 /***************************************************************************
 *          Apache Cayenne interface for RDBMS Linter source code           *
 *        This is unpublished proprietary source code of RELEX, Inc.        *
 *     The copyright notice does not evidence any actual or intended        *
 *                  publication of such source code.                        *
 *                                                                          *
 *                 Copyright (c) 2003-2014 RELEX Inc.                       *
 *                                                                          *
 *                   This software was developed by                         *
 *                                                                          *
 *                            RELEX, Inc.                                   *
 *                         Voronezh, Russia                                 *
 *                                                                          *
 *                        All rights reserved.                              *
 ****************************************************************************
 AUTHOR:
 ----------------------------------------------------------------------------
 PURPOSE:
 ----------------------------------------------------------------------------
 REVISION HISTORY:
 23.09.14 -- Galina Ermakova (Changed "ALTER COLUMN")
 24.09.14 -- Galina Ermakova (Changed "DROP FOREIGN KEY")
 25.09.14 -- Galina Ermakova (Changed "ALTER COLUMN ... NOT NULL")
 26.09.14 -- Galina Ermakova (Changed "ALTER TABLE ... DROP PRIMARY KEY")
 27.09.14 -- Galina Ermakova (Changed "ALTER COLUMN ... ENABLE NULL")
*/
package org.apache.cayenne.dba.linter;

import org.apache.cayenne.dba.QuotingStrategy;
import org.apache.cayenne.map.DbAttribute;
import org.apache.cayenne.map.DbEntity;
import org.apache.cayenne.merge.MergerFactory;
import org.apache.cayenne.merge.MergerToken;
import org.apache.cayenne.merge.SetColumnTypeToDb;
import org.apache.cayenne.dba.DbAdapter;
import org.apache.cayenne.map.DbRelationship;
import org.apache.cayenne.merge.DropRelationshipToDb;
import org.apache.cayenne.map.DbJoin;
import org.apache.cayenne.merge.SetNotNullToDb;
import org.apache.cayenne.merge.SetPrimaryKeyToDb;
import org.apache.cayenne.merge.SetAllowNullToDb;

import java.util.Collections;
import java.util.Collection;
import java.util.List;



public class LinterMergerFactory extends MergerFactory {

    @Override
    public MergerToken createSetColumnTypeToDb(
            final DbEntity entity,
            DbAttribute columnOriginal,
            final DbAttribute columnNew) {

        return new SetColumnTypeToDb(entity, columnOriginal, columnNew) {

            @Override
            protected void appendPrefix(StringBuffer sqlBuffer, QuotingStrategy context) {
                sqlBuffer.append("ALTER TABLE ");
                sqlBuffer.append(context.quoteFullyQualifiedName(entity));
                sqlBuffer.append(" ALTER COLUMN ");
                sqlBuffer.append(context.quoteString(columnNew.getName()));
                sqlBuffer.append(" SIZE ");                                /* 23.09.14 ... */
                sqlBuffer.append(columnNew.getMaxLength());
                sqlBuffer.append(" ");                                     /* ... 23.09.14 */   
            }

            @Override
            public List<String> createSql(DbAdapter adapter) {            /* 23.09.14 */
              StringBuffer sqlBuffer = new StringBuffer();
              QuotingStrategy context = adapter.getQuotingStrategy(getEntity()
                .getDataMap()
                .isQuotingSQLIdentifiers());
             appendPrefix(sqlBuffer, context); 
             return Collections.singletonList(sqlBuffer.toString());
           }
        };
    }

    @Override
    public MergerToken createDropRelationshipToDb(DbEntity entity, DbRelationship rel)   /* 24.09.14 */
    {
      return new DropRelationshipToDb(entity, rel) {

          @Override
          public List<String> createSql(DbAdapter adapter) 
          {
            QuotingStrategy context = adapter.getQuotingStrategy(getEntity()
                  .getDataMap()
                  .isQuotingSQLIdentifiers());
            StringBuilder buf = new StringBuilder();
            buf.append("ALTER TABLE ");
            buf.append(context.quoteFullyQualifiedName(getEntity()));
            buf.append(" DROP FOREIGN KEY (");
            buf.append(getColumnName(context));
            buf.append(") ");

            return Collections.singletonList(buf.toString());
          }

          private String getColumnName(QuotingStrategy context)
          {
            StringBuilder buf = new StringBuilder();
//            StringBuilder refBuf = new StringBuilder();
            boolean first = true;
            for (DbJoin join : getRelationship().getJoins()) 
            {
              if (!first) 
              {
                buf.append(", ");
//                refBuf.append(", ");
              }
              else
                first = false;
   
              buf.append(context.quoteString(join.getSourceName()));
//              refBuf.append(context.quoteString(join.getTargetName()));
            }
            return buf.toString();
          }
        };
    }

    @Override
    public MergerToken createSetNotNullToDb(DbEntity entity, final DbAttribute column)  /* 25.09.14 */ 
    {
        return new SetNotNullToDb(entity, column) {

            @Override
            public List<String> createSql(DbAdapter adapter) {
                StringBuffer sqlBuffer = new StringBuffer();
                QuotingStrategy context = adapter.getQuotingStrategy(getEntity()
                        .getDataMap()
                        .isQuotingSQLIdentifiers());
                sqlBuffer.append("ALTER TABLE ");
                sqlBuffer.append(context.quoteFullyQualifiedName(getEntity()));
                sqlBuffer.append(" ALTER COLUMN ");
                sqlBuffer.append(context.quoteString(column.getName()));
                sqlBuffer.append(" DISABLE NULL ");
                return Collections.singletonList(sqlBuffer.toString());
            }
        };
    }

    @Override
    public MergerToken createSetAllowNullToDb(DbEntity entity, DbAttribute column)  /* 27.09.14 */ 
    {
        return new SetAllowNullToDb(entity, column) {

            @Override
            public List<String> createSql(DbAdapter adapter) {
                StringBuffer sqlBuffer = new StringBuffer();
                QuotingStrategy context = adapter.getQuotingStrategy(getEntity()
                        .getDataMap()
                        .isQuotingSQLIdentifiers());
                sqlBuffer.append("ALTER TABLE ");
                sqlBuffer.append(context.quoteFullyQualifiedName(getEntity()));
                sqlBuffer.append(" ALTER COLUMN ");
                sqlBuffer.append(context.quoteString(getColumn().getName()));
                sqlBuffer.append(" ENABLE NULL ");
                return Collections.singletonList(sqlBuffer.toString());
            }
        };
    }

    @Override
    public MergerToken createSetPrimaryKeyToDb(                         /* 26.09.14 */
            DbEntity entity,
            Collection<DbAttribute> primaryKeyOriginal,
            Collection<DbAttribute> primaryKeyNew,
            String detectedPrimaryKeyName) 
    {
        return new SetPrimaryKeyToDb(
                entity,
                primaryKeyOriginal,
                primaryKeyNew,
                detectedPrimaryKeyName) {

            @Override
            protected void appendDropOriginalPrimaryKeySQL(
                    DbAdapter adapter,
                    List<String> sqls) 
            {
                sqls.add("ALTER TABLE "
                        + getQuotingStrategy(adapter)
                                .quoteFullyQualifiedName(getEntity())
                        + " DROP PRIMARY KEY");
            }

        };
    }


}
