 /***************************************************************************
 *          Apache Cayenne interface for RDBMS Linter source code           *
 *        This is unpublished proprietary source code of RELEX, Inc.        *
 *     The copyright notice does not evidence any actual or intended        *
 *                  publication of such source code.                        *
 *                                                                          *
 *                 Copyright (c) 2003-2014 RELEX Inc.                       *
 *                                                                          *
 *                   This software was developed by                         *
 *                                                                          *
 *                            RELEX, Inc.                                   *
 *                         Voronezh, Russia                                 *
 *                                                                          *
 *                        All rights reserved.                              *
 ****************************************************************************
 AUTHOR:
 ----------------------------------------------------------------------------
 PURPOSE:
 ----------------------------------------------------------------------------
 REVISION HISTORY:
*/
package org.apache.cayenne.dba.linter;

import java.sql.CallableStatement;
import java.sql.Connection;
import java.sql.ResultSet;
import java.sql.SQLException;

import org.apache.cayenne.access.OperationObserver;
import org.apache.cayenne.access.jdbc.ProcedureAction;
import org.apache.cayenne.access.jdbc.RowDescriptor;
import org.apache.cayenne.access.trans.ProcedureTranslator;
import org.apache.cayenne.dba.JdbcAdapter;
import org.apache.cayenne.map.EntityResolver;
import org.apache.cayenne.query.ProcedureQuery;

/**
 * @since 3.1
 */
class LinterProcedureAction extends ProcedureAction {

    LinterProcedureAction(ProcedureQuery query, JdbcAdapter adapter, EntityResolver entityResolver) {
        super(query, adapter, entityResolver);
    }

    @Override
    public void performAction(Connection connection, OperationObserver observer) throws SQLException, Exception {

        // cloned from super except for result processing consistent with
        // CAY-1874
        
        processedResultSets = 0;

        ProcedureTranslator transl = createTranslator(connection);

        CallableStatement statement = (CallableStatement) transl.createStatement();

        try {
            initStatement(statement);
            boolean hasResultSet = statement.execute();

            // read out parameters
            readProcedureOutParameters(statement, observer);

            // read the rest of the query
            while (true) {
                if (hasResultSet) {
                    ResultSet rs = statement.getResultSet();

                    try {
                        RowDescriptor descriptor = describeResultSet(rs, processedResultSets++);
                        readResultSet(rs, descriptor, query, observer);
                    } finally {
                        try {
                            rs.close();
                        } catch (SQLException ex) {
                        }
                    }
                } else {
                    int updateCount = statement.getUpdateCount();
                    if (updateCount == -1) {
                        break;
                    }
                    adapter.getJdbcEventLogger().logUpdateCount(updateCount);
                    observer.nextCount(query, updateCount);
                }
                hasResultSet = statement.getMoreResults();
            }
        } finally {
            try {
                statement.close();
            } catch (SQLException ex) {

            }
        }
    }
}
