/**
 * component of LINTER RDBMS dialect for Hibernate 4.
 *
 * @author Galina Ermakova
 * 16.07.14 -- Galina Ermakova (added support LinterUniqueDelegate)
 * 
 */
package org.hibernate.dialect.unique;

import java.util.Iterator;

import org.hibernate.dialect.Dialect;
import org.hibernate.metamodel.relational.Column;
import org.hibernate.metamodel.relational.Index;
import org.hibernate.metamodel.relational.UniqueKey;

/**
 * @author Galina Ermakova
 */
public class LinterUniqueDelegate extends DefaultUniqueDelegate {
	/**
	 * Constructs a LinterUniqueDelegate
	 *
	 * @param dialect The dialect
	 */
	public LinterUniqueDelegate( Dialect dialect ) {
		super( dialect );
	}

        @Override
	public String getAlterTableToAddUniqueKeyCommand(
			org.hibernate.mapping.UniqueKey uniqueKey,
			String defaultCatalog,
			String defaultSchema) 
        {
		// Do this here, rather than allowing UniqueKey/Constraint to do it.
		// We need full, simplified control over whether or not it happens.
		final String tableName = uniqueKey.getTable().getQualifiedName( dialect, defaultCatalog, defaultSchema );
		return "alter table " + tableName + " add " + uniqueConstraintSql( uniqueKey );
	}

	@Override
	public String getAlterTableToDropUniqueKeyCommand(
			org.hibernate.mapping.UniqueKey uniqueKey,
			String defaultCatalog,
			String defaultSchema) 
        {
		// Do this here, rather than allowing UniqueKey/Constraint to do it.
		// We need full, simplified control over whether or not it happens.
		final StringBuilder buf = new StringBuilder( "alter table " );
		buf.append( uniqueKey.getTable().getQualifiedName( dialect, defaultCatalog, defaultSchema ) );
		buf.append(" drop " );
		if ( dialect.supportsIfExistsBeforeConstraintName() ) {
			buf.append( "if exists " );
		}
		buf.append( dialect.quote( uniqueKey.getName() ) );
		if ( dialect.supportsIfExistsAfterConstraintName() ) {
			buf.append( " if exists" );
		}
		return buf.toString();
	}

	@Override
	public String getAlterTableToAddUniqueKeyCommand(UniqueKey uniqueKey) 
        {
		// Do this here, rather than allowing UniqueKey/Constraint to do it.
		// We need full, simplified control over whether or not it happens.
		final String tableName = uniqueKey.getTable().getQualifiedName( dialect );

		return "alter table " + tableName + " add " + uniqueConstraintSql( uniqueKey );
	}

	@Override
	public String getAlterTableToDropUniqueKeyCommand(UniqueKey uniqueKey) 
        {
		// Do this here, rather than allowing UniqueKey/Constraint to do it.
		// We need full, simplified control over whether or not it happens.
		final String tableName = uniqueKey.getTable().getQualifiedName( dialect );

		return "alter table " + tableName + " drop ";
	}
}
