/***************************************************************************
 *                           RELEX, Inc. DBMS Linter                        *
 *  This is unpublished propietary source code of RELEX, Inc.               *
 *  The copyright notice does not evidence any actual or intended           *
 *  publication of such source code.                                        *
 *                                                                          *
 *                 Copyright (c) 1990-2012 RELEX, Inc.                      *
 *                                                                          *
 *                   This software was developed by                         *
 *                                                                          *
 *                            RELEX, Inc.                                   *
 *                         Voronezh, Russia                                 *
 *                                                                          *
 *                        All rights reserved.                              *
 ****************************************************************************
 AUTHOR:
 ----------------------------------------------------------------------------
 PURPOSE: convert L_LONG int ( 32bit ) to L_DECIMAL
 ----------------------------------------------------------------------------
   14.10.91 -- first release
   20.07.97 -- Ermakov (Copyright)
   28.04.98 -- Peshkoff (Register optimization)
   24.03.99 -- DV (New data types)
   08.04.99 -- Maxim -- some changes for int64 library
   22.04.99 -- Kononyhin (Stub for DLongToDec for WIN32 kernel < 570)
   20.10.03 -- KOV (Fix for TT#10185: correction for 0x80000000L)
   19.06.08 -- Ermakov -- avoid __moddi3 for GCC
   01.08.08 -- Corrected prev for QNX4
   08.06.12 -- Glukhov (datatypes)
*/

#include <stdlib.h>                   
#include "decimals.h"

#ifdef INT64_EMULATION_ENABLE                                    /* 01.08.08 */
#define GCC_AVOID
#else
/*
For GCC we must convert long long to unsigned long long to avoid
use __moddi3 function from gcc spec. library
*/
#define GCC_AVOID (L_UDLONG) 
#endif

L_SWORD  DLLAPI LongToDec(L_LONG L, L_DECIMAL D)
/*********************************************/
{ 
  register L_SWORD i,isNeg, isMaxNeg = 0;					             /* 28.04.98 20.10.03 */

  SETDECSTATUS(DECZERO,D);
  if (L) 
  {
    isNeg = (L < 0); 
    if (isNeg ) 
    {
      if ( L == 0x80000000 )                               /* 20.10.03 ... */
      {
        isMaxNeg = 1;
        L = 0x7FFFFFFF;
      }
      else                                                 /* ... 20.10.03 */
        L = -L;
    }
    for(i=DECFRACTLNG; L && ++i<DECLNG;) 
    {
      D[i] = (L_CHAR)(L % DECLIMIT); 
      L /= DECLIMIT;   
    }
    if ( isMaxNeg )                                        /* 20.10.03 */
      D[DECFRACTLNG + 1] += 1;                             /* 20.10.03 */

    SETDECSTATUS( ( L ? ( isNeg ? DECNEGOVER : DECPOSOVER ) :
                        ( isNeg ? DECNEGATIV : DECPOSITIV ) ),D);
  }
  return OKDECSTATUS(D);  /* return (L==0); */
} /* end LongToNum */

/* 24.03.99 ... */
L_SWORD  DLLAPI DLongToDec(L_DLONG L, L_DECIMAL D)
/*********************************************/
{ 
  register L_SWORD i,isNeg, isMaxNeg = 0;					             /* 28.04.98 20.10.03 */

  SETDECSTATUS(DECZERO,D);
  if (!i64_IfZero(L)) 
  {                                                        /* 08.04.99 */ 
    isNeg = i64_IfNeg(L);                                  /* 08.04.99 */ 
    if (isNeg ) 
    {
      if ( !i64_Cmp( L, i64_MINDLONG() ) )                 /* 20.10.03 ... */
      {
        isMaxNeg = 1;
        L = i64_MAXDLONG();
      }
      else                                                 /* ... 20.10.03 */
        L = i64_Neg(L);                                    /* 08.04.99 */ 
    }
    for(i=DECFRACTLNG; !i64_IfZero(L) && ++i<DECLNG;) 
    {                                                      /* 08.04.99 */ 
      D[i] = (L_CHAR)(i64_GetLo(i64_RestLong(GCC_AVOID L,DECLIMIT)));  /* 01.08.08 */ /* 19.05.08 08.04.99 */ 
      L = i64_DivLong(GCC_AVOID L,DECLIMIT);                         /* 01.08.08 */ /* 19.06.09 08.04.99 */ 
    }
    if ( isMaxNeg )                                        /* 20.10.03 */
      D[DECFRACTLNG + 1] += 1;                             /* 20.10.03 */
    SETDECSTATUS( ( !i64_IfZero(L) ?                       /* 08.04.99 */ 
                        ( isNeg ? DECNEGOVER : DECPOSOVER ) : 
                        ( isNeg ? DECNEGATIV : DECPOSITIV ) ),D);
  }
  return OKDECSTATUS(D);  /* return (L==0); */
} /* end LongToNum */
/* ... 24.03.99 */
