/***************************************************************************
 *                           RELEX, Inc. DBMS Linter                        *
 *  This is unpublished propietary source code of RELEX, Inc.               *
 *  The copyright notice does not evidence any actual or intended           *
 *  publication of such source code.                                        *
 *                                                                          *
 *                 Copyright (c) 1990-2012 RELEX, Inc.                      *
 *                                                                          *
 *                   This software was developed by                         *
 *                                                                          *
 *                            RELEX, Inc.                                   *
 *                         Voronezh, Russia                                 *
 *                                                                          *
 *                        All rights reserved.                              *
 ****************************************************************************
 AUTHOR:
 ----------------------------------------------------------------------------
 PURPOSE: round L_DECIMAL value to given precision and scale
 ----------------------------------------------------------------------------
 17.08.01 -- DV (File created)
 18.11.03 -- KOV (Fix for TT#1903 on linter-it: added TRUNC function for L_DECIMAL)
 08.06.12 -- Glukhov (datatypes)
 20.12.12 -- DV (Round/trunc to 0 can change DECSTATUS)
*/

#include <stdlib.h>                  
#include <string.h>
#include "decimals.h"

static void RoundTruncDecimal( L_DECIMAL    D,
                               L_SBYTE      PREC,
                               L_SBYTE      SCALE,
                               L_BOOL       bROUND )
{
  L_DECIMAL D1, D2;
  register int I, J, M, N;
  int INTPLEN = PREC - SCALE;
  int INTPLENMAX = (DECLNG-DECFRACTLNG-1)*2;
  DECSTATUS S = GETDECSTATUS(D);
  if (S == DECZERO || S == DECERROR)
    return;

  /* Check fraction part */
  if (SCALE < DECFRACTLNG*2)
  {
    for (J=DECFRACTLNG*2; J >= SCALE+1; J--)
    {
      I = 1 + DECFRACTLNG - (J+1)/2;
      if (J&1)
      {
        N = D[I]/DECBASE;
        M = N*DECBASE;
      }
      else
        N = M = D[I]%DECBASE;
      D[I] = D[I] - M;
      if (J == SCALE+1 && N >= 5 && bROUND)                /* 18.11.03 */
      {
        memset(D1,0,sizeof(L_DECIMAL));
        memcpy(D2,D,sizeof(L_DECIMAL));
        I = 1 + DECFRACTLNG - (SCALE+1)/2;
        D1[0] = D[0];
        if (SCALE&1)
          D1[I] = 10;
        else
          D1[I] = 1;
        ADDDECIMAL(D1,D2,D);      
      }
    }
  }

  /* Check integer part */
  if ( INTPLEN >= 0 )                                      /* 18.11.03 */
  {
    if( INTPLEN < INTPLENMAX )
    {
      for (J=0; J<INTPLENMAX-INTPLEN; J++)
      {
        I = DECLNG-1-J/2;
        if (J&1)
          N = D[I]%DECBASE;
        else
          N = D[I]/DECBASE;
        if (N)
        {
          DECSTATUS S = DECERROR;
          SETDECSTATUS(S,D);
          return;
        }
      }
    }
  }
  else                                                     /* 18.11.03 ... */
  { /* for round() and trunc() */
    for (J=0; J <= ( - PREC - 1 ); J++)
    {
      I = 1 + DECFRACTLNG + J/2;
      if (J&1)
      {
        N = D[I]/DECBASE;
        M = N*DECBASE;
      }
      else
        N = M = D[I]%DECBASE;
      D[I] = D[I] - M;
      if (J == ( - PREC - 1 ) && N >= 5 && bROUND )
      {
        memset(D1,0,sizeof(L_DECIMAL));
        memcpy(D2,D,sizeof(L_DECIMAL));
        I = 1 + DECFRACTLNG - PREC/2;
        D1[0] = D[0];
        if ( PREC & 1 )
          D1[I] = 10;
        else
          D1[I] = 1;
        ADDDECIMAL(D1,D2,D);      
      }
    }
  }                                                        /* ... 18.11.03 */ 
  S = GETDECSTATUS(D);                                     /* 20.12.12 */
  SETDECSTATUS(S,D);                                       /* 20.12.12 */
}

void DLLAPI  TRUNCDECIMAL(L_DECIMAL    D,                  /* 18.11.03 ... */
                          L_BYTE       PREC,
                          L_BYTE       SCALE)
{
  RoundTruncDecimal( D, PREC, SCALE, L_TFALSE );
}                                                          /* ... 18.11.03 */

void DLLAPI  RNDDECIMAL(L_DECIMAL    D,
                        L_BYTE       PREC,
                        L_BYTE       SCALE)
/**************************************/
{
  RoundTruncDecimal( D, PREC, SCALE, L_TTRUE );               /* 18.11.03 */
} /* end RNDDECIMAL */

