/***************************************************************************
 *                           RELEX, Inc. DBMS Linter                        *
 *  This is unpublished propietary source code of RELEX, Inc.               *
 *  The copyright notice does not evidence any actual or intended           *
 *  publication of such source code.                                        *
 *                                                                          *
 *                 Copyright (c) 1990-2013 RELEX, Inc.                      *
 *                                                                          *
 *                   This software was developed by                         *
 *                                                                          *
 *                            RELEX, Inc.                                   *
 *                         Voronezh, Russia                                 *
 *                                                                          *
 *                        All rights reserved.                              *
 ****************************************************************************
 AUTHOR:
 ----------------------------------------------------------------------------
 PURPOSE: convert L_DECIMAL to double
 ----------------------------------------------------------------------------
   14.10.91 -- first release
   20.07.97 -- Ermakov (Copyright)
   07.06.04 -- Ermakov (For WINCE < 300)
   28.09.05 -- FLY (Warning - comparison of signed and not)
   11.12.05 -- Ermakov (corrected for CE - GPF avoid)
   03.04.07 -- Galizin ( Accuracy of sscanf higther then strtod for QNX )
   13.12.07 -- KOV     (Added function StringToDouble)
   08.06.12 -- Glukhov (datatypes)
   24.01.13 -- DV      (Standard linter definition)
   19.06.13 -- Glukhov (Don't use default value DBL_DIG: need to increase it for better precision)
   26.09.13 -- KOV     (Extra check for format + sign in exponent was allowed)
*/

#include <stdlib.h>                   
#include <string.h>
#include <float.h>                                         /* 13.12.07 */
#include "decimals.h"


/* Function convert string pStr to a double-precision value *pdVal;
   function returns L_TTRUE if conversion is OK 
   function returns L_TFALSE if pStr can't be converted to L_DOUBLE 
   or if pStr contains invalid symbols */
/* !!! SIMILAR FUNCTION - in INTLIB\convers.c !!! */
static L_BOOL StringToDouble(L_CHAR   *pStr,               /* 13.12.07 ... */
                             L_DOUBLE *pdVal)
{
  register L_BOOL   bMinus = L_TFALSE,
                    bSignAwaited = L_TTRUE;
  register L_LONG   fpt = -1,     /* A number of digits after floating point or (-1) */
                    exp = 0,
                    dig = 0;      /* A number of decimal digits */
  register L_DOUBLE x = 0;
  register L_CHAR   c;            /* A current character */

  while ((c = *pStr++),(c != 0))
  {
    if (c == ' ') 
      continue;
    if (bSignAwaited && (c == '+' || c == '-'))
    {
      bMinus = (c != '+');
      bSignAwaited = L_TFALSE;
      continue;
    }
    if (fpt < 0 && (c == '.' || c == ','))
    {
      fpt = 0;
      bSignAwaited = L_TFALSE;
      continue;
    }
    if (dig > 0 && (c == 'e' || c == 'E'))
    {
      register L_LONG e = 0,      /* An exponent */
                      edig = 0;   /* A number of decimal digits in exponent */
      register L_BOOL eminus = L_TFALSE;

      bSignAwaited = L_TTRUE; /* allow sign in exponent */ /* 26.09.13 */
      while ((c = *pStr++),(c != 0))
      {
        if (c == ' ') 
          continue;
        if (bSignAwaited && (c == '+' || c == '-'))
        {
          eminus = (c != '+');
          bSignAwaited = L_TFALSE;
          continue;
        }
        if (c < '0' || c > '9') 
          return L_TFALSE;
        e = e * 10 + (c - '0');
        ++edig;
      }
      if (edig == 0) 
        return L_TFALSE;
      exp += (eminus) ? -e : e;
      break;
    }
    if (c < '0' || c > '9') 
      return L_TFALSE;
    bSignAwaited = L_TFALSE;
    if (x != 0 || dig == 0) 
      ++dig;
    if (dig > DBL_DIG + 2)                                 /* 19.05.13 */
    {
      if (fpt < 0) 
        ++exp;
    }
    else
    {
      x = x * 10 + (c - '0');
      if (fpt >= 0) 
        ++fpt;
    }
  }
  if (dig == 0) 
    return L_TFALSE;
  if (x)                                                   /* 26.09.13 */
  {
    if (fpt >= 0) 
      exp = exp - fpt;
    while (exp > 0)
    {
      if (x > L_MAXDOUBLE / 10)                              /* 24.01.13 */
        return L_TFALSE;
      x *= 10;
      --exp;
    }
    if (exp < 0)                                         /* 03.10.13 ... */
    {
      L_DOUBLE Zero = 0.0,
               Ptr;

      while (exp < 0)
      {
        x /= 10;
        ++exp;
      }
      Ptr = x;
      if (!memcmp(&Ptr,&Zero,sizeof(L_DOUBLE))) /* the current number lost a mantissa */ 
        return L_TFALSE;                                     /* 26.09.13 */
    }                                                    /* ... 03.10.13 */
  }
  *pdVal = bMinus ? -x : x;
  return L_TTRUE;
}                                                          /* ... 13.12.07 */

/****************************************/
L_SWORD DLLAPI DecToDbl(const L_DECIMAL D, double *pd)     /* 11.12.05 ... */
{  
  L_CHAR STR[128];
 
  switch (GETDECSTATUS(D)) 
  {
    case DECNEGATIV :
    case DECPOSITIV :
    case DECZERO    : 
    {
      DECTOSTR(D, STR, DECLNG * 2 - 2, DECFRACTLNG*	2, 1);
      return StringToDouble(STR, pd);                      /* 13.12.07 */
    }
  }
  return 0;                                                /* ... 11.12.05 */
} /* end DecToDbl */
