/***************************************************************************
 *                           RELEX, Inc. DBMS Linter                        *
 *  This is unpublished propietary source code of RELEX, Inc.               *
 *  The copyright notice does not evidence any actual or intended           *
 *  publication of such source code.                                        *
 *                                                                          *
 *                 Copyright (c) 1990-2012 RELEX, Inc.                      *
 *                                                                          *
 *                   This software was developed by                         *
 *                                                                          *
 *                            RELEX, Inc.                                   *
 *                         Voronezh, Russia                                 *
 *                                                                          *
 *                        All rights reserved.                              *
 ****************************************************************************
 AUTHOR:
 ----------------------------------------------------------------------------
 PURPOSE: convert from L_DECIMAL to Long int (32 bit)
 ----------------------------------------------------------------------------
   14.10.91 -- first release
   19.01.95 -- DV
   20.07.97 -- Ermakov (Copyright)
   24.03.99 -- DV (New data types)
   08.04.99 -- Maxim -- some changes for int64 library
   22.04.99 -- Kononyhin (Stub for DecToDLong for WIN32 kernel < 570)
   06.02.06 -- DV (Fix TT#13291 - overflow check corrected)
   08.06.12 -- Glukhov (datatypes)
*/

#include <stdlib.h>                    
#include "decimals.h"

L_SWORD DLLAPI DecToLong(const L_DECIMAL D, L_LONG *L, L_SWORD Rounds)
/*****************************************************************/
{ L_LONG l,l0; L_SWORD i,isNeg;
  l=0; isNeg=0;
  switch (GETDECSTATUS(D)) {
   case DECNEGATIV : isNeg++;
   case DECPOSITIV : {
     for(i=DECLNG; --i>DECFRACTLNG;) {                 
       l0=l;
       l=l*DECLIMIT+D[i];                              
       if (l<l0) return 0;     /* overflow L_LONG !!! */  /* 19.01.95 */
     }
     if (Rounds && (D[DECFRACTLNG] >= DECLIMIT/2)) {    
       l++; 
       if (l<=0) return 0;     /* overflow L_LONG !!! */ /* 19.01.95 */
     }
     if (isNeg) l = -l;
   }
   case DECZERO     : {
     *L=l; return 1;
   }
  }
  return 0;

} /* end DecToLong */

/* 24.03.99 ... */
L_SWORD DLLAPI DecToDLong(const L_DECIMAL D, L_DLONG *L, L_SWORD Rounds)
/*****************************************************************/
{ L_DLONG l,l0,lmax;                                    /* 06.02.06 */
  L_SWORD i,isNeg;
  
  i64_Init(l,0,0); /* 08.04.99 */ 
  isNeg=0;
  i64_FromLong(DECLIMIT,l0);                            /* 06.02.06 ... */
  lmax = i64_Sub(L_MAXDLONG,l0);
  lmax = i64_Div(lmax,l0);                              /* ... 06.02.06 */

  switch (GETDECSTATUS(D)) {
   case DECNEGATIV : isNeg++;
   case DECPOSITIV : {
     for(i=DECLNG; --i>DECFRACTLNG;) {                 
       l0=l;
       if (i64_Cmp(l,lmax)>0)                                        /* 06.02.06 ... */
         return 0; /* overflow L_LONG !!! */
       l=i64_AddLong(i64_MulLong(l0,DECLIMIT),D[i]); /* 08.04.99 */  /* ... 06.02.06 */
       if (i64_Cmp(l,l0)<0)                          /* 08.04.99 */ 
         return 0; /* overflow L_LONG !!! */  /* 19.01.95 */
     }
     if (Rounds && (D[DECFRACTLNG] >= DECLIMIT/2)) {    
       i64_Inc(&l);                                 /* 08.04.99 */ 
       if ( i64_IfNeg(l) || i64_IfZero(l) )         /* 08.04.99 */ 
         return 0;     /* overflow L_LONG !!! */ /* 19.01.95 */
     }
     if (isNeg) 
       l = i64_Neg(l);
   }
   case DECZERO     : {
     *L=l; return 1;
   }
  }
  return 0;

} /* end DecToLong */
/* ... 24.03.99 */
