/***************************************************************************
 *                           RELEX, Inc. DBMS Linter                        *
 *  This is unpublished propietary source code of RELEX, Inc.               *
 *  The copyright notice does not evidence any actual or intended           *
 *  publication of such source code.                                        *
 *                                                                          *
 *                 Copyright (c) 1990-2012 RELEX, Inc.                      *
 *                                                                          *
 *                   This software was developed by                         *
 *                                                                          *
 *                            RELEX, Inc.                                   *
 *                         Voronezh, Russia                                 *
 *                                                                          *
 *                        All rights reserved.                              *
 ****************************************************************************
 AUTHOR:
 ----------------------------------------------------------------------------
 PURPOSE: convert from L_DECIMAL to string
 ----------------------------------------------------------------------------
   14.10.91 -- first release
   23.10.94 -- Peshkoff
   09.12.94 -- Borisenkov
   12.12.94 -- Borisenkov
   16.07.96 -- Borisenkov (Correcting DECTOSTR output)
   20.07.97 -- Ermakov (Copyright)
   25.08.97 -- DV (TO_CHAR correcting)
   10.03.99 -- DV (Rounding instead of truncation)
   23.07.99 -- DV (Optional clearing of tail zeros - DECTOSTRN function)
   25.10.01 -- DV (Additional check, comments, cosmetic)
   08.06.12 -- Glukhov (datatypes)
   22.01.13 -- DV (Correct 0 output)
*/

/*
#define _DEB
*/

#include <stdlib.h>                     
#include <string.h>
#include "decimals.h"


void DLLAPI DECTOSTRN(L_DECIMAL const D,      /* D->STR */ /* 23.07.99 */
                      L_CHAR   *   STR,
                      L_SWORD      LEN,
                      L_SWORD      LENFRACT,
                      L_SWORD      ISSIGN,
		      L_SWORD      CLEARTAIL)		   /* 23.07.99 */
/******************************************/
{ DECSTATUS  S;
  L_SWORD    I,J,C;
  L_CHAR     CH1, CH2;
  L_CHAR    *RS;
  L_CHAR     RSTR[DECLNG*2+1];
  L_SWORD    WH = 20;
  L_BOOL     ROUND = L_TFALSE;			/* 10.03.99 */

#ifdef _DEB
printf(" DecToStr Len = %d LenFract = %d\n",LEN,LENFRACT);
#endif

  if (LEN > 30) LEN = 30;                       /* 12.12.94 */
  if (LENFRACT > 10) LENFRACT = 10;
  if (LEN-LENFRACT>20) LEN = LENFRACT+20;	/* 25.10.01 */

  S = GETDECSTATUS(D);
  I=0;
  /* Form sign */
  RSTR[I++] = (S == DECNEGOVER || S == DECNEGATIV)
                   ? (L_CHAR)'-' :
             ((ISSIGN)
                   ? (L_CHAR)'+' : (L_CHAR) ' ');
                 
  /* Form number text from number internal representation */                             
  for (J = DECLNG; --J;)  {   /* 30.06.92 */
   if (J == DECFRACTLNG)  {   /* 30.06.92 */
     RSTR[I++] = (S == DECNEGOVER || S == DECPOSOVER) ?
              (L_CHAR)'9' /* ! */ : (L_CHAR)'.';        
   }
   switch (S) {

    case DECNEGOVER:
    case DECPOSOVER:   { CH1 = '9'; CH2 = '9'; break; }

    case DECERROR:   { CH1 = '?'; CH2 = '?';   break; }

    default: {
      C = D[J];
      CH1 = (L_CHAR)(C / DECBASE) + '0';      
      CH2 = (L_CHAR)(C % DECBASE) + '0';     
    }
   }
   RSTR[I++]=CH1;
   RSTR[I++]=CH2;
 }
 RSTR[I]='\0';

#ifdef _DEB
printf("------------------------------------------\n");
printf(" Result = '%s'\n",RSTR);
#endif

/* 23.10.94 */
  /* Clear leading zeros */                             
  RS = (L_CHAR *)&RSTR + 1;
  while (*RS == '0')
    { *RS++ = ' '; WH--; }                      /* 09.12.94 */
  /* Clear trailing zeros */                             
  RS = (L_CHAR *)&RSTR + strlen(RSTR) - 1;
  /* 23.07.99 ... */
  I = 0;
  if (CLEARTAIL)
    while (*RS == '0')
      *RS-- = ' ';
  else
    while (*RS == '0' && I < 10 - LENFRACT)
      { *RS-- = ' '; I++; }
  /* ... 23.07.99 */
  RSTR[DECLNG * 2] = '\0';
  /* Skip to the first significant digit */                             
  RS = (L_CHAR *)&RSTR + 1; 
  while (*RS == ' ')
    RS++;
/* overflow ? */

#ifdef _DEB
printf(" R1 = '%s' dl = %d\n",RSTR,RS - (L_CHAR *)&RSTR);
#endif

  /* Check if enough space for number text */                             
  if ((20 + (LENFRACT ? 1 : 0) - (RS - (L_CHAR *)&RSTR) < LEN)    /* 09.12.94 DV */
      && (WH <= LEN - LENFRACT))           /* 16.07.96 DV */ /* 25.08.97 */
    {
    if (--RS != (L_CHAR *)RSTR)
      {
      /* Copy sign before the first significant digit */                             
      *RS = RSTR[0];
      RSTR[0] = ' ';
      }

#ifdef _DEB
printf(" 1 RS= '%s' RSTR = '%s' %d\n",RS,RSTR,strlen(RS));
#endif

    /* Align number according to the borders */
    for (I = 0; I < LEN + 2; I++)
      RSTR[I] = RSTR[I + 20 + LENFRACT - LEN];			/* 25.10.01 */
    /*  10.03.99 ... */
    /* Is rounding needed? */
    /* Compare old digit value (before copying) */
    if (LENFRACT < 10 && RSTR[22+LENFRACT] >= '5')		/* 25.10.01 */
      ROUND = L_TTRUE;
    /* ... 10.03.99 */
    RSTR[LEN+2] = '\0';

    /* 10.03.99 ... */
    /* Round number if needed */
    if (ROUND)
    {
      J = LEN+1;
      for (;J>0;J--)
      {
        if (RSTR[J] == '?')
          break;
        if (RSTR[J] == '.')
          continue;
        else if (RSTR[J] == ' ')
        {
          RSTR[J] = '1';
          break;
        }
        else if (RSTR[J] == '+' || RSTR[J] == '-')
        {
          RSTR[J-1] = RSTR[J];
          RSTR[J] = '1';
          break;
        }
        RSTR[J]++;
        if (RSTR[J] != ':')
          break;
        else
          RSTR[J] = '0';
      }
      if (J == 0)
      {
        memset(RSTR,'*',LEN + 2);
        RSTR[LEN + 2] = '\0';
      }
    }
    /* ... 10.03.99 */

#ifdef _DEB
printf(" 2 RS= '%s' RSTR = '%s'\n",RS,RSTR);
printf(" Ddd = %d\n",1 + LEN - LENFRACT + 1);
#endif
    /* Add one leading and/or one trailing zero if needed */
    if (RSTR[LEN - LENFRACT] == ' ')                    /* 09.12.94 */
      RSTR[LEN - LENFRACT] = '0';
    /* 16.07.96 DV */
    else if (LEN - LENFRACT - (ISSIGN ? 1 : 0) >= 0)    /* 22.01.13 */
    {
      if (RSTR[LEN - LENFRACT] == '-')
      {
        RSTR[LEN - LENFRACT] = '0';
        RSTR[LEN - LENFRACT - 1] = '-';
      }
      else if (RSTR[LEN - LENFRACT] == '+')
      {
        RSTR[LEN - LENFRACT] = '0';
        RSTR[LEN - LENFRACT - 1] = '+';
      }
    }
    /* 16.07.96 */
    if (RSTR[LEN + 2 - LENFRACT] == ' ')
      RSTR[LEN + 2 - LENFRACT] = '0';

#ifdef _DEB
printf(" 3 RSTR = '%s'\n",RSTR);
#endif
    if (LENFRACT == 0)
      RSTR[LEN + 1] = ' '; 
    }
  else
    {
    memset(RSTR,'*',LEN + 2);
    RSTR[LEN + 2] = '\0';
    }
  strcpy(STR,RSTR);
/* 23.10.94 */

} /* end DECTOSTR */

void DLLAPI  DECTOSTR(L_DECIMAL const D,          /* D->STR */
                      L_CHAR *     STR,
                      L_SWORD      LEN,
                      L_SWORD      LENFRACT,
                      L_SWORD      ISSIGN)
{
  DECTOSTRN(D, STR, LEN, LENFRACT, ISSIGN, L_TTRUE);
}
