/***************************************************************************
 *                           RELEX, Inc. DBMS Linter                        *
 *  This is unpublished proprietary source code of RELEX, Inc.              *
 *  The copyright notice does not evidence any actual or intended           *
 *  publication of such source code.                                        *
 *                                                                          *
 *                 Copyright (c) 1990-2013 RELEX, Inc.                      *
 *                                                                          *
 *                   This software was developed by                         *
 *                                                                          *
 *                            RELEX, Inc.                                   *
 *                         Voronezh, Russia                                 *
 *                                                                          *
 *                        All rights reserved.                              *
 ****************************************************************************
 AUTHOR:
 ----------------------------------------------------------------------------
 PURPOSE: Conversions : String  --> Number
----------------------------------------------------------------------------
 21.04.92                                 
 11.01.93 -- Ustinov                       
 17.10.94 -- Borisenkov                    
 31.07.97 -- Ermakov (Copyright)
 28.11.97 -- Pasec (types)
 04.10.98 -- Peshkoff (Procedure headers)
 08.04.99 -- Maxim -- some changes for int64 library
 17.04.99 -- DV (Bugs fixed)
 22.04.99 -- Kononyhin (Stub for ConvDLong for WIN32 if not __64BIT_KERNEL__)
 24.04.99 -- DV (Fixed negative int64 reading bug)
 12.08.99 -- Pasec (new implementation of ConvLong)
 26.01.00 -- DV (ConvLong additional range checking)
 24.05.00 -- DV (Clear errno before operation and checking errno)
 05.04.01 -- Ermakov (Macro REAL replaced by L_REAL)
 04.09.01 -- DV (Return max negative long in ConvLong)
 18.10.01 -- Galizin ( Change native types to define )
 03.03.03 -- EVG   (Windows CE)  
 07.06.04 -- Ermakov (For WINCE < 300)
 11.12.05 -- Ermakov (Corrected prev)
 22.08.06 -- Pasechnik (SQL Numeric type convertion moved from OLEDB/cvtdecnum.c)
 03.04.07 -- Galizin ( accuracy of atof less then sscanf for qnx )
 20.07.07 -- FLY (Avoid warnings)
 09.10.07 -- FLY (Avoid buffer overflow.)
 29.07.08 -- Ermakov (QNX4 build)
 25.01.09 -- Ermakov (moved 13.12.07 by KOV from decimals. Convert string to double with invalid symbol checking)
 28.12.11 -- DV (Do not skip checks for QNX6)
 31.06.12 -- Glukhov (Correct datatypes)
 08.06.12 -- Glukhov (datatypes)
 16.11.12 -- Glukhov,Ermakov (Remove extra code, datatypes)
 24.01.13 -- DV (Remove extra float checks)
 19.06.13 -- Glukhov (Don't use default value DBL_DIG: need to increase it for better precision)
 26.09.13 -- KOV (Function StringToDouble instead of atod/sscanf etc.)
 03.10.13 -- Galizin, Ermakov (gcc optimization -O3 avoid)
*/

#include <ctype.h>
#ifndef WINCE                                              /* 03.03.03 ...*/
#  include <errno.h>
#else
#  include "wceclib.h"
#endif                                                    /*... 03.03.03 */
#include <math.h>
#include <string.h>      /* 11.01.93 */
#include <stdlib.h>      /* 11.01.93 */
#include <float.h>                                         /* 25.01.09 */

#include "decimals.h"                                         /* 22.08.06 */
#include "convers.h"


static const L_BYTE CFTS[209] =
{
/*  0     1     2     3     4     5     6     7     8     9    10    11     12 */
  0x00, 0x00, 0x00, 0x40, 0xEA, 0xED, 0x74, 0x46, 0xD0, 0x9C, 0x2C, 0x9F, 0x0C,
  0x00, 0x00, 0x00, 0xA0, 0xCA, 0x17, 0x72, 0x6D, 0xAE, 0x0F, 0x1E, 0x43, 0x01,
  0x00, 0x00, 0x00, 0x10, 0x61, 0x02, 0x25, 0x3E, 0x5E, 0xCE, 0x4F, 0x20,
  0x00, 0x00, 0x00, 0xE8, 0x3C, 0x80, 0xD0, 0x9F, 0x3C, 0x2E, 0x3B, 0x03,
  0x00, 0x00, 0x00, 0xE4, 0xD2, 0x0C, 0xC8, 0xDC, 0xD2, 0xB7, 0x52,
  0x00, 0x00, 0x00, 0x4A, 0x48, 0x01, 0x14, 0x16, 0x95, 0x45, 0x08,
  0x00, 0x00, 0x00, 0xA1, 0xED, 0xCC, 0xCE, 0x1B, 0xC2, 0xD3, 
  0x00, 0x00, 0x80, 0xF6, 0x4A, 0xE1, 0xC7, 0x02, 0x2D, 0x15, 
  0x00, 0x00, 0x40, 0xB2, 0xBA, 0xC9, 0xE0, 0x19, 0x1E, 0x02, 
  0x00, 0x00, 0xA0, 0xDE, 0xC5, 0xAD, 0xC9, 0x35, 0x36, 
  0x00, 0x00, 0x10, 0x63, 0x2D, 0x5E, 0xC7, 0x6B, 0x05, 
  0x00, 0x00, 0xE8, 0x89, 0x04, 0x23, 0xC7, 0x8A, 
  0x00, 0x00, 0x64, 0xA7, 0xB3, 0xB6, 0xE0, 0x0D, 
  0x00, 0x00, 0x8A, 0x5D, 0x78, 0x45, 0x63, 0x01, 
  0x00, 0x00, 0xC1, 0x6F, 0xF2, 0x86, 0x23, 
  0x00, 0x80, 0xC6, 0xA4, 0x7E, 0x8D, 0x03, 
  0x00, 0x40, 0x7A, 0x10, 0xF3, 0x5A, 
  0x00, 0xA0, 0x72, 0x4E, 0x18, 0x09, 
  0x00, 0x10, 0xA5, 0xD4, 0xE8, 
  0x00, 0xE8, 0x76, 0x48, 0x17, 
  0x00, 0xE4, 0x0B, 0x54, 0x02, 
  0x00, 0xCA, 0x9A, 0x3B, 
  0x00, 0xE1, 0xF5, 0x05, 
  0x80, 0x96, 0x98, 
  0x40, 0x42, 0x0F, 
  0xA0, 0x86, 0x01, 
  0x10, 0x27, 
  0xE8, 0x03, 
  0x64, 
  0x0A, 
  0x01
};

static const L_BYTE A_OFFSET[31][2] =
{
/* 1-7   */  {0,13}, {13,13},  {26,12}, {38,12}, {50,11}, {61,11}, {72,10},
/* 8-14  */ {82,10}, {92,10},  {102,9}, {111,9}, {120,8}, {128,8}, {136,8}, 
/* 15-21 */ {144,7}, {151,7},  {158,6}, {164,6}, {170,5}, {175,5}, {180,5},
/* 22-28 */ {185,4}, {189,4},  {193,3}, {196,3}, {199,3}, {202,2}, {204,2}, 
/* 29-31 */ {206,1}, {207,1},  {208,1}
};


#ifdef __64BIT_KERNEL__
L_DLONG ConvDLong(L_CHAR* s, L_SWORD* err)                  /* 18.10.01 */
{
  register short   i,j,lens,isneg;
  register char    c;
  auto     char    swrk [22];
  L_DLONG          l;

  lens = (short) strlen(s);
  while (lens > 0 && isspace(s[lens-1])) lens--;
  if ((*err = ((lens > 0) ? ISSUC : ENONUM)) != ISSUC)
    return I64_ZERO;                                     /* 08.04.99 */ 

  for (i = 0; i < lens && isspace(s[i]); i++);
  if ((*err = ((i < lens) ? ISSUC : ENONUM)) != ISSUC)
    return I64_ZERO;                                     /* 08.04.99 */

  c = s[i];
  isneg = c == '-';
  if (isneg || (c == '+')) i++;
  if ((*err = ((i < lens) ? ISSUC : ENONUM)) != ISSUC)
    return I64_ZERO;                                     /* 08.04.99 */

  for (; i < lens && s[i] == '0'; i++);
  if (i >= lens)
    return I64_ZERO;                                     /* 08.04.99 */

  j = 0;
  if (isneg) swrk[j++] = '-';
  while (j < 19+isneg && i < lens) {                     /* 17.04.99 */ /* 24.04.99 */
    c = s[i++];
    if ((*err = (isdigit(c) ? ISSUC : EBADSYM)) != ISSUC)
      return I64_ZERO;                                   /* 08.04.99 */
    swrk[j++] = c;
    }
  if ((*err = ((i == lens) ? ISSUC : EOVER)) != ISSUC)
    return I64_ZERO;                                     /* 08.04.99 */
  swrk[j] = '\0';

  lens = (short) strlen(swrk);
  if (isneg) lens--;

  c    = (isneg) ? swrk[1] : swrk[0];
  if ((*err = (((lens <= 19+isneg)) ? ISSUC : EOVER)) != ISSUC) /* 17.04.99 */ /* 24.04.99 */
    return I64_ZERO;                                     /* 08.04.99 */

  i64_FromLong(0,l);                                    /* 08.04.99 */ 
  for (j=isneg; j<lens+isneg; j++)                      /* 24.04.99 */
    l = i64_AddLong(i64_MulLong(l,10),(swrk[j] - '0')); /* 08.04.99 */ 
  if (isneg)
    l = i64_Neg(l);                                    /* 08.04.99 */ 

  if ( i64_IfZero(l) )                                 /* 08.04.99 */ 
    *err = ECONV;
  else if (isneg)
    *err = i64_IfNeg(l) ? ISSUC : EOVER;               /* 08.04.99 */ 
  else
    *err = i64_IfPos(l) ? ISSUC : EOVER;               /* 08.04.99 */ /* 17.04.99 */

  if (*err)
    return I64_ZERO;                                   /* 08.04.99 */ 
  else
    return l;
} /* ConvDLong */

#if 0
void PrintDLong(l,s,len)
  L_DLONG l;
  char  *s;
  short len;
{
  register short   i;
  L_BOOL sign = L_TFALSE;
 
  if (i64_IfNeg(l))                                    /* 08.04.99 */ 
  {
    sign = L_TTRUE;
    l = i64_Neg(l);                                    /* 08.04.99 */ 
  } 
  for (i = len-1; i >= 0; i--)
  {
    if (!i64_IfZero(l))                                /* 08.04.99 */ 
      s[i] = '0' + i64_GetLo(i64_RestLong(l,10));      /* 08.04.99 */ 
    else 
      if (sign)
      {
        s[i] = '-';
        sign = L_TFALSE;
      }
      else if (i == len-1)
        s[i] = '0';
      else
        s[i] = ' ';
    l = i64_DivLong(l,10);                             /* 08.04.99 */ 
  }
  s[len] = '\0';
} /* PrintDLong */
#endif
#else
# ifdef WIN32                                          /* 22.04.99 */
/* It's a stub only */
L_LONG ConvDLong(s,err)
{
  return(0);
}
# endif
#endif

#ifdef OLD_VERSION_CONVLONG /* 12.08.99 */
L_LONG ConvLong(s,err) /* 28.11.97 long -> L_LONG */
  char  *s;
  short *err;
{
  register short   i,j,lens,isneg;
  register char    c;
  auto     char    swrk [12];
  L_LONG          l;

  lens = (short) strlen(s);
  /* udalenie kone~nyh simwolow probela */
  while (lens > 0 && isspace(s[lens-1])) lens--;
  if ((*err = ((lens > 0) ? ISSUC : ENONUM)) != ISSUC)
    return 0;

  /* udalenie ha~alxnyh simwolow probela */
  for (i = 0; i < lens && isspace(s[i]); i++);
  if ((*err = ((i < lens) ? ISSUC : ENONUM)) != ISSUC)
    return 0;

  c = s[i];
  isneg = c == '-';
  if (isneg || (c == '+')) i++; /* probrositx znak ~isla */
  if ((*err = ((i < lens) ? ISSUC : ENONUM)) != ISSUC)
    return 0;

  /* udalenie wedu}ih nulej */
  for (; i < lens && s[i] == '0'; i++);
  if (i >= lens) /* ~islo = 0 */
    return 0;

  j = 0;
  if (isneg) swrk[j++] = '-';
  while (j < 11 && i < lens) {
    c = s[i++];
    if ((*err = (isdigit(c) ? ISSUC : EBADSYM)) != ISSUC)
      return 0;
    swrk[j++] = c;
    }
  if ((*err = ((i == lens) ? ISSUC : EOVER)) != ISSUC)
    return 0;
  swrk[j] = '\0';

  lens = (short) strlen(swrk);
  if (isneg) lens--;

  c    = (isneg) ? swrk[1] : swrk[0];
  if ((*err =
       (((lens < 10) ||
         ((lens == 10) && ((c == '1') || (c == '2'))))
        ? ISSUC : EOVER)) != ISSUC)
    return 0;

  l = (L_LONG) atol(swrk);
  if (l == 0)
    *err = ECONV;       /* neopoznannaq o{ibka */
  else if (isneg)
    *err = (l < 0) ? ISSUC : EOVER;
  else
    *err = (l > 0) ? ISSUC : EOVER;

  if (*err)
    return 0;
  else
    return l;
} /* ConvLong */
#else
L_LONG ConvLong(L_CHAR* str_in, L_SWORD * code) /* 12.08.99 */
{
  char *str, *s_str;
  L_LONG l = 0, result;
  int sign = 1;

  *code = ISSUC;

  str = str_in;

  while(*str == ' ') str++;

  switch(*str)
   {
    case '+':
      sign = 1;
      str++;
     break;
    case '-':
      sign = -1;
      str++;
     break;
    case '\0':
      *code = ENONUM /*printf("No number!")*/;
   }

  s_str = str;

  while (isdigit((int) *str))
   l = l * 10 + (*str++ - '0');

  if ((str - s_str) > 10)
   if (sign > 0)
    *code = EOVER /*printf("Overflow!")*/;
   else
    *code = EOVER /*printf("Underflow!")*/;
  else if (sign == -1 && l == L_MINLONG)                /* 04.09.01 */
    return l;                                           /* 04.09.01 */
  else if (l%10 != (*(str-1) - '0'))                    /* 26.01.00 */
    *code = EOVER;                                      /* 26.01.00 */

  while (isspace((int) *str)) str++;

  if (*str != '\0')
   *code = EBADSYM /*printf("Bad Symbol!")*/;

  result = l * sign;

  if (sign > 0 && l < 0 && result < 0)
   *code = EOVER /*printf("Overflow!")*/;
  else if (l < 0 && result > 0)
   *code = EOVER /*printf("Underflow!")*/;

  /*printf("in: %s\n l: %d\n res: %d\n",str_in, l, result);*/

  return result;
}
#endif


L_WORD LWord(L_LONG l, L_SWORD* err) /* 28.11.97 */         /* 18.10.01 */
{
  *err = (((L_LONG)0 <= l) && (l <= (L_LONG)L_MAXWORD))
           ? ISSUC : EOVER;
  return (L_WORD)l;
} /* LWord */


L_SWORD LInteger(L_LONG l, L_SWORD* err) /* 28.11.97 */      /* 18.10.01 */
{
  *err = (((L_LONG)L_MINSWORD <= l) && (l <= (L_LONG)L_MAXSWORD))
           ? ISSUC : EOVER;
  return (L_SWORD)l;
} /* LInteger */


L_WORD ConvWord(L_CHAR* s, L_SWORD* err) /* 28.11.97 */     /* 18.10.01 */
{
  L_LONG l;
  L_WORD w;

  w = 0;
  l = ConvLong(s,err);
  if (*err == ISSUC) w = LWord(l,err);
  return w;
} /* ConvWord */


L_SWORD ConvInteger(L_CHAR* s, L_SWORD* err) /* 28.11.97 */ /* 18.10.01 */
{
  L_LONG l;
  L_SWORD i;

  i = 0;
  l = ConvLong(s,err);
  if (*err == ISSUC) i = LInteger(l,err);
  return i;
} /* ConvInteger */


/* Function convert string pStr to a double-precision value *pdVal */
/* !!! SIMILAR FUNCTION - in DECIMALS\dtodbl.c !!! */
static L_SWORD StringToDouble(L_CHAR   *pStr,              /* 26.09.13 ... */
                              L_DOUBLE *pdVal)
{
  register L_BOOL   bMinus = L_TFALSE,
                    bSignAwaited = L_TTRUE;
  register L_LONG   fpt = -1,     /* A number of digits after floating point or (-1) */
                    exp = 0,
                    dig = 0;      /* A number of decimal digits */
  register L_DOUBLE x = 0;
  register L_CHAR   c;            /* A current character */

  while ((c = *pStr++),(c != 0))
  {
    if (c == ' ') 
      continue;
    if (bSignAwaited && (c == '+' || c == '-'))
    {
      bMinus = (c != '+');
      bSignAwaited = L_TFALSE;
      continue;
    }
    if (fpt < 0 && (c == '.' || c == ','))
    {
      fpt = 0;
      bSignAwaited = L_TFALSE;
      continue;
    }
    if (dig > 0 && (c == 'e' || c == 'E'))
    {
      register L_LONG e = 0,      /* An exponent */
                      edig = 0;   /* A number of decimal digits in exponent */
      register L_BOOL eminus = L_TFALSE;

      bSignAwaited = L_TTRUE;
      while ((c = *pStr++),(c != 0))
      {
        if (c == ' ') 
          continue;
        if (bSignAwaited && (c == '+' || c == '-'))
        {
          eminus = (c != '+');
          bSignAwaited = L_TFALSE;
          continue;
        }
        if (c < '0' || c > '9')
          return EBADSYM;
        e = e * 10 + (c - '0');
        ++edig;
      }
      if (edig == 0)
        return ENONUM;
      exp += (eminus) ? -e : e;
      break;
    }
    if (c < '0' || c > '9')
      return EBADSYM;
    bSignAwaited = L_TFALSE;
    if (x != 0 || dig == 0) 
      ++dig;
    if (dig > DBL_DIG + 2)
    {
      if (fpt < 0) 
        ++exp;
    }
    else
    {
      x = x * 10 + (c - '0');
      if (fpt >= 0) 
        ++fpt;
    }
  }
  if (dig == 0)
    return ENONUM;
  if (x)
  {
    if (fpt >= 0) 
      exp = exp - fpt;
    while (exp > 0)
    {
      if (x > L_MAXDOUBLE / 10) 
        return EOVER;
      x *= 10;
      --exp;
    }
    if (exp < 0)                                         /* 03.10.13 ... */
    {
      L_DOUBLE Zero = 0.0,
               Ptr;

      while (exp < 0)
      {
        x /= 10;
        ++exp;
      }
      Ptr = x;
      if (!memcmp(&Ptr,&Zero,sizeof(L_DOUBLE))) /* the current number lost a mantissa */ 
        return ELOSTV;
    }                                                    /* ... 03.10.13 */
  }
  *pdVal = bMinus ? -x : x;
  return ISSUC;
}                                                          /* ... 26.09.13 */

L_DOUBLE ConvDouble(L_CHAR* s, L_SWORD* err) /* 28.11.97 */ /* 18.10.01 */
{
  L_DOUBLE d;
  return (*err = StringToDouble(s, &d)), d;                /* 26.09.13 */
} /* ConvDouble */

L_REAL    DReal(L_DOUBLE d, L_SWORD *err)            /* 05.04.01 *//* 18.10.01 */
{
  double x;

  *err = ISSUC;
  if (d != 0.0) {
    x = (d > 0.0) ? d : -d;
#ifdef OLD_FLOAT_CHECKS                                 /* 24.01.13 */
    if      (x < (L_DOUBLE)L_MINABSREAL) {
      *err = ELOSTV;
      d = 0.0;
      }
    else  
#endif
    if (x > (L_DOUBLE)L_MAXREAL) {
      *err = EOVER;
      d = (d > 0.0) ? (L_DOUBLE)L_MAXREAL : (L_DOUBLE)-L_MAXREAL;
      }
    }
  return (L_REAL)d;                             /* 05.04.01 */
} /* DReal */


L_REAL ConvReal(L_CHAR  *s, L_SWORD *err)                  /* 05.04.01 *//* 18.10.01 */
{
  L_DOUBLE d;
  L_REAL   r;                                              /* 05.04.01 */

  r = 0.0;
  d = ConvDouble(s,err);
  if (*err == ISSUC) r = (L_REAL)d;             /* 05.04.01 */
  return r;
} /* ConvReal */

#ifdef __64BIT_KERNEL__
L_ULONG GetHighLongword(L_DLONG dl)
{
  return i64_GetHi(dl);                             /* 08.04.99 */ 
}

L_ULONG GetLowLongword(L_DLONG dl)
{
  return i64_GetLo(dl);                             /* 08.04.99 */ 
}

L_DLONG CreateQuadword(L_ULONG lh, L_ULONG ll)
{
  L_DLONG tmp;
  i64_Init(tmp,ll,lh);                              /* 08.04.99 */ 
  return tmp;
}
#endif

/* 22.08.06 ... */
static void Diff128(L_ULONG* rgNum1, L_ULONG* rgNum2)
{
  L_ULONG  ulTemp1, ulTemp2;
  L_ULONG  ulAdd1 = 0;
  L_ULONG  ul;

  for (ul = 0; ul < 4; ul++)
  {     

    ulTemp1 = rgNum1[ul];
    ulTemp2 = rgNum2[ul];

    if (ulTemp1 > ulTemp2)
    {      
      ulTemp1 =  ulTemp1 - ulTemp2 - ulAdd1;
      ulAdd1 = 0;
    }
    else
    {

      if ( (ulTemp1 == ulTemp2) && (ulAdd1 == 0) )
      {        
        ulTemp1 = 0;
      }
      else
      {
        ulTemp1 = L_MAXULONG - ulTemp2 + ulTemp1 + 1 - ulAdd1;
        ulAdd1 = 1;
      }

    }

    rgNum1[ul] = ulTemp1;    

  }/* endfor */

  return;
}

static L_BYTE  Cmp128(L_ULONG* rgNum1, L_ULONG* rgNum2)
{
  L_LONG i;
  L_BYTE bRet = 2;

  for (i = 3; i >= 0; i--)
  {

     if (rgNum1[i] == rgNum2[i])
     {
       continue;
     }
     else
     {
       bRet = 0;

       if (rgNum1[i] > rgNum2[i])
       {
         bRet = 1;
       }

       break;
     }

  }/* endfor */

  return bRet;
}

L_LONG  ConvDecToSQLNum(L_BYTE* pLinDec, L_BYTE* pNum, L_BYTE Precision, L_BYTE Scale)
{
  L_LONG  lRet = 0;
  L_LONG  i, k;

  L_ULONG  rgSumX[8];
  L_ULONG  rgTemp[8];
  L_ULONG  ulTemp;
  L_ULONG  ulAdd = 0;

  L_BYTE  rgCoeff[16];
  L_BYTE  rgCoeffX[32];
  L_BYTE  rgLinDecTemp[15];
  L_BYTE  bOrder = 1;
  L_BYTE  bNumber;
  L_BYTE  bBit;
  L_BYTE  bShift;

  DECSTATUS status = pLinDec[0];

  if ((Precision > 30) || (Scale > 10))
  {
    lRet = ECONV;
    goto L_EXIT;
  }

  memset(pNum, 0, 19);
  pNum[0] = Precision;
  pNum[1] = Scale;

  switch (status)
  {

    case  DECZERO:
            lRet = 0;
            goto L_EXIT;
            /*break;*/

    case  DECNEGATIV:
            break;

    case  DECPOSITIV:
            pNum[2] = 1;
            break;

    default:            
            lRet = ECONV;
            goto L_EXIT;

  }

  memset(rgSumX, 0, 32);
  memset(rgLinDecTemp, 0, 15);
  memcpy(rgLinDecTemp, &pLinDec[1 + 5 - (Scale+1)/2], 10 + (Scale+1)/2);

  if (Scale & 1)
  {
    rgLinDecTemp[0] = rgLinDecTemp[0]/10;
  }

  for (i = 14; i >= 0; i--, bOrder++)
  {

    memset(rgTemp,   0, 32);
    memset(rgCoeffX, 0, 32);
    memset(rgCoeff,  0, 16);

    if (Scale & 1)
    {
      if (i == 0)
      {
        rgSumX[0] += rgLinDecTemp[i];
        break;
      }

      memcpy(rgCoeff, &CFTS[A_OFFSET[2*bOrder+1][0]], A_OFFSET[2*bOrder+1][1]);
    }
    else
    {
      memcpy(rgCoeff, &CFTS[A_OFFSET[2*bOrder][0]], A_OFFSET[2*bOrder][1]);
    }

    for (k = 0; k < 8; k++)
    {
      memcpy( &rgCoeffX[4*k], &rgCoeff[2*k], 2);
    }

    bNumber = rgLinDecTemp[i];

    bBit = 1;
    bShift = 0;
    while (bBit > 0)
    {
          
      if (bBit & bNumber)
      {

        memcpy(rgTemp, rgCoeffX, 32);

        for (k = 0; k < 8; k++)
        {
          rgTemp[k] <<= bShift;
        }
      
        /* adding the number to resulting sum */
        for (k = 0; k < 8; k++)
        {
          rgSumX[k] += rgTemp[k];
        }

      }
          
      bShift++;
      bBit <<= 1;

    }/* endwhile */
        
  }/* endfor */

  for (i = 0; i < 8; i++)
  {
    ulTemp = rgSumX[i] + ulAdd;
    ulAdd = rgSumX[i] >> 16;
    memcpy(&pNum[2*i+3], &ulTemp, 2);
  }

L_EXIT:

  return lRet;
}

L_LONG  ConvSQLNumToDec(L_BYTE* pbNum, L_BYTE* pbLinDec, L_BYTE Precision, L_BYTE Scale)
{
  L_ULONG  rgSample[4]   = {0, 0, 0, 0};
  L_ULONG  rgCheckNum[4] = {0, 0, 0, 0};

  L_LONG   i, k;
  L_LONG   lRet = 0;

  L_BOOL   fOdd = L_TTRUE;

  L_BYTE   bFlag;
  L_BYTE   bNumPrecision = pbNum[0];
  L_BYTE   bNumScale = pbNum[1];
  L_BYTE   rgLinDecTemp[16];                              /* 09.10.07 */
  L_BYTE   bTemp = Precision - Scale;

  if ((Precision > 30) || (Scale > 10))
  {
    lRet = ECONV;
    goto L_EXIT;
  }

  memset(pbLinDec, 0, 16);
  memset(rgLinDecTemp, 0, 16);

  memcpy(rgCheckNum, &pbNum[3], 16);
  bFlag = Cmp128(rgCheckNum, rgSample);
  if (bFlag == 2)
  {

    pbLinDec[0] = DECZERO;
    lRet = 0;
    goto L_EXIT;

  }
  else
  {

    if (pbNum[2] == 1)
    {
      pbLinDec[0] = DECPOSITIV;
    }
    else
    if (pbNum[2] == 0)
    {
      pbLinDec[0] = DECNEGATIV;
    }
    else
    {
      lRet = ECONV;
      goto L_EXIT;
    }

  }

  if ( 
      (bNumPrecision > 30) ||
      (bNumScale > bNumPrecision)
     )
  {
    lRet = ECONV;
    goto L_EXIT;
  }

  memcpy(rgSample, &CFTS[A_OFFSET[0][0]], A_OFFSET[0][1]);
  bFlag = Cmp128(rgCheckNum, rgSample);
  if (bFlag)
  {
    /* error, too large number */
    lRet = EOVER;
    goto L_EXIT;
  }

  for (i = 1; i < 31; i++)
  {

    memset(rgSample, 0, 16);
    k = 0;
    memcpy(rgSample, &CFTS[A_OFFSET[i][0]], A_OFFSET[i][1]);

    bFlag = Cmp128(rgCheckNum, rgSample);
    if (bFlag > 0)
    {

      if (bFlag == 2)
      {
        k = 1;
        Diff128(rgCheckNum, rgSample);        
      }
      else
      {

        for (k = 1; k <= 9; k++)
        {
          Diff128(rgCheckNum, rgSample);

          if (Cmp128(rgCheckNum, rgSample) == L_TFALSE)
          {             
            break;          
          }        
        }/* endfor */

      }

    }/* endif (bFlag > 0) */

    if (bNumScale & 1)
    {
      
      if (fOdd)
      {
        rgLinDecTemp[15-(i-1)/2] += (L_BYTE)k;
      }
      else
      {
        rgLinDecTemp[15-(i+1)/2] = (L_BYTE)(k*10);
      }
      
    }
    else
    {

      if (fOdd)
      {
        rgLinDecTemp[15-(i-1)/2] = (L_BYTE)(k*10);
      }
      else
      {
        rgLinDecTemp[15-(i-1)/2] += (L_BYTE)k;
      }
      
    }

    fOdd = !fOdd;
    
  }/* endfor */

  if (bNumScale > 10)
  {
    memcpy(&pbLinDec[1], &rgLinDecTemp[(bNumScale+1)/2 - 5], 15 - (bNumScale+1)/2);
  }
  else
  {
    memcpy(&pbLinDec[1 + 5 - (bNumScale+1)/2], &rgLinDecTemp[0], 10 + (bNumScale+1)/2);
  }

  if (bTemp & 1)
  {
    if (pbLinDec[6 + bTemp/2] > 9)
    {
      lRet = ECONV;
      goto L_EXIT;
    }
  }

  for (i = 15; i > (5 + (bTemp + 1)/2); i--)
  {

    if (pbLinDec[i] > 0)
    {
      lRet = ECONV;
      break;
      /* goto L_EXIT; */
    }

  }/* endfor */

L_EXIT:

  return lRet;
}                                                            /* ... 22.08.06 */

/* Function convert string pStr to a double-precision value *pdVal; 
   function returns L_TTRUE if conversion is OK 
   function returns L_TFALSE if pStr can't be converted to L_DOUBLE 
   or if pStr contains invalid symbols */
L_BOOL ConvStringToDouble(L_CHAR   *pStr,           /* 25.01.09 13.12.07 ... */
                          L_DOUBLE *pdVal)
{
  return (StringToDouble(pStr, pdVal) == ISSUC);           /* 26.09.13 */
}                                                          /* ... 13.12.07 */
