 /***************************************************************************
 *                        RELEX, Inc. DBMS Linter                           *
 *  This is unpublished proprietary source code of RELEX, Inc.              *
 *  The copyright notice does not evidence any actual or intended           *
 *  publication of such source code.                                        *
 *                                                                          *
 *                  Copyright (c) 1990-2009 RELEX Inc.                      *
 *                                                                          *
 *                   This software was developed by                         *
 *                                                                          *
 *                            RELEX, Inc.                                   *
 *                         Voronezh, Russia                                 *
 *                                                                          *
 *                        All rights reserved.                              *
 ****************************************************************************
 AUTHOR:
 ----------------------------------------------------------------------------
 PURPOSE: Conversions : String  --> Number
----------------------------------------------------------------------------
 21.04.92                                 
 11.01.93 -- Ustinov                       
 31.07.97 -- Ermakov (Copyright) 
 28.11.97 -- Pasec (types)
 05.04.01 -- Ermakov (Macro REAL replaced by L_REAL)
 18.10.01 -- Galizin (Change native types to macros)
 22.08.06 -- Pasechnik (SQL Numeric type convertion)
 25.01.09 -- Ermakov (moved 13.12.07 by KOV from decimals. Convert string to double with invalid symbol checking)
*/
#ifndef _CONVERS_H_
#define _CONVERS_H_

#include "definit.h"
#include "typedef.h"                                /* 28.11.97 */

#ifdef  __cplusplus
extern "C" {
#endif

/* --- Error Code --- */
#define ISSUC     0    /* O.K. (No Changes !) */
#define ENONUM    1    /* No Number           */
#define EBADSYM   2    /* Bad Symbol          */
#define EOVER     3    /* Overflow            */
#define ECONV     4    /* Error of Conversion */
#define ESYNT     5    /* Error of Syntax     */
#define ELOSTV    6    /* Lost Value          */


/* 28.11.97 */
L_LONG    ConvLong(L_CHAR *, L_SWORD *); /* char *s; short *err; */ /* 18.10.01 */
/*
 * 'long' from string 's';
 * syntax of string:
 *   ['space'][+|-]'digits'['space'] ;
 * 'err' = error code: ISSUC,ENOTNUM,EBADSYM,EOVER,ECONV
 */

L_SWORD ConvInteger(L_CHAR *, L_SWORD *); /* char *s; short *err; */ /* 18.10.01 */ 
/*
 * 'integer' from string 's';
 * syntax of string:
 *   ['space'][+|-]'digits'['space'] ;
 * 'err' = error code: ISSUC,ENOTNUM,EBADSYM,EOVER,ECONV
 */

L_WORD    ConvWord(L_CHAR *, L_SWORD *); /* char *s; short *err; */ /* 18.10.01 */
/*
 * 'word' from string 's';
 * syntax of string:
 *   ['space'][+|-]'digits'['space'] ;
 * 'err' = error code: ISSUC,ENOTNUM,EBADSYM,EOVER,ECONV
 */

L_DOUBLE  ConvDouble(L_CHAR *, L_SWORD *); /* char *s; short *err; */ /* 18.10.01 */
/*
 * 'double' from string 's';
 * syntax of string:
 *   ['space'][+|-]'digits'[.'digits'][e|E[+|-]'digits']['space'] ;
 * 'err' = error code: ISSUC,ENOTNUM,EBADSYM,EOVER,ECONV,ESYNT,ELOSTV
 */

L_REAL    ConvReal(L_CHAR *, L_SWORD *); /* char *s; short *err; */ /* 05.04.01 */
/*
 * 'real' from string 's';
 * syntax of string:
 *   ['space'][+|-]'digits'[.'digits'][e|E[+|-]'digits']['space'] ;
 * 'err' = error code: ISSUC,ENOTNUM,EBADSYM,EOVER,ECONV,ESYNT,ELOSTV
 */

L_WORD    LWord(L_LONG, L_SWORD *); /* long l; short *err;  */ /* 18.10.01 */
/*
 * 'word' from long integer 'l';
 * 'err' = error code: ISSUC,EOVER
 */

L_SWORD   LInteger(L_LONG, L_SWORD *); /* long l; short *err;  */ /* 18.10.01 */
/*
 * 'integer' from long integer 'l';
 * 'err' = error code: ISSUC,EOVER
 */

L_REAL    DReal(L_DOUBLE, L_SWORD *); /* double d; short *err; */       /* 18.10.01 */ /* 05.04.01 */
/*
 * 'real' from double 'd';
 * 'err' = error code: ISSUC,EOVER,ELOSTV
 */
#ifdef __64BIT_KERNEL__
L_ULONG GetHighLongword (L_DLONG);
L_ULONG GetLowLongword  (L_DLONG);
L_DLONG CreateQuadword  (L_ULONG, L_ULONG);
L_DLONG ConvDLong       (L_CHAR *, L_SWORD *);                  /* 18.10.01 */
void    PrintDLong      (L_DLONG, L_CHAR *, L_SWORD);           /* 18.10.01 */
#endif

L_LONG  ConvDecToSQLNum(L_BYTE* , L_BYTE*, L_BYTE, L_BYTE);   /* 22.08.06 */
L_LONG  ConvSQLNumToDec(L_BYTE* , L_BYTE*, L_BYTE, L_BYTE);   /* 22.08.06 */
L_BOOL  ConvStringToDouble(L_CHAR   *pStr, L_DOUBLE *pdVal);  /* 25.01.09 */

#ifdef __cplusplus
}
#endif

/*#endif * of #ifndef _convers_ *    11.01.93 */
#endif
