/***************************************************************************
 *                    RELEX, Inc. DBCS source code                          *
 *  This is unpublished proprietary source code of RELEX, Inc.              *
 *  The copyright notice does not evidence any actual or intended           *
 *  publication of such source code.                                        *
 *                                                                          *
 *                Copyright (c) 1996-2012 RELEX Inc.                        *
 *                                                                          *
 *                  This software was developed by                          *
 *                                                                          *
 *                            RELEX, Inc.                                   *
 *                         Voronezh, Russia                                 *
 *                                                                          *
 *                        All rights reserved.                              *
 ****************************************************************************
 AUTHOR: Alexey G. Grachyov (RooK)
 ----------------------------------------------------------------------------
 PURPOSE: DBC API header
 ----------------------------------------------------------------------------
 REVISION HISTORY:  21.01.99 -- Kent (Use Relex's standard modifiers DLLAPI)
                    25.01.99 -- Kent (Return APIENTRY for functions)
                    03.03.99 -- RooK (timeout added)
                    01.11.00 -- Konon (WIN16)
                    25.04.01 -- Galizin ( DWORD -> L_LONG)
                    13.09.01 -- Kent (Add 'DBCEditNodetab')
                    29.11.01 -- Galizin ( Change type WORD to L_WORD, BYTE to L_BYTE)
                    26.03.02 -- Galizin ( Syncronize type of LPSTR from Win 32 )
                    23.09.02 -- Kononyhin (add/remove node to/from nodetab without dialog - WIN32)
                    03.03.03 -- Kent,EVG (Windows CE)
                    14.03.03 -- EVG (Server description)
                    20.03.03 -- EVG (MBSC interface)
                    20.03.03A - EVG (WORD -> L_WORD, BYTE -> L_BYTE, DWORD -> L_LONG)
                    26.03.03 -- EVG (small bug fixed)
                    10.04.03 -- EVG (nodetab direct editing: PT #2551)   
                    14.04.03 -- EVG (UNIX build)
                    15.04.03 -- EVG (L_LONG -> L_ULONG)
                    28.04.03 -- EVG (Porting to Unix, UNICODE/MBCS full support)
                    28.04.03a -- EVG (Exporting SRC_XXX constants)
                    13.05.03 -- EVG (TCPIPS added)
                    28.05.03 -- EVG (default nodetab file support)
                    05.06.03 -- EVG (avoid duplicate server names in nodetab)
                    19.06.03 -- Galizin ( VxWorks build )
                    29.08.03 -- Ermakov (Removed APIENTRY (DLLAPI cantain __stdcall))
                    04.09.03 -- Kononyhin (PT#3567 - defines for global variables)
                    19.08.08 -- Ermakov (corrected proto)
                    01.07.11 -- Ganin (DBC_LASTLINK, DBC_NO_REINIT, DBC_LUSED error ids)
                    13.07.11 -- Galizin (Api connect - disconnect commands)
                    10.10.11 -- Ganin (Support for UNICODE and new drivers)
                    10.08.12 -- Ganin (Corrected return value for DBCGetDefaultServerIndex)
 ----------------------------------------------------------------------------
*/
#ifndef DBCAPI_HEADER_FILE                                      /* 10.10.11 ... */
#define DBCAPI_HEADER_FILE

#if !defined(WINDOWS_NT) && !defined(INTER_MSWINDOWS)
#include "lintypes.h"                                           /* 10.10.11 */
typedef struct _SYSTEMTIME 
{
    L_WORD wYear;
    L_WORD wMonth;
    L_WORD wDayOfWeek;
    L_WORD wDay;
    L_WORD wHour;
    L_WORD wMinute;
    L_WORD wSecond;
    L_WORD wMilliseconds;
} SYSTEMTIME, *LPSYSTEMTIME;
#else
#  include <windows.h>
#endif                                                          /* ... 10.10.11 */

#include "dlldefs.h"
#include "lintypes.h"                                           /* 25.04.01 */
#include "dbcsacom.h"                                           /* 10.10.11 */

#if defined(INTER_MSDOS) || (defined(WIN3XX) && !defined(WIN32S)) /* 01.11.00 */
#   ifndef FALSE
#      define FALSE 0
#      define TRUE  1
#   endif
#   ifndef BYTE
#      define BYTE unsigned char
#   endif
#   ifndef WORD
#      define WORD unsigned short
#   endif
#endif                                                    

#if defined(UNIX) || defined(QNX) || defined(VXWORKS)           /* 19.06.03 */ /* 19.09.98*/
#   define PBYTE L_BYTE *
#   define LPBYTE L_BYTE *
#   define TRUE L_TTRUE
#   define FALSE L_TFALSE                                       /* ... 28.04.03 */
#endif
                                                                /* 10.04.03 ... */
#define DBC_SUCCESS       0x00000000L      /* */
#define DBC_ERROR         0x00000001L      /* */
#define DBC_NOMEM         0x00000002L      /* not enough virtual memory */
#define DBC_LOCKED        0x00000004L      /* nodetab is locked by another client */
#define DBC_BUFFSIZE      0x00000008L      /* insufficient output bufer size */
#define DBC_NOSERVER      0x00000010L      /* no such a server */
#define DBC_USED          0x00000020L      /* server is used */
#define DBC_EXISTS        0x00000040L      /* server already exists */
#define DBC_FOPEN         0x00000080L      /* could not open nodetab file */
#define DBC_FCLOSE        0x00000100L      /* could not close nodetab file */
#define DBC_FREAD         0x00000200L      /* could not read the nodetab file */
#define DBC_FWRITE        0x00000400L      /* couldnot write to the nodetab file */
#define DBC_ASYNC         0x00000800L      /* nodetab contents doesn't match the client's node list */
#define DBC_NOINNODETAB   0x00001000L      /* no such a node in nodetab file */
#define DBC_NOINCLIENT    0x00002000L      /* no such a node in the current nodetab client node list */
#define DBC_NOCLIENT      0x00004000L      /* no network client found */  /* 28.05.03 */
#define DBC_NOINREG       0x00008000L      /* error reading system registry (Win32 only) */ /* 28.05.03 */
#define DBC_INNODETAB     0x00010000L      /* there is already a server with such a name in nodetab */  /* 05.06.03 */
#define DBC_INCLIENT      0x00020000L      /* server exists already in client node list */              /* 05.06.03 */
#define DBC_LASTLINK      0x00040000L      /* can't remove the last link */                             /* 01.07.11 ... */
#define DBC_NO_REINIT     0x00080000L      /* can't reinitialize the link, i.e. protocol changes */
#define DBC_LUSED         0x00100000L      /* can't remove link used in logical node */                 /* ... 01.07.11 */
#define DBC_PROGRESS      0x00200000L      /* requested operation is pending */ /* 10.10.11 */

#define DBC_FILEERROR     DBC_FOPEN | DBC_FCLOSE | DBC_FREAD | DBC_FWRITE
#define DBC_SYNCERROR     DBC_ASYNC | DBC_NOINNODETAB | DBC_NOINCLIENT

#define DBC_MATCH_NAME    0x00000001L      /* names don't match */    /* 28.04.03 ... */
#define DBC_MATCH_TYPE    0x00000002L      /* protocol types don't match */
#define DBC_MATCH_ADDR    0x00000004L      /* addresses don't match */
#define DBC_MATCH_PORT    0x00000008L      /* ports don't match */
#define DBC_MATCH_CLOSE   0x00000010L      /* close timeouts don't match */
#define DBC_MATCH_ALIVE   0x00000020L      /* alive timeouts don't match */
#define DBC_MATCH_RETRY   0x00000040L      /* retry timeouts don't match */



#define LINK_DISCONNECTED        ((L_CHAR)0) /* State is disconnected */  /* 10.10.11 */
#define LINK_CONNECT_IN_PROGRESS ((L_CHAR)-1)/* In connect time */        /* 10.10.11 */
#define LINK_CONNECTED           ((L_CHAR)1) /* Link is connected */      /* 10.10.11 */



/* Old codes for compat. */
/* TODO: consider removal, not used anywhere in sources */
#define OLD_IS_LOCAL_LINTER_LOADED        1                     /* 10.10.11 ... */
#define OLD_IS_DBC_LOADED                 2
#define OLD_IS_TCP_LOADED                 3
#define OLD_IS_SPX_LOADED                 4
#define OLD_GET_SERVERS_COUNT             5
#define OLD_GET_SERVER_NAME               6
#define OLD_GET_DEFAULT_SERVER_NAME       7
#define OLD_GET_DEFAULT_SERVER_INDEX      8
#define OLD_SET_DEFAULT_SERVER            9


#define API_ISTCPLOADED                   100
#define API_ISSPXLOADED                   101  
#define API_GETSRVCOUNT                   102  
#define API_GETDEFSRVNAME                 103  
#define API_GETDEFSRVINDEX                104
#define API_GETSRVNAME                    105  
#define API_SETDEFSRV                     106  
#define API_SETDEFSRVNAME                 107  
#define API_ISNBLOADED                    108  
#define API_ISNPLOADED                    109  
#define API_ISDECLOADED                   110
#define API_ADDSERVER                     111
#define API_DELSERVER                     112
#define API_EDITSERVER                    113

#define API_GETNODETABPATH                116
#define API_LOCKNODETAB                   117
#define API_UNLOCKNODETAB                 118
#define API_ADDNODE                       119
#define API_GETNODEBYNAME                 120
#define API_GETNODEBYINDEX                121
#define API_GETDEFAULTNODE                122
#define API_DELNODEBYNAME                 123
#define API_DELNODEBYINDEX                124
#define API_DELDEFAULTNODE                125
#define API_SETNODEBYNAME                 126
#define API_SETNODEBYINDEX                127
#define API_SETDEFAULTNODE                128
#define API_NODETABSTATUS                 129

#define API_CONNECTNODE                   130
#define API_DISCONNECTNODE                131
#define API_GETNODECONNECTSTATUS          132

#define API_ADDNODE_STR                   140
#define API_GETNODE_BYNAME_STR            141
#define API_GETNODE_BYINDEX_STR           142
#define API_GETNODE_DEFAULT_STR           143
#define API_SETNODE_BYNAME_STR            144
#define API_SETNODE_BYINDEX_STR           145
#define API_SETNODE_DEFAULT_STR           146

#define API_LATEST_SUPPORTED_CMD          170                   /* ... 10.10.11 */


#define CS_MBCS           0x00000001L      /* MBCS client application */
#define CS_UNICODE        0x00000002L      /* UNICODE client application */

#define MAX_NODE_NAME     8 
#define MAX_NODE          300                                   /* ... 28.04.03 */

#ifndef MAX_PATH                                                /* 10.10.11 ... */
#  define MAX_PATH        260
#endif

#ifndef _MAX_PATH
#  define _MAX_PATH       MAX_PATH
#endif                                                          /* ... 10.10.11 */

/* defines for setting global variables from outside */
#define	GLOB_VAR_ENABLEPING	1                                   /* 04.09.03 */

#ifdef UNICODE
# define CLIENT_CS CS_UNICODE
#else
# define CLIENT_CS CS_MBCS
#endif

#ifdef __cplusplus                                              /* 10.10.11 ... */
extern "C" {
#endif                                                          /* ... 10.10.11 */

/*
   The following structures and related functions are deprecated
   If at all possible avoid using them in new code
   However they are required for DBCAPI with the old driver
   New driver supports them only for compatibility reasons
   The support is not full though
*/

typedef struct CONNSTAT_tag
{
   SYSTEMTIME tstOpened;
   SYSTEMTIME tstClosed;
   L_ULONG    dwCode;
} CONNSTAT, *PCONNSTAT;

typedef struct CLNTSTAT_tag
{
   L_ULONG   dwQueryes;
   L_ULONG   dwAnswers;
   L_ULONG   dwUnkAnswers;
   L_ULONG   dwSendBytes;
   L_ULONG   dwRecvBytes;
   L_ULONG   dwSendErrors;
   L_ULONG   dwRecvErrors;
   L_ULONG   dwSendTicks;
   L_ULONG   dwRecvTicks;
} CLNTSTAT, *PCLNTSTAT;

typedef struct SVRDESCA_tag
{
  L_ULONG            m_dwSize;
  L_ULONG            m_dwSync;
  L_ULONG            m_dwType;
  char               m_szName[12];                              /* 10.10.11 ... */
  char               m_szAddr[104];
  char               m_szPort[8];                               /* ... 10.10.11 */
  L_ULONG            m_lPort;
  L_WORD             m_wCloseTimeout;
  L_WORD             m_wAliveTimeout;
  L_WORD             m_wRetryTimeout;
  L_ULONG            m_dwConnections;                    
  L_ULONG            m_dwLastRecv;
  time_t             m_ttLastConnect;
  CLNTSTAT           m_sstStat;
  CONNSTAT           m_ConnStat;
} SVRDESCA, *PSVRDESCA;

#ifdef UNICODE                                                  /* 10.10.11 ... */
typedef struct SVRDESCW_tag
{
  L_ULONG            m_dwSize;
  L_ULONG            m_dwSync;
  L_ULONG            m_dwType;
  wchar_t            m_szName[12];
  wchar_t            m_szAddr[104];
  wchar_t            m_szPort[8];
  L_ULONG            m_lPort;
  L_WORD             m_wCloseTimeout;
  L_WORD             m_wAliveTimeout;
  L_WORD             m_wRetryTimeout;
  L_ULONG            m_dwConnections;
  L_ULONG            m_dwLastRecv;
  time_t             m_ttLastConnect;
  CLNTSTAT           m_sstStat;
  CONNSTAT           m_ConnStat;
} SVRDESCW, *PSVRDESCW;
#  define SVRDESC_tag   SVRDESCW_tag
#  define SVRDESC       SVRDESCW
#  define PSVRDESC      PSVRDESCW
#else
#  define SVRDESC_tag   SVRDESCA_tag
#  define SVRDESC       SVRDESCA
#  define PSVRDESC      PSVRDESCA
#endif /* UNICODE */


L_BOOL  DLLAPI  DBCIsLocalLinterLoaded(void);
L_BOOL  DLLAPI  DBCIsLoaded(void);                              
#ifdef WIN32
L_BOOL  DLLAPI  DBCIsLocalLinterVersionLoaded(L_WORD version);
#endif

L_BOOL  DLLAPI  DBCIsTCPLoaded(void);
L_BOOL  DLLAPI  DBCIsSPXLoaded(void);
L_BOOL  DLLAPI  DBCIsNBLoaded(void);
L_BOOL  DLLAPI  DBCIsNPLoaded(void);
L_BOOL  DLLAPI  DBCIsDECLoaded(void);

L_WORD  DLLAPI  DBCGetServersCount(void);
L_BOOL  DLLAPI  DBCGetServerNameA(L_WORD index, char *buffer, int sz);

L_BOOL  DLLAPI  DBCGetDefaultServerNameA(char *buffer, int sz);
L_SWORD DLLAPI  DBCGetDefaultServerIndex();                     /* 10.08.12 */
L_BOOL  DLLAPI  DBCSetDefaultServer(L_WORD index);

L_ULONG DLLAPI  DBCGetNodetabPathA(char *buffer, int sz);
L_ULONG DLLAPI  DBCLockNodetab();
L_ULONG DLLAPI  DBCUnlockNodetab();
L_ULONG DLLAPI  DBCGetNodetabStatus();

/* deprecated */
L_ULONG DLLAPI  DBCAddNodeA(const PSVRDESCA svr);
/* deprecated */
L_ULONG DLLAPI  DBCGetNodeByNameA(const char *name, PSVRDESCA svr);
/* deprecated */
L_ULONG DLLAPI  DBCGetNodeByIndexA(L_WORD index, PSVRDESCA svr);
/* deprecated */
L_ULONG DLLAPI  DBCGetDefaultNodeA(PSVRDESCA svr);
/* deprecated */
L_ULONG DLLAPI  DBCSetNodeByNameA(const char *name, const PSVRDESCA svr);
/* deprecated */
L_ULONG DLLAPI  DBCSetNodeByIndexA(L_WORD index, const PSVRDESCA svr);
/* deprecated */
L_ULONG DLLAPI  DBCSetDefaultNodeA(const PSVRDESCA svr);

L_ULONG DLLAPI  DBCDelNodeByNameA(const char *name);
L_ULONG DLLAPI  DBCDelNodeByIndex(L_WORD index);
L_ULONG DLLAPI  DBCDelDefaultNode();

L_ULONG DLLAPI  DBCConnectNodeA(const char *name);
L_ULONG DLLAPI  DBCDisconnectNodeA(const char *name);
L_ULONG DLLAPI  DBCGetNodeConnectStatusA(const char *name, L_CHAR *status);

L_ULONG DLLAPI  DBCAddNodeStrA(const char *szStr);
L_ULONG DLLAPI  DBCGetNodeByNameStrA(const char *name, char *szStr, int size);
L_ULONG DLLAPI  DBCGetNodeByIndexStrA(L_WORD index, char *szStr, int size);
L_ULONG DLLAPI  DBCGetNodeDefaultStrA(char *szStr, int size);
L_ULONG DLLAPI  DBCSetNodeByNameStrA(const char *name, const char *szStr);
L_ULONG DLLAPI  DBCSetNodeByIndexStrA(L_WORD index, const char *szStr);
L_ULONG DLLAPI  DBCSetNodeDefaultStrA(const char *szStr);

/*
  Return value: DBC_SUCCESS in case of success, or error code.

  In case of success *command shall be set to
  maximal value of API command supported by currently loaded driver.
*/
L_ULONG DLLAPI  DBCLatestSupportedCmd(L_ULONG *command);

#ifdef UNICODE

L_BOOL  DLLAPI  DBCGetServerNameW(L_WORD index, wchar_t *buffer, int sz);

L_BOOL  DLLAPI  DBCGetDefaultServerNameW(wchar_t *buffer, int sz);

L_ULONG DLLAPI  DBCGetNodetabPathW(wchar_t *buffer, int sz);

/* deprecated */
L_ULONG DLLAPI  DBCAddNodeW(const PSVRDESCW svr);
/* deprecated */
L_ULONG DLLAPI  DBCGetNodeByNameW(const wchar_t *name, PSVRDESCW svr);
/* deprecated */
L_ULONG DLLAPI  DBCGetNodeByIndexW(L_WORD index, PSVRDESCW svr);
/* deprecated */
L_ULONG DLLAPI  DBCGetDefaultNodeW(PSVRDESCW svr);
/* deprecated */
L_ULONG DLLAPI  DBCSetNodeByNameW(const wchar_t *name, const PSVRDESCW svr);
/* deprecated */
L_ULONG DLLAPI  DBCSetNodeByIndexW(L_WORD index, const PSVRDESCW svr);
/* deprecated */
L_ULONG DLLAPI  DBCSetDefaultNodeW(const PSVRDESCW svr);

L_ULONG DLLAPI  DBCDelNodeByNameW(const wchar_t *name);

L_ULONG DLLAPI  DBCConnectNodeW(const wchar_t *name);
L_ULONG DLLAPI  DBCDisconnectNodeW(const wchar_t *name);
L_ULONG DLLAPI  DBCGetNodeConnectStatusW(const wchar_t *name, L_CHAR *status);

L_ULONG DLLAPI  DBCAddNodeStrW(const wchar_t *szStr);
L_ULONG DLLAPI  DBCGetNodeByNameStrW(const wchar_t *name, wchar_t *szStr, int size);
L_ULONG DLLAPI  DBCGetNodeByIndexStrW(L_WORD index, wchar_t *szStr, int size);
L_ULONG DLLAPI  DBCGetNodeDefaultStrW(wchar_t *szStr, int size);
L_ULONG DLLAPI  DBCSetNodeByNameStrW(const wchar_t *name, const wchar_t *szStr);
L_ULONG DLLAPI  DBCSetNodeByIndexStrW(L_WORD index, const wchar_t *szStr);
L_ULONG DLLAPI  DBCSetNodeDefaultStrW(const wchar_t *szStr);

#  define DBCGetServerName              DBCGetServerNameW
#  define DBCGetDefaultServerName       DBCGetDefaultServerNameW
#  define DBCGetNodetabPath             DBCGetNodetabPathW
#  define DBCAddNode                    DBCAddNodeW
#  define DBCDelNodeByName              DBCDelNodeByNameW
#  define DBCGetNodeByName              DBCGetNodeByNameW
#  define DBCGetNodeByIndex             DBCGetNodeByIndexW
#  define DBCGetDefaultNode             DBCGetDefaultNodeW
#  define DBCSetNodeByName              DBCSetNodeByNameW
#  define DBCSetNodeByIndex             DBCSetNodeByIndexW
#  define DBCSetDefaultNode             DBCSetDefaultNodeW
#  define DBCConnectNode                DBCConnectNodeW
#  define DBCDisconnectNode             DBCDisconnectNodeW
#  define DBCGetNodeConnectStatus       DBCGetNodeConnectStatusW
#  define DBCAddNodeStr                 DBCAddNodeStrW
#  define DBCGetNodeByNameStr           DBCGetNodeByNameStrW
#  define DBCGetNodeByIndexStr          DBCGetNodeByIndexStrW
#  define DBCGetNodeDefaultStr          DBCGetNodeDefaultStrW
#  define DBCSetNodeByNameStr           DBCSetNodeByNameStrW
#  define DBCSetNodeByIndexStr          DBCSetNodeByIndexStrW
#  define DBCSetNodeDefaultStr          DBCSetNodeDefaultStrW

#else

#  define DBCGetServerName              DBCGetServerNameA
#  define DBCGetDefaultServerName       DBCGetDefaultServerNameA
#  define DBCGetNodetabPath             DBCGetNodetabPathA
#  define DBCAddNode                    DBCAddNodeA
#  define DBCDelNodeByName              DBCDelNodeByNameA
#  define DBCGetNodeByName              DBCGetNodeByNameA
#  define DBCGetNodeByIndex             DBCGetNodeByIndexA
#  define DBCGetDefaultNode             DBCGetDefaultNodeA
#  define DBCSetNodeByName              DBCSetNodeByNameA
#  define DBCSetNodeByIndex             DBCSetNodeByIndexA
#  define DBCSetDefaultNode             DBCSetDefaultNodeA
#  define DBCConnectNode                DBCConnectNodeA
#  define DBCDisconnectNode             DBCDisconnectNodeA
#  define DBCGetNodeConnectStatus       DBCGetNodeConnectStatusA
#  define DBCAddNodeStr                 DBCAddNodeStrA
#  define DBCGetNodeByNameStr           DBCGetNodeByNameStrA
#  define DBCGetNodeByIndexStr          DBCGetNodeByIndexStrA
#  define DBCGetNodeDefaultStr          DBCGetNodeDefaultStrA
#  define DBCSetNodeByNameStr           DBCSetNodeByNameStrA
#  define DBCSetNodeByIndexStr          DBCSetNodeByIndexStrA
#  define DBCSetNodeDefaultStr          DBCSetNodeDefaultStrA


#endif /* UNICODE */

/*
   The following functions are deprecated
   Avoid using them in new code
   They kept here for compatibility reasons only
*/

L_ULONG   DLLAPI  DBCOpenNodetab(L_BOOL bSync);
L_ULONG   DLLAPI  DBCSaveNodetab();
L_ULONG   DLLAPI  DBCCloseNodetab();

L_WORD    DLLAPI  DBCGetNodetabNodesCount();

L_ULONG   DLLAPI  DBCAddNodetabClientNodeA(PSVRDESCA pSvrDesc);

L_ULONG   DLLAPI  DBCGetNodetabClientNodeByNameA(const char *szName,
                                                 PSVRDESCA pSvrDesc,
                                                 L_BOOL bClient);
L_ULONG   DLLAPI  DBCGetNodetabClientNodeByIndexA(L_WORD index,
                                                  PSVRDESCA pSvrDesc,
                                                  L_BOOL bClient);
L_ULONG   DLLAPI  DBCGetNodetabClientDefaultNodeA(PSVRDESCA pSvrDesc,
                                                  L_BOOL bClient);

L_ULONG   DLLAPI  DBCSetNodetabClientNodeByNameA(const char *szName,
                                                 PSVRDESCA pSvrDesc);
L_ULONG   DLLAPI  DBCSetNodetabClientNodeByIndexA(L_WORD index,
                                                  PSVRDESCA pSvrDesc);
L_ULONG   DLLAPI  DBCSetNodetabClientDefaultNodeA(PSVRDESCA pSvrDesc);

L_ULONG   DLLAPI  DBCDelNodetabClientNodeByNameA(const char *szNode);
L_ULONG   DLLAPI  DBCDelNodetabClientNodeByIndex(L_WORD wIndex);
L_ULONG   DLLAPI  DBCDelNodetabClientDefaultNode();

char     *DLLAPI  DBCCreateNodeLineA(PSVRDESCA pSvrDesc, char *szNodeStr);
L_ULONG   DLLAPI  DBCParseNodeLineA(PSVRDESCA pSvrDesc, const char *szNodeStr);

#ifdef UNICODE
L_ULONG   DLLAPI  DBCAddNodetabClientNodeW(PSVRDESCW pSvrDesc);

L_ULONG   DLLAPI  DBCGetNodetabClientNodeByNameW(const wchar_t *szName,
                                                 PSVRDESCW pSvrDesc,
                                                 L_BOOL bClient);
L_ULONG   DLLAPI  DBCGetNodetabClientNodeByIndexW(L_WORD index,
                                                  PSVRDESCW pSvrDesc,
                                                  L_BOOL bClient);
L_ULONG   DLLAPI  DBCGetNodetabClientDefaultNodeW(PSVRDESCW pSvrDesc,
                                                  L_BOOL bClient);

L_ULONG   DLLAPI  DBCSetNodetabClientNodeByNameW(const wchar_t *szName,
                                                 PSVRDESCW pSvrDesc);
L_ULONG   DLLAPI  DBCSetNodetabClientNodeByIndexW(L_WORD index,
                                                  PSVRDESCW pSvrDesc);
L_ULONG   DLLAPI  DBCSetNodetabClientDefaultNodeW(PSVRDESCW pSvrDesc);

L_ULONG   DLLAPI  DBCDelNodetabClientNodeByNameW(const wchar_t *szNode);

wchar_t  *DLLAPI  DBCCreateNodeLineW(PSVRDESCW pSvrDesc, wchar_t *szNodeStr);
L_ULONG   DLLAPI  DBCParseNodeLineW(PSVRDESCW pSvrDesc, const wchar_t *szNodeStr);


#define   DBCAddNodetabClientNode           DBCAddNodetabClientNodeW
#define   DBCGetNodetabClientNodeByName     DBCGetNodetabClientNodeByNameW
#define   DBCGetNodetabClientNodeByIndex    DBCGetNodetabClientNodeByIndexW
#define   DBCGetNodetabClientDefaultNode    DBCGetNodetabClientDefaultNodeW
#define   DBCSetNodetabClientNodeByName     DBCSetNodetabClientNodeByNameW
#define   DBCSetNodetabClientNodeByIndex    DBCSetNodetabClientNodeByIndexW
#define   DBCSetNodetabClientDefaultNode    DBCSetNodetabClientDefaultNodeW
#define   DBCDelNodetabClientNodeByName     DBCDelNodetabClientNodeByNameW
#define   DBCCreateNodeLine                 DBCCreateNodeLineW
#define   DBCParseNodeLine                  DBCParseNodeLineW

#else

#define   DBCAddNodetabClientNode           DBCAddNodetabClientNodeA
#define   DBCGetNodetabClientNodeByName     DBCGetNodetabClientNodeByNameA
#define   DBCGetNodetabClientNodeByIndex    DBCGetNodetabClientNodeByIndexA
#define   DBCGetNodetabClientDefaultNode    DBCGetNodetabClientDefaultNodeA
#define   DBCSetNodetabClientNodeByName     DBCSetNodetabClientNodeByNameA
#define   DBCSetNodetabClientNodeByIndex    DBCSetNodetabClientNodeByIndexA
#define   DBCSetNodetabClientDefaultNode    DBCSetNodetabClientDefaultNodeA
#define   DBCDelNodetabClientNodeByName     DBCDelNodetabClientNodeByNameA
#define   DBCCreateNodeLine                 DBCCreateNodeLineA
#define   DBCParseNodeLine                  DBCParseNodeLineA

#endif

#ifdef WIN32
L_BOOL    DLLAPI DBCGetLocalAddresses(L_BYTE btProt,
                                      L_BYTE *pbtAddr,
                                      L_LONG *pdwAddrSize,
                                      L_LONG *pdwFlags);
L_BOOL    DLLAPI DBCEditNodetabA(const char *szNodetabName,
                                 HWND hwndParent);
L_BOOL    DLLAPI DBCEditNodetabExA(const char *szNodetabName,
                                   HWND hwndParent);

L_BOOL    DLLAPI DBCAddNodeToNodetabA(const char *szNodetabName,
                                      const char *szNewNode);
L_BOOL    DLLAPI DBCDelNodeFromNodetabA(const char *szNodetab,
                                        const char *szNode);

#  ifdef UNICODE
L_BOOL    DLLAPI DBCEditNodetabW(const wchar_t *szNodetabName,
                                 HWND hwndParent);
L_BOOL    DLLAPI DBCEditNodetabExW(const wchar_t *szNodetabName,
                                   HWND hwndParent);
L_BOOL    DLLAPI DBCAddNodeToNodetabW(const wchar_t *szNodetabName,
                                      const wchar_t *szNewNode);
L_BOOL    DLLAPI DBCDelNodeFromNodetabW(const wchar_t *szNodetab,
                                        const wchar_t *szNode);

#define   DBCEditNodetab          DBCEditNodetabW
#define   DBCEditNodetabEx        DBCEditNodetabExW
#define   DBCAddNodeToNodetab     DBCAddNodeToNodetabW
#define   DBCDelNodeFromNodetab   DBCDelNodeFromNodetabW
#else
#define   DBCEditNodetab          DBCEditNodetabA
#define   DBCEditNodetabEx        DBCEditNodetabExA
#define   DBCAddNodeToNodetab     DBCAddNodeToNodetabA
#define   DBCDelNodeFromNodetab   DBCDelNodeFromNodetabA
#  endif

#endif /* WIN32 */                                              /* ... 10.10.11 */

#ifdef __cplusplus
}
#endif
#endif /* DBCAPI_HEADER_FILE */
/*---------------------------------------------------------------------------*/
