/***************************************************************************
 *                           RELEX, Inc. DBMS Linter                        *
 *  This is unpublished propietary source code of RELEX, Inc.               *
 *  The copyright notice does not evidence any actual or intended           *
 *  publication of such source code.                                        *
 *                                                                          *
 *                 Copyright (c) 1990-2012 RELEX, Inc.                      *
 *                                                                          *
 *                   This software was developed by                         *
 *                                                                          *
 *                            RELEX, Inc.                                   *
 *                         Voronezh, Russia                                 *
 *                                                                          *
 *                        All rights reserved.                              *
 ****************************************************************************
 AUTHOR:
 ----------------------------------------------------------------------------
 PURPOSE: common header for LINTER DECIMALS library
 ----------------------------------------------------------------------------
 REVISION HISTORY:
 14.10.91 
 10.06.97 -- Kent ( Windows dll )
 20.07.97 -- Ermakov (Copyright) 
 22.05.98 -- Kent (new dlldefs.h)
 19.06.98 -- Ermakov (L_TTRUE/L_TFALSE)
 05.07.98 -- Ermakov (typedef.h)
 07.04.99 -- Ermakov (for DLONG type)
 23.07.99 -- DV (Optional clearing of tail zeros - DECTOSTRN function)
 17.08.01 -- DV (RNDDECIMAL function added)
 18.11.03 -- KOV (TRUNCDECIMAL function added)
 08.06.12 -- Glukhov (datatypes)
*/

#ifndef _DECIMALS_H_
#define _DECIMALS_H_

#include "lintypes.h"                            /* 05.07.98  */
#include "dlldefs.h"                             /* 22.05.98 ... */

#ifdef __cplusplus
extern "C" {
#endif

#define DECLNG      16   /* full L_DECIMAL length          */
#define DECFRACTLNG 5    /* fracting part L_DECIMAL length */

#define DECBASE     10
#define DECLIMIT    100

typedef L_BYTE L_DECIMAL[DECLNG];

  /* L_DECIMAL type (first byte!) */
typedef enum {
     DECZERO    /* 0 */   /* value     = 0 */
    ,DECNEGATIV /* 1 */   /* value     < 0 */
    ,DECPOSITIV /* 2 */   /* value     > 0 */
    ,DECNEGOVER /* 3 */   /* overflow  < 0 */
    ,DECPOSOVER /* 4 */   /* overflow  > 0 */
    ,DECERROR   /* 5 */   /* error         */
} DECSTATUS;

/******************************************************/
extern void DLLAPI COPYDEC(const L_DECIMAL D, L_DECIMAL DD);
/************************************/
extern DECSTATUS DLLAPI GETDECSTATUS(const L_DECIMAL D);
/***************************************/
extern L_SWORD DLLAPI OKDECSTATUS(const L_DECIMAL D);
/***************************************/
extern void    DLLAPI SETDECSTATUS(DECSTATUS    S,
                                   L_DECIMAL    D);
/***************************************/
extern void    DLLAPI ENTDECIMAL(const L_DECIMAL D,
                                       L_DECIMAL DD);
/**************************************/
extern void    DLLAPI NEGDECIMAL(const L_DECIMAL D,
                                       L_DECIMAL DD);
/**************************************/
extern void    DLLAPI ADDDECIMAL(const L_DECIMAL D1,
                                 const L_DECIMAL D2,
                                       L_DECIMAL D);
/**************************************/
extern void    DLLAPI SUBDECIMAL(const L_DECIMAL D1,
                                 const L_DECIMAL D2,
                                       L_DECIMAL D);
/**************************************/
extern L_SWORD DLLAPI CMPDECIMAL(const L_DECIMAL D1,
                                 const L_DECIMAL D2);
/**************************************/
extern void    DLLAPI MULDECIMAL(const L_DECIMAL D1,
                                 const L_DECIMAL D2,
                                       L_DECIMAL D);
/**************************************/
extern void    DLLAPI DIVDECIMAL(const L_DECIMAL D1,
                                 const L_DECIMAL D2,
                                       L_DECIMAL D);
/**************************************/
extern L_SWORD DLLAPI STRTODEC(L_CHAR  *    STR,
                               L_DECIMAL    D);
/*************************************/
extern void DLLAPI DECTOSTR(L_DECIMAL const D,
                            L_CHAR *     STR,
                            L_SWORD      LEN,
                            L_SWORD      LENFRACT,
                            L_SWORD      ISSIGN);
/******************************************/
extern void DLLAPI DECTOSTRN(L_DECIMAL const D,            /* 23.07.99 */
                             L_CHAR *     STR,
                             L_SWORD      LEN,
                             L_SWORD      LENFRACT,
                             L_SWORD      ISSIGN,
			     L_SWORD      CLEARTAIL);
/******************************************/
extern L_SWORD DLLAPI LongToDec(L_LONG L, L_DECIMAL D);
/*********************************************/
extern L_SWORD DLLAPI DecToLong(L_DECIMAL const D, L_LONG *L, L_SWORD Rounds);
/*********************************************/
extern L_SWORD DLLAPI DblToDec(double d, L_DECIMAL D);
/****************************************/
extern void    DLLAPI SetMaxDecimal(L_DECIMAL D);               /* 03.09.92 */
/****************************************/
extern void    DLLAPI SetMinDecimal(L_DECIMAL D);               /* 03.09.92 */
/****************************************/
extern L_SWORD DLLAPI DecToDbl(const L_DECIMAL D, double *pd);

extern L_SWORD DLLAPI DecToDLong(const L_DECIMAL D, L_DLONG *L, L_SWORD Rounds);
extern L_SWORD DLLAPI DLongToDec(L_DLONG L, L_DECIMAL D);

extern void    DLLAPI RNDDECIMAL(L_DECIMAL      D1,		/* 17.08.01 */
                                 L_BYTE         PREC,
                                 L_BYTE         SCALE);

extern void    DLLAPI TRUNCDECIMAL(L_DECIMAL    D,
                                   L_BYTE       PREC,
                                   L_BYTE       SCALE);         /* 18.11.03 */
#ifdef __cplusplus
}
#endif


#endif

