 /***************************************************************************
 *           RELEX, Inc. Client source include file                         *
 *  This is unpublished propietary source code of RELEX, Inc.               *
 *  The copyright notice does not evidence any actual or intended           *
 *  publication of such source code.                                        *
 *                                                                          *
 *                 Copyright (c) 1990-2005 RELEX Inc.                       *
 *                                                                          *
 *                   This software was developed by                         *
 *                                                                          *
 *                            RELEX, Inc.                                   *
 *                         Voronezh, Russia                                 *
 *                                                                          *
 *                        All rights reserved.                              *
 ****************************************************************************
 FILE:   encode.c
 ----------------------------------------------------------------------------
 AUTHOR: Maxim V. Kositsin
 ----------------------------------------------------------------------------
 PURPOSE: functions for convertion from Ansi <-> Oem and Koi <-> Oem
 ----------------------------------------------------------------------------
 REVISION HISTORY:
                  ** 22.12.95 -- Kent
                  ** 16.04.96 -- Maxim K. ( Letter 'E' )
                  ** 10.06.96 -- Maxim K. ( cod[] )
                  ** 25.06.96 -- Maxim K. ( $ ->  )
                  ** 13.08.96 -- KOM
                  ** 26.11.96 -- Maxim K. ( cToDOS , cToWIN )
                  ** 27.11.96 -- Maxim K. ( Single cod )
                  ** 30.01.97 -- Maxim K. ( Koi85 <-> Dos )
                  ** 26.01.99 -- Fedorov A. (to upper/lower )
  16.05.00 -- ReAn -- Win1251charTo866
  01.12.00 -- ReAn - moving arrays to .h
  02.04.01 -- Maxim -- new 866 <-> koi8 code tables
  04.04.01 -- DV (6.0)
  20.08.02 -- Ermakov (ENGLATIN1 tables added(850/1252))
  22.08.02 -- DV (_VER_MAX < 600)
  08.10.02 -- RAW(TT#6375. changed tables for cp866 to upper /to lower conversion.)
  23.09.05 -- FLY (Move some functions from strin.c)
  16.11.12 -- Glukhov,Ermakov (Remove extra code, datatypes)
 --------------------------------------------------------------------------*/
#include "encode.h"  /* 13.08.96 -- KOM */

#if !defined (LITE_KERNEL) || _VER_MAX < 600      /* 22.08.02 */ /* 23.09.05 */

unsigned char to_up_alt[256] = {                  /* 26.01.99 ... */
#ifdef _ENGLATIN1
  0x00,0x01,0x02,0x03,0x04,0x05,0x06,0x07,0x08,0x09,0x0A,0x0B,0x0C,0x0D,0x0E,0x0F,
  0x10,0x11,0x12,0x13,0x14,0x15,0x16,0x17,0x18,0x19,0x1A,0x1B,0x1C,0x1D,0x1E,0x1F,
  0x20,0x21,0x22,0x23,0x24,0x25,0x26,0x27,0x28,0x29,0x2A,0x2B,0x2C,0x2D,0x2E,0x2F,
  0x30,0x31,0x32,0x33,0x34,0x35,0x36,0x37,0x38,0x39,0x3A,0x3B,0x3C,0x3D,0x3E,0x3F,
  0x40,0x41,0x42,0x43,0x44,0x45,0x46,0x47,0x48,0x49,0x4A,0x4B,0x4C,0x4D,0x4E,0x4F,
  0x50,0x51,0x52,0x53,0x54,0x55,0x56,0x57,0x58,0x59,0x5A,0x5B,0x5C,0x5D,0x5E,0x5F,
  0x60,0x41,0x42,0x43,0x44,0x45,0x46,0x47,0x48,0x49,0x4A,0x4B,0x4C,0x4D,0x4E,0x4F,
  0x50,0x51,0x52,0x53,0x54,0x55,0x56,0x57,0x58,0x59,0x5A,0x7B,0x7C,0x7D,0x7E,0x7F,
  0x80,0x9A,0x90,0xB6,0x8E,0xB7,0x8F,0x80,0xD2,0xD3,0xD4,0xD8,0xD7,0xDE,0x8E,0x8F,
  0x90,0x92,0x92,0xE2,0x99,0xE3,0xEA,0xEB,0x98,0x99,0x9A,0x9D,0x9C,0x9D,0x9E,0x9F,
  0xB5,0xD6,0xE0,0xE9,0xA5,0xA5,0xA6,0xA7,0xA8,0xA9,0xAA,0xAB,0xAC,0xAD,0xAE,0xAF,
  0xB0,0xB1,0xB2,0xB3,0xB4,0xB5,0xB6,0xB7,0xB8,0xB9,0xBA,0xBB,0xBC,0xBD,0xBE,0xBF,
  0xC0,0xC1,0xC2,0xC3,0xC4,0xC5,0xC7,0xC7,0xC8,0xC9,0xCA,0xCB,0xCC,0xCD,0xCE,0xCF,
  0xD1,0xD1,0xD2,0xD3,0xD4,0x49,0xD6,0xD7,0xD8,0xD9,0xDA,0xDB,0xDC,0xDD,0xDE,0xDF,
  0xE0,0xE1,0xE2,0xE3,0xE5,0xE5,0xE6,0xE8,0xE8,0xE9,0xEA,0xEB,0xED,0xED,0xEE,0xEF,
  0xF0,0xF1,0xF2,0xF3,0xF4,0xF5,0xF6,0xF7,0xF8,0xF9,0xFA,0xFB,0xFC,0xFD,0xFE,0xFF
#else
# if 0 /* old upper case table */                                          /* 08.10.02 */
  0x00,0x01,0x02,0x03,0x04,0x05,0x06,0x07,0x08,0x09,0x0a,0x0b,0x0c,0x0d,0x0e,
  0x0f,0x10,0x11,0x12,0x13,0x14,0x15,0x16,0x17,0x18,0x19,0x1a,0x1b,0x1c,0x1d,
  0x1e,0x1f,0x20,0x21,0x22,0x23,0x24,0x25,0x26,0x27,0x28,0x29,0x2a,0x2b,0x2c,
  0x2d,0x2e,0x2f,0x30,0x31,0x32,0x33,0x34,0x35,0x36,0x37,0x38,0x39,0x3a,0x3b,
  0x3c,0x3d,0x3e,0x3f,0x40,0x41,0x42,0x43,0x44,0x45,0x46,0x47,0x48,0x49,0x4a,
  0x4b,0x4c,0x4d,0x4e,0x4f,0x50,0x51,0x52,0x53,0x54,0x55,0x56,0x57,0x58,0x59,
  0x5a,0x5b,0x5c,0x5d,0x5e,0x5f,0x60,0x41,0x42,0x43,0x44,0x45,0x46,0x47,0x48,
  0x49,0x4a,0x4b,0x4c,0x4d,0x4e,0x4f,0x50,0x51,0x52,0x53,0x54,0x55,0x56,0x57,
  0x58,0x59,0x5a,0x7b,0x7c,0x7d,0x7e,0x7f,0xc0,0xc1,0xc2,0xc3,0xc4,0xc5,0xc6,
  0xc7,0xc8,0xc9,0xca,0xcb,0xcc,0xcd,0xce,0xcf,0xd0,0xd1,0xd2,0xd3,0xd4,0xd5,
  0xd6,0xd7,0xd8,0xd9,0x9a,0xda,0xdb,0xdc,0xdd,0xde,0xdf,0xf2,0xf3,0xb3,0xf5,
  0xf6,0xf7,0xf1,0xf8,0xf9,0xfa,0xfb,0xfc,0xfd,0xfe,0xea,0xb0,0xb1,0xb2,0xb3,
  0xb4,0xb5,0xb6,0xb7,0xb8,0xb9,0xba,0xbb,0xbc,0xbd,0xbe,0xbf,0x9e,0x80,0x81,
  0x96,0x84,0x85,0x94,0x83,0x95,0x88,0x89,0x8a,0x8b,0x8c,0x8d,0x8e,0x8f,0x9f,
  0x90,0x91,0x92,0x93,0x86,0x82,0x9c,0x9b,0x87,0x98,0x9d,0x99,0x97,0xff,0x9e,
  0x80,0x81,0x96,0x84,0x85,0x94,0x83,0x95,0x88,0x89,0x8a,0x8b,0x8c,0x8d,0x8e,
  0x8f,0x9f,0x90,0x91,0x92,0x93,0x86,0x82,0x9c,0x9b,0x87,0x98,0x9d,0x99,0x97,
  0xff
# else /* new upper case table */
  0x00,0x01,0x02,0x03,0x04,0x05,0x06,0x07,0x08,0x09,0x0A,0x0B,0x0C,0x0D,0x0E,0x0F,
  0x10,0x11,0x12,0x13,0x14,0x15,0x16,0x17,0x18,0x19,0x1A,0x1B,0x1C,0x1D,0x1E,0x1F,
  0x20,0x21,0x22,0x23,0x24,0x25,0x26,0x27,0x28,0x29,0x2A,0x2B,0x2C,0x2D,0x2E,0x2F,
  0x30,0x31,0x32,0x33,0x34,0x35,0x36,0x37,0x38,0x39,0x3A,0x3B,0x3C,0x3D,0x3E,0x3F,
  0x40,0x41,0x42,0x43,0x44,0x45,0x46,0x47,0x48,0x49,0x4A,0x4B,0x4C,0x4D,0x4E,0x4F,
  0x50,0x51,0x52,0x53,0x54,0x55,0x56,0x57,0x58,0x59,0x5A,0x5B,0x5C,0x5D,0x5E,0x5F,
  0x60,0x41,0x42,0x43,0x44,0x45,0x46,0x47,0x48,0x49,0x4A,0x4B,0x4C,0x4D,0x4E,0x4F,
  0x50,0x51,0x52,0x53,0x54,0x55,0x56,0x57,0x58,0x59,0x5A,0x7B,0x7C,0x7D,0x7E,0x7F,
  0x80,0x81,0x82,0x83,0x84,0x85,0x86,0x87,0x88,0x89,0x8A,0x8B,0x8C,0x8D,0x8E,0x8F,
  0x90,0x91,0x92,0x93,0x94,0x95,0x96,0x97,0x98,0x99,0x9A,0x9B,0x9C,0x9D,0x9E,0x9F,
  0x80,0x81,0x82,0x83,0x84,0x85,0x86,0x87,0x88,0x89,0x8A,0x8B,0x8C,0x8D,0x8E,0x8F,
  0xB0,0xB1,0xB2,0xB3,0xB4,0xB5,0xB6,0xB7,0xB8,0xB9,0xBA,0xBB,0xBC,0xBD,0xBE,0xBF,
  0xC0,0xC1,0xC2,0xC3,0xC4,0xC5,0xC6,0xC7,0xC8,0xC9,0xCA,0xCB,0xCC,0xCD,0xCE,0xCF,
  0xD0,0xD1,0xD2,0xD3,0xD4,0xD5,0xD6,0xD7,0xD8,0xD9,0xDA,0xDB,0xDC,0xDD,0xDE,0xDF,
  0x90,0x91,0x92,0x93,0x94,0x95,0x96,0x97,0x98,0x99,0x9A,0x9B,0x9C,0x9D,0x9E,0x9F,
  0xF0,0xF0,0xF2,0xF2,0xF4,0xF4,0xF6,0xF6,0xF8,0xF9,0xFA,0xFB,0xFC,0xFD,0xFE,0xFF
# endif
#endif
};

unsigned char to_low_alt[256] = {
#ifdef _ENGLATIN1
  0x00,0x01,0x02,0x03,0x04,0x05,0x06,0x07,0x08,0x09,0x0A,0x0B,0x0C,0x0D,0x0E,0x0F,
  0x10,0x11,0x12,0x13,0x14,0x15,0x16,0x17,0x18,0x19,0x1A,0x1B,0x1C,0x1D,0x1E,0x1F,
  0x20,0x21,0x22,0x23,0x24,0x25,0x26,0x27,0x28,0x29,0x2A,0x2B,0x2C,0x2D,0x2E,0x2F,
  0x30,0x31,0x32,0x33,0x34,0x35,0x36,0x37,0x38,0x39,0x3A,0x3B,0x3C,0x3D,0x3E,0x3F,
  0x40,0x61,0x62,0x63,0x64,0x65,0x66,0x67,0x68,0x69,0x6A,0x6B,0x6C,0x6D,0x6E,0x6F,
  0x70,0x71,0x72,0x73,0x74,0x75,0x76,0x77,0x78,0x79,0x7A,0x5B,0x5C,0x5D,0x5E,0x5F,
  0x60,0x61,0x62,0x63,0x64,0x65,0x66,0x67,0x68,0x69,0x6A,0x6B,0x6C,0x6D,0x6E,0x6F,
  0x70,0x71,0x72,0x73,0x74,0x75,0x76,0x77,0x78,0x79,0x7A,0x7B,0x7C,0x7D,0x7E,0x7F,
  0x87,0x81,0x82,0x83,0x84,0x85,0x86,0x87,0x88,0x89,0x8A,0x8B,0x8C,0x8D,0x84,0x86,
  0x82,0x91,0x91,0x93,0x94,0x95,0x96,0x97,0x98,0x94,0x81,0x9B,0x9C,0x9B,0x9E,0x9F,
  0xA0,0xA1,0xA2,0xA3,0xA4,0xA4,0xA6,0xA7,0xA8,0xA9,0xAA,0xAB,0xAC,0xAD,0xAE,0xAF,
  0xB0,0xB1,0xB2,0xB3,0xB4,0xA0,0x83,0x85,0xB8,0xB9,0xBA,0xBB,0xBC,0xBD,0xBE,0xBF,
  0xC0,0xC1,0xC2,0xC3,0xC4,0xC5,0xC6,0xC6,0xC8,0xC9,0xCA,0xCB,0xCC,0xCD,0xCE,0xCF,
  0xD0,0xD0,0x88,0x89,0x8A,0xD5,0xA1,0x8C,0x8B,0xD9,0xDA,0xDB,0xDC,0xDD,0x8D,0xDF,
  0xA2,0xE1,0x93,0x95,0xE4,0xE4,0xE6,0xE7,0xE7,0xA3,0x96,0x97,0xEC,0xEC,0xEE,0xEF,
  0xF0,0xF1,0xF2,0xF3,0xF4,0xF5,0xF6,0xF7,0xF8,0xF9,0xFA,0xFB,0xFC,0xFD,0xFE,0xFF
#else
# if 0 /* old lower case table */                                          /* 08.10.02 */
  0x00,0x01,0x02,0x03,0x04,0x05,0x06,0x07,0x08,0x09,0x0a,0x0b,0x0c,0x0d,0x0e,
  0x0f,0x10,0x11,0x12,0x13,0x14,0x15,0x16,0x17,0x18,0x19,0x1a,0x1b,0x1c,0x1d,
  0x1e,0x1f,0x20,0x21,0x22,0x23,0x24,0x25,0x26,0x27,0x28,0x29,0x2a,0x2b,0x2c,
  0x2d,0x2e,0x2f,0x30,0x31,0x32,0x33,0x34,0x35,0x36,0x37,0x38,0x39,0x3a,0x3b,
  0x3c,0x3d,0x3e,0x3f,0x40,0x61,0x62,0x63,0x64,0x65,0x66,0x67,0x68,0x69,0x6a,
  0x6b,0x6c,0x6d,0x6e,0x6f,0x70,0x71,0x72,0x73,0x74,0x75,0x76,0x77,0x78,0x79,
  0x7a,0x5b,0x5c,0x5d,0x5e,0x5f,0x60,0x61,0x62,0x63,0x64,0x65,0x66,0x67,0x68,
  0x69,0x6a,0x6b,0x6c,0x6d,0x6e,0x6f,0x70,0x71,0x72,0x73,0x74,0x75,0x76,0x77,
  0x78,0x79,0x7a,0x7b,0x7c,0x7d,0x7e,0x7f,0xc0,0xc1,0xc2,0xc3,0xc4,0xc5,0xc6,
  0xc7,0xc8,0xc9,0xca,0xcb,0xcc,0xcd,0xce,0xcf,0xd0,0xd1,0xd2,0xd3,0xd4,0xd5,
  0xd6,0xd7,0xd8,0xd9,0x9a,0xda,0xdb,0xdc,0xdd,0xde,0xdf,0xf2,0xf3,0xf4,0xf5,
  0xf6,0xf7,0xf1,0xf8,0xf9,0xfa,0xfb,0xfc,0xfd,0xfe,0xea,0xb0,0xb1,0xb2,0xf4,
  0xb4,0xb5,0xb6,0xb7,0xb8,0xb9,0xba,0xbb,0xbc,0xbd,0xbe,0xbf,0xee,0xa0,0xa1,
  0xe6,0xa4,0xa5,0xe4,0xa3,0xe5,0xa8,0xa9,0xaa,0xab,0xac,0xad,0xae,0xaf,0xef,
  0xe0,0xe1,0xe2,0xe3,0xa6,0xa2,0xec,0xeb,0xa7,0xe8,0xed,0xe9,0xe7,0xf0,0xee,
  0xa0,0xa1,0xe6,0xa4,0xa5,0xe4,0xa3,0xe5,0xa8,0xa9,0xaa,0xab,0xac,0xad,0xae,
  0xaf,0xef,0xe0,0xe1,0xe2,0xe3,0xa6,0xa2,0xec,0xeb,0xa7,0xe8,0xed,0xe9,0xe7,
  0xf0
# else /* new lower case table */
  0x00,0x01,0x02,0x03,0x04,0x05,0x06,0x07,0x08,0x09,0x0A,0x0B,0x0C,0x0D,0x0E,0x0F,
  0x10,0x11,0x12,0x13,0x14,0x15,0x16,0x17,0x18,0x19,0x1A,0x1B,0x1C,0x1D,0x1E,0x1F,
  0x20,0x21,0x22,0x23,0x24,0x25,0x26,0x27,0x28,0x29,0x2A,0x2B,0x2C,0x2D,0x2E,0x2F,
  0x30,0x31,0x32,0x33,0x34,0x35,0x36,0x37,0x38,0x39,0x3A,0x3B,0x3C,0x3D,0x3E,0x3F,
  0x40,0x61,0x62,0x63,0x64,0x65,0x66,0x67,0x68,0x69,0x6A,0x6B,0x6C,0x6D,0x6E,0x6F,
  0x70,0x71,0x72,0x73,0x74,0x75,0x76,0x77,0x78,0x79,0x7A,0x5B,0x5C,0x5D,0x5E,0x5F,
  0x60,0x61,0x62,0x63,0x64,0x65,0x66,0x67,0x68,0x69,0x6A,0x6B,0x6C,0x6D,0x6E,0x6F,
  0x70,0x71,0x72,0x73,0x74,0x75,0x76,0x77,0x78,0x79,0x7A,0x7B,0x7C,0x7D,0x7E,0x7F,
  0xA0,0xA1,0xA2,0xA3,0xA4,0xA5,0xA6,0xA7,0xA8,0xA9,0xAA,0xAB,0xAC,0xAD,0xAE,0xAF,
  0xE0,0xE1,0xE2,0xE3,0xE4,0xE5,0xE6,0xE7,0xE8,0xE9,0xEA,0xEB,0xEC,0xED,0xEE,0xEF,
  0xA0,0xA1,0xA2,0xA3,0xA4,0xA5,0xA6,0xA7,0xA8,0xA9,0xAA,0xAB,0xAC,0xAD,0xAE,0xAF,
  0xB0,0xB1,0xB2,0xB3,0xB4,0xB5,0xB6,0xB7,0xB8,0xB9,0xBA,0xBB,0xBC,0xBD,0xBE,0xBF,
  0xC0,0xC1,0xC2,0xC3,0xC4,0xC5,0xC6,0xC7,0xC8,0xC9,0xCA,0xCB,0xCC,0xCD,0xCE,0xCF,
  0xD0,0xD1,0xD2,0xD3,0xD4,0xD5,0xD6,0xD7,0xD8,0xD9,0xDA,0xDB,0xDC,0xDD,0xDE,0xDF,
  0xE0,0xE1,0xE2,0xE3,0xE4,0xE5,0xE6,0xE7,0xE8,0xE9,0xEA,0xEB,0xEC,0xED,0xEE,0xEF,
  0xF1,0xF1,0xF3,0xF3,0xF5,0xF5,0xF7,0xF7,0xF8,0xF9,0xFA,0xFB,0xFC,0xFD,0xFE,0xFF
# endif
#endif
};                                                   /* ... 26.01.99 */


unsigned char DosToKoi[] = { /* 30-01-97 -- Maxim K. */
#ifdef _ENGLATIN1
/* 02.04.01... */
0xC7,0xFC,0xE9,0xE2,0xE4,0xE0,0xE5,0xE7,0xEA,0xEB,0xE8,0xEF,0xEE,0xEC,0xC4,0xC5,
0xC9,0xE6,0xC6,0xF4,0xF6,0xF2,0xFB,0xF9,0xFF,0xD6,0xDC,0xF8,0xA3,0xD8,0xD7,0x80,
0xE1,0xED,0xF3,0xFA,0xF1,0xD1,0xAA,0xBA,0xBF,0xAE,0xAC,0xBD,0xBC,0xA1,0xAB,0xBB,
0x81,0x82,0x83,0x84,0x85,0xC1,0xC2,0xC0,0xA9,0x86,0x87,0x88,0x89,0xA2,0xA5,0x8A,
0x8B,0x8C,0x8D,0x8E,0x8F,0x90,0xE3,0xC3,0x91,0x92,0x93,0x94,0x95,0x96,0x97,0xA4,
0xF0,0xD0,0xCA,0xCB,0xC8,0x98,0xCD,0xCE,0xCF,0x99,0x9A,0x9B,0x9C,0xA6,0xCC,0x9D,
0xD3,0xDF,0xD4,0xD2,0xF5,0xD5,0xB5,0xFE,0xDE,0xDA,0xDB,0xD9,0xFD,0xDD,0xAF,0xB4,
0xAD,0xB1,0x9E,0xBE,0xB6,0xA7,0xF7,0xB8,0xB0,0xA8,0xB7,0xB9,0xB3,0xB2,0x9F,0xA0
#else
225,226,247,231,228,229,246,250,233,234,235,236,237,238,239,240,
242,243,244,245,230,232,227,254,251,253,255,249,248,252,224,241,
193,194,215,199,196,197,214,218,201,202,203,204,205,206,207,208,
144,145,146,129,135,178,180,167,166,181,161,168,174,173,172,131,
132,137,136,134,128,138,175,176,171,165,187,184,177,160,190,185,
186,182,183,170,169,162,164,189,188,133,130,141,140,142,143,139,
210,211,212,213,198,200,195,222,219,221,223,217,216,220,192,209,
179,163,153,152,147,155,159,151,156,149,158,150,191,157,148,154
#endif
};
/* ...02.04.01 */

unsigned char KoiToDos[] = { /* 30-01-97 -- Maxim K. */
#ifdef _ENGLATIN1
0x9F,0xB0,0xB1,0xB2,0xB3,0xB4,0xB9,0xBA,0xBB,0xBC,0xBF,0xC0,0xC1,0xC2,0xC3,0xC4,
0xC5,0xC8,0xC9,0xCA,0xCB,0xCC,0xCD,0xCE,0xD5,0xD9,0xDA,0xDB,0xDC,0xDF,0xF2,0xFE,
0xFF,0xAD,0xBD,0x9C,0xCF,0xBE,0xDD,0xF5,0xF9,0xB8,0xA6,0xAE,0xAA,0xF0,0xA9,0xEE,
0xF8,0xF1,0xFD,0xFC,0xEF,0xE6,0xF4,0xFA,0xF7,0xFB,0xA7,0xAF,0xAC,0xAB,0xF3,0xA8,
0xB7,0xB5,0xB6,0xC7,0x8E,0x8F,0x92,0x80,0xD4,0x90,0xD2,0xD3,0xDE,0xD6,0xD7,0xD8,
0xD1,0xA5,0xE3,0xE0,0xE2,0xE5,0x99,0x9E,0x9D,0xEB,0xE9,0xEA,0x9A,0xED,0xE8,0xE1,
0x85,0xA0,0x83,0xC6,0x84,0x86,0x91,0x87,0x8A,0x82,0x88,0x89,0x8D,0xA1,0x8C,0x8B,
0xD0,0xA4,0x95,0xA2,0x93,0xE4,0x94,0xF6,0x9B,0x97,0xA3,0x96,0x81,0xEC,0xE7,0x98
#else
/* 02.04.01... */
196,179,218,191,192,217,195,180,194,193,197,223,220,219,221,222,
176,177,178,244,254,249,251,247,243,242,255,245,248,253,250,246,
205,186,213,241,214,201,184,183,187,212,211,200,190,189,188,198,
199,204,181,240,182,185,209,210,203,207,208,202,216,215,206,252,
238,160,161,230,164,165,228,163,229,168,169,170,171,172,173,174,
175,239,224,225,226,227,166,162,236,235,167,232,237,233,231,234,
158,128,129,150,132,133,148,131,149,136,137,138,139,140,141,142,
143,159,144,145,146,147,134,130,156,155,135,152,157,153,151,154 
#endif
};
/* ...02.04.01 */

unsigned char cToDOS[] = {
#ifdef _ENGLATIN1
0xB0,0xB1,0xB2,0x9F,0xB3,0xB4,0xB9,0xBA,0xBB,0xBC,0xBF,0xC0,0xC1,0xC2,0xC3,0xC4,
0xC5,0xC8,0xC9,0xCA,0xCB,0xCC,0xCD,0xCE,0xD5,0xD9,0xDA,0xDB,0xDC,0xDF,0xF2,0xFE,
0xFF,0xAD,0xBD,0x9C,0xCF,0xBE,0xDD,0xF5,0xF9,0xB8,0xA6,0xAE,0xAA,0xF0,0xA9,0xEE,
0xF8,0xF1,0xFD,0xFC,0xEF,0xE6,0xF4,0xFA,0xF7,0xFB,0xA7,0xAF,0xAC,0xAB,0xF3,0xA8,
0xB7,0xB5,0xB6,0xC7,0x8E,0x8F,0x92,0x80,0xD4,0x90,0xD2,0xD3,0xDE,0xD6,0xD7,0xD8,
0xD1,0xA5,0xE3,0xE0,0xE2,0xE5,0x99,0x9E,0x9D,0xEB,0xE9,0xEA,0x9A,0xED,0xE8,0xE1,
0x85,0xA0,0x83,0xC6,0x84,0x86,0x91,0x87,0x8A,0x82,0x88,0x89,0x8D,0xA1,0x8C,0x8B,
0xD0,0xA4,0x95,0xA2,0x93,0xE4,0x94,0xF6,0x9B,0x97,0xA3,0x96,0x81,0xEC,0xE7,0x98
#else
      0xC8,0xB4,0xB5,0xB6,0xB7,0xB8,0xB9,0xBA,0xB2,0xBC,0xBD,
      0xBE,0xBF,0xC0,0xC1,0xC2,0xC3,0xCA,0xC5,0xC6,0xC7,0xF9,
      0xC9,0xC4,0xFE,0xCC,0xCD,0xCE,0xCF,0xD0,0xD1,0xD2,0xD3,
      0xF7,0xF6,0xD6,0xFD,0xD8,0xB3,0xDA,0xF0,0xB0,0xF2,0xDE,
      0xDF,0xBB,0xB1,0xF4,0xF8,0xDD,0xFB,0xD9,0xDB,0xD5,0xD4,
      0xFA,0xF1,0xFC,0xF3,0xDC,0xD7,0xCB,0xFF,0xF5,0x80,0x81,
      0x82,0x83,0x84,0x85,0x86,0x87,0x88,0x89,0x8A,0x8B,0x8C,
      0x8D,0x8E,0x8F,0x90,0x91,0x92,0x93,0x94,0x95,0x96,0x97,
      0x98,0x99,0x9A,0x9B,0x9C,0x9D,0x9E,0x9F,0xA0,0xA1,0xA2,
      0xA3,0xA4,0xA5,0xA6,0xA7,0xA8,0xA9,0xAA,0xAB,0xAC,0xAD,
      0xAE,0xAF,0xE0,0xE1,0xE2,0xE3,0xE4,0xE5,0xE6,0xE7,0xE8,
      0xE9,0xEA,0xEB,0xEC,0xED,0xEE,0xEF
#endif
};

unsigned char cToWIN[] = {
#ifdef _ENGLATIN1
0xC7,0xFC,0xE9,0xE2,0xE4,0xE0,0xE5,0xE7,0xEA,0xEB,0xE8,0xEF,0xEE,0xEC,0xC4,0xC5,
0xC9,0xE6,0xC6,0xF4,0xF6,0xF2,0xFB,0xF9,0xFF,0xD6,0xDC,0xF8,0xA3,0xD8,0xD7,0x83,
0xE1,0xED,0xF3,0xFA,0xF1,0xD1,0xAA,0xBA,0xBF,0xAE,0xAC,0xBD,0xBC,0xA1,0xAB,0xBB,
0x80,0x81,0x82,0x84,0x85,0xC1,0xC2,0xC0,0xA9,0x86,0x87,0x88,0x89,0xA2,0xA5,0x8A,
0x8B,0x8C,0x8D,0x8E,0x8F,0x90,0xE3,0xC3,0x91,0x92,0x93,0x94,0x95,0x96,0x97,0xA4,
0xF0,0xD0,0xCA,0xCB,0xC8,0x98,0xCD,0xCE,0xCF,0x99,0x9A,0x9B,0x9C,0xA6,0xCC,0x9D,
0xD3,0xDF,0xD4,0xD2,0xF5,0xD5,0xB5,0xFE,0xDE,0xDA,0xDB,0xD9,0xFD,0xDD,0xAF,0xB4,
0xAD,0xB1,0x9E,0xBE,0xB6,0xA7,0xF7,0xB8,0xB0,0xA8,0xB7,0xB9,0xB3,0xB2,0x9F,0xA0
#else
      0xC0,0xC1,0xC2,0xC3,0xC4,0xC5,0xC6,0xC7,0xC8,0xC9,0xCA,0xCB,0xCC,
      0xCD,0xCE,0xCF,0xD0,0xD1,0xD2,0xD3,0xD4,0xD5,0xD6,0xD7,0xD8,0xD9,
      0xDA,0xDB,0xDC,0xDD,0xDE,0xDF,0xE0,0xE1,0xE2,0xE3,0xE4,0xE5,0xE6,
      0xE7,0xE8,0xE9,0xEA,0xEB,0xEC,0xED,0xEE,0xEF,0xA9,0xAE,0x88,0xA6,
      0x81,0x82,0x83,0x84,0x85,0x86,0x87,0xAD,0x89,0x8A,0x8B,0x8C,0x8D,
      0x8E,0x8F,0x90,0x97,0x92,0x93,0x94,0x80,0x96,0x91,0xBD,0x99,0x9A,
      0x9B,0x9C,0x9D,0x9E,0x9F,0xA0,0xB6,0xB5,0xA3,0xBC,0xA5,0xB3,0xA7,
      0xB4,0xBB,0xB1,0xAB,0xAC,0xF0,0xF1,0xF2,0xF3,0xF4,0xF5,0xF6,0xF7,
      0xF8,0xF9,0xFA,0xFB,0xFC,0xFD,0xFE,0xFF,0xA8,0xB8,0xAA,0xBA,0xAF,
      0xBF,0xA2,0xA1,0xB0,0x95,0xB7,0xB2,0xB9,0xA4,0x98,0xBE
#endif
      };

/*----------------------01-30-97 03:38pm----------------------
  Call:
  Desc:
  Args:
  Output:
  ------------------------------------------------------------*/
char *Oem2Koi(const char *s, char *d, unsigned short len)
{
 unsigned short i;

 for( i=0; i<len && s[i]; i++ )
   d[i] = alt2koi( s[i] );

 if (i<len && !s[i])    /* 26.01.99 */
   d[i]=0;

 return (char *)d;
}
/*----------------------01-30-97 03:38pm----------------------
  Call:
  Desc:
  Args:
  Output:
  ------------------------------------------------------------*/
char *Koi2Oem(const char *s, char *d, unsigned short len)
{
 unsigned short i;

 for( i=0; i<len && s[i]; i++ )
   d[i] = koi2alt( s[i] );

 if (i<len && !s[i])    /* 26.01.99 */
   d[i]=0;

 return (char *)d;
}

/* 22.12.95 */
char *Oem2Ansi(const char *s, char *d, unsigned short len)
{
 unsigned short i;

 for( i=0; i<len && s[i]; i++ )
   d[i] = alt2win( s[i] );

 if (i<len && !s[i])    /* 26.01.99 */
   d[i]=0;

 return (char *)d;
}

char *Ansi2Oem(const char *s, char *d, unsigned short len)
{
 unsigned short i;

 for( i=0; i<len && s[i]; i++ )
   d[i] = win2alt( s[i] );

 if (i<len && !s[i])    /* 26.01.99 */
   d[i]=0;

 return (char *)d;
}
/* 22.12.95 */

char *Ansi2Koi(const char *s, char *d, unsigned short len)
{
 unsigned short i;

 for( i=0; i<len && s[i]; i++ )
   {
   d[i] = win2alt( s[i] );
   d[i] = alt2koi( d[i] );
   }

 if (i<len && !s[i])
   d[i]=0;

 return (char *)d;
}

char *Koi2Ansi(const char *s, char *d, unsigned short len)
{
 unsigned short i;

 for( i=0; i<len && s[i]; i++ )
   {
   d[i] = koi2alt( s[i] );
   d[i] = alt2win( d[i] );
   }

 if (i<len && !s[i])
   d[i]=0;

 return (char *)d;
}
                                                
#define char_to_upper_alt(c) to_up_alt[(unsigned char)c]     /* 26.01.99 ... */
#define char_to_lower_alt(c) to_low_alt[(unsigned char)c]

#define char_to_upper_koi(c) alt2koi(to_up_alt[(unsigned char)koi2alt(c)])
#define char_to_lower_koi(c) alt2koi(to_low_alt[(unsigned char)koi2alt(c)])

#define char_to_upper_win(c) alt2win(to_up_alt[(unsigned char)win2alt(c)])
#define char_to_lower_win(c) alt2win(to_low_alt[(unsigned char)win2alt(c)])

char*   str_to_upper_alt(char* src, char* dst, unsigned short len)
{
    unsigned short i;
    
    for (i=0;i<len && src[i];i++)
        dst[i]=char_to_upper_alt(src[i]);
        
    if (i<len && !src[i])
        dst[i]=0;
        
    return dst;
}

char*   str_to_lower_alt(char* src, char* dst, unsigned short len)
{
    unsigned short i;
    
    for (i=0;i<len && src[i];i++)
        dst[i]=char_to_lower_alt(src[i]);
        
    if (i<len && !src[i])
        dst[i]=0;
        
    return dst;
}

char*   str_to_upper_koi(char* src, char* dst, unsigned short len)
{
    unsigned short i;
    
    for (i=0;i<len && src[i];i++)
        dst[i]=char_to_upper_koi(src[i]);
        
    if (i<len && !src[i])
        dst[i]=0;
        
    return dst;
}

char*   str_to_lower_koi(char* src, char* dst, unsigned short len)
{
    unsigned short i;
    
    for (i=0;i<len && src[i];i++)
        dst[i]=char_to_lower_koi(src[i]);
        
    if (i<len && !src[i])
        dst[i]=0;
        
    return dst;
}

char*   str_to_upper_win(char* src, char* dst, unsigned short len)
{
    unsigned short i;
    
    for (i=0;i<len && src[i];i++)
        dst[i]=char_to_upper_win(src[i]);
        
    if (i<len && !src[i])
        dst[i]=0;
        
    return dst;
}

char*   str_to_lower_win(char* src, char* dst, unsigned short len)
{
    unsigned short i;
    
    for (i=0;i<len && src[i];i++)
        dst[i]=char_to_lower_win(src[i]);
        
    if (i<len && !src[i])
        dst[i]=0;
        
    return dst;
}

/* ... 26.01.99 */

char Win1251charTo866(char Ch) /* 16.05.00 */
{
  return win2alt(Ch);
}

#else                                                     /* 23.09.05 ... */

char ToUp(char c)
{
  register unsigned char uc = (unsigned char)c;

  if (uc >= 'a' && uc <= 'z')
    return c - 'a' + 'A';
  if (uc >= 160 && uc <= 175)
    return c - 32;
  if (uc >= 224 && uc <= 239)
    return c - 80;
  if (uc == 241)
    return (char)240;
  return c;
}

char ToLow(char c)
{
  register unsigned char uc = (unsigned char)c;

  if (uc >= 'A' && uc <= 'Z') 
    return c + 'a' - 'A';
  if (uc >= 128 && uc <= 143)
    return c + 32;
  if (uc >= 144 && uc <= 159)
    return c + 80;
  if (uc == 240) 
    return (char)241;
  return c;
}

int  Izalpha(char c)
{
  register unsigned char uc = (unsigned char)c;

  return  uc >= 'A' && uc <= 'Z' || uc >= 'a' && uc <= 'z' ||
          uc >= 128 && uc <= 175 || uc >= 224 && uc <= 241;
}                              /* ... 23.09.05 */
#endif                                                  /* 22.08.02 */
