/****************************************************************************
 *           RELEX, Inc. RDBMS Linter hot reserve source code               *
 *  This is unpublished propietary source code of RELEX, Inc.               *
 *  The copyright notice does not evidence any actual or intended           *
 *  publication of such source code.                                        *
 *                                                                          *
 *                  Copyright (c) 1990-2005 RELEX Inc.                      *
 *                                                                          *
 *                   This software was developed by                         *
 *                                                                          *
 *                            RELEX, Inc.                                   *
 *                         Voronezh, Russia                                 *
 *                                                                          *
 *                        All rights reserved.                              *
 ----------------------------------------------------------------------------
 PURPOSE: application interface for hot  reserve control
 
 @author Dmitry Prohorov
 ----------------------------------------------------------------------------
 REVISION HISTORY:
   02.08.08 -- Prohorov ( first edition )
   27.12.11 -- Galizin (Openssl ciphers & digest)
   16.04.14 -- Galizin (allocation error)
 ****************************************************************************/

#include <unistd.h>
#include <netdb.h>
#include  <fcntl.h>
#include <errno.h>

#if defined(CRYPTO_OPENSSL)
#include <openssl/evp.h>                                    /* 27.12.11 */
#endif /* CRYPTO_OPENSSL */

#include "server.h"
#include "klib.h"                                           /* 27.12.11 */
#include "netproc.h"
#include "hresapi.h"
#define socklen_t int
#define BINARY_MODIFIER 0xa0000000
#define INIT_ERR     -10
#define RECEIVE_ERR  -20
#define PROTOCOL_ERR -30
/* structure for reading/writing current state */
int Port;
static void *Context = 0;

typedef struct
{
  L_WORD FragmentBitMask;
  L_WORD MaxFragmentNumber;
  L_WORD MsgLen;
  char AnswerBuff[1];
} GetStateContext;
static int workSocket = -1;
#define RECVBUFLEN  10000

# define TO_NETWORK      0
# define FROM_NETWORK    1
static char RecvBuf[RECVBUFLEN];

char MsgTxt[300];
char  *GetHresApiErrorMsg(int Error)
{
  char *Ptr;
  sprintf(MsgTxt, "HresApi() return error: ");
  Ptr = &MsgTxt[strlen(MsgTxt)];
  switch (Error)
  {
  case HRESAPI_OPENSOCKET_ERR:
    sprintf(Ptr, "socket(), ");
    break;
  case HRESAPI_BIND_ERR:
    sprintf(Ptr, "bind(), ");
    break;
  case HRESAPI_FCNTL_ERR:
    sprintf(Ptr, "fcntl(), ");
    break;
  case HRESAPI_SENDTO_ERR:
    sprintf(Ptr, "send data , ");
    break;
  case HRESAPI_CRC_ERR:
    sprintf(Ptr, "CRC, ");
    break;
  case HRESAPI_TIMEOUT_ERR:
    sprintf(Ptr, "timeout expiered, ");
    break;
  case HRESAPI_NETWORK_ERR:
    sprintf(Ptr, "network , ");
    break;
  case HRESAPI_UNEXPECTPACKET_ERR:
    sprintf(Ptr, "unexpected packet type, ");
    break;
  case HRESAPI_PACKETLEN_ERR:
    sprintf(Ptr, "packet len, ");
    break;
  case HRESAPI_CSUM_ERR:
    sprintf(Ptr, "packet control sum, ");
    break;
  case HRESAPI_MD5_ERR:
    sprintf(Ptr, "packet control sum, ");
    break;
  case HRESAPI_FRAGMENTNUM_ERR:
    sprintf(Ptr, "fragment number, ");
    break;
  case HRESAPI_UNEXPECTANSWER_ERR:
    sprintf(Ptr, "unexpected answer, ");
    break;
  case HRESAPI_HOSTADDR_ERR:
    sprintf(Ptr, "wrong host address, ");
    break;
  case HRESAPI_FRAGMENTLEN_ERR:
    sprintf(Ptr, "frogment len , ");
    break;
  case HRESAPI_FRAGMENTOFFS_ERR:
    sprintf(Ptr, "fragment offset, ");
    break;
  case HRESAPI_COMMAND_ERR:
    sprintf(Ptr, "wrong command, ");
    break;
  default:
    sprintf(Ptr, " code %d, ", Error);
  }
  Ptr = &MsgTxt[strlen(MsgTxt)];
  sprintf(Ptr, " errno = %d\n", errno);
  return MsgTxt;
}


static int ClearSocketError(int s)
{
  int Err;
  int ErrLen = sizeof(Err);
 return(getsockopt(s, SOL_SOCKET, SO_ERROR,(char *)&Err, &ErrLen)); /*WIN32*/
}

static int InitSocket(void)
{
  struct sockaddr_in MyAddr;

  /*init DGRAM socket*/
  if (workSocket != -1)
    return 0;
  memset(&MyAddr, 0, sizeof(MyAddr));
  MyAddr.sin_family = AF_INET;
  MyAddr.sin_addr.s_addr  = INADDR_ANY;

  if (( workSocket = socket(AF_INET, SOCK_DGRAM, IPPROTO_UDP) ) < 0)
  {
    return HRESAPI_OPENSOCKET_ERR;
  }

  if (bind(workSocket, (struct sockaddr *)&MyAddr, sizeof( MyAddr )) < 0)
  {
    close(workSocket);
    return HRESAPI_BIND_ERR;                                              /*for "already running" diagnostich*/
  }

  if (fcntl(workSocket, F_SETFL, O_NONBLOCK) == -1)
  {
    close(workSocket);
    return HRESAPI_FCNTL_ERR ;
  }
  return 0;
}


void RotateValue(char *Adr, char Len)
{
  char *adrend, rb;

  adrend = Adr + Len - 1;
  while ( Adr < adrend )
    {
    rb        = *Adr;
    *Adr++    = *adrend;
    *adrend-- = rb;
    }
}

#ifdef DATA_MSBF
static void  RotateGetStateStruct(t_GetStateAnswer *GA)
{
  RotateValue((L_BYTE *)&GA->MsgLen,sizeof(GA->MsgLen));
  RotateValue((L_BYTE *)&GA->MaxFragmentNumber,sizeof(GA->MaxFragmentNumber));
  RotateValue((L_BYTE *)&GA->FragmentNumber,sizeof(GA->FragmentNumber));
  RotateValue((L_BYTE *)&GA->Offset,sizeof(GA->Offset));
  RotateValue((L_BYTE *)&GA->FragmentLen,sizeof(GA->FragmentLen));
}

static void RotateCommand(t_Command* c, int FromToNetwork)
{
  if( FromToNetwork == TO_NETWORK )
    if ( c->Command == COMMAND_GETS || c->Command == A_COMMAND_GETS )
      RotateGetStateStruct((t_GetStateAnswer *)&c->Data);

  RotateValue((L_BYTE *)&c->Command, sizeof(c->Command));

  if( FromToNetwork == FROM_NETWORK )
    if ( c->Command == COMMAND_GETS || c->Command == A_COMMAND_GETS )
      RotateGetStateStruct((t_GetStateAnswer *)&c->Data);

  return;
}
#endif

static void RotatePacket(t_Packet* p)
{
  RotateValue((L_BYTE *)&p->Type, sizeof(p->Type));
  RotateValue((L_BYTE *)&p->Len, sizeof(p->Len));
  RotateValue((L_BYTE *)&p->PacketId, sizeof(p->PacketId));
  return;
}

static int IsOurByteOrder(t_Packet* packet)
{
#ifdef DATA_MSBF
  if (((char *)(&packet->Type))[0] == 0)
#else
  if (((char *)(&packet->Type))[1] == 0)
#endif
    return 0;
  else
    return -1;
}


static ssize_t recvfrom_cerr(int s, void *buf, size_t len, int flags,
                             struct sockaddr *addr, socklen_t *addr_len)
{
  ssize_t ReceivedLen;

  while (1)
  {
    if (( ReceivedLen = recvfrom(s, (char *)buf, len, flags, addr, addr_len) ) < 0)  /*WIN32*/
    {
      if (errno == ECONNREFUSED)
      if (!ClearSocketError(s))
        continue;
    }
    break;
  }
  return ReceivedLen;
}

static L_BYTE Crc8(L_BYTE data, L_BYTE CRC)
{
  L_BYTE n;
  int count=8;
  do
  {
    n = (1 & (data^CRC));
    CRC >>= 1;
    data >>= 1;
    if (n)
      CRC ^= 0x8C;
  } while (--count);
  return CRC;
}



L_BYTE GetCrc( const t_Packet *Packet, int Len )
{
  int i = 0;
  L_BYTE CRC = 0;
  
  i = Len;
  if (i > MAX_NET_MSG_LEN+PACKET_HEADER_SIZE)
    i = MAX_NET_MSG_LEN+PACKET_HEADER_SIZE;

  do
  {
    i--;
    if (i == 6)                                             /* Skip the crc in packet */
      continue;
    CRC = Crc8(((L_BYTE*)Packet)[i], CRC);
  }
  while( i > 0);

  return CRC;
}

static ssize_t sendto_cerr(int s, t_Packet *buf, size_t len, int flags,
                           const struct sockaddr *addr, socklen_t addr_len)
{
  ssize_t SentLen;

  buf->Crc = GetCrc(buf, len);

  while (1)
  {
    if (( SentLen = sendto(s, (char *)buf, len, flags, addr, addr_len) ) < 0)
    {
     if (errno == ECONNREFUSED)
       if (!ClearSocketError(s))
         continue;
    }
    break;
  }
  return SentLen;
}

int SendCmd(t_Command* Command)
{
  t_Packet Packet;
  MD5_CTX  Ctx;
  L_LONG   Len;
  int      PasswordLen = 0;

  Len = Command->Len;

#ifdef DATA_MSBF
  RotateCommand(Command, TO_NETWORK);
#endif

  if ( Command->UserPassword )
    PasswordLen = strlen(Command->UserPassword)+1;
  memset(&Packet, 0, sizeof(Packet) - MAX_NET_MSG_LEN + 20 );
  /* CRC16 - Command2 - PasworLen1 - reserv1 - DATA - PASSWORD */
  memcpy(Packet.Data + 16, &Command->Command, sizeof(Command->Command));    /* 02.06.08 ...*/
  Packet.Data[16+sizeof(Command->Command)] = PasswordLen; 
  memcpy(Packet.Data + 20, Command->Data, Len);
  if ( PasswordLen )
    memcpy(Packet.Data + 20 + Len, Command->UserPassword, PasswordLen);
  Packet.Type = PACKET_COMMAND;
  Packet.Len  = 20 + PasswordLen + Len;

  MD5Init(&Ctx);
  MD5Update(&Ctx, (unsigned char *)Packet.Data, Packet.Len);
  MD5Final((unsigned char *)Packet.Data, &Ctx);
  
  {
    int PacketLen = Packet.Len;

    if ((Len = sendto_cerr(workSocket, &Packet, PacketLen + PACKET_HEADER_SIZE, 0, 
                          (struct sockaddr *) &Command->Address,
                          sizeof(Command->Address))) != (PacketLen + PACKET_HEADER_SIZE))  
    {
      return HRESAPI_SENDTO_ERR;
    }
    return 0;
  }
}

static int ReceivePacket(t_Packet* Packet, int* Len, 
                       struct sockaddr_in* Addr, struct timeval *timeout)
{
  fd_set readFDS, writeFDS, errorFDS;
  int selectResult = 0;
  int ReceivedLen = 0;
  socklen_t AddrLen = sizeof(struct sockaddr_in);

  while(1)
  {
  FD_ZERO(&readFDS);
  FD_SET(workSocket, &readFDS);
  FD_ZERO(&writeFDS);
  FD_ZERO(&errorFDS);

  selectResult = select(workSocket + 1, &readFDS, &writeFDS, &errorFDS, timeout); 

  switch( selectResult )
  {
    case -1:
    
    if (errno != EINTR)
    {
      goto ErrRet;
    }
  else
    {
    continue;
    }

    case  0:
      goto NoData;
    default:
      break;
  }
    break;
  }
  /* check network socket activity */
  if (FD_ISSET(workSocket, &readFDS))
  {
    while(1)
    {
      ReceivedLen = recvfrom_cerr(workSocket, RecvBuf, sizeof(t_Packet), 0,
                                  (struct sockaddr *)Addr, &AddrLen);
      if (ReceivedLen < 0 )
      {
        if (errno == EINTR)
        {
          continue;
        }

        if (errno == EAGAIN)
          break;

        goto ErrRet;
      }

      break;
    }

    if (ReceivedLen > 0)
    {
#if defined(DEBUG)
      printf("%d bytes received\n", ReceivedLen);
#endif
      memcpy((char *)Packet, RecvBuf, ReceivedLen);
      *Len = ReceivedLen;

      if (Packet->Crc != GetCrc(Packet, ReceivedLen))
      {
        return HRESAPI_CRC_ERR;
      }
      return 0;
    }
  }

NoData:
  return HRESAPI_TIMEOUT_ERR;

ErrRet:
  shutdown(workSocket, 2);
  close(workSocket);
  workSocket = -1;
  return HRESAPI_NETWORK_ERR;
}




static int ReceiveData(struct sockaddr_in *Address, char **ContextPtr, int *ContextLen, int TimeoutSec) 
{
  struct timeval timeout;
  struct sockaddr_in SrcAddr;
  t_Packet Packet;
  int Len;
  static  t_Command Answer;
  char    ReceivedMd5[16], ThetMd5[16];
  MD5_CTX   Ctx;

  if (Context)
  {
    free(Context);
    Context = 0;
  }
  memset(&timeout, 0, sizeof( timeout ));
  timeout.tv_sec = TimeoutSec;
  while(1)
  { 
    int Err;
    if (Err = ReceivePacket(&Packet, &Len, &SrcAddr, &timeout))
    {
      return (Err);
    }

    if ( IsOurByteOrder(&Packet) != 0 )
      RotatePacket(&Packet); 
#if defined(DEBUG)
    printf("packet type = %d\n", Packet.Type);
#endif

    if (Packet.Type != PACKET_COMMAND)
    {
      return HRESAPI_UNEXPECTPACKET_ERR;
    }
  
    if (Packet.Len < 16+sizeof(Answer.Command))
    {
      return HRESAPI_PACKETLEN_ERR;
    }

    memcpy(ReceivedMd5, Packet.Data, 16);
    memset(Packet.Data, 0, 16);

    MD5Init(&Ctx);
    MD5Update(&Ctx, (unsigned char *)Packet.Data, Packet.Len);
    MD5Final((unsigned char *)ThetMd5, &Ctx);

    if (memcmp(ThetMd5, ReceivedMd5, 16)!= 0)
    {
      return HRESAPI_MD5_ERR;
    }
  
    memset(&Answer, 0, sizeof(Answer));
    memcpy(&Answer.Address, Address, sizeof(struct sockaddr_in));
    Answer.Command = *(L_WORD*)(Packet.Data + 16 );
    Answer.Len = Packet.Len - 20;
    memcpy(Answer.Data, Packet.Data+20, Answer.Len);

#ifdef DATA_MSBF
    RotateCommand(&Answer, FROM_NETWORK);
#endif
    if (!memcmp(&Answer.Address.sin_addr, &Address->sin_addr,
              sizeof(Answer.Address.sin_addr)) &&
              (Answer.Address.sin_port == Address->sin_port))
      switch( Answer.Command )
      {
        case A_COMMAND_GETS:
        {
           t_GetStateAnswer * GA = 0;
           GetStateContext *CPtr = 0;

           GA = (t_GetStateAnswer *)Answer.Data;
           if ( !Context )
           {
             Context = (void *)calloc(1, GA->MsgLen + 1 + (sizeof(L_WORD) * 3) );
             if ( Context == NULL )
               return HRESAPI_ALLOC_ERR;                    /* 16.04.14 */
             CPtr = (GetStateContext *)Context;
             CPtr->MsgLen = GA->MsgLen;
             CPtr->FragmentBitMask = 0;
             CPtr->MaxFragmentNumber = GA->MaxFragmentNumber;
          
             if ( CPtr->MaxFragmentNumber > sizeof(CPtr->FragmentBitMask) * 8 - 1 )
             {
               return HRESAPI_FRAGMENTNUM_ERR;
             }
           }
           else
             CPtr = (GetStateContext *)Context;

           if (GA->FragmentNumber > CPtr->MaxFragmentNumber)
           {
             return HRESAPI_FRAGMENTNUM_ERR;
           }
        
           if ( GA->Offset < CPtr->MsgLen )
           {
             if ( GA->Offset + GA->FragmentLen > CPtr->MsgLen )
             {
               return HRESAPI_FRAGMENTLEN_ERR;
             }

             memcpy(&CPtr->AnswerBuff[GA->Offset], GA->Data, GA->FragmentLen);
           }
           else
             return HRESAPI_FRAGMENTOFFS_ERR;

           CPtr->FragmentBitMask |= 1 << GA->FragmentNumber;

           if (CPtr->FragmentBitMask == ( 1 << (CPtr->MaxFragmentNumber+1) ) - 1 )
           {
             CPtr->AnswerBuff[CPtr->MsgLen] = 0;
             *ContextLen = CPtr->MsgLen;
             *ContextPtr = CPtr->AnswerBuff;
             return 0;
           }
           break;
        }
      case A_COMMAND_STOP:
      case A_COMMAND_EXCHANGE:
      case A_COMMAND_SHUT:
      
      if ( Answer.Len == 0 )
      {
        *ContextLen = Answer.Len;                            /* 02.08.08 */
        return 0;
      }
      else
      {
        *ContextPtr = Answer.Data;
        *ContextLen = Answer.Len;
        Answer.Data[Answer.Len] = 0;
        return 0;
      }

      default:
        return HRESAPI_UNEXPECTANSWER_ERR;
    }
  }
  return 1;
}


int HresApi(char * CommandName, char *UsrPassw, char *HostAddr, int Port, void *AddBuff, char **ContextPtr, int *ContextLen, int TimeoutSec)
{
  t_Command Command;
  int CmdCode;
  int Ret = 0;

  if ( !strcmp(CommandName , "exch") )
    CmdCode = COMMAND_EXCHANGE;
  else if ( !strcmp(CommandName , "stop" ) )
    CmdCode = COMMAND_STOP;
  else if ( !strcmp(CommandName , "shut" ) )
    CmdCode = COMMAND_SHUT;
  else if ( (!strcmp(CommandName , "getstate")) ||
          (!strcmp(CommandName , "show")) )
    CmdCode = COMMAND_GETS;
  else if ( !strcmp(CommandName , "main") )
    CmdCode = COMMAND_MAIN;
  else if ( !strcmp(CommandName , "slave") )
    CmdCode = COMMAND_SLAVE;
  else if ( !strcmp(CommandName , "testdb") )
    CmdCode = COMMAND_TESTDB;
  else if ( !strcmp(CommandName , "copy") )
    CmdCode = COMMAND_COPY;
  else if ( !strcmp(CommandName , "arc") )
    CmdCode = COMMAND_ARC;
  else if ( !strcmp(CommandName , "dead_node") )
    CmdCode = COMMAND_DEAD_NODE;
  else if (!strcmp(CommandName , "dead_server") )
    CmdCode = COMMAND_DEAD_SERVER;
  else
    return HRESAPI_COMMAND_ERR;
    
  memset(&Command, 0, sizeof(Command));
  Command.Command = CmdCode;


  if (CmdCode == COMMAND_DEAD_NODE || CmdCode == COMMAND_DEAD_SERVER )
  {
     Command.Len = 8;
     strncpy(Command.Data, AddBuff, 8);
  }   

  if (Ret = InitSocket())
    return Ret;
  
  if ((Command.Address.sin_addr.s_addr = inet_addr(HostAddr) ) == -1)
  {
    struct hostent* HostEnt;
    
    HostEnt = gethostbyname(HostAddr);
    if (HostEnt == NULL)
    {
      return HRESAPI_HOSTADDR_ERR;
    }
    memcpy((void *)(&Command.Address.sin_addr), HostEnt->h_addr_list[0], 
                                sizeof(Command.Address.sin_addr));
  }

  Command.Address.sin_family = AF_INET;
  Command.Address.sin_port = htons(1060);

  if (Port)
    Command.Address.sin_port = htons(Port);

  Command.UserPassword = UsrPassw;
  if (Ret = SendCmd(&Command))
    return Ret;

  return(ReceiveData(&Command.Address, ContextPtr, ContextLen, TimeoutSec));
}

struct sockaddr_in **getBackupNodes(void)
{
  static struct sockaddr_in *AddrArray[20];
  return AddrArray;
}
