/****************************************************************************
 *                           RELEX, Inc. DBMS Linter                        *
 *  This is unpublished propietary source code of RELEX, Inc.               *
 *  The copyright notice does not evidence any actual or intended           *
 *  publication of such source code.                                        *
 *                                                                          *
 *                  Copyright (c) 1990-2013 RELEX Inc.                      *
 *                                                                          *
 *                   This software was developed by                         *
 *                                                                          *
 *                            RELEX, Inc.                                   *
 *                         Voronezh, Russia                                 *
 *                                                                          *
 *                        All rights reserved.                              *
 ****************************************************************************
 AUTHOR: Maxim V. Kositsin -- 07.04.99
 ----------------------------------------------------------------------------
 PURPOSE: 64 bit arithmetic
 ----------------------------------------------------------------------------
 15.06.99 -- signed long overflow
 31.01.00 -- Maxim -- correct emulation
 15.05.00 -- Maxim -- correct negative divergention
 29.01.01 -- Maxim -- skip spaces before the first digit
 30.03.01 -- Maxim -- ignore end of string spaces
 24.09.01 -- Maxim -- add new finction double convert
 02.10.01 -- BOLT  -- Add i64_InvertBytes function
 15.10.01 -- Maxim -- correct signed compare
 15.10.01 -- Maxim -- correct error with signed compare
 17.10.01 -- Maxim -- correct double->bigint convert
 02.04.02 -- BOLT  -- Stack overflow if convert to string max bigint value
 03.04.02 -- Maxim -- make function static
 03.09.02 -- Galizin -- Remove global temporary variable PT#568
 23.08.03 -- Ermakov -- Remove warnings
 28.08.03 -- KOV   -- Added two functions, PT#3537
 28.09.05 -- FLY   -- Use another symbolic names for constants
 24.04.06 -- Galizin -- _i64_Div without recursion. Correct error neg div. Faster IsZero IsNeg.
 19.06.08 -- Ermakov -- avoid __moddi3 for GCC
 17.08.12 -- DV    -- Correct overflow check
*/
#include <stdio.h>
#include <string.h>        /* 13.05.99 */
#include "int64.h"


#ifdef INT64_EMULATION_ENABLE
L_DLONG __i64_ZeroConst = {0,0};
L_DLONG __i64_MaxDoubleConst = {0xa4c67fff,0x38d7e};
/*-----------------------------------------------------------
  Created:    24.09.01 11:35 by Maxim V. Kositsin
  Comments:
  -----------------------------------------------------------*/
L_DLONG i64_FromDouble(double d, L_DLONG x)
  {
  if ( d < 0.0 )
    {
    i64_FromPosDouble(-d,x);
    x=i64_Neg(x);
    }
  else
    i64_FromPosDouble(d,x); /* 17.10.01 */ 
  return x;
  }

/*-----------------------------------------------------------
  Created:    07.04.99 17:27 by Maxim V. Kositsin
  Comments:
  -----------------------------------------------------------*/
L_DLONG i64_NILDL(void)
  {
  L_DLONG __current__rest__;                                /* 03.09.02 */
  
  i64_Init(__current__rest__,0xffffffff,0xffffffff);
  return __current__rest__;
  }
/*-----------------------------------------------------------
  Created:    07.04.99 17:27 by Maxim V. Kositsin
  Comments:
  -----------------------------------------------------------*/
L_DLONG i64_MINDLONG(void)
  {
  L_DLONG __current__rest__;                                /* 03.09.02 */
  
  i64_Init(__current__rest__,0x00000000,0x80000000);
  return __current__rest__;
  }
/*-----------------------------------------------------------
  Created:    07.04.99 17:27 by Maxim V. Kositsin
  Comments:
  -----------------------------------------------------------*/
L_DLONG i64_MAXDLONG(void)
  {
  L_DLONG __current__rest__;                                /* 03.09.02 */
  
  i64_Init(__current__rest__,0xffffffff,0x7fffffff);
  return __current__rest__;
  }
/*-----------------------------------------------------------
  Created:    07.04.99 17:27 by Maxim V. Kositsin
  Comments:
  -----------------------------------------------------------*/
L_UDLONG i64_MAXUDLONG(void)
  {
  L_DLONG __current__rest__;                                /* 03.09.02 */
  
  i64_Init(__current__rest__,0xffffffff,0xffffffff);
  return __current__rest__;
  }
/*-----------------------------------------------------------
  Created:    05.04.99 15:15 by Maxim V. Kositsin
  Comments:
  -----------------------------------------------------------*/
L_DLONG i64_AddLong( L_DLONG num1, L_LONG num2 )
  {
  L_DLONG tmp;
  i64_FromLong(num2,tmp);
  return i64_Add(num1,tmp);
  }
/*-----------------------------------------------------------
  Created:    05.04.99 15:15 by Maxim V. Kositsin
  Comments:
  -----------------------------------------------------------*/
L_DLONG i64_SubLong( L_DLONG num1, L_LONG num2 )
  {
  L_DLONG tmp;
  i64_FromLong(num2,tmp);
  return i64_Sub(num1,tmp);
  }
/*-----------------------------------------------------------
  Created:    05.04.99 15:15 by Maxim V. Kositsin
  Comments:
  -----------------------------------------------------------*/
L_DLONG i64_MulLong( L_DLONG num1, L_LONG num2 )
  {
  L_DLONG tmp;
  i64_FromLong(num2,tmp);
  return i64_Mul(num1,tmp);
  }
/*-----------------------------------------------------------
  Created:    05.04.99 15:15 by Maxim V. Kositsin
  Comments:
  -----------------------------------------------------------*/
L_DLONG i64_DivLong( L_DLONG num1, L_LONG num2 )
  {
  L_DLONG tmp;
  i64_FromLong(num2,tmp);
  return i64_Div(num1,tmp);
  }
/*-----------------------------------------------------------
  Created:    05.04.99 15:15 by Maxim V. Kositsin
  Comments:
  -----------------------------------------------------------*/
L_DLONG i64_Add( L_DLONG num1, L_DLONG num2 )
  {
  L_LONG i;
  L_DLONG res;
  L_LONG rest;

  for ( i = 0 , rest = 0; i < 4 ; i++ )
    {
    rest = (L_LONG)i64_GetWord(num1,i) + (L_LONG)i64_GetWord(num2,i) + (rest>>16);
    i64_GetWord(res,i) = rest & 0xFFFF;
    } /* for */
  return res;
  }
/*-----------------------------------------------------------
  Created:    05.04.99 15:15 by Maxim V. Kositsin
  Comments:
  -----------------------------------------------------------*/
L_DLONG i64_Sub( L_DLONG num1, L_DLONG num2 )
  {
  L_LONG i;
  L_DLONG res;
  L_LONG rest;

  for ( i = 0 , rest = 0; i < 4 ; i++ )
    {
    rest = (L_LONG)i64_GetWord(num1,i) - (L_LONG)i64_GetWord(num2,i) - ((rest>>16)&1);
    i64_GetWord(res,i) = rest & 0xFFFF;
    } /* for */
  return res;
  }
/*-----------------------------------------------------------
  Created:    05.04.99 15:15 by Maxim V. Kositsin
  Comments:
  -----------------------------------------------------------*/
L_DLONG i64_Inc( L_DLONG *num )
  {
  L_LONG i;

  for ( i = 0 ; i < 4 ; i++ )
    {
    i64_GetWord(*num,i)++;
    if ( i64_GetWord(*num,i) )
      break;
    } /* for */
  return *num;
  }
/*-----------------------------------------------------------
  Created:    05.04.99 15:15 by Maxim V. Kositsin
  Comments:
  -----------------------------------------------------------*/
L_DLONG i64_Dec( L_DLONG *num )
  {
  L_LONG i;

  for ( i = 0 ; i < 4 ; i++ )
    {
    i64_GetWord(*num,i)--;
    if ( i64_GetWord(*num,i) != 0xFFFF )
      break;
    } /* for */
  return *num;
  }
/*-----------------------------------------------------------
  Created:    05.04.99 15:15 by Maxim V. Kositsin
  Comments:
  -----------------------------------------------------------*/
L_DLONG i64_Neg( L_DLONG num )
  {
  L_DLONG num1;
  i64_Init(num1,0,0);
  return i64_Sub( num1, num );
  }
/*-----------------------------------------------------------
  Created:    05.04.99 15:15 by Maxim V. Kositsin
  Comments:
  -----------------------------------------------------------*/
L_DLONG _i64_Mul_Word( L_DLONG num1, L_WORD num2 )
  {
  L_LONG i;
  L_DLONG res;
  L_LONG rest;

  for ( i = 0 , rest = 0; i < 4 ; i++ )
    {
    rest = i64_GetWord(num1,i) * (L_LONG)num2 + (rest>>16); /* 31.01.00 */
    i64_GetWord(res,i) = rest & 0xFFFF;
    } /* for */
  return res;
  }
/*-----------------------------------------------------------
  Created:    06.04.99 12:44 by Maxim V. Kositsin
  Comments:
  -----------------------------------------------------------*/
L_DLONG _i64_ShiftWordLeft( L_DLONG num )
  {
  i64_GetWord(num,3) = i64_GetWord(num,2);
  i64_GetWord(num,2) = i64_GetWord(num,1);
  i64_GetWord(num,1) = i64_GetWord(num,0);
  i64_GetWord(num,0) = 0;
  return num;
  }
/*-----------------------------------------------------------
  Created:    05.04.99 15:15 by Maxim V. Kositsin
  Comments:
  -----------------------------------------------------------*/
L_DLONG i64_Mul( L_DLONG num1, L_DLONG num2 )
  {
  L_DLONG n0,n1,n2,n3;
  n0 = _i64_Mul_Word ( num1, i64_GetWord(num2,0) );
  n1 = _i64_Mul_Word ( num1, i64_GetWord(num2,1) );
  n1 = _i64_ShiftWordLeft( n1 );
  n2 = _i64_Mul_Word ( num1, i64_GetWord(num2,2) );
  n2 = _i64_ShiftWordLeft( n2 );
  n2 = _i64_ShiftWordLeft( n2 );
  n3 = _i64_Mul_Word ( num1, i64_GetWord(num2,3) );
  n3 = _i64_ShiftWordLeft( n3 );
  n3 = _i64_ShiftWordLeft( n3 );
  n3 = _i64_ShiftWordLeft( n3 );
  n0 = i64_Add( n0, n1 );
  n0 = i64_Add( n0, n2 );
  n0 = i64_Add( n0, n3 );
  return n0;
  }
/*-----------------------------------------------------------
  Created:    16.10.01 12:45 by Maxim V. Kositsin
  Comments:
  -----------------------------------------------------------*/
L_LONG i64_CmpPositive ( L_DLONG num1, L_DLONG num2 )
  {
  if ( (L_LONG)i64_GetHi(num1) > (L_LONG)i64_GetHi(num2) ) /* 15.10.01 */ 
    return 1;
  else if ( (L_LONG)i64_GetHi(num1) < (L_LONG)i64_GetHi(num2) ) /* 15.10.01 */ 
    return -1;
  else if ( i64_GetLo(num1) > i64_GetLo(num2) )
    return 1;
  else if ( i64_GetLo(num1) < i64_GetLo(num2) )
    return -1;
  return 0;
  }
/*-----------------------------------------------------------
  Created:    07.04.99 11:19 by Maxim V. Kositsin
  Comments:
  -----------------------------------------------------------*/
L_LONG i64_Cmp ( L_DLONG num1, L_DLONG num2 )
  {
  L_DLONG zero,n1,n2;
  if ( i64_IfNeg(num1) ) /* num1 < 0 */
    {
    if ( i64_IfNeg(num2) ) /* num2 < 0 */
      {
      zero = I64_ZERO;
      n1 = i64_Sub(zero,num1);
      n2 = i64_Sub(zero,num2);
      return i64_CmpPositive(n2,n1);
      }
    else
      return -1;
    }
  else
    {
    if ( i64_IfNeg(num2) ) /* num2 < 0 */
      return 1;
    else
      return i64_CmpPositive(num1,num2);
    }
  }
/*-----------------------------------------------------------
  Created:    07.04.99 11:19 by Maxim V. Kositsin
  Comments:
  -----------------------------------------------------------*/
L_DLONG i64_And ( L_DLONG num1, L_DLONG num2 )
  {
  i64_GetHi(num1) &= i64_GetHi(num2);
  i64_GetLo(num1) &= i64_GetLo(num2);
  return num1;
  }
/*-----------------------------------------------------------
  Created:    07.04.99 11:19 by Maxim V. Kositsin
  Comments:
  -----------------------------------------------------------*/
L_DLONG i64_Or ( L_DLONG num1, L_DLONG num2 )
  {
  i64_GetHi(num1) |= i64_GetHi(num2);
  i64_GetLo(num1) |= i64_GetLo(num2);
  return num1;
  }
/* 28.08.03 ...*/
/*-----------------------------------------------------------
  Created:    28.08.03 21:46 by Dmitry Koval
  Comments:
  -----------------------------------------------------------*/
L_DLONG i64_Xor ( L_DLONG num1, L_DLONG num2 )
  {
  i64_GetHi(num1) ^= i64_GetHi(num2);
  i64_GetLo(num1) ^= i64_GetLo(num2);
  return num1;
  }
/*-----------------------------------------------------------
  Created:    28.08.03 21:47 by Dmitry Koval
  Comments:
  -----------------------------------------------------------*/
L_DLONG i64_Not ( L_DLONG num )
  {
  i64_GetHi(num) = ~i64_GetHi(num);
  i64_GetLo(num) = ~i64_GetLo(num);
  return num;
  }
/*... 28.08.03 */

                                                            /* 24.04.06 .. */
#define _i64_ShiftBitLeft(num) \
  i64_GetHi(num) = i64_GetHi(num) << 1;\
  if ( i64_GetLo(num) & 0x80000000 )\
    i64_GetHi(num) |= 1;\
  i64_GetLo(num) = i64_GetLo(num) << 1;

#define _i64_ShiftBitRight(num) \
  i64_GetLo(num) = i64_GetLo(num) >> 1;\
  if ( i64_GetHi(num) & 0x1 )\
    i64_GetLo(num) |= 0x80000000;\
  i64_GetHi(num) = i64_GetHi(num) >> 1;

/* Sub for unsigned */
#define i64_USub(num1, num2) \
 { int Carry = (i64_GetLo(num1) < i64_GetLo(num2));\
   i64_GetLo(num1) -= i64_GetLo(num2);\
   i64_GetHi(num1) -= (i64_GetHi(num2) + Carry); }

/*-----------------------------------------------------------
  Created:    05.04.99 15:15 by Maxim V. Kositsin
  Comments:
  -----------------------------------------------------------*/
static void _i64_Div( L_DLONG *num1, L_DLONG num2, L_DLONG *res) /* 03.04.02 */ 
{
  L_DLONG bit;
  
  i64_Init(bit, 1, 0);
  *res = I64_ZERO;
  
  while ( !(i64_IfNeg(num2)) && 
           !i64_IfZero(bit) && 
           /* UNSIGNED *num1 > num2 */
           ( (i64_GetHi(*num1) != i64_GetHi(num2)) ? 
              (i64_GetHi(*num1) > i64_GetHi(num2)):
              (i64_GetLo(*num1) > i64_GetLo(num2))
           )
        )
  {
    _i64_ShiftBitLeft(num2);
    _i64_ShiftBitLeft(bit);
  }
  
  while ( !i64_IfZero(bit) )
  {
    /* *num1 >= num2 */
    if ( (i64_GetHi(*num1) != i64_GetHi(num2)) ? 
            (i64_GetHi(*num1) >= i64_GetHi(num2)):
            (i64_GetLo(*num1) >= i64_GetLo(num2)) )
    {
      /* *num1 = *num1 - num2 */
      i64_USub(*num1, num2);
      /* *res = *res | bit */
      i64_GetLo(*res) |= i64_GetLo(bit);
      i64_GetHi(*res) |= i64_GetHi(bit);
    }
    _i64_ShiftBitRight(num2);
    _i64_ShiftBitRight(bit);
  }

  return;
}                                                           /* .. 24.04.06 */

/*-----------------------------------------------------------
  Created:    05.04.99 15:15 by Maxim V. Kositsin
  Comments:
  -----------------------------------------------------------*/
L_DLONG i64_DivRest( L_DLONG num1, L_DLONG num2 , L_DLONG *rest )
  {
  L_DLONG res;
  L_LONG lminus,rminus=0; /* 15.05.00 */

  if ( i64_IfZero ( num2 ) )
    {
    i64_Init(res,0xffffffff,0x7fffffff);
    return res;
    }
  lminus = (i64_IfNeg ( num1 ) != i64_IfNeg ( num2 ));
  if ( i64_IfNeg ( num1 ) )
  {
    rminus = 1;                                             /* 24.04.06 */
    num1 = i64_Neg ( num1 );
  }
  if ( i64_IfNeg ( num2 ) )
    num2 = i64_Neg ( num2 );
  _i64_Div( &num1, num2, &res);
  if ( lminus )
    res = i64_Neg ( res );
  if ( rest )
    {
    if ( rminus ) /* 15.05.00 */
      num1 = i64_Neg ( num1 );
    *rest = num1;
    }
  return res;
  }
/*-----------------------------------------------------------
  Created:    05.04.99 15:15 by Maxim V. Kositsin
  Comments:
  -----------------------------------------------------------*/
L_DLONG i64_Div( L_DLONG num1, L_DLONG num2 )
  {
  return i64_DivRest( num1, num2, NULL );
  }
/*-----------------------------------------------------------
  Created:    05.04.99 15:15 by Maxim V. Kositsin
  Comments:
  -----------------------------------------------------------*/
L_DLONG i64_Rest( L_DLONG num1, L_DLONG num2 )
  {
  L_DLONG rest;
  i64_DivRest( num1, num2, &rest );
  return rest;
  }
/*-----------------------------------------------------------
  Created:    05.04.99 15:15 by Maxim V. Kositsin
  Comments:
  -----------------------------------------------------------*/
/*-----------------------------------------------------------
  Created:    05.04.99 15:15 by Maxim V. Kositsin
  Comments:
  -----------------------------------------------------------*/
L_DLONG i64_RestLong( L_DLONG num1, L_LONG num2 )
  {
  L_DLONG tmp;
  i64_FromLong(num2,tmp);
  return i64_Rest(num1,tmp);
  }

#endif /* INT64_EMULATION_ENABLE */

/*----------------- 02.10.01  18:48:10 -----------------
  Created:  by Andrew BOLT (Boldyrev)
  Comments: 
------------------------------------------------------*/
void   i64_InvertBytes(L_DLONG * Val)
{
  L_BYTE    TmpTyp[sizeof(L_DLONG)] = {0};
  L_BYTE    *p = (L_BYTE*)Val;
  int       i  = 0;
  memcpy(TmpTyp, Val, sizeof(L_DLONG));
  for (i; i < sizeof(L_DLONG); i++)
    memcpy(p + i, &TmpTyp[sizeof(L_DLONG) - i - 1], 1);
}

/*-----------------------------------------------------------
  Created:    07.04.99 13:59 by Maxim V. Kositsin
  Comments:
  -----------------------------------------------------------*/
L_DLONG i64_atol ( L_CHAR *str )
  {
  L_DLONG num,_10,tmp;
  L_LONG     lminus = 0;

  while ( *str == ' ' || *str == '\t' ) /* 29.01.01 */
    str++;

  if ( *str == '-' )
    {
    str++;
    lminus = 1;
    } /* if */
  i64_Init(num,0,0);
  i64_Init(_10,10,0);
  while ( *str >= '0' && *str <= '9' )
    {
    i64_Init(tmp,*str-'0',0);
    num = i64_Mul ( num , _10 );
    num = i64_Add ( num , tmp );
  str++;
    } /* while */
  if ( lminus )
    num = i64_Neg ( num );
  return num;
  }
/*-----------------------------------------------------------
  Created:    07.04.99 13:59 by Maxim V. Kositsin
  Comments:
  -----------------------------------------------------------*/
char i64_maxnum_string[]="9223372036854775807";
char i64_minnum_string[]="9223372036854775808";         /* 17.08.12 */
L_DLONG i64_atol_ext ( L_CHAR *str , L_WORD *err )
  {
  L_DLONG num,_10,tmp;
  L_LONG     lminus = 0;
  L_CHAR  *s;

  while ( *str == ' ' || *str == '\t' ) /* 29.01.01 */
    str++;
  if ( *str == '-' )
    {
    str++;
    lminus = 1;
    } /* if */
  *err = I64_NOERR;
  s = str;
  i64_Init(num,0,0);
  i64_Init(_10,10,0);
  while ( *str >= '0' && *str <= '9' )
    {
    i64_Init(tmp,*str-'0',0);
    num = i64_Mul ( num , _10 );
    num = i64_Add ( num , tmp );
    str++;
    } /* while */
  if ( ( *str || str == s ) && *str != ' ' ) /* 30.03.01 */
    *err = I64_ERR_NODIGIT;
  if ( lminus && str-s == 19 && strncmp(s,i64_minnum_string,19) == 0)      /* 17.08.12 ... */
    {
    i64_Init(num,0x00000000,0x80000000);
    return num;
    }
  if ( str-s > 19 || (str-s == 19 && strncmp(s,i64_maxnum_string,19)>0) ) /* ... 17.08.12 */
    *err = I64_ERR_OVERFLOW;
  if ( lminus )
    num = i64_Neg ( num );
  return num;
  }
/*-----------------------------------------------------------
  Created:    07.04.99 13:59 by Maxim V. Kositsin
  Comments:
  -----------------------------------------------------------*/
L_CHAR * i64_ltoa ( L_DLONG num , L_CHAR *str )
  {
  L_DLONG _10;
#ifdef INT64_EMULATION_ENABLE        /* 13.05.99 */
  L_DLONG rest;
#endif
  L_LONG     lminus = 0;
  L_CHAR s[40], *ptr = s + 38;

#ifndef INT64_EMULATION_ENABLE
  if (num == L_MINDLONG)                  /* 28.09.05 ... */
#else
  if ( i64_IfZero(i64_Sub(num, L_MINDLONG)) )                                /* 28.09.05 */
#endif                            /* ... 28.09.05 */
    {
    strcpy ( str , "-9223372036854775808" );
    return str;
    }

  if ( i64_IfNeg (num) )
    {
    lminus = 1;
    num = i64_Neg ( num );
    } /* if */
  i64_Init(_10,10,0);
  *ptr-- = 0;
  do
    {
#ifdef INT64_EMULATION_ENABLE
    num = i64_DivRest ( num , _10, &rest );
    *ptr-- = '0' + (char)i64_GetLo(rest);          /* 28.09.05 */
#else
    *ptr-- = '0' + (char)((L_UDLONG)num%10);       /* 19.06.08 */ /* 23.08.03 */
    num /= 10;
#endif
    } while ( !i64_IfZero (num) );
  if ( lminus )
    *ptr-- = '-';
  strcpy ( str , ptr+1 );
  return str;
  }
/*-----------------------------------------------------------
  Created:    19.04.99 11:47 by Maxim V. Kositsin
  Comments:   if overflow - return FALSE (0)
  -----------------------------------------------------------*/
L_BOOL i64_TestToLongOfl ( L_DLONG num )
  {
  if ( i64_GetHi ( num ) != 0 && i64_GetHi ( num ) != 0xFFFFFFFF )
    return L_TFALSE;
  return L_TTRUE;
  }
/*-----------------------------------------------------------
  Created:    15.06.99 13:57 by Maxim V. Kositsin
  Comments:
  -----------------------------------------------------------*/
L_BOOL i64_TestToSLongOfl ( L_DLONG num )
  {
  if ( i64_TestToLongOfl(num) == L_TFALSE ||
       ((!i64_GetHi(num)) && (i64_GetLo(num)&0x80000000)) ||
       (i64_GetHi(num) && !(i64_GetLo(num)&0x80000000)) )
    return L_TFALSE;
  return L_TTRUE;
  }
/*-----------------------------------------------------------
  Created:    19.04.99 11:47 by Maxim V. Kositsin
  Comments:   if overflow - return FALSE
  -----------------------------------------------------------*/
L_BOOL i64_TestToDoubleOfl ( L_DLONG num )
  {
  if ( i64_IfNeg (num) )
    num = i64_Neg (num);
  if ( i64_Cmp ( num , I64_MAXDOUBLE ) > 0 )
    return L_TFALSE;
  return L_TTRUE;
  }
/*-----------------------------------------------------------
  Created:    19.04.99 11:47 by Maxim V. Kositsin
  Comments:   if overflow - return FALSE
  -----------------------------------------------------------*/
L_BOOL i64_TestFromDoubleOfl ( double d )
  {
  if ( d < 0 )
    d = -d;
  if ( d > I64_DLIMIT )
    return L_TFALSE;
  return L_TTRUE;
  }

#undef i64_IfNeg                                            /* 24.04.06 */
#undef i64_IfZero                                           /* 24.04.06 */
/*-----------------------------------------------------------
  Created:    05.04.99 15:15 by Maxim V. Kositsin
  Comments:
  -----------------------------------------------------------*/
L_LONG i64_IfNeg( L_DLONG num )
  {
  if ( i64_GetHi(num) & 0x80000000 )
    return 1;
  return 0;
  }
/*-----------------------------------------------------------
  Created:    05.04.99 15:15 by Maxim V. Kositsin
  Comments:
  -----------------------------------------------------------*/
L_LONG i64_IfZero( L_DLONG num )
  {
  if ( i64_GetHi(num) == 0 && i64_GetLo(num) == 0 )
    return 1;
  return 0;
  }


/* #define ___TEST_I64_LIBRARY_PROGRAM___ */
#ifdef ___TEST_I64_LIBRARY_PROGRAM___

lll = 0;
L_DLONG GetRandom ( void )
  {
  L_DLONG n;
  L_LONG i,j;
  if ( lll & 1 )
  {
    j = (rand () % 4) + 1;
  }
  else
    j = 4;

  i64_Init(n,0,0);
  for ( i = 0 ; i < j ; i++ )
    i64_GetWord(n,i) = rand ();
  return n;
  }

main ()
{
__int64 i,j,i1;
L_DLONG k,l,k1,a;
L_LONG allnum=0;

long aaa;

char s1[100], s2[100], str1[100],str2[100];

i64_Init( a , 0 , 0x80000000 );
i64_ltoa( a , s1 );
printf ( "%s\n", s1 );

i64_Init( l , 1 , 0 );
k = i64_Div( a , l );
i64_ltoa( a , s1 );
printf ( "%s\n", s1 );

i64_Init( l , 0xFFFFFFFF , 0xFFFFFFFF );
k = i64_Div( a , l );
i64_ltoa( a , s1 );
printf ( "%s\n", s1 );

/*
a = 0x80000000 << 1;

i64_Init( a , 0 , 0x80000000 );

k = a*-1;
k = a/-1;
k = a/-2;
k = k*-2;
k = a/2;


i64_ltoa( a , s1 );
printf ( "%s\n", s1 );

#ifdef INT64_EMULATION_ENABLE
aaa = i64_GetLo(i64_RestLong(l,10));
#endif
*/
while ( !kbhit() )
  {
  if ( (allnum++ & 0xFF) == 0 )
    {
  lll++;
    printf ( "\r%d OK" , allnum );
    fflush ( stdout );
    } /* if */
beg:
  k = GetRandom ();
  l = GetRandom ();
  memcpy ( &i , &k , 8 );
  memcpy ( &j , &l , 8 );
  if ( !j )
    goto beg;
  i1 = i/j;
  k1 = i64_Div(k,l);

  if ( /*strcmp(str1,str2)*/memcmp ( &i1 , &k1 , 8 ) != 0 )
    {
    _i64toa(i,s1,16);
    _i64toa(j,s2,16);
    /*
    i64_ltoa ( *((__int64*)&i1) , str  );
    i64_ltoa ( *((__int64*)&k1) , str1 );
    */
    printf ( "Div Error!!! (%s/%s) %s != %s\n", s1,s2,str1 , str2 );
    goto end;
    } /* if */
  i1 = i*j;
  k1 = i64_Mul(k,l);
  if ( /*strcmp(str1,str2)*/memcmp ( &i1 , &k1 , 8 ) != 0 )
    {
    _i64toa(i,s1,16);
    _i64toa(j,s2,16);
    /*
    i64_ltoa ( *((__int64*)&i1) , str );
    i64_ltoa ( *((__int64*)&k1) , str1 ); */
    printf ( "Mul Error!!! (%s*%s) %s != %s\n", s1,s2,str1 , str2 );
    goto end;
    return 1;
    } /* if */
  i1 = i+j;
  k1 = i64_Add(k,l);
  if ( /*strcmp(str1,str2)*/memcmp ( &i1 , &k1 , 8 ) != 0 )
    {
    _i64toa(i,s1,16);
    _i64toa(j,s2,16);
    /*
    i64_ltoa ( *((__int64*)&i1) , str );
    i64_ltoa ( *((__int64*)&k1) , str1 ); */
    printf ( "Add Error!!! (%s+%s) %s != %s\n", s1,s2,str1 , str2 );
    goto end;
    return 1;
    } /* if */
#if 1
  i1 = i-j;
  k1 = i64_Sub(k,l);
  if ( /*strcmp(str1,str2)*/memcmp ( &i1 , &k1 , 8 ) != 0 )
    {
    _i64toa(i,s1,16);
    _i64toa(j,s2,16);
    /*
    i64_ltoa ( *((__int64*)&i1) , str );
    i64_ltoa ( *((__int64*)&k1) , str1 ); */
    printf ( "Sub Error!!! (%s-%s) %s != %s\n", s1,s2,str1 , str2 );
    goto end;
    return 1;
    } /* if */
  {
  char sss[100];
  i64_ltoa ( k , sss );
  l = i64_atol ( sss );
  if ( memcmp ( &l , &k , 8 ) != 0 )
    {
    printf ( "Atol Error!!! %s \n", sss );
    goto end;
    return 1;
    } /* if */


  }

#endif
  } /* while */
return 1;
end:
  printf ( "Press a key..." );
  fflush ( stdout );
  getch();
  return 1;

}

#endif /* ___TEST_I64_LIBRARY_PROGRAM___ */
