/***************************************************************************
 *                           RELEX, Inc. DBMS Linter                        *
 *  This is unpublished propietary source code of RELEX, Inc.               *
 *  The copyright notice does not evidence any actual or intended           *
 *  publication of such source code.                                        *
 *                                                                          *
 *                  Copyright (c) 1990-2013 RELEX Inc.                      *
 *                                                                          *
 *                   This software was developed by                         *
 *                                                                          *
 *                            RELEX, Inc.                                   *
 *                         Voronezh, Russia                                 *
 *                                                                          *
 *                        All rights reserved.                              *
 ****************************************************************************
 AUTHOR: Maxim V. Kositsin -- 07.04.99
 ----------------------------------------------------------------------------
 PURPOSE: 64 bit arithmetic
 ----------------------------------------------------------------------------
 15.06.99 -- signed long overflow
 21.07.99 -- DV (MAX values for Sparc SUN)
 30.09.99 -- DV (_BSD_)
 18.05.00 -- Kent (Declare all functions as extern "C")
 27.04.01 -- KOV  (Added 4 defines)
 18.09.01 -- Maxim -- correct macros
 24.09.01 -- Maxim add new finction double convert
 02.10.01 -- BOLT  Add i64_InvertBytes function
 28.08.03 -- KOV   Added two functions, PT#3537
 28.09.05 -- FLY   Use constants from lintypes.h
 24.04.06 -- Galizin (Defines for IfZero && IfNeg are faster )
 03.07.12 -- Ganin (Changed to use with MIDL)
 16.11.12 -- Glukhov,Ermakov (Remove extra code, datatypes)
*/                     
#ifndef __INT64_H__
#define __INT64_H__

#ifndef __64BIT_KERNEL__
#define __64BIT_KERNEL__
#endif

#ifndef __midl                                             /* 03.07.12 */

#include "lintypes.h"

#ifdef INT64_EMULATION_ENABLE

L_DLONG  i64_Add( L_DLONG num1, L_DLONG num2 );
L_DLONG  i64_Sub( L_DLONG num1, L_DLONG num2 );
L_LONG   i64_IfNeg( L_DLONG num );
L_LONG   i64_IfZero( L_DLONG num );
L_DLONG  i64_Neg( L_DLONG num );
L_DLONG  i64_Mul( L_DLONG num1, L_DLONG num2 );
L_LONG   i64_Cmp ( L_DLONG num1, L_DLONG num2 );
L_DLONG  i64_And ( L_DLONG num1, L_DLONG num2 );
L_DLONG  i64_Or ( L_DLONG num1, L_DLONG num2 );
L_DLONG  i64_Xor ( L_DLONG num1, L_DLONG num2 );            /* 28.08.03 */
L_DLONG  i64_Not ( L_DLONG num );                           /* 28.08.03 */
L_DLONG  i64_Div( L_DLONG num1, L_DLONG num2 );
L_DLONG  i64_Inc( L_DLONG *num );
L_DLONG  i64_Dec( L_DLONG *num );
L_DLONG  i64_NILDL(void);
L_DLONG  i64_MINDLONG(void);
L_DLONG  i64_MAXDLONG(void);
L_UDLONG i64_MAXUDLONG(void);
L_DLONG i64_AddLong( L_DLONG num1, L_LONG num2 );
L_DLONG i64_SubLong( L_DLONG num1, L_LONG num2 );
L_DLONG i64_MulLong( L_DLONG num1, L_LONG num2 );
L_DLONG i64_DivLong( L_DLONG num1, L_LONG num2 );
L_DLONG i64_DivRest( L_DLONG num1, L_DLONG num2 , L_DLONG *rest );
L_DLONG i64_Rest( L_DLONG num1, L_DLONG num2 );
L_DLONG i64_RestLong( L_DLONG num1, L_LONG num2 );

#define i64_ToLong(i,l)     (l=i64_GetLo(i))
#define i64_FromLong(l,i)   (i64_GetLo(i)=l,i64_GetHi(i)=(l)<0?0xFFFFFFFF:0)
#define i64_ToPosDouble(i,d) (d=(double)i64_GetLo(i)+(double)i64_GetHi(i)*((double)0xFFFFFFFF+1))
#define i64_ToDouble(i,d)   (i64_IfNeg(i)?(i=i64_Neg(i),i64_ToPosDouble(i,d),i=i64_Neg(i),d=-d):(i64_ToPosDouble(i,d)))
#define i64_FromPosDouble(d,i) (i64_GetHi(i)=(d)/((double)0xFFFFFFFF+1),i64_GetLo(i)=(d)-((double)i64_GetHi(i))*((double)0xFFFFFFFF+1))
/* #define i64_FromDouble(d,x) ((d<0.0)?(i64_FromPosDouble(-d,x),x=i64_Neg(x)):i64_FromPosDouble(-d,x)) */
L_DLONG i64_FromDouble(double d, L_DLONG x); /* 24.09.01 */ 


#define I64_ZERO          __i64_ZeroConst
#define I64_MAXDOUBLE     __i64_MaxDoubleConst

extern L_DLONG            __i64_ZeroConst;
extern L_DLONG            __i64_MaxDoubleConst;

#define i64_GetLo(i)      ((i).lo)
#define i64_GetHi(i)      ((i).hi)
#define i64_Init(i,lo,hi) (i64_GetLo(i)=lo,i64_GetHi(i)=hi)

#define i64_IfNeg(num) ( (i64_GetHi(num) & 0x80000000) != 0 )  /* 24.04.06 */
#define i64_IfZero(num) ( i64_GetHi(num) == 0 && i64_GetLo(num) == 0 ) /* 24.04.06 */
#define i64_IfPos(num)       (!i64_IfNeg(num)&&!i64_IfZero(num))

#define i64_Equ(n1,n2)       (memcmp(&(n1),&(n2),8)==0)
# ifdef DATA_MSBF
#  define i64_Initializer(lo, hi) { hi, lo }
# else
#  define i64_Initializer(lo, hi) { lo, hi }
# endif
#else

#if defined(_MSC_VER) && _MSC_VER >= 900 || defined (__WATCOMC__)
# define  I64(i) i ##  i64
# define UI64(i) i ## ui64
#elif defined(__GNUC__) || \
     (defined(SUN) && defined (DATA_MSBF)) || \
      defined(_BSD_)                                       /* 21.07.99 ... 30.09.99 */
# define  I64(i) i ##  LL
# define UI64(i) i ## ULL
#else
# define  I64(i) i
# define UI64(i) i                                         /* ... 21.07.99 */
#endif

#define I64_ZERO              0
#define I64_MAXDOUBLE     I64(0x00038d7ea4c67fff)
#define I64_MAXNEGATIVE  UI64(0x8000000000000000)

#define i64_Add(i1,i2)       ((i1)+(i2))
#define i64_Sub(i1,i2)       ((i1)-(i2))
#define i64_Mul(i1,i2)       ((i1)*(i2))
#define i64_Div(i1,i2)       ((i1)/(i2))
#define i64_Neg(num)         (-(num))
#define i64_IfNeg(num)       ((num)<0)
#define i64_IfPos(num)       ((num)>0)
#define i64_IfZero(num)      ((num)==0)
#define i64_Cmp(n1,n2)       ((n1)<(n2)?-1:(n1)>(n2))
#define i64_Equ(n1,n2)       ((n1)==(n2))
#define i64_And(n1,n2)       ((n1)&(n2))
#define i64_Or(n1,n2)        ((n1)|(n2))
#define i64_Xor(n1,n2)       ((n1)^(n2))                    /* 28.08.03 */
#define i64_Not(n)           (~n)                           /* 28.08.03 */
#define i64_Inc(num)         ((*num)++)
#define i64_Dec(num)         ((*num)--)
#define i64_NILDL()          (L_NILDL)                     /* 27.04.01 ... */
#define i64_MINDLONG()       (L_MINDLONG)
#define i64_MAXDLONG()       (L_MAXDLONG)
#define i64_MAXUDLONG()      (L_MAXUDLONG)                 /* ... 27.04.01 */
#define i64_AddLong          i64_Add
#define i64_SubLong          i64_Sub
#define i64_MulLong          i64_Mul
#define i64_DivLong          i64_Div
#define i64_DivRest(n1,n2,r) (*r=(n1)%(n2),(n1)/(n2))
#define i64_Rest(n1,n2)      ((n1)%(n2))
#define i64_RestLong         i64_Rest
#define i64_ToLong(i,l)      ((l)=(L_ULONG)(i))        /* 28.09.05 */
#define i64_FromLong(l,i)    ((i)=(l))
#define I64_ZERO             0
#define i64_GetLo(i)         ((L_ULONG)((i)&0xffffffff))
#define i64_GetHi(i)         ((L_ULONG)((i)>>32))
#define i64_Init(i,lo,hi)    ((i)=(((L_DLONG)(hi))<<32)+(lo))
#define i64_ToDouble(i,d)    ((d)=(double)(i))
#define i64_FromDouble(d,i)  ((i)=(L_DLONG)(d))
#define i64_Initializer(lo,hi) ((((L_DLONG)(hi)) << 32) + (lo))
#endif

#define I64_DLIMIT        9223372036854776000.000000

#ifdef __cplusplus                                         /* 18.05.00 ... */
extern "C" {
#endif

L_DLONG i64_atol ( L_CHAR *str );
L_DLONG i64_atol_ext ( L_CHAR *str , L_WORD *err );
#define I64_NOERR         0
#define I64_ERR_NODIGIT   1
#define I64_ERR_OVERFLOW  2
L_CHAR * i64_ltoa ( L_DLONG num , L_CHAR *str );
L_BOOL i64_TestToLongOfl ( L_DLONG num );
L_BOOL i64_TestToDoubleOfl ( L_DLONG num );
L_BOOL i64_TestToSLongOfl ( L_DLONG num );    /* 15.06.99 */ 
L_BOOL i64_TestFromDoubleOfl ( double d );
void   i64_InvertBytes(L_DLONG * Val);        /* 02.10.01 */

#ifdef __cplusplus                                         /* ... 18.05.00 */
}
#endif

#ifdef DATA_MSBF
#define i64_GetWord(i,n)  (((L_WORD *)(&i))[3-n])
#else
#define i64_GetWord(i,n)  (((L_WORD *)(&i))[n])
#endif

#endif                                                     /* 03.07.12 */

#endif /* __INT64__ */
