/****************************************************************************
 *                       RELEX, Inc. DBMS Linter                            *
 *  This is unpublished proprietary source code of RELEX, Inc.              *
 *  The copyright notice does not evidence any actual or intended           *
 *  publication of such source code.                                        *
 *                                                                          *
 *                  Copyright (c) 1990-2013 RELEX Inc.                      *
 *                                                                          *
 *                   This software was developed by                         *
 *                                                                          *
 *                            RELEX, Inc.                                   *
 *                         Voronezh, Russia                                 *
 *                                                                          *
 *                        All rights reserved.                              *
 ****************************************************************************
 AUTHOR: Glukhov
 ----------------------------------------------------------------------------
 PURPOSE: Common intlib definition
 ----------------------------------------------------------------------------
 REVISION HISTORY:
 15.04.2013 - Initial revision
 24.08.14 - Galizin, Pashentsev (Kerberos authentication)
*/

#ifndef __INTLIB2_H__
#define __INTLIB2_H__

#include "lintypes.h"

#define M_BINARY     0x00000000
#define M_SPEC       0x00000003
#define M_OPTIMISTIC 0x00000100
#define M_SHARE      0x00000200
#define M_EXCLUSIVE  0x00000400
#define Q_ASYNC      0x00001000
#define Q_ASYNCDONE  0x00002000
#define Q_ENCODE     0x00004000
#define Q_KOI8       0x00008000
#define Q_866        (Q_ENCODE+Q_KOI8)  /* Is cleared in INTLIB */ /* 24.01.00 */ /* 01.02.12 */
#define Q_COMPRESS   0x00010000
#define Q_RECEIVE    0x00020000
#define Q_LOLTP      0x00040000
#define Q_REPLICATOR 0x00100000
#define Q_EV_CATCH   0x00200000
#define Q_USE_UTF8   0x00400000
#define Q_USE_ADO    0x00800000

#if _VER_MAX >= 610
# define M_READ_UNCOMMITTED 0x04000000
# define M_READ_COMMITTED   0x01000000
# define M_SERIALIZABLE     0x02000000
#endif

#define M_LARGE_EXCHANGE    0x10000000                /* 19.10.09 */
#define M_SEC_LABELS        0x20000000
#define Q_KRBREQ            0x40000000                /* 24.08.14 ... */


/*Inter Control requests for `inter_control' */
#define ICR_ENABLE_MSG  0
#define ICR_GET_CP      1
#define ICR_SET_CP      2
#define ICR_ADD_NODE    3
#define ICR_DEL_NODE    4
#define ICR_USE_OEM_CP  5                                  /* 26.12.01 */
#define ICR_USE_UTF8    6                                  /* 26.12.01 */
#define ICR_REREAD_ENV  7                                  /* 24.06.02 */
#define ICR_SET_STACK_SIZE  8                              /* 10.05.12 */
#define ICR_FORCE_KRB   9
                                                      /* ... 24.08.14 */

#ifdef _ENGLATIN1
# define ANSI_CP  "ANSI"
# define ISO_CP   "ISO"
# define OEM_CP   "OEM"
# define ANSI_CP6 "ANSI"                             /* 01.08.10 ... */
# define ISO_CP6  "ISO"
# define OEM_CP6  "OEM"                              /* ... 01.08.10 */
# define INET_ANSI_CP "windows-1252"
# define INET_OEM_CP  "ibm850"
# define INET_ISO_CP  "iso-8859-1"
#else
# define ANSI_CP  "1251"
# define ISO_CP   "koi8"
# define OEM_CP   "866"
# define ANSI_CP6 "CP1251"                           /* 01.08.10 ... */
# define ISO_CP6  "KOI8-R"
# define OEM_CP6  "CP866"                            /* ... 01.08.10 */
# define INET_ANSI_CP "windows-1251"
# define INET_OEM_CP  "cp866"
# define INET_ISO_CP  "koi8-r"
#endif

/* -- Control block --------------------------------------------------------- */

typedef struct TCBL
{
  L_LONG       CodErr;      /* Error code                  */
  L_WORD       Prior;       /* Channel priority            */
  L_WORD       NumChan;     /* Channel number              */
  L_CHAR       UserName[4]; /* User name                   */
  L_CHAR       Command[4];  /* Command name                */
  L_CHAR       Node[8];     /* Node name                   */
  L_LONG       RowId;       /* Row number                  */
  L_LONG       RowCount;    /* Rows account                */
  L_LONG       PrzExe;      /* Query's accomplishment flag */
  L_LONG       SysErr;      /* System error                */
  L_WORD       LnBufRow;    /* Row buffer length           */
  L_WORD       CharSet;                                      
} TCBL;

#define LENCBL sizeof(TCBL)

#ifndef MAX_IDENT
#define MAX_IDENT 66
#endif

typedef struct 
{
  L_BYTE  SavePoint;
  L_BYTE  Named;
  L_CHAR  Name[MAX_IDENT];
} SAVE_POINT;

typedef struct
{
  L_WORD cp_type;
  L_WORD cp_len;
  L_WORD pad;
  union
  {
    L_WORD n_cp;
    L_CHAR s_cp[MAX_IDENT];
  } cp;
} LCODEPAGE;

#define CP_AS_NUM       0
#define CP_AS_STR       1

#if defined(INTER_MSWINDOWS) && !defined(WIN32) && !defined(WIN16)
#  define HPCHAR char huge *
#  define HPVOID void huge *
#else
#  define HPCHAR char *
#  define HPVOID void *
#endif

#ifndef HPCHAR
#  define HPCHAR char *
#endif
#ifndef HPVOID
#  define HPVOID void *
#endif

typedef void (*USR_PROC)(TCBL *, HPVOID, HPVOID);

#endif /* __INTLIB2_H__ */
