 /***************************************************************************
 *           RELEX, Inc. Linter API (version 3.0.44-6x) include file        *
 *  This is unpublished proprietary source code of RELEX, Inc.              *
 *  The copyright notice does not evidence any actual or intended           *
 *  publication of such source code.                                        *
 *                                                                          *
 *                 Copyright (c) 1997-2014 RELEX Inc.                       *
 *                                                                          *
 *                   This software was developed by                         *
 *                                                                          *
 *                            RELEX, Inc.                                   *
 *                         Voronezh, Russia                                 *
 *                                                                          *
 *                        All rights reserved.                              *
 ****************************************************************************
 AUTHOR: Anatoly Peshkoff, Pavel Pasechnik (Pasec)
 ----------------------------------------------------------------------------
 PURPOSE: to define LINAPI for user's program
 ----------------------------------------------------------------------------
 REVISION HISTORY: last modification date -- 18.01.06
 ----------------------------------------------------------------------------
  
  28.02.96 -- creation date

  08.11.96 -- renamed LINTER_ShutDown to LINTER_CloseAPI
  08.11.96 -- return values
  08.11.96 -- new functions LINTER_Error, LINTER_ErrorMessage 
  15.11.96 -- LINTER_Complete, LINTER_CursorComplete - Linter, System error
  23.01.97 -- eExistActiveChannel 
  28.01.97 -- cChannelID
  08.02.97 -- cIsAutoInc, sIsAutoInc
  19.08.97 -- syStartAppend, syEndAppend
  29.09.97 -- cProcErrCount, cProcArgNum, cProcArgDesc, cProcArgName,
              eNoMoreErrors, eNoProcArgName
              stored procedures exceptions,
              stored procedure arguments description,
              arguments flags,
              parameters types
  08.10.97 -- cWaitComplete
  14.11.97 -- LINAPI_INV_CONTEXT, LINAPI_EXCEPTION,
              LINAPI_ID_NUM_EXCEEDED
  04.05.98 -- sStmtType
  21.12.98 -- cProcCursorArg, types definition from inter.h
  01.02.99 -- typedef USR_AST_FUNC
  05.05.99 -- new types added: tBigInt, tVarChar, tVarByte, tBoolean
  04.02.00 -- IsUpdatable
  10.02.00 -- cWaitTimeout
  23.05.00 -- cBlobLength, cBlobType,
              LINTER_PurgeBlob, LINTER_AddBlob, LINTER_FetchBlob
  05.06.00 -- new types added: tNChar, tNVarChar
  29.08.00 -- new type tExtFile added
  26.12.00 -- Ermakov (IDENT's)
  04.01.01 -- Pasec (LINTER_ServerInfo)
  09.01.01 -- Pasec (cUserData, sUserData, LINTER_SetStatementOption)
  07.05.02 -- Pasec (syTestTable)
  02.10.02 -- Goncharov, Pasec (locale check added)
  28.05.03 -- Pasechnik (LINTER_Cancel)
  25.06.03 -- Pasechnik (eCharsetChanged)
  14.08.03 -- Pasechnik (LINTER_FetchBlobEx)
  26.08.03 -- Pasechnik (LINTER_ExecControlQuery)
  04.09.03 -- Pasechnik (LINTER_CreateCursor)
  09.12.05 -- Pasechnik (isolation level bitmasks added)
  18.01.06 -- Pasechnik (LINTER_AddBlob2)
  04.04.06 -- KAA (enum instead of defines for fetch direction)
  04.05.06 -- Pasechnik (LINAPI_ERROR_ON_CLOSE)
  14.12.06 -- Ermakov (Removed old code. Corrected Async Proto)
  15.11.07 -- Baranov (Added cConnFlags)
  16.11.07 -- Ermakov (Added sParamMap)
  28.07.08 -- KOV     (LINTER_FreeConnect)
  24.06.09 -- Ermakov (LINTER_ConnectCS)
  22.01.10 -- Egorov (Modified for RTOS)
  08.02.10 -- KOV     (Added constructions 'TABLE <table_name>' and 'VALUES (<value_list>) [,(<value_list>)] ...')
  15.12.11 -- Ganin   (Added mOem for OEM mode flag)
  17.04.13 -- KOV     (Added LINTER_ConnectEnv/LINTER_AsyncConnectEnv)
  21.05.13 -- KOV     (Support of 'execute block' command)
  20.02.14 -- Ganin   (Options to check for IsLastAst)
  23.09.14 -- Glukhov (change define to enum for parameter type)
*/

#ifndef __LINAPI_H
#define __LINAPI_H

/* Platform depended macro:
 *    INTER_MSWINDOWS           - Win16
 *    INTER_MSWINDOWS && WIN32  - Win32
 *
 * Version depended macro:
 *    LINAPI_STATIC   - make/use static library
 *    LINAPI_DLL      - make DLL
 */

#include "dlldefs.h"

#define _lincall_ DLLAPI

#include "typedef.h"
#include "inter.h"

/* return values */
#define LINAPI_SUCCESS           0
#define LINAPI_ERROR            -1
#define LINAPI_INV_ID           -2
#define LINAPI_NO_MEM            3
#define LINAPI_BUSY             -3
#define LINAPI_INV_CONTEXT      -4
#define LINAPI_ID_NUM_EXCEEDED  -5
#define LINAPI_ERROR_ON_CLOSE   -6

/* LinAPI internal errors */

/*        Other           */
#define eLinterError             14000
#define eNoMemory                14001
#define eNullPointer             14003
#define eIllegalSequence         14004
#define eSmallBuffer             14005
#define eIllegalDirection        14006
#define eFunctionNotAvailable    14007
#define eIllegalParameterNumber  14009
#define eIllegalParam            14010
#define eInvalidBlobOperation    14011
#define eExistActiveChannel      14012
#define eMaxIDNumberExceeded     14013
#define eInvalidContext          14014
#define eErrorInternalDiagnostic 14015
#define eOperationCanceled       14016


/*        Connect       */
#define eConnectNotFound         14100
#define eTooLongUserName         14101
#define eTooLongPassWord         14102
#define eTooLongServerName       14103
#define eIllegalConnectOption    14104
#define eConnectIsBusy           14105
#define eIncompatibilityVersions 14106
#define eWrongLocale             14107                     /* 02.10.02 */
#define eCharsetChanged          14108                     /* 25.06.03 */
#define eTooLongCfgName          14109                     /* 17.04.13 ... */
#define eEnvVarNotDef            14110
#define eIncEnvVar               14111
#define eInvConnStr              14112
#define eEnvVarError             14113
#define eErrOpenCfgFile          14114                     /* ... 17.04.13 */

/*        Cursor        */
#define eCursorNotFound          14200
#define eIllegalCursorOption     14201
#define eCursorIsBusy            14202

/*        Statement     */
#define eStatementNotFound       14300
#define eIllegalStatementOption  14301
#define eInvalidConnectID        14302
#define eStatementIsBusy         14303

/*        Query         */
#define eQueryTooLong            14400
#define eQueryNotSelect          14401
#define eNoBlobColumn            14403

/*        Object       */
#define eIllegalObjectType       14500
#define eObjectNotView           14501

/* Procedures          */
/* only for v.5.1      */
#define eNoMoreErrors            14601
#define eNoProcArgName           14602


/* LinAPI types */
#define tChar               DT_CHAR
#define tInteger            DT_INTEGER
#define tInt                tInteger
#define tReal               DT_REAL
#define tDate               DT_DATE
#define tTimeStamp          tDate
#define tDecimal            DT_DECIMAL
#define tDec                tDecimal
#define tNumeric            tDecimal
#define tBlob               DT_BLOB
#define tByte               DT_BYTE
#define tSmallInt           8
#define tDouble             9
#define tBigInt             10
#define tVarChar            11
#define tVarByte            12
#define tBoolean            13
#define tString             15
#define tNChar              16
#define tNVarChar           17
#define tExtFile            18

/* Fetch directions */                                           /* 04.04.06 */
enum FetchDirection
{
  toNext      = 1,
  toPrevious  = 2,
  toFirst     = 3,
  toLast      = 4,
  toAbsNumber = 5,
  toRelNumber = 6,
  toFromEnd   = 7
};

/* Parameters types */
enum
{
  pInput       = 0x0000,
  pInputOutput = 0x0100,
  pOutput      = 0x0200
};

/* Connect options */
#define cDBDesc             101
#define cNodeName           102
#define cChannelID          103
#define cConnFlags          104                                  /* 15.11.07 */

/* Cursor options  */
#define cSelectRowCount     200
#define cRowCount           201
#define cColumnCount        202
#define cStmtType           203
#define cCurrentRow         204
#define cCurrentRowID       205
#define cExecStmt           206
#define cExecRow            207
#define cAnswerDesc         208
#define cStmtNumber         209
#define cParamNumber        210
#define cConnectID          211
#define cAnswerSize         212
#define cCursorName         213
#define cNullIndicator      214
#define cIsAutoInc          215
/* only for v.5.1      */
#define cProcErrNum         216
#define cProcArgNum         217
#define cProcArgName        219
#define cProcCursorArg      220
/* don't use cProcArgDesc option,  */
/* for backward compatibility only */
#define cProcArgDesc        218
#define cIsUpdatable        221
#define cBlobLength         222
#define cBlobType           223
#define cIsInCallback       224

/* Connect/Cursor options */
#define cTransMode          300
#define cPriority           301
#define cLinCode            302
#define cSysCode            303
#define cStrNumber          304
#define cPosNumber          305
#define cApiCode            306
#define cWaitComplete       307
#define cWaitTimeout        308
#define cUserData           309

/* Statement options */
#define sParamCount         401
#define sAnswerDesc         402
#define sColumnCount        403
#define sParamDesc          404
#define sConnectID          405
#define sApiCode            406
#define sIsAutoInc          407
#define sStmtType           408
#define sParamType          409
#define sParamMap           410
#define sUserData           cUserData

/* Object Types      */
#define oView               501

/* SQL keywords */
#define sySelect       12
#define syInsert       13
#define syUpdate       14
#define syDelete       15
#define syCreate       16
#define syAlter        17
#define syDrop         18
#define syGrant        19
#define syRevoke       20
#define sySet          21
#define syCommit       22
#define syRollBack     23
#define syPress        24
#define syRebuild      25
#define syWait         26
#define syClear        27
#define syLock         28
#define syUnlock       29
#define syExecute      30
#define syStartAppend  31
#define syEndAppend    32
#define syTestTable    33
#define syTable        34                                  /* 08.02.10 */
#define syValues       35                                  /* 08.02.10 */
#define syExecuteBlock 36                                  /* 21.05.13 */

/* argument flags                  */
/* for backward compatibility only */
#define fNULL            PARM_FLAG_ISNULL
#define fCursor          PARM_FLAG_ISCURSOR
#define fName            PARM_FLAG_NAMESENT

/* Transaction mode */
#define mAutocommit      0L
#define mOptimistic      M_OPTIMISTIC
#define mExclusive       M_EXCLUSIVE
#if _VER_MAX >= 610                                             /* 09.12.05 */
#define mReadUncommitted M_READ_UNCOMMITTED
#define mReadCommitted   M_READ_COMMITTED
#define mSerializable    M_SERIALIZABLE
#endif
/* encoding mode */
#define mAnsi            Q_ENCODE
#define mKoi8            Q_KOI8
#define mOem             (Q_ENCODE + Q_KOI8)                    /* 15.12.11 */

#define linNameLen       MAX_ID_LEN                             /* 26.12.00 */

/* LINTER_Cancel options (28.05.03) */
#define C_CANCEL               0x00000001
#define C_CANCEL_AST           0x00000002
#define C_CANCEL_AST_CLOSE_CH  0x00000004

/* LINTER_FetchBlobEx options (14.08.03) */
#define FBE_CONV_PROC   0x00000001

typedef struct  
      {
      L_CHAR     Owner[MAX_ID_LEN];
      L_CHAR     Table[MAX_ID_LEN];
      L_CHAR     Column[MAX_ID_LEN];
      L_WORD     Length;
      L_BYTE     Type;
      L_BYTE     Prec;
      L_BYTE     Scale;
      L_BYTE     NullIndicator;
      L_LONG     RealLength;
      } t_ParamDesc;

/* don't use t_ProcArgDesc struct, */
/* for backward compatibility only */
/* stored procedure arguments description */
/* only for Linter v.5.1 (don't use for later versions) */
typedef struct
{
  L_WORD  Length;     
  L_BYTE  Ntype,       
          Prec,       
          Scale,      
          Reserv;     
} t_ArgType;                 

typedef struct
{
  L_BYTE Flags;      
  L_BYTE Reserv;     
  L_WORD Expr;            
  L_WORD Value;      
  t_ArgType     Type;
  L_WORD Reserv2;         
} t_ProcArgDesc;        

/* Database description */
typedef DB_DESC t_DBDesc;

/* user AST function */
typedef void (* USR_AST_FUNC)( L_WORD, L_LONG, void *); /* 01.02.99 */

#ifdef  __cplusplus
extern "C" {
#endif

extern L_LONG DLLAPI LINTER_CloseAPI(void);

extern L_LONG DLLAPI LINTER_Connect(L_CHAR  *, /* UserName           */
                                    L_SWORD,   /* Name length        */
                                    L_CHAR  *, /* PassL_WORD         */
                                    L_SWORD,   /* PassL_WORD length  */
                                    L_CHAR  *, /* Server Name        */
                                    L_LONG   , /* Transaction mode   */
                                    L_WORD *); /* Connection ID      */

extern L_LONG DLLAPI LINTER_AsyncConnect(L_CHAR  *, /* UserName           */
                                    L_SWORD,   /* Name length        */
                                    L_CHAR  *, /* PassL_WORD         */
                                    L_SWORD,   /* PassL_WORD length  */
                                    L_CHAR  *, /* Server Name        */
                                    L_LONG   , /* Transaction mode   */
                                    L_WORD *,  /* Connection ID      */
                                    void *,
                                    void *);

extern L_LONG DLLAPI LINTER_ConnectCS(L_CHAR  *, /* UserName           */ /* 24.06.09 */
                                      L_SWORD,   /* Name length        */
                                      L_CHAR  *, /* PassL_WORD         */
                                      L_SWORD,   /* PassL_WORD length  */
                                      L_CHAR  *, /* Server Name        */
                                      L_CHAR  *, /* Charset            */
                                      L_LONG   , /* Transaction mode   */
                                      L_WORD *); /* Connection ID      */

extern L_LONG DLLAPI LINTER_AsyncConnectCS(L_CHAR  *, /* UserName           */  /* 24.06.09 */
                                           L_SWORD,   /* Name length        */
                                           L_CHAR  *, /* PassL_WORD         */
                                           L_SWORD,   /* PassL_WORD length  */
                                           L_CHAR  *, /* Server Name        */
                                           L_CHAR  *, /* Charset            */
                                           L_LONG   , /* Transaction mode   */
                                           L_WORD *,  /* Connection ID      */
                                           void *,
                                           void *);

#ifndef WINCE                                              /* 17.04.13 ... */
extern L_LONG DLLAPI LINTER_ConnectEnv(L_CHAR  *, /* CfgName           */
                                       L_SWORD,   /* CfgName length    */
                                       L_LONG *,  /* Transaction mode   */
                                       L_WORD *); /* Connection ID     */

extern L_LONG DLLAPI LINTER_AsyncConnectEnv(L_CHAR  *, /* CfgName           */
                                            L_SWORD,   /* CfgName length    */
                                            L_LONG *,  /* Transaction mode   */
                                            L_WORD *,  /* Connection ID     */
                                            void *,
                                            void *);
#endif /* WINCE */                                         /* ... 17.04.13 */

extern L_LONG DLLAPI LINTER_CloseConnect(L_WORD); /* Connection ID      */

extern L_LONG DLLAPI LINTER_FreeConnect(L_WORD,    /* Connection ID     */
                                        L_LONG *); /* Linter error code */ /* 28.07.08 */

extern L_LONG DLLAPI LINTER_OpenCursor(L_WORD  , /* Connection ID      */
                                     L_WORD *,   /* Cursor     ID      */
                                     L_CHAR  *,  /* Cursor Name        */
                                     L_SWORD,    /* Name length        */
                                     L_LONG);    /* Transaction Mode   */

extern L_LONG DLLAPI LINTER_CreateCursor(L_WORD,    /* (in) Connection ID */
                                         TCBL *,    /* (in) CBL control block */
                                         L_WORD *); /* (out) Cursor ID */

extern L_LONG DLLAPI LINTER_CloseCursor(L_WORD);  /* Cursor ID   */


extern L_LONG DLLAPI LINTER_CreateStatement(L_WORD,  /* Connection ID  */
                                          L_CHAR *,  /* Query text     */
                                          L_LONG,    /* Length         */
                                          L_WORD *); /* StatementID    */

extern L_LONG DLLAPI LINTER_FreeStatement(L_WORD);    /* Statement ID   */

extern L_LONG DLLAPI LINTER_ExecuteStatement(L_WORD, /* Cursor ID            */
                                           L_WORD,   /* Statement ID         */
                                           L_LONG *, /* number executed stmt */
                                           void *,   /* Ast-function         */
                                           void *);  /* User argument        */

extern L_LONG DLLAPI LINTER_Fetch(L_WORD,  /* Cursor ID                  */
                                L_SWORD,   /* Direction                  */
                                L_LONG,    /* Position number            */
                                L_LONG,    /* Requirement rows counter   */
                                L_LONG * , /* real count rows of answer  */
                                void * ,   /* Ast-function               */
                                void *);   /* User argument              */

extern L_LONG DLLAPI LINTER_ExecControlQuery(L_WORD,    /* Conection ID   */
                                             L_CHAR *,  /* Query text     */
                                             L_LONG,    /* Query length   */
                                             void *,    /* Ast_function   */
                                             void *);   /* User argument  */

extern L_LONG DLLAPI LINTER_ExecuteDirect(L_WORD,    /* Cursor ID      */
                                        L_CHAR *,    /* Query text     */
                                        L_LONG,      /* Query length   */
                                        void *,      /* Ast_function   */
                                        void *);     /* User argument  */

extern L_LONG DLLAPI LINTER_GetRowBuffer(L_WORD ,   /* Cursor ID          */
                                       void *,      /* Answer buffer      */
                                       L_LONG *);   /* Answer buffer size */

extern L_LONG DLLAPI LINTER_RollBackCursor(L_WORD,   /* Cursor ID        */
                                         void *,     /* Async function   */
                                         void *);    /* User argument    */

extern L_LONG DLLAPI LINTER_RollBack(L_WORD,   /* Connection ID    */
                                   void *,     /* Async function   */
                                   void *);    /* User argument    */

extern L_LONG DLLAPI LINTER_Commit(L_WORD,   /* Connection ID    */
                                 void *,     /* Async function   */
                                 void *);    /* User argument    */

extern L_LONG DLLAPI LINTER_CommitCursor(L_WORD,   /* Cursor ID        */
                                       void *,     /* Async function   */
                                       void *);    /* User argument    */

extern L_LONG DLLAPI LINTER_LockRow(L_WORD);   /* Cursor ID        */

extern L_LONG DLLAPI LINTER_UnlockRow(L_WORD); /* Cursor ID        */

extern L_LONG DLLAPI LINTER_ClearBlob(L_WORD,    /* Cursor ID        */
                                    void *,      /* Async function   */
                                    void *);     /* User argument    */

extern L_LONG DLLAPI LINTER_PurgeBlob(L_WORD,      /* Cursor ID        */
                                      L_SWORD,     /* Column number    */
                                      void *,      /* Async function   */
                                      void *);     /* User argument    */

extern L_LONG DLLAPI LINTER_GetBlobType(L_WORD,     /* Cursor ID        */
                                      L_WORD *);    /* Blob type        */

extern L_LONG DLLAPI LINTER_GetBlobLength(L_WORD,   /* Cursor ID                  */
                                        L_LONG *);  /* Full blob size in database */

extern L_LONG DLLAPI LINTER_AppendBlob(L_WORD,    /* Cursor ID           */
                                     L_WORD,      /* Blob type           */
                                     void *,      /* Blob buffer pointer */
                                     L_LONG,      /* Blob buffer size    */
                                     void *,      /* Async function      */
                                     void *);     /* User argument       */

#if _VER_MAX >= 590                                           /* 18.01.06 */
extern L_LONG DLLAPI LINTER_AddBlob2(L_WORD,    /* Cursor ID           */
                                     L_SWORD,   /* Column number       */
                                     L_LONG,    /* Blob type           */
                                     void *,    /* Blob buffer pointer */
                                     L_LONG,    /* Blob buffer size    */
                                     void *,    /* must be NULL (reserved for future use) */
                                     void *,    /* Async function      */
                                     void *);   /* User argument       */
#endif

extern L_LONG DLLAPI LINTER_AddBlob(L_WORD,    /* Cursor ID           */
                                    L_SWORD,   /* Column number       */
                                    L_WORD,    /* Blob type           */
                                    void *,    /* Blob buffer pointer */
                                    L_LONG,    /* Blob buffer size    */
                                    void *,    /* Async function      */
                                    void *);   /* User argument       */

extern L_LONG DLLAPI LINTER_GetBlob(L_WORD,  /* Cursor ID             */
                                  L_LONG,    /* Blob portion position */
                                  L_LONG *,  /* Blob buffer size      */
                                  void *,    /* Blob buffer pointer   */
                                  void *,    /* Async function        */
                                  void *);   /* User argument         */

extern L_LONG DLLAPI LINTER_FetchBlob(L_WORD,    /* Cursor ID             */
                                      L_SWORD,   /* Column number         */
                                      L_LONG,    /* Blob portion position */
                                      L_LONG *,  /* Blob buffer size      */
                                      void *,    /* Blob buffer pointer   */
                                      void *,    /* Async function        */
                                      void *);   /* User argument         */

#if _VER_MAX >= 600                                           /* 14.08.03 */
extern L_LONG DLLAPI LINTER_FetchBlobEx(L_WORD,    /* Cursor ID             */
                                        L_SWORD,   /* Column number         */
                                        L_LONG,    /* Blob portion position */
                                        L_LONG *,  /* Blob buffer size      */
                                        void *,    /* Blob buffer pointer   */
                                        void *,    /* Async function        */
                                        void *,    /* User argument         */
                                        L_LONG);   /* reserved              */
#endif

extern L_LONG DLLAPI LINTER_SetBlobType(L_WORD,     /* Cursor ID        */
                                      L_WORD);      /* BLOB type        */

extern L_LONG DLLAPI LINTER_SetStatementOption(L_WORD,    /* Statement ID        */
                                               L_SWORD,   /* Statement option    */
                                               void *,    /* Option value buffer */
                                               L_LONG *); /* Option value length */

extern L_LONG DLLAPI LINTER_GetStatementOption(L_WORD,  /* Statement ID     */
                                             L_SWORD,   /* Statement option */
                                             L_SWORD,   /* Column number    */
                                             void *,    /* Option buffer    */
                                             L_LONG *); /* Output length    */

extern L_LONG DLLAPI LINTER_SetConnectOption(L_WORD ,  /* Connect ID      */
                                           L_SWORD ,   /* Connect option  */
                                           void *,     /* Option buffer   */
                                           L_LONG *);  /* output length   */

extern L_LONG DLLAPI LINTER_GetConnectOption(L_WORD,  /* Connect ID      */
                                           L_SWORD,   /* Connect option  */
                                           void *,    /* Option buffer   */
                                           L_LONG *); /* output length   */

extern L_LONG DLLAPI LINTER_GetCursorOption(L_WORD,  /* Cursor ID              */
                                          L_SWORD,   /* Cursor option          */
                                          L_SWORD,   /* Column number          */
                                          void *,    /* Option value buffer    */
                                          L_LONG *); /* Real option value size */

extern L_LONG DLLAPI LINTER_SetCursorOption(L_WORD,  /* Cursor ID           */
                                          L_SWORD,   /* Cursor option       */
                                          void *,    /* Option value buffer */
                                          L_LONG *); /* Option value length */

extern L_LONG DLLAPI LINTER_Error(L_WORD,    /* Connection ID          */
                                L_WORD,      /* Cursor ID              */
                                L_WORD,      /* Statement ID           */
                                L_LONG *,    /* LinAPI return code     */
                                L_LONG *,    /* Linter return code     */
                                L_LONG *,    /* System error           */
                                L_CHAR *,    /* Message                */
                                L_SWORD *);  /* Message length         */

extern L_LONG DLLAPI LINTER_ErrorMessage(L_LONG,    /* LinAPI return code */
                                       L_LONG,      /* Linter return code */
                                       L_CHAR  *,   /* Message            */
                                       L_SWORD *);  /* Message length     */

extern L_LONG DLLAPI LINTER_BindParameter(L_WORD,   /* Cursor ID         */
                                        L_WORD,     /* Statement ID      */
                                        L_SWORD,    /* Parameter Number  */
                                        L_CHAR *,   /* Parameter Name    */
                                        L_CHAR *,   /* Null Indicator    */
                                        void *,     /* Parameter Address */
                                        L_LONG,     /* Item Counter      */
                                        L_LONG,     /* Shift             */
                                        L_SWORD,    /* Parameter Type    */
                                        L_LONG *,   /* not used          */
                                        L_LONG *);  /* Parameter Length  */


extern L_LONG DLLAPI LINTER_BindAnswer(L_WORD,   /* Cursor ID                  */
                                     L_WORD,     /* Statement ID               */
                                     L_SWORD,    /* Column number              */
                                     void *,     /* Answer buffer pointer      */
                                     L_CHAR *,   /* Null indicator pointer     */
                                     L_LONG,     /* Shift to next column value */
                                     L_SWORD,    /* Output column type         */
                                     L_LONG,     /* Output column length       */
                                     L_SWORD,    /* Output column precision    */
                                     L_SWORD,    /* Output column scale        */
                                     L_LONG *);  /* Real value length          */

extern L_LONG DLLAPI LINTER_GetData(L_WORD,  /* Cursor ID                  */
                                  L_SWORD,   /* Column number              */
                                  void *,    /* Answer buffer pointer      */
                                  L_LONG,    /* Output buffer length       */
                                  L_SWORD,   /* Output column type         */
                                  L_SWORD,   /* Output column precision    */
                                  L_SWORD,   /* Output column scale        */
                                  L_LONG *); /* Real value length          */

extern L_LONG DLLAPI LINTER_UnBindAnswer(L_WORD,    /* Cursor ID        */
                                       L_WORD,      /* Statement ID     */
                                       L_SWORD );   /* Column Number    */

extern L_LONG DLLAPI LINTER_CursorComplete(L_WORD,   /* Cursor ID      */
                                         L_SWORD *,  /* Complete or no */
                                         L_LONG *,   /* LinAPI error   */
                                         L_LONG *,   /* Linter error   */
                                         L_LONG *);  /* System error   */

extern L_LONG DLLAPI LINTER_ConnectComplete(L_WORD,    /* Connect ID      */
                                          L_SWORD *,   /* Complete or no  */
                                          L_LONG *,    /* LinAPI error    */
                                          L_LONG *,    /* Linter error    */
                                          L_LONG *);   /* System error    */

extern L_LONG DLLAPI LINTER_GetObjDesc(L_WORD,    /* Connect ID      */
                                     L_SWORD,     /* Object Type     */
                                     L_CHAR *,    /* Object Name     */
                                     L_SWORD,     /* Name Length     */
                                     void *,      /* Out Buffer      */
                                     L_LONG *);   /* Buffer size     */

extern L_LONG DLLAPI LINTER_KillChannel(L_WORD,   /* Connect ID */ 
                                      L_WORD );   /* Channel ID */

extern L_LONG DLLAPI LINTER_ServerInfo(L_CHAR *,  /* Server name        */
                                       L_SWORD,   /* Server name length */
                                       L_SWORD,   /* Info type          */
                                       void *,    /* Info buffer        */
                                       L_WORD,    /* Buffer length      */
                                       L_WORD *,  /* Output info length */
                                       L_LONG *,  /* LinAPI error       */
                                       L_LONG *,  /* Linter error       */
                                       L_LONG *); /* System error       */

extern L_LONG DLLAPI LINTER_Cancel(L_WORD ConnectionID, 
                                   L_WORD CursorID, 
                                   L_ULONG option, 
                                   L_ULONG reserved);

#ifdef __cplusplus
}
#endif

#endif
#ifdef RTOS32
# define GetProcType GetPrcType /*names conflict*/
#endif
