/***************************************************************************
 *           RELEX, Inc. Linter CPP API source code                         *
 *  This is unpublished proprietary source code of RELEX, Inc.              *
 *  The copyright notice does not evidence any actual or intended           *
 *  publication of such source code.                                        *
 *                                                                          *
 *                 Copyright (c) 1997-2013 RELEX Inc.                       *
 *                                                                          *
 *                   This software was developed by                         *
 *                                                                          *
 *                            RELEX, Inc.                                   *
 *                         Voronezh, Russia                                 *
 *                                                                          *
 *                        All rights reserved.                              *
 ****************************************************************************
 AUTHOR: Koval Dmitry (KOV)
 ----------------------------------------------------------------------------
 REVISION HISTORY: 
 ----------------------------------------------------------------------------
  11.07.08 -- KOV (Creation)
  19.06.09 -- KOV (Added class linDataSet)
  27.12.10 -- KOV (Correction: added possibility of transactions on separate cursors, not just on connection)
  21.01.11 -- KOV, Oleg (FOR UNIX)
  02.02.11 -- KOV (Alignment to 8)
  06.12.11 -- KOV (New options for internal buffer & temporary files)
  20.08.12 -- KOV (Changes for case cursors in MT mode)
  20.03.13 -- Ermakov (GCC 4 adopt. MinGW adopt)
  17.04.13 -- KOV (Added linConnection::ConnectEnv function)
*/

#ifndef _LINCPPAPI_H_
#define _LINCPPAPI_H_

#include "linapi.h"

#ifdef WIN32
#  if defined(LINCPPAPI_DLL_USE) || defined(LINCPPAPI_DLL_BUILD)
#    ifdef LINCPPAPI_DLL_BUILD
#      define DLL_CALL __declspec(dllexport)
#    else
#      define DLL_CALL __declspec(dllimport)
#    endif
#  else
#    define DLL_CALL
#  endif
#else
#  define DLL_CALL
#endif

/* type definition for MUTEX: */
#if defined(WINDOWS_NT) || defined(WIN32)
#  define LINMUTEX void *
#  define LINMUTEX_INIT(m)   { m = NULL; }
#  define LINMUTEX_EXISTS(m) (m != NULL)
#elif defined(LIN_PTHREAD)
#  include <pthread.h>
#  include <semaphore.h>
#  define LINMUTEX pthread_mutex_t
#  define LINMUTEX_INIT(m)   { lin_mutex_init(&m_linmutex); }
#  define LINMUTEX_EXISTS(m) 1
#else
#  if defined(SUN) || defined (UNIXWARE)
#    include <thread.h>
#    define LINMUTEX mutex_t
#    define LINMUTEX_INIT(m)   { lin_mutex_init(&m_linmutex); }
#    define LINMUTEX_EXISTS(m) 1
#  endif
#  if defined(AIX) || defined(BSD) || defined(LINUX) || defined(SGI)
#    error Use macro PTHREADS for this platform
#  endif
#  ifdef DIGITAL_UNIX
#  endif
#endif

#define LINAPI_ERROR_FOR_RE_CONNECT -101  /* was returned specific error code (connection error, need to make re-comment): (4000;5000),1001,1044,1046,1069,6712 */
#define LINCPPAPI_ERROR             -102  /* ~ equals LINAPI_ERROR, error successfully processed and error text was placed into error buffer */
#define LINCPPAPI_STATEMENT_DIED    -103  /* con't execute statement operation (statement is died), need free statement */ /* 20.08.12 */

class linQBuffer;
class linParamBuffer;
class linErrBuf;
class linStatement;
class linCursor;
class linConnection;
class linPtrArray;                                         /* 19.06.09 */
class linLongArray;                                        /* 19.06.09 */

class DLL_CALL linStatement
{
private:
  friend class linCursor;
  friend class linConnection;

  L_WORD          m_wStatementId;       /* statement identifier */
  L_WORD          Pad;                  /* align to 8 */
  L_LONG          m_lFlags;             /* statement flags */
  linCursor      *m_pCursor;            /* parent cursor */
  linStatement   *m_pNextStatement;     /* next statement into chain (first & last - into linCursor class ) */
  linStatement   *m_pPrevStatement;     /* previous statement into chain (first & last - into linCursor class ) */
  linStatement   *m_pNextFreeStatement; /* next free statement into chain (first - into linCursor class ) */
  linParamBuffer *m_pParamBuf;          /* buffer for parameters */

  linStatement(linCursor *); /* parentCursor */
  ~linStatement();

  linStatement   *GetNextStatement();
  linStatement   *GetPrevStatement();
#define LIN_STATEMENT_DIED  0x00000001                     /* 20.08.12 */
  L_LONG          GetFlags();
  void            SetFlags(L_LONG lFlags);
  L_WORD          GetStatementId();
  linParamBuffer *GetParamBuf();

  L_LONG Create(L_CHAR *, /* Query text */
                L_LONG,   /* Length     */
                linErrBuf*,
                L_BOOL);                                   /* 27.12.10 ... 20.08.12 */
  L_LONG Free(linErrBuf*);
  L_LONG RestoreParameters();

  L_LONG SetStatementOptionEx(L_SWORD, void *, L_LONG *, L_BOOL); /* 20.08.12 */
  L_LONG ExecuteStatementEx(L_LONG *, void *, void *, L_BOOL);
  L_LONG BindParameterEx(L_SWORD, L_CHAR *, L_CHAR *, void *, L_LONG, L_LONG, L_SWORD, L_LONG *, L_BOOL); /* ... 20.08.12 */

public:
  linCursor      *GetCursor();
  linConnection  *GetConnection();

  L_LONG ExecuteStatement(L_LONG *, /* number executed stmt */
                          void *,   /* Ast-function         */
                          void *);  /* User argument        */
  L_LONG SetStatementOption(L_SWORD,   /* Statement option    */
                            void *,    /* Option value buffer */
                            L_LONG *); /* Option value length */
  L_LONG GetStatementOption(L_SWORD,   /* Statement option */
                            L_SWORD,   /* Column number    */
                            void *,    /* Option buffer    */
                            L_LONG *); /* Output length    */
  L_LONG BindParameter(L_SWORD,    /* Parameter Number  */
                       L_CHAR *,   /* Parameter Name    */
                       L_CHAR *,   /* Null Indicator    */
                       void *,     /* Parameter Address */
                       L_LONG,     /* Item Counter      */
                       L_LONG,     /* Shift             */
                       L_SWORD,    /* Parameter Type    */
                       L_LONG *);  /* Parameter Length  */
  L_LONG BindAnswer(L_SWORD,    /* Column number              */
                    void *,     /* Answer buffer pointer      */
                    L_CHAR *,   /* Null indicator pointer     */
                    L_LONG,     /* Shift to next column value */
                    L_SWORD,    /* Output column type         */
                    L_LONG,     /* Output column length       */
                    L_SWORD,    /* Output column precision    */
                    L_SWORD,    /* Output column scale        */
                    L_LONG *);  /* Real value length          */
  L_LONG UnBindAnswer(L_SWORD);   /* Column Number    */
};

class DLL_CALL linCursor
{
private:
  friend class linStatement;
  friend class linConnection;

  L_WORD         m_wCursorId;           /* cursor identifier */  
  L_SWORD        m_swCursorNameLen;     /* length of cursor name */
  L_LONG         m_lFlags;              /* cursor flags */
  L_LONG         m_lQBufBufSize;        /* size of query buffer */ /* 06.12.11 ... */
  L_BOOL         m_bQBufUseTmpFile;     /* use temporary file or not */
  L_BYTE         bRes[3];                                   /* ... 06.12.11 */
  L_CHAR        *m_sCursorName;         /* name of cursor */
  linConnection *m_pConnection;         /* parent connection */
  linCursor     *m_pNextCursor;         /* next cursor into chain (first & last - into linConnection class ) */
  linCursor     *m_pPrevCursor;         /* previous cursor into chain (first & last - into linConnection class ) */
  linCursor     *m_pNextFreeCursor;     /* next free cursor into chain (first - into linConnection class ) */
  linStatement  *m_pFirstStatement;     /* first statement into chain */  
  linStatement  *m_pLastStatement;      /* last statement into chain */
  linStatement  *m_pFirstFreeStatement; /* first free statement into chain */  
  linErrBuf     *m_pErrBuf;             /* buffer for errors */ /* 27.12.10 */
  linQBuffer    *m_pQBuf;               /* buffer with logged queries */ /* 27.12.10 */

  linCursor(linConnection *pConnection); /* parentConenction */
  ~linCursor();

  void FreeCursor();

  L_LONG Open(L_CHAR  *,    /* Cursor Name */
              L_SWORD,      /* Name length */
              linErrBuf*,
              L_BOOL);                                     /* 27.12.10 20.08.12 */
  L_LONG Close(linErrBuf*);                                /* 27.12.10 */
  void SetQBufSettings(L_LONG lBufSize /* size of buffer */, L_BOOL bUseTmpFile = L_TTRUE /* can use temparary file */); /* 06.12.11 */

  linCursor     *GetNextCursor();
  linCursor     *GetPrevCursor();
  L_LONG         GetFlags();
  void           SetFlags(L_LONG lFlags);
  L_LONG         ReConnect(linErrBuf *, L_BOOL);           /* 27.12.10 20.08.12 */
  linStatement  *GetFirstStatement();
  L_WORD         GetCursorId();
  L_CHAR        *GetCursorName();
  L_SWORD        GetCursorNameLen();
  L_LONG         CloseFreeStatements();                    /* 27.12.10 ... */
  linErrBuf     *GetErrBuf();
  linQBuffer    *GetQBuf();                                /* ... 27.12.10 */

  L_LONG ExecuteDirectEx(L_CHAR *, L_LONG, void *, void *, L_BOOL); /* 20.08.12 ... */
  L_LONG LockRowEx(L_BOOL);
  L_LONG UnlockRowEx(L_BOOL);
  L_LONG ClearBlobEx(void *, void *, L_BOOL);
  L_LONG PurgeBlobEx(L_SWORD, void *, void *, L_BOOL);
  L_LONG AppendBlobEx(L_WORD, void *, L_LONG, void *, void *, L_BOOL);
  L_LONG AddBlobEx(L_SWORD, L_LONG, void *, L_LONG, void *, void *, L_BOOL);
  L_LONG SetBlobTypeEx(L_WORD, L_BOOL);
  L_LONG SetCursorOptionEx(L_SWORD, void *, L_LONG *, L_BOOL);
  L_LONG FetchEx(L_SWORD, L_LONG, L_LONG, L_LONG * , void *, void *, L_BOOL);
  L_LONG CreateStatementEx(L_CHAR *, L_LONG, linStatement **, L_BOOL);
  L_LONG CommitCursorEx(void *, void *, L_BOOL);
  L_LONG RollBackCursorEx(void *, void *, L_BOOL);          /* ... 20.08.12 */

public:
  linConnection *GetConnection();
  L_LONG CreateStatement(L_CHAR *,         /* Query text                    */
                         L_LONG,           /* Length                        */
                         linStatement **); /* (out) pointer to linStatement */
  L_LONG FreeStatement(linStatement *);
#define LINAPI_NO_DATA  -10 /* Fetch returns LINAPI_NO_DATA if answer is empty */
  L_LONG Fetch(L_SWORD,   /* Direction                  */
               L_LONG,    /* Position number            */
               L_LONG,    /* Requirement rows counter   */
               L_LONG * , /* real count rows of answer  */
               void *pAstFunction = NULL, /* Async function */
               void *pUserArg = NULL);    /* User argument  */
  L_LONG ExecuteDirect(L_CHAR *,  /* Query text     */
                       L_LONG,    /* Query length   */
                       void *pAstFunction = NULL, /* Async function */
                       void *pUserArg = NULL);    /* User argument  */
  L_LONG GetRowBuffer(void *,      /* Answer buffer      */
                      L_LONG *);   /* Answer buffer size */
  L_LONG LockRow();
  L_LONG UnlockRow();
  L_LONG ClearBlob(void *pAstFunction = NULL, /* Async function */
                   void *pUserArg = NULL);    /* User argument  */
  L_LONG PurgeBlob(L_SWORD,     /* Column number    */
                   void *pAstFunction = NULL, /* Async function */
                   void *pUserArg = NULL);    /* User argument  */
  L_LONG GetBlobType(L_WORD *);    /* Blob type        */
  L_LONG GetBlobLength(L_LONG *);  /* Full blob size in database */
  L_LONG AppendBlob(L_WORD,      /* Blob type           */
                    void *,      /* Blob buffer pointer */
                    L_LONG,      /* Blob buffer size    */
                    void *pAstFunction = NULL, /* Async function */
                    void *pUserArg = NULL);    /* User argument  */
  L_LONG AddBlob(L_SWORD,   /* Column number       */
                 L_LONG,    /* Blob type           */
                 void *,    /* Blob buffer pointer */
                 L_LONG,    /* Blob buffer size    */
                 void *pAstFunction = NULL, /* Async function */
                 void *pUserArg = NULL);    /* User argument  */
  L_LONG GetBlob(L_LONG,    /* Blob portion position */
                 L_LONG *,  /* Blob buffer size      */
                 void *,    /* Blob buffer pointer   */
                 void *pAstFunction = NULL, /* Async function */
                 void *pUserArg = NULL);    /* User argument  */
  L_LONG FetchBlob(L_SWORD,   /* Column number         */
                   L_LONG,    /* Blob portion position */
                   L_LONG *,  /* Blob buffer size      */
                   void *,    /* Blob buffer pointer   */
                   void *pAstFunction = NULL, /* Async function */
                   void *pUserArg = NULL);    /* User argument  */
  L_LONG SetBlobType(L_WORD); /* BLOB type */
  L_LONG GetCursorOption(L_SWORD,   /* Cursor option          */
                         L_SWORD,   /* Column number          */
                         void *,    /* Option value buffer    */
                         L_LONG *); /* Real option value size */
  L_LONG SetCursorOption(L_SWORD,   /* Cursor option       */
                         void *,    /* Option value buffer */
                         L_LONG *); /* Option value length */
  L_LONG GetData(L_SWORD,   /* Column number              */
                 void *,    /* Answer buffer pointer      */
                 L_LONG,    /* Output buffer length       */
                 L_SWORD,   /* Output column type         */
                 L_SWORD,   /* Output column precision    */
                 L_SWORD,   /* Output column scale        */
                 L_LONG *); /* Real value length          */
  L_LONG CursorComplete(L_SWORD *,   /* Complete or no  */
                        L_LONG *,    /* LinAPI error    */
                        L_LONG *,    /* Linter error    */
                        L_LONG *);   /* System error    */
  L_LONG CommitCursor(void *pAstFunction = NULL, /* Async function */
                      void *pUserArg = NULL);    /* User argument  */ /* 27.12.10 */
  L_LONG RollBackCursor(void *pAstFunction = NULL, /* Async function */
                        void *pUserArg = NULL);    /* User argument  */ /* 27.12.10 */
  /* errors processing: */
  L_ULONG GetErrorsCount();
  void    ClearErrors();
  L_LONG  GetError(L_CHAR  *,  /* buffer for name of module      */
                   L_ULONG *,  /* buffer with sModuleName size   */
                   L_CHAR  *,  /* buffer for text of error       */
                   L_ULONG *,  /* buffer with size of sErrorText */
                   L_LONG  *,  /* buffer for linapi error        */
                   L_LONG  *,  /* buffer for linter error        */
                   L_LONG  *); /* buffer for system error        */
};

class DLL_CALL linConnection
{
private:
  friend class linCursor;
  friend class linStatement;

  L_SWORD     m_swUserNameLen;    /* length of user name */
  L_SWORD     m_swPasswordLen;    /* length of password */
  L_WORD      m_wConnectionId;    /* connection identifier */
  L_BOOL      m_bUseConnQBuf;     /* use m_pQBuf for connection - TRUE by default,
                                     otherwise use m_pQBuf for cursors */ /* 27.12.10 */
  L_BYTE      bPad;
  L_LONG      m_lTransactionMode; /* transaction mode */
  L_LONG      m_lFlags;           /* connection flags */
  L_ULONG     m_ulTimeout;        /* timeout for trying to re-open connect */
  L_LONG      m_lQBufBufSize;     /* size of query buffer */ /* 06.12.11 ... */
  L_BOOL      m_bQBufUseTmpFile;  /* use temporary file or not */
  L_LONG      m_lFunctionCounter; /* count of LINAPI function call (now) */ /* 20.08.12 */
  L_SWORD     m_swCfgNameLen;     /* length of config name */ /* 17.04.13 */
  L_BYTE      bRes[1];                                     /* ... 06.12.11 */
  L_CHAR     *m_sUserName;        /* name of user */
  L_CHAR     *m_sPassword;        /* password */
  L_CHAR     *m_sServerName;      /* name of server */
  L_CHAR     *m_sCharSet;         /* character set */      /* 27.12.10 */
  L_CHAR     *m_sCfgName;         /* config name */        /* 17.04.13 */
  linCursor  *m_pFirstCursor;     /* first cursor into chain */
  linCursor  *m_pLastCursor;      /* last cursor into chain */
  linCursor  *m_pFirstFreeCursor; /* first free cursor into chain */
  linQBuffer *m_pQBuf;            /* buffer with logged queries */
  linErrBuf  *m_pErrBuf;          /* buffer for errors */
  void       *m_pAstFunction;     /* stored argument of Connect function */
  void       *m_pUserArg;         /* stored argument of Connect function */
  LINMUTEX    m_linmutex;                                  /* 27.12.10 */

  void           FreeConnection();
  linQBuffer    *GetQBuf();
  linErrBuf     *GetErrBuf();
  L_LONG         ProcessingError(const L_CHAR*,     /* module name */
                                 L_LONG,      /* LINAPI function return code */
                                 L_WORD,      /* cursor identifier    */
                                 L_WORD,      /* statement identifier */
                                 L_LONG,      /* timeout counter (in ms) */
                                 linErrBuf *, /* buffer for errors */ /* 27.12.10 */
                                 L_BOOL*,     /* (out) repeat function indicator */
                                 L_BOOL);     /* external (out of library) or internal call */ /* 20.08.12 */
  L_LONG         IsErrorForReConnect(L_LONG lReturnCode, /* LINAPI function return code */
                                     L_WORD wCursorId,   /* cursor identifier    */       
                                     L_WORD wStatId);    /* statement identifier */       
  L_LONG         ReConnect(linErrBuf *pErrBuf);            /* 27.12.10 */
  L_LONG         PlayQBuffer(linQBuffer *pQBuf, linCursor  *pCurs); /* 27.12.10 */
  linCursor     *GetFirstCursor();
  linConnection *GetConnection();
  L_WORD         GetConnectionId();
  L_CHAR        *GetUserName();
  L_SWORD        GetUserNameLen();
  L_CHAR        *GetPassword();
  L_SWORD        GetPasswordLen();
  L_CHAR        *GetServerName();
  L_CHAR        *GetCharSet();                             /* 27.12.10 */
  L_CHAR        *GetCfgName();                             /* 17.04.13 */
  L_SWORD        GetCfgNameLen();                          /* 17.04.13 */
  L_LONG         GetTransactionMode();
  L_ULONG        GetMaxTimeout();
  L_LONG         GetFlags();
  void          *GetAstFunction();
  void          *GetUserArg();
  void           SetFlags(L_LONG lFlags);
  L_LONG         RestoreParameters(linCursor *pCurs);      /* 27.12.10 */
  void           FreeConnectParams();                      /* 27.12.10 */
  L_LONG         CloseFreeCursors();                       /* 27.12.10 */
  L_LONG SetConnectParams(L_CHAR*, /* UserName           */
                          L_SWORD, /* Name length        */
                          L_CHAR*, /* Password           */
                          L_SWORD, /* Password length    */
                          L_CHAR*, /* Server Name        */
                          L_CHAR*, /* Charset            */
                          L_CHAR*, /* Config name        */
                          L_SWORD, /* Config name length */
                          L_LONG,  /* Transaction mode   */
                          linErrBuf *pErrBuf = NULL);      /* 27.12.10 17.04.13 */
  L_LONG SetConnectOptionEx(L_SWORD, void *, L_LONG *, L_BOOL); /* 20.08.12 ... */
  L_LONG ReConnectEx(L_CHAR*, L_SWORD, L_CHAR*, L_SWORD, L_CHAR*, L_CHAR*, L_CHAR*, L_SWORD, L_LONG, linErrBuf *, L_BOOL); /* 17.04.13 */
  L_LONG ConnectCSEx(L_CHAR *, L_SWORD, L_CHAR  *, L_SWORD, L_CHAR *, L_CHAR *, L_LONG, void *, void *, linErrBuf *, L_BOOL);
  L_LONG ConnectEnvEx(L_CHAR *, L_SWORD, void *, void *, linErrBuf *, L_BOOL); /* 17.04.13 */
  void           IncFunctionCounter();
  void           DecFunctionCounter();
  L_LONG         GetFunctionCounter();
  static void WaitQuant(L_LONG lCount = 1);                /* ... 20.08.12 */

public:
  linConnection();
  ~linConnection();

  /* Define max. timeout between state 'server down' and state 'server start', by default - 30 sec.: */
  void SetMaxTimeout(L_ULONG ulTimeout);
  /* Ignore or not SQL errors during process of restore transaction, by default - ignore */
  void IgnoreLinterErrorsDuringRestore(L_BOOL);
  /* use query buffer or not: */
  L_LONG SetUseConnQBuf(L_BOOL bSet);                      /* 27.12.10 */
  L_BOOL GetUseConnQBuf();                                 /* 27.12.10 */

  L_LONG Connect(L_CHAR  *, /* UserName           */
                 L_SWORD,   /* Name length        */
                 L_CHAR  *, /* Password           */
                 L_SWORD,   /* Password length    */
                 L_CHAR  *, /* Server Name        */
                 L_LONG,    /* Transaction mode   */
                 void *pAstFunction = NULL, /* Async function */
                 void *pUserArg = NULL);    /* User argument  */
  L_LONG ConnectCS(L_CHAR  *, /* UserName           */
                   L_SWORD,   /* Name length        */
                   L_CHAR  *, /* Password           */
                   L_SWORD,   /* Password length    */
                   L_CHAR  *, /* Server Name        */
                   L_CHAR  *, /* Charset            */
                   L_LONG,    /* Transaction mode   */
                   void *pAstFunction = NULL, /* Async function */
                   void *pUserArg = NULL,     /* User argument  */
                   linErrBuf *pErrBuf = NULL);             /* 27.12.10 */
  L_LONG ConnectEnv(L_CHAR    *sCfgName = NULL, 
                    L_SWORD    swCfgNameLength = 0,
                    void      *pAstFunction = NULL, /* Async function */
                    void      *pUserArg = NULL,     /* User argument  */
                    linErrBuf *pErrBuf = NULL);             /* 17.04.13 */
  L_LONG ReConnect(L_CHAR*, /* UserName           */
                   L_SWORD, /* Name length        */
                   L_CHAR*, /* Password           */
                   L_SWORD, /* Password length    */
                   L_CHAR*, /* Server Name        */
                   L_CHAR*, /* Charset            */
                   L_LONG,  /* Transaction mode   */
                   linErrBuf *pErrBuf = NULL);             /* 27.12.10 */
  L_LONG ReConnect(L_CHAR    *sCfgName = NULL,
                   L_SWORD    swCfgNameLength = 0,
                   linErrBuf *pErrBuf = NULL);             /* 17.04.13 */
  L_LONG CloseConnect();
  L_LONG OpenCursor(L_CHAR  *,      /* Cursor Name        */
                    L_SWORD,        /* Name length        */
                    linCursor **);  /* (out) pointer to linCursor */
  L_LONG CloseCursor(linCursor *);  /* linCursor              */
  L_LONG RollBack(void *pAstFunction = NULL, /* Async function */
                  void *pUserArg = NULL);    /* User argument  */
  L_LONG Commit(void *pAstFunction = NULL, /* Async function */
                void *pUserArg = NULL);    /* User argument  */
  L_LONG SetConnectOption(L_SWORD ,   /* Connect option  */
                          void *,     /* Option buffer   */
                          L_LONG *);  /* output length   */
  L_LONG GetConnectOption(L_SWORD,    /* Connect option  */
                          void *,     /* Option buffer   */
                          L_LONG *);  /* output length   */
  L_LONG GetObjDesc(L_SWORD,     /* Object Type     */
                    L_CHAR *,    /* Object Name     */
                    L_SWORD,     /* Name Length     */
                    void *,      /* Out Buffer      */
                    L_LONG *);   /* Buffer size     */
  L_LONG ConnectComplete(L_SWORD *,   /* Complete or no  */
                         L_LONG *,    /* LinAPI error    */
                         L_LONG *,    /* Linter error    */
                         L_LONG *);   /* System error    */
  void SetQBufSettings(L_LONG lBufSize /* size of buffer */, L_BOOL bUseTmpFile = L_TTRUE /* can use temparary file */); /* 06.12.11 */
  /* errors processing: */
  L_ULONG GetErrorsCount();
  void    ClearErrors();
  L_LONG  GetError(L_CHAR  *,  /* buffer for name of module      */
                   L_ULONG *,  /* buffer with sModuleName size   */
                   L_CHAR  *,  /* buffer for text of error       */
                   L_ULONG *,  /* buffer with size of sErrorText */
                   L_LONG  *,  /* buffer for linapi error        */
                   L_LONG  *,  /* buffer for linter error        */
                   L_LONG  *); /* buffer for system error        */
};
                                                           /* 19.06.09 ... */
#define LDS_NLOAD_BLOB_BODY 0x00000001 /* don't load bodies of blobs int linDataSet */

/* Flags for rows & cells: */
#define LDS_ADDED           0x00000001
#define LDS_MODIFIED        0x00000002
#define LDS_NULL            0x00000004

/* Flags for GetCellData & SetCellData */
#define LDS_CD_SMALLBUFFER 0x00000001
#define LDS_CD_NULL        0x00000002

typedef struct
{
  L_CHAR szUserName[MAX_ID_LEN + 2];
  L_CHAR szTableName[MAX_ID_LEN + 2];
  L_CHAR szColumnName[MAX_ID_LEN + 2];
  L_BYTE Pad[4];
}
t_ColDesc;

class DLL_CALL linDataSet
{
private:
  linConnection *m_pConnection;
  linCursor     *m_pCursor;

  L_LONG         m_lFlags;
  L_LONG         m_lColCount;        /* amount of columns */
  L_LONG         m_lRowCount;        /* amount of records */
  L_LONG         m_lIsUpdatable;     /* flag of updatable */
  L_LONG         m_lTransMode;       /* translation mode */
  L_LONG         m_lUniqColsCount;   /* amount of numbers of unique column(s) */
  t_ParamDesc   *m_pColInfo;         /* array of descriptions of columns */
  t_ColDesc     *m_pColInfo2;        /* array of 0-terminated strig descriptions of columns */
  linErrBuf     *m_pErrBuf;          /* buffer for errors */
  L_LONG        *m_plUniqCols;       /* array for numbers of unique column(s) */
  linPtrArray   *m_AnsBuf;           /* pointer to array of answers */
  linPtrArray   *m_AnsRealLen;       /* pointer to array of real length of answers */
  linLongArray  *m_RowFlags;         /* flag for each row */
  linLongArray  *m_ColFlags;         /* flag for each column */
  linPtrArray   *m_CellFlags;        /* flag for each cell */
  linPtrArray   *m_CellFlags2;       /* spec. flag for each cell (for example, blob type) */
  linPtrArray   *m_DelQueryBuf;      /* buffer for queries for delete rows */
  linPtrArray   *m_Index;            /* array with indexes for each column */
  linPtrArray   *m_IndexNULL;        /* array with NULL for each indexed column */
  linLongArray  *m_SortColumns;      /* array with numbers of columns for sort */
  linLongArray  *m_SortRows;         /* array with numbers of rows (with apply sorting) */
  linPtrArray   *m_FindConditions;   /* array with conditions for find operation */
  linLongArray  *m_FindRows;         /* array with founded rows (with apply finding) */

  linErrBuf     *GetErrBuf();
  L_SWORD        AppendStrValue(L_CHAR      *SqlBuffer, 
                                t_ParamDesc *pColInfo, 
                                L_LONG       lCellFlag,
                                L_BYTE      *pValue,
                                L_LONG       lMaxBufLen);
  L_SWORD        GenerateWhereCondition(L_LONG lRow, L_CHAR *SqlBuffer, L_CHAR *SqlBuffer2);
  L_SWORD        InsertRowIntoIndex(L_LONG lRow, L_LONG lColumn);
  L_SWORD        DeleteRowFromIndex(L_LONG lRow, L_LONG lColumn, L_BOOL bDeleteRow);
  L_SWORD        CompareValues(L_LONG lType, L_BYTE *pBuf1, L_LONG lLen1, L_BYTE *pBuf2, L_LONG lLen2, L_LONG *plCode);
  L_SWORD        DichotomiaSearch(L_LONG lRow, L_LONG lColumn, L_LONG *plIndex, L_LONG *plLastIndex);
  L_SWORD        DichotomiaSearchBuf(L_BYTE *pBuf, L_LONG lColumn, L_LONG *plIndex, L_LONG *plLastIndex);
  L_SWORD        SortByNextColumn(linLongArray *pRows, L_LONG lColumnIndex, linLongArray *pNewRows);
  L_SWORD        CheckType(L_LONG lType, L_BYTE *pData, L_LONG lDataLen);
  L_SWORD        GetDataWithType(const L_CHAR *pFuncName, L_LONG lRow, L_LONG lColumn, L_LONG lType, void *Data, L_LONG *plDataLen, L_LONG *plFlags);
  L_SWORD        ConvertAnswerToBuf(const L_CHAR *pFuncName, L_LONG lInLinapiType, L_CHAR *InBuffer, L_LONG lInLen, L_BYTE bPrec, L_BYTE bScale, L_LONG lOutLinapiType, L_CHAR *OutBuffer, L_LONG *plOutLen);
  L_SWORD        GetPrecisionScaleForColumn(const L_CHAR *pFuncName, L_LONG lColumn, L_BYTE *pbPrec, L_BYTE *pbScale);
  L_LONG         RealGetRowCount();

public:
  linDataSet();
  ~linDataSet();

  L_SWORD Create(linConnection* pConnect, L_CHAR *SQLStr, L_LONG lFlags = 0, L_LONG *plUniqCols = NULL, L_LONG lUniqColsCount = 0);
  L_SWORD Close();

  linConnection *GetConnection();
  linCursor     *GetCursor();                              /* 27.12.10 */
  L_LONG GetFlags();

  L_LONG  GetColCount();
  L_LONG  GetRowCount();
  L_BOOL  IsUpdatable(); /* query into this DataSet is updatable */

  t_ParamDesc *GetColInfo(L_LONG lColumn);
  L_SWORD SetCellData(L_LONG lRow, L_LONG lColumn, L_LONG lType, void *Data, L_LONG DataLen, L_LONG lFlags = 0);
  L_SWORD GetCellData(L_LONG lRow, L_LONG lColumn, L_LONG lType, void *Data, L_LONG *plDataLen, L_LONG *plFlags);

  L_SWORD GetValueLength(L_LONG lRow, L_LONG lColumn, L_LONG *plLength);
  L_SWORD SetBlobType(L_LONG lRow, L_LONG lColumn, L_LONG lBlobType);
  L_SWORD GetBlobType(L_LONG lRow, L_LONG lColumn, L_LONG *plBlobType);

  L_SWORD AddRow(L_LONG *plRow = NULL);
  L_SWORD DeleteRow(L_LONG lRow);

  L_SWORD CreateIndex(L_LONG lColumn);
  L_SWORD DropIndex(L_LONG lColumn);

  enum FindCondition{fdEQ=1, fdNEQ, fdGT, fdLT, fdGE, fdLE, fdNL, fdNNL};
  L_SWORD AddCondition(L_LONG lColumn, L_LONG lType, void *Data, L_LONG lDataLen, L_SWORD Condition=fdEQ);
  L_SWORD Find(L_LONG lFindFrom=1);
  L_SWORD FreeFind();

  enum GetAggFunc{gaMin=1, gaMax, gaCount, gaAvg, gaSum};
  L_SWORD GetAggregate(L_LONG lColumn, L_SWORD AggFunc, L_LONG lType, void *Data, L_LONG *plDataLen, L_LONG *plFlags);

  enum SortDirection{sAsc=1, sDesc}; 
  L_SWORD AddSortColumn(L_LONG lColumn, L_WORD SortDirection=sAsc);
  L_SWORD Sort();
  L_SWORD FreeSort();

  L_SWORD SaveChanges(L_CHAR *szFileName=NULL);
  /* errors processing: */
  L_ULONG GetErrorsCount();
  void    ClearErrors();
  L_LONG  GetError(L_CHAR  *,  /* buffer for name of module      */
                   L_ULONG *,  /* buffer with sModuleName size   */
                   L_CHAR  *,  /* buffer for text of error       */
                   L_ULONG *,  /* buffer with size of sErrorText */
                   L_LONG  *,  /* buffer for linapi error        */
                   L_LONG  *,  /* buffer for linter error        */
                   L_LONG  *); /* buffer for system error        */
};                                                         /* ... 19.06.09 */
#endif

