 /***************************************************************************
 *                 RELEX, Inc. Linter DBMS source code                      *
 *              This is public source code of RELEX, Inc.                   *
 *                                                                          *
 *      You have a royalty-free right to use, modify, reproduce and         *
 *     distribute the Sample Files (and/or any modified version) in         *
 *        any way you find useful, provided that you agree that             *
 *       RELEX has no warranty obligations or liability for any             *
 *           Sample Application Files which are modified.                   *
 *                                                                          *
 *                 Copyright (c) 1998-2008 RELEX Inc.                       *
 *                                                                          *
 *                   This software was developed by                         *
 *                                                                          *
 *                            RELEX, Inc.                                   *
 *                         Voronezh, Russia                                 *
 *                                                                          *
 *                        All rights reserved.                              *
 ****************************************************************************
 AUTHOR:  Galizin O.V.
 ----------------------------------------------------------------------------
 PURPOSE: Linter control library header
 ----------------------------------------------------------------------------
 REVISION HISTORY:
 15.07.08 - Initially created
 01.08.08 - Ermakov - C++ support
 ----------------------------------------------------------------------------
*/
#include "inter.h"

#define E_LINCTRL_NOMEM       1 
#define E_LINCTRL_NOINIT      2
#define E_LINCTRL_ALREADYINIT 3
#define E_LINCTRL_GETPATH     4
#define E_LINCTRL_NOTFOUND    5
#define E_LINCTRL_NOTDIR      6
#define E_LINCTRL_OPENDIR     7
#define E_LINCTRL_REMOVE      8
#define E_LINCTRL_RUN         9

#define E_LINCTRL_PIPE        10
#define E_LINCTRL_INVARG      11
#define E_LINCTRL_GENDBFAILED 12
#define E_LINCTRL_WRITE       13
#define E_LINCTRL_LINFAILED   14
#define E_LINCTRL_SHMGET      15
#define E_LINCTRL_LINPID      16
#define E_LINCTRL_ROLE        17
#define E_LINCTRL_SHUTFAILED  18
#define E_LINCTRL_KILLFAILED  19
#define E_LINCTRL_CHKFAILED   20

#define E_LINCTRL_INL         21
#define E_LINCTRL_LOAREL      22

typedef struct 
{
  char *DbPath;
  char *LinterPath;
  char *LinterMbx;
  char *TmpPath;
  int  LinterPid;
#define LINDB_ATTACHED 0x0001
#define LINDB_INIT_MASK ~LINDB_ATTACHED
#define LINDB_INIT_FLG 0x35798642
  L_LONG  Flg;
  L_LONG OpenFlags;
  L_LONG RetStatus;
  L_LONG SysStatus;
}  t_LINDBCtrl;
  
#ifdef __cplusplus
extern "C" {
#endif

L_LONG LINDBCtrlInit(t_LINDBCtrl* LINDBCtrl, char* LinterPath);
L_LONG LINDBCtrlSetDbPath(t_LINDBCtrl* LINDBCtrl, char* DbPath);
L_LONG LINDBCtrlSetLinterMbx(t_LINDBCtrl* LINDBCtrl, char* LinterMbx);
L_LONG LINDBCtrlSetTmpPath(t_LINDBCtrl* LINDBCtrl, char* TmpPath);
L_LONG LINDBCtrlUninit(t_LINDBCtrl* LINDBCtrl);

#ifdef __cplusplus
           }
#endif

typedef enum
{
  LIN_DATABASE_CREATE=1, /* Create database in the specified directory   */
  LIN_DATABASE_RUN,      /* Activate database in the specified directory */
  LIN_DATABASE_ATTACH,   /* Start managing already running DBMS          */
/*  LIN_DATABASE_RUN_TCP,   * Start TCP/IP network agent (NIY)   */
/*  LIN_DATABASE_RUN_JDBC,  * Start JDBC agent (NIY)             */
/*  LIN_DATABASE_RUN_CHECK, * Check if specified DB is activated  
                              and shut down DBMS (NIY)           */
  LIN_DATABASE_SHUT,     /* Shut down DBMS */
/*  LIN_DATABASE_SHUT_TCP,  * Stop TCP/IP network agent (NIY)    */
/*  LIN_DATABASE_SHUT_JDBC, * Stop JDBC agent (NIY)              */
  LIN_DATABASE_DICT,     /* Create database dictionary */
  LIN_DATABASE_ALIVE,    /* Check if DBMS is running   */
  LIN_DATABASE_SET,      /* Reserved */
  LIN_DATABASE_GET       /* Reserved */
} t_LinDatabaseCommand; 

typedef enum
{
  /* Database creation flags */
  LIN_DATABASE_CREATE_PURGE=0x1,     /* Remove all database files before database creation */
  LIN_DATABASE_CREATE_IGNORE=0x2,    /* Re-create DB. Be cautious with this mode!!! */
  LIN_DATABASE_CREATE_GENDB_ADD=0x4, /* The last argument is followed by pointer to additional gendb parameters string */
  
  /* DBMS activation flags */
  LIN_DATABASE_RUN_RDONLY=0x10, /* Run DBMS in read only mode */
  LIN_DATABASE_RUN_ARG=0x20,    /* Specify DBMS kernel starting command line arguments */
  LIN_DATABASE_RUN_TCP=0x40,    /* Start dbs_tcp after DBMS kernel start  */
  LIN_DATABASE_RUN_JDBC=0x80,   /* Start linapid after DBMS kernel start */

  /* DBMS shutdown flags */
  LIN_DATABASE_SHUT_FORCE=0x1000,       /* Shut down DBMS immediately without last data updates saving */
  LIN_DATABASE_SHUT_IGNOREROLE=0x2000,  /* Shut down DBMS which management has been started by LIN_DATABASE_ATTACH command    */
  LIN_DATABASE_SHUT_BY_SHUT_RUN=0x4000, /* Shut down DBMS by shut utility. Additional argument contains username and password */

  /* Kernel state checking flags */
  LIN_DATABASE_ALIVE_SIG=0x10000, /* Check kernel state by sending signal to pid, otherwise by calling chklinter */
  
  /* Dictionary creation flags */
  LIN_DATABASE_DICT_NAME=0x20000  /* Additional argument contains name of SQL script file for database dictionary creation  */
} t_LinDatabaseFlags;

#ifdef __cplusplus
extern "C" {
#endif

L_LONG LinDatabaseControl(t_LINDBCtrl* LINDBCtrl, t_LinDatabaseCommand Command,
                         L_LONG Flags, ...);

#ifdef __cplusplus
e          }
#endif
