 /***************************************************************************
 *                   RELEX, Inc. Linter source code                         *
 *      This is unpublished propietary source code of RELEX, Inc.           *
 *    The copyright notice does not evidence any actual or intended         *
 *                   publication of such source code.                       *
 *                                                                          *
 *                  Copyright (c) 1990-2014 RELEX Inc.                      *
 *                                                                          *
 *                    This software was developed by                        *
 *                                                                          *
 *                             RELEX, Inc.                                  *
 *                          Voronezh, Russia                                *
 *                                                                          *
 *                         All rights reserved.                             *
 ****************************************************************************
 AUTHOR: Ermakov Michael V.
 ----------------------------------------------------------------------------
 PURPOSE: basic data type definitions
 ----------------------------------------------------------------------------
 CREATION DATE: 28.02.97 19:48 ( GMT+3 )
 ----------------------------------------------------------------------------
 USED MACROSES:
  __LINTYPES_H__   -  Internal macros. Prevnt from multiple includes
        __64BIT_ARCH__   -  For 64bit per long datatype architecture
        __UNSIG_CHAR__   -  For 'default char - unsigned char' compilers
        __64BIT_KERNEL__ -  For 64bit datatypes definition
        OS9000           -  For Microware non-POSIX .h file header
        _BCPP_           -  For Borland-C (16Bit) non-POSIX offset definition
        __INT_OFF_T      -  Internal temporary off_t value (for compat.)
        _MSC_VER         -  For Microsoft compilers identification (auto)
        _VMS_            -  For old VMS non-posix .h file naming
        _BSD_
        __WATCOMC__
 ----------------------------------------------------------------------------
 REVISION HISTORY:
        17.05.98  --  Ermakov. New edition.
        28.06.98  --  Ermakov. (MIN values corrected)
        29.06.98  --  Kent, DV (values corrected)
        05.07.98  --  Ermakov (DLONG for 64BIT_ARCH)
        29.07.98  --  Ermakov. (MIN values corrected - 64 bit)
        31.01.99  --  Ermakov. (fpos_t replaced by POSIX off_t structure (.h))
        18.02.99  --  Ermakov. (New integral types for WINNT && int64_bits i/o)
        06.04.99  --  Maxim -- add int64 emulation
        07.04.99  --  Maxim -- define int64 types for some platforms
        12.04.99  --  Ermakov -- L_OFFSET refer to DLONG only for 64BIT_KERNEL
        24.04.99  --  Ermakov. (IRIX && SUNOS)
        26.04.99  --  Ermakov. (AIX)
        27.04.99  --  Ermakov. (LINUX)
        22.07.99  --  Ermakov. (USIX)
        22.07.99  --  DV (MAX values for Sparc SUN)
        21.09.99  --  Ermakov. (MSC_VER check)
        30.09.99  --  DV (_BSD_)
        24.01.00  --  Galizin ( INT64 )
        26.01.00  --  Galizin ( INT64 - default - emulation )
        09.02.00  --  Ermakov ( remove extra comments )
        29.02.00  --  Ermakov (OS9000)
        08.04.02  --  Kent (Windows CE)
        23.12.03  --  Ermakov (For MIDL compatible)
        17.05.04  --  Galizin ( Linux glibc 2.3 TT#11078)
        07.06.04  --  Ermakov (included windef.h for WINCE)
        08.06.04  --  Ermakov (Corrected for IDL compiler)
        19.07.04  --  Galizin ( Linux glibc macro corrected )
        11.09.04  --  Ermakov ( Added ARCH macro for windef.h )
        24.10.04  --  Ermakov ( Added status code )
        19.06.05  --  Ermakov ( Continued 19.07.04 )
        28.09.05  --  FLY (Correct constants)
        15.02.06  --  DV, Ermakov (Unsigned types for addresses)
        23.02.06  --  Ermakov (CE problem avoid for RC compilation)
        08.06.06  --  Galizin (Macroses for memcpy change. Compilers optimization exclude )
        01.08.06  --  DV      (Add FLOAT(N) syntax)
        05.11.06  --  Ermakov (Added L_MINABSx constant - to float value checking)
        13.10.07  --  Ermakov (For QNX enable long long support)
        01.05.09  --  Danila (windows build with __GNUC__, add definition section for L_DLONG and L_UDLONG)
        26.10.09  --  Chernyshov (Added support 'long long' for OS2000)
        06.02.12  --  Galizin (gcc LONG_ALIGN sparc build. Memcpy... defines)
        20.02.12  --  Galizin (L_MEMCPY macro correction)
        31.06.12  --  Glukhov (Correct datatypes)
        08.06.12  --  Glukhov (datatypes)
        27.06.12  --  Egorov  (AMD64 && VC>VC6)
        05.07.12  --  DV      (SQL needs LONG_ROWS_SUPPORTED definition)
        24.01.13  --  DV      (Remove extra float checks)
        22.02.13  --  Galizin (src is too often used in source)
        13.02.14  --  Ermakov (Some macroses moved from global.h)
 ----------------------------------------------------------------------------
*/

#ifndef __LINTYPES_H__
#define __LINTYPES_H__

#ifdef WIN32
#  ifndef __midl                                           /* 08.06.04 23.12.03 */
#    if !defined(_PPC_) && !defined(_ALPHA_) && !defined(_MIPS_) && !defined(_X86_) && defined(_M_IX86) /* 11.09.04 ... */
#      define _X86_
#    endif
#    if !defined(_PPC_) && !defined(_ALPHA_) && !defined(_MIPS_) && !defined(_AMD64_) && defined(_M_AMD64) /* 11.09.04 ... */
#      define _AMD64_
#    endif
#    if !defined(_PPC_) && !defined(_ALPHA_) && !defined(_X86_) && !defined(_MIPS_) && defined(_M_MRX000)
#      define _MIPS_
#    endif
#    if !defined(_PPC_) && !defined(_ALPHA_) && !defined(_X86_) && !defined(_MIPS_) && defined(_M_ALPHA)
#      define _ALPHA_
#    endif
#    if !defined(_PPC_) && !defined(_ALPHA_) && !defined(_X86_) && !defined(_MIPS_) && defined(_M_PPC)
#      define _PPC_
#    endif                                                 /* ... 11.09.04 */
#    include <windef.h>                                    /* 07.06.04 */
#  endif
#endif

#if defined(OS9000)                                        /* 31.01.99 ... */
#  include <sys/ctypes>                                    /* 29.02.00 */
#  define __INT_OFF_T off_t
#elif defined(_BCPP_)
#  include <stdio.h>
#  define __INT_OFF_T fpos_t
#elif defined(_MSC_VER)
#  ifndef WINCE                                            /* 08.04.02 */
#    include <sys/types.h>
#  endif /* WINCE */
#  define __INT_OFF_T _off_t
#elif defined(_VMS_)
#  include <stat.h>
#  define __INT_OFF_T off_t
#else
#  ifndef WINCE  /* RC compiler is'nt set MSC_VER */       /* 23.02.06 */
#    include <sys/types.h>  /* For off_t definition */     /* 31.01.99 */
#  endif
#  define __INT_OFF_T off_t
#endif                                                     /* .. 31.01.99 */

#define __64BIT_KERNEL__

#if defined(_AMD64_) && !defined(__64BIT_ARCH__)
# define __64BIT_ARCH__
#endif

/* Basic integer types */
#define L_BYTE   unsigned char         /* unsigned, 1 byte  */

#ifdef __UNSIG_CHAR__
#  define L_SBYTE  signed char         /* signed,   1 byte  */
#else
#  define L_SBYTE  char                /* signed,   1 byte  */
#endif /* __UNSIG_CHAR__ */

#define L_WORD   unsigned short        /* unsigned, 2 bytes */
#define L_SWORD  short                 /* signed,   2 bytes */

#if defined(__64BIT_ARCH__)
#  define L_LONG   int                 /* signed,   4 bytes */
#  define L_ULONG  unsigned int        /* unsigned, 4 bytes */
#else
#  define L_LONG   long                /* signed,   4 bytes */
#  define L_ULONG  unsigned long       /* unsigned, 4 bytes */
#endif /* __64BIT_ARCH__ */

#ifdef __64BIT_KERNEL__                                    /* 06.04.99... 28.09.05 ... */
#  if ((defined(__GNUC__) && defined(WIN32)) || defined(OS2000)) && !defined(SUPP_LONGLONG) /* 01.05.09 ... 26.10.09 */
#    define SUPP_LONGLONG
#  endif
#  if (defined(_MSC_VER) && _MSC_VER >= 900) || /* For Microsoft compilers identification (auto) 21.09.99 */ \
      (defined(__WATCOMC__) && defined(__WATCOM_INT64__))
#    if _MSC_VER <= 1200
#      define L_DLONG  __int64
#      define L_UDLONG unsigned __int64
#    else
#      define L_DLONG  long long           /* signed,   8 bytes */
#      define L_UDLONG unsigned long long  /* unsigned, 8 bytes */
#    endif
#  elif defined(__64BIT_ARCH__)                            /* 05.07.98 */
#    define L_DLONG  long                /* signed,   8 bytes */
#    define L_UDLONG unsigned long       /* unsigned, 8 bytes */
#  elif defined(_VMS_)    /* For old VMS non-posix .h file naming */
#    define L_DLONG  signed__int64       /* signed,   8 bytes */
#    define L_UDLONG unsigned__int64     /* unsigned, 8 bytes */
#  elif defined(SUPP_LONGLONG) /* && (defined (UNIX) || defined (QNX)) */ /* 13.10.07 26.01.00 */
#    define L_DLONG  long long           /* signed,   8 bytes */
#    define L_UDLONG unsigned long long  /* unsigned, 8 bytes */
#  else
#    define INT64_EMULATION_ENABLE
#    ifndef __midl                                         /* 06.04.99 ... 23.12.03 */
typedef struct 
{
#ifdef DATA_MSBF
  L_ULONG  hi, lo;
#else
  L_ULONG  lo, hi;
#endif
} L_DLONG, L_UDLONG;
#    endif  /* __midl */                                   /* ... 06.04.99 */
#  endif  /* __64BIT_ARCH__ */
#endif  /* __64BIT_KERNEL__ */                             /* ... 28.09.05 */

/* Integer type limits */
#define L_MAXBYTE   ((L_BYTE)0xFF)                         /* 29.06.98 ... */
#define L_MAXSBYTE  ((L_SBYTE)0x7F)                    
#define L_MINSBYTE  ((L_SBYTE)0x80)                        /* 28.06.98 */

#define L_MAXWORD   ((L_WORD)0xFFFF)
#define L_MAXSWORD  ((L_SWORD)0x7FFF)
#define L_MINSWORD  ((L_SWORD)0x8000)                      /* ... 29.06.98 */

#define L_MINLONG   ((L_LONG)0x80000000)                   /* 29.07.98 */
#define L_MAXLONG   ((L_LONG)0x7FFFFFFF)
#define L_MAXULONG  ((L_ULONG)0xFFFFFFFF)                  /* ... 29.07.98 */

#ifdef __64BIT_KERNEL__
#  ifdef INT64_EMULATION_ENABLE                            /* 07.04.99 ...*/
#    define L_MINDLONG      i64_MINDLONG()
#    define L_MAXDLONG      i64_MAXDLONG()
#    define L_MAXUDLONG     i64_MAXUDLONG()
#  else                                                    /* 08.04.99 28.09.05 ... */
#    define L_MINDLONG ((L_DLONG) UI64(0x8000000000000000))
#    define L_MAXDLONG             I64(0x7FFFFFFFFFFFFFFF)
#    define L_MAXUDLONG           UI64(0xFFFFFFFFFFFFFFFF)
#  endif                                                   /* ...07.04.99 ... 28.09.05 */
#endif /* __64BIT_KERNEL__ */

/* Basic float types */
#define L_REAL   float       /* float, 4 bytes     */
#define L_DOUBLE double      /* float, 8 bytes     */

/* Float type limits */
#ifdef OLD_FLOAT_CHECKS                                    /* 24.01.13 */
#define L_MAXREAL       +1.0E+38
#define L_MINREAL       -1.0E+38
#define L_MAXDOUBLE     L_MAXREAL
#define L_MINDOUBLE     L_MINREAL
#define L_MINABSREAL    1.0e-38                            /* 05.11.06 */
#define L_MINABSDOUBLE  1.0e-38                            /* 05.11.06 */
#else                                                      /* 24.01.13 ... */
#define L_MAXREAL       3.402823466e+38F                   
#define L_MINREAL       -L_MAXREAL
#define L_MAXDOUBLE     1.7976931348623158e+308
#define L_MINDOUBLE     -L_MAXDOUBLE
#endif                                                     /* ... 24.01.13 */

#define L_MAXREALBINPREC 24 /* FLT_MANT_DIG */             /* 01.08.06 */
#define L_MAXDBLBINPREC  53 /* DBL_MANT_DIG */             /* 01.08.06 */

/* Boolean type */
#define L_BOOL    unsigned char
#define L_TFALSE   0
#define L_TTRUE    1
#define L_TUNKNOWN 2
#define L_ISTRUE(expr) ((L_BOOL) !!(expr))

/* Character types */
#define L_CHAR    char
#define L_UNICHAR L_WORD       /* Reserverd for UNICODE symbols */

/* Addresses and offsets */
#define L_ADDRESS void*        /* Pointer datatype */
#define L_CHARPTR char*        /* Pointer to char datatype */

#if defined(__64BIT_KERNEL__) && defined(WIN32) && defined(WINDOWS_NT) && \
   (defined(__WATCOM_INT64__) || \
   (defined(_MSC_VER) && (_INTEGRAL_MAX_BITS >= 64)))      /* 12.04.99 */
# define L_OFFSET  L_DLONG     /* Direct file addresing */ /* 18.02.99 */
#else
# define L_OFFSET  __INT_OFF_T /* Direct file addresing */ /* 31.01.99 */
#endif

/* Simple datatype for address storing */
#ifdef __64BIT_ARCH__
#  define L_ADRTYPE L_UDLONG                               /* 15.02.06 */
#else
#  define L_ADRTYPE L_ULONG                                /* 15.02.06 */
#endif /* __64BIT_ARCH__ */

#define L_NULL  NULL

#define L_NIL   0xFF
#define L_NILW  0xFFFF
#define L_NILL  0xFFFFFFFF

#ifdef __64BIT_KERNEL__
#  ifdef INT64_EMULATION_ENABLE                            /* 07.04.99...*/
#    define L_NILDL i64_NILDL()
#  else
#    define L_NILDL ((L_DLONG)L_MAXUDLONG)                 /* 28.09.05 */
#  endif                                                   /*...07.04.99 */
#endif /* __64BIT_KERNEL__ */

#ifdef __64BIT_KERNEL__                                    /* 28.09.05 */
#  include "int64.h"                                       /* 07.04.99 */
#endif

#if defined(LONG_ALIGN)                                    /* 08.06.06 .. */
#define L_MEMCPY2(DST, SRC) \
do\
{\
  char *LocalMacroDst = (char*)(DST);                                 /* 06.02.12 22.02.13 ... */\
  char *LoacalMacroSrc = (char*)(SRC);                                 /* 06.02.12 */\
  *LocalMacroDst++ = *LoacalMacroSrc++;\
  *LocalMacroDst = *LoacalMacroSrc;\
} while(0)

#define L_MEMCPY4(DST, SRC) \
do\
{\
  char *LocalMacroDst = (char*)(DST);\
  char *LoacalMacroSrc = (char*)(SRC);\
\
  *LocalMacroDst++ = *LoacalMacroSrc++;                                          /* 06.02.12 ... */\
  *LocalMacroDst++ = *LoacalMacroSrc++;\
  *LocalMacroDst++ = *LoacalMacroSrc++;\
  *LocalMacroDst = *LoacalMacroSrc;                                              /* ... 06.02.12 */\
}while (0)

#define L_MEMCPY8(DST, SRC) \
do\
{\
  char *LocalMacroDst = (char*)(DST);\
  char *LoacalMacroSrc = (char*)(SRC);\
  \
  *LocalMacroDst++ = *LoacalMacroSrc++;                                          /* 06.02.12 ... */\
  *LocalMacroDst++ = *LoacalMacroSrc++;\
  *LocalMacroDst++ = *LoacalMacroSrc++;\
  *LocalMacroDst++ = *LoacalMacroSrc++;\
  *LocalMacroDst++ = *LoacalMacroSrc++;\
  *LocalMacroDst++ = *LoacalMacroSrc++;\
  *LocalMacroDst++ = *LoacalMacroSrc++;\
  *LocalMacroDst = *LoacalMacroSrc;                                              /* ... 06.02.12 */\
} while(0)


#define L_MEMCPY(DST, SRC, size) \
  do\
  {\
    char* LocalMacroDst = (char*)(DST);\
    char* LoacalMacroSrc = (char*)(SRC);\
\
    switch ( (size) )\
    {\
      case 8:\
        *LocalMacroDst++ = *LoacalMacroSrc++;                                    /* 06.02.12 ... */\
        *LocalMacroDst++ = *LoacalMacroSrc++;\
      case 6:\
        *LocalMacroDst++ = *LoacalMacroSrc++;\
        *LocalMacroDst++ = *LoacalMacroSrc++;\
      case 4:\
        *LocalMacroDst++ = *LoacalMacroSrc++;\
        *LocalMacroDst++ = *LoacalMacroSrc++;\
      case 2:\
        *LocalMacroDst++ = *LoacalMacroSrc++;\
        *LocalMacroDst++ = *LoacalMacroSrc++;                                    /* ... 06.02.12 */\
        break;\
      default:\
      {\
        int i = (size) - 1;                               /* 20.02.12 */ \
        \
        do\
        {\
          ((char*)(LocalMacroDst))[i] = ((char*)(LoacalMacroSrc))[i]; /* ... 22.02.13 */\
          i--;\
        } while ( i >= 0 );\
      }\
    }\
  } while(0)

#define L_MEMCPY_DECIMAL(DST, SRC) L_MEMCPY(DST, SRC, sizeof(L_DECIMAL)) /* 06.02.12 */

#define L_MEMSET(DST, C, LEN)\
  do\
  {\
    int i = (LEN)-1;\
    do\
    {\
      ((char*)(DST))[i] = (C);\
      i--;\
    } while ( i >= 0 );\
  } while(0)
#else
# define L_MEMCPY2(DST, SRC) do { *(L_WORD  *)(DST) = *(L_WORD  *)(SRC); } while (0)
# define L_MEMCPY4(DST, SRC) do { *(L_LONG  *)(DST) = *(L_LONG  *)(SRC); } while (0)
# define L_MEMCPY8(DST, SRC) do { memcpy((DST), (SRC), 8); } while (0)
# define L_MEMCPY_DECIMAL(DST, SRC) do { memcpy((DST), (SRC), sizeof(L_DECIMAL)); } while(0)
# define L_MEMCPY(DST, SRC, size) memcpy((DST), (SRC), size)
# define L_MEMSET(DST, C, LEN)    memset((DST), (C), LEN)
#endif                                                     /* .. 08.06.06 */
#define L_MEMCPY_DOUBLE(DST, SRC) L_MEMCPY8(DST, SRC)
#define L_MEMCPY_REAL(DST, SRC)   L_MEMCPY4(DST, SRC)      /* ... 06.02.12 */


#if defined(LINUX)                                         /* 17.05.04 ... */
#  undef isdigit
#  undef isspace
#  undef islower
#  undef isupper
#  undef isalpha
/* WARNUNG: don't use next : LAnsiTolower(*c++) */
#  define LAnsiToupper(c) (((int)(c)<='z' && (int)(c)>='a')?(int)(c) + 'A' - 'a':(int)(c)) /* 19.07.04 */
#  define LAnsiTolower(c) (((int)(c)<='Z' && (int)(c)>='A')?(int)(c) + 'a' - 'A':(int)(c)) /* 19.06.05 19.07.04 */
#else
#  define LAnsiToupper(a) toupper(a)
#  define LAnsiTolower(a) tolower(a)
#endif                                                     /* ... 17.05.04 */

/* Return codes */
#define LINTER_ERROR  -1                                   /* 24.10.04 */
#define LINTER_SUCCESS 0                                   /* 24.10.04 */

#ifdef _VER_MAX
#if _VER_MAX >= 600
# define LINTER_UNICODE
#endif

#if _VER_MAX >= 610
# define LINTER_MVCC
#endif

#if _VER_MAX >= 600 && !defined(LITE_NOLONG) && !defined(LITE_NOBLOB) /* 05.07.12 ... */
# define LONG_ROWS_SUPPORTED
#endif                                                                /* ... 05.07.12 */
#endif

#ifdef __cplusplus
# define L_EXTERN_C extern "C"
#else
# define L_EXTERN_C
#endif

#include <limits.h>                                        /* 13.02.14 ... */

#ifndef __midl
#include <stdlib.h>

#ifdef PATH_MAX
#define L_MAX_PATH PATH_MAX
#elif defined(MAX_PATH)
#define L_MAX_PATH MAX_PATH
#elif defined(_MAX_PATH)
#define L_MAX_PATH _MAX_PATH
#elif defined(VMS)
#define L_MAX_PATH 1024
#elif defined(OS9000)
#define L_MAX_PATH MAXPATHLEN
#else
#warning "_MAX_PATH, MAX_PATH and PATH_MAX not defined"
#endif 
#endif 

#define L_MAX_MBX     32

/* Must be 260 - DB_INFO in WinNT use this constant */
#define MAX_SY00_PATH 260                                  /* ... 13.02.14 */

#endif /* __LINTYPES_H__ */
