 /***************************************************************************
 *           RELEX, Inc. Linter Kernel source code                          *
 *  This is unpublished propietary source code of RELEX, Inc.               *
 *  The copyright notice does not evidence any actual or intended           *
 *  publication of such source code.                                        *
 *                                                                          *
 *                 Copyright (c) 1990-2014 RELEX Inc.                       *
 *                                                                          *
 *                   This software was developed by                         *
 *                                                                          *
 *                            RELEX, Inc.                                   *
 *                         Voronezh, Russia                                 *
 *                                                                          *
 *                        All rights reserved.                              *
 ****************************************************************************
 AUTHOR:
 ----------------------------------------------------------------------------
 PURPOSE: UTF support
 ----------------------------------------------------------------------------
 REVISION HISTORY:
 creation date: 17.10.02

 03.12.01 -- KOV (s_LINNLS_Utf8Ucs2 corrected)
 04.01.02 -- KOV (changes for correct length)
 21.12.05 -- Ermakov, KOV (Fix for translation ucs2 to utf8)
 07.02.14 -- Galizin (Working with unaligned L_UNICHAR strings)
 09.10.14 -- Muratova (Added functions for convertion Ucs4->Ucs2 and Ucs2->Ucs4)
*/

#include "lnlsutf.h"

/*
 *  Utf8 -> Ucs2 conversion
 */
L_LONG LINNLS_Utf8Ucs2(L_CHAR   *InStr,
                       L_LONG    InStrSize,
                       L_LONG    OutBufSize,
                       L_UNICHAR *OutStr,
                       L_LONG    *OutStrSize)
{
  L_UNICHAR *ucstr;

  ucstr = (L_UNICHAR*) OutStr;
  *OutStrSize = 0;

  while (InStrSize > 0)
  {
    if (OutBufSize)
    {
      OutBufSize --;
      if (!(*InStr & 0x80))  /* US-ASCII */
      {
        if (ucstr)
          *ucstr = *InStr;
        InStrSize --;
        InStr++;                                          /* 03.12.01 */
      }
      else if ((*InStr & 0xE0) == 0xC0)
      {
        if (ucstr)
          *ucstr = ( ( (L_UNICHAR)( *(InStr + 1) ) ) & 0x3F) | 
                   ( ( ( (L_UNICHAR)(*InStr) ) & 0x1F) << 6);
        InStrSize -= 2;
        InStr += 2;                                        /* 03.12.01 */
      }
      else if ((*InStr & 0xF0) == 0xE0)
      {
        if (ucstr)
          *ucstr = ( ( (L_UNICHAR)( *(InStr + 2) ) ) & 0x3F) | 
                   ( ( ( (L_UNICHAR)( *(InStr + 1) ) ) & 0x3F) << 6) | 
                   ( ( ( (L_UNICHAR)(*InStr) ) & 0x0F) << 12);
        InStrSize -= 3;
        InStr += 3;                                        /* 03.12.01 */
      }
      else if ((*InStr & 0xF8) == 0xF0) 
      {
        return LINTER_ERROR; /* UCS-4 does not supported */
      }
      else if ((*InStr & 0xFC) == 0xF8)
      {
        return LINTER_ERROR; /* UCS-4 does not supported */
      }
      else if ((*InStr & 0xFE) == 0xFC)
      {
        return LINTER_ERROR; /* UCS-4 does not supported */
      }
      else
      {
        return LINTER_ERROR; /* not utf-8 symbol */
      }
      if (ucstr)
        ucstr ++;
      (*OutStrSize) += sizeof(L_UNICHAR);                  /* 03.12.01 *//* 04.01.02 */
    }
  }
  if (OutBufSize > 0)
  {
    if (ucstr)
      *ucstr = L'\0';
  }
  return LINTER_SUCCESS;
}


/*
 *  Ucs2 -> Utf8 conversion
 */
L_LONG LINNLS_Ucs2Utf8(L_UNICHAR *InStr,
                       L_LONG     InStrSize,
                       L_LONG     OutBufSize,
                       L_CHAR    *OutStr,
                       L_LONG    *OutStrSize)
{
  L_UNICHAR *ucstr;

  ucstr = (L_UNICHAR*)InStr;
  *OutStrSize = 0;

  while (InStrSize --)
  {
    if (OutBufSize > 0)
    {
      L_UNICHAR UcsCh;                                     /* 07.02.14 */

      L_MEMCPY2(&UcsCh, ucstr);                            /* 07.02.14 */
      if (!(UcsCh & 0xFF80))        /* 0000-007F   0xxxxxxx */ /* 07.02.14 */
      {
        if (OutStr)
          *OutStr ++ = (L_CHAR)UcsCh;                          /* 07.02.14 */
        OutBufSize --;
        (*OutStrSize) ++;
      }
      else if ((UcsCh & 0xFF80) &&                             /* 07.02.14 */
               !(UcsCh & 0xF800) )  /* 0080-07FF   110xxxxx 10xxxxxx */
      {
        if(OutBufSize >= 2)
        {
          OutBufSize -= 2;
          if (OutStr)
          {
            *OutStr ++ = (UcsCh >> 6)   | 0xC0;            /* 07.02.14 */
            *OutStr ++ = (UcsCh & 0x3F) | 0x80;
          }
          (*OutStrSize) += 2;
        }
        else
        {
          OutBufSize = 0;
        }
      }
      else                           /* 0800-FFFF   1110xxxx 10xxxxxx 10xxxxxx */
      {
        if(OutBufSize >= 3)
        {
          OutBufSize -= 3;
          if (OutStr)
          {
            *OutStr ++ = (UcsCh >> 12)          | 0xE0;    /* 07.02.14 */
            *OutStr ++ = ((UcsCh & 0xFC0) >> 6) | 0x80;      /* 21.12.05 */
            *OutStr ++ = (UcsCh & 0x3F)         | 0x80;
          }
          (*OutStrSize) += 3;
        }
        else
        {
          OutBufSize = 0;
        }
      }
    }
    ucstr++;                                            /* 03.12.01 */
  }
  if (OutBufSize > 0)
  {
    if (OutStr)
      *OutStr = '\0';
  }
  return LINTER_SUCCESS;
}

/*
 *  Ucs4 -> Ucs2 conversion
 */
L_LONG LINNLS_Ucs4Ucs2(L_CHAR   *InUcs4,             /* 09.10.14 ... */
                 L_LONG    InStrSize,
                 L_UNICHAR *OutUcs2)
{
  L_ULONG *In;
  for( In = (L_ULONG*)InUcs4; InStrSize--;)
    *OutUcs2++ = (unsigned short)*In++;
  return LINTER_SUCCESS;
}

/*
 *  Ucs2 -> Ucs4 conversion
 */
L_LONG LINNLS_Ucs2Ucs4(L_UNICHAR *InUcs2,
                       L_LONG     InStrSize,                       
                       L_CHAR    *OutUcs4)
{
  L_ULONG *Out;

  for( Out = (L_ULONG*)OutUcs4; InStrSize--; )
    *Out++ = *InUcs2++;

  return LINTER_SUCCESS;
}                                                    /* ... 09.10.14 */