/****************************************************************************
 *           RELEX, Ltd. Linter precompiler library source code             *
 *  This is unpublished propietary source code of RELEX, Ltd.               *
 *  The copyright notice does not evidence any actual or intended           *
 *  publication of such source code.                                        *
 *                                                                          *
 *  Copyright (c) 1994-2003 RELEX Ltd.                                      *
 *                                                                          *
 *                   This software was developed by                         *
 *                                                                          *
 *                            RELEX, Ltd.                                   *
 *                         Voronezh, Russia                                 *
 *                                                                          *
 *                        All rights reserved.                              *
 ****************************************************************************
 AUTHOR: Razuwaev A.V.
 ----------------------------------------------------------------------------
 PURPOSE:
 ----------------------------------------------------------------------------
 REVISION HISTORY:
 ----------------------------------------------------------------------------
*/
/* 04.04.03 - RAW (code revision; lite client)                                */
/* 02.09.03 - RAW (GET DIAGNOSTICS)                                           */
/******************************************************************************/

#ifndef __sqlda__
#define __sqlda__


#define SQL93_CHARACTER_TYP           1
#define SQL93_NUMERIC_TYP             2
#define SQL93_DECIMAL_TYP             3
#define SQL93_INTEGER_TYP             4
#define SQL93_SMALLINT_TYP            5
#define SQL93_FLOAT_TYP               6
#define SQL93_REAL_TYP                7
#define SQL93_DOUBLE_TYP              8
#define SQL93_DATETIME_TYP            9
#define SQL93_INTERVAL_TYP           10
#define SQL93_CHARACTER_VARYING_TYP  12
#define SQL93_EMUMERATED_TYP         13
#define SQL93_BIT_TYP                14
#define SQL93_BIT_VARYING_TYP        15
#define SQL93_BOOLEAN_TYP            16
#define SQL93_ABSTRACT_TYP           17
#define SQL93_ROWID_TYP              18

#define SQL93_BLOB_TYP               30
#define SQL93_BLOB_LOCATOR_TYP       31
#define SQL93_CLOB_TYP               40
#define SQL93_CLOB_LOCATOR_TYP       41

/* Codes associated with datetime data types in Dynamic SQL*/
#define SQL93_DATE                    1
#define SQL93_TIME                    2
#define SQL93_TIMESTAMP               3
#define SQL93_TIME_WITH_TIME_ZONE     4
#define SQL93_TIMESTAMP_WITH_TIME_ZONE 5

/* interval qualifier in dynamic SQL*/
#define SQL93_YEAR                    1
#define SQL93_MONTH                   2
#define SQL93_DAY                     3
#define SQL93_HOUR                    4
#define SQL93_MINUTE                  5
#define SQL93_SECOND                  6
#define SQL93_YEAR_TO_MONTH           7
#define SQL93_DAY_TO_HOUR             8
#define SQL93_DAY_TO_MINUTE           9
#define SQL93_DAY_TO_SECOND          10
#define SQL93_HOUR_TO_MINUTE         11
#define SQL93_HOUR_TO_SECOND         12
#define SQL93_MINUTE_TO_SECOND       13

/* Precompiler Data Types */
#define PCC_INT_TYP   SQL93_INTEGER_TYP           /*1*/ /* integer        (int)     */
#define PCC_FLT_TYP   SQL93_FLOAT_TYP             /*2*/ /* real           (float)   */
#define PCC_DEC_TYP   SQL93_DECIMAL_TYP           /*3*/ /* DECIMAL/NUMERIC          */
#define PCC_CHR_TYP   SQL93_CHARACTER_TYP         /*4*/ /* character      (char)    */
#define PCC_TXT_TYP   SQL93_CHARACTER_VARYING_TYP /*5*/ /* text           (VARCHAR) */
#define PCC_SML_TYP   SQL93_SMALLINT_TYP    /* SMALLINT */
#define PCC_REL_TYP   SQL93_REAL_TYP        /* REAL     */
#define PCC_DBL_TYP   SQL93_DOUBLE_TYP      /* DOUBLE   */
#define PCC_DAT_TYP   SQL93_DATETIME_TYP    /* DATETIME */
#define PCC_BLB_TYP   SQL93_BLOB_TYP        /* BLOB */
#define PCC_BIT_TYP   SQL93_BIT_TYP         /* BIT  */
#define PCC_BVT_TYP   SQL93_BIT_VARYING_TYP /* BIT VARYING*/
#define PCC_BOO_TYP   SQL93_BOOLEAN_TYP     /* BOOLEAN */

/* implementation defined types : */
#define PCC_INV_TYP   -1   /* undefined                */
#define PCC_BIGINT_TYP -5  /* BIGINT                   */
#define PCC_CHR_CPY   -6   /* character      (IMMED)   */
#define PCC_TXT_CPY   -7   /* text           (IMMED)   */

#define PCC_CUR_TYP   -8   /* CURSOR_PCI               */ 
#define PCC_CTX_TYP   -9   /* CONTEXT_PCI              */ 
#define PCC_DSC_TYP  -10   /* sqlda */

typedef enum                                                               /* 02.09.03 */
{
  e_number                  = 0,
  e_more                    = 1,
  e_command_function        = 2,
  e_command_function_code   = 3,
  e_row_count               = 4,
  e_transactions_committed  = 5,
  e_transactions_rolled_back= 6,
  e_transaction_active      = 7,
  e_condition_number        = 8,
  e_returned_sqlstate       = 9,  
  e_class_origin            = 10,
  e_subclass_origin         = 11,
  e_server_name             = 12,
  e_connection_name         = 13,
  e_constraint_catalog      = 14,
  e_constraint_schema       = 15,
  e_constraint_name         = 16,
  e_trigger_catalog         = 17,
  e_trigger_schema          = 18,
  e_trigger_name            = 19,
  e_catalog_name            = 20,
  e_schema_name             = 21,
  e_table_name              = 22,
  e_column_name             = 23,
  e_cursor_name             = 24,
  e_routine_catalog         = 25,
  e_routine_schema          = 26,
  e_routine_name            = 27,
  e_specific_name           = 28,
  e_parameter_name          = 29,
  e_message_text            = 30,
  e_message_length          = 31,
  e_message_octet_length    = 32,
  e_dynamic_function        = 33,
  e_dynamic_function_code   = 34
}e_diag_item;


#ifdef __cplusplus
extern "C" {
#endif

/*
** SQL Descriptor Area
*/
struct t_sqlda
{
  int     N; /* Descriptor entries count                                */
#ifndef PCL_LITE
  void(*DF)(void);/* dynamic function                                   */
#endif
  char  **V; /* Pointer to main variables addresses array               */
  int    *L; /* Pointer to buffer lengths array (HOST language lengths) */
  short  *T; /* Pointer to types array (HOST language types)            */
  int    *D; /* Pointer to dimensions array                       (!!!) */
#ifndef PCL_LITE
  short  *P; /* Pointer to precision array       */
  short  *SC;/* Pointer to scale array           */
#endif
  short  *RL;/* Pointer to returned length array */
  short  *BL;/* Pointer to binding length array:internal */
  short  **I;/* Pointer to indicator variables addresses array          */
  int     F; /* Count of variables found by DESCRIBE                    */
#ifndef PCL_LITE
  char  **S; /* Pointer to main variable name addresses array           */
  short  *M; /* Pointer to max main var name lengths array              */
  short  *C; /* Pointer to current main var name lengths array          */
  char  **X; /* Pointer to indicator variable name addresses array      */
  short  *Y; /* Pointer to max indicator var name lengths array         */
  short  *Z; /* Pointer to current indicator var name lengths array     */
#endif
  short  *MN; /* Descriptor value filled manually - TRUE/FALSE           */
  int    NL; /* Reserved descriptor entries count                 (!!!) */
  int    FR; /* Descriptor was used by DESCRIBE                   (!!!) */
  int    UD; /* Descriptor was allocated by ALLOCATE SQL DESCRIPTOR  st */
#ifdef CHECKING
  L_LONG   sign;
#endif
};

typedef struct t_sqlda  sqlda;
typedef struct t_sqlda *psqlda;

#ifdef __cplusplus
}
#endif

#endif /* __sqlda__ */


/******************************************************************************/
/********************************** SOURCE END ********************************/
/******************************************************************************/
