 /***************************************************************************
 *           RELEX, Inc. Linter sql queries formatter source code           *
 *  This is unpublished proprietary source code of RELEX, Inc.              *
 *  The copyright notice does not evidence any actual or intended           *
 *  publication of such source code.                                        *
 *                                                                          *
 *                 Copyright (c) 1990-2013 RELEX Inc.                       *
 *                                                                          *
 *                   This software was developed by                         *
 *                                                                          *
 *                            RELEX, Inc.                                   *
 *                         Voronezh, Russia                                 *
 *                                                                          *
 *                        All rights reserved.                              *
 ****************************************************************************
 AUTHOR:
 ----------------------------------------------------------------------------
 PURPOSE: format sql queries for better readability
 ----------------------------------------------------------------------------
 REVISION HISTORY:
*/

#ifndef __SQLF_H__
#define __SQLF_H__

/*
    L_LONG FormatQueries(const L_CHAR *inBuf, L_CHAR **outBuf, L_LONG caseFlag, L_LONG numOfTabs)

    The 'FormatQueries' function formats SQL query stored at 'inBuf' for better readability.
  Function dynamically allocates a string to hold the output. The 'outBuf' is the address of a 
  'L_CHAR *', and a successful call stores a pointer to the newly allocated string at that location.
  The return value is zero on success format, -1 if not enough memory, -2 if internal error occurs.
  The 'caseFlag' argument controls conversion of the key words and the identifies. This is a bit mask.
    The following macros are defined:
    - macro KW_CASE
        If set all key words will be converted to upper case.
    - macro ID_CASE
        If set all identifies in SQL query will be converted to upper case except ones in double
        quotes.
    The 'numOfTabs' argument defines the number of indents to be performed in sub-queries. Each
  indent is equals two spaces;
*/

#define KW_CASE 1
#define ID_CASE 2

L_LONG FormatQueries(const L_CHAR *inBuf, L_CHAR **outBuf, L_LONG caseFlag, L_LONG numOfTabs);

#endif /* __SQLF_H__ */
