 /***************************************************************************
 *           RELEX, Inc. Linter UNICODE library source code                 *
 *  This is unpublished proprietary source code of RELEX, Inc.              *
 *    The copyright notice does not evidence any actual or intended         *
 *                   publication of such source code.                       *
 *                                                                          *
 *                  Copyright (c) 1990-2014 RELEX Inc.                      *
 *                                                                          *
 *                    This software was developed by                        *
 *                                                                          *
 *                             RELEX, Inc.                                  *
 *                          Voronezh, Russia                                *
 *                                                                          *
 *                         All rights reserved.                             *
 ****************************************************************************
  AUTHOR: Pavel Pasechnik (Pasec)
 ----------------------------------------------------------------------------
 PURPOSE: UNICODE support
 ----------------------------------------------------------------------------
 REVISION HISTORY:
 ----------------------------------------------------------------------------

  23.12.96 -- creation date
  14.09.99 -- Ermakov (For OS9000)
   8.12.99 -- Pasec (added KOI8 codepage)
  16.05.00 -- ReAn -- UnicharTo866
  10.09.01 -- Pasec (MSBF)
  20.08.02 -- Ermakov (ENGLATIN1 tables added(850/1252). Changed tables names)
  22.08.02 -- DV (_VER_MAX < 600)
  08.10.02 -- DV (Against Intel compiler feature - TT#6319)
  19.03.03 -- DV (LITE version compilation corrected)
  02.04.03 -- Galizin ( For lite <600 need convertation functions)
  20.07.07 -- FLY (Use Linter types)
                  (Unified code)
  27.02.13 -- Galizin ( Unaligned data conversion)
  23.09.13 -- Galizin (Utf-8)
  24.01.14 -- KOV (Correction for correct work of ClientUnicode2Target function after CODEPAGE command (INL), TT#24257)
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "uniconv.h"
#include "inter.h"

#if !defined (LITE_NOSPPREPQ) || _VER_MAX < 600 || defined (PHRASE_FILTER)       /* 02.04.03 19.03.03 */

#ifdef _ENGLATIN1
#include "uc850.h"
#include "uc1252.h"
#include "uc8859_1.h"

static  L_UNICHAR *uc_To_Unicode_1  = uc850_To_Unicode;
static  L_WORD    *To_OffSet_1      = To_850_OffSet;
static  L_BYTE    *cUnicode_To_1    = cUnicode_To_850;
static  L_UNICHAR *uc_To_Unicode_2  = uc1252_To_Unicode;
static  L_WORD    *To_OffSet_2      = To_1252_OffSet;
static  L_BYTE    *cUnicode_To_2    = cUnicode_To_1252;
static  L_UNICHAR *uc_To_Unicode_3  = uc8859_1_To_Unicode;
static  L_WORD    *To_OffSet_3      = To_8859_1_OffSet;
static  L_BYTE    *cUnicode_To_3    = cUnicode_To_8859_1;
#else
#include "uc866.h"
#include "uc1251.h"
#include "uckoi8.h"

static  L_UNICHAR *uc_To_Unicode_1  = uc866_To_Unicode;
static  L_WORD    *To_OffSet_1      = To_866_OffSet;
static  L_BYTE    *cUnicode_To_1    = cUnicode_To_866;
static  L_UNICHAR *uc_To_Unicode_2  = uc1251_To_Unicode;
static  L_WORD    *To_OffSet_2      = To_1251_OffSet;
static  L_BYTE    *cUnicode_To_2    = cUnicode_To_1251;
static  L_UNICHAR *uc_To_Unicode_3  = ucKOI8_To_Unicode;
static  L_WORD    *To_OffSet_3      = To_KOI8_OffSet;
static  L_BYTE    *cUnicode_To_3    = cUnicode_To_KOI8;
#endif

#define ToUni(map, out, in, len)  \
  for ( ; len > 0; --len) *out++ = map[*in++];

#define FromUni(map, off, out, in, len) \
  for ( ; len; ++in, --len)                             /* 27.02.13 ... */ \
  {\
    L_UNICHAR InpChar;\
    \
    L_MEMCPY2(&InpChar, in);\
    *out++ = map[off[InpChar >> 8] | (L_BYTE)InpChar];\
  }                                                         /* ... 27.02.13 */

typedef L_LONG (*ConversionFunc_t)(L_UNICHAR* In, L_LONG InLen, L_BYTE* Out); /* 23.09.13 ... */
/* This function used for all time of conversion for single process.
   Ignored MT specific. */
static ConversionFunc_t ConversionFunc = NULL;              /* ... 23.09.13 */

#ifdef LOADMAP
#define MaxCP        3
#define MaxMapTables 16
#define MaxSize      MaxMapTables * 256

static L_WORD     map_offset[256];
static L_BYTE     map[MaxSize];
static L_UNICHAR  mapuni[256];
static L_WORD     nCPage;
static L_CHAR    *arlCPage[MaxCP] = {"866", "1251", "KOI8"};


static int LoadRuleTable(L_WORD codepage)
{
  FILE * fp;
  L_CHAR name[51];
  L_BYTE cc[256], c;
  L_WORD i, i1, c_count;

  if (nCPage == codepage)
   return 0;
  if (codepage > MaxCP || !codepage)
   return 1;
  sprintf(name, "%s_uni.map", arlCPage[codepage - 1]);
  if ( !(fp = fopen(name, "rb")) )
   return 1;
  nCPage = codepage;
  fgets(name, 50, fp);
  fgets(name, 50, fp);
  fgets(name, 50, fp);
  fread(mapuni, sizeof(L_UNICHAR), 256, fp);
  fclose(fp);

  for (i = 0, c_count = 0; i < 256; i++)
  {
    c = ((L_BYTE *)mapuni)[i << 1 | 1];
    for (i1 = 0; i1 < c_count && cc[i1] != c; i1++)
      ;
    if (i1 == c_count)
      cc[c_count++] = c;
  }

  if (c_count > MaxMapTables)
    return 1;

  for (i = 0; i < 256; i++)
   map_offset[i] = c_count * 256;

  for (i = 0; i < c_count; i++)
   map_offset[cc[i]] = i * 256;

  memset(map, '.', c_count * 256);

  for (i = 0; i < 256; i++)
    map[map_offset[*((L_BYTE *)&mapuni[i] + 1)] + *((L_BYTE *)&mapuni[i])] = (L_BYTE)i;
  return 0;
}

L_UNICHAR * CharToUnichar(L_BYTE    * cInStr,
                          L_LONG      InLen,
                          L_UNICHAR * ucOutStr,
                          L_WORD      codepage)
{
  if (LoadRuleTable(codepage))
    return NULL;
  ToUni(mapuni, ucOutStr, cInStr, InLen);
  return ucOutStr;
}

L_BYTE * UnicharToChar(   L_UNICHAR * ucInStr,
                          L_LONG      InLen,
                          L_BYTE    * cOutStr,
                          L_WORD      codepage)
{
  if (LoadRuleTable(codepage))
    return NULL;
  FromUni(map, map_offset, cOutStr, ucInStr, InLen);
  return cOutStr;                                       /* 14.09.99 */
}
#endif

void API_866ToUnichar(const L_BYTE *cInStr, L_LONG InLen, L_UNICHAR *ucOutStr)
{
  ToUni(uc_To_Unicode_1, ucOutStr, cInStr, InLen);
}

void API_UnicharTo866(const L_UNICHAR *ucInStr, L_LONG InLen, L_BYTE *cOutStr)
{
  FromUni(cUnicode_To_1, To_OffSet_1, cOutStr, ucInStr, InLen);
}

void API_1251ToUnichar(const L_BYTE *cInStr, L_LONG InLen, L_UNICHAR *ucOutStr)
{
  ToUni(uc_To_Unicode_2, ucOutStr, cInStr, InLen);
}

void API_UnicharTo1251(const L_UNICHAR *ucInStr, L_LONG InLen, L_BYTE *cOutStr)
{
  FromUni(cUnicode_To_2, To_OffSet_2, cOutStr, ucInStr, InLen);
}

/*  8.12.99 ... */
void API_KOI8ToUnichar(const L_BYTE *cInStr, L_LONG InLen, L_UNICHAR *ucOutStr)
{
  ToUni(uc_To_Unicode_3, ucOutStr, cInStr, InLen);
}

void API_UnicharToKOI8(const L_UNICHAR *ucInStr, L_LONG InLen, L_BYTE *cOutStr)
{
  FromUni(cUnicode_To_3, To_OffSet_3, cOutStr, ucInStr, InLen);
}
/* ... 8.12.99 */

L_BYTE UnicharTo866(L_UNICHAR UCh)                      /* 16.05.00 */
{
  return cUnicode_To_1[To_OffSet_1[UCh >> 8] | (L_BYTE)UCh];
}

int API_UniTo866(const L_UNICHAR *ucInStr, L_LONG InLen, L_BYTE *cOutStr)
{
  for ( ; InLen > 0; ++ucInStr, --InLen)
  {
    L_ULONG c = cUnicode_To_1[To_OffSet_1[*ucInStr >> 8] | (L_BYTE)*ucInStr];
    *cOutStr++ = (L_BYTE)c;
    if (uc_To_Unicode_1[c] != *ucInStr)
      return 1;
  }
  return 0;
}

#if !defined(LINTER_KERNEL) && !defined(TESTDB)          /* 23.09.13 ... */
static L_LONG ClientUnicharTo1251(L_UNICHAR* In, L_LONG InLen, L_BYTE* Out)
{
  API_UnicharTo1251( In, InLen, Out);
  return InLen/2;
}

static L_LONG ClientUnicharToKOI8(L_UNICHAR* In, L_LONG InLen, L_BYTE* Out)
{
  API_UnicharToKOI8( In, InLen, Out);
  return InLen/2;
}

static L_LONG ClientUnicharTo866(L_UNICHAR* In, L_LONG InLen, L_BYTE* Out)
{
  API_UnicharTo866( In, InLen, Out);
  return InLen/2;
}

static L_LONG ClientUnicharToUtf8(L_UNICHAR* In, L_LONG InLen, L_BYTE* Out)
{
  L_BYTE TmpBuf[MAX_CLIENT_BUFSIZE+1];
  L_LONG OutSize;

  /* For LINNLS_Ucs2Utf8 in len in chars  out size - in bytes */
  LINNLS_Ucs2Utf8(In,  InLen/2, MAX_CLIENT_BUFSIZE, (char *)TmpBuf, &OutSize);
  if (InLen < OutSize) /* Out buf size == InLen. For some codepages unicode can be less then utf8 */
    OutSize = InLen;
  memcpy(Out, TmpBuf, OutSize);

  return OutSize;
}

void ClientUnicode2TargetInvalidate()                      /* 24.01.14 ... */
{
  ConversionFunc = NULL;
}                                                          /* ... 24.01.14 */

L_LONG ClientUnicode2Target(L_BYTE * In, L_LONG InOutLen, L_BYTE * Out)
{
  char *cp = NULL;

  if ( ConversionFunc == NULL)
  {
#if (_VER_MAX < 600 )
#ifndef WINCE                                              /* 12.02.03 */
    cp = getenv("LINTER_CP");
#endif
#else
    LCODEPAGE CodePage;

    memset(&CodePage, 0, sizeof(CodePage));
    inter_control( NULL, ICR_GET_CP, &CodePage, sizeof(CodePage));
    cp = CodePage.cp.s_cp;
#endif
    if ( cp )
    {
      if ( strcmp(cp, ISO_CP) == 0 || strcmp(cp, ISO_CP6) == 0 ) /* koi8 */
        ConversionFunc = ClientUnicharToKOI8;
      else if ( strcmp(cp, ANSI_CP) == 0 || strcmp(cp, ANSI_CP6) == 0 ) /* 1251 */
        ConversionFunc = ClientUnicharTo1251;
      else if ( strcmp(cp, "UTF-8") == 0 )
        ConversionFunc = ClientUnicharToUtf8;
      else  /* 866 */
        ConversionFunc = ClientUnicharTo866; /* Default conversion to 1 byte codepage. For unknown codepage we can see count of characters */
    }
    else
    {
#if (defined (UNIX) && !(defined(__RUSSIAN__LANG_) && defined(_KOI8)))/* 06.07.01 */
      ConversionFunc = ClientUnicharToKOI8;
#else
#if defined(INTER_MSWINDOWS) && !defined(WIN32)
      ConversionFunc = ClientUnicharTo1251;
#endif
      ConversionFunc = ClientUnicharTo866;
#endif
    }
  }
  return ConversionFunc((L_UNICHAR*)In, InOutLen, Out);
}                                                          /* ... 23.09.13 */
#endif /* LINTER_KERNEL */

#endif                                                  /* 22.08.02 */
