 /***************************************************************************
 *           RELEX, Inc. Linter UNICODE library source code                 *
 *  This is unpublished proprietary source code of RELEX, Inc.              *
 *    The copyright notice does not evidence any actual or intended         *
 *                   publication of such source code.                       *
 *                                                                          *
 *                  Copyright (c) 1990-2014 RELEX Inc.                      *
 *                                                                          *
 *                    This software was developed by                        *
 *                                                                          *
 *                             RELEX, Inc.                                  *
 *                          Voronezh, Russia                                *
 *                                                                          *
 *                         All rights reserved.                             *
 ****************************************************************************
 AUTHOR: Pavel Pasechnik (Pasec)
 ----------------------------------------------------------------------------
 PURPOSE: UNICODE support
 ----------------------------------------------------------------------------
 REVISION HISTORY:
 ----------------------------------------------------------------------------

  23.12.96 -- creation date
   8.12.99 -- Pasec (added KOI8 codepage)
  16.05.00 -- ReAn -- UnicharTo866
  24.01.01 -- Pasec (#ifdef  __cplusplus)
  20.07.07 -- FLY (Use Linter types)
  23.09.13 -- Galizin (Utf-8)
  24.01.14 -- KOV (Correction for correct work of ClientUnicode2Target function after CODEPAGE command (INL), TT#24257)
*/

#ifndef __UNICONV_H__
#define __UNICONV_H__

#include "lintypes.h"
#ifndef LINTER_KERNEL
#include "lnlsutf.h"                                     /* 23.09.13 */
#endif

/* 24.01.01 */
#ifdef  __cplusplus
extern "C" {
#endif

void  API_866ToUnichar( const L_BYTE    * cInStr,       /* input string          */
                              L_LONG      InLen,        /* length in characters  */
                              L_UNICHAR * ucOutStr);    /* output unicode string */

void  API_UnicharTo866( const L_UNICHAR * ucInStr,      /* input unicode string  */
                              L_LONG      InLen,        /* length in characters  */
                              L_BYTE    * cOutStr);     /* output string         */

void  API_1251ToUnichar(const L_BYTE    * cInStr,       /* input string          */
                              L_LONG      InLen,        /* length in characters  */
                              L_UNICHAR * ucOutStr);    /* output unicode string */

void  API_UnicharTo1251(const L_UNICHAR * ucInStr,      /* input unicode string  */
                              L_LONG      InLen,        /* length in characters  */
                              L_BYTE    * cOutStr);     /* output string         */

void  API_KOI8ToUnichar(const L_BYTE    * cInStr,       /* input string          */
                              L_LONG      InLen,        /* length in characters  */
                              L_UNICHAR * ucOutStr);    /* output unicode string */

void  API_UnicharToKOI8(const L_UNICHAR * ucInStr,      /* input unicode string  */
                              L_LONG      InLen,        /* length in characters  */
                              L_BYTE    * cOutStr);     /* output string         */

int   API_UniTo866(     const L_UNICHAR * ucInStr,
                              L_LONG      InLen,
                              L_BYTE    * cOutStr);

L_BYTE UnicharTo866(L_UNICHAR UCh);                     /* 16.05.00 */

/* Converting string from unicode to intlib charset */
L_LONG ClientUnicode2Target(L_BYTE * In,                /* Input array */   /* 23.09.13 ... */
                            L_LONG InOutLen,            /* In-out Array length */
                            L_BYTE * Out);              /* Output null terminated array. Max length the same as In */ /* ... 23.09.13 */
/* Need to invalidate the conversion function for case change of codepage: */
void ClientUnicode2TargetInvalidate();                     /* 24.01.14 */

#ifdef LOADMAP
 /* defined code pages */
#define LCP_866         1
#define LCP_1251        2
#define LCP_KOI8        3

L_UNICHAR * CharToUnichar(L_BYTE  * cInStr,
                        L_LONG      InLen,
                        L_UNICHAR * ucOutStr,
                        L_WORD      codepage);          /* code page */

L_BYTE  * UnicharToChar(L_UNICHAR * ucInStr,
                        L_LONG      InLen,
                        L_BYTE    * cOutStr,
                        L_WORD      codepage);          /* code page */
#endif

#ifdef __cplusplus
}
#endif

#endif
