/****************************************************************************
 *          RELEX, Inc. Linter interface library source code                *
 *              This is public source code of RELEX, Inc.                   *
 *                                                                          *
 *       RELEX has no warranty obligations or liability for any             *
 *      usage of interface library source code which is modified.           *
 *                                                                          *
 *                 Copyright (c) 2000-2013 RELEX Inc.                       *
 *                                                                          *
 *                   This software was developed by                         *
 *                                                                          *
 *                            RELEX, Inc.                                   *
 *                         Voronezh, Russia                                 *
 *                                                                          *
 *                        All rights reserved.                              *
 ****************************************************************************
 AUTHOR: Gluhov
 ----------------------------------------------------------------------------
 PURPOSE:
 ----------------------------------------------------------------------------
 REVISION HISTORY:
 28.11.05 -- Gluhov ( file created )     
 10.01.06 -- Gluhov ( support for prepared queries ) 
 19.04.06 -- Gluhov ( Added new linapi calls - blobs, Correct names )
 06.05.06 -- Galizin ( Add stdlib.h for calloc, free calls )
 17.05.06 -- Gluhov ( fixed crush with var types)
 26.01.09 -- Ermakov (fix some bugs related to BLOB processing
                      - Parameter count with AddBlob
                      - Use extended finction insted sinple (5.7)
                      - Added CBLB before ABLB in update operation
                      - Correct paraneter type with BLOB operaion (Input instead of Output)
                      - Correct return value in some error case)
 29.07.09 -- Muratova (fetchNext, fetchPrevious)
 20.09.09 -- Muratova (add LINTER_ConnectCS)
 15.10.09 -- Muratova (editing CharSet processing) 
 13.11.09 -- Muratova (extend getError by adding params)
 23.12.09 -- Muratova (correct getError return value)
 24.03.10 -- Muratova (correct buflen for Nchar)
 22.07.10 -- DV       (use er_type always - for old versions)
 28.07.10 -- DV       (encoding conversion for QT 4.x)
 17.08.10 -- Andreeva (fix bug insert BLOB data
                      - move BLOB data clearing out of cycle)
 27.09.10 -- Muratova (escapeIdentifier)
 27.10.10 -- Ermakov (LINTER_SYSTEM_USER for dict queries)
 27.06.11 -- Muratova (add isIdentifierEscaped)
 17.08.11 -- DV       (remove 'l' from formats)
 29.11.11 -- Muratova (add Qt3Quotes;
                       open with 5 params for qt < 4)
 22.02.12 -- Muratova (add inl style working with blob)
 01.03.12 -- Muratova (add ACCESS_BLOBFILE for work blob and files)
 11.03.12 -- Muratova (add LINTER_NO_DATA for fix error 2 in getting data)
 08.06.12 -- Glukhov (MTINDEX + datatypes)
 02.08.12 -- Galizin  (IdentifierType for 4.2.1 and 3.3 qt )
 10.08.12 -- DV (Remove unused QLinterDriver::NCurs field)
 18.09.12 -- DV (call = execute)
 24.09.12 -- DV (Increase max column length by 1)
 24.09.12a - DV (Set field length)
 09.10.12 -- Muratova (Warning added)
 22.02.13 -- DV (Warnings)
 04.04.13 -- Ermakov (BLOB column number)
 28.06.13 -- DV,Ermakov (Remove non-const call from const method)
 07.08.13 -- Galizin (Read blob update)
 07.08.13a-- Galizin (Unused argument warning)
 07.08.13b-- Galizin (Linapi error print if no linter error)
 07.08.13c-- Galizin (InOut procedure parameters qt3)
 07.08.13d-- Galizin (Try to find field without and whith ")
 07.08.13f-- Galizin (Transaction modes for qt >= 3.2)
 09.08.13 -- Galizin (Default bind values)
 09.08.13a-- Galizin (Saving rowset on blob pseudoqueries)
 09.08.13b-- Galizin (Speed optimisation)
 12.08.13 -- Galizin (Using char and hex values for blob append)
 13.08.13 -- Galizin (Prepared queries is supported fully from qt 3.2)
 13.08.13a-- Galizin (Changing QByteArray length by real blob length)
 14.08.13 -- Galizin (Changing TRUE to true and FALSE to false)
 04.03.14 -- Galizin (Initialization)
 ----------------------------------------------------------------------------
*/

#include <stdlib.h>                                         /* 06.05.06 */
#include <qglobal.h>
#include <qsqldriver.h>
#include <qsqlresult.h>
#include <qsqlerror.h>
#include <qdatetime.h>
#include <qsqlrecord.h>
#include <qsqlfield.h>
#include <qsqlindex.h>
#include <qsqlquery.h>
#include <qstringlist.h>

#if QT_VERSION < 300
#"This driver can be used only with QT 3.0 or higher" 
#endif

#if QT_VERSION >= 0x030200                                  /* 13.08.13 */
#  define ENABLE_PREPARED_EXTENSION
#endif

#ifdef ACCESS_BLOBFILE
#  define ENABLE_ACCESS_BLOBFILE
#endif

#if QT_VERSION < 0x040000                                    /* 27.06.11 ... */
#  define QSQL_DRIVER_IDENTIFIER_TYPE int                        /* 02.08.12 */
#  define FieldName 0
#  define TableName 1
#else
#  define QSQL_DRIVER_IDENTIFIER_TYPE QSqlDriver::IdentifierType /* 02.08.12 */
#endif                                                       /* ... 27.06.11 */

#ifdef ENABLE_PREPARED_EXTENSION
#  if QT_VERSION < 0x040000
#    include <private/qsqlextension_p.h>
#    include <private/qinternal_p.h>
#    include <qfile.h>                                       /* 22.02.12 */
#  else
#    include <QHash>
#    include <QFile>                                         /* 22.02.12 */
#  endif
#endif

#include "lintypes.h"
#include "int64.h"
#include "lnlsutf.h"
#include "linapi.h"
#include "decimals.h"
#include "tick.h"

#include "qsql_linter.h"

#ifdef LINQT_DEBUG
int donothing( char* format, ...) { return 0; };
#include "log.h"
#else
   int donothing( const char*, ...) { return 0; };
#  define LOG_INIT(LevelString, FileName)
#  define LOG_SHUT
#  define TRS_ENTER
#  define TRS_RETURN
#  define TRS_MSG   donothing(
#  define TRS_ERROR donothing(
#  define TRS_END )
#endif    

#if QT_VERSION < 0x040000                                    /* 22.02.12 ... */
  QString::SectionFlags scis = QString::SectionCaseInsensitiveSeps;
  const bool ci = L_TFALSE;
#else
  QString::SectionFlag scis = QString::SectionCaseInsensitiveSeps;
  const Qt::CaseSensitivity ci = Qt::CaseInsensitive;              /* ... 22.02.12 */
#endif

#if QT_VERSION < 0x040000
#  define QS2C(qstr) qstr.local8Bit().data()
#  define QBYTEARRAY(str, len) QByteArray().duplicate( str, len ) 
#  define UPPER(qstr) qstr.upper()
#  define LOWER(qstr) qstr.lower()
#  define QS2QB(qstr) qstr.local8Bit()                       /* 22.02.12 */
class MyByteArray:public QByteArray                        /* 13.08.13a ... */
{
public:
  MyByteArray():QByteArray() {};
  ~MyByteArray() {};
  inline void DecreaseLenUpTo(int len)
  {
    int *Size = (int*)(&sharedBlock()->len);
    if ( len < *Size )
      *Size = len;
  }
};
#else
class MyByteArray:public QByteArray
{
public:
  MyByteArray():QByteArray() {};
  ~MyByteArray() {};
  inline void DecreaseLenUpTo(int len)
  {
    int *Size = &(data_ptr()->size);
    if ( len < *Size )
      *Size = len;
  }
};                                                         /* ... 13.08.13a */
#  define QS2C(qstr) qstr.toLocal8Bit().data()
#  define QBYTEARRAY(str, len) QByteArray( str, len ) 
#  define UPPER(qstr) qstr.toUpper()
#  define LOWER(qstr) qstr.toLower()
#  define Q_LLONG qlonglong
#  define QS2QB(qstr) qstr.toLocal8Bit()                     /* 22.02.12 */
#endif

#if QT_VERSION < 0x040000
#  define er_type QSqlError::Statement
#else
#  define er_type QSqlError::StatementError
#endif

#define DEFAULT_BLOB_PORTION_SIZE 50000

static const char * date_format = "yyyyMMddhhmiss";

#if QT_VERSION < 0x040600                                    /* 27.06.11 ... */
static bool isIdentifierEscaped(const QString &identifier, 
                                QSQL_DRIVER_IDENTIFIER_TYPE type)            /* 02.08.12 */
{
  bool result;

  Q_UNUSED(type);                                          /* 07.08.13a */
  result = false;
  if(!identifier.isEmpty() && identifier.startsWith( QChar('"') ) && identifier.endsWith( QChar('"') ) )   
  {
    result = true;
  }
  
  return result;
}
#endif                                                       /* ... 27.06.11 */

static char *strrtrim(char *str, char *chs)
{
  int i;

  i = strlen(str) - 1;
  while (strrchr(chs, str[i]) && i >= 0)
    str[i--] = 0;
  return (str);
}

static char *strltrim(char *str, char *chs)
{
  while (strrchr(chs, str[0]) && *str)
    strcpy(str, str + 1);
  return (str);
}

QVariant::Type qDecodeLinterType( L_BYTE lintype )
{
  QVariant::Type type;
  switch ( lintype ) 
  {
    case tChar:
    case tVarChar:
#if QT_VERSION < 0x040000
      type = QVariant::CString;
#else
      type = QVariant::String;                          /* 28.07.10 */                       
#endif
      break;
    case tNChar:
    case tNVarChar:
      type = QVariant::String;
      break;
    case tDecimal:
    case tReal:
    case tDouble:
      type = QVariant::Double;
      break;
    case tDate:
      type = QVariant::DateTime;
      break;
    case tByte:
    case tVarByte:
      type = QVariant::ByteArray;
      break;
    case tSmallInt:
    case tInt:
      type = QVariant::Int;
      break;
    case tBigInt:
#  ifdef Q_LLONG
      type = QVariant::LongLong;
#  else
      type = QVariant::Double;
#  endif
      break;
    case tBlob:
      type = QVariant::ByteArray;
      break;
    case tBoolean:
      type = QVariant::Bool;
      break;
    default:
      type = QVariant::Invalid;
  }
  return type;
}

QString ByteArray2Hex(QByteArray ba)
{
  QString res;
  static const char hexchars[] = "0123456789abcdef";
   
  for ( int i = 0; i < (int)ba.size(); ++i ) 
  {
    uchar s = (uchar) ba[(int)i];
    res += hexchars[s >> 4];
    res += hexchars[s & 0x0f];
  }
  return res;
}


QByteArray String2Ucs2(QString s)
{
  L_LONG tmplen;
  L_UNICHAR * tmpstr;
#if QT_VERSION < 0x040000
  QCString real_data = s.utf8();
#else
  QByteArray real_data = s.toUtf8();
#endif
  L_LONG real_len = real_data.length();

  if (!(tmpstr = (L_UNICHAR *)calloc(1, real_len * 4)))
  {
    TRS_ERROR "Not enough memory" TRS_END;
    return QByteArray();
  }
  
  LINNLS_Utf8Ucs2( real_data.data(),
                   real_len,
                   L_LONG(real_len * 4),
                   tmpstr,
                   &tmplen );
  
  QByteArray ba = QBYTEARRAY( (char*)tmpstr, tmplen );

  TRS_MSG "String '%s' successfully converted to ucs2", QS2C(s) TRS_END;
  
  free(tmpstr);
  return ba;
}

QVariant toVariant ( void* buf, L_LONG buflen, L_LONG lin_type, int prec, int scale )
{
  QVariant result;

  TRS_ENTER;

  switch ( lin_type )
  {
    case tChar:
    {
      result = QVariant( QString::fromLocal8Bit( (char*)buf, buflen ) );
      break;
    }  

    case tVarChar:
    {
      char * real_data = (char *)buf + sizeof(L_WORD);
#if !defined(LONG_ALIGN)
      L_WORD real_len = *(L_WORD *)buf;
#else
      L_WORD real_len;
      memcpy(&real_len, buf, sizeof(L_WORD));
#endif
      result = QVariant( QString::fromLocal8Bit( real_data, real_len ) );
      break;
    }
    case tByte:
    {
      QByteArray ba = QBYTEARRAY( (char *)buf, buflen );
      result = QVariant( ba );
      break;
    }

    case tVarByte:
    {
      char * real_data = (char *)buf + sizeof(L_WORD);
#if !defined(LONG_ALIGN)
      L_WORD real_len = *(L_WORD *)buf;
#else
      L_WORD real_len;
      memcpy(&real_len, buf, sizeof(L_WORD));
#endif 
      QByteArray ba = QBYTEARRAY( real_data, real_len );
      result = QVariant( ba );
      break;
    }

    case tNChar:
    {
      L_LONG tmplen;
      char * tmpstr; 

      if (!(tmpstr = (char *)calloc(1, buflen * 4)))
      {
        TRS_ERROR "Not enough memory" TRS_END;
        result = QVariant();
        break;
      }

      LINNLS_Ucs2Utf8( (L_UNICHAR *)buf,
                       (L_LONG)(buflen / 2),              /* 24.03.10 */
                       L_LONG(buflen * 4),
                       tmpstr,
                       &tmplen );
      result = QVariant( QString::fromUtf8( tmpstr, tmplen ) );
      free(tmpstr);      
      break;
    }

    case tNVarChar:
    {
      char * real_data = (char *)buf + sizeof(L_WORD);
      L_LONG tmplen;
      char * tmpstr; 
#if !defined(LONG_ALIGN)
      L_WORD real_len = *(L_WORD *)buf;
#else 
      L_WORD real_len;
      memcpy(&real_len, buf, sizeof(L_WORD));
#endif

      if (!(tmpstr = (char *)calloc(1, real_len * 4)))
      {
        TRS_ERROR "Not enough memory" TRS_END;
        result = QVariant();
        break;
      }
      LINNLS_Ucs2Utf8( (L_UNICHAR *)real_data,
                       (L_LONG)(real_len),
                       (L_LONG) real_len * 4,
                       tmpstr,
                       &tmplen );
      result = QVariant( QString::fromUtf8( tmpstr, tmplen ) );
      free(tmpstr);      
      break;
    }

    case tBigInt:
    {
#ifdef Q_LLONG
#  if !defined(LONG_ALIGN)
      result = QVariant( *(Q_LLONG *)buf );
#  else                                                       
      L_DLONG Val;
      memcpy(&Val, buf, sizeof(L_DLONG));
      result = QVariant( *(Q_LLONG *)&Val );
#  endif                                                      
#else
      double dVal;
#  if !defined(LONG_ALIGN)
      i64_ToDouble(*(L_DLONG *)buf, dVal);
#  else                                                       
      L_DLONG Val;
      memcpy(&Val, buf, sizeof(L_DLONG));
      i64_ToDouble(Val, dVal);
#  endif                                                      
      result = QVariant(dVal);
#endif
      break;
    }

    case tInt:
    {
#if !defined(LONG_ALIGN)
      result = QVariant((int)(*(L_LONG *)buf));
#else
      L_LONG Val;
      memcpy( &Val, buf, sizeof(L_LONG));
      result = QVariant((int)Val);
#endif
      break;
    }

    case tBoolean:
    {
      result = QVariant((bool)(*(char *)buf));
      break;
    }

    case tSmallInt:
    {
#if !defined(LONG_ALIGN)
      result = QVariant( (int)(*(L_SWORD *)buf) );
#else
      L_SWORD Val;
      memcpy( &Val, buf, sizeof(L_SWORD) );
      result = QVariant( (int)Val );
#endif
      break;
    }
      
    case tReal:
    {
#if !defined(LONG_ALIGN)
      result = QVariant(*((L_REAL *)buf));
#else
      L_REAL Val;
      memcpy(&Val, buf, sizeof(L_REAL));
      result = QVariant(Val);
#endif
      break;
    }

    case tDouble:
    {
#if !defined(LONG_ALIGN)
      result = QVariant(*((L_DOUBLE *)buf));
#else
      L_DOUBLE Val;
      memcpy(&Val, buf, sizeof(L_DOUBLE));
      result = QVariant(Val);
#endif
      break;
    }

    case tDecimal:
    {
      char dec_val[DECLNG*2+1];

      DECTOSTR(*(L_DECIMAL*)buf, dec_val, prec, scale, L_TTRUE); /* 14.08.13 */
      strltrim(dec_val,(char *)" ");
      strrtrim(dec_val,(char *)" ");
      result = QVariant( QString::fromLatin1( dec_val ) );
      break;                                                                                                    
    }
                                                                                    
    case tDate:                                                                                              
    {
      char date_val[32];
     
      TICKTOSTRF(*(L_DECIMAL*)buf, date_format, date_val);
      QString val = QString::fromLatin1( date_val, strlen(date_format) );
      val.insert(4, "-").insert(7, "-").insert(10, 'T').insert(13, ':').insert(16, ':');
      result = QVariant( QDateTime::fromString( val, Qt::ISODate ) );
      break;
    }
    default:
    {
      TRS_MSG "data type %i is unknown, size is %i", (int)lin_type, (int)buflen TRS_END;
    }  
  }
  
  TRS_RETURN;
  return result;
}

static QSqlError getError ( L_WORD con_id,                   /* 13.11.09 ... */
                            L_WORD cur_id,
                            L_WORD stmt_id,
                            QString text,
#if QT_VERSION < 0x040000                            
                            QSqlError::Type type,
#else
                            QSqlError::ErrorType type,
#endif                            
                            L_LONG ret_cod )                 /* ... 13.11.09 */                       
{
  L_LONG lRet;
  L_LONG apierr = 0, err_cod = 0, syserr = 0;
  QSqlError res;
  QString err_text;
  
  Q_UNUSED(type);                                          /* 07.08.13a */
  if (ret_cod == LINAPI_ERROR)
  {
    if ((lRet = LINTER_Error(con_id, cur_id, stmt_id,
                             &apierr, &err_cod, &syserr, NULL, NULL)))
    {
      err_text = QString().sprintf( (char *)"Unknown error. LinApi return code is %d", (int)ret_cod );
    }
    else
    {
      err_text = QString().sprintf( (char *)"Linter error %d, ", (int)err_cod );
      if ((apierr == eLinterError) && err_cod > 2000 && err_cod < 3000)
      {
        /* getting string and position if syntax error is */
        err_text = err_text + QString().sprintf((char *)"syntax error in line %d, position %d",
               (short) syserr, *(short *) ((char *) &syserr + 2));
      }
      else if (apierr != eLinterError)                     /* 07.08.13b */
        err_text = err_text + QString().sprintf( (char *)"LinApiErr = %d", (int)apierr ); /* 07.08.13b */
      else
        err_text = err_text + QString().sprintf( (char *)"SysErr = %d", (int)syserr );
    }
  }
  else
  {
    err_text = QString().sprintf( (char *)"Unknown error. LinApi return code is %d", (int)ret_cod );
    err_cod = ret_cod;                                           /* 13.04.10 */
  }

  TRS_ERROR "%s", QS2C(err_text) TRS_END;
                                  
  
#if QT_VERSION < 0x040000
  return QSqlError( text + err_text, QString(), QSqlError::None, err_cod );
#else
  return QSqlError( text + err_text, text, type, err_cod );      /* 23.12.09 */ /* 13.04.10 */
#endif
}


#ifdef ENABLE_PREPARED_EXTENSION
#  if QT_VERSION < 0x040000
class QLinterPreparedExtension : public QSqlExtension
{
public:
  QLinterPreparedExtension( QLinterResult * r )
    : result( r ) {}

  bool prepare( const QString& query )
  {
    return result->prepare( query );
  }

  bool exec()
  {
    return result->exec();
  }
  
  QLinterResult * result;
};
#  endif
#endif /* ENABLE_PREPARED_EXTENSION */


QLinterResult::QLinterResult( const QSqlDriver* db )
    : QSqlResult( db )
{
  L_LONG lRet;

  NConn = ((QLinterDriver *)db)->NConn;
  NCurs = 0;
  NMainCurs = 0;
  NStatement = 0;
  LastDataField = -1;
  LastDataLength = -1;
  ColCount = 0;                                             /* 04.03.14 */

  L_LONG lOptions = ((QLinterDriver *)driver())->Options;
  if ( ( lRet = LINTER_OpenCursor(NConn, &NCurs, NULL, 0, lOptions) ) )
  {
    setLastError( getError( NConn, 0, 0 , QString("QLinterResult::QLinterResult() : "), er_type, lRet ) ); /* 13.11.09 */
    TRS_ERROR "Error in LINTER_OpenCursor." TRS_END;
  }
#ifdef ENABLE_PREPARED_EXTENSION   
#  if QT_VERSION < 0x040000
  setExtension( new QLinterPreparedExtension( this ) );
#  endif
#endif

}

QLinterResult::~QLinterResult()
{
  if ( NStatement )
    LINTER_FreeStatement( NStatement );
  if ( NCurs ) 
    LINTER_CloseCursor( NCurs );
  if ( NMainCurs )
    LINTER_CloseCursor( NMainCurs );
}

#if QT_VERSION >= 0x040000
Extension* QLinterResult::extension()
{
  return &ext;
}
#endif


bool QLinterResult::prepare( const QString& query )
{
  L_LONG lRet;

  TRS_ENTER;

  TRS_MSG "Preparing query: %s", QS2C(query) TRS_END;

  if ( query.startsWith("AddBlob") ||                      /* 09.08.13a ... */
       query.startsWith("FetchBlob") ||
       query.startsWith("PurgeBlob") ||
       query.startsWith("blob") )
  { /* pseudoquery. Do noting*/
    TRS_MSG "Special command detected" TRS_END;
    TRS_RETURN;
    return true;
  }                                                        /* ... 09.08.13a */

  if ( NStatement )
    LINTER_FreeStatement( NStatement );
 
  if ( NMainCurs )
  {
    LINTER_CloseCursor( NCurs );
    NCurs = NMainCurs;
    NMainCurs = 0;
  }
  
  setActive( L_TFALSE );
#if QT_VERSION < 0x040000
  setAt( QSql::BeforeFirst );
#else
  setAt( QSql::BeforeFirstRow );
#endif

  if ( ( lRet = LINTER_CreateStatement(NConn, QS2C(query), 0, &NStatement) ) )
  {
    setLastError( getError( NConn, 0, 0 , QString("QLinterResult::prepare() : "), er_type, lRet ) );/* 13.11.09 */
    TRS_ERROR "Error in LINTER_CreateStatement." TRS_END;
    TRS_RETURN;
    return false;
  }
  TRS_MSG "NStatement = %i", NStatement TRS_END;

  TRS_RETURN;
  return true;
}


bool QLinterResult::exec()
{
  L_LONG lRet;

  TRS_ENTER;
  
  QMap<L_SWORD,char> NullIndicators;
  QMap<L_SWORD,L_LONG> ParameterLengths;
  QMap<L_SWORD,void *> BindedValues;

  t_ParamDesc ParamDesc; 

#ifdef ENABLE_PREPARED_EXTENSION   
#if QT_VERSION >= 0x040000
  /* emulate extension() structures from QT3 */
  ext.index.clear();
  ext.values.clear();
  for ( int i=0; i < boundValueCount(); i++ )
  {
    QString s = QString().setNum(i);  
    ext.index.insert( i, s );
    Param p; p.value = boundValues()[i];
    ext.values.insert( s, p );
    TRS_MSG "values[%s] = %s", QS2C(s), QS2C(p.value.toString()) TRS_END;
  }  
#endif


  if ( lastQuery().startsWith("AddBlob") || lastQuery().startsWith("FetchBlob") || 
       lastQuery().startsWith("PurgeBlob") || lastQuery().startsWith("blob") ) /* 22.02.12 ... */
  {
    QString main_query = lastQuery();                      /* 12.08.13 ... */
    L_LONG coln = 0;
    bool StringData = false;
    QString textData;
    
    if ( main_query.contains('\'') )
    {
      StringData = true;
      textData = main_query.section('\'', 1, 1);
      main_query = main_query.section('\'', 0, 0);
    }                                                      /* ... 12.08.13 */

#if QT_VERSION < 0x040000
    QStringList list = QStringList::split( ' ', main_query );
#else
    QStringList list = main_query.split( ' ' );
#endif

    if ( main_query.contains("rowid=", ci) > 0 )
    {
      int rowid = 0;
      rowid = main_query.section("rowid=", 1, 1, scis).section(" ", 0, 0).toInt();
      
      if ( list.size() > 3 )
      {
        QString nameString = "";
        
#if QT_VERSION < 0x040000
        nameString = list[3];
        QStringList nameList = QStringList::split( '.', nameString );
#else
        nameString = list.at(3);
        QStringList nameList = QString(nameString).split('.');
#endif
        coln = 0;

        switch ( nameList.size() )
        {
          case 2:
          {
#if QT_VERSION < 0x040000
            QString tableName = nameList[0];
            QString colName = nameList[1];
#else            
            QString tableName = nameList.at(0);
            QString colName = nameList.at(1);
#endif	    
            QString stmt = "select %1 from %2 where rowid= %3";                   
            setQuery( stmt.arg(colName).arg(tableName).arg(rowid) );
            reset( stmt.arg(colName).arg(tableName).arg(rowid) );     
            break;
          }
          case 3:
          {
#if QT_VERSION < 0x040000
            QString userName = nameList[0];
            QString tableName = nameList[1];
            QString colName = nameList[2];
#else
            QString userName = nameList.at(0);
            QString tableName = nameList.at(1);
            QString colName = nameList.at(2);
#endif	    
            QString stmt = "select %1 from %2.%3 where rowid= %4";                     
            setQuery( stmt.arg(colName).arg(userName).arg(tableName).arg(rowid) );
            reset( stmt.arg(colName).arg(userName).arg(tableName).arg(rowid) );            
            break;
          }
          default:
          {
            setLastError( QSqlError( QString( "QLinterResult::exec() : wrong blob column name for blob command" ) ) );
            TRS_ERROR "QLinterResult::exec()" TRS_END;
            TRS_RETURN;
            return false;
          }
        }                   
      }  
      else
      {	
        setLastError( QSqlError( QString( "QLinterResult::exec() : wrong param count for blob command" ) ) );
        TRS_ERROR "QLinterResult::exec()" TRS_END;
        TRS_RETURN;
        return false;
      }            
    }
        
    QByteArray ba;         
    QFile file;

    if ( main_query.contains("column=", ci) > 0 )
    {
      coln = main_query.section("column=", 1, 1, scis).section(" ", 0, 0).toInt();
    }
    
    L_LONG blobsize = 0;
    L_LONG blobtype = 0;
    
    if ( main_query.contains("insert", ci) > 0 || main_query.contains("append", ci) > 0 || 
         main_query.contains("get", ci) > 0 )
    {
      if ( main_query.contains("file=", ci) > 0 )
      {
#ifdef ENABLE_ACCESS_BLOBFILE
        QString blobFileName = "";
        blobFileName = QS2C(main_query.section("file=", 1, 1, scis));
#if QT_VERSION < 0x040000
        file.setName(blobFileName);
#else
        file.setFileName(blobFileName);
#endif
        if ( main_query.contains("insert", ci) > 0 || main_query.contains("append", ci) > 0 )
        {
#if QT_VERSION < 0x040000
          file.open(IO_ReadOnly);
#else
          file.open(QIODevice::ReadOnly);
#endif
          ba = file.readAll();
          blobsize = ba.size();
          file.close();
        }
        else
        {
#if QT_VERSION < 0x040000
          file.open(IO_WriteOnly);
#else
          file.open(QIODevice::WriteOnly);
#endif
        }
#else /* ! ENABLE_ACCESS_BLOBFILE */
        qWarning( "ACCESS_BLOBFILE mode is disabled\nWorking with option 'file=' is not available" ); /* 09.10.12 */
#endif
      }
      else if ( main_query.contains("insert", ci) > 0 || main_query.contains("append", ci) > 0 )
      {
        int lastItem = list.size() - 1;                     /* 12.08.13 ... */

        if (StringData) /* it has string data */
        {
          ba = QS2QB(textData);
#if QT_VERSION < 0x040000
          blobsize = ba.size()-1; /* size - it is string length + 1 zero char */
#else
          blobsize = ba.size();
#endif
        }
        else if ( lastItem == 3 || lastItem == 4 ) /* List size is 5 or 4 - last field has 16 data */
        {
          QString dataString;

#if QT_VERSION < 0x040000
          dataString = list[lastItem];
#else
          dataString = list.at(lastItem);
#endif
          /* hex value parsing */
          ba = QByteArray();
          QString num;
          int i;

          for (i = 0; i < (int)dataString.length(); i++ )
          {
            num.append(dataString[i]);
            if ( (i & 0x1) == 0 )
              continue;
            ba.resize(i/2 + 1);
            ba[i/2] = (char)num.toUShort(NULL, 16);
            num = "";
          }
          blobsize = ba.size();
        }                                                   /* ... 12.08.13 */
      }
    }

    TRS_MSG "Executing special command with blob" TRS_END;

    if ( main_query.contains("insert", ci) > 0  || main_query.contains("append", ci) > 0  || 
         main_query.startsWith("AddBlob") )
    {
      const char* ColumnIndex = "0";                       /* 09.08.13 ... */
      const char* TypeIndex = "1";
      const char* ByteArrayIndex = "2";
      const char* LengthIndex = "3";

      if ( main_query.startsWith("blob") )
      {
        ByteArrayIndex = "0";
        LengthIndex = "1";
        TypeIndex = "2";
        ColumnIndex = "3";
      }

      QVariant& Variant = extension()->values[ColumnIndex].value;
      if (Variant.type() != QVariant::Invalid )
        coln = Variant.toInt(); /* column number */

      Variant = extension()->values[TypeIndex].value;
      if (Variant.type() != QVariant::Invalid )
        blobtype = Variant.toInt(); /* blobtype */

      Variant = extension()->values[ByteArrayIndex].value;
      if (Variant.type() != QVariant::Invalid )
      {
        ba = Variant.toByteArray(); /* Data */
        blobsize = ba.size();
      }

      Variant = extension()->values[LengthIndex].value;
      if (Variant.type() != QVariant::Invalid )
        blobsize = Variant.toInt(); /* size of data */

      if ( blobsize < 0 )
        blobsize = 0;
      if (blobsize > (int)ba.size())
        blobsize = ba.size();                              /* ... 09.08.13a */

      if ( main_query.contains("insert", ci) > 0 )
      {
        if ( ( lRet = LINTER_PurgeBlob( NCurs, coln, NULL, NULL ) ) )
        {
          setLastError( getError( 0, NCurs, 0 , QString( "QLinterResult::exec() : " ), er_type, lRet ) );
          TRS_ERROR "Error in LINTER_PurgeBlob" TRS_END;
          TRS_RETURN;
          return false;
        }
      }

      for ( L_LONG offset = 0; offset < blobsize; offset += DEFAULT_BLOB_PORTION_SIZE )
      {
        L_LONG len = ( (blobsize - offset) < DEFAULT_BLOB_PORTION_SIZE ) ? blobsize - offset : DEFAULT_BLOB_PORTION_SIZE;

        if ((lRet = LINTER_AddBlob2( NCurs, coln, blobtype, ba.data() + offset, len, NULL, NULL, NULL)))
        {
          setLastError( getError( 0, NCurs, 0, QString( "QLinterResult::exec() : " ), er_type, lRet ) ); 
          TRS_ERROR "Error in LINTER_AddBlob" TRS_END;
          TRS_RETURN;
          return false;
        }
      }
    } /* AddBlob, insert, append */
    else                                                   /* 07.08.13 ... */
    if ( main_query.contains("get", ci) > 0 || main_query.startsWith("FetchBlob") )
    {
      L_LONG BlobCommandFlag = main_query.startsWith("blob");

      char * buf;
      L_LONG BlobOffset = 0;
      L_LONG ReqDataSize = 0;
      const char* ByteArrayIndex;
      const char* SizeIndex;
      const char* ColumnIndex;
      const char* OffsetIndex;

      if ( BlobCommandFlag ) /* blob get */
      {
        ByteArrayIndex = "0"; /* output QByteArray */
        SizeIndex = "1"; /* Output buffer size */
        OffsetIndex = "2"; /* Blob start downloading position (from 0) */
        ColumnIndex = "3"; /* Column number */
      }
      else /* FetchBlob */
      {
        ColumnIndex = "0"; /* Column number */
        OffsetIndex = "1"; /* Blob start downloading position (from 0) */
        ByteArrayIndex = "2"; /* output QByteArray */
        SizeIndex = "3"; /* Data size to save */
      }
      
      QVariant& Variant = extension()->values[ColumnIndex].value;
      if ( Variant.type() != QVariant::Invalid )
        coln = Variant.toInt();
      Variant = extension()->values[OffsetIndex].value;
      if ( Variant.type() != QVariant::Invalid )
        BlobOffset = Variant.toInt();
      Variant = extension()->values[SizeIndex].value;
      if ( Variant.type() != QVariant::Invalid )
        ReqDataSize = Variant.toInt(); /* Buffer size */

#if !defined(ENABLE_PREPARED_EXTENSION)
      if ( !file.isOpen ) /* boundValue is not support - only file posible */
      {
        setLastError( QSqlError( QString( "This QT version is not support boundValue" ) ) );
        TRS_ERROR "Not support boundValue" TRS_END;
        TRS_RETURN;
        return false;
      }
#endif

      L_BYTE NeedResize = 0;

      Variant = extension()->values[ByteArrayIndex].value;
      if ( Variant.type() == QVariant::ByteArray)
      {
        QByteArray InArray = Variant.toByteArray();
#if QT_VERSION < 0x040000
        int Size = InArray.size();
#else
        int Size = InArray.length();
#endif
        ba = InArray;
        if (ReqDataSize == 0 || /* Data size is not set */ 
            Size < ReqDataSize ) /* QByteArray is to short */
          ReqDataSize = Size; /* Max len correction */
      }
      else /* ByteArray is not bound */
      {
        if ( ReqDataSize == 0 ) /* Size is not set */
          ReqDataSize = DEFAULT_BLOB_PORTION_SIZE - 1;
        ba.resize(ReqDataSize); /* allocate new space */
        NeedResize = 1;
      }

      L_LONG DataLen = 0;

      while (1)
      {
        L_LONG PortionLen = ReqDataSize;

#if QT_VERSION >= 0x040000
        buf = (char*)ba.constData(); /* special trick for exclude reallocate QByteArray data !!! */
#else
        buf = (char*)((const char*)ba); /* special trick for exclude reallocate QByteArray data !!! */
#endif

        if ( (ReqDataSize - DataLen) > DEFAULT_BLOB_PORTION_SIZE )
          PortionLen = DEFAULT_BLOB_PORTION_SIZE;
        else
          PortionLen = ReqDataSize - DataLen;
        L_LONG len = PortionLen;

        if ((lRet = LINTER_FetchBlob( NCurs, coln, BlobOffset + 1, &len,
                                                buf + DataLen , NULL, NULL )))
        {
          setLastError( getError( 0, NCurs, 0, QString( "QLinterResult::exec() : " ), er_type, lRet ) );
          TRS_ERROR "Error in LINTER_FetchBlob" TRS_END;
          TRS_RETURN;
          return false;
        }
        BlobOffset += len;
        DataLen += len;
        if ( ( (len < PortionLen) || /* No data in the blob */
               (DataLen == ReqDataSize) ) && /* All QByteArray is fill */
             (PortionLen != DEFAULT_BLOB_PORTION_SIZE) ) /* all data read */
          break;

        if ( NeedResize )
        {
          ReqDataSize += DEFAULT_BLOB_PORTION_SIZE;
          ba.resize(ReqDataSize);
          continue;
        }
      }

      if (DataLen < ReqDataSize)
        ((MyByteArray*)&ba)->DecreaseLenUpTo(DataLen);     /* 13.08.13a */
      memset(buf + DataLen, 0, ReqDataSize - DataLen); /* zero all unused tail */

      if ( file.isOpen() )
      {
#ifdef ENABLE_ACCESS_BLOBFILE
#if QT_VERSION < 0x040000
        file.writeBlock(buf, DataLen);
#else
        file.write(buf,DataLen);
#endif
        file.close();
#endif
      }
#ifdef ENABLE_PREPARED_EXTENSION
      else
      if ( NeedResize ) /* Need to bind new QByteArray. It can be got by boundValue */
      {
#if QT_VERSION < 0x040000
        extension()->
#endif
                      bindValue(QString(ByteArrayIndex).toInt(), ba, QSql::InOut);
      }

      /* return real data len */
#if QT_VERSION < 0x040000
      extension()->
#endif
                    bindValue((int)QString(SizeIndex).toInt(), (int)DataLen, QSql::InOut);
#endif /* ENABLE_PREPARED_EXTENSION */
    } /* FetchBlob, blob get */
    else                                                   /* ... 07.08.13 */
    if ( main_query.contains("clear", ci) > 0 || main_query.startsWith("PurgeBlob") )
    {
      QVariant& Variant = extension()->values["0"].value;  /* 09.08.13 ... */
      if ( Variant.type() != QVariant::Invalid )
        coln = Variant.toInt();                            /* ... 09.08.13 */

      if ( ( lRet = LINTER_PurgeBlob( NCurs, coln, NULL, NULL ) ) )
      {
        setLastError( getError( 0, NCurs, 0 , QString( "QLinterResult::exec() : " ), er_type, lRet ) ); /* 13.11.09 */
        TRS_ERROR "Error in LINTER_PurgeBlob" TRS_END;
        TRS_RETURN;
        return false;  
      }
    } /* clear PurgeBlob */

    L_LONG CurrentRow;                                     /* 09.08.13a ... */
    L_LONG Len =  sizeof(CurrentRow);

    lRet = LINTER_GetCursorOption( NCurs, cCurrentRow , 0,
                                    &CurrentRow, &Len);
    if ( lRet == 0 ) /* Last query was select */
    { /* restoring current position back */
      setSelect(true);
      setAt( CurrentRow - 1);

      LastDataField = -1;
      LastDataLength = -1;
    }                                                      /* ... 09.08.13a */

    setActive( true );                                     /* 14.08.13 */
    TRS_RETURN;
    return true;  
  }                                                          /* ... 22.02.12 */

  /* bind parameters - positional binding */
  if ( extension()->index.count() > 0 )
  {    
    QVariant val;
    void* ParameterAddress = NULL;
    char* NullIndicator;
    char* ParameterName = NULL;
    L_SWORD ParameterNumber = 0;
    L_SWORD ParameterType = 0;    
    L_LONG* ParameterLength = NULL; 

    for ( QMap<int, QString>::Iterator it = extension()->index.begin(); it != extension()->index.end(); it++ )
    {
      QString ind = *it;
      ParameterNumber = ind.toInt() + 1;
      val = extension()->values[ ind ].value;   
      L_LONG ParamLen = sizeof( ParamDesc );
      if ( ( lRet = LINTER_GetStatementOption( NStatement, sParamDesc, ParameterNumber, &ParamDesc, &ParamLen ) ) )
      {                    
        setLastError( getError( 0, 0, NStatement, QString( "QLinterResult::exec() : " ), er_type, lRet ) ); /* 13.11.09 */
        TRS_ERROR "Error in LINTER_GetStatementOption" TRS_END;
        TRS_RETURN;
        return false;  
      }
      NullIndicator = &NullIndicators[ParameterNumber];
      ParameterLength = &ParameterLengths[ParameterNumber];

      if ( val.isNull() )
        *NullIndicator = (char)true;
      else
        *NullIndicator = (char)false;

      TRS_MSG "NullIndicator is %i", *NullIndicator TRS_END;

      switch ( qDecodeLinterType(ParamDesc.Type) )
      {
        case QVariant::Int:
        {
          TRS_MSG "Binding Int parameter" TRS_END;
          int *v = (int*) calloc(1, sizeof(int));
          *v = val.toInt();
          ParameterAddress = v;
          ParameterType = tInteger;
          *ParameterLength = sizeof(int);
          BindedValues[ParameterNumber] = ParameterAddress;
          break;
        }
#ifdef Q_LLONG
        case QVariant::LongLong:
        {
          TRS_MSG "Binding LongLong parameter" TRS_END;
          Q_LLONG* v = (Q_LLONG*) calloc(1, sizeof(Q_LLONG));
          *v = val.toLongLong();
          ParameterAddress = v;
          ParameterType = tBigInt;
          *ParameterLength = sizeof(Q_LLONG);
          BindedValues[ParameterNumber] = ParameterAddress;
          break;
        }  
#endif

        case QVariant::Double:
        {
          TRS_MSG "Binding Double parameter" TRS_END;
          double* v = (double*) calloc(1, sizeof(double));
          *v = val.toDouble();
          ParameterAddress = v;
          ParameterType = tDouble;
          *ParameterLength = sizeof(double);
          BindedValues[ParameterNumber] = ParameterAddress;
          break;
        }
 
        case QVariant::ByteArray:
        {
          TRS_MSG "Binding ByteArray parameter" TRS_END;
          QByteArray ba = val.toByteArray();
          void* v = calloc(ba.size(), 1); 
          memcpy(v, ba.data(), ba.size());
          ParameterAddress = v;
          ParameterType = tByte;
          *ParameterLength = ba.size();
          BindedValues[ParameterNumber] = ParameterAddress;
          break;
        }

        case QVariant::String:
        {
          TRS_MSG "Binding String parameter" TRS_END;
          QByteArray ba = String2Ucs2(val.toString()); 
          void* v = calloc(ba.size(), 1); 
          memcpy(v, ba.data(), ba.size());
          ParameterAddress = v;
          ParameterType = tNChar;
          *ParameterLength = ba.size();
          BindedValues[ParameterNumber] = ParameterAddress;
          break;
        }

#if QT_VERSION < 0x040000
        case QVariant::CString:
        {
          TRS_MSG "Binding CString parameter" TRS_END;
          QByteArray ba = val.toByteArray();
          void* v = calloc(ba.size(), 1); 
          memcpy(v, ba.data(), ba.size());
          ParameterAddress = v;
          ParameterType = tChar;
          *ParameterLength = ba.size();
          BindedValues[ParameterNumber] = ParameterAddress;
          break;
        }
#endif

        case QVariant::DateTime:
        {
          TRS_MSG "Binding DateTime parameter" TRS_END;
          if (val.isNull())
            val = QDateTime::currentDateTime();
          QString tmp = val.toDateTime().toString( Qt::ISODate );        
          tmp.replace("-","");
          tmp.replace("T","");
          tmp.replace(":","");
          L_BYTE* v = (L_BYTE *)calloc(1, sizeof(L_DECIMAL)); 
          STRTOTICKF( QS2C(tmp), date_format, v);         
          ParameterAddress = v;
          ParameterType = tDate;
          *ParameterLength = sizeof(L_DECIMAL);
          BindedValues[ParameterNumber] = ParameterAddress;
          break;
        }

        case QVariant::Bool:
        {
          TRS_MSG "Binding Bool parameter" TRS_END;
          L_BOOL* v = (L_BOOL*) calloc(1, sizeof(L_BOOL));
          *v = val.toBool();
          ParameterAddress = v;
          ParameterType = tBoolean;
          *ParameterLength = sizeof(L_BOOL);
          BindedValues[ParameterNumber] = ParameterAddress;
          break;
        }
        
        default:
          TRS_MSG "Binding NULL parameter" TRS_END;
          ParameterAddress = NullIndicator;
          ParameterType = ParamDesc.Type;    
          *ParameterLength = 0;
      }

      if ((!ParameterAddress) && (*ParameterLength == 0)) 
        ParameterAddress = NullIndicator;

      TRS_MSG "NCurs = %i, NStatement = %i, ParamNumber = %i, ParamAddr = %p , ParamType = %i, ParamLen = %i", NCurs, NStatement, ParameterNumber, ParameterAddress, ParameterType, *ParameterLength TRS_END;
      TRS_MSG "Param Value: %s", (char *) ParameterAddress TRS_END;
      
      if ( ( lRet = LINTER_BindParameter( NCurs, NStatement, ParameterNumber, ParameterName, NullIndicator, ParameterAddress, 1, *ParameterLength, ParameterType, NULL, ParameterLength ) ) )
      {
        for (int i=1; i<=ParameterNumber; i++ )
          free(BindedValues[i]);

        setLastError( getError( 0, NCurs, 0, QString( "QLinterResult::exec() : " ), er_type, lRet ) ); /* 13.11.09 */
        TRS_ERROR "Error in LINTER_BindParameter" TRS_END;
        TRS_RETURN;
        return false;  
      }   
    }
  }
  if ( ( lRet = LINTER_ExecuteStatement( NCurs, NStatement, NULL, NULL, NULL ) ) )
  {
    for (int i=1; i<=int(extension()->index.count()); i++ )
      free(BindedValues[i]);
    
    setLastError( getError( 0, NCurs, 0, QString( "QLinterResult::exec() : " ), er_type, lRet ) ); /* 13.11.09 */
    TRS_ERROR "Error in LINTER_ExecuteStatement" TRS_END;
    TRS_RETURN;
    return false;  
  }   

  for (int i=1; i<= int(extension()->index.count()); i++ )
    free(BindedValues[i]);

  setSelect( true );                                       /* 14.08.13 */
  ColCount = extension()->index.count(); 
  
  int outparamn = 0;

  if ( LOWER(lastQuery()).startsWith( "execute into" ) ) 
    outparamn = -1;
  
  /* fetch blob values and process output parameters */
  for ( QMap<int, QString>::Iterator it = extension()->index.begin(); it != extension()->index.end(); it++ )
  {
    QString ind = *it;

    L_LONG ParamLen ;
    ParamLen =  sizeof( ParamDesc );
    if ( ( lRet = LINTER_GetStatementOption( NStatement, sParamDesc, ind.toInt() + 1, &ParamDesc, &ParamLen ) ) )
    {
      setLastError( getError( 0, 0, NStatement, QString( "QLinterResult::exec() : " ), er_type, lRet ) ); /* 13.11.09 */
      TRS_ERROR "Error in LINTER_GetStatementOption" TRS_END;
      TRS_RETURN;
      return false;  
    }

    L_LONG ParamType;
    ParamLen = sizeof( ParamType );
    if ( ( lRet = LINTER_GetStatementOption( NStatement, sParamType, ind.toInt() + 1, &ParamType, &ParamLen ) ) )
    {
      setLastError( getError( 0, 0, NStatement, QString( "QLinterResult::exec() : " ), er_type, lRet ) ); /* 13.11.09 */
      TRS_ERROR "Error in LINTER_GetStatementOption" TRS_END;
      TRS_RETURN;
      return false;  
    }

    if ( ParamDesc.Type == tBlob )
    {
      if ( (ParamType == pInput) || (ParamType == pInputOutput) ) /* 26.01.09 */
      { 
        L_LONG ColNum;                                  /* 04.04.13 */

        /* we must insert blob data into current record */
        QByteArray ba = extension()->values[ ind ].value.toByteArray();
        
        if ( ( lRet = LINTER_GetStatementOption( NStatement, sParamMap, ind.toInt() + 1, &ColNum, &ParamLen ) ) ) /* 04.04.13 ... */
        {
          setLastError( getError( 0, 0, NStatement, QString( "QLinterResult::exec() : " ), er_type, lRet ) );
          TRS_ERROR "Error in LINTER_GetStatementOption" TRS_END;
          TRS_RETURN;
          return false;  
        }                                                    /* ... 04.04.13 */

        if ( ( lRet = LINTER_PurgeBlob( NCurs, ColNum, NULL, NULL ) ) ) /* 17.08.10 ... */ /* 04.04.13 */
        {
          setLastError( getError( 0, NCurs, 0, QString( "QLinterResult::exec() : " ), er_type, lRet ) ); /* 13.11.09 */
          TRS_ERROR "Error in LINTER_PurgeBlob" TRS_END;
          TRS_RETURN;
          return false;  
        }                                                    /* ... 17.08.10 */
        
#if QT_VERSION >= 0x040000                                 /* 09.08.13b ... */
        char* buf = (char*)ba.constData();
#else
        char* buf = (char*)((const char*)ba);
#endif
        L_LONG Size = ba.size();

        for ( L_LONG offset = 0; offset < Size; offset += DEFAULT_BLOB_PORTION_SIZE )
        {
          L_LONG len = ( (Size - offset) < DEFAULT_BLOB_PORTION_SIZE ) ? Size - offset : DEFAULT_BLOB_PORTION_SIZE; /* ... 09.08.13b */ 

          if ((lRet = LINTER_AddBlob2( NCurs, ColNum, 0, buf + offset, len, NULL, NULL, NULL))) /* 04.04.13 */
          {
            setLastError( getError( 0, NCurs, 0, QString( "QLinterResult::exec() : " ), er_type, lRet ) ); /* 13.11.09 */
            TRS_ERROR "Error in LINTER_AddBlob" TRS_END;
            TRS_RETURN;
            return false;  
          }                                                  /* ... 26.01.09 */
        }
        TRS_MSG "Blob data successfully added." TRS_END;
      }

      if ( (ParamType == pOutput) || (ParamType == pInputOutput) )
      {                 
        /* we must fetch blob data from current record */
        QByteArray ba;
        L_LONG bloblen;
        L_LONG bufsize;
       
        if ( ( lRet = LINTER_GetCursorOption( NCurs, cBlobLength, ind.toInt() + 1, (void*)&bloblen, NULL ) ) )  /* 26.01.09 */ 
        {
          setLastError( getError( 0, NCurs, 0, QString( "QLinterResult::exec() : " ), er_type, lRet ) ); /* 13.11.09 */
          TRS_ERROR "Error in LINTER_GetCursorOption" TRS_END;
          TRS_RETURN;
          return false;  
        } 
    
        ba.resize( bloblen );
        if ( (L_LONG)ba.size() != bloblen )
        {
          setLastError( QSqlError( QString("QLinterResult::exec() : Not enough memory") ) );
          TRS_ERROR "Not enough memory" TRS_END;
          TRS_RETURN;
          return false;  
        }
       
        for ( L_LONG offset = 0; offset < bloblen; offset += DEFAULT_BLOB_PORTION_SIZE )
        {
          bufsize = ( (bloblen - offset) < DEFAULT_BLOB_PORTION_SIZE ) ? bloblen - offset + 1: DEFAULT_BLOB_PORTION_SIZE;
          if ((lRet = LINTER_FetchBlob( NCurs, ind.toInt() + 1, offset, &bufsize, ba.data() + offset, NULL, NULL ))) /* 26.01.09 ... */
          {
            setLastError( getError( 0, NCurs, 0, QString( "QLinterResult::exec() : " ), er_type, lRet ) ); /* 13.11.09 */
            TRS_ERROR "Error in LINTER_FetchBlob" TRS_END;
            TRS_RETURN;
            return false;  
          }
        }
        extension()->values[ ind ].value = QVariant( ba );
        TRS_MSG "Blob data successfully readed." TRS_END;
      }  
    } /* if ( ParamDesc.Type == tBlob ) */

    if ( (ParamType == pOutput) || (ParamType == pInputOutput) )
    {            
      outparamn++;

      QVariant v = data(outparamn - 1);
      TRS_MSG "output param %s(%i) is %s", QS2C(ind), outparamn, QS2C(v.toString()) TRS_END;
      extension()->values[ ind ].value = v;
#if QT_VERSION < 0x040000
      extension()->                                        /* 07.08.13c */
#endif
      bindValue(ind.toInt(), v, QSql::InOut); /* 03.03.2010 */
    }
  }  
#else /* not ENABLE_PREPARED_EXTENSION */
  if ( ( lRet = LINTER_ExecuteStatement( NCurs,
                                         NStatement,
                                         NULL,
                                         NULL,
                                         NULL ) ) )
  {
    setLastError( getError( 0, NCurs, 0, QString( "QLinterResult::exec() : " ), er_type, lRet ) );
    TRS_ERROR "Error in LINTER_ExecuteStatement" TRS_END;
    TRS_RETURN;
    return false;  
  }   
#endif
  if ( LOWER(lastQuery()).startsWith( "execute" ) || LOWER(lastQuery()).startsWith( "call" )) /* 18.09.12 */
  {
    if ( ( lRet = LINTER_GetCursorOption( NCurs, cAnswerDesc, 0, (void*)&ParamDesc, NULL ) ) ) 
    {
      setLastError( getError( 0, NCurs, 0, QString( "QLinterResult::exec() : " ), er_type, lRet ) ); /* 13.11.09 */
      TRS_ERROR "Error in LINTER_GetCursorOption - can't get column description." TRS_END;
      TRS_RETURN;    
      return false;
    } 
    
    /* if procedure returns cursor */
    if ( ParamDesc.Type == 0 )
    {
      L_LONG tmpcurn;
      L_LONG len;
    
      if ( ( lRet = LINTER_GetData( NCurs, 0, &tmpcurn, sizeof(tmpcurn), tInteger, 0, 0, &len ) ) )
      {      
        setLastError( getError( 0, NCurs, 0, QString( "QLinterResult::exec() : " ), er_type, lRet ) ); /* 13.11.09 */
        TRS_ERROR "Error in LINTER_GetData for procedure result" TRS_END;
        TRS_RETURN;
        return false;
      }

      NMainCurs = NCurs; 
      NCurs = tmpcurn;
    }
  }

  if ( ( lRet = LINTER_GetCursorOption(NCurs, cColumnCount, 0, &ColCount, NULL ) ) )
  {
    TRS_MSG "Can't get columns count, ColCount set to 0" TRS_END;
    ColCount = 0;
  }
  
  if ( ( lRet = LINTER_GetCursorOption(NCurs, cRowCount, 0, &RowCount, NULL ) ) ) 
  {
    TRS_MSG "Can't get rows count, RowCount set to -1" TRS_END;
    RowCount = -1;    
  }

  TRS_MSG "ColCount = %i", ColCount TRS_END;
  setSelect( ColCount > 0 );
  setActive( true );                                       /* 14.08.13 */
  TRS_RETURN;
  return true;
}


QVariant QLinterResult::data( int field )
{
  /* NOTE: fields numeration in qt starts from 0, in linter from 1 */

  L_LONG       lRet,vLen;
  t_ParamDesc  pdAnsDesc;
  void        *vBuff;
  QVariant     result;

  TRS_ENTER;

  LastDataField = -1;

  if ( !isSelect() || field >= ColCount )
  {  
    qWarning( "QLinterResult::data: column %d out of range", field );
    TRS_RETURN;
    return false;                                                /* 26.01.09 */
  }

  if ( ( lRet = LINTER_GetCursorOption( NCurs, cAnswerDesc, field + 1, (void*)&pdAnsDesc, NULL ) ) ) 
  {
    setLastError( getError( 0, NCurs, 0, QString( "QLinterResult::data() : " ), er_type, lRet ) ); /* 13.11.09 */
    TRS_ERROR "Error in LINTER_GetCursorOption - can't get column description." TRS_END;
    TRS_RETURN;    
    return false;                                                /* 26.01.09 */
  } 

  if ( pdAnsDesc.Type == tBlob )
  {
    QByteArray ba;
    L_LONG bloblen;
    L_LONG bufsize;

    result = QVariant();

    if ( ( lRet = LINTER_GetCursorOption( NCurs, cBlobLength, field + 1, (void*)&bloblen, NULL ) ) )  /* 26.01.09 */
    {
      TRS_ERROR "Error in LINTER_GetCursorOption" TRS_END;
      TRS_RETURN;
      return false; 
    } 
    
    ba.resize( bloblen );
    if ( (L_LONG)ba.size() != bloblen )
    {
      setLastError( QSqlError( QString("QLinterResult::data() : Not enough memory") ) );
      TRS_ERROR "Not enough memory" TRS_END;
      TRS_RETURN;
      return false;                                              /* 26.01.09 */
    }
    
    for ( L_LONG offset = 1; offset <= bloblen; offset += DEFAULT_BLOB_PORTION_SIZE )
    {
      bufsize = ( (bloblen - offset) <= DEFAULT_BLOB_PORTION_SIZE ) ? bloblen - offset + 1: DEFAULT_BLOB_PORTION_SIZE;
      if ( ( lRet = LINTER_FetchBlob( NCurs,field + 1, offset, &bufsize, ba.data() + offset - 1, NULL, NULL ) ) )
      {                                    
        TRS_ERROR "Error in LINTER_FetchBlob" TRS_END;
        TRS_RETURN;
        return result;
      }   
    }
    result = QVariant( ba );    
    LastDataField = field;
    LastDataLength = bloblen;    

    TRS_RETURN;
    return result;
  }

  if ((pdAnsDesc.Type == tVarChar) ||                        /* 17.05.06 ... */
      (pdAnsDesc.Type == tNVarChar) ||
      (pdAnsDesc.Type == tVarByte)
     )
    vLen = pdAnsDesc.Length + 2;
  else
    vLen = pdAnsDesc.Length;                                 /* ... 17.05.06 */
  vBuff = calloc( vLen, 1 );
  
  if ( !vBuff)
  {
    setLastError( QSqlError( QString("QLinterResult::data() : Not enough memory.") ) );
    TRS_ERROR "Not enough memory" TRS_END;
    TRS_RETURN;    
    return result;
  } 

  if ( ( lRet = LINTER_GetData( NCurs, field + 1, vBuff, vLen, pdAnsDesc.Type, 0, 0, &LastDataLength ) ) )
  {
    setLastError( getError( 0, NCurs, 0, QString( "QLinterResult::data() : " ), er_type, lRet ) ); /* 13.11.09 */
    free(vBuff);
    TRS_ERROR "Error in LINTER_GetData for field %i", field TRS_END;
    TRS_RETURN;
    return result;
  } 

  LastDataField = field;
  if ( LastDataLength == -1 )
  {
    /* NULL value */
    free(vBuff);
    TRS_MSG "data(%i) = NULL", field TRS_END;
#if QT_VERSION < 0x040000
    result.cast(qDecodeLinterType(pdAnsDesc.Type));
#else
    result.convert(qDecodeLinterType(pdAnsDesc.Type));
#endif
    TRS_RETURN;
    return result;
  }  

  result = toVariant( vBuff, pdAnsDesc.Length, pdAnsDesc.Type, pdAnsDesc.Prec, pdAnsDesc.Scale );
  free(vBuff);

  TRS_MSG "data(%i) = %s", field, QS2C(result.toString()) TRS_END;
  TRS_RETURN;
  return result;
}


bool QLinterResult::isNull( int field )
{
  bool result;

  TRS_ENTER;

  if ( !isSelect() || field >= ColCount )
  {  
    qWarning( "QLinterResult::isNull: column %d out of range", field );
    TRS_RETURN;
    return true;
  }

  if ( LastDataField != field )
    data(field);

  result = ( ( LastDataField != -1 ) && ( LastDataLength != -1 ) ) ? false : true;

  TRS_MSG "isNull(%i) = %i", field, (int)result TRS_END;
  TRS_RETURN;
  return result; 
}


bool QLinterResult::reset( const QString& query )
{
  TRS_ENTER;
  if ( prepare( query ) )
    if ( exec() )
    {
      TRS_RETURN;
      return true;
    }

  TRS_RETURN;
  return false;
}

bool LINTER_NO_DATA (L_WORD cur_id)                          /* 11.03.2012 ... */    
{
  L_LONG apierr = 0, err_cod = 0, syserr = 0;

  LINTER_Error(0, cur_id, 0, &apierr, &err_cod, &syserr, NULL, NULL);

  if ( err_cod == 2 )
    return true;
  else
    return false;
}                                                            /* ... 11.03.2012 */

bool QLinterResult::fetch( int i )
{
  L_LONG lRet;

  TRS_ENTER;
 
  TRS_MSG "Fetch to record %i", i TRS_END;

  if ( i >= RowCount )
  {
#if QT_VERSION < 0x040000
    setAt( QSql::AfterLast );
#else
    setAt( QSql::AfterLastRow );
#endif
    TRS_RETURN;
    return false;
  }

  if ( ( lRet = LINTER_Fetch( NCurs, toAbsNumber, i + 1, 1, NULL, NULL, NULL ) ) )
  {
    if ( !LINTER_NO_DATA(NCurs) )                            /* 11.03.2012 */
    {
      setLastError( getError( 0, NCurs, 0, QString( "QLinterResult::fetch() : " ), er_type, lRet ) ); /* 13.11.09 */
      TRS_RETURN;
    }
    return false;
  }

  setAt( i );

  LastDataField = -1;
  LastDataLength = -1;

  TRS_RETURN;
  return true;
}


bool QLinterResult::fetchFirst()
{
  return fetch( 0 );
}


#  if QT_VERSION >= 0x040000
bool QLinterResult::fetchNext()                               /* 29.07.09... */
{
  L_LONG lRet;

  TRS_ENTER;
 
  TRS_MSG "Fetch to next record" TRS_END;

  if ( ( lRet = LINTER_Fetch( NCurs, toNext, 0, 1, NULL, NULL, NULL ) ) )
  {
    if ( !LINTER_NO_DATA(NCurs) )                            /* 11.03.2012 */
    {
      setLastError( getError( 0, NCurs, 0, QString( "QLinterResult::fetchNext() : " ), er_type, lRet ) ); /* 13.11.09 */
      TRS_RETURN;
    }
    return false;
  }

  setAt(at() + 1);

  TRS_RETURN;
  return true;
}


bool QLinterResult::fetchPrevious()
{
  L_LONG lRet;

  TRS_ENTER;
 
  TRS_MSG "Fetch to next record" TRS_END;

  if ( ( lRet = LINTER_Fetch( NCurs, toPrevious, 0, 1, NULL, NULL, NULL ) ) )
  {
    if ( !LINTER_NO_DATA(NCurs) )                            /* 11.03.2012 */
    {
      setLastError( getError( 0, NCurs, 0, QString( "QLinterResult::fetchPrevious() : " ), er_type, lRet ) ); /* 13.11.09 */
      TRS_RETURN;
    }
    return false;
  }

  setAt(at() - 1);

  TRS_RETURN;
  return true;
}                                                             /* ...29.07.09 */
#endif

bool QLinterResult::fetchLast()
{
  return fetch( size() - 1 );
}


int QLinterResult::size()
{
  TRS_ENTER;
  TRS_RETURN; 
  return isSelect()?RowCount:-1;
}


int QLinterResult::numRowsAffected()
{
  TRS_ENTER;
  TRS_RETURN; 
  return RowCount;
}


QSqlRecord QLinterResult::record() const
{
  L_LONG lRet;
  t_ParamDesc pdAnsDesc;
  L_CHAR      columnName[MAX_ID_LEN+1];                 /* 24.09.12 */

  TRS_ENTER;

  QSqlRecord rec;
  QSqlField  *f;
  for (L_LONG i=1; i<=(ColCount); i++)
  {
    if ( ( lRet = LINTER_GetCursorOption( NCurs, cAnswerDesc, i, (void*)&pdAnsDesc, NULL ) ) ) 
    {
      /* ((QLinterDriver *)this)->setLastError( getError( 0, NCurs, 0, QString( "QLinterDriver::record() : " ), er_type, lRet ) ); */ /* 13.11.09 */ /* 28.06.13 */
      TRS_ERROR "Error in LINTER_GetCursorOption - can't get column description." TRS_END;
      TRS_RETURN;    
      return QSqlRecord();
    }
    
    memcpy(columnName,pdAnsDesc.Column,MAX_ID_LEN);     /* 24.09.12 ... */
    columnName[MAX_ID_LEN] = 0;
    strltrim(columnName, (char *)" ");
    strrtrim(columnName, (char *)" ");

    TRS_MSG "Column '%s' added to QSqlRecord", columnName TRS_END;

#if QT_VERSION < 0x040000                                    /* 27.06.11 ... */
    if ( ((QLinterDriver *)this)->Qt3Quotes )                /* 29.11.11 ... */
      f = new QSqlField( "\"" + QString(columnName) + "\"", qDecodeLinterType(pdAnsDesc.Type) );
    else
      f = new QSqlField( QString(columnName), qDecodeLinterType(pdAnsDesc.Type) ); /* ... 29.11.11 */
#else
    f = new QSqlField( QString(columnName), qDecodeLinterType(pdAnsDesc.Type) ); /* ... 24.09.12 */
    f->setLength((pdAnsDesc.Type == tNChar || pdAnsDesc.Type == tNVarChar) ? pdAnsDesc.Length/2 : 
                                                                             pdAnsDesc.Length); /* 24.09.12a */
#endif                                                       /* ... 27.06.11 */
    rec.append ( *f );
    delete f;
  } 
 
  TRS_RETURN;
  return rec;
}

QVariant QLinterResult::lastInsertId() const
{
  TRS_ENTER;
  TRS_RETURN;
    return QVariant();
}

#if defined(QT3_TRANSACTIONS) && QT_VERSION >= 0x030200 && QT_VERSION < 0x040000        /* 07.08.13f ... */
/* Special class. If this class defined - driver use open whith options */
Q_EXPORT QPtrDict<QSqlOpenExtension> *qSqlOpenExtDict();
class QLinterSQLOpenExtension:public QSqlOpenExtension
{
private:
  QLinterDriver* driver;
public:
  QLinterSQLOpenExtension(QLinterDriver* drv): QSqlOpenExtension(), driver(drv) {};
  virtual ~QLinterSQLOpenExtension() {};
  virtual bool open( const QString& db,
             const QString& user,
             const QString& password,
             const QString& host,
             int port,
             const QString& connOpts )
     { return driver->open(db, user, password, host, port, connOpts); };
};
#endif                                                    /* ... 07.08.13f */

QLinterDriver::QLinterDriver( QObject* parent, const char* name )
{
  Q_UNUSED(parent);
  Q_UNUSED(name);

  NConn = 0;
#if QT_VERSION < 0x040000                                    /* 29.11.11 ... */
  Qt3Quotes = L_TFALSE;
#if defined(QT3_TRANSACTIONS) && QT_VERSION >= 0x030200
  qSqlOpenExtDict()->insert( this, new QLinterSQLOpenExtension(this) ); /* 07.08.13f */
#endif
#endif                                                       /* ... 29.11.11 */
  LOG_INIT("0xFFFFFFFF", "log.txt");
}

QLinterDriver::~QLinterDriver()
{
  TRS_ENTER;
  if ( NConn ) 
    LINTER_CloseConnect(NConn);
  TRS_RETURN;
  
  LOG_SHUT;  
}

#if QT_VERSION < 0x040000                                    /* 29.11.11 ... */
bool QLinterDriver::open( const QString & db,
                    const QString & user,
                    const QString & password,
                    const QString & host,
                    int port )
{
  return open( db, user, password, host, port, "EXCLUSIVE");
}
#endif                                                       /* ... 29.11.11 */

bool QLinterDriver::open( const QString & db,
                    const QString & user,
                    const QString & password,
                    const QString & host,
                    int port,
                    const QString & options )
{
  L_LONG  lRet;
  L_LONG  lOptions = 0;
  
  TRS_ENTER;

  Q_UNUSED( host );
  Q_UNUSED( port );

#if QT_VERSION < 0x040000
  QStringList raw = QStringList::split( ';', options );
#else
  QStringList raw = QString( options ).split( ';' );
#endif
  QStringList opts;
  QStringList::ConstIterator it;
  QString Charset;
  for ( it = raw.begin(); it != raw.end(); it++ )
  {
    QString tmp( *it );
#if QT_VERSION >= 0x040000
    tmp = tmp.trimmed();                                      /* 17.11.11 */
#endif

    tmp = UPPER(tmp);                                         /* 15.10.09 */

    if ( !QString::compare( tmp, QString( "AUTOCOMMIT" ) ) )
      lOptions |= mAutocommit;
    if ( !QString::compare( tmp, QString( "OPTIMISTIC" ) ) )
      lOptions |= mOptimistic;
    if ( !QString::compare( tmp, QString( "EXCLUSIVE" ) ) )
      lOptions |= mExclusive;

    if ( tmp.startsWith( "CHARSET=" ) )                      /* 15.10.09 ... */
    {
      tmp = tmp.section('=', 1);
#if _VER_MAX < 600
      if ( !QString::compare( tmp, QString( "KOI8" ) ) || !QString::compare( tmp, QString( "KOI8-R" ) ) || 
           !QString::compare( tmp, QString( "ISO" ) )  || !QString::compare( tmp, QString( "ISO-8859-1" ) ) )
        lOptions |= mKoi8;
      if ( !QString::compare( tmp, QString( "ANSI" ) )  || !QString::compare( tmp, QString( "CP1251" ) ) || 
           !QString::compare( tmp, QString( "1251" ) ) || !QString::compare( tmp, QString( "WINDOWS-1251" ) ) || 
           !QString::compare( tmp, QString( "WINDOWS-1252" ) ) || !QString::compare( tmp, QString( "CP1252" ) ))
        lOptions |= mAnsi;
      if ( !QString::compare( tmp, QString( "OEM" )) || !QString::compare( tmp, QString( "CP866" ) ) || 
           !QString::compare( tmp, QString( "866" )) || !QString::compare( tmp, QString( "IBM850" ) ))
      {
        lOptions |= mAnsi;
        lOptions |= mKoi8;
      }
#else          
      Charset = tmp;
#endif
    }                                                        /* ... 15.10.09 */
#if _VER_MAX >= 610                                             
    if ( !QString::compare( tmp, QString( "READ_UNCOMMITTED" ) ) )
      lOptions |= mReadUncommitted;
    if ( !QString::compare( tmp, QString( "READ_COMMITTED" ) ) )
      lOptions |= mReadCommitted;
    if ( !QString::compare( tmp, QString( "SERIALIZABLE" ) ) )
      lOptions |= mSerializable;
#endif

#if QT_VERSION < 0x040000                                    /* 29.11.11 ... */
    if ( !QString::compare( tmp, "NONASCIICOMPATIVE3" ) )
      this->Qt3Quotes = L_TTRUE;
#endif                                                       /* ... 29.11.11 */
  }

  this->Options = lOptions;

  if ( ( lRet = LINTER_ConnectCS( QS2C(user), 0, QS2C(password), 0, QS2C(db), QS2C(Charset), lOptions, &NConn ) ) ) 
  {
    setLastError( getError( NConn, 0, 0, QString( "QLinterDriver::open() : " ), er_type, lRet ) ); /* 13.11.09 */
    setOpenError( true );                                  /* 14.08.13 */
    TRS_ERROR "Can't connect to DB." TRS_END;
    TRS_RETURN;
    return false;
  }

  setOpen( true );                                         /* 14.08.13 */
  setOpenError( false );                                   /* 14.08.13 */

  TRS_MSG "Succesfully connected to DB, NConn = %i", NConn TRS_END;

  TRS_RETURN;
  return true;
}

void QLinterDriver::close()
{
  TRS_ENTER;
  setOpen( false );                                        /* 14.08.13 */
  if ( NConn )
    LINTER_CloseConnect(NConn);
  NConn = 0;
  TRS_RETURN;
}

bool QLinterDriver::hasFeature( DriverFeature f ) const
{
  switch ( f )
  {
    case Transactions:
      return true;                                         /* 14.08.13 ... */
    case QuerySize:
      return true;
    case BLOB:
      return false;
#ifdef ENABLE_PREPARED_EXTENSION
    case PreparedQueries:
      return true;
    case PositionalPlaceholders:
      return true;
    case NamedPlaceholders:
      return false;
#endif
    default:
      return false;                                        /* .... 14.08.13 */
  }
}

QSqlQuery QLinterDriver::createQuery() const
{
  TRS_ENTER; 
  TRS_RETURN;
  return QSqlQuery( new QLinterResult( this ) );
}

QSqlRecord QLinterDriver::record( const QString& tablename ) const
{
  TRS_ENTER;  

  QSqlRecord rec;
  QString stmt = "select * from %1 limit 1;";  
  QSqlQuery q = createQuery();
  QString table;                                             /* 27.09.10 ... */

  if (isIdentifierEscaped(tablename, TableName))             /* 27.06.11 ... */
    table = tablename;
  else
    table = "\"" + UPPER(tablename) + "\"";                  /* ... 27.06.11 */

  q.exec( stmt.arg(table) );	                               /* ... 27.09.10 */
  rec = record(q);

  TRS_RETURN;
  return rec;
}

QSqlRecord QLinterDriver::record( const QSqlQuery& query ) const
{
  TRS_ENTER;
 
  QSqlRecord rec;
  if ( isOpen() && query.isActive() && query.isSelect() && (query.driver() == this) )
     rec = ((QLinterResult *)query.result())->record();

  TRS_RETURN;
  return rec;
}

bool QLinterDriver::beginTransaction()
{
  L_LONG lRet;
  
  TRS_ENTER;

  if ( !isOpen() ) 
  {
    qWarning( "QLinterDriver::beginTransaction: Database not open" );
    TRS_RETURN;
    return false;
  }

  if ( ( lRet = LINTER_Commit( NConn, NULL, NULL ) ) )
  {
    setLastError( getError( NConn, 0, 0, QString( "QLinterDriver::beginTransaction() : " ), er_type, lRet ) ); /* 13.11.09 */
    TRS_ERROR "Error in LINTER_RollBack." TRS_END;
    TRS_RETURN;
    return false;
  }

  TRS_RETURN;
  return true; 
}

bool QLinterDriver::commitTransaction()
{
  L_LONG lRet;

  TRS_ENTER;

  if ( !isOpen() )
  {
    qWarning( "QLinterDriver::commitTransaction: Database not open" );
    TRS_RETURN;
    return false;
  }

  if ( ( lRet = LINTER_Commit( NConn, NULL, NULL ) ) )
  {
    setLastError( getError( NConn, 0, 0, QString( "QLinterDriver::commitTransaction() : " ), er_type, lRet ) ); /* 13.11.09 */
    TRS_ERROR "Error in LINTER_ExecuteDirect." TRS_END;
    TRS_RETURN;
    return false;
  }
 
  TRS_RETURN;
  return true;
}


bool QLinterDriver::rollbackTransaction()
{
  L_LONG lRet;
  
  TRS_ENTER;

  if ( !isOpen() )
  {
    qWarning( "QLinterDriver::rollbackTransaction: Database not open" );
    return false;                                          /* 14.08.13 */
  }

  if ( ( lRet = LINTER_RollBack( NConn, NULL, NULL ) ) )
  {
    setLastError( getError( NConn, 0, 0, QString( "QLinterDriver::rollbackTransaction() : " ), er_type, lRet ) ); /* 13.11.09 */
    TRS_ERROR "Error in LINTER_ExecuteDirect." TRS_END;
    TRS_RETURN;
    return false;
  }

  TRS_RETURN;
  return true;
}


#if QT_VERSION < 0x040000
QString QLinterDriver::formatValue( const QSqlField* field, bool trimStrings ) const
#else
QString QLinterDriver::formatValue(const QSqlField &field1, bool trimStrings) const
#endif
{
  TRS_ENTER;
#if QT_VERSION >= 0x040000
  const QSqlField* field = &field1;
#endif
  TRS_MSG "Unformatted string : %s, type %i", QS2C(field->value().toString()), field->type() TRS_END;  

  QString r;
  if ( field->isNull() )
  {
    TRS_MSG "field is null" TRS_END;
    r = QString("NULL");
  }
  else
  {
    switch ( field->type() )
    {
      case QVariant::Double:
      {
        r = QString().setNum(field->value().toDouble(),'f',15);
        break;
      }
      case QVariant::Bool:
      {
      if ( field->value().toBool() == true )
          r = QString("TRUE");
        else
          r = QString("FALSE");
        break;
      }
      case QVariant::DateTime:
      {
        r = field->value().toDateTime().toString( "'dd.MM.yyyy:hh:mm:ss'" );
        break;
      }
      case QVariant::String:
      {
        r = "hex('" + ByteArray2Hex( String2Ucs2( field->value().toString() ) ) + "')";
        break;
      }
      case QVariant::ByteArray:
      {
        r = "hex('" + ByteArray2Hex( field->value().toByteArray() ) + "')";
        break;
      }
      default:  
#if QT_VERSION < 0x040000
        r = QSqlDriver::formatValue( field, trimStrings );
#else
        r = QSqlDriver::formatValue( *field, trimStrings );
#endif
    }
  }

  TRS_MSG "Formated string: %s", QS2C(r) TRS_END; 

  TRS_RETURN;
  return r;
}


#if QT_VERSION >= 0x040000
QSqlResult* QLinterDriver::createResult() const
{
  return new QLinterResult(this);
}
#endif


QSqlIndex QLinterDriver::primaryIndex(const QString &tablename) const
{
  TRS_ENTER; 

  QSqlIndex idx( tablename );
  QSqlRecord rec = record( tablename );
  QString table;
  
  QString stmt = "select column_name, pk_name from LINTER_SYSTEM_USER.primary_keys where table_name = '%1' order by key_seq asc;";  
  QSqlQuery q = createQuery();

  if (isIdentifierEscaped(tablename, TableName))             /* 27.06.11 ... */
  {
    table = tablename;
    table = table.mid(1, table.length()-2);
  }
  else
    table = UPPER(tablename);                                /* ... 27.06.11 */

  q.exec( stmt.arg(table) );                                 /* ... 27.09.10 */
  while ( q.isActive() && q.next() )
  {
#if QT_VERSION < 0x040000
    QSqlField* Field = rec.field( q.value(0).toString());  /* 07.08.13d ... */
    
    if ( Field == NULL )
      Field = rec.field( "\"" + q.value(0).toString() + "\"");
    idx.append( *Field );                                  /* ... 07.08.13d */
#else
    idx.append( rec.field( q.value(0).toString() ) );
#endif
    idx.setName( q.value(1).toString() );
  }
  TRS_RETURN;
  return idx;
}

#if QT_VERSION < 0x040000
QStringList QLinterDriver::tables() const
#else
QStringList QLinterDriver::tables(QSql::TableType type) const
#endif
{
  QSqlQuery q;
  QString stmt; 
  QStringList res;

  TRS_ENTER;

  stmt = QString("select table_name from LINTER_SYSTEM_USER.tables;");
#if QT_VERSION >= 0x040000
  switch (type)
  {
    case QSql::SystemTables:
      stmt = QString("select table_name from LINTER_SYSTEM_USER.tables where table_type = 'SYSTEM TABLE';");
      break;

    case QSql::Tables:
      stmt = QString("select table_name from LINTER_SYSTEM_USER.tables where table_type = 'TABLE';");
      break;
    
    case QSql::Views:
      stmt = QString("select table_name from LINTER_SYSTEM_USER.tables where table_type = 'VIEW';");
      break;

    case QSql::AllTables:
      break;
  }
#endif
    
  q.exec( stmt );
  while ( q.isActive() && q.next() )
  {
    res << q.value(0).toString();
  }
  return res;
}

#if QT_VERSION >= 0x040000
QString QLinterDriver::escapeIdentifier(const QString &identifier, QSQL_DRIVER_IDENTIFIER_TYPE type) const /* 02.08.12 */ /* 27.09.10 ... */ 
{
    QString res = identifier;
   
    type = type;
    if(!identifier.isEmpty() && !identifier.startsWith(QLatin1Char('"')) && !identifier.endsWith(QLatin1Char('"')) ) {
        res.replace(QLatin1Char('"'), QLatin1String("\"\""));
        res.prepend(QLatin1Char('"')).append(QLatin1Char('"'));
        res.replace(QLatin1Char('.'), QLatin1String("\".\""));
    }
    return res;
}                                                            /* ... 27.09.10 */
#endif
