/****************************************************************************
 *          RELEX, Inc. Linter interface library source code                *
 *              This is public source code of RELEX, Inc.                   *
 *                                                                          *
 *       RELEX has no warranty obligations or liability for any             *
 *      usage of interface library source code which is modified.           *
 *                                                                          *
 *                 Copyright (c) 2000-2012 RELEX Inc.                       *
 *                                                                          *
 *                   This software was developed by                         *
 *                                                                          *
 *                            RELEX, Inc.                                   *
 *                         Voronezh, Russia                                 *
 *                                                                          *
 *                        All rights reserved.                              *
 ****************************************************************************
 AUTHOR: Gluhov
 ----------------------------------------------------------------------------
 PURPOSE:
 ----------------------------------------------------------------------------
 REVISION HISTORY:
 28.11.05 -- Gluhov ( file created )     
 19.04.06 -- Gluhov ( Added new linapi calls - blobs, Correct names )
 27.07.09 -- Muratova ( fetchNext, fetchPrevious )
 27.09.10 -- Muratova (escapeIdentifier)
 29.11.11 -- Muratova (add Qt3Quotes;
                       open with 5 params for qt < 4)
 10.08.12 -- DV (Remove unused QLinterDriver::NCurs field)
 ----------------------------------------------------------------------------*/

#ifndef QSQL_LINTER_H
#define QSQL_LINTER_H

#include <qsqldriver.h>
#include <qsqlresult.h>
#include <qsqlfield.h>
#include <qsqlindex.h>
#if QT_VERSION < 0x040000
#include <qsqlquery.h>
#else
#include <QSqlQuery>                                             /* 13.08.09 */
#endif

#include "linapi.h"

#if QT_VERSION >= 0x040000
struct Param {
  QVariant value;
};

typedef QMap<QString, Param> ValueMap;

struct Extension
{
  QMap<int, QString> index;
  ValueMap values;
};
#endif


class QLinterResult : public QSqlResult
{
  friend class QLinterDriver;

public:
  QLinterResult( const QSqlDriver* db );
  ~QLinterResult();

  bool prepare( const QString& query );
  bool exec();

protected:
  QVariant   data( int field );
  bool       isNull( int field );
  bool       reset( const QString& query );
  bool       fetch( int i );
  bool       fetchFirst();
#  if QT_VERSION >= 0x040000
  bool       fetchNext();                                        /* 29.07.09 */
  bool       fetchPrevious();                                    /* 29.07.09 */
#  endif
  bool       fetchLast();
  int        size();
  int        numRowsAffected();
  QVariant   lastInsertId() const;
  QSqlRecord record() const;

protected:
  L_WORD NConn;      /* connection identifier */
  L_WORD NCurs;      /* cursor identifier     */
  L_WORD NMainCurs;  /* used when stored proc returns cursor */
  L_WORD NStatement; /* statement identifier  */
  
  L_LONG ColCount;
  L_LONG RowCount;
  L_LONG LastDataField;  /* field id of last data fetched with data() or -1 */
  L_LONG LastDataLength; /* length of this data */

#if QT_VERSION >= 0x040000
  Extension* extension();
  Extension ext;
#endif
};

class QLinterDriver : public QSqlDriver
{
  friend class QLinterResult;

public:
    QLinterDriver( QObject* parent = 0, const char* name = 0 );
    ~QLinterDriver();

#if QT_VERSION < 0x040000                                    /* 29.11.11 ... */
    bool        open( const QString & db,
                      const QString & user = QString::null,
                      const QString & password = QString::null,
                      const QString & host = QString::null,
                      int port = -1 );
#endif                                                       /* ... 29.11.11 */

    bool        open( const QString& db,
                      const QString& user,
                      const QString& password,
                      const QString& host,
                      int port,
                      const QString& connOpts );

    void        close();
    bool        hasFeature( DriverFeature f ) const;
    QSqlQuery   createQuery() const;
    QSqlRecord  record( const QString & tablename ) const;
    QSqlRecord  record( const QSqlQuery & query   ) const;
    QSqlIndex   primaryIndex(const QString &tablename) const;
#if QT_VERSION >= 0x040000                                       /* 27.09.10 ... */                
    QString     escapeIdentifier(const QString &identifier, IdentifierType type) const; 
#endif                                                           /* ... 27.09.10 */

#if QT_VERSION < 0x040000
    QStringList tables() const;
    QString     formatValue( const QSqlField* field, bool trimStrings ) const;
#else
    QStringList tables(QSql::TableType type) const;
    QString     formatValue( const QSqlField &field, bool trimStrings ) const;
    QSqlResult *createResult() const;
#endif

protected:
    bool        beginTransaction();
    bool        commitTransaction();
    bool        rollbackTransaction();

    L_WORD NConn;   /* connection identifier */
    L_WORD Options; /* connection options */ 

#if QT_VERSION < 0x040000                                    /* 29.11.11 ... */
    L_BOOL Qt3Quotes; /* quotes for qt3 */
#endif                                                       /* ... 29.11.11 */

};

#endif
