/****************************************************************************
 *           RELEX, Ltd. ODBC driver source code                            *
 *  This is unpublished propietary source code of RELEX, Ltd.               *
 *  The copyright notice does not evidence any actual or intended           *
 *  publication of such source code.                                        *
 *                                                                          *
 *  Copyright (c) 1994-2000 RELEX Ltd.                                      *
 *                                                                          *
 *                   This software was developed by                         *
 *                                                                          *
 *                            RELEX, Ltd.                                   *
 *                         Voronezh, Russia                                 *
 *                                                                          *
 *                        All rights reserved.                              *
 ****************************************************************************
 AUTHOR: Galizin Oleg
 ----------------------------------------------------------------------------
 PURPOSE: make unixODBC configuration interface 
 ----------------------------------------------------------------------------
 REVISION HISTORY:
 creation date: 12.01.2000

  21.01.00 -- Galizin ( Change malloc on calloc and test allocation )
  29.11.2001 -- Korotchenko (new implemention)
  30.10.04 -- Ermakov (buffered fetch)
*/

#ifdef UNIXODBC

#include <odbcinstext.h>
extern void get_SERVER_default(char*  buf);
extern char** get_SERVER_list(void);
extern char** get_TXN_list();
extern char** get_CODEPAGE_list();
extern char** get_YesNo_list();                                  /* 30.10.04 */

static char *help_strings[] = 
{
    "Name of the server to connect to.",
    "User name to connect with.",
    "Password of user.",
    "Transaction mode",
    "Language",
    "Codepage",
    "Channel priority",
    "Buffered fetch",                                            /* 30.10.04 */
    (char*)0
};

int ODBCINSTGetProperties(
    HODBCINSTPROPERTY hLastProperty)
{
    hLastProperty->pNext =
        (HODBCINSTPROPERTY)malloc( sizeof(ODBCINSTPROPERTY) );
    hLastProperty = hLastProperty->pNext;
    memset( hLastProperty, 0, sizeof(ODBCINSTPROPERTY) );
    hLastProperty->nPromptType = ODBCINST_PROMPTTYPE_TEXTEDIT;
    strncpy( hLastProperty->szName, "UID", INI_MAX_PROPERTY_NAME );
    strncpy( hLastProperty->szValue, "", INI_MAX_PROPERTY_VALUE );
    hLastProperty->pszHelp = malloc(strlen(help_strings[1]) + 1);
    strcpy(hLastProperty->pszHelp, help_strings[1]);

    hLastProperty->pNext =
        (HODBCINSTPROPERTY)malloc( sizeof(ODBCINSTPROPERTY) );
    hLastProperty = hLastProperty->pNext;
    memset( hLastProperty, 0, sizeof(ODBCINSTPROPERTY) );
    hLastProperty->nPromptType = ODBCINST_PROMPTTYPE_TEXTEDIT;
    strncpy( hLastProperty->szName, "PWD", INI_MAX_PROPERTY_NAME );
    strncpy( hLastProperty->szValue, "", INI_MAX_PROPERTY_VALUE );
    hLastProperty->pszHelp = malloc(strlen(help_strings[2]) + 1);
    strcpy(hLastProperty->pszHelp, help_strings[2]);

    hLastProperty->pNext = 
        (HODBCINSTPROPERTY)malloc( sizeof(ODBCINSTPROPERTY) );
    hLastProperty = hLastProperty->pNext;
    memset( hLastProperty, 0, sizeof(ODBCINSTPROPERTY) );
    hLastProperty->nPromptType = ODBCINST_PROMPTTYPE_LISTBOX;
    strncpy( hLastProperty->szName, "TXN", INI_MAX_PROPERTY_NAME );
    hLastProperty->aPromptData=get_TXN_list();
    strncpy( hLastProperty->szValue, hLastProperty->aPromptData[0], INI_MAX_PROPERTY_VALUE );
    hLastProperty->pszHelp = malloc(strlen(help_strings[3]) + 1);
    strcpy(hLastProperty->pszHelp, help_strings[3]);

    hLastProperty->pNext =
        (HODBCINSTPROPERTY)malloc( sizeof(ODBCINSTPROPERTY) );
    hLastProperty = hLastProperty->pNext;
    memset( hLastProperty, 0, sizeof(ODBCINSTPROPERTY) );
    hLastProperty->nPromptType = ODBCINST_PROMPTTYPE_LISTBOX;
    strncpy( hLastProperty->szName, "SERVER", INI_MAX_PROPERTY_NAME );
    hLastProperty->pszHelp = malloc(strlen(help_strings[0]) + 1);
    strcpy(hLastProperty->pszHelp, help_strings[0]);
    hLastProperty->aPromptData=get_SERVER_list();
    get_SERVER_default(hLastProperty->szValue);

#if 0
    hLastProperty->pNext = 
    (HODBCINSTPROPERTY)malloc( sizeof(ODBCINSTPROPERTY) );
    hLastProperty = hLastProperty->pNext;
    memset( hLastProperty, 0, sizeof(ODBCINSTPROPERTY) );
    hLastProperty->nPromptType = ODBCINST_PROMPTTYPE_TEXTEDIT;
    strncpy( hLastProperty->szName, "LANGUAGE", INI_MAX_PROPERTY_NAME );
    strncpy( hLastProperty->szValue, "", INI_MAX_PROPERTY_VALUE );
    hLastProperty->pszHelp = malloc(strlen(help_strings[4]) + 1);
    strcpy(hLastProperty->pszHelp, help_strings[4]);
#endif
    
    hLastProperty->pNext = 
    (HODBCINSTPROPERTY)malloc( sizeof(ODBCINSTPROPERTY) );
    hLastProperty = hLastProperty->pNext;
    memset( hLastProperty, 0, sizeof(ODBCINSTPROPERTY) );
    hLastProperty->nPromptType = ODBCINST_PROMPTTYPE_LISTBOX;
    strncpy( hLastProperty->szName, "CODEPAGE", INI_MAX_PROPERTY_NAME );
    hLastProperty->pszHelp = malloc(strlen(help_strings[5]) + 1);
    strcpy(hLastProperty->pszHelp, help_strings[5]);
    hLastProperty->aPromptData=get_CODEPAGE_list();
    strncpy( hLastProperty->szValue, hLastProperty->aPromptData[0], INI_MAX_PROPERTY_VALUE );
    
    hLastProperty->pNext = 
    (HODBCINSTPROPERTY)malloc( sizeof(ODBCINSTPROPERTY) );
    hLastProperty = hLastProperty->pNext;
    memset( hLastProperty, 0, sizeof(ODBCINSTPROPERTY) );
    hLastProperty->nPromptType = ODBCINST_PROMPTTYPE_TEXTEDIT;
    strncpy( hLastProperty->szName, "PRIORITY", INI_MAX_PROPERTY_NAME );
    strncpy( hLastProperty->szValue, "", INI_MAX_PROPERTY_VALUE );
    hLastProperty->pszHelp = malloc(strlen(help_strings[6]) + 1);
    strcpy(hLastProperty->pszHelp, help_strings[6]);

    hLastProperty->pNext =                                   /* 30.10.04 ... */
        (HODBCINSTPROPERTY)malloc( sizeof(ODBCINSTPROPERTY) );
    hLastProperty = hLastProperty->pNext;
    memset( hLastProperty, 0, sizeof(ODBCINSTPROPERTY) );
    hLastProperty->nPromptType = ODBCINST_PROMPTTYPE_LISTBOX;
    strncpy( hLastProperty->szName, "BUFFERED_FETCH", INI_MAX_PROPERTY_NAME );
    hLastProperty->pszHelp = malloc(strlen(help_strings[7]) + 1);
    strcpy(hLastProperty->pszHelp, help_strings[7]);
    hLastProperty->aPromptData = get_YesNo_list();
    strcpy(hLastProperty->szValue,"No");                     /* ... 30.10.04 */

    return 1;
}

#endif
