# -*-Perl-*-
#/***************************************************************************
#*                 RELEX, Inc. Linter DBMS source code                      *
#*        This is unpublished propietary source code of RELEX, Inc.         *
#*     The copyright notice does not evidence any actual or intended        *
#*                  publication of such source code.                        *
#*                                                                          *
#*                 Copyright (c) 1990-2007 RELEX Inc.                       *
#*                                                                          *
#*                   This software was developed by                         *
#*                                                                          *
#*                            RELEX, Inc.                                   *
#*                         Voronezh, Russia                                 *
#*                                                                          *
#*                        All rights reserved.                              *
#****************************************************************************
# AUTHOR:  Andrew V Razuwaev
#----------------------------------------------------------------------------
# PURPOSE: Linter DBD
#----------------------------------------------------------------------------
# REVISION HISTORY:
#
# 13.06.00 (RAW) default server name in connect()
# 20.06.00 (RAW) parse DB name in connect()
# 29.08.00 (Galizin) rebind in execte correction
# 18.12.00 (RAW) store selected rows count by execute method and check it in fetch
# 21.12.00 (RAW) added sql types support (BIGINT, VARCHAR, VARBYTE)
# 03.04.01 (RAW) memory leak
# 05.04.01 (RAW) multiple blobs (lin5.8) (DBD ver 2.00 now)
# 24.03.03 (RAW) (BT#7962. fills 'rows' in the SP returned cursor)
# 01.10.04 (Gluhov) Fixed bug in function fetchrow_array
# 02.02.06 (Gluhov) BT#13383
# 13.02.06 (Gluhov) Translation of ESC sequences implemented
# 16.02.06 (Gluhov) Linter 6.x transaction isolation levels support
# 26.10.07 (Ermakov) Correct PROC cursor handling
# 18.05.10 (Glukhov) Using unnecessary statement's field removed
# 18.05.10A(Glukhov) Input params fixed in finish and getblob functions
# 20.05.10 (Glukhov) Using commit and rollback with exclusive mode
# 16.09.10 (Glukhov) Return value fixed in do function
# 22.09.10 (Glukhov) Getting database param in snap and shut functions fixed
# 22.09.10A(Glukhov) Calling internal functions in snap and shut functions fixed
# 22.09.10B(Glukhov) Commit can be called in autocommit mode
# 30.09.10 (Glukhov) Deprecated method call removed
# 23.06.11 (Galizin) By specifacation finish is used to inform the RDBMS about end of fetching. It should not delete the statement.
# 23.06.11a(Galizin) Destructor for statement
# 09.09.13 (Galizin) Deprecated warnings
# 18.02.14 (Pashentsev) Check possibly undefined variables
# 25.04.14 (Muratova) Correct info about types (linter's native types like sql types)  
#   You may distribute under the terms of either the GNU General Public
#   License or the Artistic License, as specified in the Perl README file.
#   This module based on  ExampleP.pm
#   Copyright (c) 1994,1997,1998 Tim Bunce;
#   DBD::Sybase  0.13
#   Copyright (c) 1996, 1997, 1998   Michael Peppler

$DBD::Linter::VERSION = '2.00';
{
    package DBD::Linter;

    use DBI qw(:sql_types);
    use DynaLoader ();

    @ISA = qw(DynaLoader);

    require_version DBI 1.02;

    @EXPORT = qw(); # Do NOT @EXPORT anything.

    bootstrap DBD::Linter;

#    @statnames = qw(dev ino mode nlink
#	uid gid rdev size
#	atime mtime ctime
#	blksize blocks name);
#    @statnames{@statnames} = (0 .. @statnames-1);

#    @stattypes = (SQL_INTEGER, SQL_INTEGER, SQL_INTEGER, SQL_INTEGER,
#	SQL_INTEGER, SQL_INTEGER, SQL_INTEGER, SQL_INTEGER,
#	SQL_INTEGER, SQL_INTEGER, SQL_INTEGER,
#	SQL_INTEGER, SQL_INTEGER, SQL_VARCHAR);
#    @stattypes{@statnames} = @stattypes;
# accordance between linter's native types and sql types
# not change!:
    @sqltypes=(SQL_CHAR,	SQL_INTEGER,	SQL_SMALLINT,
               SQL_DOUBLE,	SQL_FLOAT,	SQL_DATE,
               SQL_DECIMAL,	SQL_BINARY,	SQL_BLOB,
               SQL_WVARCHAR,	SQL_WCHAR,	SQL_ALL_TYPES,         # 21.12.00
               SQL_VARCHAR,	SQL_ALL_TYPES,	SQL_BIGINT,  
               SQL_VARBINARY,	SQL_ALL_TYPES,  SQL_ALL_TYPES);  # 25.04.14
 
    $drh = undef;	# holds driver handle once initialised

    sub err {return DBD::Linter::st::_errno();}
    sub errstr {return DBD::Linter::st::_errstr();}
    sub state {return DBD::Linter::st::_errstate();}

    sub driver{
	return $drh if $drh;
	my($class, $attr) = @_;
	$class .= "::dr";
#        $ctx = 0;
        my($major,$minor,$build);
        return undef if DBD::Linter::dr::_version(\$major,\$minor,\$build);
	($drh) = DBI::_new_drh($class, {'Name' => 'Linter',
					'Version'=>'Revision: 2.0', #05.04.01
					'VerMajor'=>$major,
					'VerMinor'=>$minor,
					'VerBuild'=>$build,
					'Attribution'=>'Linter 4.xx-6.xx DBD',
					},{}); #{'DBDctx'=>$ctx}
	$drh;
    }
    1;
}


{   package DBD::Linter::dr; # ====== DRIVER ======
    $imp_data_size = 0;
    use strict;

    sub err {return DBD::Linter::st::_errno();}
    sub errstr {return DBD::Linter::st::_errstr();}
    sub state {return DBD::Linter::st::_errstate();}

#    sub my_handler {
#	my($self, $type, @args) = @_;
#	return 0 unless $type eq 'ERROR';
#	${$self->{Err}}    = $args[0];
#	${$self->{Errstr}} = $args[1];
#	1;	# handled
#    }

# method connect : return dbh or undef if error
    sub connect {
        my($drh, $dbname, $user, $pass, $attr)= @_;
        my($server,$major,$minor,$build);
	$server = ($dbname =~ /^dbi:Linter:(\w+):/i)? $1:''; #20.06.00
        return undef if DBD::Linter::dr::_server_version($server,\$major,\$minor,\$build);
# check for compatible server and driver :
        return undef if DBD::Linter::dr::_ver_compat($drh->{'VerMajor'},$drh->{'VerMinor'},$major,$minor);
#set default modes if not defined
        my($mode)=0x0;
        my($trans)=0x0; # no ESCAPe translation
        my($CP)=0x0; # CP 866
        if (exists $attr->{'mode'}){
          if ($attr->{'mode'} =~ /autocommit/i){
           $mode=0x000;
          }
          elsif($attr->{'mode'} =~ /optimistic/i){
           $mode=0x100;
          }
          elsif($attr->{'mode'} =~ /exclusive/i){
           $mode=0x400
          }
          elsif($attr->{'mode'} =~ /read_committed/i){
           $mode=0x01000000;
          }
          elsif($attr->{'mode'} =~ /read_uncommitted/i){
           $mode=0x04000000;
          }
          elsif($attr->{'mode'} =~ /serializable/i){
           $mode=0x02000000;
          }
        }
        if (exists $attr->{'codepage'}) {
          if ($attr->{'codepage'} =~ /CP_866/i){
            $CP=0x0000;
          }elsif($attr->{'codepage'} =~ /CP_1251/i){
            $CP=0x4000;
          }elsif($attr->{'codepage'} =~ /KOI8/i){
            $CP=0x8000;
          }
        }
        if (exists $attr->{'translate_esc'}) {
          if ($attr->{'translate_esc'} == 1){
            $trans = 0x100000;
          }
        }
# now context not uses yet
# getting Linter connection
        $pass = '' unless defined $pass;
	my($connnum)=DBD::Linter::db::_connect($server,$user,$pass,$mode|$trans,$CP);#13.06.00
        return undef unless ($connnum >= 0);
#note: password no stores
        my($this) = DBI::_new_dbh($drh,{'Name'=>$dbname,
					'User'=>$user,
					'ConnNum'=>$connnum, # connection number
					'Mode'=>$attr->{'mode'},
					'Translate_Esc'=>$attr->{'translate_esc'},
					'Codepage'=>$attr->{'codepage'},
					},{});
        $this;
    }

    sub data_sources {
	return ("dbi:Linter:dir=.");	# possibly usefully meaningless
    }

    sub disconnect_all {
        DBD::Linter::dr::_disconnect_all();
    }

#direct inter() call
    sub inter{
	my($drh,$arg1,$arg2,$arg3,$arg4,$arg5)=@_;
	return undef unless defined $arg1 && defined $arg2 && defined $arg3 && defined $arg4 &&defined $arg5;
        return DBD::Linter::dr::_linter($arg1,$arg2,$arg3,$arg4,$arg5);
    }

    sub func{
    }

    sub DESTROY { undef }
}


{   package DBD::Linter::db; # ====== DATABASE ======
    $imp_data_size = 0;
    use strict;

    sub err {return DBD::Linter::st::_errno();}
    sub errstr {return DBD::Linter::st::_errstr();}
    sub state {return DBD::Linter::st::_errstate();}

    sub prepare {
	my($dbh, $statement, $attr)= @_;

        if (!defined $dbh->{'ConnNum'}){
          return undef;
        }

        my($prepattr)=undef;
        my($statnum)=DBD::Linter::st::_prepare($dbh->{'ConnNum'},$statement);
#print "\n in prepare statnum = $statnum";
#print "\n error num  = ",DBD::Linter::st::_errno();
        return undef unless ($statnum >=0);
        my $field_num= DBD::Linter::st::_fieldcnt($statnum);
        my $pcnt     = DBD::Linter::st::_paramcnt($statnum);
        my ($ret,$fields,$types) =
	  DBD::Linter::st::_titles($statnum, \@DBD::Linter::sqltypes);	# 03.04.01
        return undef if ($ret); #error

        return undef unless ($field_num>=0 && $pcnt >=0);
# register new statement
	my ($sth) = DBI::_new_sth($dbh,{'Statement'=>$statement,
					'StatNum'=>$statnum,
					'state'=>0,},{});
	$sth->{NAME} = $fields;					# 03.04.01
        $sth->{TYPE} = $types;
        $sth->{NULLABLE} = [ (0) x @$fields ] if(defined $fields);# 18.02.14 09.09.13 03.04.01
	$sth->{NUM_OF_FIELDS} = $field_num;
	$sth->{NUM_OF_PARAMS} = $pcnt;
	$sth->{'ConnNum'} = $dbh->{'ConnNum'};
	$sth;
    }


    sub type_info_all {
	my ($dbh) = @_;
	my $ti = [
	    {	TYPE_NAME	=> 0,
		DATA_TYPE	=> 1,
		PRECISION	=> 2,
		LITERAL_PREFIX	=> 3,
		LITERAL_SUFFIX	=> 4,
		CREATE_PARAMS	=> 5,
		NULLABLE	=> 6,
		CASE_SENSITIVE	=> 7,
		SEARCHABLE	=> 8,
		UNSIGNED_ATTRIBUTE=> 9,
		MONEY		=> 10,
		AUTO_INCREMENT	=> 11,
		LOCAL_TYPE_NAME	=> 12,
		MINIMUM_SCALE	=> 13,
		MAXIMUM_SCALE	=> 14,
	    },
	    [ 'VARCHAR', DBI::SQL_VARCHAR, undef, "'","'", undef, 0, 1, 1, 0, 0,0,undef,0,0 ],
	    [ 'INTEGER', DBI::SQL_INTEGER, undef, "","",   undef, 0, 0, 1, 0, 0,0,undef,0,0 ],
	];
	return $ti;
    }


    sub disconnect {
        my ($dbh, $action)= @_;
        return 0 unless defined $dbh;
        $action = 'rollback' unless (defined $action && $action =~ /commit/i);
	return (!DBD::Linter::db::_disconnect($dbh->{'ConnNum'}, $action));
    }

    sub commit {
      my $dbh=shift;
      return 0 unless (defined $dbh->{'ConnNum'});

      my $rv=DBD::Linter::db::_commit($dbh->{'ConnNum'}); # 22.09.10B
      if ($rv!=0){
        print "commit error $rv\n";                       # 30.09.10
        return 0;
      }else{
        return 1;
      }
    }

    sub rollback {
      my $dbh=shift;
      return 0 unless (defined $dbh->{'ConnNum'});
      if ($dbh->{'Mode'} !~ /(optimistic)|(exclusive)/i){ # 20.05.10
        print "Rollback error. Wrong connecion mode (not Optimistic or Exclusive)\n"; # 30.09.10
        return 0;
      }
      my $rv=DBD::Linter::db::_rollback($dbh->{'ConnNum'},'');
      if ($rv){
	print "rollback error $rv\n";                     # 30.09.10
        return 0;
      }else{
        return 1;
      }
    }

# force store data to disk
    sub snap {
     my $database = pop;                          # 22.09.10
     return (!DBD::Linter::db::_snap($database)); # 22.09.10A
    }

#remote shutdown
    sub shut {
     my $database = pop;                          # 22.09.10
     return (!DBD::Linter::db::_shut($database)); # 22.09.10A
    }

# makes for best processing's speed
    sub do {
      my ($dbh,$statement,$attr,@values)=@_;
      my $rv = DBD::Linter::db::_do($dbh->{'ConnNum'},$statement,$attr,\@values);
      if ($rv>=0){
       $rv = ($rv == 0) ? "0E0" : $rv;            # 16.09.10
       return $rv;
      }else{
       return undef;
      }
    }

#execute SP
    sub proc {
      my ($dbh, $name,@params) = @_;
      return undef unless defined $dbh && defined $name;
      my $sp='execute ';$sp .= $name; $sp .='(';
      if (defined $params[0]){			# 05.04.01
       my $cnt =scalar @params;
       if($cnt>1){$sp = 'execute '.$name.'('.('?,')x($cnt-1).'?);';}
       elsif ($cnt==1){$sp = 'execute '.$name.'(?);';}
       else {$sp = 'execute '.$name.'();';}
      }else{$sp = 'execute '.$name.'();';
      }
      my ($result,$isProc,$field_num,$fields,$types,$rv);
      $rv=DBD::Linter::st::_proc($dbh->{ConnNum},$sp,\@params,\@DBD::Linter::sqltypes,\$result,\$isProc,\$field_num, $fields, $types);
      if($rv>=0){
       if($isProc!=-1){                                           # 26.10.07
	my ($sth) = DBI::_new_sth($dbh, {'Statement'=>$sp,
					'StatNum'=>$result,
                                        'ConnNum' =>$isProc,
                                        'CursNum' =>$isProc,
					'rows'=>DBD::Linter::st::_rows($result),#24.03.03
					'state'=>2,},{});
	$sth->{NAME} = \@$fields;
	$sth->{TYPE} = \@$types;
	$sth->{NULLABLE} = [ (0) x @$fields ];
	$sth->{NUM_OF_FIELDS} = $field_num;
	$sth->{NUM_OF_PARAMS} = 0;
	return $sth;
	}
	return $result;
      }else{
       return undef;
      }
    }

    sub func {
    }

    sub native_sql {  #13.02.06
    my ($dbh, $stat) = @_;
	my $res = "";
    DBD::Linter::db::_nativesql($stat, \$res);
	return $res;
    }

    sub FETCH {
	my ($dbh, $attrib) = @_;
	# In reality this would interrogate the database engine to
	# either return dynamic values that cannot be precomputed
	# or fetch and cache attribute values too expensive to prefetch.
	return 1 if $attrib eq 'AutoCommit';
	# else pass up to DBI to handle
	return $dbh->DBD::_::db::FETCH($attrib);
    }

    sub STORE {
	my ($dbh, $attrib, $value) = @_;
	# would normally validate and only store known attributes
	# else pass up to DBI to handle
	if ($attrib eq 'AutoCommit') {
	    return 1 if $value;	# is already set
	    croak("Can't disable AutoCommit");
	}
	return $dbh->DBD::_::db::STORE($attrib, $value);
    }
    sub DESTROY { undef }
}


{   package DBD::Linter::st; # ====== STATEMENT ======
    $imp_data_size = 0;
    use strict; no strict 'refs'; # cause problems with filehandles

    sub err {return DBD::Linter::st::_errno();}
    sub errstr {return DBD::Linter::st::_errstr();}
    sub state {return DBD::Linter::st::_errstate();}

    sub bind_param {
	my($sth, $param, $value, $paramtyp) = @_;
        return 0 unless defined $sth->{'StatNum'};
        my($typ) = -1;

        my($rv)=DBD::Linter::st::_bind($sth->{'StatNum'},$param,
                  $value,\$sth->{'dbd_length'}->[$param]); # 18.05.10
        return (!$rv);
    }

    sub execute {
	my($sth, @dir) = @_;
	my $dir;
#already executed
#        return 1 unless ($sth->{'state'} <= 0);
# statement not prepared
        return undef unless (defined $sth->{'state'} && $sth->{'state'} >=0);

	if (@dir) {
	    my ($cnt)= scalar(@dir);

            my($fcnt)= DBD::Linter::st::_paramcnt($sth->{'StatNum'});
            unless ($fcnt>=0){
             print "Invoke LINDBD_paramcnt error";                           # 30.09.10
             return undef;
            }
            unless ($cnt == $fcnt){
             print "Actual paramater count not equal formal paramter count"; # 30.09.10
             return undef;
            }
            while(--$cnt>=0){                                       # 18.05.10
                my($rv)=DBD::Linter::st::_bind($sth->{'StatNum'},$cnt+1,
                      $dir[$cnt],\$sth->{'dbd_length'}->[$cnt]);
             return undef unless (!$rv);
            }
	} #else {                                                   # 28.03.10
#            my($cnt) = DBD::Linter::st::_paramcnt($sth->{'StatNum'});
#            unless ($cnt>=0){
#             $sth->event("ERROR", 2, "Invoke LINDBD_paramcnt error");
#             print "\nInvoke LINDBD_paramcnt error\n";
#             return undef;
#            }
#            while(--$cnt>=0){
#                my($rv)=DBD::Linter::st::_bind($sth->{'StatNum'},$cnt+1,
#                      $sth->{'dbd_param'}->[$cnt+1],\$sth->{'dbd_length'}->[$cnt+1]); # 29.08.00
#             return undef unless (!$rv);
#            }
#	}
        my ($rv, $curs)=DBD::Linter::st::_execute($sth->{'StatNum'});# 03.04.01
#print "   cursor num ->$curs<-\n";
        $sth->{'CursNum'} = $curs;
        $sth->{'rows'} = $rv; # 18.12.00
        if ($rv>=0){$sth->{'state'} = 1;
                    ($rv == 0) ? "0E0" : $rv;
        }else{
          return undef;
        }
    }

# fetch row
    sub fetchrow_arrayref {
     my($sth, $attr) = @_;
     if ($sth->{'state'} < 1){
	print "Statement not executed";                              # 30.09.10
      return undef;
     }
     if ($sth->{'rows'} <=0){ #18.12.00
      return undef;
     }
     my(@dir);
     $dir[0]='GETN';
     if (exists $attr->{'direction'}&& defined $attr->{'direction'}){
      if($attr->{'direction'} =~ /next/i){
        $dir[0]='GETN';
      }
      elsif($attr->{'direction'} =~ /prev/i){
        $dir[0]='GETP';
      }
      elsif($attr->{'direction'} =~ /first/i){
        $dir[0]='GETF';
      }
      elsif($attr->{'direction'} =~ /last/i){
        $dir[0]='GETL';
      }
      elsif($attr->{'direction'} =~ /number/i){
        if (exists $attr->{'number'} && defined $attr->{'number'}){
         $dir[0]='GETS';
         $dir[1]=$attr->{'number'};
        }
      }
     }
     my ($data)=DBD::Linter::st::_fetch($sth->{'StatNum'},$sth->{'ConnNum'},\@dir,$sth->{'state'});
     if(!DBD::Linter::st::_errno()){
       $sth->{'state'}=2;
       if (defined $data){                                    # 18.02.14 09.09.13
        $sth->_set_fbav(\@$data);
        return $data;
       }
      }
      return undef;
    }

    *fetch = \&fetchrow_arrayref;

    sub fetchrow_array{
      my($sth, $attr) = @_;
#print "\nattr=", %$attr;
      my $ary_ref=fetchrow_arrayref(@_);
#      return @$ary if (defined @$ary);  # 01.10.04 ...
#      return undef;
      my @ary=@$ary_ref;
      return @ary[0..$#ary]               # ... 01.10.04 , 02.02.06
    }

    sub rows { my($sth) = @_;
#print "->", $sth->{'StatNum'}, "<-\n";
      return DBD::Linter::st::_rows($sth->{'StatNum'});
    }

    sub finish {
	my $sth = shift;
        return 0 unless (defined $sth->{'state'});
#        return 1 unless ($sth->{'state'} >=1);		# 03.04.01
        my ($rv)=DBD::Linter::st::_finish($sth->{'StatNum'},$sth->{'ConnNum'}); # 18.05.10A
#        $sth->{'ConnNum'}=undef;   # 23.06.11 18.05.10A
#        $sth->{'state'}=-1;
	return (!$rv);
    }

    sub blob_read {
      my($sth,$field, $offset, $len, $blobref)=@_;
      unless ($sth->{'state'}==2){
print "\n blob_read aborted. fetch not processed...";
        return undef;
      }
      my @a =($offset,$len);
      if (ref $blobref){
       $$blobref = DBD::Linter::st::_getblob($sth->{'StatNum'},$sth->{'ConnNum'},$field,\@a); #05.04.01 18.05.10A
       return $$blobref;
      }else{
        return DBD::Linter::st::_getblob($sth->{'StatNum'},$sth->{'ConnNum'},$field,\@a); #05.04.01 18.05.10A
      }
    }

    sub blob_copy_to_file {	# returns length or undef on error
	my($self, $field, $filename_or_handleref, $blocksize) = @_;
	my $fh = $filename_or_handleref;
	my($len, $buf) = (0, "");
	$blocksize ||= 1024;	# not too ambitious
	local(*FH);
	unless(ref $fh) {
	    open(FH, ">$fh") || return undef;
            binmode FH;
	    $fh = \*FH;
	}
	while(defined($self->blob_read($field, $len, $blocksize, \$buf))) {
	    syswrite $fh, $buf, length $buf;
	    $len += length $buf;
	}
	close(FH);
	$len;
    }

    sub blob_write {
      my($sth,$field,$len,$blob)=@_;
      unless ($sth->{'state'}==2){
print "\n blob_write aborted. fetch not processed...";
        return 0;
      }
      unless (defined $len && $len>0 && $len<4080 ){
print "\n blob_write aborted. wrong parameter's list...";
      }
      return (!DBD::Linter::st::_addblob($sth->{'StatNum'},$sth->{'CursNum'},$field,$len,$blob));# 05.04.01
    }

    sub blob_delete {
      my($sth,$field,@attr)=@_;
      unless ($sth->{'state'}==2){
print "\n blob_delete aborted. fetch not processed...";
        return 0;
      }
      return (!DBD::Linter::st::_delblob($sth->{'StatNum'},$sth->{'CursNum'},$field,\@attr));# 05.04.01
    }

#call native Linter's methods
    sub func {
    }

#    sub FETCH {
#	my ($sth, $attrib) = @_;
#	# In reality this would interrogate the database engine to
#	# either return dynamic values that cannot be precomputed
#	# or fetch and cache attribute values too expensive to prefetch.
#	if ($attrib eq 'TYPE'){
#	    my @t = @DBD::Linter::stattypes{@{$sth->{NAME}}};
#	    return \@t;
#	}
#	# else pass up to DBI to handle
#	return $sth->DBD::_::st::FETCH($attrib);
#    }

    sub STORE {
	my ($sth, $attrib, $value) = @_;
	return $sth->{$attrib}=$value
          if $attrib eq 'NAME' or $attrib eq 'NULLABLE' or $attrib eq 'TYPE' or $attrib eq 'CursNum' or $attrib eq 'ConnNum';
	return $sth->DBD::_::st::STORE($attrib, $value);
    }

    sub DESTROY
    {
      my($sth) = @_;
      my($ret) = DBD::Linter::st::_freestmt($sth->{'StatNum'}); #23.06.11
      undef
    }
}

1;
