 /***************************************************************************
 *                 RELEX, Ltd. Linter DBMS source code                      *
 *        This is unpublished propietary source code of RELEX, Ltd.         *
 *     The copyright notice does not evidence any actual or intended        *
 *                  publication of such source code.                        *
 *                                                                          *
 *                 Copyright (c) 1990-1999 RELEX Ltd.                       *
 *                                                                          *
 *                   This software was developed by                         *
 *                                                                          *
 *                            RELEX, Ltd.                                   *
 *                         Voronezh, Russia                                 *
 *                                                                          *
 *                        All rights reserved.                              *
 ****************************************************************************
 AUTHOR:  Andrew V Razuwaev
 ----------------------------------------------------------------------------
 PURPOSE: errors & exceptions management 
 ----------------------------------------------------------------------------
 REVISION HISTORY:
 11.06.00 -- Ermakov (Can't write in string constant area.)
*/

#ifdef UNIX
#include <signal.h>
#endif
#include "lindbd.h"

#define iabs(x) ((x>=0)? x:-x)
void __llasgmul()
{
}

void __llasgrems()
{
}
void __llasgdivs()
{
}
void Except(L_LONG cause)
{
 if (cause==ERR_NOMEM){
  LINDBD_disconnect_all();
#ifdef WIN32
  RaiseException(STATUS_NO_MEMORY,EXCEPTION_NONCONTINUABLE,NULL,NULL);
#endif
#ifdef UNIX
  raise(SIGTERM);
#endif
 }else
 {
  LINDBD_disconnect_all();
#ifdef WIN32
  RaiseException(STATUS_NONCONTINUABLE_EXCEPTION,EXCEPTION_NONCONTINUABLE,NULL,NULL);
#endif
#ifdef UNIX
  raise(SIGTERM);
#endif
 }
}

L_CHAR*textbycode(L_LONG code, L_CHAR*attr)
{
 L_CHAR *str;
 static L_CHAR VolatailStr[sizeof(Msg_LINERR) + 1];		/* 11.06.00 */

 switch(code)
 {
   case ERR_NOCTH:        str=Msg_NOCTH;      break; /* invalid context number */
   case ERR_NODBH:        str=Msg_NODBH;      break; /* invalid connetion number */
   case ERR_NOSH:         str=Msg_NOSH;       break; /* invalid statement number */
   case ERR_NOTCURSOR:    str=Msg_NOTCURSOR;  break; /* this channel is not cursor */
   case ERR_NOTPREPARED:  str=Msg_NOTPREPARED;break; /* statement is not prepared */
   case ERR_INVIDX:       str=Msg_INVIDX;     break; /* invalid bind parameter's number */
   case ERR_NOREF:        str=Msg_NOREF;      break; /* parameter not defined */
   case ERR_INVTYP:       str=Msg_INVTYP;     break; /* invalid parameter's type */
   case ERR_INVVAL:       str=Msg_INVVAL;     break; /* invalid parameter's value */
   case ERR_EXEPROC:      str=Msg_EXEPROC;    break; /* can not execute procedure by execute() */
   case ERR_NOTMAINCH:    str=Msg_NOTMAINCH;  break; /* this channel is cursor */
   case ERR_CORRUPT:      str=Msg_CORRUPT;    break; /* internal structures corrupted */
   case ERR_SPNOTSUPP:    str=Msg_SPNOTSUPP;  break; /* SP call not supports */
   case ERR_NOCOMPAT:     str=Mag_NOCOMPAT;   break; /* non-compatible versions*/
   case ERR_NONSELECT:    str=Msg_NONSELECT;  break; /* non-select statement was executed */

   case NOERR:            str=Msg_NOERR;      break;
   case -2:
   case NOTFOUND:         str=Msg_NOTFOUND;   break;
/* linter errors : */
   default:               str = VolatailStr;			/* 11.06.00 */
                          strcpy(VolatailStr,Msg_LINERR);
                          sprintf(&str[strlen(Msg_LINERRPATT)],"%.8ld",iabs(code));
   break;
 }
 return str;
}

/*
 get standard SQLSTATE code
*/
L_CHAR*get_sqlstate(L_LONG err)
{
 return "000000";
}

/**************************** source end **************************************/
