 /***************************************************************************
 *                 RELEX, Inc. Linter DBMS source code                      *
 *        This is unpublished proprietary source code of RELEX, Inc.        *
 *     The copyright notice does not evidence any actual or intended        *
 *                  publication of such source code.                        *
 *                                                                          *
 *                 Copyright (c) 1990-2013 RELEX Inc.                       *
 *                                                                          *
 *                   This software was developed by                         *
 *                                                                          *
 *                            RELEX, Inc.                                   *
 *                         Voronezh, Russia                                 *
 *                                                                          *
 *                        All rights reserved.                              *
 ****************************************************************************
 AUTHOR:  Andrew V Razuwaev
 ----------------------------------------------------------------------------
 PURPOSE: memory management procedures
 ----------------------------------------------------------------------------
 REVISION HISTORY:
  03.04.01 -- (RAW) Memory leak
  28.09.05 -- (FLY) Type of result changed
  15.02.06 -- (Gluhov) Correct argument checking
  14.03.06 -- (Gluhov) WARNINGS
  17.08.11 -- (DV)  Remove 'l' from formats
  09.09.13 -- Galizin (calloc redefine is absent in perl)
*/

#define _DBDMEM_
#include "lindbd.h"

static L_LONG initialized=0; /* was binit()? */

static DBDCONTEXT    *ctxlist;  /* context    list */
static DBDCONNECTION *connlist; /* connetions list */
static DBDSTATEMENT  *statlist; /* statements list */

static L_LONG       ectx;  /*  allowed entry point for make new context */
static L_LONG       econn; /*  allowed entry point for make new connection */
static L_LONG       estat; /*  allowed entry point for make new statement  */

/* values for default; may change by call LINDBD_init() */
static L_LONG       max_conn =_MAX_CONN; /* 32 by default */
static L_LONG       max_stat =_MAX_STAT; /* 32 by default */
static L_LONG       max_ctx  =_MAX_CTX;  /* 1/32 by default */

#if defined DEBUG_MODE && defined CSUM
static L_LONG connsum=0;
/*
 calc check sum 
*/
static L_LONG csum(L_CHAR*s,L_LONG l)
{L_LONG i=0,v=0;
 for(;i<l;i+=sizeof(DBDCONNECTION))
 {
  v ^= *((L_LONG*)(&s[i]));
  v ^= *((L_LONG*)(&s[i+4]));
 }
 return v; 
}
#endif


/*
 descriptor list commit 
*/
static void _bcommit(L_LONG lst, L_LONG n)              /* 28.09.05 */
{
L_CHAR*clist;
L_LONG i;
L_LONG siz;
L_LONG*entry;

 if (lst == CTX_LIST) /* context list */
 {
  clist = (L_CHAR*)ctxlist;
  entry= &ectx;
  siz = sizeof(DBDCONTEXT);
  i=max_ctx;
 }
 else if  (lst == CONN_LIST) /* connection list */
 {
  clist = (L_CHAR*)connlist;
  entry= &econn;
  siz = sizeof(DBDCONNECTION);
  i=max_conn;
 }
 else /* statement list*/
 {
  clist = (L_CHAR*)statlist;
  entry= &estat;
  siz = sizeof(DBDSTATEMENT);
  i=max_stat;
 }
 for (; i<n; i++)
 {
  if (*(L_LONG*)(clist+siz*i) == -1)  /* block not commited*/
  {
   memset(clist+siz*i, 0x0, siz);
   if (*entry == -1)
    *entry = i;

   if (i<n-1) /* non last block */
    *(L_LONG*)(clist+siz*i) = i+1;
   else
    *(L_LONG*)(clist+siz*i) = -1;

   *(L_LONG*)(clist+siz*i+sizeof(L_LONG)) = 0; /* block is free */
  }
 }
}

/*
 block list expand
*/
static L_CHAR*  _bexpand(L_LONG lst)
{
L_CHAR*clist;
L_CHAR*ptr;
L_LONG *entry;
L_LONG *max_entry;
L_LONG new_max_entry;
L_LONG siz;

 if (lst == CTX_LIST) /* context list */
 {
  clist= (L_CHAR*)ctxlist;
  entry=&ectx;
  siz = sizeof(DBDCONTEXT);
  max_entry = &max_ctx;
  new_max_entry= max_ctx+_MAX_CTX;
 }
 else if  (lst == CONN_LIST) /* connection list */
 {
  clist= (L_CHAR*)connlist;
  entry=&econn;
  siz = sizeof(DBDCONNECTION);
  max_entry = &max_conn;
  new_max_entry= max_conn+_MAX_CONN;
 }
 else /* statement list*/
 {
  clist= (L_CHAR*)statlist;
  entry=&estat;
  siz = sizeof(DBDSTATEMENT);
  max_entry = &max_stat;
  new_max_entry= max_stat+_MAX_STAT;
 }

 if (new_max_entry <=0) /* too many entries */
  Except(ERR_NOMEM);

 ptr = MYMEM_CALLOC (new_max_entry*siz, sizeof(L_CHAR));		/* 03.04.01 */
 if (!ptr) /* no memory */
  Except(ERR_NOMEM);

 memset(&ptr[*max_entry*siz], 0xFF,(new_max_entry - *max_entry)*siz); /* set 'uncommit' for new blocks */
 memcpy(ptr, clist, *max_entry*siz); /* copy blocks to new location  */
 MYMEM_FREE(clist);

 if (lst == CTX_LIST) /* context list */
  ctxlist = (DBDCONTEXT*)ptr;
 else if  (lst == CONN_LIST) /* connection list */
  connlist= (DBDCONNECTION*)ptr;
 else /* statement list*/
  statlist= (DBDSTATEMENT*)ptr;

/* commit allocated blocks */
 _bcommit(lst, new_max_entry);
 *max_entry=new_max_entry;
 return ptr;
}

/*
 allocate new descriptor 
*/
static L_LONG _balloc(L_LONG lst)
{
 L_LONG siz;
 L_LONG d;
 L_CHAR*clist;
 L_LONG *entry;

 if (lst == CTX_LIST) /* context list */
 {
  clist= (L_CHAR*)ctxlist;
  siz = sizeof(DBDCONTEXT);
  entry=&ectx;
 }
 else if  (lst == CONN_LIST) /* connection list */
 {
  clist= (L_CHAR*)connlist;
  siz = sizeof(DBDCONNECTION);
  entry=&econn;
 }
 else /* statement list*/
 {
  clist= (L_CHAR*)statlist;
  siz = sizeof(DBDSTATEMENT);
  entry=&estat;
 }
 if (*entry == -1)
 {
  clist = _bexpand(lst);
 }
 d = *entry;
#if defined DEBUG_MODE && defined DEBUG_DBHNUM
printf ("\n allocate block : %d",d);
printf ("\n next free block : %d", *(L_LONG*)(clist+d*siz));
#endif
 *entry = *(L_LONG*)(clist+d*siz); /*->next*/
 *(L_LONG*)(clist+d*siz+sizeof(L_LONG))=1; /*->status*/

#if defined DEBUG_MODE && defined DEBUG_DBHNUM && defined CSUM
 connsum = csum(connlist, sizeof(DBDCONNECTION)*max_conn);
#endif
 return d;
}

/*
 free allocated descriptor
*/
static L_LONG _bfree(L_LONG lst, L_LONG d)
{
 L_LONG siz;
 L_CHAR*clist;
 L_LONG *entry;

 if (lst == CTX_LIST) /* context list */
 {
  clist= (L_CHAR*)ctxlist;
  siz = sizeof(DBDCONTEXT);
  entry=&ectx;
 }
 else if  (lst == CONN_LIST) /* connection list */
 {
  clist= (L_CHAR*)connlist;
  siz = sizeof(DBDCONNECTION);
  entry=&econn;
 }
 else /* statement list*/
 {
  clist= (L_CHAR*)statlist;
  siz = sizeof(DBDSTATEMENT);
  entry=&estat;
 }
 if (*(L_LONG*)(clist+d*siz+sizeof(L_LONG))==0) /* free block*/
  return 0;

#if defined DEBUG_MODE && defined DEBUG_DBHNUM
printf ("\n(free) free block : %d",d);
printf ("\n(free) next free block : %d",(*entry));
#endif
 *(L_LONG*)(clist+d*siz) =*entry;
 *(L_LONG*)(clist+d*siz+sizeof(L_LONG)) =0;
 *entry = d;

#if defined DEBUG_MODE && defined DEBUG_DBHNUM  && defined CSUM
 connsum = csum(connlist, sizeof(DBDCONNECTION)*max_conn);
#endif
 return 1;
} 

/*
 lists initilaize
*/
static L_LONG _binit(void)
{
 L_LONG nconn, nctx, nstat;
 ectx = econn = estat = -1; /* undefined entrypoint's number */

 ctxlist  = MYMEM_CALLOC(max_ctx*sizeof(DBDCONTEXT), sizeof(L_CHAR));	/* 03.04.01 */
 connlist = MYMEM_CALLOC(max_conn*sizeof(DBDCONNECTION), sizeof(L_CHAR));
 statlist= MYMEM_CALLOC(max_stat*sizeof(DBDSTATEMENT), sizeof(L_CHAR));
 if (!ctxlist || !connlist || !statlist)
  Except(ERR_NOMEM);

 memset(ctxlist, 0xFF, max_ctx*sizeof(DBDCONTEXT));
 memset(connlist, 0xFF, max_conn*sizeof(DBDCONNECTION));
 memset(statlist, 0xFF, max_stat*sizeof(DBDSTATEMENT));

 nconn =max_conn;
 nstat =max_stat;
 nctx  =max_ctx;
 max_ctx =0;
 max_conn=0;
 max_stat=0;

 _bcommit(CTX_LIST,nctx);
 _bcommit(CONN_LIST,nconn);
 _bcommit(STAT_LIST,nstat);

 max_conn=nconn;
 max_stat=nstat;
 max_ctx=nctx;
 return 1;
}

/*
 lists free
*/
static L_LONG _bdeinit(void)
{
 if (ctxlist)
  MYMEM_FREE((L_CHAR*)ctxlist);
 if (connlist)
  MYMEM_FREE((L_CHAR*)connlist);
 if (statlist)
  MYMEM_FREE((L_CHAR*)statlist);
 connlist = NULL;
 statlist = NULL;
 ctxlist = NULL;
 return 0;
}


static DBDCONNECTION*_getconn(L_LONG dbh)
{
#if defined DEBUG_MODE && defined DEBUG_DBHNUM && defined CSUM
 if (csum(connlist, sizeof(DBDCONNECTION)*max_conn) != connsum)
  printf("\ncheck sum error");
#endif
 if (dbh <0 || dbh >=max_conn)                       /* 15.02.06 */
  return 0;
 if (connlist[dbh].status==0)
 {
#if defined DEBUG_MODE && defined DEBUG_DBHNUM
printf ( "\nbad connectin number: %d",dbh);
printf(  "\nconnlist[%d].status: %d", dbh,connlist[dbh].status); 
printf(  "\nconnlist[%d].next: %d", dbh,connlist[dbh].next); 
#endif
  return 0;
 }
 else
  return &connlist[dbh];
}

static DBDCONTEXT*_getctx(L_LONG cth)
{
#if defined DEBUG_MODE && defined DEBUG_DBHNUM && defined CSUM
 if (csum(connlist, sizeof(DBDCONNECTION)*max_conn) != connsum)
  printf("\ncheck sum error");
#endif
 if (cth <0 || cth >=max_ctx)                            /* 15.02.06 */
  return 0;
 if (ctxlist[cth].status==0)
  return 0;
 else
  return &ctxlist[cth];
}

static DBDSTATEMENT*_getstat(L_LONG sh)
{
#if defined DEBUG_MODE && defined DEBUG_DBHNUM && defined CSUM
 if (csum(connlist, sizeof(DBDCONNECTION)*max_conn) != connsum)
  printf("\ncheck sum error");
#endif
 if (sh <0 || sh >=max_stat)                                /* 15.02.06 */
  return 0;
 if ( statlist[sh].status==0)
  return 0;
 else
  return &statlist[sh];
}

/*get chlld connections */
static L_LONG _childconn(L_LONG dbnp, L_LONG dbn)
{
 DBDCONNECTION*conn;
 if (dbn <0 || dbn >=max_conn)
  return -1;

 conn = &connlist[dbn];
 if (conn->parent==-1) /* not child connection */
  return -1;

 for(;dbn<max_conn;dbn++)
 {
  conn = &connlist[dbn];
  if (dbnp >=0 && conn->parent == dbnp && conn->status ==1)
    return dbn;
 }
 return -1;
}

/* get next allocated channel */
L_LONG _nextconn(L_LONG dbn)
{
 if (dbn <0 || dbn >=max_conn)
  return -1;

 for(;dbn<max_conn;dbn++)
 {
  if (connlist[dbn].status ==1)
    return dbn;
 }
 return -1;
}

/*
 lists allocate
*/
L_LONG binit(void)
{
L_LONG d=-1;
 inimutex();
 mutex();
 if (!initialized)
 {  _binit();
    initialized=1;
    d = balloc(CTX_LIST);  /* initialize main context */
 }
 unmutex();
 return d;
}

/*
 lists deallocate
*/
L_LONG bdeinit(void)
{
L_LONG d=0;
 mutex();
 if (initialized)
 {
  d=_bdeinit();
  initialized=0;
 }
 unmutex();
 return d;
}

/*
 block (connection, cursor, statement) allocate 
*/
L_LONG balloc(L_LONG lst)
{
L_LONG d;
 mutex();
  d=_balloc(lst);
 unmutex();
 return d;
}

/*
 block  free 
*/
L_LONG bfree(L_LONG lst, L_LONG d)
{
L_LONG rc;
 mutex();
 rc =_bfree(lst,d);
 unmutex();
return rc;
}

DBDCONNECTION*getconn(L_LONG dbh)
{
DBDCONNECTION*adr;
#if defined DEBUG_MODE && defined  DEBUG_DBHNUM
printf("\nconnection number: %d",dbh);
#endif
mutex();
 adr=_getconn(dbh);
unmutex();
return adr;
}

DBDCONTEXT*getctx(L_LONG cth)
{
DBDCONTEXT*adr;
#if defined DEBUG_MODE && defined  DEBUG_CTHNUM
printf("\ncontext number: %d",cth);
#endif
mutex();
 adr=_getctx(cth);
unmutex();
return adr;
}

DBDSTATEMENT*getstat(L_LONG sh)
{
DBDSTATEMENT*adr;
#if defined DEBUG_MODE && defined  DEBUG_SHNUM
printf("\nstatement number: %d",sh);
#endif
mutex();
adr=_getstat(sh);
unmutex();
return adr;
}

L_LONG childconn(L_LONG dbnp, L_LONG dbn)
{
 L_LONG n;
 mutex();
 n= _childconn(dbnp, dbn);
 unmutex();
 return n;
}

L_LONG nextconn(L_LONG dbn)
{
 L_LONG n;
 mutex();
 n= _nextconn(dbn);
 unmutex();
 return n;
}

void bfreeall(L_LONG lst)
{
}

void* LINDBD_calloc(                                        /* 09.09.13 ... */
#ifdef MEM_DEBUG
                    L_CHAR*file, L_LONG line,
#endif
                   int n, int size)
{                                                          /* 03.04.01 */
  void* m = malloc(n * size);

#ifdef MEM_DEBUG
  printf("file: %s line:%d (%p)=calloc(%d,%d)\n",file, line, m, n, size);
#endif
  if (m)
    memset(m, 0, n * size);                                  /* ... 09.09.13 */
  return m;
}
/**************************** source end **************************************/
