 /***************************************************************************
 *                 RELEX, Inc. Linter DBI driver source code                *
 *        This is unpublished proprietary source code of RELEX, Inc.        *
 *     The copyright notice does not evidence any actual or intended        *
 *                  publication of such source code.                        *
 *                                                                          *
 *                 Copyright (c) 1990-2014 RELEX Inc.                       *
 *                                                                          *
 *                   This software was developed by                         *
 *                                                                          *
 *                            RELEX, Inc.                                   *
 *                         Voronezh, Russia                                 *
 *                                                                          *
 *                        All rights reserved.                              *
 ****************************************************************************
 AUTHOR:  Andrew V Razuwaev
 ----------------------------------------------------------------------------
 PURPOSE: DBD entry points  
 ----------------------------------------------------------------------------
 REVISION HISTORY:

 17.03.99  - RAV (check for nonSELECT queries in fieldcnt)
 16.12.99  - Kononyhin (set crName before SETO calling)
 22.11.00  - RAW (Varchar , Varbyte support for Linter >= 5.7)
 06.12.00  - Galizin ( LnBufRow ...)
 21.12.00  - RAW (L_LONG support; BOOLEAN support)
 27.12.00  - RAW (MAX_ID_LEN instead directly given length)
 03.04.01  - RAW (memory leak)
 05.04.01  - RAW (multiple blobs)
 05.04.01  - RAW (EXTFILE type support)
 21.09.01  - Galizin ( remove comments in ++ stile)
 30.07.02  - Galizin ( Remove compiler warning PT#1046)
 24.03.03  - RAW (BT#7962. SP calling; tab deleted; some optimization)
             (memory leak while SP calling; fetch. Initialization of the stack variables.)
 31.03.03  - RAW (BT#8075. Filling the output parameters in the SP)
 01.10.04  - Gluhov,Ermakov (fixed bug in LINDBD_titles column name length)
 15.10.04  - Gluhov ( NCHAR & NVARCHAR support added,
                      corercted VAR types length  )
 03.11.04  - Gluhov (continue - used utf8 flag)
 11.02.05  - Gluhov (fixed bug in LinDBD_Execute with error 1009) 
 11.02.05A - FBV, Ermakov (Added min definition)
 30.11.05  - Gluhov ( fixed bug with binded parameters in procedures )
 01.02.06 -- Boris Fedorov (Corrected date format)
 13.02.06 -- Gluhov (Translation of ESC sequences implemented)
 16.02.06 -- Gluhov ( Linter 6.x transaction isolation levels support )
 31.02.06 -- Gluhov (Work with BLOBs)
 13.06.06 -- Ermakov (removed s_esc.h)
 28.10.06 -- Ermakov (Remove warnings)
 28.03.10 -- Glukhov (Params bind remake)
 06.04.10 -- Glukhov (Using MYMEM_CALLOC allocation memory macros added)
 06.04.10A-- Glukhov (Inserting/updating binded blobs error fixed)
 06.04.10B-- Glukhov (Binded param number bug fixed)
 18.05.10 -- Glukhov (Get param value from SV acording with SvType fixed) 
 21.06.10 -- Glukhov (Memory allocated for LINNLS_Ucs2Utf8 string size fixed)
 24.06.10 -- Glukhov (Rollback and commit return value fixed)
 14.07.10 -- Glukhov (Setting codepage fixed)
 14.07.10A-- Glukhov (Using special PrzExe flags removed)
 20.08.10  - Ermakov (Correct buffer size)
 25.08.10 -- Glukhov (Using internal answer description added)
 20.09.10 -- Glukhov (Blob length during blob adding bug fixed)
 20.09.10A-- Glukhov (Returning affected rows count on non-select query added)
 23.06.11 -- Galizin (Destructor for statement)
 23.06.11a-- Galizin (return 0 on _Bind always)
 23.06.11b-- Galizin (Freeing only answer resourses in finish)
 17.08.11  - DV      (Remove 'l' from formats)
 24.11.11  - DV      (Perl version check - copied from linperl.c)
 19.06.12  - Ermakov (Warning corrected)
 14.02.13  - Parkhomenko (Correct value type checking)
 09.09.13  - Galizin (DT_DEFAULT definition)
 13.11.14  - Kononov (fixed output empty VARCHAR value when constants exist in the select query)
 14.11.14 -- Kononov (removed excess symbol in BIGINT value)
 17.11.14 -- Kononov (fixed overwriting memory of null mask buffer)
 19.11.14 -- Kononov (removed NUL characters in EXTFILE value)
 20.11.14 -- Kononov (fixed return BLOB value for Perl > 5.6)
 21.11.14 -- Kononov (fixed support for Perl >= 5.19.x)
*/

#define _LINDBD_

#include "lindbd.h"
#include "decimals.h"
#include "tick.h"
#include "lnlsutf.h"
#if !defined(DT_DEFAULT)
#define DT_DEFAULT -1                                       /* 09.09.13 */
#endif

/*
#include "s_esc.h"
*/
int ReplaceEscapeSequences(char *SrcPtr, unsigned short SrcLen,  /* 13.06.06 */
                           char *ResPtr, unsigned short *ResLen, 
                           unsigned short ResMaxLen);

#ifndef min
#  define min(a, b) (((a) < (b)) ? (a) : (b))              /* 11.02.05A */
#endif

#define DATE_FORMAT "DD.MM.YYYY:HH:MI:SS.FF"               /* 01.02.06 */
#define BLOB_PORTION_SIZE 4048

/****************************** local methods *********************************/
/*
 sql types equals linter's native types
*/
static L_LONG _sqltype(L_LONG typ, L_LONG len)
{
 switch(typ)
 {
    case DT_CHAR:
      return 0; /* SQL_CHARACTER*/
    break;
    case DT_INTEGER:
     if(len==4)
       return 1; /* SQL_INTEGER */
     else if (len == 8)
       return 14; /* SQL_BIGINT */
     else 
       return 2; /* SQL_SMALLINT*/
    break;
    case DT_REAL:
     if(len==8)
      return 3; /*SQL_DOUBLE*/
     else
      return 4;/*SQL_FLOAT*/
    break;
    case DT_DATE:
      return 5; /* SQL_DATETIME */
    break;
    case DT_DECIMAL:
      return 6; /* SQL_DECIMAL */
    break;
    case DT_BYTE: /* tBYTE*/
      return 7; /* SQL_BIT */
    break;
    case DT_BLOB: /* tBLOB*/
      return 8; /* SQL_BLOB */
    break;
    case DT_VARCHAR:
      return 12; /* SQL_CHARACTER_VARYING_TYP*/
    case DT_VARBYTE:
      return 15; /* SQL_BIT_VARYING_TYP*/
    case DT_BOOL:     /* BOOLEAN */                /* 21.12.00 */
      return 1;  /* return SQL_INTEGER */
    case DT_EXTFILE:   
      return 0;   /* SQL_CHARACTER */
    case DT_NCHAR:                               /* 15.10.04 ... */
      return -8;  /* SQL_WCHAR */
    case DT_NVARCHAR:
      return -9;  /* SQL_WVARCHAR*/           /* ... 15.10.04 */
    default:
      return 16; /* UNDEFINED TYPE */         /* 21.12.00 */
 }
}

/*
sql statement class 
*/
static L_WORD _opertype(L_CHAR*tq)
{
L_WORD iOperType;
 iOperType = *((L_WORD*)&tq[4]);
 return iOperType;
}

/*
 set and pass thru error codes 
*/
static L_LONG _seterr(DBDCONTEXT*ctx, L_LONG code)
{
 if(!ctx)
  return ERR_NOCTH;
 ctx->errn=code;
 return code;
}
#define SERR(x) _seterr(ctx, x)

/*
 fills SP output argument
*/
static L_LONG fillarg(SV*arg, L_CHAR*adr, L_SWORD len, L_SWORD typ)
{
 L_CHAR     date[32];
 L_DOUBLE pd;
 char*    tmpstr = NULL;                        /* 15.10.04 */
 L_LONG   tmplen;                               /* 15.10.04 */

 switch(typ)
 {
   case DT_CHAR: 
    sv_setpvn(arg,adr,len);                     /* 05.04.01 */
   break;
   case DT_INTEGER: 
    if(len==4)
      sv_setiv(arg, *(L_LONG*)adr);
    else if (len ==2)
      sv_setiv(arg, *(L_SWORD*)adr);
    else if (len == 8){
      char strbuf[128];
      i64_ltoa(*((L_DLONG*)adr), strbuf);
      sv_setpv(arg, strbuf);
    }
    else
      sv_setiv(arg, *(L_BYTE*)adr);
   break;
   case DT_REAL:
    if(len==8)
      sv_setnv(arg, *(L_DOUBLE*)adr);
    else
      sv_setnv(arg, *(L_REAL*)adr); 
   break;
   case DT_DATE:
     TICKTOSTRF(adr, DATE_FORMAT, date);                  /* 01.02.06 */
     sv_setpvn(arg, date, sizeof(DATE_FORMAT)-1);         /* 01.02.06 */
   break;
   case DT_DECIMAL:
     DecToDbl(adr, &pd);
     sv_setnv(arg, pd);
   break;
   case DT_BYTE:
     sv_setpvn(arg, adr, len);
   break;
   case DT_BLOB:
     sv_setpvn(arg, adr, len);
   break;
   case DT_VARCHAR:
   case DT_VARBYTE:
     sv_setpvn(arg, adr+2, (STRLEN)(*((short*)adr)));
   break;
   case DT_BOOL:                  /* 21.12.00 */
     sv_setiv(arg, (STRLEN)(*(unsigned char*)adr));
   break;
   case DT_EXTFILE:   
     sv_setpvn(arg,adr+10,len-10);/* data part always has offset 10 */
   break;
                                                              
   case DT_NCHAR:                                          /* 15.10.04 ... */
     /* define tmpstr length*/
     LINNLS_Ucs2Utf8( (L_UNICHAR *)adr,
                      (L_LONG)(len / 2),
                      (L_LONG)(len * 4),
                      NULL,
                      &tmplen );
     if(!(tmpstr = (char *)MYMEM_CALLOC(1, tmplen+1)))     /* 06.04.10 */ /* 21.06.10 */ 
       Except(ERR_NOMEM);
     LINNLS_Ucs2Utf8( (L_UNICHAR *)adr,
                      (L_LONG)(len / 2),
                      (L_LONG)(len * 4),
                      tmpstr,
                      &tmplen );
     sv_setpvn(arg, tmpstr, (STRLEN)tmplen);
     SvUTF8_on( arg ); 
     MYMEM_FREE(tmpstr);                                   /* 06.04.10 */  
   break;
     
   case DT_NVARCHAR:
     /* define tmpstr length*/
     LINNLS_Ucs2Utf8( (L_UNICHAR *)(adr + 2),
                      *(L_WORD*)adr / 2,
                      (L_LONG)( *(L_WORD*)adr * 4),
                      NULL,
                      &tmplen );
     if(!(tmpstr = (char *)MYMEM_CALLOC(1, tmplen+1)))      /* 06.04.10 */ /* 21.06.10 */ 
       Except(ERR_NOMEM);
     LINNLS_Ucs2Utf8( (L_UNICHAR *)(adr + 2),
                      *(L_WORD*)adr / 2,
                      (L_LONG)( *(L_WORD*)adr * 4),
                      tmpstr,
                      &tmplen );
     sv_setpvn(arg, tmpstr, (STRLEN)tmplen);
     SvUTF8_on( arg ); 
     MYMEM_FREE(tmpstr);                                   /* 06.04.10 */  
   break;                                                  /* ... 15.10.04 */
   default:
      return ERR_CORRUPT;
  }
  return NOERR;
}

/*
 fill SP result and arguments 
*/
static L_LONG fill(SV*value, ARGPROC_OUT*res, L_CHAR*answ)
{
 L_SWORD      typ, len;
 L_CHAR         *adr;
 L_LONG      err;
 switch(res->Flags)
 {
  case 8:
    len = *(short*)(answ + res->Value);
    typ = res->Type.ntyp; 
    res->Value += sizeof(short) + len;
    len = res->Type.length;
    adr = answ+res->Value;
    if((err=fillarg(value, adr, len, typ)))
      return err;
  break;
  case 4: /* cursor */
  break;
  case 2: /* NULL value */
    sv_setsv(value,&sv_undef);
  break; 
  default:
    len = res->Type.length;
    typ = res->Type.ntyp; 
    adr = answ+res->Value;
    if((err=fillarg(value,adr, len, typ)))
      return err;
  break;
 }
 return NOERR;
}

/*
 prepare SV to bind 
*/
L_LONG prepareSVtoBind(SV* sv, short *type, short *len, char **value) /* 28.03.10 ... */
{  
  switch (SvTYPE(sv))                                      /* 18.05.10 */                          
  {
    case SVt_PV: *type = DT_CHAR;                     
                 *len = strlen(SvPVX(sv));
                 *value = SvPVX(sv);
                 break; 
    case SVt_PVNV: if (SvPOK(sv))
                 {
                   *type = DT_CHAR;                     
                   *len = strlen(SvPVX(sv));
                   *value = SvPVX(sv);
                   break; 
                 }                  
    case SVt_NV: if (SvNOK(sv))
                 {
                   *type = DT_REAL;                            
                   *value = (char *)&SvNVX(sv);                 
#ifdef NVTYPE                                             
                   *len = sizeof(NV);
#else
                   *len = sizeof(double);
#endif
                   break;   
                 }
    case SVt_PVIV: if (SvPOK(sv))
                 {
                   *type = DT_CHAR;                     
                   *len = strlen(SvPVX(sv));
                   *value = SvPVX(sv);
                   break; 
                 }                 
    case SVt_IV: *type = DT_INTEGER;
                 *len = sizeof(IV);           
                 *value = (char *)&SvIVX(sv);
                 break;
    case SVt_NULL:
#if PERL_REVISION < 5 || (PERL_REVISION == 5 && PERL_VERSION < 12) /* 24.11.11 ... */     
    case SVt_RV:
#endif
    case SVt_PVMG: 
#if PERL_REVISION < 5 || (PERL_REVISION == 5 && PERL_VERSION < 10)
      case SVt_PVBM:
#elif PERL_REVISION == 5 && PERL_VERSION >= 10 && PERL_VERSION <= 19 && PERL_SUBVERSION < 3  /* 21.11.14 ... */
      case SVt_BIND:
#elif PERL_REVISION == 5 && PERL_VERSION >= 19 && PERL_SUBVERSION >= 3
    case SVt_INVLIST:                                                                        /* ... 21.11.14 */
#endif                                                             /* ... 24.11.11 */
    case SVt_PVLV:
    case SVt_PVAV:
    case SVt_PVHV:
    case SVt_PVCV:
    case SVt_PVGV:
    case SVt_PVFM:
    case SVt_PVIO:
    default:     *type = DT_DEFAULT;
                 *len = 0;
                 *value = (char *)sv;
                 break;
  }  
  return NOERR;
}                                                          /* ... 28.03.10 */

/*
 generate statement for cursor that SP returns 
*/
static L_LONG genstat(int dbh,
                       CR_INFO*cr,
                       SV*sqltypes,/* input: array of standard sql types */
                       SV*result, /* output: result (may be cursor number ) */
                       SV*isProc, /* output: reference to 'cursor returns' flag*/
                       SV*field_cnt, /*output: fileds count that cursor has */
                       SV**fields,   /*output: ref. to fields array*/
                       SV**types)   /*output: ref. to types array*/
{
DBDCONNECTION*c,*crs;
DBDSTATEMENT *s;
GETA_OUT     *row;
AV           *typ, *nam;
SV           *name, *type;
SV           **sqltyp;
L_LONG      n, rn, i, j; 
L_LONG      err=0;
L_CHAR         *buf=NULL;
L_WORD       *dsc;
TCBL         cbl;
int          sn=-1,cn=-1; /* must be -1 */

 c = getconn(dbh);
 if(!c){
    return ERR_NODBH;
 }
 if(!SvROK(isProc)){
    return ERR_NOREF;
 }
 if (cr->CodErr){
  return -cr->CodErr;
 }

 cn = balloc(CONN_LIST);
 crs  = getconn(cn);
 if(!crs){
  err=ERR_NODBH;
  goto END;
 }
 sn = balloc(STAT_LIST);
 s  = getstat(sn);
 if(!s){
  err=ERR_NOSH;
  goto END;
 }

/* now describe output fields */

 if(!(buf=MYMEM_CALLOC(4096, sizeof(L_CHAR))))                    /* 03.04.01 */
  Except(ERR_NOMEM);
 memset((L_CHAR*)&cbl, 0, sizeof(TCBL));
 strncpy(cbl.Node, c->database, 8);                             /* 26.10.07 */
 cbl.NumChan=cr->NumChan;
 cbl.RowId=0; 
 cbl.PrzExe=0;
 cbl.LnBufRow=4096;                                         /* 06.12.00 */
 memcpy((char *)cbl.Command,"GETA",4);
 err= -inter(&cbl,NULL,NULL,NULL,buf);
 if(err)
 { goto END;
 }

 nam = newAV();
 typ = newAV();

 n =cbl.RowCount; /*<- columns count */
 if (!(s->dsc=MYMEM_CALLOC( sizeof(L_WORD)+2*sizeof(L_WORD)*n, sizeof(L_CHAR)))) /* {colunms count},{type,length},{type,length}...*//* 03.04.01 */
  Except(ERR_NOMEM);
 *(L_WORD*)(s->dsc)    = n; /* columns count */
 dsc = (L_WORD*)(s->dsc)+1;
 rn = 4096/sizeof(GETA_OUT);

 row = (GETA_OUT*)buf;

 for (i=0,j=0; i<n; i++,j++,row++)
 {
  if(j>=rn) 
  {
   rn=0;
   cbl.NumChan=cr->NumChan;
   cbl.RowId =i;
   cbl.PrzExe=0;
   cbl.LnBufRow=4096;
   memcpy((char *)cbl.Command,"GETA",4);
   if((err =-inter(&cbl,NULL,NULL,NULL,buf))!=0)
   {goto END;
   }
   row= (GETA_OUT*)buf;
  } 
  *(dsc+(i<<1))  = row->Type;
  *(dsc+(i<<1)+1)= row->Length;
  name= newSVpv(row->Column, min( strlen(row->Column), MAX_ID_LEN ) );  /* 01.10.04 */
  sqltyp=av_fetch((AV*)SvRV(sqltypes), _sqltype(row->Type,row->Length),0);
  if (sqltyp!=0){
   type= newSVsv(*sqltyp);
  }else
  {type= newSVsv(&sv_undef);
  }
  av_push(nam, name);
  av_push(typ, type); 
 }

 crs->parent=dbh;
 memset((L_CHAR*)&crs->cbl,0,sizeof(TCBL));
 crs->cbl.NumChan = cr->NumChan;
 strncpy(crs->cbl.Node, c->database, 8);                         /* 26.10.07 */

 s->rows = cr->RowCount;
 s->dbh  = dbh;
 s->tq   = NULL;
 s->bind_params = NULL;                                         /* 28.03.10 */
 s->parm_leng = NULL;                                           /* 28.03.10 */
 if (!(s->answ=MYMEM_CALLOC(cr->LnBufRow, sizeof(L_CHAR))))       /* 03.04.01 */
   Except(ERR_NOMEM);
 if (!(s->nulls = MYMEM_CALLOC(sizeof(L_SWORD)*n+4, sizeof(L_CHAR))))/* 03.04.01 */
   Except(ERR_NOMEM);

 *fields = newRV_noinc((SV*)nam);                       /* 03.04.01 */
 *types  = newRV_noinc((SV*)typ);                       /* 03.04.01 */
 sv_setiv(SvRV(field_cnt),n);
 sv_setiv(SvRV(isProc),cn); /* channel number */
 sv_setiv(SvRV(result),sn); /* statement number */

END:
 if (buf)
   MYMEM_FREE(buf);
 if(err<0){
  if(s->dsc!=0){
   MYMEM_FREE(s->dsc);
   s->dsc=NULL;
  }

  if(sn !=-1)
   bfree(STAT_LIST,sn);
  if(cn !=-1)
   bfree(CONN_LIST,cn);
 }
 return err;
}

/*
 clear answer description 
*/

void clearStatementDesc(int sh, int all, int force_unload) /* 25.08.10 ... */
{  
  DBDSTATEMENT *s;
  
  s = getstat(sh);
  if(!s)
    return;

  if ( s->answ )                                            /* 23.06.11 ... */
  {
    MYMEM_FREE(s->answ);
    s->answ = NULL;
  }
  if ( s->answ_desc )
  {
    MYMEM_FREE(s->answ_desc);
    s->answ_desc = NULL;
  }
  if ( s->nulls )
  {
    MYMEM_FREE(s->nulls);
    s->nulls = NULL;
  }
  if (s->colOffset )
  {
    MYMEM_FREE(s->colOffset);
    s->colOffset = NULL;
  }
  s->cols = 0;
  s->answ_len = 0;
  s->hasFirstRow = 0;
  if (all)
  {
    int i;
    
    if ( s->tq )
    {
      MYMEM_FREE(s->tq);
      s->tq = NULL;
    }
    if ( s->parm_leng )
    {
      MYMEM_FREE(s->parm_leng);
      s->parm_leng = NULL;
    }
    if ( s->bind_params )
    {
      if (!force_unload)
        for (i=0;i<s->param_count;i++)
          if (s->bind_params[i])
            SvREFCNT_dec(s->bind_params[i]);
      MYMEM_FREE(s->bind_params);
      s->param_count = 0;                                   /* ... 23.06.11 */
    }
  }
}

/*
 fill answer description 
*/
static L_LONG fillStatementDesc(int sh)          
{
  DBDCONNECTION *crs;
  DBDSTATEMENT *s;
  TCBL cbl;
  int  i, cols, delta = ANSWBUF_SIZ / sizeof(GETA_OUT);  
  char *firstRowBuf = NULL;           
  L_CHAR *nullMaskBuf;                                     /* 17.11.14 */
  
  s = getstat(sh);
  if(!s)
    return ERR_NOSH;

  crs = getconn(s->dbh);
  if(!crs)
    return ERR_NODBH;
    
  if (s->hasFirstRow) {
    cols = inter_column_number(&crs->cbl, s->tq);
    firstRowBuf = (L_BYTE *)MYMEM_CALLOC(1,crs->cbl.LnBufRow);
    memcpy(firstRowBuf,s->answ,crs->cbl.LnBufRow);
    nullMaskBuf = (L_CHAR*)MYMEM_CALLOC(cols + 2 * sizeof(L_SWORD), sizeof(L_CHAR));  /* 17.11.14 ... */
    memcpy(nullMaskBuf, s->nulls, cols + 2 * sizeof(L_SWORD));                        /* ... 17.11.14 */
  }      

  clearStatementDesc(sh, L_TFALSE, L_TFALSE);              
  if (crs->cbl.PrzExe & Q_ASYNC)  
    inter_wait_single(&crs->cbl, -1);                       
  memcpy(&cbl, &crs->cbl, sizeof(TCBL));
  crs->cbl.RowId = crs->cbl.LnBufRow = 0;
  strncpy((char *)crs->cbl.Command, "GETA", 4);
  inter(&crs->cbl, NULL, NULL, NULL, NULL);
  if (crs->cbl.PrzExe & Q_ASYNC)  
    inter_wait_single(&crs->cbl, -1);                       
  if (crs->cbl.CodErr)
  {
    if ( firstRowBuf )
      MYMEM_FREE(firstRowBuf);
    if (nullMaskBuf)                                                                  /* 17.11.14 ... */
      MYMEM_FREE(nullMaskBuf);                                                        /* ... 17.11.14 */
    return -crs->cbl.CodErr;
  }
  
  cols = s->cols = crs->cbl.RowCount;
  s->answ_desc = (GETA_OUT *)MYMEM_CALLOC(cols, sizeof(GETA_OUT));  
  if (firstRowBuf != NULL)                                
    s->answ = firstRowBuf;
  else
    s->answ = (L_BYTE *)MYMEM_CALLOC(1, cbl.LnBufRow); 

  if (nullMaskBuf != NULL)                                                            /* 17.11.14 ... */
    s->nulls = nullMaskBuf;
  else
    s->nulls = (L_CHAR*)MYMEM_CALLOC(cols + 2 * sizeof(L_SWORD), sizeof(L_CHAR));     /* ... 17.11.14 */

  s->colOffset = (long *)MYMEM_CALLOC(cols,sizeof(long));     
  if (!s->answ_desc || !s->nulls)
    return ERR_NOMEM;
  do {
    crs->cbl.LnBufRow = ANSWBUF_SIZ; 
    inter(&crs->cbl, NULL, NULL, NULL, &s->answ_desc[crs->cbl.RowId]);
    if (crs->cbl.PrzExe & Q_ASYNC)  
      inter_wait_single(&crs->cbl, -1);                       
    if (crs->cbl.CodErr) 
      return -crs->cbl.CodErr;
    crs->cbl.RowId += delta;
    cols           -= delta;
    } while(cols > 0);

  s->answ_len = 0;    
  for (i = 0; i < s->cols; i++)   
  {
    s->colOffset[i] = s->answ_len;                      
    s->answ_len    += s->answ_desc[i].Length;
    if (s->answ_desc[i].Type == DT_VARCHAR ||
        s->answ_desc[i].Type == DT_VARBYTE ||
        s->answ_desc[i].Type == DT_NVARCHAR)    
      s->answ_len += sizeof(L_WORD);          
  }    

  memcpy(&crs->cbl, &cbl, sizeof(TCBL));    
  
  return NOERR;
}                                                          /* ... 25.08.10 */

/************************** initialize & tuning *******************************/
/*
 driver initalize 
*/
int LINDBD_init(void)
{
 DBDCONTEXT *ctx=NULL; /* 21.12.00 */
 L_LONG ct;
#ifdef DEBUG_MODE
printf("\nLINDBD_init();\n");
#endif
 ct = binit(); /* internal structures initialize and allocate main context */
 if (ct>=0) /* first binit() call */
 {
   ctx= getctx(ct);
   if (!ctx)
    return ERR_NOCTH;
   ctx->last_func=DBD_INIT;
#ifdef DBI_THREAD
   ctx->mt = newmutex();
#endif
   return SERR(NOERR);
 }else
 {
   return SERR(NOERR);
 }
}

/*
 DBD version 
*/
int LINDBD_version(int cth,SV*major,SV*minor,SV*build)
{
 DBDCONTEXT   *ctx;
 ctx=getctx(cth);
 if (!ctx)
  return ERR_NOCTH;
 ctx->last_func=DBD_DRVVER;
if(!SvROK(major) || !SvROK(minor)|| !SvROK(build))
 return SERR(ERR_NOREF);

 sv_setiv(SvRV(major), _VER_MAX/100);
 sv_setiv(SvRV(minor), (_VER_MAX-(_VER_MAX/100)*100)/10);
 sv_setiv(SvRV(build),LINDBDBUILD);
 return SERR(NOERR);
}

/*
 Linter's core version 
*/
int LINDBD_server_version(int cth,SV*database, SV*major,SV*minor,SV*build)
{
 DBDCONTEXT   *ctx;
 TCBL cbl;
 DB_DESC desc;
 ctx=getctx(cth);
 if (!ctx)
  return ERR_NOCTH;
 ctx->last_func=DBD_SERVER;
 if(!SvROK(major) || !SvROK(minor) || !SvROK(build))
  return SERR(ERR_NOREF);
 if(!SvOK(database) || !SvPOKp(database))
  return SERR(ERR_INVVAL);
 memset(&cbl,0,sizeof(TCBL));
 memcpy(cbl.Command,"DESC",4);
 strncpy(cbl.Node,SvPVX(database),8);
 cbl.LnBufRow =sizeof(desc);
 inter(&cbl,NULL,NULL,NULL,&desc);
 if (cbl.CodErr)
  return SERR(-cbl.CodErr);
 sv_setiv(SvRV(major),desc.VerMajor);
 sv_setiv(SvRV(minor),desc.VerMinor);
 sv_setiv(SvRV(build),desc.VerBuild);
 return SERR(NOERR);
}

/*
 version's compatibility 
*/
int LINDBD_ver_compat(int cth,int drvmajor,int drvminor,int srvmajor,int srvminor)
{
 DBDCONTEXT   *ctx;
 ctx=getctx(cth);
 if (!ctx)
  return ERR_NOCTH;
 ctx->last_func=DBD_VERCOMPAT;

 if (drvmajor!=srvmajor)
  return SERR(ERR_NOCOMPAT);
 if (drvmajor==4) /* 4.x */
  return SERR(NOERR);
 if (drvminor==srvminor)
  return SERR(NOERR);
 if (drvminor==0 && srvminor!=0 || srvminor==0 && drvminor!=0) /* 5.0*/
  return SERR(ERR_NOCOMPAT);
 if (drvminor>=1 && drvminor<=3 && srvminor>3 ||  /*5.1-5.3*/
     srvminor>=1 && srvminor<=3 && drvminor>3)
  return SERR(ERR_NOCOMPAT);
 if (drvminor==4 && srvminor!=4 || srvminor==4 && drvminor!=4) /* 5.4*/
  return SERR(ERR_NOCOMPAT);
 if (drvminor>=5 && drvminor<=6 && srvminor>6 ||  /*5.5-5.6*/
     srvminor>=5 && srvminor<=6 && drvminor>6)
  return SERR(ERR_NOCOMPAT);
 if (drvminor==7 && srvminor!=7 || srvminor==7 && drvminor!=7) /* 5.7*/
  return SERR(ERR_NOCOMPAT);

 return SERR(NOERR);
}

/*************************** context management *******************************/
/****************************  (for future) ***********************************/
/*
 allocated new context
*/
int LINDBD_allocctx(SV**cth) /* output: context number */
{
 L_LONG ct= balloc(CTX_LIST);
#ifdef DEBUG_MODE
printf("\nLINDBD_allocctx();\n");
#endif
 *cth=newSViv(ct);
 return NOERR;
} 

/*
 free context
*/
int LINDBD_freectx(SV*cth) /* input: context number */
{
#ifdef DEBUG_MODE
printf("\nLINDBD_freectx();\n");
#endif
 if (!SvIOKp(cth))
  return ERR_INVVAL;
 bfree(CTX_LIST,SvIV(cth));
 return NOERR;
}

/************************* transaction management *****************************/
/*
 commit changes 
*/
int LINDBD_commit(int cth, /* context number */
                  int dbh) /* connection number */
{
 DBDCONTEXT   *ctx;
 DBDCONNECTION*c;
 TCBL *cbl;
#ifdef DEBUG_MODE
printf("\nLINDBD_commit();\n");
#endif
 ctx=getctx(cth);
 if (!ctx)
  return ERR_NOCTH;
 ctx->last_func=DBD_COMMIT;
 c=getconn(dbh);
 if (!c)
  return SERR(ERR_NODBH);

 cbl=&c->cbl;
 memcpy(cbl->Command,"COMT",4);
 inter(cbl,NULL,NULL,NULL,NULL);                           /* 24.06.10 */ 
 return SERR(-cbl->CodErr);                                /* 24.06.10 */
}

/*
 rollback changes
*/
int LINDBD_rollback(int cth, /* context number */
                    int dbh, /* connection number */
                    SV*spoint) /* savepoint name : not supported yet*/
{
 DBDCONTEXT   *ctx;
 DBDCONNECTION*c;
 TCBL *cbl;
#ifdef DEBUG_MODE
printf("\nLINDBD_rollback();\n");
#endif
 ctx=getctx(cth);
 if (!ctx)
  return ERR_NOCTH;
 ctx->last_func=DBD_ROLLBACK;
 c=getconn(dbh);
 if (!c)
  return SERR(ERR_NODBH);

 cbl=&c->cbl;
 memcpy(cbl->Command,"RBAC",4);
 inter(cbl,NULL,NULL,NULL,NULL);                           /* 24.06.10 */
 return SERR(-cbl->CodErr);                                /* 24.06.10 */
}

/*
 save point : for future
*/
int LINDBD_savepoint(int cth, /* context number */
                     int dbh, /* connection number */
                     SV*spoint) /*savepoint name : not supported yet*/
{
 DBDCONTEXT   *ctx;
 ctx=getctx(cth);
 if (!ctx)
  return ERR_NOCTH;
 ctx->last_func=DBD_SAVEPOINT;
 return SERR(NOERR);
}
/***************************** session management *****************************/
/*
 open cursor (for v4xx open channel)
*/
int LINDBD_cursor_open(int cth,  /* context number    */
                       int dbh,  /* connection number */
                       int mode, /* child channel mode*/
                       char*name,/* child channel name or NULL*/
                       int*crh)   /* output: cursor number reference */
{
 DBDCONTEXT   *ctx;
 DBDCONNECTION*c,*cr;
 TCBL*cbl;
 L_LONG crn;
#ifdef DEBUG_MODE
printf("\nLINDBD_cursor_open();\n");
#endif
 ctx=getctx(cth);
 if (!ctx)
   return ERR_NOCTH;
 ctx->last_func=DBD_CURSOPEN;
 c=getconn(dbh);
 if (!c)
   return SERR(ERR_NODBH);

 crn=balloc(CONN_LIST);
 cr=getconn(crn);
 if (!cr)
   return SERR(ERR_NODBH);

 memcpy(&cr->cbl,&c->cbl,sizeof(TCBL));

#if defined DEBUG_MODE && defined DEBUG_DBHNUM
printf ( "\nparent: %d",c->parent);
#endif
 if (c->parent ==-1) /* all cursors have one root. There is main connection */
  cr->parent=dbh;
 else
  cr->parent=c->parent;

 cr->attr= mode;
 cbl=&cr->cbl;

#if _VER_MAX >= 610
 cbl->PrzExe = mode & (TM_AUTOCOMMIT | TM_OPTIMISTIC |TM_EXCLUSIVE | CP_866 | CP_1251 | CP_KOI8 | TIL_OPTIMISTIC | TIL_READ_COMMITTED | TIL_READ_UNCOMMITTED | TIL_SERIALIZABLE ); /* 16.02.06 13.02.06 */ 
#else
 cbl->PrzExe = mode & (TM_AUTOCOMMIT | TM_OPTIMISTIC |TM_EXCLUSIVE | CP_866 | CP_1251 | CP_KOI8 ); /* 13.02.06 */ 
#endif

 memcpy(cbl->Command,"OCUR",4);
 if(inter(cbl,NULL,NULL,NULL,NULL))
 {bfree(CONN_LIST,crn);
  goto END;
 }
 if (name)
 {
  cbl->RowId = crName;                                     /* 16.12.99 */
  memcpy(cbl->Command,"SETO",4);
  cbl->LnBufRow=MAX_ID_LEN;                             /* 27.12.00 */
  if(inter(cbl,NULL,NULL,NULL,name))
  {bfree(CONN_LIST,crn);
   goto END;
  }
 }
 *crh = crn;                                    /* 03.04.01 */
END:
 return SERR(-cbl->CodErr);
}

/*
 close cursor 
*/
int LINDBD_cursor_close(int cth,  /* context number */
                        int dbh)  /* cursor number  */
{
 DBDCONTEXT   *ctx;
 DBDCONNECTION*c;
 TCBL*cbl;
 L_LONG err;
#ifdef DEBUG_MODE
printf("\nLINDBD_cursor_close();\n");
#endif
 ctx=getctx(cth);
 if (!ctx)
  return ERR_NOCTH;
 ctx->last_func=DBD_CURSCLOSE;
 c=getconn(dbh);
 if (!c)
   return SERR(ERR_NODBH);

 if (c->parent== -1) /* this channel is no cursor (child channel )*/
   return SERR(ERR_NOTCURSOR);
 cbl=&c->cbl;

 memcpy(cbl->Command,"CLOS",4);
 inter(cbl,NULL,NULL,NULL,NULL);
 err=cbl->CodErr;
 bfree(CONN_LIST,dbh);
 return SERR(-err);
}

/*
 open connection
*/
int LINDBD_connect(int cth,      /* current context number */
                   SV* database, /* database name (node) */
                   SV* username, /* username : casesens.; less or eq. then MAX_ID_LEN syms*//* 27.12.00*/
                   SV* password, /* username : casesens.; less or eq. then 18 syms*/
                   SV* mode,     /* channel modes  */
                   SV* cp,       /* codepage */
                   SV**dbh)      /* output: connection number */
{
 DBDCONNECTION*c;
 DBDCONTEXT   *ctx;
 TCBL*cbl;
 L_CHAR User[48];
 L_CHAR cp_name_buf[7] = "";                                 /* 14.07.10 */
 L_LONG CodErr;
 L_LONG  dbn;

#ifdef DEBUG_MODE
printf("\nLINDBD_connect();\n");
#endif
 ctx=getctx(cth);
 if (!ctx)
   return ERR_NOCTH;

 ctx->last_func=DBD_CONNECT;

 dbn = balloc(CONN_LIST);
 *dbh = newSViv(dbn);

 c=getconn(dbn);
 if (!c)
  return SERR(ERR_NODBH);

 cbl=&c->cbl;
 c->attr = SvIV(mode)|SvIV(cp); /* channel mode + codepage */ /* 13.02.06 */ 
#if _VER_MAX >= 610
 cbl->PrzExe = c->attr & (TM_AUTOCOMMIT | TM_OPTIMISTIC |TM_EXCLUSIVE | CP_866 | CP_1251 | CP_KOI8 | TIL_OPTIMISTIC | TIL_READ_COMMITTED | TIL_READ_UNCOMMITTED | TIL_SERIALIZABLE ); /* 16.02.06 13.02.06 */ 
#else
 cbl->PrzExe = c->attr & (TM_AUTOCOMMIT | TM_OPTIMISTIC |TM_EXCLUSIVE | CP_866 | CP_1251 | CP_KOI8 ); /* 13.02.06 */ 
#endif
 memset(c->database,0,sizeof(c->database));
 memset(c->user    ,0,sizeof(c->user));
 memset(c->pass    ,0,sizeof(c->pass));
 strncpy(c->database, SvPVX(database), 8);
 strncpy(c->user    , SvPVX(username),MAX_ID_LEN); /* 27.12.00 */
 strncpy(c->pass    , SvPVX(password),18);
#ifdef DEBUG_MODE
printf("\nNODE: %s",c->database);
printf("\nUSER: %s",c->user);
printf("\nPASS: %s",c->pass);
#endif
 c->parent=-1;

 strncpy(cbl->Node, c->database, 8);
 sprintf(User, "\"%s\"/\"%s\"", c->user, c->pass);
 memcpy(cbl->Command,"OPEN",4);
#if _VER_MAX >=600                                         /* 14.07.10 ... */
  if ((c->attr & CP_KOI8) == CP_KOI8)
    strncpy(cp_name_buf,"KOI8-R",6);
  else if ((c->attr & CP_1251) == CP_1251)
    strncpy(cp_name_buf,"CP1251",6);
  else
    strncpy(cp_name_buf,"CP866",5);
  
  CodErr = inter(cbl, User, &cp_name_buf, NULL, NULL);  
#else                                                      
  CodErr = inter(cbl, User, NULL, NULL, NULL);
#endif
 if(CodErr)                                                /* ... 14.07.10 */
 {bfree(CONN_LIST,dbn);
  sv_setiv(*dbh,-1);
 }
 else
 {
  sv_setiv(*dbh,dbn);
  memset(cbl->Command, ' ',4);
 }
 return SERR(-CodErr);
}

/*
 close connection
*/
int LINDBD_disconnect(int cth,  /* context namber */
                      int dbh,  /* connection number */
                      char*action)  /* disconnect attributes : commit/rollback*/
{
 DBDCONNECTION*c;
 DBDCONTEXT   *ctx;
 TCBL         *cbl;
 L_LONG       CodErr;
 L_LONG       comt,i ;

#ifdef DEBUG_MODE
printf("\nLINDBD_disconnect();\n");
#endif
 if (cth !=-2)
 {
  ctx=getctx(cth);
  if (!ctx)
   return ERR_NOCTH;
  ctx->last_func=DBD_DISCONNECT;
 }

 c= getconn(dbh);
 if (!c)
   return SERR(ERR_NODBH);

 comt = (!strcmp(action, "commit"));

 if(comt && (c->attr&0x100))
   LINDBD_commit(cth,dbh);
 else if ((c->attr&0x100))
   LINDBD_rollback(cth,dbh, NULL);

 if (c->parent==-1) /* main channel */
 {
  for (i=0;(i=childconn(dbh,i))!=-1;)
  {
    if(comt && (c->attr&0x100))
     LINDBD_commit(cth,i);
    else if ((c->attr&0x100))
     LINDBD_rollback(cth,i, NULL);

     LINDBD_cursor_close(cth,i);
  }
  cbl=&c->cbl;
  memcpy(cbl->Command,"CLOS",4);
  CodErr = inter(cbl, NULL, NULL, NULL, NULL);
  bfree(CONN_LIST,dbh);
  return SERR(-CodErr);
 }
 else /* child channel (cursor) */
 {
  return SERR(LINDBD_cursor_close(cth,dbh));
 }
}

/* disconnect all channels */
int  LINDBD_disconnect_all(void)
{
 L_LONG i;
 TCBL*cbl;
#ifdef DEBUG_MODE
printf("\nLINDBD_disconnect_all();\n");
#endif
  for (i=0; (i=nextconn(i))!=-1;)
  {
   if (getconn(i)->parent==-1)
   {
    cbl = &getconn(i)->cbl;
    memcpy(cbl->Command,"CLOS",4);
    inter(cbl, NULL, NULL, NULL, NULL);
   }
   bfree(CONN_LIST,i);
  }
  bfreeall(CONN_LIST);
#ifdef DEBUG_MODE
printf("\nLINDBD_disconnect_all() done;\n");
#endif
 return NOERR;
}


/*
 flash linter buffers to disk
*/
int LINDBD_snap(int cth, /* context number */
                SV*database) /* linter server name*/
{
 DBDCONTEXT   *ctx;
 TCBL cbl;
 ctx=getctx(cth);
 if (!ctx)
  return ERR_NOCTH;
 ctx->last_func=DBD_SNAP;
 memcpy(cbl.Command,"SNAP",4);
 memset(cbl.Node, '\0',8);
 if (SvPOKp(database))
  strncpy(cbl.Node, SvPVX(database),8);
 return SERR(-inter(&cbl,NULL,NULL,NULL,NULL));
}


/*
 linter-system shutdown
*/
int LINDBD_shut(int cth, /* context number */
                SV*database) /* linter server name*/
{
 DBDCONTEXT   *ctx;
 TCBL cbl;
 ctx=getctx(cth);
 if (!ctx)
  return ERR_NOCTH;
 ctx->last_func=DBD_SHUT;
 memcpy(cbl.Command,"SHUT",4);
 memset(cbl.Node, '\0',8);
 if (SvPOKp(database))
  strncpy(cbl.Node, SvPVX(database),8);
 return SERR(-inter(&cbl,NULL,NULL,NULL,NULL));
}

/****************************** query management ******************************/
/*
 prepare statement from query text.
 NOTE: text may consists dynamic parameters (<?> - unnamed parameter; <:name> - named parameter)
*/
int LINDBD_prepare(int cth,  /* context number */
                   int dbh,  /* connection number */
                   SV*stat, /* query string */  
                   SV*attr, /* attributes hash */
                   int*sh)  /* output: statement number */      /* 03.04.01 */
{
 DBDCONTEXT   *ctx;
 DBDCONNECTION*c;
 DBDSTATEMENT*s;
 TCBL*cbl;
 L_CHAR*buf;
 L_CHAR*sstat;                                    /* 03.04.01 */
 L_LONG  len;                                  /* 03.04.01 */
 L_LONG  shn;
 short reslen;  
 short p_count;                                 /* 28.03.10 */

#ifdef DEBUG_MODE
printf("\nLINDBD_prepare();\n");
#endif
 ctx=getctx(cth);
 if (!ctx)
  return ERR_NOCTH;

 ctx->last_func=DBD_PREPARE;

 c= getconn(dbh);
 if (!c)
   return SERR(ERR_NODBH);

 if (!(buf=MYMEM_CALLOC(65535, sizeof(L_CHAR)))) /* no memory */ /* 20.08.10 */ /* 03.04.01 ... */
  Except(ERR_NOMEM);

 if (c->attr & TRANSLATE_ESC)                                 /* 13.02.06 ... */
 {
   if(!(sstat = MYMEM_CALLOC(L_MAXWORD+2, sizeof(L_CHAR))))
     Except(ERR_NOMEM);

   ReplaceEscapeSequences(SvPVX(stat),         /* source */
                          strlen(SvPVX(stat)), /* source length */
                          sstat,               /* result */
                          &reslen,             /* result length */
                          L_MAXWORD);            /* result max length */
   sstat[reslen] = ';';
   sstat[reslen + 1] = 0;
 }
 else
 {
   len = strlen(SvPVX(stat));     /* duplicate statement and force set ';' */
   if(!(sstat = MYMEM_CALLOC(len+2, sizeof(L_CHAR))))
     Except(ERR_NOMEM);
   strcpy(sstat, SvPVX(stat));
   sstat[len] = ';';
 }                                                             /* ... 13.02.06 */

 
 cbl=&c->cbl;
 memcpy(cbl->Command,"TRAN",4); 
 cbl->LnBufRow=65535;                            /* 14.07.10A */ /* 20.08.10 */                   
 
#ifdef PRINT_QUERY
printf("\n\nquery: %s",SvPVX(stat));
#endif
 inter(cbl, NULL, sstat,NULL,buf);              /* 14.07.10A */

#ifdef PRINT_QUERY
printf("\nquery's translate code: %d",cbl->CodErr);
#endif

 MYMEM_FREE(sstat);     /* free temporary buffer */
                                                /* ... 03.04.01 */
 if (cbl->CodErr)
 {
   MYMEM_FREE(buf);
   *sh = -1;                                    /* 03.04.01 */
   return SERR(-cbl->CodErr);
 }
 
 shn = balloc(STAT_LIST);
 *sh = shn;                                     /* 03.04.01 */
 s = getstat(shn);
 s->tq = buf;
 s->dbh= dbh;

 inter_GetParamCount(s->tq, &p_count);          /* 28.03.10 ... */
 s->param_count = p_count; 
 s->parm_leng = MYMEM_CALLOC(1, (sizeof(short) * p_count)); /* 06.04.10 */
 memset(s->parm_leng,0, (sizeof(short) * p_count));
 s->bind_params = MYMEM_CALLOC(1,(sizeof(SV*)*p_count));    /* 06.04.10 */
 memset(s->bind_params,0, (sizeof(SV*)*p_count)); /* ... 28.03.10 */        

 return SERR(-cbl->CodErr);
}

/* return query paramter's count or -1 of error */
int LINDBD_paramcnt(int cth, /* context number */
                    int sh) /* statement number */
{
 DBDCONTEXT   *ctx;
 DBDCONNECTION*c;
 DBDSTATEMENT*s; 
                                                           /* 28.03.10 */
#ifdef DEBUG_MODE
printf("\nLINDBD_paramcnt();\n");
#endif
 ctx=getctx(cth);
 if (!ctx)
  return ERR_NOCTH;
 ctx->last_func=DBD_PARAMCNT;

 s= getstat(sh);
 if (!s)
   return SERR(ERR_NOSH);
 c= getconn(s->dbh);
 if (!c)
   return SERR(ERR_NODBH);

 if (!s->tq)
   return SERR(ERR_NOTPREPARED);
 
 SERR(NOERR);
 return (int)s->param_count;                               /* 28.03.10 */
}

int LINDBD_fieldcnt(int cth, /* context number */
                    int sh) /* statement number */
{
 DBDCONTEXT   *ctx;
 DBDCONNECTION*c;
 DBDSTATEMENT*s;
 TCBL*cbl;
 L_SWORD n;

#ifdef DEBUG_MODE
printf("\nLINDBD_fieldcnt();\n");
#endif
 ctx=getctx(cth);
 if (!ctx)
  return ERR_NOCTH;
 ctx->last_func=DBD_FIELDCNT;

 s= getstat(sh);
 if (!s)
   return SERR(ERR_NOSH);
 c= getconn(s->dbh);
 if (!c)
   return SERR(ERR_NODBH);

 if (!s->tq)
   return SERR(ERR_NOTPREPARED);

 if(_opertype(s->tq) != 1) /* nonSELECT query*//* 17.03.99*/
 {return 0;
 }
 cbl=&c->cbl;

 n = inter_column_number(cbl,s->tq);
 SERR(cbl->CodErr);
 if (cbl->CodErr)
  return (-cbl->CodErr);
 else
  return n;
}



int LINDBD_freestmt(int sh) /* statement number */          /* 23.06.11 ... */
{
   clearStatementDesc(sh, /* All */ 1, /*force_unload*/ 0 /* dec ref count */);
   bfree(STAT_LIST, sh);
   return 0;
}                                                           /* ... 23.06.11 */


/*
 array of column names 
*/
int LINDBD_titles(int cth,   /* context number */
                  int sh,    /* statement number */
                  SV*sqltypes, /* reference to hash with sql types */
                  SV**tlist, /* output: column, column_type */
                  SV**types)
{
 DBDCONTEXT   *ctx;
 DBDCONNECTION*c;
 DBDSTATEMENT*s;
 L_SWORD n,i;
 AV*nam, *typ;
 SV*name, *type;
 SV**sqltyp;
 GETA_OUT*row;
 TCBL*cbl;
 L_CHAR *buf =NULL;

 ctx=getctx(cth);
 if (!ctx)
  return ERR_NOCTH;
 ctx->last_func=DBD_TITLES;
 s= getstat(sh);
 if (!s)
   return SERR(ERR_NOSH);
 c= getconn(s->dbh);
 if (!c)
   return SERR(ERR_NODBH);

 if (!s->tq)
   return SERR(ERR_NOTPREPARED);
 cbl=&c->cbl;

 if (_opertype(s->tq) !=1)  /* non select query */
 {
   *tlist = &sv_undef;
   *types = &sv_undef;
   return 0;
 }
  
 n = inter_column_number(cbl,s->tq);

 if (cbl->CodErr)
  return SERR(-cbl->CodErr);

 nam=newAV(); /* may be mortal ? */
 typ=newAV();

 if (!(buf = MYMEM_CALLOC(n * sizeof(GETA_OUT), sizeof(L_CHAR)))) /* 03.04.01 */
   Except(ERR_NOMEM);

 cbl->RowId =0;
 cbl->LnBufRow = n * sizeof(GETA_OUT);                      /* 06.12.00 */
 inter_get_description(cbl,s->tq,buf);
 if (cbl->CodErr)
 {MYMEM_FREE(buf);
  return SERR(-cbl->CodErr);
 }
 for (i =0, row = (GETA_OUT*)buf ;i<n; i++, row++ )
 {
   name= newSVpv( row->Column, min( strlen(row->Column), MAX_ID_LEN ) );  /* 01.10.04 */
   sqltyp=av_fetch((AV*)SvRV(sqltypes), _sqltype(row->Type,row->Length),0);
   if (sqltyp!=0){
    type= newSVsv(*sqltyp);
   }else
   {type= newSVsv(&sv_undef);
   }
   av_push(nam, name);
   av_push(typ, type); 
 }                                                      /* ...03.04.01 */
 MYMEM_FREE(buf);

 *tlist = newRV_noinc((SV*)nam);
 *types = newRV_noinc((SV*)typ);
 return SERR(-cbl->CodErr);
}


/* bind actual parameters  */
int LINDBD_bind(int cth,  /* context number */
                int sh,  /* statement number */
                int parnum,    /* param number*/
                SV* value,    /* param value */
                SV* parlen)  /* param type */
{
 DBDCONTEXT    *ctx;
 DBDCONNECTION *c;
 DBDSTATEMENT  *s;
 TCBL          *cbl;
 STRLEN         len = 0;  /* must be force set */
 L_CHAR          *pname = NULL;                                  /* 03.11.04 */ 

#ifdef DEBUG_MODE
printf("\nLINDBD_bind();\n");
#endif
 ctx=getctx(cth);
 if (!ctx)
  return ERR_NOCTH;

 ctx->last_func=DBD_BIND;

 s= getstat(sh);
 if (!s)
   return SERR(ERR_NOSH);
 c= getconn(s->dbh);
 if (!c)
   return SERR(ERR_NOCTH);

 if (!s->tq)
   return SERR(ERR_NOTPREPARED);
 cbl=&c->cbl;
 
 if (parnum < 1 || parnum>s->param_count)                  /* 28.03.10 ... */ /* 06.04.10B */
   return SERR(ERR_INVIDX);

 s->bind_params[parnum-1] = value;
 value->sv_refcnt++;                                       /* ... 28.03.10 */

 return SERR(0);                                             /* 23.06.11a */
}

/*
 execute prepared statement 
*/
int LINDBD_execute(int cth,  /* context number */
                   int sh,   /* statement number */
                   int*curs) /* cursor number */
{
 DBDCONTEXT   *ctx;
 DBDCONNECTION*c;
 DBDSTATEMENT *s;
 TCBL         *cbl;
 L_SWORD      n;
 L_LONG       i;
 L_WORD        ParamType;
 SV           *param;
 char         *ParamValue;
 L_LONG        ParamLen; 

#ifdef DEBUG_MODE
printf("\nLINDBD_execute();\n");
#endif
 ctx=getctx(cth);
 if (!ctx)
  return ERR_NOCTH;
 ctx->last_func=DBD_EXECUTE;
 s = getstat(sh);
 if (!s)
  return SERR(ERR_NOSH);
 if (!s->tq)
  return SERR(ERR_NOTPREPARED);

 if (_opertype(s->tq) == 52) /* execute procedure */
 {return SERR(ERR_EXEPROC);
 }

 if (!(c=getconn(s->dbh)))
  return SERR(ERR_NODBH);
 cbl=&c->cbl;
 
 for (i = 0; i<s->param_count; i++)                                         /* 28.03.10 ... */
 {
   if (!s->bind_params[i])
      continue;
   prepareSVtoBind(s->bind_params[i], &ParamType, &s->parm_leng[i], &ParamValue);                            
   inter_bind_num (cbl, s->tq, i+1, ParamType, &s->parm_leng[i], ParamValue);  
 }                                                                          /* ... 28.03.10 */

/* SELECT statement : open cursor */
/*                                                      commented 11.02.05 
 if (_opertype(s->tq) ==1 ) 
 {
   if((err=LINDBD_cursor_open(cth,s->dbh,c->attr,NULL,curs))!=0)
     return SERR(err);
   if (!(c=getconn(*curs)))
     return SERR(ERR_NODBH);
   cbl=&c->cbl;
 }
 else
*/
 { *curs = sh; /* the same channel */                      /* 03.04.01 */
 }

/* execute query */
 memcpy(cbl->Command,"EXEC",4);                            /* 14.07.10 */         
 cbl->LnBufRow=ANSWBUF_SIZ;
 if (!(s->answ=MYMEM_CALLOC(ANSWBUF_SIZ, sizeof(L_CHAR))))   /* 03.04.01 */
  Except(ERR_NOMEM);

 n = inter_column_number(cbl,s->tq);
 if(n>0)
 {
   if (!(s->nulls = (L_CHAR*)MYMEM_CALLOC(n + 2 * sizeof(L_SWORD), sizeof(L_CHAR))))    /* 03.04.01 */ /* 17.11.14 */
    Except(ERR_NOMEM);
 }
 inter(cbl,s->nulls,s->tq,NULL,s->answ); 
#ifdef DEBUG_MODE
 printf("\nexecute status: %d\n", cbl->CodErr);
#endif
 if (cbl->CodErr ==2){
   SERR(0);
   s->rows =0;
   return 0;
 }
 else if (cbl->CodErr){
  s->rows =0;
  return SERR(-cbl->CodErr);
 }
 else if (_opertype(s->tq)==1) {                           /* 25.08.10 ... */
   s->hasFirstRow = 1;                
   if ((i = fillStatementDesc(sh)) != 0)
     return i;
 }
 else { /* rows for fetch or affected rows count */        /* ... 25.08.10 */
   
   /* check if we must append blob value */                   /* 31.02.06 ... */   
   for (i=1; i<=s->param_count; i++)                          /* 28.03.10 */
   {
     inter_ExtDescParam(s->tq, i, NULL, &ParamType, NULL, NULL, NULL, NULL, NULL, NULL);
     if ((ParamType == DT_BLOB) &&
         ((_opertype(s->tq) == 2) || /* s_INSERT */
          (_opertype(s->tq) == 3)))  /* s_UPDATE */
     {
       param = newSVpv(*(char **)inter_get_paradr(cbl, s->tq, i), 0); /* 06.04.10A */
       ParamValue = (char *)SvPV_nolen(param);
       ParamLen = s->parm_leng[i - 1];                                /* 20.11.14 */
       memcpy(cbl->Command,"ABLB",4);
       cbl->RowCount=inter_GetBlobColNum(s->tq, i);
       while (ParamLen>0) 
       {
         cbl->LnBufRow = (ParamLen > BLOB_PORTION_SIZE)?BLOB_PORTION_SIZE : ParamLen; /* size of blob data without ending zero */
         if (inter(cbl,NULL,NULL,NULL, ParamValue))
           return SERR(-cbl->CodErr);
         ParamValue = ParamValue + BLOB_PORTION_SIZE;
         ParamLen = ParamLen - BLOB_PORTION_SIZE;
       }
     } 
   }                                                           /* ... 31.02.06 */   
 }
 
 SERR(0);
 s->rows =cbl->RowCount;
 return cbl->RowCount;                                       /* 20.09.10A */   
}


/*
 fetch selected items 
*/
int LINDBD_fetch(int cth,  /* context */
                 int sh,   /* statement number */
                 int dbh,   /* channel number */
                 SV*dir,  /* fetch direction: next, prev, first, last, num*/
                 int fst, /* fetches 1-st row*/
                 SV**cols) /* output: columns */
{
DBDCONTEXT   *ctx;
DBDCONNECTION*c;
DBDSTATEMENT *s;
TCBL         *cbl;
AV           *row; 
SV           *d, *off;
L_CHAR         *adr;
L_WORD         *dsc;
L_CHAR         date[32];
L_DOUBLE     pd;
L_SWORD      n;
L_LONG      i;
L_WORD       typ, len;
char*        tmpstr = NULL;                   /* 15.10.04 */
L_LONG       tmplen;                          /* 15.10.04 */
SV           *sv;
SV           *blobsv;  
L_LONG      lasterr;
GETA_OUT*    col_desc;                        /* 25.08.10 */
EXTFILE_DESC *ef;                             /* 19.11.14 */

#ifdef DEBUG_MODE
printf("\nLINDBD_fetch();\n");
#endif

 ctx=getctx(cth);
 if (!ctx)
  return ERR_NOCTH;
 ctx->last_func=DBD_FETCH;

 s = getstat(sh);
 if (!s)
  return SERR(ERR_NOSH);
 c= getconn(dbh);
 if (!c)
   return SERR(ERR_NODBH);

 if (!s->tq && !s->dsc)
  return SERR(ERR_NOTPREPARED);  /* if SP executution is born cursor : tq==NULL*/

 if (s->tq && _opertype(s->tq)!=1) /* non-select statement */
  return SERR(ERR_NONSELECT); 

 cbl=&c->cbl;

 if (s->tq)
  n = inter_column_number(cbl,s->tq);
 else /* procedure return cursor */
  n = *(L_WORD*)s->dsc; /* {colunms count},{type,length},{type,length}...*/

 if (cbl->CodErr)
  return SERR(-cbl->CodErr);

 if (!SvROK(dir) || SvTYPE(SvRV(dir))!=SVt_PVAV)
   return SERR(ERR_NOREF);
 d=*(av_fetch((AV*)SvRV(dir), 0, 0));                   /* 03.04.01 */
 if (SvTYPE(d)!=SVt_PV)
 { 
   return SERR(ERR_INVTYP);
 }
/* not first fetch after execute() and !GETN,GETP*/
 if (fst==2 || (fst==1 && (!strncmp(SvPVX(d),"GETS",4) || 
                           !strncmp(SvPVX(d),"GETL",4) ||
                           !strncmp(SvPVX(d),"GETF",4)
                          ) 
               ) 
    )
 {
  if (!strncmp(SvPVX(d),"GETS",4))
  { 
/*    off= av_shift((AV*)SvRV(dir));*/
    off= *(av_fetch((AV*)SvRV(dir), 1, 0));             /* 03.04.01 */
    if (!(SvIOK(off) || SvIOKp(off)))                   /* 14.02.13 */
    { 
      return SERR(ERR_INVTYP);
    }
    cbl->RowId = SvIV(off);
  }
  memcpy(cbl->Command,SvPVX(d),4);                      /* 14.07.10A */ 
  cbl->LnBufRow =ANSWBUF_SIZ;
  if (inter(cbl,s->nulls,NULL,NULL,s->answ))
    return SERR(-cbl->CodErr);
 }

 /* getting fields :*/
 row = newAV();                                                            /* 24.03.03 */

 if (s->tq) /* cursor maked by 'EXEC'*/
 {
  for(i=0; i < n; i++) /* 15.10.04 */ /* 25.08.10 */
  {
    col_desc = &s->answ_desc[i];                           /* 25.08.10 */
    adr      = &s->answ[s->colOffset[i]];                  /* 25.08.10 */
   if (s->nulls && (s->nulls[i+4] == 1)) /* NULL value */  /* 25.08.10 */
   {
     av_push(row, &sv_undef);
   }
   else
   {
    switch(col_desc->Type)                                 /* 25.08.10 */
    {
     case DT_CHAR:
      av_push(row, newSVpv(adr, col_desc->Length));        /* 25.08.10 */
     break;
     case DT_INTEGER:
      if(col_desc->Length==4)                              /* 25.08.10 */
        av_push(row, newSViv(*((L_LONG*)adr)));
      else if(col_desc->Length==8){                        /* 25.08.10 */   
        char strbuf[128];
        i64_ltoa(*((L_DLONG*)adr), strbuf);
        av_push(row, newSVpv(strbuf, strlen(strbuf)));     /* 14.11.14 */
      }
      else
        av_push(row, newSViv(*((L_SWORD*)adr)));
     break;
     case DT_REAL: 
      if(col_desc->Length==8)                              /* 25.08.10 */
        av_push(row, newSVnv(*((L_DOUBLE*)adr)));
      else
        av_push(row, newSVnv(*((L_REAL*)adr)));
     break;
     case DT_DATE: 
      TICKTOSTRF(adr, DATE_FORMAT, date);                  /* 01.02.06 */
      av_push(row, newSVpv(date, sizeof(DATE_FORMAT)-1));  /* 01.02.06 */
     break;
     case DT_DECIMAL: 
      DecToDbl(adr, &pd);
      av_push(row, newSVnv(pd));
     break;
     case DT_BYTE: 
      av_push(row, newSVpv(adr, col_desc->Length));        /* 25.08.10 */
     break;
     case DT_BLOB:
       memcpy(cbl->Command,"GBLB",4);
       cbl->RowCount=i+1;
       cbl->RowId = 1;          
       cbl->LnBufRow = BLOB_PORTION_SIZE;
       tmpstr = (char*)MYMEM_CALLOC(1, BLOB_PORTION_SIZE); /* 06.04.10 */
       blobsv = newSVpvn(NULL, 0);
       lasterr = cbl->CodErr;
       do
       {
         if (inter(cbl,NULL,NULL,NULL, tmpstr))
         {
           if (cbl->CodErr == 2) /* no data */
             break;
           else
             return SERR(-cbl->CodErr);
         }
         sv_catpvn(blobsv, tmpstr, cbl->LnBufRow);
         cbl->RowId += BLOB_PORTION_SIZE;
       } while (cbl->LnBufRow>=BLOB_PORTION_SIZE);
       MYMEM_FREE(tmpstr);                                 /* 06.04.10 */ /* 25.08.10 */ 
       cbl->CodErr = lasterr;
       av_push(row, blobsv);                               /* ... 31.02.06 */   
     break;
     case DT_VARCHAR: 
     case DT_VARBYTE: 
      len = (STRLEN)(*((short*)adr));                       
      adr+= sizeof(L_WORD);
      if (len > 0)                   /* 13.11.14 ... */
      {
        sv = newSVpv(adr, len);
      }
      else
      {
        sv = newSV(0);
        sv_setpv(sv, "");
      }
      av_push(row, sv);              /* ... 13.11.14 */
     break;
     case DT_BOOL:                   /* 21.12.00 */
      av_push(row, newSViv((STRLEN)(*(unsigned char*)adr)));
     break;
     case DT_EXTFILE:
      ef = (EXTFILE_DESC*)adr;                                    /* 19.11.14 ... */
      av_push(row, newSVpv(ef->FileName, strlen(ef->FileName)));  /* ... 19.11.14 */
     break;
                                                           /* 15.10.04 ... */
     case DT_NCHAR:                                            
       /* define tmpstr length*/
       LINNLS_Ucs2Utf8( (L_UNICHAR *)adr,
                        (L_LONG)(col_desc->Length / 2),    /* 25.08.10 */
                        (L_LONG)(col_desc->Length * 4),    /* 25.08.10 */
                        NULL,
                        &tmplen );
       if(!(tmpstr = (char *)MYMEM_CALLOC(1, tmplen+1)))   /* 06.04.10 */ /* 21.06.10 */
         Except(ERR_NOMEM);
       LINNLS_Ucs2Utf8( (L_UNICHAR *)adr,
                        (L_LONG)(col_desc->Length / 2),    /* 25.08.10 */
                        (L_LONG)(col_desc->Length * 4),    /* 25.08.10 */
                        tmpstr,
                        &tmplen );
       sv = newSVpv(tmpstr,(STRLEN)tmplen);
       SvUTF8_on( sv ); 
       av_push(row,sv);
       MYMEM_FREE(tmpstr);                                 /* 06.04.10 */
     break;
     
     case DT_NVARCHAR:
       /* define tmpstr length*/
       LINNLS_Ucs2Utf8( (L_UNICHAR *)(adr + 2),
                        *(L_WORD*)adr / 2,
                        (L_LONG)( *(L_WORD*)adr * 4),
                        NULL,
                        &tmplen );
       if(!(tmpstr = (char *)MYMEM_CALLOC(1, tmplen+1)))   /* 06.04.10 */ /* 21.06.10 */
         Except(ERR_NOMEM);
       LINNLS_Ucs2Utf8( (L_UNICHAR *)(adr + 2),
                        *(L_WORD*)adr / 2,
                        (L_LONG)( *(L_WORD*)adr * 4),
                        tmpstr,
                        &tmplen );
       sv = newSVpv(tmpstr,(STRLEN)tmplen);
       SvUTF8_on( sv ); 
       av_push(row,sv);
       MYMEM_FREE(tmpstr);                                 /* 06.04.10 */
     break;                                                /* ... 15.10.04 */
     default:
       return SERR(ERR_CORRUPT);
    }
   }
  }
 }
 else /* cursor maked by SP */
 {
  dsc = (L_WORD*)(s->dsc)+1;
  adr = s->answ;
  for (i=0; i<n; i++, adr+= IS_VAR_TYPE(typ) ? len+2 : len )  /* 15.10.04 */
  {
   typ=*(dsc+(i<<1));
   len=*(dsc+(i<<1)+1);
   switch (typ) /*type*/
   {
    case DT_CHAR:
     av_push(row, newSVpv(adr, len));
    break;
    case DT_INTEGER: 
     if(len==4)
       av_push(row, newSViv(*((L_LONG*)adr)));
     else if(len==8){
       char strbuf[128];
       i64_ltoa(*((L_DLONG*)adr), strbuf);
       av_push(row, newSVpv(strbuf, strlen(strbuf)));     /* 14.11.14 */
     }
     else
       av_push(row, newSViv(*((L_SWORD*)adr)));
    break;
    case DT_REAL:
     if(len==8)
       av_push(row, newSVnv(*((L_DOUBLE*)adr)));
     else
       av_push(row, newSVnv(*((L_REAL*)adr)));
    break;
    case DT_DATE:
     TICKTOSTRF(adr, DATE_FORMAT, date);                  /* 01.02.06 */
     av_push(row, newSVpv(date, sizeof(DATE_FORMAT)-1));  /* 01.02.06 */
    break;
    case DT_DECIMAL:
     DecToDbl(adr, &pd);
     av_push(row, newSVnv(pd));
    break;
    case DT_BYTE:
     av_push(row, newSVpv(adr, len));
    break;
    case DT_BLOB:                                         /* 31.02.06 ... */
       memcpy(cbl->Command,"GBLB",4);
       cbl->RowCount=i+1;
       cbl->RowId = 1;          
       cbl->LnBufRow = BLOB_PORTION_SIZE;
       tmpstr = (char*)MYMEM_CALLOC(1, BLOB_PORTION_SIZE); /* 06.04.10 */
       blobsv = newSVpvn(NULL, 0);
       lasterr = cbl->CodErr;
       do
       {
         if (inter(cbl,NULL,NULL,NULL, tmpstr))
         {
           if (cbl->CodErr == 2) /* no data */
             break;
           else
             return SERR(-cbl->CodErr);
         }
         sv_catpvn(blobsv, tmpstr, cbl->LnBufRow);
         cbl->RowId += BLOB_PORTION_SIZE;
       } while (cbl->LnBufRow>=BLOB_PORTION_SIZE);
       MYMEM_FREE(tmpstr);                                /* 06.04.10 */ /* 25.08.10 */
       cbl->CodErr = lasterr;
       av_push(row, blobsv);                              /* ... 31.02.06 */
    break;        	
    case DT_VARCHAR: 
    case DT_VARBYTE: 
     len = (STRLEN)(*((short*)adr));  /* 13.11.14 ... */
     adr += sizeof(L_WORD);
     if (len > 0)
     {
       sv = newSVpv(adr, len);
     }
     else
     {
       sv = newSV(0);
       sv_setpv(sv, "");
     }
     av_push(row, sv);                /* ... 13.11.14 */
    break;
    case DT_BOOL:                  /* 21.12.00 */
     av_push(row, newSViv((STRLEN)(*(unsigned char*)adr)));
    break;
    case DT_EXTFILE:
     ef = (EXTFILE_DESC*)adr;                                    /* 19.11.14 ... */
     av_push(row, newSVpv(ef->FileName, strlen(ef->FileName)));  /* ... 19.11.14 */
    break;
                                                           /* 15.10.04 ... */
     case DT_NCHAR:                                            
       /* define tmpstr length*/
       LINNLS_Ucs2Utf8( (L_UNICHAR *)adr,
                        (L_LONG)(len / 2),
                        (L_LONG)(len * 4),
                        NULL,
                        &tmplen );
       if(!(tmpstr = (char *)MYMEM_CALLOC(1, tmplen+1)))   /* 06.04.10 */ /* 21.06.10 */
         Except(ERR_NOMEM);
       LINNLS_Ucs2Utf8( (L_UNICHAR *)adr,
                        (L_LONG)(len / 2),
                        (L_LONG)(len * 4),
                        tmpstr,
                        &tmplen );
       sv = newSVpv(tmpstr,(STRLEN)tmplen);
       SvUTF8_on( sv ); 
       av_push(row,sv);
       MYMEM_FREE(tmpstr);                                 /* 06.04.10 */
     break;
     
     case DT_NVARCHAR:
       /* define tmpstr length*/
       LINNLS_Ucs2Utf8( (L_UNICHAR *)(adr + 2),
                        *(L_WORD*)adr / 2,
                        (L_LONG)( *(L_WORD*)adr * 4),
                        NULL,
                        &tmplen );
       if(!(tmpstr = (char *)MYMEM_CALLOC(1, tmplen+1)))   /* 06.04.10 */ /* 21.06.10 */
         Except(ERR_NOMEM);
       LINNLS_Ucs2Utf8( (L_UNICHAR *)(adr + 2),
                        *(L_WORD*)adr / 2,
                        (L_LONG)( *(L_WORD*)adr * 4),
                        tmpstr,
                        &tmplen );
       sv = newSVpv(tmpstr,(STRLEN)tmplen);
       SvUTF8_on( sv ); 
       av_push(row,sv);
       MYMEM_FREE(tmpstr);                                 /* 06.04.10 */
     break;                                                /* ... 15.10.04*/
    default:
      return SERR(ERR_CORRUPT);
   }
  }
 }

 *cols = newRV_noinc((SV*)row);                 /* 03.04.01 */
 return SERR(-cbl->CodErr);
}

int LINDBD_rows(int cth,int sh) /* return total rows to fetch or -<Code of error>*/
{
 DBDCONTEXT   *ctx;
 DBDSTATEMENT *s;
#ifdef DEBUG_MODE
printf("\nLINDBD_rows();\n");
#endif
 ctx=getctx(cth);
 if (!ctx)
  return ERR_NOCTH;
 ctx->last_func=DBD_ROWS;

 s = getstat(sh);
 if (!s)
  return SERR(ERR_NOSH);
 return s->rows;
}

/*
 fetch finish
*/
int LINDBD_finish(int cth,  /* context */
                  int sh,   /* statement number */
                  int dbh)  /* channel number */
{
 DBDCONTEXT   *ctx;
 DBDCONNECTION*c;
 DBDSTATEMENT *s;
 TCBL*cbl;
 L_LONG err = 0;
 short i;                                                  /* 28.03.10 */
#ifdef DEBUG_MODE
printf("\nLINDBD_finish();\n");
#endif
 ctx=getctx(cth);
 if (!ctx)
  return ERR_NOCTH;
 ctx->last_func=DBD_FINISH;
 s=getstat(sh);
 if (!s)
  return SERR(ERR_NOSH);
 c=getconn(dbh);
 if (!c)
  return SERR(ERR_NODBH);

 cbl=&c->cbl;
#if defined DEBUG_MODE && defined DEBUG_DBHNUM
printf ( "\nparent: %d",c->parent);
#endif
 if (c->parent!=-1)
 {
  err= LINDBD_cursor_close(cth, dbh);
  c->parent = -1;                                          /* 19.06.12 23.06.11b ... */
 }

 if (s->answ){
   MYMEM_FREE(s->answ);
   s->answ=NULL;
 }

 if (s->nulls){
   MYMEM_FREE(s->nulls);
   s->nulls=NULL;
 }

 if(s->bind_params)
 {                    /* 28.03.10 ... */
   for (i=0;i<s->param_count;i++)
      if (s->bind_params[i])
      {
        SvREFCNT_dec(s->bind_params[i]);
        s->bind_params[i] = NULL;
      }
 }                                                          /* ... 23.06.11b */

 return SERR(err);
}

/*
 execute immediate non-select statement 
*/
int LINDBD_do(int cth,   /* context number */
              int dbh,   /* connection number */
              SV*stat,   /* statement text */
              SV*attr,   /* attributes: not used */
              SV*argv)   /* placeholders   */
{
 DBDCONTEXT   *ctx;
 DBDCONNECTION*c;
 TCBL*cbl;
 L_CHAR*buf=NULL, *adr, *statement;
 L_LONG  i, err=0;
 L_WORD iOperType; 
 L_SWORD n, typ, size, *parlen = NULL;                                     /* 24.03.03 */
 STRLEN len;  /* must be force set */
 SV*value, *curs;                                                          /* 24.03.03 */

#ifdef DEBUG_MODE
printf("\nLINDBD_do();\n");
#endif
 ctx=getctx(cth);
 if (!ctx)
  return ERR_NOCTH;
 ctx->last_func=DBD_DO;
 c=getconn(dbh);
 if (!c)
  return SERR(ERR_NODBH);
 if (c->parent!=-1)
 { return SERR(ERR_NOTMAINCH);
 }

/* only from here we may say goto END; */
 if (!(buf=MYMEM_CALLOC(65535, sizeof(L_CHAR)))) /* no memory */ /* 20.08.10 */ /* 03.04.01 ... */
  Except(ERR_NOMEM);

 len = strlen(SvPVX(stat));     /* duplicate statement and force set ';' */
 if (len <= L_MAXWORD-3)                                         /* 20.08.10 */
 {
   if (!(statement = MYMEM_CALLOC(len+2, sizeof(L_CHAR))))
     Except(ERR_NOMEM);
   strcpy(statement, SvPVX(stat));
   statement[len] = ';';                                       /*... 03.04.01 */
 }

 cbl=&c->cbl;
 memcpy(cbl->Command,"TRAN",4);                                /* 14.07.10A */ /* 20.08.10 */
 cbl->LnBufRow=65535;
#ifdef PRINT_QUERY
printf("\n\nquery: %s",statement);
#endif
 err=inter(cbl, NULL, statement,NULL,buf);                     /* 14.07.10A */
 
#ifdef PRINT_QUERY
printf("\nquery's translate code: %d",cbl->CodErr);
#endif
 MYMEM_FREE(statement);                         /* 03.04.01 */
 if (cbl->CodErr)
 {
   err= -cbl->CodErr;
   goto END;
 }
 if ((iOperType=_opertype(buf))==52){ /* select query or execute procedure */
  err=ERR_EXEPROC;
  goto END;
 }

 inter_GetParamCount(buf, (L_SWORD*)&n);
 if ((n==0 && av_len((AV*)SvRV(argv)) != -1) || (n > 0 && av_len((AV*)SvRV(argv))+1 != n)){
   err=ERR_NOREF;
   goto END;
 }

 if (!(parlen = MYMEM_CALLOC(n, sizeof(L_SWORD))))                         /* 24.03.03 */
   Except(ERR_NOMEM);

 for (i=0; i < n; i++)
 {
   value= *(av_fetch((AV*)SvRV(argv), i, 0));           /* 03.04.01 */
   if (SvIOKp(value))
   {
    adr = (L_CHAR*)&SvIVX(value); /* address if INT*/
    typ = DT_INTEGER; 
    len = sizeof(int);
   }
   else if (SvNOKp(value))
   {
    adr = (L_CHAR*)&SvNVX(value); /* address if REAL*/
    typ = DT_REAL;  
    len = sizeof(L_DOUBLE);
   }
   else if (SvPOKp(value))
   {
    adr = SvPVX(value); /* address if L_CHAR**/
    typ = DT_CHAR; 
    SvPV(value, len);
   }
   else
   {
    err= ERR_INVTYP;
    goto END;
   }
   parlen[i] = len;
   inter_bind_num(cbl, buf, i+1, typ, &parlen[i], adr);                    /* 24.03.03 */

   if (cbl->CodErr)
   {err=-cbl->CodErr;
    goto END;
   }
 }

 memcpy(cbl->Command,"EXEC",4);
 cbl->LnBufRow=0;

 err= -inter(cbl,NULL,buf,NULL,NULL); 
/* we don't need get answer : */
 if (iOperType == 1 && cbl->CodErr==1014)
 {cbl->CodErr=0;
  err=0;
 }

END:
 if (buf)
   MYMEM_FREE(buf);
 if (parlen)                                                               /* 24.03.03 */
   MYMEM_FREE(parlen);
#ifdef DEBUG_MODE
printf("\nLINDBD_do() done; error code %d\n",err);
#endif
 if (err ==-2){ /* nothing found */
   SERR(0);
   return 0;
 }
 else if (err){ /* other errors */
  return SERR(err);
 }
 else{ /* rows for fetch or affected rows count */
  SERR(0);
  return cbl->RowCount;                                                    /* 20.09.10A */     
 }
}

/*************************** Locking  management ******************************/

/*
 lock row
*/
int LINDBD_lrow(int cth,   /* context number */
                int dbh)   /* connection number */
{
 DBDCONTEXT   *ctx;
 DBDCONNECTION*c;
 TCBL*cbl;
 ctx=getctx(cth);
 if (!ctx)
  return ERR_NOCTH;
 ctx->last_func=DBD_LOCKROW;
 c=getconn(dbh);
 if(!c)
  return SERR(ERR_NODBH);

 cbl=&c->cbl;
 memcpy(cbl->Command,"LROW",4);
 inter(cbl,NULL,NULL,NULL,NULL);
 return SERR(-cbl->CodErr);
}

/*
 unlock row
*/
int LINDBD_ulrow(int cth, /* context number */
                 int dbh) /* connection number */
{
 DBDCONTEXT   *ctx;
 DBDCONNECTION*c;
 TCBL*cbl;
 ctx=getctx(cth);
 if (!ctx)
  return ERR_NOCTH;
 ctx->last_func=DBD_ULOCKROW;
 c=getconn(dbh);
 if(!c)
  return SERR(ERR_NODBH);

 cbl=&c->cbl;
 memcpy(cbl->Command,"UROW",4);
 inter(cbl,NULL,NULL,NULL,NULL);
 return SERR(-cbl->CodErr);
}
/***************************** BLOB  management *******************************/
/*
 get blob portion after fetch
*/
int LINDBD_getblob(int cth,  /* context number */
                   int sh,   /* statememt number */
                   int dbh,  /* connection number */
                   SV*fn,    /* blob field number in dataset */ /* 05.04.01 */
                   SV*attr, /* attributes : offset, length*/
                   SV**blob) /*output: blob portion */
{
 DBDCONTEXT   *ctx;
 DBDCONNECTION*c;
 DBDSTATEMENT *s;
 TCBL*cbl;
 SV*b;
 L_CHAR*ptr;
 L_LONG leng;
 L_LONG offs;
#ifdef DEBUG_MODE
printf("\nLINDBD_getblob();\n");
#endif
 ctx=getctx(cth);
 if (!ctx)
   return ERR_NOCTH;
 ctx->last_func=DBD_GETBLOB;
 s = getstat(sh);
 if(!s)
   return SERR(ERR_NOSH);
 c  =getconn(dbh);
 if(!c)
   return SERR(ERR_NODBH);

 cbl=&c->cbl;
 offs=0;
 leng=DEF_BLOB_PORTION;
 if (!SvROK(attr) || SvTYPE(SvRV(attr))!=SVt_PVAV)
   return SERR(ERR_NOREF);

 if(av_len((AV*)SvRV(attr))==0){
  offs=SvIV((*av_fetch((AV*)SvRV(attr),0,0)));
 }
 else if(av_len((AV*)SvRV(attr))==1){
  offs=SvIV((*av_fetch((AV*)SvRV(attr),0,0)));
  leng=SvIV((*av_fetch((AV*)SvRV(attr),1,0)));
 }
                                                                /* 05.04.01 ...*/
 cbl->LnBufRow = leng;                                          /* 14.07.10A */
 cbl->RowId    = offs+1;
 if(SvOK(fn) && SvIOKp(fn) && SvIV(fn) > 0){ /* fn likes to valid dataset index */
  memcpy(cbl->Command,"GBLB",4);
  cbl->RowCount = SvIV(fn);
 }else{
  memcpy(cbl->Command,"GOBJ",4);
 }

 ptr = MYMEM_CALLOC(leng, sizeof(L_CHAR));                        /* 03.04.01 */
 if(!ptr)
  Except(ERR_NOMEM);
 b=newSVpv(ptr,leng);
 MYMEM_FREE(ptr);
 ptr=SvPVX(b);
 inter(cbl,NULL,NULL,NULL,ptr);
 if(!cbl->CodErr){
  if (leng != cbl->LnBufRow){
   SvCUR_set(b, cbl->LnBufRow);
  }
  *blob=b;
 }else{
  *blob =&sv_undef;
 }
 return SERR(-cbl->CodErr);
}


/*
 add blob portion
*/
int LINDBD_addblob(int cth,  /* context number */
                   int sh,   /* statememt number */
                   int dbh,  /* connection number */
                   SV*fn,    /* blob field number in dataset */ /* 05.04.01 */
                   SV*len, /* blob portion length */
                   SV*blob) /* blob portion */
{
 DBDCONTEXT   *ctx;
 DBDCONNECTION*c;
 DBDSTATEMENT *s;
 TCBL*cbl;
 L_CHAR*ptr;
 L_LONG length, err;
#ifdef DEBUG_MODE
printf("\nLINDBD_addblob();\n");
#endif
 ctx=getctx(cth);
 if (!ctx)
   return ERR_NOCTH;
 ctx->last_func=DBD_ADDBLOB;
 s = getstat(sh);
 if(!s)
   return SERR(ERR_NOSH);
 c  =getconn(dbh);
 if(!c)
   return SERR(ERR_NODBH);

 cbl=&c->cbl;

 if (!SvIOKp(len))
  return SERR(ERR_INVTYP);
 if (!SvROK(blob) || !SvPOKp(SvRV(blob))) /* blob is really reference to string */
  return SERR(ERR_NOREF);

 length=SvIV(len);
 length = min((STRLEN)SvLEN(SvRV(blob))-1, length);        /* 20.09.10 */
 ptr=SvPVX(SvRV(blob));

 LINDBD_lrow(cth,dbh);
 
 cbl->LnBufRow = length;                                   /* 05.04.01 ... */ /* 14.07.10A */
 if(SvOK(fn) && SvIOKp(fn) && SvIV(fn) > 0){ /* fn likes to valid dataset index */
  memcpy(cbl->Command,"ABLB",4);
  cbl->RowCount = SvIV(fn);
 }else{
  memcpy(cbl->Command,"AOBJ",4);
 }

 err=inter(cbl,NULL,NULL,NULL,ptr);
 LINDBD_ulrow(cth,dbh);

 return SERR(-err);
}


/*
 delete blob portion
*/
int LINDBD_delblob(int cth,  /* context number */
                   int sh,   /* statememt number */
                   int dbh,  /* connection number */
                   SV*fn,    /* blob field number in dataset */ /* 05.04.01 */
                   SV*attr) /* attributes : not used*/
{
 DBDCONTEXT   *ctx;
 DBDCONNECTION*c;
 DBDSTATEMENT *s;
 TCBL*cbl;
 L_LONG err;
#ifdef DEBUG_MODE
printf("\nLINDBD_delblob();\n");
#endif
 ctx=getctx(cth);
 if (!ctx)
  return ERR_NOCTH;
 ctx->last_func=DBD_DELBLOB;
 s = getstat(sh);
 if(!s)
   return SERR(ERR_NOSH);
 c  =getconn(dbh);
 if(!c)
   return SERR(ERR_NODBH);

 cbl=&c->cbl;

 LINDBD_lrow(cth,dbh);
 if(SvOK(fn) && SvIOKp(fn) && SvIV(fn) > 0){ /* fn likes to valid dataset index */
  memcpy(cbl->Command,"CBLB",4);
  cbl->RowCount = SvIV(fn);
 }else{
  memcpy(cbl->Command,"COBJ",4);
 }
 err=inter(cbl,NULL,NULL,NULL,NULL);
 LINDBD_ulrow(cth,dbh);

 return SERR(-err);
}

/*
 execute SP : for Linter 5.xx
*/
int LINDBD_proc(int cth,    /* context number          */
                int dbh,    /* connection number       */  
                SV*proc,    /* proc. execute statement */
                SV*arg,    /* reference to input/output args  */
                SV*sql_types,/* input: array of standard sql types */
                SV*result, /* output: result (may be cursor number ) */
                SV*isProc, /* output: reference to 'cursor returns' flag*/
                SV*field_cnt, /*output: fileds count that cursor has */
                SV**fields,   /*output: ref. to fields array*/
                SV**types)   /*output: ref. to types array*/
{
 DBDCONTEXT   *ctx;
 DBDCONNECTION*c;
 TCBL         *cbl;
 L_CHAR         *tq=NULL,*answ=NULL,*statement = NULL, *adr = NULL;          /* 24.03.03 */
 L_LONG      i, err=0;
 L_SWORD      n, typ, prec ,scal, nullable, argnum, size, paridx, *parlen = NULL; /* 24.03.03 */
 L_CHAR         inp, outp;
 STRLEN       len=0; /* must be force set */
 SV           *value = NULL, *valref = NULL, **valarr = NULL;              /* 24.03.03 */
 ARGPROC_OUT  *res;
 L_WORD       *pn;
 L_CHAR         *preparedValue;                                              /* 28.03.10 */

#ifdef DEBUG_MODE
printf("\nLINDBD_proc();\n");
#endif

 if(SvROK(isProc)){
   sv_setiv(SvRV(isProc),-1);
 }
 ctx=getctx(cth);
 if (!ctx)
  return ERR_NOCTH;
 ctx->last_func=DBD_EXEPROC;
 c = getconn(dbh);
 if (!c)
  return SERR(ERR_NODBH);
 if (c->parent!=-1)
 {return SERR(ERR_NOTMAINCH);
 }
/* only from here we may say goto END; */
 cbl=&c->cbl;

 if (!(tq=MYMEM_CALLOC(65535, sizeof(L_CHAR)))) /* no memory */    /* 03.04.01 */ /* 20.08.10 */
  Except(ERR_NOMEM);

                                                        /* 03.04.01 ... */
 len = strlen(SvPVX(proc));     /* duplicate statement and force set ';' */
 if(!(statement = MYMEM_CALLOC(len+2, sizeof(L_CHAR))))
  Except(ERR_NOMEM);
 strcpy(statement, SvPVX(proc));
 statement[len] = ';';                                  /*... 03.04.01 */

 memcpy(cbl->Command,"TRAN",4);                         /* 14.07.10A */ 
 cbl->LnBufRow=65535;                                   /* 20.08.10 */
#ifdef PRINT_QUERY
printf("\n\nquery: %s",statement);
#endif
 err=inter(cbl, NULL, statement,NULL,tq);               /* 14.07.10A */ 
 
#ifdef PRINT_QUERY
printf("\nquery's translate code: %d",cbl->CodErr);
#endif
 MYMEM_FREE(statement);                                 /* 03.04.01 */
 if (cbl->CodErr)
 {
   err= -cbl->CodErr;
   goto END;
 }

 inter_GetParamCount(tq, (L_SWORD*)&n);

 /* alloc the array: */
 if (!(valarr = (SV**)MYMEM_CALLOC(n, sizeof(SV*))))                       /* 24.03.03 */
    Except(ERR_NOMEM);

 if (!(parlen = MYMEM_CALLOC(n, sizeof(L_SWORD))))
   Except(ERR_NOMEM);

 for (paridx = 0, i=0; i < n; i++)                                         /* 24.03.03 */
 {
  valref= *(av_fetch((AV*)SvRV(arg), i, 0));            /* 03.04.01 */
  if (SvROK(valref))
    value =SvRV(valref);
  else
    value =valref;

  inter_ExtDescParam(tq,(L_SWORD)i+1, &size, &typ, &prec, &scal, &nullable, &inp, &outp, &argnum); /* 24.03.03 */
  len = (STRLEN)size;

  if (outp) /* add to reference list */
  {
    valarr[paridx ++] = SvRV(valref);                                      /* 31.03.03 */
  }

  prepareSVtoBind(value, &typ, &parlen[i], &preparedValue);                              
  inter_bind_num(cbl, tq, i+1, typ, &parlen[i], preparedValue);            /* 30.11.05 */ /* 28.03.10 */ /* 28.03.10 */        

  if (cbl->CodErr)
  {
    err=-cbl->CodErr;
    goto END;
  }
 }

 memcpy(cbl->Command,"EXEC",4);                                         /* 14.07.10A */ 
 cbl->LnBufRow=ANSWBUF_SIZ;
 if (!(answ=MYMEM_CALLOC(ANSWBUF_SIZ, sizeof(L_CHAR))))                   /* 03.04.01 */
  Except(ERR_NOMEM);
 err= -inter(cbl,NULL,tq,NULL,answ); 
 if (err!=0)
  goto END;

 res = (ARGPROC_OUT*)answ;
 if(res->Flags==4) /* cursor return */
 {
  if((err=genstat(dbh,(CR_INFO*)(answ+res->Value),sql_types,result,isProc,field_cnt,fields,types))<0)
   goto END;
 }
 else
 {
  if (SvROK(result))
  {
   if((err=fill(SvRV(result), res, answ)))
    goto END;
  }
 }

 pn = (L_WORD*)(res+1); /* arguments count */
 if (*pn > 0)
 {
  res = (ARGPROC_OUT*)(pn+1);
  for (i=0; i < *pn; i++, res++)
  {
   value = valarr[i];                                                      /* 24.03.03 */
   if (SvOK(value))
   {
    if((err=fill(value, res, answ)))
      goto END;
   }
  }
 }

END:
 if (tq)
   MYMEM_FREE(tq);
 if (answ)
   MYMEM_FREE(answ);
 if (parlen)                                                               /* 24.03.03 */
   MYMEM_FREE(parlen);
 if (valarr)
   MYMEM_FREE(valarr);

 return SERR(err);
}
/***************************** Error  management ******************************/
/* 
 get errno (get codes for all _this_ context errors )
*/
int LINDBD_errno(int cth,  /* context number */
                 int*errn) /* output: errno  */
{
 DBDCONTEXT   *ctx;
 ctx=getctx(cth);
 if (!ctx)
  return ERR_NOCTH;
 ctx->last_func=DBD_ERRNO;
 *errn=ctx->errn;
 return ctx->errn;
}

/*
 get error text associated with errno
*/
void LINDBD_errstr(int cth,    /* context number */
                  SV**errstr)  /* output: errstr */
{
 DBDCONTEXT   *ctx;
 int errn;                                                  /* 30.07.02 */
 ctx=getctx(cth);
 if (ctx){
   ctx->last_func=DBD_ERRSTR;
   LINDBD_errno(cth,&errn);
 }else
 {errn= ERR_NOCTH;
 }
 *errstr=newSVpv(textbycode(errn,NULL),0);
 return;
}

/*
 get last native error code for dbh handle
*/
void LINDBD_errstate(int cth,  /* context number    */
                    int dbh,  /* connecion numder  */
                    int *errn) /* output: error code*/
{
 DBDCONTEXT   *ctx;
 DBDCONNECTION*cn;
 ctx=getctx(cth);
 if (ctx)
  ctx->last_func=DBD_ERRSTATE;
 cn=getconn(dbh);
 if(!cn)
  *errn=1001;
 else
  *errn=cn->cbl.CodErr;
 return;
}

/*
 get last native error text for dbh handle
*/
void LINDBD_errmsg(int cth,  /* context number    */
                   int dbh,  /* connecion numder  */
                   SV*msg)   /* output: error message text*/
{
 DBDCONTEXT   *ctx;
 DBDCONNECTION*c;
 ctx=getctx(cth);
 if (ctx)
  ctx->last_func=DBD_ERRMSG;
 if (!SvROK(msg))
   return;

 c=getconn(dbh);
 if(!c)
  sv_setiv(SvRV(msg),1001);
 else
  sv_setiv(SvRV(msg),c->cbl.CodErr);
 return;
}

/*
 get last native error code (with SQL-92 standard accordance) for dbh handle
*/
void LINDBD_sqlstate(int cth,     /* context number    */
                    int dbh,     /* connecion numder  */
                    SV*sqlstat) /* output: SQLSTATE  */
{
 DBDCONTEXT   *ctx;
 DBDCONNECTION*c;
 ctx=getctx(cth);
 if (ctx)
  ctx->last_func=DBD_ERRSTATE;
 if (!SvROK(sqlstat))
   return;

 c=getconn(dbh);
 if(!c)
  sv_setpv(SvRV(sqlstat),get_sqlstate(1001));
 else
  sv_setpv(SvRV(sqlstat),get_sqlstate(c->cbl.CodErr));
 return;
}

/*
 direct linter's invoke
*/
int LINDBD_linter(SV*arg1,SV*arg2,SV*arg3,SV*arg4,SV*arg5)
{
TCBL cbl;
 L_CHAR*a2,*a3,*a4,*a5;
 SV**coderr,**prior,**numchan,**username,**command,**node,**rowid,
   **rowcount,**przexe,**syserr,**lnbufrow,**reserve;
 memset(&cbl, 0, sizeof(cbl));

 if (!SvROK(arg1)||!SvROK(arg2)||!SvROK(arg3)||!SvROK(arg4)||!SvROK(arg5))
 {return ERR_INVTYP;
 }
 if (SvTYPE(SvRV(arg1))!=SVt_PVHV)
 {return ERR_INVTYP;
 }
 coderr=hv_fetch((HV*)SvRV(arg1),"CodErr",sizeof("CodErr")-1,0);
 prior =hv_fetch((HV*)SvRV(arg1),"Prior",sizeof("Prior")-1,0);
 numchan=hv_fetch((HV*)SvRV(arg1),"NumChan",sizeof("NumChan")-1,0);
 username=hv_fetch((HV*)SvRV(arg1),"UserName",sizeof("UserName")-1,0);
 command=hv_fetch((HV*)SvRV(arg1),"Command",sizeof("Command")-1,0);
 node=hv_fetch((HV*)SvRV(arg1),"Node",sizeof("Node")-1,0);
 rowid=hv_fetch((HV*)SvRV(arg1),"RowId",sizeof("RowId")-1,0);
 rowcount=hv_fetch((HV*)SvRV(arg1),"RowCount",sizeof("RowCount")-1,0);
 przexe=hv_fetch((HV*)SvRV(arg1),"PrzExe",sizeof("PrzExe")-1,0);
 syserr=hv_fetch((HV*)SvRV(arg1),"SysErr",sizeof("SysErr")-1,0);
 lnbufrow=hv_fetch((HV*)SvRV(arg1),"LnBufRow",sizeof("LnBufRow")-1,0);
 reserve=hv_fetch((HV*)SvRV(arg1),"Reserve",sizeof("Reserve")-1,0);
 if (!coderr||!prior||!numchan||!username||!command||!node||!rowid||!rowcount||!przexe||!syserr||!lnbufrow)
 {return ERR_INVTYP;
 }
 if (SvOK(*command) && SvROK(*command) && SvPOKp(SvRV(*command)))
  strncpy(cbl.Command, SvPVX(SvRV(*command)),4);
 else
 {return ERR_INVTYP;
 }
 if (SvOK(*coderr) && SvROK(*coderr) && SvIOKp(SvRV(*coderr)))
  cbl.CodErr=SvIVX(SvRV(*coderr));
 if (SvOK(*prior) && SvROK(*prior) && SvIOKp(SvRV(*prior)))
  cbl.Prior=SvIVX(SvRV(*prior));
 if (SvOK(*numchan) && SvROK(*numchan) && SvIOKp(SvRV(*numchan)))
  cbl.NumChan=SvIVX(SvRV(*numchan));
 if (SvOK(*rowid) && SvROK(*rowid) && SvIOKp(SvRV(*rowid)))
  cbl.RowId=SvIVX(SvRV(*rowid));
 if (SvOK(*rowcount) && SvROK(*rowcount) && SvIOKp(SvRV(*rowcount)))
  cbl.RowCount=SvIVX(SvRV(*rowcount));
 if (SvOK(*przexe) &&  SvROK(*przexe) && SvIOKp(SvRV(*przexe)))
  cbl.PrzExe=SvIVX(SvRV(*przexe));
 if (SvOK(*syserr) && SvROK(*syserr) && SvIOKp(SvRV(*syserr)))
  cbl.SysErr=SvIVX(SvRV(*syserr));
 if (SvOK(*username) && SvROK(*username) && SvPOKp(SvRV(*username)))
  strncpy(cbl.UserName, SvPVX(SvRV(*username)),4);
 if (SvOK(*node) && SvROK(*node) && SvPOKp(SvRV(*node)))
  strncpy(cbl.Node, SvPVX(SvRV(*node)),8);
 if (SvOK(*lnbufrow) && SvROK(*lnbufrow) && SvIOKp(SvRV(*lnbufrow)))
  cbl.LnBufRow=SvIVX(SvRV(*lnbufrow));

 if(cbl.LnBufRow)
 { a5 = MYMEM_CALLOC(cbl.LnBufRow, sizeof(L_CHAR));                       /* 03.04.01 */
   if(!a5)
    Except(ERR_NOMEM);
 }else{
   a5 =NULL;
 }


 if(SvOK(SvRV(arg2)) && SvPOKp(SvRV(arg2)))
 {a2 = SvGROW(SvRV(arg2), 8192);
 }else
 {a2 = NULL;
 }

 a3=(SvOK(SvRV(arg3)) && SvPOKp(SvRV(arg3)))?SvPVX(SvRV(arg3)):NULL;
 a4=(SvOK(SvRV(arg4)) && SvPOKp(SvRV(arg4)))?SvPVX(SvRV(arg4)):NULL;

 inter(&cbl,a2,a3,a4,a5);

 sv_setiv(SvRV(*coderr),cbl.CodErr);
 sv_setiv(SvRV(*prior),cbl.Prior);
 sv_setiv(SvRV(*numchan),cbl.NumChan);
 sv_setiv(SvRV(*rowid),cbl.RowId);
 sv_setiv(SvRV(*rowcount),cbl.RowCount);
 sv_setiv(SvRV(*przexe),cbl.PrzExe);
 sv_setiv(SvRV(*syserr),cbl.SysErr);
 sv_setiv(SvRV(*lnbufrow),cbl.LnBufRow);

 if(a5){
  sv_setpvn(SvRV(arg5),a5,cbl.LnBufRow);
  MYMEM_FREE(a5);
 }

 return cbl.CodErr;
}


#if 0
char *GetParamValue(SV *sv, char type, int plen)    /* 03.11.04 ... */
{
  int    vlen = 0;
  char  *sval;
  static char val[4096];
  L_LONG tmplen;
  L_WORD space,i;

  memset(val, 0, sizeof(val));
  switch(type)
    {
    case DT_CHAR:
      memset(val, ' ', sizeof(val));
    case DT_BYTE:
      sval = SvPV(sv, vlen);
      memcpy(val, sval, (plen > vlen) ? vlen : plen);
      break;
    case DT_INTEGER:
      if (plen == 8)
        i64_FromDouble(SvNV(sv), *(L_DLONG *)val);
      else
      if (plen == 4)
        (*(L_LONG *)val) = (L_LONG)SvIV(sv);
      else
        (*(L_WORD *)val) = (L_WORD)SvIV(sv);
      break;
    case DT_REAL:
      if (plen == 4)
        (*(L_REAL   *)val) = (L_REAL)SvNV(sv);
      else
        (*(L_DOUBLE *)val) = (L_DOUBLE)SvNV(sv);
      break;
    case DT_DECIMAL:
      DblToDec(SvNV(sv), *(L_DECIMAL*)val);
      break;
    case DT_DATE:
      sval = SvPV(sv, vlen);
      STRTOTICK(sval, *(L_DECIMAL*)val);
      break;
    case DT_VARCHAR:                                      
    case DT_VARBYTE:
      sval = SvPV(sv, vlen);
      vlen = (plen > vlen) ? vlen : plen;
      *(L_WORD *)val = (L_WORD)vlen;
      memcpy(&val[sizeof(L_WORD)], sval, vlen);
      break;
    case DT_BOOL:
      if (SvIOKp(sv))
        vlen = (int)SvIV(sv);
      else if SvNOKp(sv)
        vlen = (int)SvNV(sv);
      else if SvPOKp(sv)
        {
        sval = SvPV(sv, vlen);
        vlen = (int)*sval;
        }
      strcpy(val, vlen ? "TRUE" : "FALSE");
      break;                                               
    case DT_NCHAR:
      sval = SvPV(sv, vlen);
      if (SvUTF8(sv))  /* vlue is UTF-8 string, then decode it into ucs2 string */
      {
        LINNLS_Utf8Ucs2( sval,
                         (L_LONG) strlen(sval),
                         (L_LONG) sizeof(val),
                         (L_UNICHAR *)val,
                         &tmplen );
        space = 0x0020;      
        for (i=tmplen; i<plen; i=i+2)
          memcpy(val+i, &space, 2);
      }
      else               /* value is normal string */
      {
        memcpy(val, sval, (plen > vlen) ? vlen : plen);
      };
      break;
    case DT_NVARCHAR:
      sval = SvPV(sv, vlen);
      if (SvUTF8(sv))  /* vlue is UTF-8 string, then decode it into ucs2 string */
      {
        LINNLS_Utf8Ucs2( sval,
                         (L_LONG) strlen(sval),
                         (L_LONG) sizeof(val) - sizeof(L_WORD),
                         (L_UNICHAR *)(val + sizeof(L_WORD)),
                         &tmplen );
        *(L_WORD *)val = (L_WORD) min(tmplen / 2,plen);
      }
      else               /* value is normal string */
      {
        vlen = (plen > vlen) ? vlen : plen;
        *(L_WORD *)val = (L_WORD)vlen / 2;
        memcpy(&val[sizeof(L_WORD)], sval, vlen);
      };
      break;
    };

  return ((char *)val);
}                                                      /* ... 03.11.04 */
#endif
/* 28.03.10 */


void LINDBD_nativesql(SV* stat,    /* query with ODBS ESC sequences */ /* 13.02.06 */
                      SV* res )    /* output: native linter query */
{
  char *buf = NULL;
  unsigned short reslen;
  
#ifdef DEBUG_MODE
printf("\nLINDBD_nativesql();\n");
#endif

  if (SvROK(res))
    if (buf = MYMEM_CALLOC(1,L_MAXWORD))                     /* 06.04.10 */
    {     
      ReplaceEscapeSequences(SvPVX(stat),         /* source */
                             strlen(SvPVX(stat)), /* source length */
                             &buf[0],      /* result */
	                     &reslen,      /* result length */
	                     L_MAXWORD); /* result max length */

      sv_setpvn(SvRV(res), &buf[0], reslen);
      MYMEM_FREE(buf);                                     /* 06.04.10 */      
      return;
    }
  sv_setsv(res,&sv_undef);
  return;
}


/**************************** source end **************************************/
