 /***************************************************************************
 *                 RELEX, Inc. Linter DBMS source code                      *
 *        This is unpublished proprietary source code of RELEX, Inc.        *
 *     The copyright notice does not evidence any actual or intended        *
 *                  publication of such source code.                        *
 *                                                                          *
 *                 Copyright (c) 1990-2013 RELEX Inc.                       *
 *                                                                          *
 *                   This software was developed by                         *
 *                                                                          *
 *                            RELEX, Inc.                                   *
 *                         Voronezh, Russia                                 *
 *                                                                          *
 *                        All rights reserved.                              *
 ****************************************************************************
 AUTHOR:  Andrew V Razuwaev
 ----------------------------------------------------------------------------
 PURPOSE: definitions, prototypes 
 ----------------------------------------------------------------------------
 REVISION HISTORY:
 27.12.00  - RAW (MAX_ID_LEN instead directly given length)
 03.04.01  - RAW (memory leak)
 24.10.04  - Ermakov (Added IS_VAR_TYPE)
 14.03.06  - Gluhov (WARNINGS)
 28.03.10  - Glukhov (Params bind remake)
 20.08.10  - Ermakov (Correct buffer size)
 25.08.10  - Glukhov (Using internal answer description added)
 17.08.11  - DV      (Remove 'l' from formats)
 09.09.13  - Galizin (calloc redefine is absent in perl)
*/

#ifndef __LINDBD_H__
#define __LINDBD_H__

#ifdef WIN32
#include <windows.h>
#endif


#include <stdio.h>
#include <string.h>
#include <stdlib.h>

#include "EXTERN.h"
#include "perl.h"
#include "XSUB.h"

#include "inter.h"
#include "linter.h"

/* Linter DBD build */
#ifndef LINDBDBUILD
#define  LINDBDBUILD 100
#endif

#define IS_VAR_TYPE(t) ( (t == DT_VARCHAR) || \
                         (t == DT_VARBYTE) || \
                         (t == DT_NVARCHAR) )           /* 24.10.04 */

#ifndef MEM_DEBUG                                          /* 03.04.01 */
void* LINDBD_calloc(int n, int size);                      /* 09.09.13 */
# define MYMEM_CALLOC(a,b)      LINDBD_calloc(a,b)         /* 09.09.13 */
# define MYMEM_FREE(a)          free(a)
#else
void* LINDBD_calloc(L_CHAR*file, L_LONG line, int n, int size);
# define MYMEM_CALLOC(a,b)      LINDBD_calloc(__FILE__, __LINE__, a,b)
# define MYMEM_FREE(a)          {printf("file: %s line:%d free(%p)\n",__FILE__, __LINE__, a);free(a);}
#endif

/* DBD error codes */

#define NOERR           0 /* no error */ 
#define NOTFOUND        100 /* not found */
#define ERR_NOCTH        -3050 /* invalid context number */
#define ERR_NODBH        -3051 /* invalid connetion number */
#define ERR_NOSH         -3052 /* invalid statement number */
#define ERR_NOTCURSOR    -3053 /* this channel is not cursor */
#define ERR_NOTPREPARED  -3054 /* statement is not prepared */
#define ERR_INVIDX       -3055 /* invalid bind parameter's number */
#define ERR_NOREF        -3056 /* parameter not defined */
#define ERR_INVTYP       -3057 /* invalid parameter's type */
#define ERR_INVVAL       -3058 /* invalid parameter's value */
#define ERR_EXEPROC      -3059 /* can not execute procedure by execute() */
#define ERR_NOTMAINCH    -3060 /* this channel is cursor */
#define ERR_CORRUPT      -3061 /* internal structures corrupted */
#define ERR_NOMEM        -3062 /* no memory */
#define ERR_SPNOTSUPP    -3063 /* SP call not supports */
#define ERR_NOCOMPAT     -3064 /* non-compatible versions */ 
#define ERR_NONSELECT    -3065 /* non-select statement was executed */

/* DBD functions */
#define DBD_INIT          "LINBDD_init"
#define DBD_CONNECT       "LINDBD_connect"
#define DBD_DISCONNECT    "LINDBD_disconnect"
#define DBD_CURSOPEN      "LINDBD_cursor_open"
#define DBD_CURSCLOSE     "LINDBD_cursor_close"
#define DBD_SNAP          "LINDBD_snap"
#define DBD_SHUT          "LINDBD_shut"
#define DBD_PREPARE       "LINDBD_prepare"
#define DBD_EXECUTE       "LINDBD_execute"
#define DBD_PARAMCNT      "LINDBD_paramcnt"
#define DBD_FIELDCNT      "LINDBD_fieldcnt"
#define DBD_TITLES        "LINDBD_titles"
#define DBD_BIND          "LINDBD_bind"
#define DBD_FETCH         "LINDBD_fetch"
#define DBD_FINISH        "LINDBD_finish"
#define DBD_DO            "LINDBD_do"
#define DBD_GETBLOB       "LINDBD_getblob"
#define DBD_ADDBLOB       "LINDBD_addblob"
#define DBD_DELBLOB       "LINDBD_delblob"
#define DBD_EXEPROC       "LINDBD_proc"
#define DBD_COMMIT        "LINDBD_commit"
#define DBD_ROLLBACK      "LINDBD_rollback"
#define DBD_SAVEPOINT     "LINDBD_savepoint"
#define DBD_LOCKROW       "LINDBD_lrow"
#define DBD_ULOCKROW      "LINDBD_ulrow"
#define DBD_ERRNO         "LINDBD_errno"
#define DBD_ERRSTR        "LINDBD_errstr"
#define DBD_ERRSTATE      "LINDBD_errstate"
#define DBD_ERRMSG        "LINDBD_errmsg"
#define DBD_SQLSTATE      "LINDBD_sqlstate"
#define DBD_ROWS          "LINDBD_rows"
#define DBD_DRVVER        "LINDBD_version"
#define DBD_SERVER        "LINDBD_server_version"
#define DBD_VERCOMPAT     "LINDBD_ver_compat"

/* answer buffer size */ 
#if _VER_MAX >= 600                                          /* 20.08.10 ... */
#define ANSWBUF_SIZ  0xFFFF
#else
#define ANSWBUF_SIZ  8192                                 
#endif                                                       

/* default blob portion*/
#define DEF_BLOB_PORTION 4048                                /* ... 20.08.10 */

/* inital values for connection , statement , context lists */
#define _MAX_CONN  32
#define _MAX_STAT  32
#ifndef DBI_THREAD
#define _MAX_CTX   1
#else
#define _MAX_CTX   32
#endif

/* lists numbers */
#define CTX_LIST   0
#define CONN_LIST  1
#define STAT_LIST  2

typedef struct _DBDCONTEXT
{
 L_LONG next; /* next free context descriptor number or -1 */
 L_LONG status;

 L_LONG errn;
 L_CHAR*   last_func;
 L_CHAR    sqlstate[8];
#if defined DBI_THREAD
#if defined WIN32
 L_CHAR*   mt;   /* mutex object */
#endif
#endif
}DBDCONTEXT;

typedef struct _DBDCONNECTION
{
 L_LONG next;  /* next free connection descriptor number or -1 */
 L_LONG status;/* this connection status */

 L_LONG  parent; /* parent channel or -1 */

 L_LONG attr; /* connetion properties */
 L_CHAR database[16];
 L_CHAR user[MAX_ID_LEN+8];				/* 27.12.00 */
 L_CHAR pass[24];
 TCBL cbl;
#if defined DBI_THREAD
#if defined WIN32
 L_CHAR*   mt;   /* mutex object */
#endif
#endif
}DBDCONNECTION;

typedef struct _DBDSTATEMENT
{
 L_LONG next; /* next free statement descriptor number or -1 */
 L_LONG status;

 L_LONG dbh; 
 L_LONG rows; /* rows to fetch or affected */
 L_CHAR    *tq; /* translated query */
 L_CHAR    *answ;  /* answer buffer */
 long     answ_len;                                        /* 25.08.10 */
 GETA_OUT *answ_desc;                                      /* 25.08.10 */  
 L_BOOL   hasFirstRow;                                     /* 25.08.10 */
 int      cols;                                            /* 25.08.10 */
 long    *colOffset;                                       /* 25.08.10 */
 short    param_count; /* query params count */            /* 28.03.10 ... */
 SV     **bind_params; /* binded params array */ 
 short   *parm_leng; /* binded params length array */      /* ... 28.03.10 */
 L_CHAR    *nulls; /* nulls */
 L_CHAR    *dsc;   /* cursor's description (for SP) or NULL*/
#if defined DBI_THREAD
#if defined WIN32
 L_CHAR*   mt;   /* mutex object */
#endif
#endif
}DBDSTATEMENT;

/* dbdmem.c */
L_LONG binit(void);
L_LONG bdeinit(void);
L_LONG balloc(L_LONG lst);
L_LONG bfree(L_LONG lst, L_LONG d);
DBDCONNECTION*getconn(L_LONG dbh);
DBDCONTEXT*getctx(L_LONG cth);
DBDSTATEMENT*getstat(L_LONG sh);
L_LONG childconn(L_LONG dbnp, L_LONG dbn);
L_LONG nextconn(L_LONG dbn);
void bfreeall(L_LONG lst);
void*mymem_calloc(L_CHAR*file, L_LONG line, size_t n, size_t size);/* 03.04.01 */

/* dbdasn.c */
#ifdef DBI_THREAD
void inimutex(void);
void mutex(void);
void unmutex(void);
L_CHAR*newmutex(void);
void emutex(L_CHAR*);
void lmutex(L_CHAR*);
#else
#define inimutex() {}
#define mutex() {}
#define unmutex() {}
#endif

/* dbderr.c */
void Except(L_LONG cause);
L_CHAR*textbycode(L_LONG code, L_CHAR*attr);
L_CHAR*get_sqlstate(L_LONG err);

#ifdef _RUSSIAN
#include "msgrus.h"
#else
#include "msgeng.h"
#endif


#endif
/**************************** source end **************************************/
