 /***************************************************************************
 *                 RELEX, Ltd. Linter DBMS source code                      *
 *        This is unpublished propietary source code of RELEX, Ltd.         *
 *     The copyright notice does not evidence any actual or intended        *
 *                  publication of such source code.                        *
 *                                                                          *
 *                 Copyright (c) 1990-1999 RELEX Ltd.                       *
 *                                                                          *
 *                   This software was developed by                         *
 *                                                                          *
 *                            RELEX, Ltd.                                   *
 *                         Voronezh, Russia                                 *
 *                                                                          *
 *                        All rights reserved.                              *
 ****************************************************************************
 AUTHOR:  Andrew V Razuwaev
 ----------------------------------------------------------------------------
 PURPOSE: DBD entry points for mutli-thread library
 ----------------------------------------------------------------------------
 REVISION HISTORY:
 27.12.00  - RAW (MAX_ID_LEN instead directly given object name length)
 03.04.01  - RAW (memory leak)
 05.04.01  - RAW (multiple blobs)
*/

#define _LINDBDA_

#ifdef DBI_THREAD

#include "lindbd.h"

int aLINDBD_version(int cth,SV*major,SV*minor,SV*build)
{return LINDBD_version(cth,major,minor,build);
}

int aLINDBD_server_version(int cth,SV*database, SV*major,SV*minor,SV*build)
{return LINDBD_server_version(cth,database,major,minor,build);
}

int aLINDBD_ver_compat(int cth,int drvmajor,int drvminor,int srvmajor,int srvminor)
{return LINDBD_ver_compat(cth,drvmajor,drvminor,srvmajor,srvminor);
}

int aLINDBD_allocctx(SV**cth) /* allocate context output: context number */
{
 DBDCONTEXT*ctx;
 int err;
 mutex();
  err= LINDBD_allocctx(cth);
  if(!err)
  {
    ctx = getctx(SvIV(*cth));
    if (!ctx)
    {err=ERR_NOCTH;
     goto END;
    }
    ctx->mt = newmutex();
  }
END:
 unmutex();
 return err;
}

int aLINDBD_freectx(SV*cth) /* free context number */
{
int err;
DBDCONTEXT*ctx;
 mutex();
   if (!SvIOKp(cth))
   {err=ERR_INVVAL;
    goto END;
   }
   ctx = getctx(SvIV(cth));
   if (!ctx)
   {err= ERR_NOCTH;
    goto END;
   }
   MYMEM_FREE(ctx->mt);
   ctx->mt=NULL;
   err= LINDBD_freectx(cth);
END:
 unmutex();
 return err;
}



int aLINDBD_commit(int cth, /* context number */
                   int dbh) /* connection number */
{
 int err;
 DBDCONNECTION*c;
 c=getconn(dbh);
 if (!c)
  return ERR_NODBH;
 emutex(c->mt);
 err=LINDBD_commit(cth,dbh);
 lmutex(c->mt);
 return err;
}

int aLINDBD_rollback(int cth, /* context number */
                     int dbh, /* connection number */
                     SV*spoint) /* savepoint name : not supported yet*/
{
 int err;
 DBDCONNECTION*c;
 c=getconn(dbh);
 if (!c)
  return ERR_NODBH;
 emutex(c->mt);
 err=LINDBD_rollback(cth,dbh,spoint);
 lmutex(c->mt);
 return err;
}

int aLINDBD_savepoint(int cth, /* context number */
                      int dbh, /* connection number */
                      SV*spoint) /*savepoint name : not supported yet*/
{
 int err;
 DBDCONNECTION*c;
 c=getconn(dbh);
 if (!c)
  return ERR_NODBH;
 emutex(c->mt);
 err=LINDBD_savepoint(cth,dbh,spoint);
 lmutex(c->mt);
 return err;
}

int aLINDBD_cursor_open(int cth,  /* context number    */
                        int dbh,  /* connection number */
                        int mode, /* child channel mode*/
                        char*name,/* child channel name or NULL*/
                        int*crh)   /* output: cursor number reference *//* 03.04.01 */
{
 int err;
 DBDCONNECTION*c, *cr;
 c=getconn(dbh);
 if (!c)
  return ERR_NODBH;
 emutex(c->mt);
 err=LINDBD_cursor_open(cth,dbh,mode,name,crh);
 if (!err) /* make mutex for cursor */
 {
  if((cr  = getconn(*crh))>=0)
  {cr->mt = newmutex();
  }
  else
  {err = ERR_NODBH;
  }
 }
 lmutex(c->mt);
 return err;
}


int aLINDBD_cursor_close(int cth,  /* context number */
                         int dbh)  /* cursor number  */
{
 int err;
 DBDCONNECTION*c, *cr;
 CHAR*mt;
 c=getconn(dbh);
 if (!c)
  return ERR_NODBH;
 mt=c->mt;
 emutex(mt);
 err=LINDBD_cursor_close(cth,dbh);
 lmutex(mt);
 mutex();
  MYMEM_FREE(c->mt);
  c->mt=NULL;
 unmutex();
 return err;
}

int aLINDBD_connect(int cth,      /* current context number */
                    SV* database, /* database name (node) */
                    SV* username, /* username : casesens.; less or eq. then MAX_ID_LEN syms*//* 27.12.00*/
                    SV* password, /* username : casesens.; less or eq. then 18 syms*/
                    SV* mode,     /* channel modes  */
                    SV* cp,       /* codepage */
                    SV**dbh)      /* output: connection number */
{
 int err;
 DBDCONNECTION*c;
 err= LINDBD_connect(cth,database,username,password,mode,cp,dbh);
 if(!err) /* create new mutex on connection */
 {
  c=getconn( SvIV(*dbh));
  if (!c)
   err= ERR_NODBH;
  else
   c->mt = newmutex();
 }
 return err;
}


int aLINDBD_disconnect(int cth,  /* context namber */
                       int dbh,  /* connection number */
                       char*action)  /* disconnect attributes : commit/rollback*/
{
 int err;
 CHAR*mt;
 DBDCONNECTION*c;
 c=getconn(dbh);
 if (!c)
  return ERR_NODBH;
 mt=c->mt;
 emutex(mt);
 err=LINDBD_disconnect(cth,dbh,action);
 lmutex(mt);
 mutex();
  MYMEM_FREE(c->mt);
  c->mt=NULL;
 unmutex();
 return err;
}

int aLINDBD_snap(int cth, /* context number */
                 SV*database) /* linter server name*/
{
 int err;
 DBDCONTEXT*ctx;
 ctx=getctx(cth);
 if (!ctx)
  return ERR_NODBH;
 emutex(ctx->mt);
 err=LINDBD_snap(cth,database);
 lmutex(ctx->mt);
 return err;
}

int aLINDBD_shut(int cth, /* context number */
                 SV*database) /* linter server name*/
{
 int err;
 DBDCONTEXT*ctx;
 ctx=getctx(cth);
 if (!ctx)
  return ERR_NODBH;
 emutex(ctx->mt);
 err=LINDBD_shut(cth,database);
 lmutex(ctx->mt);
 return err;
}

int aLINDBD_prepare(int cth,  /* context number */
                    int dbh,  /* connection number */
                    SV*stat, /* query string */  
                    SV*attr, /* attributes hash */
                    int*sh)  /* output: statement number */	/* 03.04.01 */
{
 int err;
 DBDSTATEMENT*s;

 err= LINDBD_prepare(cth,dbh,stat,attr,sh);
 if(!err) /* create new mutex on connection */
 {
  s=getstat(*sh);						/* 03.04.01 */
  if (!s)
   err= ERR_NOSH;
  else
   s->mt = newmutex();
 }
 return err;
}

int aLINDBD_paramcnt(int cth, /* context number */
                     int sh) /* statement number */
{int err;
 DBDSTATEMENT*s;
 if (!(s=getstat(sh)))
   return ERR_NOSH;
 emutex(s->mt);
 err= LINDBD_paramcnt(cth,sh);
 lmutex(s->mt);
 return err;
}

int aLINDBD_fieldcnt(int cth, /* context number */
                    int sh) /* statement number */
{int err;
 DBDSTATEMENT*s;
 if (!(s=getstat(sh)))
   return ERR_NOSH;
 emutex(s->mt);
 err= LINDBD_fieldcnt(cth,sh);
 lmutex(s->mt);
 return err;
}

int aLINDBD_titles(int cth,   /* context number */
                  int sh,    /* statement number */
                  SV*sqltypes) /* reference to hash with sql types *//* 03.04.01 */
{int err;
 DBDSTATEMENT*s;
 if (!(s=getstat(sh)))
   return ERR_NOSH;
 emutex(s->mt);
 err= LINDBD_titles(cth,sh,sqltypes);
 lmutex(s->mt);
 return err;
}

int aLINDBD_bind(int cth,  /* context number */
                int sh,  /* statement number */
                int parnum,    /* param number*/
                SV* value,    /* param value */
                SV* parlen)  /* param type */
{int err;
 DBDSTATEMENT*s;
 if (!(s=getstat(sh)))
   return ERR_NOSH;
 emutex(s->mt);
 err= LINDBD_bind(cth,sh,parnum,value,parlen);
 lmutex(s->mt);
 return err;
}

int aLINDBD_execute(int cth,  /* context number */
                   int sh,   /* statement number */
                   int*curs) /* cursor number */	/* 03.04.01 */
{int err;
 DBDSTATEMENT*s;
 if (!(s=getstat(sh)))
   return ERR_NOSH;
 emutex(s->mt);
 err= LINDBD_execute(cth,sh,curs);
 lmutex(s->mt);
 return err;
}

int aLINDBD_fetch(int cth,  /* context */
                 int sh,   /* statement number */
                 int dbh,   /* channel number */
                 SV*dir,  /* fetch direction: next, prev, first, last, num*/
                 int fst, /* fetches 1-st row*/
                 SV**cols) /* output: columns */
{int err;
 DBDSTATEMENT*s;
 if (!(s=getstat(sh)))
   return ERR_NOSH;
 emutex(s->mt);
 err= LINDBD_fetch(cth,sh,dbh,dir,fst,cols);
 lmutex(s->mt);
 return err;
}

int aLINDBD_rows(int cth,int sh) /* return total rows to fetch or -<Code of error>*/
{int err;
 DBDSTATEMENT*s;
 if (!(s=getstat(sh)))
   return ERR_NOSH;
 emutex(s->mt);
 err= LINDBD_rows(cth,sh);
 lmutex(s->mt);
 return err;
}

int aLINDBD_finish(int cth,  /* context */
                  int sh,   /* statement number */
                  int dbh)  /* channel number */
{int err;
 DBDSTATEMENT*s;
 if (!(s=getstat(sh)))
   return ERR_NOSH;
 emutex(s->mt);
 err= LINDBD_finish(cth,sh,dbh);
 lmutex(s->mt);
 mutex();
  MYMEM_FREE(s->mt);
  s->mt=NULL;
 unmutex();
 return err;
}

int aLINDBD_do(int cth,   /* context number */
              int dbh,   /* connection number */
              SV*stat,   /* statement text */
              SV*attr,   /* attributes: not used */
              SV*argv)   /* placeholders   */
{
 return LINDBD_do(cth,dbh,stat,attr,argv);
}


int aLINDBD_lrow(int cth,   /* context number */
                int dbh)   /* connection number */
{
 return LINDBD_lrow(cth,dbh);
}

int aLINDBD_ulrow(int cth, /* context number */
                 int dbh) /* connection number */
{
 return LINDBD_ulrow(cth,dbh);
}

int aLINDBD_getblob(int cth,  /* context number */
                    int sh,   /* statememt number */
                    int dbh,  /* connection number */
		    SV*fn,   /* field number */		/* 05.04.01 */
                    SV*attr, /* attributes : offset, length*/
                    SV**blob) /*output: blob portion */
{int err;
 DBDSTATEMENT*s;
 if (!(s=getstat(sh)))
   return ERR_NOSH;
 emutex(s->mt);
 err= LINDBD_getblob(cth,sh,dbh,fn,attr,blob);
 lmutex(s->mt);
 return err;
}

int aLINDBD_addblob(int cth,  /* context number */
                   int sh,   /* statememt number */
                   int dbh,  /* connection number */
		    SV*fn,   /* field number */		/* 05.04.01 */
                   SV*len, /* blob portion length */
                   SV*blob) /* blob portion */
{int err;
 DBDSTATEMENT*s;
 if (!(s=getstat(sh)))
   return ERR_NOSH;
 emutex(s->mt);
 err= LINDBD_addblob(cth,sh,dbh,fn,len,blob);
 lmutex(s->mt);
 return err;
}

int aLINDBD_delblob(int cth,  /* context number */
                   int sh,   /* statememt number */
                   int dbh,  /* connection number */
		    SV*fn,   /* field number */		/* 05.04.01 */
                   SV*attr) /* attributes : not used*/
{int err;
 DBDSTATEMENT*s;
 if (!(s=getstat(sh)))
   return ERR_NOSH;
 emutex(s->mt);
 err= LINDBD_delblob(cth,sh,dbh,fnattr);
 lmutex(s->mt);
 return err;
}

int aLINDBD_proc(int cth,    /* context number         */
                 int dbh,    /* connection number       */  
                 SV*proc,    /* proc. execute statement */
                 SV*arg,    /* reference to input/output args  */
                 SV*sql_types,/* input: array of standard sql types */
                 SV*result, /* output: result (may be cursor number ) */
                 SV*isProc, /* output: reference to 'cursor returns' flag*/
                 SV*field_cnt, /*output: fileds count that cursor has */
                 SV**fields,   /*output: ref. to fields array*/
                 SV**types)   /*output: ref. to types array*/
{int err;
 DBDCONNECTION*c;
 if (!(c=getconn(dbh)))
   return ERR_NOSH;
 emutex(c->mt);
 err= LINDBD_proc(cth,dbh,proc,arg,sql_types,result,isProc,field_cnt,fields,types);
 lmutex(c->mt);
 return err;
} 

int aLINDBD_errno(int cth,  /* context number */
                 int*errn) /* output: errno  */
{
 return LINDBD_errno(cth,errn);
}

void aLINDBD_errstr(int cth,    /* context number */
                  SV**errstr)  /* output: errstr */
{
 LINDBD_errstr(cth,errstr);
}
#endif /* DBI_THREAD */
/**************************** source end **************************************/
