 /***************************************************************************
 *                 RELEX, Inc. Linter DBI driver source code                *
 *        This is unpublished propietary source code of RELEX, Inc.         *
 *     The copyright notice does not evidence any actual or intended        *
 *                  publication of such source code.                        *
 *                                                                          *
 *                 Copyright (c) 1990-2006 RELEX Inc.                       *
 *                                                                          *
 *                   This software was developed by                         *
 *                                                                          *
 *                            RELEX, Inc.                                   *
 *                         Voronezh, Russia                                 *
 *                                                                          *
 *                        All rights reserved.                              *
 ****************************************************************************
 AUTHOR:  Andrew V Razuwaev
 ----------------------------------------------------------------------------
 PURPOSE: definitions, prototypes for Linter.xs
 ----------------------------------------------------------------------------
 REVISION HISTORY:
 06.06.00  - RAV (support Perl 5.6 changes)
 27.12.00  - RAW (MAX_ID_LEN instead directly given object name length)
 03.04.01  - RAW (memory leak)
 05.04.01  - RAW (multiple blobs)
 13.02.06 -- Gluhov (Translation of ESC sequences implemented)
 16.02.06 -- Gluhov ( Linter 6.x transaction isolation levels support )
*/

#ifndef __LINTER_H__
#define __LINTER_H__

/* Linter's datatypes */
#define LIN_CHAR                1
#define LIN_INTEGER             2
#define LIN_REAL                3
#define LIN_DATE                4
#define LIN_NUMERIC             5
#define LIN_BYTE                6
#define LIN_BLOB                7
#define LIN_SMALLINT            8
#define LIN_DOUBLE              9

/* Fetch directions */
#define FETCH_FIRST             1
#define FETCH_LAST              2
#define FETCH_NEXT              3
#define FETCH_PREV              4
#define FETCH_ABSNUM            5

/* Transaction modes */
#define TM_AUTOCOMMIT           0x00000000
#define TM_OPTIMISTIC           0x00000100
#define TM_EXCLUSIVE            0x00000400

/* Supported codepages */
#define CP_866                  0x00000000
#define CP_1251                 0x00004000
#define CP_KOI8                 0x00008000

#define TRANSLATE_ESC           0x00100000                 /* 13.02.06 */

/* Transaction isolation levels */                         /* 16.02.06 */
#define TIL_OPTIMISTIC        0x00000100
#define TIL_READ_COMMITTED    0x01000000  
#define TIL_READ_UNCOMMITTED  0x04000000  
#define TIL_SERIALIZABLE      0x02000000

#if ((PERL_VERSION == 5 && PERL_SUBVERSION >= 600) || PERL_VERSION >= 6) /* 06.06.00 */
#  define na       PL_na
#  define sv_undef PL_sv_undef
#endif

#ifdef __cplusplus
extern "C"{
#endif

int LINDBD_init();

int LINDBD_version(int cth,SV*major,SV*minor,SV*build); /* Linter DBD version */
int LINDBD_server_version(int cth,SV*database,SV*major,SV*minor,SV*build); /*Linter SQL server version */
int LINDBD_ver_compat(int cth,int drvmajor,int drvminor,int srvmajor,int srvminor);/* version compatibility check*/

int LINDBD_allocctx(SV**cth); /* output: context number */

int LINDBD_freectx(SV*cth); /* input: context number */

int LINDBD_disconnect_all(void);

int LINDBD_connect(int cth,      /* current context number */
                   SV* database, /* database name (node) */
                   SV* username, /* username : casesens.; less or eq. then MAX_ID_LEN syms*//* 27.12.00 */
                   SV* password, /* username : casesens.; less or eq. then 18 syms*/
                   SV* mode,     /* channel open mode: autocommit,exclusive,optimistic */
                   SV* cp,       /* codepage */
                   SV** dbh);    /* output: connection number */

int LINDBD_disconnect(int cth,  /* context namber */
                      int dbh,  /* connection number */
                      char* act);  /*disconnect with commit /rollback*/

int LINDBD_cursor_open(int cth,  /* context number    */
                       int dbh,  /* connection number */
                       int mode, /* child channel mode*/
                       char*name,/* child channel name or NULL*/
                       int*crh);   /* output: cursor number reference *//* 03.04.01 */

int LINDBD_cursor_close(int cth,  /* context number */
                        int dbh);  /* cursor number  */


int LINDBD_snap(int cth, /* context number */
                SV*database); /* linter server name*/

int LINDBD_shut(int cth, /* context number */
                SV*database); /* linter server name*/

int LINDBD_prepare(int cth,  /* context number */
                   int dbh,  /* connection number */
                   SV*stat, /* query string */  
                   SV*attr, /* attributes hash */
                   int*sh);  /* output: statement number */	/* 03.04.01 */

int LINDBD_paramcnt(int cth, /* context number */
                    int sh); /* statement number */
			     /* return: query paramter's count */

int LINDBD_fieldcnt(int cth, /* context number */
                    int sh); /* statement number */
			     /* return: output fields count */

int LINDBD_bind(int cth,  /* context number */
                int sh,  /* statement number */
                int parnum,    /* param number*/
                SV*value,    /* param value */
                SV*partyp);  /* param type */

int LINDBD_execute(int cth,  /* context number */
                   int sh,   /* statement number */
                   int*curs); /* output: channel consists selected rows or same channel if non-select query*//* 03.04.01 */
                                
int LINDBD_titles(int cth,   /* context number */
                  int sh,    /* statement number */
                  SV*sqltypes, /* reference to hash with sql types */
                  SV**tlist, /* output: column names */
                  SV**types); /* output: column types */

int LINDBD_fetch(int cth,  /* context */
                 int sh,   /* statement number */
                 int dbh,   /*connection number */
                 SV*dir,  /* fetch direction*/
                 int fst,  /* if 1: fetches 1-st row*/
                 SV**cols); /* output: columns */

int LINDBD_rows(int cth,int sh); /* return total rows to fetch or -<Code of error>*/

int LINDBD_finish(int cth,  /* context */
                  int sh,   /* statement number */
                 int dbh);   /*connection number */

int LINDBD_do(int cth,    /* context number */
              int dbh,    /* connection number */
              SV*stat,   /* stement text */
              SV*attr,   /* stement attributes: not used */
              SV*argv);   /* placeholders  reference */

int LINDBD_getblob(int cth,  /* context number */
                   int sh,   /* statememt number */
                   int dbh,   /* connection number */
                   SV*fn,    /* blob field number in dataset */ /* 05.04.01 */
                   SV*attr, /* attributes : offset & portion length */
                   SV**blob); /*output:reference to blob portion */

int LINDBD_addblob(int cth,  /* context number */
                   int sh,   /* statememt number */
                   int dbh,  /* connection number */
                   SV*fn,    /* blob field number in dataset */ /* 05.04.01 */
                   SV*len, /* blob portion length */
                   SV*blob); /* blob portion */

int LINDBD_delblob(int cth,  /* context number */
                   int sh,   /* statememt number */
                   int dbh,   /* connection number */
                   SV*fn,    /* blob field number in dataset */ /* 05.04.01 */
                   SV*attr); /* attributes */

int LINDBD_proc(int cth,    /* context number         */
                int dbh,    /* connection number      */  
                SV*proc,    /* proc. name             */
                SV*arg,    /* reference to input/output args  */
                SV*sql_types,/* input: array of standard sql types */
                SV*result, /* output: result (may be cursor number ) */
                SV*isProc, /* output: reference to 'cursor returns' flag*/
                SV*field_cnt, /*output: fileds count that cursor has */
                SV**fields,   /*output: ref. to fields array*/
                SV**types);   /*output: ref. to types array*/

int LINDBD_commit(int cth, /* context number */
                  int dbh); /* connection number */

int LINDBD_rollback(int cth, /* context number */
                    int dbh, /* connection number */
                    SV*spoint); /* savepoint name : not supported yet*/

int LINDBD_savepoint(int cth, /* context number */
                     int dbh, /* connection number */
                     SV*spoint); /*savepoint name : not supported yet*/


int LINDBD_lrow(int cth,   /* context number */
                int dbh);   /* connection number */

int LINDBD_ulrow(int cth,   /* context number */
                int dbh);   /* connection number */

int LINDBD_errno(int cth,  /* context number */
                 int*errn); /* output: errno  */

void LINDBD_errstr(int cth,    /* context number */
                  SV**errstr); /* output: errstr */

void LINDBD_errstate(int cth,  /* context number    */
                    int dbh,  /* connecion numder  */
                    int*errn); /* output: error code*/

void LINDBD_errmsg(int cth,  /* context number    */
                  int dbh,  /* connecion numder  */
                  SV*msg);  /* output: error message text*/

void LINDBD_sqlstate(int cth,     /* context number    */
                    int dbh,     /* connecion numder  */
                    SV*sqlstat); /* output: SQLSTATE  */

int LINDBD_linter(SV*arg1,SV*arg2,SV*arg3,SV*arg4,SV*arg5);

void LINDBD_nativesql(SV* stat,     /* query with ODBS ESC sequences */
                      SV* res );    /* output: native linter query */


#ifdef __cplusplus
}
#endif

#endif /*__LINTER_H__*/
