# /***************************************************************************
# *           RELEX, Inc. Client source include file                         *
# *  This is unpublished propietary source code of RELEX, Inc.               *
# *  The copyright notice does not evidence any actual or intended           *
# *  publication of such source code.                                        *
# *                                                                          *
# *                 Copyright (c) 1999-2010 RELEX Inc.                       *
# *                                                                          *
# *                   This software was developed by                         *
# *                                                                          *
# *                            RELEX, Inc.                                   *
# *                         Voronezh, Russia                                 *
# *                                                                          *
# *                        All rights reserved.                              *
# ****************************************************************************
# FILE:    LinPerl.pm
# ----------------------------------------------------------------------------
# AUTHOR:  Alexander V. Kefeli
# ----------------------------------------------------------------------------
# PURPOSE: Interface from Perl to Linter SQL Server
# ----------------------------------------------------------------------------
# REVISION HISTORY:
#
# **.**.** -- Kent (file creation)
# 17.02.99 -- RAV  (change package name to LinPerl)
# 17.02.99 -- RAV  (insert direct inter() call)
# 16.12.99 -- Kent (New cursor options for 5.7)
# 24.02.00 -- Kent (LAST ROWID cursor option)
# 11.05.00 -- Kent (New function `Cleanup')
# 08.11.00 -- Kent (Add stored procedures exception)
# 30.11.09 -- Glukhov (BindParamArray wrapper added)
# 24.12.09 -- Glukhov (BlobGetSize wrapper added, new functions export added)
# 09.02.10 -- Glukhov (Module description updated, constant export fixed)
# 15.02.10 -- Glukhov (CO_FETCH_BLOBS_AS_USUAL_DATA cursor option added)
# 26.02.10 -- Glukhov (New cursor options added)
#
# --------------------------------------------------------------------------*/

package LinPerl;

$VERSION = '1.10';

require Exporter;
require DynaLoader;
require AutoLoader;

@ISA    = qw(Exporter DynaLoader);
@EXPORT = qw(&OpenConnect
             &CloseConnect
             &GetConnectInfo
             &OpenCursor
             &CloseCursor
             &GetCursorOption
             &SetCursorOption
             &SetCodepage
             &ExecDirect
             &Prepare
             &BindParameter
             &BindParamArray
             &Execute
             &Fetch
             &BindColumn
             &UnbindColumn
             &GetDataColumn
             &GetDataRow
             &GetDataArray
             &GetDataRows
             &GETM
             &TestNull
             &BLOBAppend
             &BLOBAppendEx
             &BLOBAdd
             &BLOBAddEx
             &BLOBClear
             &BLOBGetSize
             &BLOBGetType
             &BLOBPurge
             &BLOBGetData
             &BLOBFetch
             &Commit
             &Rollback
             &GetColInfo
             &GetError
             &GetErrorMsg
             &Cleanup
             CO_CURNAME
             CO_SYNCMODE
             CO_ASYNCMODE
             CO_ASYNCDONE
             CO_DTFORMAT
             CO_ERRPOS
             CO_ERRROW
             CO_COLCOUNT
             CO_CONNECTID
             CO_CURROW
             CO_CURROWID
             CO_NODENAME
             CO_ROWCOUNT
             CO_SYNCMODE
             CO_TRANSMODE
             CO_PRIORITY
             CO_QUERY_PRIORITY
             CO_CANCEL
             CO_PAUSE
             CO_CONTINUE
             CO_LAST_ROWID
             CO_FETCH_BLOBS_AS_USUAL_DATA
             CP_866
             CP_1251
             CP_KOI8
             EXC_NONE
             EXC_DIVZERO
             EXC_UNDEFPROC
             EXC_BADPARAM
             EXC_BADINDEX
             EXC_BADRETVAL
             EXC_NULLDATA
             EXC_NOMEM
             EXC_BADCURSOR
             EXC_CURNOTOPEN
             EXC_BADCODE
             EXC_TRIGQUERY
             EXC_APPENDNOTSTARTED
             EXC_QUERYWHENAPPEND
             EXC_APPENDACTIVE
             EXC_APPLICATIONERROR
             EXC_INVTRSTATE
             EXC_CUSTOM
             FETCH_FIRST
             FETCH_LAST
             FETCH_NEXT
             FETCH_PREV
             FETCH_ABSNUM
             LDT_CHAR
             LDT_INTEGER
             LDT_REAL
             LDT_DATE
             LDT_NUMERIC
             LDT_BYTE
             LDT_BLOB
             LDT_VARCHAR
             LDT_VARBYTE
             LDT_BOOLEAN
             LDT_NCHAR
             LDT_NVARCHAR
             LPERR_SUCCESS
             LPERR_NOMEMORY
             LPERR_INVCURSOR
             LPERR_INVCOLNUM
             LPERR_INVPARAMNUM
             LPERR_INVSTATE
             LPERR_INVDIRECT
             LPERR_UNKNOWNOPT
             LPERR_BADDTFORMAT
             LPERR_NOBLOBCOL
             LPERR_NOPARENT
             LPERR_INVPARAMETER
             LPERR_LINERROR
             LPERR_NOTIMPLEMENTED
             TM_AUTOCOMMIT
             TM_OPTIMISTIC
             TM_EXCLUSIVE
);

sub AUTOLOAD {
    my($constname, $val);
    ($constname = $AUTOLOAD) =~ s/.*:://;
    $val = constant($constname, @_ ? $_[0] : 0);
    if ($! != 0) {
      print "Constant $constname has not defined in package Linter\n";
    }
    eval "sub $AUTOLOAD { $val }";
    goto &$AUTOLOAD;
}

sub BindParamArray {
  $argsCount = @_;
  if ($argsCount == 2 && ref($_[1]) eq 'ARRAY') {
    BindParamArrayInternal($_[0],$_[1],-1,-1);
  }
  elsif ($argsCount == 4 && ref($_[1]) eq 'ARRAY') {    
    BindParamArrayInternal($_[0],$_[1],$_[2],$_[3]);
  }
  else {
    printf ("Usage: %s(%s) or %s(%s)", "LinPerl::BindParamArray", "cursor, av","LinPerl::BindParamArray", "cursor, av, begin, end") ;
  }

}

sub BLOBGetSize {
  $argsCount = @_;

  if ($argsCount == 2) {
    BLOBGetSizeInternal($_[0],$_[1],-1);
  }
  else {
    BLOBGetSizeInternal(@_);
  }
}

bootstrap LinPerl $VERSION;

1;

__END__

=head1 NAME

LinPerl - Perl extension for Linter SQL Server

=head1 SYNOPSIS

  use LinPerl;

=head1 DESCRIPTION

This module is an interface from Perl to Linter SQL Server.
It's implemented for Win32 and various UNIX platforms.

=head1 INSTALLATION

  1. Create subdirectory `LinPerl` in $PERL5LIB/auto
     (or %PERL5LIB%\auto under Win32 platforms).
  2. Copy file LinPerl.so (LinPerl.dll under Win32) into this directory.
  3. Copy file LinPerl.pm into $PERL5LIB (%PERL5LIB% under Win32).
  4. Enjoy using Linter.

=head1 EXPORTED FUNCTIONS

All functions return completion status - integer value, which contains module
internal error code or error code returned by Linter SQL Server. 
Result[s] of any function are always returned via parameter[s]. 
Module internal error codes are always negative. Zero code means succesful 
completion.


OpenConnect(username, password, server, mode, connect);

The OpenConnect function opens a new connection to the Linter SQL Server.
+-----------+-----+---------------------------------------------------+
| Parameter | Out | Description                                       |
+-----------+-----+---------------------------------------------------+
| user uame |  -  | User name     (1)(2)                              |
| password  |  -  | User password (1)(2)                              |
| server    |  -  | Name of Linter server (3)                         |
| mode      |  -  | Codepage and transaction mode  (4)                |
| connect   |  +  | Identifier of the new connection                  |
+-----------+-----+---------------------------------------------------+
1 - maximum 18 characters
2 - case-sensitive
3 - maximum 8 characters
4 - should be zero or an OR-combination of constants "Transaction modes" and
    "Supported codepages".

Example:
$err = OpenConnect("KENT", "ALEX", "LINTER", CP_866 | MODE_EXCLUSIVE, $con);
$err && ... to do something for error handling ...

See Also:
CloseConnect


CloseConnect(connect);

The CloseConnect function closes an open connection.
+-----------+-----+---------------------------------------------------+
| Parameter | Out | Description                                       |
+-----------+-----+---------------------------------------------------+
| connect   |  -  | Connection identifier                             |
+-----------+-----+---------------------------------------------------+

Example:
$err = CloseConnect($con);
$err && ... to do something for error handling ...

See Also:
OpenConnect


GetConnectInfo(cursor, info);

The GetConnectInfo function return information about the connection 
and database.
+-----------+-----+---------------------------------------------------+
| Parameter | Out | Description                                       |
+-----------+-----+---------------------------------------------------+
| cursor    |  -  | Cursor identifier                                 |
| info      |  +  | Perl variable (hash reference) for result (1)     |
+-----------+-----+---------------------------------------------------+
1 - see "Available connect info keys" for more information.

Example:
$err = GetConnectInfo($cur, \%info);
$err && ... to do something for error handling ...

See Also:
OpenConnect, GetColInfo


SetCodepage(codepage);

The SetCursorOption function gets a cursor option[s].
+-----------+-----+---------------------------------------------------+
| Parameter | Out | Description                                       |
+-----------+-----+---------------------------------------------------+
| codepage  |  -  | New codepage name as string                       |
+-----------+-----+---------------------------------------------------+

Example:
$err = SetCodepage("KOI8-R");
$err && ... to do something for error handling ...

See Also:
OpenConnect, GetConnectInfo


OpenCursor(connect, name, cursor)

The OpenCursor function opens a new cursor.
+-----------+-----+---------------------------------------------------+
| Parameter | Out | Description                                       |
+-----------+-----+---------------------------------------------------+
| connect   |  -  | Parent connection                                 |
| name      |  -  | Cursor name  (1)(2)                               |
| cursor    |  +  | Identifier of the new cursor                      |
+-----------+-----+---------------------------------------------------+
1 - maximum 18 characters
2 - not impelemented in current version

Note:
Transaction mode and codepage for cursor are inherited from the
parent connection.

Example:
$err = OpenCursor($con, "MY_CURSOR", $cur);
$err && ... to do something for error handling ...

See Also:
OpenConnect, CloseCursor


CloseCursor(cursor);

The CloseCursor function closes an open cursor.
+-----------+-----+---------------------------------------------------+
| Parameter | Out | Description                                       |
+-----------+-----+---------------------------------------------------+
| cursor    |  -  | Cursor identifier                                 |
+-----------+-----+---------------------------------------------------+

Example:
$err = CloseCursor($cur);
$err && ... to do something for error handling ...

See Also:
OpenCursor


GetCursorOption(cursor, option, value);

The GetCursorOption function gets a cursor option[s].
+-----------+-----+---------------------------------------------------+
| Parameter | Out | Description                                       |
+-----------+-----+---------------------------------------------------+
| cursor    |  -  | Cursor identifier                                 |
| option    |  -  | Option                   (1)                      |
| value     |  +  | Option value                                      |
+-----------+-----+---------------------------------------------------+
1 - should be one of 'Cursor/connection options'

Example:
$err = GetCursorOption($cursor, CO_ROWCOUNT, $rows);
$err && ... to do something for error handling ...

See Also:
SetCursorOption, GetConnectInfo, GetColInfo


SetCursorOption(cursor, option, value);

The SetCursorOption function gets a cursor option[s].
+-----------+-----+---------------------------------------------------+
| Parameter | Out | Description                                       |
+-----------+-----+---------------------------------------------------+
| cursor    |  -  | Cursor identifier                                 |
| option    |  -  | Option                   (1)                      |
| value     |  -  | Option value                                      |
+-----------+-----+---------------------------------------------------+
1 - should be one of 'Cursor/connection options'

Example:
$err = SetCursorOption($cursor, CO_PRIORITY, 10);
$err && ... to do something for error handling ...

See Also:
GetCursorOption


ExecDirect(cursor, query);

The ExecDirect function executes query without preparing.
+-----------+-----+---------------------------------------------------+
| Parameter | Out | Description                                       |
+-----------+-----+---------------------------------------------------+
| cursor    |  -  | Cursor identifier                                 |
| query     |  -  | Query body in plane text (1)                      |
+-----------+-----+---------------------------------------------------+
1 - must be ended by ';' (semicolon).

Example:
$err = ExecDirect($cur, "select * from auto;");
$err && ... to do something for error handling ...

See Also:
Fetch, Prepare, BindParameter, BindParamArray, Execute


Prepare(cursor, query);

The Prepare function prepares query for execution.
+-----------+-----+---------------------------------------------------+
| Parameter | Out | Description                                       |
+-----------+-----+---------------------------------------------------+
| cursor    |  -  | Cursor identifier                                 |
| query     |  -  | Query body in plane text (1)                      |
+-----------+-----+---------------------------------------------------+
1 - must be ended by ';' (semicolon).

Example:
$err = Prepare($cur, "select * from auto where personid = ?;");
$err && ... to do something for error handling ...

See Also:
BindParameter, BindParamArray, Execute, ExecDirect


BindParameter(cursor, number, param);

The BindParameter function binds variables to query parameters.
+-----------+-----+---------------------------------------------------+
| Parameter | Out | Description                                       |
+-----------+-----+---------------------------------------------------+
| cursor    |  -  | Cursor identifier                                 |
| number    |  -  | Parameter number (1)                              |
| param     |  -  | Perl variable                                     |
+-----------+-----+---------------------------------------------------+
1 - parameter numbering starts from 1.

Note:
If a parameter has been already bound, it will be unbound automatically.

Example:
$err = BindParameter($cur, 1, $my_param);
$err && ... to do something for error handling ...

See Also:
Prepare, Execute, BindParamArray


BindParamArray(cursor, param[, begin, end]);

The BindParamArray function binds Perl array to query parameters.
+-----------+-----+---------------------------------------------------+
| Parameter | Out | Description                                       |
+-----------+-----+---------------------------------------------------+
| cursor    |  -  | Cursor identifier                                 |
| param     |  -  | Perl array's reference                            |
| begin     |  -  | First parameter number (1)                        |
| end       |  -  | Last bparameter number                            |
+-----------+-----+---------------------------------------------------+
1 - parameter numbering starts from 1.

Note:
If parameters have been already bound, they will be unbound automatically.

Example:
$err = BindParamArray($cur, \@my_param);
$err && ... to do something for error handling ...

See Also:
Prepare, Execute, BindParameter 


Execute(cursor)

The Execute function executes query, which was prepared by Prepare.
+-----------+-----+---------------------------------------------------+
| Parameter | Out | Description                                       |
+-----------+-----+---------------------------------------------------+
| cursor    |  -  | Cursor identifier                                 |
+-----------+-----+---------------------------------------------------+

Example:
$err = Execute($cur);
$err && ... to do something for error handling ...

See Also:
ExecDirect, Prepare, BindParameter, BindParamArray, Fetch


Fetch(cursor, direct, number);

The Fetch function fetches query result row by row.
+-----------+-----+---------------------------------------------------+
| Parameter | Out | Description                                       |
+-----------+-----+---------------------------------------------------+
| cursor    |  -  | Cursor identifier                                 |
| direct    |  -  | Direction for fetch (1)                           |
| number    |  -  | Absolute row number (2)(3)                        |
+-----------+-----+---------------------------------------------------+
1 - see "Directions for fetch" for direction code values.
2 - used with FETCH_ABSNUM only.
3 - row numbering starts from 1.

Example:
$err = Fetch($cur, FETCH_NEXT, 0);
$err = Fetch($cur, FETCH_ABSNUM, 100);
$err && ... to do something for error handling ...

See Also:
ExecDirect, Execute


BindColumn(cursor, number, param);

The BindColumn function binds variables to query result columns.
+-----------+-----+---------------------------------------------------+
| Parameter | Out | Description                                       |
+-----------+-----+---------------------------------------------------+
| cursor    |  -  | Cursor identifier                                 |
| number    |  -  | Column number (1)                                 |
| param     |  -  | Perl variable                                     |
+-----------+-----+---------------------------------------------------+
1 - column numbering starts from 1.

Example:
$err = BindColumn($cur, 1, $make);
$err && ... to do something for error handling ...

See Also:
GetDataColumn, Fetch, UnbindColumn


UnbindColumn(cursor, number);

The UnbindColumn function unbinds variables from query result columns.
+-----------+-----+---------------------------------------------------+
| Parameter | Out | Description                                       |
+-----------+-----+---------------------------------------------------+
| cursor    |  -  | Cursor identifier                                 |
| number    |  -  | Column number (1)                                 |
+-----------+-----+---------------------------------------------------+
1 - column numbering starts from 1.

Example:
$err = UnbindColumn($cur, 1);
$err && ... to do something for error handling ...

See Also:
BindColumn


GetDataColumn(cursor, number, var);

The GetDataColumn function gets data for one column.
+-----------+-----+---------------------------------------------------+
| Parameter | Out | Description                                       |
+-----------+-----+---------------------------------------------------+
| cursor    |  -  | Cursor identifier                                 |
| number    |  -  | Column number (1)                                 |
| var       |  +  | Perl variable for output (2)                      |
+-----------+-----+---------------------------------------------------+
1 - columns numbering starts from 1.
2 - result is NULL-value var will be set to undefined state.

Example:
$err = GetDataColumn($cur, 1, $make);
$err && ... to do something for error handling ...

See Also:
ExecDirect, Execute, Fetch, BLOBGetData, GetDataRow


GetDataRow(cursor, var);

The GetDataRow function gets data for current row as array.
+-----------+-----+---------------------------------------------------+
| Parameter | Out | Description                                       |
+-----------+-----+---------------------------------------------------+
| cursor    |  -  | Cursor identifier                                 |
| var       |  +  | Perl variable (array reference) for output        |
+-----------+-----+---------------------------------------------------+

Example:
$err = GetDataRow($cur, \@res);
$err && ... to do something for error handling ...

See Also:
ExecDirect, Execute, Fetch, GetDataColumn, GetDataArray, GetDataRows


GetDataArray(cursor, var);

The GetDataRow function gets data for current row as hash.
+-----------+-----+---------------------------------------------------+
| Parameter | Out | Description                                       |
+-----------+-----+---------------------------------------------------+
| cursor    |  -  | Cursor identifier                                 |
| var       |  +  | Perl variable (hash reference) for output         |
+-----------+-----+---------------------------------------------------+

Example:
$err = GetDataRow($cur, \%res);
$err && ... to do something for error handling ...

See Also:
ExecDirect, Execute, Fetch, GetDataColumn, GetDataRow


GetDataRows(cursor, var, rows);

The GetDataRows function gets data for several rows as array.
+-----------+-----+---------------------------------------------------+
| Parameter | Out | Description                                       |
+-----------+-----+---------------------------------------------------+
| cursor    |  -  | Cursor identifier                                 |
| var       |  +  | Perl variable (array reference) for output        |
| rows      |  -  | Rows count to get                                 |
+-----------+-----+---------------------------------------------------+

Example:
$err = GetDataRows($cur, \@res, 5);
$err && ... to do something for error handling ...

See Also:
ExecDirect, Execute, Fetch, GetDataColumn, GetDataRow, GetDataArray


GETM(cursor, var, number);

The GETM function gets buffered data for definite row as array.
+-----------+-----+---------------------------------------------------+
| Parameter | Out | Description                                       |
+-----------+-----+---------------------------------------------------+
| cursor    |  -  | Cursor identifier                                 |
| var       |  +  | Perl variable (array reference) for output        |
| number    |  -  | Row number (1)                                    |
+-----------+-----+---------------------------------------------------+
1 - rows numbering starts from 1.

Example:
$err = GETM($cur, \@res, 5);
$err && ... to do something for error handling ...

See Also:
ExecDirect, Execute, Fetch, GetDataColumn, GetDataRow, GetDataArray


TestNull(cursor, number, var);

The TestNull function tests data for NULL value.
+-----------+-----+---------------------------------------------------+
| Parameter | Out | Description                                       |
+-----------+-----+---------------------------------------------------+
| cursor    |  -  | Cursor identifier                                 |
| number    |  -  | Column number (1)                                 |
| var       |  +  | Perl variable for output                          |
+-----------+-----+---------------------------------------------------+
1 - columns numbering starts from 1.

Example:
$err = TestNull($cur, 1, $is_null);
$err && ... to do something for error handling ...
if ($is_null != 0) ...

See Also:
GetDataColumn, BLOBGetData, GetDataRow, GetDataArray


BLOBAppend(cursor, var);

The BLOBAppend function appends portion to first BLOB value of the 
current answer row.
+-----------+-----+---------------------------------------------------+
| Parameter | Out | Description                                       |
+-----------+-----+---------------------------------------------------+
| cursor    |  -  | Cursor identifier                                 |
| var       |  -  | Perl variable which contains BLOB body            |
+-----------+-----+---------------------------------------------------+

Example:
$err = BLOBAppend($cur, $blob);
$err && ... to do something for error handling ...

See Also:
BLOBClear, BLOBGetSize, BLOBGetData, BLOBAdd, BLOBAppendEx


BLOBAppendEx(cursor, var, type);

The BLOBAppendEx function appends portion to first BLOB value of the 
current answer row. If it is a first portition, BLOB type settings.
+-----------+-----+---------------------------------------------------+
| Parameter | Out | Description                                       |
+-----------+-----+---------------------------------------------------+
| cursor    |  -  | Cursor identifier                                 |
| var       |  -  | Perl variable which contains BLOB body            |
| type      |  -  | Numerical BLOB type                               |
+-----------+-----+---------------------------------------------------+

Example:
$err = BLOBAppendEx($cur, $blob, $type);
$err && ... to do something for error handling ...

See Also:
BLOBAppend, BLOBClear, BLOBGetSize, BLOBGetData, BLOBAddEx


BLOBAdd(cursor, var, number);

The BLOBAdd function appends portion to certain BLOB value of the 
current answer row.
+-----------+-----+---------------------------------------------------+
| Parameter | Out | Description                                       |
+-----------+-----+---------------------------------------------------+
| cursor    |  -  | Cursor identifier                                 |
| var       |  -  | Perl variable which contains BLOB body            |
| number    |  -  | BLOB column number (1)                            |
+-----------+-----+---------------------------------------------------+
1 - columns numbering starts from 1.

Note:
number ignored for Linter SQL Server version before 5.9.

Example:
$err = BLOBAdd($cur, $blob, 3);
$err && ... to do something for error handling ...

See Also:
BLOBAppend, BLOBAddEx, BLOBPurge, BLOBFetch


BLOBAddEx(cursor, var, number, type);

The BLOBAddEx function appends portion to certain BLOB value of the 
current answer row. If it is a first portition, BLOB type settings.
+-----------+-----+---------------------------------------------------+
| Parameter | Out | Description                                       |
+-----------+-----+---------------------------------------------------+
| cursor    |  -  | Cursor identifier                                 |
| var       |  -  | Perl variable which contains BLOB body            |
| number    |  -  | BLOB column number (1)                            |
| type      |  -  | Numerical BLOB type                               |
+-----------+-----+---------------------------------------------------+
1 - columns numbering starts from 1.

Note:
number ignored for Linter SQL Server version before 5.9.

Example:
$err = BLOBAddEx($cur, $blob, 3, $type);
$err && ... to do something for error handling ...

See Also:
BLOBAppend, BLOBAppendEx, BLOBAdd, BLOBPurge, BLOBFetch


BLOBClear(cursor);

The BLOBClear function clears first BLOB value of the current answer 
row.
+-----------+-----+---------------------------------------------------+
| Parameter | Out | Description                                       |
+-----------+-----+---------------------------------------------------+
| cursor    |  -  | Cursor identifier                                 |
+-----------+-----+---------------------------------------------------+

Example:
$err = BLOBClear($cur);
$err && ... to do something for error handling ...

See Also:
BLOBAppend, BLOBGetSize, BLOBGetData, BLOBPurge


BLOBPurge(cursor, number);

The BLOBClear function clears certain BLOB value of the current answer 
row.
+-----------+-----+---------------------------------------------------+
| Parameter | Out | Description                                       |
+-----------+-----+---------------------------------------------------+
| cursor    |  -  | Cursor identifier                                 |
| number    |  -  | Column number (1)                                 |
+-----------+-----+---------------------------------------------------+
1 - columns numbering starts from 1.

Note:
number ignored for Linter SQL Server version before 5.9.

Example:
$err = BLOBPurge($cur, 3);
$err && ... to do something for error handling ...

See Also:
BLOBAppend, BLOBAdd, BLOBGetSize, BLOBGetData, BLOBFetch, BLOBClear


BLOBGetSize(cursor, var[, number]);

The BLOBGetSize function gets size of BLOB value of the current 
answer row.
+-----------+-----+---------------------------------------------------+
| Parameter | Out | Description                                       |
+-----------+-----+---------------------------------------------------+
| cursor    |  -  | Cursor identifier                                 |
| var       |  +  | Perl variable for output                          |
| number    |  -  | Column number (1)                                 |
+-----------+-----+---------------------------------------------------+
1 - columns numbering starts from 1.

Example:
$err = BLOBGetSize($cur, $blob_size);
$err && ... to do something for error handling ...

See Also:
BLOBAppend, BLOBAdd, BLOBClear, BLOBPurge, BLOBGetData, BLOBFetch. BLOBGetType


BLOBGetType(cursor, var, number);

The BLOBGetType function gets type of BLOB value of the current 
answer row.
+-----------+-----+---------------------------------------------------+
| Parameter | Out | Description                                       |
+-----------+-----+---------------------------------------------------+
| cursor    |  -  | Cursor identifier                                 |
| var       |  +  | Perl variable for output                          |
| number    |  -  | Column number (1)                                 |
+-----------+-----+---------------------------------------------------+
1 - columns numbering starts from 1.

Example:
$err = BLOBGetType($cur, $blob_size, 3);
$err && ... to do something for error handling ...

See Also:
BLOBAppendEx, BLOBAddEx, BLOBGetSize


BLOBGetData(cursor, pos, len, var);

The BLOBGetData function gets portion of first BLOB value of the current 
answer row.
+-----------+-----+---------------------------------------------------+
| Parameter | Out | Description                                       |
+-----------+-----+---------------------------------------------------+
| cursor    |  -  | Cursor identifier                                 |
| pos       |  -  | Starting position (1)                             |
| len       | -+  | Perl var. which sets size of BLOB portion (2)     |
| var       |  +  | Perl variable for output                          |
+-----------+-----+---------------------------------------------------+
1 - position numbering starts from 1.
2 - after function completion this parameter contain number of bytes 
    which was really read.

Example:
$err = BLOBGetData($cur, 1, 1000, $blob);
$err && ... to do something for error handling ...

See Also:
BLOBAppend, BLOBClear, BLOBGetSize, GetDataColumn, BLOBFetch


BLOBFetch(cursor, pos, len, var, number);

The BLOBFetch function gets portion of certain BLOB value of the current 
answer row.
+-----------+-----+---------------------------------------------------+
| Parameter | Out | Description                                       |
+-----------+-----+---------------------------------------------------+
| cursor    |  -  | Cursor identifier                                 |
| pos       |  -  | Starting position (1)                             |
| len       | -+  | Perl var. which sets size of BLOB portion (2)     |
| var       |  +  | Perl variable for output                          |
| number    |  -  | Column number (3)                                 |
+-----------+-----+---------------------------------------------------+
1 - position numbering starts from 1.
2 - after function completion this parameter contain number of bytes 
    which was really read.
3 - columns numbering starts from 1.

Note:
number ignored for Linter SQL Server version before 5.9.

Example:
$err = BLOBFetch($cur, 1, 1000, $blob, $column);
$err && ... to do something for error handling ...

See Also:
BLOBAppend, BLOBAdd, BLOBClear, BLOBPurge, BLOBGetData, GetDataColumn


Commit(cursor);

The Commit function commits results of the current transaction.
+-----------+-----+---------------------------------------------------+
| Parameter | Out | Description                                       |
+-----------+-----+---------------------------------------------------+
| cursor    |  -  | Cursor identifier                                 |
+-----------+-----+---------------------------------------------------+

Example:
$err = Commit($cur);
$err && ... to do something for error handling ...

See Also:
ExecDirect, Execute, Rollback


Rollback(cursor);

The Rollback function rolls back results of the current transaction.
+-----------+-----+---------------------------------------------------+
| Parameter | Out | Description                                       |
+-----------+-----+---------------------------------------------------+
| cursor    |  -  | Cursor identifier                                 |
+-----------+-----+---------------------------------------------------+

Example:
$err = Rollback($cur);
$err && ... to do something for error handling ...

See Also:
ExecDirect, Execute, Commit


GetColInfo(cursor, number, info);

The GetColInfo function return information about the specified column.
+-----------+-----+---------------------------------------------------+
| Parameter | Out | Description                                       |
+-----------+-----+---------------------------------------------------+
| cursor    |  -  | Cursor identifier                                 |
| number    |  -  | Column number (1)                                 |
| info      |  +  | Perl variable (hash reference) for column info (2)|
+-----------+-----+---------------------------------------------------+
1 - columns numbering starts from 1.
2 - see "Available column info keys" for more information.

Example:
$err = GetColInfo($cur, 1, \%info);
$err && ... to do something for error handling ...

See Also:
ExecDirect, Execute, GetConnectInfo, GetCursorOption


GetError(cursor, linerr, syserr);

The GetError functions return Linter SQL Server error code and OS error code.
+-----------+-----+---------------------------------------------------+
| Parameter | Out | Description                                       |
+-----------+-----+---------------------------------------------------+
| cursor    |  -  | Cursor identifier                                 |
| linerr    |  +  | Perl variable for Linter SQL Server error code    |
| syserr    |  +  | Perl variable for OS error code                   |
+-----------+-----+---------------------------------------------------+

Example:
$err = GetError($cur, $linerr, $syserr);
$err && ... to do something for error handling ...

See Also:
GetErrorMsg


GetErrorMsg(cursor, linerr, errmsg)

The GetErrorMsg function returns text message for Linter SQL Server error.
+-----------+-----+---------------------------------------------------+
| Parameter | Out | Description                                       |
+-----------+-----+---------------------------------------------------+
| cursor    |  -  | Cursor identifier                                 |
| linerr    |  -  | Linter SQL Server error code                      |
| errmsg    |  +  | Perl variable for message text                    |
+-----------+-----+---------------------------------------------------+

Note:
Table "SYSTEM.ERRORS" must exist in database. Otherwise GetErrorMsg returns
an empty string.

Example:
$err = GetErrorMsg($cur, $linerr, $errmsg);
$err && ... to do something for error handling ...

See Also:
GetError

=head1 Exported constants

 Available datatypes:
 +--------------+-----------+-------------+
 |  Macro name  | Type name | Max. length |
 +--------------+-----------+-------------+
 | LDT_CHAR     | char      |     4000*   |
 | LDT_INTEGER  | integer   |        4    |
 | LDT_REAL     | real      |        4    |
 | LDT_DATE     | timestamp |       16    |
 | LDT_NUMERIC  | numeric   |       16    |
 | LDT_BYTE     | byte      |     4000*   |
 | LDT_BLOB     | BLOB      |      2GB    |
 | LDT_VARCHAR  | var char  |     4000+   |
 | LDT_VARBYTE  | var byte  |     4000+   |
 | LDT_BOLLEAN  | boolean   |        5+   |
 | LDT_NCHAR    | national  |     2000    |
 |              | char      |             |
 | LDT_NVARCHAR | national  |     2000    |
 |              | var char  |             |
 +--------------+-----------+-------------+
 * - 240 for Linter SQL Server version 4.xx
 + - Implemented in Linter SQL Server version 5.7 or higher

 Directions for fetch:
 +--------------+------------------------+
 |  Macro name  |      Description       |
 +--------------+------------------------+
 | FETCH_FIRST  | Get first row          |
 | FETCH_LAST   | Get last row           |
 | FETCH_NEXT   | Get next row           |
 | FETCH_PREV   | Get previous row       |
 | FETCH_ABSNUM | Get row by number      |
 +--------------+------------------------+

 Transaction modes:
 +---------------+---------------------------------------+
 |  Macro name   |             Description               |
 +---------------+---------------------------------------+
 | TM_AUTOCOMMIT | Set auto commit mode for transaction  |
 | TM_OPTIMISTIC | Set optimistic mode for transaction   |
 | TM_EXCLUSIVE  | Set exclusive mode for transaction    |
 +---------------+---------------------------------------+

 Supported codepages:
 +------------+---------------------------------+
 | Macro name |           Description           |
 +------------+---------------------------------+
 | CP_866     | MS-DOS  866  cyrillic           |
 | CP_1251    | Windows 1251 cyrillic           |
 | CP_KOI8    | Unix    KOI8 cyrillic           |
 +------------+---------------------------------+

 Cursor/connection options:
 +-----------------------------+-----+-----+----------------------------------+
 |        Macro name           | Set | Get |        Description               |
 +-----------------------------+-----+-----+----------------------------------+
 | CO_CURNAME                  |  +  |  +  | Cursor name                      |
 | CO_ASYNCMODE                |  +  |  +  | Asynchronous execution mode      |
 | CO_SYNCMODE                 |  +  |  +  | Synchronous execution mode       |
 | CO_ASYNCDONE                |  -  |  +  | Asynchronous execution done      |
 | CO_DTFORMAT                 |  +  |  +  | Format for "DATE" datatype       |
 | CO_COLCOUNT                 |  -  |  +  | Number of columns in answer      |
 | CO_ROWCOUNT                 |  -  |  +  | Number of rows in answer         |
 | CO_ERRROW                   |  -  |  +  | Number of query line with error  |
 | CO_ERRPOS                   |  -  |  +  | Number of error position in line |
 | CO_TRANSMODE                |  +  |  +  | Transaction mode            (1)  |
 | CO_CURROW                   |  -  |  +  | Current row in SQL cursor        |
 | CO_CURROWID                 |  -  |  +  | ROWID of current row             |
 | CO_CONNECTID                |  -  |  +  | Connection ID                    |
 | CO_NODENAME                 |  -  |  +  | Name of Linter server            |
 | CO_PRIORITY                 |  +  |  +  | Cursor priority             (2)  |
 | CO_QUERY_PRIORITY           |  +  |  -  | Priority for current query  (2)  |
 | CO_CANCEL                   |  +  |  -  | Cancel query                (2)  |
 | CO_PAUSE                    |  +  |  -  | Pause query                 (2)  |
 | CO_CONTINUE                 |  +  |  -  | Continue query              (2)  |
 | CO_LAST_ROWID               |  -  |  +  | Get last row ID             (3)  |
 | CO_FETCH_BLOBS_AS_USUAL_DATA|  +  |  +  | Fetch blobs as usual data flag   |
 +-----------------------------+-----+-----+----------------------------------+
 1 - can be set in version 5.7 or higher
 2 - implemented in version 5.7 or higher
 3 - should be used after INSERT query only

 Extension internal error codes:
 +----------------------+---------------------------------+
 |     Macro name       |           Description           |
 +----------------------+---------------------------------+
 | LPERR_SUCCESS        | Operation successed             | 
 | LPERR_NOMEMORY       | Not enough memory               |
 | LPERR_INVCURSOR      | Invalid cursor/connection       |
 | LPERR_INVCOLNUM      | Invalid column number           |
 | LPERR_INVPARAMNUM    | Invalid parameter number        |
 | LPERR_INVSTATE       | Invalid cursor/connection state |
 | LPERR_INVDIRECT      | Invalid fetch direction         |
 | LPERR_UNKNOWNOPT     | Unknown cursor option           |
 | LPERR_BADDTFORMAT    | Bad format for datatype 'DATE'  |
 | LPERR_NOBLOBCOL      | BLOB column is absent in answer |
 | LPERR_NOPARENT       | Cursor doesn't have a parent    |
 | LPERR_INVPARAMETER   | Invalid parameter               |
 | LPERR_LINERROR       | Internal Linter error           |
 | LPERR_NOTIMPLEMENTED | Feature not implemented         |
 +----------------------+---------------------------------+

=head1 Available column/connect info keys
 
 Available column info keys
 +--------------+--------------------+
 |  Key name    |      Description   |
 +--------------+--------------------+
 | is_null      | Is null flag       |
 | type         | Datatype           |
 | type_name    | Datatype name      |
 | length       | Datatype length    |
 | precision    | Datatype precision |
 | scale        | Datatype scale     |
 | name         | Name               |
 | table        | Table's name       |
 | user         | Username           |
 +--------------+--------------------+

 Available connect info keys
 +--------------+--------------------------------------------------+
 |  Key name                |              Description             |
 +--------------+--------------------------------------------------+
 | VERMAJOR                 | Linter major version number          |
 | VERMINOR                 | Linter minor version number          |
 | VERBUILD                 | Linter version build number          |
 | SORTPOOLSIZE             | Sort pool size (pages count)         |
 | KERNELPOOLSIZE           | Kernel pool size (pages count)       |
 | FILEQUEUESIZE            | File queue size                      |
 | USERQUEUESIZE            | User queue size                      |
 | TABLEQUEUESIZE           | Table queue size                     |
 | COLUMNQUEUESIZE          | Column queue size                    |
 | CHANNELQUEUESIZE         | Channel queue size                   |
 | SNAPTIMEOUT              | Snap timeout                         |
 | KILLTIMEOUT              | Kill timeout                         |
 | BASENAME                 | Database name                        |
 | SYSLOG                   | Transaction file active flag         |
 | SYNC                     | I/O sync flag                        |
 | LOG                      | Log file is writing flag             |
 | OS                       | OS id                                |
 | NUMOFSORT                | Sort files count                     |
 | FLAGS_CSETCHG            | Codepage not found, uses default (1) |
 | FLAGS_KERNELENGLOCALE    | Only english locale exists (2)       |
 | FLAGS_EXTPASS            | User's password given by admin,      |
 |                          | must be changed                      |
 | FLAGS_PASSEND            | User's password must be              |
 |                          | changed (timeout)                    |
 | FLAGS_KERNELINVBYTEORDER | Server and client byte order         |
 |                          | don't match                          |
 | FLAGS_KERNELDEMOLIC      | This is Linter demo license          |
 | FLAGS_KERNELDEMOLICEXP   | Linter Demo license expires          |
 | MAXRECSIZE               | Max record size (1)                  |
 | CHARSET                  | Database charset (1)                 |
 | DEFCHARSET               | Database default charset (1)         |
 | USECHARSET               | Charset, that was setup (1)          |
 | USECHARSETNAME           | Charset, that was setup, name (1)    |
 +--------------+--------------------------------------------------+
 1 - Implemented in Linter SQL Server version 6.0 or higher
 2 - Implemented in Linter SQL Server version under 6.0
                  
=head1 SEE ALSO

perl(1), Linter SQL Server documentation

=cut

