use 5.000001;
use ExtUtils::MakeMaker qw(&WriteMakefile $Verbose );

use Config;
use File::Copy;

my %opts = (
    'NAME'         => 'LinPerl',
    'VERSION_FROM' => 'LinPerl.pm',
    'PREREQ_PM'    => {},
    ($] ge '5.005')
       ? ( 'AUTHOR' => 'Linter Perl (support@relex.ru)',
           'ABSTRACT' => 'Perl driver for Linter Database', )
       : (),
    ( $Config{archname} =~ /-object\b/i )
       ? ( 'CAPI' => 'TRUE', )
       : (),
    'OBJECT'       => '$(O_FILES)'
);

# --- Operating system
my $run_os = $^O;

my $envvar = 'LINTER_HOME';
my $LINTER = $ENV{$envvar};

unless ( $LINTER )
{
  if( $run_os eq 'MSWin32' )
  {
    open(PIPE, 'Reg Query "HKEY_LOCAL_MACHINE\SOFTWARE\Relex, Ltd.\Linter SQL Server\5.1" /v "Linter Home" |');
    my @str = <PIPE>;
    close PIPE;
    my $lindir;
    for ($i=$[;$i<=$#str;$i++)
    {
      if ($str[$i]=~/Linter Home/)
      {
        ($_,$lindir) = split('REG_SZ', $str[$i]);
        $lindir =~ s/^\s+//;
        $lindir =~ s/\s+$//;
      };
    };

    if ( -e "$lindir\\intlib\\lnlsutf.c" )
    {
      $LINTER = "$lindir";
    };
  }
  else
  {
    if ( -e "/usr/linter/intlib/lnlsutf.c" )
    {
      $LINTER = "/usr/linter";
    };
  };
};

unless ( $LINTER ) {
  die "LINTER_HOME environment variable must be set to installed location of LINTER_HOME.\n"
    unless $LINTER;
};

$LINTER =~ s/\"//g; #delete all "

print qq(Using LINTER in "$LINTER"\n);

# --- Setup include paths and libraries
$opts{INC} .= qq( -I"$LINTER/intlib" );

my $liblist;
my $LINTERLIB = "$LINTER/intlib/LIB";

# libraries required to build LINTER driver
if( $run_os eq 'MSWin32' )
{
  my $LINTERLIB = "$LINTER\\intlib\\LIB";
  $liblist = qq(-L"$LINTERLIB" dectic32.lib inter325.lib);
  open(PIPE, "\"$LINTER\\bin\\inl\" -briefversion |") or die "Can't find inl";
  $ver=<PIPE>;
  close PIPE;
  ($ver_mj, $ver_mi) =  split('\.', $ver);
  $opts{DEFINE} = '-D_MBCS -D_VER_MAX=' . "$ver_mj$ver_mi" . '0 -DUSE_THREADS -DINTER_MSWINDOWS -DWIN32 -DWINDOWS_NT';
  copy("$LINTER\\intlib\\lnlsutf.c", ".\\lnlsutf.c") or die "Copy failed: $!:";
}
else
{
  $liblist = qq(-L$LINTER/perl -llinstab);
  $opts{LDDLFLAGS} = qq(-nostartfiles -shared);
  
  open(F1, "$LINTER/Definition");
  @str = <F1>;

  $sys="^SYSTEMS.*=";
  $os="^OS.*=";
  $pack="^PACKING.*=";
  $ver="^VERSION.*=";

  close F1;


  for ($i=$[;$i<=$#str;$i++)
  {
    if ($str[$i]=~/$sys/){
      $SYSTEM = $str[$i];
      $SYSTEM =~ s/\w+\s*=//; 
      $SYSTEM =~ s/\n?$//;
      $opts{DEFINE} .= $SYSTEM;
    }

    if ($str[$i]=~/$os/){
      $OS = $str[$i];
      $OS =~ s/\w+\s*=//;
      $OS =~ s/\n?$//;
      $opts{DEFINE} .= $OS;
    }

    if ($str[$i]=~/$pack/){
      $PACKING = $str[$i];
      $PACKING =~ s/\w+\s*=//;
      $PACKING =~ s/\n?$//;
      $opts{DEFINE} .= $PACKING;
    }

    if ($str[$i]=~/$ver/){
      $VERSION = $str[$i];
      $VERSION =~ s/\w+\s*=//;
      $VERSION =~ s/\n?$//;
      $opts{DEFINE} .= $VERSION;
    }
  } # for
}

$opts{LIBS} = [ $liblist ];

sub MM::all_target {
    my $self = shift;
    my $wint = "";

    if ( $run_os eq 'MSWin32' )
    {
      $wint = "copylintlib";
    };
    return << "MAKE_EXT";
all :: pure_all manifypods $wint
\t\$\(NOECHO\) \$\(NOOP\)
MAKE_EXT
}


sub MY::postamble {
  my $ret = "";
  if ( $run_os eq 'MSWin32' )
  {
    $ret = "\n" . 
    "copylintlib:\n\t\$\(CP\) \"$LINTER\\bin\\dectic32.dll\" \$(INST_ARCHAUTODIR)\n" .
    "\t\$\(CP\) \"$LINTER\\bin\\inter325.dll\" \$(INST_ARCHAUTODIR)\n";
  }
  return $ret;
}


print "System: perl$] DBI$DBI::VERSION @Config{qw(myuname archname dlsrc)}\n";
print "Compiler: @Config{qw(cc optimize ccflags)}\n";
print "Includes: $opts{INC}\n";
print "Libraries: @{$opts{LIBS}}\n";
print "\n";

WriteMakefile(%opts);
