/****************************************************************************
 *           RELEX, Inc. Client source include file                         *
 *  This is unpublished proprietary source code of RELEX, Inc.              *
 *  The copyright notice does not evidence any actual or intended           *
 *  publication of such source code.                                        *
 *                                                                          *
 *                 Copyright (c) 1999-2014 RELEX Inc.                       *
 *                                                                          *
 *                   This software was developed by                         *
 *                                                                          *
 *                            RELEX, Inc.                                   *
 *                         Voronezh, Russia                                 *
 *                                                                          *
 *                        All rights reserved.                              *
 ****************************************************************************
 FILE:    linperl.c
 ----------------------------------------------------------------------------
 AUTHOR:  Alexander V. Kefeli
 ----------------------------------------------------------------------------
 PURPOSE: Interface from Perl to Linter SQL Server
 ----------------------------------------------------------------------------
 REVISION HISTORY:

 **.**.** -- Kent (file creation)
 17.02.99 -- RAV  (change Linter to LinPerl)
 17.02.99 -- RAV  (insert direct inter() call)
 19.02.99 -- Alx  (portting to LINUX, undef "word" macros to remove conflict)
 16.12.99 -- Kononyhin (set crName before SETO)
 16.12.99 -- Kent (New cursor options for 5.7)
 27.12.99 -- Kent (free memory after delete from list)
 24.02.00 -- Kent (LAST ROWID cursor option)
 09.02.00 -- Ermakov (bool definition moved up)
 12.04.00 -- Kent (Support 5.7 datatypes)
 12.04.00 -- Kent (Fix some bugs and cosmetic)
 18.04.00 -- Kent (Get answer info if no records)
 11.05.00 -- Kent (New function `Cleanup')
 02.06.00 -- Kent (Call UninitLinterClient() on unload)
 16.06.00 -- Kononyhin (rename input to _input)
 31.10.00 -- Ermakov (Use new function from cquery, DT_DEFAULT)
 08.11.00 -- Kent (New routines for stored procedures)
 06.12.00 -- Galizin ( RowId ...)
 27.12.00 -- Kent (MAX_ID_LEN instead 18[+x]
                   Adapted for multithreaded env.)
 28.12.00 -- Kent (Perl lower 5.005 doesn't support threads)
 03.01.01 -- Kent (Bug fixed: cursor ID instead channel number)
 26.09.02 -- Goncharov (Correcting code page names for english locale)
 16.06.04 -- Peshkoff (varchar/varbyte data length should be taken to an account)
 26.10.04 -- Gluhov (nchar support added)
 05.11.04 -- Gluhov,Ermakov (fixed bug with codepage in binded values)
 01.04.08 -- Ermakov (Corrected isselect usage)
 08.09.08 -- KOV (Added some exceptions)
 15.04.09 -- Glukhov (GetParamValue using removed)
 15.04.09A-- Glukhov (REAL & DOUBLE fields fetching fixed)
 07.05.09 -- Glukhov (Automatic cursor closing commented - it leads to error)
 07.05.09A-- Glukhov (Get blob size after varchar, varbyte, nvarchar column fixed)
 08.05.09 -- Glukhov (Get blob data column impementation added)
 12.05.09 -- Glukhov (Bind params fixed)
 12.05.09A-- Glukhov (Blobs bind implementation added)
 01.06.09 -- Glukhov (Using codepages in Linter 6.x fixed)
 14.06.09 -- Glukhov (Using len param in BLOBGetData fixed)
 14.06.09A-- Glukhov (Default date format fixed)
 21.06.09 -- Glukhov (Some critical error fixed)
 21.06.09A-- Glukhov (Some constants added)
 21.06.09B-- Glukhov (Procedures work fixed)
 29.06.09 -- Glukhov (force_unload param add in ClearCursorDesc)
 30.10.09 -- Glukhov (BindParamArray and some bugs fixed)
 23.12.09 -- Glukhov (ClearCursorDesc, FillCursorDesc bug fixed)
 24.12.09 -- Glukhov (Blobs using modified, some functions added)          
 29.12.09 -- Glukhov (GetConnectInfo added)
 12.01.10 -- Glukhov (SetCodepage added)
 25.01.10 -- Glukhov (pass_cnt added)
 01.02.10 -- Glukhov (GetDataRow, GetDataArray, GetDataRows added)
 03.02.10 -- Glukhov (ColOffset CURSOR parameter added, GetExchBuffOff and GetBlobExchBuffOff funcions removed)
 03.02.10A-- Glukhov (GETM added)
 04.02.10 -- Glukhov (GetColInfo updated)
 08.02.10 -- Glukhov (Column number check fixed)
 09.02.10 -- Glukhov (Constants export fixed)
 15.02.10 -- Glukhov (Constants from linapiin.h using added, some constants using error fixed)
 15.02.10A-- Glukhov (First row fetch after select fixed)
 15.02.10B-- Glukhov (Blob behaviour cursor option added, BlFlag reset added)
 15.02.10C-- Glukhov (Long row fetch fixed in LINPL_GETM)
 26.02.10 -- Glukhov (Asynchronous execution improved)
 07.06.10 -- Glukhov (_strdup using removed (Perl don't support internal 
                      version of _strdup, but redefine free, malloc, etc))
 08.06.10 -- Glukhov (Get param value from SV acording with SvType fixed)
 16.06.10 -- Glukhov (GETM using error fixed)
 21.06.10 -- Glukhov (Memory allocated for LINNLS_Ucs2Utf8 string size fixed)
 24.09.10 -- Glukhov (SvTYPE macros using fixed for Perl 5.12 and beyond)
 11.09.12 -- Pashentsev (Free allocated memory inside CURSOR) 
 13.09.12 -- Pashentsev (Free intlib context on library unload)
 26.09.12 -- Ermakov (Corrected 64bit access)
 08.09.13 -- Muratova (Defining DT_DEFAULT instead including cquery.h)
 09.09.13 -- Galizin (calloc redefinition is absent in perl)
 24.04.14 -- Muratova (smallint error fixed)
 29.09.14 -- Kononov (fixed date format)
 11.11.14a-- Kononov (fixed deleting elements from an array when binding params)
 11.11.14b-- Kononov (fixed zero values of blob size and blob type)
 12.11.14a-- Kononov (fixed updating value of user charset name after setting new codepage)
 12.11.14b-- Kononov (fixed logic of LINPL_GETM function - behavior is the same as LINPL_GetDataRows function,
                      but using a batch loading data)
 13.11.14 -- Kononov (fixed setting cursor options)
 14.11.14 -- Kononov (added EXTFILE type)
 17.11.14 -- Kononov (added support 'WITH' construction)
 20.11.14 -- Kononov (fixed support for Perl >= 5.19.x)
 --------------------------------------------------------------------------*/

#define __LINPERL_C__

#ifdef WIN32
#  define WIN32_LEARN_AND_MEAN
#  define DONT_USE_CRITICAL_SECTION
#include <windows.h>
#endif
#include <errno.h>
#include <string.h>

#include "inter.h"
#include "decimals.h"
#include "tick.h"
#include "int64.h"
#include "lnlsutf.h"                                    /* 26.10.04 */

#define DT_DEFAULT     -1                               /* 08.09.13 */

#define BLOCK PL_BLOCK


#include "EXTERN.h"
#include "perl.h"
#include "XSUB.h"

#include "linperl.h"
#include "tools.h"

#ifndef min
#  define min(a, b) (a < b ? a : b)
#endif
#ifndef s_free
#  define s_free(p) {if (p != NULL) free(p); p = NULL;}
#endif

static void* s_calloc(int nmemb, int size)                 /* 09.09.13 ... */
{
  void* Ret = malloc(nmemb*size);
  if (Ret)
    memset(Ret, 0, nmemb*size);
  return Ret;
}                                                          /* ... 09.09.13 */
                                                           /* 15.02.10 */
#define SQL_UNKNOWN    0
#define SQL_SELECT     1
#define SQL_ALTER      42                                  /* 21.06.09A ... */
#define SQL_GRANT      47
#define SQL_EXEC_PROC  52
#define SQL_TSTTBL     112                                 /* ... 21.06.09A */

#define GET_LINTER_CODE(c) ((int)((c)->CBL.CodErr))
#define GET_SYSTEM_CODE(c) ((int)((c)->CBL.SysErr))

#ifndef MUTEX_INIT                                         /* 28.12.00 ... */
#  define MUTEX_INIT(m)
#endif

#ifndef MUTEX_DESTROY
#  define MUTEX_DESTROY(m)
#endif

#ifndef MUTEX_LOCK
#  define MUTEX_LOCK(m)
#endif

#ifndef MUTEX_UNLOCK
#  define MUTEX_UNLOCK(m)
#endif                                                     /* ... 28.12.00 */

#define MaxAnswerSize        4096                          /* 15.02.10 ... */
#define LongQueryMaxLength   65535
#if _VER_MAX >= 600
#define MAX_ROW_SIZE  0xFFFF
#else
#define MAX_ROW_SIZE  8192                                 
#endif                                                     /* ... 15.02.10 */

#define DEFAULT_BLOB_BEHAVIOUR L_TTRUE                     /* 15.02.10B */

extern void UninitLinterClient(void);

typedef struct t_CURSOR CURSOR;
struct t_CURSOR {
  TCBL       CBL;
  int        id;
  int        cols;
  int        rows;
  int        crow;
  int        errpos;
  int        last_oper;
  long       answ_len;                                    /* 03.02.10A */
  long      *ColOffset;                                   /* 03.02.10 */
  long       FstRow;                                      /* 03.02.10A ... */
  long       LstRow;                                            
  char      *getm_buff;                                   /* ... 03.02.10A */      
  L_LONG     Flags;                                       /* 05.11.04 */
  char       name[MAX_ID_LEN+1];
  char      *dt_format;
  char      *exch_buff;
  char      *null_flag;
  char      *pqry_body;
  GETA_OUT  *answ_desc;
  long       pass_cnt;                                    /* 25.01.10 */
  L_BOOL     hasFirstRow;                                 /* 15.02.10A */
  OPEN_DESC  Dsc;                                         /* 01.06.09 */
  L_LONG     DscLen;                                      /* 29.12.09 ... */
  DB_DESC    DbDsc;                                       
  L_LONG     DbDscLen;                                    /* ... 29.12.09 */
  short     *parm_leng;                                   /* 15.04.09 */
  short      param_count;                                 /* 15.04.09 */
  SV       **bind_cols;
  SV       **bind_params;                                 /* 12.05.09 */
  L_BOOL     BlFlag;                                      /* 12.05.09A */
  L_BOOL     BlobBehaviour;                               /* 15.02.10B */
  CURSOR    *parent;
  CURSOR    *next;
};

#if defined(USE_THREADS) || defined(USE_ITHREADS)
static perl_mutex mutex;
#endif
static CURSOR     List;
static int        max_id        = 1;
static char      *STR_DELIM     = " \t\n\r";
static char      *DEF_DT_FORMAT = "DD.MM.YYYY HH:MI:SS.FF"; /* 14.06.09A */ /* 29.09.14 */

static char *ldt_names[] = {                                 /* 04.02.10 ... */
  "CHAR", "INTEGER", "REAL", "DATE", "NUMERIC", "BYTE",
  "BLOB", "VARCHAR", "VARBYTE", "BOOLEAN", "NCHAR",        
  "NVARCHAR", "EXTFILE", "SMALLINT", "DOUBLE"              
};

static char* ldt_int_names[] = {"SMALLINT", "INTEGER", "", "BIGINT"}; 
static char* ldt_real_names[] = {"REAL", "DOUBLE"};          /* ... 04.02.10 */

CURSOR *GetFromList(int id)
{
  CURSOR *crsr;

  MUTEX_LOCK(&mutex);
  crsr = List.next;

  while(crsr)
    {
    if (crsr->id == id)
      break;
    crsr = crsr->next;
    }
  MUTEX_UNLOCK(&mutex);
  return (crsr);
}

void AddToList(CURSOR *c)
{
  CURSOR *crsr = &List;

  MUTEX_LOCK(&mutex);
  c->id = max_id++;
  while(crsr->next)
    crsr = crsr->next;
  crsr->next = (CURSOR *)s_calloc(1, sizeof(CURSOR));      /* 09.09.13 */
  memcpy(crsr->next, c, sizeof(CURSOR));
  crsr->next->BlobBehaviour = DEFAULT_BLOB_BEHAVIOUR;      /* 15.02.10B */
  MUTEX_UNLOCK(&mutex);
}

void DelFromList(CURSOR *c)
{
  CURSOR *crsr = &List;

  MUTEX_LOCK(&mutex);
  while(crsr)
    {
    if (crsr->next == c)
      {
      crsr->next = c->next;
      break;
      }
    crsr = crsr->next;
    }
  MUTEX_UNLOCK(&mutex);
}

int AddBlob(CURSOR *crsr, char* data, L_LONG blob_size, L_LONG blob_type, L_LONG colnum) /* 24.12.09 ... */
{
  L_LONG    psize, sent = 0;
  L_LONG    full_size;
  L_LONG    offst;

  full_size = blob_size;   
  
  if (crsr->CBL.PrzExe & Q_ASYNC)                          /* 26.02.10 */
    inter_wait_single(&crsr->CBL, -1);                       

  if (colnum > 0)
  {
    strncpy(crsr->CBL.Command, "ABLB", 4);
    crsr->CBL.RowCount = colnum;
  }
  else
    strncpy(crsr->CBL.Command, "AOBJ", 4);
  crsr->CBL.RowId = blob_type;
  do
  {
    psize = min(OPTIMUM_PORTION_SIZE, blob_size);
    crsr->CBL.LnBufRow = (unsigned short)psize;
    inter(&crsr->CBL, NULL, NULL, NULL, &data[sent]);
    if (crsr->CBL.PrzExe & Q_ASYNC)                        /* 26.02.10 */
      inter_wait_single(&crsr->CBL, -1);                       
    if (GET_LINTER_CODE(crsr) != 0)
      return (GET_LINTER_CODE(crsr));
    sent += psize;
    blob_size -= psize;
  } while(sent < full_size);

  offst = crsr->ColOffset[(colnum > 0) ? (colnum - 1) : 0];   /* 11.11.14b ... */
  ((BLOB_ATR*)&crsr->exch_buff[offst])->Size = sent;
  ((BLOB_ATR*)&crsr->exch_buff[offst])->TypeObj = blob_type;  /* ... 11.11.14b */
    
  return(LPERR_SUCCESS);
}        

int FetchBlob(CURSOR *crsr, L_LONG blob_start, SV *blob_size, L_LONG colnum, SV *blob_value)   
{
  char   data[OPTIMUM_PORTION_SIZE];
  L_LONG got = 0;  

  if (crsr->CBL.PrzExe & Q_ASYNC)                          /* 26.02.10 */
    inter_wait_single(&crsr->CBL, -1);                       

  if (colnum > 0)
  {
    strncpy(crsr->CBL.Command, "GBLB", 4);  
    crsr->CBL.RowCount = colnum;
  }
  else
    strncpy(crsr->CBL.Command, "GOBJ", 4);
  sv_setpvn(blob_value, NULL, 0);  
  do
  {
    crsr->CBL.LnBufRow = (L_WORD)min(OPTIMUM_PORTION_SIZE, SvIV(blob_size));
    crsr->CBL.RowId    = blob_start + got;    
    inter(&crsr->CBL, NULL, NULL, NULL, data);
    if (crsr->CBL.PrzExe & Q_ASYNC)                        /* 26.02.10 */
      inter_wait_single(&crsr->CBL, -1);                       
    if (GET_LINTER_CODE(crsr) != 0)
      break;    
    sv_catpvn(blob_value, data, (int)crsr->CBL.LnBufRow);
    got += crsr->CBL.LnBufRow;
  } while(got < SvIV(blob_size));

  if (GET_LINTER_CODE(crsr) == EORR)
    crsr->CBL.CodErr = 0;
  if (crsr->CBL.CodErr == 0)
    sv_setiv(blob_size, got);
  else
  {
    sv_setpvn(blob_value, NULL, 0);
    sv_setiv(blob_size, 0);
  }
  return (GET_LINTER_CODE(crsr));
}       

int ClearBlob(CURSOR *crsr, L_LONG colnum)                
{
  if (crsr->CBL.PrzExe & Q_ASYNC)                          /* 26.02.10 */
    inter_wait_single(&crsr->CBL, -1);                       

  if (colnum > 0)
  {
    strncpy(crsr->CBL.Command, "CBLB", 4);
    crsr->CBL.RowCount = colnum;
  }
  else
    strncpy(crsr->CBL.Command, "COBJ", 4);
  
  inter(&crsr->CBL, NULL, NULL, NULL, NULL);

  return (GET_LINTER_CODE(crsr));
}     

int GetTypeName(long type, long length, char *type_name, long max_name_len) /* 04.02.10 ... */
{
  if (type == LDT_INTEGER)
    strncpy(type_name,ldt_int_names[length / 2 - 1], max_name_len);          
  else
  if (type == LDT_REAL )
    strncpy(type_name, ldt_real_names[length / 4 -1], max_name_len);         
  else
    strcpy(type_name, ldt_names[type - 1]);
  return(LPERR_SUCCESS);
}                                                          /*... 04.02.10 */

int LINPL_OpenConnect(char *usr, char *pwd, char *node,
                      int flags, SV *connect)
{
  CURSOR crsr;
  char   user[128];                                        /* 27.12.00 */  
  char   cp_name_buf[7] = "";                              /* 01.06.09 */
  
  memset(&crsr, 0, sizeof(CURSOR));
  crsr.Flags = crsr.CBL.PrzExe = flags;                    /* 05.11.04 */
  strncpy((char *)crsr.CBL.Node, node, MAX_NODE_LEN);
  strncpy((char *)crsr.CBL.Command, "OPEN", 4);
  crsr.CBL.LnBufRow = sizeof(crsr.Dsc);                    /* 01.06.09 */
  sprintf(user, "\"%.66s\"/\"%.18s\"", usr, pwd);
#if _VER_MAX >=600                                         /* 01.06.09 ... */
  if ((crsr.Flags & CP_866) == CP_866)
    strncpy(cp_name_buf,"CP866",5);
  else if ((crsr.Flags & CP_KOI8) == CP_KOI8)
    strncpy(cp_name_buf,"KOI8-R",6);
  else if ((crsr.Flags & CP_1251) == CP_1251)
    strncpy(cp_name_buf,"CP1251",6);
  
  inter(&crsr.CBL, user, &cp_name_buf,NULL, &crsr.Dsc);  
#else                                                      /* ... 01.06.09 */
  inter(&crsr.CBL, user, NULL, NULL, NULL);
#endif
  if (crsr.CBL.PrzExe & Q_ASYNC)                           /* 26.02.10 */
    inter_wait_single(&crsr.CBL, -1);                       
  crsr.DscLen = crsr.CBL.LnBufRow;                         /* 29.12.09 */
  if (crsr.CBL.CodErr == 0)
    AddToList(&crsr);
  else
    memcpy(&List.CBL, &crsr.CBL, sizeof(TCBL));
  sv_setiv(connect, (IV)crsr.id);
  return (GET_LINTER_CODE(&crsr));
}

static void ClearCursorDesc(CURSOR *crsr, int all, int force_unload) /* 09.09.13 29.06.09 */
{  
  s_free(crsr->exch_buff);
  s_free(crsr->answ_desc);
  s_free(crsr->bind_cols);
  s_free(crsr->null_flag);                                  /* 21.06.09 */
  s_free(crsr->ColOffset);                                  /* 03.02.10 */
  crsr->cols = 0;                                           /* 23.12.09 */
  crsr->pass_cnt = 0;                                       /* 25.01.10 */
  crsr->answ_len = 0;                                       /* 03.02.10A ... */
  crsr->FstRow   = 0;
  crsr->LstRow   = 0;                                       /* ... 03.02.10A */
  crsr->hasFirstRow = 0;                                    /* 15.02.10A */
  crsr->BlFlag   = L_TFALSE;                                /* 15.02.10B */  
   
  if (all)
  {    
    int i;                                                  /* 12.05.09 */  
    s_free(crsr->pqry_body);
    s_free(crsr->getm_buff);                                /* 03.02.10A */
    s_free(crsr->dt_format);    
    s_free(crsr->parm_leng);                                /* 15.04.09 */
    if (!force_unload)                                       /* 29.06.09 */
      for (i=0;i<crsr->param_count;i++)                     /* 12.05.09 ... */
        if (crsr->bind_params[i])
          crsr->bind_params[i]->sv_refcnt--;      
    s_free(crsr->bind_params); 
    crsr->param_count = 0;                                  /* ... 12.05.09 */    
  }
}

int LINPL_CloseConnect(int connect)
{
  int     rc;
  CURSOR *crsr, *tmp;
  CURSOR *conn = GetFromList(connect);

  if (conn == NULL)
    return (LPERR_INVCURSOR);
  if (conn->CBL.PrzExe & Q_ASYNC)                           /* 26.02.10 */
    inter_wait_single(&conn->CBL, -1);                       
  strncpy((char *)conn->CBL.Command, "CLOS", 4);
  inter(&conn->CBL, NULL, NULL, NULL, NULL);
  rc = GET_LINTER_CODE(conn);
  ClearCursorDesc(conn, L_TTRUE, L_TFALSE);                 /* 29.06.09 */
  DelFromList(conn);
  crsr = List.next;
  while(crsr != NULL)
  {
    if (crsr->parent == conn)
    {
      DelFromList(crsr);
      tmp = crsr;
      crsr = crsr->next;
      s_free(tmp->pqry_body);                               /* 11.09.12 ...*/
      s_free(tmp->answ_desc);
      s_free(tmp->exch_buff);
      s_free(tmp->ColOffset);
      s_free(tmp->bind_cols);
      s_free(tmp->dt_format);
      s_free(tmp->null_flag);
      s_free(tmp->parm_leng);
      s_free(tmp->bind_params);                             /*... 11.09.12 */
      s_free(tmp);                                          /* 27.12.99 */
      continue;
    }
    crsr = crsr->next;
  }
  s_free(conn);                                             /* 27.12.99 */
  return (rc);
}

int LINPL_GetConnectInfo(int connect, HV * info)          /* 29.12.09 ...*/
{
  CURSOR *conn = GetFromList(connect);
  
  hv_clear(info);
  if (conn == NULL)
    return (LPERR_INVCURSOR);

  if (!conn->DbDscLen)
  {
    if (conn->CBL.PrzExe & Q_ASYNC)                        /* 26.02.10 */
      inter_wait_single(&conn->CBL, -1);                       

    strncpy((char *)conn->CBL.Command, "DESC", 4);   
    conn->CBL.LnBufRow = sizeof(conn->DbDsc);                                     
    inter(&conn->CBL, NULL, NULL, NULL, &conn->DbDsc);          
    if (conn->CBL.CodErr != 0)
      conn->DbDscLen = 0;                                        
    else
      conn->DbDscLen = conn->CBL.LnBufRow;             
  }
      
  /* from DB_DESC */
  
  if ( (char *)&(conn->DbDsc) - (char *)&(conn->DbDsc.VerMajor) < conn->DbDscLen )      
    hv_store(info, "VERMAJOR", 8, newSViv((IV)conn->DbDsc.VerMajor), 0);        
  if ( (char *)&(conn->DbDsc) - (char *)&(conn->DbDsc.VerMinor) < conn->DbDscLen )
    hv_store(info, "VERMINOR", 8, newSViv((IV)conn->DbDsc.VerMinor), 0);                
  if ( (char *)&(conn->DbDsc) - (char *)&(conn->DbDsc.VerBuild) < conn->DbDscLen )
    hv_store(info, "VERBUILD", 8, newSViv((IV)conn->DbDsc.VerBuild), 0);                
  if ( (char *)&(conn->DbDsc) - (char *)&(conn->DbDsc.SortPoolSize) < conn->DbDscLen )
    hv_store(info, "SORTPOOLSIZE", 12, newSViv((IV)conn->DbDsc.SortPoolSize), 0);                        
  if ( (char *)&(conn->DbDsc) - (char *)&(conn->DbDsc.KernelPoolSize) < conn->DbDscLen )
    hv_store(info, "KERNELPOOLSIZE", 14, newSViv((IV)conn->DbDsc.KernelPoolSize), 0);                        
  if ( (char *)&(conn->DbDsc) - (char *)&(conn->DbDsc.FileQueueSize) < conn->DbDscLen )
    hv_store(info, "FILEQUEUESIZE", 13, newSViv((IV)conn->DbDsc.FileQueueSize), 0);                    
  if ( (char *)&(conn->DbDsc) - (char *)&(conn->DbDsc.UserQueueSize) < conn->DbDscLen )
    hv_store(info, "USERQUEUESIZE", 13, newSViv((IV)conn->DbDsc.UserQueueSize), 0);                    
  if ( (char *)&(conn->DbDsc) - (char *)&(conn->DbDsc.TableQueueSize) < conn->DbDscLen )
    hv_store(info, "TABLEQUEUESIZE", 14, newSViv((IV)conn->DbDsc.TableQueueSize), 0);                    
  if ( (char *)&(conn->DbDsc) - (char *)&(conn->DbDsc.ColumnQueueSize) < conn->DbDscLen )
    hv_store(info, "COLUMNQUEUESIZE", 15, newSViv((IV)conn->DbDsc.ColumnQueueSize), 0);                      
  if ( (char *)&(conn->DbDsc) - (char *)&(conn->DbDsc.ChannelQueueSize) < conn->DbDscLen )
    hv_store(info, "CHANNELQUEUESIZE", 16, newSViv((IV)conn->DbDsc.ChannelQueueSize), 0);                          
  if ( (char *)&(conn->DbDsc) - (char *)&(conn->DbDsc.SnapTimeout) < conn->DbDscLen )
    hv_store(info, "SNAPTIMEOUT", 11, newSViv((IV)conn->DbDsc.SnapTimeout), 0);                          
  if ( (char *)&(conn->DbDsc) - (char *)&(conn->DbDsc.KillTimeout) < conn->DbDscLen )
    hv_store(info, "KILLTIMEOUT", 11, newSViv((IV)conn->DbDsc.KillTimeout), 0);                          
  if ( (char *)&(conn->DbDsc) - (char *)&(conn->DbDsc.BaseName) < conn->DbDscLen )
    hv_store(info, "BASENAME", 8, newSVpvn(conn->DbDsc.BaseName, sizeof(conn->DbDsc.BaseName)), 0);                          
  if ( (char *)&(conn->DbDsc) - (char *)&(conn->DbDsc.SysLog) < conn->DbDscLen )
    hv_store(info, "SYSLOG", 6, newSViv((IV)conn->DbDsc.SysLog), 0);                          
  if ( (char *)&(conn->DbDsc) - (char *)&(conn->DbDsc.Sync) < conn->DbDscLen )
    hv_store(info, "SYNC", 4, newSViv((IV)conn->DbDsc.Sync), 0);                          
  if ( (char *)&(conn->DbDsc) - (char *)&(conn->DbDsc.Log) < conn->DbDscLen )
    hv_store(info, "LOG", 3, newSViv((IV)conn->DbDsc.Log), 0);                          
  if ( (char *)&(conn->DbDsc) - (char *)&(conn->DbDsc.Os) < conn->DbDscLen )
    hv_store(info, "OS", 2, newSViv((IV)conn->DbDsc.Os), 0);                          
  if ( (char *)&(conn->DbDsc) - (char *)&(conn->DbDsc.NumOfSort) < conn->DbDscLen )
    hv_store(info, "NUMOFSORT", 9, newSViv((IV)conn->DbDsc.NumOfSort), 0);                            
  
  /* from OPEN_DESC */ 
  
  if ( (char *)&(conn->Dsc) - (char *)&(conn->Dsc.Flags) < conn->DscLen )
  {
#if _VER_MAX >= 600
    hv_store(info, "FLAGS_CSETCHG", 13, newSViv((conn->Dsc.Flags & OPENDESC_CSET_CHANGED)?1:0), 0);                                
#else
    hv_store(info, "FLAGS_KERNELENGLOCALE", 20, newSViv((conn->DbDsc.Flags & KERNEL_ENG_LOCALE)?1:0), 0);                                    
#endif    
    hv_store(info, "FLAGS_EXTPASS", 13, newSViv((conn->Dsc.Flags & OPENDESC_PASS_EXTERNAL)?1:0), 0);                                    
    hv_store(info, "FLAGS_PASSEND", 13, newSViv((conn->Dsc.Flags & OPENDESC_PASSLIFEEND)?1:0), 0);                                        
    hv_store(info, "FLAGS_KERNELINVBYTEORDER", 24, newSViv((conn->DbDsc.Flags & KERNEL_INVBYTEORD)?1:0), 0);
    hv_store(info, "FLAGS_KERNELDEMOLIC", 19, newSViv((conn->DbDsc.Flags & KERNEL_DEMOLIC)?1:0), 0);                                                
    hv_store(info, "FLAGS_KERNELDEMOLICEXP", 22, newSViv((conn->DbDsc.Flags & KERNEL_DEMOLICEXP)?1:0), 0);  
  }

#if _VER_MAX >= 600
  if ( (char *)&(conn->DbDsc) - (char *)&(conn->DbDsc.MaxRecSize) < conn->DbDscLen )
    hv_store(info, "MAXRECSIZE", 10, newSViv(conn->DbDsc.MaxRecSize), 0);                                                           
  if ( (char *)&(conn->DbDsc) - (char *)&(conn->DbDsc.CharSet) < conn->DbDscLen )
    hv_store(info, "CHARSET", 7, newSViv(conn->DbDsc.CharSet), 0);                                                               
  if ( (char *)&(conn->Dsc) - (char *)&(conn->Dsc.DefCharSet) < conn->DscLen )
    hv_store(info, "DEFCHARSET", 10, newSViv(conn->Dsc.DefCharSet), 0);                                                                   
  if ( (char *)&(conn->Dsc) - (char *)&(conn->Dsc.UseCharSet) < conn->DscLen )
    hv_store(info, "USECHARSET", 10, newSViv(conn->Dsc.UseCharSet), 0);                                                                       
  if ( (char *)&(conn->Dsc) - (char *)&(conn->Dsc.UseCharSetName) < conn->DscLen )
    hv_store(info, "USECHARSETNAME", 14, newSVpvn(conn->Dsc.UseCharSetName, sizeof(conn->Dsc.UseCharSetName)), 0);                              
#endif  
  
  return (LPERR_SUCCESS);
}                                                          /* ... 29.12.09 */

int LINPL_OpenCursor(int connect, char *name, SV *cursor)
{
  CURSOR crsr;

  memset(&crsr, 0, sizeof(CURSOR));
  crsr.parent = GetFromList(connect);
  if (crsr.parent == NULL)
    return (LPERR_INVCURSOR);
  crsr.Flags = crsr.parent->Flags;                           /* 05.11.04 */
  memcpy(&crsr.CBL, &(crsr.parent->CBL), sizeof(TCBL));
  strncpy((char *)crsr.CBL.Command, "OCUR", 4);
  inter(&crsr.CBL, NULL, NULL, NULL, NULL);
  if (crsr.CBL.CodErr == 0)
  {
    AddToList(&crsr);
    sv_setiv(cursor, (IV)crsr.id);
  }
  crsr.BlobBehaviour = crsr.parent->BlobBehaviour;         /* 15.02.10B */
  return (GET_LINTER_CODE(&crsr));
}

int LINPL_CloseCursor(int cursor)
{
  int rc;
  CURSOR *crsr = GetFromList(cursor);

  if (crsr == NULL)
    return (LPERR_INVCURSOR);
  if (crsr->CBL.PrzExe & Q_ASYNC)                          /* 26.02.10 */
    inter_wait_single(&crsr->CBL, -1);                       
  strncpy((char *)crsr->CBL.Command, "CLOS", 4);
  inter(&crsr->CBL, NULL, NULL, NULL, NULL);
  rc = GET_LINTER_CODE(crsr);
  ClearCursorDesc(crsr, L_TTRUE, L_TFALSE);           /* 29.06.09 */
  DelFromList(crsr);
  return (rc);
}

int LINPL_GetCursorOption(int cursor, int option, SV *sv)
{
  CURSOR *crsr = GetFromList(cursor);

  if (crsr == NULL)
    return (LPERR_INVCURSOR);
  switch(option)
  {
    case CO_CURNAME:
      sv_setpv(sv, crsr->name);
      break;
    case CO_DTFORMAT:
      sv_setpv(sv, crsr->dt_format);
      break;
    case CO_COLCOUNT:
      sv_setiv(sv, crsr->cols);
      break;
    case CO_ROWCOUNT:
      sv_setiv(sv, crsr->rows);
      break;
    case CO_ERRROW:
      sv_setiv(sv, (IV)(crsr->errpos & 0x000000FF));
      break;
    case CO_ERRPOS:
      sv_setiv(sv, (IV)(crsr->errpos & 0xFFFF0000) >> 16);
      break;
    case CO_TRANSMODE:
      sv_setiv(sv, TM_AUTOCOMMIT);
      if (crsr->CBL.PrzExe & TM_OPTIMISTIC)
        sv_setiv(sv, TM_OPTIMISTIC);
      if (crsr->CBL.PrzExe & TM_EXCLUSIVE)
        sv_setiv(sv, TM_EXCLUSIVE);
      break;
    case CO_CURROW:
      sv_setiv(sv, crsr->crow);
      break;
    case CO_CURROWID:
      sv_setiv(sv, crsr->CBL.RowId);
      break;
    case CO_CONNECTID:
      if (crsr->parent)
        sv_setiv(sv, (IV)(crsr->parent->CBL.NumChan));
      else
        return (LPERR_NOPARENT);
      break;
    case CO_NODENAME:
      sv_setpvn(sv, crsr->CBL.Node, 8);
      break;
    case CO_LAST_ROWID:                                    /* 24.02.00 ... */
      sv_setiv(sv, crsr->CBL.RowId);                       /* 06.12.00 */
      break;                                               /* ... 24.02.00 */
    case CO_PRIORITY:
      sv_setiv(sv, crsr->CBL.Prior);
      break;
    case CO_FETCH_BLOBS_AS_USUAL_DATA:                     /* 15.02.10B */
      sv_setiv(sv, crsr->BlobBehaviour);
      break;
    case CO_SYNCMODE:                                      /* 26.02.10 */
      sv_setiv(sv,((crsr->CBL.PrzExe & Q_ASYNC)?0:1));
      break;
    case CO_ASYNCMODE:                                     /* 26.02.10 */
      sv_setiv(sv,((crsr->CBL.PrzExe & Q_ASYNC)?1:0));
      break;
    case CO_ASYNCDONE:                                     /* 26.02.10 */
      sv_setiv(sv,((crsr->CBL.PrzExe & Q_ASYNCDONE)?1:0));
      break;
    default:
      return (LPERR_UNKNOWNOPT);
  }
  return (LPERR_SUCCESS);
}

int LINPL_SetCursorOption(int cursor, int option, SV *sv)
{
  int     len;
  L_LONG  mode;
  L_SWORD curVal;
  char   *tmp  = SvPV(sv, len);
  CURSOR *crsr = GetFromList(cursor);

  if (crsr == NULL)
    return (LPERR_INVCURSOR);
  if (crsr->CBL.PrzExe & Q_ASYNC)                          /* 26.02.10 */
    inter_wait_single(&crsr->CBL, -1);                       
  strncpy(crsr->CBL.Command, "SETO", 4);
  switch(option)
  {
    case CO_CURNAME:
      crsr->CBL.RowId = crName;                            /* 16.12.99 */
      crsr->CBL.LnBufRow = MAX_ID_LEN;                     /* 13.11.14 ... */
      crsr->CBL.RowCount = crsr->CBL.NumChan;
      inter(&crsr->CBL, NULL, NULL, NULL, tmp);            /* ... 13.11.14 */
      if (GET_LINTER_CODE(crsr) == 0)
      {
        memset(crsr->name, 0, sizeof(crsr->name));
        strncpy(crsr->name, tmp, MAX_ID_LEN);              /* 27.12.00 */
      }
      break;
    case CO_ASYNCMODE:
      crsr->CBL.PrzExe |= Q_ASYNC;
      return (LPERR_SUCCESS);
    case CO_SYNCMODE:
      crsr->CBL.PrzExe &= ~Q_ASYNC;
      return (LPERR_SUCCESS);
    case CO_DTFORMAT:
      s_free(crsr->dt_format);
      crsr->dt_format = malloc(strlen(tmp)+1);                       /* 07.06.10 */
      crsr->dt_format = memcpy(crsr->dt_format, tmp, strlen(tmp)+1); /* 07.06.10 */  
      return (LPERR_SUCCESS);
    case CO_PRIORITY:
      curVal = (L_WORD)SvIV(sv);                           /* 13.11.14 */
      crsr->CBL.RowId    = crPrior;
      crsr->CBL.Prior    = curVal;
      crsr->CBL.RowCount = crsr->CBL.NumChan;
      inter(&crsr->CBL, NULL, NULL, NULL, &curVal);        /* 13.11.14 */
      break;
    case CO_QUERY_PRIORITY:
      curVal = (L_WORD)SvIV(sv);                           /* 13.11.14 */
      crsr->CBL.RowId    = crQueryPrior;
      crsr->CBL.Prior    = curVal;
      crsr->CBL.RowCount = crsr->CBL.NumChan;
      inter(&crsr->CBL, NULL, NULL, NULL, &curVal);        /* 13.11.14 */
      break;
    case CO_CANCEL:
      crsr->CBL.RowId    = crCancel;
      crsr->CBL.RowCount = crsr->CBL.NumChan;
      inter(&crsr->CBL, NULL, NULL, NULL, tmp);            /* 13.11.14 */
      break;
    case CO_PAUSE:
      crsr->CBL.RowId    = crPause;
      crsr->CBL.RowCount = crsr->CBL.NumChan;
      inter(&crsr->CBL, NULL, NULL, NULL, tmp);            /* 13.11.14 */
      break;
    case CO_CONTINUE:
      crsr->CBL.RowId    = crContinue;
      crsr->CBL.RowCount = crsr->CBL.NumChan;
      inter(&crsr->CBL, NULL, NULL, NULL, tmp);            /* 13.11.14 */
      break;
    case CO_TRANSMODE:
      crsr->CBL.RowId = crMode;
      mode = (L_LONG)SvIV(sv);
      if (mode && mode != M_OPTIMISTIC && mode != M_EXCLUSIVE)
        return (LPERR_INVPARAMETER);
      crsr->CBL.PrzExe &= ~(M_OPTIMISTIC | M_EXCLUSIVE);
      crsr->CBL.PrzExe |= mode;
      crsr->CBL.RowCount = crsr->CBL.NumChan;              /* 13.11.14 ... */
      inter(&crsr->CBL, NULL, NULL, NULL, &mode);          /* ... 13.11.14 */
      break;
    case CO_FETCH_BLOBS_AS_USUAL_DATA:                     /* 15.02.10B */
      crsr->BlobBehaviour = (L_BOOL)SvIV(sv);
      return (LPERR_SUCCESS);                              /* 13.11.14 */
    default:
      return (LPERR_UNKNOWNOPT);
  }
  if (GET_LINTER_CODE(crsr) != 0)
    return (GET_LINTER_CODE(crsr));
  return (LPERR_SUCCESS);
}

int LINPL_SetCodepage(SV *cpname)                          /* 12.01.10 ... */  
{
#if _VER_MAX >= 600
  CURSOR *crsr = &List;
  char *cp;
  int size;
  if ( !inter_control( NULL, ICR_SET_CP, SvPV_nolen(cpname), CP_AS_STR) )
    return (LPERR_LINERROR);
  while (crsr)                                             /* 12.11.14a ... */
  {
    cp = SvPV(cpname, size);
    strncpy(crsr->Dsc.UseCharSetName, cp, size);
    crsr = crsr->next;
  }                                                        /* ...12.11.14a */
#endif
  return (LPERR_SUCCESS);
}                                                          /* ... 12.01.10 */

static int FillCursorDesc(CURSOR *crsr)
{
  TCBL RCBL;
  int  i, cols, delta = MaxAnswerSize / sizeof(GETA_OUT);  /* 03.02.10 */ /* 15.02.10 */
  char *firstRowBuf = NULL;                                /* 15.02.10A ... */

  if (crsr->hasFirstRow)
  {
    firstRowBuf = (L_BYTE *)s_calloc(1,crsr->CBL.LnBufRow);  /* 09.09.13 */
    memcpy(firstRowBuf,crsr->exch_buff,crsr->CBL.LnBufRow);
  }                                                        /* ... 15.02.10A */

  ClearCursorDesc(crsr, L_TFALSE, L_TFALSE);               /* 29.06.09 */ /* 23.12.09 */
  if (crsr->CBL.PrzExe & Q_ASYNC)                          /* 26.02.10 */
    inter_wait_single(&crsr->CBL, -1);                       
  memcpy(&RCBL, &crsr->CBL, sizeof(TCBL));
  crsr->CBL.RowId = crsr->CBL.LnBufRow = 0;
  strncpy((char *)crsr->CBL.Command, "GETA", 4);
  inter(&crsr->CBL, NULL, NULL, NULL, NULL);
  if (crsr->CBL.PrzExe & Q_ASYNC)                          /* 26.02.10 */
    inter_wait_single(&crsr->CBL, -1);                       
  if (GET_LINTER_CODE(crsr))
    return (GET_LINTER_CODE(crsr));
  cols = crsr->cols = crsr->CBL.RowCount;                  /* 06.12.00 */  
  crsr->answ_desc = (GETA_OUT *)s_calloc(cols, sizeof(GETA_OUT)); /* 09.09.13 */
  crsr->bind_cols = (SV **)s_calloc(cols, sizeof(SV *));   /* 09.09.13 */
  if (firstRowBuf != NULL)                                 /* 15.02.10A ... */  
    crsr->exch_buff = firstRowBuf;
  else
    crsr->exch_buff = (L_BYTE *)s_calloc(1, RCBL.LnBufRow);    /* ... 15.02.10A */

  crsr->null_flag = (L_BYTE *)s_calloc(cols + 4, 1); 
  crsr->ColOffset = (long *)s_calloc(cols,sizeof(long));     /* 03.02.10 */  
  if (!crsr->answ_desc || !crsr->null_flag ||
      !crsr->bind_cols || !crsr->exch_buff ||
      !crsr->ColOffset)
    return (LPERR_NOMEMORY);
  do{
    crsr->CBL.LnBufRow = MaxAnswerSize;                    /* 15.02.10 */
    inter(&crsr->CBL, NULL, NULL, NULL, &crsr->answ_desc[crsr->CBL.RowId]);
    if (crsr->CBL.PrzExe & Q_ASYNC)                        /* 26.02.10 */
      inter_wait_single(&crsr->CBL, -1);                       
    if (GET_LINTER_CODE(crsr))
      return (GET_LINTER_CODE(crsr));
    crsr->CBL.RowId += delta;
    cols            -= delta;
    } while(cols > 0);

  crsr->answ_len = 0;                                      /* 03.02.10 ... */
  for (i = 0; i < crsr->cols; i++)   
  {
    crsr->ColOffset[i] = crsr->answ_len;                      
    crsr->answ_len   += crsr->answ_desc[i].Length;
    if (crsr->answ_desc[i].Type == LDT_VARCHAR ||
        crsr->answ_desc[i].Type == LDT_VARBYTE ||
        crsr->answ_desc[i].Type == LDT_NVARCHAR)    
      crsr->answ_len += sizeof(L_WORD);          
  }                                                        /* ... 03.02.10 */

  memcpy(&crsr->CBL, &RCBL, sizeof(TCBL));  
  crsr->dt_format = malloc(strlen(DEF_DT_FORMAT)+1);       /* 07.06.10 */
  crsr->dt_format = memcpy(crsr->dt_format, DEF_DT_FORMAT, strlen(DEF_DT_FORMAT)+1); /* 07.06.10 */  
  
  return (LPERR_SUCCESS);
}

int LINPL_ExecDirect(int cursor, char *query)
{
  int     rc;
  char   *s    = query;
  CURSOR *crsr = GetFromList(cursor);

  if (crsr == NULL)
    return (LPERR_INVCURSOR);
  if (crsr->CBL.PrzExe & Q_ASYNC)                              /* 26.02.10 */
    inter_wait_single(&crsr->CBL, -1);                       
  ClearCursorDesc(crsr, L_TFALSE, L_TFALSE);                    /* 21.06.09 29.06.09 */
  crsr->crow = crsr->errpos = 0;
  while(strchr(STR_DELIM, *s) || *s == '(')
    s++;
  if (!strnicmp(s, "SELECT", 6) || !strnicmp(s, "WITH", 4))    /* 17.11.14 */
    {
    crsr->last_oper = SQL_SELECT;
    strncpy((char *)crsr->CBL.Command, "SLCT", 4);
    crsr->exch_buff = s_calloc(1, MAX_ROW_SIZE);               /* 09.09.13 15.02.10 */     
    }
  else
  {
    memset((char *)crsr->CBL.Command, ' ', 4);
    if (!strnicmp(s, "GRANT", 5))                              /* 25.01.10 ... */
      crsr->last_oper = SQL_GRANT;
    else if (!strnicmp(s, "ALTER", 5))
      crsr->last_oper = SQL_ALTER;
    else if ((!strnicmp(s, "EXECUTE", 7)) || (!strnicmp(s, "CALL", 4))) /* 21.06.09 */
    {
      crsr->last_oper = SQL_EXEC_PROC; 
      crsr->exch_buff = s_calloc(1, MAX_ROW_SIZE);               /* 09.09.13 15.02.10 */      
    }
    else                                                       /* ... 25.01.10 */ 
      crsr->last_oper = SQL_UNKNOWN;                           /* 21.06.09 */
  }
  
  crsr->CBL.LnBufRow = MAX_ROW_SIZE;                           /* 15.02.10 */  
  inter(&crsr->CBL, NULL, query, NULL, crsr->exch_buff); 
  if (GET_LINTER_CODE(crsr) > EORR)                            /* 18.04.00 */
    {
    if (GET_LINTER_CODE(crsr) >= SQLBADCH && GET_LINTER_CODE(crsr) < SQLENDSYMEXP)
      crsr->errpos = (int)crsr->CBL.SysErr;
    return (GET_LINTER_CODE(crsr));
    }
  crsr->rows = crsr->CBL.RowCount;                         /* 06.12.00 */ 
  if (crsr->last_oper == SQL_SELECT)
  {
    crsr->hasFirstRow = 1;                                 /* 15.02.10A */
    if ((rc = FillCursorDesc(crsr)) != 0)
      return (rc);    
  }                                                        /* 15.02.10A */
  else if ((crsr->last_oper == SQL_GRANT) || (crsr->last_oper == SQL_ALTER)) /* 25.01.10 ... */
  {
    if (crsr->CBL.LnBufRow)
      crsr->pass_cnt = crsr->CBL.LnBufRow/18;
  }                                                            /* ... 25.01.10 */  
  return (GET_LINTER_CODE(crsr));
}

int LINPL_Prepare(int cursor, char *query)
{
  CURSOR *crsr = GetFromList(cursor);
  short   p_count;                                        /* 15.04.09 */
  int     i;                                              /* 12.05.09 */

  if (crsr == NULL)
    return (LPERR_INVCURSOR);  
  crsr->pqry_body = realloc(crsr->pqry_body,LongQueryMaxLength); /* 12.05.09 */ /* 15.02.10 */ 
  if (crsr->pqry_body == NULL)
    return (LPERR_NOMEMORY);  
  if (crsr->CBL.PrzExe & Q_ASYNC)                          /* 26.02.10 */
    inter_wait_single(&crsr->CBL, -1);                       
  memset(crsr->pqry_body,0,LongQueryMaxLength);            /* 21.06.09 */ /* 15.02.10 */
  crsr->crow = crsr->errpos = 0;
  crsr->CBL.LnBufRow = LongQueryMaxLength;
  strncpy((char *)crsr->CBL.Command, "TRAN", 4);
  inter(&crsr->CBL, NULL, query, NULL, crsr->pqry_body);
  if (GET_LINTER_CODE(crsr) >= SQLBADCH && GET_LINTER_CODE(crsr) < SQLENDSYMEXP)
  {
    crsr->errpos = (int)crsr->CBL.SysErr;                 /* 30.10.09 ... */
    free(crsr->pqry_body);
    crsr->pqry_body = NULL;
    return (GET_LINTER_CODE(crsr));                       /* ... 30.10.09 */
  }

  inter_SetEncoding(crsr->pqry_body,crsr->Flags);
  inter_GetParamCount(crsr->pqry_body, &p_count);         /* 15.04.09... */
  crsr->parm_leng = realloc(crsr->parm_leng, sizeof(short) * p_count); 
  if (p_count && !crsr->parm_leng)
    return (LPERR_NOMEMORY);                              /* ... 15.04.09 */
  
  for (i=0;i<crsr->param_count;i++)                       /* 12.05.09 ...*/  
    if (crsr->bind_params[i])
        crsr->bind_params[i]->sv_refcnt--;      

  crsr->param_count = p_count;                            /* 15.04.09 */
  crsr->bind_params = realloc(crsr->bind_params, sizeof(SV*)*p_count); 
  memset(crsr->bind_params,0,sizeof(SV*)*p_count);         
  if (p_count && !crsr->bind_params)
    return (LPERR_NOMEMORY);                              /* ... 12.05.09 */

  return (GET_LINTER_CODE(crsr));
}

int LINPL_BindParameter(int cursor, int num, SV *sv)
{  
  CURSOR     *crsr  = GetFromList(cursor);    

  if (crsr == NULL)
    return (LPERR_INVCURSOR);
  if (crsr->pqry_body == NULL)
    return (LPERR_INVSTATE);
  if (num < 1 || num > crsr->param_count)                  /* 15.04.09 */
    return (LPERR_INVPARAMETER);
 
  if (num <= 0 || num > crsr->param_count)                 /* 12.05.09 */
    return (LPERR_INVPARAMNUM);

  crsr->bind_params[num-1] = sv;                           /* 12.05.09 */  
  sv->sv_refcnt++;                                         /* 12.05.09 */
  return (GET_LINTER_CODE(crsr));
}

int LINPL_BindParamArray(int cursor, AV *av, int begin, int end) /* 30.10.09  ... */
{
  int         i, j, rc, av_length;                           
  L_SWORD     cnt;
  SV         *sv;
  CURSOR     *crsr  = GetFromList(cursor);

  if (crsr == NULL)
    return (LPERR_INVCURSOR);
  if (crsr->pqry_body == NULL)
    return (LPERR_INVSTATE);
  rc = LPERR_SUCCESS;
  begin = begin<1?1:begin;

  inter_GetParamCount(crsr->pqry_body, &cnt);
  if (end > 0)
    cnt = min(cnt,end);

  av_length = av_len(av);  
  j=0;  
  for(i = 0; i < (cnt-begin+1); i++)
  {    
    do
    {
      sv = *av_fetch(av, i, 0);                              /* 11.11.14a */
      j++;
    }
    while (!SvOK(sv) && j<=av_length);
    if (SvOK(sv))
    {
      rc = LINPL_BindParameter(cursor, i+begin, sv);        
      if (rc != LPERR_SUCCESS)
        break;
    }    
  }  
  return (rc);                                               /* ... 30.10.09 */
}

int LINPL_Execute(int cursor)
{
  int     rc;
  short   p_type, i;                              /* 12.05.09 */
  char   *p_value;                                /* 12.05.09 */
  CURSOR *crsr = GetFromList(cursor);

  if (crsr == NULL)
    return (LPERR_INVCURSOR);
  if (crsr->pqry_body == NULL)                    /* 30.10.09 */
    return (LPERR_INVSTATE);
  if (crsr->CBL.PrzExe & Q_ASYNC)                 /* 26.02.10 */
    inter_wait_single(&crsr->CBL, -1);                       

  for (i = 0; i<crsr->param_count; i++)           /* 12.05.09 ... */
  {
    inter_DescribeParam(crsr->pqry_body, i+1, &crsr->parm_leng[i], 
                        &p_type, NULL, NULL, NULL);
    if (p_type == LDT_BLOB)       
      crsr->BlFlag = L_TTRUE;      
    
    if (!crsr->bind_params[i])
      continue;

    switch (SvTYPE(crsr->bind_params[i]))                  /* 08.06.10 */                            
    {
      case SVt_PV: p_type = LDT_CHAR;                     
                 crsr->parm_leng[i] = strlen(SvPVX(crsr->bind_params[i]));
                 p_value = SvPVX(crsr->bind_params[i]);
                 break; 
      case SVt_PVNV: if (SvPOK(crsr->bind_params[i]))
                 {
                   p_type = LDT_CHAR;                     
                   crsr->parm_leng[i] = strlen(SvPVX(crsr->bind_params[i]));
                   p_value = SvPVX(crsr->bind_params[i]);
                   break; 
                 }    
                 /* NO BREAK */
      case SVt_NV: if (SvNOK(crsr->bind_params[i]))
                 {
                   p_type = LDT_REAL;                 
                   p_value = (char *)&SvNVX(crsr->bind_params[i]);
#ifdef NVTYPE                                               /* 30.10.09 */
                   crsr->parm_leng[i] = sizeof(NV);
#else
                   crsr->parm_leng[i] = sizeof(double);
#endif
                   break;   
                 }
                 /* NO BREAK */
      case SVt_PVIV: if (SvPOK(crsr->bind_params[i]))
                 {
                   p_type = LDT_CHAR;                     
                   crsr->parm_leng[i] = strlen(SvPVX(crsr->bind_params[i]));
                   p_value = SvPVX(crsr->bind_params[i]);
                   break; 
                 }                 
                 /* NO BREAK */
      case SVt_IV: p_type = LDT_INTEGER;
                 crsr->parm_leng[i] = sizeof(IV);           
                 p_value = (char *)&SvIVX(crsr->bind_params[i]);
                 break;                 
      case SVt_NULL:
#if PERL_REVISION < 5 || (PERL_REVISION == 5 && PERL_VERSION < 12) /* 24.09.10 */      
      case SVt_RV:
#endif
      case SVt_PVMG: 
#if PERL_REVISION < 5 || (PERL_REVISION == 5 && PERL_VERSION < 10) /* 08.06.10 */
      case SVt_PVBM:
#elif PERL_REVISION == 5 && PERL_VERSION >= 10 && PERL_VERSION <= 19 && PERL_SUBVERSION < 3  /* 20.11.14 ... */
      case SVt_BIND:
#elif PERL_REVISION == 5 && PERL_VERSION >= 19 && PERL_SUBVERSION >= 3
      case SVt_INVLIST:                                                                      /* ... 20.11.14 */
#endif
      case SVt_PVLV:
      case SVt_PVAV:
      case SVt_PVHV:
      case SVt_PVCV:
      case SVt_PVGV:
      case SVt_PVFM:
      case SVt_PVIO:
      default:     p_type = DT_DEFAULT;
                   crsr->parm_leng[i] = 0;
                   p_value = (char *)crsr->bind_params[i];
                   break;
    }       

    inter_bind_num(&crsr->CBL, crsr->pqry_body, i+1, p_type, &crsr->parm_leng[i], p_value);

  }                                                        /* ... 12.05.09 */
  

  crsr->CBL.LnBufRow = MAX_ROW_SIZE;                       /* 15.02.10 */
  strncpy((char *)crsr->CBL.Command, "EXEC", 4);
  crsr->last_oper = inter_isselect(&crsr->CBL, crsr->pqry_body); /* 21.06.09 */

  if (crsr->last_oper == SQL_EXEC_PROC || crsr->last_oper == SQL_SELECT) /* 21.06.09B ... */ /* 15.02.10A */
    crsr->exch_buff = s_calloc(1,MAX_ROW_SIZE);              /* 09.09.13 15.02.10 */

  inter(&crsr->CBL, NULL, crsr->pqry_body, NULL, crsr->exch_buff); /* ... 21.06.09B */  

  if (GET_LINTER_CODE(crsr) > EORR)                        /* 18.04.00 */
    return (GET_LINTER_CODE(crsr));
  crsr->rows = crsr->CBL.RowCount;                         /* 06.12.00 */ 
  if (crsr->last_oper == SQL_SELECT)                       /* 31.10.00 */ /* 21.06.09 */
  {
    crsr->hasFirstRow = 1;                                 /* 15.02.10A */
    if ((rc = FillCursorDesc(crsr)) != 0)
      return (rc);    
  }                                                        /* 15.02.10A */
  else                                                     /* 12.05.09A ... */ 
  {
    if ((crsr->last_oper == SQL_GRANT) || (crsr->last_oper == SQL_ALTER)) /* 25.01.10 ... */ 
    {
      if (crsr->CBL.LnBufRow)
        crsr->pass_cnt = crsr->CBL.LnBufRow/18;
      return (GET_LINTER_CODE(crsr));
    }                                                      /* ... 25.01.10 */            
    if (crsr->BlFlag)                                              
    {       
      for(i = 1; i <= crsr->param_count; i++)              /* 24.12.09 */
      {
        inter_DescribeParam(crsr->pqry_body, i, NULL, &p_type, NULL, NULL, NULL);                              /* 24.12.09 */ 
        if (p_type != LDT_BLOB)
          continue;         
        if (ClearBlob(crsr,inter_GetBlobColNum(crsr->pqry_body,i))!=LPERR_SUCCESS)                             /* 24.12.09 */  
          return LPERR_INVPARAMETER;          
        if (LINPL_BLOBAdd(cursor, crsr->bind_params[i-1],inter_GetBlobColNum(crsr->pqry_body,i))!=LPERR_SUCCESS) /* 24.12.09 */  
          return LPERR_INVPARAMETER;
      }  
    }   
  }                                                          /* ... 12.05.09A */
  return (GET_LINTER_CODE(crsr));
}

int LINPL_Fetch(int cursor, int direct, int pos)
{
  int     i;
  CURSOR *crsr = GetFromList(cursor);

  if (crsr == NULL)
    return (LPERR_INVCURSOR);
  if (crsr->CBL.PrzExe & Q_ASYNC)                          /* 26.02.10 */
    inter_wait_single(&crsr->CBL, -1);                       
  switch(direct)
    {
    case FETCH_FIRST:
      strncpy((char *)crsr->CBL.Command, "GETF", 4);
      crsr->crow = 1;
      break;
    case FETCH_LAST:
      strncpy((char *)crsr->CBL.Command, "GETL", 4);
      crsr->crow = crsr->rows;
      break;
    case FETCH_NEXT:
      strncpy((char *)crsr->CBL.Command, "GETN", 4);
      crsr->crow++;
      break;
    case FETCH_PREV:
      strncpy((char *)crsr->CBL.Command, "GETP", 4);
      crsr->crow--;
      break;
    case FETCH_ABSNUM:
      strncpy((char *)crsr->CBL.Command, "GETS", 4);
      crsr->CBL.RowId = pos;
      break;
    default:
      return (LPERR_INVDIRECT);
    }
  crsr->CBL.LnBufRow = MAX_ROW_SIZE;                       /* 21.06.09 */ /* 15.02.10 */
  inter(&crsr->CBL, crsr->null_flag, NULL, NULL, crsr->exch_buff);
  if (GET_LINTER_CODE(crsr) != 0)
    return (GET_LINTER_CODE(crsr));
  if (direct == FETCH_ABSNUM)
    crsr->crow = pos;
  for(i = 0; i < crsr->cols; i++)
    if (crsr->bind_cols[i] != NULL)
      LINPL_GetDataColumn(cursor, i + 1, crsr->bind_cols[i]);
  return (LPERR_SUCCESS);
}

int LINPL_Commit(int cursor)
{
  char   *q    = NULL;
  CURSOR *crsr = GetFromList(cursor);

  if (crsr == NULL)
    return (LPERR_INVCURSOR);
  if (crsr->CBL.PrzExe & Q_ASYNC)                          /* 26.02.10 */
    inter_wait_single(&crsr->CBL, -1);                       

  strncpy(crsr->CBL.Command, "    ", 4);
  q = "commit;";
  inter(&crsr->CBL, NULL, q, NULL, NULL);
  return (GET_LINTER_CODE(crsr));
}

int LINPL_Rollback(int cursor)
{
  char   *q    = NULL;
  CURSOR *crsr = GetFromList(cursor);

  if (crsr == NULL)
    return (LPERR_INVCURSOR);
  if (crsr->CBL.PrzExe & Q_ASYNC)                          /* 26.02.10 */
    inter_wait_single(&crsr->CBL, -1);                       
  strncpy(crsr->CBL.Command, "    ", 4);
  q = "rollback;";
  inter(&crsr->CBL, NULL, q, NULL, NULL);
  return (GET_LINTER_CODE(crsr));
}

int LINPL_BindColumn(int cursor, int col, SV *sv)
{
  CURSOR *crsr = GetFromList(cursor);

  if (crsr == NULL)
    return (LPERR_INVCURSOR);
  if (col <= 0 || col > crsr->cols)
    return (LPERR_INVCOLNUM);
  crsr->bind_cols[col - 1] = sv;
  return (LPERR_SUCCESS);
}

int LINPL_UnbindColumn(cursor, col)
{
  CURSOR   *crsr = GetFromList(cursor);

  if (crsr == NULL)
    return (LPERR_INVCURSOR);
  if (col <= 0 || col > crsr->cols)                        /* 08.02.10 */
    return (LPERR_INVCOLNUM);
  crsr->bind_cols[col - 1] = NULL;
  return (LPERR_SUCCESS);
}

/*
int LINPL_BindColArray(int, int, int, AV *av)
{
 return (LPERR_NOTIMPLEMENTED);
}
*/

int GetProcOutData(int cursor, int par_num, SV *sv) /* 01.02.10 */
{
  double       dec_val;
  char         dt_val[32];
  L_WORD      *par_count;
  L_BYTE      *par_value;
  L_WORD      *par_val_len;
  ARGPROC_OUT *par_info;
  CURSOR      *crsr = GetFromList(cursor);
  char        *tmpstr = NULL;                   /* 26.10.04 */
  L_LONG       tmplen;                          /* 26.10.04 */

  if (crsr == NULL)
    return (LPERR_INVCURSOR);
  if (crsr->exch_buff == NULL)
    return (LPERR_INVSTATE);
  if (crsr->CBL.PrzExe & Q_ASYNC)               /* 26.02.10 */
    inter_wait_single(&crsr->CBL, -1);                       

  if (GET_LINTER_CODE(crsr) == Proc_Raised_Exception)
    {
    sv_setiv(sv, (IV)GET_SYSTEM_CODE(crsr));
    return (LPERR_SUCCESS);
    }
  par_num--;                                    /* 21.06.09 */
  par_info  = (ARGPROC_OUT *)crsr->exch_buff;
  par_count = (L_WORD *)(par_info + 1);
  if ((L_WORD)par_num > *par_count)
    return (LPERR_INVPARAMETER);

  if (par_num > 0)
    {
    par_info  = (ARGPROC_OUT *)(par_count + 1);
    par_info += par_num - 1;
    }
  par_value = crsr->exch_buff + par_info->Value;

  if (par_info->Flags & PARM_FLAG_NAMESENT)
    {
    L_WORD par_name_len = *((L_WORD *)par_value);
    par_value += sizeof(L_WORD) + par_name_len;
    }

  if (par_info->Flags & PARM_FLAG_ISCURSOR)
    {
    CURSOR   new_crsr;
    CR_INFO *cr_info = (CR_INFO *)par_value;
    memset(&new_crsr, 0, sizeof(CURSOR));
    if (crsr->parent == NULL)
      new_crsr.parent = crsr;
    else
      new_crsr.parent = crsr->parent;
    memcpy(&new_crsr.CBL, &(crsr->CBL), sizeof(TCBL));
    new_crsr.CBL.CodErr   = cr_info->CodErr;
    new_crsr.CBL.NumChan  = cr_info->NumChan;
    new_crsr.CBL.RowId    = cr_info->RowId;
    new_crsr.CBL.RowCount = cr_info->RowCount;
    new_crsr.CBL.LnBufRow = cr_info->LnBufRow;
    FillCursorDesc(&new_crsr);
    AddToList(&new_crsr);
    LINPL_Fetch((int)new_crsr.id, FETCH_FIRST, 0);         /* 03.01.01 */
    sv_setiv(sv, (IV)new_crsr.id);                         /* 03.01.01 */
    return (LPERR_SUCCESS);
    }

  if (par_info->Flags & PARM_FLAG_ISNULL)
    {
    sv_setsv(sv, &sv_undef);
    return (LPERR_SUCCESS);
    }

  switch(par_info->Type.ntyp)
    {
    case LDT_BYTE:
    case LDT_CHAR:
      sv_setpvn(sv, par_value, par_info->Type.length);
      break;
    case LDT_INTEGER:
      if (par_info->Type.length == 8)
        {
        i64_ToDouble(*(L_DLONG *)par_value, dec_val);
        sv_setnv(sv, dec_val);
        }
      else
        if (par_info->Type.length == 4)
          sv_setiv(sv, (IV)(*((L_LONG *)par_value)));
        else
          sv_setiv(sv, (IV)(*((L_WORD *)par_value)));
      break;
    case LDT_REAL:
      if (par_info->Type.length == 4)
        sv_setnv(sv, (double)(*((float *)par_value)));
      else
        sv_setnv(sv, (double)(*((double*)par_value)));
    case LDT_NUMERIC:
      DecToDbl(*(L_DECIMAL*)par_value, &dec_val);
      sv_setnv(sv, dec_val);
      break;
    case LDT_DATE:
      TICKTOSTRF(*(L_DECIMAL*)par_value, crsr->dt_format, dt_val);
      sv_setpv(sv, dt_val);
      break;
    case LDT_BLOB:
      return (LPERR_NOTIMPLEMENTED);
    case LDT_VARCHAR:
    case LDT_VARBYTE:
      par_val_len = (L_WORD *)par_value;
      par_value += sizeof(L_WORD);
      sv_setpvn(sv, par_value, (IV)(*par_val_len));
      break;
    case LDT_BOOLEAN:
      sv_setpv(sv, (*par_value != 0) ?  "TRUE" : "FALSE");
      break;
    case LDT_NCHAR:                                        /* 26.10.04 ... */
      /* define tmpstr length*/
      LINNLS_Ucs2Utf8( (L_UNICHAR *)par_value,
                       (L_LONG)(par_info->Type.length / 2),
                       (L_LONG)(par_info->Type.length * 4),
                       NULL,
                       &tmplen );
      if(!(tmpstr = (char *)s_calloc(1, tmplen+1)))          /* 09.09.13 21.06.10 */
        return (LPERR_NOMEMORY);
      LINNLS_Ucs2Utf8( (L_UNICHAR *)par_value,
                       (L_LONG)(par_info->Type.length / 2),
                       (L_LONG)tmplen,
                       tmpstr,
                       &tmplen );
      sv_setpvn(sv,tmpstr,(STRLEN)tmplen);
      SvUTF8_on( sv ); 
      free(tmpstr); 
    break;
    case LDT_NVARCHAR:
      /* define tmpstr length*/
      LINNLS_Ucs2Utf8( (L_UNICHAR *)(par_value + 2),
                       *(L_WORD*)par_value / 2,
                       (L_LONG)( *(L_WORD*)par_value * 4),
                       NULL,
                       &tmplen );
      if(!(tmpstr = (char *)s_calloc(1, tmplen+1)))          /* 09.09.13 21.06.10 */
        return (LPERR_NOMEMORY);
      LINNLS_Ucs2Utf8( (L_UNICHAR *)(par_value + 2),
                       *(L_WORD*)par_value / 2,
                       (L_LONG)tmplen,
                       tmpstr,
                       &tmplen );
      sv_setpvn(sv,tmpstr,(STRLEN)tmplen);
      SvUTF8_on( sv ); 
      free(tmpstr); 
      break;                                               /* ... 26.10.04 */
    default:
      return (LPERR_NOTIMPLEMENTED);
    }
  return (LPERR_SUCCESS);
}

int GetProcOutDataArray(int cursor, AV *av)                /* 01.02.10 ... */
{
  CURSOR      *crsr = GetFromList(cursor);
  SV          *sv;   
  int          i,res;
  L_WORD      *par_count;  
  ARGPROC_OUT *par_info;
  
  if (crsr == NULL)
    return (LPERR_INVCURSOR);
  if (crsr->exch_buff == NULL)
    return (LPERR_INVSTATE);
  if (crsr->last_oper != SQL_EXEC_PROC)     
    return (LPERR_INVSTATE);

  av_clear(av);
  if (GET_LINTER_CODE(crsr) == Proc_Raised_Exception)
  {
    sv = newSViv((IV)GET_SYSTEM_CODE(crsr));
    av_push(av, sv);
    return (LPERR_SUCCESS);
  }

  par_info  = (ARGPROC_OUT *)crsr->exch_buff;
  par_count = (L_WORD *)(par_info + 1);

  for (i = 0; i <= *par_count; i++)
  {
    sv = newSV(0);
    res = GetProcOutData(cursor, i+1, sv);
    if (res != LPERR_SUCCESS)
      return (res);
    av_push(av, sv);
  }

  return (LPERR_SUCCESS);
}                                                          /* ... 01.02.10 */

int DataToSV(CURSOR *crsr, int col, int type, int length, char *val, SV *sv) /* 03.02.10A ... */
{
  double    dec_val;
  char      dt_val[32];
  char     *tmpstr = NULL;                                 /* 26.10.04 */
  L_LONG    tmplen;                                        /* 26.10.04 */  
  L_WORD    len;
  EXTFILE_DESC *ef;                                        /* 14.11.14 */

  switch(type)
    {
    case LDT_BYTE:
    case LDT_CHAR:
      sv_setpvn(sv, val, length);
      break;
    case LDT_INTEGER:
      if (length == 8)
        {
        i64_ToDouble(*(L_DLONG *)val, dec_val);
        sv_setnv(sv, dec_val);
        }
      else
        if (length == 4)
          sv_setiv(sv, (IV)(*((L_LONG *)val)));            /* 26.09.12 */
        else
          sv_setiv(sv, (IV)(*((L_SWORD*)val)));            /* 24.04.14 */
      break;
    case LDT_REAL:      
      if (length == 4)        
        sv_setnv(sv, (double)(*((float *)val)));
      else
        sv_setnv(sv, (double)(*((double*)val)));
      break;                                               /* 15.04.09A */
    case LDT_NUMERIC:
      DecToDbl(*(L_DECIMAL*)val, &dec_val);
      sv_setnv(sv, dec_val);
      break;
    case LDT_DATE:
      TICKTOSTRF(*(L_DECIMAL*)val, crsr->dt_format, dt_val);
      sv_setpv(sv, dt_val);
      break;
    case LDT_BLOB:                                         /* 08.05.09 ... */
      if (crsr->BlobBehaviour)                             /* 15.02.10B */
      {        
        L_LONG  res;        
        SV     *tempSv;
        
        tempSv = malloc(sizeof(SV));                
        memset(tempSv,0,sizeof(SV));
        sv_setiv(tempSv,OPTIMUM_PORTION_SIZE);             /* 15.02.10 */            
        
        res = FetchBlob(crsr, 1, tempSv, col+1, sv);       /* 24.12.09 */
        free(tempSv);                                                               
        if (res != LPERR_SUCCESS)                 
          return (res);      
      }      
      else                                                 /* 15.02.10B */
      {
        sv_setpv(sv, "");
        return (LPERR_SUCCESS);
      }
      break;                                               /* ... 08.05.09 */
    case LDT_VARCHAR:                                      /* 12.04.00 ... */
    case LDT_VARBYTE:
      len  = *(L_WORD *)val;
      val += sizeof(L_WORD);                               /* 03.02.10A */      
      sv_setpvn(sv, val, (IV)len);
      break;
    case LDT_BOOLEAN:
      sv_setpv(sv, (*val != 0) ?  "TRUE" : "FALSE");
      break;
    case LDT_NCHAR:                                        /* 26.10.04 ... */
      /* define tmpstr length*/
      LINNLS_Ucs2Utf8( (L_UNICHAR *)val,
                       (L_LONG)(length / 2),
                       (L_LONG)(length * 4),
                       NULL,
                       &tmplen );
      if(!(tmpstr = (char *)s_calloc(1, tmplen+1)))          /* 09.09.13 21.06.10 */
        return (LPERR_NOMEMORY);
      LINNLS_Ucs2Utf8( (L_UNICHAR *)val,
                       (L_LONG)(length / 2),
                       (L_LONG)tmplen,
                       tmpstr,
                       &tmplen );
      sv_setpvn(sv,tmpstr,(STRLEN)tmplen);
      SvUTF8_on( sv ); 
      free(tmpstr); 
    break;
    case LDT_NVARCHAR:
      /* define tmpstr length*/
      LINNLS_Ucs2Utf8( (L_UNICHAR *)(val + 2),
                       *(L_WORD*)val / 2,
                       (L_LONG)( *(L_WORD*)val * 4),
                       NULL,
                       &tmplen );
      if(!(tmpstr = (char *)s_calloc(1, tmplen+1)))          /* 09.09.13 21.06.10 */
        return (LPERR_NOMEMORY);
      LINNLS_Ucs2Utf8( (L_UNICHAR *)(val + 2),
                       *(L_WORD*)val / 2,
                       (L_LONG)tmplen,
                       tmpstr,
                       &tmplen );
      sv_setpvn(sv,tmpstr,(STRLEN)tmplen);
      SvUTF8_on( sv ); 
      free(tmpstr); 
      break;                                            /* ... 26.10.04 */
    case LDT_EXTFILE:                                     /* 14.11.14 ... */
      ef = (EXTFILE_DESC*)val;
      sv_setpvn(sv, ef->FileName, (STRLEN)strlen(ef->FileName));
      break;                                              /* ... 14.11.14 */
    default:
      return (LPERR_NOTIMPLEMENTED);
    }
  return (LPERR_SUCCESS);  

}                                                          /* ... 03.02.10A */

int LINPL_GetDataColumn(int cursor, int col, SV *sv)
{
  GETA_OUT *col_desc;
  CURSOR   *crsr = GetFromList(cursor);  
  int       off = 0;                                       /* 03.02.10 */  
    
  if (crsr == NULL)
    return (LPERR_INVCURSOR);  
  if (crsr->CBL.PrzExe & Q_ASYNC)                          /* 26.02.10 */
    inter_wait_single(&crsr->CBL, -1);                       
  if (crsr->CBL.CodErr)                                    /* 30.10.09 */
    return GET_LINTER_CODE(crsr);                          /* 30.10.09 */  
  if (crsr->exch_buff == NULL)
    return (LPERR_INVSTATE); 
  if (crsr->last_oper == SQL_EXEC_PROC)     
    return (GetProcOutData(cursor, col, sv));  
  if (col <= 0 || col > crsr->cols)                        /* 08.02.10 */
    return (LPERR_INVCOLNUM);  
  col--;
  if (crsr->null_flag[col + 4])
  {
    sv_setsv(sv, &sv_undef);
    return (LPERR_SUCCESS);
  }
  col_desc = &crsr->answ_desc[col];
  off      = crsr->ColOffset[col];                         /* 03.02.10 */

  return (DataToSV(crsr, col, col_desc->Type, col_desc->Length, &crsr->exch_buff[off], sv)); /* 03.02.10A */  
}

int LINPL_GetDataRow(int cursor, AV *av)                   /* 01.02.10 ... */ 
{  
  CURSOR   *crsr = GetFromList(cursor);
  SV       *sv;
  L_LONG    i    = 0;
  int       ret;

  if (crsr == NULL)
    return (LPERR_INVCURSOR);  
  if (crsr->CBL.PrzExe & Q_ASYNC)                          /* 26.02.10 */
    inter_wait_single(&crsr->CBL, -1);                       
  if (crsr->CBL.CodErr)      
    return GET_LINTER_CODE(crsr);                    
  if (crsr->exch_buff == NULL)
    return (LPERR_INVSTATE); 
    
  if (crsr->last_oper == SQL_EXEC_PROC)     
    return (GetProcOutDataArray(cursor, av));  
  
  if (((crsr->last_oper == SQL_ALTER) || (crsr->last_oper == SQL_GRANT)) &&
       (crsr->pass_cnt))    
  {    
    av_clear(av);
    
    for (i=0;i < crsr->pass_cnt;i++)
    {
      sv = newSVpv(&crsr->exch_buff[i*18], 18);
      av_push(av, sv);
    }
    return (LPERR_SUCCESS);
  }
  
  if (crsr->last_oper != SQL_SELECT)
    return (LPERR_INVSTATE);
    
  av_clear(av);  

  for (i = 1; i <= crsr->cols; i++)
  {
    sv = newSV(0);
    ret = LINPL_GetDataColumn(cursor, i, sv);
    if (ret != LPERR_SUCCESS)
      return (ret);
    av_push(av, sv);
  }
  
  return (LPERR_SUCCESS);
}           

int LINPL_GetDataRows(int cursor, AV *av, int rows) 
{
  int     i;
  L_LONG  Ret;    
  AV     *avRow;  
  CURSOR *crsr = GetFromList(cursor);

  if (crsr == NULL)
    return (LPERR_INVCURSOR);  
  if (crsr->CBL.PrzExe & Q_ASYNC)                          /* 26.02.10 */
    inter_wait_single(&crsr->CBL, -1);                       
  if (crsr->CBL.CodErr)                              
    return GET_LINTER_CODE(crsr);                    
  if (crsr->last_oper != SQL_SELECT)
    return (LPERR_INVSTATE); 
  if (rows <= 0)
    return (LPERR_INVPARAMETER); 
  
  av_clear(av);
  for (i = 0; (i < rows) && (crsr->CBL.CodErr < EORR); i++)
  {    
    if (crsr->CBL.CodErr != NOKOR)                               
    {
      avRow = newAV();
            
      if (Ret = LINPL_GetDataRow(cursor, avRow)) 
        return (Ret);            
      av_push(av, newRV_noinc((SV*)avRow));            
    }
    LINPL_Fetch(cursor, FETCH_NEXT, 0);
  }
  if (crsr->CBL.CodErr > EORR)                                 
    return (LPERR_LINERROR);
  
  crsr->CBL.CodErr = 0;

  return (LPERR_SUCCESS);
}              

int LINPL_GetDataArray (int cursor, HV *hv) 
{
  int     i;
  L_LONG  ret;   
  SV     *sv;
  CURSOR *crsr = GetFromList(cursor);
  char    ColName[MAX_ID_LEN + 1];    
    
  if (crsr == NULL)
    return (LPERR_INVCURSOR);  
  if (crsr->CBL.PrzExe & Q_ASYNC)                          /* 26.02.10 */
    inter_wait_single(&crsr->CBL, -1);                       
  if (crsr->CBL.CodErr)                              
    return GET_LINTER_CODE(crsr);                    
  if (crsr->exch_buff == NULL)
    return (LPERR_INVSTATE); 
      
  if (crsr->last_oper != SQL_SELECT)
    return (LPERR_INVSTATE);
    
  memset(ColName, 0, sizeof(ColName));  
  hv_clear(hv);   
  for (i = 1; i <= crsr->cols; i++)
  {    
    strncpy(ColName, crsr->answ_desc[i-1].Column, MAX_ID_LEN);
    ColName[MAX_ID_LEN] = 0;
    strrtrim(ColName, " ");                                

    sv = newSV(0);
    ret = LINPL_GetDataColumn(cursor, i, sv);
    if (ret != LPERR_SUCCESS)
      return (ret);
    hv_store(hv, ColName, strlen(ColName), sv, 0);
  }
  
  return (LPERR_SUCCESS);
}                                                          /* ... 01.02.10 */

int LINPL_GETM(int cursor, AV *av, int ansnum)             /* 03.02.10A ... */
{
  CURSOR *crsr = GetFromList(cursor);
  long    rc, rows_inbuf = 0;
  long    len, i, j, off = 0;
  SV     *sv;
  AV     *avRow;
  char   *row;
    
  if (crsr == NULL)
    return (LPERR_INVCURSOR);  
  if (crsr->CBL.PrzExe & Q_ASYNC)                          /* 26.02.10 */
    inter_wait_single(&crsr->CBL, -1);                       
  if (crsr->CBL.CodErr)                              
    return GET_LINTER_CODE(crsr);                    
  if (crsr->last_oper != SQL_SELECT)
    return (LPERR_INVSTATE); 
  if ((ansnum <= 0) || (ansnum > crsr->rows))
    return (LPERR_INVPARAMETER); 
    
  rows_inbuf = MAX_CLIENT_BUFSIZE / crsr->answ_len;        /* 15.02.10 */
  if (!rows_inbuf)
  {
    L_BOOL blobBehaviour = crsr->BlobBehaviour;            /* 15.02.10B ... */
    long   res;                                                   
    crsr->BlobBehaviour = L_TFALSE;                        /* ... 15.02.10B */
    rc = LINPL_Fetch(cursor, FETCH_ABSNUM, ansnum);        /* 15.02.10C */        
    if (rc != LPERR_SUCCESS)
      return (rc);    
    res = LINPL_GetDataRow(cursor, av);                    /* 15.02.10B ... */
    crsr->BlobBehaviour = blobBehaviour;                   
    return (res);                                          /* ... 15.02.10B */
  }
  if ((ansnum < crsr->FstRow) || (ansnum > crsr->LstRow ))
  {
    L_LONG SecPos = ((ansnum-1)/rows_inbuf)*rows_inbuf + 1;
    strncpy((char *)crsr->CBL.Command, "GETM", 4);
    crsr->CBL.RowCount = rows_inbuf;
    crsr->CBL.RowId = SecPos;
    if (!crsr->getm_buff)
      crsr->getm_buff = s_calloc(MAX_CLIENT_BUFSIZE,1);      /* 09.09.13 15.02.10 */            
    crsr->null_flag = realloc(crsr->null_flag, rows_inbuf*crsr->cols); /* 16.06.10 */
    crsr->CBL.LnBufRow = MAX_CLIENT_BUFSIZE;               /* 15.02.10 */    
    inter(&crsr->CBL, crsr->null_flag, NULL, NULL, crsr->getm_buff); 
    if (crsr->CBL.CodErr != 0)
      return (LPERR_LINERROR);
    crsr->FstRow = SecPos;
    crsr->LstRow = crsr->FstRow + crsr->CBL.RowCount - 1;
  }
  ansnum = ansnum % rows_inbuf;                            /* 12.11.14b ... */
  av_clear(av);
  for (i = 0; i < ansnum; i++)
  {
    avRow = newAV();
    row = (char *)&crsr->getm_buff[i * crsr->answ_len];
    for (j = 0; j < crsr->cols; j++)
    {
      sv = newSV(0);
      off = crsr->ColOffset[j];
      len = crsr->answ_desc[j].Length;
      if (crsr->null_flag[i * crsr->cols + j + 4])         /* 16.06.10 */
      {
        sv_setsv(sv, &sv_undef);
      }
      else if (crsr->answ_desc[j].Type != LDT_BLOB)
      {
        rc = DataToSV(crsr, j, crsr->answ_desc[j].Type, len, &row[off], sv);
        if (rc != LPERR_SUCCESS)
          return (rc);
      }
      else
      {
        sv_setpv(sv, "");
      }
      av_push(avRow, sv);
    }
    av_push(av, newRV_noinc((SV*)avRow));
  }                                                        /* ... 12.11.14b */
  
  return (LPERR_SUCCESS);
}                                                          /* ... 03.02.10A */

int LINPL_TestNull(int cursor, int col, SV *sv)
{
  IV      iv   = 0;
  CURSOR *crsr = GetFromList(cursor);

  if (crsr == NULL)
    return (LPERR_INVCURSOR);
  if (crsr->CBL.PrzExe & Q_ASYNC)                          /* 26.02.10 */
    inter_wait_single(&crsr->CBL, -1);                       
  if (crsr->exch_buff == NULL)
    return (LPERR_INVSTATE);
  if (col <= 0 || col > crsr->cols)                        /* 08.02.10 */
    return (LPERR_INVCOLNUM);
  iv = (IV)(crsr->null_flag[col - 1 + 4]);
  sv_setiv(sv, iv);
  return (LPERR_SUCCESS);
}

int LINPL_BLOBAppend(int cursor, SV *sv)
{
  char   *data;
  int     size;                                            /* 24.12.09 */  
  CURSOR *crsr = GetFromList(cursor);
  
  if (crsr == NULL)
    return (LPERR_INVCURSOR);
  data = SvPV(sv, size);
  
  return (AddBlob(crsr, data, size, 0, 0));                /* 24.12.09 */
}

int LINPL_BLOBAppendEx(int cursor, SV *sv, int type)       /* 24.12.09 ... */
{
  char   *data;
  int     size;      
  CURSOR *crsr = GetFromList(cursor);

  if (crsr == NULL)
    return (LPERR_INVCURSOR);
  data = SvPV(sv, size); 
  
  return (AddBlob(crsr, data, size, type, 0));                
}                                                          /* ... 24.12.09 */ 

int LINPL_BLOBClear(int cursor)
{
  CURSOR *crsr = GetFromList(cursor);  

  if (crsr == NULL)
    return (LPERR_INVCURSOR);
  
  return (ClearBlob(crsr, 0));                             /* 24.12.09 */
}

int LINPL_BLOBGetData(int cursor, int pos, SV *len, SV *sv) /* 24.12.09 ... */
{  
  CURSOR *crsr = GetFromList(cursor);
  int i;

  if (crsr == NULL)
    return (LPERR_INVCURSOR);  

  i = -1;
  for(i = 0; i < crsr->cols; i++)
  {
    if (crsr->answ_desc[i].Type == LDT_BLOB)
      break;
  }
  if (i == crsr->cols) 
    return LPERR_NOBLOBCOL;

  return (FetchBlob(crsr, pos, len, i+1, sv));
}                                                          /* ... 24.12.09 */

int LINPL_BLOBGetSize(int cursor, SV *sv, int colnum)      /* 24.12.09 */
{  
  CURSOR   *crsr = GetFromList(cursor);
  BLOB_ATR *desc;                                          /* 31.10.00 */
  L_LONG    off;                                           /* 24.12.09 */  
  
  if (crsr == NULL)
    return (LPERR_INVCURSOR);
    
  if (colnum > crsr->cols)                                 /*03.02.10 */ 
    return (LPERR_INVCOLNUM);
  if (colnum <= 0)                                         /* 24.12.09 ... */ 
  {
    for(colnum = 0; colnum < crsr->cols; colnum++)
    {
      if (crsr->answ_desc[colnum].Type == LDT_BLOB)
        break;
    }
    if (colnum == crsr->cols) 
      return (LPERR_NOBLOBCOL);
    colnum++;
  }    
  if (crsr->answ_desc[colnum-1].Type != LDT_BLOB)          /* 03.02.10 */
    return (LPERR_NOBLOBCOL);         

  off = crsr->ColOffset[colnum-1];                         /* 07.05.09A 03.02.10 *//* ... 24.12.09 */                       
   
  desc = (BLOB_ATR *)&crsr->exch_buff[off];                /* 31.10.00 */
  sv_setiv(sv, (IV)desc->Size);
  
  return (LPERR_SUCCESS);                                 
}

int LINPL_BLOBGetType(int cursor, SV *sv, int colnum)      /* 24.12.09 ... */
{  
  CURSOR   *crsr = GetFromList(cursor);
  BLOB_ATR *desc;                                         
  L_LONG    off;                                          
  
  if (crsr == NULL)
    return (LPERR_INVCURSOR);
  
  if (colnum < 1 || colnum > crsr->cols)                   /* 03.02.10 ... */
    return (LPERR_INVCOLNUM);

  if (crsr->answ_desc[colnum-1].Type == LDT_BLOB)
    off = crsr->ColOffset[colnum-1];
  else
    return (LPERR_NOBLOBCOL);                              /* ... 03.02.10 */
    
  desc = (BLOB_ATR *)&crsr->exch_buff[off];   
  sv_setiv(sv, (IV)desc->TypeObj);
  
  return (LPERR_SUCCESS);                                 
}                                                          

int LINPL_BLOBAdd(int cursor, SV *sv, int col)            
{
  char   *data;
  int     size;
  CURSOR *crsr = GetFromList(cursor);

  if (crsr == NULL)
    return (LPERR_INVCURSOR);
  data = SvPV(sv, size);  
  return (AddBlob(crsr, data, size, 0, col)); 
}                     

int LINPL_BLOBAddEx(int cursor, SV *sv, int col, int type)            
{
  char   *data;
  int     size;
  CURSOR *crsr = GetFromList(cursor);

  if (crsr == NULL)
    return (LPERR_INVCURSOR);
  data = SvPV(sv, size);  
  return (AddBlob(crsr, data, size, type, col)); 
}                     

int LINPL_BLOBFetch(int cursor, int pos, SV *len, SV *sv, int col)
{  
  CURSOR *crsr = GetFromList(cursor);
  
  if (crsr == NULL)
    return (LPERR_INVCURSOR);

  return (FetchBlob(crsr, pos, len, col, sv));
}             

int LINPL_BLOBPurge(int cursor, int col)
{
  CURSOR *crsr = GetFromList(cursor);

  if (crsr == NULL)
    return (LPERR_INVCURSOR);
  
  return (ClearBlob(crsr, col));                           
}                                                          /* ... 24.12.09 */

int LINPL_GetColInfo(int cursor, int col, HV *hv)          /* 04.02.10 */
{
  GETA_OUT *col_desc = NULL;
  CURSOR   *crsr= GetFromList(cursor);
  char      c_prop[MAX_ID_LEN + 2];   
  SV       *sv;  

  if (crsr == NULL)
    return (LPERR_INVCURSOR);
  if (crsr->last_oper != SQL_SELECT)                       /* 04.02.10 */
    return (LPERR_INVSTATE);
  if (col == 0 || col > crsr->cols)
    return (LPERR_INVCOLNUM);

  hv_clear(hv);                                            /* 04.02.10 ... */
  col--;
  
  sv = newSViv((long)crsr->null_flag[2 * sizeof(L_WORD) + col]); 
  hv_store(hv, "is_null", 7, sv, 0);  
  sv = newSViv((long)crsr->answ_desc[col].Type);
  hv_store(hv, "type", 4, sv, 0);    
  GetTypeName(SvIV(sv),(long)crsr->answ_desc[col].Length, (char *)&c_prop, MAX_ID_LEN + 2); 
  sv = newSVpv((char *)&c_prop, 0);
  hv_store(hv, "type_name", 9, sv, 0);  
  sv = newSViv((long)crsr->answ_desc[col].Length);
  hv_store(hv, "length", 6, sv, 0);    
  sv = newSViv((long)crsr->answ_desc[col].Precision);
  hv_store(hv, "precision", 9, sv, 0);      
  sv = newSViv((long)crsr->answ_desc[col].Scale);
  hv_store(hv, "scale", 5, sv, 0);        
  memset(c_prop, 0, sizeof(c_prop));
  memcpy(c_prop, (char *)crsr->answ_desc[col].Column, MAX_ID_LEN); 
  strrtrim(c_prop, " ");
  sv = newSVpv(c_prop, 0);
  hv_store(hv, "name", 4, sv, 0);   
  memset(c_prop, 0, sizeof(c_prop));
  memcpy(c_prop, (char *)crsr->answ_desc[col].Table, MAX_ID_LEN); 
  strrtrim(c_prop, " ");
  sv = newSVpv(c_prop, 0);
  hv_store(hv, "table", 5, sv, 0);
  memset(c_prop, 0, sizeof(c_prop));
  memcpy(c_prop, (char *)crsr->answ_desc[col].User, MAX_ID_LEN); 
  strrtrim(c_prop, " ");
  sv = newSVpv(c_prop, 0);
  hv_store(hv, "user", 4, sv, 0);                          /* ... 04.02.10 */
  
  return (LPERR_SUCCESS);    
}

int LINPL_GetError(int cursor, SV *linerr, SV *syserr)
{
  CURSOR *crsr = GetFromList(cursor);

  if (crsr == NULL)
    crsr = &List;
  if (crsr->CBL.PrzExe & Q_ASYNC)                          /* 26.02.10 */
    inter_wait_single(&crsr->CBL, -1);                       
  sv_setiv(linerr, GET_LINTER_CODE(crsr));
  sv_setiv(syserr, GET_SYSTEM_CODE(crsr));
  return (LPERR_SUCCESS);
}

int LINPL_GetErrorMsg(int cursor, int linerr, SV *errmsg)
{
  L_CHAR    msg[256];
  CURSOR *crsr = GetFromList(cursor);

  memset(msg, 0, sizeof(msg));
  if (crsr == NULL)
    return (LPERR_INVCURSOR);
  if (crsr->CBL.PrzExe & Q_ASYNC)                          /* 26.02.10 */
    inter_wait_single(&crsr->CBL, -1);                       
  crsr->CBL.RowId    = linerr;
  crsr->CBL.LnBufRow = sizeof(msg) - 1;
  strncpy((char *)crsr->CBL.Command, "GETE", 4);
  inter(&crsr->CBL, NULL, NULL, NULL, msg);
  if (GET_LINTER_CODE(crsr) == 0)
    {
    strltrim(msg, " ");
    sv_setpv(errmsg, msg);
    }
  return (GET_LINTER_CODE(crsr));
}

int LINPL_Init(void)
{
  MUTEX_INIT(&mutex);
  return (LPERR_SUCCESS);
}

int LINPL_Cleanup(void)                                    /* 11.05.00 ... */
{
  CURSOR *crsr;

  while(List.next)
    {
    crsr = List.next;
#if 0                                                      /* 07.05.09 */    
    if (crsr->parent == NULL)
      {
      strncpy((char *)crsr->CBL.Command, "CLOS", 4);
      inter(&crsr->CBL, NULL, NULL, NULL, NULL);
      }    
#endif      
    List.next = List.next->next;
    ClearCursorDesc(crsr, L_TTRUE, L_TTRUE);               /* 29.06.09 */
    s_free(crsr);
    }
  UninitLinterClient();                                    /* 02.06.00 */
  return (LPERR_SUCCESS);
}                                                          /* ... 11.05.00 */

int LINPL_inter(SV *arg1, SV *arg2, SV *arg3,              /* 17.02.99 ... */
                SV *arg4, SV *arg5)
{
  TCBL cbl;
  char *a2,*a3,*a4,*a5;
  SV **coderr,**prior, **numchan, **username, **command, **node;
  SV **rowid, **rowcount, **przexe, **syserr, **lnbufrow, **reserve;

  memset(&cbl, 0, sizeof(cbl));

  if (!SvROK(arg1) || !SvROK(arg2) || !SvROK(arg3) || !SvROK(arg4) || !SvROK(arg5))
    {
    errno = EINVAL;
    return (0);
    }
  if (SvTYPE(SvRV(arg1)) != SVt_PVHV)
    {
    errno = EINVAL;
    return (0);
    }
  coderr   = hv_fetch((HV*)SvRV(arg1), "CodErr", sizeof("CodErr") - 1, 0);
  prior    = hv_fetch((HV*)SvRV(arg1), "Prior", sizeof("Prior") - 1, 0);
  numchan  = hv_fetch((HV*)SvRV(arg1), "NumChan", sizeof("NumChan") - 1, 0);
  username = hv_fetch((HV*)SvRV(arg1), "UserName", sizeof("UserName") - 1, 0);
  command  = hv_fetch((HV*)SvRV(arg1), "Command", sizeof("Command") - 1, 0);
  node     = hv_fetch((HV*)SvRV(arg1), "Node", sizeof("Node") - 1, 0);
  rowid    = hv_fetch((HV*)SvRV(arg1), "RowId", sizeof("RowId") - 1, 0);
  rowcount = hv_fetch((HV*)SvRV(arg1), "RowCount", sizeof("RowCount") - 1, 0);
  przexe   = hv_fetch((HV*)SvRV(arg1), "PrzExe", sizeof("PrzExe") - 1, 0);
  syserr   = hv_fetch((HV*)SvRV(arg1), "SysErr", sizeof("SysErr") - 1,0);
  lnbufrow = hv_fetch((HV*)SvRV(arg1), "LnBufRow", sizeof("LnBufRow") - 1, 0);
  reserve  = hv_fetch((HV*)SvRV(arg1), "Reserve", sizeof("Reserve") - 1, 0);
  if (!coderr || !prior || !numchan || !username ||
      !command|| !node || !rowid || !rowcount || !przexe ||
      !syserr || !lnbufrow)
    {
    errno = EINVAL;
    return (0);
    }
  if (SvOK(*command) && SvROK(*command) && SvPOKp(SvRV(*command)))
    strncpy(cbl.Command, SvPVX(SvRV(*command)),4);
  else
    {
    errno = EINVAL;
    return (0);
    }
  if (SvOK(*coderr) && SvROK(*coderr) && SvIOKp(SvRV(*coderr)))
    cbl.CodErr = SvIVX(SvRV(*coderr));
  if (SvOK(*prior) && SvROK(*prior) && SvIOKp(SvRV(*prior)))
    cbl.Prior = SvIVX(SvRV(*prior));
  if (SvOK(*numchan) && SvROK(*numchan) && SvIOKp(SvRV(*numchan)))
    cbl.NumChan = SvIVX(SvRV(*numchan));
  if (SvOK(*rowid) && SvROK(*rowid) && SvIOKp(SvRV(*rowid)))
    cbl.RowId = SvIVX(SvRV(*rowid));
  if (SvOK(*rowcount) && SvROK(*rowcount) && SvIOKp(SvRV(*rowcount)))
    cbl.RowCount = SvIVX(SvRV(*rowcount));
  if (SvOK(*przexe) &&  SvROK(*przexe) && SvIOKp(SvRV(*przexe)))
    cbl.PrzExe = SvIVX(SvRV(*przexe));
  if (SvOK(*syserr) && SvROK(*syserr) && SvIOKp(SvRV(*syserr)))
    cbl.SysErr = SvIVX(SvRV(*syserr));
  if (SvOK(*username) && SvROK(*username) && SvPOKp(SvRV(*username)))
    strncpy(cbl.UserName, SvPVX(SvRV(*username)), 4);
  if (SvOK(*node) && SvROK(*node) && SvPOKp(SvRV(*node)))
    strncpy(cbl.Node, SvPVX(SvRV(*node)), 8);
  if (SvOK(*lnbufrow) && SvROK(*lnbufrow) && SvIOKp(SvRV(*lnbufrow)))
    cbl.LnBufRow = SvIVX(SvRV(*lnbufrow));

  if (cbl.LnBufRow)
    {
    if ((a5 = malloc(cbl.LnBufRow)) == NULL)
      return (LPERR_NOMEMORY);
    }
  else
    a5 = NULL;

  if (SvOK(SvRV(arg2)) && SvPOKp(SvRV(arg2)))
    a2 = SvGROW(SvRV(arg2), MAX_ROW_SIZE);
  else
    a2 = NULL;

  a3 = (SvOK(SvRV(arg3)) && SvPOKp(SvRV(arg3))) ? SvPVX(SvRV(arg3)) : NULL;
  a4 = (SvOK(SvRV(arg4)) && SvPOKp(SvRV(arg4))) ? SvPVX(SvRV(arg4)) : NULL;

  inter(&cbl, a2, a3, a4, a5);

  sv_setiv(SvRV(*coderr), cbl.CodErr);
  sv_setiv(SvRV(*prior), cbl.Prior);
  sv_setiv(SvRV(*numchan), cbl.NumChan);
  sv_setiv(SvRV(*rowid), cbl.RowId);
  sv_setiv(SvRV(*rowcount), cbl.RowCount);
  sv_setiv(SvRV(*przexe), cbl.PrzExe);
  sv_setiv(SvRV(*syserr), cbl.SysErr);
  sv_setiv(SvRV(*lnbufrow), cbl.LnBufRow);
  if (a5 != NULL)
    {
    sv_setpvn(SvRV(arg5), a5, cbl.LnBufRow);
    s_free(a5);
    }
  return (cbl.CodErr);
}                                                          /* ... 17.02.99 */

int LINPL_constant(char *name, int arg)
{
  errno = 0;
  switch(*name)
    {
    case 'C':
      if (strEQ(name, "CP_866") || strEQ(name, "CP_OEM"))   /* 26.09.02 */
        return (CP_866);
      if (strEQ(name, "CP_1251") || strEQ(name, "CP_ANSI")) /* 26.09.02 */
        return (CP_1251);
      if (strEQ(name, "CP_KOI8") || strEQ(name, "CP_ISO"))  /* 26.09.02 */
        return (CP_KOI8);
      if (strEQ(name, "CO_CURNAME"))
        return (CO_CURNAME);
      if (strEQ(name, "CO_ASYNCMODE"))
        return (CO_ASYNCMODE);
      if (strEQ(name, "CO_SYNCMODE"))
        return (CO_SYNCMODE);
      if (strEQ(name, "CO_ASYNCDONE"))                     /* 26.02.10 */
        return (CO_ASYNCDONE);
      if (strEQ(name, "CO_DTFORMAT"))
        return (CO_DTFORMAT);
      if (strEQ(name, "CO_COLCOUNT"))
        return (CO_COLCOUNT);
      if (strEQ(name, "CO_ROWCOUNT"))
        return (CO_ROWCOUNT);
      if (strEQ(name, "CO_ERRROW"))
        return (CO_ERRROW);
      if (strEQ(name, "CO_ERRPOS"))
        return (CO_ERRPOS);
      if (strEQ(name, "CO_TRANSMODE"))
        return (CO_TRANSMODE);
      if (strEQ(name, "CO_CURROW"))
        return (CO_CURROW);
      if (strEQ(name, "CO_CURROWID"))
        return (CO_CURROWID);
      if (strEQ(name, "CO_CONNECTID"))
        return (CO_CONNECTID);
      if (strEQ(name, "CO_NODENAME"))
        return (CO_NODENAME);
      if (strEQ(name, "CO_PRIORITY"))                      /* 16.12.99 ... */
        return (CO_PRIORITY);
      if (strEQ(name, "CO_QUERY_PRIORITY"))
        return (CO_QUERY_PRIORITY);
      if (strEQ(name, "CO_CANCEL"))
        return (CO_CANCEL);
      if (strEQ(name, "CO_PAUSE"))
        return (CO_PAUSE);
      if (strEQ(name, "CO_CONTINUE"))
        return (CO_CONTINUE);                              /* ... 16.12.99 */
      if (strEQ(name, "CO_LAST_ROWID"))                    /* ... 24.02.00 */
        return (CO_LAST_ROWID);
      if (strEQ(name, "CO_FETCH_BLOBS_AS_USUAL_DATA"))     /* 15.02.10B */
        return (CO_FETCH_BLOBS_AS_USUAL_DATA);
      break;
    case 'E':
      if (strEQ(name, "EXC_NONE"))
        return (EXC_NONE);
      if (strEQ(name, "EXC_DIVZERO"))
        return (EXC_DIVZERO);
      if (strEQ(name, "EXC_UNDEFPROC"))
        return (EXC_UNDEFPROC);
      if (strEQ(name, "EXC_BADPARAM"))
        return (EXC_BADPARAM);
      if (strEQ(name, "EXC_BADINDEX"))
        return (EXC_BADINDEX);
      if (strEQ(name, "EXC_BADRETVAL"))
        return (EXC_BADRETVAL);
      if (strEQ(name, "EXC_NULLDATA"))
        return (EXC_NULLDATA);
      if (strEQ(name, "EXC_NOMEM"))
        return (EXC_NOMEM);
      if (strEQ(name, "EXC_BADCURSOR"))
        return (EXC_BADCURSOR);
      if (strEQ(name, "EXC_CURNOTOPEN"))
        return (EXC_CURNOTOPEN);
      if (strEQ(name, "EXC_BADCODE"))
        return (EXC_BADCODE);
      if (strEQ(name, "EXC_TRIGQUERY"))
        return (EXC_TRIGQUERY);
      if (strEQ(name, "EXC_APPENDNOTSTARTED"))             /* 08.09.08 ... */
        return (EXC_APPENDNOTSTARTED);
      if (strEQ(name, "EXC_QUERYWHENAPPEND"))
        return (EXC_QUERYWHENAPPEND);
      if (strEQ(name, "EXC_APPENDACTIVE"))
        return (EXC_APPENDACTIVE);
      if (strEQ(name, "EXC_APPLICATIONERROR"))
        return (EXC_APPLICATIONERROR);
      if (strEQ(name, "EXC_INVTRSTATE"))
        return (EXC_INVTRSTATE);                           /* ... 08.09.08 */
      if (strEQ(name, "EXC_CUSTOM"))
        return (EXC_CUSTOM);
      break;
    case 'F':
      if (strEQ(name, "FETCH_FIRST"))
        return (FETCH_FIRST);
      if (strEQ(name, "FETCH_LAST"))
        return (FETCH_LAST);
      if (strEQ(name, "FETCH_NEXT"))
        return (FETCH_NEXT);
      if (strEQ(name, "FETCH_PREV"))
        return (FETCH_PREV);
      if (strEQ(name, "FETCH_ABSNUM"))
        return (FETCH_ABSNUM);
      break;
    case 'L':
      if (strEQ(name, "LDT_CHAR"))
        return (LDT_CHAR);
      if (strEQ(name, "LDT_INTEGER"))
        return (LDT_INTEGER);
      if (strEQ(name, "LDT_REAL"))
        return (LDT_REAL);
      if (strEQ(name, "LDT_DATE"))
        return (LDT_DATE);
      if (strEQ(name, "LDT_NUMERIC"))
        return (LDT_NUMERIC);
      if (strEQ(name, "LDT_BYTE"))
        return (LDT_BYTE);
      if (strEQ(name, "LDT_BLOB"))
        return (LDT_BLOB);
      if (strEQ(name, "LDT_VARCHAR"))
        return (LDT_VARCHAR);
      if (strEQ(name, "LDT_VARBYTE"))
        return (LDT_VARBYTE);
      if (strEQ(name, "LDT_BOOLEAN"))
        return (LDT_BOOLEAN);
      if (strEQ(name, "LDT_NCHAR"))
        return (LDT_NCHAR);
      if (strEQ(name, "LDT_NVARCHAR"))
        return (LDT_NVARCHAR);

      if (strEQ(name, "LPERR_SUCCESS"))
        return (LPERR_SUCCESS);
      if (strEQ(name, "LPERR_NOMEMORY"))
        return (LPERR_NOMEMORY);
      if (strEQ(name, "LPERR_INVCURSOR"))
        return (LPERR_INVCURSOR);
      if (strEQ(name, "LPERR_INVCOLNUM"))
        return (LPERR_INVCOLNUM);
      if (strEQ(name, "LPERR_INVPARAMNUM"))
        return (LPERR_INVPARAMNUM);
      if (strEQ(name, "LPERR_INVSTATE"))
        return (LPERR_INVSTATE);
      if (strEQ(name, "LPERR_INVDIRECT"))
        return (LPERR_INVDIRECT);
      if (strEQ(name, "LPERR_UNKNOWNOPT"))
        return (LPERR_UNKNOWNOPT);
      if (strEQ(name, "LPERR_BADDTFORMAT"))
        return (LPERR_BADDTFORMAT);
      if (strEQ(name, "LPERR_NOBLOBCOL"))
        return (LPERR_NOBLOBCOL);
      if (strEQ(name, "LPERR_NOPARENT"))
        return (LPERR_NOPARENT);
      if (strEQ(name, "LPERR_INVPARAMETER"))               /* 09.02.10 ... */
        return (LPERR_INVPARAMETER);  
      if (strEQ(name, "LPERR_LINERROR"))
        return (LPERR_LINERROR);                           /* ... 09.02.10 */       
      if (strEQ(name, "LPERR_NOTIMPLEMENTED"))
        return (LPERR_NOTIMPLEMENTED);
      break;
    case 'T':
      if (strEQ(name, "TM_AUTOCOMMIT"))
        return (TM_AUTOCOMMIT);
      if (strEQ(name, "TM_OPTIMISTIC"))
        return (TM_OPTIMISTIC);
      if (strEQ(name, "TM_EXCLUSIVE"))
        return (TM_EXCLUSIVE);
      break;
    }
  errno = EINVAL;
  return (0);
}

#ifdef WIN32
BOOL WINAPI DllMain(HINSTANCE hInstance, DWORD dwReason, LPVOID lpvReserved)
{
 lpvReserved = lpvReserved;

 switch (dwReason)
   {
   case DLL_PROCESS_ATTACH:
     LINPL_Init();
     break;
   case DLL_PROCESS_DETACH:
     LINPL_Cleanup();
     break;
   case DLL_THREAD_ATTACH:
     break;
   case DLL_THREAD_DETACH:
     break;
   }
 return (TRUE);
}

#endif

#if (defined(UNIX) || defined(QNX)) /* 13.09.12 ... */
void _fini()
{
  UninitLinterClient();
}
#endif                              /* ... 13.09.12 */

