 /***************************************************************************
 *           RELEX, Inc. Client source include file                         *
 *  This is unpublished proprietary source code of RELEX, Inc.               *
 *  The copyright notice does not evidence any actual or intended           *
 *  publication of such source code.                                        *
 *                                                                          *
 *                 Copyright (c) 1999-2014 RELEX Inc.                       *
 *                                                                          *
 *                   This software was developed by                         *
 *                                                                          *
 *                            RELEX, Inc.                                   *
 *                         Voronezh, Russia                                 *
 *                                                                          *
 *                        All rights reserved.                              *
 ****************************************************************************
 FILE:    linperl.h
 ----------------------------------------------------------------------------
 AUTHOR:  Alexander V. Kefeli
 ----------------------------------------------------------------------------
 PURPOSE: Interface from Perl to Linter SQL Server
 ----------------------------------------------------------------------------
 REVISION HISTORY:

 **.**.** -- Kent (file creation)
 16.12.99 -- Kent (New cursor options for 5.7)
 24.02.00 -- Kent (LAST ROWID cursor option)
 11.05.00 -- Kent (New function `Cleanup')
 01.06.00 -- Kent (Check Perl version & subversion)
 08.11.00 -- Kent (New routines for stored procedures)
 17.11.00 -- Kent (New routine for init module)
 05.11.04 -- Gluhov, Ermakov (fixed bug with codepage in binded values)
 30.10.09 -- Glukhov (BindParamArray fixed)
 24.12.09 -- Glukhov (Blobs using modified, some functions added)
 29.12.09 -- Glukhov (GetConnectInfo added)
 11.01.10 -- Glukhov (New error code for common errors added)
 12.01.10 -- Glukhov (SetCodepage added)
 27.01.10 -- Glukhov (LINPL_GetProcOutData removed)
 01.02.10 -- Glukhov (GetDataRow, GetDataArray, GetDataRows added)
 03.02.10A-- Glukhov (GETM added)
 04.02.10 -- Glukhov (GetColInfo updated)
 15.02.10 -- Glukhov (Blob behaviour cursor option added)
 26.02.10 -- Glukhov (CO_ASYNCDONE added)
 14.11.14 -- Kononov (added EXTFILE type)
 --------------------------------------------------------------------------*/

#ifndef __LINPERL_H__
#define __LINPERL_H__

/* Linter's datatypes */
#define LDT_CHAR                1
#define LDT_INTEGER             2
#define LDT_REAL                3
#define LDT_DATE                4
#define LDT_NUMERIC             5
#define LDT_BYTE                6
#define LDT_BLOB                7
#define LDT_VARCHAR             8
#define LDT_VARBYTE             9
#define LDT_BOOLEAN             10
#define LDT_NCHAR               11
#define LDT_NVARCHAR            12
#define LDT_EXTFILE             13  /* 14.11.14 */

/* Fetch directions */
#define FETCH_FIRST             1
#define FETCH_LAST              2
#define FETCH_NEXT              3
#define FETCH_PREV              4
#define FETCH_ABSNUM            5

/* Transaction modes */
#define TM_AUTOCOMMIT           0x00000000
#define TM_OPTIMISTIC           0x00000100
#define TM_EXCLUSIVE            0x00000400

/* Supported codepages */
#define CP_1251                 0x00004000
#define CP_KOI8                 0x00008000
#define CP_866                  Q_ENCODE+Q_KOI8            /* 05.11.04 */

/* Connect/cursor options */
#define CO_CURNAME              1
#define CO_ASYNCMODE            2
#define CO_SYNCMODE             3
#define CO_DTFORMAT             4
#define CO_COLCOUNT             5
#define CO_ROWCOUNT             6
#define CO_ERRROW               7
#define CO_ERRPOS               8
#define CO_TRANSMODE            9
#define CO_CURROW               10
#define CO_CURROWID             11
#define CO_CONNECTID            12
#define CO_NODENAME             13
#define CO_PRIORITY             14                         /* 16.12.99 ... */
#define CO_QUERY_PRIORITY       15
#define	CO_CANCEL               16
#define CO_PAUSE                17
#define CO_CONTINUE             18                         /* ... 16.12.99 */
#define CO_LAST_ROWID           19                         /* 24.02.00 */
#define CO_FETCH_BLOBS_AS_USUAL_DATA 20                    /* 15.02.10B */
#define CO_ASYNCDONE            21                         /* 26.02.10 */

/* Module's internal errors */
#define LPERR_SUCCESS            0
#define LPERR_NOMEMORY          -1
#define LPERR_INVCURSOR         -2
#define LPERR_INVCOLNUM         -3
#define LPERR_INVPARAMNUM       -4
#define LPERR_INVSTATE          -5
#define LPERR_INVDIRECT         -6
#define LPERR_UNKNOWNOPT        -7
#define LPERR_BADDTFORMAT       -8
#define LPERR_NOBLOBCOL         -9
#define LPERR_NOPARENT          -10
#define LPERR_INVPARAMETER      -11
#define LPERR_LINERROR          -12                        /* 11.01.10 */
#define LPERR_NOTIMPLEMENTED    -999


#if ((PERL_VERSION == 5 && PERL_SUBVERSION >= 600) || PERL_VERSION >= 6) /* 01.06.00 */
#  define na       PL_na
#  define sv_undef PL_sv_undef
#endif

#ifdef __cplusplus
extern "C" {
#endif

int LINPL_OpenConnect(char *, char *, char *, int, SV *);
int LINPL_CloseConnect(int);
int LINPL_GetConnectInfo(int, HV *);                       /* 29.12.09 */
int LINPL_OpenCursor(int, char *, SV *);
int LINPL_CloseCursor(int);
int LINPL_GetCursorOption(int, int, SV *);
int LINPL_SetCursorOption(int, int, SV *);
int LINPL_SetCodepage(SV *);                               /* 12.01.10 */

int LINPL_ExecDirect(int, char *);
int LINPL_Prepare(int, char *);
int LINPL_BindParameter(int, int, SV *);
int LINPL_BindParamArray(int, AV *, int, int);             /* 30.10.09 */
int LINPL_Execute(int);
int LINPL_Fetch(int, int, int);
int LINPL_UnbindParameter(int, int);

int LINPL_Commit(int);
int LINPL_Rollback(int);

int LINPL_BindColumn(int, int, SV *);
int LINPL_BindColArray(int, int, int, AV *);
int LINPL_GetDataColumn(int , int, SV *);
int LINPL_GetDataRow(int, AV *);                           /* 01.02.10 ... */
int LINPL_GetDataArray(int, HV *);                         
int LINPL_GetDataRows(int, AV *, int);                     /* ... 01.02.10 */
int LINPL_GETM(int cursor, AV *av, int ansnum);            /* 03.02.10A */
int LINPL_TestNull(int, int, SV *);
int LINPL_UnbindColumn(int, int);

int LINPL_GetColInfo(int, int, HV *);                      /* 04.02.10 */

int LINPL_BLOBAppend(int, SV *);
int LINPL_BLOBAppendEx(int, SV *, int);                    /* 24.12.09 */
int LINPL_BLOBClear(int);
int LINPL_BLOBGetData(int, int, SV *, SV *);
int LINPL_BLOBGetSize(int, SV *, int colnum);              /* 24.12.09 */
int LINPL_BLOBGetType(int, SV *, int colnum);              /* 24.12.09 */

int LINPL_BLOBAdd(int, SV *, int);                         /* 24.12.09 ... */
int LINPL_BLOBAddEx(int, SV *, int, int);      
int LINPL_BLOBFetch(int, int, SV *, SV *, int);
int LINPL_BLOBPurge(int, int);                             /* ... 24.12.09 */

int LINPL_GetError(int, SV *, SV *);
int LINPL_GetErrorMsg(int, int, SV *);

int LINPL_Init(void);
int LINPL_Cleanup(void);

int LINPL_constant(char *, int);

int LINPL_inter(SV *, SV *,SV *,SV *, SV *);

#ifdef __cplusplus
}
#endif

#endif /* __LINPERL_H__ */

