/****************************************************************************
 *                       RELEX, Inc. DBMS Linter                            *
 *             This is propietary source code of RELEX, Inc.                *
 *                                                                          *
 *                    Copyright (c) 2006 RELEX, Inc.                        *
 *                                                                          *
 *                   This software was developed by                         *
 *                                                                          *
 *                            RELEX, Inc.                                   *
 *                         Voronezh, Russia                                 *
 *                                                                          *
 *                        All rights reserved.                              *
 ****************************************************************************
*/
#include "dbx.h"
#include "dbx_linter.h"

#ifndef min                                                 
#define min(a,b) ((a)>(b)?(b):(a))
#endif

int dbx_linter_connect(zval **rv, zval **host, zval **db, zval **username, zval **password, INTERNAL_FUNCTION_PARAMETERS)
{
	/* returns 1 as long on success or 0 as long on failure */
	int number_of_arguments=4;
	zval **arguments[4];
	zval *returned_zval=NULL;
	zval *mode=NULL;

	MAKE_STD_ZVAL(mode);
	ZVAL_LONG(mode, 0); /* default mode */

	arguments[0]=username;
	arguments[1]=password;
	arguments[2]=host;
	arguments[3]=&mode;

	dbx_call_any_function(INTERNAL_FUNCTION_PARAM_PASSTHRU, "linter_open_connect", &returned_zval, number_of_arguments, arguments);
	zval_ptr_dtor(&mode);
	if (!returned_zval || (returned_zval->value.lval < 0))
	{
	  if (returned_zval) zval_ptr_dtor(&returned_zval);
		return 0;
	}
	MOVE_RETURNED_TO_RV(rv, returned_zval);
	return 1;
}

int dbx_linter_pconnect(zval **rv, zval **host, zval **db, zval **username, zval **password, INTERNAL_FUNCTION_PARAMETERS)
{
	return 0;
}

int dbx_linter_close(zval **rv, zval **dbx_handle, INTERNAL_FUNCTION_PARAMETERS)
{
	/* returns 1 as long on success or 0 as long on failure */
	int number_of_arguments=1;
	zval **arguments[1];
	zval *returned_zval=NULL;

	arguments[0]=dbx_handle;
	dbx_call_any_function(INTERNAL_FUNCTION_PARAM_PASSTHRU, "linter_close_connect", &returned_zval, number_of_arguments, arguments);
	if (!returned_zval || (returned_zval->value.lval < 0))
	{
	  if (returned_zval) zval_ptr_dtor(&returned_zval);
		return 0;
	}
	ZVAL_TRUE(returned_zval);
	MOVE_RETURNED_TO_RV(rv, returned_zval);
	return 1;
}

int dbx_linter_query(zval **rv, zval **dbx_handle, zval **db_name, zval **sql_statement, INTERNAL_FUNCTION_PARAMETERS)
{
	/* returns 1 as long or a result identifier as resource on success  or 0 as long on failure */
	
	int number_of_arguments=2;
	zval **arguments[2];
	zval *returned_zval=NULL;
	zval *curs_id=NULL;
	zval *new_sql_statement = NULL;
	char *query = NULL;
	int query_len = 0;

	MAKE_STD_ZVAL(curs_id);
	ZVAL_LONG(curs_id, (*dbx_handle)->value.lval);

	/* add ";" to the end of query */
	query_len = Z_STRLEN_PP(sql_statement);
	query = (char *) calloc(query_len + 2, 1);
	strncpy(query, Z_STRVAL_PP(sql_statement), query_len);
	query[query_len] = ';';
	query[query_len + 1] = 0;
	MAKE_STD_ZVAL(new_sql_statement);
	ZVAL_STRING(new_sql_statement, query, 1);

	arguments[0]=dbx_handle;
	arguments[1]=&new_sql_statement;
	dbx_call_any_function(INTERNAL_FUNCTION_PARAM_PASSTHRU, "linter_exec_direct", &returned_zval, number_of_arguments, arguments);
  zval_ptr_dtor(&new_sql_statement);
  free(query);
	if (!returned_zval || (returned_zval->value.lval < 0))
	{
	  if (returned_zval) zval_ptr_dtor(&returned_zval);
	  zval_ptr_dtor(&curs_id);
		return 0;
	}
	/* check for select query */
	if ((!strncasecmp(Z_STRVAL_PP(sql_statement), "select", min((int)strlen("select"), Z_STRLEN_PP(sql_statement)))) ||
	    (!strncasecmp(Z_STRVAL_PP(sql_statement), "execute", min((int)strlen("execute"), Z_STRLEN_PP(sql_statement)))))
	{
	  /* select query */
    zval_ptr_dtor(&returned_zval);
	  MOVE_RETURNED_TO_RV(rv, curs_id);
	}
  else
  {
    /* non select query */ 
    zval_ptr_dtor(&curs_id);
    ZVAL_TRUE(returned_zval);
    MOVE_RETURNED_TO_RV(rv, returned_zval);
  }
  return 1;
}

int dbx_linter_getcolumncount(zval **rv, zval **result_handle, INTERNAL_FUNCTION_PARAMETERS)
{
	/* returns column-count as long on success or 0 as long on failure */
	int number_of_arguments=2;
	zval **arguments[2];
	zval *returned_zval=NULL;
	zval *cursor_option=NULL;
	zval *col_count=NULL;
	arguments[0]=result_handle;
	arguments[1]=&cursor_option;

	dbx_call_any_function(INTERNAL_FUNCTION_PARAM_PASSTHRU, "linter_get_data_row", &returned_zval, 1, arguments);
	if (!returned_zval || (Z_TYPE_P(returned_zval) != IS_ARRAY))
	{
	  if (returned_zval) zval_ptr_dtor(&returned_zval);
	  
	  MAKE_STD_ZVAL(cursor_option);
	  ZVAL_LONG(cursor_option, 5); /* CO_COL_COUNT */
  	dbx_call_any_function(INTERNAL_FUNCTION_PARAM_PASSTHRU, "linter_get_cursor_opt", &returned_zval, number_of_arguments, arguments);
    zval_ptr_dtor(&cursor_option);
    if (!returned_zval || (returned_zval->value.lval < 0))
    {
      if (returned_zval) zval_ptr_dtor(&returned_zval);
      return 0;
   	}
    zval_ptr_dtor(&cursor_option);
	  MOVE_RETURNED_TO_RV(rv, returned_zval);
	}
	else
	{
	  MAKE_STD_ZVAL(col_count);
	  ZVAL_LONG(col_count, zend_hash_num_elements(Z_ARRVAL_P(returned_zval)));
	  if (returned_zval) zval_ptr_dtor(&returned_zval);
	  MOVE_RETURNED_TO_RV(rv, col_count);
	}
  return 1;
}

int dbx_linter_getcolumnname(zval **rv, zval **result_handle, long column_index, INTERNAL_FUNCTION_PARAMETERS)
{
	/* returns column-name as string on success or 0 as long on failure */
	int number_of_arguments=2;
	zval **arguments[2];
	zval *returned_zval=NULL;
	zval *column_num=NULL;
  zval **column_name=NULL;
	
	MAKE_STD_ZVAL(column_num);
	ZVAL_LONG(column_num, column_index+1); 

	arguments[0]=result_handle;
	arguments[1]=&column_num;
	dbx_call_any_function(INTERNAL_FUNCTION_PARAM_PASSTHRU, "linter_get_col_prop", &returned_zval, number_of_arguments, arguments);
  zval_ptr_dtor(&column_num);

	if ( !returned_zval ||
	     (Z_TYPE_P(returned_zval)!=IS_OBJECT) ||
       (zend_hash_find(Z_OBJPROP_P(returned_zval), "name", strlen("name") + 1, (void **) &column_name) == FAILURE)
     )
  {
	  if (returned_zval) zval_ptr_dtor(&returned_zval);
    MAKE_STD_ZVAL(returned_zval);
    ZVAL_STRING(returned_zval, "", 1);
	  MOVE_RETURNED_TO_RV(rv, returned_zval);
		return 1;
	}
  php_strtolower(Z_STRVAL_PP(column_name), Z_STRLEN_PP(column_name));
	MOVE_RETURNED_TO_RV(rv, *column_name);
  zval_ptr_dtor(&returned_zval);
	return 1;
}

int dbx_linter_getcolumntype(zval **rv, zval **result_handle, long column_index, INTERNAL_FUNCTION_PARAMETERS)
{
	/* returns column-type as string on success or 0 as long on failure */
  zval *returned_zval=NULL;

  MAKE_STD_ZVAL(returned_zval);
  ZVAL_STRING(returned_zval, "string", 1); /* fake field type */
	MOVE_RETURNED_TO_RV(rv, returned_zval);
	return 1;
}

int dbx_linter_getrow(zval **rv, zval **result_handle, long row_number, INTERNAL_FUNCTION_PARAMETERS)
{
	/* returns array[0..columncount-1] as strings on success or 0 as long on failure */
	int number_of_arguments=1;
	zval **arguments[1];
	zval *returned_zval=NULL;

	arguments[0]=result_handle;
	dbx_call_any_function(INTERNAL_FUNCTION_PARAM_PASSTHRU, "linter_fetch_row", &returned_zval, number_of_arguments, arguments);
	if (!returned_zval || (Z_TYPE_P(returned_zval) != IS_ARRAY))
	{
	  if (returned_zval) zval_ptr_dtor(&returned_zval);
		return 0;
	}
	MOVE_RETURNED_TO_RV(rv, returned_zval);
	return 1;
}

int dbx_linter_error(zval **rv, zval **dbx_handle, INTERNAL_FUNCTION_PARAMETERS)
{
	/* returns string */
	int number_of_arguments=2;
	zval **arguments[2];
	zval *returned_zval=NULL;
	zval *error_type=NULL;
                       
	MAKE_STD_ZVAL(error_type);
	ZVAL_LONG(error_type, 0); /* LINTER_ERROR */

	arguments[0]=dbx_handle;
	arguments[1]=&error_type;
	dbx_call_any_function(INTERNAL_FUNCTION_PARAM_PASSTHRU, "linter_last_error", &returned_zval, number_of_arguments, arguments);
  zval_ptr_dtor(&error_type);
  if (!returned_zval || (returned_zval->value.lval < 0))
	{
	  if (returned_zval) zval_ptr_dtor(&returned_zval);
		return 0;
	}
	
	if ((Z_LVAL_P(returned_zval) == 2) || /* no data - it's not real error */
	    (Z_LVAL_P(returned_zval) == 0))   /* no errors */
    ZVAL_EMPTY_STRING(returned_zval);
  
  MOVE_RETURNED_TO_RV(rv, returned_zval);
	return 1;
}

int dbx_linter_esc(zval **rv, zval **dbx_handle, zval **string, INTERNAL_FUNCTION_PARAMETERS)
{
	/* returns escaped string */
	/* replace ' with '' */
	char * str;
	int len;
	char * tmpstr;
	int tmplen;

	if (Z_STRLEN_PP(string) == 0) {
		ZVAL_EMPTY_STRING(*rv);
		return 1;
	}
	tmpstr = estrdup(Z_STRVAL_PP(string));
	tmplen = Z_STRLEN_PP(string);
	/* php_str_to_str uses a smart_str that allocates memory */
	/* this memory must be freed or passed on to rv */
	str = php_str_to_str(tmpstr, tmplen, "'", 1, "''", 2, &len);
	efree(tmpstr);

	ZVAL_STRINGL(*rv, str, len, 0);

	return 1;
}
