 /***************************************************************************
 *           RELEX, Inc. PHP core client source file                        *
 *  This is unpublished proprietary source code of RELEX, Inc.              *
 *  The copyright notice does not evidence any actual or intended           *
 *  publication of such source code.                                        *
 *                                                                          *
 *                   Copyright (c) 1996-2008 RELEX Inc.                     *
 *                                                                          *
 *                   This software was developed by                         *
 *                                                                          *
 *                            RELEX, Inc.                                   *
 *                         Voronezh, Russia                                 *
 *                                                                          *
 *                        All rights reserved.                              *
 ****************************************************************************
 FILE:   linmutex.c 
 ----------------------------------------------------------------------------
 AUTHOR: Pasec
 ----------------------------------------------------------------------------
 PURPOSE: Threaded interface for PHP
 ----------------------------------------------------------------------------
 REVISION HISTORY:
  17.11.04 -- Gluhov (moved from LINAPI)
  07.04.08 -- Ermakov (Avoid cond incompatible for linux)
*/

#ifdef USE_THREADS
#include "linmutex.h"

#if defined(LIN_PTHREAD)                                   

int RecursiveInit(RECURSIVE_MUTEX* RMutex)                 
{
  int RetVal;
  
  if ( RetVal = pthread_mutex_init(&RMutex->Mutex, NULL) )
    return RetVal;
  if ( RetVal = pthread_cond_init(&RMutex->LCond.Cond, NULL) )   /* 07.04.08 */
    return RetVal;
  RMutex->CountLock = 0;
  RMutex->ThreadID = 0;
  return 0;
};

int RecursiveDestroy(RECURSIVE_MUTEX* RMutex)              
{
  int RetVal;
  
  if (RetVal = pthread_cond_destroy(&RMutex->LCond.Cond) )       /* 07.04.08 */
    return RetVal;
  if ( RetVal = pthread_mutex_destroy(&RMutex->Mutex) )
    return RetVal;
  return 0;
};

int RecursiveLock(RECURSIVE_MUTEX* RMutex)                 
{
  int RetVal;
  pthread_t ThreadID =  pthread_self();
  
  if ( RetVal = pthread_mutex_lock(&RMutex->Mutex) )
    return RetVal;
  
  if ( ThreadID == RMutex->ThreadID )
    RMutex->CountLock++;
  else
  {
    if ( RMutex->CountLock <= 0 )
    {
      RMutex->ThreadID = ThreadID;
      RMutex->CountLock = 1;
    }
    else
    {  
      while ( RMutex->CountLock > 0 )
        pthread_cond_wait(&RMutex->LCond.Cond, &RMutex->Mutex);  /* 07.04.08 */
      RMutex->ThreadID = ThreadID;
      RMutex->CountLock = 1;
    }
  }
  pthread_mutex_unlock(&RMutex->Mutex);
  return 0;
};

int RecursiveUnlock(RECURSIVE_MUTEX* RMutex)               
{
  int RetVal;
  pthread_t ThreadID = pthread_self();
  
  if ( RetVal = pthread_mutex_lock(&RMutex->Mutex) )
    return RetVal;
  
  if ( ThreadID == RMutex->ThreadID )
  {
    RMutex->CountLock--;
    if ( RMutex->CountLock <= 0)
      pthread_cond_signal(&RMutex->LCond.Cond);                  /* 07.04.08 */
  }
  else
    return -1;
  pthread_mutex_unlock(&RMutex->Mutex);
};
#endif
#endif
