 /***************************************************************************
 *           RELEX, Inc. PHP core client source file                        *
 *  This is unpublished proprietary source code of RELEX, Inc.              *
 *  The copyright notice does not evidence any actual or intended           *
 *  publication of such source code.                                        *
 *                                                                          *
 *                   Copyright (c) 1996-2008 RELEX Inc.                     *
 *                                                                          *
 *                   This software was developed by                         *
 *                                                                          *
 *                            RELEX, Inc.                                   *
 *                         Voronezh, Russia                                 *
 *                                                                          *
 *                        All rights reserved.                              *
 ****************************************************************************
 FILE:   linmutex.h
 ----------------------------------------------------------------------------
 AUTHOR: Pasec
 ----------------------------------------------------------------------------
 PURPOSE: hrader for threaded interface for PHP
 ----------------------------------------------------------------------------
 REVISION HISTORY:
  17.11.04 -- Gluhov (moved from LINAPI)
  07.04.08 -- Ermakov (Avoid cond incompatible for linux)
*/

#ifndef __LINMUTEX_H__
#define __LINMUTEX_H__

#ifdef WIN32
#include <windows.h>
#endif


#ifdef LIN_PTHREAD                                          
#include <pthread.h>
typedef union
{
  pthread_cond_t  Cond;
  char            Pad[48];
} L_pthread_cond_t;

typedef struct                                               /* 07.04.08 ... */
  {
  pthread_t         ThreadID;
  int               CountLock;
  pthread_mutex_t   Mutex;
  L_pthread_cond_t  LCond;
  } RECURSIVE_MUTEX;                                         /* ... 07.04.08 */
#endif                                                      
                                                            

#ifdef _THR_UNIXWARE
#include <synch.h>
#endif


#ifdef LIN_PTHREAD
int RecursiveInit(RECURSIVE_MUTEX* RMutex);
int RecursiveDestroy(RECURSIVE_MUTEX* RMutex);
int RecursiveLock(RECURSIVE_MUTEX* RMutex);
int RecursiveUnlock(RECURSIVE_MUTEX* RMutex);
#endif

#ifdef WIN32
#define LINAPI_MUTEX  CRITICAL_SECTION
#define LINAPI_MutexInit(a)  InitializeCriticalSection(a)
#define LINAPI_MutexLock(a)  EnterCriticalSection(a)
#define LINAPI_MutexUnlock(a)  LeaveCriticalSection(a)
#define LINAPI_MutexDestroy(a)  DeleteCriticalSection(a)
#elif LIN_PTHREAD
#define LINAPI_MUTEX  RECURSIVE_MUTEX
#define LINAPI_MutexInit(a)  RecursiveInit(a)
#define LINAPI_MutexLock(a)  RecursiveLock(a)
#define LINAPI_MutexUnlock(a)  RecursiveUnlock(a)
#define LINAPI_MutexDestroy(a)  RecursiveDestroy(a)
#elif _THR_VMS
#define LINAPI_MUTEX  cma$t_mutex
#define LINAPI_MutexInit(a)  cma$mutex_create(a,cma$c_null)
#define LINAPI_MutexLock(a)  cma$mutex_lock(a)
#define LINAPI_MutexUnlock(a)  cma$mutex_unlock(a)
#define LINAPI_MutexDestroy(a)  cma$mutex_delete(a)
#elif _THR_UNIXWARE
#define LINAPI_MUTEX  mutex_t
#define LINAPI_MutexInit(a)  mutex_init(a, USYNC_THREAD, NULL)
#define LINAPI_MutexLock(a)  mutex_lock(a)
#define LINAPI_MutexUnlock(a)  mutex_lock(a)
#define LINAPI_MutexDestroy(a)  mutex_destroy(a)
#else
#define LINAPI_MutexInit(a)
#define LINAPI_MutexLock(a)
#define LINAPI_MutexUnlock(a)
#define LINAPI_MutexDestroy(a)
#endif


#endif
