/***************************************************************************
 *           RELEX, Inc. Client source include file                         *
 *  This is unpublished proprietary source code of RELEX, Inc.              *
 *  The copyright notice does not evidence any actual or intended           *
 *  publication of such source code.                                        *
 *                                                                          *
 *                 Copyright (c) 2008-2012 RELEX Inc.                       *
 *                                                                          *
 *                   This software was developed by                         *
 *                                                                          *
 *                            RELEX, Inc.                                   *
 *                         Voronezh, Russia                                 *
 *                                                                          *
 *                        All rights reserved.                              *
 ****************************************************************************
 FILE:    linter_driver.c
 ----------------------------------------------------------------------------
 AUTHOR: Dmitrii Glukhov
 ----------------------------------------------------------------------------
 PURPOSE: PDO interface from PHP language to Linter SQL Server
 ----------------------------------------------------------------------------
 INFORMATION:E-allocation functions (emalloc, ecalloc, etc) are used for 
             external variables, that free by higher layer pdo driver
 ----------------------------------------------------------------------------
 REVISION HISTORY:
 04.08.08 -- Creation
 28.11.08 -- Glukhov (remove global variables)
 22.01.09 -- Glukhov (linter_handle_doer fixed)
 29.01.09 -- Glukhov (error handling fixed)
 10.02.09 -- Glukhov (error handling fixed)
 12.02.09 -- Glukhov (PDO_ATTR_FETCH_BLOB_AS_DATA attribut removed)
 13.02.09 -- Glukhov (PDO_ATTR_AUTOCOMMIT attribut uses fixed)
 17.02.09 -- Glukhov (Transaction's work fixed, ChanLink added) 
 18.02.09 -- Glukhov (error handling in linter_handle_doer fixed, set cursor name modified)
 12.03.09 -- Glukhov (PDO_ATTR_AUTOCOMMIT attribut uses fixed)
 18.03.09 -- Glukhov (ATTR_FETCH_TABLE_NAMES and ATTR_FETCH_CATALOG_NAMES attributes added for statement)
 30.03.09 -- Glukhov (GENERATE_COL_NAME and RENAME_DUBBED_COL settings added)
 27.04.12 -- Medvedev (Correct headers for 5.x)
 25.08.14 -- Fatkhudinov (close handling fixed)
 14.09.14 -- Fatkhudinov (binded_param_count added)
 30.09.14 -- Fatkhufinov (commit mode check added)
 01.10.14 -- Fatkhudinov (Condition sequence changed)
*/

#ifdef PDO

#include <php.h>
#include <sys/types.h>
#include <php_ini.h>
#include <ext/standard/info.h>
#include <Zend/zend_exceptions.h>
#include <TSRM/TSRM.h>

#include <ext/pdo/php_pdo.h>
#include <ext/pdo/php_pdo_driver.h>

#include "inter.h"
#include "php_linter.h"

#include "php_pdo_linter.h"
#include "php_pdo_linter_int.h"


int _pdo_linter_error(pdo_dbh_t *dbh, pdo_stmt_t *stmt, const char *file, int line TSRMLS_DC)
{   
  CURSOR *csr = (CURSOR *)dbh->driver_data;
  CURSOR *stmt_csr = NULL;    
  pdo_error_type *pdo_err;
  pdo_linter_error_info *einfo; 
  
  if (stmt) 
  {    
    stmt_csr = (CURSOR *)stmt->driver_data;
    einfo = &stmt_csr->err_info;        
    pdo_err = &stmt->error_code;
  }
  else
  {   
    if (csr)
    {
      einfo = &csr->err_info;        
      pdo_err = &dbh->error_code;
    }
  }

  if ((csr || stmt)&&(einfo->err_code != LPE_NO_MEMORY))                                     /* 29.01.09 ... */
  {    
    get_sql_state(pdo_err,einfo->err_code);
    if (csr->CBL.NumChan != 0)
    {    
      memcpy(csr->CBL.Command,"GETE",4);
      csr->CBL.RowId = einfo->err_code;  
      csr->CBL.LnBufRow = sizeof(einfo->err_msg)-1;  
      inter(&csr->CBL,NULL,NULL,NULL,einfo->err_msg);  
    }
    else    
      strcpy(einfo->err_msg,"Native error");    

    einfo->file = file;
    einfo->line = line;  
     
    if (!dbh->methods)    
      zend_throw_exception_ex(php_pdo_get_exception(),0 TSRMLS_CC, "SQLSTATE[%s] [%d] %s",*pdo_err,einfo->err_code,einfo->err_msg);
    
    return einfo->err_code;      
  }
  else
  {    
    zend_throw_exception_ex(php_pdo_get_exception(),0 TSRMLS_CC, "Memory allocation error",*pdo_err);
    return LPE_NO_MEMORY;
  }                                                                                           /* ... 29.01.09 */


} 

static int linter_handle_fetch_error_func(pdo_dbh_t *dbh, pdo_stmt_t *stmt, zval *info TSRMLS_DC)
{
  CURSOR *csr = (CURSOR *) dbh->driver_data;
  pdo_linter_error_info *einfo = &csr->err_info;  
  
  if (stmt)
  {
    CURSOR *stmtCsr = (CURSOR *)stmt->driver_data;
    einfo = &stmtCsr->err_info;
  }

  if (einfo->err_code)
  {    
    add_next_index_long(info,einfo->err_code);    
    add_next_index_string(info,einfo->err_msg,1);
  }

  return 0;
} 

static int linter_handle_closer(pdo_dbh_t *dbh TSRMLS_DC)
{ 
  CURSOR *csr = (CURSOR *)dbh->driver_data;  

  if (csr) 
  {
    if (csr->CBL.PrzExe != TM_AUTOCOMMIT)                                      /* 17.02.09 ... */
    {
      memcpy(csr->CBL.Command,"    ",4);
      inter(&csr->CBL,"COMMIT;",NULL,NULL,NULL);
    }                                                                          /* ... 17.02.09 */

    close_sub_chans(csr,0);                                                   /* 17.02.09 */    

    memcpy(csr->CBL.Command,"CLOS",4);    
    inter(&csr->CBL,NULL,NULL,NULL,NULL);   
    
    ClearCursorDesc(csr, 1);    

    s_free(csr);    
    dbh->driver_data = NULL;
  }
  
  return 0;
}

static int linter_handle_preparer(pdo_dbh_t * dbh, const char *sql, long sql_len, pdo_stmt_t *stmt, zval *driver_options TSRMLS_DC)
{
  CURSOR *csr      = (CURSOR *)dbh->driver_data;
  CURSOR *stmt_csr = (CURSOR *)stmt->driver_data;  
  t_ChanLink *ChanLink = csr->ChanLink;
  char   *query;
  char   *nsql;
  int     nsql_len = 0;
  int     ret;
  short   p_count;
    
  if (ChanLink)                                                                   /* 17.02.09 ... */
  {
    stmt_csr            = ChanLink->subCsr;
    stmt_csr->CBL.Prior = csr->CBL.Prior;
    csr->ChanLink       = ChanLink->Next;
    s_free(ChanLink);
  }
  else                                                                            /* ... 17.02.09 */
  {
    stmt_csr = calloc(1,sizeof(CURSOR));
    if (stmt_csr == NULL)
    { 
      csr->err_info.err_code = LPE_NO_MEMORY;                                     /* 29.01.09 */
      pdo_linter_error(dbh);                                                      /* 29.01.09 */
      return 0;
    }  
  
    stmt_csr->CBL.Prior   = csr->CBL.Prior;
    stmt_csr->CBL.NumChan = csr->CBL.NumChan;  
    stmt_csr->CBL.PrzExe  = csr->CBL.PrzExe;
    stmt_csr->ChanLink    = NULL;                                                  /* 17.02.09 */

    strncpy((char *)stmt_csr->CBL.Command, "OCUR",        4);
    strncpy((char *)stmt_csr->CBL.Node,    csr->CBL.Node, 8);
  
    inter(&stmt_csr->CBL, NULL, NULL, NULL, NULL);  
  
    stmt_csr->err_info.err_code = stmt_csr->CBL.CodErr;                            /* 29.01.09, 10.02.09 */ 
    if (stmt_csr->CBL.CodErr != 0)                                                 /* 10.02.09 */
    {     
      pdo_linter_error(dbh);                                                       /* 29.01.09 */
      return 0;
    }
  }
  stmt->driver_data           = stmt_csr;
  stmt->methods               = &linter_stmt_methods;
  stmt->supports_placeholders = PDO_PLACEHOLDER_POSITIONAL;   

  ret = pdo_parse_params(stmt, (char*)sql,sql_len,&nsql,&nsql_len TSRMLS_CC);

  if (ret == 1)
  {
    sql     = nsql;
    sql_len = nsql_len;
  }
  else if (ret == -1)
  {
    strcpy(dbh->error_code,stmt->error_code);
    return 0;
  }
  
  query = malloc(MAX_BUFF);
  if (query == NULL)
  {
    stmt_csr->err_info.err_code = LPE_NO_MEMORY;                                 /* 29.01.09 */ 
    pdo_linter_error(dbh);                                                       /* 29.01.09 */
    return 0;
  }

  strncpy(query,sql,min(MAX_BUFF,sql_len));

  /*query must be terminated by semicolumn */

  if ((sql_len < (MAX_BUFF-1)) && (*(query+sizeof(char)*(sql_len-1))!=';'))       /* 01.10.14 */
  {
    *(query+sizeof(char)*(sql_len))=';';
    *(query+sizeof(char)*sql_len+1)='\0';
  }  
  
  stmt_csr->parm_leng   = NULL;  
  stmt_csr->param_count = 0;                                
  stmt_csr->binded_param_count = 0;                                               /* 14.09.14 */
  stmt_csr->pqry_body   = realloc(stmt_csr->pqry_body, MAX_BUFF);
  
  if (stmt_csr->pqry_body == NULL)
  {
    s_free(query);
    stmt_csr->err_info.err_code = LPE_NO_MEMORY;                                  /* 29.01.09 */
    pdo_linter_error(dbh);                                                        /* 29.01.09 */
    return 0;
  }

  stmt_csr->CBL.LnBufRow = MAX_ANSWER;
  
  /*Query translation*/

  strncpy((char *)stmt_csr->CBL.Command, "TRAN", 4);
  inter(&stmt_csr->CBL, NULL, query, NULL, stmt_csr->pqry_body);
  s_free(query);

  if (stmt_csr->CBL.CodErr == SMALLBUFKOR) 
  {
    stmt_csr->CBL.CodErr = 0;    
    stmt_csr->pqry_body  = realloc(stmt_csr->pqry_body, stmt_csr->CBL.LnBufRow);
    if (stmt_csr->pqry_body != NULL) 
    {
      stmt_csr->CBL.PrzExe |= Q_RECEIVE;
      inter(&stmt_csr->CBL, NULL, NULL, NULL, stmt_csr->pqry_body);
      stmt_csr->CBL.PrzExe &= ~Q_RECEIVE;      
    }
    else
    {  
      stmt_csr->err_info.err_code = LPE_NO_MEMORY;                                /* 29.01.09 */
      pdo_linter_error(dbh);                                                      /* 29.01.09 */
      return 0;
    }
  }
  
  stmt_csr->err_info.err_code = stmt_csr->CBL.CodErr;                             /* 10.02.09 */
  csr->err_info.err_code = stmt_csr->CBL.CodErr;                                  /* 10.02.09 */
  
  if (stmt_csr->CBL.CodErr != 0)
  {    
    pdo_linter_error(dbh);
        
    return 0;
  }
   
  inter_GetParamCount(stmt_csr->pqry_body, &p_count);
  stmt_csr->parm_leng = calloc(1, sizeof(short) * p_count);   
  if (p_count && !stmt_csr->parm_leng)
  {
    stmt_csr->err_info.err_code = LPE_NO_MEMORY;                                 /* 29.01.09 */ 
    pdo_linter_error(dbh);                                                       /* 29.01.09 */

    return 0;
  }
  
  stmt_csr->param_count = p_count;                                               

  inter_SetEncoding(stmt_csr->pqry_body,csr->Flags);
    
  stmt_csr->dt_format                = strdup(csr->dt_format);                   /* 12.02.09 */  
  stmt_csr->fetch_table_names        = csr->fetch_table_names;                   /* 18.03.09 */
  stmt_csr->fetch_catalog_names      = csr->fetch_catalog_names;                 /* 18.03.09 */
  stmt_csr->GenerateColNameBehaviour = csr->GenerateColNameBehaviour;            /* 30.03.09 */
  stmt_csr->RenameDubbedColBehaviour = csr->RenameDubbedColBehaviour;            /* 30.03.09 */
  return 1;
}

static long linter_handle_doer(pdo_dbh_t *dbh, const char *sql, long sql_len TSRMLS_DC)
{
  CURSOR *csr   = (CURSOR *)dbh->driver_data;
  char   *query = malloc(MAX_BUFF);
  if (query == NULL)
  {
    csr->err_info.err_code = LPE_NO_MEMORY;                      /* 29.01.09 */ 
    pdo_linter_error(dbh);                                       /* 29.01.09 */

    return -1;                                                   /* 22.01.09, 18.02.09 */
  }
 
  strncpy(query,sql,min(sql_len,MAX_BUFF));                      /* 28.11.08, 22.01.09*/

  /*query must be terminated by semicolumn */

  if ((*(query+sizeof(char)*(sql_len-1))!=';') && (sql_len < (MAX_BUFF-1)))
  {    
    *(query+sizeof(char)*(sql_len))=';';
    *(query+sizeof(char)*sql_len+1)='\0';
  } 

  strncpy((char *)csr->CBL.Command, "    ", 4);
  csr->CBL.LnBufRow = MAX_ANSWER;
  csr->CBL.RowCount = 0;                                         /* 28.11.08 */
  inter(&csr->CBL, NULL, query, NULL, NULL);
  
  s_free(query);

  csr->err_info.err_code = csr->CBL.CodErr;                      /* 10.02.09 */
  if (csr->CBL.CodErr != 0)
  {    
    pdo_linter_error(dbh);    
    return -1;                                                   /* 22.01.09, 18.02.09 */
  }
  return csr->CBL.RowCount;                                      /* 28.11.08 */
}

static int linter_handle_quoter(pdo_dbh_t *dbh, const char *unquoted, int uquoted_len, char **quoted, int *quotedlen, enum pdo_param_type param_type TSRMLS_DC)
{   
  int unquoted_ptr = 0;  
  int ret_val = 0;  

  if (param_type != PDO_PARAM_STR)  
    return ret_val;  

  *quoted = emalloc(2*uquoted_len+3);    
  *quotedlen = 0;

  if ((*unquoted == '\'')&&(*(unquoted+sizeof(char)*uquoted_len-1) == '\''))
  {    
    uquoted_len -= 2;
    unquoted_ptr ++;
  }
  else  
    ret_val = 1;    

  (*quotedlen)++;

  while (uquoted_len > 0)
  {
    if (*(unquoted+unquoted_ptr) == '\'')
    {
      *(*quoted+(*quotedlen)) = '\'';
      (*quotedlen)++;
      ret_val = 1;
    }
    *(*quoted+(*quotedlen)) = *(unquoted+(unquoted_ptr++));
    uquoted_len--;
    (*quotedlen)++;
  }
   
  (**quoted) = '\'';
  (*(*quoted+((*quotedlen)++))) = '\'';
  (*(*quoted+(*quotedlen))) = '\0';  
  
  return ret_val;
}

static int linter_handle_begin(pdo_dbh_t *dbh TSRMLS_DC)
{
  CURSOR *csr = (CURSOR *)dbh->driver_data; 
  strncpy((char *)csr->CBL.Command, "SETO", 4); 

  csr->CBL.RowId = crMode;
  csr->CBL.RowCount = csr->CBL.NumChan;  
  csr->CBL.LnBufRow = 0;
  csr->CBL.PrzExe = TM_EXCLUSIVE;                                          /* 17.02.09 */

  inter(&csr->CBL, NULL, NULL, NULL, NULL);

  csr->err_info.err_code = csr->CBL.CodErr;                                /* 10.02.09 */
  if (csr->CBL.CodErr != 0)
  {    
    pdo_linter_error(dbh);

    return 0;
  }
  dbh->auto_commit = 0;                                                    /* 12.03.09 */

  return 1;
}

static int linter_handle_commit(pdo_dbh_t *dbh TSRMLS_DC)
{
  CURSOR *csr = (CURSOR *)dbh->driver_data;
  char *query = "COMMIT;"; 
   
  strncpy((char *)csr->CBL.Command, "    ", 4);
  csr->CBL.LnBufRow = MAX_ANSWER;
  
  inter(&csr->CBL, NULL, query, NULL, NULL);
  
  close_sub_chans(csr,0);                                                    /* 17.02.09 */

  csr->err_info.err_code = csr->CBL.CodErr;                                  /* 10.02.09 */
  if (csr->CBL.CodErr != 0)
  {    
    pdo_linter_error(dbh);

    return 0;
  }

  if (to_autocommit_mode(&csr->CBL) == 0)
  {
    csr->err_info.err_code = csr->CBL.CodErr;
    pdo_linter_error(dbh);

    return 0;
  }
  dbh->auto_commit = 1;                                                      /* 12.03.09 */

  return 1;

}

static int linter_handle_rollback(pdo_dbh_t *dbh TSRMLS_DC)
{  
  CURSOR *csr = (CURSOR *)dbh->driver_data;
  char *query = "ROLLBACK;"; 
   
  strncpy((char *)csr->CBL.Command, "    ", 4);
  csr->CBL.LnBufRow = MAX_ANSWER;
  if (dbh->auto_commit != 1)                                                    /* 30.09.14 */
    inter(&csr->CBL, NULL, query, NULL, NULL);
  close_sub_chans(csr,1);

  csr->err_info.err_code = csr->CBL.CodErr;                                     /* 10.02.09 */
  if (csr->CBL.CodErr != 0)
  {    
    pdo_linter_error(dbh);

    return 0;
  }

  if (to_autocommit_mode(&csr->CBL) == 0)
  {
    csr->err_info.err_code = csr->CBL.CodErr;
    pdo_linter_error(dbh);

    return 0;
  }
  dbh->auto_commit = 1;                                                       /* 12.03.09 */

  return 1;
}

static int linter_handle_set_attribute(pdo_dbh_t *dbh, long attr, zval *val TSRMLS_DC)
{
  CURSOR *csr = (CURSOR *)dbh->driver_data;  

  switch (attr)
  { 
    case PDO_ATTR_AUTOCOMMIT: if (dbh->auto_commit != (unsigned)val->value.lval)
                              {
                                dbh->auto_commit = val->value.lval;         
                                if (dbh->auto_commit == 0)
                                {
                                  linter_handle_begin(dbh TSRMLS_CC);
                                  dbh->in_txn = 1;                                             /* 13.02.09 */
                                }
                                else
                                {              
                                  if (linter_handle_commit(dbh TSRMLS_CC))                    /* 17.02.09 12.03.09 */                                  
                                    csr->err_info.err_code = 0;                                /* 10.02.09 */                                  
                                  else                                                         /* 12.03.09 */                                  
                                    return 0;                                  
                                }
                              }
                              break;
    case PDO_ATTR_CURSOR_NAME:strncpy((char *)csr->CBL.Command, "SETO", 4);

                              csr->CBL.RowId = crName;                              
                              csr->CBL.LnBufRow = val->value.str.len;                          /* 18.02.09 */
                              
                              inter(&csr->CBL, NULL, NULL, NULL, val->value.str.val);

                              csr->err_info.err_code = csr->CBL.CodErr;                        /* 29.01.09, 10.02.09 */
                              if (csr->CBL.CodErr != 0)
                              {                                
                                pdo_linter_error(dbh);                                         /* 29.01.09 */

                                return 0;
                              }
                              
                              strncpy(csr->name,val->value.str.val,min(sizeof(csr->name),MAX_ID_LEN+2));

                              break;
    case PDO_ATTR_FETCH_TABLE_NAMES: csr->fetch_table_names = (L_BOOL)val->value.lval;
                              break;
    case PDO_ATTR_FETCH_CATALOG_NAMES: csr->fetch_catalog_names = (L_BOOL)val->value.lval;
                              break;
    case PDO_ATTR_DT_FORMAT:  s_free(csr->dt_format);                                          
                              csr->dt_format = strdup(val->value.str.val);
                              break;                                                           /* 12.02.09 */
    case PDO_ATTR_GENERATE_COL_NAME: csr->GenerateColNameBehaviour = (L_BOOL)val->value.lval;  /* 30.03.09 ... */
                              break;
    case PDO_ATTR_RENAME_DUBBED_COL: csr->RenameDubbedColBehaviour = (L_BOOL)val->value.lval;  
                              break;                                                           /* ... 30.03.09 */
    default: return 0;                                                                         
             break;
  }
  return 1;
}

static int linter_handle_get_attribute(pdo_dbh_t *dbh, long attr, zval *return_value TSRMLS_DC)
{
  CURSOR *csr = (CURSOR *)dbh->driver_data;
  char *buf;  
   
  switch (attr)
  {
    case PDO_ATTR_AUTOCOMMIT: ZVAL_LONG(return_value,dbh->auto_commit);
            
                              break;
    case PDO_ATTR_CURSOR_NAME:ZVAL_STRING(return_value,csr->name,1);                              
                              break;
    case PDO_ATTR_CURSOR:     ZVAL_LONG(return_value,PDO_CURSOR_SCROLL);
                              break;
    case PDO_ATTR_SERVER_VERSION: buf = malloc(30);                                  
                                  sprintf(buf,"%d.%d.%d",csr->Dsc.VerMajor,csr->Dsc.VerMinor,csr->Dsc.VerBuild);                                  
                                  ZVAL_STRING(return_value,buf,1);
                                  s_free(buf);
                                  break;
    case PDO_ATTR_CONNECTION_STATUS:  strncpy((char *)csr->CBL.Command, "DESC", 4);            
                                      csr->CBL.LnBufRow = sizeof(csr->DbDsc);                                     
                                      inter(&csr->CBL, NULL, NULL, NULL, &csr->DbDsc);          
                                      if (csr->CBL.CodErr != 0)
                                      {
                                        csr->DbDscLen = 0;                                        
                                        ZVAL_LONG(return_value,0);
                                      }
                                      else                                      
                                        ZVAL_LONG(return_value,1);                                      
                                      csr->err_info.err_code = 0;                                 /* 10.02.09 */ 
                                     
                                     break;
    case PDO_ATTR_SERVER_INFO: GetConnectInfo(csr,return_value);
                               break;

    case PDO_ATTR_CLIENT_VERSION: ZVAL_STRING(return_value,"1.0",1);
                               break;
    case PDO_ATTR_STRINGIFY_FETCHES: ZVAL_LONG(return_value,dbh->stringify);
                               break;
    case PDO_ATTR_FETCH_TABLE_NAMES: ZVAL_BOOL(return_value,csr->fetch_table_names);
                               break;
    case PDO_ATTR_FETCH_CATALOG_NAMES: ZVAL_BOOL(return_value,csr->fetch_catalog_names);
                               break;
    case PDO_ATTR_MAX_COLUMN_LEN: ZVAL_LONG(return_value,4000);
                               break;
    case PDO_ATTR_DT_FORMAT:  ZVAL_STRING(return_value,csr->dt_format,1);                              
                               break;                                                             
    case PDO_ATTR_GENERATE_COL_NAME: ZVAL_BOOL(return_value,csr->GenerateColNameBehaviour);       /* 30.03.09 ... */
                               break;
    case PDO_ATTR_RENAME_DUBBED_COL: ZVAL_BOOL(return_value,csr->RenameDubbedColBehaviour);
                               break;                                                             /* 30.03.09 */
    default: return 0;                                                                            /* 12.02.09 */                                
             break;
  }

  return 1;
}

#if PHP_API_VERSION >= 20070116                                 /* 28.11.08 */
static char *linter_handle_last_insert_id(pdo_dbh_t *dbh, const char *name, int *len TSRMLS_DC)
#else
static char *linter_handle_last_insert_id(pdo_dbh_t *dbh, const char *name, unsigned int *len TSRMLS_DC)
#endif

{  
  CURSOR *csr = (CURSOR *)dbh->driver_data;
  char *return_string;
  
  if (name == NULL)
  {
    L_ULONG id;
    char temp_str[10];
    
    strncpy(csr->CBL.Command,"    ",4);
    inter(&csr->CBL, NULL, "SELECT LAST_ROWID;", NULL, NULL);
  
    csr->err_info.err_code = csr->CBL.CodErr;                                     /* 10.02.09 */
    if (csr->CBL.CodErr != 0)
    {      
      pdo_linter_error(dbh);
      return NULL;
    }

    strncpy(csr->CBL.Command,"GETF",4);
    csr->CBL.LnBufRow = 4;
    inter(&csr->CBL, NULL, NULL, NULL, &id);

    csr->err_info.err_code = csr->CBL.CodErr;                                     /* 10.02.09 */
    if (csr->CBL.CodErr != 0)
    {      
      pdo_linter_error(dbh);
      return NULL;
    }
    *len = sprintf(temp_str,"%i",id);
    return_string = emalloc(*len+1);
    strncpy(return_string,temp_str,*len+1);    
  }
  else
  {
    L_DLONG sequence_val; 
    char *sql_str;

    sql_str = malloc(strlen(name)+19);
    sprintf(sql_str,"SELECT \"%s\".CURRVAL;",name);

    strncpy(csr->CBL.Command,"    ",4);

    inter(&csr->CBL, NULL, sql_str, NULL, NULL);
    s_free (sql_str);
  
    csr->err_info.err_code = csr->CBL.CodErr;                                      /* 10.02.09 */
    if (csr->CBL.CodErr != 0)
    {      
      pdo_linter_error(dbh);
      return NULL;
    }

    strncpy(csr->CBL.Command,"GETF",4);
    csr->CBL.LnBufRow = 8;
    inter(&csr->CBL, NULL, NULL, NULL, &sequence_val);

    csr->err_info.err_code = csr->CBL.CodErr;                                      /* 10.02.09 */
    if (csr->CBL.CodErr != 0)
    {      
      pdo_linter_error(dbh);
      return NULL;
    }
    return_string = php_pdo_int64_to_str(sequence_val TSRMLS_CC); 
  }

  *len = strlen(return_string);
  return return_string;
}

static int linter_handle_check_liveness(pdo_dbh_t *dbh TSRMLS_DC)
{
  CURSOR *csr = (CURSOR *)dbh->driver_data;
    
  strncpy((char *)csr->CBL.Command, "DESC", 4);            
  csr->CBL.LnBufRow = 0;                                     
  inter(&csr->CBL, NULL, NULL, NULL, NULL);  
  csr->err_info.err_code = csr->CBL.CodErr;                                         /* 10.02.09 */
  if (csr->CBL.CodErr != 0)  
    return 0;  
  else  
    return 1;  
  
}

static struct pdo_dbh_methods linter_methods = {
        linter_handle_closer,  
        linter_handle_preparer,
        linter_handle_doer,
        linter_handle_quoter,
        linter_handle_begin,
        linter_handle_commit,
        linter_handle_rollback,
        linter_handle_set_attribute,
        linter_handle_last_insert_id,
        linter_handle_fetch_error_func,
        linter_handle_get_attribute,
        linter_handle_check_liveness, 
        NULL,                                                                       /* 28.11.08 */
        NULL
};
/*
static function_entry *linter_get_driver_methods(pdo_dbh_t *dbh, int kind TSRMLS_DC)
{   
  return NULL;
}
*/

static int pdo_linter_handle_factory(pdo_dbh_t *dbh, zval *driver_options TSRMLS_DC)
{  
  char    user[MAX_ID_LEN*2];  
  CURSOR *csr;    
    
  struct pdo_data_src_parser vars[] = {		
                                        {"node"   ,"",0},
                                        {"dbname" ,"",0},		
                                      };   
  
  int connect_opts    = 0;  
  int autocommit_mode = 1;
  
  csr = calloc(1,sizeof(CURSOR));
    
  if (csr == NULL)
  {     
    pdo_linter_error(dbh);
    return 0;
  }  

  dbh->driver_data = csr;    

  php_pdo_parse_data_source(dbh->data_source, dbh->data_source_len, vars, 2);
	  
  snprintf(user, sizeof(user), "\"%s\"/\"%s\"",dbh->username, dbh->password);
  strncpy((char *)csr->CBL.Node, vars[0].optval, 8);
  csr->CBL.PrzExe = autocommit_mode == 1? TM_AUTOCOMMIT : TM_EXCLUSIVE;           /* 17.02.09 */
  strncpy((char *)csr->CBL.Command, "OPEN", 4);             
  csr->CBL.LnBufRow = sizeof(csr->Dsc);                    
  csr->CBL.Prior = 0;
    
  strncpy(csr->Dsc.BaseName,vars[1].optval,4);  

  inter(&csr->CBL, user, NULL, NULL, &csr->Dsc);
  
  csr->DscLen               = csr->CBL.LnBufRow;  
  csr->DbDscLen             = 0;  
  csr->fetch_catalog_names  = 0;
  csr->fetch_table_names    = 0;  
  csr->GenerateColNameBehaviour = GetGenerateColNameFlag();                       /* 30.03.09 */
  csr->RenameDubbedColBehaviour = GetRenameDubbedColFlag();                       /* 30.03.09 */
    
  csr->pqry_body = NULL;
  csr->dt_format = strdup(GetLinterDateFormat());                                 /* 12.02.09 */  
  csr->ChanLink  = NULL;                                                          /* 17.02.09 */
  
  csr->err_info.err_code = 0;
  strncpy(csr->err_info.err_msg,"",sizeof(csr->err_info.err_msg));    


  csr->err_info.err_code = csr->CBL.CodErr;                                       /* 10.02.09 */
  if (csr->CBL.CodErr != 0)
  {     
    pdo_linter_error(dbh);    
    return 0;
  }

  dbh->methods = &linter_methods;  
  dbh->alloc_own_columns = 1;
  
  return 1;  
}

pdo_driver_t pdo_linter_driver = {
	PDO_DRIVER_HEADER(linter),
	pdo_linter_handle_factory
};

#endif
