/***************************************************************************
 *           RELEX, Inc. Client source include file                         *
 *  This is unpublished proprietary source code of RELEX, Inc.              *
 *  The copyright notice does not evidence any actual or intended           *
 *  publication of such source code.                                        *
 *                                                                          *
 *                 Copyright (c) 2008-2012 RELEX Inc.                       *
 *                                                                          *
 *                   This software was developed by                         *
 *                                                                          *
 *                            RELEX, Inc.                                   *
 *                         Voronezh, Russia                                 *
 *                                                                          *
 *                        All rights reserved.                              *
 ****************************************************************************
 FILE:    linter_statement.c 
 ----------------------------------------------------------------------------
 AUTHOR: Dmitrii Glukhov
 ----------------------------------------------------------------------------
 PURPOSE: PDO interface from PHP language to Linter SQL Server
 ----------------------------------------------------------------------------
 INFORMATION:E-allocation functions (emalloc, ecalloc, etc) are used for 
             external variables, that free by higher layer pdo driver
 ----------------------------------------------------------------------------
 REVISION HISTORY:
 04.08.08 -- Creation
 29.01.09 -- Glukhov (error handling fixed)
 06.02.09A-- Glukhov (LDT_REAL trim fixed)
 06.02.09B-- Glukhov (malloc/emalloc bug fixed)
 10.02.09 -- Glukhov (error handling fixed)
 11.02.09A-- Glukhov (dec & int fetching fixed)
 11.02.09B-- Glukhov (fetch_blob_as_data setting unused now, blobs fetchs as streams)
 12.02.09A-- Glukhov (pdo_linter_stmt_col_meta fixed)
 12.02.09B-- Glukhov (PDO_ATTR_FETCH_BLOB_AS_DATA attribut removed)
 17.02.09 -- Glukhov (Transaction's work fixed, ChanLink added) 
 17.02.09A-- Glukhov (PDO_ATTR_FETCH_CATALOG_NAMES using fixed)
 18.02.09 -- Glukhov (Set cursor name modified)
 19.02.09 -- Glukhov (Using ClearBlob,AddBlob fixed)
 26.02.09 -- Glukhov (Stored procedures params using fixed)
 05.03.09 -- Glukhov (Getting BYTE column fixed)
 11.03.09 -- Glukhov (GetParamValue using removed, binding params type using fixed)
 17.03.09 -- Glukhov (DecimalBehaviour deleted (PDO don't support double))
 18.03.09 -- Glukhov (ATTR_FETCH_TABLE_NAMES and ATTR_FETCH_CATALOG_NAMES attributes added for statement)
 30.03.09 -- Glukhov (GENERATE_COL_NAME and RENAME_DUBBED_COL settings added)
 07.04.09 -- Glukhov (cursor_closer and offset type in pdo_linter_stmt_get_col fixed)
 27.04.12 -- Medvedev (Correct headers for 5.x)
 06.08.14 -- Glukhov, Ermakov (64bit PHP changes)
 14.09.14 -- Fatkhudinov (binded_param_count added)
 30.09.14 -- Fatkhudinov (passing params length in param_hook fixed, checks added)
 02.10.14 -- Fatkhudinov (NULL flag checks added)
 14.10.14 -- Fatkhudinov ( returning long values on 32-bit machines changes )
 16.10.14 -- Fatkhudinov ( EXTFILE column type processing added )
 20.10.14 -- Fatkhudinov ( changes connected with SMALLINT column type )
 23.10.14 -- Fatkhudinov ( check parameter correspondence moved in stmt_execute )
 27.10.14 -- Fatkhudinov ( binded_param_count for blob values added, reseting added )
*/

#ifdef PDO

#include <php.h>
#include <sys/types.h>
#include <php_ini.h>
#include <ext/standard/info.h>
#include <TSRM/TSRM.h>

#include <ext/pdo/php_pdo.h>
#include <ext/pdo/php_pdo_driver.h>

#include "inter.h"
#include "decimals.h"
#include "tick.h"

#include "php_linter.h"
#include "php_pdo_linter.h"
#include "php_pdo_linter_int.h"

#include "tools.h"

static int pdo_linter_stmt_dtor(pdo_stmt_t *stmt TSRMLS_DC)
{
  CURSOR *stmt_csr = (CURSOR *)stmt->driver_data;
  CURSOR *csr = (CURSOR *)stmt->dbh->driver_data;                                                  /* 17.02.09 ... */
  t_ChanLink *ChanLink = NULL;    

  stmt->driver_data = NULL;       


  if (stmt_csr) 
  {
    ClearCursorDesc(stmt_csr,1);        

    if (stmt_csr->CBL.Node)
    {
      if (stmt_csr->CBL.PrzExe != TM_AUTOCOMMIT)                              
      {
        ChanLink = calloc(1,sizeof(t_ChanLink));
        ChanLink->Next = csr->ChanLink;
        ChanLink->subCsr = stmt_csr;
        csr->ChanLink = ChanLink;
        
        return 1;
      }                            
      else                                                                                         /* ... 17.02.09 */
      {
        strncpy(stmt_csr->CBL.Command,"CLOS",4);      
        inter(&stmt_csr->CBL,NULL,NULL,NULL,NULL);      
      }
    }            
    s_free(stmt_csr);         
  }
  
  return 1;
}

static int pdo_linter_stmt_execute(pdo_stmt_t *stmt TSRMLS_DC)
{
  CURSOR *stmt_csr = (CURSOR *)stmt->driver_data;    
  CURSOR *csr = (CURSOR *)stmt->dbh->driver_data;

  if (stmt_csr == NULL)                                                                            /* 29.01.09 ... */
  { 
    csr->err_info.err_code = LPE_NO_MEMORY;      
    pdo_linter_error(stmt->dbh);                       
    return 0;
  }

  if (stmt_csr->pqry_body == NULL)
  { 
    stmt_csr->err_info.err_code = LPE_NO_MEMORY; 
    pdo_linter_error(stmt->dbh);                       
    return 0;
  }                                                                                                /* ... 29.01.09 */

  if (stmt_csr->CBL.PrzExe != csr->CBL.PrzExe)
  {    
    strncpy((char *)stmt_csr->CBL.Command, "SETO", 4);

    stmt_csr->CBL.RowId = crMode;
    stmt_csr->CBL.RowCount = stmt_csr->CBL.NumChan;
    stmt_csr->CBL.LnBufRow = 0;
    stmt_csr->CBL.PrzExe = csr->CBL.PrzExe;

    inter(&stmt_csr->CBL, NULL, NULL, NULL, NULL);

    stmt_csr->err_info.err_code = stmt_csr->CBL.CodErr;                                            /* 29.01.09, 10.02.09 */ 

    if (stmt_csr->CBL.CodErr != 0)
    {            
      pdo_linter_error(stmt->dbh);                                                                 /* 29.01.09 */
      return 0;
    }
  }
  
  /* Execute translated query from pqry_body */

  stmt_csr->CBL.LnBufRow = MAX_ANSWER;                                                             /* 04.08.08 */
  strncpy((char *)stmt_csr->CBL.Command, "EXEC", 4);

  inter(&stmt_csr->CBL, NULL, stmt_csr->pqry_body, NULL, stmt_csr->exch_buff); 

  stmt_csr->err_info.err_code = stmt_csr->CBL.CodErr;                                              /* 10.02.09 */
  
  if (stmt_csr->param_count != stmt_csr->binded_param_count)                                       /* 23.10.14 */
    pdo_raise_impl_error(stmt->dbh,stmt,"HY000", "not enough params" TSRMLS_CC);

  if (stmt_csr->CBL.CodErr > EORR)                              
  {    
    pdo_linter_error_stmt(stmt);
    return 0;
  }  
    
  stmt_csr->rows               = stmt_csr->CBL.RowCount;
  stmt_csr->cols               = 0;  
  stmt_csr->crow               = 0;
  stmt_csr->last_oper          = inter_isselect(&stmt_csr->CBL, stmt_csr->pqry_body);
  stmt_csr->binded_param_count = 0;                                                                /* 27.10.14 */

  FillCursorDesc(stmt_csr,1);    

  stmt->row_count     = stmt_csr->rows;
  stmt->column_count  = stmt_csr->cols;

  return 1;
}

static int pdo_linter_stmt_fetch(pdo_stmt_t *stmt, enum pdo_fetch_orientation ori, long offs TSRMLS_DC)
{
  CURSOR *stmt_csr = (CURSOR *)stmt->driver_data;  
  L_LONG res;
  
  if ((((ori == PDO_FETCH_ORI_ABS) || (ori == PDO_FETCH_ORI_REL)) && (offs == 0)) 
            || (stmt_csr  == NULL) || (stmt_csr->cols == 0))
  {
    pdo_linter_error(stmt->dbh);
    return 0;
  }
  
  if ((ori == PDO_FETCH_ORI_NEXT) && (stmt_csr->crow == 0))  
    ori = PDO_FETCH_ORI_FIRST;  

  if (ori == PDO_FETCH_ORI_REL)  
    offs += stmt_csr->crow;  
  
  switch (ori)
  {
    case PDO_FETCH_ORI_FIRST: res=FetchCursor(stmt_csr,FETCH_FIRST,offs,0);
	      break;
    case PDO_FETCH_ORI_LAST:  res=FetchCursor(stmt_csr,FETCH_LAST,offs,0);     
              break;
    case PDO_FETCH_ORI_NEXT:  res=FetchCursor(stmt_csr,FETCH_NEXT,offs,0);
              break;
    case PDO_FETCH_ORI_PRIOR: res=FetchCursor(stmt_csr,FETCH_PREV,offs,0);
              break;
    case PDO_FETCH_ORI_ABS:   res=FetchCursor(stmt_csr,FETCH_ABSNUM,offs,0);
              break;
    case PDO_FETCH_ORI_REL:   res=FetchCursor(stmt_csr,FETCH_ABSNUM,offs,0);
              break;
    default:                  res=LPE_INVALID_PARAM;
	      break;
  } 

  stmt_csr->err_info.err_code = stmt_csr->CBL.CodErr;                                              /* 10.02.09 */
  
  if (res != LPE_SUCCESS)  
    return 0;    
 
  return 1;
} 

static int pdo_linter_stmt_param_hook(pdo_stmt_t *stmt, struct pdo_bound_param_data *param, enum pdo_param_event event_type TSRMLS_DC)
{ 
  CURSOR *stmt_csr = (CURSOR *)stmt->driver_data;

  L_WORD      *proc_par_count;                                                                     /* 26.02.09 ... */
  L_BYTE      *proc_par_value; 
  L_WORD      *proc_par_val_len;
  L_WORD       proc_par_name_len;
  ARGPROC_OUT *proc_par_info;
  long         proc_paramno;
  double       dec_val;  
  char         dt_val[32];
  char         ds_val[DECLNG*2+1];                                                            
  char        *blob_value; 

  L_LONG  blob_size;                                                                               /* ... 26.02.09 */
  short param_number;
  short p_type = 1; 
 
  if (param->is_param)
  {
    switch (event_type)
    {
      case PDO_PARAM_EVT_ALLOC:
        if (param->paramno < 0 || param->paramno >= stmt_csr->param_count)         
        { 
            pdo_raise_impl_error(stmt->dbh,stmt,"HY093", "parameter was not defined" TSRMLS_CC);    /* 14.09.14 */
            return 0;        
        }
        break;
      case PDO_PARAM_EVT_FREE:
        break;
      case PDO_PARAM_EVT_EXEC_PRE:
        param_number = (short)param->paramno;                                      
                                      
        inter_DescribeParam(stmt_csr->pqry_body, (short)(param_number+1), &stmt_csr->parm_leng[param_number], &p_type, NULL, NULL, NULL);
                                      
        if (p_type == DT_BLOB)  
        {
          stmt_csr->BlFlag = 1;

          if ((PDO_PARAM_TYPE(param->param_type)==PDO_PARAM_LOB)&&(Z_TYPE_P(param->parameter) == IS_RESOURCE))                     
          {
            php_stream *stm;
            php_stream_from_zval_no_verify(stm,&param->parameter);
            if (stm)
            {
              SEPARATE_ZVAL_IF_NOT_REF(&param->parameter);
              Z_TYPE_P(param->parameter) = IS_STRING;
              Z_STRLEN_P(param->parameter) = php_stream_copy_to_mem(stm,&Z_STRVAL_P(param->parameter), PHP_STREAM_COPY_ALL, 0);
            }
            else
            {
              pdo_raise_impl_error(stmt->dbh,stmt,"HY105", "Expected a stream resource" TSRMLS_CC);
              return 0;
            }
          }
          stmt_csr->binded_param_count ++;                                                                      /* 27.10.14 */                             
          return 1;
        }                                      

        if (PDO_PARAM_TYPE(param->param_type) == PDO_PARAM_NULL || Z_TYPE_P(param->parameter) == IS_NULL) 
        {
          stmt_csr->parm_leng[param_number] = -1;          
          inter_bind_num(&stmt_csr->CBL, stmt_csr->pqry_body, (short)(param_number+1), 
                         LDT_INTEGER,&stmt_csr->parm_leng[param_number],(char *)&param->parameter->value.lval); /* 11.03.09 */
          stmt_csr->binded_param_count ++;                                                                      /* 14.09.14 */
          return 1;
        }

        if (PDO_PARAM_TYPE(param->param_type) == PDO_PARAM_STMT)
          return 0;

        switch (PDO_PARAM_TYPE(param->param_type))                                                      /* 11.03.09 ... */
        {
          case PDO_PARAM_STR:        	   
            convert_to_string(param->parameter);            
            break;
          case PDO_PARAM_INT:            
            convert_to_long(param->parameter);
            break;          
        }                                                                                               /* ... 11.03.09 */
                                                                                                        /* 11.02.09B */                                      
        switch (Z_TYPE_P(param->parameter))                                                             /* 11.03.09... */
        {
          case IS_BOOL:
            stmt_csr->parm_leng[param_number] = 4;                                                       /* 30.09.14 */
            inter_bind_num(&stmt_csr->CBL, stmt_csr->pqry_body, (short)(param_number+1), LDT_INTEGER, &stmt_csr->parm_leng[param_number], (char *)&param->parameter->value.lval);          
            stmt_csr->binded_param_count ++;                                                             /* 14.09.14 */
            break;
          case IS_STRING:
            inter_bind_num(&stmt_csr->CBL, stmt_csr->pqry_body, (short)(param_number+1), LDT_CHAR, (short *)&param->parameter->value.str.len, (char *)param->parameter->value.str.val);          
            stmt_csr->binded_param_count ++;                                                             /* 14.09.14 */
            break;
          case IS_LONG:   
            stmt_csr->parm_leng[param_number] = sizeof(long);                                            /* 30.09.14 */
            inter_bind_num(&stmt_csr->CBL, stmt_csr->pqry_body, (short)(param_number+1), LDT_INTEGER, &stmt_csr->parm_leng[param_number], (char *)&param->parameter->value.lval);
            stmt_csr->binded_param_count ++;                                                             /* 14.09.14 */ 
            break;
          case IS_DOUBLE:     
            stmt_csr->parm_leng[param_number] = 8;                                                       /* 30.09.14 */
            inter_bind_num(&stmt_csr->CBL, stmt_csr->pqry_body, (short)(param_number+1), LDT_REAL, &stmt_csr->parm_leng[param_number], (char *)&param->parameter->value.dval);
            stmt_csr->binded_param_count ++;                                                             /* 14.09.14 */  
            break;              
        }                                                                                               /* ... 11.03.09 */                                                                                                                                                                                        
        break;
      case PDO_PARAM_EVT_EXEC_POST:
        if ((stmt_csr->last_oper != SQL_SELECT)&&(stmt_csr->last_oper != SQL_TSTTBL))
        {
          if (stmt_csr->last_oper == SQL_EXEC_PROC)                                                     /* 26.02.09 ... */                                                                                                           
          {                                          
            /* first value - returned value, must be skiped */
            proc_paramno = param->paramno+1;       

            if (stmt_csr->CBL.CodErr == Proc_Raised_Exception)            
              return 0;            
                                       
            proc_par_info  = (ARGPROC_OUT *)stmt_csr->exch_buff;
            proc_par_count = (L_WORD *)(proc_par_info + 1);                                            
            proc_par_info  = (ARGPROC_OUT *)(proc_par_count + 1);
            proc_par_info += proc_paramno-1;                                            
            proc_par_value = stmt_csr->exch_buff + proc_par_info->Value;

            if (proc_par_info->Flags & PARM_FLAG_NAMESENT)
            {                              
#if !defined(LONG_ALIGN)
              proc_par_name_len = *((L_WORD *)proc_par_value);
#else
              memcpy(&proc_par_name_len, proc_par_value, sizeof(L_WORD)); 
#endif
              proc_par_value += sizeof(L_WORD) + proc_par_name_len;
            }

            if (proc_par_info->Flags & PARM_FLAG_ISCURSOR)            
              return 1;            

            if (proc_par_info->Flags & PARM_FLAG_ISNULL)            
              return 1;            

            switch(proc_par_info->Type.ntyp)
            {
              case LDT_BYTE:
              case LDT_CHAR:
              case LDT_NCHAR:  
                convert_to_string(param->parameter);                                          /* 11.03.09 */
                param->parameter->value.str.len = proc_par_info->Type.length;
                strncpy(param->parameter->value.str.val,proc_par_value,param->parameter->value.str.len);                                                
                break;

              case LDT_EXTFILE:                                                               /* 16.10.14 */
                convert_to_string(param->parameter + 10);                                          
                param->parameter->value.str.len = strlen(proc_par_value + 10);
                strncpy(param->parameter->value.str.val,proc_par_value,param->parameter->value.str.len);                                                
                break;

              case LDT_INTEGER:
                convert_to_long(param->parameter);                                            /* 11.03.09 */
                if (proc_par_info->Type.length == 8)
                {
#if !defined(LONG_ALIGN)
                  i64_ToDouble(*(L_DLONG *)proc_par_value, dec_val);
#else                                   
                  L_DLONG Val;          
                  memcpy(&Val, proc_par_value, sizeof(L_DLONG));
                  i64_ToDouble(Val, dec_val);
#endif                                            
                  param->parameter->value.dval = dec_val;                                                  
                }
                else
                {
                  if (proc_par_info->Type.length == 4)
#if !defined(LONG_ALIGN)
                    param->parameter->value.lval = (long)(*(L_LONG *)proc_par_value);                                                    
#else  
                  {
                    L_LONG Val;
                    memcpy(&Val, proc_par_value, sizeof(L_LONG));
                    param->parameter->value.lval = (long)Val;                                                    
                  }
#endif                                                    
                  else
#if !defined(LONG_ALIGN)
                    param->parameter->value.lval = (long)(*(L_SWORD *)proc_par_value);                                                    
#else     
                  {
                    L_SWORD Val;                 
                    memcpy(&Val, proc_par_value, sizeof(L_SWORD));
                    param->parameter->value.lval = (long)Val;                                                    
                  }
#endif                                          
                }
                break;
              case LDT_REAL:
                convert_to_double(param->parameter);                                       /* 11.03.09 */
                if (proc_par_info->Type.length == 4)
#if !defined(LONG_ALIGN)
                  param->parameter->value.dval = (double)(*((L_REAL *)proc_par_value));                                                
#else                                 
                {
                  L_REAL Val;          
                  memcpy(&Val, proc_par_value, sizeof(L_REAL));
                  param->parameter->value.dval = (double)Val;                                                
                }
#endif                                          
                else
#if !defined(LONG_ALIGN)
                  param->parameter->value.dval = (double)(*((L_DOUBLE *)proc_par_value));                                                
#else                                   
                {
                  L_DOUBLE Val;          
                  memcpy(&Val, proc_par_value, sizeof(L_DOUBLE));
                  param->parameter->value.dval = (dobule)Val;                                                
                }
#endif   
                break;
              case LDT_NUMERIC:                                                             /* 17.03.09 */
                convert_to_string(param->parameter);                                        /* 11.03.09 */
                DECTOSTR(*(L_DECIMAL*)proc_par_value, ds_val, proc_par_info->Type.prcs, proc_par_info->Type.scale, TRUE); 
                strrtrim(ds_val," ");                                                    
                param->parameter->value.str.len = strlen(ds_val);
                strncpy(param->parameter->value.str.val,ds_val,param->parameter->value.str.len);                                                                
                break;                                                                                                    
              case LDT_DATE:   
                convert_to_string(param->parameter);                                        /* 11.03.09 */
                TICKTOSTRF(*(L_DECIMAL*)proc_par_value, stmt_csr->dt_format, dt_val);                                                  
                param->parameter->value.str.len = strlen(dt_val);
                strncpy(param->parameter->value.str.val,dt_val,param->parameter->value.str.len);                                                
                break;
              case LDT_BLOB:  
                convert_to_string(param->parameter);                                        /* 11.03.09 */
                if (GetBlobSize(stmt_csr, proc_paramno + 1, &blob_size) != LPE_SUCCESS)
                  return 0;

                blob_value = (char *)calloc(1, blob_size);
                if (blob_value == NULL)
                  return 0;                                                  
                if (FetchBlob(stmt_csr, 1, &blob_size, proc_paramno + 1, blob_value) != LPE_SUCCESS)
                {
                  efree(blob_value); 
                  return 0;
                }
                strncpy(param->parameter->value.str.val,blob_value,blob_size);
                param->parameter->value.str.len = blob_size;                                                
                free(blob_value);                                                  
                break;
              case LDT_VARCHAR:
              case LDT_VARBYTE:
              case LDT_NVARCHAR: 
                convert_to_string(param->parameter);                                         /* 11.03.09 */
#if !defined(LONG_ALIGN)                
                proc_par_val_len = (L_WORD *)proc_par_value;
                proc_par_value += sizeof(L_WORD);
                param->parameter->value.str.len = (unsigned int)(*proc_par_val_len);
                strncpy(param->parameter->value.str.val,proc_par_value,param->parameter->value.str.len);                                                                                                  
#else                                       
                {
                  L_WORD ParLen;

                  memcpy(&ParLen, proc_par_value, sizeof(L_WORD));
                  proc_par_value += sizeof(L_WORD);
                  param->parameter->value.str.len = (unsigned int)ParLen;
                  strncpy(param->parameter->value.str.val,proc_par_value,param->parameter->value.str.len);                                                  
                }
#endif  
                break;
              case LDT_BOOLEAN:
                convert_to_long(param->parameter);                                            /* 11.03.09 */
                param->parameter->value.lval = *proc_par_value;                                                
                break;
              default:
                return 0;
            }

            switch (PDO_PARAM_TYPE(param->param_type))                                                      /* 11.03.09 ... */
            {              
              case PDO_PARAM_INT:            
                convert_to_long(param->parameter);
                break;
              case PDO_PARAM_BOOL:
                convert_to_boolean(param->parameter);
                break;
              default:              
                convert_to_string(param->parameter);                
            }
                                                                                               /* ... 11.03.09 */
            return 1;
          }                                                          /* ... 26.02.09 */
          if (stmt_csr->BlFlag == 1)                                           
          {
            param_number = (short)param->paramno;
                                                                                            
            inter_DescribeParam(stmt_csr->pqry_body, (short)(param_number+1), &stmt_csr->parm_leng[param_number],
                  &p_type, NULL, NULL, NULL);

             if (p_type != DT_BLOB)               
               return 1;             

             if (!ClearBlob(stmt_csr,inter_GetBlobColNum(stmt_csr->pqry_body,(short)(param_number+1))))                                                                     /* 19.02.09 */             
               return 0;             
         
             if (!AddBlob(stmt_csr, param->parameter->value.str.val, param->parameter->value.str.len, 0, inter_GetBlobColNum(stmt_csr->pqry_body,(short)(param_number+1)))) /* 19.02.09 */             
               return 0;             
          }
        }
        break;
      case PDO_PARAM_EVT_FETCH_PRE:   break;
      case PDO_PARAM_EVT_FETCH_POST:  break;
      case PDO_PARAM_EVT_NORMALIZE:   break;
    }
  }

  return 1;
}

static int pdo_linter_stmt_describe(pdo_stmt_t *stmt, int colno TSRMLS_DC)
{
  CURSOR *stmt_csr  = (CURSOR *)stmt->driver_data;
  CURSOR *csr       = (CURSOR *)stmt->dbh->driver_data;

  struct pdo_column_data *cols = stmt->columns;  
  char   col_name[MAX_ID_LEN*2 +28],
         buf[MAX_ID_LEN*2 +28];

  if (colno >= stmt->column_count)                                                                 /* 04.08.08 */
    return 0;
 
  cols[colno].precision = stmt_csr->answ_desc[colno].Precision;
  cols[colno].maxlen = stmt_csr->answ_desc[colno].Length;    
    
  if (stmt_csr->fetch_catalog_names)                                                               /* 18.03.09 */
  {    
    sprintf(col_name,"%s",csr->CBL.Node);                                                          /* 17.02.09A */                    
    strncpy(buf,csr->Dsc.BaseName,18);            
    *(buf+sizeof(char)*18)='\0';
    strrtrim(buf," ");
    strcpy((buf+(strlen(buf))*sizeof(char)),".");      
    *(buf+sizeof(char)*19)='\0';
    strcat(col_name,buf);
      
  }
  if (stmt_csr->fetch_table_names)                                                                 /* 18.03.09 */
  {    
    strncpy(buf,stmt_csr->answ_desc[colno].Table,MAX_ID_LEN);
    *(buf+sizeof(char)*MAX_ID_LEN)='\0';
    strrtrim(buf," ");
    strcpy((buf+(strlen(buf))*sizeof(char)),".\0");            
    if (stmt_csr->fetch_catalog_names==0)                                                          /* 18.03.09 */    
      strcpy(col_name,buf);    
    else    
      strcat(col_name,buf);    
  }
  strncpy(buf,stmt_csr->answ_desc[colno].Column,MAX_ID_LEN);
  *(buf+sizeof(char)*MAX_ID_LEN)='\0';
  strrtrim(buf," ");    
  if (!stmt_csr->fetch_catalog_names && !stmt_csr->fetch_table_names)                              /* 18.03.09 */  
    strcpy(col_name,buf);  
  else  
    strcat(col_name,buf);  
    
  cols[colno].namelen = strlen(col_name);
  cols[colno].name = ecalloc(cols[colno].namelen+1,sizeof(char));
  strncpy(cols[colno].name,col_name,cols[colno].namelen);
      
  switch (stmt_csr->answ_desc[colno].Type)
  {
    case LDT_BYTE:   
    case LDT_CHAR:
    case LDT_NCHAR:  cols[colno].param_type = PDO_PARAM_STR;
                     break;
    case LDT_EXTFILE:cols[colno].param_type = PDO_PARAM_STR;                                       /* 16.10.14 */
                     break;
    case LDT_INTEGER:if ((stmt_csr->answ_desc[colno].Length > 4) &&                                /* 11.02.09A ... */                     
                         (sizeof(long) == 4))                                                     /* 22.10.14 */
                       cols[colno].param_type = PDO_PARAM_STR;
                     else                     
                       cols[colno].param_type = PDO_PARAM_INT;                                     /* ... 11.02.09A */
                     break;
    case LDT_REAL:   cols[colno].param_type = PDO_PARAM_STR;
                     break;
    case LDT_NUMERIC:cols[colno].param_type = PDO_PARAM_STR;
                     break;
    case LDT_DATE:   cols[colno].param_type = PDO_PARAM_STR;
                     break;
    case LDT_BLOB:   cols[colno].param_type = PDO_PARAM_LOB;
                     break;
    case LDT_VARCHAR:
    case LDT_VARBYTE:
    case LDT_NVARCHAR:cols[colno].param_type = PDO_PARAM_STR; 
                     break;
    case LDT_BOOLEAN:cols[colno].param_type = PDO_PARAM_BOOL;
                     break;
  } 
  
  return 1;
}

static int pdo_linter_stmt_get_col(pdo_stmt_t *stmt, int colno, char **ptr, unsigned long *len, int *caller_frees TSRMLS_DC)
{  
  CURSOR *stmt_csr = (CURSOR *)stmt->driver_data;
  long    off=0;                                                                    /* 07.04.09 */
  char   *buf;
  char   *blob_value;
  L_LONG  blob_size;  
  static long    l_val;
  double  d_val;
  static char t_val[64];
  static char ds_val[DECLNG*2+1];  

  if (colno > stmt_csr->cols)
  {
    *ptr = NULL;
    *len = 0;
    return 0;
  }

  off = stmt_csr->ColOffset[colno];
  if (stmt_csr->null_flag[sizeof(L_WORD) * 2 + colno])        /* 16.10.14 */
  {
    *ptr = NULL;
    *len = 0;
    return 1;
  }  
  else
    switch (stmt_csr->answ_desc[colno].Type)
    {   
      case LDT_NCHAR:buf = malloc(stmt_csr->answ_desc[colno].Length);
                     memcpy(buf,&stmt_csr->exch_buff[off],stmt_csr->answ_desc[colno].Length);                     
                     *len = stmt_csr->answ_desc[colno].Length-2;
                     while (strrchr(" ", buf[*len]) && (*len)>= 0)
                     {
                       buf[(*len)] = 0;
                       (*len) -=2;
                     }
                     (*len)+=2;
                     
                     s_free(buf);
                     *ptr = &stmt_csr->exch_buff[off];                                                                      
                     break;              
      case LDT_CHAR: buf = malloc(stmt_csr->answ_desc[colno].Length+1);        
                     memcpy(buf,&stmt_csr->exch_buff[off],stmt_csr->answ_desc[colno].Length);                   
                     buf[stmt_csr->answ_desc[colno].Length] = 0;
                     strrtrim(buf," ");       
                     *len = strlen(buf);
                     *ptr = &stmt_csr->exch_buff[off];  
                     s_free(buf);	
                     break;
      case LDT_EXTFILE:                                                                             /* 16.10.14 */
                     buf = malloc(stmt_csr->answ_desc[colno].Length+1);        
                     memcpy(buf,&stmt_csr->exch_buff[off] + 10,stmt_csr->answ_desc[colno].Length);                   
                     buf[stmt_csr->answ_desc[colno].Length] = 0;
                     strrtrim(buf," ");       
                     *len = strlen(buf);
                     *ptr = &stmt_csr->exch_buff[off] + 10;  
                     s_free(buf);	
                     break;
      case LDT_BYTE: *ptr = &stmt_csr->exch_buff[off];                   
                     *len = min(strlen(*ptr),stmt_csr->answ_desc[colno].Length); 
                     break;
      case LDT_INTEGER: 
                       if ((stmt_csr->answ_desc[colno].Length > 4)                                     /* 11.02.09A... */
                            &&(sizeof(long) == 4))                                                                    /* 22.10.14 */
                       {
                         buf = emalloc(100);                                                         
                         l_val = GetDataColInteger(stmt_csr,colno, off, buf);

                         *ptr = buf;                                                                   /* 02.10.14 */

                         *len = strlen(buf);
                         *caller_frees = 1;                   
                       }                                                                               /* ... 11.02.09A */
                       else
                       {
                         l_val = GetDataColInteger(stmt_csr,colno, off, NULL);                       
                         *ptr = (char *)&l_val;
                         *len = sizeof(l_val);                         /* 06.08.14 */
                       }
                     break;
      case LDT_REAL: d_val = GetDataColReal(stmt_csr,colno, off);
                         
                     buf = emalloc(100);                                                             /* 06.02.09B */
                     *len = sprintf(buf,"%f",d_val);                   
                     *(buf+(*len))='\0';                   
                     strrtrim(buf,"0");
                     if (buf[strlen(buf)-1]=='.')                                                    /* 06.02.09A */                   
                       buf[strlen(buf)-1]='\0';                                                      /* 06.02.09A */                          

                     *len = strlen(buf);
                     *ptr = buf;
                     *caller_frees = 1;                   
                     break;

      case LDT_NUMERIC: DECTOSTR(*((L_DECIMAL*)&stmt_csr->exch_buff[off]), ds_val, 30, 10, FALSE);     /* 04.08.08, 11.02.09A*/
                        strrtrim(ds_val, " ");
                        strltrim(ds_val, " ");
                        *ptr = ds_val;
                        *len = strlen(ds_val);
                        break;
      case LDT_DATE:    TICKTOSTRF(*((L_DECIMAL*)&stmt_csr->exch_buff[off]), stmt_csr->dt_format, t_val);
                        *ptr = t_val;
                        *len = strlen(t_val);
                        break;    
      case LDT_BLOB:    if(GetBlobSize(stmt_csr, colno+1, &blob_size) != LPE_SUCCESS)                /* 11.02.09B */                      
                          return 0;                      

                        blob_value = (char *)emalloc(blob_size);                                     /* 06.02.09B */
                        if (blob_value == NULL)
                        {
                          stmt_csr->err_info.err_code = LPE_NO_MEMORY;                               /* 29.01.09 */ 
                          pdo_linter_error(stmt->dbh);                                               /* 29.01.09 */

                          return 0;
                        }
                       
                        if (FetchBlob(stmt_csr, 1, &blob_size, colno + 1, blob_value)!= LPE_SUCCESS)
                        {
                          efree(blob_value);                                                         /* 06.02.09B */
                          return 0;
                        }                               
                       
                        *ptr = blob_value;
                        *len = blob_size;
                        *caller_frees = 1;                    
                       
                        break; 

      case LDT_VARBYTE:
      case LDT_VARCHAR:     
      case LDT_NVARCHAR:*ptr = (char *)&stmt_csr->exch_buff[off];                      
  #if !defined(LONG_ALIGN)
                          *len = (long)(*(L_WORD *)*ptr);
  #else /* LONG_ALIGN */                  
                        {
                          L_WORD  Val;
                          memcpy((char *)&Val, (CHAR *)*ptr, sizeof(L_WORD));
                          *len = (long)Val;
                        }
  #endif /* LONG_ALIGN */

                        *ptr += sizeof(L_WORD);                                                                 


                        break;
          
      case LDT_BOOLEAN: 
                        *ptr = (char *)&stmt_csr->exch_buff[off];                                            
                        *len = 1; 
                        break;
    }  


  return 1;
}
                                   
static int pdo_linter_stmt_set_attr(pdo_stmt_t *stmt, long attr, zval *val TSRMLS_DC)
{  
  CURSOR *stmt_csr = (CURSOR *)stmt->driver_data;

  switch (attr)
  {
    case PDO_ATTR_CURSOR_NAME:strncpy((char *)stmt_csr->CBL.Command, "SETO", 4);

                              stmt_csr->CBL.RowId = crName;                              
                              stmt_csr->CBL.LnBufRow = val->value.str.len;                         /* 18.02.09 */                              

                              inter(&stmt_csr->CBL, NULL, NULL, NULL, val->value.str.val);

                              stmt_csr->err_info.err_code = stmt_csr->CBL.CodErr;                  /* 29.01.09, 10.02.09 */ 
                              if (stmt_csr->CBL.CodErr != 0)
                              {                                
                                pdo_linter_error(stmt->dbh);                                       /* 29.01.09 */

                                return 0;
                              }
                              
                              strncpy(stmt_csr->name,val->value.str.val,min(sizeof(stmt_csr->name),MAX_ID_LEN+2));

                              break;
    case PDO_ATTR_DT_FORMAT:  s_free(stmt_csr->dt_format);
                              stmt_csr->dt_format = strdup(val->value.str.val);                    /* 04.08.08 */
                              break;  
    case PDO_ATTR_FETCH_TABLE_NAMES: stmt_csr->fetch_table_names = (L_BOOL)val->value.lval;             /* 18.03.09 */
                              break;
    case PDO_ATTR_FETCH_CATALOG_NAMES: stmt_csr->fetch_catalog_names = (L_BOOL)val->value.lval;         /* 18.03.09 */
                              break;
    case PDO_ATTR_GENERATE_COL_NAME: stmt_csr->GenerateColNameBehaviour = (L_BOOL)val->value.lval; /* 30.03.09 ... */
                              break;
    case PDO_ATTR_RENAME_DUBBED_COL: stmt_csr->RenameDubbedColBehaviour = (L_BOOL)val->value.lval;  
                              break;                                                               /* ... 30.03.09 */
    default: return 0;                                                                             /* 12.02.09B */
             break;
  }

  return 1;
}

static int pdo_linter_stmt_get_attr(pdo_stmt_t *stmt, long attr, zval *return_value TSRMLS_DC)
{
  CURSOR *stmt_csr = (CURSOR *)stmt->driver_data;
    
  switch (attr)
  {    
    case PDO_ATTR_CURSOR_NAME:ZVAL_STRING(return_value,stmt_csr->name,1);                              
                              break;
    case PDO_ATTR_CURSOR:     ZVAL_LONG(return_value,PDO_CURSOR_SCROLL);
                              break;            
    case PDO_ATTR_MAX_COLUMN_LEN: ZVAL_LONG(return_value,4000);
                               break;                               
    case PDO_ATTR_DT_FORMAT:  ZVAL_STRING(return_value,stmt_csr->dt_format,1);                     /* 04.08.08 */
                              break;  
    case PDO_ATTR_FETCH_TABLE_NAMES: ZVAL_BOOL(return_value,stmt_csr->fetch_table_names);          /* 18.03.09 */
                               break;
    case PDO_ATTR_FETCH_CATALOG_NAMES: ZVAL_BOOL(return_value,stmt_csr->fetch_catalog_names);      /* 18.03.09 */
                               break;
    case PDO_ATTR_GENERATE_COL_NAME: ZVAL_BOOL(return_value,stmt_csr->GenerateColNameBehaviour);   /* 30.03.09 ... */
                               break;
    case PDO_ATTR_RENAME_DUBBED_COL: ZVAL_BOOL(return_value,stmt_csr->RenameDubbedColBehaviour);
                               break;                                                              /* ... 30.03.09 */
    default: return 0;                                                                             /* 12.02.09B */
             break;
  }
  
  return 1;
}

static int pdo_linter_stmt_col_meta(pdo_stmt_t *stmt, long colno, zval *return_value TSRMLS_DC)
{
  CURSOR *stmt_csr = (CURSOR *)stmt->driver_data;
  char    type_name[MAX_ID_LEN + 2];
  zval *flags;

  array_init(return_value);

  if ((!stmt_csr->answ_desc) || (colno >= stmt_csr->cols))  
    return -1;                                                                                     /* 12.02.09A */    
  
  switch (stmt_csr->answ_desc[colno].Type)
  {
    case LDT_BOOLEAN:  add_assoc_string(return_value,"native_type","bool",1);                      /* 12.02.09A ... */
                       break;
    case LDT_BYTE:     add_assoc_string(return_value,"native_type","string",1);  
    case LDT_INTEGER:  if ((stmt_csr->answ_desc[colno].Length > 4) &&
                           (sizeof(long) == 4))                       
                         add_assoc_string(return_value,"native_type","string",1);                        
                       else                       
                         add_assoc_string(return_value,"native_type","integer",1);                        
                       break;                                                                      /* ... 12.02.09A */
    case LDT_REAL:     add_assoc_string(return_value,"native_type","float",1); 
                       break;
    case LDT_CHAR:
    case LDT_DATE:
    case LDT_BLOB:
    case LDT_VARCHAR:
    case LDT_VARBYTE:                                                                              /* 12.02.09A */
    case LDT_NCHAR:
    case LDT_NVARCHAR:
    case LDT_NUMERIC:
    case LDT_EXTFILE:  add_assoc_string(return_value,"native_type","string",1); 
                       break;
    default: add_assoc_string(return_value,"native_type","",1);
             break;
  }

  GetTypeName((long)stmt_csr->answ_desc[colno].Type,(long)stmt_csr->answ_desc[colno].Length,(char *)&type_name, MAX_ID_LEN + 2);   /* 12.02.09A */  
  add_assoc_string(return_value,"decl_type",type_name,1);                                                                          /* 12.02.09A */  

  MAKE_STD_ZVAL(flags);
  array_init(flags);
  add_assoc_zval(return_value, "flags", flags);

  return 1;
}

static int pdo_linter_stmt_next_rowset(pdo_stmt_t *stmt TSRMLS_DC)
{   
  CURSOR *stmt_csr = (CURSOR *)stmt->driver_data;
  
  stmt_csr->cols = 0;
  stmt_csr->rows = 0;  
  
  return 0;
}

static int pdo_linter_stmt_cursor_closer(pdo_stmt_t *stmt TSRMLS_DC)
{
  CURSOR *stmt_csr = (CURSOR *)stmt->driver_data;  
  ClearCursorDesc(stmt_csr,0);                                                      /* 07.04.09 */

  return 0;
}

struct pdo_stmt_methods linter_stmt_methods = {  
  pdo_linter_stmt_dtor,
  pdo_linter_stmt_execute,
  pdo_linter_stmt_fetch,
  pdo_linter_stmt_describe,
  pdo_linter_stmt_get_col,
  pdo_linter_stmt_param_hook,
  pdo_linter_stmt_set_attr,
  pdo_linter_stmt_get_attr,
  pdo_linter_stmt_col_meta,
  pdo_linter_stmt_next_rowset,
  pdo_linter_stmt_cursor_closer
};

#endif
