/***************************************************************************
 *           RELEX, Inc. Client source include file                         *
 *  This is unpublished proprietary source code of RELEX, Inc.              *
 *  The copyright notice does not evidence any actual or intended           *
 *  publication of such source code.                                        *
 *                                                                          *
 *                 Copyright (c) 2008-2012 RELEX Inc.                       *
 *                                                                          *
 *                   This software was developed by                         *
 *                                                                          *
 *                            RELEX, Inc.                                   *
 *                         Voronezh, Russia                                 *
 *                                                                          *
 *                        All rights reserved.                              *
 ****************************************************************************
 FILE:    pdo_linter.c
 ----------------------------------------------------------------------------
 AUTHOR: Dmitrii Glukhov
 ----------------------------------------------------------------------------
 PURPOSE: PDO interface from PHP language to Linter SQL Server
 ----------------------------------------------------------------------------
 REVISION HISTORY:
 04.08.08 -- Creation
 28.11.08 -- Glukhov (remove global variables)
 17.02.09 -- Glukhov (close_sub_chans added) 
 27.04.12 -- Medvedev (Correct headers for 5.x)
*/

#ifdef PDO

#include <php.h>
#include <sys/types.h>
#include <stdio.h>
#include <php_ini.h>
#include <ext/standard/info.h>
#include <TSRM/TSRM.h>

#include <ext/pdo/php_pdo.h>
#include <ext/pdo/php_pdo_driver.h>

#include "inter.h"
#include "php_linter.h"                                         /* 17.02.09 */
#include "php_pdo_linter.h"

int get_sql_state(pdo_error_type *sql_state,int linter_error_code)
{ 
  strcpy((char*)sql_state,"HY000"); /* HY000 - General error */

  switch (linter_error_code)
  {     
    case NOFREEKAN: /* Communication link failure */
    case ERRWRITEMSG:
    case ERRREADMSG:
                    strcpy((char *)sql_state, "08S01");
                    break;
    case INVPARNUM: /* Invalid descriptor index */
                    strcpy((char *)sql_state, "07009");
                    break;
    case SQLLSTLONG: /* Insert value list does not match column list */
    case SQLLSTSHRT:
                    strcpy((char *)sql_state, "21S01");
                    break;
    case CursorNameTooLong:
                    strcpy((char *)sql_state, "01004");
                    break;
    case Invalid_User_Name: /* Invalid authorization specification */
    case Invalid_User_Passwd:
                    strcpy((char *)sql_state, "28000");
                    break;
    case NOSUCHCUR: /* Invalid cursor name */
                    strcpy((char *)sql_state, "34000");
                    break;
    case DupCurName: /* Duplicate cursor name */
                    strcpy((char *)sql_state, "3C000");
                    break;
    case SQLNUMBSYNT: /* Syntax error or access violation */
    case SQLSYNTAX:
    case SQLFNCSYNT:
    case SQLEXPRSYNT:
    case SQLSQSYNT:
    case SQLSCRTAB:
    case SQLSCOL:
    case SQLSDEF:
    case SQLSCNULL:
    case SQLPRVSYNT:
    case SQLBPASSW:
    case Pattern_Syntax_Error:
                    strcpy((char *)sql_state, "42000");
                    break;
    case RELEXIST: /* Base table or view already exists */
                    strcpy((char *)sql_state, "42S01");
                    break;
    case SQLUNKTAB: /* Base table or view not found */
                    strcpy((char *)sql_state, "42S02");
                    break;
    case ATREXIST: /* Column already exists */
                    strcpy((char *)sql_state, "42S21");
                    break;
    case SQLUNKCRF: /* Column not found */
                    strcpy((char *)sql_state, "42S22");
                    break;
    case SQLIEXIST: /* Index already exists */
                    strcpy((char *)sql_state, "42S11");
                    break;
    case SQLINEXIST: /* Index not found */
                    strcpy((char *)sql_state, "42S11");
                    break;
  }  

  return 1;
}

int to_autocommit_mode(TCBL *CBL)
{
  strncpy((char *)CBL->Command, "SETO", 4);

  CBL->RowId = crMode;
  CBL->RowCount = CBL->NumChan;
  CBL->LnBufRow = 0;
  CBL->PrzExe = TM_AUTOCOMMIT;                                          /* 17.02.09 */

  inter(CBL, NULL, NULL, NULL, NULL);

  if (CBL->CodErr != 0)  
    return 0;  

  return 1;
}

int close_sub_chans(CURSOR *csr, int is_rollback)                       /* 17.02.09 ... */
{
  t_ChanLink *ChanLink = csr->ChanLink;
  char *query = malloc(10);

  if (is_rollback != 1)  
    strncpy(query,"COMMIT;",7);  
  else  
    strncpy(query,"ROLLBACK;",9);  
  
  while (ChanLink)            
  {  
    /* commit/rollback */
    strncpy((char *)ChanLink->subCsr->CBL.Command, "    ", 4);
    ChanLink->subCsr->CBL.LnBufRow = MAX_ANSWER;  
    inter(&ChanLink->subCsr->CBL, NULL, query, NULL, NULL);

    /* close */
    strncpy(ChanLink->subCsr->CBL.Command,"CLOS",4);
    inter(&ChanLink->subCsr->CBL,NULL,NULL,NULL,NULL);
    
    csr->ChanLink = ChanLink->Next;

    s_free(ChanLink->subCsr);        
    s_free(ChanLink);
        
    ChanLink = csr->ChanLink;    
  } 
  return 1;
}                                                                       /* ... 17.02.09 */

#endif