 /***************************************************************************
 *           RELEX, Inc. PHP core client source file                        *
 *  This is unpublished propietary source code of RELEX, Inc.               *
 *  The copyright notice does not evidence any actual or intended           *
 *  publication of such source code.                                        *
 *                                                                          *
 *                   Copyright (c) 1996-2012 RELEX Inc.                     *
 *                                                                          *
 *                   This software was developed by                         *
 *                                                                          *
 *                            RELEX, Inc.                                   *
 *                         Voronezh, Russia                                 *
 *                                                                          *
 *                        All rights reserved.                              *
 ****************************************************************************
 FILE:   php_linter.c
 ----------------------------------------------------------------------------
 AUTHOR: Alexander V. Kefeli
 ----------------------------------------------------------------------------
 PURPOSE: Interface from PHP language to Linter SQL Server
 ----------------------------------------------------------------------------
 REVISION HISTORY:

 22.07.99 -- Kent (file creation)
 16.12.99 -- Kononyhin (set crName before SETO)
 20.12.99 -- Galizin (free cursor memory in DelFromList)
 21.12.99 -- Kent, Galizin (correct length for VAR types)
 18.02.00 -- Kent (Clear cursor before execute query)
 21.02.99 -- Kent (Yet another bug with length for VAR types)
 25.02.00 -- Kent (LAST ROWID cursor option)
 06.03.00 -- Kononyhin(ecalloc instead of calloc, PHP3 - PHP4)
 12.03.00 -- Ermakov (php_linter.h instead of php3_linter.h )
 15.03.00 -- Kent (Close all cursors on shutdown)
 29.03.00 -- Kent (Save pointer on next cursor for open connect)
 18.04.00 -- Kent (Get answer info if no records)
 19.04.00 -- Kent (Correct constant for codepage 866)
 26.05.00 -- Kent,Kononyhin (bugs correction in LinPHP_BLOBGetData & LinPHP_BLOBGetSize)
 01.06.00 -- Kononyhin (bug with VAR* checked for get blob size)
 02.06.00 -- Kent (Call UninitLinterClient on unload)
 06.06.00 -- Kononyhin (bug fixed)
 07.06.00 -- Kononyhin (use calloc,malloc,free,strdup instead of ecalloc etc)
 14.06.00 -- Kononyhin (duplicate return value - because of freeing ret_val later)
 16.06.00 -- Kononyhin (input -> _input)
 27.06.00 -- Kononyhin (bug with null indicators fixed)
 18.08.00 -- Kononyhin (add associative array)
 22.08.00 -- Kononyhin (don't clean CodErr in linter_last_error)
 24.08.00 -- Kononyhin (restore buffer length after GOBJ)
 21.09.00 -- Kononyhin (add BIGINT)
 02.10.00 -- Kent (Add `LinPHP_GetDataRows')
 25.10.00 -- Kent (All constants are persistent)
 30.10.00 -- Kent (No global.h and other like it)
 09.11.00 -- Kent (New routines for stored procedures)
 13.11.00 -- Kent (Don't clear prepared query body after execute)
 17.11.00 -- Kent (Don't use channel number as cursor ID)
 21.11.00 -- Kononyhin (correct default date format)
 28.11.00 -- Kononyhin (correct Info information)
 07.12.00 -- Kent (Correct syntax mistake (add comma `,'))
 27.12.00 -- Kent (MAX_ID_LEN instead 18[+x])
 03.01.01 -- Kent (Bug fixed: cursor ID instead channel number)
 04.01.01 -- Kent (Correct work with hashes for version 3 & 4)
 13.02.01 -- Kononyhin (use PAGE_BUFF in FillCursorDesc)
 07.03.01 -- Kent (Don't pass a buffer not `EXECUTE' queries)
 27.03.01 -- Kononyhin (working with multiple blobs)
 28.03.01 -- Kononyhin (set operation as SQL_UNKNOWN if not SELECT)
 14.05.01 -- Kononyhin (using GETM)
 26.05.01 -- Kononyhin (include modules.h or zend_modules.h)
 20.06.01 -- Kononyhin (bug in blob loading corrected)
 09.08.01 -- Kononyhin (remove direct including of modules.h -> it includes from php.h)
 22.11.01 -- Kononyhin (new interface functions for adding blob &&
                        using internal api function for real blob adding)
 17.12.01 -- Kononyhin (building php4.1.0 for ZEND_MODULE_API_NO >= 20010901)
 21.03.02 -- Kononyhin (check demo license)
 08.04.02 -- Galizin   (Move check demo license in tools.c ) 
 31.05.02 -- Kononyhin (always set LnBufRow before fetch)
 31.07.02 -- Galizin   (Remove compiler warning PT#1046)
 27.09.02 -- Goncharov (Correcting code page names for english locale)
 14.05.03 -- Galizin   (Long align systems)
 15.05.03 -- Galizin   ( Correct return type names. Correct index of associative array)
 29.08.03 -- Kononyhin (Close cursors and uninit intlib in LinPHP_Req_Uninit)
 06.02.04 -- Ermakov   (For NULL values don't use kernel buffer)
 20.02.04 -- Galizin   (MAC Os X build)
 04.03.04 -- Ermakov   (new UPDATABLE && QUERY_TYPE options)
 28.06.04 -- Peshkoff  (NCHAR & NVARCHAR support added)
 18.08.04 -- Peshkoff  (numeric data should be converted to string)
 13.09.04 -- Gluhov    (new functions FetchRow, FetchArray, SetCodepage)
 17.09.04 -- Gluhov    (fixed bug in fucntion GetParamValue)
 27.09.04 -- Gluhov    (remake 18.08.04)
 08.10.04 -- Gluhov    (fixed bug with type SMALLINT )
 29.10.04 -- Gluhov    (multithread support) 
 05.11.04 -- Gluhov,Ermakov (fixed bug with codepage in binded values)
 17.11.04 -- Ermakov   ( corrected max_id infinite usage)
 13.01.05 -- Matasov   (fixed parent pointer access in OpenCursor)  
 09.03.05 -- Gluhov    (changes in LinPHP_LastError & LinPHP_ErrorMsg)
 28.07.05 -- Ermakov   (TT#12264. Corrected GETM)
 01.02.06 -- Ermakov, Gluhov (new function LinPHP_ConnectInfo ) 
 12.04.06 -- Gluhov    ( Error checking added - GPF avoid )
 05.05.06 -- Gluhov    ( Added EXEC proc handling )
 13.02.07 -- Ermakov   ( VC 8.0 )
 24.04.07 -- Max       ( PHP 6.x compat )
 11.10.07 -- Ermakov   ( Corrected RETURN values. memory leaks, invalid arguments)
 13.10.07 -- Ermakov   ( Added LINTER_GetParamProp )
 20.10.07 -- Gluhov    ( Corrected work with blobs, added FETCH_BLOBS_AS_USUAL_DATA cursor option)
 27.02.08 -- Ermakov   ( Avoid double increment for SEQUENCE value )
 01.03.08 -- Ermakov   ( Added BLOB binding on insert && update, unified blob function)
 11.03.08 -- Ermakov   ( Connect bind string to double && BIGINT value handle )
 28.03.08A-- Ermakov   ( Don't extra CALL for GETA size less when initial buffer)
 28.03.08B-- Ermakov   ( Don't call GETN if cursor already at end)
 28.03.08C-- Ermakov   ( Added ';' at and of query)
 29.03.08 -- Ermakov   ( Added CO_NULL_AS_NULL_OBJECT - for bind and get data)
 30.03.08 -- Ermakov   ( unified ini - added new constants )
 28.04.08 -- Ermakov   ( Correct 28.03.08A.)
 28.04.08A-- Ermakov   ( Added check on error before make result, added error handling in get_data_rows)
 24.06.08 -- Ermakov   ( Removed old compat. code,
                         Remove extra buffers, Corrected memory handling)
 22.01.09 -- Glukhov   ( Added pdo interface)
 12.02.09 -- Glukhov   ( GetTypeName function added)
 12.02.09A-- Glukhov   ( ATTR_FETCH_BLOB_AS_DATA PDO constant removed)
 19.02.09 -- Glukhov   ( Using ClearBlob,AddBlob fixed)
 02.03.09 -- Glukhov   ( added CO_FETCH_BOOL_AS_BOOL cursor option )
 05.03.09 -- Glukhov   ( added CO_FETCH_BLOBS_AS_STREAM cursor option )
 11.03.09 -- Glukhov   ( GetParamValue using removed )
 30.03.09 -- Glukhov   ( GENERATE_COL_NAME and RENAME_DUBBED_COL settings added )
 07.04.09 -- Glukhov   ( ClearCursorDesc and LinPHP_Execute fixed )
 07.04.09A-- Glukhov   ( PrepareColNames logic modified )
 07.04.09B-- Glukhov   ( Some defines for PHP3 where added)
 02.04.12 -- Galizin   ( Php 5.4 does not support php3 compatible)
 27.04.12 -- Medvedev  ( Correct headers for 5.x)
 27.11.12 -- Ermakov   ( var for length for all parameters must be allocated separately )
 26.08.13 -- Galizin   ( New macroses for types)
 29.08.13 -- Galizin   ( Building as internal PHP extension)
 06.08.14 -- Glukhov, Ermakov (64bit PHP changes)
 30.09.14 -- Fatkhudinov ( 64-bit php changes )
 14.10.14 -- Fatkhudinov ( returning long values on 32-bit machines changes )
 15.10.14 -- Fatkhudinov ( actions sequence in GetDataRow, linter_get_data_array reorginized, LDT_EXTFILE constant registration added )
 16.10.14 -- Fatkhudinov ( EXTFILE column type process added )
 --------------------------------------------------------------------------*/

#define __PHP_LINTER_C__

#include "php.h"

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/types.h>

#if defined (_MSC_VER) && (_MSC_VER >= 1400)                 /* 13.02.07 ... */
#include <sys/stat.h>
#include <sys/utime.h>
/*#include "msvc80.h"*/
#endif                                                       /* ... 13.02.07 */

#ifdef UNIX
#  include <unistd.h>
#endif

#ifdef _MOD_PHP3_H                                         /* 06.03.00 */
#  include "internal_functions.h"
#  define add_assoc_null(x,y)                              /* 29.03.08 ... */
#  define add_next_index_null(x)
#  define ZVAL_IS_NULL(x) 0                                /* ... 29.03.08 */
#  define MAKE_STD_ZVAL(x)
#  define ALLOC_INIT_ZVAL(x)     
#  define zval_dtor(x)
#  define FREE_ZVAL(x)
#  define add_index_zval(x,y,z)  _php3_hash_index_update((x)->value.ht, y, return_value, sizeof(pval), NULL)
#  define zval_copy_ctor         pval_copy_constructor
#  define zend_hash_index_find   _php3_hash_index_find 
#  define zend_module_entry      php3_module_entry
#  define zend_get_parameters    getParameters
#  define Z_TYPE(pval)		 (pval).type               /* 07.04.09B ... */
#  define Z_TYPE_P(pval_p)	 Z_TYPE(*pval_p)
#  define IS_BOOL                0x3                       
#  define zval                   pval                      
#  define TSRMLS_DC                                        
#  define TSRMLS_CC                                        /* ... 07.04.09B */
#  define zend_function_entry function_entry                /* 02.04.12 */
#  define zval pval                                         /* 02.04.12 */
                                                           /* 29.08.13 ... */
#define PHP_MINIT_FUNCTION(module) \
  int php3_##module##_Init(INIT_FUNC_ARGS)
#define PHP_MSHUTDOWN_FUNCTION(module) \
  int php3_##module##_Shutdown(SHUTDOWN_FUNC_ARGS)
#define PHP_RINIT_FUNCTION(module) \
  int php3_##module##_RInit(INIT_FUNC_ARGS)
#define PHP_RSHUTDOWN_FUNCTION(module) \
  int php3_##module##_RShutdown(SHUTDOWN_FUNC_ARGS)
#define PHP_MINFO_FUNCTION(module) \
  void php3_##module##_Info(void)
#define  PHP_MINIT(module) php3_##module##_Init
#define  PHP_MSHUTDOWN(module) php3_##module##_Shutdown
#define  PHP_RINIT(module) php3_##module##_RInit
#define  PHP_RSHUTDOWN(module) php3_##module##_RShutdown
#define  PHP_MINFO(module) php3_##module##_Info
#define  PHP_FN(function) php3_##function                  /* ... 29.08.13 */
#else
# if !defined(PHP4)                                        /* 31.07.02 */
#  define PHP4
# endif
# if PHP_API_VERSION < 20010119                            /* 29.03.08 ... */
#  define add_assoc_null(x,y)  add_assoc_unset(x,y)        
#  define add_next_index_null(x) add_next_index_unset(x)
#  define add_index_zval(x,y,z)  _php3_hash_index_update((x)->value.ht, y, z, sizeof(pval), NULL)
# endif                                                    /* ... 29.03.08 */
# if PHP_API_VERSION < 20010901
#  define TSRMLS_DC                                        
#  define TSRMLS_CC
# endif                                                    /* ... 29.03.08 */
#endif    /* _MOD_PHP3_H */

#ifdef PHP4
# include "ext/standard/info.h"                            /* 28.11.00 */
#endif

#if PHP_API_VERSION >= 20010119                            /* 27.09.04 ... */
# include "php_ini.h"
#endif                                                     /* ... 27.09.04 */

#ifdef PDO                                                 /* 22.01.09 ... */ 
# include "ext/pdo/php_pdo.h"             
# include "ext/pdo/php_pdo_driver.h"      
#endif                                                     /* ... 22.01.09 */

#include "inter.h"
#include "decimals.h"
#include "tick.h"

#include "php_linter.h"					   /* 12.03.00 */
#include "php_linter_int.h"                                /* 29.08.13 */
#include "tools.h"

#ifdef PDO                                                 /* 22.01.09... */
# include "php_pdo_linter.h"        
# include "php_pdo_linter_int.h"    
#endif                                                     /* ... 22.01.09 */

#define MODULE_NAME "linter"                                    /* 28.11.00 */

#ifdef USE_THREADS                                         /* 29.10.04 ... */
#  if PHP_API_VERSION >= 20020918 
#    include "linmutex.h"
#    include "TSRM/TSRM.h"
#    define MUTEX_INIT(a)    LINAPI_MutexInit(&a)
#    define MUTEX_DESTROY(a) LINAPI_MutexDestroy(&a)
#    define MUTEX_LOCK(a)    LINAPI_MutexLock(&a)
#    define MUTEX_UNLOCK(a)  LINAPI_MutexUnlock(&a)
#  else
#    error "Can't make threaded version PHP for this PHP version"
#  endif
#else
#  define MUTEX_LOCK(a)
#  define MUTEX_UNLOCK(a)
#  define MUTEX_INIT(a)   
#  define MUTEX_DESTROY(a)
#endif                                                     /* ... 29.10.04 */

extern void UninitLinterClient(void);                      /* 02.06.00 */

#ifdef USE_THREADS                                         /* 29.10.04 ... */       
typedef struct t_THREADBUF THREADBUF;                        
struct t_THREADBUF {
  THREAD_T   thread_id;
  char      *val;
  L_LONG    CodErr;
  L_LONG    SysErr;
  THREADBUF *next;
};                                                         
#endif                                                     /* ... 29.10.04 */

static CURSOR CsrList = {{0}};                                   /* 20.02.04 */
static const char * STR_SEP = " \t\n\r";
static L_ULONG max_id = 1;                                       /* 17.11.04 */

#define DEF_DT_FORMAT "DD.MM.YYYY HH:MI:SS"

extern int ConvertParameter(char *adr1, L_BYTE typ1, L_WORD len1, /* 26.08.13 ... */
                     char *adr2, L_BYTE typ2, L_WORD len2,L_LONG * RealLen,
                     L_BYTE islike, L_LONG CodePage);
extern int ConvertAnswer(char *adr1, L_BYTE typ1, L_WORD len1, L_BYTE Prec, L_BYTE Scale,
                     char *adr2, L_BYTE typ2, L_WORD len2,L_LONG * RealLen,
                     L_LONG CodePage);
extern int ConvertData(char *adr1, L_BYTE typ1, L_WORD len1, L_BYTE Prec, L_BYTE Scale,
                     char *adr2, L_BYTE typ2, L_WORD len2, L_LONG * RealLen,
                     L_BYTE islike,
                     L_BYTE fConvert, L_LONG CodePage);
extern int ConvertType(char *, L_BYTE, L_WORD, L_BYTE, L_BYTE,
                       char *, L_BYTE, L_WORD, L_LONG *, L_BYTE, L_BYTE, L_LONG); /* ... 26.08.13 */

#define DT_DEFAULT     -1                               

extern void inter_bind_num( TCBL *, char *, int, int, short *, char *);
extern void inter_bind_name( TCBL *, char *, char *, int, short *, char *);

#if _VER_MAX >= 600                                     
extern void inter_bind_num_cs( TCBL *, char *, int, int, short *, char *, short);
extern void inter_bind_name_cs( TCBL *, char *, char *, int, short *, char *, short);
extern void FillBufferSize( char* Range, L_WORD RLnBufRow ); /* 26.08.13 */
#endif                                                  

extern void inter_fill_parameters( TCBL *, char * );
extern int  inter_check_bind(char *, int, char *, short *);
extern int  inter_column_number(TCBL * Cbl,char *);
extern void inter_get_description(TCBL * Cbl,char *,char *);
extern int  inter_isselect(TCBL * Cbl,char * Query_Buff);
extern int  inter_get_blobnum(char *);
extern void inter_GetParamCount(char* QueryBuf, short* pParamCount);
extern void inter_DescribeParam( char*  QueryBuf,
                                 short  ParamNumber,
                                 short* pLength,
                                 short* pType,
                                 short* pPrecision,
                                 short* pScale,
                                 short* pNullable );
extern void inter_ExtDescParam(char*  QueryBuf,
                               short  ParamNumber,
                               short* pLength,
                               short* pType,
                               short* pPrecision,
                               short* pScale,
                               short* pNullable,
                               char*  pInput,
                               char*  pOutput,
                               short* pArgNum );
extern short inter_GetParamNumber(char * PreparedQuery, char * Name, short * Counter);
extern char* inter_GetParamName(char * PreparedQuery, short Number);
extern void inter_SetEncoding(char * QueryBuf, L_LONG Encoding);
extern void* inter_GetStartSet( char* QueryBuf, L_WORD nSet ); /* 26.08.13 ... */
extern void* inter_GetQuery( char* QueryBuf, L_WORD nQuery );
extern void* inter_GetMainQuery( char* QueryBuf );
extern void* inter_GetOperQuery( char* QueryBuf );
extern void* inter_GetExpr( char* QueryBuf, L_WORD nExpr );
extern void* inter_GetColInfo( char* QueryBuf, L_WORD nOper );
extern L_WORD inter_GetBlobColNum( char* QueryBuf, L_WORD nPar ); /* ... 26.08.13 */
extern void * inter_get_paradr(TCBL *CBL, char * buffer, L_WORD parnum);
extern L_LONG inter_GetProcType(TCBL * Cbl, char * Query_Buf, char * Out_Buf);


static L_CHAR *glDateFormat;
static L_BOOL glReturnDecimalAsDouble;                           /* 27.09.04 */
static L_BOOL glFetchBlobAsData;                                 /* 30.03.08 */
static L_BOOL glFetchBlobAsStream;                               /* 05.03.09 */
static L_BOOL glNullsAsNullObject;                               /* 30.03.08 */
static L_BOOL glFetchBoolAsBool;                                 /* 02.03.09 */
static L_BOOL glGenerateColName;                                 /* 30.03.09 */
static L_BOOL glRenameDubbedCol;                                 /* 30.03.09 */

#ifdef USE_THREADS                                                /* 29.10.04 ... */
  static THREADBUF* thread_buf = NULL; /* for thread specific data */
  static LINAPI_MUTEX mutex1;
#endif                                                            /* ... 29.10.04 */

PHP_FUNCTION(linter_open_connect);                         /* 29.08.13 ... */
PHP_FUNCTION(linter_connect_info);
PHP_FUNCTION(linter_close_connect);
PHP_FUNCTION(linter_open_cursor);
PHP_FUNCTION(linter_close_cursor);
PHP_FUNCTION(linter_get_cursor_opt);
PHP_FUNCTION(linter_set_cursor_opt);
PHP_FUNCTION(linter_exec_direct);
PHP_FUNCTION(linter_prepare);
PHP_FUNCTION(linter_execute);
PHP_FUNCTION(linter_fetch);
PHP_FUNCTION(linter_fetch_row);
PHP_FUNCTION(linter_fetch_array);
PHP_FUNCTION(linter_set_codepage);
PHP_FUNCTION(linter_get_data_row);
PHP_FUNCTION(linter_get_data_rows);
PHP_FUNCTION(linter_get_data_array);
PHP_FUNCTION(linter_getm);
PHP_FUNCTION(linter_get_col_prop);
PHP_FUNCTION(linter_get_param_prop);
PHP_FUNCTION(linter_blob_clear);
PHP_FUNCTION(linter_blob_append);
PHP_FUNCTION(linter_blob_append_ex);
PHP_FUNCTION(linter_blob_get_data);
PHP_FUNCTION(linter_blob_get_size);
PHP_FUNCTION(linter_blob_purge);
PHP_FUNCTION(linter_blob_add);
PHP_FUNCTION(linter_blob_add_ex);
PHP_FUNCTION(linter_blob_fetch);
PHP_FUNCTION(linter_last_error);
PHP_FUNCTION(linter_error_msg);


static PHP_MINIT_FUNCTION(linter);
static PHP_MSHUTDOWN_FUNCTION(linter);
static PHP_RINIT_FUNCTION(linter);
static PHP_RSHUTDOWN_FUNCTION(linter);
static PHP_MINFO_FUNCTION(linter);



#if !defined(PHP_FE_END)
#define PHP_FE_END {NULL, NULL, NULL}
#endif

zend_function_entry linter_functions[] = {                  /* 02.04.12 */
  PHP_FE(linter_open_connect,    NULL)
  PHP_FE(linter_connect_info,    NULL)
  PHP_FE(linter_close_connect,   NULL)
  PHP_FE(linter_open_cursor,     NULL)
  PHP_FE(linter_close_cursor,    NULL)
  PHP_FE(linter_get_cursor_opt,  NULL)
  PHP_FE(linter_set_cursor_opt,  NULL)
  PHP_FE(linter_exec_direct,     NULL)
  PHP_FE(linter_prepare,         NULL)
  PHP_FE(linter_execute,         NULL)
  PHP_FE(linter_fetch,           NULL)
  PHP_FE(linter_fetch_row,       NULL)                     /* 13.09.04 */
  PHP_FE(linter_fetch_array,     NULL)                     /* 13.09.04 */
  PHP_FE(linter_set_codepage,    NULL)                     /* 13.09.04 */
  PHP_FE(linter_get_data_row,    NULL)
  PHP_FE(linter_get_data_rows,   NULL)                     /* 02.10.00 */
  PHP_FE(linter_get_data_array,  NULL)                     /* 18.08.00 */
  PHP_FE(linter_getm,            NULL)                     /* 14.05.01 */
  PHP_FE(linter_get_col_prop,    NULL)
  PHP_FE(linter_get_param_prop,  NULL)                     /* 13.10.07 */
  PHP_FE(linter_blob_clear,      NULL)
  PHP_FE(linter_blob_append,     NULL)
  PHP_FE(linter_blob_append_ex,  NULL)                     /* 22.11.01 */
  PHP_FE(linter_blob_get_data,   NULL)
  PHP_FE(linter_blob_get_size,   NULL)
  PHP_FE(linter_blob_purge,      NULL)
  PHP_FE(linter_blob_add,        NULL)
  PHP_FE(linter_blob_add_ex,     NULL)                     /* 22.11.01 */
  PHP_FE(linter_blob_fetch,      NULL)
  PHP_FE(linter_last_error,      NULL)
  PHP_FE(linter_error_msg,       NULL)
  PHP_FE_END
};                                                         /* ... 29.08.13 */

#ifdef PDO
# if ZEND_EXTENSION_API_NO >= 220050617
    static zend_module_dep pdo_linter_deps[] = {
      ZEND_MOD_REQUIRED("pdo")
          {NULL, NULL, NULL}
        };
# endif
#endif

zend_module_entry linter_module_entry = {
#if ZEND_EXTENSION_API_NO >= 220050617                     /* 22.01.09... */
  STANDARD_MODULE_HEADER_EX, NULL,
# ifdef PDO
  pdo_linter_deps,
# endif
#else
# if ZEND_MODULE_API_NO >= 20010901
    STANDARD_MODULE_HEADER,
# endif
#endif                                                     /* ... 22.01.09 */
  MODULE_NAME,
  linter_functions,
  PHP_MINIT(linter),                                       /* 29.08.13 ... */
  PHP_MSHUTDOWN(linter),
  PHP_RINIT(linter),
  PHP_RSHUTDOWN(linter),
  PHP_MINFO(linter),                                       /* ... 29.08.13 */
#if ZEND_MODULE_API_NO >= 20010901
  NO_VERSION_YET,
#endif
  STANDARD_MODULE_PROPERTIES
};

char *ldt_names[] = {
  "CHAR", "INTEGER", "REAL", "DATE", "NUMERIC", "BYTE",
  "BLOB", "VARCHAR", "VARBYTE", "BOOLEAN", "NCHAR",        /* 07.12.00 */
  "NVARCHAR", "EXTFILE", "SMALLINT", "DOUBLE"              /* 15.05.03 */  /* 28.06.04 */
};

char* ldt_int_names[] = {"SMALLINT", "INTEGER", "", "BIGINT"};  /* 15.05.03 */
char* ldt_real_names[] = {"REAL", "DOUBLE"};                    /* 15.05.03 */

#ifdef USE_THREADS                                         /* 29.10.04 ... */

THREADBUF *GetBuf( THREAD_T thread_id )
{
  THREADBUF *buf,*last;

  MUTEX_LOCK( mutex1 );
  buf = last = thread_buf;
  while ( buf )
  {
    if ( buf->thread_id == thread_id )
    {
      MUTEX_UNLOCK(mutex1);
      return buf;
    }
    last = buf;
    buf = buf->next;
  }
  /* if not found, then create */
  buf = (THREADBUF *)calloc(1, sizeof(THREADBUF));
  if ( buf != NULL )
  {
    buf->thread_id = thread_id;
    buf->CodErr = 0;  /* 09.03.05 */
    buf->SysErr = 0;  /* 09.03.05 */
    buf->val = NULL;
    buf->next = NULL;
  }
  if (last)
    last->next = buf;
  else
    thread_buf = buf;
  MUTEX_UNLOCK(mutex1);
  return last;
}

void DelBuf( THREAD_T thread_id )
{
  THREADBUF *buf,*last;

  MUTEX_LOCK(mutex1);
  buf = last = thread_buf;
  while ( buf )
  {
    if ( buf->thread_id == thread_id )
    {
      if ( buf == thread_buf )
        thread_buf = buf->next;
      last->next = buf->next;
      if ( buf->val != NULL )
        free( buf->val );
      free( buf );
      break;
    }
    last = buf;
    buf = buf->next;
  }
  MUTEX_UNLOCK(mutex1);
}

#endif /* ifdef USE_THREADS*/                              /* ... 29.10.04 */

L_CHAR * GetLinterDateFormat()                             /* 22.01.09 ... */
{
  return glDateFormat;
}

static L_BOOL SetLinterDateFormat(const char * newFormat, L_BOOL Init)
{
  if (!Init && glDateFormat)
    free(glDateFormat);
  glDateFormat = strdup(newFormat);
  return 1;
}

static int GetFetchBlobAsDataFlag()
{
  return glFetchBlobAsData;
}

static L_BOOL SetFetchBlobAsDataFlag(L_BOOL newValue)
{
  glFetchBlobAsData = newValue;
  return 1;
}                                                          /* ... 22.01.09 */

static int GetFetchBlobAsStreamFlag()                      /* 05.03.09 ... */
{
  return glFetchBlobAsStream;
}

static L_BOOL SetFetchBlobAsStreamFlag(L_BOOL newValue)
{
  glFetchBlobAsStream = newValue;
  return 1;
}                                                          /* ... 05.03.09 */

static int GetFetchBoolAsBoolFlag()                        /* 02.03.09... */
{
  return glFetchBoolAsBool;
}

static L_BOOL SetFetchBoolAsBoolFlag(L_BOOL newValue)
{
  glFetchBoolAsBool = newValue;
  return 1;
}                                                          /* ... 02.03.09 */

int GetGenerateColNameFlag()                               /* 30.03.09 ... */
{
  return glGenerateColName;
}

static L_BOOL SetGenerateColNameFlag(L_BOOL newValue)
{
  glGenerateColName = newValue;
  return 1;
}

int GetRenameDubbedColFlag()
{
  return glRenameDubbedCol;
}

static L_BOOL SetRenameDubbedColFlag(L_BOOL newValue)
{
  glRenameDubbedCol = newValue;
  return 1;
}                                                          /* ... 30.03.09 */

void ClearCursorDesc(CURSOR *csr, int all)                 /* 24.06.08 ... 13.11.00 */
{
 if (all)
 {
    s_free(csr->dt_format);
    s_free(csr->parm_leng);
    s_free(csr->pqry_body);
    s_free(csr->getm_buff);
    s_free(csr->answ_desc);
    s_free(csr->ColOffset);                                /* 22.01.09 07.04.09 */
    csr->param_count = 0;                                  /* 13.10.07, 22.01.09 */
  }
  csr->pass_cnt = 0;
  csr->rows = csr->cols = 0;                               /* 18.02.00 */
  csr->FstRow = 0;                                         /* 28.07.05 */
  csr->LstRow = 0;                                         /* 14.05.01 */ /* 28.07.05 */ 
  csr->crow = 0;
  csr->errpos = 0;
}
                                                           /* ... 24.06.08  */
CURSOR *GetFromList(int n)
{
  CURSOR *csr;

  MUTEX_LOCK(mutex1);                                   /* 29.10.04 */
  csr = CsrList.next;
  if (n == 0)
  {
    MUTEX_UNLOCK(mutex1);                               /* 29.10.04 */
    return (NULL);
  }
  while(csr)
  {
    if (csr->id == (long)n)                             /* 17.11.00 */
      break;
    csr = csr->next;
  }
  MUTEX_UNLOCK(mutex1);                                 /* 29.10.04 */
  return (csr);
}

CURSOR *AddToList(CURSOR *c)
{
  CURSOR *csr = &CsrList;
  MUTEX_LOCK(mutex1);                                   /* 29.10.04 */
  while(csr->next)
    csr = csr->next;
  csr->next = (CURSOR *)calloc(1, sizeof(CURSOR));
  if (csr->next != NULL)
    {
    memcpy(csr->next, c, sizeof(CURSOR));
    csr->next->next = NULL;
    csr->next->dt_format = strdup(GetLinterDateFormat());     /* 13.11.00 */ /* 22.01.09 */
    csr->next->id = max_id++;
    csr->next->FetchBlobBehaviour = GetFetchBlobAsDataFlag(); /* 30.03.08 20.10.07 */ /* 22.01.09 */
    csr->next->FetchBlobStreamBehaviour = GetFetchBlobAsStreamFlag(); /* 05.0.09 */
    csr->next->FetchBoolBehaviour = GetFetchBoolAsBoolFlag(); /* 02.03.09 */
    csr->next->NullsBehaviour = glNullsAsNullObject;          /* 29.03.08 */
    csr->next->DecimalBehaviour = glReturnDecimalAsDouble;    /* 30.03.08 */
    csr->next->GenerateColNameBehaviour = GetGenerateColNameFlag(); /* 30.03.09 */
    csr->next->RenameDubbedColBehaviour = GetRenameDubbedColFlag(); /* 30.03.09 */
    /* When ulong overflow */
    if (!csr->next->id)                                 /* 17.11.04 */
      csr->next->id = max_id++;
#ifdef USE_THREADS                                      /* 29.10.04 */
    csr->next->thread_id = tsrm_thread_id();
#endif
    }
  MUTEX_UNLOCK(mutex1);                                 /* 29.10.04 */
  return (csr->next);
}

void DelFromList(CURSOR *c)
{
  CURSOR *csr = &CsrList;
  if (c == NULL)
    return;

  MUTEX_LOCK(mutex1);                                   /* 29.10.04 */
  while(csr)
  {
    if (csr->next == c)
    {
      csr->next = c->next;
      break;
    }
    csr = csr->next;
  }
  ClearCursorDesc(c, 1);                                /* 13.11.00 */
  s_free(c);
  MUTEX_UNLOCK(mutex1);                                 /* 29.10.04 */
}

int SetUniqueColName(CURSOR *csr, int colNum)                                                  /* 07.04.09A ... */
{
  int i,dubbedCounter;
  L_BOOL setFlag                   = L_TFALSE;
  char ColName[MAX_ID_LEN + 1]     = "";
  char tempColName[MAX_ID_LEN + 1] = "";
  char tempStr[MAX_ID_LEN+1];

  memcpy(tempColName,csr->answ_desc[colNum].Column,min(strlen(csr->answ_desc[colNum].Column),MAX_ID_LEN));

  while (!setFlag)
  {
    dubbedCounter = 1;
    snprintf(ColName,MAX_ID_LEN+1,"%s_%u",tempColName, dubbedCounter);
    while (!setFlag && (strlen(tempColName)+snprintf(tempStr,MAX_ID_LEN,"%u",dubbedCounter)+1)<=MAX_ID_LEN)
    {
      setFlag = L_TTRUE;
      for (i = 0; i < colNum; i++)
      {
        if (strncmp(csr->answ_desc[i].Column,ColName,strlen(ColName))==0)
          setFlag = L_TFALSE;
      }
      if (!setFlag)
        snprintf(ColName,MAX_ID_LEN+1,"%s_%u",tempColName, ++dubbedCounter);
    }
    if (!setFlag)
    {
      if (strlen(tempColName))
        tempColName[strlen(tempColName)-1] = 0;
      else
        setFlag = L_TTRUE; /* return the last proccessed variant (dubbed, that's why this block must be FIXED), its hardly probable */
    }
  }
  memset(csr->answ_desc[colNum].Column,0,MAX_ID_LEN);
  memcpy(csr->answ_desc[colNum].Column,ColName,strlen(ColName));
  return 0;
}

static int PrepareColNames(CURSOR *csr)
{
  int i,j,k;

  for (i = 0; i < csr->cols; i++)
  {
    k = MAX_ID_LEN-1;
    while (strrchr(" ", csr->answ_desc[i].Column[k]) && k >= 0)
      csr->answ_desc[i].Column[k--] = 0;

    if (csr->GenerateColNameBehaviour)
    {
      // check column names for empty names
      if (strlen(csr->answ_desc[i].Column) == 0)
        strncpy(csr->answ_desc[i].Column,"AUTO_GENERATED_NAME",MAX_ID_LEN);
    }
  }

  if (csr->RenameDubbedColBehaviour)
  {
    // check column names for dubbed
    for (i = 1; i < csr->cols; i++)
    {
      j = 0;
      while ((j < i) && strcmp(csr->answ_desc[i].Column,csr->answ_desc[j].Column)!=0)
        j++;
      if (j < i)
        SetUniqueColName(csr, i);
    }
  }

  return 1;
}                                                                                                    /* ... 07.04.09A */

long FillCursorDesc(CURSOR *csr, int DontFree)
{
  TCBL RCBL;
  int  cols, delta = PAGE_BUFF / sizeof(GETA_OUT);
  int  i;                                               /* 14.05.01 */

  ClearCursorDesc(csr, 0);                              /* 13.11.00 */
  csr->rows = csr->CBL.RowCount;
  memcpy(&RCBL, &csr->CBL, sizeof(TCBL));
  csr->CBL.RowId = 0;                                   /* 28.03.08A ... */

  csr->CBL.LnBufRow = PAGE_BUFF;
  csr->answ_desc = (GETA_OUT *)realloc(csr->answ_desc,PAGE_BUFF);  /* 24.06.08  */
  strncpy((char *)csr->CBL.Command, "GETA", 4);
  inter(&csr->CBL, NULL, NULL, NULL, csr->answ_desc);
  cols = csr->cols = csr->CBL.RowCount;
  csr->ColOffset = (long *)calloc(cols,sizeof(long));

  switch (csr->CBL.CodErr)
  {
    case SMALLBUFKOR:
            csr->answ_desc = (GETA_OUT *)realloc(csr->answ_desc, cols * sizeof(GETA_OUT));
            csr->CBL.CodErr = 0;
            csr->CBL.RowId = 0; /* Start from begin */
            break;
    case 0:
            if (cols > delta)                                /* 28.04.08 ... */
              csr->answ_desc = (GETA_OUT *)realloc(csr->answ_desc, cols * sizeof(GETA_OUT));
            else                                             /* ... 28.04.08 */
              cols = 0; /* Don't need next reading */
            break;
    default:
            i = csr->CBL.CodErr;
            memcpy(&csr->CBL, &RCBL, sizeof(TCBL));
            return (i);
  }

  if (!csr->answ_desc)
  {
    memcpy(&csr->CBL, &RCBL, sizeof(TCBL));
    return (LPE_NO_MEMORY);
  }

  while (cols > 0)                                               /* 28.04.08 */
  {
    csr->CBL.LnBufRow = PAGE_BUFF;
    inter(&csr->CBL, NULL, NULL, NULL, &csr->answ_desc[csr->CBL.RowId]);
    if (csr->CBL.CodErr != 0)
    {
      memcpy(&csr->CBL, &RCBL, sizeof(TCBL));
      return (csr->CBL.CodErr);
    }
    csr->CBL.RowId += delta;
    cols           -= delta; /* cols can set to negative value */
  }

  csr->answ_len = 0;                                          /* ... 28.03.08A */
  for (i = 0; i < csr->cols; i++)                             /* 14.05.01 ...*/
  {
    csr->ColOffset[i] = csr->answ_len;                        /* 22.01.09 */
    csr->answ_len   += csr->answ_desc[i].Length;
    if (csr->answ_desc[i].Type == LDT_VARCHAR ||
        csr->answ_desc[i].Type == LDT_VARBYTE ||
        csr->answ_desc[i].Type == LDT_NVARCHAR)
      csr->answ_len += sizeof(L_WORD);
  }                                                           /*... 14.05.01 */
  memcpy(&csr->CBL, &RCBL, sizeof(TCBL));
  PrepareColNames(csr);                                       /* 30.03.09 */
  return (LPE_SUCCESS);
}

long FetchCursor(CURSOR *csr, long direct, long pos, long DontCall)
{
  switch(direct)
  {
    case FETCH_FIRST:
      strncpy((char *)csr->CBL.Command, "GETF", 4);
      csr->crow = 1;
      break;
    case FETCH_LAST:
      strncpy((char *)csr->CBL.Command, "GETL", 4);
      csr->crow = csr->rows;
      break;
    case FETCH_NEXT:
      strncpy((char *)csr->CBL.Command, "GETN", 4);
      if (csr->crow >= csr->rows)                           /* 28.03.08B ... */
      {
        csr->CBL.CodErr = EORR;
        return (LPE_LINTER_ERROR);
      }
      csr->crow++;                                          /* ... 28.03.08B */
      break;
    case FETCH_PREV:
      strncpy((char *)csr->CBL.Command, "GETP", 4);
      csr->crow--;
      break;
    case FETCH_ABSNUM:
      strncpy((char *)csr->CBL.Command, "GETS", 4);
      csr->CBL.RowId = pos;
      break;
    default:
      return (LPE_INVALID_DIRECT);
  }
  if (!DontCall)                                             /* 27.02.08 */
  {
    csr->CBL.LnBufRow = MAX_ANSWER;                          /* 24.06.08  */ /* 31.05.02 */
    inter(&csr->CBL, csr->null_flag, NULL, NULL, csr->exch_buff);
  }
  if (csr->CBL.CodErr != 0)
    return (LPE_LINTER_ERROR);
  if (direct == FETCH_ABSNUM)
    csr->crow = pos;
  return (LPE_SUCCESS);
}

L_BOOL ClearBlob(CURSOR *csr, L_LONG column)                 /* 01.03.08 ... */
{
  strncpy(csr->CBL.Command, "CBLB", 4);
  csr->CBL.RowCount = column;
  inter(&csr->CBL, NULL, NULL, NULL, NULL);
  if (csr->CBL.CodErr != 0)
    return(L_TFALSE);
  return(L_TTRUE);
}                                                            /* ... 01.03.08 */
/* 22.11.01 ...*/
L_BOOL AddBlob(CURSOR *csr, char* data, L_LONG blob_size, L_LONG blob_type, L_LONG colnum)
{
  L_LONG    psize, sent = 0;
  L_LONG    full_size;

  full_size = blob_size;
  csr->CBL.RowCount = colnum;
  strncpy(csr->CBL.Command, "ABLB", 4);
  csr->CBL.RowId = blob_type;
  do
  {
    psize = min(OPTIMUM_PORTION_SIZE, blob_size);
    csr->CBL.LnBufRow = (unsigned short)psize;
    inter(&csr->CBL, NULL, NULL, NULL, &data[sent]);
    if (csr->CBL.CodErr != 0)
      return(L_TFALSE);
    sent += psize;
    blob_size -= psize;
  } while(sent < full_size);
  return(L_TTRUE);
} /* AddBlob */
/*... 22.11.01 */

L_LONG FetchBlob(CURSOR *csr, L_LONG blob_start, L_LONG *blob_size, L_LONG colnum, char *blob_value)     /* 20.10.07 */
{
  char   data[OPTIMUM_PORTION_SIZE];
  L_LONG got = 0;

  strncpy(csr->CBL.Command, "GBLB", 4);
  csr->CBL.RowCount = colnum;

  do
  {
    csr->CBL.LnBufRow = (L_WORD)min(OPTIMUM_PORTION_SIZE, *blob_size);
    csr->CBL.RowId    = blob_start + got;
    inter(&csr->CBL, NULL, NULL, NULL, data);
    if (csr->CBL.CodErr != 0)
      break;
    memcpy(&blob_value[got], data, csr->CBL.LnBufRow);
    got += csr->CBL.LnBufRow;
  } while(got < *blob_size);

  *blob_size = got;

  if (csr->CBL.CodErr == EORR)
    csr->CBL.CodErr = 0;

  if (csr->CBL.CodErr == 0)
    return LPE_SUCCESS;

  return LPE_LINTER_ERROR;
} /* FetchBlob */


L_LONG GetBlobSize(CURSOR *csr, L_LONG colnum, L_LONG* blob_size)       /* 20.10.07 */
{
  BLOB_ATR *desc;
  L_LONG    i, off = 0;

  for(i = 0; i < csr->cols; i++)
  {
    if((i+1) == colnum)
    {
      if (csr->answ_desc[i].Type == LDT_BLOB)
        break;
      else
        return LPE_NO_BLOB_COLUMN;
    }
    off += csr->answ_desc[i].Length;
    if (csr->answ_desc[i].Type == LDT_VARCHAR ||           /* 01.06.00 */
        csr->answ_desc[i].Type == LDT_VARBYTE ||
        csr->answ_desc[i].Type == LDT_NVARCHAR )           /* 28.06.04 */
      off += sizeof(L_WORD);
  }
  if (i >= csr->cols)
    return LPE_NO_BLOB_COLUMN;

  desc = (BLOB_ATR *)&csr->exch_buff[off];
  *blob_size = desc->Size;

  return LPE_SUCCESS;
} /* GetBlobSize */

int GetConnectInfo(CURSOR *csr, zval *return_value)
{
  if (!csr->DbDscLen)
  {
    strncpy((char *)csr->CBL.Command, "DESC", 4);             /* 01.02.06 ... */
    csr->CBL.LnBufRow = sizeof(csr->DbDsc);
    inter(&csr->CBL, NULL, NULL, NULL, &csr->DbDsc);
    if (csr->CBL.CodErr != 0)
      csr->DbDscLen = 0;
    else
      csr->DbDscLen = csr->CBL.LnBufRow;                      /* ... 01.02.06 */ 
  }

  array_init(return_value);

  /* from DB_DESC */
  if ( (char *)&(csr->DbDsc) - (char *)&(csr->DbDsc.VerMajor) < csr->DbDscLen )
    add_assoc_long(return_value, "VERMAJOR", csr->DbDsc.VerMajor);
  if ( (char *)&(csr->DbDsc) - (char *)&(csr->DbDsc.VerMinor) < csr->DbDscLen )
    add_assoc_long(return_value, "VERMINOR", csr->DbDsc.VerMinor);
  if ( (char *)&(csr->DbDsc) - (char *)&(csr->DbDsc.VerBuild) < csr->DbDscLen )
    add_assoc_long(return_value, "VERBUILD", csr->DbDsc.VerBuild);
  if ( (char *)&(csr->DbDsc) - (char *)&(csr->DbDsc.SortPoolSize) < csr->DbDscLen )
    add_assoc_long(return_value, "SORTPOOLSIZE", csr->DbDsc.SortPoolSize);
  if ( (char *)&(csr->DbDsc) - (char *)&(csr->DbDsc.KernelPoolSize) < csr->DbDscLen )
    add_assoc_long(return_value, "KERNELPOOLSIZE", csr->DbDsc.KernelPoolSize);
  if ( (char *)&(csr->DbDsc) - (char *)&(csr->DbDsc.FileQueueSize) < csr->DbDscLen )
    add_assoc_long(return_value, "FILEQUEUESIZE", csr->DbDsc.FileQueueSize);
  if ( (char *)&(csr->DbDsc) - (char *)&(csr->DbDsc.UserQueueSize) < csr->DbDscLen )
    add_assoc_long(return_value, "USERQUEUESIZE", csr->DbDsc.UserQueueSize);
  if ( (char *)&(csr->DbDsc) - (char *)&(csr->DbDsc.TableQueueSize) < csr->DbDscLen )
    add_assoc_long(return_value, "TABLEQUEUESIZE", csr->DbDsc.TableQueueSize);
  if ( (char *)&(csr->DbDsc) - (char *)&(csr->DbDsc.ColumnQueueSize) < csr->DbDscLen )
    add_assoc_long(return_value, "COLUMNQUEUESIZE", csr->DbDsc.ColumnQueueSize);
  if ( (char *)&(csr->DbDsc) - (char *)&(csr->DbDsc.ChannelQueueSize) < csr->DbDscLen )
    add_assoc_long(return_value, "CHANNELQUEUESIZE", csr->DbDsc.ChannelQueueSize);
  if ( (char *)&(csr->DbDsc) - (char *)&(csr->DbDsc.SnapTimeout) < csr->DbDscLen )
    add_assoc_long(return_value, "SNAPTIMEOUT", csr->DbDsc.SnapTimeout);
  if ( (char *)&(csr->DbDsc) - (char *)&(csr->DbDsc.KillTimeout) < csr->DbDscLen )
    add_assoc_long(return_value, "KILLTIMEOUT", csr->DbDsc.KillTimeout);
  if ( (char *)&(csr->DbDsc) - (char *)&(csr->DbDsc.BaseName) < csr->DbDscLen )
    add_assoc_stringl(return_value, "BASENAME", csr->DbDsc.BaseName, sizeof(csr->DbDsc.BaseName), L_TTRUE);
  if ( (char *)&(csr->DbDsc) - (char *)&(csr->DbDsc.SysLog) < csr->DbDscLen )
    add_assoc_long(return_value, "SYSLOG", csr->DbDsc.SysLog);
  if ( (char *)&(csr->DbDsc) - (char *)&(csr->DbDsc.Sync) < csr->DbDscLen )
    add_assoc_long(return_value, "SYNC", csr->DbDsc.Sync);
  if ( (char *)&(csr->DbDsc) - (char *)&(csr->DbDsc.Log) < csr->DbDscLen )
    add_assoc_long(return_value, "LOG", csr->DbDsc.Log);
  if ( (char *)&(csr->DbDsc) - (char *)&(csr->DbDsc.Os) < csr->DbDscLen )
    add_assoc_long(return_value, "OS", csr->DbDsc.Os);
  if ( (char *)&(csr->DbDsc) - (char *)&(csr->DbDsc.NumOfSort) < csr->DbDscLen )
    add_assoc_long(return_value, "NUMOFSORT", csr->DbDsc.NumOfSort);
  /* from OPEN_DESC */
  if ( (char *)&(csr->Dsc) - (char *)&(csr->Dsc.Flags) < csr->DscLen )
  {
#if _VER_MAX >= 600
    add_assoc_long(return_value, "FLAGS_CSETCHG",(csr->Dsc.Flags & OPENDESC_CSET_CHANGED)?1:0);
#endif
    add_assoc_long(return_value, "FLAGS_KERNELENGLOCALE",(csr->DbDsc.Flags & KERNEL_ENG_LOCALE)?1:0);
    add_assoc_long(return_value, "FLAGS_EXTPASS",(csr->Dsc.Flags & OPENDESC_PASS_EXTERNAL)?1:0);
    add_assoc_long(return_value, "FLAGS_PASSEND",(csr->Dsc.Flags & OPENDESC_PASSLIFEEND)?1:0);
    add_assoc_long(return_value, "FLAGS_KERNELINVBYTEORDER",(csr->DbDsc.Flags & KERNEL_INVBYTEORD)?1:0);
    add_assoc_long(return_value, "FLAGS_KERNELDEMOLIC",(csr->DbDsc.Flags & KERNEL_DEMOLIC)?1:0);
    add_assoc_long(return_value, "FLAGS_KERNELDEMOLICEXP",(csr->DbDsc.Flags & KERNEL_DEMOLICEXP)?1:0);
  }

#if _VER_MAX >= 600
  if ( (char *)&(csr->DbDsc) - (char *)&(csr->DbDsc.MaxRecSize) < csr->DbDscLen )
    add_assoc_long(return_value, "MAXRECSIZE", csr->DbDsc.MaxRecSize);
  if ( (char *)&(csr->DbDsc) - (char *)&(csr->DbDsc.CharSet) < csr->DbDscLen )
    add_assoc_long(return_value, "CHARSET", csr->DbDsc.CharSet);

  if ( (char *)&(csr->Dsc) - (char *)&(csr->Dsc.DefCharSet) < csr->DscLen )
    add_assoc_long(return_value, "DEFCHARSET", csr->Dsc.DefCharSet);
  if ( (char *)&(csr->Dsc) - (char *)&(csr->Dsc.UseCharSet) < csr->DscLen )
    add_assoc_long(return_value, "USECHARSET", csr->Dsc.UseCharSet);
  if ( (char *)&(csr->Dsc) - (char *)&(csr->Dsc.UseCharSetName) < csr->DscLen )
    add_assoc_stringl(return_value, "USECHARSETNAME", csr->Dsc.UseCharSetName, sizeof(csr->Dsc.UseCharSetName), L_TTRUE);
#endif  

  return 1;
}

long GetDataColInteger(CURSOR *csr, long i, long off, char *str_val)
{
  long l_val;

  if (csr->answ_desc[i].Length == 4)
  {
#if !defined(LONG_ALIGN)
    l_val = (long)(*(L_LONG *)(&csr->exch_buff[off]));
#else                                                                       /* 14.05.03 ... */
    L_LONG   Val;

    memcpy((char *)&Val, (char *)(&csr->exch_buff[off]), sizeof(L_LONG));
    l_val = (long)Val;
#endif                                                                      /* ... 14.05.03 */
  }
  else if (csr->answ_desc[i].Length == 8)
  {
#if !defined(LONG_ALIGN)
    if (str_val != NULL)                                                    /* 14.10.14 */
        str_val = i64_ltoa((*(L_DLONG *)(&csr->exch_buff[off])), str_val);
#ifdef __64BIT_ARCH__
    l_val = (*(L_DLONG *)(&csr->exch_buff[off]));                           /* 06.08.14 */
#else
    l_val = i64_GetLo((*(L_DLONG *)(&csr->exch_buff[off])));
#endif
#else                                                                       /* 14.05.03 ... */
    L_DLONG Val;

    memcpy(&Val, &csr->exch_buff[off], sizeof(L_DLONG));
    if (str_val != NULL)                                                 /* 14.10.14 */
        str_val = i64_ltoa(&Val, str_val);
#ifdef __64BIT_ARCH__
    l_val = Val;                                                            /* 06.08.14 */
#else    
    l_val = i64_GetLo(Val);
#endif
#endif                                                                      /* ... 14.05.03 */
  }
  else
  {
#if !defined(LONG_ALIGN)
    l_val = (long)(*(L_SWORD *)(&csr->exch_buff[off]));                     /* 08.10.04 */
#else                                                                       /* 14.05.03 ... */
    L_SWORD Val;                                                            /* 08.10.04 */

    memcpy(&Val, &csr->exch_buff[off], sizeof(L_SWORD));
    l_val = (long)Val;
#endif                                                                      /* ... 14.05.03 */
  }
  return l_val;
}

double GetDataColReal(CURSOR *csr, long i, long off)
{
  double d_val;

  if (csr->answ_desc[i].Length == 4)
#if !defined(LONG_ALIGN)
    d_val = (double)(*(L_REAL   *)(&csr->exch_buff[off]));
#else                                                                        /* 14.05.03 ... */
  {
    L_REAL Val;

    memcpy(&Val, &csr->exch_buff[off], sizeof(L_REAL));
    d_val = (double)Val;
  }
#endif                                                                       /* ... 14.05.03 */
  else
#if !defined(LONG_ALIGN)
    d_val = (double)(*(L_DOUBLE *)(&csr->exch_buff[off]));
#else                                                                        /* 14.05.03 ... */
  {
    L_DOUBLE Val;

    memcpy(&Val, &csr->exch_buff[off], sizeof(L_DOUBLE));
    d_val = (double)Val;
  }
#endif                                                                       /* ... 14.05.03 */
  return d_val;
}

int GetTypeName(long type, long length, char *type_name, long max_name_len)  /* 12.02.09 ... */
{
  if (type == LDT_INTEGER)
    strncpy(type_name,ldt_int_names[length / 2 - 1], max_name_len);          /* 15.05.03 */
  else
  if (type == LDT_REAL )
    strncpy(type_name, ldt_real_names[length / 4 -1], max_name_len);         /* 15.05.03 */
  else
    strcpy(type_name, ldt_names[type - 1]);
  return LPE_SUCCESS;
}                                                                            /*... 12.02.09 */                                                                             

PHP_FUNCTION(linter_open_connect)
{
  char    user[MAX_ID_LEN*2];                              /* 17.11.04 */
  CURSOR *csr = &CsrList;
  CURSOR *nxt;                                             /* 29.03.00 */
  zval   *usr, *pwd, *node, *mode;                          /* 02.04.12 */
#ifdef USE_THREADS                                         /* 09.03.05 ... */
  THREADBUF *buf = GetBuf(tsrm_thread_id());
#endif                                                     /* ... 09.03.05 */
  if (ARG_COUNT(ht) != 4)
    WRONG_PARAM_COUNT;

  MUTEX_LOCK(mutex1);                                      /* 29.10.04 */
  nxt = CsrList.next;
  memset(csr, 0, sizeof(CURSOR));
  CsrList.next = nxt;                                      /* 29.03.00 */
  zend_get_parameters(ht, 4, &usr, &pwd, &node, &mode);
  convert_to_string(usr);
  convert_to_string(pwd);
  convert_to_string(node);
  convert_to_long(mode);
  snprintf(user, sizeof(user), "\"%s\"/\"%s\"",
           usr->value.str.val, pwd->value.str.val);
  strncpy((char *)csr->CBL.Node, node->value.str.val, 8);

  csr->Flags = csr->CBL.PrzExe = mode->value.lval;          /* 05.11.04 */
  strncpy((char *)csr->CBL.Command, "OPEN", 4);
  csr->CBL.LnBufRow = sizeof(csr->Dsc);                     /* 01.02.05 ... */
  inter(&csr->CBL, user, NULL, NULL, &csr->Dsc);
  csr->DscLen = csr->CBL.LnBufRow;                          /* ... 01.02.05 */

  if (csr->CBL.CodErr != 0)
  {
#ifdef USE_THREADS                                          /* 09.03.05 ... */
    buf->CodErr = csr->CBL.CodErr;
    buf->SysErr = csr->CBL.SysErr;
#endif                                                      /* ... 09.03.05 */
    MUTEX_UNLOCK(mutex1);                                   /* 29.10.04 */
    RETURN_LONG(LPE_LINTER_ERROR);
  }

/* 21.03.02 ...*/
  if ( PhpCheckDemoLicense (&csr->CBL, &csr->Dsc) < 0 )     /* 08.04.02 */
  {
    strncpy((char *)csr->CBL.Command, "CLOS", 4);
    inter(&csr->CBL, NULL, NULL, NULL, NULL);
    MUTEX_UNLOCK(mutex1);                                   /* 29.10.04 */
    RETURN_LONG(LPE_LINTER_ERROR);
  }
/*... 21.03.02 */

  if ((csr = AddToList(csr)) == NULL)
  {
    MUTEX_UNLOCK(mutex1);                                   /* 29.10.04 */
    RETURN_LONG(LPE_NO_MEMORY);
  }
  MUTEX_UNLOCK(mutex1);                                     /* 29.10.04 */
  RETURN_LONG(csr->id);                                     /* 17.11.00 */
}

PHP_FUNCTION(linter_connect_info)                           /* 01.02.06 */
{
  CURSOR *csr;
  zval   *cid;                                              /* 02.04.12 */

  if (ARG_COUNT(ht) < 1)
    WRONG_PARAM_COUNT;
  zend_get_parameters(ht, 1, &cid);
  convert_to_long(cid);
  csr = GetFromList(cid->value.lval);
  if (csr == NULL)
    RETURN_LONG(LPE_INVALID_CURSOR);

  GetConnectInfo(csr, return_value);                      /* 22.01.09 */
}

PHP_FUNCTION(linter_close_connect)
{
  zval   *cid;                                              /* 02.04.12 */
  CURSOR *con, *csr, *csrNext;

  if (ARG_COUNT(ht) != 1)
    WRONG_PARAM_COUNT;
  zend_get_parameters(ht, 1, &cid);
  convert_to_long(cid);
  con = GetFromList(cid->value.lval);
  if (con == NULL)
    RETURN_LONG(LPE_INVALID_CONNECT);
  strncpy((char *)con->CBL.Command, "CLOS", 4);
  inter(&con->CBL, NULL, NULL, NULL, NULL);
  MUTEX_LOCK(mutex1);                                   /* 29.10.04 */
  memcpy(&CsrList, con, sizeof(TCBL));
  csr = CsrList.next;
  while(csr != NULL)
  {
    csrNext = csr->next;                                /* 20.12.99 */
    if (csr->parent == con)
      DelFromList(csr);
    csr = csrNext;                                      /* 20.12.99 */
  }
  DelFromList(con);
  if (CsrList.CBL.CodErr != 0)
  {
    MUTEX_UNLOCK(mutex1);                               /* 29.10.04 */
    RETURN_LONG(LPE_LINTER_ERROR);
  }
  MUTEX_UNLOCK(mutex1);                               /* 29.10.04 */
  RETURN_LONG(LPE_SUCCESS);
}

PHP_FUNCTION(linter_open_cursor)
{
  zval   *cid;                                              /* 02.04.12 */
  CURSOR *con, *csr = &CsrList;
#ifdef USE_THREADS                                         /* 09.03.05 ... */
  THREADBUF *buf = GetBuf(tsrm_thread_id());
#endif                                                     /* ... 09.03.05 */
  if (ARG_COUNT(ht) != 1)
    WRONG_PARAM_COUNT;
  zend_get_parameters(ht, 1, &cid);
  convert_to_long(cid);
  con = GetFromList(cid->value.lval);
  if (con == NULL)
    RETURN_LONG(LPE_INVALID_CONNECT);
  MUTEX_LOCK(mutex1);                                      /* 29.10.04 */

  if (con->parent != NULL)                                 /* 13.01.05 */
    con->Flags = con->parent->Flags;                       /* 05.11.04 */
  else
    con->Flags = con->CBL.PrzExe;                          /* 13.01.05 */

  memcpy(&csr->CBL, &con->CBL, sizeof(TCBL));
  strncpy((char *)csr->CBL.Command, "OCUR", 4);
  inter(&csr->CBL, NULL, NULL, NULL, NULL);
  if (csr->CBL.CodErr != 0)
  {
#ifdef USE_THREADS                                         /* 09.03.05 ... */
    buf->CodErr = csr->CBL.CodErr;
    buf->SysErr = csr->CBL.SysErr;
#endif                                                     /* ... 09.03.05 */
    MUTEX_UNLOCK(mutex1);                                  /* 29.10.04 */
    RETURN_LONG(LPE_LINTER_ERROR);
  }
  if ((csr = AddToList(csr)) == NULL)                      /* 06.06.00 */
  {
    MUTEX_UNLOCK(mutex1);                                  /* 29.10.04 */
    RETURN_LONG(LPE_NO_MEMORY);
  }
  csr->parent = con;
  csr->FetchBlobBehaviour = con->FetchBlobBehaviour;       /* 30.03.08 */
  csr->FetchBlobStreamBehaviour = con->FetchBlobStreamBehaviour; /* 05.03.09 */
  csr->FetchBoolBehaviour = con->FetchBoolBehaviour;       /* 02.03.09 */
  csr->NullsBehaviour = con->NullsBehaviour;               /* 30.03.08 */
  csr->DecimalBehaviour = con->DecimalBehaviour;           /* 30.03.08 */
  csr->GenerateColNameBehaviour = con->GenerateColNameBehaviour; /* 30.03.09 */
  csr->RenameDubbedColBehaviour = con->RenameDubbedColBehaviour; /* 30.03.09 */
  s_free(csr->dt_format);
  csr->dt_format = strdup(con->dt_format);

  MUTEX_UNLOCK(mutex1);                                    /* 29.10.04 */
  RETURN_LONG(csr->id);                                    /* 17.11.00 */
}

PHP_FUNCTION(linter_close_cursor)
{
  CURSOR *csr;
  zval   *cid;                                              /* 02.04.12 */

  if (ARG_COUNT(ht) != 1)
    WRONG_PARAM_COUNT;
  zend_get_parameters(ht, 1, &cid);
  convert_to_long(cid);
  csr = GetFromList(cid->value.lval);
  if (csr == NULL)
    RETURN_LONG(LPE_INVALID_CURSOR);
  strncpy((char *)csr->CBL.Command, "CLOS", 4);
  inter(&csr->CBL, NULL, NULL, NULL, NULL);
  MUTEX_LOCK(mutex1);                                    /* 29.10.04 */
  memcpy(&CsrList, csr, sizeof(TCBL));
  DelFromList(csr);
  if (CsrList.CBL.CodErr != 0)
  {
    MUTEX_UNLOCK(mutex1);                                /* 29.10.04 */
    RETURN_LONG(LPE_LINTER_ERROR);
  }
  MUTEX_UNLOCK(mutex1);                                  /* 29.10.04 */
  RETURN_LONG(LPE_SUCCESS);
}

PHP_FUNCTION(linter_get_cursor_opt)
{
  CURSOR *csr;
  zval   *cid, *opt;                                        /* 02.04.12 */
  FCUR_OUT CursorDesc;                                           /* 04.03.04 */

  if (ARG_COUNT(ht) != 2)
    WRONG_PARAM_COUNT;
  zend_get_parameters(ht, 2, &cid, &opt);
  convert_to_long(cid);
  csr = GetFromList(cid->value.lval);

  if (csr == NULL)
    RETURN_LONG(LPE_INVALID_CURSOR);
  convert_to_long(opt);
  switch(opt->value.lval)
  {
    case CO_NAME:
      RETURN_STRING(csr->name, L_TTRUE);
    case CO_DT_FORMAT:
      RETURN_STRING(csr->dt_format, L_TTRUE);
    case CO_COL_COUNT:
      RETURN_LONG(csr->cols);
    case CO_ROW_COUNT:
      RETURN_LONG(csr->rows);
    case CO_ERR_ROW:
      RETURN_LONG(csr->errpos & 0x0000FFFF);
    case CO_ERR_POS:
      RETURN_LONG((csr->errpos & 0xFFFF0000) >> 16);
    case CO_TRANS_MODE:
      if (csr->CBL.PrzExe & TM_OPTIMISTIC)
        RETURN_LONG(TM_OPTIMISTIC);
      if (csr->CBL.PrzExe & TM_EXCLUSIVE)
        RETURN_LONG(TM_EXCLUSIVE);
      RETURN_LONG(TM_AUTOCOMMIT);
    case CO_CUR_ROW:
      RETURN_LONG(csr->crow);
    case CO_CUR_ROWID:
      RETURN_LONG(csr->CBL.RowId);
    case CO_CONNECT_ID:
      if (csr->parent)
        RETURN_LONG((long)csr->parent->CBL.NumChan);
      RETURN_LONG(LPE_NOT_CURSOR);
    case CO_NODE_NAME:
      RETURN_STRINGL(csr->CBL.Node, 8, L_TTRUE);                /* 11.10.07 */
    case CO_LAST_ROWID:                                      /* 25.02.00 */
      RETURN_LONG(csr->CBL.RowId);
    case CO_UPDATABLE:                                       /* 04.03.04 ... */
      strncpy(csr->CBL.Command, "FCUR", 4);
      memset(&CursorDesc,0,sizeof(CursorDesc));
      csr->CBL.LnBufRow = sizeof(CursorDesc);
      inter(&csr->CBL, "", NULL, NULL, (void *) &CursorDesc);
      if (csr->CBL.CodErr != 0)
        RETURN_LONG(LPE_LINTER_ERROR);
      RETURN_LONG((long)CursorDesc.Updatable);
    case CO_QUERY_TYPE:
      RETURN_LONG((long)csr->last_oper);
    case CO_PARAM_COUNT:                                     /* 13.10.07 */
      RETURN_LONG((long)csr->param_count);
    case CO_FETCH_BLOBS_AS_USUAL_DATA:
      RETURN_LONG((long)csr->FetchBlobBehaviour);
    case CO_FETCH_BLOBS_AS_STREAM:                           /* 05.03.09 */
      RETURN_LONG((long)csr->FetchBlobStreamBehaviour);
    case CO_FETCH_BOOL_AS_BOOL:                              /* 02.03.09 */
      RETURN_LONG((long)csr->FetchBoolBehaviour);
    case CO_NULL_AS_NULL_OBJECT:                             /* 29.03.08 */
      RETURN_LONG((long)csr->NullsBehaviour);
    case CO_DECIMAL_AS_DOUBLE:                               /* 30.03.08 */
      RETURN_LONG((long)csr->DecimalBehaviour);
    case CO_GENERATE_COL_NAME:                               /* 30.03.09 */
      RETURN_LONG((long)csr->GenerateColNameBehaviour);
    case CO_RENAME_DUBBED_COL:                               /* 30.03.09 */
      RETURN_LONG((long)csr->RenameDubbedColBehaviour);
  }                                                          /* ... 04.03.04 */
  RETURN_LONG(LPE_UNKNOWN_OPTION);
}

PHP_FUNCTION(linter_set_cursor_opt)
{
  CURSOR *csr;
  zval   *cid, *opt, *val;                                  /* 02.04.12 */

  cid = opt = val = NULL;
  if (ARG_COUNT(ht) < 2)
    WRONG_PARAM_COUNT;
  zend_get_parameters(ht, min(ARG_COUNT(ht), 3), &cid, &opt, &val);
  convert_to_long(cid);
  csr = GetFromList(cid->value.lval);
  if (csr == NULL)
    RETURN_LONG(LPE_INVALID_CURSOR);
  convert_to_long(opt);
  switch(opt->value.lval)
  {
    case CO_NAME:
      if (val == NULL)
        WRONG_PARAM_COUNT;
      convert_to_string(val);
      csr->CBL.RowId = crName;                             /* 16.12.99 */
      strncpy(csr->CBL.Command, "SETO", 4);
      csr->CBL.LnBufRow = val->value.str.len;
      inter(&csr->CBL, NULL, NULL, NULL, val->value.str.val);
      if (csr->CBL.CodErr != 0)
        RETURN_LONG(LPE_LINTER_ERROR);;
      memset(csr->name, 0, sizeof(csr->name));
      strncpy(csr->name, val->value.str.val,
              min(MAX_ID_LEN, val->value.str.len));        /* 27.12.00 */
      break;
    case CO_ASYNC_MODE:
      csr->CBL.PrzExe |= Q_ASYNC;
      break;
    case CO_SYNC_MODE:
      csr->CBL.PrzExe &= ~Q_ASYNC;
      break;
    case CO_DT_FORMAT:
      s_free(csr->dt_format);
      convert_to_string(val);
      csr->dt_format = strdup(val->value.str.val);
      break;
    case CO_FETCH_BLOBS_AS_USUAL_DATA:
      convert_to_long(val);
      csr->FetchBlobBehaviour = (val->value.lval != 0);
      break;
    case CO_FETCH_BLOBS_AS_STREAM:                               /* 05.03.09 */
      convert_to_long(val);
      csr->FetchBlobStreamBehaviour = (val->value.lval != 0);
      break;
    case CO_FETCH_BOOL_AS_BOOL:                                  /* 02.03.09 */
      convert_to_long(val);
      csr->FetchBoolBehaviour = (val->value.lval != 0);
      break;
    case CO_NULL_AS_NULL_OBJECT:                                 /* 29.03.08 */
      convert_to_long(val);
#if defined(_MOD_PHP3_H)
      RETURN_LONG(LPE_NOT_IMPLEMENTED);
#else
      csr->NullsBehaviour = (val->value.lval != 0);
#endif
      break;
    case CO_DECIMAL_AS_DOUBLE:                                   /* 30.03.08 */
      convert_to_long(val);
      csr->DecimalBehaviour = (val->value.lval != 0);
      break;                                                     /* 30.03.09 */
    case CO_GENERATE_COL_NAME:                                   /* 30.03.09 */
      convert_to_long(val);
      csr->GenerateColNameBehaviour = (val->value.lval != 0);
      break;
    case CO_RENAME_DUBBED_COL:                                   /* 30.03.09 */
      convert_to_long(val);
      csr->RenameDubbedColBehaviour = (val->value.lval != 0);
      break;
    default:
      RETURN_LONG(LPE_UNKNOWN_OPTION);
  }
  RETURN_LONG(LPE_SUCCESS);
}

PHP_FUNCTION(linter_exec_direct)
{
  CURSOR *csr;
  zval   *cid, *qry;                                        /* 02.04.12 */
  char   *query,                                                /* 28.03.08C */
         *s;
  int     Len;
  long    rc;

  if (ARG_COUNT(ht) != 2)
    WRONG_PARAM_COUNT;
  zend_get_parameters(ht, 2, &cid, &qry);
  convert_to_long(cid);
  convert_to_string(qry);
  csr = GetFromList(cid->value.lval);
  if (csr == NULL)
    RETURN_LONG(LPE_INVALID_CURSOR);
  ClearCursorDesc(csr, 0);
  s = query = qry->value.str.val;
  while(strchr(STR_SEP, *s) || *s == '(')
    s++;
  if (!strnicmp(s, "SELECT", 6))
  {
    csr->last_oper = SQL_SELECT;
    strncpy((char *)csr->CBL.Command, "SLCT", 4);
  }
  else
  {
    memset((char *)csr->CBL.Command, ' ', 4);
    if (!strnicmp(s, "GRANT", 5))
      csr->last_oper = SQL_GRANT;
    else if (!strnicmp(s, "ALTER", 5))
      csr->last_oper = SQL_ALTER;
    else if (!strnicmp(s, "EXECUTE", 7))
      csr->last_oper = SQL_EXEC_PROC;
    else
      csr->last_oper = SQL_UNKNOWN;                          /* 28.03.01 */
  }
  s = NULL;                                                  /* 28.03.08C ... */
  Len = qry->value.str.len;
  if (Len && query[Len-1] != ';')
  {
    s = malloc(Len+2);
    if (s)
    {
      memcpy(s,query,Len);
      s[Len] = ';';
      s[Len+1] = 0;
      query = s;
    }
  }
  csr->CBL.LnBufRow = MAX_ANSWER;                                /* 24.06.08  */
  inter(&csr->CBL, csr->null_flag, query, NULL, csr->exch_buff); /* 27.02.08 07.03.01 */
  s_free(s); /* From this point query can be isn't valid !!! */ /* ... 28.03.08C */
  if (csr->CBL.CodErr > EORR)                              /* 18.04.00 */
  {
    if (csr->CBL.CodErr >= SQLBADCH && csr->CBL.CodErr < SQLENDSYMEXP)
      csr->errpos = csr->CBL.SysErr;
    RETURN_LONG(LPE_LINTER_ERROR);
  }
  csr->rows = csr->CBL.RowCount;
  csr->cols = 0;                                                 /* 13.10.07 */
  if (csr->last_oper == SQL_SELECT)
  {
    rc = FillCursorDesc(csr,1);
    if (rc != 0)
      RETURN_LONG(rc);
    rc = FetchCursor(csr, FETCH_FIRST, 0, 1);
    if (rc != LPE_SUCCESS)
      RETURN_LONG(rc);
  }
  else if ((csr->last_oper == SQL_GRANT) || (csr->last_oper == SQL_ALTER))
  {
    if (csr->CBL.LnBufRow)
      csr->pass_cnt = csr->CBL.LnBufRow/18;
  }
  RETURN_LONG(LPE_SUCCESS);
}

PHP_FUNCTION(linter_prepare)
{
  CURSOR *csr;
  zval   *cid, *qry;                                        /* 02.04.12 */
  char   *query,                                                /* 28.03.08C */
         *s;
  int     Len;
  short   p_count;

  if (ARG_COUNT(ht) != 2)
    WRONG_PARAM_COUNT;
  zend_get_parameters(ht, 2, &cid, &qry);
  convert_to_long(cid);
  csr = GetFromList(cid->value.lval);
  if (csr == NULL)
    RETURN_LONG(LPE_INVALID_CURSOR);
  convert_to_string(qry);
  ClearCursorDesc(csr, 0);
  query = qry->value.str.val;
  csr->pqry_body = realloc(csr->pqry_body, MAX_BUFF);
  if (!csr->pqry_body)
    RETURN_LONG(LPE_NO_MEMORY);
  csr->last_oper = SQL_UNKNOWN;

  s = NULL;                                                 /* 28.03.08C ... */
  Len = qry->value.str.len;
  if (Len && query[Len-1] != ';')
  {
    s = malloc(Len+2);
    if (s)
    {
      memcpy(s,query,Len);
      s[Len] = ';';
      s[Len+1] = 0;
      query = s;
    }
  }
  csr->CBL.LnBufRow = MAX_BUFF;
  strncpy((char *)csr->CBL.Command, "TRAN", 4);
  inter(&csr->CBL, NULL, query, NULL, csr->pqry_body);
  s_free(s); /* From this point query can be isn't valid !!! */ /* ... 28.03.08C */
  if (csr->CBL.CodErr >= SQLBADCH && csr->CBL.CodErr <= SQLENDSYMEXP)
    csr->errpos = csr->CBL.SysErr;
  if (csr->CBL.CodErr == SMALLBUFKOR)
  {
    csr->CBL.CodErr = 0;
    csr->pqry_body = realloc(csr->pqry_body, csr->CBL.LnBufRow);
    if (csr->pqry_body == NULL)
      RETURN_LONG(LPE_NO_MEMORY);
    csr->CBL.PrzExe |= Q_RECEIVE;
    inter(&csr->CBL, NULL, NULL, NULL, csr->pqry_body);
    csr->CBL.PrzExe &= ~Q_RECEIVE;
  }
  if (csr->CBL.CodErr != 0)
    RETURN_LONG(LPE_LINTER_ERROR);
  inter_GetParamCount(csr->pqry_body, &p_count);
  csr->parm_leng = realloc(csr->parm_leng, sizeof(short) * p_count); /* 11.10.07 */
  if (p_count && !csr->parm_leng)
    RETURN_LONG(LPE_NO_MEMORY);
  csr->param_count = p_count;                              /* 13.10.07 */
  csr->last_oper = inter_isselect(&csr->CBL, csr->pqry_body);
  inter_SetEncoding(csr->pqry_body,csr->Flags);
  RETURN_LONG(LPE_SUCCESS);
}

PHP_FUNCTION(linter_execute)
{
  CURSOR     *csr;
  L_BOOL      BlFlag = L_TFALSE;                                 /* 01.03.08 */
  long        rc;
  char       *p_name;
  short       p_type;
  short       p_len[256];                                   /* 11.03.09 */
  short       i, p_count;
  zval       *cid, *params;                                 /* 02.04.12 */
  zval       *param;                                        /* 02.04.12 */

  if (ARG_COUNT(ht) < 1)
    WRONG_PARAM_COUNT;
  cid = params = NULL;
  zend_get_parameters(ht, min(ARG_COUNT(ht), 2), &cid, &params);
  convert_to_long(cid);
  csr = GetFromList(cid->value.lval);
  if (csr == NULL) 
    RETURN_LONG(LPE_INVALID_CURSOR);
  if (csr->pqry_body == NULL)
    RETURN_LONG(LPE_INVALID_STATE);
  inter_GetParamCount(csr->pqry_body, &p_count);
  if (p_count && (!params || params->type != IS_ARRAY))
    RETURN_LONG(LPE_INVALID_PARAM);
  for(i = 1; i <= p_count; i++)
  {
    p_name = inter_GetParamName(csr->pqry_body, i);
    inter_DescribeParam(csr->pqry_body, i, &csr->parm_leng[i - 1],
                        &p_type, NULL, NULL, NULL);
    if (p_type == LDT_BLOB)                                  /* 01.03.08 ... */
    {
      BlFlag = L_TTRUE;
      continue;
    }                                                        /* ... 01.03.08 */
    if (zend_hash_index_find(params->value.ht, i - 1, (void **)&param) != SUCCESS) /* 12.04.06 */
      RETURN_LONG(LPE_INVALID_PARAM);                                               /* 12.04.06 */
#ifdef PHP4
    param = *(zval **)param;                                /* 02.04.12 */
#endif
    switch (Z_TYPE_P(param))                                   /* 07.04.09 11.03.09... */
    {
      case IS_BOOL:
          p_len[i-1] = 4;                                      /* 27.11.12 ... */
          if (p_name) 
            inter_bind_name(&csr->CBL, csr->pqry_body, p_name, LDT_INTEGER, &p_len[i-1], (char *)&param->value.lval);          
          else
            inter_bind_num(&csr->CBL, csr->pqry_body, i, LDT_INTEGER, &p_len[i-1], (char *)&param->value.lval);          
          break;
      case IS_STRING:
          p_len[i-1] = param->value.str.len;
          if (p_name)
            inter_bind_name(&csr->CBL, csr->pqry_body, p_name, LDT_CHAR, &p_len[i-1], (char *)param->value.str.val);          
          else
            inter_bind_num(&csr->CBL, csr->pqry_body, i, LDT_CHAR, &p_len[i-1], (char *)param->value.str.val);          
          break;
      case IS_LONG:
          p_len[i-1] = sizeof(long);                             /* 30.09.14 */
          if (p_name)
            inter_bind_name(&csr->CBL, csr->pqry_body, p_name, LDT_INTEGER, &p_len[i-1], (char *)&param->value.lval);
          else
            inter_bind_num(&csr->CBL, csr->pqry_body, i, LDT_INTEGER, &p_len[i-1], (char *)&param->value.lval);
          break;
      case IS_DOUBLE:
          p_len[i-1] = 8;
          if (p_name)
            inter_bind_name(&csr->CBL, csr->pqry_body, p_name, LDT_REAL, &p_len[i-1], (char *)&param->value.dval);
          else
            inter_bind_num(&csr->CBL, csr->pqry_body, i, LDT_REAL, &p_len[i-1], (char *)&param->value.dval);
          break;
      case IS_NULL:
          if (csr->NullsBehaviour && ZVAL_IS_NULL(param))              /* 29.03.08 */
            p_len[i-1] = -1;                                                /* 29.03.08 11.03.09*/
          else
            p_len[i-1] = 4;

          if (p_name)
            inter_bind_name(&csr->CBL, csr->pqry_body, p_name, LDT_INTEGER, &p_len[i-1], (char *)&param->value.lval);
          else
            inter_bind_num(&csr->CBL, csr->pqry_body, i, LDT_INTEGER, &p_len[i-1], (char *)&param->value.lval);
          break;                                                       /* ... 27.11.12 */
      default:
          RETURN_LONG(LPE_LINTER_ERROR);
          break;
    }                                                          /* ... 11.03.09 07.04.09 */      

    if (csr->CBL.CodErr != 0)
      RETURN_LONG(LPE_LINTER_ERROR);
  }
  csr->CBL.LnBufRow = MAX_ANSWER;                               /* 24.06.08  */
  strncpy((char *)csr->CBL.Command, "EXEC", 4);
  inter(&csr->CBL, csr->null_flag, csr->pqry_body, NULL, csr->exch_buff); /* 24.06.08  */ /* 13.09.04 */ 
  if (csr->CBL.CodErr > EORR)                                   /* 18.04.00 */
    RETURN_LONG(LPE_LINTER_ERROR);
  csr->rows = csr->CBL.RowCount;
  csr->cols = 0;                                                /* 13.10.07 */
  if ((csr->last_oper == SQL_SELECT)||(csr->last_oper == SQL_TSTTBL))
  {
    rc = FillCursorDesc(csr, 0);
    if (rc != 0)
      RETURN_LONG(rc);
    rc = FetchCursor(csr, FETCH_FIRST, 0, 1);
    if (rc != LPE_SUCCESS)
      RETURN_LONG(rc);
    csr->last_oper = SQL_SELECT;
  }
  else
  {
    if ((csr->last_oper == SQL_GRANT) || (csr->last_oper == SQL_ALTER)) /* 24.06.08 ... */
    {
      if (csr->CBL.LnBufRow)
        csr->pass_cnt = csr->CBL.LnBufRow/18;
      return;
    }                                                                   /* ... 24.06.08 */
    if (BlFlag)                                              /* 01.03.08 ... */
    {
      for(i = 1; i <= p_count; i++)
      {
        inter_DescribeParam(csr->pqry_body, i, NULL, &p_type, NULL, NULL, NULL);
        if (p_type != LDT_BLOB)
          continue;
        if (zend_hash_index_find(params->value.ht, i - 1, (void **)&param) != SUCCESS)
          RETURN_LONG(LPE_INVALID_PARAM);
#ifdef PHP4
        param = *(zval **)param;                            /* 02.04.12 */
#endif
        if (!ClearBlob(csr,inter_GetBlobColNum(csr->pqry_body,i)))                                                /* 19.02.09 */
          RETURN_LONG(LPE_LINTER_ERROR);
        if (!AddBlob(csr, param->value.str.val, param->value.str.len, 0, inter_GetBlobColNum(csr->pqry_body,i)))  /* 19.02.09 */
          RETURN_LONG(LPE_LINTER_ERROR);
      }
    }                                                        /* ... 01.03.08 */
  }
  RETURN_LONG(LPE_SUCCESS);
}

PHP_FUNCTION(linter_fetch)
{
  CURSOR *csr;
  zval   *cid, *dir, *pos;                                  /* 02.04.12 */

  if (ARG_COUNT(ht) < 2)
    WRONG_PARAM_COUNT;
  cid = dir = pos = NULL;
  zend_get_parameters(ht, min(ARG_COUNT(ht), 3), &cid, &dir, &pos);
  if (dir->value.lval == FETCH_ABSNUM && pos == NULL)
    WRONG_PARAM_COUNT;
  convert_to_long(cid);
  csr = GetFromList(cid->value.lval);
  if (csr == NULL)
    RETURN_LONG(LPE_INVALID_CURSOR);
  if (csr->last_oper != SQL_SELECT)
    RETURN_LONG(LPE_INVALID_STATE);
  convert_to_long(dir);
  if (pos != NULL)
    convert_to_long(pos);
  RETURN_LONG(FetchCursor(csr, dir->value.lval, pos ? pos->value.lval : 0, 0));
}

PHP_FUNCTION(linter_fetch_row)                              /* 13.09.04 */
{
  CURSOR *csr;
  zval   *cid = NULL;                                       /* 02.04.12 */
  zval   *tmp = NULL;                                       /* 02.04.12 */

  if (ARG_COUNT(ht) < 1)
    WRONG_PARAM_COUNT;

  zend_get_parameters(ht, min(ARG_COUNT(ht), 2), &cid, &tmp);
  convert_to_long(cid);

  csr = GetFromList(cid->value.lval);
  if (csr == NULL)
    RETURN_LONG(LPE_INVALID_CURSOR);
  if ((csr->last_oper != SQL_SELECT) && (csr->last_oper != SQL_EXEC_PROC)) /* 05.05.06 */
    RETURN_LONG(LPE_INVALID_STATE);
  if (csr->CBL.CodErr == EORR)
    RETURN_LONG(LPE_LINTER_ERROR);
  if (csr->crow > csr->rows )
    RETURN_LONG(LPE_LINTER_ERROR);

  PHP_FN(linter_get_data_row)(INTERNAL_FUNCTION_PARAM_PASSTHRU);
  FetchCursor(csr, FETCH_NEXT, 0, 0);
}

PHP_FUNCTION(linter_fetch_array)                            /* 13.09.04 */
{
  CURSOR *csr;
  zval   *cid = NULL;                                       /* 02.04.12 */

  if (ARG_COUNT(ht) != 1)
    WRONG_PARAM_COUNT;

  zend_get_parameters(ht, 1, &cid);
  convert_to_long(cid);

  csr = GetFromList(cid->value.lval);
  if (csr == NULL)
    RETURN_LONG(LPE_INVALID_CURSOR);

  if ((csr->last_oper != SQL_SELECT) && (csr->last_oper != SQL_EXEC_PROC)) /* 05.05.06 */
    RETURN_LONG(LPE_INVALID_STATE);
  if (csr->CBL.CodErr == EORR)
    RETURN_LONG(LPE_LINTER_ERROR);
  if (csr->crow > csr->rows )
    RETURN_LONG(LPE_LINTER_ERROR);

  PHP_FN(linter_get_data_array)(INTERNAL_FUNCTION_PARAM_PASSTHRU);
  FetchCursor(csr, FETCH_NEXT, 0, 0);
}


PHP_FUNCTION(linter_set_codepage)                           /*13.09.04 */
{
  zval   *cpname;                                           /* 02.04.12 */

  if (ARG_COUNT(ht) != 1)
    WRONG_PARAM_COUNT;

  zend_get_parameters(ht, 1, &cpname);
  convert_to_string(cpname);

#if _VER_MAX >= 600
  if ( !inter_control( NULL, ICR_SET_CP, cpname->value.str.val, CP_AS_STR) )
    RETURN_LONG (LPE_LINTER_ERROR)
#endif
  RETURN_LONG (LPE_SUCCESS)
}

void LinPHP_GetProcOutData(INTERNAL_FUNCTION_PARAMETERS)   /* 09.11.00 ... */
{
  double       dec_val;
  char         dt_val[32];
  char         ds_val[DECLNG*2+1];  /* 27.09.04 */
  zval         *cid;                                        /* 02.04.12 */
  L_WORD       i;
  L_WORD      *par_count;
  L_BYTE      *par_value;
  L_WORD      *par_val_len;
  ARGPROC_OUT *par_info;
  CURSOR      *csr;

  if (ARG_COUNT(ht) < 1)
    WRONG_PARAM_COUNT;
  zend_get_parameters(ht, 1, &cid);
  convert_to_long(cid);
  csr = GetFromList(cid->value.lval);
  if (csr == NULL)
    RETURN_LONG(LPE_INVALID_CURSOR);
  if (csr->last_oper != SQL_EXEC_PROC)
    RETURN_LONG(LPE_INVALID_STATE);
  array_init(return_value);

  if (csr->CBL.CodErr == Proc_Raised_Exception)
  {
    add_index_long(return_value, 0, csr->CBL.SysErr);
    RETURN_LONG(LPE_SUCCESS);
  }

  par_info  = (ARGPROC_OUT *)csr->exch_buff;
  par_count = (L_WORD *)(par_info + 1);

  for(i = 0; i <= *par_count; i++)
  {
    if (i > 0)
    {
      par_info  = (ARGPROC_OUT *)(par_count + 1);
      par_info += i - 1;
    }
    par_value = csr->exch_buff + par_info->Value;

    if (par_info->Flags & PARM_FLAG_NAMESENT)
    {
      L_WORD par_name_len;
#if !defined(LONG_ALIGN)
      par_name_len = *((L_WORD *)par_value);
#else
      memcpy(&par_name_len, par_value, sizeof(L_WORD));     /* 14.05.03 */
#endif
      par_value += sizeof(L_WORD) + par_name_len;
    }

    if (par_info->Flags & PARM_FLAG_ISCURSOR)
    {
      CURSOR   new_csr, *c;
      CR_INFO *cr_info;
#if !defined(LONG_ALIGN)
      cr_info = (CR_INFO *)par_value;
#else                                                       /* 14.05.03 ... */
      CR_INFO CursorInfo;
      cr_info = &CursorInfo;
      memcpy(cr_info, par_value, sizeof(CR_INFO));
#endif                                                      /* ... 14.05.03 */
      memset(&new_csr, 0, sizeof(CURSOR));
      if (csr->parent == NULL)
        new_csr.parent = csr;
      else
        new_csr.parent = csr->parent;
      memcpy(&new_csr.CBL, &(csr->CBL), sizeof(TCBL));
      new_csr.CBL.CodErr   = cr_info->CodErr;
      new_csr.CBL.NumChan  = cr_info->NumChan;
      new_csr.CBL.RowId    = cr_info->RowId;
      new_csr.CBL.RowCount = cr_info->RowCount;
      new_csr.CBL.LnBufRow = cr_info->LnBufRow;
      new_csr.last_oper    = SQL_SELECT;
      c = AddToList(&new_csr);                            /* 03.01.01 ... */
      add_next_index_long(return_value, c->id);
      FillCursorDesc(c, 0);
      FetchCursor(c, FETCH_FIRST, 0, 0);                  /* ... 03.01.01 */
      c->FetchBlobBehaviour = csr->FetchBlobBehaviour;    /* 20.10.07 */
      c->FetchBlobStreamBehaviour = csr->FetchBlobStreamBehaviour; /* 05.03.09 */
      c->FetchBoolBehaviour = csr->FetchBoolBehaviour;    /* 02.03.09 */
      c->NullsBehaviour = csr->NullsBehaviour;            /* 29.03.08 */
      c->DecimalBehaviour = csr->DecimalBehaviour;        /* 30.03.08 */
      c->GenerateColNameBehaviour = csr->GenerateColNameBehaviour; /* 30.03.09 */
      c->RenameDubbedColBehaviour = csr->RenameDubbedColBehaviour; /* 30.03.09 */
      continue;
    }

    if (par_info->Flags & PARM_FLAG_ISNULL)
      continue;    

    switch(par_info->Type.ntyp)
    {
      case LDT_BYTE:
      case LDT_CHAR:
      case LDT_NCHAR:   /* 28.06.04 */
        add_next_index_stringl(return_value, par_value, par_info->Type.length, L_TTRUE);
        break;
      case LDT_EXTFILE:   /* 16.10.14 */
        add_next_index_stringl(return_value, (par_value + 10), strlen(par_value + 10), L_TTRUE);
        break;

      case LDT_INTEGER:
        if (par_info->Type.length == 8)
        {
#if !defined(LONG_ALIGN)
          i64_ToDouble(*(L_DLONG *)par_value, dec_val);
#else                                                       /* 14.05.03 ... */
          L_DLONG Val;

          memcpy(&Val, par_value, sizeof(L_DLONG));
          i64_ToDouble(Val, dec_val);
#endif                                                      /* ... 14.05.03 */
          add_next_index_double(return_value, dec_val);
        }
        else
        {
          if (par_info->Type.length == 4)
#if !defined(LONG_ALIGN)
            add_next_index_long(return_value, (long)(*(L_LONG *)par_value));
#else                                                       /* 14.05.03 ... */
          {
            L_LONG Val;

            memcpy(&Val, par_value, sizeof(L_LONG));
            add_next_index_long(return_value, (long)Val);
          }
#endif                                                      /* ... 14.05.03 */
          else
#if !defined(LONG_ALIGN)
            add_next_index_long(return_value, (long)(*(L_SWORD *)par_value)); /* 08.10.04 */
#else                                                       /* 14.05.03 ... */
          {
            L_SWORD Val;                                                      /* 08.10.04 */

            memcpy(&Val, par_value, sizeof(L_SWORD));
            add_next_index_long(return_value, (long)Val);
          }
#endif                                                      /* ... 14.05.03 */
        }
        break;
      case LDT_REAL:
        if (par_info->Type.length == 4)
#if !defined(LONG_ALIGN)
          add_next_index_double(return_value, (double)(*((L_REAL *)par_value)));
#else                                                       /* 14.05.03 ... */
        {
          L_REAL Val;

          memcpy(&Val, par_value, sizeof(L_REAL));
          add_next_index_double(return_value, (double)Val);
        }
#endif                                                      /* ... 14.05.03 */
        else
#if !defined(LONG_ALIGN)
          add_next_index_double(return_value, (double)(*((L_DOUBLE *)par_value)));
#else                                                       /* 14.05.03 ... */
        {
          L_DOUBLE Val;

          memcpy(&Val, par_value, sizeof(L_DOUBLE));
          add_next_index_double(return_value, (double)Val);
        }
#endif                                      /* ... 14.05.03 */
        break;
      case LDT_NUMERIC:
        if ( csr->DecimalBehaviour )                         /* 27.09.04 ... */
        {
          DecToDbl(*(L_DECIMAL*)par_value, &dec_val);
          add_next_index_double(return_value, dec_val);
        }
        else
        {
          DECTOSTR(*(L_DECIMAL*)par_value, ds_val, par_info->Type.prcs, par_info->Type.scale, L_TTRUE); /* 18.08.04 */  
          strrtrim(ds_val," ");
          add_next_index_string(return_value, ds_val, L_TTRUE);                                       /* 18.08.04 */  
        }                                                  /* ... 27.09.04 */
        break;
      case LDT_DATE:
        TICKTOSTRF(*(L_DECIMAL*)par_value, csr->dt_format, dt_val);
        add_next_index_string(return_value, dt_val, L_TTRUE);
        break;
      case LDT_BLOB:
        if (csr->FetchBlobBehaviour || csr->FetchBlobStreamBehaviour)                         /* 20.10.07 ... *//* 05.03.09 */
        {
          char   *blob_value;
          L_LONG  blob_size, res;

          res = GetBlobSize(csr, i + 1, &blob_size);
          if (res != LPE_SUCCESS)
            RETURN_LONG(res);

          blob_value = (char *)ecalloc(1, blob_size);                                         /* 05.03.09 */
          if (blob_value == NULL)
            RETURN_LONG(LPE_NO_MEMORY);
          res = FetchBlob(csr, 1, &blob_size, i + 1, blob_value);
          if (res != LPE_SUCCESS)
          {
            efree(blob_value);                                                                /* 05.03.09 */
            RETURN_LONG(res);
          }
#if PHP_API_VERSION >= 20020918                                                               /* 05.03.09 ... */
          if (csr->FetchBlobStreamBehaviour)
          {
            php_stream *stm;
            stm = php_stream_memory_open(TEMP_STREAM_READONLY, blob_value, blob_size);
            if (stm)
              add_next_index_resource(return_value,php_stream_get_resource_id(stm));                          
            else
            {
              add_next_index_null(return_value);
              efree(blob_value);
            }
          }
          else
#endif
          {
            add_next_index_stringl(return_value, blob_value, blob_size, L_TTRUE);          
            efree(blob_value);
          }
        }                                                                                     /* ...05.03.09 */
        else                                                 /* ... 20.10.07 */
          add_next_index_long(return_value, 0);
        break;
      case LDT_VARCHAR:
      case LDT_VARBYTE:
      case LDT_NVARCHAR: /* 28.06.04 */
#if !defined(LONG_ALIGN)
        par_val_len = (L_WORD *)par_value;
        par_value += sizeof(L_WORD);
        add_next_index_stringl(return_value, par_value, (unsigned int)(*par_val_len), L_TTRUE); /* 14.05.03 */
#else                                       /* 14.05.03 ... */
        {
          L_WORD ParLen;

          memcpy(&ParLen, par_value, sizeof(L_WORD));
          par_value += sizeof(L_WORD);
          add_next_index_stringl(return_value, par_value, (unsigned int)ParLen, L_TTRUE);
        }
#endif                                      /* ... 14.05.03 */
        break;
      case LDT_BOOLEAN:
#if PHP_API_VERSION >= 19990421                            /* 02.03.09 ... */
        if (csr->FetchBoolBehaviour == 1)
          add_next_index_bool(return_value,*par_value);
        else
#endif                                                     /* ... 02.03.09 */
          if (*par_value != 0)
            add_next_index_string(return_value, "TRUE", L_TTRUE);
          else
            add_next_index_string(return_value, "FALSE", L_TTRUE);
        break;
      default:
        RETURN_LONG(LPE_NOT_IMPLEMENTED);
      }
    }
}                                                          /* ... 09.11.00 */

static L_LONG GetDataRow(zval *Result, CURSOR *csr, L_LONG nfl TSRMLS_DC) /* 02.04.12 24.06.08, 05.03.09 */
{
  char   *buf;                                               /* 14.10.14 */
  char   *c_val;
  long    l_val;
  double  d_val;
  char    t_val[64];
  char    ds_val[DECLNG*2+1];                                /* 27.09.04 */
  long    len, off = 0;
  long    i, j = 0, nf;                                      /* 06.02.04 */

  array_init(Result);
  for (i = 0; i < csr->cols; i++)
  {
    len = csr->answ_desc[i].Length;                        /* 21.12.99 */
    nf = csr->null_flag[sizeof(L_WORD) * 2 + i];           /* 06.02.04 */
    switch(csr->answ_desc[i].Type)
    {
      case LDT_BYTE:
      case LDT_CHAR:
      case LDT_NCHAR:                                      /* 28.06.04 */

        if (nf)                                            /* 06.02.04 ... */
        {
          if (csr->NullsBehaviour)                         /* 29.03.08 */
            add_next_index_null(Result);                   /* 29.03.08 */
          else
            add_next_index_stringl(Result, "", 0, L_TTRUE);
        }
        else                                              /* ... 06.02.04 */
        {
          c_val = (char *)&csr->exch_buff[off];
          add_next_index_stringl(Result, c_val, len, L_TTRUE);
        }
        break;
      case LDT_EXTFILE:                                      /* 16.10.14 ... */
        if (nf)                                            
        {
          if (csr->NullsBehaviour)                         
            add_next_index_null(Result);                   
          else
            add_next_index_stringl(Result, "", 0, L_TTRUE);
        }
        else                                              
        {
          len = strlen(&csr->exch_buff[off] + 10);
          c_val = (char *)(&csr->exch_buff[off] + 10);
          add_next_index_stringl(Result, c_val, len, L_TTRUE);
        }
        break;                                              /* ... 16.10.14 */
      case LDT_INTEGER:
        if (nf)                                             /* 06.02.04 ... */
        {
          if (csr->NullsBehaviour)                          /* 29.03.08 */
            add_next_index_null(Result);              /* 29.03.08 */
          else
            add_next_index_long(Result, 0);
        }
        else                                                /* ... 06.02.04 */
        {
          if ((sizeof(long) == 4) && (len > 4))             /* 14.10.14 */
          {
            buf = emalloc(100);
            l_val = GetDataColInteger(csr,i, off, buf);
            add_next_index_stringl(Result, buf, strlen(buf), L_TTRUE);
            s_free(buf);
          }
          else
          {
            l_val = GetDataColInteger(csr,i, off, NULL);
            add_next_index_long(Result, l_val);
          }
        }
        
        break;
      case LDT_REAL:
        if (nf)                                             /* 06.02.04 ... */
        {
          if (csr->NullsBehaviour)                          /* 29.03.08 */
            add_next_index_null(Result);              /* 29.03.08 */
          else
            add_next_index_double(Result, 0.0e0);
        }
        else                                                /* ... 06.02.04 */
        {
          d_val = GetDataColReal(csr,i, off);
          add_next_index_double(Result, d_val);
        }
        break;
      case LDT_NUMERIC:
        if ( csr->DecimalBehaviour )                         /* 27.09.04 ... */
        {
          if (nf)                                           /* 06.02.04 ... */
          {
            if (csr->NullsBehaviour)                        /* 29.03.08 */
              add_next_index_null(Result);            /* 29.03.08 */
            else
              add_next_index_double(Result, 0.0e0);
          }
          else                                              /* ... 06.02.04 */
          {
            DecToDbl(*((L_DECIMAL*)&csr->exch_buff[off]), &d_val);
            add_next_index_double(Result, d_val);
          }
        }
        else
        {
          if (nf)
          {
            if (csr->NullsBehaviour)                           /* 29.03.08 */
              add_next_index_null(Result);               /* 29.03.08 */
            else
              add_next_index_string(Result, "0", L_TTRUE);
          }
          else
          {
            DECTOSTR(*((L_DECIMAL*)&csr->exch_buff[off]), ds_val, 30, 10, L_TTRUE);
            strrtrim( ds_val, " ");
            add_next_index_string(Result, ds_val, L_TTRUE);
          }
        }                                                  /* ... 27.09.04 */
        break;

      case LDT_DATE:

        if (nf)                                              /* 06.02.04 ... */
        {
          if (csr->NullsBehaviour)                           /* 29.03.08 */
            add_next_index_null(Result);               /* 29.03.08 */
          else
            add_next_index_string(Result, "00.00.0000 00:00:00", L_TTRUE);
        }
        else                                                 /* ... 06.02.04 */
        {
          TICKTOSTRF(*((L_DECIMAL*)&csr->exch_buff[off]), csr->dt_format, t_val);
          add_next_index_string(Result, t_val, L_TTRUE);
        }
        break;
      case LDT_BLOB:
        if (csr->FetchBlobBehaviour || csr->FetchBlobStreamBehaviour)                         /* 20.10.07 ... *//* 05.03.09 */
        {
          char   *blob_value;
          L_LONG  blob_size, res;

          res = GetBlobSize(csr, i + 1, &blob_size);
          if (res != LPE_SUCCESS)
            return(res);

          blob_value = (char *)ecalloc(1, blob_size);                                         /* 05.03.09 */
          if (blob_value == NULL)
            return(LPE_NO_MEMORY);
          res = FetchBlob(csr, 1, &blob_size, i + 1, blob_value);
          if (res != LPE_SUCCESS)
          {
            efree(blob_value);                                                                /* 05.03.09 */
            return (res);
          }
#if PHP_API_VERSION >= 20020918                                                               /* 05.03.09 ... */
          if (csr->FetchBlobStreamBehaviour)
          {
            php_stream *stm;
            stm = php_stream_memory_open(TEMP_STREAM_READONLY, blob_value, blob_size);
            if (stm)
              add_next_index_resource(Result,php_stream_get_resource_id(stm));
            else
            {
              add_next_index_null(Result);
              efree(blob_value);
            }
          }
          else
#endif
          {
            add_next_index_stringl(Result, blob_value, blob_size, L_TTRUE);
            efree(blob_value);
          }
        }                                                                                     /* ... 05.03.09 */
        else                                                 /* ... 20.10.07 */
          add_next_index_long(Result, 0);
        break;
      case LDT_VARCHAR:
      case LDT_VARBYTE:
      case LDT_NVARCHAR:                                    /* 28.06.04 */
        if (nf)                                             /* 06.02.04 ... */
        {
          if (csr->NullsBehaviour)                          /* 29.03.08 */
            add_next_index_null(Result);              /* 29.03.08 */
          else
            add_next_index_stringl(Result, "", 0, L_TTRUE);
        }
        else                                                /* ... 06.02.04 */
        {
          c_val = (char *)&csr->exch_buff[off];
#if !defined(LONG_ALIGN)
          len   = (long)(*(L_WORD *)c_val);
#else                                       /* 14.05.03 ... */
          {
            L_WORD  Val;

            memcpy((char *)&Val, (CHAR *)c_val, sizeof(L_WORD));
            len   = (long)Val;
          }
#endif                                      /* ... 14.05.03 */
          c_val += sizeof(L_WORD);
          off   += sizeof(L_WORD);

          add_next_index_stringl(Result, c_val, len, L_TTRUE);
        }
        break;
      case LDT_BOOLEAN:                                     /* 02.03.09 */
        if (nf)                                             /* 06.02.04 ... */
        {
          if (csr->NullsBehaviour)                          /* 29.03.08 */
            add_next_index_null(Result);                    /* 29.03.08 */
          else
#if PHP_API_VERSION >= 19990421                             /* 02.03.09 ... */
            if (csr->FetchBoolBehaviour)
              add_next_index_bool(Result,0);
            else
#endif                                                      /* ... 02.03.09 */
            {
              add_next_index_string(Result, "FALSE", L_TTRUE);
            }
        }
        else                                                /* ... 06.02.04 */
        {
#if PHP_API_VERSION >= 19990421                             /* 02.03.09 ... */
          if (csr->FetchBoolBehaviour)
            add_next_index_bool(Result,csr->exch_buff[off]);
          else
#endif                                                      /* ... 02.03.09 */
          {
            if (csr->exch_buff[off] != 0)
              c_val = "TRUE";
            else
              c_val = "FALSE";
            add_next_index_string(Result, c_val, L_TTRUE);
          }
        }
        break;
    }
    if (nfl != 0)
      add_next_index_long(Result, csr->null_flag[sizeof(L_WORD) * 2 + i]);
    off += csr->answ_desc[i].Length;                       /* 21.02.99 */
  }
  return LPE_SUCCESS;
}

PHP_FUNCTION(linter_get_data_row)
{
  CURSOR *csr;
  zval   *cid;                                              /* 02.04.12 */
  zval   *nif = NULL;                                       /* 02.04.12 */
  long    off = 0,nfl = 0;                             /* 06.02.04 */
  L_LONG  Ret;

  if (ARG_COUNT(ht) < 1)
    WRONG_PARAM_COUNT;
  zend_get_parameters(ht, min(ARG_COUNT(ht), 2), &cid, &nif);
  convert_to_long(cid);
  csr = GetFromList(cid->value.lval);
  if (csr == NULL)
    RETURN_LONG(LPE_INVALID_CURSOR);
  if (csr->last_oper == SQL_EXEC_PROC)
  {
    LinPHP_GetProcOutData(INTERNAL_FUNCTION_PARAM_PASSTHRU);
    return;
  }
  if (((csr->last_oper == SQL_ALTER) || (csr->last_oper == SQL_GRANT)) &&
       (csr->pass_cnt))
  {
    L_LONG i = 0;
    array_init(return_value);

    for (i=0;i < csr->pass_cnt;i++)
      add_next_index_stringl(return_value, &csr->exch_buff[i*18], 18, L_TTRUE);
    return;
  }

  if (csr->last_oper != SQL_SELECT)
    RETURN_LONG(LPE_INVALID_STATE);
  if (csr->CBL.CodErr)                                          /* 28.04.08A */
    RETURN_LONG(LPE_LINTER_ERROR);                              /* 28.04.08A */
  if (nif != NULL)
  {
    convert_to_long(nif);
    nfl = (nif->value.lval != 0);                                /* 06.02.04 */
  }
  array_init(return_value);
  if (Ret = GetDataRow(return_value,csr,nfl TSRMLS_CC))                    /* 24.06.08, 05.03.09 */
    RETURN_LONG(Ret);
}

PHP_FUNCTION(linter_get_data_array)
{
  char   *buf;                                              /* 14.10.14 */
  CURSOR *csr;
  zval   *cid;                                              /* 02.04.12 */
  char   *c_val;
  long    l_val;
  double  d_val;
  char    t_val[64];
  char    ds_val[DECLNG*2+1];                              /* 27.09.04 */
  long    i, off = 0, len;
  char    ColName[MAX_ID_LEN + 1];                         /* 27.12.00 */
  L_BYTE    nf;                                              /* 06.02.04 */

  memset(ColName, 0, sizeof(ColName));
  if (ARG_COUNT(ht) != 1)
    WRONG_PARAM_COUNT;
  zend_get_parameters(ht, 1, &cid);
  convert_to_long(cid);
  csr = GetFromList(cid->value.lval);
  if (csr == NULL)
    RETURN_LONG(LPE_INVALID_CURSOR);
  if (csr->last_oper != SQL_SELECT)
    RETURN_LONG(LPE_INVALID_STATE);
  if (csr->CBL.CodErr > 0)                                      /* 28.04.08A */
    RETURN_LONG(LPE_LINTER_ERROR);                              /* 28.04.08A */
  array_init(return_value);
  for(i = 0; i < csr->cols; i++)
  {
    strncpy(ColName, csr->answ_desc[i].Column, MAX_ID_LEN);
    ColName[MAX_ID_LEN] = 0;
    strrtrim(ColName, " ");                                 /* 15.05.03 */
    len = csr->answ_desc[i].Length;                        /* 21.12.99 */
    nf = csr->null_flag[sizeof(L_WORD) * 2 + i];      /* 06.02.04 */
    switch(csr->answ_desc[i].Type)
    {
      case LDT_BYTE:
      case LDT_CHAR:
      case LDT_NCHAR:                                       /* 28.06.04 */
        
        if (nf)                                             /* 06.02.04 ... */
        {
          if (csr->NullsBehaviour)                          /* 29.03.08 */
            add_assoc_null(return_value,ColName);           /* 29.03.08 */
          else
            add_assoc_stringl(return_value, ColName, "", 0, L_TTRUE);
        }
        else                                                /* ... 06.02.04 */
        {
          c_val = (char *)&csr->exch_buff[off];
          add_assoc_stringl(return_value, ColName, c_val, len, L_TTRUE);
        }
        break;
      case LDT_EXTFILE:                                       /* 16.10.14 */
        
        if (nf)
        {
          if (csr->NullsBehaviour)
            add_assoc_null(return_value,ColName);
          else
            add_assoc_stringl(return_value, ColName, "", 0, L_TTRUE);
        }
        else
        {
          len = strlen(&csr->exch_buff[off] + 10);
          c_val = (char *)(&csr->exch_buff[off] + 10);
          add_assoc_stringl(return_value, ColName, c_val, len, L_TTRUE);
        }
        break;

      case LDT_INTEGER:
        if (nf)                                             /* 06.02.04 ... */
        {
          if (csr->NullsBehaviour)                          /* 29.03.08 */
            add_assoc_null(return_value,ColName);           /* 29.03.08 */
          else
            add_assoc_long(return_value, ColName, 0);
        }
        else                                                /* ... 06.02.04 */
        {
          if ((sizeof(long) == 4) && (len > 4))             /* 14.10.14 */
          {
            buf = emalloc(100);
            l_val = GetDataColInteger(csr,i, off, buf);
            add_assoc_stringl(return_value, ColName, buf, strlen(buf), L_TTRUE);
            s_free(buf);
          }
          else
          {
            l_val = GetDataColInteger(csr,i, off, NULL);
            add_assoc_long(return_value, ColName, l_val);
          }
        }
        break;
      case LDT_REAL:
        if (nf)                                             /* 06.02.04 ... */
        {
          if (csr->NullsBehaviour)                          /* 29.03.08 */
            add_assoc_null(return_value,ColName);           /* 29.03.08 */
          else
            add_assoc_double(return_value, ColName, 0.0e0);
        }
        else                                              /* ... 06.02.04 */
        {
          d_val = GetDataColReal(csr,i, off);                 /* 22.01.09 */
          add_assoc_double(return_value, ColName, d_val);
        }
        break;
      case LDT_NUMERIC:
        if ( csr->DecimalBehaviour )                         /* 27.09.04 ... */
        {
          if (nf)                                             /* 06.02.04 ... */
          {
            if (csr->NullsBehaviour)                          /* 29.03.08 */
              add_assoc_null(return_value,ColName);           /* 29.03.08 */
            else
              add_assoc_double(return_value, ColName, 0.0e0);
          }
          else                                                /* ... 06.02.04 */
          {
            DecToDbl(*((L_DECIMAL*)&csr->exch_buff[off]), &d_val);
            add_assoc_double(return_value, ColName, d_val);
          }
        }
        else
        {
          if (nf)
          {
            if (csr->NullsBehaviour)                            /* 29.03.08 */
              add_assoc_null(return_value,ColName);             /* 29.03.08 */
            else
              add_assoc_string(return_value, ColName, "0", L_TTRUE);
          }
          else
          {
            DECTOSTR(*((L_DECIMAL*)&csr->exch_buff[off]), ds_val, 30, 10, L_TTRUE);
            strrtrim(ds_val," ");                                                                                    /* 27.09.04 */
            add_assoc_string(return_value, ColName, ds_val, L_TTRUE);
          }
        }                                                       /* ... 27.09.04 */
        break;
      case LDT_DATE:
        if (nf)                                              /* 06.02.04 ... */
        {
          if (csr->NullsBehaviour)                           /* 29.03.08 */
            add_assoc_null(return_value,ColName);            /* 29.03.08 */
          else
            add_assoc_string(return_value, ColName, "00.00.0000 00:00:00", L_TTRUE);
        }
        else                                                 /* ... 06.02.04 */
        {
          TICKTOSTRF(*((L_DECIMAL*)&csr->exch_buff[off]), csr->dt_format, t_val);
          add_assoc_string(return_value, ColName, t_val, L_TTRUE);
        }
        break;
      case LDT_BLOB:
        if (csr->FetchBlobBehaviour || csr->FetchBlobStreamBehaviour)                         /* 20.10.07 ... *//* 05.03.09 */
        {
          char   *blob_value;
          L_LONG  blob_size, res;

          res = GetBlobSize(csr, i + 1, &blob_size);
          if (res != LPE_SUCCESS)
            RETURN_LONG(res);

          blob_value = (char *)ecalloc(1, blob_size);                                         /* 05.03.09 */
          if (blob_value == NULL)
            RETURN_LONG(LPE_NO_MEMORY);
          res = FetchBlob(csr, 1, &blob_size, i + 1, blob_value);
          if (res != LPE_SUCCESS)
          {
            efree(blob_value);                                                                /* 05.03.09 */
            RETURN_LONG(res);
          }
#if PHP_API_VERSION >= 20020918                                                               /* 05.03.09 ... */
          if (csr->FetchBlobStreamBehaviour)
          {
            php_stream *stm;
            stm = php_stream_memory_open(TEMP_STREAM_READONLY, blob_value, blob_size);
            if (stm)
              add_assoc_resource(return_value, ColName, php_stream_get_resource_id(stm));
            else
            {
              add_assoc_null(return_value, ColName);
              efree(blob_value);
            }
          }
          else
#endif
          {
            add_assoc_stringl(return_value, ColName, blob_value, blob_size, L_TTRUE);
            efree(blob_value);
          }
        }                                                                                     /* ... 05.03.09 */
        else                                                 /* ... 20.10.07 */
          add_assoc_long(return_value, ColName, 0);
        break;
      case LDT_VARCHAR:
      case LDT_VARBYTE:
      case LDT_NVARCHAR:                                    /* 28.06.04 */
        if (nf)                                             /* 06.02.04 ... */
        {
          if (csr->NullsBehaviour)                          /* 29.03.08 */
            add_assoc_null(return_value,ColName);           /* 29.03.08 */
          else
            add_assoc_stringl(return_value, ColName, "", 0, L_TTRUE);
        }
        else                                                /* ... 06.02.04 */
        {
          c_val = (char *)&csr->exch_buff[off];
#if !defined(LONG_ALIGN)
          len   = (long)(*(L_WORD *)c_val);
#else                                       /* 14.05.03 ... */
          {
            L_WORD Val;
        
            memcpy(&Val, c_val, sizeof(L_WORD));
            len = (long)Val;
          }
#endif                                      /* ... 14.05.03 */
          c_val += sizeof(L_WORD);
          off   += sizeof(L_WORD);

          add_assoc_stringl(return_value, ColName, c_val, len, L_TTRUE);
        }
        break;
      case LDT_BOOLEAN:                                     /* 02.03.09 */
        if (nf)                                             /* 06.02.04 ... */
        {
          if (csr->NullsBehaviour)                          /* 29.03.08 */
            add_assoc_null(return_value,ColName);           /* 29.03.08 */
          else
          {
#if PHP_API_VERSION >= 19990421                             /* 02.03.09 ... */
            if (csr->FetchBoolBehaviour)
              add_assoc_bool(return_value,ColName,0);
            else
#endif                                                      /* ... 02.03.09 */
            {
              add_assoc_string(return_value, ColName, "FALSE", L_TTRUE);
            }
          }
        }
        else                                                /* ... 06.02.04 */
        {
#if PHP_API_VERSION >= 19990421                             /* 02.03.09 ... */
          if (csr->FetchBoolBehaviour)
            add_assoc_bool(return_value,ColName,csr->exch_buff[off]);
          else
#endif                                                      /* ... 02.03.09 */
          {
            if (csr->exch_buff[off] != 0)
              c_val = "TRUE";
            else
              c_val = "FALSE";
            add_assoc_string(return_value, ColName, c_val, L_TTRUE);
          }
        }
        break;
      }
    off += csr->answ_desc[i].Length;                        /* 21.02.99 */
    }
}                                                           /* ... 18.08.00 */

PHP_FUNCTION(linter_get_data_rows)                          /* 02.10.00 ... */
{
  int     i;
  zval   *cid;                                              /* 02.04.12 ... */
  zval   *rows;
  zval   *ret_val;                                          /* ... 02.04.12 */
  CURSOR *csr;

#ifdef _MOD_PHP3_H
  RETURN_LONG(LPE_INVALID_PARAM);
#endif
  if (ARG_COUNT(ht) != 2)
    WRONG_PARAM_COUNT;
  zend_get_parameters(ht, 2, &cid, &rows);
  convert_to_long(cid);
  convert_to_long(rows);
  csr = GetFromList(cid->value.lval);
  if (csr == NULL)
    RETURN_LONG(LPE_INVALID_CURSOR);
  if (csr->last_oper != SQL_SELECT)
    RETURN_LONG(LPE_INVALID_STATE);
  if (rows <= 0)
    RETURN_LONG(LPE_INVALID_PARAM);

  MAKE_STD_ZVAL(ret_val);
  array_init(ret_val);
  for (i = 0; (i < rows->value.lval) && (csr->CBL.CodErr < EORR); i++) /* 28.04.08A */
  {
    if (csr->CBL.CodErr != NOKOR)                               /* 28.04.08A */
    {
      zval   *Child = NULL;                                     /* 02.04.12 24.06.08  ... */
      L_LONG  Ret;

      ALLOC_INIT_ZVAL(Child);

      if (Ret = GetDataRow(Child,csr,0 TSRMLS_CC))              /* 05.03.09 */
        RETURN_LONG(Ret);
      if (Child->type != IS_ARRAY)
        RETURN_LONG(LPE_LINTER_ERROR);
      add_index_zval(ret_val,i,Child);
    }
    FetchCursor(csr, FETCH_NEXT, 0, 0);                         /* ... 24.06.08  */
  }
  if (csr->CBL.CodErr > EORR)                                   /* 28.04.08A */
    RETURN_LONG(LPE_LINTER_ERROR);
  *return_value = *ret_val;
  zval_copy_ctor(return_value);
  zval_dtor(ret_val);
  FREE_ZVAL(ret_val);
}                                                               /* ... 02.10.00 */

/* 14.05.01 ...*/
PHP_FUNCTION(linter_getm)
{
  char *buf = NULL;                                         /* 14.10.14 */
  CURSOR *csr;
  zval   *cid;                                              /* 02.04.12 */
  zval   *ansid;                                            /* 02.04.12 */
  long    ansnum;
  long    rows_inbuf = 0;
  char   *row;
  char   *c_val;
  long    l_val;
  double  d_val;
  char    t_val[64];
  char    ds_val[DECLNG*2+1];                              /* 27.09.04 */
  long    len, off = 0;
  long    i, j = 0, nf = 0;

  if (ARG_COUNT(ht) < 1)
    WRONG_PARAM_COUNT;
  zend_get_parameters(ht, min(ARG_COUNT(ht), 2), &cid, &ansid);
  convert_to_long(cid);
  convert_to_long(ansid);
  ansnum = ansid->value.lval;
  csr = GetFromList(cid->value.lval);
  if (csr == NULL)
    RETURN_LONG(LPE_INVALID_CURSOR);
  if (csr->last_oper != SQL_SELECT)
    RETURN_LONG(LPE_INVALID_STATE);
  if ((ansnum <= 0) || (ansnum > csr->rows))                 /* 28.07.05 ... */
    RETURN_LONG(LPE_INVALID_PARAM);
  rows_inbuf = GETM_BUFF / csr->answ_len;
  if (!rows_inbuf)
  {
    long rc = FetchCursor(csr, FETCH_ABSNUM, ansnum, 0);
    if (rc != LPE_SUCCESS)
      RETURN_LONG(rc);
    ansid->value.lval = 0; /* avoid NullFlag set into GetDataRow */
    PHP_FN(linter_get_data_row)(INTERNAL_FUNCTION_PARAM_PASSTHRU);
    ansid->value.lval = ansnum;
    return;
  }
  if ((ansnum < csr->FstRow) || (ansnum > csr->LstRow ))
  {
    L_LONG SecPos = ((ansnum-1)/rows_inbuf)*rows_inbuf + 1;
    strncpy((char *)csr->CBL.Command, "GETM", 4);
    csr->CBL.RowCount = rows_inbuf;
    csr->CBL.RowId = SecPos;
    if (!csr->getm_buff)
      csr->getm_buff = calloc(GETM_BUFF,1);
    csr->CBL.LnBufRow = GETM_BUFF;
    inter(&csr->CBL, csr->null_flag, NULL, NULL, csr->getm_buff);
    if (csr->CBL.CodErr != 0)
      RETURN_LONG(LPE_LINTER_ERROR);
    csr->FstRow = SecPos;
    csr->LstRow = csr->FstRow + csr->CBL.RowCount - 1;
  }                                                          /* 28.07.05 ... */
  ansnum = (ansnum-1) % rows_inbuf;
  row = (char *)&csr->getm_buff[ansnum*csr->answ_len];
  array_init(return_value);
  for(i = 0; i < csr->cols; i++)
  {
    nf = csr->null_flag[sizeof(L_WORD) * 2 + ansnum * csr->cols + i]; /* 06.02.04 */
    len = csr->answ_desc[i].Length;                        /* 21.12.99 */
    switch(csr->answ_desc[i].Type)
    {
      case LDT_BYTE:
      case LDT_CHAR:
      case LDT_NCHAR:                                     /* 28.06.04 */
        c_val = (char *)&row[off];
        if (nf)                                           /* 06.02.04 ... */
        {
          if (csr->NullsBehaviour)                        /* 29.03.08 */
            add_next_index_null(return_value);            /* 29.03.08 */
          else
            add_next_index_stringl(return_value, "", 0, L_TTRUE);
        }
        else                                              /* ... 06.02.04 */
          add_next_index_stringl(return_value, c_val, len, L_TTRUE);
        break;
      case LDT_EXTFILE:                                     /* 16.10.14 */
        len = strlen(&row[off] + 10);
        c_val = (char *)(&row[off] + 10);
        if (nf)
        {
          if (csr->NullsBehaviour)
            add_next_index_null(return_value);
          else
            add_next_index_stringl(return_value, "", 0, L_TTRUE);
        }
        else
          add_next_index_stringl(return_value, c_val, len, L_TTRUE);
        break;

      case LDT_INTEGER:
        if (csr->answ_desc[i].Length == 4)
#if !defined(LONG_ALIGN)
          l_val = (long)(*(L_LONG *)(&row[off]));
#else                                       /* 14.05.03 ... */
        {
          L_LONG Val;

          memcpy(&Val, &row[off], sizeof(L_LONG));
          l_val = (long) Val;
        }
#endif                                      /* ... 14.05.03 */
        else if (csr->answ_desc[i].Length == 8)
        {                                                           
#if !defined(LONG_ALIGN)
          if (sizeof(long) == 4)                                    /* 14.10.14 */
          {
            buf = emalloc(100);
            buf = i64_ltoa((*(L_DLONG *)(&row[off])), buf);
          }
          else
          {
#ifdef __64BIT_ARCH__
            l_val = (*(L_DLONG *)(&row[off]));
#else    
            l_val = i64_GetLo((*(L_DLONG *)(&row[off])));
#endif
          }
#else                                      /* 14.05.03 ... */
          L_DLONG Val;

          memcpy(&Val, &row[off], sizeof(L_DLONG));
          if (sizeof(long) == 4)                                    /* 14.10.14 */
          {
            buf = emalloc(100);
            buf = i64_ltoa(&Val, buf);
          }
          else
          {
#ifdef __64BIT_ARCH__
            l_val = Val;
#else    
            l_val = i64_GetLo(Val);
#endif
          }
#endif  
        }                                    /* ... 14.05.03 */
        else
#if !defined(LONG_ALIGN)
          l_val = (long)(*(L_WORD *)(&row[off]));
#else                                       /* 14.05.03 ... */
        {
          L_WORD Val;

          memcpy(&Val, &row[off], sizeof(L_WORD));
          l_val = (long)Val;
        }
#endif                                      /* ... 14.05.03 */
        if (nf)                                             /* 06.02.04 ... */
        {
          if (csr->NullsBehaviour)                          /* 29.03.08 */
            add_next_index_null(return_value);              /* 29.03.08 */
          else
            add_next_index_long(return_value, 0);
        }                                                   /* ... 06.02.04 */
        else if (buf)                                       /* 14.10.14 */
        {  
          add_next_index_stringl(return_value, buf, strlen(buf), L_TTRUE);
          s_free(buf);
        }
        else
          add_next_index_long(return_value, l_val);
        break;
      case LDT_REAL:
        if (csr->answ_desc[i].Length == 4)
#if !defined(LONG_ALIGN)
          d_val = (double)(*(L_REAL   *)(&row[off]));
#else                                       /* 14.05.03 ... */
        {
          L_REAL Val;

          memcpy(&Val, &row[off], sizeof(L_REAL));
          d_val = (double)Val;
        }
#endif                                      /* ... 14.05.03 */
        else
#if !defined(LONG_ALIGN)
          d_val = (double)(*(L_DOUBLE *)(&row[off]));
#else                                       /* 14.05.03 ... */
        {
          L_DOUBLE Val;

          memcpy(&Val, &row[off], sizeof(L_DOUBLE));
          d_val = (double)Val;
        }
#endif                                      /* ... 14.05.03 */
        if (nf)                                             /* 06.02.04 ... */
        {
          if (csr->NullsBehaviour)                          /* 29.03.08 */
            add_next_index_null(return_value);              /* 29.03.08 */
          else
            add_next_index_double(return_value, 0.0e0);
        }
        else                                                /* ... 06.02.04 */
          add_next_index_double(return_value, d_val);
        break;
      case LDT_NUMERIC:
        if ( csr->DecimalBehaviour )                         /* 27.09.04 ... */
        {
          DecToDbl(*((L_DECIMAL*)&row[off]), &d_val);
          if (nf)                                              /* 06.02.04 ... */
          {
            if (csr->NullsBehaviour)                           /* 29.03.08 */
              add_next_index_null(return_value);               /* 29.03.08 */
            else
              add_next_index_double(return_value, 0.0e0);
          }
          else                                                 /* ... 06.02.04 */
            add_next_index_double(return_value, d_val);
        }
        else
        {
          DECTOSTR(*((L_DECIMAL*)&row[off]), ds_val, 30, 10, L_TTRUE);
          strrtrim(ds_val," ");                                  /* 27.09.04 */
          if (nf)
          {
            if (csr->NullsBehaviour)                             /* 29.03.08 */
              add_next_index_null(return_value);                 /* 29.03.08 */
            else
              add_next_index_string(return_value, "0", L_TTRUE);
          }
          else
            add_next_index_string(return_value, ds_val, L_TTRUE);
        }                                                        /* ... 27.09.04 */
        break;
      case LDT_DATE:
        TICKTOSTRF(*((L_DECIMAL*)&row[off]), csr->dt_format, t_val);
        if (nf)                                             /* 06.02.04 ... */
        {
          if (csr->NullsBehaviour)                          /* 29.03.08 */
            add_next_index_null(return_value);              /* 29.03.08 */
          else
            add_next_index_string(return_value, "00.00.0000 00:00:00", L_TTRUE);
        }
        else                                                /* ... 06.02.04 */
          add_next_index_string(return_value, t_val, L_TTRUE);
        break;
      case LDT_BLOB:
        add_next_index_long(return_value, 0);
        break;
      case LDT_VARCHAR:
      case LDT_VARBYTE:
      case LDT_NVARCHAR:                                    /* 28.06.04 */
        c_val = (char *)&row[off];
#if !defined(LONG_ALIGN)
        len   = (long)(*(L_WORD *)c_val);
#else                                       /* 14.05.03 ... */
        {
          L_WORD Val;

          memcpy(&Val, c_val, sizeof(L_WORD));
          len   = (long)Val;
        }
#endif                                      /* ... 14.05.03 */
        c_val += sizeof(L_WORD);
        off   += sizeof(L_WORD);
        if (nf)                                             /* 06.02.04 ... */
        {
          if (csr->NullsBehaviour)                          /* 29.03.08 */
            add_next_index_null(return_value);              /* 29.03.08 */
          else
            add_next_index_stringl(return_value, "", 0, L_TTRUE);
        }
        else                                                /* ... 06.02.04 */
          add_next_index_stringl(return_value, c_val, len, L_TTRUE);
        break;
      case LDT_BOOLEAN:                                     /* 02.03.09 */
        if (nf)                                             /* 06.02.04 ... */
        {
          if (csr->NullsBehaviour)                          /* 29.03.08 */
            add_next_index_null(return_value);              /* 29.03.08 */
          else
          {
#if PHP_API_VERSION >= 19990421                             /* 02.03.09 ... */
            if (csr->FetchBoolBehaviour)
              add_next_index_bool(return_value,0);
            else
#endif                                                      /* ... 02.03.09 */
              add_next_index_string(return_value, "FALSE", L_TTRUE);
          }
        }
        else                                                /* ... 06.02.04 */
        {
#if PHP_API_VERSION >= 19990421                             /* 02.03.09 ... */
          if (csr->FetchBoolBehaviour)
            add_next_index_bool(return_value,row[off]);
          else
#endif                                                      /* ... 02.03.09 */
          {
            if (row[off] != 0)
              c_val = "TRUE";
            else
              c_val = "FALSE";
            add_next_index_string(return_value, c_val, L_TTRUE);
          }
        }
        break;
      }
    off += csr->answ_desc[i].Length;                       /* 21.02.99 */
    }
}
/*... 14.05.01 */

PHP_FUNCTION(linter_get_col_prop)
{
  CURSOR *csr;
  zval   *cid, *col;                                        /* 02.04.12 */
  char    c_prop[MAX_ID_LEN + 2];                          /* 27.12.00 */
  long    l_prop, fld;

  if (ARG_COUNT(ht) != 2)
    WRONG_PARAM_COUNT;
  zend_get_parameters(ht, 2, &cid, &col);
  convert_to_long(cid);
  convert_to_long(col);
  csr = GetFromList(cid->value.lval);
  if (csr == NULL)
    RETURN_LONG(LPE_INVALID_CURSOR);
  if (csr->last_oper != SQL_SELECT)
    RETURN_LONG(LPE_INVALID_STATE);
  if (col->value.lval == 0 || col->value.lval > csr->cols)
    RETURN_LONG(LPE_INVALID_COL_NUM);
  fld = col->value.lval - 1;
  object_init(return_value);
  l_prop = (long)csr->null_flag[2 * sizeof(L_WORD) + fld]; /* 27.06.00 */
  add_property_long(return_value, "is_null", l_prop);
  l_prop = (long)csr->answ_desc[fld].Type;
  add_property_long(return_value, "type", l_prop);
  GetTypeName(l_prop,(long)csr->answ_desc[fld].Length,(char *)&c_prop, MAX_ID_LEN + 2);   /* 12.02.09 */  
  add_property_string(return_value, "type_name", c_prop, L_TTRUE);
  l_prop = (long)csr->answ_desc[fld].Length;
  add_property_long(return_value, "length", l_prop);
  l_prop = (long)csr->answ_desc[fld].Precision;
  add_property_long(return_value, "precision", l_prop);
  l_prop = (long)csr->answ_desc[fld].Scale;
  add_property_long(return_value, "scale", l_prop);
  memset(c_prop, 0, sizeof(c_prop));
  memcpy(c_prop, (char *)csr->answ_desc[fld].Column, MAX_ID_LEN); /* 27.12.00 */
  strrtrim(c_prop, " ");
  add_property_string(return_value, "name", c_prop, L_TTRUE);
  memcpy(c_prop, (char *)csr->answ_desc[fld].Table, MAX_ID_LEN);  /* 27.12.00 */
  strrtrim(c_prop, " ");
  add_property_string(return_value, "table", c_prop, L_TTRUE);
  memcpy(c_prop, (char *)csr->answ_desc[fld].User, MAX_ID_LEN);   /* 27.12.00 */
  strrtrim(c_prop, " ");
  add_property_string(return_value, "user", c_prop, L_TTRUE);
}

PHP_FUNCTION(linter_get_param_prop)                         /* 13.10.07 ... */
{
  CURSOR *csr;
  zval   *cid,                                              /* 02.04.12 */
         *col;
  char   *p_name;
  long    l_prop;
  char    c_prop[MAX_ID_LEN + 2];
  L_WORD  Len,Typ,Prc,Scl;

  if (ARG_COUNT(ht) != 2)
    WRONG_PARAM_COUNT;
  cid = col = NULL;
  zend_get_parameters(ht, 2, &cid, &col);
  convert_to_long(cid);
  convert_to_long(col);
  csr = GetFromList(cid->value.lval);
  if (csr == NULL)
    RETURN_LONG(LPE_INVALID_CURSOR);
  if ((col->value.lval > csr->param_count) ||
      (col->value.lval <= 0))
    RETURN_LONG(LPE_INVALID_COL_NUM);
  object_init(return_value);
  p_name = inter_GetParamName(csr->pqry_body, (L_WORD)col->value.lval);
  inter_DescribeParam(csr->pqry_body, (L_WORD)col->value.lval, &Len, &Typ, &Prc, &Scl, NULL);
  l_prop = (long)Typ;
  add_property_long(return_value, "type", l_prop);
  GetTypeName(l_prop,(long)Len,(char *)&c_prop, MAX_ID_LEN+2);  /* 12.02.09 */
  add_property_string(return_value, "type_name", c_prop, L_TTRUE);
  l_prop = (long)Len;
  add_property_long(return_value, "length", l_prop);
  l_prop = (long)Prc;
  add_property_long(return_value, "precision", l_prop);
  l_prop = (long)Scl;
  add_property_long(return_value, "scale", l_prop);
  memset(c_prop, 0, sizeof(c_prop));
  if (p_name)
    strncpy(c_prop, p_name, MAX_ID_LEN);
  else
    *c_prop = 0;
  add_property_string(return_value, "name", c_prop, L_TTRUE);
}                                                               /* ... 13.10.07 */

PHP_FUNCTION(linter_blob_clear)
{
  CURSOR *csr;
  zval   *cid;                                              /* 02.04.12 */

  if (ARG_COUNT(ht) != 1)
    WRONG_PARAM_COUNT;
  zend_get_parameters(ht, 1, &cid);
  convert_to_long(cid);
  csr = GetFromList(cid->value.lval);
  if (csr == NULL)
    RETURN_LONG(LPE_INVALID_CURSOR);
  if (ClearBlob(csr,0))                                          /* 01.03.08 */
  {
    RETURN_LONG(LPE_SUCCESS);
  }
  else
  {
    RETURN_LONG(LPE_LINTER_ERROR);
  }
}

PHP_FUNCTION(linter_blob_append)
{
  CURSOR *csr;
  zval   *cid, *data, *size;                                /* 02.04.12 */
  L_BOOL bRet;

  if (ARG_COUNT(ht) != 3)
    WRONG_PARAM_COUNT;
  zend_get_parameters(ht, 3, &cid, &data, &size);
  convert_to_long(cid);
  csr = GetFromList(cid->value.lval);
  if (csr == NULL)
    RETURN_LONG(LPE_INVALID_CURSOR);
  convert_to_long(size);

  bRet = AddBlob(csr, data->value.str.val, size->value.lval, 0, 0); /* 01.03.08 */
  if (bRet)
  {
    RETURN_LONG(LPE_SUCCESS);
  }
  else
  {
    RETURN_LONG(LPE_LINTER_ERROR);
  }
}

/* 22.11.01 ...*/
PHP_FUNCTION(linter_blob_append_ex)
{
  CURSOR *csr;
  zval   *cid, *data, *size, *type;                         /* 02.04.12 */
  L_BOOL bRet;

  if (ARG_COUNT(ht) != 4)
    WRONG_PARAM_COUNT;
  zend_get_parameters(ht, 4, &cid, &data, &size, &type);
  convert_to_long(cid);
  csr = GetFromList(cid->value.lval);
  if (csr == NULL)
    RETURN_LONG(LPE_INVALID_CURSOR);
  convert_to_long(size);
  convert_to_long(type);

  bRet = AddBlob(csr, data->value.str.val, size->value.lval, type->value.lval, 0); /* 01.03.08 */
  if (bRet)
  {
    RETURN_LONG(LPE_SUCCESS);
  }
  else
  {
    RETURN_LONG(LPE_LINTER_ERROR);
  }
}
/*... 22.11.01 */

PHP_FUNCTION(linter_blob_get_data)
{
  CURSOR *csr;
  zval   *cid, *from, *size;                                /* 02.04.12 */
  char   *blob_value;
  int     res, i;

  if (ARG_COUNT(ht) != 3)
    WRONG_PARAM_COUNT;
  zend_get_parameters(ht, 3, &cid, &from, &size);
  convert_to_long(cid);
  convert_to_long(from);
  convert_to_long(size);

  csr = GetFromList(cid->value.lval);
  if (csr == NULL)
    RETURN_LONG(LPE_INVALID_CURSOR);

  if (csr->last_oper != SQL_SELECT)
    RETURN_LONG(LPE_INVALID_STATE);

  blob_value = (char *)calloc(1, size->value.lval);            /* 20.10.07 ... */
  if (blob_value == NULL)
    RETURN_LONG(LPE_NO_MEMORY);

  for(i = 0; i < csr->cols; i++)
  {
    if (csr->answ_desc[i].Type == LDT_BLOB)
      break;
  }
  if (i == csr->cols) 
    RETURN_LONG(LPE_NO_BLOB_COLUMN);

  res = FetchBlob(csr, from->value.lval, &(size->value.lval), i + 1, blob_value);
  if (res != LPE_SUCCESS)
  {
    free(blob_value);
    RETURN_LONG(res);
  }

  RETVAL_STRINGL(blob_value, size->value.lval, 1);             /* ... 20.10.07  */          
  free(blob_value);
  return;
}

PHP_FUNCTION(linter_blob_purge)
{
  CURSOR *csr;
  zval   *cid, *num;                                        /* 02.04.12 */

  if (ARG_COUNT(ht) != 2)
    WRONG_PARAM_COUNT;
  zend_get_parameters(ht, 2, &cid, &num);
  convert_to_long(cid);
  csr = GetFromList(cid->value.lval);
  if (csr == NULL)
    RETURN_LONG(LPE_INVALID_CURSOR);
  convert_to_long(num);
  if (ClearBlob(csr,num->value.lval))                            /* 01.03.08 */
  {
    RETURN_LONG(LPE_SUCCESS);
  }
  else
  {
    RETURN_LONG(LPE_LINTER_ERROR);
  }
}

PHP_FUNCTION(linter_blob_add)
{
  CURSOR *csr;
  zval   *cid, *data, *size, *num;                          /* 02.04.12 */
  L_BOOL bRet;

  if (ARG_COUNT(ht) != 4)
    WRONG_PARAM_COUNT;

  zend_get_parameters(ht, 4, &cid, &data, &size, &num);
  convert_to_long(cid);
  csr = GetFromList(cid->value.lval);
  if (csr == NULL)
    RETURN_LONG(LPE_INVALID_CURSOR);
  convert_to_long(size);
  convert_to_long(num);

  bRet = AddBlob(csr, data->value.str.val, size->value.lval, 0, num->value.lval); /* 22.11.01 */
  if(bRet)
  {
    RETURN_LONG(LPE_SUCCESS);
  }
  else
  {
    RETURN_LONG(LPE_LINTER_ERROR);
  }
}

/* 22.11.01 ...*/
PHP_FUNCTION(linter_blob_add_ex)
{
  CURSOR *csr;
  zval   *cid, *data, *size, *num, *type;                   /* 02.04.12 */
  L_BOOL bRet;

  if (ARG_COUNT(ht) != 5)
    WRONG_PARAM_COUNT;

  zend_get_parameters(ht, 5, &cid, &data, &size, &num, &type);
  convert_to_long(cid);
  csr = GetFromList(cid->value.lval);
  if (csr == NULL)
    RETURN_LONG(LPE_INVALID_CURSOR);
  convert_to_long(size);
  convert_to_long(num);
  convert_to_long(type);

  bRet = AddBlob(csr, data->value.str.val, size->value.lval, type->value.lval, num->value.lval);
  if(bRet)
  {
    RETURN_LONG(LPE_SUCCESS);
  }
  else
  {
    RETURN_LONG(LPE_LINTER_ERROR);
  }
}
/*... 22.11.01 */

PHP_FUNCTION(linter_blob_fetch)
{
  CURSOR *csr;
  zval   *cid, *from, *size, *num;                          /* 02.04.12 */
  char   *blob_value;
  L_LONG  res;

  if (ARG_COUNT(ht) != 4)
    WRONG_PARAM_COUNT;

  zend_get_parameters(ht, 4, &cid, &from, &size, &num);
  convert_to_long(cid);
  convert_to_long(from);
  convert_to_long(size);
  convert_to_long(num);

  csr = GetFromList(cid->value.lval);
  if (csr == NULL)
    RETURN_LONG(LPE_INVALID_CURSOR);
  
  blob_value = (char *)calloc(1, size->value.lval);
  if (blob_value == NULL)
    RETURN_LONG(LPE_NO_MEMORY);

  res = FetchBlob(csr, from->value.lval, &(size->value.lval), num->value.lval, blob_value);
  if (res != LPE_SUCCESS)
  {
    free(blob_value);
    RETURN_LONG(res);
  }

  RETVAL_STRINGL(blob_value, size->value.lval, 1);
  free(blob_value);
  return;
}

PHP_FUNCTION(linter_blob_get_size)
{
  CURSOR   *csr;
  zval     *cid, *num;                                      /* 02.04.12 */
  int       arg_count = ARG_COUNT(ht);
  L_LONG    blob_size, res, colnum;

  if (arg_count<1 || arg_count>2 ||
      zend_get_parameters(ht, arg_count, &cid, &num)==FAILURE) {
    WRONG_PARAM_COUNT;
  }
  convert_to_long(cid);
  csr = GetFromList(cid->value.lval);
  if (csr == NULL)
    RETURN_LONG(LPE_INVALID_CURSOR);
  if (csr->last_oper != SQL_SELECT)
    RETURN_LONG(LPE_INVALID_STATE);

  if(arg_count == 1)                                        /* 20.10.07 ... */
  {
    for(colnum = 0; colnum < csr->cols; colnum++)
    {
      if (csr->answ_desc[colnum].Type == LDT_BLOB)
        break;
    }
    if (colnum == csr->cols)
      RETURN_LONG(LPE_NO_BLOB_COLUMN);
    colnum++;
  }

  if(arg_count == 2)
  {
    convert_to_long(num);
    colnum = num->value.lval;
  }

  res = GetBlobSize(csr, colnum, &blob_size);                /* ... 20.10.07 */
  if (res != LPE_SUCCESS)
    RETURN_LONG(res)
  else
    RETURN_LONG(blob_size)
}


PHP_FUNCTION(linter_last_error)
{
  CURSOR *csr;
  zval   *cid, *typ;                                        /* 02.04.12 */
#ifdef USE_THREADS                                         /* 09.03.05 ... */
  THREADBUF *buf = GetBuf(tsrm_thread_id());
#endif                                                     /* ... 09.03.05 */

  if (ARG_COUNT(ht) == 1)
  {
    zend_get_parameters(ht, 1, &typ);
#ifdef USE_THREADS                                         /* 09.03.05 ... */
    if (typ->value.lval == SYSTEM_ERROR)
      RETURN_LONG(buf->SysErr);
    RETURN_LONG(buf->CodErr);
#else
    if (typ->value.lval == SYSTEM_ERROR)
      RETURN_LONG(CsrList.CBL.SysErr);
    RETURN_LONG(CsrList.CBL.CodErr);
#endif                                                     /* ... 09.03.05 */
  }
  else if (ARG_COUNT(ht) != 2)
    WRONG_PARAM_COUNT;

  zend_get_parameters(ht, 2, &cid, &typ);
  convert_to_long(cid);
  convert_to_long(typ);
  csr = GetFromList(cid->value.lval);

  if (csr == NULL)
    RETURN_LONG(LPE_INVALID_CURSOR);
  if (typ->value.lval == SYSTEM_ERROR)
    RETURN_LONG(csr->CBL.SysErr);
  RETURN_LONG(csr->CBL.CodErr);
}

PHP_FUNCTION(linter_error_msg)
{
  CURSOR *csr;
  zval   *cid;                                              /* 02.04.12 */
  char    msg[256];
  L_LONG LastCodErr;                                      /* 22.08.00 */

  if (ARG_COUNT(ht) != 1)
    WRONG_PARAM_COUNT;
  zend_get_parameters(ht, 1, &cid);
  convert_to_long(cid);
  csr = GetFromList(cid->value.lval);
  if (csr == NULL)
    RETURN_LONG(LPE_INVALID_CURSOR);;
  memset(msg, 0, sizeof(msg));
  LastCodErr = csr->CBL.CodErr;                            /* 22.08.00 */
  csr->CBL.RowId    = csr->CBL.CodErr;
  csr->CBL.LnBufRow = sizeof(msg) - 1;
  strncpy((char *)csr->CBL.Command, "GETE", 4);
  inter(&csr->CBL, NULL, NULL, NULL, msg);
  if (csr->CBL.CodErr != 0)
    RETURN_LONG(LPE_LINTER_ERROR);
  csr->CBL.CodErr = LastCodErr;                            /* 22.08.00 */
  strltrim(msg, " ");
  RETURN_STRING(msg, L_TTRUE);
}
                                                           /* 11.03.09 */

#if PHP_API_VERSION >= 20010901              /* 27.09.04 ... */
PHP_INI_MH(OnChangeDecimal)
{
  if ( !strcmp( new_value, "1" ) || !strcmp( new_value, "On" ) || !strcmp( new_value, "on" ))
    glReturnDecimalAsDouble = 1;
  else
    glReturnDecimalAsDouble = 0; 
  return(SUCCESS);
}
PHP_INI_MH(OnChangeBlob)
{
  if ( !strcmp( new_value, "1" ) || !strcmp( new_value, "On" ) || !strcmp( new_value, "on" ))
    SetFetchBlobAsDataFlag(1);               /* 22.01.09 */
  else
    SetFetchBlobAsDataFlag(0);               /* 22.01.09 */
  return(SUCCESS);
}
PHP_INI_MH(OnChangeBlobStream)               /* 05.03.09 ... */
{
  if ( !strcmp( new_value, "1" ) || !strcmp( new_value, "On" ) || !strcmp( new_value, "on" ))
    SetFetchBlobAsStreamFlag(1);
  else
    SetFetchBlobAsStreamFlag(0);
  return(SUCCESS);
}                                            /* ... 05.03.09 */
PHP_INI_MH(OnChangeBool)                     /* 02.03.09... */
{
  if ( !strcmp( new_value, "1" ) || !strcmp( new_value, "On" ) || !strcmp( new_value, "on" ))
    SetFetchBoolAsBoolFlag(1);
  else
    SetFetchBoolAsBoolFlag(0);
  return(SUCCESS);
}                                            /* ... 02.03.09 */
PHP_INI_MH(OnChangeNulls)
{
  if ( !strcmp( new_value, "1" ) || !strcmp( new_value, "On" ) || !strcmp( new_value, "on" ))
    glNullsAsNullObject = 1;
  else
    glNullsAsNullObject = 0;
  return(SUCCESS);
}
PHP_INI_MH(OnChangeDateF)
{
  SetLinterDateFormat(new_value,0);          /* 22.01.09 */
  return(SUCCESS);
}

PHP_INI_MH(OnChangeGenerateColName)          /* 30.03.09 ... */
{
  if ( !strcmp( new_value, "1" ) || !strcmp( new_value, "On" ) || !strcmp( new_value, "on" ))
    glGenerateColName = 1;
  else
    glGenerateColName = 0;
  return(SUCCESS);
}

PHP_INI_MH(OnChangeRenameDubbedCol)
{
  if ( !strcmp( new_value, "1" ) || !strcmp( new_value, "On" ) || !strcmp( new_value, "on" ))
    glRenameDubbedCol = 1;
  else
    glRenameDubbedCol = 0;
  return(SUCCESS);
}                                            /* ... 30.03.09 */

#endif                                       /* ... 27.09.04 */

DLLEXP zend_module_entry *get_module(void)
{
  return (&linter_module_entry);
}

static PHP_MINIT_FUNCTION(linter)
{
  MUTEX_INIT(mutex1);                                            /* 29.10.04 */         

  glReturnDecimalAsDouble = 0;                                   /* 30.03.08 */
  SetFetchBlobAsDataFlag(0);                                     /* 30.03.08 */ /* 22.01.09 */
  SetFetchBlobAsStreamFlag(0);                                   /* 05.03.09 */
  SetFetchBoolAsBoolFlag(0);                                     /* 02.03.09 */
  SetGenerateColNameFlag(0);                                     /* 30.03.09 */
  SetRenameDubbedColFlag(0);                                     /* 30.03.09 */
  glNullsAsNullObject = 0;                                       /* 30.03.08 */  
  SetLinterDateFormat(DEF_DT_FORMAT,1);                          /* 22.01.09 */

  {
#if PHP_API_VERSION >= 20010901                              /* 27.09.04 ... */
  PHP_INI_BEGIN()
    PHP_INI_ENTRY("linter.decimal_as_double","Off", PHP_INI_SYSTEM, OnChangeDecimal)
    PHP_INI_ENTRY("linter.fetch_blob_as_data","Off", PHP_INI_SYSTEM, OnChangeBlob)
    PHP_INI_ENTRY("linter.fetch_blob_as_stream","Off", PHP_INI_SYSTEM, OnChangeBlobStream)  /* 05.03.09 */
    PHP_INI_ENTRY("linter.fetch_bool_as_bool","Off", PHP_INI_SYSTEM, OnChangeBool)          /* 02.03.09 */     
    PHP_INI_ENTRY("linter.generate_col_name","Off", PHP_INI_SYSTEM, OnChangeGenerateColName)/* 30.03.09 */
    PHP_INI_ENTRY("linter.rename_dubbed_col","Off", PHP_INI_SYSTEM, OnChangeRenameDubbedCol)/* 30.03.09 */
    PHP_INI_ENTRY("linter.null_as_null_object","Off", PHP_INI_SYSTEM, OnChangeNulls)
    PHP_INI_ENTRY("linter.date_format",DEF_DT_FORMAT, PHP_INI_SYSTEM, OnChangeDateF)
  PHP_INI_END()
  REGISTER_INI_ENTRIES();
#endif                                                       /* ... 27.09.04 */
  }

  REGISTER_LONG_CONSTANT("LDT_CHAR",     LDT_CHAR,     CONST_CS | CONST_PERSISTENT);
  REGISTER_LONG_CONSTANT("LDT_INTEGER",  LDT_INTEGER,  CONST_CS | CONST_PERSISTENT);
  REGISTER_LONG_CONSTANT("LDT_REAL",     LDT_REAL,     CONST_CS | CONST_PERSISTENT);
  REGISTER_LONG_CONSTANT("LDT_DATE",     LDT_DATE,     CONST_CS | CONST_PERSISTENT);
  REGISTER_LONG_CONSTANT("LDT_NUMERIC",  LDT_NUMERIC,  CONST_CS | CONST_PERSISTENT);
  REGISTER_LONG_CONSTANT("LDT_BYTE",     LDT_BYTE,     CONST_CS | CONST_PERSISTENT);
  REGISTER_LONG_CONSTANT("LDT_BLOB",     LDT_BLOB,     CONST_CS | CONST_PERSISTENT);
  REGISTER_LONG_CONSTANT("LDT_VARCHAR",  LDT_VARCHAR,  CONST_CS | CONST_PERSISTENT);
  REGISTER_LONG_CONSTANT("LDT_VARBYTE",  LDT_VARBYTE,  CONST_CS | CONST_PERSISTENT);
  REGISTER_LONG_CONSTANT("LDT_BOOLEAN",  LDT_BOOLEAN,  CONST_CS | CONST_PERSISTENT);
  REGISTER_LONG_CONSTANT("LDT_NCHAR",    LDT_NCHAR,    CONST_CS | CONST_PERSISTENT);
  REGISTER_LONG_CONSTANT("LDT_NVARCHAR", LDT_NVARCHAR, CONST_CS | CONST_PERSISTENT);
  REGISTER_LONG_CONSTANT("LDT_EXTFILE",  LDT_EXTFILE,  CONST_CS | CONST_PERSISTENT); /* 15.10.14 */

  REGISTER_LONG_CONSTANT("FETCH_FIRST",  FETCH_FIRST,  CONST_CS | CONST_PERSISTENT);
  REGISTER_LONG_CONSTANT("FETCH_LAST",   FETCH_LAST,   CONST_CS | CONST_PERSISTENT);
  REGISTER_LONG_CONSTANT("FETCH_NEXT",   FETCH_NEXT,   CONST_CS | CONST_PERSISTENT);
  REGISTER_LONG_CONSTANT("FETCH_PREV",   FETCH_PREV,   CONST_CS | CONST_PERSISTENT);
  REGISTER_LONG_CONSTANT("FETCH_ABSNUM", FETCH_ABSNUM, CONST_CS | CONST_PERSISTENT);

  REGISTER_LONG_CONSTANT("TM_AUTOCOMMIT", 0x00000000,   CONST_CS | CONST_PERSISTENT);
  REGISTER_LONG_CONSTANT("TM_OPTIMISTIC", M_OPTIMISTIC, CONST_CS | CONST_PERSISTENT);
  REGISTER_LONG_CONSTANT("TM_EXCLUSIVE",  M_EXCLUSIVE,  CONST_CS | CONST_PERSISTENT);

  REGISTER_LONG_CONSTANT("CP_866",  Q_866,      CONST_CS | CONST_PERSISTENT); /* 19.04.00 */
  REGISTER_LONG_CONSTANT("CP_1251", Q_ENCODE,   CONST_CS | CONST_PERSISTENT);
  REGISTER_LONG_CONSTANT("CP_KOI8", Q_KOI8,     CONST_CS | CONST_PERSISTENT);

  REGISTER_LONG_CONSTANT("CP_OEM",  Q_866,      CONST_CS | CONST_PERSISTENT); /* 27.09.02 */
  REGISTER_LONG_CONSTANT("CP_ANSI", Q_ENCODE,   CONST_CS | CONST_PERSISTENT);
  REGISTER_LONG_CONSTANT("CP_ISO",  Q_KOI8,     CONST_CS | CONST_PERSISTENT);

  REGISTER_LONG_CONSTANT("TIL_OPTIMISTIC",       TIL_OPTIMISTIC,       CONST_CS | CONST_PERSISTENT); /* 13.10.04 ... */
  REGISTER_LONG_CONSTANT("TIL_READ_COMMITTED",   TIL_READ_COMMITTED,   CONST_CS | CONST_PERSISTENT);
  REGISTER_LONG_CONSTANT("TIL_READ_UNCOMMITTED", TIL_READ_UNCOMMITTED, CONST_CS | CONST_PERSISTENT);
  REGISTER_LONG_CONSTANT("TIL_SERIALIZABLE",     TIL_SERIALIZABLE,     CONST_CS | CONST_PERSISTENT); /* ... 13.10.04 */

  REGISTER_LONG_CONSTANT("CO_NAME",       CO_NAME,       CONST_CS | CONST_PERSISTENT);
  REGISTER_LONG_CONSTANT("CO_ASYNC_MODE", CO_ASYNC_MODE, CONST_CS | CONST_PERSISTENT);
  REGISTER_LONG_CONSTANT("CO_SYNC_MODE",  CO_SYNC_MODE,  CONST_CS | CONST_PERSISTENT);
  REGISTER_LONG_CONSTANT("CO_DT_FORMAT",  CO_DT_FORMAT,  CONST_CS | CONST_PERSISTENT);
  REGISTER_LONG_CONSTANT("CO_COL_COUNT",  CO_COL_COUNT,  CONST_CS | CONST_PERSISTENT);
  REGISTER_LONG_CONSTANT("CO_ROW_COUNT",  CO_ROW_COUNT,  CONST_CS | CONST_PERSISTENT);
  REGISTER_LONG_CONSTANT("CO_ERR_ROW",    CO_ERR_ROW,    CONST_CS | CONST_PERSISTENT);
  REGISTER_LONG_CONSTANT("CO_ERR_POS",    CO_ERR_POS,    CONST_CS | CONST_PERSISTENT);
  REGISTER_LONG_CONSTANT("CO_TRANS_MODE", CO_TRANS_MODE, CONST_CS | CONST_PERSISTENT);
  REGISTER_LONG_CONSTANT("CO_CUR_ROW",    CO_CUR_ROW,    CONST_CS | CONST_PERSISTENT);
  REGISTER_LONG_CONSTANT("CO_CUR_ROWID",  CO_CUR_ROWID,  CONST_CS | CONST_PERSISTENT);
  REGISTER_LONG_CONSTANT("CO_CONNECT_ID", CO_CONNECT_ID, CONST_CS | CONST_PERSISTENT);
  REGISTER_LONG_CONSTANT("CO_NODE_NAME",  CO_NODE_NAME,  CONST_CS | CONST_PERSISTENT);
  REGISTER_LONG_CONSTANT("CO_LAST_ROWID", CO_LAST_ROWID, CONST_CS | CONST_PERSISTENT);
  REGISTER_LONG_CONSTANT("CO_UPDATABLE",  CO_UPDATABLE,  CONST_CS | CONST_PERSISTENT); /* 04.03.04 */
  REGISTER_LONG_CONSTANT("CO_QUERY_TYPE", CO_QUERY_TYPE, CONST_CS | CONST_PERSISTENT); /* 04.03.04 */
  REGISTER_LONG_CONSTANT("CO_PARAM_COUNT", CO_PARAM_COUNT, CONST_CS | CONST_PERSISTENT); /* 13.10.07 */
  REGISTER_LONG_CONSTANT("CO_FETCH_BLOBS_AS_USUAL_DATA", CO_FETCH_BLOBS_AS_USUAL_DATA, CONST_CS | CONST_PERSISTENT);     /* 20.10.07 */
  REGISTER_LONG_CONSTANT("CO_FETCH_BLOBS_AS_STREAM", CO_FETCH_BLOBS_AS_STREAM, CONST_CS | CONST_PERSISTENT);             /* 20.10.07 */
  REGISTER_LONG_CONSTANT("CO_FETCH_BOOL_AS_BOOL", CO_FETCH_BOOL_AS_BOOL, CONST_CS | CONST_PERSISTENT);                   /* 02.03.09 */
  REGISTER_LONG_CONSTANT("CO_NULL_AS_NULL_OBJECT", CO_NULL_AS_NULL_OBJECT, CONST_CS | CONST_PERSISTENT);
  REGISTER_LONG_CONSTANT("CO_DECIMAL_AS_DOUBLE", CO_DECIMAL_AS_DOUBLE, CONST_CS | CONST_PERSISTENT);
  REGISTER_LONG_CONSTANT("CO_GENERATE_COL_NAME", CO_GENERATE_COL_NAME, CONST_CS | CONST_PERSISTENT);                     /* 30.03.09 */
  REGISTER_LONG_CONSTANT("CO_RENAME_DUBBED_COL", CO_RENAME_DUBBED_COL, CONST_CS | CONST_PERSISTENT);                     /* 30.03.09 */

  REGISTER_LONG_CONSTANT("LINTER_ERROR", PHPLINTER_ERROR, CONST_CS | CONST_PERSISTENT);
  REGISTER_LONG_CONSTANT("SYSTEM_ERROR", SYSTEM_ERROR, CONST_CS | CONST_PERSISTENT);

  REGISTER_LONG_CONSTANT("LPE_SUCCESS",            LPE_SUCCESS,            CONST_CS | CONST_PERSISTENT);
  REGISTER_LONG_CONSTANT("LPE_LINTER_ERROR",       LPE_LINTER_ERROR,       CONST_CS | CONST_PERSISTENT);
  REGISTER_LONG_CONSTANT("LPE_NO_MEMORY",          LPE_NO_MEMORY,          CONST_CS | CONST_PERSISTENT);
  REGISTER_LONG_CONSTANT("LPE_INVALID_PARAM",      LPE_INVALID_PARAM,      CONST_CS | CONST_PERSISTENT);
  REGISTER_LONG_CONSTANT("LPE_INVALID_CONNECT",    LPE_INVALID_CONNECT,    CONST_CS | CONST_PERSISTENT);
  REGISTER_LONG_CONSTANT("LPE_INVALID_CURSOR",     LPE_INVALID_CURSOR,     CONST_CS | CONST_PERSISTENT);
  REGISTER_LONG_CONSTANT("LPE_INVALID_COL_NUM",    LPE_INVALID_COL_NUM,    CONST_CS | CONST_PERSISTENT);
  REGISTER_LONG_CONSTANT("LPE_INVALID_PARAM_NUM",  LPE_INVALID_PARAM_NUM,  CONST_CS | CONST_PERSISTENT);
  REGISTER_LONG_CONSTANT("LPE_INVALID_PARAM_NAME", LPE_INVALID_PARAM_NAME, CONST_CS | CONST_PERSISTENT);
  REGISTER_LONG_CONSTANT("LPE_INVALID_STATE",      LPE_INVALID_STATE,      CONST_CS | CONST_PERSISTENT);
  REGISTER_LONG_CONSTANT("LPE_INVALID_DIRECT",     LPE_INVALID_DIRECT,     CONST_CS | CONST_PERSISTENT);
  REGISTER_LONG_CONSTANT("LPE_UNKNOWN_OPTION",     LPE_UNKNOWN_OPTION,     CONST_CS | CONST_PERSISTENT);
  REGISTER_LONG_CONSTANT("LPE_BAD_DT_FORMAT",      LPE_BAD_DT_FORMAT,      CONST_CS | CONST_PERSISTENT);
  REGISTER_LONG_CONSTANT("LPE_NO_BLOB_COLUMN",     LPE_NO_BLOB_COLUMN,     CONST_CS | CONST_PERSISTENT);
  REGISTER_LONG_CONSTANT("LPE_NOT_CURSOR",         LPE_NOT_CURSOR,         CONST_CS | CONST_PERSISTENT);
  REGISTER_LONG_CONSTANT("LPE_WRONG_PARAM_COUNT",  LPE_WRONG_PARAM_COUNT,  CONST_CS | CONST_PERSISTENT);
  REGISTER_LONG_CONSTANT("LPE_NOT_IMPLEMENTED",    LPE_NOT_IMPLEMENTED,    CONST_CS | CONST_PERSISTENT);

  REGISTER_LONG_CONSTANT("LQT_SQL_UNKNOWN",    SQL_UNKNOWN,      CONST_CS | CONST_PERSISTENT);
  REGISTER_LONG_CONSTANT("LQT_SQL_SELECT",     SQL_SELECT,       CONST_CS | CONST_PERSISTENT);
  REGISTER_LONG_CONSTANT("LQT_SQL_EXEC_PROC",  SQL_EXEC_PROC,    CONST_CS | CONST_PERSISTENT);
  REGISTER_LONG_CONSTANT("LQT_SQL_GRANT",      SQL_GRANT,        CONST_CS | CONST_PERSISTENT);
  REGISTER_LONG_CONSTANT("LQT_SQL_ALTER",      SQL_ALTER,        CONST_CS | CONST_PERSISTENT);

#ifdef PDO                                                 /* 22.01.09 ... */

  REGISTER_PDO_CLASS_CONST_LONG("ATTR_DT_FORMAT",               (long)PDO_ATTR_DT_FORMAT);           /* 12.02.09 */  
  REGISTER_PDO_CLASS_CONST_LONG("ATTR_GENERATE_COL_NAME",       (long)PDO_ATTR_GENERATE_COL_NAME);   /* 30.03.09 */
  REGISTER_PDO_CLASS_CONST_LONG("ATTR_RENAME_DUBBED_COL",       (long)PDO_ATTR_RENAME_DUBBED_COL);   /* 30.03.09 */

  php_pdo_register_driver(&pdo_linter_driver);

#endif                                                     /* ... 22.01.09 */

  return (SUCCESS);
}

static PHP_RINIT_FUNCTION(linter)
{
#ifdef USE_THREADS                                         /* 29.10.04 */
  GetBuf( tsrm_thread_id() );
#endif
  return (SUCCESS);
}

static PHP_MSHUTDOWN_FUNCTION(linter)
{
  s_free(glDateFormat);
  UninitLinterClient();
  MUTEX_DESTROY(mutex1);                                   /* 29.10.04 */

#ifdef PDO
  php_pdo_unregister_driver(&pdo_linter_driver);           /* 22.01.09 */
#endif
  return (SUCCESS);
}

static PHP_RSHUTDOWN_FUNCTION(linter)
{
  CURSOR *csr,*pred;

#ifdef USE_THREADS                                           /* 29.10.04 ... */
  DelBuf( tsrm_thread_id() );
#endif
  MUTEX_LOCK(mutex1);
  csr = CsrList.next;
  pred = &CsrList;
  while(csr)
  {
#ifdef USE_THREADS
    if ( csr->thread_id == tsrm_thread_id() )
    {
#endif
      pred->next = csr->next;
      if (csr->parent == NULL)
      {
        strncpy((char *)csr->CBL.Command, "CLOS", 4);
        inter(&csr->CBL, NULL, NULL, NULL, NULL);
      }
      ClearCursorDesc(csr, 1);
      s_free(csr);
      csr = pred;
#ifdef USE_THREADS
    }
#endif
    pred = csr; csr = csr->next;
  }

  MUTEX_UNLOCK(mutex1);                                      /* ... 29.10.04 */

  return (SUCCESS);
}

static PHP_MINFO_FUNCTION(linter)                          /* 29.08.13 */
{
/* 28.11.00 ...*/
#ifdef PHP4
  char  buf[32];
  php_info_print_table_start();
  php_info_print_table_header(2, "Linter SQL Server support", "enabled");
  sprintf(buf, "%d.%d", _VER_MAX/100, (_VER_MAX/10)%10);
  php_info_print_table_row(2, "version", buf);
#if PHP_API_VERSION >= 20010901                              /* 27.09.04 ... */
  php_info_print_table_row(2, "decimal_as_double", glReturnDecimalAsDouble?"On":"Off");
  php_info_print_table_row(2, "fetch_blob_as_data", GetFetchBlobAsDataFlag()?"On":"Off");
  php_info_print_table_row(2, "fetch_blob_as_stream", GetFetchBlobAsStreamFlag()?"On":"Off"); /* 05.03.09 */
  php_info_print_table_row(2, "fetch_bool_as_bool", GetFetchBoolAsBoolFlag()?"On":"Off");     /* 02.03.09 */
  php_info_print_table_row(2, "generate_col_name", GetGenerateColNameFlag()?"On":"Off");      /* 30.03.09 */
  php_info_print_table_row(2, "rename_dubbed_col", GetRenameDubbedColFlag()?"On":"Off");      /* 30.03.09 */
  php_info_print_table_row(2, "null_as_null_object", glNullsAsNullObject?"On":"Off");
  php_info_print_table_row(2, "date_format", GetLinterDateFormat());
#endif
  php_info_print_table_end();
#else
  php3_printf("%s\n", "PHP3 Linter SQL Server extension");
#endif
/*... 28.11.00 */
}
