 /***************************************************************************
 *           RELEX, Inc. Client source include file                         *
 *  This is unpublished propietary source code of RELEX, Inc.               *
 *  The copyright notice does not evidence any actual or intended           *
 *  publication of such source code.                                        *
 *                                                                          *
 *                 Copyright (c) 1996-2008 RELEX Inc.                       *
 *                                                                          *
 *                   This software was developed by                         *
 *                                                                          *
 *                            RELEX, Inc.                                   *
 *                         Voronezh, Russia                                 *
 *                                                                          *
 *                        All rights reserved.                              *
 ****************************************************************************
 FILE:    php_linter.h
 ----------------------------------------------------------------------------
 AUTHOR:  Alexander V. Kefeli
 ----------------------------------------------------------------------------
 PURPOSE: Interface from PHP language to Linter SQL Server
 ----------------------------------------------------------------------------
 REVISION HISTORY:

 22.07.99 -- Kent (file creation)
 16.12.99 -- Kent (New cursor options for 5.7)
 25.02.00 -- Kent (LAST ROWID cursor option)
 19.04.00 -- Kononyhin (change CP_866)
 18.08.00 -- Kononyhin (add associative array)
 02.10.00 -- Kent (Add `LinPHP_GetDataRows')
 30.10.00 -- Kent (New prototype LinPHP_Info)
 27.03.01 -- Kononyhin (working with multiple blobs)
 14.05.01 -- Kononyhin (using GETM)
 12.09.01 -- Kononyhin (add LDT_EXTFILE)
 22.11.01 -- Kononyhin
 17.12.01 -- Kononyhin (building php4.1.0 for ZEND_MODULE_API_NO >= 20010901)
 04.03.04 -- Ermakov   (new UPDATABLE && QUERY_TYPE options)
 05.11.04 -- Ermakov   (Correct LINTER_ERROR)
 01.02.06 -- Ermakov, Gluhov (new function LinPHP_ConnectInfo ) 
 13.10.07 -- Ermakov   (Added LINTER_GetParamProp)
 20.10.07 -- Gluhov    (Corrected work with blobs, added FETCH_BLOBS_AS_USUAL_DATA cursor option)
 29.03.08 -- Ermakov   (Added CO_NULL_AS_NULL_OBJECT)
 24.06.08 -- Ermakov   (Removed old compat. code)
 22.01.09 -- Glukhov   (Added pdo interface)
 12.02.09 -- Glukhov   (GetTypeName function added)
 17.02.09 -- Glukhov   (ChanLink added) 
 02.03.09 -- Glukhov   (added CO_FETCH_BOOL_AS_BOOL cursor option )
 05.03.09 -- Glukhov   (added CO_FETCH_BLOBS_AS_STREAM cursor option )
 30.03.09 -- Glukhov   (GENERATE_COL_NAME and RENAME_DUBBED_COL settings added)
 07.03.09 -- Glukhov   (Unnecessary functions' prototypes are removed from this)
 14.09.14 -- Fatkhudinov (binded_param_count added)
 21.11.14.-- Galizin   (Prototype like in c file)

 --------------------------------------------------------------------------*/

#ifndef __PHP_LINTER_INT_H__
#define __PHP_LINTER_INT_H__
#if defined(USE_THREADS) && !defined(ZTS)
#Zend Thread Safety if off. Theading is not support
#endif

/* Linter datatypes */
#define LDT_CHAR                1
#define LDT_INTEGER             2
#define LDT_REAL                3
#define LDT_DATE                4
#define LDT_NUMERIC             5
#define LDT_BYTE                6
#define LDT_BLOB                7
#define LDT_VARCHAR             8
#define LDT_VARBYTE             9
#define LDT_BOOLEAN             10
#define LDT_NCHAR               11
#define LDT_NVARCHAR            12
#define LDT_EXTFILE             13                            /* 12.09.01 */

#define LDT_MAX                 12

#define PAGE_BUFF      4096                                /* 13.02.01 */

#define SQL_UNKNOWN    0
#define SQL_SELECT     1
#define SQL_EXEC_PROC  52
#define SQL_GRANT      47
#define SQL_ALTER      42
#define SQL_TSTTBL     112


/* Fetch directions */
#define FETCH_FIRST             1
#define FETCH_LAST              2
#define FETCH_NEXT              3
#define FETCH_PREV              4
#define FETCH_ABSNUM            5

/* Transaction modes */
#define TM_AUTOCOMMIT           0x00000000
#define TM_OPTIMISTIC           0x00000100
#define TM_EXCLUSIVE            0x00000400

/* Transaction isolation levels */    /* 13.09.04 */
#define TIL_OPTIMISTIC        0x00000100
#define TIL_READ_COMMITTED    0x01000000  
#define TIL_READ_UNCOMMITTED  0x04000000  
#define TIL_SERIALIZABLE      0x02000000

/* Supported codepages */
#define CP_1251                 0x00004000
#define CP_KOI8                 0x00008000
#define CP_866                  CP_ENCODE+CP_KOI8 /* 19.04.00 */

/* Connect/Cursor options */
#define CO_NAME                 1
#define CO_ASYNC_MODE           2
#define CO_SYNC_MODE            3
#define CO_DT_FORMAT            4
#define CO_COL_COUNT            5
#define CO_ROW_COUNT            6
#define CO_ERR_ROW              7
#define CO_ERR_POS              8
#define CO_TRANS_MODE           9
#define CO_CUR_ROW              10
#define CO_CUR_ROWID            11
#define CO_CONNECT_ID           12
#define CO_NODE_NAME            13
#define CO_PRIORITY             14                         /* 16.12.99 ... */
#define CO_QUERY_PRIORITY       15
#define	CO_CANCEL               16
#define CO_PAUSE                17
#define CO_CONTINUE             18                         /* ... 16.12.99 */
#define CO_LAST_ROWID           19                         /* 25.02.00 */
#define CO_UPDATABLE            20                         /* 04.03.04 */
#define CO_QUERY_TYPE           21                         /* 04.03.04 */
#define CO_PARAM_COUNT          22                         /* 13.10.07 */
#define CO_FETCH_BLOBS_AS_USUAL_DATA 23                    /* 20.10.07 */
#define CO_NULL_AS_NULL_OBJECT  24                         /* 29.03.08 */
#define CO_DECIMAL_AS_DOUBLE    25                         /* 29.03.08 */
#define CO_FETCH_BOOL_AS_BOOL   26                         /* 02.03.09 */
#define CO_FETCH_BLOBS_AS_STREAM 27                        /* 05.03.09 */
#define CO_GENERATE_COL_NAME    28                         /* 30.03.09 */
#define CO_RENAME_DUBBED_COL    29                         /* 30.03.09 */

/* Error type for `Linter_LastError' */
#define PHPLINTER_ERROR          0                         /* 19.11.04 */
#define SYSTEM_ERROR             1

/* Module internal errors */
#define LPE_SUCCESS              0
#define LPE_LINTER_ERROR        -1
#define LPE_NO_MEMORY           -2
#define LPE_INVALID_PARAM       -3
#define LPE_INVALID_CONNECT     -4
#define LPE_INVALID_CURSOR      -5
#define LPE_INVALID_COL_NUM     -6
#define LPE_INVALID_PARAM_NUM   -7
#define LPE_INVALID_PARAM_NAME  -8
#define LPE_INVALID_STATE       -9
#define LPE_INVALID_DIRECT      -10
#define LPE_UNKNOWN_OPTION      -11
#define LPE_BAD_DT_FORMAT       -12
#define LPE_NO_BLOB_COLUMN      -13
#define LPE_NOT_CURSOR          -14
#define LPE_WRONG_PARAM_COUNT   -15
#define LPE_NOT_IMPLEMENTED     -999

#define MAX_BUFF       8192
#define GETM_BUFF      4096                                /* 14.05.01 */
#if _VER_MAX >= 600                                        /* 27.02.08 ... */
#define MAX_ANSWER     0xFFFF
#else
#define MAX_ANSWER     MAX_BUFF
#endif                                                     /* ... 27.02.08 */

#if defined(WIN32) && (!defined (_MSC_VER) || (_MSC_VER < 1400))
#ifdef snprintf
#undef snprintf
#endif

#  define snprintf _snprintf
#endif
#ifndef min
#  define min(a, b) (a < b ? a : b)
#endif
#ifndef s_free
#  define s_free(p) { if (p) free(p); p = NULL; }         /* 22.01.09 */
#endif

typedef struct                                             /* 22.01.09 .... */
{
  const char *file;
  L_LONG      line;
  L_LONG      err_code;
  char        err_msg[256];
} pdo_linter_error_info;                                   /* ... 22.01.09 */

typedef struct t_CURSOR CURSOR;

typedef struct ChanLink                                    /* 17.02.09 ... */
{
  CURSOR * subCsr;  
  struct ChanLink *Next;
} t_ChanLink;                                              /* ... 17.02.09 */

struct t_CURSOR {
  TCBL       CBL;
  long       id;                                           /* 17.11.00 */
  long       cols;
  long       rows;
  long       crow;
  long       errpos;        
  long       last_oper;
  long       answ_len;                                           /* 14.05.01 ...*/
  long      *ColOffset;                                          /* 22.01.09 */
  long       FstRow;                                             /* 28.07.05 */
  long       LstRow;                                             /* 28.07.05 */
  char      *getm_buff;                                          /*... 14.05.01 */
  char       name[MAX_ID_LEN+2];
  char      *dt_format;
  char       exch_buff[MAX_ANSWER];
  char       null_flag[sizeof(L_WORD) * 2 + GETM_BUFF];
  long       pass_cnt;
  char      *pqry_body;
  short     *parm_leng; /* Parameter length - used for prepare/bind/execute */
  short      param_count;                                           /* 13.10.07 */
  short      binded_param_count;                                    /* 14.09.14*/
  GETA_OUT  *answ_desc;
  CURSOR    *next;
  CURSOR    *parent;
  L_LONG     Flags;                                                 /* 05.11.04 */
  OPEN_DESC  Dsc;
  L_LONG     DscLen;
  DB_DESC    DbDsc;
  L_LONG     DbDscLen;
#ifdef USE_THREADS
  THREAD_T   thread_id;                                          /* 29.10.04 */
#endif
  L_BOOL     BlFlag;                                             /* 22.01.09 */
  L_BOOL     FetchBlobBehaviour;                                 /* 20.10.07 */
  L_BOOL     FetchBlobStreamBehaviour;                           /* 05.03.09 */
  L_BOOL     FetchBoolBehaviour;                                 /* 02.03.09 */
  L_BOOL     NullsBehaviour;                                     /* 29.03.08 */
  L_BOOL     DecimalBehaviour;                                   /* 30.03.08 */
  L_BOOL     GenerateColNameBehaviour;                           /* 30.03.09 */
  L_BOOL     RenameDubbedColBehaviour;                           /* 30.03.09 */
  L_BOOL     fetch_table_names;                                  /* 22.01.09 .. */
  L_BOOL     fetch_catalog_names;   
  pdo_linter_error_info err_info;                                /* .. 22.01.09 */
  t_ChanLink *ChanLink;                                          /* 17.02.09 */
};

extern long GetDataColInteger(CURSOR *csr, long i, long off, char *str_val); /* 21.11.14 */
extern double GetDataColReal(CURSOR *csr, long i, long off);
extern int    GetTypeName(long type, long type_length, char *type_name, long max_name_len);               /* 12.02.09 */

extern void ClearCursorDesc(CURSOR *csr, int all);
extern long FillCursorDesc(CURSOR *csr, int DontFree);
extern long FetchCursor(CURSOR *csr, long direct, long pos, long DontCall);
extern L_LONG FetchBlob(CURSOR *csr, L_LONG blob_start, L_LONG *blob_size, L_LONG colnum, char *blob_value);
extern L_LONG GetBlobSize(CURSOR *csr, L_LONG colnum, L_LONG* blob_size);


extern L_BOOL ClearBlob(CURSOR *csr, L_LONG column);
extern L_BOOL AddBlob(CURSOR *csr, char* data, L_LONG blob_size, L_LONG blob_type, L_LONG colnum);

#endif /* __PHP_LINTER_INT_H__ */

