 /***************************************************************************
 *                 RELEX, Inc. Linter Python driver source code             *
 *        This is unpublished proprietary source code of RELEX, Inc.        *
 *     The copyright notice does not evidence any actual or intended        *
 *                  publication of such source code.                        *
 *                                                                          *
 *                 Copyright (c) 2003-2013 RELEX Inc.                       *
 *                                                                          *
 *                   This software was developed by                         *
 *                                                                          *
 *                            RELEX, Inc.                                   *
 *                         Voronezh, Russia                                 *
 *                                                                          *
 *                        All rights reserved.                              *
 ****************************************************************************
 AUTHOR: Boris Fedorov (fbv@relex.ru)
 ----------------------------------------------------------------------------
 PURPOSE: 
 ----------------------------------------------------------------------------
 REVISION HISTORY:
 31.10.03 -- Boris Fedorov (Removed C++ style comments, disable unicode 
 support for Python older whan 2.2)
 29.07.08 -- Ermakov (Corrected warnings)
 22.09.08 -- Ermakov (Added date and unicode configuration)
 25.12.13 -- Muratova (Python 3 support)
*/

#ifndef ARRAY_H
#define ARRAY_H

struct _t_ArrayType;

typedef struct
{
  PyObject_HEAD
  t_ParamDesc desc;
  int         index;
  PyObject   *boundName;
  t_Cursor   *boundCursor;
  L_LONG      allocatedElements;
  L_LONG      actualElements;
  char       *indicator;
  L_LONG     *actualLength;
  int         maxLength;
  struct _t_ArrayType *type;
  char       *data;
  L_LONG      blobPortion;
} t_Array;

typedef int (*InitializeProc)(t_Array*, t_Cursor*);
typedef void (*FinalizeProc)(t_Array*);
typedef int (*SetValueProc)(t_Array*, unsigned, PyObject*);
typedef PyObject * (*GetValueProc)(t_Array*, unsigned);
typedef int (*BindAnswerProc)(t_Array*, t_Cursor*);
typedef int (*BeforePosChangedProc)(t_Array*, t_Cursor*);
typedef int (*AfterPosChangedProc)(t_Array*);

typedef struct _t_ArrayType
{
  InitializeProc initializeProc;
  FinalizeProc finalizeProc;
  SetValueProc setValueProc;
  GetValueProc getValueProc;
  BindAnswerProc bindAnswerProc;

  BeforePosChangedProc beforePosChanged;
  AfterPosChangedProc afterPosChanged;

  PyTypeObject *pythonType;
  int linterType;
  int elementLength;
  int isVariableLength;
} t_ArrayType;

extern PyTypeObject g_ArrayType;                             /* 25.12.13 */

#define Array_ELEMENT(a,p) (((char*)(a)->data)+((p)*(a)->maxLength))

extern void Array_Free(
  t_Array *var);

extern t_Array *Array_Define(
  t_Cursor *cursor,
  unsigned numElements,
  unsigned column);

extern t_Array *Array_New(
  t_Cursor *cursor,
  unsigned numElements,
  t_ArrayType *type,
  t_ParamDesc *desc);

extern t_Array *Array_NewParameter(
  t_Cursor *cursor,
  int index,
  int arraySize);

t_Array *Array_NewByType(
  t_Cursor *cursor,
  PyObject *value,
  unsigned numElements);

extern int Array_Check(
  PyObject *object);

extern int Array_Resize(
  t_Array *var,
  unsigned maxLength);

extern int Array_SetValue(
  t_Array *var,
  unsigned arrayPos,
  PyObject *value);

extern PyObject *Array_GetValue(
  t_Array *var,
  unsigned arrayPos);

extern int Array_GetData(
  t_Array *array,
  t_Cursor *cursor);

extern int Array_BindNamedParameter(
  t_Array *var,
  t_Cursor *cursor,
  PyObject *name);

extern int Array_BindQmarkParameter(
  t_Array *var,
  t_Cursor *cursor,
  int index);

extern t_ArrayType *Array_TypeByPythonType(
  PyObject *type);

extern t_ArrayType *Array_TypeByDataType(
  int dataType);

extern int Array_InternalBindAnswer(
  t_Array *var,
  t_Cursor *cursor);

extern PyTypeObject g_StringType;
extern PyTypeObject g_BinaryType;
extern PyTypeObject g_NumberType;

extern PyObject *g_DateEmulation;                           /* 22.09.08 */
extern PyObject *g_UnicodeData;                             /* 22.09.08 */

extern t_ArrayType vt_Char;
extern t_ArrayType vt_VarChar;
#if (PY_VERSION_HEX >= 0x02020000)                          /* 31.10.03 */
extern t_ArrayType vt_NChar;
extern t_ArrayType vt_NVarChar;
#endif                                                     /* 31.10.03 */
extern t_ArrayType vt_Byte;
extern t_ArrayType vt_VarByte;
extern t_ArrayType vt_SmallInt;
extern t_ArrayType vt_Integer;
extern t_ArrayType vt_BigInt;
extern t_ArrayType vt_Boolean;                              /* 29.07.08 */
extern t_ArrayType vt_Real;
extern t_ArrayType vt_Double;
extern t_ArrayType vt_Decimal;
extern t_ArrayType vt_Timestamp;
extern t_ArrayType vt_Blob;

#endif /* ARRAY_H */
