"""
LINTER database backend for Django.
"""
from django.db.backends import *
                                        
from django.db.backends.linter import query
from django.db.backends.linter.client import DatabaseClient
from django.db.backends.linter.creation import DatabaseCreation
from django.db.backends.linter.introspection import DatabaseIntrospection
from django.utils.encoding import smart_str, force_unicode

try:
    import LinPy as Database
except ImportError, e:
    from django.core.exceptions import ImproperlyConfigured
    raise ImproperlyConfigured, "Error loading LinPy module: %s" % e

DatabaseError = Database.Error
IntegrityError = Database.IntegrityError

class DatabaseFeatures(BaseDatabaseFeatures):
    uses_custom_query_class = True
   
class DatabaseOperations(BaseDatabaseOperations):

    _lookup_types_dict = {'day' : 'D', 'month' : 'M', 'year' : 'Y'}

    def date_extract_sql(self, lookup_type, table_name):      
        return "DATESPLIT (%s, '%s')" % (table_name, self._lookup_types_dict[lookup_type])

    def date_trunc_sql(self, lookup_type, field_name):
        return "TRUNC(%s, '%s')" % (field_name, self._lookup_types_dict[lookup_type])

    def drop_foreignkey_sql(self):
        return "DROP FOREIGN KEY"

    def fulltext_search_sql(self, field_name):
        return "%s CONTAINS '%%s'" % field_name

    def last_executed_query(self, cursor, sql, params):
        """
        Returns a string of the query last executed by the given cursor, with
        placeholders replaced with actual values.
        """
        from django.utils.encoding import smart_unicode, force_unicode

        if not params:
            return smart_unicode(sql)

        # Convert params to contain Unicode values.
        to_unicode = lambda s: force_unicode(s, strings_only=True)
        if isinstance(params, (list, tuple)):
            u_params = tuple([to_unicode(val) for val in params])
        else:
            u_params = dict([(to_unicode(k), to_unicode(v)) for k, v in params.items()])

        return smart_unicode(sql) % u_params

    def last_insert_id(self, cursor, table_name, pk_name):
        query = "SELECT %s FROM %s WHERE ROWID = LAST_ROWID" % (self.quote_name(pk_name), self.quote_name(table_name)) # 23.09.
        cursor.execute(query)
        return cursor.fetchone()[0]

    def lookup_cast(self, lookup_type):
        if lookup_type in ('iexact', 'icontains', 'iregex', 'istartswith', 'iendswith'):
            return "UPPER(%s)"
	return "%s"

    def no_limit_value(self):
        return -1

    def query_class(self, DefaultQueryClass):
        return query.query_class(DefaultQueryClass, Database)

    def quote_name(self, name):   
        if name.startswith('"') and name.endswith('"'):
            return name # Quoting once is enough.
        return '"%s"' % name.upper()
    
    def random_function_sql(self):
        return "RAND()"

    def sql_flush(self, style, tables, sequences):  
        # Return a list of 'TRUNCATE x;', 'TRUNCATE y;',
        # 'TRUNCATE z;'... style SQL statements          
	
        sql = ['%s %s %s;' % \
                (style.SQL_KEYWORD('TRUNCATE'),
                 style.SQL_KEYWORD('TABLE'),
                 style.SQL_FIELD(self.quote_name(table))
                 )  for table in tables]
        return sql

class DatabaseWrapper(BaseDatabaseWrapper):

    operators = {
        'exact': '= %s',
        'iexact': '= UPPER(%s)',
        'contains': 'LIKE %s',
        'icontains': 'LIKE UPPER(%s)',
        'regex': 'SIMILAR TO %s',
        'iregex': 'SIMILAR TO UPPER(%s)',
        'gt': '> %s',
        'gte': '>= %s',
        'lt': '< %s',
        'lte': '<= %s',
        'startswith': 'LIKE %s',
        'endswith': 'LIKE %s',                                  
        'istartswith': 'LIKE UPPER(%s)',
        'iendswith': 'LIKE UPPER(%s)',
    }
    
    def __init__(self, **kwargs):
        super(DatabaseWrapper, self).__init__(**kwargs)

        self.features = DatabaseFeatures()   
        self.ops = DatabaseOperations()
        self.client = DatabaseClient()
        self.creation = DatabaseCreation(self)
        self.introspection = DatabaseIntrospection(self)
        self.validation = BaseDatabaseValidation()

    def _valid_connection(self):
        return self.connection is not None

    def _cursor(self, settings):
	# SetUnicodeData(1) - set output data mode in unicode
        Database.SetUnicodeData(1)
        from django.conf import settings

        if not self._valid_connection():
            self.connection = Database.connect(settings.DATABASE_USER, settings.DATABASE_PASSWORD, settings.DATABASE_NAME, **self.options)
        cursor = self.connection.cursor(mode=Database.M_EXCLUSIVE)   
        cursor = LinterCursorWrapper(cursor)
        return cursor

    def _commit(self):
	if self.connection is not None:
            self.connection.commit()
                       
    def _rollback(self):
        if self.connection is not None:
            try:
                self.connection.rollback()
            except Database.NotSupportedError:
                pass

    def set_date_emulation(self, value=0):
	# set emulation mode (compatibility mode) DateTime,
        # on default - always TimeStamp.
	Database.SetDateEmulation(value)

class LinterCursorWrapper(object):
    """
    Django uses "format" style placeholders, but LinPy uses "qmark" style.
    This fixes it -- but note that if you want to use a literal "%s" in a query,
    you'll need to use "%%s".
    """
    def __init__(self, cursor):
        self.cursor = cursor
                                         
    def __getattr__(self, attr):
        if self.__dict__.has_key(attr):
            return self.__dict__[attr]
        else:
            return getattr(self.cursor, attr)

    def execute(self, query, params=()):
        query = self.convert_query(query, len(params))

        try:
            return self.cursor.execute(query, params)    
	except DatabaseError, e:
	    if type(e) != IntegrityError:
		e = IntegrityError(e)
	raise e

    def executemany(self, query, param_list):
        try:
            query = self.convert_query(query, len(param_list[0]))
            return self.cursor.executemany(query, param_list)
        except (IndexError,TypeError):
            # No parameter list provided
            return None
                                          
    def convert_query(self, query, num_params):
        if num_params:
            return smart_str(query % tuple("?" * num_params))
        else: 	   
            return smart_str(query)
