import sys
from django.conf import settings
from django.db.backends.creation import BaseDatabaseCreation

TEST_USER_PREFIX = 'TEST_'

class DatabaseCreation(BaseDatabaseCreation):
    # This dictionary maps Field objects to their associated LINTER column
    # types, as strings. Column-type strings can contain format strings; they'll
    # be interpolated against the values of Field.__dict__ before being output.
    # If a column type is set to None, it won't be included in the output.
    data_types = {
        'AutoField':       	        'integer AUTOINC',
        'BooleanField':     	        'boolean',
        'CharField':           	        'nvarchar(%(max_length)s)',
        'CommaSeparatedIntegerField':   'varchar(%(max_length)s)',
        'DateField':     	        'date',
        'DateTimeField':    		'date',
        'DecimalField':                 'number(%(max_digits)s, %(decimal_places)s)',
        'FileField':       		'varchar(%(max_length)s)',
        'FilePathField':  		'varchar(%(max_length)s)',
        'FloatField':     		'double',
        'IntegerField':     		'integer',
        'IPAddressField':  		'char(15)',
        'NullBooleanField':		'integer',
        'OneToOneField':    		'integer',
        'PositiveIntegerField': 	'integer',
        'PositiveSmallIntegerField':	'smallint',
        'SlugField':        		'varchar(%(max_length)s)',
        'SmallIntegerField': 		'smallint',
        'TextField':        		'varchar(1000)',    
        'TimeField':        		'date',
        'URLField':                     'varchar(%(max_length)s)',
    }

    remember = {}

    def _create_test_db(self, verbosity, autoclobber):
        """
        We don't created test database, we only create test user 
        """

	TEST_DATABASE_USER = TEST_USER_PREFIX + settings.DATABASE_USER
	TEST_DATABASE_PASSWD = settings.DATABASE_PASSWORD

        parameters = {
            'user' : TEST_DATABASE_USER,
	    'password' : TEST_DATABASE_PASSWD,
	}

	self.remember['user'] = settings.DATABASE_USER
	self.remember['passwd'] = settings.DATABASE_PASSWORD

	cursor = self.connection.cursor()

        if verbosity >= 1:
     	    print "Creating test user..."
	try:
	    self._create_test_user(cursor, parameters, verbosity) 
	except Exception, e:
	    sys.stderr.write("Got an error creating the test user: %s\n" % e)
	    if not autoclobber:
	        confirm = raw_input("It appears the test user, %s, already exists. Type 'yes'  to delete it? or 'no' to cancel: " % TEST_DATABASE_USER)
	    if autoclobber or confirm == 'yes':
	        try:
	    	    if verbosity >= 1:
		        print "Destroying old test user..."
		    self._destroy_test_user(cursor, parameters, verbosity) 
		    if verbosity >= 1:
		        print "Creating test user..."
		    self._create_test_user(cursor, parameters, verbosity)  
		except Exception, e:
		    sys.stderr.write("Got an error recreating the test user: %s\n" % e)
		    sys.exit(2)
    	    else:
	        print "Tests cancelled."
	        sys.exit(1)

	settings.DATABASE_USER = TEST_DATABASE_USER

        return settings.DATABASE_NAME

    def _destroy_test_db(self, test_database_name, verbosity):
	"""
	Don't destroy database. Desrtoy test user.
	"""
	TEST_DATABASE_USER = settings.DATABASE_USER
	TEST_DATABASE_PASSWD = settings.DATABASE_PASSWORD

	parameters = {
	    'user' : TEST_DATABASE_USER,
	    'password' : TEST_DATABASE_PASSWD
	}

        settings.DATABASE_USER = self.remember['user']
        settings.DATABASE_PASSWORD = self.remember['passwd']

	cursor = self.connection.cursor()

	if verbosity >= 1:
	    print "Destroying test user..."
	self._destroy_test_user(cursor, parameters, verbosity)
	
	self.connection.close()

    def _create_test_user(self, cursor, parameters, verbosity):
	if verbosity >= 2:
	    print "_crete_test_user(): username = %s" % parameters['user']
        statements = [
	    "CREATE USER %(user)s IDENTIFIED BY '%(password)s'", 
	    "GRANT RESOURCE TO %(user)s"
	]
	self._execute_statements(cursor, statements, parameters, verbosity)
    
    def _destroy_test_user(self, cursor, parameters, verbosity):
	if verbosity >= 2:
	    print "_destroy_test_user(): user = %s" % parameters['user']
	statements = ["DROP USER %(user)s CASCADE"]
	
	self._execute_statements(cursor, statements, parameters, verbosity)
	    
    def _execute_statements(self, cursor, statements, parameters, verbosity):
	for template in statements:
	    stmt = template % parameters
	    if verbosity >= 2:
		print stmt
	    try:
		cursor.execute(stmt)
	    except Exception, err:
		sys.stderr.write("Failed (%s)\n" % (err))
		raise

    def sql_for_pending_references(self, model, style, pending_references):
        """
	Returns any ALTER TABLE statements to add constraints after the fact.
	"""                                                                  
        qn = self.connection.ops.quote_name
        final_output = []
        opts = model._meta
        if model in pending_references:
            for rel_class, f in pending_references[model]:
                rel_opts = rel_class._meta
                r_table = rel_opts.db_table
                r_col = f.column
                table = opts.db_table
                col = opts.get_field(f.rel.field_name).column
                final_output.append(style.SQL_KEYWORD('ALTER TABLE') + ' %s ADD FOREIGN KEY (%s) REFERENCES %s (%s)%s;' % \
                    (qn(r_table), 
                    qn(r_col), qn(table), qn(col),
                    self.connection.ops.deferrable_sql()))
            del pending_references[model]
        return final_output

    def sql_indexes_for_field(self, model, f, style):
        """
	Return the CREATE INDEX SQL statements for a single model field
	""" 
        from django.db.models.fields.related import ForeignKey
	# LINTER create index for primary keys, unique and foreign keys 
	# automatically
        if f.db_index and not f.unique and (not isinstance(f, ForeignKey)):
            qn = self.connection.ops.quote_name
            tablespace = f.db_tablespace or model._meta.db_tablespace
            if tablespace:
                sql = self.connection.ops.tablespace_sql(tablespace)
                if sql:
                    tablespace_sql = ' ' + sql
                else:
                    tablespace_sql = ''
            else:
                tablespace_sql = ''
            output = [style.SQL_KEYWORD('CREATE INDEX') + ' ' +
                style.SQL_TABLE(qn('%s_%s' % (model._meta.db_table, f.column))) + ' ' +
                style.SQL_KEYWORD('ON') + ' ' +
                style.SQL_TABLE(qn(model._meta.db_table)) + ' ' +
                "(%s)" % style.SQL_FIELD(qn(f.column)) +
                "%s;" % tablespace_sql]
        else:
            output = []
        return output           

    def sql_remove_table_constraints(self, model, references_to_delete, style):
        from django.db.backends.util import truncate_name

        output = []
        qn = self.connection.ops.quote_name
        for rel_class, f in references_to_delete[model]:
            table = rel_class._meta.db_table
            col = f.column
            output.append('%s %s %s (%s);' % \
                (style.SQL_KEYWORD('ALTER TABLE'),
                style.SQL_TABLE(qn(table)),
                style.SQL_KEYWORD(self.connection.ops.drop_foreignkey_sql()),
                style.SQL_FIELD(col)))
        del references_to_delete[model]
        return output
