from django.db.backends import BaseDatabaseIntrospection
import string

class DatabaseIntrospection(BaseDatabaseIntrospection):
    # Maps type codes to Django Field types.
    data_types_reverse = {
        'CHAR': 'CharField',
        'NCHAR': 'CharField',
        'VARCHAR': 'CharField',   
        'NACHAR VARYING': 'CharField',
        'SMALLINT': 'SmallIntegerField',
        'INTEGER': 'IntegerField',
        'BIGINT': 'IntegerField',
        'REAL': 'FloatField', 
        'DOUBLE': 'FloatField',
        'NUMERIC': 'FloatField',
        'BOOLEAN': 'BooleanField',
        'DATE': 'DateField',
        'DATE': 'DateFieldWrapper',
        'BLOB': 'TextField',
        'EXTFILE': 'CharField',
    }

    def get_table_list(self, cursor):
        "Returns a list of table names in the current database."
        cursor.execute("""
            SELECT TABLE_NAME  
            FROM LINTER_SYSTEM_USER.TABLES 
            WHERE TABLE_SCHEM = USER AND TABLE_TYPE ='TABLE' 
            AND TABLE_NAME NOT LIKE '$$$%' AND TABLE_NAME NOT LIKE 'L\_%' 
            AND TABLE_NAME NOT IN ('SERVERS','PRIV_TYPES','TYPEINFO','COUNTER6',
            'COUNTER8','COUNTER31','PROVIDER_TYPES','ERRORS')""")
        return ([string.lower(row[0]) for row in cursor.fetchall()])

    def get_table_description(self, cursor, table_name):
	qn = self.connection.ops.quote_name
        cursor.execute("SELECT * FROM %s LIMIT 1" % qn(table_name))
        return cursor.description

    def _name_to_index(self, cursor, table_name):
        """
        Returns a dictionary of {field_name: field_index} for the given table.
        Indexes are 0-based.
        """
        return dict([(d[0], i) for i, d in enumerate(self.get_table_description(cursor, table_name))])

    def get_relations(self, cursor, table_name):
        """
        Returns a dictionary of {field_index: (field_index_other_table, other_table)}
        representing all relationships to the given table. Indexes are 0-based.
        """
        my_field_dict = self._name_to_index(cursor, table_name)
        relations = {}

        cursor.execute("""
            SELECT FKCOLUMN_NAME, PKTABLE_NAME, PKCOLUMN_NAME 
            FROM LINTER_SYSTEM_USER.FOREIGN_KEYS 
            WHERE FKTABLE_NAME = %s""" % dbo.quote_name(table_name))
        for row in cursor.fetchall():
            other_field_index = self._name_to_index(cursor, row[1])[row[2]]
            my_field_index = my_field_dict[row[0]]
            relations[my_field_index] = (other_field_index, row[1])

        return relations	

    def get_indexes(self, cursor, table_name):
        """
        Returns a dictionary of fieldname -> infodict for the given table,
        where each infodict is in the format:
            {'primary_key': boolean representing whether it's the primary key,
             'unique': boolean representing whether it's a unique index}
        """
        indexes = {}
     
        cursor.execute("""
            SELECT NON_UNIQUE, COLUMN_NAME 
            FROM LINTER_SYSTEM_USER.TABLESTATISTICS 
            WHERE TABLE_NAME = %s""" % dbo.quote_name(table_name))
        for row in cursor.fetchall():
            if row[0] != None:
                indexes[row[1]] = {'primary_key': bool(row[1]), 'unique': not bool(row[0])}

        return indexes

