"""
Custom Query class for Linter.
Derives from: django.db.models.sql.query.Query
"""

import datetime

from django.db.backends import util

# Cache. Maps default query class to new Linter query class.
_classes = {}

def query_class(QueryClass, Database):
    """
    Returns a custom django.db.models.sql.query.Query subclass that is
    appropriate for Linter.

    """
    global _classes
    try:
        return _classes[QueryClass]
    except KeyError:
        pass

    class LinterQuery(QueryClass):

        def as_sql(self, with_limits=True, with_col_aliases=False):
            """
            Creates the SQL for this query. Returns the SQL string and list
            of parameters.  This is overriden from the original Query class
            to accommodate Linter's limit/offset SQL.

            If 'with_limits' is False, any limit/offset information is not
            included in the query.
            """

            # The `do_offset` flag indicates whether we need to construct
            # the SQL needed to use limit/offset.        
            do_offset = with_limits and (self.high_mark is not None
                                         or self.low_mark)

            # If no offsets, just return the result of the base class
            # `as_sql`.
            if not do_offset:
                return super(LinterQuery, self).as_sql(with_limits=False,
                        with_col_aliases=with_col_aliases)

            # `get_columns` needs to be called before `get_ordering` to
            # populate `_select_alias`.
            self.pre_sql_setup()
            out_cols = self.get_columns()
            ordering = self.get_ordering()

            sql, params = super(LinterQuery, self).as_sql(with_limits=False,
                    with_col_aliases=True)

            # Constructing the result SQL, using the initial select SQL
            # obtained above.
            result =['%s' % sql]      

            if self.low_mark:
	        if self.high_mark == self.low_mark:
		    return None
	        elif self.high_mark is not None:
	            result.append('LIMIT %d, %d' % (self.low_mark, (self.high_mark - self.low_mark)))
    	        else:
		    val = self.connection.ops.no_limit_value()
                    if val:
    	                result.append('LIMIT %d, %d' % (self.low_mark, val))
 	    else:
                result.append('LIMIT %d' % self.high_mark)	

            # Returning the SQL w/params.
            return ' '.join(result), params

    _classes[QueryClass] = LinterQuery
    return LinterQuery
