 /***************************************************************************
 *                 RELEX, Inc. Linter Python driver source code             *
 *        This is unpublished propietary source code of RELEX, Inc.         *
 *     The copyright notice does not evidence any actual or intended        *
 *                  publication of such source code.                        *
 *                                                                          *
 *                 Copyright (c) 2003-2014 RELEX Inc.                       *
 *                                                                          *
 *                   This software was developed by                         *
 *                                                                          *
 *                            RELEX, Inc.                                   *
 *                         Voronezh, Russia                                 *
 *                                                                          *
 *                        All rights reserved.                              *
 ****************************************************************************
 AUTHOR: Boris Fedorov
 ----------------------------------------------------------------------------
 PURPOSE: Error interface (DBAPI)
 ----------------------------------------------------------------------------
 REVISION HISTORY:
 31.10.03 -- Boris Fedorov (Remove C++ style comments)
 25.12.13 -- Muratova (Python 3 support)
 30.09.14 -- Kononov (Removed an unreferenced local variables, fixed compiler warnings)
 06.10.14 -- Kononov (Fixed support for Python < 2.6)
*/

#undef _DEBUG
#include <Python.h>
#include <linapi.h>
#include "LinPy.h"

static void Error_Free(t_Error*);
static int Error_Init(t_Error*);                             /* 25.12.13 */
static PyObject *Error_Str(t_Error*);
#if PY_VERSION_HEX < 0x02060000                              /* 06.10.14 ... */
static PyObject *Error_GetAttr(t_Error*, PyObject*);
#endif                                                       /* ... 06.10.14 */

//-----------------------------------------------------------------------------
// declaration of members
//-----------------------------------------------------------------------------
#if PY_VERSION_HEX >= 0x02060000                             /* 06.10.14 ...*/
static PyMemberDef g_ErrorMembers[] = {                      /* 25.12.13 ... */
    { "code", T_INT, offsetof(t_Error, apiCode), READONLY },
    { "apiCode", T_INT, offsetof(t_Error, apiCode), READONLY },
    { "linCode", T_INT, offsetof(t_Error, linCode), READONLY },
    { "sysCode", T_INT, offsetof(t_Error, sysCode), READONLY },
    { "message", T_OBJECT, offsetof(t_Error, message), READONLY },
    { NULL }
};                                                           /* ... 25.12.13 */
#endif                                                       /* ... 06.10.14 */

PyTypeObject g_ErrorType = {
  PyVarObject_HEAD_INIT(NULL, 0)                         /* 25.12.13 */
  "Error",                       /* tp_name */
  sizeof(t_Error),               /* tp_basicsize */
  0,                             /* tp_itemsize */
  (destructor) Error_Free,       /* tp_dealloc */
  0,                             /* tp_print */
  0,                             /* tp_getattr */
  0,                             /* tp_setattr */
  0,                             /* tp_compare */
  0,                             /* tp_repr */
  0,                             /* tp_as_number */
  0,                             /* tp_as_sequence */
  0,                             /* tp_as_mapping */
  0,                             /* tp_hash */
  0,                             /* tp_call */
  0,                             /* tp_str */
#if PY_VERSION_HEX >= 0x02060000                         /* 06.10.14 ... */
  0,                             /* tp_getattro*/        /* 25.12.13 ... */
#else
  (getattrofunc) Error_GetAttr,  /* tp_getattro*/
#endif                                                   /* ... 06.10.14 */
  0,                             /* tp_setattro */
  0,                             /* tp_as_buffer */
  Py_TPFLAGS_DEFAULT,            /* tp_flags */
  0,                             /* tp_doc */
  0,                             /* tp_traverse */
  0,                             /* tp_clear */
  0,                             /* tp_richcompare */
  0,                             /* tp_weaklistoffset */
  0,                             /* tp_iter */
  0,                             /* tp_iternext */
  0,                             /* tp_methods */
#if PY_VERSION_HEX >= 0x02060000                         /* 06.10.14 ... */
  g_ErrorMembers,                /* tp_members */
#else
  0,
#endif                                                   /* ... 06.10.14 */
  0,                             /* tp_getset */
  0,                             /* tp_base */
  0,                             /* tp_dict */
  0,                             /* tp_descr_get */
  0,                             /* tp_descr_set */
  0,                             /* tp_dictoffset */
#if PY_VERSION_HEX >= 0x02060000                         /* 06.10.14 ... */
  (initproc) Error_Init,         /* tp_init */
#else
  0,                             /* tp_init */
#endif                                                   /* ... 06.10.14 */
  0,                             /* tp_alloc */
  (newfunc) Error_New,           /* tp_new */            /* 30.09.14 */
  0,                             /* tp_free */
  0,                             /* tp_is_gc */
  0                              /* tp_bases */
};                                                       /* ... 25.12.13  */


/**
 * Create error.
 */
#if PY_VERSION_HEX >= 0x02060000         /* 06.10.14 */
PyObject *Error_New(PyTypeObject *type)  /* 25.12.13 */
{
#else
t_Error *Error_New()                     /* 06.10.14 */
{
  t_Error *error;
#endif
  trace(("Error_New()\n"));
#if PY_VERSION_HEX >= 0x02060000
  return type->tp_alloc(type, 0);
#else                                    /* 06.10.14 ... */
  error = PyObject_NEW(t_Error, &g_ErrorType);
  if (error == NULL)
    return NULL;
  Error_Init(error);
  return error;
#endif                                   /* ... 06.10.14 */
}


static int Error_Init(t_Error *error)                         /* 25.12.13 ... */
{
  error->code = -1; 
  error->apiCode = -1;
  error->linCode = -1;
  error->sysCode = -1;

  trace(("Error_Init(): error %08x\n", error));

  return 0;
}                                                            /* ... 25.12.13 */


/**
 * Destroy error object
 */
static void Error_Free(t_Error *error)
{
  trace(("Error_Free(): error %08x\n", error));
  PyObject_DEL(error);
}

#if PY_VERSION_HEX < 0x02060000                                        /* 06.10.14 ... */
/**
 * Retrives error attributes
 */
static PyObject *Error_GetAttr(t_Error *error, PyObject *nameObject)
{
  char *name;

  name = PyString_AS_STRING(nameObject);
  
  if (name[0] == 'c' && strcmp(name, "code") == 0)
    return PyInt_FromLong(error->linCode);

  if (name[0] == 'm' && strcmp(name, "message") == 0)
    return PyString_FromString(PyString_AS_STRING(error->message));

  if (name[0] == 'l' && strcmp(name, "linCode") == 0)
    return PyInt_FromLong(error->linCode);
  
  if (name[0] == 'a' && strcmp(name, "apiCode") == 0)
    return PyInt_FromLong(error->apiCode);

  if (name[0] == 's' && strcmp(name, "sysCode") == 0)
    return PyInt_FromLong(error->sysCode);

  PyErr_SetObject(PyExc_AttributeError, nameObject);
  return NULL;
}
#endif                                                                 /* ... 06.10.14 */
