 /***************************************************************************
 *                 RELEX, Inc. Linter Python driver source code             *
 *        This is unpublished proprietary source code of RELEX, Inc.        *
 *     The copyright notice does not evidence any actual or intended        *
 *                  publication of such source code.                        *
 *                                                                          *
 *                 Copyright (c) 2003-2014 RELEX Inc.                       *
 *                                                                          *
 *                   This software was developed by                         *
 *                                                                          *
 *                            RELEX, Inc.                                   *
 *                         Voronezh, Russia                                 *
 *                                                                          *
 *                        All rights reserved.                              *
 ****************************************************************************
 AUTHOR: Boris Fedorov (fbv@relex.ru)
 ----------------------------------------------------------------------------
 PURPOSE: 
 ----------------------------------------------------------------------------
 REVISION HISTORY:
 31.10.03 -- Boris Fedorov (Remove C++ style comments)
 04.09.08 -- Ermakov ( added native DATETIME support (2.4 and higher) )
 25.12.13 -- Muratova (Python 3 support)
 30.09.14 -- Kononov (Fixed Python 3.x support)
*/

#ifndef LINPY_H
#define LINPY_H

#include <stdarg.h>

#include "Connection.h"
#include "Cursor.h"
#include "Error.h"
#include "Array.h"
#include "Timestamp.h"
#include "Blob.h"

#if PY_VERSION_HEX >= 0x02060000
  #include "structmember.h"
  #include <datetime.h>
#endif

// define Py_TYPE for versions before Python 2.6 (support Python 2.5 or earlier)
#ifndef Py_TYPE
  #define Py_TYPE(ob) (((PyObject*)(ob))->ob_type)
#endif 
#ifndef PyVarObject_HEAD_INIT
    #define PyVarObject_HEAD_INIT(type, size) \
        PyObject_HEAD_INIT(type) size,
#endif
 
#if PY_MAJOR_VERSION >= 3                                         /* 25.12.13 ... */
  #define PyExc_StandardError PyExc_Exception  
  #define INIT_LINPY PyInit_LinPy
  #define INITERROR return NULL
  
  #define PyString_Check PyBytes_Check
  #define PyString_FromStringAndSize PyUnicode_FromStringAndSize
  #define PyString_GET_SIZE PyBytes_GET_SIZE
  #define PyString_FromString PyUnicode_FromString                /* 30.09.14 */
  #define PyString_AS_STRING PyBytes_AS_STRING

  #define PyInt_FromLong PyLong_FromLong
  #define PyInt_Check(op) PyLong_Check(op)
  #define PyInt_AS_LONG PyLong_AS_LONG
#else
  #define INIT_LINPY initLinPy
  #define INITERROR return
#endif
// define macro for adding objects
#define ADD_OBJECT(name, type) \
    Py_INCREF(type); \
    if (PyModule_AddObject(module, name, (PyObject*) type) < 0) \
        INITERROR;                                                /* ... 25.12.13 */

extern PyObject *g_WarningException;
extern PyObject *g_ErrorException;
extern PyObject *g_InterfaceErrorException;
extern PyObject *g_DatabaseErrorException;
extern PyObject *g_DataErrorException;
extern PyObject *g_OperationalErrorException;
extern PyObject *g_IntegrityErrorException;
extern PyObject *g_InternalErrorException;
extern PyObject *g_ProgrammingErrorException;
extern PyObject *g_NotSupportedErrorException;
#if PY_VERSION_HEX < 0x02060000
extern PyObject *g_ModuleDictionary;
extern void InitTimeStampObject();
#endif
extern void TimeStampInit();                                     /* 04.09.08 */
#ifdef DEBUG
void linter_trace(char *fmt, ...);
#define trace(a) linter_trace a
#else
#define trace(a)
#endif

#endif /* LINPY_H */
