 /***************************************************************************
 *                 RELEX, Inc. Linter Python driver source code             *
 *        This is unpublished proprietary source code of RELEX, Inc.        *
 *     The copyright notice does not evidence any actual or intended        *
 *                  publication of such source code.                        *
 *                                                                          *
 *                 Copyright (c) 2003-2014 RELEX Inc.                       *
 *                                                                          *
 *                   This software was developed by                         *
 *                                                                          *
 *                            RELEX, Inc.                                   *
 *                         Voronezh, Russia                                 *
 *                                                                          *
 *                        All rights reserved.                              *
 ****************************************************************************
 AUTHOR: Boris Fedorov
 ----------------------------------------------------------------------------
 PURPOSE: String interface (DBAPI)
 ----------------------------------------------------------------------------
 REVISION HISTORY:
 31.10.03 -- Boris Fedorov (Removed C++ style comments, disable unicode 
 support for Python older whan 2.2)
 29.07.08 -- Ermakov (Corrected warnings)
 04.09.08 -- Ermakov ( added unicode string support )
 22.09.08 -- Ermakov (Added date and unicode configuration)
 28.11.08 -- Ermakov (Added unicode for some cases)
 25.12.08 -- Ermakov (Added string support for nchar values. Corrected size for unicode data)
 25.12.13 -- Muratova (Python 3 support)
 01.10.14 -- Kononov (Fixed return of Py_None value if NCHAR and NVARCHAR are NULL)
 08.10.14 -- Kononov (Added tByte and tVarbyte types for processing)
 09.10.14 -- Kononov (Truncated length of EXTFILE value to null-terminated character)
*/

#undef _DEBUG
#include <Python.h>
#include <linapi.h>
#include "LinPy.h"
#ifdef Py_UNICODE_WIDE
#include "lnlsutf.h"
#endif

static int String_SetValue(t_Array*, unsigned, PyObject*);
static PyObject *String_GetValue(t_Array*, unsigned);
//#if PY_MAJOR_VERSION >= 3
static int Byte_SetValue(t_Array*, unsigned, PyObject*);     /* 25.12.13 ... */
static PyObject *Byte_GetValue(t_Array*, unsigned);          /* ... 25.12.13 */
//#endif
#if (PY_VERSION_HEX >= 0x02020000)                         /* 31.10.03 */
static int Unicode_SetValue(t_Array*, unsigned, PyObject*);
static PyObject *Unicode_GetValue(t_Array*, unsigned);
#endif                                                     /* 31.10.03 */


PyTypeObject g_StringType = {
	  PyVarObject_HEAD_INIT(NULL, 0)                         /* 25.12.13 */
	  "STRING",			 /*tp_name*/
	  sizeof(t_Array),					/*tp_basicsize*/
	  0,									 /*tp_itemsize*/
	  (destructor)Array_Free, /*tp_dealloc*/
	  0,									 /*tp_print*/
	  0,									 /*tp_getattr*/
	  0,									 /*tp_setattr*/
	  0,									 /*tp_compare*/
	  0,									 /*tp_repr*/
	  0,									 /*tp_as_number*/
	  0,									 /*tp_as_sequence*/
	  0,									 /*tp_as_mapping*/
	  0,									 /*tp_hash */
	  0,									 /*tp_call*/
	  0,									 /*tp_str*/
	  0,                   /*tp_getattro*/
	  0,									 /*tp_setattro*/
	  0,									 /*tp_as_buffer*/
	  Py_TPFLAGS_DEFAULT,					/*tp_flags*/
	  0	                 /* tp_doc */                       /* 25.12.13 ... */
	#if PY_VERSION_HEX >= 0x02060000
	  , 0,									 /* tp_traverse */
	  0,									 /* tp_clear */
	  0,									 /* tp_richcompare */
	  0,									 /* tp_weaklistoffset */
	  0,									 /* tp_iter */
	  0,									 /* tp_iternext */
	  0,									 /* tp_methods */
	  0,									 /* tp_members */
	  0,									 /* tp_getset */
	  0,									 /* tp_base */
	  0,									 /* tp_dict */
	  0,									 /* tp_descr_get */
	  0,									 /* tp_descr_set */
	  0,									 /* tp_dictoffset */
	  0,									 /* tp_init */    /* ... 25.12.13 */
	 #endif
};



PyTypeObject g_BinaryType = {
	  PyVarObject_HEAD_INIT(NULL, 0)                         /* 25.12.13 */
	  "BINARY",			 /*tp_name*/
	  sizeof(t_Array),					/*tp_basicsize*/
	  0,									 /*tp_itemsize*/
	  (destructor)Array_Free, /*tp_dealloc*/
	  0,									 /*tp_print*/
	  0,									 /*tp_getattr*/
	  0,									 /*tp_setattr*/
	  0,									 /*tp_compare*/
	  0,									 /*tp_repr*/
	  0,									 /*tp_as_number*/
	  0,									 /*tp_as_sequence*/
	  0,									 /*tp_as_mapping*/
	  0,									 /*tp_hash */
	  0,									 /*tp_call*/
	  0,									 /*tp_str*/
	  0,                   /*tp_getattro*/
	  0,									 /*tp_setattro*/
	  0,									 /*tp_as_buffer*/
	  Py_TPFLAGS_DEFAULT,					/*tp_flags*/
	  0	                 /* tp_doc */                        /* 25.12.13 ... */
	#if PY_VERSION_HEX >= 0x02060000
	 , 0,									 /* tp_traverse */
	  0,									 /* tp_clear */
	  0,									 /* tp_richcompare */
	  0,									 /* tp_weaklistoffset */
	  0,									 /* tp_iter */
	  0,									 /* tp_iternext */
	  0,									 /* tp_methods */
	  0,									 /* tp_members */
	  0,									 /* tp_getset */
	  0,									 /* tp_base */
	  0,									 /* tp_dict */
	  0,									 /* tp_descr_get */
	  0,									 /* tp_descr_set */
	  0,									 /* tp_dictoffset */
	  0,									 /* tp_init */     /* ... 25.12.13 */
	 #endif
};


t_ArrayType vt_Char =
{
  (InitializeProc) NULL,
  (FinalizeProc) NULL,
  (SetValueProc) String_SetValue,
  (GetValueProc) String_GetValue,
  Array_InternalBindAnswer,
  NULL,
  NULL,
  &g_StringType,
  tChar,
  4000,
  0
};


t_ArrayType vt_VarChar =
{
  (InitializeProc) NULL,
  (FinalizeProc) NULL,
  (SetValueProc) String_SetValue,
  (GetValueProc) String_GetValue,
  Array_InternalBindAnswer,
  NULL,
  NULL,
  &g_StringType,
  tVarChar,
  4000,
  1
};


t_ArrayType vt_Byte =
{
  (InitializeProc) NULL,
  (FinalizeProc) NULL,
  (SetValueProc) String_SetValue,
  (GetValueProc) String_GetValue,
  Array_InternalBindAnswer,
  NULL,
  NULL,
  &g_BinaryType,
  tByte,                             /* 08.10.14 */
  4000,
  0
};


t_ArrayType vt_VarByte =
{
  (InitializeProc) NULL,
  (FinalizeProc) NULL,
  (SetValueProc) String_SetValue,
  (GetValueProc) String_GetValue,
  Array_InternalBindAnswer,
  NULL,
  NULL,
  &g_BinaryType,
  tVarByte,                          /* 08.10.14 */
  4000,
  1
};


#if (PY_VERSION_HEX >= 0x02020000)                          /* 31.10.03 */
t_ArrayType vt_NChar =
{
  (InitializeProc) NULL,
  (FinalizeProc) NULL,
  (SetValueProc) Unicode_SetValue,
  (GetValueProc) Unicode_GetValue,
  Array_InternalBindAnswer,
  NULL,
  NULL,
  &g_StringType,
  tNChar,
  4000,
  0
};


t_ArrayType vt_NVarChar =
{
  (InitializeProc) NULL,
  (FinalizeProc) NULL,
  (SetValueProc) Unicode_SetValue,
  (GetValueProc) Unicode_GetValue,
  Array_InternalBindAnswer,
  NULL,
  NULL,
  &g_StringType,
  tNVarChar,
  4000,
  1
};
#endif                                                     /* 31.10.03 */

/**
 *
 */
static int String_SetValue(t_Array *array, unsigned pos, PyObject *value)
{
  long   length,ReqLen;
  char   *data;
  L_BOOL UFlag = 0;                                          /* 04.09.08 ... */
  PyObject *val;
  
  trace(("String_SetValue: array %08x pos %i value %08x\n", array, pos, value));
  if (value == Py_None)
  {
    array->indicator[pos] = 1;
    return 0;
  }

  UFlag = PyUnicode_Check(value);

  if (!PyString_Check(value) && !UFlag)
  {
    PyErr_SetString(PyExc_TypeError, "expecting string or unicode data");
    return -1;
  }

  if (UFlag)
  {                                           /* 25.12.13 ... */
#ifdef Py_UNICODE_WIDE
    val = PyUnicode_AsUTF8String(value);
#else
    val = PyUnicode_AsEncodedString(value, NULL, NULL);
#endif
  }
  
  length = PyString_GET_SIZE((UFlag)?val:value); /* 28.11.08 */  /* 25.12.13 */
  ReqLen = length+(array->type->isVariableLength?sizeof(L_WORD):0); /* 28.11.08 */

/*  
  if (length == 0)
  {
    array->indicator[pos] = 1;
    return 0;
  }
*/
  array->indicator[pos] = 0;
  if (ReqLen > array->maxLength)                                 /* 28.11.08 */
  {
    if (Array_Resize(array, ReqLen) < 0)
      return -1;
  }
  data = Array_ELEMENT(array, pos);
  if (array->type->isVariableLength)                             /* 28.11.08 */
  {    
    * (L_SWORD *) data = (L_SWORD)length;
    data += sizeof(L_SWORD);
  }
  
  memcpy(data, PyString_AS_STRING((UFlag)?val:value), (length));         /* ... 04.09.08 */ /* 25.12.13 */
  array->actualLength[pos] = length;
  return 0;
}

/**
 *
 */
static PyObject *String_GetValue(t_Array *array, unsigned pos)
{
  char *data = array->data + pos * array->maxLength;

  trace(("String_GetValue: array %08x pos %i\n", array, pos));  /* 25.12.13 */
  if (array->indicator[pos])
  {
    Py_INCREF(Py_None);
    return Py_None;
  }

  if (array->type->isVariableLength)
  {
    if (PyInt_AS_LONG(g_UnicodeData) == 1)                     /* 22.09.08 */
    {
      return PyUnicode_Decode(data + sizeof(L_SWORD), * (L_SWORD *) data, NULL, NULL);
    }
    else
    {
      if (array->type->linterType == tVarByte)                                         /* 08.10.14 ... */
      {
#if PY_VERSION_HEX >= 0x02060000
        return PyBytes_FromStringAndSize(data + sizeof(L_SWORD), *(L_SWORD *)data);
#else
        return PyString_FromStringAndSize(data + sizeof(L_SWORD), *(L_SWORD *)data);
#endif
      }                                                                                /* ... 08.10.14 */
      else
      {
#if PY_MAJOR_VERSION >= 3
        return PyUnicode_Decode(data + sizeof(L_SWORD), (array->desc.Type == tExtFile) ? strlen(data + sizeof(L_SWORD)) : *(L_SWORD *)data, NULL, NULL); /* 09.10.14 */
#else
        return PyString_FromStringAndSize(data + sizeof(L_SWORD), (array->desc.Type == tExtFile) ? strlen(data + sizeof(L_SWORD)) : *(L_SWORD *)data);   /* 09.10.14 */
#endif
      }
    }
  }
  else
  {
    if (PyInt_AS_LONG(g_UnicodeData) == 1)                     /* 22.09.08 */
    {    
      return PyUnicode_Decode(data, array->maxLength, NULL, NULL);
    }
    else
    {
      if (array->type->linterType == tByte)                                            /* 08.10.14 ... */
      {
#if PY_VERSION_HEX >= 0x02060000
        return PyBytes_FromStringAndSize(data, (array->actualLength[pos] < array->maxLength) ? array->actualLength[pos] : array->maxLength);
#else
        return PyString_FromStringAndSize(data, (array->actualLength[pos] < array->maxLength) ? array->actualLength[pos] : array->maxLength);
#endif
      }                                                                                /* ... 08.10.14 */
      else
      {
#if PY_MAJOR_VERSION >= 3
        return PyUnicode_Decode(data, (array->actualLength[pos] < array->maxLength) ? array->actualLength[pos] : array->maxLength, NULL, NULL);
#else
        return PyString_FromStringAndSize(data, (array->actualLength[pos] < array->maxLength) ? array->actualLength[pos] : array->maxLength);
#endif
      }
    }
  }
}

#if PY_MAJOR_VERSION >= 3
static int Byte_SetValue(t_Array *array, unsigned pos, PyObject *value) /* 25.12.13 ... */
{
  long   length,ReqLen;
  char   *data;
  L_BOOL UFlag = 0;
  
  trace(("Byte_SetValue: array %08x pos %i value %08x\n", array, pos, value));
  if (value == Py_None)
  {
    array->indicator[pos] = 1;
    return 0;
  }

  UFlag = PyUnicode_Check(value);

  if (!/*PyBytes_Check*/PyString_Check(value) && !UFlag)
  {
    PyErr_SetString(PyExc_TypeError, "expecting string or unicode data");
    return -1;
  }
  
  if (UFlag)
  {
#ifdef Py_UNICODE_WIDE
    value = PyUnicode_AsUTF8String(value);
#else
    value = PyUnicode_AsEncodedString(value, NULL, NULL);
#endif
  } 
  
  length = /*PyBytes_GET_SIZE*/PyString_GET_SIZE(value); 
  ReqLen = length+(array->type->isVariableLength?sizeof(L_WORD):0); 
  
  array->indicator[pos] = 0;
  if (ReqLen > array->maxLength)
  {
    if (Array_Resize(array, ReqLen) < 0)
      return -1;
  }
  data = Array_ELEMENT(array, pos);
  if (array->type->isVariableLength)                             
  {       
    * (L_SWORD *) data = (L_SWORD)length;                 
    data += sizeof(L_SWORD);
  }

  memcpy(data, /*PyBytes_AS_STRING*/PyString_AS_STRING(value), length);
  array->actualLength[pos] = length;

  return 0;
}

static PyObject *Byte_GetValue(t_Array *array, unsigned pos)
{
  char *data = array->data + pos * array->maxLength;

  trace(("Byte_GetValue: array %08x pos %i\n", array, pos));
  if (array->indicator[pos])
  {
    Py_INCREF(Py_None);
    return Py_None;
  }

  if (array->type->isVariableLength)
  {
    if (PyInt_AS_LONG(g_UnicodeData) == 1)
    {
      return PyUnicode_Decode(data + sizeof(L_SWORD), * (L_SWORD *) data, NULL, NULL);
    }
    else
    {
      return PyBytes_FromStringAndSize(data + sizeof(L_SWORD), * (L_SWORD *) data);
    }
  }
  else
  {
    if (PyInt_AS_LONG(g_UnicodeData) == 1)
    {    
      return PyUnicode_Decode(data, (array->actualLength[pos] < array->maxLength) ? array->actualLength[pos] : array->maxLength, NULL, NULL);
    }
    else 
    {     
      return PyBytes_FromStringAndSize(data, (array->actualLength[pos] < array->maxLength) ? array->actualLength[pos] : array->maxLength);
    }
  }
}                                                            /* ... 25.12.13 */
#endif

#if (PY_VERSION_HEX >= 0x02020000)                          /* 31.10.03 */
/**
 *
 */
static int Unicode_SetValue(t_Array *array, unsigned pos, PyObject *value)
{
  long   length,ReqLen;                                      /* 25.12.08 */
  char   *data;
  L_BOOL UFlag = 0;                                          /* 04.09.08 ... */

  trace(("Unicode_SetValue: array %08x pos %i value %08x\n", array, pos, value));

  if (value == Py_None)                                      /* 25.12.08 */
  {
    array->indicator[pos] = 1;
    return 0;
  }

  UFlag = PyUnicode_Check(value);                            /* 25.12.08 ... */
  if (!PyString_Check(value) && !UFlag)
  {
    PyErr_SetString(PyExc_TypeError, "expecting string or unicode data");
    return -1;
  }

  length = (UFlag) ? PyUnicode_GET_SIZE(value): PyString_GET_SIZE(PyUnicode_FromObject(value)); /* 25.12.13 */
  ReqLen = (length*sizeof(L_WORD))+(array->type->isVariableLength?sizeof(L_WORD):0); /* 28.11.08 */ /* 25.12.13 */

  trace((" length %i\n", length));                           /* ... 25.12.08 */
/*
  if (length == 0)
  {
    array->indicator[pos] = 1;
  }
  else
*/
  array->indicator[pos] = 0;
  if (ReqLen > array->maxLength)                                 /* 28.11.08 */
  {
    if (Array_Resize(array, ReqLen) < 0)
      return -1;
  }
  data = Array_ELEMENT(array, pos);
  if (array->type->isVariableLength)                             /* 28.11.08 */
  {
    * (L_SWORD *) data = (L_SWORD)length;
    data += sizeof(L_SWORD);
  }

  if (UFlag)                                                 /* 25.12.08 ... */ /* 25.12.13 ... */
  {
#ifdef Py_UNICODE_WIDE
    LINNLS_Ucs4Ucs2(PyUnicode_AsUnicode(value),
              length,
              (L_UNICHAR *)data);
#else
    memcpy(data, PyUnicode_AsUnicode(value), length << 1);
#endif   
  }
  else
  {
#ifdef Py_UNICODE_WIDE
    L_LONG tmplen;
    LINNLS_Utf8Ucs2(PyBytes_AS_STRING(value),
                    PyBytes_GET_SIZE(value), 
                    length, 
                    (L_UNICHAR *)data,
                    &tmplen);
#else
    PyObject *val;
    val = PyUnicode_FromObject(value); 
    memcpy(data, PyUnicode_AsUnicode(val), length << 1);
#endif
  }                                                          /* ... 25.12.08 */ /* 25.12.13 ...*/

  array->actualLength[pos] = length << 1;
  return 0;                                                  /* ... 28.11.08 */
}


/**
 *
 */
static PyObject *Unicode_GetValue(t_Array *array, unsigned pos)
{
  int length;
  char *data;
  
  trace(("Unicode_GetValue: array %08x pos %i\n", array, pos));
  if (array->indicator[pos])                                      /* 01.10.14 ... */
  {
    Py_INCREF(Py_None);
    return Py_None;
  }                                                               /* ... 01.10.14 */
  data = Array_ELEMENT(array, pos);
  if (array->type->isVariableLength)
  {
    length = (*(L_SWORD *) data)/* << 1*/;                       /* 25.12.08 */
    data += sizeof(L_SWORD);
  }
  else
  {
    length = array->maxLength >> 1;
  }
  
#ifdef Py_UNICODE_WIDE                                       /* 25.12.13 ... */
  char * tmpstr;
  tmpstr = (char *)calloc(1, length *4);
  LINNLS_Ucs2Ucs4((L_UNICHAR *)data,
            (L_LONG)(length),
             (unsigned long *)tmpstr);

  return PyUnicode_FromUnicode((Py_UNICODE *) tmpstr, length);

#else
  return PyUnicode_FromUnicode((Py_UNICODE *) data, length);  
#endif                                                       /* ... 25.12.13 */

}
#endif                                                     /* 31.10.03 */
