 /***************************************************************************
 *                 RELEX, Inc. Linter Python driver source code             *
 *        This is unpublished proprietary source code of RELEX, Inc.        *
 *     The copyright notice does not evidence any actual or intended        *
 *                  publication of such source code.                        *
 *                                                                          *
 *                 Copyright (c) 2003-2014 RELEX Inc.                       *
 *                                                                          *
 *                   This software was developed by                         *
 *                                                                          *
 *                            RELEX, Inc.                                   *
 *                         Voronezh, Russia                                 *
 *                                                                          *
 *                        All rights reserved.                              *
 ****************************************************************************
 AUTHOR: Boris Fedorov
 ----------------------------------------------------------------------------
 PURPOSE: Timestamp interface (DBAPI)
 ----------------------------------------------------------------------------
 REVISION HISTORY:
 06.09.03 -- Boris Fedorov (Removed datetime.h dependence)
 31.10.03 -- Boris Fedorov (Remove C++ style comments)
 29.07.08 -- Ermakov (Corrected warnings)
 18.08.08 -- Ermakov (Added conversion support)
 28.08.08 -- Ermakov (Added unicode support for timestamp conversion)
 04.09.08 -- Ermakov ( added native DATETIME support (2.4 and higher) )
 22.09.08 -- Ermakov (Added date and unicode configuration)
 01.09.10 -- Galizin ( Time stamp compare )
 25.12.13 -- Muratova (Python 3 support)
 24.03.14 -- Andreeva (Python 3 support)
 30.09.14 -- Kononov (Removed an unreferenced local variables, fixed compiler warnings)
 06.10.14 -- Kononov (Fixed support for Python < 2.6)
 07.10.14 -- Kononov (Fixed binding DATE as datetime.datetime type to a prepared query)
*/

#undef _DEBUG
#include <Python.h>

#if (PY_VERSION_HEX >= 0x02040000)                           /* 04.09.08 ... */
#include <datetime.h>
#define NATIVE_DATETIME
#endif                                                       /* ... 04.09.08 */

#include <linapi.h>
#include "LinPy.h"
#include <decimals.h>
#include <tick.h>

static void Timestamp_Free(t_Timestamp*);
#if PY_VERSION_HEX < 0x02060000
static PyObject *Timestamp_GetAttr(t_Timestamp*, PyObject*);
#endif
static PyObject *Timestamp_Str(t_Timestamp*);
static int Timestamp_Cmp(t_Timestamp*, t_Timestamp*);
static int TimestampArray_SetValue(t_Array *array, unsigned pos, PyObject *value);
static PyObject *TimestampArray_GetValue(t_Array *array, unsigned pos);


PyTypeObject g_TimestampType = {                             
	  PyVarObject_HEAD_INIT(NULL, 0)                         /* 25.12.13 */
	  "Timestamp",			 /*tp_name*/  
	  sizeof(t_Timestamp),					 /*tp_basicsize*/
	  0,									 /*tp_itemsize*/
	  (destructor)Timestamp_Free, /*tp_dealloc*/
	  0,									 /*tp_print*/
	  0,									 /*tp_getattr*/
	  0,									 /*tp_setattr*/
#if PY_VERSION_HEX >= 0x03000000
  0, /* tp_reserved in 3.0.1, tp_compare in 3.0.0 but not used */
#else
  (cmpfunc) Timestamp_Cmp,		/* tp_compare */
#endif					
	  (reprfunc) Timestamp_Str,	/*tp_repr*/
	  0,									 /*tp_as_number*/
	  0,									 /*tp_as_sequence*/
	  0,									 /*tp_as_mapping*/
	  0,									 /*tp_hash */
	  0,									 /*tp_call*/
	  (reprfunc) Timestamp_Str,									 /*tp_str*/
#if PY_VERSION_HEX < 0x02060000
      (getattrofunc) Timestamp_GetAttr,	/* tp_getattro */
#else
	  0,                                   /*tp_getattro*/ /* 25.12.13 ... */
#endif
	  0,									 /*tp_setattro*/
	  0,									 /*tp_as_buffer*/
	  Py_TPFLAGS_DEFAULT,  /*tp_flags*/
	  0,	                 /* tp_doc */                    /* 25.12.13 ...*/
	  0,									 /* tp_traverse */
	  0,									 /* tp_clear */
	  0,									 /* tp_richcompare */
	  0,									 /* tp_weaklistoffset */
	  0,									 /* tp_iter */
	  0,									 /* tp_iternext */
	  0,									 /* tp_methods */
	  0,									 /* tp_members */
	  0,									 /* tp_getset */
	  0,									 /* tp_base */
	  0,									 /* tp_dict */
	  0,									 /* tp_descr_get */
	  0,									 /* tp_descr_set */
	  0,									 /* tp_dictoffset */
	  0,									 /* tp_init */   /* ... 25.12.13 */
};


PyTypeObject g_TimestampArrayType = {
	  PyVarObject_HEAD_INIT(NULL, 0)                         /* 25.12.13 */
	  "DATETIME",			 /*tp_name*/
	  sizeof(t_Array),					     /*tp_basicsize*/
	  0,									 /*tp_itemsize*/
	  (destructor)Array_Free, /*tp_dealloc*/
	  0,									 /*tp_print*/
	  0,									 /*tp_getattr*/
	  0,									 /*tp_setattr*/
	  0,                   /*tp_compare*/
	  0,	                 /*tp_repr*/
	  0,									 /*tp_as_number*/
	  0,									 /*tp_as_sequence*/
	  0,									 /*tp_as_mapping*/
	  0,									 /*tp_hash */
	  0,									 /*tp_call*/
	  0,									 /*tp_str*/
	  0,                   /*tp_getattro*/
	  0,									 /*tp_setattro*/
	  0,									 /*tp_as_buffer*/
	  Py_TPFLAGS_DEFAULT,					 /*tp_flags*/
	  0,	                 /* tp_doc */                      /* 25.12.13 ...*/
	  0,									 /* tp_traverse */
	  0,									 /* tp_clear */
	  0,									 /* tp_richcompare */
	  0,									 /* tp_weaklistoffset */
	  0,									 /* tp_iter */
	  0,									 /* tp_iternext */
	  0,									 /* tp_methods */
	  0,									 /* tp_members */
	  0,									 /* tp_getset */
	  0,									 /* tp_base */
	  0,									 /* tp_dict */
	  0,									 /* tp_descr_get */
	  0,									 /* tp_descr_set */
	  0,									 /* tp_dictoffset */
	  0,									 /* tp_init */   /* ... 25.12.13 */
};


t_ArrayType vt_Timestamp =
{
  (InitializeProc) NULL,
  (FinalizeProc) NULL,
  (SetValueProc) TimestampArray_SetValue,
  (GetValueProc) TimestampArray_GetValue,
  Array_InternalBindAnswer,
  NULL,
  NULL,
  &g_TimestampArrayType,
  tTimeStamp,
  sizeof(L_DECIMAL),
  0
};


void TimeStampInit()                                         /* 04.09.08 ... */
{
#ifdef NATIVE_DATETIME
   PyDateTime_IMPORT;
#endif
}

#if PY_VERSION_HEX < 0x02060000                                                            /* ... 04.09.08 */
void InitTimeStampObject()
{
#ifdef NATIVE_DATETIME
  PyDict_SetItemString(g_ModuleDictionary, "DATETIME", (PyObject*) PyDateTimeAPI->DateTimeType);
#else
  PyDict_SetItemString(g_ModuleDictionary, "DATETIME", (PyObject*) &g_TimestampType);
#endif
}                                                            /* ... 04.09.08 */
#endif

PyObject *Timestamp_New(
  unsigned type,                                             /* 22.09.08 */
  unsigned year,
  unsigned month,
  unsigned day,
  unsigned hour,
  unsigned minute,
  unsigned second,
  unsigned tick)
{
#ifdef NATIVE_DATETIME                                       /* 22.09.08 ... */
  switch(type)
  {
    case 0:
           return (PyObject *) PyDateTime_FromDateAndTime(year, month, day, hour, minute, second, tick*10);
    case 1:
           return (PyObject *) PyDate_FromDate(year, month, day);
    case 2:
           return (PyObject *) PyTime_FromTime(hour, minute, second, tick*10);
  }
  return NULL;
#else                                                        /* ... 22.09.08 */
  t_Timestamp *ts;

  type = type;
  ts = PyObject_NEW(t_Timestamp, &g_TimestampType);
  if (ts == NULL)
    return NULL;

  ts->year = year;
  ts->month = month;
  ts->day = day;
  ts->hour = hour;
  ts->minute = minute;
  ts->second = second;
  ts->tick = tick;

  return (PyObject *) ts;
#endif
}

static void Timestamp_Free(t_Timestamp *var)
{
  PyObject_DEL(var);                                             /* 04.09.08 */
}

#if PY_VERSION_HEX < 0x02060000
static PyObject *Timestamp_GetAttr(t_Timestamp *var, PyObject *nameObject)
{
  const char *name;

  name = PyString_AS_STRING(nameObject);
  if (name[0] == 'y' && strcmp(name, "year") == 0)
    return PyInt_FromLong(var->year);
  if (name[0] == 'm' && strcmp(name, "month") == 0)
    return PyInt_FromLong(var->month);
  if (name[0] == 'd' && strcmp(name, "day") == 0)
    return PyInt_FromLong(var->day);
  if (name[0] == 'h' && strcmp(name, "hour") == 0)
    return PyInt_FromLong(var->hour);
  if (name[0] == 'm' && strcmp(name, "minute") == 0)
    return PyInt_FromLong(var->minute);
  if (name[0] == 's' && strcmp(name, "second") == 0)
    return PyInt_FromLong(var->second);
  if (name[0] == 't' && strcmp(name, "tick") == 0)
    return PyInt_FromLong(var->tick);
  PyErr_SetObject(PyExc_AttributeError, nameObject);
  return NULL;
}
#endif
static PyObject *Timestamp_Str(t_Timestamp *var)
{
  char value[100];
  sprintf(value, "%.4d.%.2d.%.2d %.2d:%.2d:%.2d",
          var->year, var->month, var->day,
          var->hour, var->minute, var->second);
  return PyString_FromString(value);
}


static int Timestamp_Cmp(t_Timestamp *var1, t_Timestamp *var2)
{
  if ( var1->year > var2->year )                            /* 01.09.10 ... */
    return 1;
  else if ( var1->year < var2->year )
    return -1;

  if ( var1->month > var2->month)
    return 1;
  else if ( var1->month < var2->month)
    return -1;

  if ( var1->day > var2->day)
    return 1;
  else if ( var1->day < var2->day)
    return -1;

  if ( var1->hour > var2->hour)
    return 1;
  else if ( var1->hour < var2->hour)
    return -1;

  if ( var1->minute > var2->minute)
    return 1;
  else if ( var1->minute < var2->minute)
    return -1;

  if ( var1->second > var2->second)
    return 1;
  if ( var1->second < var2->second)
    return -1;

  if ( var1->tick > var2->tick)
    return 1;
  else if ( var1->tick < var2->tick)
    return -1;                                              /* ... 01.09.10 */

  return 0;
}

static int TimestampArray_SetValue(t_Array *array, unsigned pos, PyObject *value)
{
  L_LONG kd, kt;
  trace(("TimestampArray_SetValue: array %08x pos %i value %08x\n", array, pos, value));

  if (value == Py_None)
  {
    array->indicator[pos] = 1;
    return 0;
  }

  kd = 0;
  kt = 0;
#if PY_VERSION_HEX >= 0x02060000                                                   /* 06.10.14 ... */
  if (!PyDate_Check(value) && !PyTime_Check(value) && !PyDateTime_Check(value) &&
    !PyBytes_Check(value) && !PyUnicode_Check(value))      /* 25.12.13 *//* 28.08.08 18.08.08 */
#else
  if ((value->ob_type != &g_TimestampType) &&
    !PyString_Check(value) && !PyUnicode_Check(value))
#endif                                                                             /* ... 06.10.14 */
  {                                                              
    PyErr_SetString(PyExc_TypeError, "expecting timestamp, string or unicode data");
    return -1;
  }

  if (PyUnicode_Check(value))                                /* 28.08.08 ... */
  {
    Py_UNICODE  uniValue[100];
    char        stringValue[100];
    int         Len, status;

    Len = PyUnicode_AsWideChar((PyObject *)value, (wchar_t *)uniValue,99);
    uniValue[Len] = 0;
    wcstombs(stringValue,(wchar_t *)uniValue,100);
    trace((" str %s\n", stringValue));
    status = STRTOTICK(stringValue, ((L_DECIMAL *) array->data)[pos]);
    if (status == 0)                                            /* 24.03.14 ... */
    {
      PyErr_SetString(PyExc_TypeError, "expecting date in correct format");
      return -1;
    }                                                           /* ... 24.03.14 */
  }
  else                                                           /* ... 28.08.08 */
  {
    if (PyString_Check(value))                                   /* 18.08.08 ... */
    {
      char        *stringValue;
      int         status;
      stringValue = PyString_AS_STRING(value);
      trace((" str %s\n", stringValue));
      status = STRTOTICK(stringValue, ((L_DECIMAL *)array->data)[pos]); /* 30.09.14 */
      if (status == 0)                                          /* 24.03.14 ... */
      {
        PyErr_SetString(PyExc_TypeError, "expecting date in correct format");
        return -1;
      }                                                         /* ... 24.03.14 */
    }
    else
    {                                                            /* ... 18.08.08 */
#if PY_VERSION_HEX < 0x02040000                                  /* 06.10.14 ... */
      t_Timestamp *ts = (t_Timestamp *) value;
      kd = DAYNUMBERDATE((L_WORD)ts->day, (L_WORD)ts->month, (L_WORD)ts->year);
      kt = ts->tick + (ts->second * 100) + (ts->minute * 6000) + (ts->hour * 360000);
#else                                                            /* ... 06.10.14 */
      if (PyDateTime_Check(value))                           /* 25.12.13 ... */
      {
        L_DECIMAL ticksInDay;                                                                                                        /* 07.10.14 ... */
        char buf[20];
        sprintf(buf, "%u:%u:%u.%d", (L_WORD)PyDateTime_DATE_GET_HOUR(value), (L_WORD)PyDateTime_DATE_GET_MINUTE(value),
          (L_WORD)PyDateTime_DATE_GET_SECOND(value), (L_LONG)PyDateTime_DATE_GET_MICROSECOND(value));
        STRTOTICKF(buf, "HH:MI:SS.MS", ticksInDay);
        TICKTODATE(ticksInDay, &kd, &kt);                                                                                            /* ... 07.10.14 */
        kd = DAYNUMBERDATE((L_WORD)PyDateTime_GET_DAY(value),
                           (L_WORD)PyDateTime_GET_MONTH(value),
                           (L_WORD)PyDateTime_GET_YEAR(value));
      }
      else if (PyTime_Check(value))
      {
	      kd = 0;
        kt = PyDateTime_TIME_GET_MICROSECOND(value) + (PyDateTime_TIME_GET_SECOND(value) * 100) + (PyDateTime_TIME_GET_MINUTE(value)
             * 6000) + (PyDateTime_TIME_GET_HOUR(value) * 360000);
      }
      else if (PyDate_Check(value))
      {
        kd = DAYNUMBERDATE((L_WORD)PyDateTime_GET_DAY(value),
                           (L_WORD)PyDateTime_GET_MONTH(value),
                           (L_WORD)PyDateTime_GET_YEAR(value));
        kt = 0;
      }                                                      /* ... 25.12.13 */
#endif
      DATETOTICK(kd, kt, ((L_DECIMAL *) array->data)[pos]);
    }
  }

  array->indicator[pos] = 0;
  array->actualLength[pos] = sizeof(L_DECIMAL);

  return 0;
}

static PyObject *TimestampArray_GetValue(t_Array *array, unsigned pos)
{
  L_SWORD year, month, day, hour, minute, second, tick;
  L_LONG kd, kt;

  if (array->indicator[pos])
  {
    Py_INCREF(Py_None);
    return Py_None;
  }

  TICKTODATE(((L_DECIMAL *) array->data)[pos], &kd, &kt);

  DATEDAYNUMBER(kd, &day, &month, &year);

  hour = (L_SWORD)(kt / 360000);
  kt %= 360000;
  minute = (L_SWORD)(kt / 6000);
  kt %= 6000;
  second = (L_SWORD)(kt / 100);
  tick = kt % 100;

  if (PyInt_AS_LONG(g_DateEmulation) == 1)                       /* 22.09.08 */
  {
    if (!hour && !minute && !second && !tick)
      return Timestamp_New(1, year, month, day, hour, minute, second, tick);
    if (!day && !month && !year)
      return Timestamp_New(2, year, month, day, hour, minute, second, tick);
  }
  return Timestamp_New(0, year, month, day, hour, minute, second, tick); /* 22.09.08 */
}
