 /***************************************************************************
 *           RELEX, Inc. RUBY core client source file                       *
 *  This is unpublished proprietary source code of RELEX, Inc.              *
 *  The copyright notice does not evidence any actual or intended           *
 *  publication of such source code.                                        *
 *                                                                          *
 *                   Copyright (c) 2007-2014 RELEX Inc.                     *
 *                                                                          *
 *                   This software was developed by                         *
 *                                                                          *
 *                            RELEX, Inc.                                   *
 *                         Voronezh, Russia                                 *
 *                                                                          *
 *                        All rights reserved.                              *
 ****************************************************************************
 FILE:   LinRuby.c
 ----------------------------------------------------------------------------
 AUTHOR: Vera Muratova
 ----------------------------------------------------------------------------
 PURPOSE: Interface from Ruby language to Linter SQL Server
 ----------------------------------------------------------------------------
 REVISION HISTORY:
 02.08.08 -- (file creation)
 04.09.08 -- Muratova, Ermakov (Warnings)
 23.10.08 -- Muratova (Changes for GetLastAutoInc, GetLastRowId)
 28.11.08 -- Muratova (Add Error_Check, processing exceptions)
 20.03.09 -- Muratova (Changed in processing exceptions)
 30.03.09 -- Muratova (Changed in SQLFetchManyRow)
 30.07.09 -- Muratova (Changed processing of name/password)
 27.04.10 -- Muratova (Correct last error number in Error_Check)
 28.05.10 -- Muratova (Changed CloseConnection and added FreeConnection function)
 31.05.10 -- Muratova (mem_free added)
 02.06.10 -- Muratova (Corrected coping varbyte)
 03.06.10 -- Muratova (DEFAULT_FORMAT added)
 03.06.10 -- Muratova (Corrected precessing nchar, nvarchar in ToValue)
 04.06.10 -- Muratova (Corrected precessing blob in ToValue)
 11.06.10 -- Muratova (SetColCount, SetAnswerInfo added)
 22.06.10 -- Muratova (Changed parameters count in GetBlob)
 23.06.10 -- Muratova (Changed parameters count in AddBlob)
 23.06.10 -- Muratova (Bug fixed in BlobInfo processing in SetColumnInfo)
 08.07.10 -- Muratova (Bug fixed: added parameters checking in proc_)
 08.07.10 -- Muratova (Changed free of AnswerInfo, BlobInfo)
 09.07.10 -- Muratova (Added checking for procedure in request)
 17.05.11 -- Muratova (Replacement STR2CSTR on StringValuePtr)
 17.05.11 -- Muratova (Replacement RSTRING(str)->len on RSTRING_LEN, RARRAY(ary)->len on RARRAY_LEN)
 24.11.11 -- Muratova (SetColumnInfo if selection not empty;
                       Corrected getting blob length;
                       Corrected getting bool data)
 12.03.12 -- Muratova (Bug fixed in output of GetAnswerInfo)
 23.03.12 -- Muratova (Change keys in output of GetAnswerInfo)
 23.08.12 -- Ermakov  (Corrected wrong code)
 10.09.12a-- Pashentsev (Removed calloc - ALLOC is inside macro Data_Make_Struct)
 10.09.12b-- Pashentsev (Corrected memory freeing)
 13.09.12 -- Pashentsev (Free intlib context on library unload) 
 05.11.13 -- Galizin, Ermakov(Warnings, memory allocation for int types)
 08.04.14 -- Muratova (Bind int params like string)
 10.04.14 -- Muratova (Ruby 2 compatibility)
 20.10.14 -- Kononov (Fixed length of the column name)
 21.10.14 -- Kononov (Fixed binding integers and floating-point values as string for prepared query)
 22.10.14a-- Kononov (Added processing of EXTFILE type)
 22.10.14b-- Kononov (Added supporting construction WITH)
 24.10.14 -- Kononov (Fixed NCHAR and NVARCHAR values by using LINNLS_Ucs2Utf8() function)
*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "ruby.h"
#include "inter.h"
#include "cquery.h"
#include "decimals.h"
#include "tick.h"

#define  MAX_BUFF          8192
#define  MAX_BUFF_BLOB     4048
#define  BUFF              255
#if _VER_MAX >= 600
#define  MAX_ROW_SIZE     0xFFFF
#else
#define  MAX_ROW_SIZE     MAX_BUFF
#endif

#define MAX_DATABASE_NAME 18
/* Fetch directions */
#define FETCH_FIRST             1
#define FETCH_LAST              2
#define FETCH_NEXT              3
#define FETCH_PREV              4
#define FETCH_ABSNUM            5

#define HashResult 1
#define RowResult  2

#define NO_MEMORY             -1
#define FETCH_DIR             -2
#define WRONG_COUNT_PARAM     -3
#define WRONG_BIND_PARAM      -4
#define WRONG_TYPE_PARAM      -5
#define WRONG_NUMBER_PARAM    -6
#define NO_BLOB_COLUMNS       -7
#define UNKNOWN_OPTION        -8                                   /* 20.03.09 */

#define FETCH_BLOB_MODE       1
#define CHANNEL_PRIORITY      2
#define DATE_FORMAT           3
#define SET_SAVE_POINT        4
#define SET_CURSOR_NAME       5

#define BindByNumber          1
#define BindByName            2

#define DEFAULT_FORMAT "YYYY-MM-DD HH24:MI:SS"                     /* 03.06.10 */

#define SQL_SELECT            1
#define SQL_EXEC_PROC        52

#ifndef RSTRING_LEN                                                /* 17.05.11... */
#define RSTRING_LEN(str) RSTRING(str)->len
#endif

#ifndef RARRAY_LEN
#define RARRAY_LEN(ary) RARRAY(ary)->len
#endif                                                             /* ...17.05.11 */

#ifndef rb_float_new_in_heap                                       /* 10.04.14... */
#define rb_float_new_in_heap rb_float_new
#endif                                                             /* ...10.04.14 */
const char s[] = "select";                                         /* 09.06.10 */
const char e[] = "execute"; 
const char w[] = "with";                                           /* 22.10.14b */

static VALUE rb_cLinterConnection;
static VALUE rb_cLinterCursor;

extern int ReplaceEscapeSequences(char *SrcPtr, unsigned short SrcLen, 
                           char *ResPtr, unsigned short *ResLen, 
                           unsigned short ResMaxLen);
extern L_LONG GetProcCursorDesc(TCBL * Cbl, P_TYPE * ptype, char * buffer, GETA_OUT * coldesc);

typedef void (* t_Convert)(VALUE Text,
                            void  * c,
                            int   i);
typedef struct t_BindElement
{
  short     len;
  short     scale;
  short     precision;
  short     type;
  void      *str;
  char      name[MAX_ID_LEN];
  t_Convert fConvert;
} t_BindElement;

struct LinConnection
{
  L_LONG               Err;
  L_WORD                  Priority;
  L_LONG               PrzExe;
  OPEN_DESC             *info; /* answer structure */
  GETA_OUT              *AnswerInfo;
  t_BindElement         *Bind;
  char                  *PrepareQuery;/* buffer for compiled query */
  L_BOOL                flagOfProc;
  L_BOOL                flagOfSelect;
  L_BOOL                flagFetch;
  L_BOOL                flagOfBind;
  int                   ParamCount;
  int                   CurrentCol;
  int                   CurrentRow;
  int                   NumBlobCol;
  int                   CurrentBlobCol;
  int                   BlobColCount;
  TCBL                  *CBL;
  int                   RowCount;  /* quantity of answers     */
  int                   ColCount;  /* quantity of colomns     */
  BLOB_ATR              *BlobInfo; /* parameter of blob       */
  char                  *RowBuf;
  char                  *format;   /* format of date      */
  L_WORD                  RowBufLen; /* length of answer buffer */
/*
for cursor "list" points to connection, which this cursor submission
*/
  struct LinConnection  *list; /* cursors list for connection */
  struct LinConnection  *next;
  VALUE                 cur;
};
typedef struct LinConnection t_LinConnection;

static VALUE Get_Blob(t_LinConnection *c, int len, int nCol, int start);

static t_LinConnection *linter_connection_list; /* connections list */

static void Error_Check(t_LinConnection *c)                        /* 28.11.08 */
{
  char *res;

  c->Err = c->CBL->CodErr;                                         /* 27.04.10 */

  res = (char *) calloc(1, MAX_BUFF);
  if ( res == NULL )
  {
    c->Err = NO_MEMORY;
    rb_raise(rb_eStandardError, "alloc failed");                   /* 28.11.08 */
  }

  if ( c->Err != 0 && c->Err != 2 )
  {
    memcpy(c->CBL->Command, "GETE", 4);
    c->CBL->LnBufRow = MAX_BUFF;
    c->CBL->RowId = c->Err;
    inter(c->CBL, NULL, NULL, NULL, res);
    *(res + ((c->CBL->LnBufRow > MAX_BUFF) ? (MAX_BUFF-1) : (c->CBL->LnBufRow))) = 0;

    if ( c->CBL->CodErr == 1001 )                                  /* 27.04.10 */
    {
      strcpy(res, "Linter hasn't loaded");
    }
    if ( c->CBL->CodErr == 1069 )                                  /* 27.04.10 */
    {
      strcpy(res, "Channel hasn't opened");
    }
    if ((c->CBL->CodErr >= 4000) && (c->CBL->CodErr <= 4999))      /* 27.04.10 */
    {
      strcpy(res, "Network error");
    }

    rb_raise(rb_eTypeError, "%s", res);                    /* 05.11.13 */
                                 /* 10.09.12b ...*/
  }
  if (res)
  {
    free(res);
  }                              /*... 10.09.12b */
}

static VALUE GetStatus(VALUE self)
{
  t_LinConnection   *c;

  Data_Get_Struct(self, t_LinConnection, c);

  return LL2NUM(c->Err);
}

static VALUE GetStatusOS(VALUE self)
{
  t_LinConnection   *c;

  Data_Get_Struct(self, t_LinConnection, c);

  return LL2NUM(c->CBL->SysErr);
}

/*
set option for connection (object method)
input params:
Option - connection option
val - value of option
output param:
self
*/
static VALUE SetOption(VALUE self, VALUE Option, VALUE val)
{
  t_LinConnection   *c;
  int                option;
  char              *v;
  char query[100] = "set savepoint ";

  Data_Get_Struct(self, t_LinConnection, c);

  if ( TYPE(Option) == T_FIXNUM )
  {
    option = NUM2INT(Option);                              /* 05.11.13 */
    switch( option )
    {
      case FETCH_BLOB_MODE:
        if ( TYPE(val) == T_TRUE )
          c->flagFetch = L_TTRUE;
        else if ( TYPE(val) == T_FALSE )
          c->flagFetch = L_TFALSE;
        else
        {
          c->Err = WRONG_TYPE_PARAM;
          rb_raise(rb_eTypeError, "wrong parameter type %s (expected TRUE/FALSE)",
                   rb_obj_classname(val));                         /* 28.11.08 */
        }
        break;
      case CHANNEL_PRIORITY:
        if ( TYPE(val) == T_FIXNUM )
          c->Priority = (unsigned short)NUM2LL(val);
        else
        {
          c->Err = WRONG_TYPE_PARAM;
          rb_raise(rb_eTypeError, "wrong parameter type %s (expected FIXNUM)",
                   rb_obj_classname(val));                         /* 28.11.08 */
        }
        break;
      case DATE_FORMAT:
        if ( TYPE(val) == T_STRING )
        {
          v = StringValuePtr(val);                                 /* 17.05.11 */
          c->format = (char *) calloc(1, 100);

          if ( c->format == NULL )
          {
            c->Err = NO_MEMORY;
            rb_raise(rb_eStandardError, "alloc failed");           /* 28.11.08 */
          }

          memcpy(c->format, v, strlen(v));
        }
        else
        {
          c->Err = WRONG_TYPE_PARAM;
          rb_raise(rb_eTypeError, "wrong parameter type %s (expected STRING)",
                   rb_obj_classname(val));                         /* 28.11.08 */
        }
        break; 
      case SET_SAVE_POINT:   
        if ( TYPE(val) == T_STRING )
        {
          v = StringValuePtr(val);                                 /* 17.05.11 */
          strcat( query, "\"" );
          strcat( query, v);
          strcat( query, "\"" );
          strcat( query, ";" );          
          memcpy(c->CBL->Command, "SLCT", 4);   
          c->CBL->LnBufRow = MAX_BUFF;   
          inter(c->CBL, NULL, query, NULL, NULL);
          Error_Check(c);
        }
        else
        {
          c->Err = WRONG_TYPE_PARAM;
          rb_raise(rb_eTypeError, "wrong parameter type %s (expected STRING)", 
                   rb_obj_classname(val));                         /* 28.11.08 */
        }
        break; 
      case SET_CURSOR_NAME:                           
        if ( TYPE(val) == T_STRING ) 
        {
          v  = StringValuePtr(val);                                /* 17.05.11 */
          c->CBL->RowId = crName; 
          c->CBL->LnBufRow = (unsigned short)strlen(v);
          memcpy(c->CBL->Command, "SETO", 4);       
          inter(c->CBL, NULL, NULL, NULL, v);
          Error_Check(c);
        }
        else
        {
          c->Err = WRONG_TYPE_PARAM;                     
          rb_raise(rb_eTypeError, "wrong parameter type %s (expected STRING)", 
                   rb_obj_classname(val));                         /* 28.11.08 */
        }
        break;
      default:
        c->Err = UNKNOWN_OPTION;                                   /* 20.03.09 */
        rb_raise(rb_eTypeError, "unknown option");                 /* 28.11.08 */
    }     
  }
  else
  {
    c->Err = UNKNOWN_OPTION;                                       /* 20.03.09 */
    rb_raise(rb_eTypeError, "unknown option");                     /* 28.11.08 */
  }

  return self;
}

/*
creation of connection (class method)
output param:
obj - ruby object
*/
static VALUE Create(VALUE self)
{
  VALUE             obj;
  t_LinConnection   *c;

  /* 10.09.12a -- c = calloc() removed */
 
  obj = Data_Make_Struct(self, t_LinConnection, 0, NULL, c);
  c->RowCount = 0;
  c->Priority = 0;
  c->cur = Qnil;
  c->Err = 0;	
  c->ParamCount = 0;

  c->CBL = (TCBL *) calloc(1, sizeof(TCBL)); 
  c->RowBuf = (unsigned char *) calloc(1, MAX_BUFF);

  if ( c->CBL == NULL || c->RowBuf == NULL ) 
  {    
    c->Err = NO_MEMORY;
    rb_raise(rb_eStandardError, "alloc failed");                   /* 28.11.08 */
  }

  c->next = linter_connection_list;
  linter_connection_list = c;      
    
  rb_obj_call_init(obj, 0, NULL);  

  return obj;
}

/*
opening connection (class method)
input params:
Name - user name
Pasword - user pasword
Node - Linter node
output param:
obj - ruby object
*/    
static VALUE Open1 (int argc, VALUE* argv, VALUE self)
{ 
  VALUE             obj, Name, Password, Node, PrzExe, OpBuf;
  t_LinConnection   *c;
  char              Name_password[MAX_ID_LEN+18+5]; 

  switch( argc )
  {
    case 2:
      rb_scan_args(argc, argv, "02", &Name, &Password);
      if ( (TYPE(Name) != T_STRING) && (TYPE(Password) != T_STRING) )
      {        
        rb_raise(rb_eTypeError, "wrong parameter(s) type");        /* 28.11.08 */
      }
      else
      {
        Node = Qnil; 
        PrzExe = Qnil;
      }
      break;
    case 3:
      rb_scan_args(argc, argv, "03", &Name, &Password, &Node);  
      if ( TYPE(Name) != T_STRING || TYPE(Password) != T_STRING ||
           (TYPE(Node) != T_STRING && TYPE(Node) != T_NIL) ) 
      {        
        rb_raise(rb_eTypeError, "wrong parameter(s) type");        /* 28.11.08 */
      }
      else
      {
        PrzExe = Qnil;
      }
      break;   
    case 4:
      rb_scan_args(argc, argv, "04", &Name, &Password, &Node, &PrzExe);        
      if ( TYPE(Name) != T_STRING || TYPE(Password) != T_STRING ||
           (TYPE(Node) != T_STRING && TYPE(Node) != T_NIL) || 
           (TYPE(PrzExe) != T_FIXNUM && TYPE(PrzExe) != T_NIL) )
      {        
        rb_raise(rb_eTypeError, "wrong parameter(s) type");        /* 28.11.08 */
      }
      break;   
    case 5:
      rb_scan_args(argc, argv, "05", &Name, &Password, &Node, &PrzExe, &OpBuf);        
      if ( TYPE(Name) != T_STRING || 
           TYPE(Password) != T_STRING ||
           (TYPE(Node) != T_STRING && TYPE(Node) != T_NIL) || 
           (TYPE(PrzExe) != T_FIXNUM && TYPE(PrzExe) != T_NIL)|| 
           TYPE(OpBuf) != T_STRING )
      {        
        rb_raise(rb_eTypeError, "wrong parameter(s) type");        /* 28.11.08 */
      }
      break; 
    default:      
      rb_raise(rb_eArgError, "too many arguments");                /* 28.11.08 */
  }  

  /* 10.09.12a -- c = calloc() removed */
  obj = Data_Make_Struct(self, t_LinConnection, 0, NULL, c);
  c->RowCount = 0;
  c->Priority = 0;  
  c->CurrentRow = 1;
  c->ParamCount = 0;
  c->cur = Qnil;
  c->next = linter_connection_list;
  linter_connection_list = c; 

  c->info = ALLOC(OPEN_DESC);
  c->CBL = (TCBL *) calloc(1, sizeof(TCBL));
  c->RowBuf = (unsigned char *) calloc(1, MAX_BUFF);

  if ( c->CBL == NULL || c->info == NULL || 
       c->RowBuf == NULL  ) 
  {    
    c->Err = NO_MEMORY;
    rb_raise(rb_eStandardError, "alloc failed");                 /* 28.11.08 */
  }    	          
       
  if ( TYPE(Node) != T_NIL )
    strcpy(c->CBL->Node, StringValuePtr(Node));                  /* 17.05.11 */
  else
    strcpy(c->CBL->Node, "   ");

  if ( TYPE(PrzExe) != T_NIL )            
    c->CBL->PrzExe = c->PrzExe = FIX2LONG(PrzExe);               
  else
    c->CBL->PrzExe = c->PrzExe = Q_ENCODE | M_BINARY;

  memcpy(c->CBL->Command, "OPEN", 4); 
  c->CBL->Prior = c->Priority;
  c->CBL->PrzExe &= ~Q_ASYNC;      
  c->CBL->LnBufRow = sizeof(OPEN_DESC);  

  strcpy(Name_password, "\"");
  strncat(Name_password, StringValuePtr(Name), MAX_ID_LEN);            /* 30.07.09... *//* 17.05.11 */
  strcat(Name_password, "\"/\"");
  strncat(Name_password, StringValuePtr(Password),18);                 /* 17.05.11 */
  strcat(Name_password, "\""); 

  if (argc < 5)
    inter(c->CBL, Name_password, NULL, NULL, c->info);
#if _VER_MAX <=590                                                 
  else
    inter(c->CBL, Name_password, NULL, NULL, c->info);
#else
  else
    inter(c->CBL, Name_password, StringValuePtr(OpBuf), NULL, c->info); /* 17.05.11 */
#endif                                                             /* ...30.07.09 */
     
  Error_Check(c);                                                /* 28.11.08 */

  c->flagOfSelect = L_TFALSE;
  c->flagOfProc = L_TFALSE;
  c->flagFetch = L_TTRUE;    
    
  rb_obj_call_init(obj, 0, NULL);
  /* 10.09.12a -- calloc-related code removed */

  return obj;
}
/*
creation of connection (object method)
input params:
Name - user name
Pasword - user pasword
Node - Linter node
output param:
obj - ruby object
*/
static VALUE Open2 (int argc, VALUE* argv, VALUE obj)
{ 
  t_LinConnection   *c;
  VALUE             Name, Password, Node, PrzExe, OpBuf;
  char              Name_password[MAX_ID_LEN+18+5];

  Data_Get_Struct(obj, t_LinConnection, c);

  switch( argc )
  {
    case 2:
      rb_scan_args(argc, argv, "02", &Name, &Password);
      if ( (TYPE(Name) != T_STRING) || (TYPE(Password) != T_STRING) ) 
      {
        c->Err = WRONG_TYPE_PARAM;
        rb_raise(rb_eTypeError, "wrong parameter(s) type");        /* 28.11.08 */
      }
      else
      {
        Node = Qnil; 
        PrzExe = Qnil;
      }
      break;
    case 3:
      rb_scan_args(argc, argv, "03", &Name, &Password, &Node);  
      if ( TYPE(Name) != T_STRING || TYPE(Password) != T_STRING ||
           (TYPE(Node) != T_STRING && TYPE(Node) != T_NIL) )              
      {
        c->Err = WRONG_TYPE_PARAM;
        rb_raise(rb_eTypeError, "wrong parameter(s) type");        /* 28.11.08 */
      }
      else
        PrzExe = Qnil;
      break;   
      case 4:
      rb_scan_args(argc, argv, "04", &Name, &Password, &Node, &PrzExe);  
      if ( TYPE(Name) != T_STRING || 
           TYPE(Password) != T_STRING ||
           (TYPE(Node) != T_STRING && TYPE(Node) != T_NIL) || 
           (TYPE(PrzExe) != T_FIXNUM && TYPE(PrzExe) != T_NIL ) ) 
      {
        c->Err = WRONG_TYPE_PARAM;
        rb_raise(rb_eTypeError, "wrong parameter(s) type");        /* 28.11.08 */
      }
      break; 
    case 5:
      rb_scan_args(argc, argv, "05", &Name, &Password, &Node, &PrzExe, &OpBuf);        
      if ( TYPE(Name) != T_STRING || 
           TYPE(Password) != T_STRING ||
           (TYPE(Node) != T_STRING && TYPE(Node) != T_NIL) || 
           (TYPE(PrzExe) != T_FIXNUM && TYPE(PrzExe) != T_NIL) || 
           TYPE(OpBuf) != T_STRING )
      {
        c->Err = WRONG_TYPE_PARAM;
        rb_raise(rb_eTypeError, "wrong parameter(s) type");        /* 28.11.08 */
      }
      break; 
    default:
      c->Err = WRONG_COUNT_PARAM;
      rb_raise(rb_eArgError, "too many arguments");                /* 28.11.08 */
  }  
           
  c->CurrentRow = 1;

  c->info = ALLOC(OPEN_DESC);	

  if ( c->info == NULL ) 
  {    
    c->Err = NO_MEMORY;
    rb_raise(rb_eStandardError, "alloc failed");                   /* 28.11.08 */
  } 

  if ( TYPE(Node) != T_NIL )
    strcpy(c->CBL->Node, StringValuePtr(Node));                    /* 17.05.11 */
  else
      strcpy(c->CBL->Node, "   ");

  if ( TYPE(PrzExe) != T_NIL )
    c->CBL->PrzExe = c->PrzExe = FIX2LONG(PrzExe);
  else
    c->CBL->PrzExe = c->PrzExe = Q_ENCODE | M_BINARY;    

  memcpy(c->CBL->Command, "OPEN", 4); 
  c->CBL->Prior = c->Priority;
  c->CBL->PrzExe &= ~Q_ASYNC; 
  c->CBL->LnBufRow = sizeof(OPEN_DESC);     

  strcpy(Name_password, "\"");                                     /* 30.07.09 ... */
  strncat(Name_password, StringValuePtr(Name), MAX_ID_LEN);        /* 17.05.11 */
  strcat(Name_password, "\"/\"");
  strncat(Name_password, StringValuePtr(Password),18);             /* 17.05.11 */
  strcat(Name_password, "\"");

  if (argc < 5)
    inter(c->CBL, Name_password, NULL, NULL, c->info);
#if _VER_MAX <=590                                                 
  else
    inter(c->CBL, Name_password, NULL, NULL, c->info);
#else
  else
    inter(c->CBL, Name_password, StringValuePtr(OpBuf), NULL, c->info); /* 17.05.11 */
#endif                                                            /* ... 30.07.09 */ 
    
  Error_Check(c);                                                  /* 28.11.08 */

  c->flagOfSelect = L_TFALSE;
  c->flagOfProc = L_TFALSE;
  c->flagFetch = L_TTRUE;

  return obj;
}

/*
connection info (object method)
output param:
hash - associate ruby array
*/
static VALUE GetDBInfo(VALUE self)
{
  VALUE hash = rb_hash_new();
  t_LinConnection *c;
  Data_Get_Struct(self, t_LinConnection, c);
  rb_hash_aset(hash, rb_str_new2("channel"), INT2FIX(c->CBL->NumChan));
  rb_hash_aset(hash, rb_str_new2("Node"), rb_str_new2(c->CBL->Node));
  if ( c->info == NULL && c->list )
  {    
    rb_hash_aset(hash, rb_str_new2("BaseName"), rb_str_new(c->list->info->BaseName,MAX_DATABASE_NAME));
    rb_hash_aset(hash, rb_str_new2("DefCharSet"), CHR2FIX(c->list->info->DefCharSet));
    rb_hash_aset(hash, rb_str_new2("Flags"), CHR2FIX(c->list->info->Flags));      
    rb_hash_aset(hash, rb_str_new2("Log"), CHR2FIX(c->list->info->Log));
    rb_hash_aset(hash, rb_str_new2("MaxRecSize"), CHR2FIX(c->list->info->MaxRecSize));
    rb_hash_aset(hash, rb_str_new2("Os"), CHR2FIX(c->list->info->Os));
    rb_hash_aset(hash, rb_str_new2("Sync"),CHR2FIX(c->list->info->Sync));
    rb_hash_aset(hash, rb_str_new2("SysLog"), CHR2FIX(c->list->info->SysLog));
    rb_hash_aset(hash, rb_str_new2("UseCharSet"), CHR2FIX(c->list->info->UseCharSet));
    rb_hash_aset(hash, rb_str_new2("UseCharSetName"), rb_str_new2(c->list->info->UseCharSetName));
    rb_hash_aset(hash, rb_str_new2("VerBuild"), LL2NUM(c->list->info->VerBuild));
    rb_hash_aset(hash, rb_str_new2("VerMajor"), LL2NUM(c->list->info->VerMajor));
    rb_hash_aset(hash, rb_str_new2("VerMinor"), LL2NUM(c->list->info->VerMinor));    
  }
  rb_hash_aset(hash, rb_str_new2("BaseName"), rb_str_new(c->info->BaseName, MAX_DATABASE_NAME));
  rb_hash_aset(hash, rb_str_new2("DefCharSet"), CHR2FIX(c->info->DefCharSet));
  rb_hash_aset(hash, rb_str_new2("Flags"), CHR2FIX(c->info->Flags));  
  rb_hash_aset(hash, rb_str_new2("Log"), CHR2FIX(c->info->Log));
  rb_hash_aset(hash, rb_str_new2("MaxRecSize"), CHR2FIX(c->info->MaxRecSize));
  rb_hash_aset(hash, rb_str_new2("Os"), CHR2FIX(c->info->Os));
  rb_hash_aset(hash, rb_str_new2("Sync"),CHR2FIX(c->info->Sync));
  rb_hash_aset(hash, rb_str_new2("SysLog"), CHR2FIX(c->info->SysLog));
  rb_hash_aset(hash, rb_str_new2("UseCharSet"), CHR2FIX(c->info->UseCharSet));
  rb_hash_aset(hash, rb_str_new2("UseCharSetName"), rb_str_new2(c->info->UseCharSetName));
  rb_hash_aset(hash, rb_str_new2("VerBuild"), LL2NUM(c->info->VerBuild));
  rb_hash_aset(hash, rb_str_new2("VerMajor"), LL2NUM(c->info->VerMajor));
  rb_hash_aset(hash, rb_str_new2("VerMinor"), LL2NUM(c->info->VerMinor));

  return hash;
}

static void escape(char *buf, int length)                    /* 23.08.12 */
{
  while (length && (buf[length-1] == ' ' || !buf[length-1]))
    buf[--length] = 0;
}

static void SetColCount(t_LinConnection *c)                  /* 11.06.10 ... */
{
  memcpy(c->CBL->Command, "GETA", 4);  
  c->CBL->LnBufRow = 0;                                     
  inter(c->CBL, NULL, NULL, NULL, NULL);
  c->ColCount = (L_WORD) c->CBL->RowCount;
  Error_Check(c);    
}

static void SetAnswerInfo(t_LinConnection *c)
{
  memcpy(c->CBL->Command, "GETA", 4);
  c->CBL->RowId = 0;            
  c->CBL->LnBufRow = (L_WORD) (c->ColCount * sizeof(GETA_OUT));      
  inter(c->CBL, NULL, NULL, NULL, c->AnswerInfo);
  Error_Check(c);      
}

static void SetBlobInfo(t_LinConnection *c)
{
  int             i;

  for ( i = 0; i < c->ColCount; i++ )
  {   		
    if ( c->AnswerInfo[i].Type == DT_BLOB )    
    {
      c->BlobColCount++;      
    }						
  }

  if ( c->BlobColCount > 0 )
  {
    c->BlobInfo = (BLOB_ATR *) calloc(c->BlobColCount, sizeof(BLOB_ATR));
    if ( c->BlobInfo == NULL )
    {
      c->Err = NO_MEMORY;
      rb_raise(rb_eStandardError, "alloc failed");                 /* 28.11.08 */
    }
  }     
}                                                            /* ... 11.06.10 */


static void SetColumnInfo(t_LinConnection *c)
{  
  int             i, blob_num = 0;   
  unsigned char   *Buf;    

  if ( c->RowCount > 0 )                                             /* 24.11.11 */
  {
    Buf = (unsigned char *) calloc(1, MAX_BUFF);
    if ( Buf == NULL )
    {
      c->Err = NO_MEMORY;
      rb_raise(rb_eStandardError, "alloc failed");                   /* 28.11.08 */
    }

    memcpy( Buf, c->RowBuf, MAX_BUFF );    
    for ( i = 1; i <= c->ColCount; i++ )
    {   
      if ( c->AnswerInfo[i-1].Type == DT_BLOB )
      {                            
        memcpy(&c->BlobInfo[blob_num], Buf, sizeof(BLOB_ATR));       /* 23.06.10 */
        memcpy( Buf, Buf + c->AnswerInfo[i-1].Length, c->RowBufLen ); 
        blob_num++;
      }     
      else if ( c->AnswerInfo[i-1].Type == DT_VARCHAR || 
                c->AnswerInfo[i-1].Type == DT_VARBYTE ||
                c->AnswerInfo[i-1].Type == DT_NVARCHAR )
      {			
        memcpy(Buf, Buf + c->AnswerInfo[i-1].Length + 2, c->RowBufLen); 
      }
      else 
        memcpy( Buf, Buf + c->AnswerInfo[i-1].Length, c->RowBufLen ); 
    }
    free(Buf);         
  }
}


/*
feeder function for view result of query
input params:
c - current connection
text - string with data
type - ruby type of text
Length - length of text
output param:
res - text presented as ruby VALUE
*/
static VALUE ToValue (t_LinConnection *c, char * buf, unsigned char Type, unsigned short Length)
{
  VALUE     res;
  L_DECIMAL DecBuf;
  L_LONG outLen;
  char     *str;
		
  switch ( Type )
  {
    case DT_CHAR:
    case DT_EXTFILE:           /* 22.10.14a */
      escape(buf, Length); 
      res = rb_str_new2(buf);
      break;
    case DT_INTEGER:
      if ( Length == sizeof(L_WORD) )
      {        
        short     SmallBuf;
        memcpy(&SmallBuf, buf, sizeof(L_WORD));
        res = INT2FIX(SmallBuf);
      }
      else
      {
        L_LONG    IntBuf; 
        memcpy(&IntBuf, buf, sizeof(L_LONG));
        res = LL2NUM(IntBuf);
      }       
      break;
    case DT_REAL:              
      if ( Length == sizeof(float) )
      {
        float     FloatBuf;
        memcpy(&FloatBuf, buf, Length);        
        res = rb_float_new_in_heap(FloatBuf);              /* 10.04.14 */
      }
      else        
      {
        double    DoubleBuf;
        memcpy(&DoubleBuf, buf, Length);  
        res =  rb_float_new_in_heap(DoubleBuf);            /* 10.04.14 */
      }
      break;
    case DT_DATE:	
      memcpy(DecBuf, buf, sizeof(L_DECIMAL));
      if ( str = (unsigned char *) calloc(1, MAX_BUFF) )
      {
        if ( c->format != NULL )
        {             
          TICKTOSTRF(DecBuf, c->format, str);              
        }
        else
        {
          TICKTOSTRF(DecBuf, DEFAULT_FORMAT, str);                 /* 03.06.10 */
        }

        res = rb_str_new2(str);
        free(str);        				
      }      	      
      break;   
    case DT_DECIMAL:     
      if ( str = (unsigned char *) calloc(1, MAX_BUFF) )
      {
        memcpy(DecBuf, buf, sizeof(L_DECIMAL));		  
        DECTOSTR(DecBuf, str, 20, 2, 1);
        res = rb_str_new2(str);
        free(str);
      }
      else
      {
        c->Err = NO_MEMORY;
        rb_raise(rb_eStandardError, "alloc failed");               /* 28.11.08 */
      }
      break;
    case DT_BYTE:   
      res = rb_tainted_str_new(buf, strlen(buf));                  /* 01.06.10 */
      break;       
    case DT_BLOB:                                                  /* 04.06.10 ... */
      if ( c->BlobInfo[c->NumBlobCol - 1].Size > 0 && c->flagFetch ) 
      {
        res = Get_Blob(c, c->BlobInfo[c->NumBlobCol - 1].Size, c->CurrentCol, 1);
      }
      else
      {
        res = Qnil; 
      }
      break;                                                       /* ... 04.06.10 */
    case DT_VARCHAR:           
      res = rb_str_new(buf, Length); 
      break;
    case DT_VARBYTE:            
      res = rb_tainted_str_new(buf, strlen(buf));                  /* 01.06.10 */
      break;
    case DT_BOOL:
      if ( buf[0] == 1 )
      {
        res = rb_str_new2("true"); 
      }
      else
      {
        res = rb_str_new2("false"); 
      }
      break;
    case DT_NCHAR:      
    case DT_NVARCHAR:
      {
        LINNLS_Ucs2Utf8(buf, Length, Length * 4, NULL, &outLen); /* 24.10.14 ... */
        str = calloc(outLen + 1, sizeof(char));
        if (str == NULL)
        {
          c->Err = NO_MEMORY;
          rb_raise(rb_eStandardError, "alloc failed");
        }
        LINNLS_Ucs2Utf8(buf, Length, Length * 4, str, &outLen); /* ... 24.10.14 */

        escape(str, outLen);
        res = rb_str_new2(str);
        free(str);
        break;
      }
  }
	
  return res;
}


static void proc_(t_LinConnection *c)
{
  t_LinConnection   *cursor;
  ARGPROC_OUT       *ParInfo;
  L_WORD              *cnt, i, Len;
  char              *ptr;
  VALUE             obj; 
  CR_INFO           *cr;

  c->cur = rb_ary_new();  
  ParInfo = (ARGPROC_OUT *) c->RowBuf;  

  if ( ParInfo->Flags == PARM_FLAG_ISCURSOR )
  {              
    if ( c->AnswerInfo != NULL )                             /* 08.07.10 ... */
    {  
      free(c->AnswerInfo);
      c->AnswerInfo = NULL;
    }                                                        /* ... 08.07.10 */

    ptr = c->RowBuf + ParInfo->Value;                 	
    cr = (CR_INFO *) ptr;
    /* 10.09.12a -- cursor = calloc() removed */   
    obj = Data_Make_Struct( rb_cLinterCursor, t_LinConnection, 0, NULL, cursor );

    cursor->RowBuf = (unsigned char *) calloc(1, MAX_BUFF);
    cursor->CBL = (TCBL *) calloc(1, sizeof(TCBL));

    if ( cursor->RowBuf == NULL || cursor->CBL == NULL )
    {
      cursor->Err = NO_MEMORY; 
      rb_raise(rb_eStandardError, "alloc failed");               /* 28.11.08 */
    }
             				
    cursor->CBL->LnBufRow = MAX_ROW_SIZE;
    cursor->CBL->NumChan = cr->NumChan; 
    cursor->CBL->Prior = c->CBL->Prior;
    cursor->CBL->PrzExe = c->PrzExe;
    cursor->CBL->PrzExe &= ~Q_ASYNC;
    cursor->CBL->CodErr = cr->CodErr;
    Error_Check(cursor);                                         /* 28.11.08 */
    strcpy( cursor->CBL->Node, c->CBL->Node );
    cursor->RowBufLen = cr->LnBufRow;
    cursor->RowCount = cr->RowCount;
    cursor->next = c->list;
    c->list = cursor;
    cursor->list = c;          

    memcpy(cursor->CBL->Command, "GETA", 4);        
    cursor->CBL->LnBufRow = 0;                                     
    inter(cursor->CBL, NULL, NULL, NULL, NULL);
    cursor->ColCount = (L_WORD) cursor->CBL->RowCount;
    Error_Check(cursor);                                         /* 28.11.08 */

    cursor->AnswerInfo = (GETA_OUT *) calloc(cursor->ColCount, sizeof(GETA_OUT));

    if ( cursor->AnswerInfo == NULL )
    {
      cursor->Err = NO_MEMORY;
      rb_raise(rb_eStandardError, "alloc failed");               /* 28.11.08 */
    }
        
    memcpy(cursor->CBL->Command, "GETA", 4);
    cursor->CBL->RowId = 0;
    cursor->CBL->LnBufRow = (L_WORD) (cursor->ColCount * sizeof(GETA_OUT));            
    inter(cursor->CBL, NULL, NULL, NULL, cursor->AnswerInfo);      
    Error_Check(cursor);                                         /* 28.11.08 */
         
    rb_obj_call_init(obj, 0, NULL);
    rb_ary_push(c->cur, obj);
    /* 10.09.12b -- calloc-related code removed */
  }
  else
    if ( ParInfo->Flags != PARM_FLAG_ISNULL )
    {
      ptr = c->RowBuf + ParInfo->Value;
      obj = ToValue(c, ptr, ParInfo->Type.ntyp, ParInfo->Type.length);
      rb_ary_push(c->cur, obj);
    }
    else
      rb_ary_push(c->cur, Qnil);
  
  cnt = (L_WORD *) (ParInfo + 1);
  ParInfo = (ARGPROC_OUT *) (cnt + 1);

  if (*cnt > 0)                                                    /* 08.07.10 ... */
  {
    for ( i = 0; i < *cnt; ++i )
    {
      int l = -1;
      if ( ParInfo->Flags & PARM_FLAG_NAMESENT)
      {
        ptr = c->RowBuf + ParInfo->Value;
        Len = *((L_WORD *) ptr);
        ParInfo->Value += sizeof(L_WORD) + Len;
      }              
      ptr = c->RowBuf + ParInfo->Value;
      if ( ParInfo->Type.ntyp == DT_VARCHAR || 
           ParInfo->Type.ntyp == DT_VARBYTE ||
           ParInfo->Type.ntyp == DT_NVARCHAR )
      {  
         l = *((L_WORD *) ptr);
         ParInfo->Value += sizeof(L_WORD);
         ptr = c->RowBuf + ParInfo->Value;
      }	      
      obj = ToValue(c, ptr, ParInfo->Type.ntyp, (l > -1)? l:ParInfo->Type.length);      
      rb_ary_push(c->cur, obj);        
      ParInfo++;
    }
  }                                                                /* ... 08.07.10 */
}

/*
executing query (object method)
input params:
Query - line of query
output param:
self
*/
static VALUE SQLExecuteDirect(VALUE self, VALUE Query)
{
  t_LinConnection   *c;
  char              *query;  
  int               i = 0;
  char              szLocQuery[10] = {0};

  Data_Get_Struct(self, t_LinConnection, c); 

  if (TYPE(Query) == T_STRING )
  {    
    query = StringValuePtr(Query);                                 /* 17.05.11 */

    do
    {
      szLocQuery[i] = LAnsiTolower(query[i]);
    } while ((i < 9) && query[i++] );

    szLocQuery[i] = 0;

    if ( c->AnswerInfo != NULL )                             /* 11.06.10 ... */
    {  
      free(c->AnswerInfo);
      c->AnswerInfo = NULL;
    }

    if ( c->BlobInfo != NULL )
    {
      free(c->BlobInfo);
      c->BlobInfo = NULL;
    }                                                        /* ... 11.06.10 */

    memcpy(c->CBL->Command, "    ", 4);
    c->flagOfProc = c->flagOfSelect = L_TFALSE;
    
    if (strstr(szLocQuery, s) || strstr(szLocQuery, w)) 								 /* 09.06.10 ... */ /* 22.10.14b */
    {
      c->flagOfSelect = L_TTRUE;
    }                                                           

    if ( strstr(szLocQuery, e))								 /* ... 09.06.10 */
    {
      c->flagOfProc = L_TTRUE;
    }

    c->CBL->LnBufRow = MAX_ROW_SIZE;        
    inter(c->CBL, NULL, query, NULL, c->RowBuf);
    Error_Check(c); 

    if ( c->flagOfProc )
    {          
      proc_(c);          
    }
    else 
    {           
      c->RowCount = c->CBL->RowCount;
      c->RowBufLen = c->CBL->LnBufRow;						

      if ( c->flagOfSelect )                                 /* 09.06.10 ... */
      {				
        SetColCount(c);
        c->AnswerInfo = (GETA_OUT *) calloc(c->ColCount, sizeof(GETA_OUT));

        if ( c->AnswerInfo == NULL  )
        {
          c->Err = NO_MEMORY;
          rb_raise(rb_eStandardError, "alloc failed");             /* 28.11.08 */
        }	
        
        c->CurrentBlobCol = 0;
        c->BlobColCount = 0;	
        SetAnswerInfo(c);
        SetBlobInfo(c);
        SetColumnInfo(c);						
      }                                                      /* ... 09.06.10 */
    }

  }
  else
  {
    c->Err = WRONG_TYPE_PARAM; 		
    rb_raise(rb_eTypeError, "wrong parameter type %s (expected String)", 
             rb_obj_classname(Query));                             /* 28.11.08 */
  }

  c->CurrentRow = 1;	
   
  return self;
}

static void Bind(t_LinConnection * c, int i)
{       
  switch( c->flagOfBind )
  {
    case BindByNumber:             
      inter_bind_num(c->CBL, c->PrepareQuery, i + 1, c->Bind[i].type, &c->Bind[i].len, c->Bind[i].str);
      break;
    case BindByName:        
      inter_bind_name(c->CBL, c->PrepareQuery, c->Bind[i].name, c->Bind[i].type, &c->Bind[i].len, c->Bind[i].str);
      break;
  }
  Error_Check(c);                                                  /* 28.11.08 */
}


static void * sConvert(VALUE InText, void *c, int i)
{
  t_LinConnection   * lc = (t_LinConnection *)c;

  switch  ( TYPE(InText) )                                         /* 23.06.10 ... */
  {
    case T_STRING:
      switch ( lc->Bind[i].type )
      {
        case DT_VARCHAR:
          *(L_WORD *)lc->Bind[i].str = (unsigned short)RSTRING_LEN(InText); /* 17.05.11 */          
          memcpy((L_WORD *)lc->Bind[i].str + 1, StringValuePtr(InText), RSTRING_LEN(InText)); /* 17.05.11 */
          break;
        case DT_CHAR:
        case DT_NCHAR:
        case DT_NVARCHAR:
          lc->Bind[i].type = 1;      
          memcpy(lc->Bind[i].str, StringValuePtr(InText), RSTRING_LEN(InText)); /* 17.05.11 */              
          break;
        case DT_BLOB:
          lc->Bind[i].str = calloc(1, RSTRING_LEN(InText));          /* 24.11.11 */ /* 17.05.11 */       
          if ( lc->Bind[i].str == NULL )
          {
            lc->Err = NO_MEMORY;
            rb_raise(rb_eStandardError, "alloc failed");
          }
          memcpy(lc->Bind[i].str, StringValuePtr(InText), RSTRING_LEN(InText)); /* 17.05.11 */              
          break;
      }                                                            /* ... 23.06.10 */
      lc->Bind[i].len = (short)RSTRING_LEN(InText); /* 17.05.11 */
      break; 
    case T_NIL:
      lc->Bind[i].len = -1;
      break; 
    default:
      lc->Err = WRONG_BIND_PARAM;
      rb_raise(rb_eTypeError, "wrong bind parameter type");        /* 28.11.08 */
      break;
  }
  Bind(lc, i);

  return 0;
}


static void  * sInt(VALUE InText, void *c, int i)
{
  t_LinConnection   * lc  = (t_LinConnection *)c;

  switch  ( TYPE(InText) )
  {
    case T_STRING:                                         /* 08.04.14 ... */
      {
        if (lc->Bind[i].len == sizeof(L_SWORD))                              /* 21.10.14 ... */
          *(L_SWORD*)lc->Bind[i].str = (L_SWORD)atoi(StringValuePtr(InText));
        else if (lc->Bind[i].len == sizeof(L_LONG))
          *(L_LONG*)lc->Bind[i].str = (L_LONG)atoi(StringValuePtr(InText));
        else if (lc->Bind[i].len == sizeof(L_DLONG))
          *(L_DLONG*)lc->Bind[i].str = (L_DLONG)i64_atol(StringValuePtr(InText));  /* ... 21.10.14 */
      }
      break;                                               /* ... 08.04.14 */
    case T_FIXNUM:
      {
        int Tmp = FIX2INT(InText);                         /* 05.11.13 ... */
        memcpy(lc->Bind[i].str, &Tmp, sizeof(int));
        lc->Bind[i].len = sizeof(int);                     /* ... 05.11.13 */
      }
      break;
    case T_BIGNUM:
      {
        L_DLONG tmp = NUM2LL(InText);                      /* 05.11.13 ... */
        memcpy(lc->Bind[i].str, &tmp, sizeof(L_DLONG));
        lc->Bind[i].len = sizeof(L_DLONG);                 /* ... 05.11.13 */
      }
      break;
    case T_NIL:
      lc->Bind[i].len = -1;
      break;
    default:
      lc->Err = WRONG_BIND_PARAM;
      rb_raise(rb_eTypeError, "wrong bind parameter type");        /* 28.11.08 */
      break;
  }
  Bind(lc, i);

  return 0;
}

static void * sReal(VALUE InText, void *c, int i)
{
  t_LinConnection * lc = (t_LinConnection *)c;   
  
  switch  ( TYPE(InText) )
  {
    case T_STRING:                                                       /* 21.10.14 ... */
      if (lc->Bind[i].len == sizeof(L_REAL))
        *(L_REAL*)lc->Bind[i].str = (L_REAL)atof(StringValuePtr(InText));
      else
        *(L_DOUBLE*)lc->Bind[i].str = (L_DOUBLE)atof(StringValuePtr(InText));
      break;                                                             /* ... 21.10.14 */
    case T_FLOAT:      
      if ( lc->Bind[i].len == sizeof(L_REAL) )
        *(L_REAL *)(lc->Bind[i].str) = (float)NUM2DBL(InText);
      else
        *(L_DOUBLE *)(lc->Bind[i].str) = NUM2DBL(InText);             
      break;
    case T_FIXNUM:
      if ( lc->Bind[i].len == sizeof(L_REAL) )
        *(L_REAL *)(lc->Bind[i].str) = (float)NUM2INT(InText); 
      else
        *(L_DOUBLE *)(lc->Bind[i].str) = NUM2INT(InText);
      break;
    case T_BIGNUM:
      if ( lc->Bind[i].len == sizeof(L_REAL) )
        *(L_REAL *)(lc->Bind[i].str) = (float)NUM2LL(InText);     
      else
        *(L_DOUBLE *)(lc->Bind[i].str) = (double )NUM2LL(InText); 
      break; 
    case T_NIL:
      lc->Bind[i].len = -1;
      break; 
    default:
      lc->Err = WRONG_BIND_PARAM;
      rb_raise(rb_eTypeError, "wrong bind parameter type");        /* 28.11.08 */
      break;
  }  

  Bind(lc, i);  

  return 0;
}

static void * sDate(VALUE InText, void *c, int i)
{
  t_LinConnection * lc = (t_LinConnection *)c;

  if ( TYPE(InText) == T_STRING )
  {
    STRTOTICK(StringValuePtr(InText), lc->Bind[i].str);            /* 17.05.11 */     
    Bind(lc, i);     
  }
  else if ( TYPE(InText) == T_NIL )
  {
    lc->Bind[i].len = -1;   
    Bind(lc, i);
  }
  else
  {
    lc->Err = WRONG_BIND_PARAM;
    rb_raise(rb_eTypeError, "wrong bind parameter type");          /* 28.11.08 */
  }

  return 0;
}


static void * sByte(VALUE InText, void *c, int i)
{
  t_LinConnection * lc  = (t_LinConnection *)c;

  if ( TYPE(InText) == T_STRING )
  {   
    if ( lc->Bind[i].type == DT_VARBYTE )
    {       
      *(L_WORD *)lc->Bind[i].str = (unsigned short)RSTRING_LEN(InText);  /* 02.06.10 ... */ /* 17.05.11 */
      memcpy((L_WORD *)lc->Bind[i].str + 1, StringValuePtr(InText), RSTRING_LEN(InText)); /* 17.05.11 */
    }
    else                                                                  /* ... 02.06.10 */
    {
      memcpy((L_WORD *)lc->Bind[i].str, StringValuePtr(InText), RSTRING_LEN(InText)); /* 17.05.11 */   
    }
    Bind(lc, i);
  }
  else if ( TYPE(InText) == T_NIL )
  {
    lc->Bind[i].len = -1;    
    Bind(lc, i);
  }
  else
  {
    lc->Err = WRONG_BIND_PARAM;
    rb_raise(rb_eTypeError, "wrong bind parameter type");          /* 28.11.08 */
  }

  return 0;
} 

static void * sBool(VALUE InText, void *c, int i)
{
  t_LinConnection   * lc = (t_LinConnection *)c;
  L_CHAR            *value;
  int               d;
  int               res = -1;

  switch  ( TYPE(InText) )
  {      
    case T_STRING:
      value = (L_CHAR*)StringValuePtr(InText);                 /* 24.11.11 */ /* 17.05.11 */ /* 21.10.14 ... */

      if (strlen(value) > 1 && (strlen(value) == 4 || strlen(value) == 5))
      {
        for (d = 0; d < (int)strlen(value); d++)
          value[d] = tolower((int)value[d]);
        if (strcmp(value, "true") == 0)
          res = 1;
        else if (strcmp(value, "false") == 0)
          res = 0;
      }
      else if (strlen(value) == 1 && (*value ==  't' || *value ==  'T' || *value ==  '1'))
      {
        res = 1;
      }
      else if (strlen(value) == 1 && (*value ==  'f' || *value ==  'F' || *value ==  '0'))
      {
        res = 0;
      }

      if (res != -1)
      {
        *(L_BYTE *)lc->Bind[i].str = (L_BYTE)res;
      }
      else 
      {
        lc->Err = WRONG_BIND_PARAM;
        rb_raise(rb_eTypeError, "wrong bind parameter type");      /* 28.11.08 */
      }                                                                                      /* ... 21.10.14 */
      break;
    case T_FIXNUM:
      d = NUM2INT(InText);
      if ( d == 1 )
      {
        *(L_BYTE *)lc->Bind[i].str = 1;
      }
      else if ( d == 0 )
      {
        *(L_BYTE *)lc->Bind[i].str = 0;
      }
      else
      {
        lc->Err = WRONG_BIND_PARAM;
        rb_raise(rb_eTypeError, "wrong bind parameter type");      /* 28.11.08 */
      }
      break;  
    case T_NIL:
      lc->Bind[i].len = -1;
      break; 
    default:
      lc->Err = WRONG_BIND_PARAM;
      rb_raise(rb_eTypeError, "wrong bind parameter type");        /* 28.11.08 */
      break;
  }  
  Bind(lc, i);

  return 0;
} 


static void *sDecimal(VALUE InText, void *c, int i)  /* 21.10.14 ... */
{
  t_LinConnection *lc = (t_LinConnection*)c;
  L_CHAR *strVal;
  int intVal;
  double dblVal;
  L_DLONG lngVal;

  strVal = (L_CHAR*)calloc(1, BUFF);
  if (strVal == NULL)
  {
    lc->Err = NO_MEMORY;
    rb_raise(rb_eStandardError, "alloc failed");
  }

  switch (TYPE(InText))
  {
    case T_FLOAT:
      dblVal = NUM2DBL(InText);
      sprintf(strVal, "%f", dblVal);
      break;
    case T_FIXNUM:
      intVal = NUM2INT(InText);
      sprintf(strVal, "%i", intVal);
      break;
    case T_BIGNUM:
      lngVal = NUM2LL(InText);
      i64_ltoa(lngVal, strVal);
      break;
    case T_STRING:
      memcpy(strVal, StringValuePtr(InText), RSTRING_LEN(InText));
      break;
    case T_NIL:
      lc->Bind[i].len = -1;
      break;
    default:
      lc->Err = WRONG_BIND_PARAM;
      rb_raise(rb_eTypeError, "wrong bind parameter type");
      break;
  }

  if (TYPE(InText) != T_NIL)
  {
    STRTODEC(strVal, lc->Bind[i].str);
    lc->Bind[i].len = BUFF;
  }

  Bind(lc, i);
  free(strVal);

  return 0;
}                                                    /* ... 21.10.14 */


static void mem_free(t_LinConnection *c)                           /* 31.05.10 ... */
{
  int i;

  if ( c->PrepareQuery != NULL )
  {
    free(c->PrepareQuery);
    c->PrepareQuery = NULL;
  }

  for ( i = 0; i < c->ParamCount; i++ )
  {     
    free(c->Bind[i].str);
  }

  if ( c->AnswerInfo != NULL )
  {
    free(c->AnswerInfo);
    c->AnswerInfo = NULL;
  }

  if ( c->Bind != NULL )
  {
    free(c->Bind);
    c->Bind = NULL;
  }
}                                                                  /* ... 31.05.10 */

/*
compiling query (object method)
input param:
str - query
out param:
self
*/
static VALUE SQLPrepare_(VALUE self, VALUE str)
{  
  t_LinConnection   *c;
  char              buffer[MAX_BUFF];
  char              *query;
  P_TYPE            AnswerInfo;     
  int               i = 0;
  
  Data_Get_Struct(self, t_LinConnection, c);                    
  if ( TYPE(str) == T_STRING )
  {           
    c->CBL->LnBufRow = MAX_BUFF;
    memcpy(c->CBL->Command, "TRAN", 4);  

    query = StringValuePtr(str);                                   /* 17.05.11 */

    mem_free(c);
	
    c->PrepareQuery = (char *) calloc(1, MAX_BUFF);

    if ( c->PrepareQuery == NULL )
    {
      c->Err = NO_MEMORY;
      rb_raise(rb_eStandardError, "alloc failed");                 /* 28.11.08 */
    }    		

    inter(c->CBL, NULL, query, NULL, c->PrepareQuery);    
    Error_Check(c);                                                /* 28.11.08 */
    
    c->flagOfProc = c->flagOfSelect = L_TFALSE;					   /* 09.07.10 */

    if ( inter_isselect(c->CBL, c->PrepareQuery) == SQL_SELECT )
    {
      c->flagOfSelect = L_TTRUE;
    }    
   
    if ( inter_isselect(c->CBL, c->PrepareQuery) == SQL_EXEC_PROC )
    {
      c->flagOfProc = L_TTRUE;
    }
          
    c->RowBufLen = c->CBL->LnBufRow;                 

    if ( c->flagOfSelect )
    {
      c->ColCount = inter_column_number(c->CBL, c->PrepareQuery);
      Error_Check(c);                                              /* 28.11.08 */

      c->AnswerInfo = (GETA_OUT *) calloc(c->ColCount, sizeof(GETA_OUT));
      if ( c->AnswerInfo == NULL )
      {
        c->Err = NO_MEMORY;
        rb_raise(rb_eStandardError, "alloc failed");               /* 28.11.08 */
      }
         
      c->CBL->LnBufRow = (L_WORD) (c->ColCount * sizeof(GETA_OUT));
      inter_get_description(c->CBL, c->PrepareQuery, (char *)c->AnswerInfo);
      Error_Check(c);                                              /* 28.11.08 */
    }

    if ( c->flagOfProc )
    {              
      c->CBL->UserName[0] = 0;                                   
      inter_GetProcType(c->CBL, c->PrepareQuery, buffer);
      Error_Check(c);                                              /* 28.11.08 */
      
      memcpy(&AnswerInfo, buffer, sizeof(P_TYPE));
      if (AnswerInfo.ntyp == 101 )
      {
        c->ColCount = 0;              
        inter_GetProcType(c->CBL, c->PrepareQuery, buffer);  
        Error_Check(c);                                            /* 28.11.08 */

        c->AnswerInfo = calloc(sizeof(GETA_OUT) * AnswerInfo.length, sizeof(char));
        if ( c->AnswerInfo == NULL )
        {
          c->Err = NO_MEMORY;
          rb_raise(rb_eStandardError, "alloc failed");             /* 28.11.08 */
        }

        while ( c->ColCount < AnswerInfo.length )
        {
          if ( c->ColCount > 0 )
          {                         
            c->CBL->UserName[0] = (char)c->ColCount;                    
            inter_GetProcType(c->CBL, c->PrepareQuery, buffer);   
            Error_Check(c);                                        /* 28.11.08 */
          }
          c->ColCount += GetProcCursorDesc(c->CBL, &AnswerInfo, buffer, ((GETA_OUT *) c->AnswerInfo) + c->ColCount);
        }                                                 
      }                           
    }        
        
    inter_GetParamCount(c->PrepareQuery, (short *) &c->ParamCount); 
   
    c->Bind = (t_BindElement *)calloc(c->ParamCount, sizeof(t_BindElement));
    if ( c->Bind == NULL && c->ParamCount > 0 )
    {
      c->Err = NO_MEMORY;
      rb_raise(rb_eStandardError, "alloc failed");                 /* 28.11.08 */
    }
    for ( i = 0; i < c->ParamCount; i++ )
    {    
      inter_DescribeParam(c->PrepareQuery, 
                          (L_WORD)(i + 1),
                          &c->Bind[i].len, 
                          (short *)&c->Bind[i].type, 
                          &c->Bind[i].precision, 
                          &c->Bind[i].scale, 
                          NULL);  

      if ( c->Bind[i].type != DT_BLOB && c->Bind[i].type != DT_INTEGER )
      {
        c->Bind[i].str = calloc(1, MAX_BUFF);
      }
      else
      {
        c->Bind[i].str = calloc(1, sizeof(L_DLONG));       /* 05.11.13 */
      }
      if ( c->Bind[i].str == NULL )
      {
        c->Err = NO_MEMORY;
        rb_raise(rb_eStandardError, "alloc failed");             /* 28.11.08 */
        return self;                                       /* 05.11.13 */
      }

      switch  ( c->Bind[i].type )
      {            
        case DT_INTEGER:              
          c->Bind[i].fConvert = (void *)sInt;
          break;
        case DT_REAL:             
          c->Bind[i].fConvert = (void *)sReal;              
          break;
        case DT_DATE:
          c->Bind[i].fConvert = (void *)sDate;
          break;           
        case DT_BYTE:
          c->Bind[i].fConvert = (void *)sByte;
          break;            
        case DT_VARBYTE:
          c->Bind[i].fConvert = (void *)sByte;
          break;
        case DT_BOOL:
          c->Bind[i].fConvert = (void *)sBool;
          break;          
        case DT_DECIMAL:                          /* 21.10.14 ... */
          c->Bind[i].fConvert = (void *)sDecimal;
          break;                                  /* ... 21.10.14 */
        default:
          c->Bind[i].fConvert = (void *)sConvert;
      }/*switch*/                
    }     
  }
  else
  {
    c->Err = WRONG_TYPE_PARAM;
    rb_raise(rb_eTypeError, "wrong parameter type %s (expected String)", 
             rb_obj_classname(str));                               /* 28.11.08 */
  }

  return self;
}


static VALUE BindParamInfo(t_LinConnection *c, int i)
{
  VALUE a;
  a = rb_hash_new();
  rb_hash_aset(a,rb_str_new2("Type"),INT2FIX(c->Bind[i].type));	 
  rb_hash_aset(a, rb_str_new2("Length"), INT2FIX(c->Bind[i].len));	  
  rb_hash_aset(a, rb_str_new2("Precision"), INT2FIX(c->Bind[i].precision));	 
  rb_hash_aset(a, rb_str_new2("Scale"), INT2FIX(c->Bind[i].scale));	        
  return a;
}

/*
get info for bind parameter (object method)
input param:
[argv - number or name of parameter]
out param:
ar - ruby array
*/
static VALUE GetBindParamInfo(int argc, VALUE* argv, VALUE self)
{  
  t_LinConnection   *c;
  char              *name;    
  int               i = 0, N;
  VALUE             ar, param;
  short             n = 0;

  ar = rb_ary_new(); 
  Data_Get_Struct(self, t_LinConnection, c); 

  if ( argc == 0 )
  {
    for ( i = 0; i < c->ParamCount; i++ )
    {
      rb_ary_push(ar, BindParamInfo(c, i));      
    }
  }
  else if ( argc == 1 )
  {
    rb_scan_args(argc, argv, "01", &param);  
    switch  ( TYPE(param) )
    {
      case T_FIXNUM:                     
        N = FIX2INT(param); 
        if ( N <= c->ParamCount )
          rb_ary_push(ar, BindParamInfo(c, N - 1)); 
        else
        {
          c->Err = WRONG_NUMBER_PARAM;
          rb_raise(rb_eTypeError, "wrong bind parameter number");  /* 28.11.08 */
        }
        break;
      case T_STRING:
        name = StringValuePtr(param);                              /* 17.05.11 */
        inter_GetParamNumber(c->PrepareQuery, name, &n);           /* 24.09.08 */
        rb_ary_push(ar, BindParamInfo(c, n - 1));        
        break;
      default:
        c->Err = WRONG_TYPE_PARAM;
        rb_raise(rb_eTypeError, "wrong bind parameter type %s (expected FIXNUM/STRING)",
                 rb_obj_classname(param));                         /* 28.11.08 */
    }
  }
  else
  {
    c->Err = WRONG_COUNT_PARAM;
    rb_raise(rb_eArgError, "too many arguments");                  /* 28.11.08 */
  }

  return ar;
}

/*
binding parameter for compiled query (object method)
input parameters:
i - parameter number
param - significance of parameter's
out param:
self
*/
static VALUE SQLBindParameter_(VALUE self, VALUE i, VALUE param)                                
{
  t_LinConnection   *c;  
  short             N = 0;
  char              *q;
  char              szLocQuery[MAX_ID_LEN] = {0};
  int               j = 0;

  Data_Get_Struct(self, t_LinConnection, c);
  
  switch  ( TYPE(i) )
  {
    case T_FIXNUM:             
      c->flagOfBind = BindByNumber;
      N = FIX2INT(i); 
      break;
    case T_STRING:
      c->flagOfBind = BindByName;  
      q = StringValuePtr(i);                                       /* 17.05.11 */
      /*       inter_GetParamNumber*/
      do
      {
        szLocQuery[j] = LAnsiToupper(q[j]);
      } while ((j < MAX_ID_LEN-1) && q[j++] );

      szLocQuery[j] = 0;

      inter_GetParamNumber(c->PrepareQuery, szLocQuery, &N);                /* 17.06.10 */
      memcpy(c->Bind[N-1].name, q, RSTRING_LEN(i));                         /* 17.05.11 */
      break;
    default:
      c->Err = WRONG_TYPE_PARAM;
      rb_raise(rb_eTypeError, "wrong bind parameter type %s (expected FIXNUM/STRING)",
                 rb_obj_classname(i));                             /* 28.11.08 */
  }

  if ( N > c->ParamCount )
  {
    c->Err = WRONG_NUMBER_PARAM;
    rb_raise(rb_eTypeError, "wrong bind parameter number");        /* 28.11.08 */
  }
           
  c->Bind[N-1].fConvert(param, c, N-1); 

  return self;
}



static void AddBl(t_LinConnection *c, char * str, int len, int N)
{  
  memcpy( c->CBL->Command, "ABLB", 4 );
  c->CBL->RowId = 0;  
  c->CBL->RowCount = N;
   
  while (len >= 4048) 
  {
    c->CBL->LnBufRow = MAX_BUFF_BLOB;
    inter(c->CBL, NULL, NULL, NULL, str); 
    Error_Check(c);                                                /* 28.11.08 */
    c->CBL->RowId += MAX_BUFF_BLOB;         
    str += MAX_BUFF_BLOB;
    len -= MAX_BUFF_BLOB;
  }
  if (len) 
  {
    c->CBL->LnBufRow = len;
    inter(c->CBL, NULL, NULL, NULL, str);  
    Error_Check(c);                                                /* 28.11.08 */
  }  
}

static VALUE Get_Blob(t_LinConnection *c, int len, int nCol, int start)
{  
  VALUE             res;    
  char              *str, *a;
  int               i = 0, l;  
  
  c->CBL->RowId = start;
  memcpy(c->CBL->Command, "GBLB", 4);  
  l = len;
  c->CBL->RowCount = nCol;

  str = (unsigned char *) calloc(1, l);
  a = (unsigned char *) calloc(1, l);

  if ( str == NULL || a == NULL )
  {
    c->Err = NO_MEMORY;
    rb_raise(rb_eStandardError, "alloc failed");                   /* 28.11.08 */
  }
    
  while ((l >= 4048) && (c->Err == 0) ) 
  {     
    c->CBL->LnBufRow = MAX_BUFF_BLOB;
    inter(c->CBL, NULL, NULL, NULL, a); 
    Error_Check(c);                                                /* 28.11.08 */
    c->CBL->RowId += MAX_BUFF_BLOB;
    memcpy(str, a, l);        
    str += MAX_BUFF_BLOB;           
    l -= MAX_BUFF_BLOB;  
    i++;
  }        
  if (l) 
  {          
    c->CBL->LnBufRow = l;
    inter(c->CBL, NULL, NULL, NULL, a);                               
    Error_Check(c);                                                /* 28.11.08 */
    memcpy(str, a, l);                 
  }    
  str = str - i * MAX_BUFF_BLOB;    
  res = rb_tainted_str_new(str, len); 
  free(a);  
  free(str);

  return res;
}


/*
executing compiled query (object method)
input parameters:
argc - number of parameters
argv - VALUE with parameters ([array with parameters])
output param:
self
*/
static VALUE SQLExecute_(int argc, VALUE* argv, VALUE self)
{
  t_LinConnection   *c;
  int               i, l;
  VALUE             ar, v;

  Data_Get_Struct(self, t_LinConnection, c);    

  c->flagOfBind = BindByNumber; 
  if ( argc == 1 )
  {   
    rb_scan_args(argc, argv, "01", &ar);    
    if ( TYPE(ar) == T_ARRAY )
    {
      l = RARRAY_LEN(ar);               /* 17.05.11 */
      if ( l > c->ParamCount )          /* 15.07.09 */
        l = c->ParamCount;              /* 15.07.09 */  
      for ( i = 0; i < l; i++ )
      {    
        v = rb_ary_shift(ar);                         
        c->Bind[i].fConvert(v, c, i); 
        c->Err = c->CBL->CodErr;                     
      }
    }
    else
    {
      c->Err = WRONG_TYPE_PARAM;
      rb_raise(rb_eTypeError, "wrong bind parameter type %s (expected ARRAY)",
               rb_obj_classname(ar));                              /* 28.11.08 */
    }
  }
  else if ( argc > 1 )
  {
    c->Err = WRONG_COUNT_PARAM;
    rb_raise(rb_eArgError, "too many arguments");                  /* 28.11.08 */
  }     
  
  c->CBL->LnBufRow = MAX_ROW_SIZE;  
  memcpy(c->CBL->Command, "EXEC", 4);    
  inter(c->CBL, NULL, c->PrepareQuery, NULL, c->RowBuf);   
  Error_Check(c);                                                  /* 28.11.08 */

  c->RowCount = c->CBL->RowCount;
  c->RowBufLen = c->CBL->LnBufRow; 
  c->CurrentRow = 1;

  if ( c->flagOfSelect )                                     /* 09.06.10 ... */
  {
    SetColCount(c);
    c->CurrentBlobCol = 0;
    c->BlobColCount = 0;	
    SetAnswerInfo(c);
    SetBlobInfo(c);
    SetColumnInfo(c);  
  }                                                          /* ... 09.06.10 */
  else
  {      
    for ( i = 0; i < c->ParamCount; i++ )
    {              
      if ( c->Bind[i].type == DT_BLOB )
      {    
        c->CurrentBlobCol = inter_GetBlobColNum(c->PrepareQuery, (L_WORD)(i+1));
        AddBl(c, c->Bind[i].str, c->Bind[i].len, c->CurrentBlobCol);                   
      }         
    }
  }

  if (c->flagOfProc)
  {      
    proc_(c);      
  }           

  return self;
}


static VALUE Fetch(t_LinConnection *c, long Direct, long Pos, int ResultView)
{  
  unsigned char     *VarBuf; 
  L_WORD              i;
  VALUE             ar, text, key;
  char              *buf;
  L_WORD            Length = 0;
  EXTFILE_DESC      *extfileDesc;  /* 22.10.14a */
    
  text = rb_ary_new();  

  switch( ResultView )
  {
    case HashResult:
      ar = rb_hash_new();              
      break;
    case RowResult:
    default:                                                /* 05.11.13 */
      ar = rb_ary_new();                               
      break;
  }
             
  c->CBL->LnBufRow = c->RowBufLen; 
  switch( Direct )
  {
    case FETCH_FIRST:
      c->CurrentRow = 1;
      memcpy(c->CBL->Command, "GETF", 4);                                                   
      break;
    case FETCH_LAST:
      c->CurrentRow = c->RowCount;
      memcpy(c->CBL->Command, "GETL", 4);                                                                
      break;
    case FETCH_NEXT:
      (c->CurrentRow++);
      memcpy(c->CBL->Command, "GETN", 4);                         
      break;
    case FETCH_PREV:
      (c->CurrentRow--);
      memcpy(c->CBL->Command, "GETP", 4);                   
      break;
    case FETCH_ABSNUM:
      c->CurrentRow = Pos;
      memcpy(c->CBL->Command, "GETS", 4);
      c->CBL->RowId = Pos;                            
      break;
    default:
      c->Err = FETCH_DIR; 
      break;
  } 

  c->NumBlobCol = 0; 

  VarBuf = (unsigned char *) calloc(1, BUFF);
  buf = (unsigned char *) calloc(1, c->RowBufLen);

  if ( VarBuf == NULL || buf == NULL )
  {
    c->Err = NO_MEMORY;
    rb_raise(rb_eStandardError, "alloc failed");                   /* 28.11.08 */
  }
  
  inter(c->CBL, VarBuf, NULL, NULL, c->RowBuf);
  Error_Check(c);                                                  /* 28.11.08 */
  
  VarBuf += 2 * sizeof(L_WORD);				
  
  SetColumnInfo(c);
  if ( c->RowCount != 0 )
  {
    for ( i = 1; i <= c->ColCount; i++ )
    {                 
      c->CurrentCol = i;
      if ( c->AnswerInfo[i-1].Type == DT_BLOB )              
      {      
        c->NumBlobCol++;
      }
      /* TODO: need to remove memmove operation. Use sequential scan for input buffer */
      memset(buf, 0, c->RowBufLen);                        /* 23.08.12 */
      if ( c->AnswerInfo[i-1].Type == DT_VARCHAR || 
           c->AnswerInfo[i-1].Type == DT_VARBYTE ||
           c->AnswerInfo[i-1].Type == DT_NVARCHAR )
      {  
        memcpy(&Length, c->RowBuf, 2);                     /* 23.08.12 */
        memcpy(buf, c->RowBuf+2, c->AnswerInfo[i-1].Length);/* 23.08.12 */
        memmove(c->RowBuf, c->RowBuf + c->AnswerInfo[i-1].Length + 2, c->RowBufLen); /* 23.08.12 */
      }
      else if (c->AnswerInfo[i - 1].Type == DT_EXTFILE)                              /* 22.10.14a ... */
      {
        Length = L_EXTFILENAMESIZE;
        extfileDesc = (EXTFILE_DESC*)calloc(1, sizeof(EXTFILE_DESC));
        memcpy(extfileDesc, c->RowBuf, c->AnswerInfo[i - 1].Length);
        memcpy(buf, extfileDesc->FileName, L_EXTFILENAMESIZE);
        memmove(c->RowBuf, c->RowBuf + c->AnswerInfo[i - 1].Length, c->RowBufLen);
        free(extfileDesc);
      }                                                                              /* ... 22.10.14a */
      else
      {			
        Length = c->AnswerInfo[i-1].Length;
        memcpy(buf, c->RowBuf, c->AnswerInfo[i-1].Length);
        memmove(c->RowBuf, c->RowBuf + c->AnswerInfo[i-1].Length, c->RowBufLen); /* 23.08.12 */
      }

      if (  VarBuf[i-1] == 0 )
      {			                    
        text = ToValue(c, buf, c->AnswerInfo[i-1].Type, Length);   
      }
      else
      {
        text = Qnil;
      }
			      
      switch( ResultView )
      {
        case HashResult:                    		
          escape(c->AnswerInfo[i-1].Column, MAX_ID_LEN);   /* 23.08.12 */
          key = rb_str_new2(c->AnswerInfo[i-1].Column);										
          rb_hash_aset(ar, key, text);            
          break;
        case RowResult:
        default:                                            /* 05.11.13 */
          rb_ary_push(ar, text);                           
          break;
      }    			
    }
  }

  VarBuf -= 2*sizeof(L_WORD);
  free(VarBuf);
  free(buf); 
  
  return ar;
}




/*
fetching the row of data from the result set and returns data for all bound columns (object method)
input params:
argc - number of parameters
argv - VALUE with parameters (direct - fetch directions [,pos - number of FETCH_ABSNUM])
output param:
ar - array with answer presented as ruby VALUE
*/
static VALUE SQLFetchRow(int argc, VALUE* argv, VALUE self)
{
  long              Direct,Pos;  
  VALUE             direct, pos;  
  t_LinConnection   *c;
 
  Data_Get_Struct(self, t_LinConnection, c);  
  
  switch( argc )
  {
    case 1:
      rb_scan_args(argc, argv, "01", &direct);
      Pos = 0;
      break;
    case 2:
      rb_scan_args(argc, argv, "02", &direct, &pos);  
      if ( TYPE(pos) == T_FIXNUM )
        Pos = (long)NUM2LL(pos);
      else
      {
        c->Err = WRONG_TYPE_PARAM;
        rb_raise(rb_eTypeError, "wrong parameter type %s (expected FIXNUM)", 
                 rb_obj_classname(pos));                           /* 28.11.08 */
      }
      break;      
    default:
      c->Err = WRONG_COUNT_PARAM;
      rb_raise(rb_eArgError, "too many arguments");                /* 28.11.08 */
  }  

  if ( TYPE(direct) == T_FIXNUM )
    Direct = (long)NUM2LL(direct); 
  else
  {
    c->Err = WRONG_TYPE_PARAM;
    rb_raise(rb_eTypeError, "wrong parameter type %s (expected FIXNUM)", 
             rb_obj_classname(direct));                            /* 28.11.08 */
  }

  return Fetch(c, Direct, Pos, RowResult);
}

/*
fetching the row of data from the result set and returns data for all bound columns (object method)
input params:
argc - number of parameters
argv - VALUE with parameters 
([, count - row count]
[,pos - number of start row])
output param:
ar - array of arrays with answer presented as ruby VALUE
*/
static VALUE SQLFetchManyRow(int argc, VALUE* argv, VALUE self)
{
  long              RowCount;        
  int               i, canfetch;
  VALUE             ar, count, pos;  
  t_LinConnection   *c;

  Data_Get_Struct(self, t_LinConnection, c);
  ar = rb_ary_new();
  
  switch( argc )
  {
    case 0:
      RowCount = c->RowCount - c->CurrentRow + 1;                  /* 30.03.09 */
      for ( i = 0; i < RowCount; i++ )
      {
        rb_ary_push(ar, Fetch(c, 5, c->CurrentRow, RowResult));
        c->CurrentRow++;
      }
      break;
    case 1:
      rb_scan_args(argc, argv, "01", &count);

      if ( TYPE(count) == T_FIXNUM )
      {
        RowCount = (long)NUM2LL(count);
        canfetch = c->RowCount - c->CurrentRow + 1;
        if ( RowCount > canfetch ) 
          RowCount = canfetch;
        for ( i = 0; i < RowCount; i++ )
        {
          rb_ary_push(ar, Fetch(c, 5, c->CurrentRow, RowResult));   
          c->CurrentRow++;
        }
      }
      else
      {
        c->Err = WRONG_TYPE_PARAM;      
        rb_raise(rb_eTypeError, "wrong parameter type %s (expected FIXNUM)", 
                 rb_obj_classname(count));                         /* 28.11.08 */
      } 
      break;
    case 2:
      rb_scan_args(argc, argv, "02", &count, &pos); 
      if ( TYPE(count) == T_FIXNUM && TYPE(pos) == T_FIXNUM )
      {
        RowCount = (long)NUM2LL(count);      
        c->CurrentRow = (int)NUM2LL(pos);
        canfetch = c->RowCount - c->CurrentRow + 1;
        if ( RowCount > canfetch ) 
          RowCount = canfetch;
        for ( i = 0; i < RowCount; i++ )
        {
          rb_ary_push(ar, Fetch(c, 5, c->CurrentRow, RowResult)); 
          c->CurrentRow++;
        }
      }
      else
      {
        c->Err = WRONG_TYPE_PARAM;
        rb_raise(rb_eTypeError, "wrong parameter(s) type (expected FIXNUM)"); /* 28.11.08 */
      }
      break;      
    default:
      c->Err = WRONG_COUNT_PARAM;
      rb_raise(rb_eArgError, "too many arguments");               /* 28.11.08 */
  }          
  return ar;
}


/*
fetching the row of data from the result set and returns data for all bound columns (object method)
input params:
argc - number of parameters
argv - VALUE with parameters (direct - fetch directions [,pos - number of FETCH_ABSNUM])
output param:
ar - hash with answer presented as ruby VALUE
*/
static VALUE SQLFetchHash(int argc, VALUE* argv, VALUE self)
{
  long              Direct,Pos; 
  VALUE             direct, pos;  
  t_LinConnection   *c;
  
  Data_Get_Struct(self, t_LinConnection, c);  
  
  switch( argc )
  {
    case 1:
      rb_scan_args(argc, argv, "01", &direct);
      Pos = 0;
      break;
    case 2:
      rb_scan_args(argc, argv, "02", &direct, &pos);  
      if ( TYPE(pos) == T_FIXNUM)
        Pos = (long)NUM2LL(pos);
      else 
      {
        c->Err = WRONG_TYPE_PARAM;
        rb_raise(rb_eTypeError, "wrong parameter type %s (expected FIXNUM)", 
                 rb_obj_classname(pos));                           /* 28.11.08 */
      }
      break;      
    default:
      c->Err = WRONG_COUNT_PARAM;
      rb_raise(rb_eArgError, "too many arguments");                /* 28.11.08 */
  }        

  if ( TYPE(direct) == T_FIXNUM )
    Direct = (long)NUM2LL(direct);
  else
  {
    c->Err = WRONG_TYPE_PARAM;  
    rb_raise(rb_eTypeError, "wrong parameter type %s (expected FIXNUM)", 
             rb_obj_classname(direct));                            /* 28.11.08 */
  }

  return Fetch(c, Direct, Pos, HashResult);
}

/*
fetching the row of data from the result set and returns data for all bound columns (object method)
input params:
argc - number of parameters
argv - VALUE with parameters 
([, count - row count]
[,pos - number of start row])
output param:
ar - array of hashs with answer presented as ruby VALUE
*/
static VALUE SQLFetchManyHash(int argc, VALUE* argv, VALUE self)
{
  long              RowCount;        
  int               i, canfetch;
  VALUE             ar, count, pos;  
  t_LinConnection   *c;

  Data_Get_Struct(self, t_LinConnection, c);
  ar = rb_ary_new();
  
  switch( argc )
  {
    case 0:
      RowCount = c->RowCount - c->CurrentRow + 1;
      for ( i = 0; i < RowCount; i++ )
      {
        rb_ary_push(ar, Fetch(c, 5, c->CurrentRow, HashResult));
        c->CurrentRow++;
      }
      break;
    case 1:
      rb_scan_args(argc, argv, "01", &count);
      if ( TYPE(count) == T_FIXNUM )
      {
        RowCount = (int)NUM2LL(count);
        canfetch = c->RowCount - c->CurrentRow + 1;
        if ( RowCount > canfetch ) 
          RowCount = canfetch;
        for ( i = 0; i < RowCount; i++ )
        {
          rb_ary_push(ar, Fetch(c, 5, c->CurrentRow, HashResult));   
          c->CurrentRow++;
        }
      }
      else 
      {
        c->Err = WRONG_TYPE_PARAM;
        rb_raise(rb_eTypeError, "wrong parameter type %s (expected FIXNUM)", 
                 rb_obj_classname(count));                         /* 28.11.08 */
      }
      break;
    case 2:
      rb_scan_args(argc, argv, "02", &count, &pos); 
      if ( TYPE(count) == T_FIXNUM && TYPE(pos) == T_FIXNUM)
      {
        RowCount = (int)NUM2LL(count);         
        c->CurrentRow = (int)NUM2LL(pos);
        canfetch = c->RowCount - c->CurrentRow + 1;
        if ( RowCount > canfetch ) 
          RowCount = canfetch;
        for ( i = 0; i < RowCount; i++ )
        {
          rb_ary_push(ar, Fetch(c, 5, c->CurrentRow, HashResult)); 
          c->CurrentRow++;
        }
      }
      else
      {
        c->Err = WRONG_TYPE_PARAM;
        rb_raise(rb_eTypeError, "wrong parameter(s) type (expected FIXNUM)"); /* 28.11.08 */
      }
      break;      
    default:
      c->Err = WRONG_COUNT_PARAM;
      rb_raise(rb_eArgError, "too many arguments");               /* 28.11.08 */
  }    
  
  return ar;
}


/*
returning the SQL string as modified(object method)
input params:
str - SQL text string to be translated
output param:
v - query presented as ruby VALUE after transformation
*/
static VALUE SQLNativeSql_(VALUE self, VALUE str)
{ 
  unsigned short    ResLen, ResMaxLen = MAX_BUFF;
  VALUE             v;
  char              *res, *a;
  int               len;
  t_LinConnection   *c;

  Data_Get_Struct(self, t_LinConnection, c);

  if ( TYPE(str) == T_STRING )
  {  
    a = (char *) calloc(1, MAX_BUFF);
    res = (char *) calloc(1, MAX_BUFF);

    if ( res == NULL || a == NULL )
    {
      c->Err = NO_MEMORY;
      rb_raise(rb_eStandardError, "alloc failed");                 /* 28.11.08 */
    }

    strcat(a, StringValuePtr(str));                                /* 17.05.11 */
    len = (int)strlen(a);				           
    ReplaceEscapeSequences(a, (L_WORD)len, res, &ResLen, ResMaxLen);
    v = rb_str_new2(res);   
    free(res);   
  }
  else
  {
    c->Err = WRONG_TYPE_PARAM;
    rb_raise(rb_eTypeError, "wrong type of parameter");            /* 28.11.08 */
  }
	   
  return v;
}



static VALUE GetProcOutParams(VALUE self)
{
  t_LinConnection *c;
  Data_Get_Struct(self, t_LinConnection, c);

  return (TYPE(c->cur) != T_NIL) ? c->cur : Qnil;
}


/*
viewing quantity of answers (object method)
output param:
INT2FIX(c->RowCount) - row count presented as ruby VALUE
*/
static VALUE GetRowCount(VALUE self)
{
  t_LinConnection *c;
  Data_Get_Struct(self, t_LinConnection, c);

  return INT2FIX(c->RowCount);
}

static VALUE ViewColumnInfo(t_LinConnection *c, int ans_n, int blob_n)
{
  VALUE     a;

  a = rb_hash_new();
  rb_hash_aset(a, rb_str_new2("Type"), INT2FIX(c->AnswerInfo[ans_n-1].Type));
  escape(c->AnswerInfo[ans_n-1].Table, MAX_ID_LEN);
  rb_hash_aset(a, rb_str_new2("Table"), rb_str_new2(c->AnswerInfo[ans_n-1].Table));
  escape(c->AnswerInfo[ans_n-1].Column, MAX_ID_LEN);       /* 23.08.12 */
  if (strlen(c->AnswerInfo[ans_n - 1].Column) == MAX_ID_LEN + 1)                                    /* 20.10.14 ... */
    rb_hash_aset(a, rb_str_new2("Column"), rb_str_new(c->AnswerInfo[ans_n-1].Column, MAX_ID_LEN));
  else
    rb_hash_aset(a, rb_str_new2("Column"), rb_str_new2(c->AnswerInfo[ans_n-1].Column));             /* ... 20.10.14 */
  escape(c->AnswerInfo[ans_n-1].User, MAX_ID_LEN);
  rb_hash_aset(a, rb_str_new2("User"), rb_str_new2(c->AnswerInfo[ans_n-1].User));   

  if ( c->AnswerInfo[ans_n-1].Type != DT_BLOB )
  {
#if _VER_MAX >=600
    rb_hash_aset(a,rb_str_new2("CharSet"),INT2FIX(c->AnswerInfo[ans_n-1].CharSet));	 
#endif
    rb_hash_aset(a, rb_str_new2("Length"), INT2FIX(c->AnswerInfo[ans_n-1].Length));	  
    rb_hash_aset(a, rb_str_new2("Precision"), INT2FIX(c->AnswerInfo[ans_n-1].Precision));	 
    rb_hash_aset(a, rb_str_new2("Scale"), INT2FIX(c->AnswerInfo[ans_n-1].Scale));	        
  }
  else
  {
    rb_hash_aset(a, rb_str_new2("Size"), LL2NUM(c->BlobInfo[blob_n-1].Size));    
    rb_hash_aset(a, rb_str_new2("Index_Time"), rb_str_new2(c->BlobInfo[blob_n-1].Index_Time));
    rb_hash_aset(a, rb_str_new2("TypeObj"), LL2NUM(c->BlobInfo[blob_n-1].TypeObj));
  }      

  return a;
}


/*
viewing info about answer (object method)
input param:
argc - number of parameters
argv - VALUE with parameters 
([Numb - number of colomn])
output param:
hash - associate array with info about each colomn presented as ruby VALUE
*/
static VALUE GetAnswerInfo(int argc, VALUE* argv, VALUE self)
{
  VALUE             hash, a, Numb;
  L_WORD              i, num = 0, n;
  t_LinConnection   *c;

  Data_Get_Struct(self, t_LinConnection, c);
  hash = rb_hash_new();

  switch( argc )
  {
    case 0:
      for ( i = 1; i <= c->ColCount; i++ )
      {          
        if ( c->AnswerInfo[i-1].Type == DT_BLOB )                    
          num++;
        a = ViewColumnInfo(c, i, num);        
        rb_hash_aset(hash, INT2FIX(i), a);                         /* 23.03.12 */
      }
      break;
    case 1:
      rb_scan_args(argc, argv, "01", &Numb);   
      if ( TYPE(Numb) == T_FIXNUM )
      {
        n = (int)NUM2LL(Numb);        
        if ( n < 1 || n > c->ColCount )                            /* 12.03.12 */
          rb_raise(rb_eArgError, "wrong argument number");
        if ( c->AnswerInfo[n-1].Type != DT_BLOB ) 
        {
          num = 0;         
          i = n;                                                   /* 12.03.12 */
        }
        else
        {
          i = 1;
          while ( i <= n )
          {          
            if ( c->AnswerInfo[i-1].Type == DT_BLOB )                    
              num++;                       
            i++;
          }           
        }    
        a = ViewColumnInfo(c, n, num); 
        rb_hash_aset(hash, INT2FIX(i), a);                         /* 23.03.12 */
      }
      else
      {
        c->Err = WRONG_TYPE_PARAM;
        rb_raise(rb_eTypeError, "wrong parameter type %s (expected FIXNUM)", 
                 rb_obj_classname(Numb));                          /* 28.11.08 */
      }
      break;      
    default:
      c->Err = WRONG_COUNT_PARAM;
      rb_raise(rb_eArgError, "too many arguments");                /* 28.11.08 */
  }       

  return hash;
}


static VALUE SQLTables_(VALUE self, VALUE TableOwner, VALUE TableName, VALUE TableType)
{
  t_LinConnection   *c;   
  char              *Query, *type;
  L_BOOL            is_where = L_TFALSE;
  int               cnt = 0;

  Data_Get_Struct(self, t_LinConnection, c); 

  if ( (TYPE(TableName) == T_STRING || TYPE(TableName) == T_NIL) && 
       (TYPE(TableType) == T_STRING || TYPE(TableType) == T_NIL) &&
       (TYPE(TableOwner) == T_STRING || TYPE(TableOwner) == T_NIL) )
  {   
    if ( c->AnswerInfo != NULL )                             /* 05.07.10 ... */
    {  
      free(c->AnswerInfo);
      c->AnswerInfo = NULL;
    }

    if ( c->BlobInfo != NULL )
    {
      free(c->BlobInfo);
      c->BlobInfo = NULL;
    }                                                        /* ... 05.07.10 */

    Query = (char *) calloc(1, MAX_BUFF);
    if ( Query == NULL ) 
    {
      c->Err = NO_MEMORY;
      rb_raise(rb_eStandardError, "alloc failed");                 /* 28.11.08 */
    }   
    strcat(Query, "SELECT * FROM LINTER_SYSTEM_USER.tables");

    if ( TYPE(TableOwner) == T_STRING )
    {        
      strcat(Query, " WHERE TABLE_SCHEM LIKE '");
      strcat(Query, StringValuePtr(TableOwner));                   /* 17.05.11 */
      strcat(Query, "'");
      is_where = L_TTRUE;
    }
    
    if ( TYPE(TableName) == T_STRING )
    {        
      strcat(Query, is_where ? " AND" : " WHERE");
      strcat(Query, " TABLE_NAME LIKE '");
      strcat(Query, StringValuePtr(TableName));                    /* 17.05.11 */
      strcat(Query, "'");
      is_where = L_TTRUE;
    }
    
    if ( TYPE(TableType) == T_STRING )
    {        
      strcat(Query, is_where ? " AND" : " WHERE");
      strcat(Query, " TABLE_TYPE IN (");                
      type = StringValuePtr(TableType);                            /* 17.05.11 */
      while ( *type )
      {
        if ( cnt > 0 )
        {
          strcat(Query, ", ");  
        }
        strcat(Query, "'"); 
        if ( strstr(type, ",") == 0 )                        /* 09.06.10 ... */
        {           
          while (*type == ' ')
             memcpy(type, type + 1, strlen(type));
          memcpy(Query + (int)strlen(Query), type, strlen(type));
          memcpy(Query + (int)strlen(Query), "'", 1);
          memcpy(type, type+strlen(type), strlen(type));
        }
        else
        {       
          cnt++;
          while (*type == ' ')
             memcpy(type, type + 1, strlen(type));
          memcpy(Query + (int)strlen(Query), type, strcspn(type, ","));
          memcpy(Query + (int)strlen(Query), "'", 1);
          memcpy(type, type+strcspn(type, ",")+1, strlen(type));
        }                                                    /* ... 09.06.10 */
      }                    
      strcat(Query, ")");         
    }      
         
    memcpy(c->CBL->Command, "    ", 4);                            
    strcat(Query, " Order By TABLE_TYPE,TABLE_SCHEM,TABLE_NAME;");                
    c->CBL->LnBufRow = MAX_BUFF;    
    inter(c->CBL, NULL, Query, NULL, c->RowBuf);
    Error_Check(c);                                                /* 28.11.08 */
    c->RowCount = c->CBL->RowCount;
    c->RowBufLen = c->CBL->LnBufRow;  
    
    SetColCount(c);                                          /* 11.06.10 ... */
    c->AnswerInfo = (GETA_OUT *) calloc(c->ColCount, sizeof(GETA_OUT));
    if ( c->AnswerInfo == NULL  )
    {
      c->Err = NO_MEMORY;
      rb_raise(rb_eStandardError, "alloc failed");                   
    }                                                        /* ... 11.06.10 */
    c->CurrentBlobCol = 0;
    c->BlobColCount = 0;	
    SetAnswerInfo(c);
    SetBlobInfo(c);
    SetColumnInfo(c);      
    free(Query);   
  }  
  else
  {
    c->Err = WRONG_TYPE_PARAM;
    rb_raise(rb_eTypeError, "wrong parameter(s) type (expected STRING/NIL)"); /* 05.11.13 28.11.08 */
  }

  c->CurrentRow = 1;

  return self;
}


static VALUE SQLColumns_(VALUE self, VALUE TableOwner, VALUE TableName, VALUE TableColumn)
{
  t_LinConnection *c;  
  char            *Query;
  L_BOOL          is_where = L_TFALSE;

  Data_Get_Struct(self, t_LinConnection, c); 

  if ( (TYPE(TableName) == T_STRING || TYPE(TableName) == T_NIL) && 
       (TYPE(TableColumn) == T_STRING || TYPE(TableColumn) == T_NIL) &&
       (TYPE(TableOwner) == T_STRING || TYPE(TableOwner) == T_NIL) )
  {
    if ( c->AnswerInfo != NULL )                             /* 05.07.10 ... */
    {  
      free(c->AnswerInfo);
      c->AnswerInfo = NULL;
    }

    if ( c->BlobInfo != NULL )
    {
      free(c->BlobInfo);
      c->BlobInfo = NULL;
    }                                                        /* ... 05.07.10 */

    Query = (char *) calloc(1, MAX_BUFF);
    if ( Query == NULL ) 
    {
      c->Err = NO_MEMORY;
      rb_raise(rb_eStandardError, "alloc failed");                 /* 28.11.08 */
    }     
    strcat(Query, "SELECT * FROM LINTER_SYSTEM_USER.columns");

    if ( TYPE(TableOwner) == T_STRING )
    {        
      strcat(Query, " WHERE TABLE_SCHEM LIKE '");
      strcat(Query, StringValuePtr(TableOwner));                   /* 17.05.11 */
      strcat(Query, "'");
      is_where = L_TTRUE;
    }

    if ( TYPE(TableName) == T_STRING )
    {        
      strcat(Query, is_where ? " AND" : " WHERE");
      strcat(Query, " TABLE_NAME LIKE '");
      strcat(Query, StringValuePtr(TableName));                    /* 17.05.11 */
      strcat(Query, "'");
      is_where = L_TTRUE;
    }
    
    if ( TYPE(TableColumn) == T_STRING )
    {        
      strcat(Query, is_where ? " AND" : " WHERE");
      strcat(Query, " COLUMN_NAME LIKE '");  
      strcat(Query, StringValuePtr(TableColumn));                  /* 17.05.11 */
      strcat(Query, "'");  
    }
                  
    memcpy(c->CBL->Command, "    ", 4);                                     
    strcat(Query, " Order By TABLE_SCHEM,TABLE_NAME,ORDINAL_POSITION;");                
    c->CBL->LnBufRow = MAX_BUFF;    
    inter(c->CBL, NULL, Query, NULL, c->RowBuf);
    Error_Check(c);                                                /* 28.11.08 */
    c->RowCount = c->CBL->RowCount;
    c->RowBufLen = c->CBL->LnBufRow; 
    SetColCount(c);                                          /* 11.06.10 ... */
    c->AnswerInfo = (GETA_OUT *) calloc(c->ColCount, sizeof(GETA_OUT));
    if ( c->AnswerInfo == NULL  )
    {
      c->Err = NO_MEMORY;
      rb_raise(rb_eStandardError, "alloc failed");                   
    }                                                        /* ... 11.06.10 */
    c->CurrentBlobCol = 0;
    c->BlobColCount = 0;	
    SetAnswerInfo(c);
    SetBlobInfo(c);
    SetColumnInfo(c);
    free(Query);   
  }
  else
  {
    c->Err = WRONG_TYPE_PARAM;
    rb_raise(rb_eTypeError, "wrong parameter(s) type (expected STRING/NIL)"); /* 05.11.13 28.11.08 */
  }

  c->CurrentRow = 1;
   
  return self;
}


/*
getting current Id (object method)
output param:
INT2FIX(i) - current id presented as ruby VALUE
*/
static VALUE GetLastRowId(VALUE self)                              /* 23.10.08 */
{  
  t_LinConnection   *c;
  int               i; 

  Data_Get_Struct(self, t_LinConnection, c);
  memcpy(c->CBL->Command, "SLCT", 4);    
  c->CBL->LnBufRow = MAX_BUFF;   
  inter(c->CBL, NULL, "select last_rowid;", NULL, &i);             /* 23.10.08 */
  Error_Check(c);                                                  /* 28.11.08 */
  
  return INT2FIX(i);
}


/*
getting last auto increment Id (object method)
output param:
INT2FIX(i) - last auto inc id presented as ruby VALUE
*/
static VALUE GetLastAutoInc(VALUE self)                            /* 23.10.08 */
{  
  t_LinConnection   *c;
  int               i;   
  
  Data_Get_Struct(self, t_LinConnection, c);
  memcpy(c->CBL->Command, "SLCT", 4);    
  c->CBL->LnBufRow = MAX_BUFF; 
  inter(c->CBL, NULL, "select last_autoinc;", NULL, &i);           /* 23.10.08 */
  Error_Check(c);                                                  /* 28.11.08 */
  return INT2FIX(i);
}

/*
closing connection (object method)
*/
static VALUE CloseConnection(VALUE self)
{
  t_LinConnection   *c;
  Data_Get_Struct(self, t_LinConnection, c);                       /* 28.05.10 ... */

  if ( c->list != NULL )
  {
    t_LinConnection *list = c->list;
    while ( list ) 
    {
      t_LinConnection *p;
      p = list;
      list = list->next;

      memcpy(p->CBL->Command, "CLOS", 4);      
      inter(p->CBL, NULL, NULL, NULL, NULL);
      Error_Check(p);		                 		
    }
  }

  memcpy(c->CBL->Command, "CLOS", 4); 
  inter(c->CBL, NULL, NULL, NULL, NULL);
  Error_Check(c);
 
  return self;                                                     /* ... 28.05.10 */
}

/*
dismissal memory (object method)
*/
static VALUE FreeConnection(VALUE self)                      /* 28.05.10 ... */
{
  t_LinConnection   *c;
  int               i;
  Data_Get_Struct(self, t_LinConnection, c);

  if ( linter_connection_list == c )
    linter_connection_list = c->next;
  else 
  {
    t_LinConnection *list = linter_connection_list;
    while ( list ) 
    {
      if ( list == c )
      {
        list->next = c->next;
        break;
      }
      list = list->next;
    }
  } 

  if ( c->list != NULL )
  {
    t_LinConnection *list = c->list;
    while ( list ) 
    {
      t_LinConnection *p;
      p = list;
      list = list->next;

      if ( p->PrepareQuery != NULL )
      {
        free(p->PrepareQuery);
        p->PrepareQuery = NULL;
      }
      if ( p->format != NULL )
      {
        free(p->format);   
        p->format = NULL;
      }
      
      for ( i = 0; i < p->ParamCount; i++ )
      {
        if ( p->Bind[i].type != DT_INTEGER )	
        {
          free(p->Bind[i].str);
          p->Bind[i].str = NULL; 
        }
      }
      
      if ( p->Bind != NULL )
      {
        free(p->Bind);             
        p->Bind = NULL;
      }
      if ( p->AnswerInfo != NULL )
      {
        free(p->AnswerInfo);
        p->AnswerInfo = NULL;
      }
      if ( p->CBL != NULL )
      {
        free(p->CBL); 
        p->CBL = NULL;
      }
      if ( p->RowBuf != NULL )
      {
        free(p->RowBuf);
        p->RowBuf = NULL;
      }
      if ( p->info != NULL )
      {
        free(p->info);
        p->info = NULL;
      }
      if ( p->BlobInfo != NULL )
      {
        free(p->BlobInfo);
        p->BlobInfo = NULL;
      }
      xfree(p);
    }
  }
  if ( c->PrepareQuery != NULL )
  {
    free(c->PrepareQuery);	
    c->PrepareQuery = NULL;
  }
  if ( c->format != NULL )
  {
    free(c->format);  
    c->format = NULL;
  }
  for ( i = 0; i < c->ParamCount; i++ )
  {
    if ( c->Bind[i].type != DT_INTEGER )
    {
      free(c->Bind[i].str);
      c->Bind[i].str = NULL;
    }
  }
  if ( c->Bind != NULL )
  {
    free(c->Bind);
    c->Bind = NULL;
  }
  if ( c->AnswerInfo != NULL )
  {
    free(c->AnswerInfo);
    c->AnswerInfo = NULL;
  }
  if ( c->CBL != NULL )
  {
    free(c->CBL);
    c->CBL = NULL;
  }
  if ( c->RowBuf != NULL)
  {
    free(c->RowBuf);
    c->RowBuf = NULL;
  }
  xfree(c->info);
  if ( c->BlobInfo != NULL )
  {
    free(c->BlobInfo);
    c->BlobInfo = NULL;
  }
  xfree(c);
 
  return Qnil;
}                                                       /* ... 28.05.10 */

/*
getting Blob (object method)
input params:
argc - number of parameters
argv - VALUE with parameters 
(Start - start position of blob portion, 
Len - length of portion
[,Numb - number of colomn])
output param:
res - blob info presented as ruby VALUE
*/
static VALUE GetBlob(int argc, VALUE* argv, VALUE self)
{
  VALUE             Start, Len, Numb, res, text;    
  int               i = 0, blob_num = 0, len = 0, st = 0, num = 0;
  t_LinConnection   *c;

  Data_Get_Struct(self, t_LinConnection, c);  
  res = rb_ary_new();

  if ( c->BlobColCount >= 1 )
  {
    switch( argc )
    {
      case 0:                                                      /* 22.06.10 ... */        
        for ( i = 0; i < c->ColCount; i++ )
        {
          if ( c->AnswerInfo[i].Type == DT_BLOB )
          {
            text = Get_Blob(c, c->BlobInfo[blob_num].Size, i+1, 1);
            rb_ary_push(res, text);
            blob_num++;
          }
        }       
        break;                                                     
      case 1:                                                     
        rb_scan_args(argc, argv, "01", &Start);  
        if ( TYPE(Start) == T_FIXNUM )
        {
          for ( i = 0; i < c->ColCount; i++ )
          {
            if ( c->AnswerInfo[i].Type == DT_BLOB )
            {
              text = Get_Blob(c, c->BlobInfo[blob_num].Size, i+1, (int)NUM2LL(Start));
              rb_ary_push(res, text);
              blob_num++;
            }
          }
        }
        else
        {
          c->Err = WRONG_TYPE_PARAM;
          rb_raise(rb_eTypeError, "wrong parameter(s) type (expected FIXNUM)"); /* 05.11.13 */
        }
        break;                                                     
      case 2:        
        rb_scan_args(argc, argv, "02", &Start, &Len);  
        if ( (TYPE(Start) == T_FIXNUM || TYPE(Start) == T_NIL) && 
             (TYPE(Len) == T_FIXNUM) )
        {     
          if ( TYPE(Start) == T_NIL )
          { 
            st = 1;
          }
          else
          {
            st = (int)NUM2LL(Start);
          }

          len = (int)NUM2LL(Len);
          for ( i = 0; i < c->ColCount; i++ )                     
          {                 
            if ( c->AnswerInfo[i].Type == DT_BLOB )  
            {
              if ( len > c->BlobInfo[blob_num].Size )
              {
                len = c->BlobInfo[blob_num].Size;
              }
              text = Get_Blob(c, len, i+1, st);
              rb_ary_push(res, text);
              blob_num++;
            }
          }                                                        /* ... 22.06.10 */
        }
        else
        {
          c->Err = WRONG_TYPE_PARAM;
          rb_raise(rb_eTypeError, "wrong parameter(s) type (expected FIXNUM)"); /* 05.11.13 28.11.08 */
        }
        break;
      case 3:
        rb_scan_args(argc, argv, "03", &Start, &Len, &Numb);       /* 23.07.10 ... */
        if ( ( TYPE(Start) == T_FIXNUM || TYPE(Start) == T_NIL )&& 
             ( TYPE(Len) == T_FIXNUM || TYPE(Len) == T_NIL )&& 
             ( TYPE(Numb) == T_FIXNUM ) )
        {
          num = (int)NUM2LL(Numb);

          if ( TYPE(Start) == T_NIL )
          { 
            st = 1;
          }
          else
          {
            st = (int)NUM2LL(Start);
          }

          if ( TYPE(Len) == T_NIL )
          { 
            while ( (i < c->ColCount) || (i == num))                     
            {                 
              if ( c->AnswerInfo[i].Type == DT_BLOB )  
              { 
                len = c->BlobInfo[blob_num].Size;         
                blob_num++;
              }
              i++;
            } 
          }
          else
          {
            len = (int)NUM2LL(Len);
            i = 1;
            while ( (i <= c->ColCount) || (i == num))                     
            {                 
              if ( c->AnswerInfo[i-1].Type == DT_BLOB )  
              { 
                if ( (i == num)  && (len > c->BlobInfo[blob_num].Size) )
                {
                  len = c->BlobInfo[blob_num].Size;         
                  blob_num++;
                }
              }
              i++;
            }         
          }
          
          res = Get_Blob(c, len, num, st);                         /* ... 23.07.10 */
        }
        else
        {
          c->Err = WRONG_TYPE_PARAM;
          rb_raise(rb_eTypeError, "wrong parameter(s) type (expected FIXNUM)"); /* 05.11.13 28.11.08 */
        }
        break;      
      default:
        c->Err = WRONG_COUNT_PARAM;
        rb_raise(rb_eArgError, "too many arguments");              /* 28.11.08 */
    } 
  }
  else
  {
    c->Err = NO_BLOB_COLUMNS;
    rb_raise(rb_eTypeError, "no Blob columns");                    /* 28.11.08 */
  }


  return res;
}

/*
adding Blob (object method)
input params:
argc - number of parameters
argv - VALUE with parameters 
(TypeBlob - type of Blob,
BlobInfo - blob portion, 
[,Numb - number of colomn])
*/
static VALUE AddBlob(int argc, VALUE* argv, VALUE self)
{  
  VALUE             Numb, BlobInfo;  
  t_LinConnection   *c;
  char              *str;
  int               len, Ncol, i = 0;
  L_BOOL            blob_flag;

  Data_Get_Struct(self, t_LinConnection, c);

  switch( argc )
  {
    case 1:                                                        /* 23.06.10 ... */
      rb_scan_args(argc, argv, "01", &BlobInfo);
      c->CBL->RowCount = 1;
      if ( TYPE(BlobInfo) == T_STRING )
      {
        str = StringValuePtr(BlobInfo);                            
        len = RSTRING_LEN(BlobInfo);                               /* 17.05.11 */
        blob_flag = L_TTRUE;
        while ( i < c->ColCount && blob_flag )                                   
        {                 
          if ( c->AnswerInfo[i].Type == DT_BLOB ) 
          {
            AddBl(c, str, len, i+1);
            blob_flag = L_TFALSE;

          }
          i++;
        }
      }
      else
      {
        c->Err = WRONG_TYPE_PARAM;
        rb_raise(rb_eTypeError, "wrong parameter(s) type");        /* 28.11.08 */
      }
      break;
    case 2:
      rb_scan_args(argc, argv, "02", &BlobInfo, &Numb);  
      if ( TYPE(BlobInfo) == T_STRING && TYPE(Numb) == T_FIXNUM )
      {
        Ncol = (int)NUM2LL(Numb);
        str = StringValuePtr(BlobInfo);                            /* 17.05.11 */
        len = RSTRING_LEN(BlobInfo);                               /* 17.05.11 */
        AddBl(c, str, len, Ncol); 
      }
      else
      {
        c->Err = WRONG_TYPE_PARAM;
        rb_raise(rb_eTypeError, "wrong parameter(s) type");        /* 28.11.08 */
      }
      break;                                                       /* ... 23.06.10 */ 
    default:
      c->Err = WRONG_COUNT_PARAM;
      rb_raise(rb_eArgError, "too many arguments");                /* 28.11.08 */
  }            

  return self;
}


/*
deleting blob (object method)
input params:
argc - number of parameters
argv - VALUE with parameters 
([Numb - number of colomn])
*/
static VALUE DelBlob(int argc, VALUE* argv, VALUE self)
{
  int               i;
  VALUE             Numb; 
  t_LinConnection   *c;
  Data_Get_Struct(self, t_LinConnection, c);

  switch( argc )
  {
    case 0:
      for ( i = 0; i < c->ColCount; i++ )                      
        {                 
          if ( c->AnswerInfo[i].Type == DT_BLOB ) 
          {
            c->CBL->RowCount = i+1;
            memcpy( c->CBL->Command, "CBLB", 4 );
            inter(c->CBL, NULL, NULL, NULL, NULL); 
            c->Err = c->CBL->CodErr;
            Error_Check(c);  
          }
      }
      break;
    case 1:
      rb_scan_args(argc, argv, "01", &Numb);  
      if ( TYPE(Numb) == T_FIXNUM )
      {
        c->CBL->RowCount = (int)NUM2LL(Numb);
        memcpy( c->CBL->Command, "CBLB", 4 );
        inter(c->CBL, NULL, NULL, NULL, NULL); 
        c->Err = c->CBL->CodErr;
        Error_Check(c);  
      }
      else
      {
        c->Err = WRONG_TYPE_PARAM;
        rb_raise(rb_eTypeError, "wrong parameter type %s (expected FIXNUM)",
                 rb_obj_classname(Numb));                          /* 28.11.08 */
      }
      break;      
    default:
      c->Err = WRONG_COUNT_PARAM;
      rb_raise(rb_eArgError, "too many arguments");                /* 28.11.08 */
  }      
                                   
  return self;
}

/*
openning cursor (class method)
input param:
connect - connection, which cursor will submission
output param:
obj - ruby object cursor
*/
static VALUE OpenCursor(VALUE self, VALUE connect)
{ 
  VALUE             obj;
  t_LinConnection   *cursor;
  t_LinConnection   *c;

  Data_Get_Struct(connect, t_LinConnection, c);

  /* 10.09.12a -- cursor = calloc() removed */
  obj = Data_Make_Struct(self, t_LinConnection, 0, NULL, cursor);

  cursor->CBL = (TCBL *) calloc(1, sizeof(TCBL));
  cursor->RowBuf = (unsigned char *) calloc(1, MAX_BUFF);

  if ( cursor->CBL == NULL || cursor->RowBuf == NULL )
  {
    cursor->Err = NO_MEMORY;
    rb_raise(rb_eStandardError, "alloc failed");                 /* 28.11.08 */
  }
     
  cursor->CBL->LnBufRow = MAX_ROW_SIZE;
  memcpy(cursor->CBL->Command, "OCUR", 4);
  strcpy(cursor->CBL->Node, c->CBL->Node);
  cursor->CBL->NumChan = c->CBL->NumChan;
  cursor->CBL->PrzExe = cursor->PrzExe = c->PrzExe;
   
  cursor->CurrentRow = 1;      
  cursor->CBL->Prior = c->Priority;
  cursor->CBL->PrzExe &= ~Q_ASYNC;
  inter(cursor->CBL, NULL, NULL, NULL, NULL);
  cursor->Err = cursor->CBL->CodErr;
  Error_Check(cursor);                                           /* 28.11.08 */
   
  cursor->next = c->list;
  c->list = cursor;
  cursor->list = c;             
  cursor->flagOfProc = L_TFALSE;
  cursor->flagFetch = L_TTRUE;
  
  rb_obj_call_init(obj, 0, NULL);
  /* 10.09.12a -- calloc-related code removed */

  return obj;
}

/*
closing cursor (object method)
output param:
*/
static VALUE CloseCursor(VALUE self)
{
  t_LinConnection *connection, *cursor;
  int             i;
  Data_Get_Struct(self, t_LinConnection, cursor);

  connection = cursor->list;

  if ( connection->list == cursor )
  {
    connection->list = cursor->next;
  } 
  else 
  {
    t_LinConnection *list = connection->list;
    while ( list->next )
    {
      if ( list->next == cursor )
      {
        list->next = cursor->next;
        break;
      }
      list = list->next;
    }
  } 

  if ( cursor->PrepareQuery != NULL )
  {
    free(cursor->PrepareQuery);
    cursor->PrepareQuery = NULL;
  }
  if ( cursor->format != NULL )
  {
    free(cursor->format);
    cursor->format = NULL;
  }
  if ( cursor->AnswerInfo != NULL )
  {
    free(cursor->AnswerInfo);
    cursor->AnswerInfo = NULL;
  }
  for ( i = 0; i <= cursor->ParamCount - 1; i++ )
  {
    if (cursor->Bind[i].str != NULL)
    {
      if ( cursor->Bind[i].type != DT_INTEGER )		
      {
        free(cursor->Bind[i].str);
        cursor->Bind[i].str = NULL;
      }
    }
  }
  if ( cursor->Bind != NULL)
  {
    free(cursor->Bind);
    cursor->Bind = NULL;
  }

  memcpy(cursor->CBL->Command, "CLOS", 4);  
  inter(cursor->CBL, NULL, NULL, NULL, NULL);
  cursor->Err = cursor->CBL->CodErr;

  if ( cursor->CBL != NULL )
  {
    free(cursor->CBL);
    cursor->CBL = NULL;
  }
  if ( cursor->RowBuf != NULL )
  {
    free(cursor->RowBuf);
    cursor->RowBuf = NULL;
  }
  if ( cursor->info != NULL )
  {
    free(cursor->info);
    cursor->info = NULL;
  }

  xfree(cursor);  
  
  return Qnil;
}

void Init_LinRuby()
{
  rb_cLinterConnection = rb_define_class ("LinterConnection", rb_cObject);	  

  rb_define_singleton_method(rb_cLinterConnection, "Create", Create, 0);
  rb_define_singleton_method(rb_cLinterConnection, "Open", Open1, -1);
  rb_define_method(rb_cLinterConnection, "Open", Open2, -1);
  rb_define_method(rb_cLinterConnection, "GetDBInfo", GetDBInfo, 0);
  rb_define_method(rb_cLinterConnection, "SQLExecuteDirect", SQLExecuteDirect, 1);
  rb_define_method(rb_cLinterConnection, "SQLFetchRow", SQLFetchRow, -1);
  rb_define_method(rb_cLinterConnection, "SQLFetchManyRow", SQLFetchManyRow, -1);
  rb_define_method(rb_cLinterConnection, "SQLFetchHash", SQLFetchHash, -1);
  rb_define_method(rb_cLinterConnection, "SQLFetchManyHash", SQLFetchManyHash, -1);
  rb_define_method(rb_cLinterConnection, "SQLNativeSql", SQLNativeSql_, 1);
  rb_define_method(rb_cLinterConnection, "SQLPrepare", SQLPrepare_, 1);
  rb_define_method(rb_cLinterConnection, "SQLBindParameter", SQLBindParameter_, 2);
  rb_define_method(rb_cLinterConnection, "SQLExecute", SQLExecute_, -1);
  rb_define_method(rb_cLinterConnection, "GetRowCount", GetRowCount, 0);
  rb_define_method(rb_cLinterConnection, "GetAnswerInfo", GetAnswerInfo, -1);  
  rb_define_method(rb_cLinterConnection, "GetLastRowId", GetLastRowId, 0);
  rb_define_method(rb_cLinterConnection, "GetLastAutoInc", GetLastAutoInc, 0);    
  rb_define_method(rb_cLinterConnection, "GetBlob", GetBlob, -1);
  rb_define_method(rb_cLinterConnection, "AddBlob", AddBlob, -1);
  rb_define_method(rb_cLinterConnection, "DelBlob", DelBlob, -1);  
  rb_define_method(rb_cLinterConnection, "Close", CloseConnection, 0);
  rb_define_method(rb_cLinterConnection, "Free", FreeConnection, 0);
  rb_define_method(rb_cLinterConnection, "GetStatus", GetStatus, 0);
  rb_define_method(rb_cLinterConnection, "SQLTables", SQLTables_, 3);
  rb_define_method(rb_cLinterConnection, "SQLColumns", SQLColumns_, 3);
  rb_define_method(rb_cLinterConnection, "GetProcOutParams", GetProcOutParams, 0);
  rb_define_method(rb_cLinterConnection, "GetBindParamInfo", GetBindParamInfo, -1);
  rb_define_method(rb_cLinterConnection, "SetOption", SetOption, 2);

  rb_define_method(rb_cLinterConnection, "GetStatusOS", GetStatusOS, 0);

  rb_cLinterCursor = rb_define_class ("LinterCursor", rb_cObject);

  rb_define_singleton_method(rb_cLinterCursor, "Open", OpenCursor, 1);
  rb_define_method(rb_cLinterCursor, "GetDBInfo", GetDBInfo, 0);
  rb_define_method(rb_cLinterCursor, "SQLExecuteDirect", SQLExecuteDirect, 1);
  rb_define_method(rb_cLinterCursor, "SQLFetchRow", SQLFetchRow, -1);
  rb_define_method(rb_cLinterCursor, "SQLFetchManyRow", SQLFetchManyRow, -1);
  rb_define_method(rb_cLinterCursor, "SQLFetchHash", SQLFetchHash, -1);
  rb_define_method(rb_cLinterCursor, "SQLFetchManyHash", SQLFetchManyHash, -1);
  rb_define_method(rb_cLinterCursor, "SQLNativeSql", SQLNativeSql_, 1);
  rb_define_method(rb_cLinterCursor, "SQLPrepare", SQLPrepare_, 1);
  rb_define_method(rb_cLinterCursor, "SQLBindParameter", SQLBindParameter_, 2);
  rb_define_method(rb_cLinterCursor, "SQLExecute", SQLExecute_, -1);
  rb_define_method(rb_cLinterCursor, "GetRowCount", GetRowCount, 0);
  rb_define_method(rb_cLinterCursor, "GetAnswerInfo", GetAnswerInfo, -1);  
  rb_define_method(rb_cLinterCursor, "GetLastRowId", GetLastRowId, 0);
  rb_define_method(rb_cLinterCursor, "GetLastAutoInc", GetLastAutoInc, 0);    
  rb_define_method(rb_cLinterCursor, "GetBlob", GetBlob, -1);
  rb_define_method(rb_cLinterCursor, "AddBlob", AddBlob, -1);
  rb_define_method(rb_cLinterCursor, "DelBlob", DelBlob, -1);  
  rb_define_method(rb_cLinterCursor, "Close", CloseCursor, 0);  
  rb_define_method(rb_cLinterCursor, "GetStatus", GetStatus, 0);
  rb_define_method(rb_cLinterCursor, "SQLTables", SQLTables_, 3);
  rb_define_method(rb_cLinterCursor, "SQLColumns", SQLColumns_, 3);
  rb_define_method(rb_cLinterCursor, "GetProcOutParams", GetProcOutParams, 0);
  rb_define_method(rb_cLinterCursor, "GetBindParamInfo", GetBindParamInfo, -1);
  rb_define_method(rb_cLinterCursor, "SetOption", SetOption, 2);    	

  rb_define_global_const("M_AUTOCOMMIT", LL2NUM(0));
  rb_define_global_const("M_EXCLUSIVE", LL2NUM(M_EXCLUSIVE)); 
  rb_define_global_const("M_OPTIMISTIC", LL2NUM(M_OPTIMISTIC)); 
  rb_define_global_const("M_BINARY", LL2NUM(M_BINARY)); 

#if _VER_MAX >= 600
  rb_define_global_const("Q_USE_UTF8", LL2NUM(Q_USE_UTF8)); 
#endif

#if _VER_MAX >= 610
  rb_define_global_const("M_READ_UNCOMMITTED", LL2NUM(M_READ_UNCOMMITTED)); 
  rb_define_global_const("M_READ_COMMITTED", LL2NUM(M_READ_COMMITTED));
  rb_define_global_const("M_SERIALIZABLE", LL2NUM(M_SERIALIZABLE));
#endif
  
  rb_define_global_const("Q_ENCODE", LL2NUM(Q_ENCODE)); 
  rb_define_global_const("Q_KOI8", LL2NUM(Q_KOI8)); 
  rb_define_global_const("Q_866", LL2NUM(Q_866)); 

  rb_define_global_const("FETCH_FIRST", INT2FIX(FETCH_FIRST));
  rb_define_global_const("FETCH_LAST", INT2FIX(FETCH_LAST));
  rb_define_global_const("FETCH_NEXT", INT2FIX(FETCH_NEXT));
  rb_define_global_const("FETCH_PREV", INT2FIX(FETCH_PREV));
  rb_define_global_const("FETCH_ABSNUM", INT2FIX(FETCH_ABSNUM));

  rb_define_global_const("FETCH_BLOB_MODE", INT2FIX(FETCH_BLOB_MODE));
  rb_define_global_const("CHANNEL_PRIORITY", INT2FIX(CHANNEL_PRIORITY)); 
  rb_define_global_const("DATE_FORMAT", INT2FIX(DATE_FORMAT));  
  rb_define_global_const("SET_SAVE_POINT", INT2FIX(SET_SAVE_POINT));  
  rb_define_global_const("SET_CURSOR_NAME", INT2FIX(SET_CURSOR_NAME));

}

#if (defined(UNIX) || defined(QNX))   /* 13.09.12 ... */
void _fini()
{
  UninitLinterClient();
}
#endif                                /* ... 13.09.12 */
