 /***************************************************************************
 *                 RELEX, Inc. Linter DBMS source code                      *
 *              This is public source code of RELEX, Inc.                   *
 *                                                                          *
 *      You have a royalty-free right to use, modify, reproduce and         *
 *     distribute the Sample Files (and/or any modified version) in         *
 *        any way you find useful, provided that you agree that             *
 *       RELEX has no warranty obligations or liability for any             *
 *           Sample Application Files which are modified.                   *
 *                                                                          *
 *                 Copyright (c) 1998-2005 RELEX Inc.                       *
 *                                                                          *
 *                   This software was developed by                         *
 *                                                                          *
 *                            RELEX, Inc.                                   *
 *                         Voronezh, Russia                                 *
 *                                                                          *
 *                        All rights reserved.                              *
 ****************************************************************************
 AUTHOR:  Kositsin M.V.
 ----------------------------------------------------------------------------
 PURPOSE: Sample CALL-interface command 'COMT'.
 ----------------------------------------------------------------------------
 REVISION HISTORY:
 22.08.01 - Galizin ( For VxWorks )
 25.03.03 -- Sveta (code format)

 ----------------------------------------------------------------------------
*/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "inter.h"
#if defined(VXWORKS)
#include "vxstart.h"
#endif

void PrintErrorAndCloseChannel(TCBL * Cbl);

#if defined(VXWORKS)
MainStart(c_commit, 32 * 1024, UninitLinterClient)
#else
int main()
#endif
{
  char name[] = "SYSTEM/MANAGER";
  char kort[40];
  TCBL Cbl;

  printf("\n*** Sample for CALL-interface command '    ', 'COMT'. ***\n");

  memset(&Cbl, 0, sizeof(Cbl));

  memset(&Cbl.Node, ' ', sizeof(Cbl.Node)); /* Local or default server */

  printf("Open channel...");
  Cbl.PrzExe = M_OPTIMISTIC;    /* Optimistic mode */
  memcpy(Cbl.Command, "OPEN", 4); /* Open channel    */
  inter(&Cbl, name, NULL, NULL, NULL);
  if (Cbl.CodErr)
  {
    PrintErrorAndCloseChannel(&Cbl);
    return 1;
  }
  printf("done\n");

  printf("Query execution:\"DELETE FROM AUTO WHERE MAKE='FORD'\"...");
  memcpy(Cbl.Command, "    ", 4); /* Non SELECT command */
  inter(&Cbl, NULL, "DELETE FROM AUTO WHERE MAKE='FORD';", NULL, NULL);
  if (Cbl.CodErr)
  {
    PrintErrorAndCloseChannel(&Cbl);
    return 1;
  }
  printf("done\n");
  printf("Delete %ld row\n", Cbl.RowCount);

  printf(
         "Query execution:\"INSERT INTO AUTO VALUES ( 'FORD','FORD SCORPIO','COUPE',\n"
         "8,143,302,1000,'BLACK',60,'1234567890',720308,49178,1);\"...");
  memcpy(Cbl.Command, "    ", 4);
  inter(&Cbl, NULL,
        "INSERT INTO AUTO VALUES ( 'FORD','FORD SCORPIO','COUPE',8,143,302,\
         1000,'BLACK',60,'1234567890',720308,49178,1);", NULL, NULL);
  if (Cbl.CodErr)
  {
    PrintErrorAndCloseChannel(&Cbl);
    return 1;
  }
  printf("done\n");
  printf("Insert %ld row\n", Cbl.RowCount);

  printf("Query execution:\"SELECT MAKE FROM AUTO WHERE MAKE='FORD';\"...");
  memcpy(Cbl.Command, "SLCT", 4);
  Cbl.LnBufRow = 40;
  inter(&Cbl, NULL, "SELECT MAKE FROM AUTO WHERE MAKE='FORD';", NULL, kort);
  if (Cbl.CodErr > 2)
  {
    PrintErrorAndCloseChannel(&Cbl);
    return 1;
  }
  printf("done\n");
  printf("Select %ld row\n", Cbl.RowCount);

  printf("Execute \"COMMIT\" command...");
  memcpy(Cbl.Command, "COMT", 4);
  inter(&Cbl, NULL, NULL, NULL, NULL);
  if (Cbl.CodErr)
  {
    PrintErrorAndCloseChannel(&Cbl);
    return 1;
  }
  printf("done\n");

  printf("Query execution:\"SELECT MAKE,MODEL FROM AUTO WHERE MAKE='FORD';\"...");
  memcpy(Cbl.Command, "SLCT", 4); /* Execute 'select' query and get 1-st
                                   * row */
  Cbl.LnBufRow = 40;
  inter(&Cbl, NULL, "SELECT MAKE,MODEL FROM AUTO WHERE MAKE='FORD';", NULL, kort);
  if (Cbl.CodErr > 2)
  {
    PrintErrorAndCloseChannel(&Cbl);
    return 1;
  }
  printf("done\n");
  printf("Select %ld row\n", Cbl.RowCount);
  printf("\nMake                 | Model ");
  printf("\n---------------------+---------------------");
  while (Cbl.CodErr == 0)
  {
    printf("\n%.20s | %.20s", kort, kort + 20);
    Cbl.LnBufRow = 40;
    memcpy(&Cbl.Command, "GETN", 4);
    inter(&Cbl, NULL, NULL, NULL, kort);
  }
  printf("\n---------------------+---------------------\n");


  printf("Query execution:\"UPDATE AUTO SET MODEL='FORD-1' WHERE MAKE='FORD';\"...");
  memcpy(Cbl.Command, "    ", 4);
  inter(&Cbl, NULL, "UPDATE AUTO SET MODEL='FORD-1' WHERE MAKE='FORD';", NULL, NULL);
  if (Cbl.CodErr)
  {
    PrintErrorAndCloseChannel(&Cbl);
    return 1;
  }
  printf("done\n");
  printf("Update %ld row\n", Cbl.RowCount);

  printf("Execute \"COMMIT\" command...");
  memcpy(Cbl.Command, "COMT", 4);
  inter(&Cbl, NULL, NULL, NULL, NULL);
  if (Cbl.CodErr)
  {
    PrintErrorAndCloseChannel(&Cbl);
    return 1;
  }
  printf("done\n");


  printf("Query execution:\"SELECT MAKE,MODEL FROM AUTO WHERE MAKE='FORD';\"...");
  memcpy(Cbl.Command, "SLCT", 4); /* Execute 'select' query and get 1-st
                                   * row */
  Cbl.LnBufRow = 40;
  inter(&Cbl, NULL, "SELECT MAKE,MODEL FROM AUTO WHERE MAKE='FORD';", NULL, kort);
  if (Cbl.CodErr > 2)
  {
    PrintErrorAndCloseChannel(&Cbl);
    return 1;
  }
  printf("done\n");
  printf("Select %ld row\n", Cbl.RowCount);
  printf("\nMake                 | Model ");
  printf("\n---------------------+---------------------");
  while (Cbl.CodErr == 0)
  {
    printf("\n%.20s | %.20s", kort, kort + 20);
    Cbl.LnBufRow = 40;
    memcpy(&Cbl.Command, "GETN", 4);
    inter(&Cbl, NULL, NULL, NULL, kort);
  }
  printf("\n---------------------+---------------------\n");

  printf("Execution \"CLOS\" command...");
  memcpy(Cbl.Command, "CLOS", 4); /* Close channel */
  inter(&Cbl, NULL, NULL, NULL, NULL);
  printf("done\n");

  printf("\nEnd of work.\n");
  return 0;
}

/*  Print error message and end of work  */
void PrintErrorAndCloseChannel(TCBL * pCBL)
{
  printf("\n");
  if (strncmp(pCBL->Node, "        ", MAX_NODE_LEN) != 0 && *pCBL->Node != 0)
    printf("* Linter server <%.8s>\n", pCBL->Node);
  printf("* Channel N %d\n", pCBL->NumChan);
  printf("* Command [%.4s]\n", pCBL->Command);
  printf("* Linter error: %ld\n", pCBL->CodErr);
  if (pCBL->CodErr >= 2000 && pCBL->CodErr < 3000)
    fprintf(stderr, "    Line %hd position %hd\n",
       ((L_WORD *) (&pCBL->SysErr))[0], ((L_WORD *) (&pCBL->SysErr))[1]);
  else if (pCBL->SysErr)
    printf("* System error: %ld\n", pCBL->SysErr);  
  memcpy(pCBL->Command, "CLOS", 4); /* Close channel */
  inter(pCBL, NULL, NULL, NULL, NULL);
}
