 /***************************************************************************
 *                 RELEX, Inc. Linter DBMS source code                      *
 *              This is public source code of RELEX, Inc.                   *
 *                                                                          *
 *      You have a royalty-free right to use, modify, reproduce and         *
 *     distribute the Sample Files (and/or any modified version) in         *
 *        any way you find useful, provided that you agree that             *
 *       RELEX has no warranty obligations or liability for any             *
 *           Sample Application Files which are modified.                   *
 *                                                                          *
 *                 Copyright (c) 1998-2005 RELEX Inc.                       *
 *                                                                          *
 *                   This software was developed by                         *
 *                                                                          *
 *                            RELEX, Inc.                                   *
 *                         Voronezh, Russia                                 *
 *                                                                          *
 *                        All rights reserved.                              *
 ****************************************************************************
 AUTHOR:  Kositsin M.V.
 ----------------------------------------------------------------------------
 REVISION HISTORY:
 22.08.01 - Galizin ( For VxWorks )
 25.03.03 -- Sveta (code format)

 ----------------------------------------------------------------------------
*/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "inter.h"
#if defined(VXWORKS)
#include "vxstart.h"
#endif

void PrintErrorAndCloseChannel(TCBL * Cbl);


#if defined(VXWORKS)
MainStart(c_current, 32 * 1024, UninitLinterClient)
#else
int main()
#endif
{
  char name[] = "SYSTEM/MANAGER";
  char kort[40];
  TCBL Cbl1;
  TCBL Cbl2;

  printf("\n*** Sample for commands '    ','RBAC','SLCT'. ***\n");

  memset(&Cbl1, 0, sizeof(Cbl1));
  memset(&Cbl2, 0, sizeof(Cbl2));
  memset(&Cbl1.Node, ' ', sizeof(Cbl1.Node)); /* Local or default server */
  memset(&Cbl2.Node, ' ', sizeof(Cbl2.Node));

  printf("Open channel 1. Pessimistic mode...");
  Cbl1.PrzExe = M_EXCLUSIVE;    /* Pessimistic mode */
  memcpy(Cbl1.Command, "OPEN", 4);  /* Open channel    */
  inter(&Cbl1, name, NULL, NULL, NULL);
  if (Cbl1.CodErr)
  {
    PrintErrorAndCloseChannel(&Cbl1);
    return 1;
  }
  printf("done\n");

  printf("Open channel 2. Optimistic mode ...");
  Cbl2.PrzExe = M_OPTIMISTIC;   /* Optimistic mode */
  memcpy(Cbl2.Command, "OPEN", 4);  /* Open channel    */
  inter(&Cbl2, name, NULL, NULL, NULL);
  if (Cbl2.CodErr)
  {
    PrintErrorAndCloseChannel(&Cbl2);
    PrintErrorAndCloseChannel(&Cbl1);
    return 1;
  }
  printf("done\n");


  printf("Query execution:\"DELETE FROM AUTO WHERE MAKE='FORD'\" chan. 1 ...");
  memcpy(Cbl1.Command, "    ", 4);  /* Non SELECT command */
  inter(&Cbl1, NULL, "DELETE FROM AUTO WHERE MAKE='FORD';", NULL, NULL);
  if (Cbl1.CodErr)
  {
    PrintErrorAndCloseChannel(&Cbl2);
    PrintErrorAndCloseChannel(&Cbl1);
    return 1;
  }
  printf("done\n");
  printf("Delete %ld row\n", Cbl1.RowCount);

  printf(
         "Query execution:\"INSERT INTO AUTO VALUES ( 'FORD','FORD SCORPIO','COUPE',\n"
         "8,143,302,1000,'BLACK',60,'1234567890',720308,49178,1);\" chan. 1 ...");
  memcpy(Cbl1.Command, "    ", 4);
  inter(&Cbl1, NULL,
        "INSERT INTO AUTO VALUES ( 'FORD','FORD SCORPIO','COUPE',8,143,302,\
         1000,'BLACK',60,'1234567890',720308,49178,1);", NULL, NULL);
  if (Cbl1.CodErr)
  {
    PrintErrorAndCloseChannel(&Cbl2);
    PrintErrorAndCloseChannel(&Cbl1);
    return 1;
  }
  printf("done\n");
  printf("Insert %ld row\n", Cbl1.RowCount);


  printf("Query execution:\"SELECT MAKE,MODEL FROM AUTO WHERE MAKE='FORD';\""
         "chan. 1...");
  memcpy(Cbl1.Command, "SLCT", 4);  /* Execute 'select' query and get 1-st
                                     * row */
  Cbl1.LnBufRow = 40;
  inter(&Cbl1, NULL, "SELECT MAKE,MODEL FROM AUTO WHERE MAKE='FORD';", NULL, kort);
  if (Cbl1.CodErr > 2)
  {
    PrintErrorAndCloseChannel(&Cbl2);
    PrintErrorAndCloseChannel(&Cbl1);
    return 1;
  }
  printf("done\n");
  printf("Select %ld row\n", Cbl1.RowCount);
  printf("Print first row\n");
  printf("\nMake                 | Model ");
  printf("\n---------------------+---------------------");
  if (Cbl1.CodErr == 0)
  {
    printf("\n%.20s | %.20s", kort, kort + 20);
  }
  printf("\n---------------------+---------------------\n");


  printf("Query execution:"
         "\"UPDATE AUTO SET MODEL='SCORPIO-1' WHERE CURRENT OF CURSOR;\" chan. 1...");
  memcpy(Cbl1.Command, "    ", 4);
  inter(&Cbl1, NULL, "UPDATE AUTO SET MODEL='SCORPIO-1' WHERE CURRENT OF CURSOR;",
        NULL, NULL);
  if (Cbl1.CodErr)
  {
    PrintErrorAndCloseChannel(&Cbl2);
    PrintErrorAndCloseChannel(&Cbl1);
    return 1;
  }
  printf("done\n");
  printf("Update %ld row\n", Cbl1.RowCount);

  printf("Query execution:\"SELECT MAKE,MODEL FROM AUTO WHERE MAKE='FORD';\""
         "chan. 2...");
  memcpy(Cbl2.Command, "SLCT", 4);  /* Execute 'select' query and get 1-st
                                     * row */
  Cbl2.LnBufRow = 40;
  inter(&Cbl2, NULL, "SELECT MAKE,MODEL FROM AUTO WHERE MAKE='FORD';", NULL, kort);
  if (Cbl2.CodErr > 2)
  {
    PrintErrorAndCloseChannel(&Cbl2);
    PrintErrorAndCloseChannel(&Cbl1);
    return 1;
  }
  printf("done\n");
  printf("Select %ld row\n", Cbl2.RowCount);
  printf("\nMake                 | Model ");
  printf("\n---------------------+---------------------");
  while (Cbl2.CodErr == 0)
  {
    printf("\n%.20s | %.20s", kort, kort + 20);
    Cbl2.LnBufRow = 40;
    memcpy(&Cbl2.Command, "GETN", 4);
    inter(&Cbl2, NULL, NULL, NULL, kort);
  }
  printf("\n---------------------+---------------------\n");


  printf("Query execution:\"DELETE FROM AUTO WHERE CURRENT OF CURSOR;\""
         " chan. 1...");
  inter(&Cbl1, NULL, "DELETE FROM AUTO WHERE CURRENT OF CURSOR;", NULL, NULL);
  if (Cbl1.CodErr)
  {
    PrintErrorAndCloseChannel(&Cbl2);
    PrintErrorAndCloseChannel(&Cbl1);
    return 1;
  }
  printf("done\n");
  printf("Delete %ld row\n", Cbl1.RowCount);

  printf("Query execution:\"SELECT MAKE,MODEL FROM AUTO WHERE MAKE='FORD';\""
         "chan. 2...");
  memcpy(Cbl2.Command, "SLCT", 4);  /* Execute 'select' query and get 1-st
                                     * row */
  Cbl2.LnBufRow = 40;
  inter(&Cbl2, NULL, "SELECT MAKE,MODEL FROM AUTO WHERE MAKE='FORD';", NULL, kort);
  if (Cbl2.CodErr > 2)
  {
    PrintErrorAndCloseChannel(&Cbl2);
    PrintErrorAndCloseChannel(&Cbl1);
    return 1;
  }
  printf("done\n");
  printf("Select %ld row\n", Cbl2.RowCount);
  printf("\nMake                 | Model ");
  printf("\n---------------------+---------------------");
  while (Cbl2.CodErr == 0)
  {
    printf("\n%.20s | %.20s", kort, kort + 20);
    Cbl2.LnBufRow = 40;
    memcpy(&Cbl2.Command, "GETN", 4);
    inter(&Cbl2, NULL, NULL, NULL, kort);
  }
  printf("\n---------------------+---------------------\n");


  printf("Execution \"ROLLBACK\" command. chan. 1...");
  memcpy(Cbl1.Command, "RBAC", 4);  /* Rollback update & delete */
  inter(&Cbl1, NULL, NULL, NULL, NULL);
  if (Cbl1.CodErr)
  {
    PrintErrorAndCloseChannel(&Cbl2);
    PrintErrorAndCloseChannel(&Cbl1);
    return 1;
  }
  printf("done\n");

  printf("Query execution:\"SELECT MAKE,MODEL FROM AUTO WHERE MAKE='FORD';\""
         "chan. 2...");
  memcpy(Cbl2.Command, "SLCT", 4);  /* Execute 'select' query and get 1-st
                                     * row */
  Cbl2.LnBufRow = 40;
  inter(&Cbl2, NULL, "SELECT MAKE,MODEL FROM AUTO WHERE MAKE='FORD';", NULL, kort);
  if (Cbl2.CodErr > 2)
  {
    PrintErrorAndCloseChannel(&Cbl2);
    PrintErrorAndCloseChannel(&Cbl1);
    return 1;
  }
  printf("done\n");
  printf("Select %ld row\n", Cbl2.RowCount);
  printf("\nMake                 | Model ");
  printf("\n---------------------+---------------------");
  while (Cbl2.CodErr == 0)
  {
    printf("\n%.20s | %.20s", kort, kort + 20);
    Cbl2.LnBufRow = 40;
    memcpy(&Cbl2.Command, "GETN", 4);
    inter(&Cbl2, NULL, NULL, NULL, kort);
  }
  printf("\n---------------------+---------------------\n");


  memcpy(Cbl1.Command, "CLOS", 4);  /* Close channel */
  inter(&Cbl1, NULL, NULL, NULL, NULL);
  memcpy(Cbl2.Command, "CLOS", 4);
  inter(&Cbl2, NULL, NULL, NULL, NULL);

  printf("\nEnd of work.\n");
  return 0;
}


/*  Print error message and end of work  */
void PrintErrorAndCloseChannel(TCBL * pCBL)
{
  printf("\n");
  if (strncmp(pCBL->Node, "        ", MAX_NODE_LEN) != 0 && *pCBL->Node != 0)
    printf("* Linter server <%.8s>\n", pCBL->Node);
  printf("* Channel N %d\n", pCBL->NumChan);
  printf("* Command [%.4s]\n", pCBL->Command);
  printf("* Linter error: %ld\n", pCBL->CodErr);
  if (pCBL->CodErr >= 2000 && pCBL->CodErr < 3000)
    fprintf(stderr, "    Line %hd position %hd\n",
       ((L_WORD *) (&pCBL->SysErr))[0], ((L_WORD *) (&pCBL->SysErr))[1]);
  else if (pCBL->SysErr)
    printf("* System error: %ld\n", pCBL->SysErr);
  memcpy(pCBL->Command, "CLOS", 4); /* Close channel */
  inter(pCBL, NULL, NULL, NULL, NULL);
}
