 /***************************************************************************
 *                 RELEX, Inc. Linter DBMS source code                      *
 *              This is public source code of RELEX, Inc.                   *
 *                                                                          *
 *      You have a royalty-free right to use, modify, reproduce and         *
 *     distribute the Sample Files (and/or any modified version) in         *
 *        any way you find useful, provided that you agree that             *
 *       RELEX has no warranty obligations or liability for any             *
 *           Sample Application Files which are modified.                   *
 *                                                                          *
 *                 Copyright (c) 1998-2005 RELEX Inc.                       *
 *                                                                          *
 *                   This software was developed by                         *
 *                                                                          *
 *                            RELEX, Inc.                                   *
 *                         Voronezh, Russia                                 *
 *                                                                          *
 *                        All rights reserved.                              *
 ****************************************************************************
 AUTHOR:  Kositsin M.V.
 ----------------------------------------------------------------------------
 PURPOSE: Sample for 'GETA' command
 ----------------------------------------------------------------------------
 REVISION HISTORY:
 22.08.01 - Galizin ( For VxWorks )
 25.03.03 -- Sveta (code format)

 ----------------------------------------------------------------------------
*/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "inter.h"
#if defined(VXWORKS)
#include "vxstart.h"
#endif

void PrintErrorAndCloseChannel(TCBL * Cbl);

#if defined(VXWORKS)
MainStart(c_getans, 32 * 1024, UninitLinterClient)
#else
int main()
#endif
{
  char name[] = "SYSTEM/MANAGER";
  char kort[4096];
  TCBL Cbl;
  GETA_OUT Geta;
  int j = 0;

  printf("\n*** Sample CALL-interface command 'GETA'. ***\n");

  memset(&Cbl, 0, sizeof(TCBL));

  printf("Open channel...");
  Cbl.PrzExe &= ~(M_OPTIMISTIC | M_SHARE | M_EXCLUSIVE);  /* Autocommit */
  memcpy(Cbl.Command, "OPEN", 4); /* Open channel */
  inter(&Cbl, name, NULL, NULL, NULL);
  if (Cbl.CodErr)
  {
    PrintErrorAndCloseChannel(&Cbl);
    return 1;
  }
  printf("done\n");

  printf("Query execution:\"SELECT * FROM AUTO;\"...");
  memcpy(Cbl.Command, "SLCT", 4);
  Cbl.LnBufRow = 4096;
  inter(&Cbl, NULL, "SELECT * FROM AUTO;", NULL, kort);
  if (Cbl.CodErr > 2)
  {
    PrintErrorAndCloseChannel(&Cbl);
    return 1;
  }
  printf("done\n");
  printf("%ld row into answer\n", Cbl.RowCount);
  printf("length of row %ld\n", Cbl.LnBufRow);

  printf("Get and print information about column of answer:\n");
  memcpy(&Cbl.Command, "GETA", 4);
  do
  {
    int i;
    int countGetA;

    Cbl.LnBufRow = 4096;
    Cbl.RowId = j;
    inter(&Cbl, NULL, NULL, NULL, kort);
    if (Cbl.CodErr)
    {
      PrintErrorAndCloseChannel(&Cbl);
      return 1;
    }
    countGetA = Cbl.LnBufRow / sizeof(GETA_OUT);
    for (i = 0; i < countGetA; i++)
    {
      memcpy(&Geta, kort + i * sizeof(Geta), sizeof(Geta));
      printf("\n Information about column number %d of answer:\n", i + j + 1);
#if _VER_MAX < 590
      printf("User name   : %.18s\n", Geta.User);
      printf("Table name  : %.18s\n", Geta.Table);
      printf("Column name : %.18s\n", Geta.Column);
#else
      printf("User name   : %.66s\n", Geta.User);
      printf("Table name  : %.66s\n", Geta.Table);
      printf("Column name : %.66s\n", Geta.Column);
#endif
      printf("Length of column: %d\n", Geta.Length);
    }
    j = j + countGetA;
  }
  while (j < Cbl.RowCount);
  printf("done\n");

  memcpy(&Cbl.Command, "CLOS", 4);
  inter(&Cbl, NULL, NULL, NULL, NULL);

  printf("\n\nEnd of work.\n");
  return 0;
}

/*  Print error message and end of work  */
void PrintErrorAndCloseChannel(TCBL * pCBL)
{
  printf("\n");
  if (strncmp(pCBL->Node, "        ", MAX_NODE_LEN) != 0 && *pCBL->Node != 0)
    printf("* Linter server <%.8s>\n", pCBL->Node);
  printf("* Channel N %d\n", pCBL->NumChan);
  printf("* Command [%.4s]\n", pCBL->Command);
  printf("* Linter error: %ld\n", pCBL->CodErr);
  if (pCBL->CodErr >= 2000 && pCBL->CodErr < 3000)
    fprintf(stderr, "    Line %hd position %hd\n",
       ((L_WORD *) (&pCBL->SysErr))[0], ((L_WORD *) (&pCBL->SysErr))[1]);
  else if (pCBL->SysErr)
    printf("* System error: %ld\n", pCBL->SysErr);
  memcpy(pCBL->Command, "CLOS", 4); /* Close channel */
  inter(pCBL, NULL, NULL, NULL, NULL);
}
