 /***************************************************************************
 *                 RELEX, Inc. Linter DBMS source code                      *
 *              This is public source code of RELEX, Inc.                   *
 *                                                                          *
 *      You have a royalty-free right to use, modify, reproduce and         *
 *     distribute the Sample Files (and/or any modified version) in         *
 *        any way you find useful, provided that you agree that             *
 *       RELEX has no warranty obligations or liability for any             *
 *           Sample Application Files which are modified.                   *
 *                                                                          *
 *                 Copyright (c) 1998-2006 RELEX Inc.                       *
 *                                                                          *
 *                   This software was developed by                         *
 *                                                                          *
 *                            RELEX, Inc.                                   *
 *                         Voronezh, Russia                                 *
 *                                                                          *
 *                        All rights reserved.                              *
 ****************************************************************************
 AUTHOR:  Kositsin M.V.
 ----------------------------------------------------------------------------
 PURPOSE: Sample for CALL-interface command 'LREL'.
 ----------------------------------------------------------------------------
 REVISION HISTORY:
 22.08.01 - Galizin ( For VxWorks )
 25.03.03 -- Sveta (code format)
 22.11.06 -- Kolesov (code corrected)

 ----------------------------------------------------------------------------
*/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "inter.h"
#if defined(VXWORKS)
#include "vxstart.h"
#endif

typedef FREL_IN T_Tbl_Query;
void PrintErrorAndCloseChannel(TCBL * pCBL);

#if defined(VXWORKS)
MainStart(c_lockrel, 32 * 1024, UninitLinterClient)
#else
int main()
#endif
{
  char name[] = "SYSTEM/MANAGER";
  char kort[40];
  TCBL Cbl1;
  TCBL Cbl2;
  T_Tbl_Query t1, t2;

  printf("\n*** Example for 'LREL' command. ***\n");

  memset(&Cbl1, 0, sizeof(TCBL));
  memset(&Cbl2, 0, sizeof(TCBL));

  printf("Open channel N1 ... ");
  Cbl1.PrzExe = M_OPTIMISTIC;
  memcpy(Cbl1.Command, "OPEN", 4);
  inter(&Cbl1, name, NULL, NULL, NULL);
  if (Cbl1.CodErr)
  {
    PrintErrorAndCloseChannel(&Cbl1);
    return 1;
  }
  printf("done\n");

  printf("Channel N1.Lock table \"AUTO\". ... ");
  t1.Owner = 1;
  memset(t1.TblName, ' ', MAX_ID_LEN);
  strncpy(t1.TblName, "AUTO", 4);
  memcpy(Cbl1.Command, "LREL", 4);  /* Lock table */
  inter(&Cbl1, (char *) &t1, NULL, NULL, NULL);
  if (Cbl1.CodErr)
  {
    PrintErrorAndCloseChannel(&Cbl1);
    return 1;
  }
  printf("done\n");

  printf("Open channel N2 ...");
  Cbl2.PrzExe = M_OPTIMISTIC;   /* Optimistic mode  */
  memcpy(Cbl2.Command, "OPEN", 4);  /* Open channel     */
  inter(&Cbl2, name, NULL, NULL, NULL);
  if (Cbl2.CodErr)
  {
    PrintErrorAndCloseChannel(&Cbl2);
    PrintErrorAndCloseChannel(&Cbl1);
    return 1;
  }
  printf("done\n");

  printf("Channel N2.Lock table \"PERSON\" ... ");
  t2.Owner = 1;
  memset(t2.TblName, ' ', MAX_ID_LEN);
  strncpy(t2.TblName, "PERSON", 6);
  memcpy(Cbl2.Command, "LREL", 4);  /* Lock table */
  inter(&Cbl2, (char *) &t2, NULL, NULL, NULL);
  if (Cbl2.CodErr)
  {
    PrintErrorAndCloseChannel(&Cbl2);
    PrintErrorAndCloseChannel(&Cbl1);
    return 1;
  }
  printf("done\n");


  printf("Channel N1. Query execution:\n"
         "\"SELECT MAKE,MODEL FROM AUTO WHERE MAKE='BMW';\"...");
  memcpy(Cbl1.Command, "SLCT", 4);
  Cbl1.LnBufRow = 40;
  inter(&Cbl1, NULL, "SELECT MAKE,MODEL FROM AUTO WHERE MAKE='BMW';", NULL, kort);
  if (Cbl1.CodErr > 2)
  {
    PrintErrorAndCloseChannel(&Cbl1);
    PrintErrorAndCloseChannel(&Cbl2);
    return 1;
  }
  printf("done\n");
  printf("Select %ld row\n", Cbl1.RowCount);
  printf("\nMake                 | Model ");
  printf("\n---------------------+---------------------");
  while (Cbl1.CodErr == 0)
  {
    printf("\n%.20s | %.20s", kort, kort + 20);
    Cbl1.LnBufRow = 40;
    memcpy(&Cbl1.Command, "GETN", 4);
    inter(&Cbl1, NULL, NULL, NULL, kort);
  }
  printf("\n---------------------+---------------------\n");


  printf("Channel N2. Query execution:\n"
         "\"SELECT NAME FROM PERSON WHERE PERSONID < 5;\"...");
  memcpy(Cbl2.Command, "SLCT", 4);
  Cbl2.LnBufRow = 40;
  inter(&Cbl2, NULL, "SELECT NAME FROM PERSON WHERE PERSONID < 5;", NULL, kort);
  if (Cbl2.CodErr > 2)
  {
    PrintErrorAndCloseChannel(&Cbl2);
    PrintErrorAndCloseChannel(&Cbl1);
    return 1;
  }
  printf("done\n");
  printf("Select %ld row\n", Cbl2.RowCount);
  printf("\nName");
  printf("\n--------------------");
  while (Cbl2.CodErr == 0)
  {
    printf("\n%.20s", kort);
    Cbl2.LnBufRow = 40;
    memcpy(&Cbl2.Command, "GETN", 4);
    inter(&Cbl2, NULL, NULL, NULL, kort);
  }
  printf("\n--------------------\n");

  printf("Channel N1. Unlock table ... ");
  memcpy(Cbl1.Command, "UREL", 4);
  inter(&Cbl1, NULL, NULL, NULL, NULL);
  if (Cbl1.CodErr != 0)
  {
    PrintErrorAndCloseChannel(&Cbl1);
    PrintErrorAndCloseChannel(&Cbl2);
    return 1;
  }
  printf("done\n");

  printf("Channel N2. Unlock table ... ");
  memcpy(Cbl2.Command, "UREL", 4);
  inter(&Cbl2, NULL, NULL, NULL, NULL);
  if (Cbl2.CodErr != 0)
  {
    PrintErrorAndCloseChannel(&Cbl2);
    PrintErrorAndCloseChannel(&Cbl1);
    return 1;
  }
  printf("done\n");

  printf("Close channel N1 ... ");
  memcpy(&Cbl1.Command, "CLOS", 4);
  inter(&Cbl1, NULL, NULL, NULL, NULL);
  printf("done\n");

  printf("Close channel N2 ... ");
  memcpy(&Cbl2.Command, "CLOS", 4);
  inter(&Cbl2, NULL, NULL, NULL, NULL);
  printf("done\n");

  printf("\nEnd of work.\n");
  return 0;                                  /* 22.11.06 */
}

/*  Print error message and end of work  */
void PrintErrorAndCloseChannel(TCBL * pCBL)
{
  printf("\n");
  if (strncmp(pCBL->Node, "        ", MAX_NODE_LEN) != 0 && *pCBL->Node != 0)
    printf("* Linter server <%.8s>\n", pCBL->Node);
  printf("* Channel N %d\n", pCBL->NumChan);
  printf("* Command [%.4s]\n", pCBL->Command);
  printf("* Linter error: %ld\n", pCBL->CodErr);
  if (pCBL->CodErr >= 2000 && pCBL->CodErr < 3000)
    fprintf(stderr, "    Line %hd position %hd\n",
       ((L_WORD *) (&pCBL->SysErr))[0], ((L_WORD *) (&pCBL->SysErr))[1]);
  else if (pCBL->SysErr)
    printf("* System error: %ld\n", pCBL->SysErr);
  memcpy(pCBL->Command, "CLOS", 4); /* Close channel */
  inter(pCBL, NULL, NULL, NULL, NULL);
}
