 /***************************************************************************
 *                 RELEX, Inc. Linter DBMS source code                      *
 *              This is public source code of RELEX, Inc.                   *
 *                                                                          *
 *      You have a royalty-free right to use, modify, reproduce and         *
 *     distribute the Sample Files (and/or any modified version) in         *
 *        any way you find useful, provided that you agree that             *
 *       RELEX has no warranty obligations or liability for any             *
 *           Sample Application Files which are modified.                   *
 *                                                                          *
 *                 Copyright (c) 1998-2005 RELEX Inc.                       *
 *                                                                          *
 *                   This software was developed by                         *
 *                                                                          *
 *                            RELEX, Inc.                                   *
 *                         Voronezh, Russia                                 *
 *                                                                          *
 *                        All rights reserved.                              *
 ****************************************************************************
 AUTHOR:  Kositsin M.V.
 ----------------------------------------------------------------------------
 PURPOSE: Sample for CALL-interface command 'SLCT'
 ----------------------------------------------------------------------------
 REVISION HISTORY:
 22.08.01 - Galizin ( For VxWorks )
 25.03.03 -- Sveta (code format)

 ----------------------------------------------------------------------------
*/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "inter.h"
#if defined(VXWORKS)
#include "vxstart.h"
#endif

static void PrintErrorAndCloseChannel(TCBL * pCBL);

#if defined(VXWORKS)
MainStart(c_select, 1024 * 32, UninitLinterClient)
#else
int main()
#endif
{
  char name[] = "SYSTEM/MANAGER";
  char kort[40];
  TCBL Cbl;

  printf("\n*** Sample for 'SLCT'. ***\n");

  memset(&Cbl, 0, sizeof(TCBL));

  printf("Open channel ... ");
  Cbl.PrzExe &= ~(M_OPTIMISTIC | M_SHARE | M_EXCLUSIVE);  /* Autocommit */
  memcpy(Cbl.Command, "OPEN", 4);
  inter(&Cbl, name, NULL, NULL, NULL);
  if (Cbl.CodErr)
  {
    PrintErrorAndCloseChannel(&Cbl);
    return 1;
  }
  printf("done\n");

  printf("Query execution:\n"
  "\"SELECT MAKE,MODEL FROM AUTO WHERE ROWID>100 AND ROWID<108;\" ... ");
  memcpy(Cbl.Command, "SLCT", 4);
  Cbl.LnBufRow = 40;
  inter(&Cbl, NULL, "SELECT MAKE,MODEL FROM AUTO WHERE ROWID>100 AND ROWID<108;",
        NULL, kort);
  if (Cbl.CodErr > 2)
  {
    PrintErrorAndCloseChannel(&Cbl);
    return 1;
  }
  printf("done\n");
  printf("Select %ld rows\n\n", Cbl.RowCount);

  printf("Get rows of answer ... \n");
  printf("Make                 | Model\n");
  printf("---------------------+---------------------\n");
  while (!Cbl.CodErr)
  {
    printf("%.20s | %.20s\n", kort, kort + 20);
    memcpy(Cbl.Command, "GETN", 4);
    inter(&Cbl, NULL, NULL, NULL, kort);
  }                             /* while */
  printf("---------------------+---------------------\n");
  printf("done get rows\n\n");

  printf("Get first row of answer ...\n");
  memcpy(Cbl.Command, "GETF", 4);
  inter(&Cbl, NULL, NULL, NULL, kort);
  if (Cbl.CodErr > 2)
  {
    PrintErrorAndCloseChannel(&Cbl);
    return 1;
  }
  if (!Cbl.CodErr)
  {
    printf("Make                 | Model\n");
    printf("---------------------+---------------------\n");
    printf("%.20s | %.20s\n", kort, kort + 20);
    printf("---------------------+---------------------\n");
  }
  printf("done get first row\n\n");

  printf("Get last row of answer ...\n");
  memcpy(Cbl.Command, "GETL", 4);
  inter(&Cbl, NULL, NULL, NULL, kort);
  if (Cbl.CodErr > 2)
  {
    PrintErrorAndCloseChannel(&Cbl);
    return 1;
  }
  if (!Cbl.CodErr)
  {
    printf("Make                 | Model\n");
    printf("---------------------+---------------------\n");
    printf("%.20s | %.20s\n", kort, kort + 20);
    printf("---------------------+---------------------\n");
  }
  printf("done get last row\n\n");

  printf("Get previous row ...\n");
  memcpy(Cbl.Command, "GETP", 4);
  inter(&Cbl, NULL, NULL, NULL, kort);
  if (Cbl.CodErr > 2)
  {
    PrintErrorAndCloseChannel(&Cbl);
    return 1;
  }
  if (!Cbl.CodErr)
  {
    printf("Make                 | Model\n");
    printf("---------------------+---------------------\n");
    printf("%.20s | %.20s\n", kort, kort + 20);
    printf("---------------------+---------------------\n");
  }
  printf("done get previous row\n\n");

  printf("Get 4-th row ...\n");
  memcpy(Cbl.Command, "GETS", 4);
  Cbl.RowId = 4;
  inter(&Cbl, NULL, NULL, NULL, kort);
  if (Cbl.CodErr > 2)
  {
    PrintErrorAndCloseChannel(&Cbl);
    return 1;
  }
  if (!Cbl.CodErr)
  {
    printf("Make                 | Model\n");
    printf("---------------------+---------------------\n");
    printf("%.20s | %.20s\n", kort, kort + 20);
    printf("---------------------+---------------------\n");
  }
  printf("done get 4-th row\n\n");

  printf("Close channel...");
  memcpy(Cbl.Command, "CLOS", 4);
  inter(&Cbl, NULL, NULL, NULL, NULL);
  printf("done\n");

  printf("\nEnd of work.\n");
  return 0;
}

void PrintErrorAndCloseChannel(TCBL * pCBL)
{
  printf("\n");
  if (strncmp(pCBL->Node, "        ", MAX_NODE_LEN) != 0 && *pCBL->Node != 0)
    printf("* Linter server <%.8s>\n", pCBL->Node);
  printf("* Channel N %d\n", pCBL->NumChan);
  printf("* Command [%.4s]\n", pCBL->Command);
  printf("* Linter error: %ld\n", pCBL->CodErr);
  if (pCBL->CodErr >= 2000 && pCBL->CodErr < 3000)
    fprintf(stderr, "    Line %hd position %hd\n",
       ((L_WORD *) (&pCBL->SysErr))[0], ((L_WORD *) (&pCBL->SysErr))[1]);
  else if (pCBL->SysErr)
    printf("* System error: %ld\n", pCBL->SysErr);
  memcpy(pCBL->Command, "CLOS", 4); /* Close channel */
  inter(pCBL, NULL, NULL, NULL, NULL);
}
