
 /***************************************************************************
 *                 RELEX, Inc. Linter DBMS source code                      *
 *              This is public source code of RELEX, Inc.                   *
 *                                                                          *
 *      You have a royalty-free right to use, modify, reproduce and         *
 *     distribute the Sample Files (and/or any modified version) in         *
 *        any way you find useful, provided that you agree that             *
 *       RELEX has no warranty obligations or liability for any             *
 *           Sample Application Files which are modified.                   *
 *                                                                          *
 *                    Copyright (c) 1998-2011 RELEX Inc.                    *
 *                                                                          *
 *                   This software was developed by                         *
 *                                                                          *
 *                            RELEX, Inc.                                   *
 *                         Voronezh, Russia                                 *
 *                                                                          *
 *                        All rights reserved.                              *
 ****************************************************************************
 AUTHOR:
 ----------------------------------------------------------------------------
 PURPOSE: Demonstration Call-interface's commands.
 ----------------------------------------------------------------------------
 REVISION HISTORY:
  11.08.98 --          , Creation date
  08.12.98 -- Kononyhin (NAME/PASS in SHUT for VER_MAX >= 500)
  16.12.99 -- Kononyhin (set crName before SETO)
  23.06.04 -- Ermakov   (LnBufRow cleared)
  15.12.11 -- Parkhomenko (check memory corruption)
*/

#include <string.h>
#include <stdlib.h>
#include "inter.h"

/******************** CONNECTION COMMAND *********************/
L_LONG LinterOPEN(TCBL * pCBL, L_CHAR * Name_Pass, L_CHAR * Node, L_WORD Prior, L_LONG PrzExe)
  {
  memcpy(pCBL->Command, "OPEN", 4);
  if (strlen(Node) > MAX_NODE_LEN)
    {
    return SQLLONGID;
    }
  memset(pCBL->Node, 0, MAX_NODE_LEN);          /* 15.12.11 */
  memcpy(pCBL->Node, Node, strlen(Node));
  pCBL->PrzExe = PrzExe;
  pCBL->Prior = Prior;
  pCBL->PrzExe &= ~Q_ASYNC;
  inter(pCBL, Name_Pass, NULL, NULL, NULL);
  return pCBL->CodErr;
  }

L_LONG LinterOCUR(TCBL * pCBL, L_WORD NumChan, L_WORD Prior, L_LONG PrzExe)
  {
  memset(pCBL, 0, sizeof(TCBL));
  memcpy(pCBL->Command, "OCUR", 4);
  pCBL->NumChan = NumChan;
  pCBL->PrzExe = PrzExe;
  pCBL->Prior = Prior;
  pCBL->PrzExe &= ~Q_ASYNC;
  inter(pCBL, NULL, NULL, NULL, NULL);
  return pCBL->CodErr;
  }

L_LONG LinterSETO(TCBL * pCBL, L_CHAR * NameCur)
  {
  pCBL->RowId = crName;                                     /* 16.12.99 */
  memcpy(pCBL->Command, "SETO", 4);
  if ((pCBL->LnBufRow = strlen(NameCur)) > MAX_ID_LEN)
    {
    return SQLLONGID;
    }
  pCBL->PrzExe &= ~Q_ASYNC;
  inter(pCBL, NULL, NULL, NULL, NameCur);
  return pCBL->CodErr;
  }

L_LONG LinterCLOS(TCBL * pCBL)
  {
  memcpy(pCBL->Command, "CLOS", 4);
  pCBL->PrzExe &= ~Q_ASYNC;
  inter(pCBL, NULL, NULL, NULL, NULL);
  return pCBL->CodErr;

  }

L_LONG LinterKILL(TCBL * pCBL, L_WORD NumChan)
  {
  memcpy(pCBL->Command, "KILL", 4);
  pCBL->RowId = NumChan;
  pCBL->PrzExe &= ~Q_ASYNC;
  inter(pCBL, NULL, NULL, NULL, NULL);
  return pCBL->CodErr;
  }

L_LONG LinterSNAP(L_CHAR * Node)
  {
  TCBL    CBL;

  memset(&CBL, 0, sizeof(TCBL));
  memcpy(CBL.Command, "SNAP", 4);
  if (strlen(Node) > MAX_NODE_LEN)
    {
    return SQLLONGID;
    }
  memset(CBL.Node, 0, MAX_NODE_LEN);          /* 15.12.11 */
  memcpy(CBL.Node, Node, strlen(Node));
  CBL.PrzExe &= ~Q_ASYNC;
  inter(&CBL, NULL, NULL, NULL, NULL);
  return CBL.CodErr;
  }

L_LONG LinterSHUT(TCBL * pCBL, L_CHAR * Name_Pass, L_CHAR * Node)
  {
  memcpy(pCBL->Command, "SHUT", 4);
  if (strlen(Node) > MAX_NODE_LEN)
    {
    return SQLLONGID;
    }
  memset(pCBL->Node, 0, MAX_NODE_LEN);          /* 15.12.11 */
  memcpy(pCBL->Node, Node, strlen(Node));
  pCBL->PrzExe &= ~Q_ASYNC;
  inter(pCBL, Name_Pass, NULL, NULL, NULL);
  return pCBL->CodErr;
  }

void    LinterASYNC_OPEN(TCBL * pCBL, L_CHAR * Name_Pass, L_CHAR * Node, L_WORD Prior, L_LONG PrzExe, USR_PROC func)
  {
  memcpy(pCBL->Command, "OPEN", 4);
  if (strlen(Node) > MAX_NODE_LEN)
    {
    pCBL->CodErr = SQLLONGID;
    func(pCBL, NULL, NULL);
    return;                                     /* 15.12.11 */
    }
  memset(pCBL->Node, 0, MAX_NODE_LEN);          /* 15.12.11 */
  memcpy(pCBL->Node, Node, strlen(Node));
  pCBL->PrzExe = PrzExe | Q_ASYNC;
  pCBL->Prior = Prior;
  inter(pCBL, Name_Pass, NULL, (void *) func, NULL);
  }

void    LinterASYNC_OCUR(TCBL * pCBL, L_WORD NumChan, L_WORD Prior, L_LONG PrzExe, USR_PROC func)
  {
  memset(pCBL, 0, sizeof(TCBL));
  memcpy(pCBL->Command, "OCUR", 4);
  pCBL->NumChan = NumChan;
  pCBL->PrzExe = PrzExe | Q_ASYNC;
  pCBL->Prior = Prior;
  inter(pCBL, NULL, NULL, (void *) func, NULL);
  }

void    LinterASYNC_SETO(TCBL * pCBL, L_CHAR * NameCur, USR_PROC func)
  {
  memcpy(pCBL->Command, "SETO", 4);
  if ((pCBL->LnBufRow = strlen(NameCur)) > MAX_ID_LEN)
    {
    pCBL->CodErr = SQLLONGID;
    func(pCBL, NULL, NULL);
    return;                                     /* 15.12.11 */
    }
  pCBL->PrzExe |= Q_ASYNC;
  inter(pCBL, NULL, NULL, (void *) func, NameCur);
  }

void    LinterASYNC_CLOS(TCBL * pCBL, USR_PROC func)
  {
  memcpy(pCBL->Command, "CLOS", 4);
  pCBL->PrzExe |= Q_ASYNC;
  inter(pCBL, NULL, NULL, (void *) func, NULL);

  }

void    LinterASYNC_KILL(TCBL * pCBL, L_WORD NumChan, USR_PROC func)
  {
  memcpy(pCBL->Command, "KILL", 4);
  pCBL->RowId = NumChan;
  pCBL->PrzExe |= Q_ASYNC;
  inter(pCBL, NULL, NULL, (void *) func, NULL);
  }

void    LinterASYNC_SNAP(TCBL * pCBL, L_CHAR * Node, USR_PROC func)
  {
  memcpy(pCBL->Command, "SNAP", 4);
  if (strlen(Node) > MAX_NODE_LEN)
    {
    pCBL->CodErr = SQLLONGID;
    func(pCBL, NULL, NULL);
    return;                                     /* 15.12.11 */
    }
  memset(pCBL->Node, 0, MAX_NODE_LEN);          /* 15.12.11 */
  memcpy(pCBL->Node, Node, strlen(Node));
  pCBL->PrzExe |= Q_ASYNC;
  inter(pCBL, NULL, NULL, (void *) func, NULL);
  }

void    LinterASYNC_SHUT(TCBL * pCBL, L_CHAR * Name_Pass, L_CHAR * Node, USR_PROC func)
  {
  memcpy(pCBL->Command, "SHUT", 4);
  if (strlen(Node) > MAX_NODE_LEN)
    {
    pCBL->CodErr = SQLLONGID;
    func(pCBL, NULL, NULL);
    return;                                     /* 15.12.11 */
    }
  memset(pCBL->Node, 0, MAX_NODE_LEN);          /* 15.12.11 */
  memcpy(pCBL->Node, Node, strlen(Node));
  pCBL->PrzExe |= Q_ASYNC;
  inter(pCBL, Name_Pass, NULL, (void *) func, NULL);
  }

/******************** SYSTEM DICTIONARY COMMAND *********************/
L_LONG LinterFUSR(TCBL * pCBL, L_CHAR * User, L_LONG * pUser_ID)
  {
  memcpy(pCBL->Command, "FUSR", 4);
  pCBL->LnBufRow = sizeof(L_LONG);
  pCBL->PrzExe &= ~Q_ASYNC;
  inter(pCBL, User, NULL, NULL, pUser_ID);
  return pCBL->CodErr;
  }

L_LONG LinterFREL(TCBL * pCBL, FREL_IN * pIn, FREL_OUT * pOut)
  {
  memcpy(pCBL->Command, "FREL", 4);
  pCBL->LnBufRow = sizeof(FREL_OUT);
  pCBL->PrzExe &= ~Q_ASYNC;
  inter(pCBL, pIn, NULL, NULL, pOut);
  return pCBL->CodErr;
  }

L_LONG LinterFATR(TCBL * pCBL, FATR_IN * pIn, FATR_OUT * pOut)
  {
  memcpy(pCBL->Command, "FATR", 4);
  pCBL->LnBufRow = sizeof(FATR_OUT);
  pCBL->PrzExe &= ~Q_ASYNC;
  inter(pCBL, pIn, NULL, NULL, pOut);
  return pCBL->CodErr;
  }

L_LONG LinterDESC(TCBL * pCBL, DB_DESC * pOut)
  {
  memcpy(pCBL->Command, "DESC", 4);
  pCBL->LnBufRow = sizeof(DB_DESC);
  pCBL->PrzExe &= ~Q_ASYNC;
  inter(pCBL, NULL, NULL, NULL, pOut);
  return pCBL->CodErr;
  }

L_LONG LinterFCUR(TCBL * pCBL, L_CHAR * NameCur, FCUR_OUT * pOut)
  {
  memcpy(pCBL->Command, "FCUR", 4);
  pCBL->LnBufRow = sizeof(FCUR_OUT);
  pCBL->PrzExe &= ~Q_ASYNC;
  inter(pCBL, NameCur, NULL, NULL, pOut);
  return pCBL->CodErr;
  }


L_LONG LinterFNOD(TCBL * pCBL, L_CHAR * Node, L_LONG * Out)
  {
  memcpy(pCBL->Command, "FNOD", 4);
  pCBL->LnBufRow = sizeof(L_LONG);
  pCBL->PrzExe &= ~Q_ASYNC;
  inter(pCBL, Node, NULL, NULL, Out);
  return pCBL->CodErr;
  }

void    LinterASYNC_FUSR(TCBL * pCBL, L_CHAR * User, L_LONG * pUser_ID, USR_PROC func)
  {
  memcpy(pCBL->Command, "FUSR", 4);
  pCBL->LnBufRow = sizeof(L_LONG);
  pCBL->PrzExe |= Q_ASYNC;
  inter(pCBL, User, NULL, (void *) func, pUser_ID);
  }


void    LinterASYNC_FREL(TCBL * pCBL, FREL_IN * pIn, FREL_OUT * pOut, USR_PROC func)
  {
  memcpy(pCBL->Command, "FREL", 4);
  pCBL->LnBufRow = sizeof(FREL_OUT);
  pCBL->PrzExe |= Q_ASYNC;
  inter(pCBL, pIn, NULL, (void *) func, pOut);
  }

void    LinterASYNC_FATR(TCBL * pCBL, FATR_IN * pIn, FATR_OUT * pOut, USR_PROC func)
  {
  memcpy(pCBL->Command, "FATR", 4);
  pCBL->LnBufRow = sizeof(FATR_OUT);
  pCBL->PrzExe |= Q_ASYNC;
  inter(pCBL, pIn, NULL, (void *) func, pOut);
  }

void    LinterASYNC_DESC(TCBL * pCBL, DB_DESC * pOut, USR_PROC func)
  {
  memcpy(pCBL->Command, "DESC", 4);
  pCBL->LnBufRow = sizeof(DB_DESC);
  pCBL->PrzExe |= Q_ASYNC;
  inter(pCBL, NULL, NULL, (void *) func, pOut);
  }


void    LinterASYNC_FCUR(TCBL * pCBL, L_CHAR * NameCur, FCUR_OUT * pOut, USR_PROC func)
  {
  memcpy(pCBL->Command, "FCUR", 4);
  pCBL->LnBufRow = sizeof(FCUR_OUT);
  pCBL->PrzExe |= Q_ASYNC;
  inter(pCBL, NameCur, NULL, (void *) func, pOut);
  }

void    LinterASYNC_FNOD(TCBL * pCBL, L_CHAR * Node, L_LONG * Out, USR_PROC func)
  {
  memcpy(pCBL->Command, "FNOD", 4);
  pCBL->LnBufRow = sizeof(L_LONG);
  pCBL->PrzExe |= Q_ASYNC;
  inter(pCBL, Node, NULL, (void *) func, Out);
  }

/******************** SELECT COMMAND *********************/
L_LONG LinterSLCT(TCBL * pCBL, L_LONG PrzExe, L_CHAR * Statement, void *RowBuf, L_WORD RowBufLen, void *NullBuf)
  {
  memcpy(pCBL->Command, "SLCT", 4);
  pCBL->PrzExe = PrzExe;
  pCBL->LnBufRow = RowBufLen;
  pCBL->PrzExe &= ~Q_ASYNC;
  inter(pCBL, NullBuf, Statement, NULL, RowBuf);
  return pCBL->CodErr;
  }

void    LinterASYNC_SLCT(TCBL * pCBL, L_LONG PrzExe, L_CHAR * Statement, void *RowBuf, L_WORD RowBufLen, void *VarBuf, USR_PROC func)
  {
  memcpy(pCBL->Command, "SLCT", 4);
  pCBL->PrzExe = PrzExe | Q_ASYNC;
  pCBL->LnBufRow = RowBufLen;
  inter(pCBL, VarBuf, Statement, (void *) func, RowBuf);
  }

/******************** ANSWERS SUBCOMMAND *********************/
L_LONG LinterGETA(TCBL * pCBL, L_WORD N, void *Out, L_WORD OutLen)
  {
  memcpy(pCBL->Command, "GETA", 4);
  pCBL->RowId = N;                /* start number  column */
  pCBL->LnBufRow = OutLen;
  pCBL->PrzExe &= ~Q_ASYNC;
  inter(pCBL, NULL, NULL, NULL, Out);
  return pCBL->CodErr;
  }

L_LONG LinterAnsColNum(TCBL * pCBL, L_WORD * N)
  {
  memcpy(pCBL->Command, "GETA", 4);
  pCBL->PrzExe &= ~Q_ASYNC;
  pCBL->LnBufRow = 0;                                       /* 23.06.04 */
  inter(pCBL, NULL, NULL, NULL, NULL);
  *N = (L_WORD) pCBL->RowCount;
  return pCBL->CodErr;
  }

L_LONG LinterGETE(TCBL * pCBL, L_LONG CodErr, void *Str, L_WORD StrLen)
  {
  memcpy(pCBL->Command, "GETE", 4);
  pCBL->LnBufRow = StrLen;
  pCBL->RowId = CodErr;
  pCBL->PrzExe &= ~Q_ASYNC;
  inter(pCBL, NULL, NULL, NULL, Str);
  *((char*)Str + ((pCBL->LnBufRow>StrLen)?(StrLen-1):(pCBL->LnBufRow))) = 0;
  return pCBL->CodErr;
  }


L_LONG LinterGETF(TCBL * pCBL, void *RowBuf, L_WORD RowBufLen, void *NullBuf)
  {
  memcpy(pCBL->Command, "GETF", 4);
  pCBL->LnBufRow = RowBufLen;
  pCBL->PrzExe &= ~Q_ASYNC;
  inter(pCBL, NullBuf, NULL, NULL, RowBuf);
  return pCBL->CodErr;
  }

L_LONG LinterGETN(TCBL * pCBL, void *RowBuf, L_WORD RowBufLen, void *NullBuf)
  {
  memcpy(pCBL->Command, "GETN", 4);
  pCBL->LnBufRow = RowBufLen;
  pCBL->PrzExe &= ~Q_ASYNC;
  inter(pCBL, NullBuf, NULL, NULL, RowBuf);
  return pCBL->CodErr;
  }

L_LONG LinterGETP(TCBL * pCBL, void *RowBuf, L_WORD RowBufLen, void *NullBuf)
  {
  memcpy(pCBL->Command, "GETP", 4);
  pCBL->LnBufRow = RowBufLen;
  inter(pCBL, NullBuf, NULL, NULL, RowBuf);
  return pCBL->CodErr;
  }

L_LONG LinterGETL(TCBL * pCBL, void *RowBuf, L_WORD RowBufLen, void *NullBuf)
  {
  memcpy(pCBL->Command, "GETL", 4);
  pCBL->LnBufRow = RowBufLen;
  pCBL->PrzExe &= ~Q_ASYNC;
  inter(pCBL, NullBuf, NULL, NULL, RowBuf);
  return pCBL->CodErr;
  }

L_LONG LinterGETS(TCBL * pCBL, L_LONG N, void *RowBuf, L_WORD RowBufLen, void *NullBuf)
  {
  memcpy(pCBL->Command, "GETS", 4);
  pCBL->LnBufRow = RowBufLen;
  pCBL->RowId = N;                /* number record */
  pCBL->PrzExe &= ~Q_ASYNC;
  inter(pCBL, NullBuf, NULL, NULL, RowBuf);
  return pCBL->CodErr;
  }

L_LONG LinterGETM(TCBL * pCBL, L_LONG Start, L_LONG N, void *RowBuf, L_WORD RowBufLen, void *NullBuf)
  {
  memcpy(pCBL->Command, "GETM", 4);
  pCBL->LnBufRow = RowBufLen;
  pCBL->RowId = Start;            /* start record, if RowId=0 then  next
                                 * record */
  pCBL->RowCount = N;             /* number of records */
  pCBL->PrzExe &= ~Q_ASYNC;
  inter(pCBL, NullBuf, NULL, NULL, RowBuf);
  return pCBL->CodErr;
  }

void    LinterASYNC_GETA(TCBL * pCBL, L_WORD N, void *Out, L_WORD OutLen, USR_PROC func)
  {
  memcpy(pCBL->Command, "GETA", 4);
  pCBL->RowId = N;                /* start numer column */
  pCBL->LnBufRow = OutLen;
  pCBL->PrzExe |= Q_ASYNC;
  inter(pCBL, NULL, NULL, (void *) func, Out);
  }

void    LinterASYNC_AnsColNum(TCBL * pCBL, USR_PROC func)
  {
  memcpy(pCBL->Command, "GETA", 4);
  pCBL->PrzExe |= Q_ASYNC;
  inter(pCBL, NULL, NULL, (void *) func, NULL);
  }

void    LinterASYNC_GETE(TCBL * pCBL, L_LONG CodErr, void *Str, L_WORD StrLen, USR_PROC func)
  {
  memcpy(pCBL->Command, "GETE", 4);
  pCBL->LnBufRow = StrLen;
  pCBL->RowId = CodErr;
  pCBL->PrzExe |= Q_ASYNC;
  inter(pCBL, NULL, NULL, (void *) func, Str);
  }


void    LinterASYNC_GETF(TCBL * pCBL, void *RowBuf, L_WORD RowBufLen, void *NullBuf, USR_PROC func)
  {
  memcpy(pCBL->Command, "GETF", 4);
  pCBL->LnBufRow = RowBufLen;
  pCBL->PrzExe |= Q_ASYNC;
  inter(pCBL, NullBuf, NULL, (void *) func, RowBuf);
  }

void    LinterASYNC_GETN(TCBL * pCBL, void *RowBuf, L_WORD RowBufLen, void *NullBuf, USR_PROC func)
  {
  memcpy(pCBL->Command, "GETN", 4);
  pCBL->LnBufRow = RowBufLen;
  pCBL->PrzExe |= Q_ASYNC;
  inter(pCBL, NullBuf, NULL, (void *) func, RowBuf);
  }

void    LinterASYNC_GETP(TCBL * pCBL, void *RowBuf, L_WORD RowBufLen, void *NullBuf, USR_PROC func)
  {
  memcpy(pCBL->Command, "GETP", 4);
  pCBL->LnBufRow = RowBufLen;
  pCBL->PrzExe |= Q_ASYNC;
  inter(pCBL, NullBuf, NULL, (void *) func, RowBuf);
  }

void    LinterASYNC_GETL(TCBL * pCBL, void *RowBuf, L_WORD RowBufLen, void *NullBuf, USR_PROC func)
  {
  memcpy(pCBL->Command, "GETL", 4);
  pCBL->LnBufRow = RowBufLen;
  pCBL->PrzExe |= Q_ASYNC;
  inter(pCBL, NullBuf, NULL, (void *) func, RowBuf);
  }

void    LinterASYNC_GETS(TCBL * pCBL, L_LONG N, void *RowBuf, L_WORD RowBufLen, void *NullBuf, USR_PROC func)
  {
  memcpy(pCBL->Command, "GETS", 4);
  pCBL->LnBufRow = RowBufLen;
  pCBL->RowId = N;                /* number record */
  pCBL->PrzExe |= Q_ASYNC;
  inter(pCBL, NullBuf, NULL, (void *) func, RowBuf);
  }

void    LinterASYNC_GETM(TCBL * pCBL, L_LONG Start, L_LONG N, void *RowBuf, L_WORD RowBufLen, void *NullBuf, USR_PROC func)
  {
  memcpy(pCBL->Command, "GETM", 4);
  pCBL->LnBufRow = RowBufLen;
  pCBL->RowId = Start;            /* start record, if RowId=0 then  next
                                 * record */
  pCBL->RowCount = N;             /* number of records */
  pCBL->PrzExe |= Q_ASYNC;
  inter(pCBL, NullBuf, NULL, (void *) func, RowBuf);
  }

/******************** NOT SELECT COMMAND *********************/
L_LONG LinterNotSelect(TCBL * pCBL, L_CHAR * Statement)
  {
  memset(pCBL->Command, ' ', 4);
  pCBL->PrzExe &= ~Q_ASYNC;
  inter(pCBL, NULL, Statement, NULL, NULL);
  return pCBL->CodErr;
  }
void    LinterASYNC_NotSelect(TCBL * pCBL, L_CHAR * Statement, USR_PROC func)
  {
  memset(pCBL->Command, ' ', 4);
  pCBL->PrzExe |= Q_ASYNC;
  inter(pCBL, NULL, Statement, (void *) func, NULL);
  }

/******************** TRANSACTION COMMAND *********************/
L_LONG LinterCOMT(TCBL * pCBL)
  {
  memcpy(pCBL->Command, "COMT", 4);
  pCBL->PrzExe &= ~Q_ASYNC;
  inter(pCBL, NULL, NULL, NULL, NULL);
  return pCBL->CodErr;
  }

L_LONG LinterRBAC(TCBL * pCBL)
  {
  memcpy(pCBL->Command, "RBAC", 4);
  pCBL->PrzExe &= ~Q_ASYNC;
  inter(pCBL, NULL, NULL, NULL, NULL);
  return pCBL->CodErr;
  }

L_LONG LinterLREL(TCBL * pCBL, L_CHAR * User, L_CHAR * Table)
  {
  FREL_IN Tbl;
  L_LONG User_ID;
  L_LONG Err;

  Err = LinterFUSR(pCBL, User, &User_ID);
  if (Err == NORMAL)
    {
    L_LONG len;

    Tbl.Owner = User_ID;
    len = strlen(Table);
    if (len > MAX_ID_LEN)
      {
      return SQLLONGID;
      }
    memset(Tbl.TblName, ' ', MAX_ID_LEN);
    memcpy(Tbl.TblName, Table, len);          /* 15.12.11 */
    }
  else
    return Err;

  memcpy(pCBL->Command, "LREL", 4);
  pCBL->PrzExe &= ~Q_ASYNC;
  inter(pCBL, &Tbl, NULL, NULL, NULL);
  return pCBL->CodErr;
  }

L_LONG LinterUREL(TCBL * pCBL)
  {
  memcpy(pCBL->Command, "UREL", 4);
  pCBL->PrzExe &= ~Q_ASYNC;
  inter(pCBL, NULL, NULL, NULL, NULL);
  return pCBL->CodErr;
  }

L_LONG LinterLROW(TCBL * pCBL)
  {
  memcpy(pCBL->Command, "LROW", 4);
  pCBL->PrzExe &= ~Q_ASYNC;
  inter(pCBL, NULL, NULL, NULL, NULL);
  return pCBL->CodErr;
  }

L_LONG LinterUROW(TCBL * pCBL)
  {
  memcpy(pCBL->Command, "UROW", 4);
  pCBL->PrzExe &= ~Q_ASYNC;
  inter(pCBL, NULL, NULL, NULL, NULL);
  return pCBL->CodErr;
  }

void    LinterASYNC_COMT(TCBL * pCBL, USR_PROC func)
  {
  memcpy(pCBL->Command, "COMT", 4);
  pCBL->PrzExe |= Q_ASYNC;
  inter(pCBL, NULL, NULL, (void *) func, NULL);
  }

void    LinterASYNC_RBAC(TCBL * pCBL, USR_PROC func)
  {
  memcpy(pCBL->Command, "RBAC", 4);
  pCBL->PrzExe |= Q_ASYNC;
  inter(pCBL, NULL, NULL, (void *) func, NULL);
  }

void    LinterASYNC_LREL(TCBL * pCBL, L_CHAR * User, L_CHAR * Table, USR_PROC func)
  {
  FREL_IN Tbl;
  L_LONG User_ID;
  L_LONG Err;

  pCBL->PrzExe &= ~Q_ASYNC;
  Err = LinterFUSR(pCBL, User, &User_ID);
  if (Err == NORMAL)
    {
    L_LONG len;

    Tbl.Owner = User_ID;
    len = strlen(Table);
    if (len > MAX_ID_LEN)
      {
      pCBL->CodErr = SQLLONGID;
      func(pCBL, NULL, NULL);
      return;                                 /* 15.12.11 */
      }
    memset(Tbl.TblName, ' ', MAX_ID_LEN);
    memcpy(Tbl.TblName, Table, len);          /* 15.12.11 */
    }  /*if (Err == NORMAL)*/
  else 
    {
    pCBL->CodErr = Err;
    func(pCBL, NULL, NULL);
    return;                                   /* 15.12.11 */
    }
  memcpy(pCBL->Command, "LREL", 4);
  pCBL->PrzExe |= Q_ASYNC;
  inter(pCBL, &Tbl, NULL, (void *) func, NULL);
  }

void    LinterASYNC_UREL(TCBL * pCBL, USR_PROC func)
  {
  memcpy(pCBL->Command, "UREL", 4);
  pCBL->PrzExe |= Q_ASYNC;
  inter(pCBL, NULL, NULL, (void *) func, NULL);
  }

void    LinterASYNC_LROW(TCBL * pCBL, USR_PROC func)
  {
  memcpy(pCBL->Command, "LROW", 4);
  pCBL->PrzExe |= Q_ASYNC;
  inter(pCBL, NULL, NULL, (void *) func, NULL);
  }

void    LinterASYNC_UROW(TCBL * pCBL, USR_PROC func)
  {
  memcpy(pCBL->Command, "UROW", 4);
  pCBL->PrzExe |= Q_ASYNC;
  inter(pCBL, NULL, NULL, (void *) func, NULL);
  }

/******************** TRACING COMMAND *********************/
L_LONG LinterDIRR(TCBL * pCBL, L_LONG N, DIRR_OUT * Out)
  {
  memcpy(pCBL->Command, "DIRR", 4);
  pCBL->LnBufRow = sizeof(DIRR_OUT);
  pCBL->RowId = N;
  pCBL->PrzExe &= ~Q_ASYNC;
  inter(pCBL, NULL, NULL, NULL, Out);
  return pCBL->CodErr;
  }

L_LONG LinterDIRA(TCBL * pCBL, L_LONG N, DIRA_OUT * Out)
  {
  memcpy(pCBL->Command, "DIRA", 4);
  pCBL->LnBufRow = sizeof(DIRA_OUT);
  pCBL->RowId = N;
  pCBL->PrzExe &= ~Q_ASYNC;
  inter(pCBL, NULL, NULL, NULL, Out);
  return pCBL->CodErr;
  }

L_LONG LinterDIRF(TCBL * pCBL, L_LONG N, DIRF_OUT * Out)
  {
  memcpy(pCBL->Command, "DIRF", 4);
  pCBL->LnBufRow = sizeof(DIRF_OUT);
  pCBL->RowId = N;
  pCBL->PrzExe &= ~Q_ASYNC;
  inter(pCBL, NULL, NULL, NULL, Out);
  return pCBL->CodErr;
  }

L_LONG LinterDIRK(TCBL * pCBL, L_LONG N, DIRK_OUT * Out)
  {
  memcpy(pCBL->Command, "DIRK", 4);
  pCBL->LnBufRow = sizeof(DIRK_OUT);
  pCBL->RowId = N;
  pCBL->PrzExe &= ~Q_ASYNC;
  inter(pCBL, NULL, NULL, NULL, Out);
  return pCBL->CodErr;
  }

void    LinterASYNC_DIRR(TCBL * pCBL, L_LONG N, DIRR_OUT * Out, USR_PROC func)
  {
  memcpy(pCBL->Command, "DIRR", 4);
  pCBL->LnBufRow = sizeof(DIRR_OUT);
  pCBL->RowId = N;
  pCBL->PrzExe |= Q_ASYNC;
  inter(pCBL, NULL, NULL, (void *) func, Out);
  }

void    LinterASYNC_DIRA(TCBL * pCBL, L_LONG N, DIRA_OUT * Out, USR_PROC func)
  {
  memcpy(pCBL->Command, "DIRA", 4);
  pCBL->LnBufRow = sizeof(DIRA_OUT);
  pCBL->RowId = N;
  pCBL->PrzExe |= Q_ASYNC;
  inter(pCBL, NULL, NULL, (void *) func, Out);
  }

void    LinterASYNC_DIRF(TCBL * pCBL, L_LONG N, DIRF_OUT * Out, USR_PROC func)
  {
  memcpy(pCBL->Command, "DIRF", 4);
  pCBL->LnBufRow = sizeof(DIRF_OUT);
  pCBL->RowId = N;
  pCBL->PrzExe |= Q_ASYNC;
  inter(pCBL, NULL, NULL, (void *) func, Out);
  }

void    LinterASYNC_DIRK(TCBL * pCBL, L_LONG N, DIRK_OUT * Out, USR_PROC func)
  {
  memcpy(pCBL->Command, "DIRK", 4);
  pCBL->LnBufRow = sizeof(DIRK_OUT);
  pCBL->RowId = N;
  pCBL->PrzExe |= Q_ASYNC;
  inter(pCBL, NULL, NULL, (void *) func, Out);
  }

/******************** BLOB-VALUES COMMAND *********************/
L_LONG LinterGOBJ(TCBL * pCBL, L_LONG Position, void *Out, L_WORD OutLen)
  {
  memcpy(pCBL->Command, "GOBJ", 4);
  pCBL->LnBufRow = OutLen;
  pCBL->RowId = Position;
  pCBL->PrzExe &= ~Q_ASYNC;
  inter(pCBL, NULL, NULL, NULL, Out);
  return pCBL->CodErr;
  }

L_LONG LinterAOBJ(TCBL * pCBL, void *Out, L_WORD OutLen)
  {
  memcpy(pCBL->Command, "AOBJ", 4);
  pCBL->LnBufRow = OutLen;
  pCBL->PrzExe &= ~Q_ASYNC;
  inter(pCBL, NULL, NULL, NULL, Out);
  return pCBL->CodErr;
  }

L_LONG LinterCOBJ(TCBL * pCBL)
  {
  memcpy(pCBL->Command, "COBJ", 4);
  pCBL->PrzExe &= ~Q_ASYNC;
  inter(pCBL, NULL, NULL, NULL, NULL);
  return pCBL->CodErr;
  }

void    LinterASYNC_GOBJ(TCBL * pCBL, L_LONG Position, void *Out, L_WORD OutLen, USR_PROC func)
  {
  memcpy(pCBL->Command, "GOBJ", 4);
  pCBL->LnBufRow = OutLen;
  pCBL->RowId = Position;
  pCBL->PrzExe |= Q_ASYNC;
  inter(pCBL, NULL, NULL, (void *) func, Out);
  }

void    LinterASYNC_AOBJ(TCBL * pCBL, void *Out, L_WORD OutLen, USR_PROC func)
  {
  memcpy(pCBL->Command, "AOBJ", 4);
  pCBL->LnBufRow = OutLen;
  pCBL->PrzExe |= Q_ASYNC;
  inter(pCBL, NULL, NULL, (void *) func, Out);
  }

void    LinterASYNC_COBJ(TCBL * pCBL, USR_PROC func)
  {
  memcpy(pCBL->Command, "COBJ", 4);
  pCBL->PrzExe |= Q_ASYNC;
  inter(pCBL, NULL, NULL, (void *) func, NULL);
  }

/******************** PACK LOADING COMMAND *********************/
L_LONG LinterPUTM(TCBL * pCBL, void *RowBuf, L_WORD RowBufLen)
  {
  memcpy(pCBL->Command, "PUTM", 4);
  pCBL->LnBufRow = RowBufLen;
  pCBL->PrzExe &= ~Q_ASYNC;
  inter(pCBL, NULL, NULL, NULL, RowBuf);
  return pCBL->CodErr;
  }

void    LinterASYNC_PUTM(TCBL * pCBL, void *RowBuf, L_WORD RowBufLen, USR_PROC func)
  {
  memcpy(pCBL->Command, "PUTM", 4);
  pCBL->LnBufRow = RowBufLen;
  pCBL->PrzExe |= Q_ASYNC;
  inter(pCBL, NULL, NULL, (void *) func, RowBuf);
  }

/******************** PROCEDURE FUNCTION *********************/
L_LONG LinterExec(TCBL * pCBL, L_CHAR * ExecStat, void *Buf, L_WORD LenBuf)
  {
  memcpy(pCBL->Command, "    ", 4);
  pCBL->LnBufRow = LenBuf;
  pCBL->PrzExe &= ~Q_ASYNC;
  inter(pCBL, NULL, ExecStat, NULL, Buf);
  return pCBL->CodErr;
  }

L_LONG LinterASYNC_Exec(TCBL * pCBL, L_CHAR * ExecStat, void *Buf, L_WORD LenBuf, USR_PROC func)
  {
  memcpy(pCBL->Command, "    ", 4);
  pCBL->LnBufRow = LenBuf;
  pCBL->PrzExe |= Q_ASYNC;
  inter(pCBL, NULL, ExecStat, (void *) func, Buf);
  return pCBL->CodErr;
  }
