 /***************************************************************************
 *                 RELEX, Inc. Linter DBMS source code                      *
 *              This is public source code of RELEX, Inc.                   *
 *                                                                          *
 *      You have a royalty-free right to use, modify, reproduce and         *
 *     distribute the Sample Files (and/or any modified version) in         *
 *        any way you find useful, provided that you agree that             *
 *       RELEX has no warranty obligations or liability for any             *
 *           Sample Application Files which are modified.                   *
 *                                                                          *
 *                 Copyright (c) 1998-2011 RELEX Inc.                       *
 *                                                                          *
 *                   This software was developed by                         *
 *                                                                          *
 *                            RELEX, Inc.                                   *
 *                         Voronezh, Russia                                 *
 *                                                                          *
 *                        All rights reserved.                              *
 ****************************************************************************
 AUTHOR:
 ----------------------------------------------------------------------------
 PURPOSE: Sample of stored procedure execution in asynchronous version.
 ----------------------------------------------------------------------------
 REVISION HISTORY:
  11.08.98 --          , Creation date
  04.02.00 -- Kononyhin (CBL clearing)
  30.07.02 -- Galizin   ( Function main return int type PT#1046 )
  23.05.03 -- EVG       ( Porting to WinCE: PT #784)
  02.09.03 -- Galizin   ( WorkError already declared in exlib.h )
  08.09.08 -- KOV       ( Added some exceptions )
  04.02.11 -- DV        ( Correct custom exception numbering )
*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "decimals.h"
#include "inter.h"
#include "tick.h"
#include "exlib.h"
                                                          /* 23.05.03 ...*/
static void AsyncHandler(TCBL * pCBL, HPVOID Korteg, HPVOID NullInd); /* 02.09.03 */
static void AnalizeResult(L_BYTE * Buf);
static void show_value(L_CHAR * text, ARGPROC_OUT * ParInfo, L_BYTE * Buf);
static void show_except(L_LONG  except);
static void ProcPrintError(TCBL * pCBL);
                                                          /*... 23.05.03 */
#ifndef WINCE                                             /* 23.05.03 ...*/
int    main(int argc, const char * argv[])                /* 30.07.02 */
#else
int    aexexec()
#endif                                                    /*... 23.05.03 */
  {
  TCBL    CBL;
  L_BYTE    Buf[8192];
  L_CHAR    Name_Pass[] = "SYSTEM/MANAGER";
  L_CHAR    Node[] = "        ";
  L_WORD    Priority = 0;
  L_LONG  PrzExe = M_EXCLUSIVE | Q_ENCODE | M_BINARY;
  L_LONG  Err;

  if (argc < 2)
    {
    fprintf(stderr, "Usage: exeproc <proccaller>\n");
    exit(-1);
    }

  memset(&CBL,0,sizeof(TCBL));			/* 04.02.00 */
  Err = LinterOPEN(&CBL, Name_Pass, Node, Priority, PrzExe);
  if (Err != NORMAL)
    {
    fprintf(stderr, "Error open channel: %d\n", CBL.CodErr);
    exit(-1);
    }

  sprintf((L_CHAR *) Buf, "execute %s;", argv[1]);
  LinterASYNC_Exec(&CBL, (L_CHAR *) Buf, Buf, sizeof(Buf), AsyncHandler);  /* 02.09.03 */

  /* ... */

  inter_wait_single(&CBL, -1);
  if (CBL.CodErr == Proc_Raised_Exception)
    {
    printf("Linter error code %ld\n", CBL.CodErr);
    show_except(CBL.SysErr);
    }
  AnalizeResult(Buf);

  LinterCLOS(&CBL);
  
  return 0;                                                 /* 30.07.02 */
  }

void    AnalizeResult(L_BYTE * Buf)
  {
  ARGPROC_OUT *ParInfo;
  L_WORD   *pCntArg;
  L_WORD    i;
  L_CHAR    str[100];
  L_WORD    Len;
  L_BYTE   *ptr;

  ParInfo = (ARGPROC_OUT *) Buf;
  show_value("Return value", ParInfo, Buf);
  pCntArg = (L_WORD *) (ParInfo + 1);
  ParInfo = (ARGPROC_OUT *) (pCntArg + 1);

  for (i = 0; i < *pCntArg; ++i)
    {
    if (ParInfo->Flags & PARM_FLAG_NAMESENT)
      {
      ptr = Buf + ParInfo->Value;
      Len = *((L_WORD *) ptr);
      memcpy(str, ptr + sizeof(L_WORD), Len);
      str[Len] = 0;
      ParInfo->Value += sizeof(L_WORD) + Len;
      }
    else
      sprintf(str, "Parameter number %d value", (int) ParInfo->Reserv);
    show_value(str, ParInfo, Buf);
    ParInfo++;
    }
  }

void    show_value(L_CHAR * text, ARGPROC_OUT * ParInfo, L_BYTE * Buf)
  {
  L_BYTE   *ptr = Buf + ParInfo->Value;
  CR_INFO *cr;
  L_WORD    i;

  L_CHAR    StrBuf[4000];
  L_WORD    SmallBuf;
  L_LONG  IntBuf;
  L_REAL    RealBuf;
  L_DOUBLE DblBuf;
  L_DECIMAL DecBuf;
  L_DOUBLE NumBuf;

  printf(text);
  printf(" = ");
  if (ParInfo->Flags & PARM_FLAG_ISNULL)
    printf("NULL");
  else if (ParInfo->Flags & PARM_FLAG_ISCURSOR)
    {
    cr = (CR_INFO *) ptr;
    printf("Cursor: NMRKAN = %d, RowId = %ld, KOLKOR = %ld, LNBUFKOR = %d, CODERR = %ld\n",
           cr->NumChan, cr->RowId, cr->RowCount, cr->LnBufRow, cr->CodErr);
    }
  else
    switch (ParInfo->Type.ntyp)
      {
      case DT_CHAR:
        memcpy(StrBuf, ptr, ParInfo->Type.length);
        StrBuf[ParInfo->Type.length] = 0;
        printf(StrBuf);
        break;
      case DT_INTEGER:
        if (ParInfo->Type.length == sizeof(L_WORD))
          {
          memcpy(&SmallBuf, ptr, sizeof(L_WORD));
          printf("%d", SmallBuf);
          }
        else
          {
          memcpy(&IntBuf, ptr, sizeof(L_LONG ));
          printf("%ld", IntBuf);
          }
        break;
      case DT_REAL:
        if (ParInfo->Type.length == sizeof(L_REAL))
          {
          memcpy(&RealBuf, ptr, sizeof(L_REAL));
          printf("%g", RealBuf);
          }
        else
          {
          memcpy(&DblBuf, ptr, sizeof(L_DOUBLE));
          printf("%g", DblBuf);
          }
        break;
      case DT_DATE:
        memcpy(DecBuf, ptr, sizeof(L_DECIMAL));
        TICKTOSTRF(DecBuf, "dd.mm.yyyy:hh:mi:ss", StrBuf);
        printf(StrBuf);
        break;
      case DT_DECIMAL:
        memcpy(DecBuf, ptr, sizeof(L_DECIMAL));
        DecToDbl(DecBuf, &NumBuf);
        printf("%g", NumBuf);
        break;
      case DT_BYTE:
        for (i = 0; i < ParInfo->Type.length; ++i)
          printf("%2x ", (L_WORD) ptr[i]);
        break;
      case DT_BLOB:
        printf("@:BLOB:@");
        break;
        }
  printf("\n");
  }

void    show_except(L_LONG  except)
  {
  switch (except)
    {
 case EXC_DIVZERO:
      printf("DIVZERO");
      break;
    case EXC_UNDEFPROC:
      printf("UNDEFPROC");
      break;
    case EXC_BADPARAM:
      printf("BADPARAM");
      break;
    case EXC_BADRETVAL:
      printf("BADRETVAL");
      break;
    case EXC_NULLDATA:
      printf("NULLDATA");
      break;
    case EXC_NOMEM:
      printf("NOMEM");
      break;
    case EXC_BADCURSOR:
      printf("BADCURSOR");
      break;
    case EXC_CURNOTOPEN:
      printf("CURNOTOPEN");
      break;
    case EXC_BADCODE:
      printf("BADCODE");
      break;
    case EXC_TRIGQUERY:
      printf("SQL is still not supported in triggers");
      break;
    case EXC_APPENDNOTSTARTED:                             /* 08.09.08 ... */
      printf("APPENDNOTSTARTED");
      break;
    case EXC_QUERYWHENAPPEND:
      printf("QUERYWHENAPPEND");
      break;
    case EXC_APPENDACTIVE:
      printf("APPENDACTIVE");
      break;
    case EXC_APPLICATIONERROR:
      printf("APPLICATIONERROR");
      break;
    case EXC_INVTRSTATE:
      printf("Invalid transaction state");
      break;                                               /* ... 08.09.08 */
    default:
      if (except <= EXC_CUSTOM)
        printf("CUSTOM (%d)", -except+EXC_CUSTOM+1);       /* 04.02.11 */
      else
        printf("%d", except);
      break;
      }
  printf(" exception has been raised\n");
  }

void    AsyncHandler(TCBL * pCBL, HPVOID Korteg, HPVOID NullInd)    /* 02.09.03 */
  {
  if (pCBL->CodErr == NORMAL || pCBL->CodErr == Proc_Raised_Exception)
    return;
  printf("Linter error code %ld\n", pCBL->CodErr);
  exit(1);
  }
