
 /***************************************************************************
 *                 RELEX, Inc. Linter DBMS source code                      *
 *              This is public source code of RELEX, Inc.                   *
 *                                                                          *
 *      You have a royalty-free right to use, modify, reproduce and         *
 *     distribute the Sample Files (and/or any modified version) in         *
 *        any way you find useful, provided that you agree that             *
 *       RELEX has no warranty obligations or liability for any             *
 *           Sample Application Files which are modified.                   *
 *                                                                          *
 *                  Copyright (c) 1998-2006 RELEX Inc.                      *
 *                                                                          *
 *                   This software was developed by                         *
 *                                                                          *
 *                            RELEX, Inc.                                   *
 *                         Voronezh, Russia                                 *
 *                                                                          *
 *                        All rights reserved.                              *
 ****************************************************************************
 AUTHOR:
 ----------------------------------------------------------------------------
 PURPOSE: Sample of stored procedure creation.
 ----------------------------------------------------------------------------
 REVISION HISTORY:
  11.08.98 --          , Creation date
  04.02.00 -- Kononyhin (CBL clearing)
  30.07.02 -- Galizin   ( Function main return int type PT#1046 )
  23.05.03 -- EVG       ( Porting to WinCE: PT #784)
*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "inter.h"
#include "exlib.h"

#define SIZE 8192


static void    PrintDescErr(TCBL * pCBL, L_LONG  CodErr); /* 23.05.03 */
static L_BYTE    Load(L_CHAR * Buf, const L_CHAR * fname, L_LONG  size);  /* 23.05.03 */
static void    ProcPrintError(TCBL * pCBL);                    /* 23.05.03 */

#ifndef WINCE                                             /* 23.05.03 ...*/
int    main(int argc, const char * argv[])                /* 30.07.02 */
#else
int    excreat()
#endif                                                    /*... 23.05.03 */
  {
  TCBL    CBLconnect;
  L_CHAR    Name_Pass[] = "SYSTEM/MANAGER";
  L_CHAR    Node[] = "        ";
  L_WORD    Priority = 0;
  L_LONG  PrzExe = M_EXCLUSIVE | Q_ENCODE | M_BINARY;
  L_LONG  Err;
  L_CHAR    Buf[SIZE];

  if (argc != 2)
    {
    printf("Usage:ExCreat input_file\n");
    exit(1);
    }

  memset(&CBLconnect,0,sizeof(TCBL));			/* 04.02.00 */
  Err = LinterOPEN(&CBLconnect, Name_Pass, Node, Priority, PrzExe);
  if (Err != NORMAL)
    ProcPrintError(&CBLconnect);
  printf("Connect to RDBMS Linter\n");

  if (Load(Buf, argv[1], SIZE))
    {
    Err = LinterNotSelect(&CBLconnect, Buf);
    if (Err != NORMAL)
      ProcPrintError(&CBLconnect);
    printf("created/alterd procedure\n");
    }
  else
    printf("can't Load text from file %s", argv[1]);

  printf("End Example\n");
  
  return 0;                                                 /* 30.07.02 */
  }

static void    PrintDescErr(TCBL * pCBL, L_LONG  CodErr)  /* 23.05.03 */
  {
  L_CHAR    Str[256];

  if (NORMAL != LinterGETE(pCBL, CodErr, Str, 255))
    {
    printf("LinterGETE -- Error=%ld\n", pCBL->CodErr);
    exit(1);
    }
  printf("%s", Str);
  }

static void  ProcPrintError(TCBL * pCBL)                    /* 23.05.03 */
  {
  ERRS_DESCR *ErrDesc;
  L_BYTE    Buf[4096];
  L_WORD    Len,
          CntErr,
          i;
  L_WORD    dp;
  L_CHAR    cp[101];
  L_LONG  Err;

  if (pCBL->CodErr != NORMAL)
    {
    printf("Linter Error: %ld\n", pCBL->CodErr);
    if (pCBL->CodErr != Proc_Translation_Error)
      exit(1);
    }
  else
    return;

  Err = LinterGETE(pCBL, Proc_Translation_Error, Buf, sizeof(Buf));
  if (Err != NORMAL)
    {
    printf("ProcPrintError -- Linter error %ld\n", Err);
    exit(1);
    }

  CntErr = *((L_WORD *) Buf);
  ErrDesc = (ERRS_DESCR *) (Buf + sizeof(L_WORD));

  for (i = 0; i < CntErr; i++)
    {
    if (ErrDesc->Row != -1)
      printf("Error in line %ld position %ld: ", ErrDesc->Row, ErrDesc->Position);
    else
      printf("Error: ");
    switch (ErrDesc->ArgType)
      {
      case PROC_ERR_ARGSHORT:
        dp = *((L_WORD *) (ErrDesc + 1));
        PrintDescErr(pCBL, ErrDesc->CodErr + PROC_ERR_BASE);
        printf(": %d", dp);
        ErrDesc = (ERRS_DESCR *) ((L_WORD *) (ErrDesc + 1) + 1);
        break;
      case PROC_ERR_ARGSTRING:
        Len = *((L_WORD *) (ErrDesc + 1));
        memcpy(cp, (L_WORD *) (ErrDesc + 1) + 1, Len);
        cp[Len] = 0;
        PrintDescErr(pCBL, ErrDesc->CodErr + PROC_ERR_BASE);
        printf(": %s", cp);
        ErrDesc = (ERRS_DESCR *) ((L_BYTE *) (ErrDesc + 1) + sizeof(L_WORD) + Len);
        break;
      default:
        PrintDescErr(pCBL, ErrDesc->CodErr + PROC_ERR_BASE);
        ErrDesc++;
        }
    printf("\n");
    }
  printf("End Example\n");
  exit(1);
  }
static L_BYTE    Load(L_CHAR * Buf, const L_CHAR * fname, L_LONG  size)
  {
  L_LONG  ret;
  FILE   *file;

  file = fopen(fname, "r");
  if (file == NULL)
    {
    printf("Error: can't open file %s\n", fname);
    exit(1);
    }

  ret = fread(Buf, size, 1, file);
  if (ret > 0)
    ret = feof(file);

  fclose(file);

  if (ret)
    return 0;

  return 1;
  }
