 /***************************************************************************
 *                 RELEX, Ltd. Linter DBMS source code                      *
 *              This is public source code of RELEX, Ltd.                   *
 *                                                                          *
 *      You have a royalty-free right to use, modify, reproduce and         *
 *     distribute the Sample Files (and/or any modified version) in         *
 *        any way you find useful, provided that you agree that             *
 *       RELEX has no warranty obligations or liability for any             *
 *           Sample Application Files which are modified.                   *
 *                                                                          *
 *                    Copyright (c) 1998 RELEX Ltd.                         *
 *                                                                          *
 *                   This software was developed by                         *
 *                                                                          *
 *                            RELEX, Ltd.                                   *
 *                         Voronezh, Russia                                 *
 *                                                                          *
 *                        All rights reserved.                              *
 ****************************************************************************
 AUTHOR:
 ----------------------------------------------------------------------------
 PURPOSE: Demonstration Linter JDBC driver work.
		  PreparedStatement usage.
 		  Working with blobs.	
 ----------------------------------------------------------------------------
 REVISION HISTORY:
  23.08.1998 -- Creation date
  06.01.2000 -- encoding, className
  25.07.2001 -- Alx : bugfix - start index of buffer
*/

import java.sql.*;
import java.io.*;
import java.util.*;

public class PreparedStatementDemo
extends Config
{
	public static void main (String[] args)
	{
		try 
		{
			Driver d = (Driver) Class.forName(className).newInstance();
			
			System.out.println("Driver found. Now connecting to database ... ");       

			Properties props = new Properties();
		    	props.put( "user",     user );
		    	props.put( "password", password );

		    	if (encoding != null)
				props.put( "encoding", encoding );

			Connection con = DriverManager.getConnection(address, props);

			//DROP/CREATE TABLE
			Statement stmt = con.createStatement();
			try
			{	 
				stmt.executeUpdate("drop table test2;");
			}
			catch(SQLException e)
			{ 
				//System.out.println("Error deleting table:"+e.getMessage());
			}
			
			stmt.executeUpdate("create table test2 (a blob, b int);");

			// CREATE PREPARED STATEMENT 
		    PreparedStatement prepstmt = con.prepareStatement("insert into test2 values (?,?);");
	    	System.out.println("Prepared statement created");
	
            int testSize=10*1024;

	    	byte[] buffer= new byte[testSize];
		    for(int jj=0;jj<testSize;jj++) 
			  buffer[jj]=126;
	    	
	    	System.out.println("Array filled");
	 
			// SET BINARY INPUT STREAM 
			ByteArrayInputStream arr=new ByteArrayInputStream(buffer);
			prepstmt.setBinaryStream(1, arr , arr.available()); 

			// SET INT PARAMETER
			prepstmt.setInt(2, 3); 
			
			// EXECUTE PREPARED UPDATE 
		    int res;
			res = prepstmt.executeUpdate();
		
			System.out.println("Insert result= "+res);
			
			// EXECUTE QUERY 
			ResultSet results;
			results = stmt.executeQuery("select * from test2;");

			byte bt[];
			DataInputStream dis; 
			if( results.next())
            {
				System.out.println("Second column value = "+results.getInt(2));
				dis = new DataInputStream( results.getBinaryStream(1) );
				bt  = new byte[dis.available()];

				System.out.println("Stream with length="+dis.available()+" created ... now reading ");
   			    dis.read(bt, 0, dis.available());                               /*25.07.2001*/
			}
			
			results.close();

			try
			{	 
				stmt.executeUpdate("drop table test2;");
			}
			catch(SQLException e)
			{ 
				System.out.println("Error deleting table:"+e.getMessage());
			}
			con.close(); 

		}
		catch (Exception e)
		{
		    System.out.println("Catched :"+e+" mess= "+e.getMessage() );
		    e.printStackTrace(); 
		     return;
		}
	} // main
}   