 /***************************************************************************
 *                 RELEX, Inc. Linter DBMS source code                      *
 *              This is public source code of RELEX, Inc.                   *
 *                                                                          *
 *      You have a royalty-free right to use, modify, reproduce and         *
 *     distribute the Sample Files (and/or any modified version) in         *
 *        any way you find useful, provided that you agree that             *
 *       RELEX has no warranty obligations or liability for any             *
 *           Sample Application Files which are modified.                   *
 *                                                                          *
 *                 Copyright (c) 1998-2005 RELEX Inc.                       *
 *                                                                          *
 *                   This software was developed by                         *
 *                                                                          *
 *                            RELEX, Inc.                                   *
 *                         Voronezh, Russia                                 *
 *                                                                          *
 *                        All rights reserved.                              *
 ****************************************************************************
 AUTHOR: Pavel Pasechnik
 ----------------------------------------------------------------------------
 PURPOSE: Linter API first step !
 ----------------------------------------------------------------------------
 REVISION HISTORY:
 ----------------------------------------------------------------------------

  30.10.96 -- creation date
  05.03.02 -- Galizin - VxWorks
  06.08.02 -- Ermakov ( Comments TT#3813)
  25.03.03 -- Sveta (Code format)
  23.05.03 -- EVG ( Porting to WinCE: PT #784 )
  10.02.06 -- Maria (Cosmetic)
*/

                    /* GETTING ANSWER COLUMN IN BUFFER */

#include <stdio.h>
#include <stdlib.h>

#include  "linapi.h"
#include  "decimals.h"
#include  "tick.h"

#if defined(VXWORKS)            /* 05.03.02 */
#include "vxstart.h"
#endif

#ifdef _BCPP_
extern unsigned _stklen = 16383;

#endif

/* function for error processing for LinAPI */
void PrintError(L_LONG retCod,               /* 23.05.03 ...*/
                       L_WORD conId,
                       L_WORD curId,
                       L_WORD stmtId,
                       L_CHAR *message);              /*... 23.05.03 */

#if defined(VXWORKS)            /* 05.03.02 */
MainStart(getdata, 32 * 1024, UninitLinterClient)
#elif defined(WINCE)                                      /* 23.05.03 ...*/
int getdata()
#else
int main(void)
#endif                                                    /*... 23.05.03 */
{
  L_WORD nConn;                   /* connection identifier */
  L_WORD nCurs;                   /* cursor identifier    */
  L_DECIMAL D;                    /* date in the inner format */
  L_CHAR date[30];                /* date as a string */
  L_LONG lRet;                    /* return code */

  printf("\nConnect\n");
  if (lRet = LINTER_Connect("SYSTEM", 0, "MANAGER", 0, NULL, mAutocommit, &nConn))
    PrintError(lRet, nConn, 0, 0, "ERROR Linter_Connect");

  printf("Open cursor\n");
  if (lRet = LINTER_OpenCursor(nConn, &nCurs, NULL, 0, mAutocommit))
    PrintError(lRet, nConn, 0, 0, "Error open cursor");

  printf("drop table TDATE\n");
  LINTER_ExecuteDirect(nCurs, "drop table TDATE", 0, NULL, NULL);

  printf("create table TDATE ( D date )\n");
  if (lRet = LINTER_ExecuteDirect(nCurs, "create table TDATE ( D date )", 0, 
                                  NULL, NULL))
    PrintError(lRet, 0, nCurs, 0, "Error ExecuteDirect");

  printf("insert into TDATE values(sysdate)\n");
  if (lRet = LINTER_ExecuteDirect(nCurs, "insert into TDATE values(sysdate)", 0, 
                                  NULL, NULL))
    PrintError(lRet, 0, nCurs, 0, "Error ExecuteDirect");

  printf("select D from TDATE\n");
  if (lRet = LINTER_ExecuteDirect(nCurs, "select D from TDATE", 0, NULL, NULL))
    PrintError(lRet, 0, nCurs, 0, "Error ExecuteDirect");

  /* getting date as a string */
  printf("LINTER_GetData (as tString)\n");
  if (lRet = LINTER_GetData(nCurs,
                            1,
                            date,                     /* answer bufer */
                            30,                       /* length of bufer */
                            tString,                  /* type of the answer  */
                            0, 0, NULL))
    PrintError(lRet, 0, nCurs, 0, "Error GetRowBuffer");

  printf("\nDATE : %s\n", date);

  /* getting date in the inner format */
  printf("LINTER_GetData (as tDate)\n");
  if (lRet = LINTER_GetData(nCurs, 1, D, sizeof(D), tDate, 0, 0, NULL))
    PrintError(lRet, 0, nCurs, 0, "Error Linter_GetData");

  /* converting the date to the string by function from TICK library */
  printf("TICKTOSTRF\n");
  TICKTOSTRF(D,                                       /* input date   */
             "DD:MM:YYYY HH24.MI.SS",                 /* required format */
             date);                                   /* output string */
  printf("DATE : %s\n", date);


  printf("LINTER_CloseAPI\n");
  LINTER_CloseAPI();                                /* Free library resource */

  printf("\nDone.\n");

  return 0;
  /* End */
}
