 /***************************************************************************
 *                 RELEX, Inc. Linter DBMS source code                      *
 *              This is public source code of RELEX, Inc.                   *
 *                                                                          *
 *      You have a royalty-free right to use, modify, reproduce and         *
 *     distribute the Sample Files (and/or any modified version) in         *
 *        any way you find useful, provided that you agree that             *
 *       RELEX has no warranty obligations or liability for any             *
 *           Sample Application Files which are modified.                   *
 *                                                                          *
 *                 Copyright (c) 1998-2005 RELEX Inc.                       *
 *                                                                          *
 *                   This software was developed by                         *
 *                                                                          *
 *                            RELEX, Inc.                                   *
 *                         Voronezh, Russia                                 *
 *                                                                          *
 *                        All rights reserved.                              *
 ****************************************************************************
 AUTHOR: Pavel Pasechnik
 ----------------------------------------------------------------------------
 PURPOSE: Linter API first step !
 ----------------------------------------------------------------------------
 REVISION HISTORY:
 ----------------------------------------------------------------------------

  30.10.96 -- creation date
  05.03.02 -- Galizin - VxWorks
  06.08.02 -- Ermakov ( Comments TT#3813)
  27.03.03 -- Sveta (Code format)
  23.05.03 -- EVG ( Porting to WinCE: PT #784 )
  10.02.06 -- Maria (Cosmetic)
*/

             /* GETTING ANSWER LINE IN BUFFER, DISPLACEMENT ON SAMPLE */

#include <stdio.h>
#include <stdlib.h>

#if defined(VXWORKS)            /* 05.03.02 */
#include "vxstart.h"
#endif

#include "linapi.h"             /* header file for LinAPI */


/* function for  LinAPI errors processing */
void PrintError( L_LONG retCod,               /* 23.05.03 ...*/
                        L_WORD conId,
                        L_WORD curId,
                        L_WORD stmtId,
                        L_CHAR *message );             /*... 23.05.03 */

#if defined(VXWORKS)                                      /* 05.03.02 */
MainStart(rowfetch, 32 * 1024, UninitLinterClient)
#elif defined(WINCE)                                      /* 23.05.03 ...*/
int rowfetch()
#else
int main(void)
#endif                                                    /*... 23.05.03 */
{
  L_WORD nConn;                   /* connection identifier */
  L_WORD nCurs;                   /* cursor identifier    */
  L_CHAR Query[] = "select NAME,FIRSTNAM,CITY,PHONE from PERSON where Name='CLINTON'";
  L_LONG lAnsLen;                 /* answer length */
  L_CHAR *cAnsBuf;                /* answer buffer */
  L_LONG lRet;                    /* return code */
  L_LONG lApiErr = 0;
  L_LONG lLinErr = 0;
  L_LONG lSysErr = 0;

  printf("\nConnect\n");
  if (lRet = LINTER_Connect("SYSTEM", 0, "MANAGER", 0, NULL,
                            mAutocommit,  /* transaction mode */
                            &nConn))
  {
    PrintError(lRet, nConn, 0, 0, "ERROR LINTER_Connect");
  }

  printf("Open cursor\n");
  if (lRet = LINTER_OpenCursor(nConn, &nCurs, NULL, 0, mAutocommit))
    PrintError(lRet, nConn, 0, 0, "Error LINTER_OpenCursor");

  if (lRet = LINTER_ExecuteDirect(nCurs, Query, 0, NULL, NULL))
    PrintError(lRet, 0, nCurs, 0, "Error LINTER_ExecuteDirect");

  /* get answer size */
  if (lRet = LINTER_GetCursorOption(nCurs, cAnswerSize, 0, &lAnsLen, NULL))
    PrintError(lRet, 0, nCurs, 0, "Error LINTER_GetCursorOption");
  /* allocate memory for answer string */
  if (!(cAnsBuf = calloc(lAnsLen + 1, sizeof(L_CHAR))))
  {
    printf("\nNoMemory\n");
    LINTER_CloseAPI();
    return 1;
  }

  /* get first answer string */
  if (lRet = LINTER_GetRowBuffer(nCurs,
                                 (void *) cAnsBuf,          /* answer buffer */
                                 &lAnsLen))                 /* bufer length */
  {
    PrintError(lRet, 0, nCurs, 0, "Error LINTER_GetRowBuffer");
  }

  printf("\nSelect NAME,FIRSTNAM,CITY,PHONE\n");
  printf("From PERSON \n");
  printf("Where NAME = 'CLINTON';\n");
  printf("\n");
  printf("NAME                FIRSTNAME      CITY           PHONE\n");
  printf("\n");
  printf("%s\n", cAnsBuf);

  /* travelling through the selection and getting remaining answers */
  for (;;)
  {
    if (lRet = LINTER_Fetch(nCurs,
                            toNext,                  /* travelling direction */
                            0,
                            1,                /* count of required strings   */
                            NULL, NULL, NULL))
    {
      free(cAnsBuf);
      if (lRet == LINAPI_ERROR)
      {
        LINTER_Error(0, nCurs, 0, &lApiErr, &lLinErr, &lSysErr, NULL, NULL);
        if (lApiErr == eLinterError && lLinErr == 2)  /* if there is no more
                                                       * data */
        {
          printf("End of data.\n");
          break;
        }
        else
          PrintError(lRet, 0, nCurs, 0, "Error LINTER_Fetch");
      }
      else
        PrintError(lRet, 0, nCurs, 0, "Error LINTER_Fetch");
    }
    if (lRet = LINTER_GetRowBuffer(nCurs, (void *) cAnsBuf, &lAnsLen))
      PrintError(lRet, 0, nCurs, 0, "Error LINTER_GetRowBuffer");
    printf("%s\n", cAnsBuf);
  }

  printf("Done.");
  return 0;
  /* End */
}
