 /***************************************************************************
 *                 RELEX, Inc. Linter DBMS source code                      *
 *              This is public source code of RELEX, Inc.                   *
 *                                                                          *
 *      You have a royalty-free right to use, modify, reproduce and         *
 *     distribute the Sample Files (and/or any modified version) in         *
 *        any way you find useful, provided that you agree that             *
 *       RELEX has no warranty obligations or liability for any             *
 *           Sample Application Files which are modified.                   *
 *                                                                          *
 *                 Copyright (c) 1998-2006 RELEX Inc.                       *
 *                                                                          *
 *                   This software was developed by                         *
 *                                                                          *
 *                            RELEX, Inc.                                   *
 *                         Voronezh, Russia                                 *
 *                                                                          *
 *                        All rights reserved.                              *
 ****************************************************************************
 AUTHOR: Pavel Pasechnik
 ----------------------------------------------------------------------------
 PURPOSE: Linter API second step
 ----------------------------------------------------------------------------
 REVISION HISTORY:
 ----------------------------------------------------------------------------

  30.10.96 -- creation date
  10.12.98 -- Kononyhin (LinErr = 2 - don't print error)
  05.03.02 -- Galizin ( VxWorks )
  06.08.02 -- Ermakov (Cosmetic TT#3813)
  25.03.03 -- Sveta (code format)
  23.05.03 -- EVG ( Porting to WinCE: PT #784 )
  22.11.06 -- Kolesov (code corrected)
*/

#include <stdio.h>
#include <stdlib.h>

#if defined(VXWORKS)            /* 05.03.02 */
#include "vxstart.h"
#endif

#include "linapi.h"

#define  NoStmt    0

/* LinAPI errors processing function */
static void processing_error( L_LONG ret_cod,               /* 23.05.03 ...*/
                              L_WORD con_id,
                              L_WORD cur_id,
                              L_WORD stmt_id,
                              char *message )             /*... 23.05.03 */
{
  L_LONG lRet;
  L_LONG apierr = 0, error = 0, syserr = 0;

  if (ret_cod == LINAPI_ERROR)
  {
    /* getting error code for required object */
    if (lRet = LINTER_Error(con_id, cur_id, stmt_id,
                            &apierr, &error, &syserr, NULL, NULL))
    {
      printf("diagnostic error %ld\n", lRet);
    }
    else
    {
      if (error == 2)           /* 10.12.98 */
        return;
      printf("ApiErr = %ld, LinErr = %ld, SysErr = %ld\n%s\n",
             apierr, error, syserr, message);
      if ((apierr == eLinterError) && error > 2000 && error < 3000)
      {
        /* getting string and position when sintax error is */
        printf("Syntax error : line %d, position %d\n",
               (short) syserr, *(short *) ((char *) &syserr + 2));
      }
    }
    LINTER_CloseAPI();          /* finishing of workin LinAPI */
    exit(1);
  }
  else
    printf("Return code = %ld\n", ret_cod);
}

#if defined(VXWORKS)            /* 05.03.02 */
MainStart(apibinda, 32 * 1024, UninitLinterClient)
#elif defined(WINCE)                                      /* 23.05.03 ...*/
int apibinda()
#else
void main(void)
#endif                                                    /*... 23.05.03 */
{
  L_WORD nConn;                   /* connection identifier */
  L_WORD nCurs;                   /* cursor identifier    */
  char Query[] = "select MODEL, PERSONID from AUTO where MAKE = 'LANCIA'";
  char cAnsModel[21];           /* answer buffer for column MODEL    */
  L_LONG lAnsPersonid;            /* answer buffer for column PERSONID */
  L_LONG lRet;                    /* return code                  */
  L_LONG i;

  /* Creating connection */
  printf("\nConnect\n");
  if (lRet = LINTER_Connect("SYSTEM", 0, "MANAGER", 0, NULL, mAutocommit, &nConn))
    processing_error(lRet, nConn, 0, 0, "ERROR Linter_Connect");

  /* cursor opening for query processing  */
  printf("Open cursor\n");
  if (lRet = LINTER_OpenCursor(nConn, &nCurs, NULL, 0, mAutocommit))
    processing_error(lRet, nConn, 0, 0, "Error open cursor");

  /* processing query */
  if (lRet = LINTER_ExecuteDirect(nCurs, Query, 0, NULL, NULL))
    processing_error(lRet, 0, nCurs, 0, "Error ExecuteDirect");

  /* registration of answers bufers */
  if (lRet = LINTER_BindAnswer(nCurs, /* cursore ID  */
                               NoStmt,  /* NoStmt=0 as since ExecuteDirect */
                               1, /* Coloumn number in the answer */
                               cAnsModel, /* bufer address for the answer */
                               NULL,  /* not use */
                               sizeof(cAnsModel), /* lenth of answer
                                                   * element */
                               tChar, /* necessary answer type */
                               sizeof(cAnsModel), /* lenth of answer area */
                               0, 0, NULL)) /* not use */
    processing_error(lRet, 0, nCurs, 0, "Error BindAnswer");
  if (lRet = LINTER_BindAnswer(nCurs,
                               NoStmt,
                               2,
                               &lAnsPersonid,
                               NULL,
                               sizeof(lAnsPersonid),
                               tInteger,
                               sizeof(lAnsPersonid),
                               0, 0, NULL))
    processing_error(lRet, 0, nCurs, 0, "Error BindAnswer");

  printf("\nSelect MODEL, PERSONID\n");
  printf("From AUTO \n");
  printf("Where MAKE = 'LANCIA';\n");
  printf("\n");
  printf("MODEL               PERSONID\n"); /* 06.08.02 */
  printf("\n");

  /* travelling through the selection and getting the answers  */
  for (i = 1;; i++)
  {
    /* set on the string in the selection number i */
    if (lRet = LINTER_Fetch(nCurs,
                            toAbsNumber,  /* travelling direction */
                            i,  /* begin position        */
                            1,  /* number of required strings  */
                            NULL, NULL, NULL))
    {
      processing_error(lRet, 0, nCurs, 0, "Error Fetch");
      break;                    /* 10.12.98 */
    }
    else
    {
      cAnsModel[20] = '\0';
      printf("%s%ld\n", cAnsModel, lAnsPersonid);
    }
  }  /* for */
  LINTER_CloseAPI();
  exit(0);
  /* End */
}
