 /***************************************************************************
 *                 RELEX, Inc. Linter DBMS source code                      *
 *              This is public source code of RELEX, Inc.                   *
 *                                                                          *
 *      You have a royalty-free right to use, modify, reproduce and         *
 *     distribute the Sample Files (and/or any modified version) in         *
 *        any way you find useful, provided that you agree that             *
 *       RELEX has no warranty obligations or liability for any             *
 *           Sample Application Files which are modified.                   *
 *                                                                          *
 *                 Copyright (c) 1998-2005 RELEX Inc.                       *
 *                                                                          *
 *                   This software was developed by                         *
 *                                                                          *
 *                            RELEX, Inc.                                   *
 *                         Voronezh, Russia                                 *
 *                                                                          *
 *                        All rights reserved.                              *
 ****************************************************************************
 AUTHOR: 
 ----------------------------------------------------------------------------
 PURPOSE: Linter API sample of BLOB
 ----------------------------------------------------------------------------
 REVISION HISTORY:
 ----------------------------------------------------------------------------

  30.03.04 -- creation date
*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include  "linapi.h"

#if defined(VXWORKS)          
# include "vxstart.h"
#endif

#define  MAX_QUERY_SIZE      4096
#define  BLOBS               2
#define  ROWS                10

/* function for error processing for LinAPI */
static void processing_error( L_LONG ret_cod,             
                              L_WORD con_id,
                              L_WORD cur_id,
                              L_WORD stmt_id,
                              char *message)            
{
  L_LONG lRet;
  L_LONG apierr = 0, error = 0, syserr = 0;

  if (ret_cod == LINAPI_ERROR)
  {
    /* getting error codes for required object */
    if (lRet = LINTER_Error(con_id, cur_id, stmt_id,
                            &apierr, &error, &syserr, NULL, NULL))
    {
      printf("diagnostic error %ld\n", lRet);
    }
    else
    {
      printf("ApiErr = %ld, LinErr = %ld, SysErr = %ld\n%s\n",
             apierr, error, syserr, message);
      if ((apierr == eLinterError) && error > 2000 && error < 3000)
      {
        /* getting string and position if syntax error is */
        printf("Syntax error : line %d, position %d\n",
               (short) syserr, *(short *) ((char *) &syserr + 2));
      }
    }
    LINTER_CloseAPI();          /* Free library resource */
    exit(1);
  }
  else
    printf("Return code = %ld\n", ret_cod);
}

/***************************************************************************    
                               MAIN
****************************************************************************/

#if defined(VXWORKS)           
MainStart(apidata, 32 * 1024, UninitLinterClient)
#elif defined(WINCE)                                    
int apidata()
#else
int main(void)
#endif                            
{
  L_WORD nConn,                                 /* connection identifier */
       nCurs;                                 /* cursor identifier    */
  L_LONG lRet;                                  /* return code */
  char query[MAX_QUERY_SIZE];
  char temp [MAX_QUERY_SIZE];
  char w[20];
  long i, j, l, len;
  

  /* Establish connection */
  printf("\nConnect\n");
  if (lRet = LINTER_Connect("SYSTEM", 0, "MANAGER", 0, NULL, mAutocommit, &nConn))
    processing_error(lRet, nConn, 0, 0, "ERROR Linter_Connect");

  /* cursor opening for query processing */
  printf("Open cursor\n");
  if (lRet = LINTER_OpenCursor(nConn, &nCurs, NULL, 0, mAutocommit))
    processing_error(lRet, nConn, 0, 0, "Error open cursor");

  printf("drop table LAPIBLOB\n");
  LINTER_ExecuteDirect(nCurs, "drop table LAPIBLOB", 0, NULL, NULL);

  /* creating table LAPIBLOB */
  sprintf(query, "create table LAPIBLOB(id int, ");
  for (i=0; i<BLOBS; i++)
  {
    sprintf(temp, "blb%i blob%s", i, (i<(BLOBS-1))?", ":"");
    strcat(query, temp);
  }
  strcat(query, ");");
  
  printf(query);
  printf("\n");

  if (lRet = LINTER_ExecuteDirect(nCurs, query, 0, NULL, NULL))
    processing_error(lRet, 0, nCurs, 0, "Error ExecuteDirect");

  for (j=0; j<ROWS; j++)
  {

    sprintf(query, "insert into LAPIBLOB (id, ");
    for (i=0; i<BLOBS; i++)
    {
      sprintf(temp, "blb%i%s", i, (i<(BLOBS-1))?", ":"");
      strcat(query, temp);
    }
    
    sprintf(temp, ") values (%i, ", j);
    strcat(query, temp);
    
    for (i=0; i<BLOBS; i++)
    {
      sprintf(temp, "NULL%s", (i<(BLOBS-1))?", ":"");
      strcat(query, temp);
    }
    strcat(query, ");");

    printf("\n");
    printf(query);
    printf("\n");

    if (lRet = LINTER_ExecuteDirect(nCurs, query, 0, NULL, NULL))
      processing_error(lRet, 0, nCurs, 0, "Error ExecuteDirect");

    for (i=0; i<BLOBS; i++)
    {      
      memset (temp,0,sizeof(temp));
      for (l = i; l<(i+10); l++)
      {
        sprintf(w, "%i ", l+j);
        strcat(temp, w);
      }

      if(lRet = LINTER_PurgeBlob(nCurs, (short) (i+2), NULL, NULL))   
        processing_error(lRet, 0, nCurs, 0, "Error PurgeBlob");  
  
      printf("\rColumn: %i  Row: %i : Purging Blob ... ", i+2, j+1);
      printf("\n");

      if(lRet = LINTER_AddBlob(nCurs, (short) (i+2), (short) i, temp, strlen(temp), NULL, NULL))
        processing_error(lRet, 0, nCurs, 0, "Error AddBlob");  

      printf("\rColumn: %i  Row: %i : Adding Blob : \"%s\"\n", i+2, j+1, temp);    
    }
  }
  printf("\n");

  sprintf(query, "select ");
  for (i=0; i<BLOBS; i++)
  {
    sprintf(temp, "blb%i%s", i, (i<(BLOBS-1))?", ":"");
    strcat(query, temp);
  }
  strcat(query, " from LAPIBLOB order by ID;");

  if (lRet = LINTER_ExecuteDirect(nCurs, query, 0, NULL, NULL))
    processing_error(lRet, 0, nCurs, 0, "Error ExecuteDirect");
  
  for (j=0; j<ROWS; j++)
  {

    if(lRet = LINTER_Fetch(nCurs, toAbsNumber, j+1, 1, NULL, NULL, NULL))
      processing_error(lRet, 0, nCurs, 0, "Error Fetch");  

    for (i=0; i<BLOBS ; i++)
    { 
      memset (temp,0,sizeof(temp));
      for (l = i; l<(i+10); l++)
      {
        sprintf(w, "%i ", l+j);
        strcat(temp, w);
      }

      len = strlen(temp);
      memset (query, 0, sizeof(query));
  
      if (lRet = LINTER_FetchBlob(nCurs, (short) (i+1), 1, &len, query, NULL, NULL))
        processing_error(lRet, 0, nCurs, 0, "Error FetchBlob");

      printf("\rColumn: %i  Row: %i : FetchBlob : Checking ... ", i+2, j+1);

      if (len != (int) strlen(temp))
      {
        printf("Error : BlobLength=%i != DataLength=%i\n", len, strlen(temp));
      }
      else 
      if (strncmp(temp, query, len) != 0)
      {
        printf("Error : strings not equals\n");
      }
      else
      {
        printf("Ok\n");
      }
    }
  }

  printf("\n");

  if(lRet = LINTER_CloseCursor(nCurs))
    processing_error(lRet, 0, nCurs, 0, "Close cursor");

  printf("Close cursor\n");

  if(lRet = LINTER_CloseConnect(nConn))
    processing_error(lRet, 0, nConn, 0, "Close connect");

  printf("Close connect\n");

  return 0;
}

/***************************************************************************    
                               END
****************************************************************************/


