 /***************************************************************************
 *                 RELEX, Inc. Linter DBMS source code                      *
 *              This is public source code of RELEX, Inc.                   *
 *                                                                          *
 *      You have a royalty-free right to use, modify, reproduce and         *
 *     distribute the Sample Files (and/or any modified version) in         *
 *        any way you find useful, provided that you agree that             *
 *       RELEX has no warranty obligations or liability for any             *
 *           Sample Application Files which are modified.                   *
 *                                                                          *
 *                 Copyright (c) 1998-2006 RELEX Inc.                       *
 *                                                                          *
 *                   This software was developed by                         *
 *                                                                          *
 *                            RELEX, Inc.                                   *
 *                         Voronezh, Russia                                 *
 *                                                                          *
 *                        All rights reserved.                              *
 ****************************************************************************
 AUTHOR: Pavel Pasechnik
 ----------------------------------------------------------------------------
 PURPOSE: for users
 ----------------------------------------------------------------------------
 REVISION HISTORY:
 ----------------------------------------------------------------------------

  26.09.96 -- creation date
  29.10.99 -- Kononyhin (NumCol - long)
  05.03.02 -- Galizin ( VxWorks)
  30.07.02 -- Galizin ( Remove compiler warning PT#1046 )
  06.08.02 -- Ermakov ( Comments TT#3813)
  25.03.03 -- Sveta (Code format)
  23.05.03 -- EVG ( Porting to WinCE: PT #784 )
  22.11.06 -- Kolesov (code corrected)
  01.08.11 -- Galizin  (Typization)
*/

/*
Program implements possible variant of table loader from text file(CSV).
Data is loading by LENARR records.
*/
#include "apidemo.h"
#if defined(VXWORKS)            /* 05.03.02 */
#include "vxstart.h"
#endif

#define P_LEN      240          /* maximum size of parameter value */

static char asyncFlag = 1;      /* operator processing flag           */ /* 23.05.03 ...*/
static char fileFlag = 1;       /* permitting new values reading flag */  
static char asyncErr = 1;       /* error operator processing flag     */ /*... 23.05.03 */
char fExit = 1;
char fExecDir = 1;

typedef struct
{                               /* structure for parameter values */
  L_LONG *Len;
  char   *Par;
} pParam;

/* function for error processing LinAPI */
static void processing_error( L_LONG ret_cod,               /* 23.05.03 ...*/
                              L_WORD con_id,
                              L_WORD cur_id,
                              L_WORD stmt_id,
                              char *message )             /*... 23.05.03 */
{
  L_LONG lRet;
  L_LONG apierr = 0, error = 0, syserr = 0;

  if (ret_cod == LINAPI_ERROR)
  {
    if (lRet = LINTER_Error(con_id, cur_id, stmt_id,
                            &apierr, &error, &syserr, NULL, NULL))
    {
      printf("diagnostic error %d\n", lRet);
    }
    else
    {
      printf("ApiErr = %d, LinErr = %d, SysErr = %d\n%s\n",
             apierr, error, syserr, message);
      if ((apierr == eLinterError) && error > 2000 && error < 3000)
      {
        printf("Syntax error : line %d, position %d\n",
               (short) syserr, *(short *) ((char *) &syserr + 2));
      }
    }
    LINTER_CloseAPI();
    exit(1);
  }
  else
    printf("Return code = %d\n", ret_cod);
}

/* function forms query with stated parameters number */
static void FormQuery(char *query, char *tname, L_LONG num)  /* 23.05.03 */
{
  sprintf(query, "insert into %s values (?", tname);
  for (num--; num > 0; num--)
    strcat(query, ",?");
  strcat(query, ");");
}

/* function for answer processing */
#ifdef _ASYNC_
static void AsyncFunc(L_WORD CID, L_LONG LastError, void *userarg) /* 01.08.11 23.05.03 */
{
  if (LastError)
  {
    printf("Error async %d\n", LastError);
    fExit = 0;
    asyncErr = 0;
  }
  else
  {
    asyncFlag = 1;
    fExit = (char) (long) userarg;                          /* 01.08.11 30.07.02 */
  }
}

/* ExecuteDirect*/
void ExecDirAsync(L_WORD CID, L_LONG LastError, void *userarg)
{
  if (LastError)
  {
    printf("Error async exec dir %d\n", LastError);
    if (LastError == eLinterError)
    {
      processing_error(LINAPI_ERROR, 0, CID, 0, "ExecuteDirect (async)");
    }
  }
  else
    fExecDir = 0;
}

#else
static void *AsyncFunc = (void *) NULL;
void *ExecDirAsync = (void *) NULL;

#endif

#if defined(VXWORKS)            /* 05.03.02 */
MainStart(apibparr, 32 * 1024, UninitLinterClient)
#elif defined(WINCE)                                      /* 23.05.03 ...*/
extern int    argc;
extern char * argv[];
int apibparr()
#else
int main(int argc, char *argv[])
#endif                                                    /*... 23.05.03 */
{
  L_WORD CONN1;                   /* connection identifier  */
  L_WORD CURS;                    /* cursor identifier     */
  L_WORD STMT, STMT_W;            /* operator identifier */
  char query[300] = "";         /* query */
  char sep;                     /* separator */
  L_LONG NumCol;                  /* column count in the answer *//* 01.08.11 29.10.99
                                 *  */
  L_LONG error = 1;               /* flag for ending file reading */ /* 01.08.11 */
  L_LONG len;                     /* 29.10.99 */
  L_LONG lRet;                    /* ending code */
  L_LONG lRows;                   /* strings counter */
  L_SWORD nCols;                  /* coloumns counter */ /* 01.08.11 */
  pParam *ParBind;              /* array of structs for parameter values */
  pParam *Par;                  /* array of structs for parameter values */
  L_LONG LENARR;                  /* array size */ /* 01.08.11 */

  if (argc < 2)
  {
    printf("\napibparr [table name] <LENARR> <separator>\n");
    return 1;
  }
  if (argc < 3)
    LENARR = 10;
  else
    LENARR = atol(argv[2]);
  if (argc < 4)
    sep = ',';
  else
    sep = argv[3][0];

  /* opening lod-file */
  if (!OpenFiles(argv[1]))
    return 1;

  /* establish server connection */
  if (lRet = LINTER_Connect("SYSTEM", 0, "MANAGER", 0, NULL, mOptimistic, &CONN1))
    processing_error(lRet, CONN1, 0, 0, "Error connect");

  /* getting count of columns in the table */
  sprintf(query, "select * from %s", argv[1]);
  if (lRet = LINTER_CreateStatement(CONN1, query, 0, &STMT_W))
    processing_error(lRet, CONN1, 0, 0, "Error create statement STMT_W ");
  len = sizeof(NumCol);                                     /* 01.08.11 29.10.99 */
  if (lRet = LINTER_GetStatementOption(STMT_W, sColumnCount, 0, &NumCol, &len)) /* 29.10.99 */
    processing_error(lRet, CONN1, 0, 0, "Error get stmt option STMT_W");
  if (lRet = LINTER_FreeStatement(STMT_W))
    processing_error(lRet, CONN1, 0, 0, "Error free stmt STMT_W");

  /* memory allocation for parameter array */
  ParBind = (pParam *) calloc(NumCol, sizeof(pParam));
  Par = (pParam *) calloc(NumCol, sizeof(pParam));
  if (!Par || !ParBind)
  {
    if (ParBind)
      free(ParBind);
    printf("NoMemory");
    LINTER_CloseAPI();
    return 1;
  }

  /* forming  query text for data adding */
  FormQuery(query, argv[1], NumCol);
  printf("%s\n", query);

  /* operator creation */
  if (lRet = LINTER_CreateStatement(CONN1, query, sizeof(query), &STMT))
    processing_error(lRet, CONN1, 0, 0, "Error create statement STMT");

  /* opening cursor for operator */
  if (lRet = LINTER_OpenCursor(CONN1, &CURS, NULL, 0, mOptimistic))
    processing_error(lRet, CONN1, 0, 0, "Error open cursor");

  for (nCols = 0; nCols < NumCol; nCols++)
  {
    /* memory allocation for parameters and their lengths */
    ParBind[nCols].Len = (L_LONG *) calloc(LENARR, sizeof(L_LONG)); /* 01.08.11 */
    ParBind[nCols].Par = (void *) calloc(LENARR * (P_LEN + 1), sizeof(char));
    Par[nCols].Len = (L_LONG *) calloc(LENARR, sizeof(L_LONG)); /* 01.08.11 */
    Par[nCols].Par = (void *) calloc(LENARR * (P_LEN + 1), sizeof(char));
    if (!Par[nCols].Par)
    {
      printf("NoMemory");
      LINTER_CloseAPI();
      return 1;
    }
    /* binding of parameters values buffers */
    if (lRet = LINTER_BindParameter(CURS,
                                    STMT,
                                    (nCols + 1),            /* 01.08.11 */
                                    NULL,
                                    NULL,
                                    ParBind[nCols].Par,
                                    LENARR,
                                    P_LEN,
                                    tChar,
                                    NULL,
                                    ParBind[nCols].Len))
      processing_error(lRet, 0, CURS, 0, "Error binding parameters");
  }

  /* main cycle */
  while (fExit)
  {
    if (fileFlag)
    {
      /* reading parameters from file */
      for (lRows = 0; lRows < LENARR; lRows++)
      {
        for (nCols = 0; nCols < NumCol; nCols++)
        {
          if (!(error = ReadParameter(Par[nCols].Par + lRows * P_LEN,
                                      &(Par[nCols].Len[lRows]), sep)))
            break;
        }  /* for */
        if (!error)
          break;
      }
      fileFlag = 0;
    }
    /* new values copy in to the bufer and process operator */
    if (asyncFlag)              /* if 1, then operator is processed */
    {
#ifdef _ASYNC_
      asyncFlag = 0;
#endif
      if (!error)
        LENARR = lRows;
      for (lRows = 0; lRows < LENARR; lRows++)
      {
        for (nCols = 0; nCols < NumCol; nCols++)
        {
          ParBind[nCols].Len[lRows] = Par[nCols].Len[lRows];
          memcpy(ParBind[nCols].Par + lRows * P_LEN,
                 Par[nCols].Par + lRows * P_LEN, Par[nCols].Len[lRows]);
        }
      } /* for */
#ifdef _ASYNC_
      if (lRet = LINTER_ExecuteStatement(CURS, STMT, &lRows, AsyncFunc, (void *)(long)error))  /* 01.08.11 */
#else
      fExit = (char) error;
      if (lRet = LINTER_ExecuteStatement(CURS, STMT, &lRows, AsyncFunc, NULL))
#endif
      {
        LINTER_RollBack(CONN1, NULL, NULL);
        processing_error(lRet, 0, CURS, 0, "Error execute");
      }
      fileFlag = 1;
      printf("%c", argv[1][0]);
    }
  }                             /* end while */

  /* commit changes in the database */
  printf("\nCOMMIT -- ");
  if (asyncErr)
  {
    if (lRet = LINTER_Commit(CONN1, NULL, NULL))
      processing_error(lRet, CONN1, 0, 0, "Error commit");
    else
      printf("   Ok!");
  }
  /* rollback changes by connection if error ending of operator is */
  if (!asyncErr)
  {
    if (lRet = LINTER_RollBack(CONN1, NULL, NULL))
      processing_error(lRet, CONN1, 0, 0, "Error COMMIT");
    else
      printf("RollBack!\n");
  }
  /* close file and free memory */
  CloseFiles();
  for (nCols = 0; nCols < NumCol; nCols++)
  {
    free(ParBind[nCols].Par);
    free(ParBind[nCols].Len);
    free(Par[nCols].Par);
    free(Par[nCols].Len);
  }
  free(ParBind);
  free(Par);

  /* ending of work */
  LINTER_CloseAPI();

  printf("\nDone.\n");

  return 0;
}
