 /***************************************************************************
 *                 RELEX, Inc. Linter DBMS source code                      *
 *              This is public source code of RELEX, Inc.                   *
 *                                                                          *
 *      You have a royalty-free right to use, modify, reproduce and         *
 *     distribute the Sample Files (and/or any modified version) in         *
 *        any way you find useful, provided that you agree that             *
 *       RELEX has no warranty obligations or liability for any             *
 *           Sample Application Files which are modified.                   *
 *                                                                          *
 *                 Copyright (c) 1998-2006 RELEX Inc.                       *
 *                                                                          *
 *                   This software was developed by                         *
 *                                                                          *
 *                            RELEX, Inc.                                   *
 *                         Voronezh, Russia                                 *
 *                                                                          *
 *                        All rights reserved.                              *
 ****************************************************************************
 AUTHOR: Pavel Pasechnik
 ----------------------------------------------------------------------------
 PURPOSE: for users
 ----------------------------------------------------------------------------
 REVISION HISTORY:
 ----------------------------------------------------------------------------

  26.09.96 -- creation date
  05.03.02 -- Galizin - VxWorks
  06.08.02 -- Ermakov ( Comments TT#3813)
  26.03.03 -- Sveta (Code format)
  23.05.03 -- EVG ( Porting to WinCE: PT #784 )
  22.11.06 -- Kolesov (code corrected)
*/
/*
The program implements one of the possible variants of loading
table from text file.
Records are added one by one.
If the _ASYNC_ macro is set for the compiler,
loading will be performed in the asynchronous mode.
*/

#if defined(VXWORKS)            /* 05.03.02 */
#include "vxstart.h"
#endif

#include "apidemo.h"

#define _P_LEN      240         /* maximum parameter value size */

struct pParam
{                               /* structure for parameter description */
  L_LONG Len;
  char Par[_P_LEN + 1];
} pParam;

static char asyncFlag = 1;      /* operator processing flag        */ /* 23.05.03 ...*/
static char fileFlag = 1;       /* flag permits reading new values */ 
static char asyncErr = 1;       /* error operator processing flag  */ /*... 23.05.03 */

/* function for processing answer */
#ifdef _ASYNC_
static void AsyncFunc(short CID, long LastError, void *userarg) /* 23.05.03 */
{
  if (LastError)
  {
    printf("Error async %ld\n", LastError);
    asyncErr = 0;
  }
  else
    asyncFlag = 1;
}

#else
static void *AsyncFunc = NULL;                            /* 23.05.03 */

#endif

static void FormQuery(char *query, char *tname, L_LONG num)  /* 23.05.03 */
{
  sprintf(query, "insert into %s values (?", tname);
  for (num--; num > 0; num--)
    strcat(query, ",?");
  strcat(query, ");");
}

/* function for  LinAPI errors processing */
static void processing_error( L_LONG ret_cod,               /* 23.05.03 ...*/
                              L_WORD con_id,
                              L_WORD cur_id,
                              L_WORD stmt_id,
                              char *message )             /*... 23.05.03 */
{
  L_LONG lRet;
  L_LONG apierr = 0, error = 0, syserr = 0;

  if (ret_cod == LINAPI_ERROR)
  {
    if (lRet = LINTER_Error(con_id, cur_id, stmt_id,
                            &apierr, &error, &syserr, NULL, NULL))
    {
      printf("diagnostic error %ld\n", lRet);
    }
    else
    {
      printf("ApiErr = %ld, LinErr = %ld, SysErr = %ld\n%s\n",
             apierr, error, syserr, message);
      if ((apierr == eLinterError) && error > 2000 && error < 3000)
      {
        printf("Syntax error : line %d, position %d\n",
               (short) syserr, *(short *) ((char *) &syserr + 2));
      }
    }
    LINTER_CloseAPI();
    exit(1);
  }
  else
    printf("Return code = %ld\n", ret_cod);
}

#if defined(VXWORKS)            /* 05.03.02 */
MainStart(apidemo, 32 * 1024, UninitLinterClient)
#elif defined(WINCE)                                      /* 23.05.03 ...*/
extern int    argc;
extern char * argv[];
int apidemo()
#else
int main(int argc, char *argv[])
#endif                                                    /*... 23.05.03 */
{
  L_WORD CONN1;                   /* connection identifier  */
  L_WORD CURS;                    /* cursor identifier     */
  L_WORD STMT, STMT_W;            /* operator identifiers */
  char query[300] = "";
  char sep;
  L_LONG NumCol;                  /* columns count in the answer */
  struct pParam *ParBind;       /* array of structures for parameters
                                 * values */
  struct pParam *ParRead;       /* array of structures for parameters
                                 * values */
  long error = 1;
  L_LONG lRet;
  L_LONG T_Mode = mAutocommit;
  short i;

  if (argc < 2)
  {
    printf("\napidemo [table name] <separator>\n");
    return 1;
  }
  if (argc < 3)
    sep = ',';
  else
    sep = argv[2][0];

  /* lod-file opening */
  if (!OpenFiles(argv[1]))
    return 1;

  /* creating server connection */
  if (lRet = LINTER_Connect("SYSTEM", 0, "MANAGER", 0, NULL, T_Mode, &CONN1))
    processing_error(lRet, CONN1, 0, 0, "Error connect");

  /* getting table description */
  sprintf(query, "select * from %s", argv[1]);
  if (lRet = LINTER_CreateStatement(CONN1, query, 0, &STMT_W))
    processing_error(lRet, CONN1, 0, 0, "Error create statement STMT_W ");
  if (lRet = LINTER_GetStatementOption(STMT_W, sColumnCount, 0, &NumCol, NULL))
    processing_error(lRet, CONN1, 0, 0, "Error get stmt option STMT_W");
  ParBind = (struct pParam *) calloc(NumCol, sizeof(struct pParam));
  ParRead = (struct pParam *) calloc(NumCol, sizeof(struct pParam));
  if (!ParRead || !ParBind)
  {
    if ( ParBind )
      free(ParBind);
    printf("NoMemory\n");
    LINTER_CloseAPI();
    return 1;
  }

  if (lRet = LINTER_FreeStatement(STMT_W))
    processing_error(lRet, CONN1, 0, 0, "Error free stmt STMT_W");

  /* formind query text for adding records */
  FormQuery(query, argv[1], NumCol);
  printf("%s\n", query);

  /* operator creating */
  if (lRet = LINTER_CreateStatement(CONN1, query, sizeof(query), &STMT))
    processing_error(lRet, CONN1, 0, 0, "Error create statement STMT");

  /* opening cursor for processing operator */
  if (lRet = LINTER_OpenCursor(CONN1, &CURS, NULL, 0, T_Mode))
    processing_error(lRet, CONN1, 0, 0, "Error open cursor");

  /* binding parameter values buffers */
  for (i = 1; i <= NumCol; i++)
  {
    if (lRet = LINTER_BindParameter(CURS, STMT, i, NULL, NULL, ParBind[i - 1].Par, 1,
                                  0, tChar, NULL, &(ParBind[i - 1].Len)))
      processing_error(lRet, 0, CURS, 0, "Error binding parameters");
  } /* for */
  /* main cycle */
  while (error)
  {
    if (fileFlag)               /* while processing operator, new values
                                 * are read */
    {
      /* reading parameters from file */
      for (i = 1; i <= NumCol; i++)
      {
        if (!(error = ReadParameter(ParRead[i - 1].Par, &(ParRead[i - 1].Len), sep)))
          break;
      }  /* for */
      fileFlag = 0;
    }
    if (asyncFlag)              /* if 1, then operator is processed */
    {
      if (error)
      {
#ifdef _ASYNC_
        asyncFlag = 0;
#endif
        /* new values are copying in buffer and operator processing again */
        for (i = 1; i <= NumCol; i++)
        {
          ParBind[i - 1].Len = ParRead[i - 1].Len;
          memcpy(ParBind[i - 1].Par, ParRead[i - 1].Par, ParRead[i - 1].Len);
        }
        if (lRet = LINTER_ExecuteStatement(CURS, STMT, NULL, AsyncFunc, NULL))
        {
          LINTER_RollBack(CONN1, NULL, NULL);
          processing_error(lRet, 0, CURS, 0, "Error execute");
        }
        fileFlag = 1;
      }
    }  /* if */
    /* if error, when cycle  interrupts  */
    if (!asyncErr)
      break;
  }                             /* end while */

  if (T_Mode != mAutocommit)
  {
    /* commit changes in database */
    printf("\nCOMMIT -- ");
    if (asyncErr)
    {
      if (lRet = LINTER_Commit(CONN1, NULL, NULL))
        processing_error(lRet, CONN1, 0, 0, "Error commit");
      else
        printf("   Ok!");
    }
    /* rollback changes by connection if with error occurs */
    if (!asyncErr)
    {
      if (lRet = LINTER_RollBack(CONN1, NULL, NULL))
        processing_error(lRet, CONN1, 0, 0, "Error COMMIT");
      else
        printf("RollBack!\n");
    }
  }

  /* close file and free memory */
  CloseFiles();

  free(ParBind);
  free(ParRead);

  /* free library resource */
  LINTER_CloseAPI();
  printf("\nDone.\n");

  return 0;
}
