 /***************************************************************************
 *                 RELEX, Inc. Linter DBMS source code                      *
 *              This is public source code of RELEX, Inc.                   *
 *                                                                          *
 *      You have a royalty-free right to use, modify, reproduce and         *
 *     distribute the Sample Files (and/or any modified version) in         *
 *        any way you find useful, provided that you agree that             *
 *       RELEX has no warranty obligations or liability for any             *
 *           Sample Application Files which are modified.                   *
 *                                                                          *
 *                 Copyright (c) 1998-2005 RELEX Inc.                       *
 *                                                                          *
 *                   This software was developed by                         *
 *                                                                          *
 *                            RELEX, Inc.                                   *
 *                         Voronezh, Russia                                 *
 *                                                                          *
 *                        All rights reserved.                              *
 ****************************************************************************
 AUTHOR: Pavel Pasechnik
 ----------------------------------------------------------------------------
 PURPOSE: Linter API second step
 ----------------------------------------------------------------------------
 REVISION HISTORY:
 ----------------------------------------------------------------------------

  30.10.96 -- creation date
  05.03.02 -- Galizin - VxWorks
  06.08.02 -- Ermakov ( Comments TT#3813)
  26.03.03 -- Sveta (Code format)
  23.05.03 -- EVG ( Porting to WinCE: PT #784 )
  13.09.03 -- Kononyhin (checking of lRowsCount)
*/

#include <stdio.h>
#include <stdlib.h>

#include "linapi.h"

#if defined(VXWORKS)            /* 05.03.02 */
#include "vxstart.h"
#endif

#define  NoStmt    0

typedef char Make_Type[21];

struct Answer
{                               /* structure for getting answer */
  Make_Type cMake;
  L_LONG lPersonID;
};

/* function for LinAPI errors processing */
static void processing_error( L_LONG ret_cod,               /* 23.05.03 ...*/
                              L_WORD con_id,
                              L_WORD cur_id,
                              L_WORD stmt_id,
                              char *message )             /*... 23.05.03 */
{
  L_LONG lRet;
  L_LONG apierr = 0, error = 0, syserr = 0;

  if (ret_cod == LINAPI_ERROR)
  {
    /* getting error codes for required object */
    if (lRet = LINTER_Error(con_id, cur_id, stmt_id,
                            &apierr, &error, &syserr, NULL, NULL))
    {
      printf("diagnostic error %ld\n", lRet);
    }
    else
    {
      printf("ApiErr = %ld, LinErr = %ld, SysErr = %ld\n%s\n",
             apierr, error, syserr, message);
      if ((apierr == eLinterError) && error > 2000 && error < 3000)
      {
        /* getting string and position if sintax error occurs */
        printf("Syntax error : line %d, position %d\n",
               (short) syserr, *(short *) ((char *) &syserr + 2));
      }
    }
    LINTER_CloseAPI();          /* finishing LinAPI work */
    exit(1);
  }
  else
    printf("Return code = %ld\n", ret_cod);
}

#if defined(VXWORKS)            /* 05.03.02 */
MainStart(apifetch, 32 * 1024, UninitLinterClient)
#elif defined(WINCE)                                      /* 23.05.03 ...*/
int apifetch()
#else
int main(void)
#endif                                                    /*... 23.05.03 */
{
  L_WORD nConn;                   /* connection identifier */
  L_WORD nCurs;                   /* cursor identifier    */
  char Query[] = "select MAKE, PERSONID from AUTO where "
  "PERSONID > 100 and PERSONID < 110";
  struct Answer *saAnswer;      /* poiter to answers array             */
  L_LONG lRowCount;               /* count of strings in the selection   */
  L_LONG lRowAnswer;              /* count of strings in the answer      */
  L_LONG lRet;                    /* return code                         */
  long i;

  /* Creating connection */
  printf("\nConnect\n");
  if (lRet = LINTER_Connect("SYSTEM", 0, "MANAGER", 0, NULL,
                            mAutocommit,  /* transaktion regime */
                            &nConn))
  {
    processing_error(lRet, nConn, 0, 0, "ERROR Linter_Connect");
  }

  /* opening cursor for processing query */
  printf("Open cursor\n");
  if (lRet = LINTER_OpenCursor(nConn, &nCurs, NULL, 0, mAutocommit))
    processing_error(lRet, nConn, 0, 0, "Error open cursor");

  /* processing query */
  if (lRet = LINTER_ExecuteDirect(nCurs, Query, 0, NULL, NULL))
    processing_error(lRet, 0, nCurs, 0, "Error ExecuteDirect");

  /* getting count of records in the answer */
  if (lRet = LINTER_GetCursorOption(nCurs, cRowCount, 0, &lRowCount, NULL))
    processing_error(lRet, 0, nCurs, 0, "Error GetCursorOption");
  /* memory allocation for answer */
  if(!lRowCount)                                           /* 13.09.03 */
  {
    printf("\nNo rows in the table AUTO!\n");
    LINTER_CloseAPI();
    exit(EXIT_FAILURE);
  }
  saAnswer = calloc(lRowCount, sizeof(struct Answer));
  if (!saAnswer)
  {
    printf("\nNoMemory.\n");
    LINTER_CloseAPI();
    exit(EXIT_FAILURE);
  }
  /* Registration answer bufers. Registration bufers for array of answers
   * (struct Answer) */
  /* for first coloumn */
  if (lRet = LINTER_BindAnswer(nCurs, /* cursor ID */
                               NoStmt,  /* NoStmt=0 as since ExecuteDirect */
                               1, /* Number of coloumn in the answer  */
                               saAnswer->cMake, /* address of bufer for
                                                 * answer */
                               NULL,  /* don't use */
                               sizeof(struct Answer), /* lenth of array
                                                       * element (length of
                                                       * struct Answer) */
                               tChar, /* required answer type */
                               sizeof(Make_Type), /* length of answer area */
                               0, 0, NULL)) /* don't use */
  {
    processing_error(lRet, 0, nCurs, 0, "Error BindAnswer");
  }
  /* for first coloumn */
  if (lRet = LINTER_BindAnswer(nCurs,
                               NoStmt,
                               2,
                               &saAnswer->lPersonID,
                               NULL,
                               sizeof(struct Answer),
                               tInteger,
                               sizeof(long),
                               0, 0, NULL))
  {
    processing_error(lRet, 0, nCurs, 0, "Error BindAnswer");
  }

  printf("\nSelect MODEL, PERSONID\n");
  printf("From AUTO \n");
  printf("Where PERSONID > 100 And PERSONID < 110;\n");
  printf("                  \n");
  printf("                  MAKE                PERSONID\n");
  printf("\n");

  /* travelling through the selection and getting the answers */

  /* get the first string of the selection */
  printf("First row      :  ");
  if (lRet = LINTER_Fetch(nCurs,
                          toFirst,  /* direction of traveling */
                          0,
                          1,    /* count of required strings */
                          NULL,
                          NULL,
                          NULL))
  {
    processing_error(lRet, 0, nCurs, 0, "Error Fetch");
  }
  else
  {
    saAnswer->cMake[20] = '\0';
    printf("%s%ld\n", saAnswer->cMake, saAnswer->lPersonID);
  }

  /* get last string of the selection */
  printf("Last row       :  ");
  if (lRet = LINTER_Fetch(nCurs,
                          toLast, /* direction of travelling */
                          0,
                          1,    /* count of required strings */
                          NULL,
                          NULL,
                          NULL))
  {
    processing_error(lRet, 0, nCurs, 0, "Error Fetch");
  }
  else
  {
    saAnswer->cMake[20] = '\0';
    printf("%s%ld\n", saAnswer->cMake, saAnswer->lPersonID);
  }

  /* get string from selection for absolut number 5 */
  printf("Row number 5   :  ");
  if (lRet = LINTER_Fetch(nCurs,
                          toAbsNumber,  /* travelling direction */
                          5,    /* start position       */
                          1,    /* count of required strings   */
                          NULL,
                          NULL,
                          NULL))
  {
    processing_error(lRet, 0, nCurs, 0, "Error Fetch");
  }
  else
  {
    saAnswer->cMake[20] = '\0';
    printf("%s%ld\n", saAnswer->cMake, saAnswer->lPersonID);
  }

  /* get next string from selection */
  printf("Next row       :  ");
  if (lRet = LINTER_Fetch(nCurs,
                          toNext, /* travelling direction */
                          0,
                          1,    /* count of required strings  */
                          NULL,
                          NULL,
                          NULL))
  {
    processing_error(lRet, 0, nCurs, 0, "Error Fetch");
  }
  else
  {
    saAnswer->cMake[20] = '\0';
    printf("%s%ld\n", saAnswer->cMake, saAnswer->lPersonID);
  }

  /* get previous string from selection */
  printf("Previous row   :  ");
  if (lRet = LINTER_Fetch(nCurs,
                          toPrevious, /* travelling direction */
                          0,
                          1,    /* count of required strings   */
                          NULL,
                          NULL,
                          NULL))
  {
    processing_error(lRet, 0, nCurs, 0, "Error Fetch");
  }
  else
  {
    saAnswer->cMake[20] = '\0';
    printf("%s%ld\n", saAnswer->cMake, saAnswer->lPersonID);
  }

  /* get string for number -2  about current string */
  printf("Current row -2 :  ");
  if (lRet = LINTER_Fetch(nCurs,
                          toRelNumber,  /* travelling direction    */
                          -2,   /* comparative string number */
                          1,    /* count of required strings      */
                          NULL,
                          NULL,
                          NULL))
  {
    processing_error(lRet, 0, nCurs, 0, "Error Fetch");
  }
  else
  {
    saAnswer->cMake[20] = '\0';
    printf("%s%ld\n", saAnswer->cMake, saAnswer->lPersonID);
  }

  /* get all records */
  printf("All            :\n");
  if (lRet = LINTER_Fetch(nCurs,
                          toAbsNumber,  /* direction of travelling     */
                          1,    /* starting position in the selection */
                          lRowCount,  /* count of required strings       */
                          &lRowAnswer,  /* count of gotten strings      */
                          NULL,
                          NULL))
  {
    processing_error(lRet, 0, nCurs, 0, "Error Fetch");
  }
  else
  {
    for (i = 0; i < lRowAnswer; i++)
    {
      saAnswer[i].cMake[20] = '\0';
      printf("                  %s%ld\n", saAnswer[i].cMake, saAnswer[i].lPersonID);
    }
  }

  /* get 3 records from the end of selection */
  printf("3 rows from end:\n");
  if (lRet = LINTER_Fetch(nCurs,
                          toFromEnd,  /* direction of travelling     */
                          0,
                          3,    /* count of required strings    */
                          &lRowAnswer,  /* count of gotten strings      */
                          NULL,
                          NULL))
  {
    processing_error(lRet, 0, nCurs, 0, "Error Fetch");
  }
  else
  {
    for (i = 0; i < lRowAnswer; i++)
    {
      saAnswer[i].cMake[20] = '\0';
      printf("                  %s%ld\n", saAnswer[i].cMake, saAnswer[i].lPersonID);
    }
  }
  /* Finishing of working with LinAPI. */
  LINTER_CloseAPI();

  printf("Done.\n");

  return 0;
  /* End */
}
