 /***************************************************************************
 *                 RELEX, Inc. Linter DBMS source code                      *
 *              This is public source code of RELEX, Inc.                   *
 *                                                                          *
 *      You have a royalty-free right to use, modify, reproduce and         *
 *     distribute the Sample Files (and/or any modified version) in         *
 *        any way you find useful, provided that you agree that             *
 *       RELEX has no warranty obligations or liability for any             *
 *           Sample Application Files which are modified.                   *
 *                                                                          *
 *                 Copyright (c) 1998-2005 RELEX Inc.                       *
 *                                                                          *
 *                   This software was developed by                         *
 *                                                                          *
 *                            RELEX, Inc.                                   *
 *                         Voronezh, Russia                                 *
 *                                                                          *
 *                        All rights reserved.                              *
 ****************************************************************************
 AUTHOR: Pavel Pasechnik
 ----------------------------------------------------------------------------
 PURPOSE: Linter API first step !
 ----------------------------------------------------------------------------
 REVISION HISTORY:
 ----------------------------------------------------------------------------

  30.10.96 -- creation date
  05.03.02 -- Galizin - VxWorks
  06.08.02 -- Ermakov ( Comments TT#3813)
  27.03.03 -- Sveta (Code format)
  23.05.03 -- EVG ( Porting to WinCE: PT #784 )
*/

#include <stdio.h>
#include <stdlib.h>

#if defined(VXWORKS)            /* 05.03.02 */
#include "vxstart.h"
#endif

#include "linapi.h"             /* header file for LinAPI */


/* function for  LinAPI errors processing */
static void processing_error( L_LONG ret_cod,               /* 23.05.03 ...*/
                              L_WORD con_id,
                              L_WORD cur_id,
                              L_WORD stmt_id,
                              char *message )             /*... 23.05.03 */
{
  L_LONG lRet;
  L_LONG apierr = 0, error = 0, syserr = 0;

  if (ret_cod == LINAPI_ERROR)
  {
    /* getting error codes for required object */
    if (lRet = LINTER_Error(con_id, cur_id, stmt_id,
                            &apierr, &error, &syserr, NULL, NULL))
    {
      printf("diagnostic error %ld\n", lRet);
    }
    else
    {
      printf("ApiErr = %ld, LinErr = %ld, SysErr = %ld\n%s\n",
             apierr, error, syserr, message);
      if ((apierr == eLinterError) && error > 2000 && error < 3000)
      {
        /* getting string and position if syntax error occurs */
        printf("Syntax error : line %d, position %d\n",
               (short) syserr, *(short *) ((char *) &syserr + 2));
      }
    }
    LINTER_CloseAPI();          /* free library resource */
    exit(1);
  }
  else
    printf("Return code = %ld\n", ret_cod);
}

#if defined(VXWORKS)            /* 05.03.02 */
MainStart(apistep, 32 * 1024, UninitLinterClient)
#elif defined(WINCE)                                      /* 23.05.03 ...*/
int apistep()
#else
int main(void)
#endif                                                    /*... 23.05.03 */
{
  L_WORD nConn;                   /* connection identifier */
  L_WORD nCurs;                   /* cursor identifier    */
  char Query[] = "select NAME,FIRSTNAM,CITY,PHONE from PERSON where Name='CLINTON'";
  L_LONG lAnsLen;                 /* answer length */
  char *cAnsBuf;                /* answer buffer */
  L_LONG lRet;                    /* return code */
  L_LONG lApiErr = 0;
  L_LONG lLinErr = 0;
  L_LONG lSysErr = 0;

  /* Creating connection */
  printf("\nConnect\n");
  if (lRet = LINTER_Connect("SYSTEM", 0, "MANAGER", 0, NULL,
                            mAutocommit,  /* transaction mode */
                            &nConn))
  {
    processing_error(lRet, nConn, 0, 0, "ERROR Linter_Connect");
  }

  /* opening cursor for processing query */
  printf("Open cursor\n");
  if (lRet = LINTER_OpenCursor(nConn, &nCurs, NULL, 0, mAutocommit))
    processing_error(lRet, nConn, 0, 0, "Error open cursor");

  /* processing query */
  if (lRet = LINTER_ExecuteDirect(nCurs, Query, 0, NULL, NULL))
    processing_error(lRet, 0, nCurs, 0, "Error ExecuteDirect");

  /* get answer size */
  if (lRet = LINTER_GetCursorOption(nCurs, cAnswerSize, 0, &lAnsLen, NULL))
    processing_error(lRet, 0, nCurs, 0, "Error get cursor option");
  /* allocate memory for answer string */
  if (!(cAnsBuf = calloc(lAnsLen + 1, sizeof(char))))
  {
    printf("\nNoMemory\n");
    LINTER_CloseAPI();
    return 1;
  }

  /* get first answer string */
  if (lRet = LINTER_GetRowBuffer(nCurs,
                                 (void *) cAnsBuf,  /* answer buffer */
                                 &lAnsLen)) /* bufer length */
  {
    processing_error(lRet, 0, nCurs, 0, "Error GetRowBuffer");
  }

  printf("\nSelect NAME,FIRSTNAM,CITY,PHONE\n");
  printf("From PERSON \n");
  printf("Where NAME = 'CLINTON';\n");
  printf("\n");
  printf("NAME                FIRSTNAME      CITY           PHONE\n");
  printf("\n");
  printf("%s\n", cAnsBuf);

  /* travelling through the selection and getting remaining answers */
  for (;;)
  {
    if (lRet = LINTER_Fetch(nCurs,
                            toNext, /* travelling direction */
                            0,
                            1,  /* count of required strings   */
                            NULL, NULL, NULL))
    {
      free(cAnsBuf);
      if (lRet == LINAPI_ERROR)
      {
        LINTER_Error(0, nCurs, 0, &lApiErr, &lLinErr, &lSysErr, NULL, NULL);
        if (lApiErr == eLinterError && lLinErr == 2)  /* if there is no more
                                                       * data */
        {
          printf("End of data.\n");
          break;
        }
        else
          processing_error(lRet, 0, nCurs, 0, "Error Fetch");
      }
      else
        processing_error(lRet, 0, nCurs, 0, "Error Fetch");
    }
    if (lRet = LINTER_GetRowBuffer(nCurs, (void *) cAnsBuf, &lAnsLen))
      processing_error(lRet, 0, nCurs, 0, "Error GetRowBuffer");
    printf("%s\n", cAnsBuf);
  }

  printf("Done.");
  return 0;
  /* End */
}
