 /***************************************************************************
 *                 RELEX, Inc. Linter DBMS source code                      *
 *              This is public source code of RELEX, Inc.                   *
 *                                                                          *
 *      You have a royalty-free right to use, modify, reproduce and         *
 *     distribute the Sample Files (and/or any modified version) in         *
 *        any way you find useful, provided that you agree that             *
 *       RELEX has no warranty obligations or liability for any             *
 *           Sample Application Files which are modified.                   *
 *                                                                          *
 *                 Copyright (c) 1998-2013 RELEX Inc.                       *
 *                                                                          *
 *                   This software was developed by                         *
 *                                                                          *
 *                            RELEX, Inc.                                   *
 *                         Voronezh, Russia                                 *
 *                                                                          *
 *                        All rights reserved.                              *
 ****************************************************************************
 AUTHOR: Koval Dmitry
 ----------------------------------------------------------------------------
 PURPOSE: Linter CPPAPI first step !
 ----------------------------------------------------------------------------
 REVISION HISTORY:
 ----------------------------------------------------------------------------
  27.12.10 -- KOV (Correction: added possibility of transactions on separate cursors, not just on connection)
  06.12.11 -- KOV (New options for internal buffer & temporary files)
  17.04.13 -- KOV (New test for linConnection::ConnectEnv)
  04.06.14 -- Galizin (Latin letters for any database codepages)
  04.05.14a-- Galizin (MBX for different OS)
*/

#ifdef WIN32
#include "windows.h"
#else
#include <unistd.h>
#include <string.h>
#include <math.h>
#endif
#ifdef QNX
#include <unix.h>
#endif

#include <stdio.h>
#include <stdlib.h>
#include "lincppapi.h"

#if defined(VXWORKS)
#include "vxstart.h"
#endif

#ifdef _BCPP_
extern unsigned _stklen = 16383;

#endif

/* function for error processing for LinCPPAPI */
static void processing_error(linConnection *pConnect,
                             L_LONG         ret_cod,
                             L_CHAR        *message,
                             L_BOOL         bExit = L_TTRUE)
{
  L_CHAR  sModuleName[64], sErrorText[1024];
  L_ULONG ulModuleNameLen, ulErrorTextLen;
  L_LONG  lLinApi, lLinter, lSystem, lRet;

  printf("Return code = %ld (%s)\n", ret_cod, message);
  while (pConnect->GetErrorsCount())
  {
    ulModuleNameLen = sizeof(sModuleName);                 /* 27.12.10 */
    ulErrorTextLen = sizeof(sErrorText);                   /* 27.12.10 */
    if ((lRet = pConnect->GetError(sModuleName,
                                   &ulModuleNameLen,
                                   sErrorText,
                                   &ulErrorTextLen,
                                   &lLinApi, 
                                   &lLinter, 
                                   &lSystem)) == LINAPI_SUCCESS)
      printf("%s: '%s'\n", sModuleName, sErrorText);
    else
      printf("GetError returned code #%ld\n", lRet);
  }
  if (bExit)
  {
    delete pConnect;
    exit(1);
  }
}

static void processing_error(linCursor *pCursor,           /* 27.12.10 ... */
                             L_LONG     ret_cod,
                             L_CHAR     *message,
                             L_BOOL      bExit = L_TTRUE)
{
  L_CHAR  sModuleName[64], sErrorText[1024];
  L_ULONG ulModuleNameLen, ulErrorTextLen;
  L_LONG  lLinApi, lLinter, lSystem, lRet;

  printf("Return code = %ld (%s)\n", ret_cod, message);
  while (pCursor->GetErrorsCount())
  {
    ulModuleNameLen = sizeof(sModuleName);
    ulErrorTextLen = sizeof(sErrorText);
    if ((lRet = pCursor->GetError(sModuleName,
                                  &ulModuleNameLen,
                                  sErrorText,
                                  &ulErrorTextLen,
                                  &lLinApi, 
                                  &lLinter, 
                                  &lSystem)) == LINAPI_SUCCESS)
      printf("%s: '%s'\n", sModuleName, sErrorText);
    else
      printf("GetError returned code #%ld\n", lRet);
  }
  if (bExit)
  {
    delete pCursor->GetConnection();
    exit(1);
  }
}                                                          /* ... 27.12.10 */

#define CHECK_BUFFER(lBufSize, Ch) \
{ \
  L_BYTE TmpBuf[0x20000]; \
  if (lBufLen != lBufSize) \
  { \
    printf("Error : BlobLength=%ld != DataLength=%ld\n", lBufLen, lBufSize); \
    delete pConnect; \
    exit(1); \
  } \
  else \
  { \
    memset(TmpBuf, Ch, lBufSize); \
    if (memcmp(Buf, TmpBuf, lBufSize)) \
    { \
      printf("Error: buffer damaged\n"); \
      delete pConnect; \
      exit(1); \
    } \
    else \
      printf("Ok\n"); \
  } \
}

/* test for linConnection, linCursor, linStatement classes */
void testApi()
{
  L_LONG         lRet;     /* return code */
  L_CHAR         date[30]; /* date as a string */
  linConnection *pConnect;
  linCursor     *pCursor;
  linStatement  *pStat;
  L_BYTE         Buf[0x20000];
  L_LONG         lBufLen, i, lPos = 1, lParam[4], lLen[4], lResult;

  printf("----------------- TEST FOR MAIN API FUNCTIONS -------------------\n\n");
  printf("Test #1: autocommit:\n\n");

  pConnect = new linConnection;
  if (!pConnect)
  {
    printf("No memory\n");
    exit(1);
  }

  /* Set maximum interval for restart LINTER kernel: */
  pConnect->SetMaxTimeout(300000 /* 300 sec = 5 min */);

  if (lRet = pConnect->Connect("SYSTEM", 0, "MANAGER", 0, NULL, mAutocommit))
    processing_error(pConnect, lRet, "ERROR Connect");

  printf("Open cursor\n");
  if (lRet = pConnect->OpenCursor(NULL, 0, &pCursor))
    processing_error(pConnect, lRet, "Error open cursor");

  printf("ExecuteDirect: 'create or replace table TDATE ( D date )'\n");
  if (lRet = pCursor->ExecuteDirect("create or replace table TDATE (D date)", 0))
    processing_error(pCursor, lRet, "Error ExecuteDirect 1");

  printf("ExecuteDirect: 'insert into TDATE values(sysdate)'\n");
  if (lRet = pCursor->ExecuteDirect("insert into TDATE values(sysdate)", 0))
    processing_error(pCursor, lRet, "Error ExecuteDirect 2");

  printf("ExecuteDirect: 'select D from TDATE'\n");
  if (lRet = pCursor->ExecuteDirect("select D from TDATE", 0))
    processing_error(pCursor, lRet, "Error ExecuteDirect 3");

  printf("GetData (as tString)\n");
  if (lRet = pCursor->GetData(1,
                              date,     /* answer bufer */
                              30,       /* length of bufer */
                              tString,  /* type of the answer  */
                              0, 0, NULL))
    processing_error(pCursor, lRet, "Error GetData");

  printf("    DATE : %s\n", date);

  printf(" ExecuteDirect: 'drop table TDATE'\n");
  if (lRet = pCursor->ExecuteDirect("drop table TDATE", 0))
    processing_error(pCursor, lRet, "Error ExecuteDirect 4", false);

  printf("CloseCursor\n");
  if (lRet = pConnect->CloseCursor(pCursor))
    processing_error(pConnect, lRet, "Error CloseCursor");

  printf("CloseConnect\n");
  if (lRet = pConnect->CloseConnect())
    processing_error(pConnect, lRet, "Error CloseConnect");

  delete pConnect;

  pConnect = new linConnection;

  if (!pConnect)
  {
    printf("No memory\n");
    exit(1);
  }

  printf("\nTest #2: exclusive:\n\n");
  if (lRet = pConnect->Connect("SYSTEM", 0, "MANAGER", 0, NULL, mExclusive))
    processing_error(pConnect, lRet, "ERROR Connect 2");

  printf("Open cursor\n");
  if (lRet = pConnect->OpenCursor(NULL, 0, &pCursor))
    processing_error(pConnect, lRet, "Error open cursor 2");

  printf("TEST FOR ERROR CODE, BEGIN\n");

  printf(" ExecuteDirect: 'drop table TDATE'\n");
  if (lRet = pCursor->ExecuteDirect("drop table TDATE", 0))
    processing_error(pCursor, lRet, "Error 1 ExecuteDirect", false);

  printf(" ExecuteDirect: 'create tab TDATE ( D date )'\n");
  if (lRet = pCursor->ExecuteDirect("create tab TDATE (D date)", 0))
    processing_error(pCursor, lRet, "Error 2 ExecuteDirect", false);

  printf("TEST FOR ERROR CODE, END\n");

  printf("ExecuteDirect: 'create table TDATE ( D date )'\n");
  if (lRet = pCursor->ExecuteDirect("create table TDATE (D date)", 0))
    processing_error(pCursor, lRet, "Error ExecuteDirect 21");

  printf("ExecuteDirect: 'insert into TDATE values(sysdate)'\n");
  if (lRet = pCursor->ExecuteDirect("insert into TDATE values(sysdate)", 0))
    processing_error(pCursor, lRet, "Error ExecuteDirect 22");

  printf("---------------------------------------------------------\n");
  printf("Pause 20 sec: need to shut down and restart LINTER kernel\n");
  printf("---------------------------------------------------------\n");
#ifdef WIN32
  Sleep(20000);
#else
  sleep(20);
#endif

  printf("ExecuteDirect: 'insert into TDATE values(sysdate + 1)'\n");
  if (lRet = pCursor->ExecuteDirect("insert into TDATE values(sysdate + 1)", 0))
    processing_error(pCursor, lRet, "Error ExecuteDirect 23");

  printf("ExecuteDirect: 'insert into TDATE values(sysdate + 2)'\n");
  if (lRet = pCursor->ExecuteDirect("insert into TDATE values(sysdate + 2)", 0))
    processing_error(pCursor, lRet, "Error ExecuteDirect 24");

  printf("---------------------------------------------------------\n");
  printf("Pause 20 sec: need to shut down and restart LINTER kernel\n");
  printf("---------------------------------------------------------\n");
#ifdef WIN32
  Sleep(20000);
#else
  sleep(20);
#endif

  printf("Commit\n");
  if (lRet = pConnect->Commit())
    processing_error(pConnect, lRet, "Error Commit");

  printf("ExecuteDirect: 'select D from TDATE'\n");
  if (lRet = pCursor->ExecuteDirect("select D from TDATE", 0))
    processing_error(pCursor, lRet, "Error ExecuteDirect 25");

  printf("GetData1 (as tString)\n");
  if (lRet = pCursor->GetData(1,
                              date,     /* answer bufer */
                              30,       /* length of bufer */
                              tString,  /* type of the answer  */
                              0, 0, NULL))
    processing_error(pCursor, lRet, "Error GetData 21");
  printf("    DATE : %s\n", date);

  printf("Fetch 2\n");
  if (lRet = pCursor->Fetch(toNext, /* travelling direction */
                            0,
                            1,    /* count of required strings  */
                            NULL))
    processing_error(pCursor, lRet, "Error Fetch 22");

  printf("GetData2 (as tString)\n");
  if (lRet = pCursor->GetData(1,
                              date,     /* answer bufer */
                              30,       /* length of bufer */
                              tString,  /* type of the answer  */
                              0, 0, NULL))
    processing_error(pCursor, lRet, "Error GetData 22");
  printf("    DATE : %s\n", date);

  printf("Fetch 3\n");
  if (lRet = pCursor->Fetch(toNext, /* travelling direction */
                            0,
                            1,    /* count of required strings  */
                            NULL))
    processing_error(pCursor, lRet, "Error Fetch 23");

  printf("GetData3 (as tString)\n");
  if (lRet = pCursor->GetData(1,
                              date,     /* answer bufer */
                              30,       /* length of bufer */
                              tString,  /* type of the answer  */
                              0, 0, NULL))
    processing_error(pCursor, lRet, "Error GetData 23");
  printf("    DATE : %s\n", date);

  printf(" ExecuteDirect: 'drop table TDATE'\n");
  if (lRet = pCursor->ExecuteDirect("drop table TDATE", 0))
    processing_error(pCursor, lRet, "Error ExecuteDirect 26", false);

  printf("CloseCursor\n");
  if (lRet = pConnect->CloseCursor(pCursor))
    processing_error(pConnect, lRet, "Error CloseCursor 2");

  printf("\nTest #3: blobs:\n\n");

  printf("Open cursor\n");
  if (lRet = pConnect->OpenCursor(NULL, 0, &pCursor))
    processing_error(pConnect, lRet, "Error open cursor 3");

  printf("ExecuteDirect: 'create table TBLOB(id int, bl blob)'\n");
  if (lRet = pCursor->ExecuteDirect("create or replace table TBLOB(id int, bl blob)", 0))
    processing_error(pCursor, lRet, "Error ExecuteDirect 31");

  printf("ExecuteDirect: 'insert into TBLOB(id, bl) values (1, NULL)'\n");
  if (lRet = pCursor->ExecuteDirect("insert into TBLOB(id, bl) values (1, NULL)", 0))
    processing_error(pCursor, lRet, "Error ExecuteDirect 32");

  lBufLen = 0x8000; /* 32768 */
  memset(&Buf[0], 0x65, lBufLen);
  printf("AddBlob\n");
  if (lRet = pCursor->AddBlob(2, 101, Buf, lBufLen, NULL, NULL))
    processing_error(pCursor, lRet, "Error AddBlob");
  printf("AppendBlob (3 times)\n");
  for (i = 1; i <= 3; i++)
  {
    lBufLen = 0x8000 * (i + 1); /* 32768 * (i + 1) */
    memset(&Buf[0], 0x65 + i, lBufLen);
    if (lRet = pCursor->AppendBlob(101, Buf, lBufLen, NULL, NULL))
      processing_error(pCursor, lRet, "Error AppendBlob 31");
  }

  printf("---------------------------------------------------------\n");
  printf("Pause 20 sec: need to shut down and restart LINTER kernel\n");
  printf("---------------------------------------------------------\n");
#ifdef WIN32
  Sleep(20000);
#else
  sleep(20);
#endif

  lBufLen = 100;
  memset(&Buf[0], 0x69, lBufLen);
  printf("AppendBlob\n");
  if (lRet = pCursor->AppendBlob(101, Buf, lBufLen, NULL, NULL))
    processing_error(pCursor, lRet, "Error AppendBlob 32");

  printf("Commit\n");
  if (lRet = pConnect->Commit())
    processing_error(pConnect, lRet, "Error Commit 3");

  printf("ExecuteDirect: 'select bl from TBLOB'\n");
  if (lRet = pCursor->ExecuteDirect("select bl from TBLOB", 0))
    processing_error(pCursor, lRet, "Error ExecuteDirect 33");

  printf("Fetch row\n");
  if(lRet = pCursor->Fetch(toAbsNumber, 1, 1, NULL, NULL, NULL))
    processing_error(pCursor, lRet, "Error Fetch 31");

  lBufLen = 0x8000;
  printf("Fetch & check blob\n");
  if(lRet = pCursor->FetchBlob(1, lPos, &lBufLen, Buf, NULL, NULL))
    processing_error(pCursor, lRet, "Error Fetch Blob 31");
  lPos += lBufLen;
  CHECK_BUFFER(0x8000, 0x65)

  printf("Fetch & check blob (3 times)\n");
  for (i = 1; i <= 3; i++)
  {
    lBufLen = 0x8000 * (i + 1); /* 32768 * (i + 1) */
    if (lRet = pCursor->FetchBlob(1, lPos, &lBufLen, Buf, NULL, NULL))
      processing_error(pCursor, lRet, "Error Fetch Blob 32");
    CHECK_BUFFER((0x8000 * (i + 1)), (0x65 + i))
    lPos += lBufLen;
  }

  lBufLen = 100;
  printf("Fetch & check blob\n");
  if(lRet = pCursor->FetchBlob(1, lPos, &lBufLen, Buf, NULL, NULL))
    processing_error(pCursor, lRet, "Error Fetch Blob 33");
  lPos += lBufLen;
  CHECK_BUFFER(100, 0x69)

  printf(" ExecuteDirect: 'drop table TBLOB'\n");
  if (lRet = pCursor->ExecuteDirect("drop table TBLOB", 0))
    processing_error(pCursor, lRet, "Error ExecuteDirect 34", false);

  printf("CloseCursor\n");
  if (lRet = pConnect->CloseCursor(pCursor))
    processing_error(pConnect, lRet, "Error CloseCursor 3");

  printf("\nTest #4: parameters:\n\n");

  printf("Open cursor\n");
  if (lRet = pConnect->OpenCursor(NULL, 0, &pCursor))
    processing_error(pConnect, lRet, "Error open cursor 4");

  printf("ExecuteDirect: 'create or replace table TPARAM(p1 int, p2 int, p3 int)'\n");
  if (lRet = pCursor->ExecuteDirect("create or replace table TPARAM(p1 int, p2 int, p3 int)", 0))
    processing_error(pCursor, lRet, "Error ExecuteDirect 41");

  printf("CreateStatement: 'insert into TPARAM(p1, p2, p3) values(?, :p2, ?)'\n");
  if (lRet = pCursor->CreateStatement("insert into TPARAM(p1, p2, p3) values(?, :p2, ?)", 0, &pStat))
    processing_error(pCursor, lRet, "CreateStatement");

  lLen[0] = lLen[1] = lLen[2] = lLen[3] = sizeof(L_LONG);

  printf("BindParameter (3 times)\n");

  /* try to bind dummy parameters */
  if (lRet = pStat->BindParameter(1, NULL, NULL, (L_BYTE*)&lParam[3], 1, 0, tInteger, &lLen[3]))
    processing_error(pCursor, lRet, "BindParameter 01");

  if (lRet = pStat->BindParameter(0, "p2", NULL, (L_BYTE*)&lParam[3], 1, 0, tInteger, &lLen[3]))
    processing_error(pCursor, lRet, "BindParameter 02");

  if (lRet = pStat->BindParameter(3, NULL, NULL, (L_BYTE*)&lParam[3], 1, 0, tInteger, &lLen[3]))
    processing_error(pCursor, lRet, "BindParameter 03");

  lLen[3] = 0xFFFFFFFF;
  lParam[3] = 0xFFFFFFFF;

  /* re-bind correct parameters with te same names/numbers: */

  if (lRet = pStat->BindParameter(1, NULL, NULL, (L_BYTE*)&lParam[0], 1, 0, tInteger, &lLen[0]))
    processing_error(pCursor, lRet, "BindParameter 1");

  if (lRet = pStat->BindParameter(0, "p2", NULL, (L_BYTE*)&lParam[1], 1, 0, tInteger, &lLen[1]))
    processing_error(pCursor, lRet, "BindParameter 2");

  if (lRet = pStat->BindParameter(3, NULL, NULL, (L_BYTE*)&lParam[2], 1, 0, tInteger, &lLen[2]))
    processing_error(pCursor, lRet, "BindParameter 3");

  lParam[0] = 101;
  lParam[1] = 1001;
  lParam[2] = 10001;

  printf("ExecuteStatement (101, 1001, 10001)\n");
  if (lRet = pStat->ExecuteStatement(NULL, NULL, NULL))
    processing_error(pCursor, lRet, "ExecuteStatement 41");

  printf("---------------------------------------------------------\n");
  printf("Pause 20 sec: need to shut down and restart LINTER kernel\n");
  printf("---------------------------------------------------------\n");
#ifdef WIN32
  Sleep(20000);
#else
  sleep(20);
#endif

  lParam[0] = 202;
  lParam[1] = 2002;
  lParam[2] = 20002;

  printf("ExecuteStatement (202, 2002, 20002)\n");
  if (lRet = pStat->ExecuteStatement(NULL, NULL, NULL))
    processing_error(pCursor, lRet, "ExecuteStatement 42");

  printf("---------------------------------------------------------\n");
  printf("Pause 20 sec: need to shut down and restart LINTER kernel\n");
  printf("---------------------------------------------------------\n");
#ifdef WIN32
  Sleep(20000);
#else
  sleep(20);
#endif

  printf("Commit\n");
  if (lRet = pConnect->Commit())
    processing_error(pConnect, lRet, "Error Commit 4");

  printf("ExecuteDirect: 'select p1+p2+p3 from TPARAM'\n");
  if (lRet = pCursor->ExecuteDirect("select p1+p2+p3 from TPARAM", 0))
    processing_error(pCursor, lRet, "Error ExecuteDirect 41");

  printf("GetData 1 (as tInteger)\n");
  if (lRet = pCursor->GetData(1,
                              &lResult,        /* answer bufer */
                              sizeof(L_LONG),  /* length of bufer */
                              tInteger,        /* type of the answer  */
                              0, 0, NULL))
    processing_error(pCursor, lRet, "Error GetData 41");
  if (lResult != 11103)
  {
    printf("Error: buffer damaged 41\n");
    delete pConnect;
    exit(1);
  }
  else
    printf("Ok\n");

  printf("Fetch\n");
  if (lRet = pCursor->Fetch(toNext, /* travelling direction */
                            0,
                            1,    /* count of required strings  */
                            NULL))
    processing_error(pCursor, lRet, "Error Fetch 4");

  printf("GetData 2 (as tInteger)\n");
  if (lRet = pCursor->GetData(1,
                              &lResult,        /* answer bufer */
                              sizeof(L_LONG),  /* length of bufer */
                              tInteger,        /* type of the answer  */
                              0, 0, NULL))
    processing_error(pCursor, lRet, "Error GetData 42");
  if (lResult != 22206)
  {
    printf("Error: buffer damaged 42\n");
    delete pConnect;
    exit(1);
  }
  else
    printf("Ok\n");

  printf("CloseCursor\n");
  if (lRet = pConnect->CloseCursor(pCursor))
    processing_error(pConnect, lRet, "Error CloseCursor 4");

  printf("CloseConnect\n");
  if (lRet = pConnect->CloseConnect())
    processing_error(pConnect, lRet, "Error CloseConnect 2");

  printf("\nTest for buffer overflow: \n");                /* 06.12.11 ... */

  pConnect->SetQBufSettings(4096 /* buffer size */, L_TFALSE /* don't use tmp file */);

  if (lRet = pConnect->Connect("SYSTEM", 0, "MANAGER", 0, NULL, mExclusive))
    processing_error(pConnect, lRet, "ERROR Connect 3");

  printf("Open cursor\n");
  if (lRet = pConnect->OpenCursor(NULL, 0, &pCursor))
    processing_error(pConnect, lRet, "Error open cursor 5");

  printf("ExecuteDirect: 'create or replace table TDATE ( D date )'\n");
  if (lRet = pCursor->ExecuteDirect("create or replace table TDATE (D date)", 0))
    processing_error(pCursor, lRet, "Error ExecuteDirect 51");

  while (1)
  {
    printf("ExecuteDirect: 'insert into TDATE values(sysdate)'\n");
    if (lRet = pCursor->ExecuteDirect("insert into TDATE values(sysdate)", 0))
    {
      L_CHAR  sModuleName[64], sErrorText[1024];
      L_ULONG ulModuleNameLen, ulErrorTextLen;
      L_LONG  lLinApi, lLinter, lSystem, lRet2;

      printf("Correct error about buffer overflow:\n");
      while (pConnect->GetErrorsCount())
      {
        ulModuleNameLen = sizeof(sModuleName);
        ulErrorTextLen = sizeof(sErrorText);
        if ((lRet2 = pConnect->GetError(sModuleName,
                                       &ulModuleNameLen,
                                       sErrorText,
                                       &ulErrorTextLen,
                                       &lLinApi, 
                                       &lLinter, 
                                       &lSystem)) == LINAPI_SUCCESS)
          printf("%s: '%s'\n", sModuleName, sErrorText);
        else
          printf("GetError returned code #%ld\n", lRet2);
      }
      break;
    }
  }

  printf("CloseCursor\n");
  if (lRet = pConnect->CloseCursor(pCursor))
    processing_error(pConnect, lRet, "Error CloseCursor 5");

  printf("CloseConnect\n");
  if (lRet = pConnect->CloseConnect())
    processing_error(pConnect, lRet, "Error CloseConnect 3"); /* ... 06.12.11 */

  delete pConnect;

  printf("----------------- END OF TEST -------------------\n\n");
}

/* function for error processing for LinCPPAPI */
static void processing_error2(linDataSet *pDataSet,
                              L_LONG      ret_cod,
                              L_CHAR     *message,
                              L_BOOL      bExit = L_TTRUE)
{
  L_CHAR  sModuleName[64], sErrorText[1024];
  L_ULONG ulModuleNameLen = sizeof(sModuleName), 
          ulErrorTextLen = sizeof(sErrorText);
  L_LONG  lLinApi, lLinter, lSystem, lRet;

  printf("Return code = %ld (%s)\n", ret_cod, message);
  while (pDataSet->GetErrorsCount())
  {
    if ((lRet = pDataSet->GetError(sModuleName,
                                   &ulModuleNameLen,
                                   sErrorText,
                                   &ulErrorTextLen,
                                   &lLinApi, 
                                   &lLinter, 
                                   &lSystem)) == LINAPI_SUCCESS)
      printf("%s: '%s'\n", sModuleName, sErrorText);
    else
      printf("GetError returned code #%ld\n", lRet);
  }
  if (bExit)
  {
    delete pDataSet;
    exit(1);
  }
}

#define PRINT_DATA_SET \
  for (i = 1; i <= pDataSet->GetRowCount(); i++) \
  { \
    printf("  "); \
    for (j = 1; j <= pDataSet->GetColCount(); j++) \
    { \
      lBufLen = sizeof(Buf); \
      if (pDataSet->GetColInfo(j)->Type == tBlob) \
      { \
        L_LONG lBlobLen; \
        L_LONG lBlobType; \
        if (pDataSet->GetValueLength(i, j, &lBlobLen) == LINAPI_ERROR) \
          processing_error2(pDataSet, 0, "Error GetValueLength"); \
        if (pDataSet->GetBlobType(i, j, &lBlobType) == LINAPI_ERROR) \
          processing_error2(pDataSet, 0, "Error GetValueLength"); \
        if (pDataSet->GetCellData(i, j, tByte, Buf, &lBufLen, &lFlags) == LINAPI_ERROR) \
          processing_error2(pDataSet, 0, "Error GetCellData"); \
        if (lFlags & LDS_CD_NULL) \
          printf("|NULL"); \
        else \
        { \
          printf("|(%ld,%ld)(%.2X)", lBlobLen, lBlobType, Buf[0]); \
          for (k = 0; k < lBlobLen; k++) \
          { \
            if (Buf[k] != Buf[0]) \
            { \
              printf("[Invalid symbol into blob]"); \
              break; \
            } \
          } \
        } \
      } \
      else \
      { \
        if (pDataSet->GetCellData(i, j, tString, Buf, &lBufLen, &lFlags) == LINAPI_ERROR) \
          processing_error2(pDataSet, 0, "Error GetCellData"); \
        if (lFlags & LDS_CD_NULL) \
          printf("|NULL"); \
        else \
          printf("|%s", Buf); \
      } \
    } \
    printf("|\n"); \
  }

/* test for linDataSet */
void testDataSet()
{
  L_LONG         lRet;     /* return code */
  linConnection *pConnect;
  linCursor     *pCursor;
  linDataSet    *pDataSet;
  L_BYTE         Buf[0x20000];
  L_LONG         lBufLen, i, j, k, l, lPos = 1, lFlags;
  t_ParamDesc   *pColInfo;
  L_CHAR        sValue[64];
  L_LONG        lUniqCols[8] = {1};

  printf("----------------- TEST FOR DATA SET FUNCTIONS -------------------\n\n");
  pConnect = new linConnection;
  if (!pConnect)
  {
    printf("No memory\n");
    exit(1);
  }

  /* Set maximum interval for restart LINTER kernel: */
  pConnect->SetMaxTimeout(300000 /* 300 sec = 5 min */);

  if (lRet = pConnect->Connect("SYSTEM", 0, "MANAGER", 0, NULL, mAutocommit))
    processing_error(pConnect, lRet, "ERROR Connect");

  for (l = 0; l < 2; l++)
  {
    printf("\n************************************************************\n");
    if (l == 0)
      printf("Test for linDataSet processing with 'load blob body' mode \n");
    else
      printf("Test for linDataSet processing without 'load blob body' mode \n");
    printf("************************************************************\n\n");
    printf("Open cursor\n");
    if (lRet = pConnect->OpenCursor(NULL, 0, &pCursor))
      processing_error(pConnect, lRet, "Error open cursor");

    printf("ExecuteDirect: 'create table test(i int primary key, j smallint, ch char(20), vch varchar(20), dt date, bl blob)'\n");
    if (lRet = pCursor->ExecuteDirect("create or replace table test(i int primary key, j smallint, ch char(20), vch varchar(20), dt date, bl blob)", 0))
      processing_error(pCursor, lRet, "Error ExecuteDirect 1");

    printf("ExecuteDirect: 'insert ... (1,100,'row1_col2','row1_col3', sysdate, NULL)\n");
    if (lRet = pCursor->ExecuteDirect("insert into test values(1, 100,'row1_col2','row1_col3', sysdate, NULL)", 0))
      processing_error(pCursor, lRet, "Error ExecuteDirect 2");

    printf("ExecuteDirect: 'insert ... (2,200,'row2_col2',NULL, sysdate-1, NULL)\n");
    if (lRet = pCursor->ExecuteDirect("insert into test values(2,200,'row2_col2',NULL, sysdate-1, NULL)", 0))
      processing_error(pCursor, lRet, "Error ExecuteDirect 3");

    lBufLen = 0x8000; /* 32768 */
    memset(&Buf[0], 0x65, lBufLen);
    printf("AddBlob\n");
    if (lRet = pCursor->AddBlob(6, 101, Buf, lBufLen, NULL, NULL))
      processing_error(pCursor, lRet, "Error AddBlob 1");

    printf("ExecuteDirect: 'insert ... (3,300,NULL,'row3_col3', NULL, NULL))\n");
    if (lRet = pCursor->ExecuteDirect("insert into test values(3,300,NULL,'row3_col3', NULL, NULL)", 0))
      processing_error(pCursor, lRet, "Error ExecuteDirect 3");

    lBufLen = 0x4000; /* 16384 */
    memset(&Buf[0], 0x61, lBufLen);
    printf("AddBlob\n");
    if (lRet = pCursor->AddBlob(6, 111, Buf, lBufLen, NULL, NULL))
      processing_error(pCursor, lRet, "Error AddBlob 2");

    printf("CloseCursor\n");
    if (lRet = pConnect->CloseCursor(pCursor))
      processing_error(pConnect, lRet, "Error CloseCursor");

    pDataSet = new linDataSet();
    if (!pDataSet)
    {
      printf("No memory\n");
      exit(1);
    }

    printf("Create DataSet\n");
    if (pDataSet->Create(pConnect, "select i, j, ch, vch, dt, bl from test", 
                         (l == 0) ? 0 : LDS_NLOAD_BLOB_BODY, &lUniqCols[0], 1) == LINAPI_ERROR)
      processing_error2(pDataSet, 0, "Error create DataSet");

    printf("---------------------------------------------------------\n");
    printf("Pause 20 sec: need to shut down and restart LINTER kernel\n");
    printf("---------------------------------------------------------\n");
#ifdef WIN32
    Sleep(20000);
#else
    sleep(20);
#endif

    printf("Information about columns:\n");
    for (i = 1; i <= pDataSet->GetColCount(); i++)
    {
      pColInfo = pDataSet->GetColInfo(i);
      if (!pColInfo)
        processing_error2(pDataSet, 0, "Error GetColInfo");
      else
        printf("column #%ld('%.6s'.'%.6s'.'%.6s'): type %d, len %d, prec %d, scale %d\n", 
                i, pColInfo->Owner, pColInfo->Table, pColInfo->Column,
                pColInfo->Type, pColInfo->Length, pColInfo->Prec, pColInfo->Scale);
    }

    printf("Data in data set:\n");
    PRINT_DATA_SET

    printf("Add row into data set\n");
    if (pDataSet->AddRow(&i) == LINAPI_ERROR)
      processing_error2(pDataSet, 0, "Error AddRow");

    printf("Add data into new row\n");
    if (pDataSet->SetCellData(pDataSet->GetRowCount(), 1, tInteger, &i, sizeof(L_LONG)) == LINAPI_ERROR)
      processing_error2(pDataSet, 0, "Error SetCellData 1");
    i = i * 100;
    if (pDataSet->SetCellData(pDataSet->GetRowCount(), 2, tInteger, &i, sizeof(L_LONG)) == LINAPI_ERROR)
      processing_error2(pDataSet, 0, "Error SetCellData 2");
    sprintf(sValue, "added value1");
    if (pDataSet->SetCellData(pDataSet->GetRowCount(), 3, tString, sValue, strlen(sValue)) == LINAPI_ERROR)
      processing_error2(pDataSet, 0, "Error SetCellData 3");
    sprintf(sValue, "added value2");
    if (pDataSet->SetCellData(pDataSet->GetRowCount(), 4, tString, sValue, strlen(sValue)) == LINAPI_ERROR)
      processing_error2(pDataSet, 0, "Error SetCellData 4");
    if (pDataSet->SetCellData(pDataSet->GetRowCount(), 5, tString, NULL, 0, LDS_CD_NULL) == LINAPI_ERROR)
      processing_error2(pDataSet, 0, "Error SetCellData 5");
    lBufLen = 1024;
    memset(&Buf[0], 0x68, lBufLen);
    if (pDataSet->SetCellData(pDataSet->GetRowCount(), 6, tByte, Buf, lBufLen) == LINAPI_ERROR)
      processing_error2(pDataSet, lRet, "Error SetCellData for add blob");
    if (pDataSet->SetBlobType(pDataSet->GetRowCount(), 6, 123) == LINAPI_ERROR)
      processing_error2(pDataSet, lRet, "Set blob type 1");

    printf("Modify second row\n");
    i = 222;
    if (pDataSet->SetCellData(2, 2, tInteger, &i, sizeof(L_LONG), 0) == LINAPI_ERROR)
      processing_error2(pDataSet, 0, "Error SetCellData 6");

    printf("Delete first row\n");
    if (pDataSet->DeleteRow(1) == LINAPI_ERROR)
      processing_error2(pDataSet, 0, "Error DeleteRow");

    printf("Continue modify second row (it's first row now)\n");
    if (pDataSet->SetCellData(1, 3, tString, NULL, 0, LDS_CD_NULL) == LINAPI_ERROR)
      processing_error2(pDataSet, 0, "Error SetCellData 7");
    sprintf(sValue, "modified value1");
    if (pDataSet->SetCellData(1, 4, tString, sValue, strlen(sValue)) == LINAPI_ERROR)
      processing_error2(pDataSet, 0, "Error SetCellData 8");
    lBufLen = 4096;
    memset(&Buf[0], 0x87, lBufLen);
    if (pDataSet->SetCellData(1, 6, tByte, Buf, lBufLen) == LINAPI_ERROR)
      processing_error2(pDataSet, lRet, "Error SetCellData for modify blob");
    if (pDataSet->SetBlobType(1, 6, -222) == LINAPI_ERROR)
      processing_error2(pDataSet, lRet, "Set blob type 2");

    printf("Modify blob of third row -> NULL (it's second row now)\n");
    if (pDataSet->SetCellData(2, 6, tByte, NULL, 0, LDS_CD_NULL) == LINAPI_ERROR)
      processing_error2(pDataSet, lRet, "Error SetCellData for modify blob");

    printf("Modified data set:\n");
    PRINT_DATA_SET

    printf("Save changes into database\n");
    if (pDataSet->SaveChanges() == LINAPI_ERROR)
      processing_error2(pDataSet, 0, "Error save changes");
    
    printf("Close DataSet\n");
    if (pDataSet->Close() == LINAPI_ERROR)
      processing_error2(pDataSet, 0, "Error close DataSet");

    if (pDataSet->Create(pConnect, "select i, j, ch, vch, dt, bl from test") == LINAPI_ERROR)
      processing_error2(pDataSet, 0, "Error create DataSet 2");

    printf("Check changes in data set:\n");
    PRINT_DATA_SET

    printf("Close DataSet\n");
    if (pDataSet->Close() == LINAPI_ERROR)
      processing_error2(pDataSet, 0, "Error close DataSet2");
  }

  printf("\n************************************************************\n");
  printf("Test for indexes, aggregate functions, sorting and search\n");
  printf("************************************************************\n\n");

  printf("Open cursor\n");
  if (lRet = pConnect->OpenCursor(NULL, 0, &pCursor))
    processing_error(pConnect, lRet, "Error open cursor 2");

  printf("ExecuteDirect: 'create table test(id int, i int, j int, vch varchar(20))'\n");
  if (lRet = pCursor->ExecuteDirect("create or replace table test(id int, i int, j int, vch varchar(20))", 0))
    processing_error(pCursor, lRet, "Error ExecuteDirect 2_1");

  printf("ExecuteDirect: 'insert ... (100,2,1,'c')\n");
  if (lRet = pCursor->ExecuteDirect("insert into test values(100,2,1,'c')", 0))
    processing_error(pCursor, lRet, "Error ExecuteDirect 2_2");

  printf("ExecuteDirect: 'insert ... (200,1,1,NULL,'b')\n");
  if (lRet = pCursor->ExecuteDirect("insert into test values(200,1,1,NULL)", 0))
    processing_error(pCursor, lRet, "Error ExecuteDirect 2_3");

  printf("ExecuteDirect: 'insert ... (300,2,NULL,'b')\n");
  if (lRet = pCursor->ExecuteDirect("insert into test values(300,2,NULL,'b')", 0))
    processing_error(pCursor, lRet, "Error ExecuteDirect 2_4");

  printf("ExecuteDirect: 'insert ... (400,2,1,'a')\n");
  if (lRet = pCursor->ExecuteDirect("insert into test values(400,2,1,'a')", 0))
    processing_error(pCursor, lRet, "Error ExecuteDirect 2_5");

  printf("ExecuteDirect: 'insert ... (500,1,2,'e')\n");
  if (lRet = pCursor->ExecuteDirect("insert into test values(500,1,2,'e')", 0))
    processing_error(pCursor, lRet, "Error ExecuteDirect 2_6");

  printf("CloseCursor\n");
  if (lRet = pConnect->CloseCursor(pCursor))
    processing_error(pConnect, lRet, "Error CloseCursor");

  printf("Create DataSet\n");
  if (pDataSet->Create(pConnect, "select id, i, j, vch from test") == LINAPI_ERROR)
    processing_error2(pDataSet, 0, "Error create DataSet");

  printf("Unsorted data:\n");
  PRINT_DATA_SET

  printf("Create index on column 2\n");
  if (pDataSet->CreateIndex(2) == LINAPI_ERROR)
    processing_error2(pDataSet, 0, "Error create index 1");

  if (pDataSet->AddSortColumn(2) == LINAPI_ERROR)
    processing_error2(pDataSet, 0, "Error add sort column 1");

  if (pDataSet->Sort() == LINAPI_ERROR)
    processing_error2(pDataSet, 0, "Error sort");

  printf("Sorted data by column 2 (asc):\n");
  PRINT_DATA_SET

  if (pDataSet->FreeSort() == LINAPI_ERROR)
    processing_error2(pDataSet, 0, "Error free sort");

  if (pDataSet->AddSortColumn(2, linDataSet::sDesc) == LINAPI_ERROR)
    processing_error2(pDataSet, 0, "Error add sort column 2");

  if (pDataSet->Sort() == LINAPI_ERROR)
    processing_error2(pDataSet, 0, "Error sort");

  printf("Sorted data by column 2 (desc):\n");
  PRINT_DATA_SET

  if (pDataSet->FreeSort() == LINAPI_ERROR)
    processing_error2(pDataSet, 0, "Error free sort");

  printf("Create index on column 3\n");
  if (pDataSet->CreateIndex(3) == LINAPI_ERROR)
    processing_error2(pDataSet, 0, "Error create index 2");

  printf("Create index on column 4\n");
  if (pDataSet->CreateIndex(4) == LINAPI_ERROR)
    processing_error2(pDataSet, 0, "Error create index 3");

  if (pDataSet->AddSortColumn(2) == LINAPI_ERROR)
    processing_error2(pDataSet, 0, "Error add sort column 3");

  if (pDataSet->AddSortColumn(3) == LINAPI_ERROR)
    processing_error2(pDataSet, 0, "Error add sort column 4");

  if (pDataSet->AddSortColumn(4) == LINAPI_ERROR)
    processing_error2(pDataSet, 0, "Error add sort column 5");

  if (pDataSet->Sort() == LINAPI_ERROR)
    processing_error2(pDataSet, 0, "Error sort");

  printf("Sorted data by column 2,3,4:\n");
  PRINT_DATA_SET

  if (pDataSet->FreeSort() == LINAPI_ERROR)
    processing_error2(pDataSet, 0, "Error free sort");

  if (pDataSet->AddSortColumn(2, linDataSet::sDesc) == LINAPI_ERROR)
    processing_error2(pDataSet, 0, "Error add sort column 3");

  if (pDataSet->AddSortColumn(3, linDataSet::sDesc) == LINAPI_ERROR)
    processing_error2(pDataSet, 0, "Error add sort column 4");

  if (pDataSet->AddSortColumn(4, linDataSet::sDesc) == LINAPI_ERROR)
    processing_error2(pDataSet, 0, "Error add sort column 5");

  if (pDataSet->Sort() == LINAPI_ERROR)
    processing_error2(pDataSet, 0, "Error sort");

  printf("Sorted data by column 2(desc),3(desc),4(desc):\n");
  PRINT_DATA_SET

  printf("Delete 2nd row\n");
  if (pDataSet->DeleteRow(2) == LINAPI_ERROR)
    processing_error2(pDataSet, 0, "Error DeleteRow");

  if (pDataSet->FreeSort() == LINAPI_ERROR)
    processing_error2(pDataSet, 0, "Error free sort");

  if (pDataSet->AddSortColumn(2, linDataSet::sDesc) == LINAPI_ERROR)
    processing_error2(pDataSet, 0, "Error add sort column 3");

  if (pDataSet->AddSortColumn(3, linDataSet::sDesc) == LINAPI_ERROR)
    processing_error2(pDataSet, 0, "Error add sort column 4");

  if (pDataSet->AddSortColumn(4) == LINAPI_ERROR)
    processing_error2(pDataSet, 0, "Error add sort column 5");

  printf("Add row into data set\n");
  if (pDataSet->AddRow(&i) == LINAPI_ERROR)
    processing_error2(pDataSet, 0, "Error AddRow");

  printf("Add data into new row\n");
  i = (i + 1) * 100;
  if (pDataSet->SetCellData(pDataSet->GetRowCount(), 1, tInteger, &i, sizeof(L_LONG)) == LINAPI_ERROR)
    processing_error2(pDataSet, 0, "Error SetCellData 2_1");
  i = 2;
  if (pDataSet->SetCellData(pDataSet->GetRowCount(), 2, tInteger, &i, sizeof(L_LONG)) == LINAPI_ERROR)
    processing_error2(pDataSet, 0, "Error SetCellData 2_2");
  i = 1;
  if (pDataSet->SetCellData(pDataSet->GetRowCount(), 3, tInteger, &i, sizeof(L_LONG)) == LINAPI_ERROR)
    processing_error2(pDataSet, 0, "Error SetCellData 2_3");
  sprintf(sValue, "0");
  if (pDataSet->SetCellData(pDataSet->GetRowCount(), 4, tString, sValue, strlen(sValue)) == LINAPI_ERROR)
    processing_error2(pDataSet, 0, "Error SetCellData 2_4");

  if (pDataSet->Sort() == LINAPI_ERROR)
    processing_error2(pDataSet, 0, "Error sort");

  printf("Sorted data by column 2(desc),3(desc),4:\n");
  PRINT_DATA_SET
 
  printf("Free sort (index for Find may be used without sorting only):\n");
  if (pDataSet->FreeSort() == LINAPI_ERROR)
    processing_error2(pDataSet, 0, "Error free sort");

  printf("\nFind by index [for column3 = 1]:\n");
  if (pDataSet->AddCondition(3, tString, (void *)"1", 1) == LINAPI_ERROR)
    processing_error2(pDataSet, 0, "Error add condition");
  if (pDataSet->Find() == LINAPI_ERROR)
    processing_error2(pDataSet, 0, "Error 'find'");  
  printf("The found columns:\n");
  PRINT_DATA_SET
  if (pDataSet->FreeFind() == LINAPI_ERROR)
    processing_error2(pDataSet, 0, "Error free find");

  printf("\nFind by index [column3 <= 2] and [column4 is not NULL]:\n");
  if (pDataSet->AddCondition(3, tString, (void *)"2", 1, linDataSet::fdLE) == LINAPI_ERROR)
    processing_error2(pDataSet, 0, "Error add condition");
  if (pDataSet->AddCondition(4, 0, NULL, 0, linDataSet::fdNNL) == LINAPI_ERROR)
    processing_error2(pDataSet, 0, "Error add condition");
  if (pDataSet->Find() == LINAPI_ERROR)
    processing_error2(pDataSet, 0, "Error 'find'");  
  printf("The found columns:\n");
  PRINT_DATA_SET
  if (pDataSet->FreeFind() == LINAPI_ERROR)
    processing_error2(pDataSet, 0, "Error free find");

  printf("\nAggregate functions for indexed column 3:\n");
  lBufLen = sizeof(Buf);
  if (pDataSet->GetAggregate(3, linDataSet::gaMin, tString, Buf, &lBufLen, &lFlags) == LINAPI_ERROR)
    processing_error2(pDataSet, 0, "GetAggregate - min");
  printf("MIN   for 3 column: %s\n", Buf);

  lBufLen = sizeof(Buf);
  if (pDataSet->GetAggregate(3, linDataSet::gaMax, tString, Buf, &lBufLen, &lFlags) == LINAPI_ERROR)
    processing_error2(pDataSet, 0, "GetAggregate - max");
  printf("MAX   for 3 column: %s\n", Buf);
    
  lBufLen = sizeof(Buf);
  if (pDataSet->GetAggregate(3, linDataSet::gaSum, tString, Buf, &lBufLen, &lFlags) == LINAPI_ERROR)
    processing_error2(pDataSet, 0, "GetAggregate - sum");
  printf("SUM   for 3 column: %s\n", Buf);

  lBufLen = sizeof(Buf);
  if (pDataSet->GetAggregate(3, linDataSet::gaAvg, tString, Buf, &lBufLen, &lFlags) == LINAPI_ERROR)
    processing_error2(pDataSet, 0, "GetAggregate - avg");
  printf("AVG   for 3 column: %s\n", Buf);

  lBufLen = sizeof(Buf);
  if (pDataSet->GetAggregate(3, linDataSet::gaCount, tString, Buf, &lBufLen, &lFlags) == LINAPI_ERROR)
    processing_error2(pDataSet, 0, "GetAggregate - count");
  printf("COUNT for 3 column: %s\n", Buf);
  printf("\n");

  printf("Drop index on column 3\n");
  if (pDataSet->DropIndex(3) == LINAPI_ERROR)
    processing_error2(pDataSet, 0, "Error drop index 1");

  printf("\nFind without index [column3 = 1]:\n");
  if (pDataSet->AddCondition(3, tString, (void *)"1", 1) == LINAPI_ERROR)
    processing_error2(pDataSet, 0, "Error add condition");
  if (pDataSet->Find() == LINAPI_ERROR)
    processing_error2(pDataSet, 0, "Error 'find'");  
  printf("The found columns:\n");
  PRINT_DATA_SET

  printf("\nSort into found data by column 4:\n");
  if (pDataSet->AddSortColumn(4) == LINAPI_ERROR)
    processing_error2(pDataSet, 0, "Error add sort column 6");

  if (pDataSet->Sort() == LINAPI_ERROR)
    processing_error2(pDataSet, 0, "Error sort");

  printf("Sorted found data by column 4:\n");
  PRINT_DATA_SET

  if (pDataSet->FreeSort() == LINAPI_ERROR)
    processing_error2(pDataSet, 0, "Error free sort");

  printf("\nFind in Find [column2 = 2] and [column4 = 'a']:\n");
  if (pDataSet->AddCondition(2, tString, (void *)"2", 1) == LINAPI_ERROR)
    processing_error2(pDataSet, 0, "Error add condition");
  if (pDataSet->AddCondition(4, tString, (void *)"a", 1) == LINAPI_ERROR)
    processing_error2(pDataSet, 0, "Error add condition");
  if (pDataSet->Find() == LINAPI_ERROR)
    processing_error2(pDataSet, 0, "Error 'find'");  
  printf("The found columns:\n");
  PRINT_DATA_SET
  if (pDataSet->FreeFind() == LINAPI_ERROR)
    processing_error2(pDataSet, 0, "Error free find");

  printf("\nAggregate functions for non-indexed column 3:\n");
  lBufLen = sizeof(Buf);
  if (pDataSet->GetAggregate(3, linDataSet::gaMin, tString, Buf, &lBufLen, &lFlags) == LINAPI_ERROR)
    processing_error2(pDataSet, 0, "GetAggregate - min");
  printf("MIN   for 3 column: %s\n", Buf);

  lBufLen = sizeof(Buf);
  if (pDataSet->GetAggregate(3, linDataSet::gaMax, tString, Buf, &lBufLen, &lFlags) == LINAPI_ERROR)
    processing_error2(pDataSet, 0, "GetAggregate - max");
  printf("MAX   for 3 column: %s\n", Buf);
    
  lBufLen = sizeof(Buf);
  if (pDataSet->GetAggregate(3, linDataSet::gaSum, tString, Buf, &lBufLen, &lFlags) == LINAPI_ERROR)
    processing_error2(pDataSet, 0, "GetAggregate - sum");
  printf("SUM   for 3 column: %s\n", Buf);

  lBufLen = sizeof(Buf);
  if (pDataSet->GetAggregate(3, linDataSet::gaAvg, tString, Buf, &lBufLen, &lFlags) == LINAPI_ERROR)
    processing_error2(pDataSet, 0, "GetAggregate - avg");
  printf("AVG   for 3 column: %s\n", Buf);

  lBufLen = sizeof(Buf);
  if (pDataSet->GetAggregate(3, linDataSet::gaCount, tString, Buf, &lBufLen, &lFlags) == LINAPI_ERROR)
    processing_error2(pDataSet, 0, "GetAggregate - count");
  printf("COUNT for 3 column: %s\n", Buf);
  printf("\n");

  printf("Close DataSet\n");
  if (pDataSet->Close() == LINAPI_ERROR)
    processing_error2(pDataSet, 0, "Error close DataSet 2");

  delete pDataSet;

  printf("CloseConnect\n");
  if (lRet = pConnect->CloseConnect())
    processing_error(pConnect, lRet, "Error CloseConnect");

  delete pConnect;

  printf("----------------- END OF TEST -------------------\n\n");
}

void AstFunction(L_SWORD swCursor, L_LONG lErr, void *pArg)
{
  if (*(L_LONG*)pArg == 1)
    printf("AstFunction, cursor #%d, code #%d ('0' is correct value)\n", swCursor, lErr);
  else
    printf("AstFunction, cursor #%d, code #%d ('14000' is correct value)\n", swCursor, lErr);
  *(L_LONG*)pArg += 100; /* operation is complete */
  return;
}

void testAsync(L_LONG lMode)
{
  L_LONG         lRet;     /* return code */
  L_LONG         lIsComplete = lMode;
  linConnection *pConnect;

  if (lMode == 2)
  {
    printf("----------------- TEST FOR ASYNC MODE W/O LINTER KERNEL -------------------\n\n");
    printf("---------------------------------------------\n");
    printf("Pause 20 sec: need to shut down LINTER kernel\n");
    printf("---------------------------------------------\n");
#ifdef WIN32
    Sleep(20000);
#else
    sleep(20);
#endif
  }
  else
    printf("----------------- TEST FOR ASYNC MODE WITH LINTER KERNEL -------------------\n\n");

  pConnect = new linConnection;
  if (!pConnect)
  {
    printf("No memory\n");
    exit(1);
  }

  printf("Try to open connect\n");
  lRet = pConnect->Connect("SYSTEM", 0, "MANAGER", 0, NULL, mAutocommit, (void *)AstFunction, (void*)&lIsComplete);
  if (lRet != LINAPI_SUCCESS)
  {
    if (lMode == 1)
      processing_error(pConnect, lRet, "ERROR Connect");
    else
      printf("Connect returned error, return value = #%d ('-1' is correct value)\n", lRet);
  }

  printf("Wait for async function\n");
  while (lIsComplete < 100)
  {
#ifdef WIN32
    Sleep(100);
#else
    sleep(1);
#endif
  }
  printf("End wait\n");
  if (lMode == 1 && (lRet = pConnect->CloseConnect()))
    processing_error(pConnect, lRet, "Error CloseConnect");
  delete pConnect;
  if (lMode == 2)
  {
    printf("----------------------------------------- \n");
    printf("Pause 20 sec: need to start LINTER kernel\n");
    printf("-----------------------------------------\n");
#ifdef WIN32
    Sleep(20000);
#else
    sleep(20);
#endif
  }
  printf("----------------- END OF TEST -------------------\n\n");
}

void testAsyncBind()
{
  linConnection *pConnect;
  linCursor     *pCursor;
  linStatement  *pStat;
  L_LONG         lLen, lRet, lError[3] = {0,0,0};
  L_SWORD        swComplete = 0;
  L_CHAR        *str="TEST";                               /* 04.06.14 */

  printf("----------------- TEST FOR BIND IN ASYNC MODE -------------------\n\n");
  pConnect = new linConnection;
  if (!pConnect)
  {
    printf("No memory\n");
    exit(1);
  }

  printf("Async connect\n");
  if (lRet = pConnect->Connect("SYSTEM", 0, "MANAGER", 0, NULL, mAutocommit, NULL, (void*)&swComplete))
    processing_error(pConnect, lRet, "ERROR Connect");

  do
  {
#ifdef WIN32
    Sleep(100);
#else
    sleep(1);
#endif
    lRet = pConnect->ConnectComplete(&swComplete, &lError[0], &lError[1], &lError[2]);
  }
  while (lRet == LINAPI_SUCCESS && swComplete == 0);

  if (lRet != LINAPI_SUCCESS)
    processing_error(pConnect, lRet, "Error Connect 2");
  else 
  if (lError[0] || lError[1] || lError[2])
  {
    printf("Error Connect 3, apierror #%d, linerror #%d, syserror #%d", lError[0], lError[1], lError[2]);
    delete pConnect;
    exit(1);
  }

  printf("Open cursor\n");
  if (lRet = pConnect->OpenCursor(NULL, 0, &pCursor))
    processing_error(pConnect, lRet, "Error open cursor");

  printf("Create statement\n");
  if (lRet = pCursor->CreateStatement("select count(*) from SYSTEM.$$$USR where $$$S34=?;", 0, &pStat))
    processing_error(pCursor, lRet, "CreateStatement");

  printf("Bind parameter\n");
  lLen = 4;
  if (lRet = pStat->BindParameter(1, NULL, NULL, (L_BYTE*)str, 1, 4, tChar, &lLen))
    processing_error(pCursor, lRet, "BindParameter");

  printf("Execute statement\n");
  if (lRet = pStat->ExecuteStatement(NULL, NULL, NULL))
    processing_error(pCursor, lRet, "ExecuteStatement");

  printf("Close cursor\n");
  if (lRet = pConnect->CloseCursor(pCursor))
    processing_error(pConnect, lRet, "Error CloseCursor");

  printf("Close connect\n");
  if (lRet = pConnect->CloseConnect())
    processing_error(pConnect, lRet, "Error CloseConnect");
  delete pConnect;
  printf("----------------- END OF TEST -------------------\n\n");
}

#ifdef WINDOWS_NT

#define THR_COUNT   3
#define QUERY_COUNT 160

class linThrClass
{
  private:
  L_LONG m_lThrNumber /* 0-based */;
  L_LONG m_lRepeatCount;
  L_BOOL m_bDoneFlag;

  public:
  linThrClass(L_LONG lThrNumber) { m_lThrNumber = lThrNumber; m_pCursor = NULL; m_lRepeatCount = QUERY_COUNT; m_hThrd = NULL; m_dwPid = 0; m_bDoneFlag = L_TFALSE; }
  ~linThrClass() {}

  linCursor *m_pCursor;
  HANDLE m_hThrd;
  DWORD  m_dwPid;
  L_BOOL GetDoneFlag()            { return m_bDoneFlag; }
  void   SetDoneFlag(L_BOOL bSet) { m_bDoneFlag = bSet; }

  static DWORD WINAPI StartThread(LPVOID pArg)
  {
    L_LONG lRet /* return code */, i = 0, lLen;
    linStatement *pStat;
    L_CHAR sQuery1[64], sQuery2[64], sMessage[128], sParam[32];
    linThrClass *pThrClass = (linThrClass*)pArg;

    sprintf(sQuery1, "select 't%d'", pThrClass->m_lThrNumber + 1);
    sprintf(sQuery2, "select 't%d_s',? (char(10))", pThrClass->m_lThrNumber + 1);
    while (pThrClass->m_lRepeatCount > 0)
    {
      i++;
      if ((lRet = pThrClass->m_pCursor->ExecuteDirect(sQuery1, 0)) != LINAPI_SUCCESS)
      {
        sprintf(sMessage, "Error ExecuteDirect %d", pThrClass->m_lThrNumber + 1);
        processing_error(pThrClass->m_pCursor, lRet, sMessage, L_TFALSE);
      }
      else
      if ((lRet = pThrClass->m_pCursor->CreateStatement(sQuery2, 0, &pStat)) != LINAPI_SUCCESS)
      {
        sprintf(sMessage, "Error CreateStatement %d", pThrClass->m_lThrNumber + 1);
        processing_error(pThrClass->m_pCursor, lRet, sMessage, L_TFALSE);
      }
      else
      {
        sprintf (sParam,"p#%.2d#%.3d", pThrClass->m_lThrNumber + 1, i);
        lLen = strlen(sParam);
        if ((lRet = pStat->BindParameter(1, NULL, NULL, (L_BYTE*)sParam, 1, lLen, tChar, &lLen)) != LINAPI_SUCCESS)
        {
          sprintf(sMessage, "Error BindParameter %d", pThrClass->m_lThrNumber + 1);
          processing_error(pThrClass->m_pCursor, lRet, sMessage, L_TFALSE);
        }
        if ((lRet = pStat->ExecuteStatement(NULL, NULL, NULL)) != LINAPI_SUCCESS)
        {
          sprintf(sMessage, "Error ExecuteStatement %d", pThrClass->m_lThrNumber + 1);
          processing_error(pThrClass->m_pCursor, lRet, sMessage, L_TFALSE);
        }
        if ((lRet = pThrClass->m_pCursor->FreeStatement(pStat)) != LINAPI_SUCCESS)
        {
          sprintf(sMessage, "Error FreeStatement %d", pThrClass->m_lThrNumber + 1);
          processing_error(pThrClass->m_pCursor, lRet, sMessage, L_TFALSE);
        }
      }
      printf("%d#%d ", pThrClass->m_lThrNumber + 1, i);
      pThrClass->m_lRepeatCount--;
      Sleep(500);
    }
    printf("\nThread#%d finished \n", pThrClass->m_lThrNumber + 1);
    pThrClass->SetDoneFlag(L_TTRUE);
    return 0;
  }
};

void testThreads()
{
  linConnection *pConnect;
  L_LONG i, lRet /* return code */;
  linThrClass *pThrClass[THR_COUNT];

  printf("----------------- TEST FOR THREADS -------------------\n\n");
  pConnect = new linConnection;
  if (!pConnect)
  {
    printf("No memory\n");
    exit(1);
  }

  pConnect->SetUseConnQBuf(L_TFALSE);

  if (lRet = pConnect->ConnectCS("SYSTEM", 0, "MANAGER", 0, NULL, NULL, /*mAutocommit*/mExclusive))
    processing_error(pConnect, lRet, "ERROR Connect");

  for (i = 0; i < THR_COUNT; i++)
  {
    if ((pThrClass[i] = new linThrClass(i)) == NULL)
    {
      printf("No memory for class allocation\n");
      exit(1);
    }    
    printf("Open cursor %d\n", i);
    if (lRet = pConnect->OpenCursor(NULL, 0, &pThrClass[i]->m_pCursor))
      processing_error(pConnect, lRet, "Error open cursor");
    pThrClass[i]->m_hThrd = CreateThread(NULL, 524288L, (LPTHREAD_START_ROUTINE)pThrClass[i]->StartThread, (LPVOID)pThrClass[i], 0L, &pThrClass[i]->m_dwPid);
  }
  printf("------------------------------\n"
         "Threads started (pause 10 sec)\n"
         "------------------------------\n");
  Sleep(10000);
  printf("\n---------------------------------------------\n"
         "Pause 10 sec: need to shut down LINTER kernel\n"
         "---------------------------------------------\n");
  Sleep(10000);
  printf("\n-----------------------------------------\n"
         "Pause 10 sec: need to start LINTER kernel\n"
         "-----------------------------------------\n");
  Sleep(10000);
  printf("\n-----------------------------------------------------------------------\n"
         "Pause 60 sec: need to shut down LINTER kernel and wait more than 30 sec\n"
         "-----------------------------------------------------------------------\n");
  Sleep(60000);
  printf("\n-----------------------------------------\n"
         "Pause 10 sec: need to start LINTER kernel\n"
         "-----------------------------------------\n");
  Sleep(10000);
  if (lRet = pConnect->ReConnect("SYSTEM", 0, "MANAGER", 0, NULL, NULL, /*mAutocommit*/mExclusive))
    processing_error(pConnect, lRet, "ERROR ReConnect");
  printf("\n-----------------------------\n"
         "Wait after re-connect ...\n"
         "-----------------------------\n");
  /* cycle for waiting for completion of threads: */
  for (i = 0; i < THR_COUNT; i++)
    while (!pThrClass[i]->GetDoneFlag())
      Sleep(1000);
  /* free threads and variables: */
  for (i = 0; i < THR_COUNT; i++)
  {
    printf("CloseCursor %d\n", i);
    if (lRet = pConnect->CloseCursor(pThrClass[i]->m_pCursor))
      processing_error(pConnect, lRet, "Error CloseCursor");
    TerminateThread(pThrClass[i]->m_hThrd, 0L);
    delete pThrClass[i];
    pThrClass[i] = NULL;
  }
  printf("\n");
  printf("----------------- END OF TEST -------------------\n\n");
} 
#endif

#ifndef WINCE                                              /* 17.04.13 ... */
/* buffer for environment variable must be global: */
#define CONFIG_FILE_NAME    "config.lin"
#define CONFIG_CONNECT_TYPE "1"
#if defined(WIN32)                                         /* 04.06.14a ... */
#define CONFIG_LINTER_MBX   ""
#elif defined(UNIX)
#define CONFIG_LINTER_MBX   "20561"
#elif defined(QNX) || defined(VXWORKS)
#define CONFIG_LINTER_MBX   "linter"
#else
OS UNDEFINED
#endif                                                     /* ... 04.06.14a */
L_CHAR sEnvVal1[64], sEnvVal2[64];

void testConnectEnv()
{
  L_LONG         lRet /* return code */, i;
  L_CHAR         date[30]  /* date as a string */, sName[16], sStr[64];
  linConnection *pConnect;
  linCursor     *pCursor;

  printf("----------------- TEST FOR ConnectEnv function ------------------\n\n");
  for (i = 1; i <= 2; i++)
  {
    printf("PASS #%d, ", i);

    switch (i)
    {
      case 1:  strcpy(sName, "LINTER01"); break;
      case 2:  strcpy(sName, "LINTER"); break;
    }
    printf("CfgName = '%s'\n", sName);
    sprintf(sEnvVal1, "CONNECT_%s=%s:%s", sName, CONFIG_CONNECT_TYPE, CONFIG_LINTER_MBX);
#ifdef WIN32
    if (_putenv(sEnvVal1) < 0)
#else
    if (putenv(sEnvVal1) < 0)
#endif
    {
      printf("putenv error #1\n");
      exit(1);
    }
    sprintf(sEnvVal2, "CFG_%s=%s", sName, CONFIG_FILE_NAME);
#ifdef WIN32
    if (_putenv(sEnvVal2) < 0)
#else
    if (putenv(sEnvVal2) < 0)
#endif
    {
      printf("putenv error #2\n");
      exit(1);
    }  
    switch (i)
    {
      case 1:  break;
      case 2:  sName[0]='\0' /* test for default config name LINTER */; break;
    }

    pConnect = new linConnection;
    if (!pConnect)
    {
      printf("No memory\n");
      exit(1);
    }

    printf("Open cursor\n");
    if (lRet = pConnect->ConnectEnv(sName))
      processing_error(pConnect, lRet, "ERROR ConnectEnv");

    printf("Open cursor\n");
    if (lRet = pConnect->OpenCursor(NULL, 0, &pCursor))
      processing_error(pConnect, lRet, "Error open cursor");

    printf("ExecuteDirect: 'select to_date('17.04.2013','DD.MM.YYYY')'\n");
    if (lRet = pCursor->ExecuteDirect("select to_date('17.04.2013','DD.MM.YYYY')", 0))
      processing_error(pCursor, lRet, "Error ExecuteDirect");

    printf("GetData (as tString)\n");
    if (lRet = pCursor->GetData(1,
                                date,     /* answer bufer */
                                30,       /* length of bufer */
                                tString,  /* type of the answer  */
                                0, 0, NULL))
      processing_error(pCursor, lRet, "Error GetData");

    printf("    DATE : %s\n", date);

    printf("CloseCursor\n");
    if (lRet = pConnect->CloseCursor(pCursor))
      processing_error(pConnect, lRet, "Error CloseCursor");

    printf("CloseConnect\n");
    if (lRet = pConnect->CloseConnect())
      processing_error(pConnect, lRet, "Error CloseConnect");

    delete pConnect;
  }

  printf("TEST FOR ERROR CODES 14109-14112 + 14114, BEGIN\n");

  pConnect = new linConnection;
  if (!pConnect)
  {
    printf("No memory\n");
    exit(1);
  }
  /* 14109: */
  if (lRet = pConnect->ConnectEnv("01234567890123456789012345678901234567890123456789012345678", 0)) /* maximum is 58 symbols */
    processing_error(pConnect, lRet, "Correct error for ConnectEnv", false);
  /* 14110: */
  if (lRet = pConnect->ConnectEnv("ABC$$$"))
    processing_error(pConnect, lRet, "Correct error for ConnectEnv", false);
  /* 14111: */
#ifdef WIN32
  if (_putenv("CONNECT_LINTER=1:0123456789012345678901234567890123456789012345678901234567890123456") < 0)
#else
  if (putenv("CONNECT_LINTER=1:0123456789012345678901234567890123456789012345678901234567890123456") < 0)
#endif
  {
    printf("putenv error #1\n");
    exit(1);
  }
#ifdef WIN32
  if (_putenv("CFG_LINTER=$dummy$.lin") < 0)
#else
  if (putenv("CFG_LINTER=$dummy$.lin") < 0)
#endif
  {
    printf("putenv error #2\n");
    exit(1);
  }  
  if (lRet = pConnect->ConnectEnv(""))
    processing_error(pConnect, lRet, "Correct error for ConnectEnv", false);
  /* 14112: */
#ifdef WIN32
  if (_putenv("CONNECT_LINTER=10:") < 0)
#else
  if (putenv("CONNECT_LINTER=10:") < 0)
#endif
  {
    printf("putenv error #1\n");
    exit(1);
  }
  if (lRet = pConnect->ConnectEnv())
    processing_error(pConnect, lRet, "Correct error for ConnectEnv", false);
  /* 14114: */
#ifdef WIN32
  if (_putenv("CONNECT_LINTER=2:AAA") < 0)
#else
  if (putenv("CONNECT_LINTER=2:AAA") < 0)
#endif
  {
    printf("putenv error #1\n");
    exit(1);
  }
#ifdef WIN32
  if (_putenv("CFG_LINTER=$dummy$.lin") < 0)
#else
  if (putenv("CFG_LINTER=$dummy$.lin") < 0)
#endif
  {
    printf("putenv error #2\n");
    exit(1);
  }  
  if (lRet = pConnect->ConnectEnv())
    processing_error(pConnect, lRet, "Correct error for ConnectEnv", false);
  delete pConnect;

  printf("TEST FOR ERROR CODES, END\n");

  printf("----------------- END OF TEST -------------------\n\n");
}
#endif                                                     /* ... 17.04.13 */

#if defined(VXWORKS)
MainStart(apidata, 32 * 1024, UninitLinterClient)
#elif defined(WINCE)
int cppapi()
#else
int main(void)
#endif
{
#ifndef WINCE                                              /* 17.04.13 ... */
  testConnectEnv();
#endif                                                     /* ... 17.04.13 */
#ifdef WINDOWS_NT
  testThreads();
#endif
  testAsyncBind();
  testAsync(1);
  testAsync(2);
  testApi();
  testDataSet();
  return 0;
}
