/****************************************************************************
 *                 RELEX, Inc. Linter DBMS source code                      *
 *              This is public source code of RELEX, Inc.                   *
 *                                                                          *
 *      You have a royalty-free right to use, modify, reproduce and         *
 *     distribute the Sample Files (and/or any modified version) in         *
 *        any way you find useful, provided that you agree that             *
 *       RELEX has no warranty obligations or liability for any             *
 *           Sample Application Files which are modified.                   *
 *                                                                          *
 *                 Copyright (c) 2000-2010 RELEX Inc.                       *
 *                                                                          *
 *                   This software was developed by                         *
 *                                                                          *
 *                            RELEX, Inc.                                   *
 *                         Voronezh, Russia                                 *
 *                                                                          *
 *                        All rights reserved.                              *
 ****************************************************************************
 AUTHOR: Gluhov
 ----------------------------------------------------------------------------
 PURPOSE: linqt sample. basic operations.
 30.07.13 -- Galizin(Set transaction mode)
 ----------------------------------------------------------------------------
*/

#include <iostream>

#include <qapplication.h>
#include <qsqldatabase.h>
#include <qsqlerror.h>
#include <qdatetime.h>

#if QT_VERSION < 0x040000
#  include <qsqlcursor.h>
#  include <qdatatable.h>
#  define QS2C(qstr) qstr.local8Bit().data()
#  define QBYTEARRAY(str, len) QByteArray().duplicate( str, len ) 
#else
#  include <QtSql>
#  include <QTableView>
#  include <QSqlTableModel>
#  define QS2C(qstr) qstr.toLocal8Bit().data()
#  define QBYTEARRAY(str, len) QByteArray( str, len ) 
#endif

using namespace std;

int main( int argc, char ** argv )
{
  QApplication a( argc, argv );

#if QT_VERSION < 0x040000
  QSqlDatabase * db = QSqlDatabase::addDatabase( "QLINTER" );
#else
  QSqlDatabase db1 = QSqlDatabase::addDatabase( "QLINTER" );
  QSqlDatabase* db = &db1;
#endif
  db->setUserName( "SYSTEM" );
  db->setPassword( "MANAGER" );
  db->setConnectOptions("EXCLUSIVE");                       /* 30.07.13 */

  if( !db->open() )
  {
    cout << "Line " << __LINE__ << " : " << QS2C(db->lastError().driverText()) << endl;
    return 1;
  }

  QSqlQuery q;
  q.exec( "drop table test;" );
  q.exec( "create table test(i int primary key, vc varchar(5), vnc nchar varying(5), b boolean, dt date, dc decimal, r real, vb varbyte(5), bi bigint);"); 
  if ( !q.isActive() )
  { 
    cout << "Line " << __LINE__ << " : " << QS2C(q.lastError().driverText()) << endl;
    return 1;
  }  
  cout << "Creating table test ... DONE\n"; 

  q.exec( "insert into test values (1, NULL, cast 'text1' as nchar, true, sysdate, 12345678901, 1.11, 'dbac', 12345678901);" ); 
  if ( !q.isActive() )
  { 
    cout << "Line " << __LINE__ << " : " << QS2C(q.lastError().driverText()) << endl;
    return 1;
  }  

  cout << "\nAdding one record ... DONE\n"; 
    
  db->transaction();
  q.exec( "delete from test;" );
  cout << "\nDeleting all records from test table ... DONE\n"; 
  if ( !db->rollback())
  {
    cout << "Line " << __LINE__ << " : Can't rollback transaction. " << QS2C(db->lastError().driverText()) << endl;
    return 1;
  } 
  cout << "Rollback last transaction ... DONE\n"; 


#if QT_VERSION < 0x040000
  QSqlCursor cur2("test");
  QDataTable table( &cur2 );
  table.addColumn( "i", "i" );
  table.addColumn( "vc", "vc" );
  table.addColumn( "vnc", "vnc" );
  table.addColumn( "b", "b" );
  table.addColumn( "dt", "dt" );
  table.addColumn( "dc", "dc" );
  table.addColumn( "r", "r" );
  table.addColumn( "vb", "vb" );
  table.addColumn( "bi", "bi" );

  a.setMainWidget( &table );
  table.refresh(); /* load data */
  table.show();    /* show widget */
#else
  QSqlTableModel *model = new QSqlTableModel;
  model->setTable("test");
  model->setEditStrategy(QSqlTableModel::OnFieldChange);
  
  model->select();
  
  QTableView *view = new QTableView;
  view->setModel(model);
  view->show();  
#endif
  return a.exec();
}
