/****************************************************************************
 *                 RELEX, Inc. Linter DBMS source code                      *
 *              This is public source code of RELEX, Inc.                   *
 *                                                                          *
 *      You have a royalty-free right to use, modify, reproduce and         *
 *     distribute the Sample Files (and/or any modified version) in         *
 *        any way you find useful, provided that you agree that             *
 *       RELEX has no warranty obligations or liability for any             *
 *           Sample Application Files which are modified.                   *
 *                                                                          *
 *                 Copyright (c) 2000-2012 RELEX Inc.                       *
 *                                                                          *
 *                   This software was developed by                         *
 *                                                                          *
 *                            RELEX, Inc.                                   *
 *                         Voronezh, Russia                                 *
 *                                                                          *
 *                        All rights reserved.                              *
 ****************************************************************************
 AUTHOR: Muratova
 ----------------------------------------------------------------------------
 PURPOSE: linqt sample. work with blob in inl form (use ACCESS_BLOBFILE mode). 
 ----------------------------------------------------------------------------
*/

#include <iostream>

#include <qapplication.h>
#include <qsqldatabase.h>
#include <qsqlerror.h>
#include <qdatetime.h>

#include <qlabel.h>


#if QT_VERSION < 0x040000
#  define QS2QB(qstr) qstr.local8Bit()
#  define QS2C(qstr) qstr.local8Bit().data()
#  define QBYTEARRAY(str, len) QByteArray().duplicate( str, len ) 
#  include <qfile.h>
#  include <qcstring.h>
#  include <qsqlcursor.h>
#  include <qsqlselectcursor.h>
#  include <qdatatable.h>
#else
#  define QS2QB(qstr) qstr.toLocal8Bit()
#  include <QtSql>
#  define QS2C(qstr) qstr.toLocal8Bit().data()
#  define QBYTEARRAY(str, len) QByteArray( str, len ) 
#  include <QByteArray>
#  include <qtableview.h>
#  include <qsqltablemodel.h>
#endif


using namespace std;
int main( int argc, char ** argv )
{
  QApplication a( argc, argv );
  
#if QT_VERSION < 0x040000
  QSqlDatabase * db = QSqlDatabase::addDatabase( "QLINTER" );
#else
  QSqlDatabase db1 = QSqlDatabase::addDatabase( "QLINTER" );
  QSqlDatabase* db = &db1;
#endif
  db->setUserName( "SYSTEM" );
  db->setPassword( "MANAGER" );

  
  if( !db->open() )
  {
    cout << "Line " << __LINE__ << " : " << QS2C(db->lastError().driverText()) << endl;
    return 1;
  }

  QSqlQuery q;

  q.exec( "drop table test;" );
  q.exec( "create table test(i int, bl blob);"); 

  cout << "Creating table test... DONE\n\n";

  QString str;
  
  cout << "Insert with \"blob insert rowid= table.name blob_string\"\n";
  q.exec("insert into test(i) values (1)"); 
  if ( !q.isActive() )
  { 
    cout << "Line " << __LINE__ << " : " << QS2C(q.lastError().driverText()) << endl;
    return 1;
  }     
  q.exec("blob insert rowid=1 test.bl first_blob");  
  if ( !q.isActive() )
  { 
    cout << "Line " << __LINE__ << " : " << QS2C(q.lastError().driverText()) << endl;
    return 1;
  } 

  cout << "Insert with \"blob insert column= blob_string\"\n";
  q.exec("insert into test(i) values (2)");   
  str = "select bl from test where i=2";
  if(q.exec(str) && q.next())
  {
    q.exec("blob insert column=1 second_blob");    
    if ( !q.isActive() )
    { 
      cout << "Line " << __LINE__ << " : " << QS2C(q.lastError().driverText()) << endl;
      return 1;
    } 
  }
  
  cout << "Insert with \"blob insert rowid= table.column file=\"\n";
  q.exec("insert into test(i) values (3)");
  QFile f1("1.txt");
  f1.open(QIODevice::WriteOnly);
  f1.write("file_blob_1");
  f1.close();
  q.exec("blob insert rowid=3 test.bl file=1.txt");  
  if ( !q.isActive() )
  { 
    cout << "Line " << __LINE__ << " : " << QS2C(q.lastError().driverText()) << endl;
    return 1;
  } 
  
  cout << "Insert with \"blob insert rowid= user.table.column file=\"\n";
  q.exec("insert into test(i) values (4)");
  QFile f2("2.txt");
  f2.open(QIODevice::WriteOnly);
  f2.write("file_blob_2");   
  f2.close();
  q.exec("blob insert rowid=4 system.test.bl file=2.txt");  
  if ( !q.isActive() )
  { 
    cout << "Line " << __LINE__ << " : " << QS2C(q.lastError().driverText()) << endl;
    return 1;
  } 

  cout << "Insert with \"blob insert column= file=\"\n";
  q.exec("insert into test(i) values (5)");  
  str = "select bl from test where i=5";
  if(q.exec(str) && q.next())
  {
    q.exec("blob insert column=1 file=1.txt");    
    if ( !q.isActive() )
    { 
      cout << "Line " << __LINE__ << " : " << QS2C(q.lastError().driverText()) << endl;
      return 1;
    } 
  } 

  cout << "Append with \"blob append rowid= table.column blob_string\"\n";
  q.exec("blob append rowid=1 test.bl third_blob");
  if ( !q.isActive() )
  { 
    cout << "Line " << __LINE__ << " : " << QS2C(q.lastError().driverText()) << endl;
    return 1;
  } 

  cout << "Append with \"blob append column= blob_string\"\n";
  str = "select bl from test where i=2";
  if(q.exec(str) && q.next())
  {
    q.exec("blob append column=1 fourth_blob"); 
    if ( !q.isActive() )
    { 
      cout << "Line " << __LINE__ << " : " << QS2C(q.lastError().driverText()) << endl;
      return 1;
    } 
  }
  
  cout << "Append with \"blob append rowid= table.column file=\"\n";
  q.exec("blob append rowid=3 test.bl file=1.txt");
  if ( !q.isActive() )
  { 
    cout << "Line " << __LINE__ << " : " << QS2C(q.lastError().driverText()) << endl;
    return 1;
  } 
  
  cout << "Append with \"blob append rowid user.table.cloumn file=\"\n";
  q.exec("blob append rowid=4 system.test.bl file=2.txt");
  if ( !q.isActive() )
  { 
    cout << "Line " << __LINE__ << " : " << QS2C(q.lastError().driverText()) << endl;
    return 1;
  }   
  
  cout << "Append with \"blob append column= file=\"\n";
  str = "select bl from test where i=5";
  if(q.exec(str) && q.next())
  {
    q.exec("blob append column=1 file=1.txt");
    if ( !q.isActive() )
    { 
      cout << "Line " << __LINE__ << " : " << QS2C(q.lastError().driverText()) << endl;
      return 1;
    } 
  }    

  cout << "Get with \"blob get rowid= table.column file=\"\n";
  q.exec("blob get rowid=3 test.bl file=3.txt");    
  if ( !q.isActive() )
  { 
    cout << "Line " << __LINE__ << " : " << QS2C(q.lastError().driverText()) << endl;
    return 1;
  } 

  cout << "Get with \"blob get rowid= user.table.column file=\"\n";
  q.exec("blob get rowid=4 system.test.bl file=4.txt");    
  if ( !q.isActive() )
  { 
    cout << "Line " << __LINE__ << " : " << QS2C(q.lastError().driverText()) << endl;
    return 1;
  } 

  cout << "Get with \"blob get column= file=\"\n";
  str = "select bl from test where i=5";
  if(q.exec(str) && q.next())
  {
    q.exec("blob get column=1 file=5.txt");    
    if ( !q.isActive() )
    { 
      cout << "Line " << __LINE__ << " : " << QS2C(q.lastError().driverText()) << endl;
      return 1;
    } 
  }  
  
  cout << "Get with \"blob get column=\"\n";
  QByteArray ba2 = QBYTEARRAY("", 1);
  ba2.resize(20);
  str = "select bl from test where i=6";
  if(q.exec(str) && q.next())
  {
    q.prepare("blob get column=1");
    q.bindValue(0, ba2);
    q.bindValue(1, 20);
    q.exec();
    if ( !q.isActive() )
    { 
      cout << "Line " << __LINE__ << " : " << QS2C(q.lastError().driverText()) << endl;
      return 1;
    } 
    cout << "ba2 " << ba2.data() << "\n";
  }


  q.prepare( "INSERT INTO test (i, bl) VALUES (?, ?)" );   
  q.bindValue(0, 6);
  q.bindValue(1, "six_blob");
  q.exec();
  if ( !q.isActive() )
  { 
    cout << "Line " << __LINE__ << " : " << QS2C(q.lastError().driverText()) << endl;
    return 1;
  } 

  q.prepare( "INSERT INTO test (i, bl) VALUES (?, ?)" );    
  q.bindValue(0, 7);
  q.bindValue(1, "seven_blob");
  q.exec();
  if ( !q.isActive() )
  { 
    cout << "Line " << __LINE__ << " : " << QS2C(q.lastError().driverText()) << endl;
    return 1;
  } 

  cout << "Clear with \"blob clear rowid= user.table.name\"\n";
  if(q.exec(str) && q.next())
  {
    q.exec("blob clear rowid=6 system.test.bl");        
    if ( !q.isActive() )
    { 
      cout << "Line " << __LINE__ << " : " << QS2C(q.lastError().driverText()) << endl;
      return 1;
    } 
  }

  cout << "Clear with \"blob clear column=\"\n";
  str = "select bl from test where i=7";
  if(q.exec(str) && q.next())
  {
    q.exec("blob clear column=1");        
    if ( !q.isActive() )
    { 
      cout << "Line " << __LINE__ << " : " << QS2C(q.lastError().driverText()) << endl;
      return 1;
    } 
  }

                               
  QSqlTableModel *tableModel = new QSqlTableModel;
  tableModel->setTable("test");
  tableModel->setEditStrategy(QSqlTableModel::OnFieldChange);
  tableModel->select();
  
  QTableView *table = new QTableView;
  table->setModel(tableModel);
  table->show();  
  
  return a.exec();
}
