/****************************************************************************
 *                 RELEX, Inc. Linter DBMS source code                      *
 *              This is public source code of RELEX, Inc.                   *
 *                                                                          *
 *      You have a royalty-free right to use, modify, reproduce and         *
 *     distribute the Sample Files (and/or any modified version) in         *
 *        any way you find useful, provided that you agree that             *
 *       RELEX has no warranty obligations or liability for any             *
 *           Sample Application Files which are modified.                   *
 *                                                                          *
 *                 Copyright (c) 2000-2010 RELEX Inc.                       *
 *                                                                          *
 *                   This software was developed by                         *
 *                                                                          *
 *                            RELEX, Inc.                                   *
 *                         Voronezh, Russia                                 *
 *                                                                          *
 *                        All rights reserved.                              *
 ****************************************************************************
 AUTHOR: Gluhov
 ----------------------------------------------------------------------------
 PURPOSE: linqt sample. prepared queries.
 ----------------------------------------------------------------------------
*/

#include <iostream>

#include <qapplication.h>
#include <qsqldatabase.h>
#include <qsqlerror.h>

#if QT_VERSION < 0x040000
#  include "qsqlcursor.h"
#  include "qdatetime.h"
#  include "qmemarray.h"
#  define QS2C(qstr) qstr.local8Bit().data()
#  define QBYTEARRAY(str, len) QByteArray().duplicate( str, len ) 
#else
#  include <QtSql>
#  define QS2C(qstr) qstr.toLocal8Bit().data()
#  define QBYTEARRAY(str, len) QByteArray( str, len ) 
#endif

using namespace std;

int main( int argc, char ** argv )
{
  QApplication a( argc, argv );

#if QT_VERSION < 0x040000
  QSqlDatabase * db = QSqlDatabase::addDatabase( "QLINTER" );
#else
  QSqlDatabase db1 = QSqlDatabase::addDatabase( "QLINTER" );
  QSqlDatabase* db = &db1;
#endif
  
  db->setUserName( "SYSTEM" );
  db->setPassword( "MANAGER" );

  if( !db->open() )
  {
    cout << "Line " << __LINE__ << " : " << QS2C(db->lastError().driverText()) << "\n";
    return 1;
  }

  QSqlQuery q;
 
  q.exec( "drop table test;" );
  q.exec( "create table test(i int, vc varchar(10), vnc nchar varying(10), b boolean, dt date, dc decimal, r real, vb varbyte(5), bi bigint);"); 
  if ( !q.isActive() )
  { 
    cout << "Line " << __LINE__ << " : " << QS2C(q.lastError().driverText()) << endl;
    return 1;
  }  
  cout << "Creating table test ... DONE\n"; 

  q.exec( "insert into test values(1, 'char1', cast 'text2' as nchar, true, sysdate, 12345678901, 1.11, 'dcba', 12345678901);" );
  if ( !q.isActive() )
  { 
    cout << "Line " << __LINE__ << " : " << QS2C(q.lastError().driverText()) << endl;
    return 1;
  }  
  cout << "\nInsert data using simple QSqlQuery ... DONE\n";

#if QT_VERSION >= 0x030101
  if ( !q.prepare( "insert into test values (?, ?, ?, ?, ?, ?, ?, ?, ?);" ) )
  {
    cout << "Line " << __LINE__ << " : " << QS2C(q.lastError().driverText()) << endl;
    return 1;
  }  
  char * test = "char2";

  q.bindValue(0, 2); 
  q.bindValue(1, QBYTEARRAY( test, strlen(test)));
  q.bindValue(2, QString("text2"));
  q.bindValue(3, false );
  q.bindValue(4, QDateTime::currentDateTime());
  q.bindValue(5, QVariant()); // NULL value
  q.bindValue(6, 2.22);
  q.bindValue(7, QBYTEARRAY("abcd", 4));
  q.bindValue(8, 3);

  if ( !q.exec() )   
  {
    cout << "Line " << __LINE__ << " : " << QS2C(q.lastError().driverText()) << "\n";
    return 1;
  }  
  cout << "\nInsert data using prepared QSqlQuery ... DONE\n";
#endif

  cout << "\nSelect data using QSqlQuery:\n"; 
  q.exec( "select * from test;" );

  if ( !q.isActive() ) 
  {
    cout << "Line " << __LINE__ << " : " << QS2C(q.lastError().driverText()) << "\n";
    return 1;
  }
  else
  {   
    if (q.isSelect())
      while (q.next())
      {
        cout << q.at() << " : |";
        for ( int i=0; i<9; i++ )
        {
          cout << (q.isNull(i) ? "NULL" : QS2C(q.value(i).toString()));
          cout << "|";
        } 
        cout << endl;
      }                                 
  }
  return 0;
}
