/****************************************************************************
 *                 RELEX, Inc. Linter DBMS source code                      *
 *              This is public source code of RELEX, Inc.                   *
 *                                                                          *
 *      You have a royalty-free right to use, modify, reproduce and         *
 *     distribute the Sample Files (and/or any modified version) in         *
 *        any way you find useful, provided that you agree that             *
 *       RELEX has no warranty obligations or liability for any             *
 *           Sample Application Files which are modified.                   *
 *                                                                          *
 *                 Copyright (c) 2000-2010 RELEX Inc.                       *
 *                                                                          *
 *                   This software was developed by                         *
 *                                                                          *
 *                            RELEX, Inc.                                   *
 *                         Voronezh, Russia                                 *
 *                                                                          *
 *                        All rights reserved.                              *
 ****************************************************************************
 AUTHOR: Gluhov
 ----------------------------------------------------------------------------
 PURPOSE: linqt sample. work with blobs.
 ----------------------------------------------------------------------------
 REVISION HISTORY:
 29.04.10 -- Muratova (correct blob column number)
 30.07.13 -- Galizin (Code renew)
 ----------------------------------------------------------------------------*/

#include <iostream>

#include <qapplication.h>
#include <qsqldatabase.h>
#include <qsqlerror.h>


inline QByteArray BAInit(const char* Str, int len)
{
#if QT_VERSION < 0x040000
  QByteArray Ba = QByteArray().duplicate(Str, strlen(Str));
#else
  QByteArray Ba = QByteArray(Str);
#endif
  Ba.resize(len);
  return Ba;
}
#  define QBYTEARRAY(str, len) BAInit(str, len)

#if QT_VERSION < 0x040000
#  define QS2C(qstr) qstr.local8Bit().data()
#else
#  include <QtSql>
#  define QS2C(qstr) qstr.toLocal8Bit().data()
#endif

using namespace std;

int PrintAllData(QSqlQuery& q)
{
  q.exec("select * from test;");
  if ( !q.isActive() )
  {
    cout << "Line " << __LINE__ << " : " << QS2C(q.lastError().driverText()) << "\n";
    return 1;
  }
  else
  {
    if (q.isSelect())
    {
      q.first();
      do
      {
        cout << q.at() << " : |";
        for ( int i=0; i<3; i++ )
        {
          cout << (q.isNull(i) ? "NULL" : QS2C(q.value(i).toString()));
          cout << "|";
        }
        cout << endl;
//        q.exec("blob get"); // The blob can be got in fetching process
//        cout << QS2C(q.boundValue(0).toString()) << endl;
      }
      while (q.next());
    }
  }
  cout<<endl;
  return 0;
}

int main( int argc, char ** argv )
{
#if QT_VERSION >= 0x030200
  QApplication a( argc, argv );

#if QT_VERSION < 0x040000
  QSqlDatabase * db = QSqlDatabase::addDatabase( "QLINTER" );
#else
  QSqlDatabase db1 = QSqlDatabase::addDatabase( "QLINTER" );
  QSqlDatabase* db = &db1;
#endif

  db->setUserName( "SYSTEM" );
  db->setPassword( "MANAGER" );
  
  if( !db->open() )
  {
    cout << "Line " << __LINE__ << " : " << QS2C(db->lastError().driverText()) << "\n";
    return 1;
  }

  QSqlQuery q;

  QByteArray ba = QBYTEARRAY( "blobdata", 8 );
  QByteArray ba2 = QBYTEARRAY( "blobdata2", 9 );

  q.exec( "drop table test;" );
  if ( !q.exec( "create table test(id int, b blob, b2 blob);" ) )
  {
    cout << "Line " << __LINE__ << " : " << QS2C(q.lastError().driverText()) << "\n";
    return 1;
  }
  cout << "Creating table test ... DONE\n";

  // Inserting data by binding QByteArray
  cout << "Inserting data by binding values"<<endl;
  if ( !q.prepare( "insert into test(id, b)  values(?, ?)" ) )
  {
    cout << "Line " << __LINE__ << " : " << QS2C(q.lastError().driverText()) << "\n";
    return 1;
  }
//  ba.resize(600000);
//  ba.fill('a', 600000);
  q.bindValue( 0, 1 );
  q.bindValue( 1, ba );
  if ( !q.exec() )
  {
    cout << "Line " << __LINE__ << " : " << QS2C(q.lastError().driverText()) << "\n";
    return 1;
  }

  //inserting data by "blob append rowid". It is independed query and it no need any additional query
  cout<<"inserting text data by \"blob append\" rowid= "<<endl;
  if ( !q.exec("blob append rowid=1 test.b2 'column=34 KL'") ) // text value
  {
    cout << "Line " << __LINE__ << " : " << QS2C(q.lastError().driverText()) << "\n";
    return 1;
  }

  if ( !q.exec("select * from test where rowid = 1;") )
  {
    cout << "Line " << __LINE__ << " : " << QS2C(q.lastError().driverText()) << "\n";
    return 1;
  }

  cout<<"inserting data by AddBlob"<<endl;
  ba = QBYTEARRAY("SA", 2);
//  ba.resize(600000);
//  ba.fill('a', 600000);
  q.prepare("AddBlob");
//  q.bindValue(0, 3); // column
//  q.bindValue(1, 14); // type
  q.bindValue(2, ba); // Data array
//  q.bindValue(3, 1); // Data size
  if ( !q.exec() )
  {
    cout << "Line " << __LINE__ << " : " << QS2C(q.lastError().driverText()) << "\n";
    return 1;
  }

  // The same by blob append
  cout<<"inserting data by \"blob append\""<<endl;
  ba = QBYTEARRAY("AS", 2);
  q.prepare("blob append");
  q.bindValue(0, ba); // Data array
//  q.bindValue(1, 1); // Data size
//  q.bindValue(2, 14); // type
//  q.bindValue(3, 3); // column
  if ( !q.exec() )
  {
    cout << "Line " << __LINE__ << " : " << QS2C(q.lastError().driverText()) << "\n";
    return 1;
  }

  cout<<"inserting hex data by \"blob append\""<<endl;
  for (int i = 0; i < 60; i++)
  {
    if ( !q.exec("blob append column=3 3420") ) // hex value
    {
      cout << "Line " << __LINE__ << " : " << QS2C(q.lastError().driverText()) << "\n";
      return 1;
    }
  }

  // inserting the second row
  if ( !q.prepare( "insert into test (id, b, b2) values(?, ?, ?)" ) )
  {
    cout << "Line " << __LINE__ << " : " << QS2C(q.lastError().driverText()) << "\n";
    return 1;
  }
  ba = QBYTEARRAY("TheSecondBlobVeryVeryVeryVeryVeryVeryVeryVeryVeryVeryVeryVeryVeryVeryVeryVeryVeryVeryVeryVeryVeryVeryVeryVeryVeryVeryVeryVeryVeryVeryVeryVeryLong", 145);
  q.bindValue( 0, 2 );
  q.bindValue( 1, ba );
  q.bindValue( 2, ba2 );
  if ( !q.exec() )
  {
    cout << "Line " << __LINE__ << " : " << QS2C(q.lastError().driverText()) << "\n";
    return 1;
  }

  cout<<"inserting data by AddBlob"<<endl;
  q.exec("select * from test where ID=3");
  q.prepare("AddBlob");
  q.bindValue(0, 2); //column
  q.bindValue(1, 12); //type
  q.bindValue(2, ba2); //array
  q.bindValue(3, 2);// Data length
  if ( !q.exec() )
  {
    cout << "Line " << __LINE__ << " : " << QS2C(q.lastError().driverText()) << "\n";
    return 1;
  }
  cout << "\nInsert data ... DONE\n"; // 2 rows were inserted

  /* Fetch works only after select */
  if ( !q.exec("select * from test;") )
  {
    cout << "Line " << __LINE__ << " : " << QS2C(q.lastError().driverText()) << "\n";
    return 1;
  }
  q.next();

  //getting data by value method
  cout<<"Blob data has got by value() method:"<<QS2C(q.value(2).toString())<<endl<<endl;

  cout <<"Getting data by \"blob get\" without parameters"<< endl;
  //q.exec("blob get column=3"); // the same data
  q.exec("blob get");// default first blob field in the row of the rowset
  ba = q.boundValue(0).toByteArray();
  cout<<"Blob data size="<<ba.size()<<" has got by \"blob get\":"<<QS2C(QString(ba))<<endl;
  cout<<"Blob data length got by boundValue:"<<q.boundValue(1).toInt()<<endl<<endl;

  //get blob data whith parameters
  cout <<"Getting data by \"blob get\" with offset"<< endl;
  q.prepare("blob get");
  q.bindValue(2, 3); // 3 byte offset
//  q.bindValue(3, 3); // the 3-th column
//  q.bindValue(1, 2); // 2 byte size for the blob
  q.exec();
  cout<<"Blob data with offset 3:"<<QS2C(q.boundValue(0).toString())<<endl<<endl;

  //get blob data into preallocated memory
  cout <<"Getting data by \"blob get\" into QByteArray buffer"<< endl;
  ba.resize(64);
  ba.fill('a', 64); // fill by garbage
  cout<<"QByteArray content before getting data:"<<QS2C(QString(ba))<<endl;
  q.prepare("blob get"); // prepare reset all bind values
  q.bindValue(0, ba); //QByteArray to return the data
//  q.bindValue(1, 2); // 2 byte size max for the blob
//  q.bindValue(2, 3); // 3 byte offset
//  q.bindValue(3, 3); // the 3-th column
  q.exec();
  cout<<"Blob data in QByteArray:"<<QS2C(QString(ba))<<"  size QByteArray="<<ba.size()<<endl; // ba size is not changed !!!
  cout<<"Real data size="<<ba.size()<<" in QByteArray:"<<QS2C(QString(ba))<<endl<<endl;
  
  // Save blob data into the file
  cout<<"Saving blob data to file"<<endl;
  q.exec("select * from test where rowid = 2;");
  q.exec("blob get column=2 file=test_b2.txt");
  
  //Save with parameter binding
  cout<<"Saving blob data to file with buffer"<<endl;
  ba.resize(100);
  q.prepare("blob get column=2 file=test_b2_.txt");
  q.bindValue(0, ba); // Through this buffer. Data will stored in it. Data length is limited by this buffer
//  q.bindValue(1, 80); // max buffer len
  q.bindValue(2, 4);  // offset
  q.bindValue(3, 3);  // Column
  q.exec();
  cout<<"----"<<QS2C(QString(ba))<<"----"<<" size = "<<q.boundValue(1).toInt()<<endl<<endl;

  cout<<"getting blob data by FetchBlob"<<endl;
  ba = QBYTEARRAY( "aaaaaaaaaaaaa", 64000 ); // fill by garbage
  q.prepare( "FetchBlob" );
//  q.bindValue( 0, 3 );  //blob column number. if it's skiped it's the first blob column
//  q.bindValue( 1, 3 );  //start from 3rd byte. if it's skiped it's the 0 offset
//  q.bindValue( 2, ba); //buffer for blob data. if it's skiped it can be got by boundValue(3) 
//  q.bindValue( 3, 50000 ); //size of buffer. if it's skiped it's ba.zise() or length of all blob
  if ( !q.exec() )
  {
    cout << "Line " << __LINE__ << " : " << QS2C(q.lastError().driverText()) << "\n";
    return 1;
  }
  ba = q.boundValue(2).toByteArray(); // If it was not been bound. Bound value can be got this way
  cout << "\nData fetched with FetchBlob command: |" << QS2C(QString(ba)) << "|" << "Length="<<ba.size()<< endl;
  cout << "Data size   "<< q.boundValue(3).toInt() << endl << endl; // The 3-th parameter return real read data length

  if ( PrintAllData(q) )
    return -1;

  // Clear blob sample
  cout<<"clearing blob by \"blob clear\""<<endl;
  // by blob clear rowid
  if (!q.exec("blob clear rowid=1 test.b2") )
  {
    cout << "Line " << __LINE__ << " : " << QS2C(q.lastError().driverText()) << "\n";
    return 1;
  }
  PrintAllData(q);

  // by update
  cout<<"clearing blob by update query"<<endl;
  if (!q.exec("update test set b=NULL where rowid=1"))
  {
    cout << "Line " << __LINE__ << " : " << QS2C(q.lastError().driverText()) << "\n";
    return 1;
  }
  PrintAllData(q);

  // by blob clear for rowset
  if (!q.exec("select * from test;"))
  {
    cout << "Line " << __LINE__ << " : " << QS2C(q.lastError().driverText()) << "\n";
    return 1;
  }

  cout<<"clearing all blobs in the rowset"<<endl;
  do
  {
    q.exec("blob clear column=3");
  }
  while(q.next());
  PrintAllData(q);

  cout<<"clearing blob with parameter binding"<<endl;
//  q.prepare( "PurgeBlob" );
  q.prepare( "blob clear" );
  q.bindValue( 0, 2 );  //blob column number               /* 30.07.13 29.04.10*/
  if ( !q.exec() )
  {
    cout << "Line " << __LINE__ << " : " << QS2C(q.lastError().driverText()) << "\n";
    return 1;
  }
  PrintAllData(q);


  cout<<"inserting data from file"<<endl;
  q.exec("select * from test;");
  q.exec("blob insert file=test_b2.txt");
  PrintAllData(q);

  cout<<"geting data by FetchBlob with parameters"<<endl;
  q.first();
  ba.resize( 20 );
  q.prepare( "FetchBlob" );
  q.bindValue( 0, 2 );  //blob column number
//  q.bindValue( 1, 3 );  //start from 3rd byte
  q.bindValue( 2, ba );  //Buffer for data
//  q.bindValue( 3, 15 );  //Buffer size
  if ( !q.exec() )
  {
    cout << "Line " << __LINE__ << " : " << QS2C(q.lastError().driverText()) << "\n";
    return 1;
  }
  ba = q.boundValue(2).toByteArray();
  cout << "Data fetched by FetchBlob command: |" << QS2C(QString(ba)) << "|" << endl;

#else
  cout << "Error! This example will work only with QT 3.2 or higher" << endl;
#endif
  return 0;
}


