/****************************************************************************
 *                 RELEX, Inc. Linter DBMS source code                      *
 *              This is public source code of RELEX, Inc.                   *
 *                                                                          *
 *      You have a royalty-free right to use, modify, reproduce and         *
 *     distribute the Sample Files (and/or any modified version) in         *
 *        any way you find useful, provided that you agree that             *
 *       RELEX has no warranty obligations or liability for any             *
 *           Sample Application Files which are modified.                   *
 *                                                                          *
 *                 Copyright (c) 2000-2010 RELEX Inc.                       *
 *                                                                          *
 *                   This software was developed by                         *
 *                                                                          *
 *                            RELEX, Inc.                                   *
 *                         Voronezh, Russia                                 *
 *                                                                          *
 *                        All rights reserved.                              *
 ****************************************************************************
 AUTHOR: Gluhov
 ----------------------------------------------------------------------------
 PURPOSE: linqt sample. work with stored procedures.
 ----------------------------------------------------------------------------
*/

#include <iostream>

#include <qapplication.h>
#include <qsqldatabase.h>
#include <qsqlerror.h>

#if QT_VERSION < 0x040000
#  define QS2C(qstr) qstr.local8Bit().data()
#  define QBYTEARRAY(str, len) QByteArray().duplicate( str, len ) 
#else
#  include <QtSql>
#  define QS2C(qstr) qstr.toLocal8Bit().data()
#  define QBYTEARRAY(str, len) QByteArray( str, len ) 
#endif

using namespace std;

int main( int argc, char ** argv )
{
  QApplication app( argc, argv );

#if QT_VERSION < 0x040000
  QSqlDatabase * db = QSqlDatabase::addDatabase( "QLINTER" );
#else
  QSqlDatabase db1 = QSqlDatabase::addDatabase( "QLINTER" );
  QSqlDatabase* db = &db1;
#endif
  
  db->setUserName( "SYSTEM" );
  db->setPassword( "MANAGER" );

  if( !db->open() )
  {
    cout << "Line " << __LINE__ << " : " << QS2C(db->lastError().driverText()) << "\n";
    return 1;
  }

  QSqlQuery q;

  q.exec( "drop procedure retcur;" );
  if ( !q.exec( QString( "create procedure retcur()\
                          result\
                            cursor(name char(20), city char(15))\
                          declare\
                            var c typeof(result);\
                          code\
                            open c for direct \"select NAME,FIRSTNAM from PERSON where AGE>30 and AGE<32;\";\
                            return c;\
                          end;" ) ) )
  {
    cout << "Line " << __LINE__ << " : " << QS2C(q.lastError().driverText()) << "\n";
    cout << "Current version of LINTER don't support stored procedures.\n";  
    return 1;
  }
  
  if ( !q.exec("execute retcur()") )   
  {
    cout << "Line " << __LINE__ << " : " << QS2C(q.lastError().driverText()) << "\n";
    return 1;
  }  
  if ( q.isSelect() ) 
  {
    cout << endl << "After calling procedure retcur(): " << endl;
    while ( q.next() )
    {
      cout << q.at() << " : |";
      cout << QS2C(q.value(0).toString()) << "|";
      cout << QS2C(q.value(1).toString()) << "|" << endl;
    }
  }
  else
  {
    cout << endl << "Undefined error while executing procedure retcur()" << endl;
  }

#if QT_VERSION >= 0x030200

  q.exec( QString( "drop procedure retint" ) );

  if ( !q.exec( QString( "create procedure retint(in a int;\
                                                  in b int;\
                                                  inout name char(32))\
                                                  result int\
                          code\
                            name := \"OUTPUT NAME\";\
                            return a + b;\
                          end;" ) ) )
  {
    cout << "Line " << __LINE__ << " : " << QS2C(q.lastError().driverText()) << "\n";
    cout << "Current version of LINTER don't support stored procedures.\n";
    return 1;
  }
  
  QVariant result = 0;
  int a = 5;
  int b = 6;
  QVariant name = QBYTEARRAY("qwerty", 6);

  q.prepare("execute into ? retint(?,?,?)");

  q.bindValue( 0, result );
  q.bindValue( 1, a );
  q.bindValue( 2, b );
  q.bindValue( 3, name );

  if ( !q.exec() )   
  {
    cout << "Line " << __LINE__ << " : " << QS2C(q.lastError().driverText()) << "\n";
    return 1;
  }  
  result = q.boundValue( 0 );
  name = q.boundValue( 3 );

  cout << endl << "After calling procedure retint(): " << endl;
  cout << "name   = " << QS2C(name.toString()) << endl;
  cout << "result = " << QS2C(result.toString()) << endl;
#endif

  return 0;
}
