/****************************************************************************
 *                 RELEX, Inc. Linter DBMS source code                      *
 *              This is public source code of RELEX, Inc.                   *
 *                                                                          *
 *      You have a royalty-free right to use, modify, reproduce and         *
 *     distribute the Sample Files (and/or any modified version) in         *
 *        any way you find useful, provided that you agree that             *
 *       RELEX has no warranty obligations or liability for any             *
 *           Sample Application Files which are modified.                   *
 *                                                                          *
 *                 Copyright (c) 2000-2011 RELEX Inc.                       *
 *                                                                          *
 *                   This software was developed by                         *
 *                                                                          *
 *                            RELEX, Inc.                                   *
 *                         Voronezh, Russia                                 *
 *                                                                          *
 *                        All rights reserved.                              *
 ****************************************************************************
 AUTHOR: DV
 ----------------------------------------------------------------------------
 PURPOSE: linqt sample (BLOB related)
          Made from samples received from a customer
 ----------------------------------------------------------------------------
 07.04.11 -- Ermakov (prepared query supported from 0x030101)
*/

#include <iostream>

#include <qapplication.h>
#include <qsqldatabase.h>
#include <qsqlquery.h>
#include <qsqlerror.h>

#if QT_VERSION < 0x040000
#  include "qsqlcursor.h"
#  include "qdatetime.h"
#  include "qmemarray.h"
#  define QS2C(qstr) qstr.local8Bit().data()
#  define QBYTEARRAY(str, len) QByteArray().duplicate( str, len ) 
#else
#  include <QtSql>
#  define QS2C(qstr) qstr.toLocal8Bit().data()
#  define QBYTEARRAY(str, len) QByteArray( str, len ) 
#endif

using namespace std;

int main( int argc, char ** argv )
{
  QApplication a( argc, argv );

#if QT_VERSION < 0x040000
  QSqlDatabase * db = QSqlDatabase::addDatabase( "QLINTER" );
#else
  error This sample is not support qt v 4
#endif
  
  db->setUserName( "SYSTEM" );
  db->setPassword( "MANAGER" );

  if( !db->open() )
  {
    cout << "Line " << __LINE__ << " : " << QS2C(db->lastError().driverText()) << "\n";
    return 1;
  }

  QSqlQuery q;

  QByteArray ba  = QBYTEARRAY("SAMPLE", sizeof("SAMPLE"));
  QByteArray ba2 = QBYTEARRAY("SAMPLE2 for another BLOB column", 
                             sizeof("SAMPLE2 for another BLOB column"));

  cout << "\nSample 1 (insert for BLOB)\n";
  q.exec( "drop table blobtab;" );
  q.exec( "create table blobtab(id int, data1 blob, comment varchar(100), data2 blob);"); 
  if ( !q.isActive() )
  { 
    cout << "Line " << __LINE__ << " : " << QS2C(q.lastError().driverText()) << endl;
    return 1;
  }  
  cout << "Creating table blobtab ... DONE\n"; 

  QSqlCursor* pictcursor=new QSqlCursor("BLOBTAB");
  QSqlRecord *buf=pictcursor->primeInsert();
  buf->setValue("ID",3);
  buf->setValue("COMMENT","This is a BLOB usage sample");
  buf->setValue("DATA1",ba);
  buf->setValue("DATA2",ba2);
  bool ret=pictcursor->insert();
  if(ret) 
    qDebug("Success"); 
  else
    qDebug("%s",pictcursor->lastError().driverText().ascii());

  cout << "\nSample 2 (update for BLOB - with PK)\n";
  q.exec( "drop table blobtab;" );
  q.exec( "create table blobtab(id int primary key, data1 blob, comment varchar(100), data2 blob);"); 
  if ( !q.isActive() )
  { 
    cout << "Line " << __LINE__ << " : " << QS2C(q.lastError().driverText()) << endl;
    return 1;
  }  
  cout << "Re-creating table blobtab ... DONE\n"; 

  q.exec( "insert into blobtab(id, comment) values (3,'This is a BLOB usage sample');"); 
  if ( !q.isActive() )
  { 
    cout << "Line " << __LINE__ << " : " << QS2C(q.lastError().driverText()) << endl;
    return 1;
  }  
  cout << "Filling table blobtab ... DONE\n"; 

  QSqlCursor* pictcursor1=new QSqlCursor("BLOBTAB");
  pictcursor1->select(QString("ID=%1").arg(3));
  if(pictcursor1->next())
  {
    QSqlRecord *buf=pictcursor1->primeUpdate();
    buf->setValue("DATA1",ba);
    buf->setValue("DATA2",ba2);
    bool ret=pictcursor1->update();
    if(ret)
      qDebug("Success"); 
    else
      qDebug("%s",pictcursor1->lastError().driverText().ascii());
  }

  cout << "\nSample 3 (use AddBlob)\n";
  q.exec( "drop table blobtab;" );
  q.exec( "create table blobtab(id int, data1 blob, comment varchar(100), data2 blob);"); 
  if ( !q.isActive() )
  { 
    cout << "Line " << __LINE__ << " : " << QS2C(q.lastError().driverText()) << endl;
    return 1;
  }  
  cout << "Re-creating table blobtab ... DONE\n"; 

  q.exec( "insert into blobtab(id,comment) values (3,'This is a BLOB usage sample');"); 
  if ( !q.isActive() )
  { 
    cout << "Line " << __LINE__ << " : " << QS2C(q.lastError().driverText()) << endl;
    return 1;
  }  
  cout << "Filling table blobtab ... DONE\n"; 

#if QT_VERSION >= 0x030101                              /* 07.04.11 */
  QSqlQuery* p=new QSqlQuery;
  QString str("select ID, DATA1, DATA2 from BLOBTAB where ID=3");
  if(p->exec(str) && p->next())
  {
    p->prepare("AddBlob");
    p->bindValue(0,2);
    p->bindValue(1,0);
    p->bindValue(2,ba);
    p->bindValue(3,ba.size());
    if(!p->exec())
      qDebug("%s",p->lastError().driverText().ascii());
    else
      qDebug("Success");
    p->prepare("AddBlob");
    p->bindValue(0,3);
    p->bindValue(1,0);
    p->bindValue(2,ba2);
    p->bindValue(3,ba2.size());
    if(!p->exec())
      qDebug("%s",p->lastError().driverText().ascii());
    else
      qDebug("Success");
  }
#endif

  return 0;
}
