/****************************************************************************
 *                 RELEX, Inc. Linter DBMS source code                      *
 *              This is public source code of RELEX, Inc.                   *
 *                                                                          *
 *      You have a royalty-free right to use, modify, reproduce and         *
 *     distribute the Sample Files (and/or any modified version) in         *
 *        any way you find useful, provided that you agree that             *
 *       RELEX has no warranty obligations or liability for any             *
 *           Sample Application Files which are modified.                   *
 *                                                                          *
 *                 Copyright (c) 2000-2010 RELEX Inc.                       *
 *                                                                          *
 *                   This software was developed by                         *
 *                                                                          *
 *                            RELEX, Inc.                                   *
 *                         Voronezh, Russia                                 *
 *                                                                          *
 *                        All rights reserved.                              *
 ****************************************************************************
 AUTHOR: DV
 ----------------------------------------------------------------------------
 PURPOSE: linqt sample (variable length types related). 
          Made from queries received from a customer
 ----------------------------------------------------------------------------
 REVISION HISTORY:
 29.04.10 -- Muratova (correct out of strValue)
 30.07.13 -- Galizin (Nvarchar query correction)
 ----------------------------------------------------------------------------*/

#include <iostream>

#include <qapplication.h>
#include <qsqldatabase.h>
#include <qsqlquery.h>
#include <qsqlerror.h>

#if QT_VERSION < 0x040000
#  include "qsqlcursor.h"
#  include "qdatetime.h"
#  include "qmemarray.h"
#  define QS2C(qstr) qstr.local8Bit().data()
#  define QBYTEARRAY(str, len) QByteArray().duplicate( str, len ) 
#else
#  include <QtSql>
#  define QS2C(qstr) qstr.toLocal8Bit().data()
#  define QBYTEARRAY(str, len) QByteArray( str, len ) 
#endif

using namespace std;

int main( int argc, char ** argv )
{
  QApplication a( argc, argv );

#if QT_VERSION < 0x040000
  QSqlDatabase * db = QSqlDatabase::addDatabase( "QLINTER" );
#else
  QSqlDatabase db1 = QSqlDatabase::addDatabase( "QLINTER" );
  QSqlDatabase* db = &db1;
#endif
  
  db->setUserName( "SYSTEM" );
  db->setPassword( "MANAGER" );

  if( !db->open() )
  {
    cout << "Line " << __LINE__ << " : " << QS2C(db->lastError().driverText()) << "\n";
    return 1;
  }

  QSqlQuery q;
 

  cout << "\nSample 1 (NVARHAR)\n";
  q.exec( "drop table string_tab;" );
  q.exec( "create table string_tab(id_tab int autoinc primary key, nstring nvarchar(5));"); 
  if ( !q.isActive() )
  { 
    cout << "Line " << __LINE__ << " : " << QS2C(q.lastError().driverText()) << endl;
    return 1;
  }  
  cout << "Creating table string_tab ... DONE\n"; 

  q.exec( "insert into string_tab(nstring) values (N'1234');"); /* 30.07.13 */
  if ( !q.isActive() )
  { 
    cout << "Line " << __LINE__ << " : " << QS2C(q.lastError().driverText()) << endl;
    return 1;
  }  
  cout << "Filling table string_tab ... DONE\n"; 

  QSqlQuery q1;
  QString strQuery = "select NSTRING from STRING_TAB where ID_TAB=1;";
  QString strValue;
  uint strLen;
  if(!q1.exec(strQuery))
  {
    cout << "Line " << __LINE__ << " : " << QS2C(q.lastError().driverText()) << endl;
    return 1;
  }
  else 
  {
    while (q1.next())
    {
      strValue = q1.value(0).toString();
      strLen = strValue.length();
       cout << " | " << QS2C(strValue) << " | " << strLen << endl; /*29.04.10*/
    }
  }

  cout << "\nSample 2 (VARCHAR)\n";
  q.exec( "drop table string_tab;" );
  cout << "Drop table string_tab ... DONE\n"; /*29.04.10*/
  q.exec( "create table string_tab(id_tab int autoinc primary key, nstring varchar(5));");
  if ( !q.isActive() )
  { 
    cout << "Line " << __LINE__ << " : " << QS2C(q.lastError().driverText()) << endl;
    return 1;
  }  
  cout << "Creating table string_tab ... DONE\n"; 

  q.exec( "insert into string_tab(nstring) values ('1234');"); 
  if ( !q.isActive() )
  { 
    cout << "Line " << __LINE__ << " : " << QS2C(q.lastError().driverText()) << endl;
    return 1;
  }  
  cout << "Filling table string_tab ... DONE\n"; 

  QSqlQuery q2;
  QString strQuery2 = "select NSTRING from STRING_TAB where ID_TAB=1;";
  if(!q2.exec(strQuery2))
  {
    cout << "Line " << __LINE__ << " : " << QS2C(q.lastError().driverText()) << endl;
    return 1;
  }
  else 
  {
    while (q2.next())
    {
      strValue = q2.value(0).toString();
      strLen = strValue.length();
      cout << " | " << QS2C(strValue) << " | " << strLen << endl; /*29.04.10*/
    }
  }

  return 0;
}
