/****************************************************************************
 *                 RELEX, Inc. Linter DBMS source code                      *
 *              This is public source code of RELEX, Inc.                   *
 *                                                                          *
 *      You have a royalty-free right to use, modify, reproduce and         *
 *     distribute the Sample Files (and/or any modified version) in         *
 *        any way you find useful, provided that you agree that             *
 *       RELEX has no warranty obligations or liability for any             *
 *           Sample Application Files which are modified.                   *
 *                                                                          *
 *                 Copyright (c) 2000-2010 RELEX Inc.                       *
 *                                                                          *
 *                   This software was developed by                         *
 *                                                                          *
 *                            RELEX, Inc.                                   *
 *                         Voronezh, Russia                                 *
 *                                                                          *
 *                        All rights reserved.                              *
 ****************************************************************************
 AUTHOR: Muratova
 ----------------------------------------------------------------------------
 PURPOSE: linqt sample. bind values.
 ----------------------------------------------------------------------------
*/

#include <iostream>

#include <qapplication.h>
#include <qsqldatabase.h>
#include <qsqlerror.h>
#include <qdatetime.h>

#include <QByteArray>
#include <QtSql>
#include <QTableView>
#include <QSqlTableModel>


using namespace std;
int main( int argc, char ** argv )
{
    QApplication a( argc, argv );
#if QT_VERSION < 0x040000
  QSqlDatabase * db = QSqlDatabase::addDatabase( "QLINTER" );
#else
  QSqlDatabase db1 = QSqlDatabase::addDatabase( "QLINTER" );
  QSqlDatabase* db = &db1;
#endif
  db->setUserName( "SYSTEM" );
  db->setPassword( "MANAGER" );

  if( !db->open() )
  {
    cout << "Line " << __LINE__ << " : " << db->lastError().driverText().toLocal8Bit().data() << endl;
    return 1;
  }

  QSqlQuery q;

  q.exec( "drop table test;" );
  q.exec( "create table test(i int, vc varchar(100), vnc nchar varying(100), b boolean, dt date, dc decimal, r real, vb varbyte(100), bi bigint, bl blob, bt byte(100), ch char(100), db double, nc nchar(100), si smallint);"); 
  
  cout << "Creating table test ... DONE\n"; 
  cout << "\nUsing addBindValue\n";

  if ( !q.prepare("INSERT INTO test (i, vc, b, bt) VALUES (?,?,?,?)") )
  { 
    cout << "Line " << __LINE__ << " : " << q.lastError().driverText().toLocal8Bit().data() << endl;
    return 1;
  }  
  q.addBindValue(1);
  q.addBindValue(QString(""));
  q.addBindValue(QString("true"));
  q.addBindValue(QString("dbac"));
  q.exec();
  if ( !q.isActive() )
  { 
    cout << "Line " << __LINE__ << " : " << q.lastError().driverText().toLocal8Bit().data() << endl;
    return 1;
  }

  cout << "\nFilling table test ... DONE\n";


  cout << "\nUsing bindValue and placeholder names\n";
  if ( !q.prepare("INSERT INTO test (i, vc, b, bt, vnc) VALUES (:i, :vc, :b, :bt, :vnc)" ) )
  {
    cout << "Line " << __LINE__ << " : " << q.lastError().driverText().toLocal8Bit().data() << endl;
    return 1;
  }  
  q.bindValue(":i", 2); 
  q.bindValue(":vc", QString("varchar"));
  q.bindValue(":b", QString("true"));
  q.bindValue(":bt", QString("dbac"));
  q.bindValue(":vnc", QString("nvc"));
  q.exec();    
  if ( !q.isActive() )
  { 
    cout << "Line " << __LINE__ << " : " << q.lastError().driverText().toLocal8Bit().data() << endl;
    return 1;
  }

  cout << "\nFilling table test ... DONE\n";


  cout << "\nUsing bindValue, placeholder names, posisions\n";
  if ( !q.prepare("INSERT INTO test (i, vc, b, bt, vnc) VALUES (:i, :vc, :b, :bt, :vnc)" ) )
  {
    cout << "Line " << __LINE__ << " : " << q.lastError().driverText().toLocal8Bit().data() << endl;
    return 1;
  }  
  q.bindValue(0, 3); 
  q.bindValue(1, QString("varchar"));
  q.bindValue(2, QString("true"));
  q.bindValue(3, QString("dbac"));
  q.bindValue(4, QString("nvc"));
  q.exec();
  if ( !q.isActive() )
  { 
    cout << "Line " << __LINE__ << " : " << q.lastError().driverText().toLocal8Bit().data() << endl;
    return 1;
  }

  cout << "\nFilling table test ... DONE\n";


  cout << "\nUsing bindValue and posisions\n";

  if ( !q.prepare( "INSERT INTO test (vc, vnc, b, dt, r, vb, bi, bl, bt, ch, db, nc, si) VALUES (?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?)" ) )
  {
    cout << "Line " << __LINE__ << " : " << q.lastError().driverText().toLocal8Bit().data() << endl;
    return 1;
  }

  q.bindValue(0, "varchar");
  q.bindValue(1, "varunicode");
  q.bindValue(2, 0);
  q.bindValue(3, QDate::fromString("1.30", "M.d")); 
  q.bindValue(4, 5.5);
  q.bindValue(5, "dba");
  q.bindValue(6, 12345);
  q.bindValue(7, "blob");
  q.bindValue(8, "dbac");
  q.bindValue(9, "char");
  q.bindValue(10, 12.45);
  q.bindValue(11, "unicode");
  q.bindValue(12, 12);
  q.exec();
  if ( !q.isActive() )
  { 
    cout << "Line " << __LINE__ << " : " << q.lastError().driverText().toLocal8Bit().data() << endl;
    return 1;
  }
  
  if ( !q.exec( "SELECT * FROM test" ) )
  {
    cout << "Line " << __LINE__ << " : " << q.lastError().driverText().toLocal8Bit().data() << endl;
    return 1;
  }

 
  while (q.next())
  {
    cout << q.at() << " : |";
    for ( int i=0; i<15; i++ )
    {
      cout << (q.isNull(i) ? "NULL" : q.value(i).toByteArray().constData());
      cout << "|";
    } 
    cout << endl;
  }                                 
  

  cout << "\nFilling table test ... DONE\n";

  QSqlTableModel *model = new QSqlTableModel;
  model->setTable("test");
  model->setEditStrategy(QSqlTableModel::OnFieldChange);
  
  model->select();
  
  QTableView *view = new QTableView;
  view->setModel(model);
  view->show();  
  return a.exec();
}


