 /***************************************************************************
 *                 RELEX, Inc. Linter DBMS source code                      *
 *              This is public source code of RELEX, Inc.                   *
 *                                                                          *
 *      You have a royalty-free right to use, modify, reproduce and         *
 *     distribute the Sample Files (and/or any modified version) in         *
 *        any way you find useful, provided that you agree that             *
 *       RELEX has no warranty obligations or liability for any             *
 *           Sample Application Files which are modified.                   *
 *                                                                          *
 *                Copyright (c) 1990 - 2012 RELEX Inc.                      *
 *                                                                          *
 *                   This software was developed by                         *
 *                                                                          *
 *                            RELEX, Inc.                                   *
 *                         Voronezh, Russia                                 *
 *                                                                          *
 *                        All rights reserved.                              *
 ****************************************************************************
 AUTHOR:
 ----------------------------------------------------------------------------
 PURPOSE:
 ----------------------------------------------------------------------------
 REVISION HISTORY:
  08.10.98 -- Konon
  05.03.02 -- Galizin - VxWorks support
  30.07.02 -- Galizin - Remove compiler warning PT#1046
  28.03.03 -- Sveta (code format)
  08.05.03 -- Galizin (Size of connect string)
  23.05.03 -- EVG ( Porting to WinCE: PT #784 )
  31.06.12 -- Ermakov (ODBC 3.7)
 ------------------------------------------------------------*/

#include <stdio.h>
#include <stdlib.h>

#ifdef INTER_MSWINDOWS
#include <windows.h>
#endif
#include <sql.h>
#include <sqlext.h>
#include <odbcinst.h>

#if defined(VXWORKS)            /* 05.03.02 */
#include "vxstart.h"
#endif

#define TRUE 1
#define FALSE 0

#define MAXBUFLEN		256         /* display buffer size */
#define MAX_COL			15          /* maximum column in result set */
#define MAX_ROW			100         /* maximum number of rows */
#define MAXDATALEN  256     /* maximum data length per column */  /* 05.03.02 */
#define MAXDISPLAYSIZE		MAX_COL*(MAXDATALEN+1)

#define DEFDBNAME		"?"
#define DSN_HDBC_FORMAT		"%s!%lX"
#define DSN_HDBC_HSTMT_FORMAT	"DSN=%s!HDBC=%lX!HSTMT=%lX"
#define MAKECONNECT		"Please connect to a data source using Login menu."
#define	OPENWINDOW		"Please open a window from Query menu."
#define NOHDBCERROR		"No Connection Available"
#define NOHSTMTERROR		"No Query Window Available"
#define MAXCHILDEXCEEDED	"Please close a child window and try again."
#define MAXCHLDERR		"Error - Too many Child Windows"
#define COLTRUNC_WARNG		"Number of columns in display truncated to %u"
#define ROWTRUNC_WARNG    "\nNumber of rows in display truncated to %u"
#define TRUNCERR		"Data Truncation"
#define CLOSEALLHDBC		"All open connections have been closed."
#define LOGOUTINFO		"Log Out Information"
#define SQLERR_FORMAT		"SQL Error State:%s, Native Error Code: %d, ODBC Error: %s"
#define SQLERRMSGTITLE		"SQL_ERROR results"
#define SQLERRCNTDTITLE		"SQL_ERROR results continued"
#define SQLWRNMSGTITLE		"SQL_SUCCESS_WITH_INFO results"
#define SQLWRNCNTDTITLE		"SQL_SUCCESS_WITH_INFO results continued"
#define NULLDATASTRING		"SQL_NULL_DATA"


int nChildCount;                /*  Number of child windows currently
                                 * open */
HENV henv;                      /*  Environment Handle */
char szDispBuffer[MAXDISPLAYSIZE + 1];  /*  Display Buffer */
UCHAR rgbData[MAX_COL][MAXDATALEN]; /*  Results Data Array */
SDWORD dwDataLen[MAX_COL];      /*  Results Data Length Array */

void DisplayError(RETCODE nResult, HENV henv, HDBC hdbc, HSTMT hstmt);

BOOL InitSQLEnvironment(void)
{
  /* call SQLAllocEnv to load Driver Manager */
  return (SQLAllocEnv(&henv) == SQL_SUCCESS);
}

BOOL ConnectDatabase(HDBC *hdbc,
                     char *szDBName /* DSN sting */,
                     char *szUserName /* User name */,
                     char *szPassword /* Password */
                     )
{
  RETCODE nResult;              /* Result code */

  /* Call SQLAllocConnect to allocate a fresh HDBC. If successful */
  /* then call SQLConnect using the allocated hdbc and supplied */
  /* connection information. */

  if ((nResult = SQLAllocConnect(henv, hdbc)) != SQL_SUCCESS)
  {
    DisplayError(nResult, henv, SQL_NULL_HDBC, SQL_NULL_HENV);
    return (FALSE);
  }

  nResult = SQLConnect(*hdbc, (LPSTR) szDBName, SQL_NTS, (LPSTR) szUserName, SQL_NTS, (LPSTR) szPassword, SQL_NTS);

  /* if failed to connect, free the allocated hdbc before return */
  if (nResult != SQL_SUCCESS && nResult != SQL_SUCCESS_WITH_INFO)
  {
    DisplayError(nResult, SQL_NULL_HENV, *hdbc, SQL_NULL_HENV);
    SQLFreeConnect(*hdbc);
    return (FALSE);
  }

  /* display any connection information if driver returns
   * SQL_SUCCESS_WITH_INFO */
  if (nResult == SQL_SUCCESS_WITH_INFO)
    DisplayError(nResult, SQL_NULL_HENV, *hdbc, SQL_NULL_HENV);

  printf("Connect!\n");
  return (TRUE);
}

BOOL DriverConnectDatabase(
                     char *szDBName   /* DSN sting */,
                     char *szUserName /* User name */,
                     char *szPassword /* Password */
                     )
{
  char szBuffer[MAXBUFLEN + 1]; /* display successful connection info on
                                 * hdbc(s) combo-box */
  char buf[MAXBUFLEN + 1];
  SWORD swStrLen;               /* String length                                   */
  HDBC hdbc;                    /* hdbc                                              */
  RETCODE nResult;              /* result code                                      */

  if ((nResult = SQLAllocConnect(henv, (HDBC FAR *) & hdbc)) != SQL_SUCCESS)
  {
    DisplayError(nResult, henv, SQL_NULL_HDBC, SQL_NULL_HSTMT);
    return (FALSE);
  }

  sprintf(buf, "DSN=%s;UID=%s;PWD=%s", szDBName, szUserName, szPassword);
  nResult = SQLDriverConnect(hdbc, NULL,
         buf, SQL_NTS, szBuffer, MAXBUFLEN, &swStrLen,                    /* 08.05.03 */ 
         SQL_DRIVER_COMPLETE_REQUIRED);

  if (nResult != SQL_SUCCESS && nResult != SQL_SUCCESS_WITH_INFO)
  {
    printf("DriverConnectError\n");
    SQLFreeConnect(hdbc);
    return (FALSE);
  }

  if (nResult == SQL_SUCCESS_WITH_INFO)
    DisplayError(nResult, SQL_NULL_HENV, hdbc, SQL_NULL_HSTMT);

  strcpy(szDBName, DEFDBNAME);  /* initialize the string value */

  nResult = SQLGetInfo(hdbc, SQL_DATA_SOURCE_NAME, (PTR) szDBName, MAXBUFLEN + 1, &swStrLen);

  szDBName[swStrLen] = '\0';
  printf("DriverConnect!\n");
  return (TRUE);
}

BOOL Connection(HDBC *hdbc)
{
  char szDBName[MAXBUFLEN + 1]; /* DSN sting */
  char szUserName[MAXBUFLEN + 1]; /* User name */
  char szPassword[MAXBUFLEN + 1]; /* Password */

  /* retrieve DSN, UID and PWD values from the connect dialog box */
  printf("UserName :");
  fgets(szUserName, MAXBUFLEN, stdin);
  szUserName[strlen(szUserName) - 1] = 0;
  printf("Password :");
  fgets(szPassword, MAXBUFLEN, stdin);
  szPassword[strlen(szPassword) - 1] = 0;
  printf("DSN :");
  fgets(szDBName, MAXBUFLEN, stdin);
  szDBName[strlen(szDBName) - 1] = 0;

  ConnectDatabase(hdbc, szDBName, szUserName, szPassword);
  return DriverConnectDatabase(szDBName, szUserName, szPassword);
}

void ExecuteQuery(HDBC hdbc)
{
  char szBuffer[MAXBUFLEN + 1]; /* display buffer */
  HSTMT hstmt;                  /* hstmt                */
  SWORD nCols = 0;              /* # of result columns */
  SWORD nCount;                 /* index */
  SWORD swColType;              /* column data type */
  SWORD swColScale;             /* column scale      */
  SWORD swColNull;              /* nullable column ?   */
  SWORD swColLength = MAXDATALEN; /* column data length    */
  DWORD dwText;                 /* tab stop for listbox   */
  UDWORD udwColDef;             /* precision on the column */
  RETCODE nReturn;              /* return code          */
  int nRows;                    /* # of result rows       */

  for (nCount = 0; nCount < MAX_COL; nCount++)
  {
    dwDataLen[nCount] = 0;
    rgbData[nCount][0] = '\0';
  }

  if ((nReturn = SQLAllocStmt(hdbc, &hstmt)) != SQL_SUCCESS)
  {
    DisplayError(nReturn, SQL_NULL_HENV, hdbc, SQL_NULL_HSTMT);
    return;
  }

  printf("SQL>");
  fgets(szBuffer, sizeof(szBuffer), stdin); /* 30.07.02 */

  nReturn = SQLExecDirect(hstmt, szBuffer, SQL_NTS);
  if (nReturn != SQL_SUCCESS)
  {
    DisplayError(nReturn, SQL_NULL_HENV, SQL_NULL_HDBC, hstmt);
    if (nReturn != SQL_SUCCESS_WITH_INFO)
    {
      SQLFreeStmt(hstmt, SQL_CLOSE);
      return;
    }
  }

  nReturn = SQLNumResultCols(hstmt, &nCols);
  if (nCols >= MAX_COL)
  {
    nCols = MAX_COL;
    printf(COLTRUNC_WARNG, MAX_COL);
  }
  else if (nCols == 0)
  {
    SQLFreeStmt(hstmt, SQL_CLOSE);
    return;
  }

  for (nCount = 0; nCount < nCols; nCount++)
    SQLBindCol(hstmt, (UWORD) (nCount + 1), SQL_C_CHAR, rgbData[nCount], MAXDATALEN, &dwDataLen[nCount]);


  for (nCount = 1, szDispBuffer[0] = '\0'; nCount <= nCols; nCount++)
  {
    SQLDescribeCol(hstmt, nCount, szBuffer, MAXDATALEN, &swColLength, &swColType, &udwColDef,
                   &swColScale, &swColNull);
    strcat(szDispBuffer, szBuffer);
    dwText = strlen(szDispBuffer);
    szDispBuffer[dwText++] = '\t';
    szDispBuffer[dwText] = '\0';
  }
  if (*szDispBuffer)
    szDispBuffer[strlen(szDispBuffer) - 1] = '\0';

  for (nRows = 0; (nReturn = SQLFetch(hstmt)) == SQL_SUCCESS || nReturn == SQL_SUCCESS_WITH_INFO;)
  {
    for (nCount = 0, szDispBuffer[0] = '\0'; nCount < nCols; nCount++)
    {
      strcat(szDispBuffer, (dwDataLen[nCount] == SQL_NULL_DATA) ? ((UCHAR *) NULLDATASTRING) : rgbData[nCount]);
      dwText = strlen(szDispBuffer);
      szDispBuffer[dwText++] = '\t';
      szDispBuffer[dwText] = '\0';
    }
    if (*szDispBuffer)
      szDispBuffer[strlen(szDispBuffer) - 1] = '\0';
    else
      break;
    printf("%s  |", szDispBuffer);
    if (++nRows == MAX_ROW)
    {
      printf(ROWTRUNC_WARNG, MAX_ROW);
      break;
    }
    printf("\n");
  }

  if (nReturn == SQL_ERROR)
    DisplayError(nReturn, SQL_NULL_HENV, SQL_NULL_HDBC, hstmt);

  SQLFreeStmt(hstmt, SQL_CLOSE);
  SQLFreeStmt(hstmt, SQL_UNBIND);

}

void DisplayError(RETCODE nResult, HENV henv, HDBC hdbc, HSTMT hstmt)
{
  UCHAR szErrState[SQL_SQLSTATE_SIZE + 1];  /* SQL Error State string */
  UCHAR szErrText[SQL_MAX_MESSAGE_LENGTH + 1];  /* SQL Error Text string */
  char szBuffer[SQL_SQLSTATE_SIZE + SQL_MAX_MESSAGE_LENGTH + MAXBUFLEN + 1];
                                /* formatted Error text Buffer */
  SWORD wErrMsgLen;             /* Error message length */
  SQLINTEGER dwErrCode;         /* Native Error code */
  int iSize;                    /* Display Error Text size */
  RETCODE nErrResult;           /* Return Code from SQLError */
  int fFirstRun;                /* If it was first msg box */

  fFirstRun = TRUE;
  szBuffer[0] = '\0';

  do
  {
    /* initialize display buffer with the string in error text buffer */
    strcpy(szDispBuffer, szBuffer);

    /* call SQLError function with proper ODBC handles, repeatedly until */
    /* function returns SQL_NO_DATA_FOUND. Concatenate all error strings */
    /* in the display buffer and display all results */
    while ((nErrResult = SQLError(henv, hdbc, hstmt, szErrState, &dwErrCode, szErrText,
          SQL_MAX_MESSAGE_LENGTH - 1, &wErrMsgLen)) != SQL_NO_DATA_FOUND)
    {
      sprintf(szBuffer, SQLERR_FORMAT, (LPSTR) szErrState, dwErrCode, (LPSTR) szErrText);
      iSize = strlen(szDispBuffer);
      if (iSize && (iSize + strlen(szBuffer) + 1) >= MAXDISPLAYSIZE)
        break;
      if (iSize)
        strcat(szDispBuffer, "\n");
      strcat(szDispBuffer, szBuffer);
    }

    /* display proper ERROR or WARNING message with proper title */
#if defined(WIN32) && ! defined(WINCE)
    CharToOem(szDispBuffer, szDispBuffer);
#endif
    if (nResult == SQL_SUCCESS_WITH_INFO)
      printf("Warning : %s\n", szDispBuffer);
    else
      printf("Error : %s\n", szDispBuffer);

    if (fFirstRun)
      fFirstRun = FALSE;

  }
  while (!(nErrResult == SQL_NO_DATA_FOUND));
}

void GetInfo(SQLHDBC conn)
{
  char str[20];
  SWORD len;
  RETCODE ret;

  /* Get version */
  len = 20;
  ret = SQLGetInfo(conn, SQL_DRIVER_ODBC_VER, str, len, &len);
  if (ret != SQL_SUCCESS && ret != SQL_SUCCESS_WITH_INFO)
  {
    printf("error GetInfo\n");
  }   /* if */
  else
  {
    printf("DRIVER_VER %s (%d)\n", str, len);
  }   /* else */

  return ;
}
/********************************************* END OF FILE **************************************************/


#if defined(VXWORKS)            /* 05.03.02 */
MainStart(odbctest, 64 * 1024, UninitLinterClient)
#elif defined(WINCE)                                      /* 23.05.03 ...*/
int odbctest()
#else
int main(void)
#endif                                                    /*... 23.05.03 */
{
  SQLHENV env;
  SQLHDBC conn;
  HDBC hdbc;
  RETCODE ret;
  RETCODE conret; /* Connect return code */

  ret = SQLAllocEnv(&env);
  if (ret != SQL_SUCCESS)
  {
    printf("error AllocEnv\n");
  }   /* if */

  ret = SQLAllocConnect(env, &conn);
  if (ret != SQL_SUCCESS)
  {
    printf("error AllocEnv\n");
  }   /* if */

  InitSQLEnvironment();
  conret = Connection(&hdbc);
  /* Get version */
  GetInfo(hdbc);
  if ( conret != FALSE) /* 08.10.98 */
  {
    ExecuteQuery(hdbc);
  }   /* if */
  printf("\nDone.\n");

  return 0;                     /* 30.07.02 */
}
