 /***************************************************************************
 *           RELEX, Inc. Linter sql formatter sample source code            *
 *  This is unpublished proprietary source code of RELEX, Inc.              *
 *  The copyright notice does not evidence any actual or intended           *
 *  publication of such source code.                                        *
 *                                                                          *
 *                 Copyright (c) 1990-2013 RELEX Inc.                       *
 *                                                                          *
 *                   This software was developed by                         *
 *                                                                          *
 *                            RELEX, Inc.                                   *
 *                         Voronezh, Russia                                 *
 *                                                                          *
 *                        All rights reserved.                              *
 ****************************************************************************
 AUTHOR:
 ----------------------------------------------------------------------------
 PURPOSE: format queries for better readability
 ----------------------------------------------------------------------------
 REVISION HISTORY:
*/

/*
    L_LONG FormatQueries(const L_CHAR *inBuf, L_CHAR **outBuf, L_LONG caseFlag, L_LONG numOfTabs)

    The 'FormatQueries' function formats SQL query stored at 'inBuf' for better readability.
  Function dynamically allocates a string to hold the output. The 'outBuf' is the address of a 
  'L_CHAR *', and a successful call stores a pointer to the newly allocated string at that location.
  The return value is zero on success format, -1 if not enough memory, -2 if internal error occurs.
  The 'caseFlag' argument controls conversion of the key words and the identifies. This is a bit mask.
    The following macros are defined:
    - macro KW_CASE
        If set all key words will be converted to upper case.
    - macro ID_CASE
        If set all identifies in SQL query will be converted to upper case except ones in double
        quotes.
    The 'numOfTabs' argument defines the number of indents to be performed in sub-queries. Each
  indent is equals two spaces;
*/

#include <stdio.h>
#include <stdlib.h>
#include "lintypes.h"
#include "sqlf.h"

int main()
{
  L_CHAR  query[] = "select p.salary,p.personid,a.make,a.model from \"person\" as p, (select * from auto)\
                     as a where exists (select make,model,bodytype from auto where personid = p.personid \
                     and ((0=0 and 1=0) or (1=1 and 0=1))) and 3=3;" ;
  L_CHAR *out = NULL;
  L_LONG  r   = 0;

  r = FormatQueries(query, &out, KW_CASE | ID_CASE, 1);
  if (r == 0)
  {
    fprintf(stdout, "%s", out);
    free(out);
  }
  else
    fprintf(stdout, "Failed to format query. Error # %ld.\n", r);

  return 0;
}
