 /***************************************************************************
 *           RELEX, Inc. Linter Tests source code                           *
 *  This is unpublished propietary source code of RELEX, Inc.               *
 *  The copyright notice does not evidence any actual or intended           *
 *  publication of such source code.                                        *
 *                                                                          *
 *                 Copyright (c) 1990-2007 RELEX Inc.                       *
 *                                                                          *
 *                   This software was developed by                         *
 *                                                                          *
 *                            RELEX, Inc.                                   *
 *                         Voronezh, Russia                                 *
 *                                                                          *
 *                        All rights reserved.                              *
 ****************************************************************************
 AUTHOR: Maxim V. Kositsin -- 01.09.97 11:04
 ----------------------------------------------------------------------------
 PURPOSE: command string
 ----------------------------------------------------------------------------
 REVISION HISTORY:
  08.10.99 -- Ermakov (QNX)
  11.09.00 -- Galizin (Title)
  26.10.00 -- Galizin (Version)
  28.11.01 -- Maxim (remove datetime and add strupr/lwr function)
  28.11.01 -- Maxim (remove version)
  30.11.01 -- Maxim (test accounts!=0)
  06.03.06 -- Maxim (correct error with long argument string)
  29.01.07 -- Oleg,Ermakov (Corrected versions)
  10.06.14 -- Reshetnyak -- master-slave mode, default pessimistic, a few new functions
*/
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <time.h>
#include "lintypes.h"
#include "tdef.h"
#include "tpcb.h"

#define SWITCH_SYMBOL1    '-'

#if defined(UNIX) || defined(QNX) || defined(OS9000)		/* 08.10.99 */
#define SWITCH_SYMBOL2    '-'
#else
#define SWITCH_SYMBOL2    '/'
#endif
/*-----------------------------------------------------------
  Created:    28.11.01 17:15 by Maxim V. Kositsin
  Comments:
  -----------------------------------------------------------*/
static char * __strlwr ( char *str )
  {
  char *sv = str;
  while ( *str )
    {
    if ( *str >= 65 && *str <= 90 )
      *str += 32;
    str++;
    }                           /* while */
  return  sv;
  }
/*-----------------------------------------------------------
  Created:    28.11.01 17:15 by Maxim V. Kositsin
  Comments:
  -----------------------------------------------------------*/
static char * __strupr( char *str )
  {
  char *sv = str;
  while ( *str )
    {
    if ( *str >= 97 && *str <= 122 )
      *str -= 32;
    str++;
    }/* while */
  return sv;
  }

/*----------------------22.08.97 17:03------------------------
  Call:
  Desc:
  Args:
  Output:
  ------------------------------------------------------------*/
static L_BOOL     FindArg ( L_LONG argc, L_CHAR *argv[], L_CHAR *key, L_CHAR *arg, int maxlen )
  {
  L_LONG           i,
                 len;
  L_CHAR            ku[20],
                 kl[20];

  strcpy ( ku, key );
  strcpy ( kl, key );
  __strupr ( ku );
  __strlwr ( kl );
  len = strlen ( ku );
  for ( i = 1; i < argc; i++ )
    {
    if ( ( argv[i][0] == SWITCH_SYMBOL1 || argv[i][0] == SWITCH_SYMBOL2 ) &&
         ( memcmp ( ku, argv[i] + 1, len ) == 0 ||
           memcmp ( kl, argv[i] + 1, len ) == 0 ) )
      {
      argv[i][0] = 0;
      if ( arg )
        {
        if ( argv[i][len + 1] == ':' || argv[i][len + 1] == '=' )
          strncpy ( arg, argv[i] + len + 2, maxlen );
        else if ( ( i + 1 < argc ) && ( argv[i + 1][0] != SWITCH_SYMBOL1 ) &&
                                      ( argv[i + 1][0] != SWITCH_SYMBOL2 ) )
          {
          strncpy ( arg, argv[i + 1], maxlen );
          argv[i+1][0] = 0;
          }
        else
          arg[0] = 0;
        arg[maxlen-1] = 0;
        }
      return L_TTRUE;
      }                         /* if */
    }                           /* for */
  return L_TFALSE;
  }
/*----------------------18.08.97 15:42------------------------
  Call:
  Desc:
  Args:
  Output:
  ------------------------------------------------------------*/
L_BOOL            Arguments ( L_LONG argc, L_CHAR *argv[] )
  {
  L_LONG i,retcode;
  L_CHAR               arg[256];

  if ( argc > 1 && !strcmp( __strupr(argv[1]), "-VERSION"))    /* 26.10.00.. */
    {
    /* 29.11.01... */
    printf( "RDBMS Linter SQL\n");
    printf( "Copyright (C) 1995-2007 Relex, Inc. All rights reserved.\n");
    /* ...29.11.01 */
/*
    printf("%s\n", version(VERSION_STRING));
    printf("%s\n", version(COPYRIGHT_STRING));
*/
    exit(0);
    }                                                       /* ..26.10.00 */


  printf ( "TPC Benchmark B for RDBMS Linter SQL v.%d.%d\n"
           "Copyright (C) 1996-2007 Relex, Inc. All rights reserved.\n"
           , _VER_MAX/100, _VER_MAX%100/10 );

  if ( argc == 1 ||
       strcmp ( argv[1], "/?" ) == 0 ||
       strcmp ( argv[1], "/H" ) == 0 ||
       strcmp ( argv[1], "-H" ) == 0 ||
       strcmp ( argv[1], "-HELP" ) == 0 ||
       strcmp ( argv[1], "/HELP" ) == 0 ||
       strcmp ( argv[1], "-?" ) == 0 )
    return L_TFALSE;
  __strupr ( argv[1] );
  switch ( argv[1][0] )
    {
    case 'M':
      command = TPCB_MASTER;
      break;
    case 'S':
      command = TPCB_SLAVE;
      break;
    default:
      printf ( "??? - '%s'\n", argv[1] );
      return L_TFALSE;
      }                         /* switch */
  if ( FindArg ( argc, argv, "trace", arg, sizeof(arg) ) ) /* 28.11.01 */
    {
    extern int trace;
    extern char tracefile[1024];
    trace = 1;
    if ( arg[0] )
      strncpy ( tracefile, arg, sizeof (tracefile) );
    }
  if ( FindArg ( argc, argv, "a", arg, sizeof(arg) ) )
    {
    naccounts = atol ( arg ); /* 30.11.01 */
    if ( !naccounts ) /* 30.11.01 */
      {
      printf ( "Invalid naccounts (0)\n" );
      return L_TFALSE;
      } /* if */
    }
  if ( FindArg ( argc, argv, "t", arg, sizeof(arg) ) )
    worktime = atol ( arg );
  Cbl.PrzExe = M_EXCLUSIVE;                                /* 10.06.14 */
  if ( FindArg ( argc, argv, "m", arg, sizeof(arg) ) )
    {
    __strupr ( arg );
    if ( strcmp ( arg , "PESSIMISTIC" ) == 0 )
      Cbl.PrzExe = 1024;
    else if ( strcmp ( arg , "OPTIMISTIC" ) == 0 )
      Cbl.PrzExe = 256;
    else
      {
      printf ( "??? --- '%s'\n", arg );
      return L_TFALSE;
      }
    }
  retcode = L_TTRUE;
  for ( i = 2 ; i < argc ; i++ )
    {
    if ( argv[i][0] )
      {
      retcode = L_TFALSE;
      printf ( "??? --- '%s'\n", argv[i] );
      }/* if */
    }/* for */
  return retcode?1:0;
  }
