 /***************************************************************************
 *           RELEX, Inc. Linter Tests source code                           *
 *  This is unpublished proprietary source code of RELEX, Inc.              *
 *  The copyright notice does not evidence any actual or intended           *
 *  publication of such source code.                                        *
 *                                                                          *
 *                 Copyright (c) 1990-2009 RELEX Inc.                       *
 *                                                                          *
 *                   This software was developed by                         *
 *                                                                          *
 *                            RELEX, Inc.                                   *
 *                         Voronezh, Russia                                 *
 *                                                                          *
 *                        All rights reserved.                              *
 ****************************************************************************
 AUTHOR: Maxim K. 01/03/95
 ----------------------------------------------------------------------------
 PURPOSE: TPC Benchmark B
 ----------------------------------------------------------------------------
 REVISION HISTORY:
 14/09/98	Li: out file for slave mode
 17.09.98 -- Maxim -- I corrected some errors (when didlock).
 17.09.98 -- Maxim -- I added storeproc into the file.
 24.04.00 -- Galizin -- correct \\ char in string
 28.11.00 -- Ermakov (ROWIDs)
 21.09.01 -- Maxim -- correct mode message
 28.11.01 -- Maxim -- optimize deleting
 28.11.01 -- Maxim -- add -trace key
 09.04.03 -- Maxim -- correct master waiting
 21.09.05 -- DV    -- check completion codes in inner loop
 01.10.09 -- Ermakov, Plot  -- aligning for android
 10.06.14 -- Reshetnyak -- master-slave mode, default pessimistic, a few new functions
\***************************************************************************/

#define LIN_INT_TYP   2   /* integer                  */
/*#define DOS*/

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <time.h>

#include "tpcb.h"
#include "decimals.h"

#ifdef STORE_PROC /* 17.09.98 */

LOG LogId;
L_BOOL Hlp=0;
CMD_LINE_ARG clArgs[] =
       {
       { "H",       4, 1, 1,             &Hlp,          sizeof(Hlp)          },
       };

#define ARGS_COUNT sizeof(clArgs)/sizeof(CMD_LINE_ARG)

#ifdef ERROR
#undef ERROR
#endif
#define ERROR(e) { if ( Cbl.CodErr ) {ErrorCode = e; LError();} }

char q[QueryLen], SPOutBuf[SPOutBufSize];

#else

#ifdef ERROR
#undef ERROR
#endif
#define ERROR(e) { if ( Cbl.CodErr ) {ErrorCode = e; Error();} }

#endif

#define MAX_CASE_NUM      4   /* max number of execution cases in the DoOne function */ /* 10.06.14 ... */
#define NUM_REP_FOR_ERROR 10                               /* ... 10.06.14 */
L_CHAR *ErrString[] = {
  "Can't open channel.",
  "Can't create BEGIN_TPCB.",
  "Can't select from HISTORY.",
  "Can't select from BRANCHES.",
  "Can't select from TELLERS.",
  "Can't select from ACCOUNTS.",
  "Can't delete HISTORY.",
  "Can't create tables.",
  "Can't insert values into tables.",
  "Can't create index.",
  "pretranslator error.",
  "Can't update ACCOUNTS.",
  "Can't select from ACCOUNTS.",
  "Can't update TELLERS.",
  "Can't update BRANCHES.",
  "Can't insert values into HISTORY.",
  "Can't COMMIT WORK." };
L_LONG buffer[50];
TCBL Cbl;

L_LONG tps;
L_LONG nbranches;
L_LONG ntellers;
L_LONG naccounts = 1000;
L_LONG nhistory;
L_LONG i,Abalance;
L_CHAR name[19];
L_CHAR pass[19];

double summa,summa1,summa2,summa3;
L_CHAR mess[81];
L_CHAR mess1[81];

L_DECIMAL begt,endt,df;
L_LONG ErrorCode;

L_LONG    diff;

time_t first,second;
L_LONG worktime = 5;
L_LONG rnd;

L_CHAR UpAcc[2048];
#ifdef LONG_ALIGN     /* 01.10.09 */
L_LONG SelAbl4[2048/sizeof(L_LONG)];
L_CHAR *SelAbl = (L_CHAR *)&SelAbl4[0];
#else
L_CHAR SelAbl[2048];
#endif
L_CHAR UpTell[2048];
L_CHAR UpBran[2048];
L_CHAR InsHis[2048];
L_LONG Bid,Tid,Aid,Delta;
L_DECIMAL DeltaOld = { 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0 };
L_CHAR outfile[256]="lintpcb.out";

L_WORD len1=4,len2=4,len3=4,len4=4;
void Trans(void);
void CreateDatabase(void);
L_LONG DoOne(void);
void Error(void);
void Error(void);
L_LONG RandomNumber(L_ULONG, L_ULONG);

L_LONG command;

int trace = 0;        /* 28.11.01 */ 
char tracefile[1024] = "tpcb.trc"; /* 28.11.01 */
struct _trc           /* 28.11.01 */ 
  {
  time_t tim;
  clock_t _first;
  clock_t _second;
  struct _trc *next;
  } *trc = NULL;
/*-----------------------------------------------------------
  Created:    28.11.01 11:15 by Maxim V. Kositsin
  Comments:
  -----------------------------------------------------------*/
void AddTraceString ( time_t tim, clock_t _first, clock_t _second)
  {
  struct _trc *t;
  if ( trc == NULL )
    {
    trc = malloc ( sizeof (struct _trc) );
    t = trc;
    } /* if */
  else
    {
    for ( t = trc ; t->next ; t = t->next );
    t->next = malloc ( sizeof (struct _trc) );
    t = t->next;
    }
  memset ( t, 0 , sizeof (struct _trc) );
  t->tim = tim;
  t->_first = _first;
  t->_second = _second;
  }
/*-----------------------------------------------------------
  Created:    17.09.98 14:30 by Maxim V. Kositsin
  Comments:
  -----------------------------------------------------------*/
main(L_LONG argc , L_CHAR * argv[])
{
  L_LONG j;
  FILE *fout;
  L_LONG nb,nt,na,delta;
  L_LONG  tm,tmold=0,tml;
  L_REAL tr=0,trl=0;

  tps=1;
  nbranches=1;
  ntellers=10;

  if ( Arguments ( argc, argv ) == L_TFALSE )
    {
    printf("\n\n"
"Usage: TPCB <Command> [-a ACCOUNTS] [-t TIME] [-m MODE]\n"
"\n"
"       Commands:\n"
"         m - master\n"
"         s - slave\n"
"\n"
"       Switches:\n"
"         -a      - number of accounts\n"
"         -t      - work time (minutes)\n"
"         -m      - mode (OPTIMISTIC/PESSIMISTIC) default PESSIMISTIC\n" ); /* 10.06.14 */
    exit(1);
    }/* if */


  if( worktime <= 0 )                                      /* 10.06.14 ... */
    {
    printf("\nIncorrect worktime.\n");
    exit(1);
    }
  worktime*=60;                                            /* ... 10.06.14 */

  strcpy(name,"system");
  strcpy(pass,"manager");
  printf("\nConnected...");
  memcpy(Cbl.Command,"OPEN",4);
  memcpy(Cbl.Node,"    ",4);
  inter(&Cbl,"SYSTEM/MANAGER",NULL,NULL,NULL);
  ERROR(1);
  printf("OK");

  if ( command == TPCB_SLAVE )
   sprintf(outfile,"tpc_%ld.out",Cbl.NumChan);
  fout = fopen(outfile,"w+t" );
  if ( !fout )
    printf( "\nCan't create out file !!!\n" );

  if ( command == TPCB_MASTER )
  {
    printf ( "\nMASTER\n" );
    fprintf(fout,"\nMASTER\n");
    fflush(fout);
  }
  else
  {
    printf ( "\nSLAVE\n" );
    fprintf(fout,"\nSLAVE\n");
    fflush(fout);
  }
  if ( Cbl.PrzExe & 256 ) /* 21.09.01 */ 
  {
    printf ( "Mode     = OPTIMISTIC\n" );
    fprintf(fout,"Mode     = OPTIMISTIC\n");
    fflush(fout);
  }
  else
  {
    printf ( "Mode     = PESSIMISTIC\n" );
    fprintf(fout,"Mode     = PESSIMISTIC\n");
    fflush(fout);
  }
  printf ( "Accounts = %d\n"
           "Worktime = %d\n", naccounts , worktime );
  fprintf(fout,"Accounts = %d\n"
               "Worktime = %d\n", naccounts , worktime );
    fflush(fout);

  memcpy(Cbl.Command,"    ",4);                            /* 10.06.14 ... */
  inter(&Cbl,NULL,"DROP TABLE BEGIN_TPCB;",NULL,NULL);
  /* Error check have skipped in purpose */                /* ... 10.06.14 */
  if ( command == TPCB_MASTER ) /* Master */
    {
    strncpy(Cbl.Command,"COMT",4);
    inter(&Cbl,NULL,NULL,NULL,NULL);
    CreateDatabase();
    Trans();
    strncpy(Cbl.Command,"    ",4);
    inter(&Cbl,NULL,"CREATE OR REPLACE TABLE BEGIN_TPCB(TPCB INT);",NULL,NULL); /* 10.06.14 */
    ERROR(2);
    inter(&Cbl,NULL,"INSERT INTO BEGIN_TPCB(TPCB) VALUES (0);",NULL,NULL);
    ERROR(2);
    strncpy(Cbl.Command,"COMT",4);
    inter(&Cbl,NULL,NULL,NULL,NULL);
    }
  else                     /* Slave */
    {
    printf("\nWaiting for master...");                     /* 10.06.14 ... */
    fflush(stdout);                                        /* ... 10.06.14 */
    do
      {
      memcpy(Cbl.Command,"SLCT",4);
      Cbl.LnBufRow = 4;
      inter(&Cbl,NULL,"SELECT TPCB FROM BEGIN_TPCB;",NULL,buffer);
      } while ( Cbl.CodErr );
    printf("Master started!");
    fflush(stdout);                                        /* 10.06.14 */
    Trans(); /* 09.04.03 */ 
    }
  printf("\nBegin test B (press any key for exit)");
  first=time(NULL);
  do
    {
    clock_t _first,_second; /* 28.11.01 */ 
    time_t tim; /* 28.11.01 */ 
    for (j=0; j<100 ; j++ )
      {
      nb = 0;
      nt = RandomNumber( 0L , 9L );
      na = RandomNumber( 0L , naccounts - 1 );
      delta = RandomNumber( 0L , 999999L );
      rnd = RandomNumber( 0L , 999L );
      if( rnd < 500 )
        delta=-delta;
      Bid = nb;
      Tid = nt;
      Aid = na;
      Delta = delta;
      tim=time(NULL); /* 28.11.01 */ 
      _first=clock(); /* 28.11.01 */ 
      DoOne();
      _second=clock(); /* 28.11.01 */ 
      if ( trace )
        AddTraceString (tim,_first, _second);
      tr++;
      trl++;
      }
    second=time(NULL);

    tm=(L_LONG)difftime(second,first);
    if(tm-tmold>10)
      {
      tml = tm - tmold;
      printf("\nProcessing: %2ld%%, Time: %2ld:%2ld, Loc Speed: %.2f tps, Glob Speed: %.2f tps.",tm*100/worktime,tm/60,tm%60,trl/tml,tr/tm);
      if (fout)
      {
        fprintf(fout,"\nProcessing: %2ld%%, Time: %2ld:%2ld, Loc Speed: %.2f tps, Glob Speed: %.2f tps.",tm*100/worktime,tm/60,tm%60,trl/tml,tr/tm);
        fflush(fout);
      }
      tmold=tm;
      trl=0;
      }
    }while( tm < worktime );

  if ( trace ) /* 28.11.01 */ 
    {
    FILE *f;
    struct _trc *t;
    char str[100], *p;
    int i;
    float cent, full;

    for ( t = trc, i=0, cent=0 ; t ; t = t->next, i++ )
      cent = cent + ((float)(t->_second-t->_first))/(float)CLOCKS_PER_SEC;
    cent = (float)0.01; /*(cent / i) / 5;*/

    f = fopen ( tracefile, "wt" );
    
    for ( t = trc ; t ; t = t->next )
      {
      strcpy ( str, ctime( &(t->tim)) );

      if ( (p=strchr(str,'\n')) != NULL )
        *p = 0;
      full = ((float)(t->_second-t->_first))/(float)CLOCKS_PER_SEC;
      fprintf ( f, "%s - %f sec. ", str, full );
      for ( i = 0 ; i < full/cent ; i++ )
        fprintf ( f, "o" );
      fprintf ( f, "\n" );
      }
    fclose (f);
    } /* if */

  printf("\rProcessing: 100%%\n The end - OK!\n");
  if (fout)
  {
   fprintf(fout,"\rProcessing: 100%%\n The end - OK!\n");
   fflush(fout);
  }

  if ( command == TPCB_MASTER )
    {
    printf("\nResults:\n");
    if (fout)
    {
     fprintf(fout,"\nResults:\n");
     fflush(fout);
    }

    memcpy(Cbl.Command,"SLCT",4);

    Cbl.LnBufRow = 4 + 16 + 16;
    inter(&Cbl,NULL,"SELECT COUNT(*),MIN(TIME),MAX(TIME) FROM HISTORY;",
          NULL,buffer);
    ERROR(3);

    printf("\n\tTransactions:       %ld\n",buffer[0]);
    if (fout)
    {
     fprintf(fout,"\n\tTransactions:       %ld\n",buffer[0]);
     fflush(fout);
    }
    nhistory = buffer[0];

    SUBDECIMAL(*(L_DECIMAL*)(((L_CHAR*)buffer)+20),(((L_CHAR*)buffer)+4),df);
    DecToLong(df,&diff,1);

    printf("\tWorking time:       %ld seconds\n",diff/100);
    if (fout)
    {
     fprintf(fout,"\tWorking time:       %ld seconds\n",diff/100);
     fflush(fout);
    }

  #ifdef DOS
    printf("        ͻ\n");
    printf("           Speed = %6.3f tps   \n",nhistory/((L_REAL)diff/100.0));
    printf("        ͼ\n");
    if (fout)
    {
     fprintf(fout,"        ͻ\n");
     fprintf(fout,"           Speed = %6.3f tps   \n",nhistory/((L_REAL)diff/100.0));
     fprintf(fout,"        ͼ\n");
     fflush(fout);
    }
  #else
    printf("        **************************\n");
    printf("        *   Speed = %6.3f tps   *\n",nhistory/((L_REAL)diff/100.0));
    printf("        **************************\n");
    if (fout)
    {
     fprintf(fout,"        **************************\n");
     fprintf(fout,"        *   Speed = %6.3f tps   *\n",nhistory/((L_REAL)diff/100.0));
     fprintf(fout,"        **************************\n");
     fflush(fout);
    }
  #endif

    Cbl.LnBufRow = 16;
    inter(&Cbl,NULL,"SELECT SUM(DELTA) FROM HISTORY;",NULL,buffer);
    ERROR(3);
    ADDDECIMAL(*(L_DECIMAL*)((L_CHAR *)buffer),DeltaOld,DeltaOld);
    DECTOSTR(DeltaOld,mess,20,2,0);
    printf("\tHISTORY.Delta     = %s\n",mess);
    if (fout)
    {
     fprintf(fout,"\tHISTORY.Delta     = %s\n",mess);
     fflush(fout);
    }

    Cbl.LnBufRow = 16;
    inter(&Cbl,NULL,"SELECT SUM(BBALANCE) FROM BRANCHES;",NULL,buffer);
    ERROR(4);
    DECTOSTR(*(L_DECIMAL*)(L_CHAR *)buffer,mess,20,2,0);
    printf("\tBRANCHES.Bbalance = %s\n",mess);
    if (fout)
    {
     fprintf(fout,"\tBRANCHES.Bbalance = %s\n",mess);
     fflush(fout);
    }

    Cbl.LnBufRow = 16;
    inter(&Cbl,NULL,"SELECT SUM(TBALANCE) FROM TELLERS;",NULL,buffer);
    ERROR(5);
    DECTOSTR(*(L_DECIMAL*)(L_CHAR *)buffer,mess,20,2,0);
    printf("\tTELLERS.Tbalance  = %s\n",mess);
    if (fout)
    {
     fprintf(fout,"\tTELLERS.Tbalance  = %s\n",mess);
     fflush(fout);
    }

    Cbl.LnBufRow = 16;
    inter(&Cbl,NULL,"SELECT SUM(ABALANCE) FROM ACCOUNTS;",NULL,buffer);
    ERROR(6);
    DECTOSTR(*(L_DECIMAL*)(L_CHAR *)buffer,mess,20,2,0);
    printf("\tACCOUNTS.Abalance = %s\n",mess);
    if (fout)
    {
     fprintf(fout,"\tACCOUNTS.Abalance = %s\n",mess);
     fflush(fout);
    }

    memcpy(Cbl.Command,"    ",4);

    sprintf(mess1,"INSERT INTO OLD_HISTORY (DELTA) VALUES (%s);",mess);
    inter(&Cbl,NULL,mess1,NULL,NULL);
    }
  memcpy(Cbl.Command,"CLOS",4);
  inter(&Cbl,"SYSTEM/MANAGER",NULL,NULL,NULL);

  if ( fout )
    fclose( fout );
  exit(0);
}

/*
	    --- Creates and initializes. ---
*/
void CreateDatabase( void )
{
  printf("\nTest old tables...");
  if (!CheckIfTablesOk())                                  /* 10.06.14 ... */
  {
    DropTables();
    CreateTables();
    FullTables();
  }
  else
  {
    printf("\nTables already exists, don't created.");
    printf("\nDelete from history...");
    memcpy(Cbl.Command,"    ",4);
    /* 28.11.01... */
    /* inter(&Cbl,NULL,"DELETE FROM HISTORY;",NULL,NULL); */  
    inter(&Cbl,NULL,"DROP TABLE HISTORY;",NULL,NULL);
    inter(&Cbl,NULL,"CREATE TABLE HISTORY (Tid INT,Bid INT,Aid INT,delta INT,time DATE,filler char(18)) MAXISN 70000 MAXROW 4096;",NULL,NULL);
    /* ...28.11.01 */ 
    ERROR(7);
    DeleteFromTables();
    FullTables();
    printf("OK");
  }                                                        /* ... 10.06.14 */
}

#ifdef STORE_PROC /* 17.09.98 */
/*-----------------09-05-95 10:51am-----------------

  --------------------------------------------------*/
L_BOOL Trans(void)
  {
TCBL Cbl_T;
  memset(&Cbl_T,0,sizeof(TCBL));
  memcpy(Cbl_T.Command,"OPEN",4);
  memcpy(Cbl_T.Node,"    ",4);
  Cbl_T.PrzExe = 0; /* Autocommit */
  inter(&Cbl_T,"SYSTEM/MANAGER",NULL,NULL,NULL);
  if(Cbl_T.CodErr)
    return L_TFALSE;
  memcpy(Cbl_T.Command,"    ",4);

  if(!DropProc(&Cbl_T, q, "TPCB_ONE", L_TTRUE, &LogId))
    return L_TFALSE;

  if(!CreateProc(&Cbl_T,"tpcb_one.sql"))
    return L_TFALSE;

  memcpy(Cbl_T.Command,"CLOS",4);
  inter(&Cbl_T,NULL,NULL,NULL,NULL);
  if(Cbl_T.CodErr)
    return L_TFALSE;
  else
    return L_TTRUE;

  }
/*
	    --- Exexutes a single TCP BM B transaction. ---
*/
L_LONG DoOne(L_REAL tr, L_REAL trl)
{
static int SkipNum = 0; /* 17.09.98 */
begin: /* 17.09.98 */
  sprintf(q,"TPCB_ONE( %ld, %ld, %ld, %ld)", Delta, Aid, Tid, Bid);
  if(!ExeProc(&Cbl, q, SPOutBuf, L_TTRUE))
  {
  if ( Cbl.CodErr == 1600 || Cbl.CodErr == 142 )
    {
    printf ( "Warning: CodErr = %d , %d transaction skipped\n" ,
             Cbl.CodErr , ++SkipNum );
    goto begin;
    }
   printf("\nError in transaction. tr = %13.7g, trl = %13.7g\n", tr, trl);
   printf("SP call: TPCB_ONE( %ld, %ld, %ld, %ld)\n", Delta, Aid, Tid, Bid);
   printf("Channel %ld Linter error %lu sys error %lu\n",
           (L_LONG)Cbl.NumChan, Cbl.CodErr, Cbl.SysErr);
   if(Cbl.CodErr == 7201)
    show_except(Cbl.SysErr);
   if(Cbl.CodErr == 0)
    AnalizeResult(SPOutBuf);

   exit(-1);

  }
  memcpy(Cbl.Command,"COMT",4);
  inter(&Cbl,NULL,NULL,NULL,NULL);
  if ( Cbl.CodErr == 6111 )
    return Abalance;
  ERROR(17);
  return Abalance;

}

void LError(void)
{
  printf("\nError: %s\nErrcode = %d",ErrString[ErrorCode-1], Cbl.CodErr);
  strncpy(Cbl.Command,"COMT",4);
  inter(&Cbl,NULL,NULL,NULL,NULL);
  exit(1);
}

#else /* STORE_PROC 17.09.98 */
/*-----------------09-05-95 10:51am-----------------

  --------------------------------------------------*/
void Trans(void)
  {
  memcpy(Cbl.Command,"TRAN",4);
  memcpy(Cbl.Node,"    ",4);
  Cbl.LnBufRow = 2048;
  inter(&Cbl,NULL,"UPDATE ACCOUNTS SET Abalance=Abalance+?  WHERE Aid=?;",
							      NULL,UpAcc);
    ERROR(11);
  Cbl.LnBufRow = 2048;
  inter(&Cbl,NULL,"SELECT Abalance FROM accounts WHERE Aid=?;",
							      NULL,SelAbl);
    ERROR(11);
  Cbl.LnBufRow = 2048;
  inter(&Cbl,NULL,"UPDATE TELLERS SET Tbalance=Tbalance+? WHERE Tid=?;",
							      NULL,UpTell);
    ERROR(11);
  Cbl.LnBufRow = 2048;
  inter(&Cbl,NULL,"UPDATE BRANCHES SET Bbalance=Bbalance+? WHERE Bid=?;",
							      NULL,UpBran);
    ERROR(11);
  Cbl.LnBufRow = 2048;
  inter(&Cbl,NULL,
      "INSERT INTO HISTORY(Tid,Bid,Aid,delta,time) VALUES (?,?,?,?,SYSDATE);",
      NULL,InsHis);
  /* Parameter binding */
  inter_bind_num(&Cbl,UpAcc,1,LIN_INT_TYP,&len1,&Delta);
  inter_bind_num(&Cbl,UpAcc,2,LIN_INT_TYP,&len2,&Aid);

  inter_bind_num(&Cbl,SelAbl,1,LIN_INT_TYP,&len2,&Aid);

  inter_bind_num(&Cbl,UpTell,1,LIN_INT_TYP,&len1,&Delta);
  inter_bind_num(&Cbl,UpTell,2,LIN_INT_TYP,&len3,&Tid);

  inter_bind_num(&Cbl,UpBran,1,LIN_INT_TYP,&len1,&Delta);
  inter_bind_num(&Cbl,UpBran,2,LIN_INT_TYP,&len4,&Bid);

  inter_bind_num(&Cbl,InsHis,1,LIN_INT_TYP,&len3,&Tid);
  inter_bind_num(&Cbl,InsHis,2,LIN_INT_TYP,&len4,&Bid);
  inter_bind_num(&Cbl,InsHis,3,LIN_INT_TYP,&len2,&Aid);
  inter_bind_num(&Cbl,InsHis,4,LIN_INT_TYP,&len1,&Delta);
  }
/*
	    --- Exexutes a single TCP BM B transaction. ---
*/
L_LONG DoOne()
{
  static L_LONG SkipNum = 0; /* 17.09.98 */                /* 10.06.14 ... */
  L_LONG        rndExecOrder;
  L_LONG        j;

  memcpy(Cbl.Command, "EXEC", 4);
  memcpy(Cbl.Node, "    ", 4);
  Cbl.LnBufRow = 20;
  rndExecOrder = RandomNumber(1, MAX_CASE_NUM);
  for (j = 0; j < NUM_REP_FOR_ERROR; j++)
  {
    switch(rndExecOrder)
    {
      case 1:
#if _VER_MAX >= 500
        inter(&Cbl,NULL,UpAcc,NULL,NULL);
#else
        inter(&Cbl,UpAcc,NULL,NULL,NULL);
#endif
        if ( Cbl.CodErr == 1600 || Cbl.CodErr == 142 )
        {
          printf ( "Warning: CodErr = %d , %d transaction skipped\n" ,
                  Cbl.CodErr , ++SkipNum );
          continue;
        }
        ERROR(12); /* UPDATE ACCOUNTS */				/* 21.09.05 */
        break;
        case 2:
#if _VER_MAX >= 500
        inter(&Cbl,NULL,SelAbl,NULL,NULL);
#else
        inter(&Cbl,SelAbl,NULL,NULL,NULL);
#endif
        if ( Cbl.CodErr == 1600 || Cbl.CodErr == 142 )
        {
          printf ( "Warning: CodErr = %d , %d transaction skipped\n" ,
                  Cbl.CodErr , ++SkipNum );
          continue;
        }
        ERROR(13); /* SELECT FROM ACCOUNTS */				/* 21.09.05 */
        break;
      case 3:
#if _VER_MAX >= 500
        inter(&Cbl,NULL,UpTell,NULL,NULL);
#else
        inter(&Cbl,UpTell,NULL,NULL,&Abalance);
#endif
        if ( Cbl.CodErr == 1600 || Cbl.CodErr == 142 )
        {
          printf ( "Warning: CodErr = %d , %d transaction skipped\n" ,
                  Cbl.CodErr , ++SkipNum );
          continue;
        }
        ERROR(14); /* UPDATE TELLERS */				/* 21.09.05 */
        break;
      case 4:
#if _VER_MAX >= 500
        inter(&Cbl,NULL,UpBran,NULL,NULL);
#else
        inter(&Cbl,UpBran,NULL,NULL,NULL);
#endif
        if ( Cbl.CodErr == 1600 || Cbl.CodErr == 142 )
        {
          printf ( "Warning: CodErr = %d , %d transaction skipped\n" ,
                  Cbl.CodErr , ++SkipNum );
          continue;
        }
        ERROR(15); /* UPDATE BRANCHES */				/* 21.09.05 */
        break;
    } /* switch(rndExecOrder) */
    break;
  } /* for (j = 0; j < NUM_REP_FOR_ERROR; j++) */
  for (j = 0; j < NUM_REP_FOR_ERROR; j++)
  {
#if _VER_MAX >= 500
    inter(&Cbl,NULL,InsHis,NULL,NULL);
#else
    inter(&Cbl,InsHis,NULL,NULL,NULL);
#endif
    if ( Cbl.CodErr == 1600 || Cbl.CodErr == 142 )
    {
      printf ( "Warning: CodErr = %d , %d transaction skipped\n" ,
              Cbl.CodErr , ++SkipNum );
      continue;
    }
    break;
  }
  ERROR(16); /* INSERT INTO HISTORY */				/* 21.09.05 */
  memcpy(Cbl.Command,"COMT",4);
  for (j = 0; j < NUM_REP_FOR_ERROR; j++)
  {
    inter(&Cbl,NULL,NULL,NULL,NULL);
    if ( Cbl.CodErr == 1600 || Cbl.CodErr == 142 || Cbl.CodErr == 6111)
    {
      printf ( "Warning: CodErr = %d , %d transaction skipped\n" ,
              Cbl.CodErr , ++SkipNum );
      continue;
    }
    break;
  }                                                        /* ... 10.06.14 */
  ERROR(17); /* COMMIT */					/* 21.09.05 */

  return Abalance;
}
#endif /* STORE_PROC 17.09.98 */

void Error(void)
{
  printf("\nError: %s\nErrcode = %d",ErrString[ErrorCode-1], Cbl.CodErr);
  strncpy(Cbl.Command,"COMT",4);
  inter(&Cbl,NULL,NULL,NULL,NULL);
  exit(1);
}

/*
**
**         *** Returns uniform random number between x and y. ***
**
*/
L_LONG RandomNumber(L_ULONG x, L_ULONG y){
  L_ULONG r;
  r = (L_ULONG) rand();
  r *= (L_ULONG) rand();
  return (r % (y - x + 1) + x);
}


void DropTables()                                          /* 10.06.14 ... */
{
  memset(DeltaOld,0,16);
  memcpy(Cbl.Command,"    ",4);
  printf("\nDrop old tables...");
  inter(&Cbl,NULL,"DROP TABLE BRANCHES;",NULL,NULL);
  inter(&Cbl,NULL,"DROP INDEX TID ON TELLERS;",NULL,NULL);
  inter(&Cbl,NULL,"DROP TABLE TELLERS;",NULL,NULL);
  inter(&Cbl,NULL,"DROP INDEX AID ACCOUNTS;",NULL,NULL);
  inter(&Cbl,NULL,"DROP TABLE ACCOUNTS;",NULL,NULL);
  inter(&Cbl,NULL,"DROP TABLE HISTORY;",NULL,NULL);
  printf("OK");
}


void CreateTables()
{
  printf("\nCreate table branches...");
  inter(&Cbl,NULL,"CREATE TABLE BRANCHES(Bid INT, Bbalance INT, filler char(92));",
        NULL,NULL);
  ERROR(8);
  printf("OK");
  printf("\nCreate table tellers...");
  inter(&Cbl,NULL,"CREATE TABLE TELLERS(Tid INT,Bid INT,Tbalance INT,filler char(88));",NULL,NULL);
  ERROR(8);
  printf("OK");
  printf("\nCreate table accounts...");
  if ( naccounts <= 1000 )
	{
    inter(&Cbl,NULL,"CREATE TABLE ACCOUNTS(Aid INT,Bid INT,Abalance INT,filler char(88));",NULL,NULL);
    ERROR(8);
    printf("OK");
  }
  else if ( naccounts <= 10000 )
  {
    inter(&Cbl,NULL,"CREATE TABLE ACCOUNTS(Aid INT,Bid INT,Abalance INT,filler char(88))MAXISN 10000 MAXROW 10000;",NULL,NULL);
    ERROR(8);
	}
  else
  {
    inter(&Cbl,NULL,"CREATE TABLE ACCOUNTS (Aid INT,Bid INT,Abalance INT,filler char(88)) MAXISN 100000 MAXROW 100000;",
          NULL, NULL);
    ERROR(8);
  }
  memcpy(Cbl.Command,"    ",4);
  inter(&Cbl,NULL,"CREATE INDEX TID ON TELLERS;",NULL,NULL);
  ERROR(10);
  inter(&Cbl,NULL,"CREATE INDEX AID ON ACCOUNTS;",NULL,NULL);
  ERROR(10);
  printf("\nCreate table history...");
  inter(&Cbl,NULL,"CREATE TABLE HISTORY (Tid INT,Bid INT,Aid INT,delta INT,time DATE,filler char(18)) MAXISN 70000 MAXROW 4096;",NULL,NULL);
  ERROR(8);
  printf("OK");
}

void FullTables()
{
  L_LONG tmp = 0;
  L_CHAR str[100];
  L_CHAR st[11] = "-\\|/-+*#*+";                         /* 24.04.00 */

  printf("\nInsert new values into tables...");
  for ( i=0 ; i < nbranches*tps ; i++ )
  {
    memcpy(Cbl.Command,"    ",4);
    sprintf(str,"INSERT INTO BRANCHES(Bid,Bbalance) VALUES (%d,0);",i);
    inter(&Cbl,NULL,str,NULL,NULL);
    ERROR(9);
    printf("OK");
	}
  for ( i=0 ; i < ntellers*tps ; i++ )
  {
    memcpy(Cbl.Command,"    ",4);
    sprintf(str, "INSERT INTO TELLERS(Tid,Bid,Tbalance) VALUES (%d,%d,0);",
            i,i/ntellers);
    inter(&Cbl,NULL,str,NULL,NULL);
    ERROR(9);
  }
  for ( i=0 ; i < naccounts*tps ; i++ )
  {
    memcpy(Cbl.Command,"    ",4);
    sprintf(str,"INSERT INTO ACCOUNTS(Aid,Bid,Abalance) VALUES (%d,%d,0);",
            i,i/naccounts );
	  inter(&Cbl,NULL,str,NULL,NULL);
	  ERROR(9);
	  if( !((i+1) % 10) )
    {
	    printf("%c\b",st[(tmp++)]);
      fflush(stdout);
	    if( tmp == 10 )
	      tmp = 0;
      if( !((i+1) % 100) )
      {
        memcpy(Cbl.Command,"COMT",4);
        inter(&Cbl,NULL,NULL,NULL,NULL);
        if ( Cbl.CodErr != 6111 )
        ERROR(17);
	      printf("-");
        fflush(stdout);
	      if( !((i+1) % 5000) )
	      printf("%ld\n",i+1);
        fflush(stdout);
	    }
	  }
	}
  printf("\nInsert done.");
}

void DeleteFromTables()
{
  memcpy(Cbl.Command,"    ",4);
  inter(&Cbl, NULL, "DELETE from BRANCHES;", NULL, NULL);
  ERROR(15);
  inter(&Cbl, NULL, "DELETE from TELLERS;", NULL, NULL);
  ERROR(14);
  inter(&Cbl, NULL, "DELETE from ACCOUNTS;", NULL, NULL);
  ERROR(12);
}

L_BOOL CheckIfTablesOk()
{
  L_BOOL tblFlag = 1;

  memcpy(Cbl.Command, "SLCT", 4);
  Cbl.LnBufRow = 16;
  inter(&Cbl, NULL, "SELECT COUNT(*) FROM BRANCHES;", NULL, buffer);
  if (Cbl.CodErr)
    tblFlag = 0;
  inter(&Cbl, NULL, "SELECT COUNT(*) FROM TELLERS;", NULL, buffer);
  if (Cbl.CodErr)
    tblFlag = 0;
  inter(&Cbl, NULL, "SELECT COUNT(*) FROM ACCOUNTS;", NULL, buffer);
  if (Cbl.CodErr)
    tblFlag = 0;
  inter(&Cbl, NULL, "SELECT COUNT(*) FROM HISTORY;", NULL, buffer);
  if (Cbl.CodErr)
    tblFlag = 0;
  if (tblFlag)
  {
    inter(&Cbl, NULL, "SELECT COUNT(*) FROM ACCOUNTS;", NULL, buffer);
    if (naccounts != buffer[0])
      tblFlag = 0;
  }

  return tblFlag;
}                                                          /* ... 10.06.14 */

