 /***************************************************************************
 *     RELEX, Inc. Linter interface to Tcl and Tk (version 1.0)             *
 *  This is unpublished propietary source code of RELEX, Inc.               *
 *  The copyright notice does not evidence any actual or intended           *
 *  publication of such source code.                                        *
 *                                                                          *
 *                 Copyright (c) 1996-2006 RELEX Inc.                       *
 *                                                                          *
 *                   This software was developed by                         *
 *                                                                          *
 *                            RELEX, Inc.                                   *
 *                         Voronezh, Russia                                 *
 *                                                                          *
 *                        All rights reserved.                              *
 ****************************************************************************
 AUTHOR: Pavel Pasechnik (Pasec)
 ----------------------------------------------------------------------------
 PURPOSE: interface to Tcl and Tk
 ----------------------------------------------------------------------------
 REVISION HISTORY:

  12.02.97 -- creation date
  17.09.98 -- ALX : change return value from void to int, add return TCL_OK
  16.12.99 -- Galizin: new linter_error
  03.01.01 -- Galizin: remove snprintf
  07.01.01 -- Pasec (out parameters in prepared query, MAX_ID_LEN)
  03.11.06 -- Galizin ( Using internal types )
*/  

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "configtcl.h"
#include "tcl.h"
#include "linapi.h"

int LinterCmdInit(Tcl_Interp * interp);

static char * ErrOpt[] = {"connect","cursor","stmt"};

char * str = NULL;

#define ERR_OPT_CONNECT  0 
#define ERR_OPT_CURSOR   1 
#define ERR_OPT_STMT     2 
#define MAX_ERR_OPT      3 

#define MAX_FETCH_DIR  7
static struct {
  char * str;
  int cod;
 } FetchDirection[MAX_FETCH_DIR] = 
                     {
                       {"toNext",       toNext},
                       {"toPrevious",   toPrevious},
                       {"toFirst",      toFirst},
                       {"toLast",       toLast},
                       {"toAbsNumber",  toAbsNumber},
                       {"toRelNumber",  toRelNumber},
                       {"toLast",       toLast}
                     };

/* 07.01.01 ... */
#define MAX_OUT_TYPE  11
static struct {
  char * str;
  int cod;
 } OutTypes[MAX_OUT_TYPE] = {
                             {"tChar",tChar},
                             {"tString",tString},
                             {"pOutput",tChar | pOutput},
                             {"pInputOutput",tChar | pInputOutput},
                             {"pInput",tChar},
                             {"tChar|pOutput",tChar | pOutput},
                             {"tString|pOutput",tString | pOutput},
                             {"tChar|pInputOutput",tChar | pInputOutput},
                             {"tString|pInputOutput",tString | pInputOutput},
                             {"tChar|pInput",tChar | pInput},
                             {"tString|pInput",tString | pInput},
  };

#define DEFAULT_OUT_TYPE -1
/* ... 07.01.01 */

#define MAX_T_MODE  6 
static struct {
  char * str;
  int cod;
 } TransModes[MAX_T_MODE] = {
                   {"mAutocommit",      mAutocommit},
                   {"mOptimistic",      mOptimistic},
                   {"mExclusive",       mExclusive},
                   {"mAutocommit|mAnsi",mAutocommit | mAnsi},
                   {"mOptimistic|mAnsi",mOptimistic | mAnsi},
                   {"mExclusive|mAnsi", mExclusive | mAnsi}};

#define MAX_CURS_OPT  23
static struct {
  char * str;
  int cod;
 } CursorOptions[MAX_CURS_OPT] = 
                     {
                     {"cSelectRowCount",cSelectRowCount},
                     {"cRowCount",cRowCount},
                     {"cColumnCount",cColumnCount},
                     {"cStmtType",cStmtType},
                     {"cCurrentRow",cCurrentRow},
                     {"cCurrentRowID",cCurrentRowID},
                     {"cExecStmt",cExecStmt},
                     {"cExecRow",cExecRow},
                     {"cAnswerDesc",cAnswerDesc},
                     {"cStmtNumber",cStmtNumber},
                     {"cParamNumber",cParamNumber},
                     {"cConnectID",cConnectID},
                     {"cAnswerSize",cAnswerSize},
                     {"cCursorName",cCursorName},
                     {"cNullIndicator",cNullIndicator},
                     {"cIsAutoInc",cIsAutoInc},
                     {"cTransMode",cTransMode},
                     {"cPriority",cPriority},
                     {"cLinCode",cLinCode},
                     {"cSysCode",cSysCode},
                     {"cStrNumber",cStrNumber},
                     {"cPosNumber",cPosNumber},
                     {"cApiCode",cApiCode}
                     };

#define MAX_STMT_OPT  7
static struct {
  char * str;
  int cod;
 } StmtOptions[MAX_STMT_OPT] = {
                    {"sParamCount",sParamCount},
                    {"sAnswerDesc",sAnswerDesc},
                    {"sColumnCount",sColumnCount},
                    {"sParamDesc",sParamDesc},
                    {"sConnectID",sConnectID},
                    {"sApiCode",sApiCode},
                    {"sIsAutoInc",sIsAutoInc}};


#define LINTCL_NO_MEM          -4
#define LINTCL_INTERNAL_ERROR  -5

#define ConvRetCod(cod) ((cod == LINAPI_NO_MEM) ? cod = LINTCL_NO_MEM : cod)   

#define MAX_ANSWER_SIZE  4096

#define FREE  1
#define BUSY  0

#define NOERRORNUMBER 1 /*16.12.99*/
#define NOERRORLIST   2 /*16.12.99*/
#define LINTERERROR   -1 /*16.12.99*/
#define OTHERERROR    -2 /*16.12.99*/

typedef struct {
  long    shift;
  int     Type;                                            /* 07.01.01 */
  int     Length;
  char *  LinkVar;
  } Var;

typedef struct tsBind {
  struct  tsBind * NextBind;
  L_WORD  CursID;                                           /* 03.11.06 */
  L_WORD  StmtID;                                           /* 03.11.06 */
  L_LONG  ParamCount;                                       /* 03.11.06 */
  Var   * LinkVar;
  L_BYTE  IsFree;                                           /* 03.11.06 */
  L_BYTE  Res[3];                                           /* 03.11.06 */
  } tsBind;

static tsBind * BindList = (tsBind *) NULL;

static tsBind * CreateBind(Tcl_Interp * interp, char * result,
                           L_WORD CursID, L_WORD StmtID)    /* 03.11.06 */
{
  tsBind * B = BindList, * PrevB = BindList;
  L_LONG lRet, ParamCount;                                  /* 03.11.06 */
  int i;

  while (B)
   {
    if (B->IsFree == FREE)
     break;
    else
    if (B->CursID == CursID && B->StmtID == StmtID)   
     return(B);
    PrevB = B;
    B = B->NextBind;
   }
  if (!B)
   {
    if (!(B = (tsBind *) Tcl_Alloc(sizeof(tsBind))))
     {
      sprintf(result,"%d nomemory",LINTCL_INTERNAL_ERROR);
      Tcl_SetResult(interp,result,TCL_VOLATILE);
      return((tsBind *) NULL); 
     };
    B->IsFree = FREE;
    B->NextBind = (tsBind *) NULL;
    B->LinkVar  = (Var *) NULL; /* ALX ... */
    B->CursID   = 0;
    B->StmtID   = 0;
    B->ParamCount  = 0;
    
    if (!BindList)
     BindList = B;
    else
     PrevB->NextBind = B;
   }
  if (lRet = LINTER_GetStatementOption(StmtID,sParamCount,0,     /* 03.11.06 */
                                       &ParamCount, NULL))
   {
    sprintf(result,"%ld",ConvRetCod(lRet));
    Tcl_SetResult(interp,result,TCL_VOLATILE);
    return((tsBind *) NULL); 
   }
  if (ParamCount)
   {
    if (!(B->LinkVar = (Var *) Tcl_Alloc(ParamCount * sizeof(Var))))
     {
      sprintf(result,"%d nomemory",LINTCL_INTERNAL_ERROR);
      Tcl_SetResult(interp,result,TCL_VOLATILE);
      return((tsBind *) NULL); 
     }
    for (i = 0; i < ParamCount; i++)
     B->LinkVar[i].LinkVar =  NULL;
   }
  B->ParamCount = ParamCount;
  B->CursID = CursID;
  B->StmtID = StmtID;
  B->IsFree = BUSY;
  return(B);
}

static tsBind * GetBind(L_WORD CursID, L_WORD StmtID)       /* 03.11.06 */
{
  tsBind * B = BindList;

  while (B)
   {
    if (B->CursID == CursID && B->StmtID == StmtID &&
        B->IsFree == BUSY)
     break;
    B = B->NextBind;
   }
  return(B);
}

static void ClearBind(Tcl_Interp * interp, L_WORD CursID, L_WORD StmtID)
{
  tsBind * B = BindList;
  int i;

  while (B)
   {
    if (B->CursID == CursID || B->StmtID == StmtID)   
     {
      if (B->LinkVar)
       {
        for (i = 0; i < B->ParamCount; i++)
         {
          if (B->LinkVar[i].LinkVar)
           {
            Tcl_Free(B->LinkVar[i].LinkVar); 
            B->LinkVar[i].LinkVar = NULL;
           }
         }
        Tcl_Free((char *)B->LinkVar);
        B->LinkVar = NULL;
       }
      B->ParamCount = 0;
      B->IsFree = FREE;
     }
    B = B->NextBind;
   }
}

static void FreeBind(Tcl_Interp * interp)
{
  tsBind * B = BindList, * NextB;
  int i;

#ifdef DEBUG
  int link_no = 0;
#endif  

  while (B)
  {
    NextB = B->NextBind;
    if (B->LinkVar)
     {
      for (i = 0; i < B->ParamCount; i++)
       {
        if (B->LinkVar[i].LinkVar)
         {
          Tcl_Free(B->LinkVar[i].LinkVar); 
          B->LinkVar[i].LinkVar = NULL;
         }
       }
      Tcl_Free((char *)B->LinkVar);
      B->LinkVar = NULL;

     } /* if */
#ifdef DEBUG
      printf("link_no = %d\n", link_no++);
#endif      
    Tcl_Free((char *) B);
    B = NextB;
  } /* end while */
  BindList = (tsBind *) NULL; 
}

static L_WORD GetFetchDir(Tcl_Interp * interp, char * result, char * str) /* 03.11.06 */
{
  int i;

  for(i = 0;
      i < MAX_FETCH_DIR && strcmp(str,FetchDirection[i].str) != 0;
      i++)
     ; /* BLANK */
     
  if (i == MAX_FETCH_DIR)
  {
    sprintf(result,"%d invalid fetch direction",
            LINTCL_INTERNAL_ERROR);
    Tcl_SetResult(interp, result, TCL_VOLATILE);
    return(LINTCL_INTERNAL_ERROR);
  }
  return(FetchDirection[i].cod); 
}

static int GetOutType(Tcl_Interp * interp, char * result, char * str)
{
  int i;

  for(i = 0;
      i < MAX_OUT_TYPE && strcmp(str,OutTypes[i].str) != 0;
      i++)
    ; 
  if (i == MAX_OUT_TYPE)
  {
    sprintf(result,"%d \"%s\" invalid type",
            LINTCL_INTERNAL_ERROR,str);
    Tcl_SetResult(interp,result,TCL_VOLATILE);
    return(LINTCL_INTERNAL_ERROR);
  }
  return(OutTypes[i].cod); 
}

static L_LONG GetTransMode(Tcl_Interp * interp, char * result, char * str)  /* 03.11.06 */
{
  int i;

  for(i = 0;
      i < MAX_T_MODE && strcmp(str,TransModes[i].str) != 0;
      i++)
    ; 
  if (i == MAX_T_MODE)
  {
    sprintf(result,"%d invalid transaction mode",
                LINTCL_INTERNAL_ERROR);
    Tcl_SetResult(interp,result,TCL_VOLATILE);
    return(LINTCL_INTERNAL_ERROR);
  }
  return(TransModes[i].cod); 
}

static int GetCursOpt(Tcl_Interp * interp, char * result, char * str)
{
  int i;

  for(i = 0;
      i < MAX_CURS_OPT && strcmp(str,CursorOptions[i].str) != 0;
      i++)
    ; 
  if (i == MAX_CURS_OPT)
  {
    sprintf(result,"%d invalid cursor option",
            LINTCL_INTERNAL_ERROR);
    Tcl_SetResult(interp,result,TCL_VOLATILE);
    return(LINTCL_INTERNAL_ERROR);
  }
  return(CursorOptions[i].cod); 
}

static int GetStmtOpt(Tcl_Interp * interp, char * result, char * str)
{
  int i;

  for(i = 0;
      i < MAX_STMT_OPT && strcmp(str,StmtOptions[i].str) != 0;
      i++)
    ;
  if (i == MAX_STMT_OPT)
  {
    sprintf(result,"%d invalid statement option",
                LINTCL_INTERNAL_ERROR);
    Tcl_SetResult(interp,result,TCL_VOLATILE);
    return(LINTCL_INTERNAL_ERROR);
  }
  return(StmtOptions[i].cod); 
}

/*16.12.99...*/
static void setError(Tcl_Interp * interp, L_WORD connId, L_WORD cursId,  /* 03.11.06 */
                L_WORD stmtId, short retCode)
{
  L_LONG codError,sysError,apiError;                        /* 03.11.06 */
  L_SWORD msgLen = 80;                                      /* 03.11.06 */
  L_CHAR message[80] = "";
  if (retCode < 0)
  {
    LINTER_Error(connId, cursId, stmtId, &apiError, 
                   &codError, &sysError, message, &msgLen);
    Tcl_SetVar(interp,"linter_message",message,0);

    sprintf(message,"%d", (int)codError);                   /* 03.11.06 */
    Tcl_SetVar(interp,"linter_coderror",message,0);
    sprintf(message,"%d", (int)apiError);                   /* 03.11.06 */
    Tcl_SetVar(interp,"linter_apierror",message,0);
    sprintf(message,"%d", (int)sysError);                   /* 03.11.06 */
    Tcl_SetVar(interp,"linter_syserror",message,0);
  };
  sprintf(message,"%d",(int)retCode);                       /* 03.11.06 */
  Tcl_SetVar(interp,"linter_retval",message,0);
};

static void resetError(Tcl_Interp * interp)
{
  Tcl_SetVar(interp,"linter_retcode","0",0);
};
/*...16.12.99*/


int LinterConnectCmd(ClientData clientData, Tcl_Interp * interp, int argc, char *argv[])
{
 L_WORD nConnID  = 0;
 int    nNameLen = 0;
 int    nPassLen = 0;
 L_LONG lRet;                                               /* 03.11.06 */
 L_LONG Mode = 0;                                           /* 03.11.06 */
 char *cServName = (char *) NULL;
 char *cPassWord = (char *) NULL;
 char result[MAX_ANSWER_SIZE];

 if (argc < 2)
  {
   sprintf(result,"%d wrong # args",LINTCL_INTERNAL_ERROR);
   Tcl_SetResult(interp,result, TCL_VOLATILE);
   return(TCL_ERROR); 
  }
 if (argc >= 3)
  if (Tcl_GetInt(interp,argv[2],&nNameLen) != TCL_OK)
   return(TCL_ERROR);
 if (argc >= 4)
  cPassWord = argv[3];
 if (argc >= 5)
  if (Tcl_GetInt(interp,argv[4],&nPassLen) != TCL_OK)
   return(TCL_ERROR);
 if (argc >= 6)
  cServName = argv[5]; 
 if (argc >= 7)
  if ( (Mode = GetTransMode(interp,result,argv[6])) == LINTCL_INTERNAL_ERROR)
   return(TCL_ERROR);
 if (lRet = LINTER_Connect(argv[1],(L_SWORD) nNameLen,
                           cPassWord,(L_SWORD)nPassLen,
                           cServName, Mode, &nConnID))
  {
   sprintf(result,"%d", (int)(ConvRetCod(lRet)));           /* 03.11.06 */
   setError(interp, nConnID, 0, 0, LINTERERROR);/*16.12.99*/
   Tcl_SetResult(interp, result, TCL_VOLATILE);
   return(TCL_OK); 
  }
 sprintf(result,"%d", (int)nConnID);                        /* 03.11.06 */
 setError(interp, 0, 0, 0, NOERRORNUMBER);/*16.12.99*/
 Tcl_SetResult(interp, result, TCL_VOLATILE);
 return(TCL_OK);  
}

int LinterCloseConnectCmd(ClientData clientData, Tcl_Interp * interp, int argc, char *argv[])
{
 int  nConnID;
 L_LONG lRet;                                               /* 03.11.06 */
 char result[MAX_ANSWER_SIZE] = "0";

 if (argc < 2)
  {
   sprintf(result,"%d wrong # args",LINTCL_INTERNAL_ERROR);
   Tcl_SetResult(interp,result, TCL_VOLATILE);
   return(TCL_ERROR); 
  }
 if (Tcl_GetInt(interp,argv[1],&nConnID) != TCL_OK)
  return(TCL_ERROR);
 if (lRet = LINTER_CloseConnect((L_WORD)nConnID))           /* 03.11.06 */
  {
   sprintf(result, "%d", (int)(ConvRetCod(lRet)) );         /* 03.11.06 */
   setError(interp, nConnID, 0, 0, LINTERERROR);/*16.12.99*/
   Tcl_SetResult(interp, result, TCL_VOLATILE);
   return(TCL_OK); 
  }
 setError(interp, 0, 0, 0, NOERRORNUMBER);/*16.12.99*/
 Tcl_SetResult(interp, result, TCL_VOLATILE);
 return(TCL_OK);
}

int LinterCommitCmd(ClientData clientData, Tcl_Interp * interp, int argc, char *argv[])
{
 int  nConnID;
 L_LONG lRet;                                               /* 03.11.06 */
 char result[MAX_ANSWER_SIZE] = "0";

 if (argc < 2)
  {
   sprintf(result,"%d wrong # args",LINTCL_INTERNAL_ERROR);
   Tcl_SetResult(interp,result, TCL_VOLATILE);
   return(TCL_ERROR); 
  }
 if (Tcl_GetInt(interp,argv[1],&nConnID) != TCL_OK)
  return(TCL_ERROR);
 if (lRet = LINTER_Commit((L_WORD)nConnID,NULL,NULL))       /* 03.11.06 */
  {
   sprintf(result,"%d",(int)(ConvRetCod(lRet)));            /* 03.11.06 */
   setError(interp,nConnID,0,0,LINTERERROR);/*16.12.99*/
   Tcl_SetResult(interp,result,TCL_VOLATILE);
   return(TCL_OK); 
  }
 setError(interp,0,0,0,NOERRORNUMBER);/*16.12.99*/
 Tcl_SetResult(interp,result,TCL_VOLATILE);
 return(TCL_OK);
}

int LinterRollBackCmd(ClientData clientData, Tcl_Interp * interp, int argc, char *argv[])
{
 int  nConnID;
 L_LONG lRet;                                               /* 03.11.06 */
 char result[MAX_ANSWER_SIZE] = "0";

 if (argc < 2)
  {
   sprintf(result,"%d wrong # args",LINTCL_INTERNAL_ERROR);
   Tcl_SetResult(interp,result, TCL_VOLATILE);
   return(TCL_ERROR); 
  }
 if (Tcl_GetInt(interp,argv[1],&nConnID) != TCL_OK)
  return(TCL_ERROR);
 if (lRet = LINTER_RollBack((L_WORD)nConnID,NULL,NULL))     /* 03.11.06 */
  {
   sprintf(result,"%d", (int)(ConvRetCod(lRet)) );
   setError(interp,nConnID,0,0,LINTERERROR);/*16.12.99*/
   Tcl_SetResult(interp,result,TCL_VOLATILE);
   return(TCL_OK); 
  }
 setError(interp,0,0,0,NOERRORNUMBER);/*16.12.99*/
 Tcl_SetResult(interp,result,TCL_VOLATILE);
 return(TCL_OK);
}

int LinterErrorCmd(ClientData clientData, Tcl_Interp * interp, int argc, char *argv[])
{
/*16.12.99...*/
  int codError,sysError,apiError,retCode;
  char * pChar; /*string OBJ value*/
  char result[MAX_ANSWER_SIZE];
  Tcl_Obj* listObj; /*OBJ for parce*/
  Tcl_Obj** arrayObj; /*array list elements*/
  int countObj;       /*count element in array obj*/
  int i;

/*get previon return code*/
  if ( (pChar=(char *)Tcl_GetVar(interp,"linter_retval",0)) == NULL)
  {
    sprintf(result,"%d connection is not opened",LINTCL_INTERNAL_ERROR);
    Tcl_SetResult(interp,result,TCL_VOLATILE);
    return(TCL_ERROR);
  };

  if (Tcl_GetInt(interp,pChar,&retCode) != TCL_OK)
    return(TCL_ERROR);

  switch(retCode)
  {
    case NOERRORLIST:
        if (argc == 2) /* argv[1] - result previon command*/
            {
            /*parce secons argv */
            if ((listObj=Tcl_NewStringObj(argv[1],-1)) == NULL)
                return(TCL_ERROR);
            if (Tcl_ListObjGetElements(interp,listObj,&countObj,&arrayObj) !=
                                            TCL_OK)
                return(TCL_ERROR);
            if (countObj == 2) /* return one element*/
                Tcl_SetResult(interp,Tcl_GetStringFromObj(arrayObj[1],NULL),TCL_VOLATILE);
            else        /*return list element*/    
                for (i=1;i<countObj;i++)        
                    Tcl_AppendElement(interp,
                                    Tcl_GetStringFromObj(arrayObj[i],NULL) );
            }
        else /*comman execut "eval"*/
            if (argc == 3) /* return one element*/
                Tcl_SetResult(interp,argv[2],TCL_VOLATILE);
            else /*return list elements*/
                for (i=2;i<argc;i++)    
                    Tcl_AppendElement(interp,argv[i]);
        break;
    case LINTERERROR:
        if ( (pChar=(char *)Tcl_GetVar(interp,"linter_coderror",0)) == NULL)
            return(TCL_ERROR);
        if (Tcl_GetInt(interp,pChar,&codError) != TCL_OK)
            return(TCL_ERROR);
        if ( (pChar=(char *)Tcl_GetVar(interp,"linter_apierror",0)) == NULL)
            return(TCL_ERROR);
        if (Tcl_GetInt(interp,pChar,&apiError) != TCL_OK)
            return(TCL_ERROR);
        if ( (pChar=(char *)Tcl_GetVar(interp,"linter_syserror",0)) == NULL)
            return(TCL_ERROR);
        if (Tcl_GetInt(interp,pChar,&sysError) != TCL_OK)
            return(TCL_ERROR);
        if ( (pChar=(char *)Tcl_GetVar(interp,"linter_message",0)) == NULL)
            return(TCL_ERROR);
        if ((codError > 2000) && (codError < 3000))
            sprintf(result,"LINTER:error %d api: %d line %d pos %d\n",
                            codError,apiError,(int)(L_WORD)sysError,
                            (int)(*((L_WORD*)((char*)(&sysError)+2))));  /* 03.11.06 */
        else
            sprintf(result,"LINTER:error %d api: %d system %d\n",
                            codError,apiError,sysError);
        Tcl_AppendResult(interp,result,NULL);
        Tcl_AppendResult(interp,pChar,NULL);
        break;
    default:
        if (argc == 2)
            Tcl_SetResult(interp,argv[1],TCL_VOLATILE);
        else
            for (i=1;i<argc;i++)        
                {
                Tcl_AppendElement(interp,argv[i]);
                };
        break;
    };

resetError(interp);

if (retCode > 0)
    return(TCL_OK);
else
    return(TCL_ERROR);/*...16.12.99*/
};

int LinterFreeAllCmd(ClientData clientData, Tcl_Interp * interp, int argc, char *argv[])
{
 L_LONG lRet;                                               /* 03.11.06 */
 char result[MAX_ANSWER_SIZE] = "0";

 if (lRet = LINTER_CloseAPI()) 
  {
   sprintf(result,"%d",(int)(lRet));                        /* 03.11.06 */
   setError(interp,0,0,0,LINTERERROR);/*16.12.99*/
   Tcl_SetResult(interp,result,TCL_VOLATILE);
   return(TCL_OK); 
  }
 FreeBind(interp);
 setError(interp,0,0,0,NOERRORNUMBER);/*16.12.99*/
 Tcl_SetResult(interp,result,TCL_VOLATILE);
 return(TCL_OK);
}

int LinterOpenCursorCmd(ClientData clientData, Tcl_Interp * interp, int argc, char *argv[])
{
 int  nConnID, NameLen;
 L_WORD nCursID = 0 ;
 L_LONG lRet, Mode = 0;                                     /* 03.11.06 */
 char * CursName = NULL;
 char result[MAX_ANSWER_SIZE];

 if (argc < 2)
  {
   sprintf(result,"%d wrong # args",LINTCL_INTERNAL_ERROR);
   Tcl_SetResult(interp,result, TCL_VOLATILE);
   return(TCL_ERROR); 
  }
 if (Tcl_GetInt(interp,argv[1],&nConnID) != TCL_OK)
  return(TCL_ERROR);
 if (argc >=3 )
  if ( (Mode = GetTransMode(interp,result,argv[2])) == LINTCL_INTERNAL_ERROR )
   return(TCL_ERROR);
 if (argc >= 4)
  {
   CursName = argv[3];
   if (argc >= 5)
    {
     if (Tcl_GetInt(interp,argv[4],&NameLen) != TCL_OK)
      return(TCL_ERROR);
    }
  }

 if (lRet = LINTER_OpenCursor((L_WORD)nConnID,&nCursID,CursName,    /* 03.11.06 */
                                  (L_SWORD)NameLen,Mode)) 
  {
   sprintf(result,"%d", (int)(lRet));                       /* 03.11.06 */
   setError(interp,nConnID,0,0,LINTERERROR);/*16.12.99*/
   Tcl_SetResult(interp,result,TCL_VOLATILE);
   return(TCL_OK); 
  }
 setError(interp,0,0,0,NOERRORNUMBER);/*16.12.99*/
 sprintf(result,"%d",(int)(nCursID));
 Tcl_SetResult(interp,result,TCL_VOLATILE);
 return(TCL_OK);
}

int LinterCloseCursorCmd(ClientData clientData, Tcl_Interp * interp, int argc, char *argv[])
{
 int  nCursID;
 L_LONG lRet;                                               /* 03.11.06 */
 char result[MAX_ANSWER_SIZE] = "0";

 if (argc < 2)
  {
   sprintf(result,"%d wrong # args",LINTCL_INTERNAL_ERROR);
   Tcl_SetResult(interp,result, TCL_VOLATILE);
   return(TCL_ERROR); 
  }
 if (Tcl_GetInt(interp,argv[1],&nCursID) != TCL_OK)
  return(TCL_ERROR);
 if (lRet = LINTER_CloseCursor((L_WORD)nCursID))            /* 03.11.06 */
  {
   sprintf(result,"%d",(int)(ConvRetCod(lRet)));            /* 03.11.06 */
   Tcl_SetResult(interp,result,TCL_VOLATILE);
   setError(interp,0,nCursID,0,LINTERERROR);/*16.12.99*/
   return(TCL_OK); 
  }
 ClearBind(interp,(L_WORD)nCursID,0);
 setError(interp,0,0,0,NOERRORNUMBER);/*16.12.99*/
 Tcl_SetResult(interp,result,TCL_VOLATILE);
 return(TCL_OK);
}

int LinterCommitCursorCmd(ClientData clientData, Tcl_Interp * interp, int argc, char *argv[])
{
 int  nCursID;
 L_LONG lRet;                                               /* 03.11.06 */
 char result[MAX_ANSWER_SIZE] = "0";

 if (argc < 2)
  {
   sprintf(result,"%d wrong # args",LINTCL_INTERNAL_ERROR);
   Tcl_SetResult(interp,result, TCL_VOLATILE);
   return(TCL_ERROR); 
  }
 if (Tcl_GetInt(interp,argv[1],&nCursID) != TCL_OK)
  return(TCL_ERROR);
 if (lRet = LINTER_CommitCursor((L_WORD)nCursID,NULL,NULL))   /* 03.11.06 */
  {
   sprintf(result, "%d", (int)(ConvRetCod(lRet)));            /* 03.11.06 */
   Tcl_SetResult(interp,result,TCL_VOLATILE);
   setError(interp,0,nCursID,0,LINTERERROR);/*16.12.99*/
   return(TCL_OK); 
  }
 Tcl_SetResult(interp,result,TCL_VOLATILE);
 setError(interp,0,0,0,NOERRORNUMBER);/*16.12.99*/
 return(TCL_OK);
}

int LinterRollBackCursorCmd(ClientData clientData, Tcl_Interp * interp, int argc, char *argv[])
{
 int  nCursID;
 L_LONG lRet;                                               /* 03.11.06 */
 char result[MAX_ANSWER_SIZE] = "0";

 if (argc < 2)
  {
   sprintf(result,"%d wrong # args",LINTCL_INTERNAL_ERROR);
   Tcl_SetResult(interp,result, TCL_VOLATILE);
   return(TCL_ERROR); 
  }
 if (Tcl_GetInt(interp,argv[1],&nCursID) != TCL_OK)
  return(TCL_ERROR);
 if (lRet = LINTER_RollBackCursor((L_WORD)nCursID,NULL,NULL))   /* 03.11.06 */
  {
   sprintf(result,"%d",(int)(ConvRetCod(lRet)) );               /* 03.11.06 */
   Tcl_SetResult(interp,result,TCL_VOLATILE);
   setError(interp,0,nCursID,0,LINTERERROR);/*16.12.99*/
   return(TCL_OK); 
  }
 Tcl_SetResult(interp,result,TCL_VOLATILE);
 setError(interp,0,0,0,NOERRORNUMBER);/*16.12.99*/
 return(TCL_OK);
}

int LinterExecuteDirectCmd(ClientData clientData, Tcl_Interp * interp, int argc, char *argv[])
{
 int  nCursID;
 L_LONG lRet;                                               /* 03.11.06 */
 int QueryLen = 0;
 char result[MAX_ANSWER_SIZE] = "0";

 if (argc < 3)
  {
   sprintf(result,"%d wrong # args",LINTCL_INTERNAL_ERROR);
   Tcl_SetResult(interp,result, TCL_VOLATILE);
   return(TCL_ERROR); 
  }
 if (Tcl_GetInt(interp,argv[1],&nCursID) != TCL_OK)
  return(TCL_ERROR);
 if (argc >= 4) 
  if (Tcl_GetInt(interp,argv[1],&QueryLen) != TCL_OK)
   return(TCL_ERROR);
 if (lRet = LINTER_ExecuteDirect((L_WORD)nCursID, argv[2],  /* 03.11.06 */
                                 (L_LONG)QueryLen, NULL, NULL))
  {
   sprintf(result,"%d",(int)(ConvRetCod(lRet)) );           /* 03.11.06 */
   Tcl_SetResult(interp,result,TCL_VOLATILE);
   setError(interp,0,nCursID,0,LINTERERROR);/*16.12.99*/
   return(TCL_OK); 
  }
 Tcl_SetResult(interp,result,TCL_VOLATILE);
 setError(interp,0,0,0,NOERRORNUMBER);/*16.12.99*/
 return(TCL_OK);
}

int LinterFetchCmd(ClientData clientData, Tcl_Interp * interp, int argc, char *argv[])
{
 int  nCursID;
 L_LONG lRet;                                               /* 03.11.06 */
 L_WORD Dir = toNext;                                       /* 03.11.06 */
 int Position = 1;
 int RowCounter = 1;
 L_LONG RowAnswer = 0;                                      /* 03.11.06 */
 char result[MAX_ANSWER_SIZE] = "0";

 if (argc < 2)
  {
   sprintf(result,"%d wrong # args",LINTCL_INTERNAL_ERROR);
   Tcl_SetResult(interp,result, TCL_VOLATILE);
   return(TCL_ERROR); 
  }
 if (Tcl_GetInt(interp,argv[1],&nCursID) != TCL_OK)
  return(TCL_ERROR);
 if (argc >= 3)
  {
   if ( (Dir = GetFetchDir(interp, result, argv[2])) ==     /* 03.11.06 */
                                 (L_WORD)(LINTCL_INTERNAL_ERROR) )
     return(TCL_ERROR);
   if ( (Dir == toAbsNumber) || (Dir == toRelNumber) )
    {
     if (argc >= 4)
      {
       if (Tcl_GetInt(interp,argv[3],&Position) != TCL_OK)
        return(TCL_ERROR);
       if (argc >= 5)
        {
         if (Tcl_GetInt(interp,argv[4],&RowCounter) != TCL_OK)
          return(TCL_ERROR);
        } /* if argc >= 5 */
      } /* if argc >= 4 */
    } /* if fetch by number */
   else
    if (argc >= 4)
     {
      if (Tcl_GetInt(interp,argv[3],&RowCounter) != TCL_OK)
       return(TCL_ERROR);
     } 
  } /* if argc >= 3 */

 if (lRet = LINTER_Fetch((L_WORD)nCursID, (L_WORD)Dir,      /* 03.11.06 */
                          (L_LONG)Position, (L_LONG)RowCounter,
                          &RowAnswer, NULL, NULL))
  {
   sprintf(result,"%d",(int)(ConvRetCod(lRet)) );           /* 03.11.06 */
   Tcl_SetResult(interp,result,TCL_VOLATILE);
   setError(interp,0,nCursID,0,LINTERERROR);/*16.12.99*/
   return(TCL_OK); 
  }
 sprintf(result,"%ld",(long)RowAnswer); 
 Tcl_SetResult(interp,result,TCL_VOLATILE);
 setError(interp,0,0,0,NOERRORNUMBER);/*16.12.99*/
 return(TCL_OK);
}

int LinterGetRowCmd(ClientData clientData, Tcl_Interp * interp, int argc, char *argv[])
{
 int nCursID;
 L_LONG lRet, ColumnCount, RealLength = 0;                  /* 03.11.06 */
 L_LONG iCol;                                               /* 03.11.06 */
 char AnsBuf[MAX_ANSWER_SIZE + 1];
 char result[MAX_ANSWER_SIZE + 1] = "";

 if (argc < 2)
  {
   sprintf(result,"%d wrong # args",LINTCL_INTERNAL_ERROR);
   Tcl_SetResult(interp,result, TCL_VOLATILE);
   return(TCL_ERROR); 
  }
 if (Tcl_GetInt(interp,argv[1],&nCursID) != TCL_OK)
  return(TCL_ERROR);

 if (lRet = LINTER_GetCursorOption((L_WORD)nCursID, cColumnCount, 0,  /* 03.11.06 */
                           &ColumnCount, NULL))
  {
   sprintf(result,"%d", (int)(ConvRetCod(lRet)) );
   Tcl_SetResult(interp,result,TCL_VOLATILE);
   setError(interp,0,nCursID,0,LINTERERROR);/*16.12.99*/
   return(TCL_OK); 
  }
 Tcl_ResetResult(interp);
 Tcl_AppendElement(interp,"0");
 for (iCol = 1; iCol <= ColumnCount; iCol++)
  {
   if (lRet = LINTER_GetData((L_WORD)nCursID, (L_SWORD)iCol,  /* 03.11.06 */
                             AnsBuf, MAX_ANSWER_SIZE,
                             tChar, 0, 0, &RealLength))
    {
     sprintf(result,"%d", (int)(ConvRetCod(lRet)) );        /* 03.11.06 */
     Tcl_SetResult(interp,result,TCL_VOLATILE);
     setError(interp,0,nCursID,0,LINTERERROR);/*16.12.99*/
     return(TCL_OK); 
    }
   AnsBuf[RealLength] = '\0';
   Tcl_AppendElement(interp,AnsBuf);
  }
 setError(interp,0,0,0,NOERRORLIST);/*16.12.99*/
 return(TCL_OK);
}

int LinterGetDataCmd(ClientData clientData, Tcl_Interp * interp, int argc, char *argv[])
{
 int nCursID;
 int ColumnNumber = 1;
 int OutType = tChar, OutPrec = 30, OutScale = 10;
 L_LONG lRet, RealLength;                                   /* 03.11.06 */
 char result[MAX_ANSWER_SIZE + 1] = "0";
 char length[70];

 if (argc < 3)
  {
   sprintf(result,"%d wrong # args",LINTCL_INTERNAL_ERROR);
   Tcl_SetResult(interp,result, TCL_VOLATILE);
   return(TCL_ERROR); 
  }
 if (Tcl_GetInt(interp,argv[1],&nCursID) != TCL_OK)
  return(TCL_ERROR);
 if (Tcl_GetInt(interp,argv[2],&ColumnNumber) != TCL_OK)
  return(TCL_ERROR);
 if (argc >= 4)
  {
   if ( (OutType = GetOutType(interp,result,argv[3])) == LINTCL_INTERNAL_ERROR)
    return(TCL_ERROR);
   if (argc >= 5)
    {
     if (Tcl_GetInt(interp,argv[4],&OutPrec) != TCL_OK)
      return(TCL_ERROR);
     if (argc >= 6)
      {
       if (Tcl_GetInt(interp,argv[5],&OutScale) != TCL_OK)
        return(TCL_ERROR);
      }
    }
  } /* if argc >= 4 */

 Tcl_ResetResult(interp);
 if (lRet = LINTER_GetData((L_WORD)nCursID, (L_WORD)ColumnNumber,  /* 03.11.06 */
                    result, MAX_ANSWER_SIZE,
                    (L_SWORD)OutType, (L_SWORD)OutPrec, 
                    (L_SWORD)OutScale, &RealLength))
  {
   sprintf(result,"%d", (int)(ConvRetCod(lRet)) );          /* 03.11.06 */
   Tcl_SetResult(interp,result,TCL_VOLATILE);
   setError(interp,0,nCursID,0,LINTERERROR);/*16.12.99*/
   return(TCL_OK); 
  }
 result[RealLength] = '\0';
 sprintf(length,"%d", (int)(RealLength));                   /* 03.11.06 */
 if (argc >= 7)
  {
   if ( Tcl_SetVar(interp,argv[6],length,0) == NULL )
    {
    sprintf(result,"%d not set value",
               LINTCL_INTERNAL_ERROR);
    Tcl_SetResult(interp,result,TCL_VOLATILE);
    return(TCL_ERROR);
    }
  }
 else
  if ( Tcl_SetVar(interp,"datalen",length,0) == NULL )
   {
   sprintf(result,"%d for var \"datalen\" not set value", LINTCL_INTERNAL_ERROR); /* 03.11.06 */
   Tcl_SetResult(interp,result,TCL_VOLATILE);
   return(TCL_ERROR);
   };

 Tcl_AppendElement(interp,"0");
 Tcl_AppendElement(interp,result);
 setError(interp,0,0,0,NOERRORLIST);/*16.12.99*/
 return(TCL_OK);
}

int LinterCreateStmtCmd(ClientData clientData, Tcl_Interp * interp, int argc, char *argv[])
{
 long lRet;
 L_WORD nStmtID;                                            /* 03.11.06 */
 int nConnID;
 char result[MAX_ANSWER_SIZE] = "0";

 if (argc < 3)
  {
   sprintf(result,"%d wrong # args",LINTCL_INTERNAL_ERROR);
   Tcl_SetResult(interp,result, TCL_VOLATILE);
   return(TCL_ERROR); 
  }
 if (Tcl_GetInt(interp,argv[1],&nConnID) != TCL_OK)
  return(TCL_ERROR);
 if (lRet = LINTER_CreateStatement((L_WORD)nConnID, argv[2], 0, &nStmtID)) /* 03.11.06 */
  {
   sprintf(result,"%d", (int)(ConvRetCod(lRet)) );          /* 03.11.06 */
   Tcl_SetResult(interp,result,TCL_VOLATILE);
   setError(interp,nConnID,0,0,LINTERERROR);/*16.12.99*/
   return(TCL_OK); 
  }

 sprintf(result,"%d", (int)(nStmtID) );                     /* 03.11.06 */
 Tcl_SetResult(interp,result,TCL_VOLATILE);
 setError(interp,0,0,0,NOERRORNUMBER);/*16.12.99*/
 return(TCL_OK);
}

int LinterFreeStmtCmd(ClientData clientData, Tcl_Interp * interp, int argc, char *argv[])
{
 L_LONG lRet;                                               /* 03.11.06 */
 int nStmtID;
 char result[MAX_ANSWER_SIZE] = "0";

 if (argc < 2)
  {
   sprintf(result,"%d wrong # args",LINTCL_INTERNAL_ERROR);
   Tcl_SetResult(interp,result, TCL_VOLATILE);
   return(TCL_ERROR); 
  }
 if (Tcl_GetInt(interp,argv[1],&nStmtID) != TCL_OK)
  return(TCL_ERROR);
 if (lRet = LINTER_FreeStatement((L_WORD)nStmtID))          /* 03.11.06 */
  {
   sprintf(result,"%ld",ConvRetCod(lRet));
   Tcl_SetResult(interp,result,TCL_VOLATILE);
   setError(interp,0,0,nStmtID,LINTERERROR);/*16.12.99*/
   return(TCL_OK); 
  }
 ClearBind(interp,0,(L_WORD)nStmtID);                       /* 03.11.06 */
 Tcl_SetResult(interp,result,TCL_VOLATILE);
 setError(interp,0,0,0,NOERRORNUMBER);/*16.12.99*/
 return(TCL_OK);
}

int LinterExecStmtCmd(ClientData clientData, Tcl_Interp * interp, int argc, char *argv[])
{
 L_LONG lRet;                                               /* 03.11.06 */
 int nStmtID, nCursID, ExecCount = 1, i;
 tsBind * B;
 char *Parameter[240];
 L_LONG ParLen[240];                                        /* 03.11.06 */
 char result[MAX_ANSWER_SIZE] = "0";
 L_SWORD ParType;                                           /* 07.01.01 */

 if (argc < 3)
  {
   sprintf(result,"%d wrong # args",LINTCL_INTERNAL_ERROR);
   Tcl_SetResult(interp,result, TCL_VOLATILE);
   return(TCL_ERROR); 
  }
 if (Tcl_GetInt(interp,argv[1],&nCursID) != TCL_OK)
  return(TCL_ERROR);
 if (Tcl_GetInt(interp,argv[2],&nStmtID) != TCL_OK)
  return(TCL_ERROR);
 if (argc >= 4)
  if (Tcl_GetInt(interp,argv[3],&ExecCount) != TCL_OK)
   return(TCL_ERROR);

/* 07.01.01 ... */
  memset(Parameter, 0, sizeof(Parameter));

 if (B = GetBind((L_WORD) nCursID, (L_WORD)nStmtID))        /* 03.11.06 */
  {
   for (i = 0; i < B->ParamCount; i++)
    {
     ParType = B->LinkVar[i].Type;
     if ( ((ParType & 0xFF00) == pOutput) || ((ParType & 0xFF00) == pInputOutput) )
      {
       Parameter[i] = Tcl_Alloc(B->LinkVar[i].Length + 1);
       memset(Parameter[i], 0, B->LinkVar[i].Length + 1);
       if (Parameter[i] == NULL)
        {
         sprintf(result,"%d nomemory",LINTCL_INTERNAL_ERROR);
         Tcl_SetResult(interp,result,TCL_VOLATILE);
         return(TCL_ERROR); 
        }
       ParLen[i] = B->LinkVar[i].Length;
      }
     else
      {
       if (B->LinkVar[i].LinkVar)
        {
         Parameter[i] = (char *)Tcl_GetVar(interp,B->LinkVar[i].LinkVar,0);
         if (Parameter[i])
          ParLen[i] = strlen(Parameter[i]); 
        }
      }
     if (lRet = LINTER_BindParameter((L_WORD)nCursID,              /* 03.11.06 */
                                     (L_WORD)nStmtID,
                                     (L_SWORD)(i + 1),
                                     NULL,NULL,
                                     Parameter[i],1,0,
                                     (ParType == DEFAULT_OUT_TYPE) ? tChar : ParType,
                                     NULL,
                                     &ParLen[i]))
      {
       sprintf(result,"%d",(int)(ConvRetCod(lRet)) );       /* 03.11.06 */
       Tcl_SetResult(interp,result,TCL_VOLATILE);
       goto l_err;
      }
    }
  }
 else
  {
   sprintf(result, "%d", LINTCL_INTERNAL_ERROR);            /* 03.11.06 */
   Tcl_SetResult(interp,result,TCL_VOLATILE);
   setError(interp,0,nCursID,nStmtID,LINTERERROR);/*16.12.99*/
   return(TCL_OK); 
  }
/* ... 07.01.01 */

 if (lRet = LINTER_ExecuteStatement((L_WORD)nCursID, (L_WORD)nStmtID, /* 03.11.06 */
                              NULL, NULL, NULL))
  {
   sprintf(result,"%d",(int)(ConvRetCod(lRet)) );           /* 03.11.06 */
   Tcl_SetResult(interp,result,TCL_VOLATILE);
   setError(interp,0,nCursID,nStmtID,LINTERERROR);/*16.12.99*/
   goto l_err;
  }

  for (i = 0; i < B->ParamCount; i++)                      /* 07.01.01 */
   {
    ParType = B->LinkVar[i].Type & 0xFF00;
    if ( (ParType == pOutput) || (ParType == pInputOutput) )
     {
      if (B->LinkVar[i].LinkVar)
        Tcl_SetVar(interp,B->LinkVar[i].LinkVar,Parameter[i],0);
      Tcl_Free(Parameter[i]);
     }
   }


 sprintf(result,"%ld",ExecCount);
 Tcl_SetResult(interp,result,TCL_VOLATILE);
 setError(interp,0,0,0,NOERRORNUMBER);/*16.12.99*/
 return(TCL_OK);

l_err:                                                     /* 07.01.01 */
 for (i = 0; i < B->ParamCount; i++)
  {
   ParType = B->LinkVar[i].Type & 0xFF00;
   if ( (ParType == pOutput) || (ParType == pInputOutput) )
    {
     if (Parameter[i])
      Tcl_Free(Parameter[i]);
    }
  }
 return(TCL_OK); 
}

int LinterBindParCmd(ClientData clientData, Tcl_Interp * interp, int argc, char *argv[])
{
 int nCursID, nStmtID, ParNumber;
 char result[MAX_ANSWER_SIZE] = "0";
 tsBind * B;

 if (argc < 5)
  {
   sprintf(result,"%d wrong # args",LINTCL_INTERNAL_ERROR);
   Tcl_SetResult(interp,result, TCL_VOLATILE);
   return(TCL_ERROR); 
  }
 if (Tcl_GetInt(interp,argv[1],&nCursID) != TCL_OK)
  return(TCL_ERROR);
 if (Tcl_GetInt(interp,argv[2],&nStmtID) != TCL_OK)
  return(TCL_ERROR);
 if (Tcl_GetInt(interp,argv[3],&ParNumber) != TCL_OK)
  return(TCL_ERROR);

 if (!(B = CreateBind(interp,result, (L_WORD)nCursID, (L_WORD)nStmtID))) /* 03.11.06 */
   return(TCL_ERROR); 

 if (ParNumber > B->ParamCount)
  {
   sprintf(result,"%d invalid parameter number",LINTCL_INTERNAL_ERROR);
   Tcl_SetResult(interp,result,TCL_VOLATILE);
   return(TCL_ERROR); 
  }

 if (B->LinkVar[ParNumber - 1].LinkVar)
  Tcl_Free(B->LinkVar[ParNumber - 1].LinkVar);
 if (!(B->LinkVar[ParNumber - 1].LinkVar = Tcl_Alloc(strlen(argv[4]) * sizeof(char) + 1)))
  {
   sprintf(result,"%d nomemory",LINTCL_INTERNAL_ERROR);
   Tcl_SetResult(interp,result,TCL_VOLATILE);
   return(TCL_ERROR); 
  }
 sprintf(B->LinkVar[ParNumber - 1].LinkVar,"%s",argv[4]);

/* 07.01.01 ... */
 if (argc > 5)
  {
   if ( (B->LinkVar[ParNumber - 1].Type = GetOutType(interp,result,argv[5])) == LINTCL_INTERNAL_ERROR)
    return(TCL_ERROR);
  }
 else
  B->LinkVar[ParNumber - 1].Type = DEFAULT_OUT_TYPE;
 if (argc > 6)
  {
   if (Tcl_GetInt(interp,argv[6],&B->LinkVar[ParNumber - 1].Length) != TCL_OK)
    return(TCL_ERROR);
  }
 else
  B->LinkVar[ParNumber - 1].Length = MAX_ANSWER_SIZE;
/* ... 07.01.01 */

 Tcl_SetResult(interp,result,TCL_VOLATILE);
 setError(interp,0,0,0,NOERRORNUMBER);/*16.12.99*/
 return(TCL_OK);
}

int LinterGetCursorOptionCmd(ClientData clientData, Tcl_Interp * interp, int argc, char *argv[])
{
 int nCursID, ColumnNumber = 0;
 L_LONG lRet;                                               /* 03.11.06 */
 L_SWORD OptionType;                                        /* 03.11.06 */
 char Owner[MAX_ID_LEN + 1],Table[MAX_ID_LEN + 1],Column[MAX_ID_LEN + 1];
 char result[MAX_ANSWER_SIZE];
 t_ParamDesc OutBuf;

 if (argc < 3)
  {
   sprintf(result,"%d wrong # args",LINTCL_INTERNAL_ERROR);
   Tcl_SetResult(interp,result, TCL_VOLATILE);
   return(TCL_ERROR); 
  }
 if (Tcl_GetInt(interp,argv[1],&nCursID) != TCL_OK)
  return(TCL_ERROR);

 OptionType = GetCursOpt(interp,result,argv[2]);
 if (OptionType == LINTCL_INTERNAL_ERROR)
  return(TCL_ERROR);

 if (OptionType == cAnswerDesc)
  if (argc >= 4)
   if (Tcl_GetInt(interp,argv[3],&ColumnNumber) != TCL_OK)
    return(TCL_ERROR);

 if (lRet = LINTER_GetCursorOption((L_WORD)nCursID, OptionType,  /* 03.11.06 */
                                   (L_WORD)ColumnNumber,
                                    &OutBuf,NULL))
  {
   sprintf(result,"%d",(int)(ConvRetCod(lRet)));            /* 03.11.06 */
   Tcl_SetResult(interp,result,TCL_VOLATILE);
   setError(interp,0,nCursID,0,LINTERERROR);/*16.12.99*/
   return(TCL_OK); 
  }

 if (OptionType == cAnswerDesc)
  {
   strncpy(Owner,OutBuf.Owner,MAX_ID_LEN);
   Owner[MAX_ID_LEN] = 0;
   strncpy(Table,OutBuf.Table,MAX_ID_LEN);
   Table[MAX_ID_LEN] = 0;
   strncpy(Column,OutBuf.Column,MAX_ID_LEN);
   Column[MAX_ID_LEN] = 0;
   sprintf(result,"0 %s %s %s %d %d %d %d",                 /* 03.11.06 */
           Owner,Table,Column,(int)OutBuf.Type,(int)OutBuf.Length,
           (int)OutBuf.NullIndicator,
           (int)OutBuf.RealLength);
  }
 else
  sprintf(result,"0 %d",(int)(*(L_LONG*)(&OutBuf)) );       /* 03.11.06 */

 Tcl_SetResult(interp,result,TCL_VOLATILE);
 setError(interp,0,0,0,NOERRORLIST);/*16.12.99*/
 return(TCL_OK);
}

int LinterGetStmtOptionCmd(ClientData clientData, Tcl_Interp * interp, int argc, char *argv[])
{
 int nStmtID, ColumnNumber = 0;
 long lRet;
 L_SWORD OptionType;                                        /* 03.11.06 */
 char Owner[MAX_ID_LEN + 1],Table[MAX_ID_LEN + 1],Column[MAX_ID_LEN + 1];
 char result[MAX_ANSWER_SIZE];
 t_ParamDesc OutBuf;

 if (argc < 3)
  {
   sprintf(result,"%d wrong # args",LINTCL_INTERNAL_ERROR);
   Tcl_SetResult(interp,result, TCL_VOLATILE);
   return(TCL_ERROR); 
  }
 if (Tcl_GetInt(interp,argv[1],&nStmtID) != TCL_OK)
  return(TCL_ERROR);

 OptionType = GetStmtOpt(interp,result,argv[2]);
 if (OptionType == LINTCL_INTERNAL_ERROR)
  return(TCL_ERROR);

 if (OptionType == sAnswerDesc || OptionType == sParamDesc)
  if (argc >= 4)
   if (Tcl_GetInt(interp,argv[3],&ColumnNumber) != TCL_OK)
    return(TCL_ERROR);

 if (lRet = LINTER_GetStatementOption((L_WORD)nStmtID,OptionType, /* 03.11.06 */
                                      (L_SWORD)ColumnNumber,
                                      &OutBuf,NULL))
  {
   sprintf(result,"%d",(int)(ConvRetCod(lRet)) );           /* 03.11.06 */
   Tcl_SetResult(interp,result,TCL_VOLATILE);
   setError(interp,0,0,nStmtID,LINTERERROR);/*16.12.99*/
   return(TCL_OK); 
  }

 switch (OptionType)
  {
   case sAnswerDesc:
     strncpy(Owner,OutBuf.Owner,MAX_ID_LEN);
     Owner[MAX_ID_LEN] = 0;
     strncpy(Table,OutBuf.Table,MAX_ID_LEN);
     Table[MAX_ID_LEN] = 0;
     strncpy(Column,OutBuf.Column,MAX_ID_LEN);
     Column[MAX_ID_LEN] = 0;
     sprintf(result,"0 %s %s %s %d %d",
             Owner,Table,Column,(int)OutBuf.Type,(int)OutBuf.Length);
    break;
   case sParamDesc:
     strncpy(Column,OutBuf.Column,MAX_ID_LEN);
     Column[MAX_ID_LEN] = 0;
     sprintf(result,"0 %s %d %d",
             Column,(int)OutBuf.Type,(int)OutBuf.Length);
    break;
   default:
    sprintf(result,"0 %d",(int)(*(L_LONG*)(&OutBuf)) );       /* 03.11.06 */
    break;
  }

 Tcl_SetResult(interp,result,TCL_VOLATILE);
 setError(interp,0,0,0,NOERRORLIST);/*16.12.99*/
 return(TCL_OK);
}

/*
17.09.98 ALX : change return value from void to int,
                add return TCL_OK
*/

int Tcllinter_Init(Tcl_Interp * interp)
{
 Tcl_PkgProvide(interp,"tclLinter","1.3");
 Tcl_CreateCommand(interp,"linter_connect",(Tcl_CmdProc*)LinterConnectCmd,NULL,NULL);
 Tcl_CreateCommand(interp,"linter_closeconnect",(Tcl_CmdProc*)LinterCloseConnectCmd,
                                                             NULL,NULL);
 Tcl_CreateCommand(interp,"linter_error",(Tcl_CmdProc*)LinterErrorCmd,NULL,NULL);
 Tcl_CreateCommand(interp,"linter_opencursor",(Tcl_CmdProc*)LinterOpenCursorCmd,NULL,NULL);
 Tcl_CreateCommand(interp,"linter_closecursor",(Tcl_CmdProc*)LinterCloseCursorCmd,NULL,NULL);
 Tcl_CreateCommand(interp,"linter_execdir",(Tcl_CmdProc*)LinterExecuteDirectCmd,NULL,NULL);
 Tcl_CreateCommand(interp,"linter_fetch",(Tcl_CmdProc*)LinterFetchCmd,NULL,NULL);
 Tcl_CreateCommand(interp,"linter_getrow",(Tcl_CmdProc*)LinterGetRowCmd,NULL,NULL);
 Tcl_CreateCommand(interp,"linter_getdata",(Tcl_CmdProc*)LinterGetDataCmd,NULL,NULL);
 Tcl_CreateCommand(interp,"linter_createstmt",(Tcl_CmdProc*)LinterCreateStmtCmd,NULL,NULL);
 Tcl_CreateCommand(interp,"linter_freestmt",(Tcl_CmdProc*)LinterFreeStmtCmd,NULL,NULL);
 Tcl_CreateCommand(interp,"linter_execstmt",(Tcl_CmdProc*)LinterExecStmtCmd,NULL,NULL);
 Tcl_CreateCommand(interp,"linter_bindpar",(Tcl_CmdProc*)LinterBindParCmd,NULL,NULL);
 Tcl_CreateCommand(interp,"linter_cursopt",(Tcl_CmdProc*)LinterGetCursorOptionCmd,NULL,NULL);
 Tcl_CreateCommand(interp,"linter_stmtopt",(Tcl_CmdProc*)LinterGetStmtOptionCmd,NULL,NULL);
 Tcl_CreateCommand(interp,"linter_commit",(Tcl_CmdProc*)LinterCommitCmd,NULL,NULL);
 Tcl_CreateCommand(interp,"linter_commcurs",(Tcl_CmdProc*)LinterCommitCursorCmd,NULL,NULL);
 Tcl_CreateCommand(interp,"linter_rollback",(Tcl_CmdProc*)LinterRollBackCmd,NULL,NULL);
 Tcl_CreateCommand(interp,"linter_rollbcurs",(Tcl_CmdProc*)LinterRollBackCursorCmd,NULL,NULL);
 Tcl_CreateCommand(interp,"linter_freeall",(Tcl_CmdProc*)LinterFreeAllCmd,NULL,NULL);
 return(TCL_OK);
 
}
