/***************************************************************************
 *                       RELEX, Inc. DBMS Linter                            *
 *  This is unpublished propietary source code of RELEX, Inc.               *
 *  The copyright notice does not evidence any actual or intended           *
 *  publication of such source code.                                        *
 *                                                                          *
 *                 Copyright (c) 1990-2013 RELEX, Inc.                      *
 *                                                                          *
 *                   This software was developed by                         *
 *                                                                          *
 *                            RELEX, Inc.                                   *
 *                         Voronezh, Russia                                 *
 *                                                                          *
 *                        All rights reserved.                              *
 ****************************************************************************
 AUTHOR:
 ----------------------------------------------------------------------------
 PURPOSE: DATE formats
 ----------------------------------------------------------------------------
 14.10.91 
 22.12.92 -- AVO
 09.12.94 -- DV
              String to date conversion - according to format
              Format string must be uppsercase
 07.04.97 - DV (copy string and format)
 22.05.97 - DV (interval times)
 20.07.97 -- Ermakov (Copyright) 
 12.08.97 -- Dudov (STRTOTICKF support "FFF" format)
 12.08.97 -- Dudov (STRTOTICKF skip all nonformat symbols)
 12.08.97 -- Kent
 12.05.98 -- DV (ticks < 100)
 18.09.98 -- DV (zero time bug fixed)
 02.11.98 -- Pasec (zero time bug fixed for empty input string)
 14.09.99 -- Ermakov (For OS9000)
 05.10.99 -- Ermakov (For QNX)
 01.11.99 -- DV (Do not clear formatted output if zeros only)
 29.12.99 -- DV (Y2K)
 25.03.00 -- DV (Correct previous update)
 15.01.01 -- DV (May be different second precision)
 18.12.00 -- VxWorks
 20.04.01 -- KOV (formats are added + functions NAMMON,MONNAM,NAMDAY,DAYNAM are changed)
 21.04.01 -- DV (TICKFSTRLEN function added)
 21.04.01 -- KOV Fix for "FFF" format
 07.06.01 -- KOV Fix for "HH12" format
 03.11.01 -- DV  BT#3719 fixed
 02.05.03 -- DV  Optimization
 17.05.04 -- Galizin   ( Linux glibc 2.3 TT#11078 )
 19.06.05 -- Ermakov   ( Corrected pointer manipulation in Linux (!) )
 06.10.05 -- KOV (STRTOTICKF support "MS" format)
 16.02.06 -- KOV (Added OBSOLETE section)
 17.02.06 -- DV  (Interval time)
 01.09.10 -- DV  (Interval time, including negative)
 03.07.12 -- Ganin (Const qualifiers)
 15.08.12 -- KOV (Correction for '12:01 AM' -> '00:01' [not '12:01'], '12:00' -> '12:00 P.M.' [not '12:00 A.M.'])
 19.07.12 -- Galizin (OpenVMS)
 14.12.12 -- KOV (Can't use '0 PM' and '[13-23] AM')
 08.04.13 -- DV  (Check input data length)
 11.04.13 -- Ermakov (const added. remove extra code)
*/

#include <string.h>

#ifdef __MSDOS__
#include <ctype.h>
#include "decimals.h"
#include "tick.h"           /* 19.05.92 */
#define _DATE_
#endif

#if defined(UNIX) || defined(QNX) || defined(VXWORKS) || defined(VMS)  /* 19.07.12 05.10.99 */ /* 18.12.00 */
#include <ctype.h>
#include "decimals.h"
#include "tick.h"           /* 19.05.92 */
#define _DATE_
#endif

/* 14.09.99 ... */
#ifdef OS9000
#include <sys/ctypes>	    
#include "decimals.h"	    
#include "tick.h"           /* 19.05.92 */
#define _DATE_
#endif
/* ... 14.09.99 */

#ifdef INTER_OS2
#include <ctype.h>
#include "tick.h"
#define _DATE_
#endif

#ifdef __INTER_DLL__
#ifndef INTER_MSWINDOWS
#define INTER_MSWINDOWS
#endif
#endif

#ifdef INTER_MSWINDOWS
#include <ctype.h>
#include "tick.h"
#define _DATE_
#endif

#ifndef _DATE_
ERROR: UNDEF SYSTEM
#endif


int GETFRACT(						/* 15.01.01 ... */
	   char **pt,
	   int	imp,
           int  lim);					/* ... 15.01.01 */

int GETZAL(
           char **pt,
           int  lim);

int todate(
          int   d,
          int   m,
          int   y,
          L_LONG  *dt);

int totime(
          int   h,
          int   m,
          int   s,
          int   t,
          L_LONG  *tm);

static void AllUpper(const L_CHAR *S, L_CHAR *CS)           /* 03.07.12 */
{
  while (*S != '\0')
  {
    *CS = LAnsiToupper(*S);                                 /* 17.05.04 */
    S++; CS++;
  }
  *CS = '\0';
}

/* Calculate result length */				/* 21.04.01 ... */
L_SWORD DLLAPI TICKFSTRLEN(L_CHAR *F, L_SWORD FLEN)
{
  register char *pf;                                    /* 02.05.03 */
  L_CHAR CS[255],CF[255];
  register L_SWORD len = 0, i;                              /* 02.05.03 */

  if (FLEN >= 255)                                      /* 08.04.13 */
    return 0;                                           /* 08.04.13 */

  pf = &CF[0];
  for (i=0; i<FLEN; i++)
    CS[i] = F[i];
  CS[FLEN] = '\0';

  AllUpper(&CS[0],&CF[0]);

  while (*pf != 0)
  {
    switch (*pf)                                        /* 02.05.03 ... */
    {
      case '+':
        pf += 1;                                        /* 01.09.10 ... */
        len += 1;
        break;                                          /* ... 01.09.10 */

      case 'm':
      case 'M':
        if (strncmp(pf,"MM",2) == 0)
        {
          pf += 2;
          len += 2;
        }
        else if (strncmp(pf,"MI",2) == 0)
        {
          pf += 2;
          len += 2;
        }
        else if (strncmp(pf,"MONTH",5) == 0)
        {
          pf += 5;
          len += 9;
        }
        else if (strncmp(pf,"MON",3) == 0)
        {
          pf += 3;
          len += 3;
        }
        else if (strncmp(pf,"MS",2) == 0)                   /* 06.10.05 ... */
        {
          pf += 2;
          len += 3;
        }                                                   /* ... 06.10.05 */
        else
        {
          pf++; len++;
        }
        break;

      case 'h':
      case 'H':
        if (strncmp(pf,"HH12",4) == 0)
        {
          pf += 4;
          len += 2;
        }
        else if (strncmp(pf,"HH24",4) == 0)
        {
          pf += 4;
          len += 2;
        }
        else if (strncmp(pf,"HH",2) == 0)
        {
          pf += 2;
          len += 2;
        }
        else
        {
          pf++; len++;
        }
        break;

      case 'a':
      case 'p':
      case 'A':
      case 'P':
        if (strncmp(pf,"A.M.",4) == 0 ||
            strncmp(pf,"P.M.",4) == 0)
        {
          pf += 4; 
          len += 4;
        }
        else if (strncmp(pf,"AM",2) == 0||
                 strncmp(pf,"PM",2) == 0)
        {
          pf += 2; 
          len += 2;
        }
        else
        {
          pf++; len++;
        }
        break;

      case 'd':
      case 'D':
        if (strncmp(pf,"DDD",3) == 0)
        {
          pf += 3; 
          len += 3;
        }
        else if (strncmp(pf,"DD",2) == 0)
        {
          pf += 2;
          len += 2;
        }
        else if (strncmp(pf,"DAY",3) == 0)
        {
          pf += 3; 
          len += 9;
        }
        else if (strncmp(pf,"DY",2) == 0)
        {
          pf += 2; 
          len += 3;
        }
        else
        {
          pf++; len++;
        }
        break;

      case 'y':
      case 'Y':
        if (strncmp(pf,"YYYY",4) == 0)
        {
          pf += 4;
          len += 4;
        }
        else if (strncmp(pf,"YY",2) == 0)
        {
          pf += 2;
          len += 2;
        }
        else
        {
          pf++; len++;
        }
        break;

      case 'f':
      case 'F':
        if (strncmp(pf,"FFF",3) == 0)
        {
          pf += 3;
          len += 3;
        }
        else if (strncmp(pf,"FF",2) == 0)
        {
          pf += 2;
          len += 2;
        }
        else
        {
          pf++; len++;
        }
        break;

      case 's':
      case 'S':
        if (strncmp(pf,"SS",2) == 0)
        {
          pf += 2;
          len += 2;
        }
        else
        {
          pf++; len++;
        }
        break;

      default:
        pf++; 
        len++; 
        break;
    }                                                   /* ... 02.05.03 */
  }
  return len;
}							/* ... 21.04.01 */

/*
**  Convert date + format to L_DECIMAL
*/
L_SWORD DLLAPI STRTOTICKF(const L_CHAR *S,          /* 03.07.12 */
                          const L_CHAR *F,          /* 03.07.12 */
                          L_DECIMAL D)
{
  L_SWORD Ok;
  L_CHAR NAM[10];                                   /* 20.04.01 */
  L_SWORD DAY,MON,YEAR,h,m,s,t,i,len;               /* 20.04.01 */  
  L_LONG dt,tm;
  char *ps,*pf;
  L_CHAR CS[255];                                   /* 21.04.01 */
  L_CHAR CF[255];                                   /* 21.04.01 */ 
  L_BOOL SHORT_YEAR = L_TFALSE;                     /* 17.02.06 */

  DAY=MON=YEAR=h=m=s=t=0;
  Ok = L_TTRUE;
  ps = &CS[0];
  pf = &CF[0];

  if (strlen(S) >= 255 ||                    /* 08.04.13 ... */
      strlen(F) >= 255)
    return L_TFALSE;                         /* ... 08.04.13 */

  AllUpper(S,&CS[0]);
  AllUpper(F,&CF[0]);                        /* 16.10.95 */

  while (*pf != 0 && Ok)
  {
    while(*pf == ' ') pf++;
    while(*ps == ' ') ps++;

    if (*ps == 0)
    {
      LongToDec(0,D);
      goto test_date;                       /* 02.11.98 */
/*      return Ok;*/
    }

    if (strncmp(pf,"MM",2) == 0)
    {
      pf += 2;
      MON = GETZAL(&ps,2);
    }
    /* 20.04.01 ... */
    else if (strncmp(pf,"MONTH",5) == 0)
    {
      pf += 5;
      for ( i=0, len=0; i<10; i++, len++) 
          if ( ps[i] == '-' || ps[i] == '/' || ps[i] == ' ' ||
               ps[i] == '.' || ps[i] == ',' || ps[i] == ';' || 
               ps[i] == ':' )
               break;
      for (i=0;i<len;i++) 
      { 
        NAM[i] = LAnsiTolower(*ps);                          /* 17.05.04 */
        ps++;                                                /* 19.06.05 */
      }
      NAM[len] = 0;
      MON = MONNAMex(&NAM[0],0,len);
      if (MON==0) return L_TFALSE;
    }
    else if (strncmp(pf,"HH12",4) == 0)
    {
      pf += 4;
      h = GETZAL(&ps,2);
      if (h < 12 &&
          (strstr(CS, "PM") || strstr(CS, "P.M.")))
      {
        if (h == 0)                                        /* 14.12.12 */
          return L_TFALSE; /* The hour value of 0 represents the hour after midnight (AM), regardless of whether or not you specify AM. You cannot specify PM when the hour equals 0. */
        h += 12;
      }
      else                                                 /* 15.08.12 ... */
      if (strstr(CS, "AM") || strstr(CS, "A.M."))
      {
        if (h == 12)
          h -= 12;                                         /* ... 15.08.12 */
        else
        if (h >= 13 && h <= 23)                            /* 14.12.12 */
          return L_TFALSE; /* Hour values from 13 through 23 represents hours after noon if AM or PM is not specified. They also represent the hours after noon when PM is specified. You cannot specify AM when the hour value is from 13 through 23. */
      }
    }
    else if (strncmp(pf,"A.M.",4) == 0 ||
             strncmp(pf,"P.M.",4) == 0)
    {
      pf += 4; ps += 4;
    }
    else if (strncmp(pf,"AM",2) == 0||
             strncmp(pf,"PM",2) == 0)
    {
      pf += 2; ps += 2;
    }
    else if (strncmp(pf,"DDD",3) == 0)
    {
      pf += 3; ps += 3;
    }
    else if (strncmp(pf,"DAY",3) == 0)
    {
      pf += 3; ps += 3;
    }
    else if (strncmp(pf,"DY",2) == 0)
    {
      pf += 2; ps += 2;
    }
    /* ... 20.04.01*/
    else if (strncmp(pf,"MON",3) == 0)
    {
      pf += 3;
      for (i=0;i<3;i++) 
      {
        NAM[i] = LAnsiTolower(*ps);                         /* 17.05.04 */
        ps++;                                               /* 19.06.05 */
      }
      NAM[3] = 0;
      MON = MONNAM(&NAM[0],0);
      if (MON==0) return L_TFALSE;
    }
    else if (strncmp(pf,"DD",2) == 0)
    {
      pf += 2;
      DAY = GETZAL(&ps,2);
    }
    else if (strncmp(pf,"YYYY",4) == 0)
    {
      pf += 4;
      YEAR = GETZAL(&ps,4);
    }
    else if (strncmp(pf,"YY",2) == 0)
    {
      pf += 2;
      /* 29.12.99 ... */
      YEAR = GETZAL(&ps,2);
      SHORT_YEAR = L_TTRUE;                                 /* 17.02.06 */
      /* ... 29.12.99 */
    }
    else if (strncmp(pf,"HH24",4) == 0)
    {
      pf += 4;
      h = GETZAL(&ps,2);
    }
    else if (strncmp(pf,"HH",2) == 0)
    {
      pf += 2;
      h = GETZAL(&ps,2);
    }
    else if (strncmp(pf,"MI",2) == 0)
    {
      pf += 2;
      m = GETZAL(&ps,2);
    }
    else if (strncmp(pf,"SS",2) == 0)
    {
      pf += 2;
      s = GETZAL(&ps,2);
    }
    else if (strncmp(pf,"MS",2) == 0)                       /* 06.10.05 ... */
    {
      pf += 2;
      t = GETFRACT(&ps,3,10)/10;
    }                                                       /* ... 06.10.05 */
    else if (strncmp(pf,"FFF",3) == 0)                  /* 12.08.97 */
    {
      pf += 3;
      t = GETFRACT(&ps,3,10)/10;			/* 15.01.01 */
    }
    else if (strncmp(pf,"FF",2) == 0)
    {
      pf += 2;
      t = GETFRACT(&ps,2,10);				/* 15.01.01 */				
    }
    /* 20.04.01 ... */
    else if (strncmp(pf,"D",1) == 0 )
    {
        pf += 1; ps += 1;
    }
    /* ... 20.04.01 */
    else if (/*(*pf == '/' || *pf == '-' || *pf == ':' || *pf == '.')
              &&*/ *pf == *ps)                          /* 12.08.97 */
    {
      pf++; ps++;
    }
    else
      Ok = L_TFALSE;
  }
  while(*ps == ' ') ps++;                               /* 03.11.01 */

  if (*ps != 0) 
    Ok = L_TFALSE;
  if (!Ok) 
    return L_TFALSE;


test_date:                /* 02.11.98 */
  
  if ((YEAR==0) && (MON==0))                                /* 22.05.97 ... */
  {
    Ok = totime(h,m,s,t,&tm); 
    dt = DAY;
  } /* date and hour interval */
  else 
    if ((DAY == 0) && (h == 0) && (m == 0) && (s == 0) && (t == 0))
    {
#ifdef OBSOLETE                                             /* 16.02.06 */
      if (YEAR >= 2000) 
        YEAR -= 2000; 
      else 
        if (YEAR >= 1900) 
          YEAR -= 1900;                                     /* 25.03.00 */
#endif
      if (YEAR != 0 && !SHORT_YEAR)                         /* 17.02.06 ... */
      {
        DAY = 1;
        if (MON == 0)
          MON = 1;
        Ok = todate(DAY,MON,YEAR,&dt) && totime(h,m,s,t,&tm);
      } /* Absolute time (year & month) */
      else
      {  
        Ok = todate(1,MON+1,YEAR+1,&dt);                      /* 29.12.99 */
        dt--; 
        tm = 0;
      }
    } /* year and month interval */
    else
    {
     if (SHORT_YEAR)
     {
       if (YEAR <= 37)
          YEAR += 2000;
       else
         YEAR += 1900;
     }                                                      /* ... 17.02.06 */
      Ok = todate(DAY,MON,YEAR,&dt) && totime(h,m,s,t,&tm);
    }
  DATETOTICK(dt,tm,D);
  if (DAY == 0 && tm == 0)
    if (YEAR != 0 || MON != 0)                              /* 18.09.98 */
      D[15] = 1;                 /* year and month interval*/ /* ... 22.05.97 */
  
  return Ok;
}

/*
** Convert L_DECIMAL+format to DATE
*/
L_SWORD DLLAPI TICKTOSTRF(const L_DECIMAL DD,
                          const L_CHAR   *F,     /* 03.07.12 */
                          L_CHAR   *S)
{
  L_CHAR NAM[10];                                /* 20.04.01 */
  L_SWORD i,N,Ok,Add;
  L_SWORD DAY,MON,YEAR,h,m,s,f;
  L_LONG lday,ltick;
  const register L_CHAR *pf, *pf1 = F;           /* 03.07.12 */ /* 20.04.01 */ /* 02.05.03 */
  L_CHAR CF[255];                                /* 21.04.01 */
  DECSTATUS st;                                  /* 01.09.10 ... */
  L_DECIMAL D;

  COPYDEC(DD, D);

  st = GETTICKSTATUS(D);
  Ok = (st != DECERROR);
  if (st == DECNEGATIV)
    SETDECSTATUS(DECPOSITIV,D);                /* ... 01.09.10 */

  N=0;
  Add=0;
  pf = &CF[0];

  AllUpper(F,&CF[0]);                          /* 16.10.95 07.04.97 */

  if (*pf == '+')                              /* 01.09.10 ... */
  {
    pf++;
    pf1++;
    if (st == DECNEGATIV)
      S[N++] = '-';
    else if (st == DECPOSITIV)
      S[N++] = '+';
    else
      S[N++] = ' ';
  }                                            /* ... 01.09.10 */

  TICKTODATE(D,&lday,&ltick);

  EDATEDAYNUMBER(D,lday,&DAY,&MON,&YEAR);       /* 17.02.06 */
  h = (L_SWORD) (ltick / 360000);
  ltick -= h*360000;				/* 02.05.03 */
  m = (L_SWORD) (ltick / 6000);
  ltick -= m*6000;				/* 02.05.03 */
  s = (L_SWORD) (ltick / 100);
  ltick -= s*100;				/* 02.05.03 */
  f = (L_SWORD) ltick;

  while (*pf != 0)
  {
    switch (*pf)                                /* 02.05.03 ... */
    {
      case 'd':
      case 'D':
        if (strncmp(pf,"DDD",3) == 0)
        {
          pf+=3; pf1+=3;
          PUT3(DAYYEAR(DAY,MON,YEAR),S,&N,Ok);
        }
        else if (strncmp(pf,"DD",2) == 0)
        {
          pf+=2; pf1+=2;                           /* 20.04.01 */   
          PUT2(DAY,S,&N,Ok);
        }
        else if (strncmp(pf,"DY",2) == 0)
        {
          NAMDAY(WEEKDAYNUMBER(lday),&NAM[0],0);
          for (i=0;i<3;i++) 
            S[N++] = (MON) ? ( ( i == 0 ) ? ( ( pf[0] == pf1[0] ) ? LAnsiToupper( NAM[i] ) : NAM[i] ) : 
                                            ( ( pf[1] == pf1[1] ) ? LAnsiToupper( NAM[i] ) : NAM[i] ) ) : '?';  /* 17.05.04 */
          pf+=2; pf1+=2;
        }
        else if (strncmp(pf,"DAY",3) == 0)
        {
          NAMDAYex(WEEKDAYNUMBER(lday),&NAM[0],0,10);
          for (i=0;i<10 && NAM[i]!=0;i++) 
            S[N+i] = (MON) ? ( ( i == 0 ) ? ( ( pf[0] == pf1[0] ) ? LAnsiToupper( NAM[i] ) : NAM[i] ) : 
                                            ( ( pf[1] == pf1[1] ) ? LAnsiToupper( NAM[i] ) : NAM[i] ) ) : '?'; /* 17.05.04 */
          N+=i;
          pf+=3; pf1+=3;
        }
        else if (strncmp(pf,"D",1) == 0)
        {
          pf+=1; pf1+=1;
          PUTDIG(WEEKDAYNUMBER(lday),S,&N,Ok);
        }
        else
        {
          S[N++] = *pf++;
          pf1++;
        }
        break;

      case 'm':
      case 'M':
        if (strncmp(pf,"MM",2) == 0)
        {
          pf+=2; pf1+=2;                           /* 20.04.01 */   
          PUT2(MON,S,&N,Ok);
        }
        else if (strncmp(pf,"MI",2) == 0)
        {
          pf+=2; pf1+=2;                           /* 20.04.01 */   
          PUT2(m,S,&N,Ok);
        }
        else if (strncmp(pf,"MONTH",5) == 0)
        {
          NAMMONex(MON,&NAM[0],0,10);
          for (i=0;i<10 && NAM[i]!=0;i++) 
            S[N+i] = (MON) ? ( ( i == 0 ) ? ( ( pf[0] == pf1[0] ) ? LAnsiToupper( NAM[i] ) : NAM[i] ) : 
                                            ( ( pf[1] == pf1[1] ) ? LAnsiToupper( NAM[i] ) : NAM[i] ) ) : '?'; /* 17.05.04 */
          N+=i;
          pf+=5; pf1+=5;
        }
        else if (strncmp(pf,"MON",3) == 0)
        {
          NAMMON(MON,&NAM[0],0);
          for (i=0;i<3;i++) 
            S[N++] = (MON) ? ( ( pf[i] == pf1[i] ) ? LAnsiToupper( NAM[i] ) : NAM[i] ) : '?'; /* 17.05.04 20.04.01 */
          pf+=3; pf1+=3;                           /* 20.04.01 */
        }
        else if (strncmp(pf,"MS",2) == 0)                   /* 06.10.05 ... */
        {
          pf+=2; pf1+=3;
          PUT2(f,S,&N,Ok);
          S[N++] = '0';
        }                                                   /* ... 06.10.05 */
        else
        {
          S[N++] = *pf++;
          pf1++;
        }
        break;

      case 'y':
      case 'Y':
        if (strncmp(pf,"YYYY",4) == 0)
        {
          pf+=4; pf1+=4;                           /* 20.04.01 */   
          PUTYEAR(YEAR,1,S,&N,Ok);
        }
        else if (strncmp(pf,"YY",2) == 0)
        {
          pf+=2; pf1+=2;                           /* 20.04.01 */   
          PUTYEAR(YEAR,0,S,&N,Ok);
        }
        else
        {
          S[N++] = *pf++;
          pf1++;
        }
        break;

      case 'h':
      case 'H':
        if (strncmp(pf,"HH24",4) == 0)
        {
          pf+=4; pf1+=4;                           /* 20.04.01 */   
          PUT2(h,S,&N,Ok);
          Add += 2;                                /* 20.04.01 */ 
        }
        else if (strncmp(pf,"HH12",4) == 0)
        {
          pf+=4; pf1+=4;
          PUT2((L_SWORD)(( h>12 ) ? ( h-12 ) : ( ( h==0 ) ? 12 : h )),S,&N,Ok); /* 07.06.05 */
          Add += 2;
        }
        else if (strncmp(pf,"HH",2) == 0)
        {
          pf+=2; pf1+=2;                           /* 20.04.01 */   
          PUT2(h,S,&N,Ok);
        }
        else
        {
          S[N++] = *pf++;
          pf1++;
        }
        break;

      case 'a':
      case 'A':
      case 'p':
      case 'P':
        if (strncmp(pf,"A.M.",4) == 0 ||
            strncmp(pf,"P.M.",4) == 0 )
        {
          strcpy( NAM, ( h>=12 ) ? ( ( h<25 ) ? "p.m." : "?.?." ) :"a.m." ); /* 15.08.12 */
          for (i=0;i<4;i++) 
            S[N++] = ( pf[i] == pf1[i] ) ? LAnsiToupper( NAM[i] ) : NAM[i];    /* 17.05.04 */
          pf+=4; pf1+=4;
        }
        else if (strncmp(pf,"AM",2) == 0 ||
                strncmp(pf,"PM",2) == 0)
        {
          strcpy( NAM, ( h>=12 ) ? ( ( h<25 ) ? "pm" : "??" ) :"am" );
          for (i=0;i<2;i++) 
             S[N++] = ( pf[i] == pf1[i] ) ? LAnsiToupper( NAM[i] ) : NAM[i];    /* 17.05.04 */
          pf+=2; pf1+=2;
        }
        else
        {
          S[N++] = *pf++;
          pf1++;
        }
        break;

      case 's':
      case 'S':
        if (strncmp(pf,"SS",2) == 0)
        {
          pf+=2; pf1+=2;                           /* 20.04.01 */   
          PUT2(s,S,&N,Ok);
        }
        else
        {
          S[N++] = *pf++;
          pf1++;
        }
        break;

      case 'f':
      case 'F':
        if (strncmp(pf,"FFF",3) == 0)
        {
          pf+=3; pf1+=3;
          PUT2(f,S,&N,Ok);
          S[N++] = '0';
        }
        else if (strncmp(pf,"FF",2) == 0)
        {
          pf+=2; pf1+=2;                           /* 20.04.01 */   
          PUT2(f,S,&N,Ok);
        }
        else
        {
          S[N++] = *pf++;
          pf1++;
        }
        break;

      default:
        S[N++] = *pf++;
        pf1++;                                   /* 20.04.01 */   
    }                                          /* ... 02.05.03 */
  }
  for (i=0; i<Add; i++)                        /* 20.04.01 */
    S[N++] = ' ';                              /* 20.04.01 */
  S[N]='\0';

  return Ok;
} /* end TICKTOSTRF */
