/****************************************************************************
 *                       RELEX, Inc. DBMS Linter                            *
 *  This is unpublished proprietary source code of RELEX, Inc.              *
 *  The copyright notice does not evidence any actual or intended           *
 *  publication of such source code.                                        *
 *                                                                          *
 *                 Copyright (c) 1990-2012 RELEX, Inc.                      *
 *                                                                          *
 *                   This software was developed by                         *
 *                                                                          *
 *                            RELEX, Inc.                                   *
 *                         Voronezh, Russia                                 *
 *                                                                          *
 *                        All rights reserved.                              *
 ****************************************************************************
 AUTHOR:
 ----------------------------------------------------------------------------
 PURPOSE: get Weekday from DATE
 ----------------------------------------------------------------------------
 14.10.91 
 19.05.92 -- UNIX-386, Ustinov Ju.
 09.06.92 -- Peshkoff
 17.12.92 -- AVO (VAX)
 22.12.92 -- AVO
 10.06.97 -- Kent
 15.05.97 -- DV
 20.07.97 -- Ermakov (Copyright) 
 22.05.98 -- Kent (new dlldefs.h)
 05.10.99 -- Ermakov (For QNX)
 18.12.00 -- VxWorks
 20.04.01 -- KOV (formats are added + functions NAMMON,MONNAM,NAMDAY,DAYNAM are changed)
 21.04.01 -- DV (TICKFSTRLEN function added)
 25.04.01 -- KOV added 4 functions
 03.05.01 -- KOV defines for new functions (TIMESTAMPADD, TIMESTAMPDIFF)
 17.02.06 -- DV (Interval time)
 14.12.07 -- KOV (Changes for TT#16356: date can't be greater than 31.12.9999)
 01.09.10 -- DV (Interval time operations)
 03.07.12 -- Ganin (Const qualifiers)
*/

#ifndef _TICK_H_
#define _TICK_H_

#include "dlldefs.h"                             /* 22.05.98 */
#include "decimals.h"

#ifdef __cplusplus
extern "C" {
#endif

#define MAXYEAR 9999
#define MAXMON 12
#define MAXDAY 31

extern L_DECIMAL L_MAX_DATE;                               /* 14.12.07 */

#define DAY1YEAR ((L_LONG)(365))
#define DAY4YEAR ((L_LONG)(DAY1YEAR * 4 + 1))
#define DAY1CENT ((L_LONG)(DAY4YEAR * 25 - 1))
#define DAY4CENT ((L_LONG)(DAY1CENT * 4 + 1))

typedef char NAMKAL[20];

/* 03.05.01 ... */
/* SQL_TIMEDATE_ADD_INTERVALS and SQL_TIMEDATE_DIFF_INTERVALS functions from sqlext.h */
#define TSI_FRAC_SECOND              0x00000001L
#define TSI_SECOND                   0x00000002L
#define TSI_MINUTE                   0x00000004L
#define TSI_HOUR                     0x00000008L
#define TSI_DAY                      0x00000010L
#define TSI_WEEK                     0x00000020L
#define TSI_MONTH                    0x00000040L
#define TSI_QUARTER                  0x00000080L
#define TSI_YEAR                     0x00000100L
/* ... 03.05.01 */

/******************************************************/

#ifndef _DATE_

extern void    DLLAPI NAMMON(L_SWORD MON, L_CHAR *NAM, L_SWORD koder);            /* AVO 26.12.92 */
extern L_SWORD DLLAPI MONNAM(L_CHAR *NAM, L_SWORD koder);                         /* AVO 26.12.92 */
extern void    DLLAPI NAMDAY(L_SWORD DAYWEEK,L_CHAR *NAM,L_SWORD kode);
extern L_SWORD DLLAPI DAYNAM(L_CHAR *NAM);
extern void    DLLAPI NAMMONex(L_SWORD MON, L_CHAR *NAM, L_SWORD koder, L_SWORD Count);   /* 20.04.01 25.04.01 */
extern L_SWORD DLLAPI MONNAMex(L_CHAR *NAM, L_SWORD koder, L_SWORD Count);                /* 20.04.01 25.04.01 */
extern void    DLLAPI NAMDAYex(L_SWORD DAYWEEK,L_CHAR *NAM,L_SWORD koder, L_SWORD Count); /* 20.04.01 25.04.01 */
extern L_SWORD DLLAPI DAYNAMex(L_CHAR *NAM, L_SWORD Count);                               /* 20.04.01 25.04.01 */
extern L_SWORD DLLAPI BIGYEAR(L_SWORD  YEAR);
extern L_SWORD DLLAPI DAYMON(L_SWORD MON, L_SWORD YEAR);
extern L_SWORD DLLAPI DAYYEAR(L_SWORD DAY, L_SWORD MON, L_SWORD YEAR);
extern L_LONG  DLLAPI DAYNUMBERDATE(L_SWORD DAY, L_SWORD MON, L_SWORD YEAR);
extern void    DLLAPI DATEDAYNUMBER(L_LONG NN, L_SWORD *DAY, L_SWORD *MON, L_SWORD *YEAR);
extern void    DLLAPI EDATEDAYNUMBER(L_DECIMAL D, L_LONG NN, L_SWORD *DAY, L_SWORD *MON, L_SWORD *YEAR);
extern L_SWORD DLLAPI WEEKDAYNUMBER(L_LONG N);
extern void    DLLAPI DATETOTICK(L_LONG L1, L_LONG L2, L_DECIMAL D);
extern void    DLLAPI PUTDIG(L_SWORD C, L_CHAR *S, L_SWORD *N, L_SWORD Ok);
extern void    DLLAPI PUT2(L_SWORD X, L_CHAR *S, L_SWORD *N, L_SWORD Ok);
extern void    DLLAPI PUT3(L_SWORD X, L_CHAR *S, L_SWORD *N, L_SWORD Ok);                 /* 20.04.01 */
extern void    DLLAPI PUTYEAR(L_SWORD Y, L_SWORD ISCENTURY, L_CHAR *S, L_SWORD *N, L_SWORD Ok);
extern void    DLLAPI TICKTODATE(const L_DECIMAL D, L_LONG *L1, L_LONG *L2);
extern L_SWORD DLLAPI STRTOTICK(const L_CHAR *S, L_DECIMAL D);

extern void    DLLAPI TICKTOSTR(L_DECIMAL const D,
                                L_SWORD   DATEFORMAT,
                                L_SWORD   ISCENTURY,
                                L_CHAR *  S,
                                L_SWORD   IS_LEN);              /* 23.12.92 AVO */
extern void    DLLAPI ADDDATE(L_DECIMAL const D1,
                              L_DECIMAL const D2,
                              L_DECIMAL       D3);
extern void    DLLAPI SUBDATE(L_DECIMAL const D1,
                              L_DECIMAL const D2,
                              L_DECIMAL       D3);
extern L_SWORD DLLAPI CMPDATE(L_DECIMAL const D1,
                              L_DECIMAL const D2);
extern L_SWORD DLLAPI TICKTOSTRF(const L_DECIMAL D,
                                 const L_CHAR   *F,             /* 03.07.12 */
                                 L_CHAR   *S);                  /* 09.12.94 DV */
extern L_SWORD DLLAPI STRTOTICKF(const L_CHAR   *S,             /* 03.07.12 */
                                 const L_CHAR   *F,             /* 03.07.12 */
                                 L_DECIMAL D);                  /* 09.12.94 DV */
extern L_SWORD DLLAPI TICKFSTRLEN(L_CHAR *F, L_SWORD FLEN);	/* 21.04.01 */
extern void    DLLAPI ADDMONTHSTODATE(L_DECIMAL const D1,       /* 15.05.97 DV */
                                      L_DECIMAL const D2,
                                      L_DECIMAL       D3);
extern void    DLLAPI SUBMONTHSFROMDATE(L_DECIMAL const D1,       /* 15.05.97 DV */
                                        L_DECIMAL const D2,
                                        L_DECIMAL     D3);
extern void    DLLAPI MULMONTHS(L_DECIMAL        D1,            /* 01.09.10 */
                                L_DECIMAL        D2,
                                L_DECIMAL        D3);
extern void    DLLAPI DIVMONTHS1(L_DECIMAL       D1,            /* 01.09.10 */
                                 L_DECIMAL       D2,
                                 L_DECIMAL       D3);
extern void    DLLAPI DIVMONTHS2(L_DECIMAL       D1,            /* 01.09.10 */
                                 L_DECIMAL       D2,
                                 L_DECIMAL       D3);
extern DECSTATUS DLLAPI GETTICKSTATUS(const L_DECIMAL D);       /* 01.09.10 */

#ifdef __cplusplus
}
#endif

#endif

#endif

