/***************************************************************************
 *                         RELEX, Inc. DBMS Linter                          *
 *  This is unpublished propietary source code of RELEX, Inc.               *
 *  The copyright notice does not evidence any actual or intended           *
 *  publication of such source code.                                        *
 *                                                                          *
 *                 Copyright (c) 1990-2014 RELEX, Inc.                      *
 *                                                                          *
 *                   This software was developed by                         *
 *                                                                          *
 *                            RELEX, Inc.                                   *
 *                         Voronezh, Russia                                 *
 *                                                                          *
 *                        All rights reserved.                              *
 ****************************************************************************
 AUTHOR:
 ----------------------------------------------------------------------------
 PURPOSE: convert string to TICK
 ----------------------------------------------------------------------------
 14.10.91 
 22.12.92 -- AVO
 04.01.95 -- Borisenkov
 05.12.96 -- Borisenkov (fixed bug found by A.Exarevsky)
 20.07.97 -- Ermakov (Copyright) 
 03.09.97 -- Ermakov (OS2)
 14.09.99 -- Ermakov (For OS9000)
 05.10.99 -- Ermakov (For QNX)
 29.12.99 -- DV (Y2K)
 22.12.00 -- DV (Additional default formats)
 13.01.01 -- DV (May be different second precision)
 15.01.01 -- DV (GETFRACT not static)
 18.12.00 -- VxWorks
 20.04.01 -- KOV (formats are added + functions NAMMON,MONNAM,NAMDAY,DAYNAM are changed)
 08.06.01 -- DV (Disable empty datetime string for versions >= 5.9) 
 17.05.04 -- Galizin   ( Linux glibc 2.3 TT#11078)
 19.06.05 -- Ermakov   ( Corrected pointer manipulation in Linux (!) )
 08.02.06 -- KOV (Correction STRTOTICK for '0001-01-01'<->'YYYY.MM.DD')
 09.02.06 -- KOV (Added syntax: cast '19630816' as date)
 10.02.06 -- KOV (Correction for previous update)
 19.07.12 -- Galizin (OpenVMS)
 27.02.13 -- DV  (Default time recognition)
 18.08.14 -- KOV (Initialization of variables)
*/

#if defined(__MSDOS__) || defined(INTER_OS2)                  /* 30.09.97 */
#include <ctype.h>
#include "decimals.h"
#include "tick.h"           /* 19.05.92 */
#define _DATE_
#endif

#ifdef INTER_MSWINDOWS
#include <ctype.h>
#include "tick.h"           /* 19.05.92 */
#define _DATE_
#endif

#if defined(UNIX) || defined(QNX) || defined(VXWORKS) || defined(VMS) /* 19.07.12 05.10.99 */ /* 18.12.00 */
#include <ctype.h>
#include "decimals.h"
#include "tick.h"           /* 19.05.92 */
#define _DATE_
#endif

/* 14.09.99 ... */
#ifdef OS9000
#include <sys/ctypes>
#include "decimals.h"
#include "tick.h"
#define _DATE_
#endif
/* ... 14.09.99 */

#ifndef _DATE_
ERROR: UNDEF SYSTEM
#endif


int GETFRACT(						/* 13.01.01 ... */ /* 15.01.01 */
	   const char **pt,
	   int	imp,
           int  lim);					/* ... 13.01.01 */
int GETZAL(
	   const char **pt,
	   int	lim);
int todate(
	  int	d,
	  int	m,
	  int	y,
	  L_LONG	*dt);
int totime(
	  int	h,
	  int	m,
	  int	s,
	  int	t,
	  L_LONG	*tm);
/*
**  Convert DATE from string to L_DECIMAL
**
**  AVO
*/
L_SWORD DLLAPI STRTOTICK(L_CHAR const *S,
	                 L_DECIMAL D)
{
  L_SWORD Ok;
  L_CHAR NAM[4];
  L_SWORD DAY,MON,YEAR,h,m,s,t,tmp,i,hasc;
  L_LONG dt = 0,tm = 0;                                    /* 18.08.14 */
  const char *ps, *ps1, *prevps;                           /* 08.02.06 */
  char sep = '\0';
  L_BOOL bNoSep;

  DAY=MON=YEAR=h=m=s=t=hasc=0;
  Ok = L_TTRUE;
  ps = S;

  while(*ps == ' ') ps++;
  prevps = ps;                                             /* 08.02.06 */
  DAY=GETZAL(&ps,4);
  if ( DAY > 31 ||                                         /* 22.12.00 ... */
       ( ps - prevps ) >= 4 )                              /* 08.02.06 */
  {
    YEAR = DAY;
    DAY = 0;
    hasc = 1;                                              /* 08.02.06 */
  }                                                        /* ... 22.12.00 */
  if ( *ps >= '0' && *ps <= '9' )                          /* 09.02.06 ... */ /* 10.02.06 */
  {
    bNoSep = L_TTRUE;
    sep = '\0';
  }
  else
    bNoSep = L_TFALSE;
  if ( *ps=='.' || *ps=='/' || *ps=='-' || bNoSep )
  {
    if ( !bNoSep )                                         /* ... 09.02.06 */
      sep = *ps++;
    if (sep != '-' || (*ps >= '0' && *ps <= '9'))          /* 09.11.94 */ /* 22.12.00 */
      MON = GETZAL(&ps,2);
    else
    {
      for (i=0;i<3;i++) 
      {
        NAM[i] = LAnsiTolower(*ps);                        /* 17.05.04 */
        ps++;                                              /* 19.06.05 */
      }
      NAM[3] = 0;
      MON=MONNAM(&NAM[0],0);
    }
    if ( *ps == sep || bNoSep )                            /* 09.02.06 */
    {
      if ( !bNoSep )                                       /* 09.02.06 */
	      ps++; 
      ps1 = ps; 
      if (!YEAR)					   /* 22.12.00 ... */
        YEAR = GETZAL(&ps,4); 
      else
        DAY = GETZAL(&ps,4); 				   /* ... 22.12.00 */
	    if (ps-ps1 > 2) 
        hasc = 1;			/* 04.01.95 */
    }
    if (*ps==':' || *ps == ' ')			/* 22.12.00 */
    {
	    ps++; 
      h = GETZAL(&ps,2);
	    if (*ps==':')
      {
	      ps++; 
        m = GETZAL(&ps,2);
	      if(*ps==':')
	      {
	        ps++; 
          s = GETZAL(&ps,2);
	        if (*ps=='.')
          { 
            ps++; 
            t = GETFRACT(&ps,2,10);
          }
	      }
      }
    }
    while(*ps == ' ') 
      ps++;
  }
  else if (*ps == ':')                                  /* 27.02.13 ... */
  {
    sep = *ps++; 
    h = YEAR ? YEAR : DAY;
    DAY = YEAR = MON = 0;    
    hasc = 1;
    m = GETZAL(&ps,2);
    if (*ps==':')
    {
      ps++; 
      s = GETZAL(&ps,2);
      if (*ps=='.')
      { 
        ps++; 
        t = GETFRACT(&ps,2,10);
      }
    }
  }                                                     /* ... 27.02.13 */
  else
    Ok = L_TFALSE;
  if (sep == '/') 
  { 
    tmp = DAY; 
    DAY = MON; 
    MON = tmp; 
  }	/* For ORACLE */
  if (YEAR < 100 && !hasc)      			/* 04.01.95   */
  {
    /* 29.12.99 ... */
    if (YEAR <= 37) 
      YEAR += 2000;		
     else
       YEAR += 1900;		
  }
    /* ... 29.12.99 */
  if (*ps != 0) 
    Ok = L_TFALSE;
  if (!Ok) 
    return L_TFALSE;
  if (YEAR==0 && MON==0)
  { 
    Ok = totime(h,m,s,t,&tm); 
    dt = DAY; 
  }	 /* time interval */
  else	
     Ok = todate(DAY,MON,YEAR,&dt) && totime(h,m,s,t,&tm);
  DATETOTICK(dt,tm,D);
  return Ok;
}

/*
** Convert TIME to L_LONG
**
**  AVO
*/
int totime(
	  int	h,
	  int	m,
	  int	s,
	  int	t,
	  L_LONG	*tm)
{
*tm = 0;
if((h>=24) || (h<0) || (m>=60) || (m<0) || (s>=60) || (s<0) || (t>=100) || (t<0))
  return L_TFALSE;		/* 05.12.96 */
*tm = (L_LONG)h*360000+(L_LONG)m*6000+(L_LONG)s*100+t;
return L_TTRUE;
}

/*
** Convert DATE to L_LONG
**
**  AVO
*/
int todate(
	  int	d,
	  int	m,
	  int	y,
	  L_LONG *dt)
{
*dt = 0;
if ((d<=0 || d>DAYMON((L_SWORD)m,(L_SWORD)y) || m>12 || m<=0) && (m!=0 || y!=0))	/* 04.01.95 */
  return L_TFALSE;
*dt = DAYNUMBERDATE((L_SWORD)d,(L_SWORD)m,(L_SWORD)y);
return L_TTRUE;
}

/*
**  Convert full DATE to L_DECIMAL
**
**  AVO
*/
int GETZAL(const char **pt,
	   int	lim)
{
int s,l;
s = 0; l=1;
while((l<=lim)&&(**pt<='9')&&(**pt>='0'))
 {
 s = s*10 + (**pt) - '0'; l++;
 (*pt)++;
 }
return s;
}

int GETFRACT(						/* 13.01.01 ... */
	   const char **pt,
	   int	imp,
           int  lim)
{
int s,l;
s = 0; l=1;
while((l<=lim)&&(**pt<='9')&&(**pt>='0'))
 {
 if (l <= imp)
   s = s*10 + (**pt) - '0'; 
 l++;
 (*pt)++;
 }
while (l <= imp) {
  s *= 10; l++; }
return s;
}							/* ... 13.01.01 */
