/***************************************************************************
 *                           RELEX, Inc. DBMS Linter                        *
 *  This is unpublished proprietary source code of RELEX, Inc.              *
 *  The copyright notice does not evidence any actual or intended           *
 *  publication of such source code.                                        *
 *                                                                          *
 *                 Copyright (c) 1990-2012 RELEX, Inc.                      *
 *                                                                          *
 *                   This software was developed by                         *
 *                                                                          *
 *                            RELEX, Inc.                                   *
 *                         Voronezh, Russia                                 *
 *                                                                          *
 *                        All rights reserved.                              *
 ****************************************************************************
 AUTHOR:
 ----------------------------------------------------------------------------
 PURPOSE: convert TICK to DATE
 ----------------------------------------------------------------------------
 14.10.91 
 22.12.92 -- AVO
 20.07.97 -- Ermakov (Copyright) 
 14.09.99 -- Ermakov (For OS9000)
 05.10.99 -- Ermakov (For QNX)
 18.12.00 -- VxWorks
 02.05.03 -- DV (Optimization)
 28.09.05 -- FLY (Suppress warnings)
 16.02.06 -- KOV (Check interval flag)
 19.06.08 -- Ermakov -- avoid __moddi3 for GCC
 01.08.08 -- Ermakov -- correct prev for QNX
 03.12.08 -- Ermakov -- corrected 19.06.08 for negative interval dates
 01.09.10 -- DV (Interval time operations)
 19.07.12 -- Galizin (OpenVMS)
 11.04.13 -- Ermakov (remove extra code)
*/

#ifdef __MSDOS__
#include <ctype.h>
#include "decimals.h"
#include "tick.h"           /* 19.05.92 */
#define _DATE_
#endif

#ifdef INTER_OS2
#include <ctype.h>
#include "tick.h"
#define _DATE_
#endif

#ifdef __INTER_DLL__
#ifndef INTER_MSWINDOWS
#define INTER_MSWINDOWS
#endif
#endif
#ifdef INTER_MSWINDOWS
#include <ctype.h>
#include "tick.h"
#define _DATE_
#endif

#if defined(UNIX) || defined(QNX) || defined(VXWORKS) || defined(VMS)   /* 19.07.12 05.10.99 */ /* 18.12.00 */
#include <ctype.h>
#include "decimals.h"
#include "tick.h"           /* 19.05.92 */
#define _DATE_
#endif

/* 14.09.99 ... */
#ifdef OS9000
#include <sys/ctypes>
#include "decimals.h"
#include "tick.h"
#define _DATE_
#endif
/* ... 14.09.99 */

#ifndef _DATE_
ERROR: UNDEF SYSTEM
#endif

#ifdef INT64_EMULATION_ENABLE                                    /* 01.08.08 */
#define GCC_AVOID
#else
/*
For GCC we must convert long long to unsigned long long to avoid
use __moddi3 function from gcc spec. library
*/
#define GCC_AVOID (L_UDLONG)
#endif

#include "int64.h"                                      /* 02.05.03 */

void DLLAPI TICKTODATE(const L_DECIMAL DD, L_LONG *L1, L_LONG *L2)
/********************************************/
{
  L_DLONG L, DL, L1L, L2L;        /* 02.05.03 */
  L_BOOL bInterval;                                         /* 16.02.06 ... */
  L_DECIMAL D;

  COPYDEC(DD, D);

  if ( D[15] == 1 )
  {
    bInterval = L_TTRUE;
    D[15] = 0;
  }
  else
    bInterval = L_TFALSE;                                   /* ... 16.02.06 */
  if (GETTICKSTATUS(D) != DECERROR)                         /* 01.09.10 */
  {
    L_BOOL FlNeg = L_TFALSE;                                /* 03.12.08 */

    i64_FromLong(8640000,L);                                /* 02.05.03 ... */
    DecToDLong(D,&DL,0);
    if ( i64_IfNeg (DL) )                                   /* 03.12.08 ... */
    {
      FlNeg = 1;
      DL = i64_Neg ( DL );
    }                                                       /* ... 03.12.08 */

    L1L = i64_DivRest(GCC_AVOID DL,L,&L2L);                 /* 01.08.08 */ /* 19.06.08 */ 
    if (FlNeg)                                              /* 03.12.08 */
      L1L = i64_Neg ( L1L );                                /* 03.12.08 */
    i64_ToLong(L1L,(*L1));
    i64_ToLong(L2L,(*L2));                                  /* ... 02.05.03 */
  }
  else 
    *L1=0;
}
