/***************************************************************************
 *                           RELEX, Inc. DBMS Linter                        *
 *  This is unpublished propietary source code of RELEX, Inc.               *
 *  The copyright notice does not evidence any actual or intended           *
 *  publication of such source code.                                        *
 *                                                                          *
 *                 Copyright (c) 1990-2012 RELEX, Inc.                      *
 *                                                                          *
 *                   This software was developed by                         *
 *                                                                          *
 *                            RELEX, Inc.                                   *
 *                         Voronezh, Russia                                 *
 *                                                                          *
 *                        All rights reserved.                              *
 ****************************************************************************
 AUTHOR:
 ----------------------------------------------------------------------------
 PURPOSE: convert TICK to string
 ----------------------------------------------------------------------------
 14.10.91 
 22.12.92 -- AVO
 20.07.97 -- Ermakov (Copyright) 
 28.08.97 -- Ermakov (Added prototype)
 19.09.98 -- Kent
 23.07.99 -- DV (Do not clear tail zeros if time present)
 14.09.99 -- Ermakov (For OS9000)
 05.10.99 -- Ermakov (For QNX)
 18.12.00 -- VxWorks
 20.04.01 -- KOV (formats are added + functions NAMMON,MONNAM,NAMDAY,DAYNAM are changed)
 08.06.01 -- DV (Do not output empty datetime string for versions >= 5.9) 
 17.02.06 -- DV (Interval time)
 01.09.10 -- DV (Interval time, including negative)
 19.07.12 -- Galizin (OpenVMS)
 11.04.13 -- Ermakov (remove extra code)
*/

#ifdef __MSDOS__
#include <ctype.h>
#include <string.h>                                       /* 28.08.97 */
#include "decimals.h"
#include "tick.h"           /* 19.05.92 */
#define _DATE_
#endif

#ifdef INTER_OS2
#include <ctype.h>
#include "tick.h"
#define _DATE_
#endif

#ifdef INTER_MSWINDOWS
#include <ctype.h>
#include <string.h>                                       /* 19.09.98 */
#include "tick.h"
#define _DATE_
#endif

#if defined(UNIX) || defined(QNX) || defined(VXWORKS) || defined(VMS)  /* 19.07.12 05.10.99 */ /* 18.12.00 */
#include <ctype.h>
#include "decimals.h"
#include "tick.h"           /* 19.05.92 */
#define _DATE_
#endif

/* 14.09.99 ... */
#ifdef OS9000
#include <sys/ctypes>
#include "decimals.h"
#include "tick.h"
#define _DATE_
#endif
/* ... 14.09.99 */

#ifndef _DATE_
ERROR: UNDEF SYSTEM
#endif


void DLLAPI PUTDIG(L_SWORD C, L_CHAR *S, L_SWORD *N, L_SWORD Ok)
/***********************/
{ S[(*N)++] = Ok ? (L_CHAR)(C % 10 + (L_SWORD)('0')) : '?';   /* 18.02.92 M */
}
void DLLAPI PUT2(L_SWORD X, L_CHAR *S, L_SWORD *N, L_SWORD Ok)
/*********************/
{ 
  PUTDIG((L_SWORD)(X / 10),S,N,Ok); 
  PUTDIG(X,S,N,Ok);
}
void DLLAPI PUT3(L_SWORD X, L_CHAR *S, L_SWORD *N, L_SWORD Ok)        /* 20.04.01 */
/*********************/
{ 
  PUTDIG((L_SWORD)(X / 100),S,N,Ok);
  PUTDIG((L_SWORD)(X / 10),S,N,Ok); 
  PUTDIG(X,S,N,Ok);
}
void DLLAPI PUTYEAR(L_SWORD Y, L_SWORD ISCENTURY, L_CHAR *S, L_SWORD *N, L_SWORD Ok)
/**********************************************/
{ 
  if (ISCENTURY) 
    PUT2((L_SWORD)(Y / 100),S,N,Ok);                  /* 14.09.95 */
  PUT2((L_SWORD)(Y % 100),S,N,Ok);
}
void  DLLAPI TICKTOSTR(L_DECIMAL const DD,
                       L_SWORD   DATEFORMAT,
                       L_SWORD   ISCENTURY,
                       L_CHAR   *S,
                       L_SWORD   IS_LEN)              /* 23.12.92 AVO */
/**************************************/
{
  L_CHAR NAM[10];                                  /* 09.12.94 DV *//* 20.04.01 */
  L_SWORD i,N,Ok;
  L_SWORD DAY,MON,YEAR;
  L_LONG lday,ltick;
  DECSTATUS st;                                /* 01.09.10 ... */
  L_DECIMAL D;

  COPYDEC(DD, D);
  st = GETTICKSTATUS(D);
  Ok = (st != DECERROR);
  if (st == DECNEGATIV)
    SETDECSTATUS(DECPOSITIV,D);                /* ... 01.09.10 */

  N=0;
  TICKTODATE(D,&lday,&ltick);

  EDATEDAYNUMBER(D,lday,&DAY,&MON,&YEAR);   /* 24.12.92 AVO */ /* 17.02.06 */

  switch (DATEFORMAT) 
  {
    case 0:   {
      PUT2(DAY,S,&N,Ok);
      S[N++]='.';
      PUT2(MON,S,&N,Ok);
      S[N++]='.';
      PUTYEAR(YEAR,ISCENTURY,S,&N,Ok);
    }
    break;

    case 1:   {
      PUT2(MON,S,&N,Ok);
      S[N++]='/';
      PUT2(DAY,S,&N,Ok);
      S[N++]='/';
      PUTYEAR(YEAR,ISCENTURY,S,&N,Ok);
    }
    break;

    default: {
      PUT2(DAY,S,&N,Ok);
      S[N++]='-';
      NAMMON(MON,&NAM[0],0);
      for (i=0;i<3;i++) S[N++] = Ok ? NAM[i] : '?';
      S[N++]='-';
      PUTYEAR(YEAR,ISCENTURY,S,&N,Ok);
    }
  }

  S[N++]=':';
  PUT2((L_SWORD)(ltick/360000),S,&N,Ok);
  ltick%=360000;

  S[N++]=':';
  PUT2((L_SWORD)(ltick/6000),S,&N,Ok);
  ltick%=6000;

  S[N++]=':';
  PUT2((L_SWORD)(ltick/100),S,&N,Ok);
  ltick%=100;

  S[N++]='.';
  PUT2((L_SWORD)ltick,S,&N,Ok);

  S[N]='\0';
} /* end TICKTOSTR */

