/****************************************************************************
 *                           RELEX, Inc. DBMS Linter                        *
 *  This is unpublished proprietary source code of RELEX, Inc.              *
 *  The copyright notice does not evidence any actual or intended           *
 *  publication of such source code.                                        *
 *                                                                          *
 *                  Copyright (c) 1990-2013 RELEX Inc.                      *
 *                                                                          *
 *                   This software was developed by                         *
 *                                                                          *
 *                            RELEX, Inc.                                   *
 *                         Voronezh, Russia                                 *
 *                                                                          *
 *                        All rights reserved.                              *
 ****************************************************************************
 AUTHOR: Peshkoff
 ----------------------------------------------------------------------------
 PURPOSE: process interchange
 ----------------------------------------------------------------------------
 REVISION HISTORY:
 30.08.96 - Prochrov D.  Signal mask
 12.09.96 - Ermakov      VAX/VMS system now supported
 12.10.96 - Peshkoff     Shared memory for UNIX SVR4, is Linter alive
 17.11.96 - Dudov        Win32s
 14.01.97 - Peshkoff
 15.01.97 - Peshkoff
 22.01.97 - Kent         NULL values
 10.02.97 - Kent         Server check
 12.02.97 - Peshkoff
 15.02.97 - Peshkoff     LINTER_CP
 28.05.97 - DV           FCUR
 31.07.97 - Kent         inter_wait_
 28.08.97 - Ermakov      Linux
 16.11.97 - Kent         Long msg.
 20.11.97 - Kent, ...    Backup
 27.11.97 - Peshkof      (SVR4->SVR42)
 14.12.97 - Ermakov      ( < replaced by >= and others )
 05.01.98 - Ermakov      ( MSV... )
 29.01.98 - Ermakov      CheckProcess && CreateEvent result analyzing
 11.03.98 - Kent         Remove ACL from mailslot
 12.03.98 - Kent         Check thread id when looking for free AST segment
 18.03.98 - Kent         Remove add 'space' for "OPEN"
 30.03.98 - Kent         Warnings shutdown (BYTE -> CHAR)
 06.04.98 - Kent         Reset event if already exist
 09.04.98 - Kent         Logic mistakes correction for WIN32
 04.04.98 - Ermakov      (DATA_MSBF InterMsg replaced by CBL)
 05.04.98 - Peshkoff     (QNX)
 21.05.98 - Peshkoff     (SUN - again!)
 25.05.98 - Kent         (Clear AstMsg->Actual if long msg.)
 26.05.98 - Kent         (Optimize code)
 31.05.98 - Ermakov      (UNIX'S)
 31.05.98 - Peshkoff     (LONG_ALIGN & UNIX)
 01.06.98 - Peshkoff     (Why it is only for this list of the commands?)
 03.06.98 - Ermakov      (QNX)
 04.06.98 - Kent         (Uninit bug fixed)
 20.06.98 - DV           (For OLTP)
 28.06.98 - Ermakov      (Message not read, - no ReceiveData)
 29.06.98 - Kent         (Parameter for COMT & RBAC)
 01.07.98 - Kent         (New user's AST proc.)
 01.07.98 - Kent         (BOOLEAN type for inter_wait_...)
 03.07.98 - Kent         (Add FNOD command)
 05.07.98 - Ermakov      (DIGITAL_UNIX)
 21.07.98 - Kent         (SHUT like OPEN)
 22.07.98 - Kent         (Disable sync. call from AST handler)
 27.07.98 - Kent         (ErrorHandler instead return(CBL->CodErr))
 29.07.98 - Kent         (Add parameters to ErrorHandler)
 28.08.98 - Mark         (Correct macroses min && max)
 10.09.98 - Kent         (New command 'DBCA')
 15.09.98 - Kent         (Proc_Raised_Exception handle)
 25.09.98 - Kent         (Save orginal CBL and set sync mode for M*** cmds)
 01.10.98 - Kent         (Receive data from server when Q_RECEIVE present)
 10.10.98 - Ermakov      (inter return only self-error (for async query).
                          For sync query may be wrong.)
 18.10.98 - Ermakov      (New macros SDKV2 for Linux. Instead of MSVS)
 28.11.98 - Kent         (Correct NULL-indicators buffer size calculation)
 11.12.98 - Kent         (New UNIX IPC via sockets)
 03.12.98 - Kent         (Correct logical mistake in WIN32 case & cosmetic)
 08.12.98 - Kent         (New edition for UninitLinterClient for WIN32)
 **.**.** - Kent         (Modifier `static' for some routines and remove it
                          for some vars for UNIX (for DBCAPI))
 11.12.98 - Kent         (LINTER_MBX support for Windows NT has released)
 22.12.98 - Kent         (Q_RECEIVE bug fixed)
 23.12.98 - Kent         (Win95 case bug fixed)
 11.01.99 - Kent         (New handle for SIGALRM)
 12.01.99 - Kent         (Enhanced handle case when server disappear)
 21.01.99 - Kent         (None static vars. if DBC_API defined)
 02.02.99 - Kent         (Sockets for Linux)
 05.02.99 - Kent         (Clear 0x10 bit if DATA_MSBF not defined)
 05.02.99 - Kent         (Add `static' for some functions)
 07.02.99 - Kent         (Save msg. len . in Msg_Type)
 13.02.99 - Ermakov      (naming for SUN speration systems)
 13.02.99 - Kent         (Clear server handle if msg. not sent)
 18.02.99 - Maxim        (Add 'EHRS' - End of Hot Reserve Synchronization)
 28.02.99 - Kent         (Improve work of sockets version for UNIX)
 01.03.99 - Peshkoff     (LONG_ALIGN)
 02.03.99 - Kent         (SIGALRM is not needed for SOCKETS version)
 15.03.99 - E.Exarevsky  (Use sigemptyset/sigaddset to change signal mask
                          OLD_SIGWRK macros added)
 16.03.99 - E.Exarevsky  (OS_AIX,OS_SGI)
 22.03.99 - Kent         (Check is sender-thread alive)
 29.03.99 - Kent         (If EAGAIN no read from socket)
 29.03.99 - Kent         (Via sockets SGI works like BSD)
 31.03.99 - Kent         (2nd parameter `sigaddset' bug fixed)
 07.04.99 - E.Exarevsky  (Don't use semaphores)
 29.04.99 - Kent         (Uninit if fork has occured)
 10.05.99 - Kent         (Via sockets AIX works like BSD)
 11.05.99 - Kent         (Set in/out buffer size for socket)
 17.05.99 - Kent         (Server checking bug fixed)
 26.05.99 - Maxim        (add quant counter command)
 19.05.99 - Kent         (Work with SIGALRM bug fixed)
 21.06.99 - Kent         (Receive long msg. if buffer is ok)
 26.07.99 - Kononyhin    (Add GETI to comptab)
 08.09.99 - Ermakov      (Check for 2 deinitialization)
 09.09.99 - Ermakov      (Correct Thr work. Remove thl occupation for AstInd &&
                          Added Sv. SetEvent enable continuing for inter_wait and
                          CBL->... may not exist)
 22.09.99 - Kent         (Check is server alive)
 23.10.99 - Ermakov(Kent)(For NO_ASYNC calls - for EMASS)
 27.10.99 - Ermakov      (CheckServer in QNX. Only for local servers.)
 03.11.99 - Kent         (Correct threaded work for WIN32)
 12.12.99 - Kent         (Modify threaded work for WIN95)
 21.12.99 - Kent         (Use MAXMSG instead LENBUF for socket buffer)
 24.01.00 - DV           (Encodings corrected)
 25.01.00 - Kent         (Clear "system" bits in PrzExe)
 26.01.00 - Kent         (Fix async problem for Win95, implement timeout
                          for inter_sync (Windows ** only))
 10.02.00 - Ermakov      (Added SRVR command for Os9000. Used for changes
                          default server name in network client)
 16.02.00 - Kent         (Fix bug with timeout in inter_sync)
 17.02.00 - Ermakov      (Correct socket write operation)
 29.02.00 - Ermakov      (OS9000)
 01.03.00 - Kent         (Correct socket read operation)
 10.03.00 - Ermakov      (EAGAIN coder treat as restart operation code)
 28.03.00 - Ermakov      (For IPC exchange remove extra label)
 04.04.00 - Galizin      (Unlock signals)
 04.04.00 - Kent         (Check async. query to complete before sync
                          Add new PrzExe flags for async. repl.
                          New AstMsg->Actual status 2)
 13.04.00 - Kent         (Don't try read next msg if NO_ASYNC)
 18.04.00 - DV           (New BLOB processing commands)
 02.06.00 - DV           (Unknown LINTER_CP = no LINTER_CP)
 10.06.00 - Kent         (Insert ZAPKORTEGB & check AST slot for actual)
 10.06.00 - Ermakov      (OS9000 correction)
 10.07.00 - Ermakov      (OS9000 correction)
 14.07.00 - Ermakov      (OS9000 correction)
 31.07.00 - Maxim        (correct pointer error - truncate buffer)
 01.08.00 - Kent         (Correct some syntax errors for WIN16)
 29.08.00 - Pasec        (move getenv by else)
 31.08.00 - Maxim        (correct zero truncate for BKUP - odd byte)
 05.09.00 - Ermakov      (Check answer size)
 12.09.00 - Kent         (New algorithm for `FSPC')
 24.10.00 - Kent         (Always set signal handler for PHP_LINTER)
 27.10.00 - DV           (Do not change RPZEXE bits for RPAG & WPAG)
 05.11.00 - Ermakov      (QNX)
 05.12.00 - Procholov    (not need now make global if DBC_API)
 26.12.00 - Ermakov      (IDENT's)
 27.12.00 - Kent         (IDENT's)
 28.03.01 - Galizin      ( memset for IRIX )
 05.04.01 - Galizin      ( HPUX )
 11.01.01 - M&E          ( VxWorks, OS2000. Changes for QNX: %x && %d )
 07.04.01 - Ermakov      ( Correct for VX )
 03.05.01 - Galizin      ( OS2000 )
 15.05.01 - Galizin      ( UninitLinterClient for VXWORKS )
 17.05.01 - Kent         (If NO_ASYNC defined all queries are synchro.)
 19.05.01 - Ermakov      (Unlink mqueue for OS2000 + VxWorks)
 04.06.01 - Ermakov      (VxWorks)
 06.06.01 - Peshkof,Galizin ( Not set handler SIGIO for NO_ASYNC )
 18.06.01 - Ermakov      ( memcpy -> memmove specials for IRIX )
 29.06.01 - Galizin      ( Lock intlib for VXWORKS )
 24.08.01 - Galizin      ( Work to no align data #2717)
 16.10.01 - Galizin      ( For OS2000 use koi8 as default codepage )
 18.10.01 - DV           ( Long records )
 22.10.01 - Kent         (Internationalization support)
 30.10.01 - AVE          ( GPRC command )
 02.11.01 - Ermakov      ( For NOPAR command correct Kent's at 22.10.01. Only
                           some command must have LOCAL CODEPAGE installed)
 02.11.01 - Kent         (Get user default codepage from the system for Win*)
 22.11.01 - Kent         (Adaptive to work in UNICODE environment for WIN32
                          Use TCHAR instead CHAR where needed, add macro TEXT
                          for string constants)
 02.11.01 - Kent         (Get user default codepage from thr system for Win*)
 29.11.01 - KOV          (CHARSET in parameters)
 04.12.01 - KOV          ( ifdef PREPARE added )
 10.12.01 - DV           ( Long records bug fixed )
 26.12.01 - Kent         (Add new flags and inter_control requests for
                          using UTF8 & OEM CP for Win32)
 08.04.02 - Kent         (Windows CE)
 23.05.02 - Kent         (Win32 IPC optimization PT#630)
 27.05.02 - Kent         (Cosmetic changes for compilation under WinCE and ver. 6.xx)
 14.06.02 - Galizin, Korotchenko (Update for long align architectures PT#698)
 20.06.02 - DV           (Debugging output removed)
 24.06.02 - Kent         (Reread env. request for inter_control)
 03.07.02 - RAW          (MVCC. isoation levels)
 05.07.02 - Galizin      (Reset socket descriptor values into uninit procedure PT#940 )
 10.07.02 - Galizin      (For QNX RTP bild PT#957)
 22.07.02 - KOV, Kent    (Query length checking corrected)
 29.07.02 - Galizin      (Remove warning for linux PT#1046)
 22.08.02 - Galizin      (Pthread version PT#568)
 23.09.02 - Ermakov      (For WINCE moved ReleaseMutex from !WINCE section)
 26.09.02 - Goncharov    (Correcting code page names for english locale)
 09.10.02 - Ermakov      (Allow REPLICATION with ODBC.... May by wrong...)
 28.10.02 - Galizin      ( Set close on exec mode for socket )
 28.10.02 - Ermakov      (Added Q_USE_UTF8)
 28.10.02 - Ermakov      (OPEN may return new structure)
 02.11.02 - Ermakov      ( 64k records processing corrected )
 04.11.02 - Galizin,Ermakov,Pasec ( POSIX_threads for long messages support)
 11.11.02 - Galizin      (Get tmp path from environment PT#1800)
 15.11.02 - KOV          (Changes for ADO)
 16.11.02 - KOV,Ermakov  (Code page number for WinCE)
 22.11.02 - Ermakov      (SetDefCodepage for 6.0)
 16.12.02 - KOV          (Error changed: ERROPENQUE->ERRWRITEMSG)
 08.01.03 - Kononyhin    (different exchanges for WIN32;
                          correct async working && async working under WIN95;
                          get server PID from shr. memory and use it for checking server)
 22.01.03 - Kononyhin    (use mutex instead of critical section to synchronize access to exchange objects)
 09.02.03 - Ermakov, Galizin (Close socket if disconnect and no active ast. Next query retutned 1003)
 14.02.03 - Galizin ( Prevent work from signal handler with not connected socket TT#7542)
 25.02.03 - Pasechnik (default codepage setting at ANSI/OEM changing)
 03.03.03 - EVG          (WinCE)
 24.03.03 - Galizin ( Build for qnx 4)
 08.04.03 - RAW (INTLIB_LITE - intlib w/o some rare used functions (for PCL_LITE))
 16.05.03 - Kononyhin (correct thread shutdown to exclude resources leak)
 20.05.03 - Kononyhin (16.05.03 corrected)
 21.05.03 - Kononyhin (synchronization doesn't work in DllMain - check global variable instead)
 21.05.03a- EVG(auto-launching linter for WinCE)
 21.05.03b- EVG(20.05.03 corrected for WinCE build)
 22.05.03 - EVG(def_codepages synchronization: PT#2557)
 23.05.03 - Kononyhin (16.05.03 corrected - step4 && GPF AstMsg=NULL in ReceiveData fixed)
 03.06.03 - EVG (def_codepage synchronization for Win32 corrected)
 04.06.03 - EVG (def_codepage synchronization for Unix corrected)
 06.06.03 - Galizin ( Linux threads correct )
 16.06.03 - Ermakov,Kononyhin (RecvMsgFromServer corrected)
 15.07.03 - Galizin ( Full LnBufRow in CBL always for test end of long message (MGET))
 01.08.03 - Boris Fedorov,Ermakov (Corrected WIN32 thread synchronization. For WIN32
            ACTUAL for synchronous query must be cleared after inter_sync call
            In another case we may use same AstList slot for other thread before
            WaitForSingleObject called and thread will suspend for infinite time )
 04.08.03 - DV (Get rid of differences between "SLCT" and "    " - TT#9063)
 23.08.03 - Ermakov (Remove warnings)
 05.09.03 - Boris Fedorov (Fill SA.bInheritHandle for CreateMailslot)
 11.09.03 -- Kononyhin (TT#9704: Release mutex before return from ReceiveData)
 11.09.03a- Galizin ( Multithread version for Free Bsd )
 12.09.03 - Ermakov (Correct 15.07.03. for User buffer insufficient when GPF)
 12.09.03a- Ermakov (Added required length return...)
 13.10.03 - Kononyhin (Bug fixed: 1) create thread events BEFORE thread creation
                                  2) take the ownership of hMutexExchLock BEFORE creating CheckServer thread)
 21.10.03 - Galizin ( Prevent work with uninitialised condition )
 05.11.03 - Galizin ( UNIX IPC deadlock )
 14.11.03 - Galizin ( MT IPC version did not work througth network )
 28.11.03 - Ermakov ( For WINCE restart linter only for OPEN or DESC command )
 15.01.04 - Ermakov ( Corrected for TerminalServer edition )
 27.02.04 - Galizin ( Corrected errors for OS2000 work )
 10.03.04 - Galizin ( OS2000 multythread work corrected)
 12.03.04 - Galizin ( QNX v6. LINTER_MBX incorrect work TT#10745)
 16.05.04 - Ermakov ( Correct AstMsg syncro. May be needed for UNIX?)
 07.06.04 - Ermakov ( Added wceclib.h - For CE )
 27.07.04 - Ermakov ( Corrected AstMsg usage again... )
 28.07.04 - Ermakov ( Corrected AstMsg usage again... )
 15.09.04 - Galizin ( OS2000 compilation warnings )
 01.12.04 - Galizin ( Prevent catch unused Ast )
 08.12.04 - Galizin ( OS2000 real multythread version)
 27.12.04 - Galizin ( Usage system memory pool for OS2000)
 29.12.04 - Galizin ( Exclude deadlock on kernel dead TT#11304)
 13.02.05 - Ermakov ( Continue ASYNC execution in WIN32. See comments in code.
                      LDBA unloading is hang...)
 08.04.05 - Galizin ( Close socket on every  error )
 18.05.05 - Ermakov ( Corrected WIN32 Sync )
 31.05.05 - Galizin ( Enable signals in CrtQue )
 14.10.05 - Ermakov ( 64Bit arch. and optimization on aligned archs.)
 16.10.05 - Ermakov ( mtyp_t replaced by IPC_TYP )
 16.11.05 - Ermakov,Pasec (Correction for Codepage argiment - if empty, when LINTER_CP use)
 18.11.05 - Galizin ( Change memcpy to two = for copy pid to tial )
 18.11.05a- Galizin ( For OS2000 use local variable for prevent change pid by other therad )
 28.11.05 - Pasechnik (support 6.x codepage definitions for 5.x)
 29.11.05 - Galizin ( New version for OS2000. One host is one task. MTOS2000 is old version - thread is task )
 01.12.05 - Pasechnik (alter 28.11.05)
 11.12.05 - Ermakov ( Corrected CE alignment )
 19.12.05 - Galizin ( Zero for thread id for OS2000)
 01.02.06 - Svetlana ( TT#12761)
 08.02.06 - DV      ( Ignore second buffer in TRAN processing )
 20.02.06 - Galizin ( QNX recreate user queue after fork )
 31.03.06 - Ermakov ( Remove WARNINGS, 64 bit arch adopted )
 20.04.06 - Korotchenko ( support locale charset for Unix & QNX )
 27.04.06 - Ermakov, Sach (Added check for RANGE - FCUR)
 20.04.06 - Korotchenko ( remomed MACX wrong code to support locale charset )
 19.06.06 - FLY     ( Misprint corrected )
 01.07.06 - Ermakov (corrected prev for QNX)
 06.08.06 - Ermakov (corrected prev.)
 26.10.06 - Ermakov (Corrected For CE 5.0. Added OS checking for CE, 
                     corrected CreateFileMapping argument)
 27.10.06 - Ermakov (Corrected UNICODE build)
 14.12.06 - Galizin ( Unlock signal once )
 16.01.07 - Galizin ( Compare commands as integer )
 16.01.07a- Galizin ( Usable command of comtab at begin )
 16.01.07b- Galizin ( Flag in comtab for exclude multiple comparision )
 16.01.07c- Galizin ( Align comtab )
 16.01.07d- Galizin ( Exclude recursive read_from_socket )
 16.01.07e- Galizin ( Ignore sigpipe once at init )
 16.01.07f- Galizin ( Only one read for one message )
 16.01.07h- Galizin ( Use while instead goto. Remove old unused signal work )
 17.01.07 - Galizin ( Excluding memset for message and Ast )
 17.01.07a- Galizin ( Test query ready before locking signal or thread )
 17.01.07b- Galizin ( Faster CalcMsgLen )
 17.01.07c- Galizin ( Excluding sigemptyset sigaddset from inter_sync )
 08.02.07 - Galizin ( Cashed getenv )
 13.02.07 - Ermakov ( Warning )
 22.03.07 - Galizin ( Prevent double using of one Ast )
 08.04.07 - Ermkaov ( Adopt to windows Vista Spec. Check Global, when Local object)
 12.04.07 - Galizin ( Long msg receive mutex initialize. Syncronise AstCall )
 16.04.07 - Galizin ( Exchange whith loltp nedd added flag Q_LOLTP )
 07.06.07 - Galizin ( Prevent segmantation fault on cancel 0 thread )
 25.07.07 - Galizin ( QNX pthread test dead of linter like OS2000)
 16.11.07 - Ermakov ( Init buffer on error)
 01.02.08 - Galizin ( mq_open on creation must have attr argument. OS3000 buld )
 03.04.08 - Galizin ( Correct read socket bug )
 03.04.08a- Galizin ( Fast read socket for lin and net at the same time )
 03.04.08b- Galizin ( Samplier pthread version - init cond once)
 03.04.08c- Galizin ( More undarstadable catch Ast algoritm )
 03.04.08d- Galizin ( Lost answers bug. Save previon WaitFlag )
 03.04.08e- Galizin ( Qnx. Remove queue after freeing all Ast. Prevent core dump on read from queue fail)
 07.04.08 - Ermakov ( Avoid cond incompatible for linux )
 27.05.08 - Galizin ( Qnx. Check network agent dead )
 19.06.08 - Ermakov ( Corrected Timeout 0 handling)
 03.07.08 - Ermakov ( For QNX error reading from input queue equeal to drop all
                      requests - w/o local or network difference)
 16.07.08 - Ermakov ( Correct mistake, remove extra code SVR)
 29.07.08 - Ermakov ( QNX4 build )
 15.09.08 - Ermakov ( Synchronization - inter_sync(...) - may be started if exist flag Q_ASYNCDONE )
 09.12.08 - DV      ( Save and restore errno)
 19.10.09 - Galizin (Large exchange compatible)
 14.12.09 - Galizin ( QNX native exchange)
 12.01.10 - DV,KOV  (Ignore dummy LINTER_CP value for 5.9)
 22.01.10 - Egorov  (Modified for RTOS32)
 12.02.10 - KOV     (Correction for TT#21592)
 17.03.10 - Galizin ( Qnx server dead finding )
 26.04.10 - Galizin ( Increase thread stack to max message size by using macro )
 28.05.10 - Chernyshov ( Canceled user reader thread, closed msg queues in UninitLinterAll for VXWORKS )
 09.07.10 - Galizin (Closing client on server detach. Next time the client will try reconnect)
 15.07.10 - Galizin (Previous for no thread version)
 16.07.10 - Galizin (QNXNATIVE synchronisation)
 19.07.10 - Galizin (Clearing on first call synchronization)
 23.07.10 - FLY     (Suppress warning)
 01.08.10 - Ganin   (Few KOI8-R aliases added)
 27.10.10 - Galizin (QNX. Reset timer on receiving every message)
 11.11.10 - Galizin (QNX Native upgrade)
 07.12.10 - Galizin (Exclude pthread_atfork from shlib if it is not supported)
 10.06.11 - Galizin (Cont prev)
 17.08.11 - DV      (Remove 'l' from formats)
 26.01.12 - Galizin (ru_RU.utf8 predefined charset)
 03.02.12 - KOV,Ermakov (Fix for large PUTM buffer)
 06.02.12 - Galizin (gcc LONG_ALIGN sparc build)
 12.04.12 - Galizin (Macro to on the memory exchange)
 27.04.12 - Galizin (Uninit timer on closing intlib for QNX)
 10.05.12 - Galizin (Setting intlib stack size by variable)
 08.06.12 - Glukhov (datatypes)
 30.07.12 - Galizin(OpenVMS)
 09.06.12 - Plotnikov (Android NDK, lindesk)
 16.11.12 -- Glukhov,Ermakov (Remove extra code, datatypes)
 20.11.12 -- Galizin (Global removing old types)
 29.05.13 - Plotnikov (Android)
 11.04.13 - Plotnikov (iOS. Threads, sockets)
 17.02.14 - Galizin (Mbx unification)
 25.04.14 - KOV (Don't call system functions with NULL-arguments)
 30.05.14 - Galizin (QNX immediately client uninit)
 04.06.14 - Galizin(Reading mesage hanle correction, ast call on unsuccess send)
 18.06.14 - Galizin (Unused variables)
 24.08.14 - Galizin, Pashentsev (Kerberos authentication)
 24.10.14 - Galizin (Prev correction)
 28.10.14 - Galizin (KrbInit should be called in case OPEN CKILL SHUT commands)
 31.10.14 - Pashentsev (Do not reset KrbForce in UninitLinterClient)
*/

/*  Use define for entire platforms
 *
 *  #define INTER_MSDOS     ( BC, MSC, Zortech C 3.0, WATCOM C 286, ... )
 *  #define INTER_DPMI      ( WATCOM C 386 Dos4g, Phar Lap's, ... )
 *  #define INTER_MSWINDOWS ( MS-Windows, WIN32 Win NT/95 ... )
 *  #define UNIX            ( UNIX ... )
 *  #define VMS             ( VMS, OpenVMS ... )
 *  #define OS9000          ( OS9000, OS9 ... )
 *  #define QNX             ( QNX )
 *  #define VXWORKS         ( VxWorks )
 */

#ifndef INTER_MSDOS
#  ifndef INTER_DPMI
#    ifndef INTER_MSWINDOWS
#      ifndef UNIX
#        ifndef OS9000
#          ifndef VMS
#            ifndef QNX
#              ifndef VXWORKS
#                ifndef RTOS32
#                  define INTER_MSDOS
#                endif 
#              endif
#            endif
#          endif
#        endif
#      endif
#    endif
#  endif
#endif

#if defined(VMS)                                            /* 30.07.12 ... */
#define __NEW_STARLET 1
#include <unixlib.h>
#include <iosbdef.h>
#include <descrip.h>
#include <lib$routines.h>
#include <ssdef>
#include <stsdef.h>
#include <starlet.h>
#include <time.h>
#include <stdlib.h>
#include <efndef.h>
#include <gen64def.h>
#include <iodef.h>
#include <cmbdef.h>
#include <agndef.h>
#include <time.h>
#include <gen64def.h>
#endif /* VMS */                                            /* ... 30.07.12 */

#include <string.h>
#include <stdlib.h>
#include <stdio.h>                                         /* 07.02.99 */
#if defined (INTER_MSDOS) || defined (INTER_DPMI)
#  include <dos.h>
#endif
#ifndef WINCE                                              /* 08.04.02 */
#  include <errno.h>
#else
#  include "wceclib.h"                                     /* 07.06.04 */
#endif

#ifdef VXWORKS                                             /* 11.01.01 */
# ifdef OS2000
#include <pthread.h>                                        /* 15.09.04 */
#include <mem.h>                                            /* 27.12.04 */
#define getpid pthread_self                                /* 04.06.01 */
#define MSGID_TYPE mqd_t                                    /* 15.09.04 */
# else /* OS2000 */
#  include <vxworks.h>
#  include <taskLib.h>
#  include <ioLib.h>
#  include <sigLib.h>
#  include <taskHookLib.h>
#define getpid taskIdSelf                                  /* 04.06.01 */
#define MSGID_TYPE int                                      /* 15.09.04 */
# endif /* OS2000 */
#  include <mqueue.h>
#  include <errno.h>
#  include <signal.h>
#  include <fcntl.h>
#  include <semaphore.h>                                   /* 29.06.01 */
#  define MAXMSGSIZE (LENMSG - sizeof(int))
#endif /* WXWORKS */

#ifdef RTOS32
#include <rtk32.h>
#define wsprintf sprintf
#endif

#ifdef QNX                                                 /* 02.04.98 */
#  include <errno.h>
#  include <signal.h>
#if !defined(QNXNATIVE)                                     /* 14.12.09 */
#  include <mqueue.h>
#else  /* QNXNATIVE */                                      /* 14.12.09 ... */
#include <sys/neutrino.h>
#include <sys/netmgr.h>
#include <sched.h>
#include <sys/sysctl.h>
#endif /* QNXNATIVE */                                      /* ... 14.12.09 */
#  include <sys/mman.h>                                    /* 27.05.99 */
#  include <unix.h>
#  if _VER_MAX >= 600                                       /* 20.04.06 ... */
#    include <locale.h>
#  endif                                                    /* ... 20.04.06 */
#  define MAXMSGSIZE (LENMSG - sizeof(int))
#define MSGID_TYPE mqd_t                                    /* 15.09.04 */
#endif

#ifdef UNIX
#  include <sys/types.h>
#  include <signal.h>
#  if _VER_MAX >= 600                                       /* 20.04.06 ... */
#    include <locale.h>
#    ifdef  SUNOS
#      include <langinfo.h>
#    endif
#  endif                                                    /* ... 20.04.06 */
#  ifdef SOCKETS                                       /* 12.11.98 ... */
#    include <fcntl.h>
#    include <sys/socket.h>
#    if !defined(BSD) && !defined(LINUX) && !defined(CYGWIN)  /* 28.02.99 */
#      include <sys/stropts.h>
#    endif
#if !defined(IOS)                                      /* 11.04.13 */
#    include <sys/un.h>
#else                                                  /* 11.04.13 ... */
#    include <netinet/in.h>
#    include <netdb.h>
#endif                                                 /* ... 11.04.13 */
#    include <unistd.h>
#    define MAXMSGSIZE LENMSG
#  else
#    include <sys/ipc.h>
#    include <sys/msg.h>
#   ifndef NO_SEMAPHORES                                      /* 7.04.99 */
#    include <sys/sem.h>
#   endif
#    include <sys/shm.h>
#    include <unistd.h>
#    define MAXMSGSIZE (LENMSG - sizeof(int))
#  endif                                               /* ... 12.11.98 */
#define MSGID_TYPE int
#endif

#ifdef OS9000
#  include <modes.h>
#  include <module.h>
#  include <process.h>
#endif

#ifdef INTER_MSWINDOWS
#  include <windows.h>
#  ifdef WIN16
#    undef WIN32
#  endif
#endif

#include "intlib.h"
#ifdef PREPARE
#  include "cquery.h"
#endif

#if defined(KRB_AUTH)                                     /* 24.08.14 ... */
#include "krbauth.c"
signed char KrbForce = -1;
#endif                                                    /* ... 24.08.14 */

#if defined(PTHREAD_INTLIB) && defined(OS2000)              /* 08.12.04 .. */
/* Structure for unitialize the data after dead the thread */
struct PidMsg
{
  mqd_t MsgId;
  pthread_t Pid;
};

/* UninitLinterClient call after end of every thread.
But library has to be initialised at any time. */
void UninitLinterClient()
{
};
#define UninitLinterClient IntlibUninitLinterClient
void UninitLinterClient();

#define calloc(a,b) IntlibAlloc(a, b)                       /* 27.12.04 .. */
#define free(a)  IntlibFree(a)
static void* IntlibAlloc(int count, int size)
{
  void* RetVal = memAlloc(NULL, count*size, 8);             /* 01.02.08 */
  if ( RetVal )
    memset(RetVal, 0, count*size);
  return RetVal;
}

static void IntlibFree(void* Addr)
{
  memFree(NULL, Addr);                                      /* 01.02.08 */
}                                                           /* .. 27.12.04 */
#endif /* PTHREAD_INTLIB OS2000 */                          /* .. 08.12.04 */

#ifdef WINCE                                              /* 21.05.03a ...*/
# define LIN_FVER    TEXT("\\5.1")
# define REG_KEY     TEXT("Software\\Relex, Ltd.\\Linter SQL Server")LIN_FVER
# define LINTER_HOME TEXT("Linter Home")
# define LINTER_BIN TEXT("linterce")
#endif /* WINCE */                                        /*... 21.05.03a */


/* ----------------------------------- Define ------------------------------ */
#define siz (sizeof(comtab)/sizeof(typtab))
#ifndef min
#  define min(a, b) (((a) < (b)) ? (a) : (b))              /* 28.08.98 ... */
#endif
#ifndef max
#  define max(a, b) (((a) > (b)) ? (a) : (b))
#endif

/* ----------------------------------- TypeDef ----------------------------- */
typedef struct {
  L_LONG   com;                                             /* 16.01.07 */
  unsigned char p1;
  unsigned char p2;
  unsigned CanLongMsg  : 1;                                  /* 16.01.07b */
  unsigned UseCodepage : 1;                                  /* 16.01.07b */
  unsigned BitRes      : 6;                                  /* 16.01.07b */
  unsigned char Res;                                        /* 16.01.07c */
} typtab;

#if !defined(DATA_MSBF)                                     /* 16.01.07 .. */
# define GETI 'G'+'E'*256+'T'*256*256+'I'*256*256*256
# define GETN 'G'+'E'*256+'T'*256*256+'N'*256*256*256
# define GETP 'G'+'E'*256+'T'*256*256+'P'*256*256*256
# define GETS 'G'+'E'*256+'T'*256*256+'S'*256*256*256
# define GETF 'G'+'E'*256+'T'*256*256+'F'*256*256*256
# define GETL 'G'+'E'*256+'T'*256*256+'L'*256*256*256
# define GETM 'G'+'E'*256+'T'*256*256+'M'*256*256*256
# define GETE 'G'+'E'*256+'T'*256*256+'E'*256*256*256
# define GOBJ 'G'+'O'*256+'B'*256*256+'J'*256*256*256
# define GPRC 'G'+'P'*256+'R'*256*256+'C'*256*256*256
# define AOBJ 'A'+'O'*256+'B'*256*256+'J'*256*256*256
# define COBJ 'C'+'O'*256+'B'*256*256+'J'*256*256*256
# define GBLB 'G'+'B'*256+'L'*256*256+'B'*256*256*256
# define ABLB 'A'+'B'*256+'L'*256*256+'B'*256*256*256
# define CBLB 'C'+'B'*256+'L'*256*256+'B'*256*256*256
# define SLCT 'S'+'L'*256+'C'*256*256+'T'*256*256*256
# define TRAN 'T'+'R'*256+'A'*256*256+'N'*256*256*256
# define EXEC 'E'+'X'*256+'E'*256*256+'C'*256*256*256
# define OPEN 'O'+'P'*256+'E'*256*256+'N'*256*256*256
# define OCUR 'O'+'C'*256+'U'*256*256+'R'*256*256*256
# define SETO 'S'+'E'*256+'T'*256*256+'O'*256*256*256
# define REPL 'R'+'E'*256+'P'*256*256+'L'*256*256*256
# define FATR 'F'+'A'*256+'T'*256*256+'R'*256*256*256
# define FREL 'F'+'R'*256+'E'*256*256+'L'*256*256*256
# define FCUR 'F'+'C'*256+'U'*256*256+'R'*256*256*256
# define FNOD 'F'+'N'*256+'O'*256*256+'D'*256*256*256
# define SPAC ' '+' '*256+' '*256*256+' '*256*256*256
# define SRVR 'S'+'R'*256+'V'*256*256+'R'*256*256*256
# define CLOS 'C'+'L'*256+'O'*256*256+'S'*256*256*256
# define LREL 'L'+'R'*256+'E'*256*256+'L'*256*256*256
# define LROW 'L'+'R'*256+'O'*256*256+'W'*256*256*256
# define UROW 'U'+'R'*256+'O'*256*256+'W'*256*256*256
# define UREL 'U'+'R'*256+'E'*256*256+'L'*256*256*256
# define SHUT 'S'+'H'*256+'U'*256*256+'T'*256*256*256
# define DIRR 'D'+'I'*256+'R'*256*256+'R'*256*256*256
# define DIRA 'D'+'I'*256+'R'*256*256+'A'*256*256*256
# define DIRF 'D'+'I'*256+'R'*256*256+'F'*256*256*256
# define DIRK 'D'+'I'*256+'R'*256*256+'K'*256*256*256
# define SNAP 'S'+'N'*256+'A'*256*256+'P'*256*256*256
# define FUSR 'F'+'U'*256+'S'*256*256+'R'*256*256*256
# define RBAC 'R'+'B'*256+'A'*256*256+'C'*256*256*256
# define COMT 'C'+'O'*256+'M'*256*256+'T'*256*256*256
# define GETA 'G'+'E'*256+'T'*256*256+'A'*256*256*256
# define SSRV 'S'+'S'*256+'R'*256*256+'V'*256*256*256
# define FSPC 'F'+'S'*256+'P'*256*256+'C'*256*256*256
# define DESC 'D'+'E'*256+'S'*256*256+'C'*256*256*256
# define KILL 'K'+'I'*256+'L'*256*256+'L'*256*256*256
# define MPUT 'M'+'P'*256+'U'*256*256+'T'*256*256*256
# define MGET 'M'+'G'*256+'E'*256*256+'T'*256*256*256
# define MCLR 'M'+'C'*256+'L'*256*256+'R'*256*256*256
# define MALL 'M'+'A'*256+'L'*256*256+'L'*256*256*256
# define PUTM 'P'+'U'*256+'T'*256*256+'M'*256*256*256
# define PROC 'P'+'R'*256+'O'*256*256+'C'*256*256*256
# define RPAG 'R'+'P'*256+'A'*256*256+'G'*256*256*256
# define WPAG 'W'+'P'*256+'A'*256*256+'G'*256*256*256
# define BKUP 'B'+'K'*256+'U'*256*256+'P'*256*256*256
# define DBCA 'D'+'B'*256+'C'*256*256+'A'*256*256*256
# define OKRB 'O'+'K'*256+'R'*256*256+'B'*256*256*256     /* 24.08.14 */
#else /* DATA_MSBF */
# define GETI 'I'+'T'*256+'E'*256*256+'G'*256*256*256
# define GETN 'N'+'T'*256+'E'*256*256+'G'*256*256*256
# define GETP 'P'+'T'*256+'E'*256*256+'G'*256*256*256
# define GETS 'S'+'T'*256+'E'*256*256+'G'*256*256*256
# define GETF 'F'+'T'*256+'E'*256*256+'G'*256*256*256
# define GETL 'L'+'T'*256+'E'*256*256+'G'*256*256*256
# define GETM 'M'+'T'*256+'E'*256*256+'G'*256*256*256
# define GETE 'E'+'T'*256+'E'*256*256+'G'*256*256*256
# define GOBJ 'J'+'B'*256+'O'*256*256+'G'*256*256*256
# define GPRC 'C'+'R'*256+'P'*256*256+'G'*256*256*256
# define AOBJ 'J'+'B'*256+'O'*256*256+'A'*256*256*256
# define COBJ 'J'+'B'*256+'O'*256*256+'C'*256*256*256
# define GBLB 'B'+'L'*256+'B'*256*256+'G'*256*256*256
# define ABLB 'B'+'L'*256+'B'*256*256+'A'*256*256*256
# define CBLB 'B'+'L'*256+'B'*256*256+'C'*256*256*256
# define SLCT 'T'+'C'*256+'L'*256*256+'S'*256*256*256
# define TRAN 'N'+'A'*256+'R'*256*256+'T'*256*256*256
# define EXEC 'C'+'E'*256+'X'*256*256+'E'*256*256*256
# define OPEN 'N'+'E'*256+'P'*256*256+'O'*256*256*256
# define OCUR 'R'+'U'*256+'C'*256*256+'O'*256*256*256
# define SETO 'O'+'T'*256+'E'*256*256+'S'*256*256*256
# define REPL 'L'+'P'*256+'E'*256*256+'R'*256*256*256
# define FATR 'R'+'T'*256+'A'*256*256+'F'*256*256*256
# define FREL 'L'+'E'*256+'R'*256*256+'F'*256*256*256
# define FCUR 'R'+'U'*256+'C'*256*256+'F'*256*256*256
# define FNOD 'D'+'O'*256+'N'*256*256+'F'*256*256*256
# define SPAC ' '+' '*256+' '*256*256+' '*256*256*256
# define SRVR 'R'+'V'*256+'R'*256*256+'S'*256*256*256
# define CLOS 'S'+'O'*256+'L'*256*256+'C'*256*256*256
# define LREL 'L'+'E'*256+'R'*256*256+'L'*256*256*256
# define LROW 'W'+'O'*256+'R'*256*256+'L'*256*256*256
# define UROW 'W'+'O'*256+'R'*256*256+'U'*256*256*256
# define UREL 'L'+'E'*256+'R'*256*256+'U'*256*256*256
# define SHUT 'T'+'U'*256+'H'*256*256+'S'*256*256*256
# define DIRR 'R'+'R'*256+'I'*256*256+'D'*256*256*256
# define DIRA 'A'+'R'*256+'I'*256*256+'D'*256*256*256
# define DIRF 'F'+'R'*256+'I'*256*256+'D'*256*256*256
# define DIRK 'K'+'R'*256+'I'*256*256+'D'*256*256*256
# define SNAP 'P'+'A'*256+'N'*256*256+'S'*256*256*256
# define FUSR 'R'+'S'*256+'U'*256*256+'F'*256*256*256
# define RBAC 'C'+'A'*256+'B'*256*256+'R'*256*256*256
# define COMT 'T'+'M'*256+'O'*256*256+'C'*256*256*256
# define GETA 'A'+'T'*256+'E'*256*256+'G'*256*256*256
# define SSRV 'V'+'R'*256+'S'*256*256+'S'*256*256*256
# define FSPC 'C'+'P'*256+'S'*256*256+'F'*256*256*256
# define DESC 'C'+'S'*256+'E'*256*256+'D'*256*256*256
# define KILL 'L'+'L'*256+'I'*256*256+'K'*256*256*256
# define MPUT 'T'+'U'*256+'P'*256*256+'M'*256*256*256
# define MGET 'T'+'E'*256+'G'*256*256+'M'*256*256*256
# define MCLR 'R'+'L'*256+'C'*256*256+'M'*256*256*256
# define MALL 'L'+'L'*256+'A'*256*256+'M'*256*256*256
# define PUTM 'M'+'T'*256+'U'*256*256+'P'*256*256*256
# define PROC 'C'+'O'*256+'R'*256*256+'P'*256*256*256
# define RPAG 'G'+'A'*256+'P'*256*256+'R'*256*256*256
# define WPAG 'G'+'A'*256+'P'*256*256+'W'*256*256*256
# define BKUP 'P'+'U'*256+'K'*256*256+'B'*256*256*256
# define DBCA 'A'+'C'*256+'B'*256*256+'D'*256*256*256
# define OKRB 'B'+'R'*256+'K'*256*256+'O'*256*256*256     /* 24.08.14 */
#endif /* DATA_MSBF */
#define COM2INT(COM) (*(L_LONG*)(&(COM)[0]))

static typtab comtab[] = {
  {GETN,1,4,1,0},                                               /* 16.01.07a .. */
  {GETS,1,4,1,0},
  {GETM,1,4,1,0},
  {SPAC,1,3,1,0},
  {EXEC,4,7,1,0},
  {TRAN,1,3,1,0},
  {SLCT,1,3,1,0},                                               /* .. 16.01.07a */
  {GETI,1,4,0,0},                                        /* 26.07.99 */
  {GETP,1,4,1,0},
  {GETF,1,4,1,0},
  {GETL,1,4,1,0},
  {GETE,1,4,0,0},
  {GOBJ,1,4,0,0},
#if _VER_MAX >= 600                                     /* 30.10.01 ... */
  {GPRC,1,4,0,0},
#endif                                                  /* ... 30.10.01 */
  {AOBJ,0,6,0,0},
  {COBJ,0,4,0,0},
  {GBLB,1,4,0,0},
  {ABLB,0,6,0,0},
  {CBLB,0,4,0,0},
  {OPEN,5,5,0,1},
#if defined(KRB_AUTH)                                     /* 24.08.14 ... */
  {OKRB,5,5,0,1},
#endif                                                    /* ... 24.08.14 */
  {OCUR,5,4,0,0},
  {SETO,0,6,0,0},
#ifdef AREPL_SYSLOG /* 07.06.00 */
  {REPL,1,4,0,0},
#else
  {REPL,0,0,0,0},
#endif
  {FATR,2,2,0,0},
  {FREL,1,2,0,0},
  {FCUR,3,2,0,0},         /* 28.05.97 */
  {FNOD,3,2,0,0},         /* 03.07.98 */
#ifdef OS9000                                           /* 10.02.00 */
  {SRVR,6,4,0,0},
#endif
  {CLOS,6,4,0,0},
  {LREL,0,2,0,0},
  {LROW,0,4,0,0},
  {UROW,0,4,0,0},
  {UREL,0,4,0,0},
  {SHUT,0,5,0,1},                                          /* 21.07.98 */
  {DIRR,1,4,0,1},
  {DIRA,1,4,0,1},
  {DIRF,1,4,0,1},
  {DIRK,1,4,0,1},
  {SNAP,0,4,0,0},
  {FUSR,3,2,0,0},
  {RBAC,5,2,0,0},                                          /* 29.06.98 */
  {COMT,5,2,0,0},                                          /* 29.06.98 */
  {GETA,1,4,0,0},
  {SSRV,1,4,0,0},
  {FSPC,6,2,0,0},                                          /* 12.09.00 */
  {DESC,1,4,0,1},
  {KILL,0,5,0,1},
  {MPUT,0,6,0,0},
  {MGET,1,4,0,0},
  {MCLR,0,4,0,0},
  {MALL,0,4,0,0},
  {PUTM,0,6,0,0},
  {PROC,1,8,0,0},                                          /* 01.02.06 */
  {RPAG,1,4,0,0},
  {WPAG,0,6,0,0},
  {BKUP,1,8,0,0},                                          /* 20.11.97 */
  {DBCA,4,2,0,0}                                           /* 10.09.98 */
};                                                          /* .. 16.01.07 */

#ifdef INTER_DPMI
#  define EMODE    1210
#  define EALLSEG  1211
#  define ESETPNTR 1212
#  define EGETPNTR 1213
#  define EINT     1214
#  define EFRSEG   1215
#endif

#ifdef VMS
static TMSG GlRcvMsg;                                       /* 30.07.12 ... */
static IOSB GlIOSB;
L_WORD InputMailbox = 0;                                    /* 20.11.12 ... */
L_WORD NetMailbox = 0;
L_WORD LinMailbox = 0;                                      /* ... 20.11.12 */
pid_t MyPid = 0;
GENERIC_64 RelativeTime;

#define LNMSG sizeof(TMSG)                                  /* ... 30.07.12 */
#endif

#ifdef INTER_MSWINDOWS
# ifdef WIN32
#   define LVER TEXT("5")
#  define LMS TEXT("\\\\.\\mailslot\\LinterMailslot"LVER)
#  define RMS TEXT("\\\\.\\mailslot\\LinNetMailslot"LVER)
static BYTE   nOS;
static DWORD  dwAstTid;                                    /* 24.07.98 */
static LONG   lInit = -1L;                                 /* 03.12.98 */
static HANDLE hAstThrd, hLock;                             /* 23.12.98 */
static HANDLE hMutexExchLock = NULL;                       /* 22.01.03 25.04.14 */
static TCHAR  szLinMbxEnv[L_MAX_MBX+32];                   /* 17.02.14 */
static TCHAR  szNetMbxEnv[L_MAX_MBX+32];                   /* 17.02.14 */
static ExchangeType gExchType = undefined_exch;
static PExchangeIn  gpInput = NULL;
static PExchangeOut gpOutput[2] = {NULL, NULL};
static HANDLE   hEventStopChkThrd;
static HANDLE   hEventStopAstThrd;
static L_BOOL   bThrdStopped;                              /* 21.05.03 */
#if _VER_MAX >= 600
static L_BOOL bUse_OEM_CP = L_TFALSE;                      /* 26.12.01 */
#endif
# else
static HANDLE  hLclQue  = NULL;
static HANDLE  hDbcQue  = NULL;
# endif
#endif

#ifdef DEBUG_EVENT_TRACE                                   /* 01.08.03 ... */
static CRITICAL_SECTION  sg_critSect = {0};                /* ... 13.02.05 */
static FILE *event_log = NULL;
static trace_event(char *file, int line, int pid, char *msg, int event)
{
  if (event_log == NULL)
  {
    event_log = fopen("event.log", "w");
  }
  if (event_log != NULL)
  {
    if (lInit != -1)                                       /* 13.02.05 */
      EnterCriticalSection(&sg_critSect);
    fprintf(event_log, "[%i] %s (%i): %s %i\n", pid, file, line, msg, event);
    fflush(event_log);
    if (lInit != -1)                                       /* 13.02.05 */
      LeaveCriticalSection(&sg_critSect);
  }
}
#define TRACE_EVENT(a,b) trace_event(__FILE__,__LINE__,GetCurrentThreadId(),a,((int)b))
#else
#define TRACE_EVENT(a,b)
#endif /* DEBUG_EVENT_TRACE */                            /* ... 01.08.03 */

static char *LINTER_CP = 0;                                     /* 22.11.02 */
static int LINTER_CP_R = L_TFALSE;
static L_LONG LINTER_CP_ENCODE = 0;                       /* 01.12.05 */

#if defined(QNXNATIVE)                                      /* 14.12.09 ... */
#define MAXSHMMSG MAXMSG
#define SHMSIZE ((MAXSHMMSG*2 + 2*PAGESIZE - 1)/PAGESIZE*PAGESIZE) /* 2 message + 1 page */
/* SHM: Client header|server header|client write data| server write data */
#define CLNT_HDR_OFFSET 0
#define CLNT_DATA_OFFSET PAGESIZE
#define SRV_HDR_OFFSET  (PAGESIZE/2)
#define SRV_DATA_OFFSET ((2*PAGESIZE + MAXSHMMSG - 1)/PAGESIZE*PAGESIZE)

#define SPACE_SIZE ((MAXSHMMSG + PAGESIZE -1)/PAGESIZE*PAGESIZE)
#define SPACE_MASK (SPACE_SIZE-1)

typedef struct
{
   L_LONG Pid;    /* Pid of linter or dbc */
   int    ConnId; /* Conn to server id */
   int    RcvId;  /* Server receive id */
   L_LONG ExchId; /* Server exchange id */
   int    ShmId;  /* Shared memory ID */
   char*  Shm;    /* Shared memory pointer */
   int    RemoteChId;   /* Server channel id */
   int    RemoteConnId; /* Server himself connect id */
   int    ClntChId; /* Connect to himself */
   int    ClntConnId; /* chanel of the client */
   int    Prior;  /* Default priority */                    /* 15.07.10 */
}
IntlibServer_t;

IntlibServer_t Linter = {0, -1, -1, -1, -1, NULL, -1, -1, -1, -1};
IntlibServer_t Dbc    = {0, -1, -1, -1, -1, NULL, -1, -1, -1, -1};

typedef struct
{
  L_LONG Pid;
#if !defined(PTHREAD_INTLIB)
  int Timer;
#endif
} IntlibClient_t;

IntlibClient_t Client = { 0 };
#endif /* QNXNATIVE */                                      /* ... 14.12.09 */

#if defined(QNX) || defined(VXWORKS)                            /* 02.04.98 *//* 11.01.01 */
#if !defined(PAGESIZE)
#define PAGESIZE 4096                                       /* 14.12.09 */
#endif
static sigset_t newmask;                                    /* 17.01.07c */
#if !defined(QNXNATIVE)                                     /* 14.12.09 */
static mqd_t mq_linter;
static int    sh_linter = -1,sh_net = -1;                       /* 27.10.99 */
static void  *shm_linter = (void*)-1,                             /* 22.08.02 */
             *shm_net = (void*)-1;                                /* 27.10.99 */
#if defined(OS2000) && defined(PTHREAD_INTLIB)              /* 08.12.04 */
static pthread_key_t UserKey;
#endif                             
static mqd_t  msg_user = (mqd_t)-1;
static mqd_t  msg_lin  = (mqd_t)-1;
static mqd_t  msg_net  = (mqd_t)-1;
static int    pid_net  = -1;
static int    pid_lin  = -1;
#if defined(QNX)
static int    my_pid = 0;                                   /* 20.02.06 */
#endif
#ifdef VXWORKS                                                  /* 11.01.01 */
static int    pid_user  = -1;
#else
static void (*alrm_handler)(int);                               /* 27.10.99 */
#endif
#endif /* QNXNATIVE */
void UninitLinter(int pid);
void UninitLinterAll(void);                                     /* 27.10.99 */
char ThisMbxKey[L_MAX_MBX+32];                                     /* 17.02.14... 08.02.07 .. */
char LinterMbxKey[L_MAX_MBX+32] = "linter";
char NetMbxKey[L_MAX_MBX+32] = "linter_net";                          /* ...17.02.14 ..08.02.07 */
#endif

#ifdef UNIX
#if !defined(SOCKETS) && !defined(NO_SEMAPHORES) /* 07.04.99 ... */
static struct sembuf sops;
static int sem_lin, sem_net;                            /* 08.12.98 ... */
#endif                                           /* ... 07.04.99 */
static int pid_lin, pid_net, my_pid = 0,                    /* 08.02.07 29.04.99 */
       msg_lin = -1, msg_net = -1;                  /* ... 08.12.98 */
static void (*alrm_handler)(int);                /* 11.01.99 */
static sigset_t  newmask;                                   /* 17.01.07c */
#if defined(BSD) || defined(LINUX)               /* 28.08.97 */
static struct sigaction sa;
#endif
static int LinterMbxKey = -1;                               /* 08.02.07 */
static int NetMbxKey    = -1;                               /* 08.02.07 */
#endif

#ifdef WIN32                                              /* 22.05.03 ...*/
DWORD  IntlibStackSize = ReadMsgFromServer_THREAD_STACK_SIZE;  /* 10.05.12 */
static CRITICAL_SECTION g_DefCpSync;                      /* 03.06.03 ...*/
static long g_iDefCp = -1L;                               /* 23.08.03 */

#ifndef NO_EXCEPTION
# define __TRY     __try
# define __FINALLY __finally
#else
# define __TRY
# define __FINALLY
#endif  /* NO_EXCEPTION */

# define THREAD_LOCK(Sync)   __TRY                               \
                             {                                   \
                                EnterCriticalSection(&Sync);

# define THREAD_UNLOCK(Sync) }                                   \
                             __FINALLY                           \
                             {                                   \
                                LeaveCriticalSection(&Sync);     \
                             }

# define THREAD_INIT(Sync)                                       \
  if(g_iDefCp < 0 && (InterlockedIncrement(&g_iDefCp) == 0))     \
    InitializeCriticalSection(&Sync);

# define THREAD_RELEASE(Sync)                                    \
  if(g_iDefCp == 0 && (InterlockedDecrement(&g_iDefCp) == -1L))  \
    DeleteCriticalSection(&Sync);

#elif defined(PTHREAD_INTLIB)
size_t IntlibStackSize = ReadMsgFromServer_THREAD_STACK_SIZE; /* 10.05.12 */
static pthread_mutex_t g_DefCpSync = PTHREAD_MUTEX_INITIALIZER;
                                                          /* 04.06.03 ...*/
# define THREAD_LOCK(Sync)     {pthread_mutex_lock(&(Sync));
# define THREAD_UNLOCK(Sync)   }{pthread_mutex_unlock(&(Sync));}
# define THREAD_INIT(Sync)
# define THREAD_RELEASE(Sync)

#else /* WIN32 */

#define  g_DefCpSync 1

# define THREAD_LOCK(Sync)
# define THREAD_UNLOCK(Sync)
# define THREAD_INIT(Sync)
# define THREAD_RELEASE(Sync)
                                                          /*... 04.06.03 */
#endif /* WIN32 */                        /*... 22.05.03 *//*... 03.06.03 */



#if defined(PTHREAD_INTLIB)                                 /* 22.08.02 ... */

#define WAIT_COND(PStruct, COND) \
  { \
    pthread_mutex_lock(&(PStruct)->Mutex); \
    if ( COND ) \
      pthread_cond_wait(&(PStruct)->LCond.Cond, &(PStruct)->Mutex); \
    pthread_mutex_unlock(&(PStruct)->Mutex); \
  }

#define RELEASE_INTER_THREAD(PAst) \
  { \
    pthread_mutex_lock(&(PAst)->Mutex); \
    (PAst)->CBL->PrzExe |= Q_ASYNCDONE; \
    if ((PAst)->CBL->PrzExe & Q_EV_CATCH) \
      (PAst)->Actual = 2; \
    else \
      (PAst)->Actual = 0; \
    if ( (PAst)->WaitFlag != 0 )                             /* 03.04.08b 21.10.03 */ \
      pthread_cond_signal(&(PAst)->LCond.Cond);\
    pthread_mutex_unlock(&(PAst)->Mutex); \
  }

# if defined(LINUX)
static pid_t LocalThPid = 0;                                /* 06.06.03 */
static pid_t NetThPid   = 0;
static L_pthread_cond_t RunReaderCond = {PTHREAD_COND_INITIALIZER}; /* 07.04.08 */
# endif
#if !defined(SOCKETS)
static pthread_t LocalThreadID = 0;
static pthread_t NetThreadID   = 0;
#endif
struct ASTCALLDESC;
typedef struct ASTCALLDESC
  {
  struct ASTCALLDESC* Next;
  pthread_t         ThreadID;
  pthread_mutex_t   Mutex;
  L_pthread_cond_t  LCond;                                      /* 07.04.08 */
  TASTMSG*          Ast;
  L_BYTE              IsBusy;
  } ASTCALLDESC;

static ASTCALLDESC AstCallDescFirst =
                 {
                   &AstCallDescFirst,
                   (pthread_t)0,
                   PTHREAD_MUTEX_INITIALIZER,
                   {PTHREAD_COND_INITIALIZER},
                   NULL,
                   0
                 };

static ASTCALLDESC* AstCallDescCurrent = &AstCallDescFirst;
static pthread_mutex_t IntlibMutex = PTHREAD_MUTEX_INITIALIZER;
static pthread_mutex_t InternalMutex = PTHREAD_MUTEX_INITIALIZER;
static L_BYTE IntlibIsInit = 0;
#if defined(OS2000)                                         /* 08.12.04 .. */
#if defined(PTHREAD_INTLIB) && !defined(MTOS2000)           /* 29.11.05 */
static L_pthread_cond_t RunReaderCond = {PTHREAD_COND_INITIALIZER}; /* 07.04.08 */
#endif

#if defined(MTOS2000)
/* Uninitialize data after dead the thread */
void UninitSpecific(void* Arg)
{
  if (Arg)
  {
    char str[32];
    char* ss;

    pthread_cancel(((struct PidMsg*)Arg)->Pid);
    mq_close(((struct PidMsg*)Arg)->MsgId);

    sprintf(str,"%s_%x", LinterMbxKey, pthread_self());     /* 08.02.07 */

    mq_unlink(str);
    free(Arg);
    pthread_setspecific(UserKey, NULL);
  }
}
#endif
#endif /* OS2000 */                                         /* .. 08.12.04 */

#endif  /* PTHREAD_INTLIB */                                /* ... 22.08.02 */


#ifdef OS9000
static PIPEMSG PipeMsg;
static int pid_lin = 0, pid_net = 0;
static int LinOutChan, NetOutChan, *LinChan;
static int TimeForkLin, TimeForkNet, *TimeFork;
static int AlmId = 0;
#ifdef OS9
#define CNT_FOR_INSPEC  750
#define MP_WORLD_ACCESS (MP_WORLD_READ | MP_WORLD_WRITE | MP_WORLD_EXEC)
#else
#define CNT_FOR_INSPEC  600
#endif
#endif

#if defined(INTER_MSDOS) || defined(INTER_DPMI)
static union  REGS  regs;
static struct SREGS sregs;
static TMSG  _far InterMsg;
#endif

                                                            /* 22.08.02 ... */
/* ASTBLOCKSIZE*ASTSBLOCKCOUNT = max count asyncronus calls of inter(...) */
/* ASTBLOCKSIZE and ASTSBLOCKCOUNT must be <= 255 */
#if defined (LITE) || defined (INTLIB_LITE)                                /* 08.04.03 */
#define ASTBLOCKSIZE   16
#define ASTBLOCKCOUNT  16
#else
#define ASTBLOCKSIZE   32
#define ASTBLOCKCOUNT 255
#endif                                                      /* ... 22.08.02 */

/* warning: AstListSize is L_WORD, but use as L_BYTE AstIndex... */
static L_WORD     AstListSize = 0;
static TASTMSG *AstList[ASTBLOCKCOUNT]     = {NULL};       /* 22.08.02 */
static L_BOOL  AstCall     = L_TFALSE;
#if _VER_MAX >= 600                                        /* 13.02.07 */
static LCODEPAGE def_codepage;                             /* 22.10.01 */
#endif
static L_BOOL msg_enabled = L_TFALSE;                      /* 22.10.01 */
static L_BOOL use_UTF8    = L_TFALSE;                      /* 26.12.01 */

/* -- Common functions ----------------------------------------------------- */

#ifdef WINCE                                              /* 21.05.03a ...*/
static L_BOOL LaunchDefServer();
#endif                                                     /*... 21.05.03a */

#ifdef TRACED_LINTER
static void inter_trace(char *Msg)
{
  if (msg_enabled)
  {
    fprintf(stderr, Msg);
  }
}
#endif

static L_LONG CalcMsgLen(TMSG *InterMsg, TAILMSG **TailMsg, TASTMSG *AstMsg)
{
 register L_LONG Len;
 L_WORD *WordPtr = InterMsg->Buf.Word;                      /* 17.01.07b .. */
 
 WordPtr += *WordPtr/2 + 1;
 WordPtr += *WordPtr/2 + 1;
 WordPtr += *WordPtr/2 + 1;
 WordPtr += *WordPtr/2 + 1;

 *TailMsg = (TAILMSG *)WordPtr;
 if (AstMsg)
 {
   register L_BYTE* TailPtr = (L_BYTE *)WordPtr;
   L_BYTE *From = (L_BYTE *)&(AstMsg->AstIndex);             /* 14.10.05 ... */
   L_BYTE *To = (L_BYTE *)&((*TailMsg)->AstInd);

   memset(TailPtr, 0, LENTAIL);
   To[0] = From[0];
   To[1] = From[1];
 }
 
 Len = (L_BYTE*)WordPtr - InterMsg->Buf.Byte + LENTAIL + LENCBL +
#if defined(UNIX) || defined(VMS) || defined(OS9000) || defined(QNXNATIVE) /* 14.12.09 */
          sizeof(InterMsg->Msg_Type) + sizeof(InterMsg->TaskName);
#else
          sizeof(L_LONG);
#endif

 return Len;                                                /* ... 17.07.01b 14.10.05 */
}

#if defined(PTHREAD_INTLIB)                                /* 22.08.02 ... */
static void* AsyncCallThread(void* Arg)
{
  ASTCALLDESC* MyAsyncCallDesc = (ASTCALLDESC*)Arg;
  TASTMSG* Ast;

#if !defined(OS2000)                                        /* 29.11.05 */
  for(;;)
#endif
  {
    WAIT_COND(MyAsyncCallDesc, MyAsyncCallDesc->IsBusy == 0);

    Ast = MyAsyncCallDesc->Ast;
    (*Ast->UsrProc)(Ast->CBL, Ast->Korteg, Ast->Range);
    RELEASE_INTER_THREAD(Ast);
    MyAsyncCallDesc->IsBusy = 0;
  }
#if defined(OS2000)                                         /* 19.12.05 29.11.05 */
  MyAsyncCallDesc->ThreadID = (pthread_t)0;                 /* 29.11.05 */
#endif
  return NULL;
}
#endif                                                      /* ... 22.08.02 */

static void CallUserProc(TASTMSG *AstMsg)
{
#if !defined(PTHREAD_INTLIB)                                /* 22.08.02 */
 if (!(AstMsg->CBL->PrzExe & Q_ASYNC) || (AstMsg->CBL->PrzExe & Q_RECEIVE) ) /* 04.11.02 */
   return;
 AstMsg->CBL->PrzExe |= Q_ASYNCDONE;
 AstCall = L_TTRUE;                              /* 22.07.98 */
 if (AstMsg->UsrProc)
   (*AstMsg->UsrProc)(AstMsg->CBL, AstMsg->Korteg, AstMsg->Range);
 AstCall = L_TFALSE;                             /* 22.07.98 */
#else    /* PTHREAD_INTLIB */                               /* 22.08.02 ... */
 if (!(AstMsg->CBL->PrzExe & Q_ASYNC) || ! AstMsg->UsrProc
                                || (AstMsg->CBL->PrzExe & Q_RECEIVE) ) /* 04.11.02 */
 {
   RELEASE_INTER_THREAD(AstMsg);
   return;
 }
 {
   ASTCALLDESC* AstCallDescLocal = AstCallDescCurrent->Next;

   while ( pthread_mutex_trylock( &AstCallDescLocal->Mutex ) ||
          ( AstCallDescLocal->IsBusy &&
            !pthread_mutex_unlock(&AstCallDescLocal->Mutex) ) )
   {
     if (AstCallDescLocal == AstCallDescCurrent)
     {
       ASTCALLDESC* NewCallDesc;
       static pthread_mutex_t AddCallDescMutex = PTHREAD_MUTEX_INITIALIZER;

       pthread_mutex_lock(&AddCallDescMutex);
       NewCallDesc = calloc(1, sizeof(ASTCALLDESC));
       if ( !NewCallDesc ) /* return error */
       {
         AstMsg->CBL->CodErr = NoMemoryForAsyncQuery;
         RELEASE_INTER_THREAD(AstMsg);
         pthread_mutex_unlock(&AddCallDescMutex);
         return;
       }
       pthread_mutex_init(&NewCallDesc->Mutex, NULL);
       pthread_cond_init(&NewCallDesc->LCond.Cond, NULL);
       NewCallDesc->Next = AstCallDescLocal->Next;
       AstCallDescLocal->Next = NewCallDesc;
       pthread_mutex_unlock(&AddCallDescMutex);
     }
   AstCallDescLocal = AstCallDescLocal->Next;
   }

   AstCallDescCurrent = AstCallDescLocal;
   AstCallDescLocal->IsBusy++;
   AstCallDescLocal->Ast = AstMsg;

   /* Create new thread */
   if ( AstCallDescLocal->ThreadID == (pthread_t)0 )
   {
     pthread_attr_t Attr;                                   /* 10.05.12 ... */
     
     pthread_attr_init(&Attr);
     pthread_attr_setstacksize(&Attr, IntlibStackSize);     /* ... 10.05.12 */
     if ( pthread_create(&AstCallDescLocal->ThreadID, &Attr,
                         &AsyncCallThread, AstCallDescLocal) ||
          pthread_detach( AstCallDescLocal->ThreadID ))
     {
       AstMsg->CBL->CodErr = NoMemoryForAsyncQuery;        /* 18.06.14 */
       RELEASE_INTER_THREAD(AstMsg);
       AstCallDescLocal->IsBusy = 0;
       pthread_mutex_unlock(&AstCallDescLocal->Mutex);
       return;
     }
   }
   pthread_cond_signal(&AstCallDescLocal->LCond.Cond);
   pthread_mutex_unlock(&AstCallDescLocal->Mutex);
 }
#endif   /* PTHREAD_INTLIB */                               /* ... 22.08.02 */
}

/* 08.01.03 ...*/
#ifdef WIN32
static void UninitOutputObjects(PExchangeOut *ppOutput)
{
  int i,j;

  if (hMutexExchLock)                                      /* 25.04.14 */
    WaitForSingleObject(hMutexExchLock, INFINITE);         /* 22.01.03 */

/* We must enter to critical section BEFORE checking to NULL!
   (*ppOutput) could be set NULL in other critical section
*/
  if((*ppOutput) == NULL)
    goto l_end;

  if ((*ppOutput)->hServer)
    CloseHandle((*ppOutput)->hServer);

  if((*ppOutput)->lpvMapViewLinter)
    UnmapViewOfFile((*ppOutput)->lpvMapViewLinter);
  if((*ppOutput)->hFileMapLinter)
    CloseHandle((*ppOutput)->hFileMapLinter);
  if((*ppOutput)->hEventLinterReady)
    CloseHandle((*ppOutput)->hEventLinterReady);
  if((*ppOutput)->hEventLinterAdd)
    CloseHandle((*ppOutput)->hEventLinterAdd);

  if((*ppOutput)->hMailslotLinter && (*ppOutput)->hMailslotLinter != INVALID_HANDLE_VALUE)
    CloseHandle((*ppOutput)->hMailslotLinter);

  (*ppOutput)->lpvMapViewLinter = (*ppOutput)->hFileMapLinter = (*ppOutput)->hEventLinterReady = (*ppOutput)->hEventLinterAdd = (*ppOutput)->hMailslotLinter = NULL;
  (*ppOutput)->hServer = NULL;

  for(i = 0; i < AstListSize; i++)
    for ( j = 0; j < ASTBLOCKSIZE; j++ )
    {
      TASTMSG* AstMsg= AstList[i] + j;
      if (AstMsg->Actual && AstMsg->ppOutput && (*AstMsg->ppOutput) == (*ppOutput) )
      {
        AstMsg->CBL->CodErr = ERROPENQUE;
/* We must clear pointer to output exchange object because of than
   program will reach inter_sync() this ponter may already be reinitialized
   from CrtQue() and contains incorrect for this query pointer to exchange object.
*/
        AstMsg->ppOutput = NULL;
        CallUserProc(AstMsg);
        TRACE_EVENT("Set event in UninitOutputObjects", AstMsg->hAsyncEvent); /* 01.08.03 */
        SetEvent(AstMsg->hAsyncEvent);
      }
    }

  free(*ppOutput);
  *ppOutput = NULL;

l_end:
  if (hMutexExchLock)                                      /* 25.04.14 */
    ReleaseMutex(hMutexExchLock);                          /* 22.01.03 */
  return;
} /* UninitOutputObjects */

static void UninitInputObjects()
{
  int i, j;
  DWORD dwTmp;                                             /* 23.05.03 */

  if(gpInput == NULL)
    return;

  if (lInit == -1)                                             /* 08.09.99 */
    return;

#ifndef RTOS32
  if (hAstThrd)                                                /* 08.12.98 ... */
  {
/*  16.05.03 ...*/
    bThrdStopped = L_TFALSE;                                  /* 21.05.03 */
    switch(gExchType)                                      /* 20.05.03 */
    {
      case slot_exch:
#ifndef WINCE                                             /* 21.05.03b ...*/
        SetMailslotInfo(gpInput->hMailslotClient, 0);
#endif                                                    /*... 21.05.03b */
        break;
      case memory_exch:
        TRACE_EVENT("Set event in UninitInputObjects (1)", hEventStopAstThrd);  /* 01.08.03 */
        SetEvent(hEventStopAstThrd);
        break;
    } /* switch */
    while (GetExitCodeThread(hAstThrd, &dwTmp))            /* 23.05.03 ...*/
    {
      if (dwTmp != STILL_ACTIVE || bThrdStopped == L_TTRUE)
        break;
      Sleep(100);
    }                                                      /*... 23.05.03 */
/*... 16.05.03 */
    CloseHandle(hAstThrd);
    hAstThrd = 0;                                              /* 08.09.99 */
  }
/* 16.05.03 ...*/
    if(hEventStopAstThrd)
      CloseHandle(hEventStopAstThrd);
/*... 16.05.03 */

  if (gpInput->hChkThrd)
  {
/* 16.05.03 ...*/
    bThrdStopped = L_TFALSE;                                  /* 21.05.03 */
    TRACE_EVENT("Set event in UninitInputObjects (2)", hEventStopChkThrd);  /* 01.08.03 */
    SetEvent(hEventStopChkThrd);
    while (GetExitCodeThread(gpInput->hChkThrd, &dwTmp))   /* 23.05.03 ...*/
    {
      if (dwTmp != STILL_ACTIVE || bThrdStopped == L_TTRUE)
        break;
      Sleep(100);
    }                                                      /*... 23.05.03 */
/*... 16.05.03 */
    CloseHandle(gpInput->hChkThrd);
    gpInput->hChkThrd = 0;
  }
/* 16.05.03 ...*/
  if(hEventStopChkThrd)
    CloseHandle(hEventStopChkThrd);
/*... 16.05.03 */
  if(gpInput->hEventSyncChkThrd)
    CloseHandle(gpInput->hEventSyncChkThrd);

  if(gpInput->lpvMapViewClient)
    UnmapViewOfFile(gpInput->lpvMapViewClient);
  if(gpInput->hFileMapClient)
    CloseHandle(gpInput->hFileMapClient);
  if(gpInput->hEventClientReady)
    CloseHandle(gpInput->hEventClientReady);
  if(gpInput->hEventClientAdd)
    CloseHandle(gpInput->hEventClientAdd);

  if(gpInput->hMailslotClient)
    CloseHandle(gpInput->hMailslotClient);

  gpInput->lpvMapViewClient = gpInput->hFileMapClient = gpInput->hEventClientReady = gpInput->hEventClientAdd = gpInput->hMailslotClient = NULL;

  free(gpInput);
  gpInput = NULL;

  if (hLock)
    CloseHandle(hLock);                                        /* 03.12.98 */
  hLock = NULL;

/* 22.01.03 ...*/
  if (hMutexExchLock)
    CloseHandle(hMutexExchLock);
  hMutexExchLock = NULL;
/*... 22.01.03 */
#endif /* RTOS32 */

  for(i = 0; i < AstListSize; i++)
    for ( j = 0; j < ASTBLOCKSIZE; j++)                         /* 22.08.02 ... */
    {
      TASTMSG *AstTmp = AstList[i] + j;
#ifdef RTOS32
      if  (AstTmp->dwTid == GetCurrentThreadId())
      { 
        AstTmp->dwTid = 0;
#endif
        if (AstTmp->hTid)                                    /* 03.11.99 */
        {
          CloseHandle(AstTmp->hTid);
          AstTmp->hTid = NULL;
        }
        if (AstTmp->hAsyncEvent)                                  /* 03.12.98 */
        {
          CloseHandle(AstTmp->hAsyncEvent);                      /* ... 22.08.02 */
          AstTmp->hAsyncEvent = NULL;
        }
#ifdef RTOS32
      }
#endif
    }
#ifndef RTOS32
  lInit = -1L;                                              /* 03.12.98 */
#endif
  return;
} /* UninitInputObjects */
#endif
/*... 08.01.03 */


static void ReceiveData(TMSG *RcvMsg, TCBL *CBL, L_WORD NmrCom,
                        HPVOID Korteg, void *Range)
{
 L_WORD   RLnBufRow;                                        /* 21.06.99 */
 L_LONG   SvPrz = 0;                                        /* 01.08.03 */
 TASTMSG *AstMsg  = NULL;
 TAILMSG *TailMsg = NULL;
 L_LONG   FlagClear = 0;                                   /* 28.07.04 */

 CalcMsgLen(RcvMsg, &TailMsg, NULL);

#ifdef WIN32
 WaitForSingleObject(hLock, INFINITE);                     /* 03.12.98 */
#endif
#if !defined(INTER_MSDOS) && !defined(INTER_DPMI)
   AstMsg = AstList[((L_BYTE*)&TailMsg->AstInd)[0]] +         /* 14.10.05 */
                              ((L_BYTE*)&TailMsg->AstInd)[1]; /* ... 22.08.02 */
 if (AstMsg != NULL)                                       /* 03.12.98 ... */
   {
   if (!AstMsg->Actual)                                    /* 10.06.00 */
   {
# ifdef WIN32
     ReleaseMutex(hLock);                                  /* 11.09.03 */
# endif
     return;                                               /* 10.06.00 */
   }
   CBL    = AstMsg->CBL;
   Range  = AstMsg->Range;
   Korteg = AstMsg->Korteg;
   NmrCom = AstMsg->NmrCom;
# ifdef WIN32
   if (AstMsg->hTid &&                                     /* 23.05.03 */
       WaitForSingleObject(AstMsg->hTid, 0) != WAIT_TIMEOUT) /* 22.03.99 ... */
   {
     AstMsg->Actual = 0;
     TRACE_EVENT("Set event in ReceiveData (1)", AstMsg->hAsyncEvent);  /* 01.08.03 */
     SetEvent(AstMsg->hAsyncEvent);                               /* 08.01.03 */
     AstMsg = NULL;
     ReleaseMutex(hLock);                                  /* 28.07.04 */
     return;                                               /* 28.07.04 */
   }                                                       /* ... 22.03.99 */
   ReleaseMutex(hLock);
# endif
   }
 else
   return;                                                 /* ... 03.12.98 */
#endif
#if defined(PTHREAD_INTLIB)                                /* 22.08.02 */
 AstMsg->Status = 0;
#endif
 RLnBufRow     = CBL->LnBufRow;                                /* 21.06.99  */
 SvPrz         = CBL->PrzExe;                                 /* 01.08.03 */
#if defined(KRB_AUTH)                                     /* 24.08.14 ... */
 if ( *((L_LONG*)RcvMsg->CBL.Command) == OKRB)
 {
   if ( KrbReceive(RcvMsg, AstMsg) >= 0 )
   {
     return;
   }
 }
#endif                                                    /* ... 24.08.14 */
 CBL->CodErr   = RcvMsg->CBL.CodErr;
 CBL->SysErr   = RcvMsg->CBL.SysErr;
 CBL->Prior    = RcvMsg->CBL.Prior;
 CBL->RowId    = RcvMsg->CBL.RowId;
 CBL->RowCount = RcvMsg->CBL.RowCount;
 if (CBL->CodErr == SMALLBUFKOR)                               /* 01.10.98 */
   CBL->LnBufRow = RcvMsg->CBL.LnBufRow;                       /* 01.10.98 */
 if ( comtab[NmrCom].CanLongMsg )                                 /* 16.01.07b */
   if ((CBL->CodErr == SMALLBUFKOR) && (RLnBufRow >= CBL->LnBufRow)  /* ... 18.10.01 */
       && CBL->LnBufRow > 0)                                         /* 10.12.01 */
   {
     CBL->PrzExe |= Q_RECEIVE;                                 /* ... 21.06.99 */
   }
 if ((CBL->CodErr > NOKOR) && (CBL->CodErr != Row_Locked) &&
     (CBL->CodErr != Proc_Raised_Exception))                   /* 15.09.98 */
   goto label_1;
 if (COM2INT(CBL->Command) == OPEN ||                       /* 16.01.07 */
     COM2INT(CBL->Command) == OCUR )
   CBL->NumChan = RcvMsg->CBL.NumChan;
 if (COM2INT(CBL->Command) == CLOS )                        /* 16.01.07 */
   CBL->NumChan = 0;
 if ((comtab[NmrCom].p1 && Korteg) ||                         /* 15.07.03 04.04.00 */
     (CBL->PrzExe & Q_EV_CATCH))                              /* 04.04.00 */
 {
   if ( RcvMsg->Buf.Word[3] )                                 /* 15.07.03 */
   {
     if (RcvMsg->CBL.LnBufRow > CBL->LnBufRow)                  /* 05.09.00 ... */
     {
       CBL->CodErr = SMALLBUFKOR;
       CBL->LnBufRow = RcvMsg->CBL.LnBufRow;                    /* 12.09.03a */
     }
     else                                                       /* ... 05.09.00 */
     {
       CBL->LnBufRow = RcvMsg->CBL.LnBufRow;                    /* 12.09.03 15.07.03 */
       memcpy(Korteg, &RcvMsg->Buf.Byte[8], CBL->LnBufRow);
       if (comtab[NmrCom].p1 != 5 && Range &&
           RcvMsg->Buf.Word[3] - CBL->LnBufRow > 1)    /* 28.11.98 ... */ /* 28.10.02 */
       {
#  ifdef LONG_ALIGN                                           /* 01.03.99 */
         L_CHAR* Addr = &RcvMsg->Buf.Byte[CBL->LnBufRow + 8];   /* 06.02.12 ... */
         L_WORD Size1;
         L_WORD Size2;

         L_MEMCPY2(&Size1, Addr);
         L_MEMCPY2(&Size2, Addr + sizeof(L_WORD));
         L_MEMCPY(Range, Addr, Size1 * Size2 + sizeof(L_WORD) * 2); /* ... 06.02.12 */
#  else /* LONG_ALIGN */
         L_WORD *pw = (L_WORD *)&RcvMsg->Buf.Byte[CBL->LnBufRow + 8]; /* 14.06.02 */
         memcpy(Range, pw, pw[0] * pw[1] + sizeof(L_WORD) * 2);
#  endif /* LONG_ALIGN */
       }                                                      /* ... 28.11.98 */
#ifdef PREPARE                                          /* 04.12.01 ... */
#if _VER_MAX >= 600                                     /* 29.11.01 ... */
       if ( COM2INT(CBL->Command) == TRAN )                 /* 16.01.07 */
         FillBufferSize( ( char* )Korteg, RLnBufRow );
#endif                                                  /* ... 29.11.01 */
#endif                                                  /* ... 04.12.01 */
     }
   }
   else  /* RcvMsg->Buf.Word[3] */
     CBL->LnBufRow = 0;                                 /* 12.09.03 */
 } /* Korteg */
label_1:;
#if !defined(INTER_MSDOS) && !defined(INTER_DPMI)
 CallUserProc(AstMsg);                                       /* 04.11.02 */
#endif                                                       /* 25.05.98 */
#if !defined (PTHREAD_INTLIB)
#if defined(OS2000)                                         /* 10.03.04 ... */
 {
   L_LONG DestPid;

   memcpy(&DestPid, &TailMsg->lReserv, sizeof(L_LONG));
   if ( (pthread_t)(DestPid) != pthread_self() )
   {
     pthread_kill((pthread_t)(DestPid), SIGUSR1);
   }
 }                                                          /* ... 10.03.04 */
#endif
#if defined(WIN32)                                           /* 27.07.04 ... 01.08.03 ... */
  {
    L_LONG Sv = CBL->CodErr;                                   /* 09.09.99 */

    /* For WIN32 we can send async. query in User function. In this case
       we can't SetEvent and Can't clear Actual field.
       In other case
           - For nonAsync query we must SetEvent and clear Actual
           - For Async query we must call SetEvent only
    */
    FlagClear = ((SvPrz & Q_ASYNC) &&                      /* 28.07.04 */
       (AstMsg->CBL->PrzExe & Q_ASYNCDONE));               /* 28.07.04 */
    /* Need to prevent use AstMsg btw. SetEvent and ACTUAL clearing...
       After SetEvent main thread may call new inter() ans allocate the same
       AstMsg for new query and send this query before Actual is cleared.
       After thet Actual cleared and inter_wait_single in main thread diagnose
       "query done" status for second inter (really - not done) */
    if (FlagClear)                                         /* 13.02.05 */
      WaitForSingleObject(hLock, INFINITE);                /* 13.02.05 */

    if (!(SvPrz & Q_ASYNC) ||                              /* 27.07.04 */
         (AstMsg->CBL->PrzExe & Q_ASYNCDONE))
    {
      TRACE_EVENT("Set event in ReceiveData (2)", AstMsg->hAsyncEvent);  /* 01.08.03 */
      SetEvent(AstMsg->hAsyncEvent);                              /* 08.01.03 */
    }
    if (Sv >= ERROPENQUE && Sv <= ERRREADMSG)                   /* 22.12.98 */
      UninitInputObjects();                                     /* 08.01.03 */
  }
  if (FlagClear)                              /* 28.07.04 */ /* ... 27.07.04 */
#endif
  {
    TRACE_EVENT("ASYNC clearing", AstMsg->hAsyncEvent);
    if (SvPrz & Q_EV_CATCH)
      AstMsg->Actual = 2;
    else
      AstMsg->Actual = 0;
#if defined(WIN32)
    ReleaseMutex(hLock);                                        /* 13.02.05 */
#endif
  }
#endif     /* PTHREAD_INTLIB */                          /* ... 01.08.03 */
#ifdef VMS
 SYS$WAKE(NULL, NULL); /* The message has received. Wake up the application */ /* 30.07.12 */
#endif
}

#if !defined(INTER_MSDOS) && !defined(INTER_DPMI)
static TASTMSG *FindFreeDesc(TASTMSG *Ast)
{
 L_WORD i;
 L_WORD j;                                               /* 22.08.02 */
 TASTMSG *Tmp = NULL;
#if (defined(UNIX) || defined(QNX)) && !defined(PTHREAD_INTLIB)
 volatile static L_BYTE LockFlag = 0;                       /* 22.03.07 */
                 L_BYTE GoNext = 0;
#endif
#ifdef WIN32
 DWORD dwTid = GetCurrentThreadId();                     /* 12.03.98 */
Repeat:                                                  /* 18.05.05 */
 WaitForSingleObject(hLock, INFINITE);                   /* 03.12.98 */
#endif
#ifdef VMS
 SYS$SETAST(0); /* Temporary off the AST */                 /* 30.07.12 */
#endif
#ifdef OS9000
 sigmask(1);
#endif
#if defined(PTHREAD_INTLIB)                                /* 22.08.02 ... */
 pthread_mutex_lock(&IntlibMutex);
#else
#if defined(UNIX) || defined(QNX)                           /* 22.03.07 .. */
 /* Prevent recursive FindFreeDesc from ast inter from signal handler */
 if (LockFlag == 0)
   LockFlag = 1;
 else
   GoNext = 1;  /* The recursive call. Skip the first free ast and take the second */
#endif                                                      /* .. 22.03.07 */
#endif

#ifdef RTOS32
  for(i = 0; i < AstListSize; i++)
    for ( j = 0; j < ASTBLOCKSIZE; j++)                         /* 22.08.02 ... */
    {
      TASTMSG *AstTmp = AstList[i] + j;
      if  (AstTmp->dwTid == dwTid)
      { 
        AstTmp->dwTid = 0;
        AstTmp->Actual == 0;
        if (AstTmp->hTid)                                        /* 03.11.99 */
        {
          CloseHandle(AstTmp->hTid);
          AstTmp->hTid = NULL;
        }
      }
    }
#endif 

 for (i = 0; i < AstListSize; i++)
 {
   TASTMSG* AstBlock = AstList[i];

   if ( AstBlock == NULL )
     break;
   for ( j = 0; j < ASTBLOCKSIZE; j++)
   {
     TASTMSG* TmpAst = AstList[i] + j;

     if (TmpAst->Actual == 0 )
      {
#if defined(PTHREAD_INTLIB)
        if ( pthread_mutex_trylock( &TmpAst->Mutex ) )
          continue;                                         /* 03.04.08c */
        if  ( TmpAst->WaitFlag ) /* A thread of the ast is waiting continuing */
        {
          pthread_mutex_unlock(&TmpAst->Mutex);             /* 03.04.08c */
          continue;
        }
#else
#if defined(UNIX) || defined(QNX)                           /* 22.03.07 .. */
        if ( GoNext )
        {
          GoNext = 0;
          continue; /* Skip the ast for first recursive call */
        }
#endif                                                      /* .. 22.03.07 */
#endif
#ifdef WIN32                                                 /* 01.12.04 */
        if (!Tmp)                                            /* 28.07.04 */
          Tmp = TmpAst;                                      /* 28.07.04 */
#else
        /* For unix we should break the search after find the first
        free element for prevent locking mutexes of all Ast. Or we can
        free unused Ast after locking it */
        Tmp = TmpAst;                                      /* 28.07.04 */
        goto point_exit_from_for;
#endif
      }
#ifdef WIN32                                                 /* 16.05.04 ... */
    /* This section added to prevent allocation too AstList block
       for one CBL. In this case for Async query we may lost answer
       synchroniation and receive End Of Query before kernel send data on
       this query. This section may be needed for UNIX...
       This section may be req. in additional checking - for example on
       Sleep count. */
    else
      {
      if (Ast->CBL == TmpAst->CBL)
        {
        /* For Async query (Linapi Async typical sequence) we must
           enable send another query by old CBL and reuse ola Ast segment.
           Description continued at the same date below */
        if (!(TmpAst->CBL->PrzExe & Q_ASYNC))                    /* 27.07.04 */
          while (TmpAst->Actual == 1)
          {                                                  /* 18.05.05 ... */
            /* Sleep(0); */
            ReleaseMutex(hLock);
            goto Repeat;
          }                                                  /* ... 18.05.05 */
        Tmp = TmpAst;
        goto point_exit_from_for;
        }
      }
#endif                                                       /* ... 16.05.04 */
    }
 }
point_exit_from_for:

 if ( Tmp == NULL)
 {
   if ( AstListSize >= ASTBLOCKCOUNT )
   {
#if defined(PTHREAD_INTLIB)
     pthread_mutex_unlock(&IntlibMutex);
#else
#if defined(UNIX) || defined(QNX)
     LockFlag = 0;                                          /* 22.03.07 */
#endif
#endif
     return NULL;
   }

   Tmp = (TASTMSG*)calloc(ASTBLOCKSIZE, sizeof(TASTMSG));
   if (Tmp == NULL)
   {
#if defined(PTHREAD_INTLIB)
     pthread_mutex_unlock(&IntlibMutex);
#else
#if defined(UNIX) || defined(QNX)
     LockFlag = 0;                                          /* 22.03.07 */
#endif
#endif
     return NULL;
   }
   for ( i = 0; i < ASTBLOCKSIZE; i++)
   {
     memset(Tmp + i, 0, sizeof(TASTMSG));
     ((L_BYTE*)(&Tmp[i].AstIndex))[0] = (L_BYTE)AstListSize;       /* 23.08.03 */
     ((L_BYTE*)(&Tmp[i].AstIndex))[1] = (L_BYTE)i;                 /* 23.08.03 */
#if defined(PTHREAD_INTLIB)
     pthread_mutex_init(&Tmp[i].Mutex, NULL);
     pthread_cond_init(&Tmp[i].LCond.Cond, NULL);                 /* 03.04.08b */
#endif
   }
   AstList[AstListSize] = Tmp;
   AstListSize++;
#if defined(PTHREAD_INTLIB)
   pthread_mutex_lock(&Tmp->Mutex);
#endif
 }                                                          /* ... 22.08.02 */

 Tmp->CBL     = Ast->CBL;
 Tmp->Range   = Ast->Range;
 Tmp->Korteg  = Ast->Korteg;
 Tmp->NmrCom  = Ast->NmrCom;
 Tmp->UsrProc = Ast->UsrProc;
 Tmp->Actual  = 1;

#if defined(PTHREAD_INTLIB)                                 /* 22.08.02 ... */
 Tmp->Status = 1;
 pthread_mutex_unlock(&Tmp->Mutex);
 pthread_mutex_unlock(&IntlibMutex);
#else
#if defined(UNIX) || defined(QNX)
 LockFlag = 0;                                              /* 22.03.07 */
#endif
#endif /* PTHREAD_INTLIB */                                 /* ... 22.08.02 */

#ifdef OS9000
 sigmask(0);
#endif
#ifdef VMS
 SYS$SETAST(1); /* On the ast back */                       /* 30.07.12 */
#endif
#ifdef WIN32
#ifndef WINCE                                              /* 08.04.02 */
 if (Tmp->dwTid != dwTid)                                  /* 03.11.99 */
   {
   Tmp->dwTid = dwTid;                                     /* 03.12.98 */
   if (Tmp->hTid != NULL)                                  /* 03.11.99 */
     CloseHandle(Tmp->hTid);
   DuplicateHandle(GetCurrentProcess(), GetCurrentThread(),/* 22.03.99 */
                   GetCurrentProcess(), &Tmp->hTid, 0, FALSE,
                   DUPLICATE_SAME_ACCESS);
   }
#endif /* WINCE */
 ReleaseMutex(hLock);                                      /* 23.09.02 03.12.98 */
#endif

 Tmp->CBL->PrzExe &= ~Q_ASYNCDONE;                         /* 18.05.05 */
 return (Tmp);
}

#if defined(UNIX) && defined(SOCKETS)                      /* 28.02.99 ... */
#if !defined(TWICE_READ)
static int read_from_sock(int sock, L_BYTE *buf)             /* 16.01.07f ... */
{
  int    len;                                     /* 01.03.00 ... */
  L_BYTE *Buf = buf;
  int    size = sizeof(TMSG);
  static int SavedLenLin = 0;                               /* 03.04.08a  ... */
  static char SavedBufLin[sizeof(TMSG)];
  static int SavedLenNet = 0;
  static char SavedBufNet[sizeof(TMSG)];
  int*   SavedLen;
  char*  SavedBuf;
  
  if ( sock == msg_lin )
  {
    SavedLen = &SavedLenLin;
    SavedBuf = SavedBufLin;
  }
  else
  {
    SavedLen = &SavedLenNet;
    SavedBuf = SavedBufNet;
  }
 
  if ( *SavedLen != 0 )
  {
    memcpy(Buf, SavedBuf, *SavedLen);
    len = *SavedLen;
    *SavedLen = 0;
    goto skip_read;
  }                                                         /* ... 03.04.08a */
  
  do
  {
    len = read(sock, Buf, size);
 
    if (len <= 0)
    {
      if ( len == 0 )
        return 0;
      if (errno == EINTR)
        continue;
      if ( errno == EAGAIN && Buf != buf ) /* Nothing to read and It is begin of the next message */
        continue;
      return (-1);
    }
    
skip_read:
    if ( (len >= sizeof(IPC_TYP)) || /* Message has been read now */
         (Buf - buf >= sizeof(IPC_TYP)) ) /* Begin of message has been read in prev time */
    {
      register int LenMsg = *(IPC_TYP *)buf;
      register int LenBuf = Buf - buf + len;
    
      if ( LenMsg == LenBuf )
        return LenMsg;
      if ( LenMsg < LenBuf )
      {
        *SavedLen = LenBuf - LenMsg;                        /* 03.04.08a */
        memcpy(SavedBuf, &buf[LenMsg], *SavedLen);          /* 03.04.08 */
        return LenMsg;
      }
    }
 
    Buf = Buf + len;
    size = size - len;
  }
  while ( size > 0 );  
 
  return Buf - buf;
 }                                                           /* ... 16.01.07f */
#else
static int read_from_sock(int sock, L_BYTE    *buf)           /* 16.01.07d .. */
{
 int     len;                                               /* 01.03.00 ... */
 L_BYTE *Buf = buf;
 int     size = sizeof(IPC_TYP);
 int     DataFlag = 0;
 
 while(1)
 {
   len = read(sock, Buf, size);

   if (len <= 0)
   {
     if ( len == 0 )
       return 0;
     if (errno == EINTR)
       continue;
     return (-1);
   }
   Buf = Buf + len;
   size = size - len;


   if ( size == 0 )
   {
      if ( DataFlag == 0 )
      {
        size = (size_t)(*(IPC_TYP *)buf) - sizeof(IPC_TYP);
        DataFlag = 1;
      }
      else
      {
        return len;
      }
   }
 }
 /* No return here */
}                                                           /* .. 16.01.07d */
#endif
#endif                                                        /* ... 28.02.99 */

#ifdef OS9000                                           /* 10.07.00 ... */
int ConnCheck(int flag)
{
 mh_com *mh;
 int    pid_srv,TimeSrv;
 char   *s, pipe[L_MAX_MBX+32];                             /* 17.02.14 ... */

 s = getenv(flag ? LIN_MBX : NET_MBX)
 if (s)
 {
   if ( strlen(s) > L_MAX_MBX )
    s = NULL;
 }
 if (s == NULL)
 {
   if (flag)
     s = KEYLIN;
   else
     s = KEYNET;
 }                                                          /* ... 17.02.14 */
 sprintf(pipe, "PID_%s", s);
 mh = (mh_com *)modlink(pipe, 0);
 if ((int)mh == -1)
   return (L_TFALSE);
 pid_srv  = *(ADRDAT(mh));
 TimeSrv  = *(ADRDAT(mh) + 1);
 munload(pipe, 0);
 if (!TimeSrv)
   return(L_TFALSE);
 return L_TTRUE;
}
#endif                                                  /* ... 10.07.00 */

#if defined(PTHREAD_INTLIB)
# if !defined(OS2000) && !defined(SOCKETS)                  /* 08.12.04 */
static void AlarmHandler(int Sig)
{
  pthread_t MyTID = pthread_self();

  if ( LocalThreadID != 0 && MyTID != LocalThreadID )       /* 14.12.09 */
      pthread_kill(LocalThreadID, Sig);
  if ( NetThreadID != 0 && MyTID != NetThreadID )           /* 14.12.09 */
      pthread_kill(NetThreadID, Sig);
  return;
}
#endif /* UNIX !SOCKETS */
#if defined(OC2000) || (defined(QNX) && defined(PTHREAD_INTLIB)) /* 25.07.07 */ /* 08.12.04 .. */
/* The function test queue by try open it */
static int TestKernel(int IsLocal)
{
  static char* LocalMbx = NULL;
  static char* NetMbx = NULL;
  char* Mbx;

#if !defined(QNXNATIVE)                                     /* 14.12.09 */
  if ( IsLocal )
  {
#if defined(QNX)
    if ( kill( pid_lin, 0 ) != 0 && (errno != EPERM) )      /* 25.07.07 */
      return 1;
#endif
    Mbx = LinterMbxKey;                                     /* 08.02.07 */
  }
  else /* IsLocal == L_TFALSE */
  {
#if defined(QNX)
    if ( kill( pid_net, 0 ) != 0 && (errno != EPERM) )      /* 27.05.08 */
      return 1;
#endif
    Mbx = NetMbxKey;                                        /* 08.02.07 */
  }
  /* Try open queue */
  {
    mqd_t mqstat;
    struct mq_attr  attr;

    memset(&attr,0,sizeof(attr));
    attr.mq_maxmsg  = 3;
    attr.mq_msgsize = MAXMSGSIZE;
    attr.mq_flags   = 0;

    /* Try open queue in exclusive mode */
    while ( (mqstat = mq_open(Mbx, O_WRONLY|O_CREAT|O_EXCL, 0777, &attr)) == /* 01.02.08 */
                                                        (mqd_t)-1)
    {
      if ( errno == EINTR )
      {
        errno = 0;
        continue;
      }
      /* Test queue for opening */
      if ( errno == EEXIST )
        return 0;  /* SUCCESS - queue exist */
      return 1;
    }
    mq_close(mqstat);
    mq_unlink(Mbx);
  }
  return 1;
#else /* QNXNATIVE */                                       /* 14.12.09 ... */
  if ( IsLocal )
  {
    if ( kill( Linter.Pid, 0 ) != 0 && (errno != EPERM) )
      return 1;
  }
  else /* IsLocal == L_TFALSE */
  {
    if ( kill( Dbc.Pid, 0 ) != 0 && (errno != EPERM) )
      return 1;
  }
  return 0;
#endif /* QNXNATIVE */                                      /* ... 14.12.09 */
}  /* TestKernel */

/* This function call from new thread for prevent deadlock on mutexes */
static void* UninitFromOtherThread(void* a)
{
  UninitLinterClient();
}

/* Run deinitialization function in new thread for prevent deadlock on mutexes */
static void RunUninitThread()
{
  pthread_attr_t attr;
  pthread_t thread;

  pthread_attr_init(&attr);
  pthread_attr_setdetachstate(&attr, PTHREAD_CREATE_DETACHED);
  pthread_create(&thread, &attr, UninitFromOtherThread, NULL);
}
#endif /* OS2000 */                                         /* .. 08.12.04 */
#endif /* PTHREAD_INTLIB */                                 /* ... 22.08.02 */


#ifdef VMS
static void ReadMsgFromServer(TMSG *Msg)
#endif
# if defined(PTHREAD_INTLIB)
static void* ReadMsgFromServer(void* flag)                  /* 22.08.02 */
# else
#if (defined(QNX) && !defined(QNXNATIVE)) || defined(VXWORKS) /* 14.12.09 02.04.98 */ /* 11.01.01 */
static void ReadMsgFromServer(mqd_t msg_id)
#endif
#if defined(QNXNATIVE)
static void ReadMsgFromServer(int flag)                     /* 14.12.09 */
#endif
#ifdef UNIX
static void ReadMsgFromServer(int msg_id)
#endif
#endif /* PTHREAD_INTLIB */
#ifdef OS9000
static void ReadMsgFromServer(int sig)
#endif
#ifdef INTER_MSWINDOWS
static VOID WINAPI ReadMsgFromServer(LPVOID lpvAst)
#endif
{
#ifdef OS9000
 if (sig == SIGCHECKSHUT)
   {
   L_BYTE ShutLin = 0, ShutNet = 0;
   int i,j;                                                 /* 22.08.02 */

   AlmId = 0;
   if (pid_lin)
     ShutLin = !CheckServer(&pid_lin, TimeForkLin);
   if (pid_net)
     {
     ShutNet = !ConnCheck(0);                                   /* 10.07.00 */
     if (!ShutNet)
       ShutNet = !CheckServer(&pid_net, TimeForkNet);
     else
       CheckServer(&pid_net, TimeForkNet);                      /* 14.07.00 */
     }
   if (ShutLin || ShutNet)
     {
     for(i = 0; i < AstListSize; i++)
       for ( j = 0; j < ASTBLOCKCOUNT; j ++ )               /* 22.08.02 ... */
       {
         TASTMSG* AstMsg= AstList[i] + j;

         if (AstMsg->Actual)
         {
           if ((AstMsg->Server == pid_lin && ShutNet)||
               (AstMsg->Server == pid_net && ShutLin))
             continue;
           AstMsg->ShMem->CBL.CodErr = ERROPENQUE;          /* ... 22.08.02 */
           raise(BASESIG + i);
         }
       }
     }
   if (!pid_lin && !pid_net)
     {
     alm_delete(AlmId);
     AlmId = 0;
     }
   else
     {
     signal(SIGCHECKSHUT, ReadMsgFromServer);
     AlmId = alm_set(SIGCHECKSHUT, TIMECHECK(10));
     }
   }
 {
   int  i = sig - BASESIG;                                  /* 22.08.02 ... */
   L_WORD AstInd, ri, rj;

   if ( i < 0 )
     return;

   AstInd = i;
   ri=((L_BYTE*)&AstInd)[0];                                 /* 18.06.14 */
   rj=((L_BYTE*)&AstInd)[1];                                 /* 18.06.14 */

   if ( ri < AstListSize && rj < ASTBLOCKCOUNT)
     ReceiveData(AstList[ri][rj].ShMem, NULL, 0, NULL, NULL);  /* ... 22.08.02 */
 }
 return;
#endif
#ifdef VMS
    if (GlIOSB.iosb$w_status != SS$_NORMAL)                 /* 30.07.12 ... */
    {
      GlRcvMsg.CBL.CodErr = ERRREADMSG;
      GlRcvMsg.CBL.SysErr = GlIOSB.iosb$w_status;
    }
    ReceiveData(&GlRcvMsg, NULL, 0, NULL, NULL);
    /* Next async read */
    if ((SYS$QIO(EFN$C_ENF, /* No event */
                 InputMailbox, /* Mailbox channel */
                 IO$_READVBLK, /* reading operation */
                 &GlIOSB, /* Status block */
                 ReadMsgFromServer, /* AST */
                 (__int64)&GlRcvMsg, /* AST arg */
                 &GlRcvMsg, /* Buffer */
                 LNMSG, /* Buffer size */
                 0, 0, 0, 0)) != SS$_NORMAL)
    {
      UninitLinterClient();
    }                                                       /* ... 30.07.12 */
    return;
#endif
#if defined(UNIX) || defined(QNX) || defined(VXWORKS)   /* 11.01.01 */
 int  i;
 int  j;                                                    /* 22.08.02 */
#if defined(QNX) || defined(VXWORKS)                    /* 11.01.01 */
 uint_t prio;
#endif
 TMSG RcvMsg;

#if defined(QNXNATIVE)                                      /* 14.12.09 ... */
 IntlibServer_t* Server = flag?&Linter:&Dbc;
 int LocOk;
 int NetOk;
#endif /* QNXNATIVE */                                      /* ... 14.12.09 */

#if defined(PTHREAD_INTLIB)                                 /* 22.08.02 ... */
#if !defined(SOCKETS)
 struct  sigaction Action;
 int* PMsgId;
#endif

#if defined(UNIX)
 int msg_id;

#if defined(SOCKETS)
 if (flag)
   msg_id = msg_lin;
 else
   msg_id = msg_net;
#else
 if (flag)
   PMsgId = &msg_lin;
 else
   PMsgId = &msg_net;
#endif  /* SOCKETS */
#elif (defined(QNX) && !defined(QNXNATIVE)) || defined(VXWORKS)  /* UNIX */ /* 14.12.09 */
 mqd_t msg_id;
#if defined(OS2000) || (defined(QNX) && defined(PTHREAD_INTLIB)) /* 25.07.07 */
 struct timespec time_break = {0, 0};
 int LocOk;
 int NetOk;
#endif
#endif

#if defined(LINUX)
 pthread_mutex_lock(&InternalMutex);
 if (flag)
   LocalThPid = getpid();
 else
   NetThPid = getpid();
 pthread_cond_signal(&RunReaderCond.Cond);
 pthread_mutex_unlock(&InternalMutex);
#endif /* LINUX */
#if !defined(SOCKETS)
#if !defined(OS2000)                                        /* 08.12.04 */
 if (flag)
   LocalThreadID = pthread_self();
 else
   NetThreadID = pthread_self();
#if !defined(QNXNATIVE)                                     /* 14.12.09 ... */
 memset(&Action, 0, sizeof(Action));
 Action.sa_handler = AlarmHandler;
 sigaction(SIGALRM, &Action, NULL);
 alarm(TIMEOUT);
#endif /* QNXNATIVE */                                      /* ... 14.12.09 */
#else  /* OS2000 */                                         /* 08.12.04 .. */

#if defined(MTOS2000)
 msg_id = ((struct PidMsg*)flag)->MsgId;
#else /* MTOS2000 */
 pthread_mutex_lock(&InternalMutex);
 if ( msg_user == (mqd_t) -1 )
   pthread_cond_wait(&RunReaderCond.Cond, &InternalMutex);
 pthread_mutex_unlock(&InternalMutex);

 msg_id = msg_user;
#endif /* MTOS2000 */

#endif /* OS2000 */                                         /* .. 08.12.04 */
#endif /* SOCKETS */


 for(;;)
 {
#if !defined(SOCKETS)
#if !defined(QNX) && !( defined(OS2000) && defined(PTHREAD_INTLIB))   /* 08.12.04 */
 msg_id = *PMsgId;
#elif defined(OS2000)
 time_break.tv_sec = time( NULL ) + TIMEOUT;                /* 08.12.04 */
#elif (defined(QNX) && !defined(QNXNATIVE)) && defined(PTHREAD_INTLIB) /* 14.12.09 */
 time_break.tv_sec = time( NULL ) + TIMEOUT;                /* 25.07.07 */
#endif
#if !defined(QNXNATIVE)                                     /* 14.12.09 */
 msg_id = msg_user;
#endif /* QNXNATIVE */
#endif /* SOCKETS */
#endif  /* PTHREAD_INTLIB */                                /* ... 22.08.02 */


l5:                                                     /* 12.02.97 */
 errno = 0;
#if (defined(QNX) && !defined(QNXNATIVE)) || defined(VXWORKS)     /* 14.12.09 11.01.01 */
#if (defined(OS2000)|| defined(QNX)) && defined(PTHREAD_INTLIB) /* 25.07.07 */ /* 08.12.04 */
 if (mq_timedreceive(msg_id, (char*)&RcvMsg, MAXMSGSIZE, &prio, &time_break)
                                                               < 0)
#else /* OS2000 PTHREAD_INTLIB */
 if (mq_receive(msg_id, (char*)&RcvMsg, MAXMSGSIZE, &prio) < 0)/* 02.04.98 03.06.98 */
#endif  /* OS2000 PTHREAD_INTLIB */
#else /* QNX VXWORKS */
#ifdef SOCKETS                                          /* 12.11.98 */
try_next:                                               /* 01.03.00 */
 i = read_from_sock(msg_id, (L_BYTE *)&RcvMsg);               /* 16.01.07d 28.02.99 */
 if (i <= 0)
#else
#if !defined(QNXNATIVE)                                     /* 14.12.09 */
 if (msgrcv(msg_id, (char*)&RcvMsg, MAXMSGSIZE, getpid(),
#if !defined(PTHREAD_INTLIB)                                /* 22.08.02 ... */
                                                   IPC_NOWAIT
#else
                                                   0
#endif  /* PTHREAD_INTLIB */
                                                          ) < 0)
#endif /* QNXNATIVE */                                      /* 14.12.09 */
#endif  /* SOCKETS */                                       /* ... 22.08.02 */
#endif  /* QNX || VXWORKS*/
#if defined(QNXNATIVE)                                      /* 14.12.09 ... */
#if defined(PTHREAD_INTLIB)
 alarm(TIMEOUT);
 if ( MsgReceivePulse(Server->ClntChId, &RcvMsg, sizeof(RcvMsg), NULL ) == -1 )
#else /* PTHREAD_INTLIB */
 {                                                          /* 27.10.10 ... */
    struct itimerspec Timeout;

    memset(&Timeout, 0, sizeof(Timeout));
    Timeout.it_value.tv_sec = TIMEOUT;
    Timeout.it_interval.tv_sec = TIMEOUT;
    timer_settime(Client.Timer, 0, &Timeout, NULL);
 }                                                          /* ... 27.10.10 */
 if ( Server->Shm == 0 || Server->Shm == MAP_FAILED) /* The connection to server was closed in alarm handler */
#endif /* PTHREAD_INTLIB */
#endif /* QNXNATIVE */                                      /* ... 14.12.09 */
   {
#if !((defined(OS2000) || defined(QNX)) && defined(PTHREAD_INTLIB)) /* 25.07.07 */ /* 08.12.04 */
   if (errno == EINTR)                                  /* 12.02.97 */
     {
#if !defined(SOCKETS) && defined(PTHREAD_INTLIB)            /* 22.08.02 ... */
     alarm(TIMEOUT);
     continue;
#endif                                                      /* ... 22.08.02 */
     goto l5;                                           /* 12.02.97 */
     }
#endif  /* OS2000 PTHREAD_INTLIB */
#if defined(QNX) || defined(VXWORKS)                    /* 27.10.99 */ /* 11.01.01 */
   if (errno == EAGAIN)
     return;                                            /* 27.02.04 */
#if (defined(OS2000) || defined(QNX)) && defined(PTHREAD_INTLIB)  /* 25.07.07 */ /* 08.12.04 .. */
   if ( errno == ETIMEDOUT || errno == EINTR )
   {
     LocOk = -1;
     NetOk = -1;

#if !defined(QNXNATIVE)                                     /* 14.12.09 */
     if ( msg_net != (mqd_t) -1 )
     {
       if (NetOk = TestKernel(0) )
       {
         msg_id = msg_net;
         pthread_mutex_lock(&IntlibMutex);                  /* 25.07.07 */
         UninitLinter(pid_net);
         pthread_mutex_unlock(&IntlibMutex);                /* 25.07.07 */
       }
     }

     if ( msg_lin != (mqd_t) -1 )
     {
       if ( LocOk = TestKernel(1) )
       {
         msg_id = msg_lin;
         pthread_mutex_lock(&IntlibMutex);                  /* 25.07.07 */
         UninitLinter(pid_lin);
         pthread_mutex_unlock(&IntlibMutex);                /* 25.07.07 */
       }
     }
#else /* QNXNATIVE */                                       /* 14.12.09 ... */
     if ( Dbc.Pid != 0 )
     {
       if ( LocOk = TestKernel(1) )
       {
         pthread_mutex_lock(&IntlibMutex);
         UninitLinter(Dbc.Pid);
         pthread_mutex_unlock(&IntlibMutex);
       }
     }

     if ( Linter.Pid != 0 )
     {
       if ( LocOk = TestKernel(1) )
       {
         pthread_mutex_lock(&IntlibMutex);
         UninitLinter(Linter.Pid);
         pthread_mutex_unlock(&IntlibMutex);
       }
     }
#endif                                                      /* ... 14.12.09 */

     if ( (NetOk <= 0) && (LocOk <= 0) )
     {
       continue;
     }
   }
#endif   /* OS2000 PTHREAD_INTLIB */                        /* .. 08.12.04 */

   if (1)
#else                                                   /* 27.10.99 */
   if (errno == EAGAIN)                                 /* 20.06.98 */
#if !defined(PTHREAD_INTLIB)                            /* 11.04.13 */
     return;                                            /* 20.06.98 */
#else                                                   /* 11.04.13 ... */
     return NULL;
#endif                                                  /* ... 11.04.13 */
#ifdef SOCKETS                                          /* 12.11.98 */
   /* If it was write to socket but peer at the time did not read but
      peer closed socket, error 104 was. After that setup code error but
      did not free waiting AST. Programm deaded in inter_wait_single.
      It must free the AST on any error and close the socket */
   if (1)                                               /* 08.04.05 */
#else
#ifdef BSD
   if (errno == EINVAL)
#else
   if (errno != ENOMSG)
#endif
#endif
#endif                                                   /* 27.10.99 */
     {
#if defined(PTHREAD_INTLIB)                              /* 22.08.02 ... */
     pthread_mutex_lock(&IntlibMutex);
#endif
     for (i = 0; i < AstListSize; i++)
      for ( j = 0; j < ASTBLOCKSIZE; j++)
      {
       TASTMSG* TmpAst = AstList[i] + j;

#if defined (PTHREAD_INTLIB)
       pthread_mutex_lock(&InternalMutex);
#endif
       if (TmpAst->Actual &&
#if !defined(SOCKETS) || defined(PTHREAD_INTLIB)
           TmpAst->Status &&
#endif
#ifndef QNX                                                      /* 03.07.08 */
           (*(TmpAst->msg_id) == msg_id ||
            *(TmpAst->msg_id) == (MSGID_TYPE)-1 ) &&             /* 15.09.04 */
#endif
          1                                                      /* 16.07.08 */
          )
         {
#ifdef SOCKETS                                           /* 12.11.98 */
         if (*(TmpAst->msg_id) >= 0)
           {
           close(*(TmpAst->msg_id));
           *(TmpAst->msg_id) = -1;
           }
#endif
         memset((char *)(RcvMsg.Buf.Byte), 0, 8);
         ((TAILMSG *)(&RcvMsg.Buf.Byte[8]))->AstInd = TmpAst->AstIndex;
         RcvMsg.CBL.CodErr = (i == 0?ERROPENQUE:ERRREADMSG);  /* 08.04.05 */
         RcvMsg.CBL.SysErr = errno;
#if defined(PTHREAD_INTLIB)
         pthread_mutex_unlock(&InternalMutex);
#endif
         ReceiveData(&RcvMsg, NULL, 0, NULL, NULL);
         }
#if defined(PTHREAD_INTLIB)
       else
           pthread_mutex_unlock(&InternalMutex);
#endif
      }
#if defined(SOCKETS)                                        /* 09.02.03 ... */
    if (msg_id == msg_lin)
    {
      if (msg_lin >= 0)
        close(msg_lin);
      msg_lin = -1;
      pid_lin = -1;                                        /* 04.06.14 */
    }
    else
    {
      if (msg_net >= 0)
        close(msg_net);
      msg_net = -1;
      pid_net = -1;                                        /* 04.06.14 */
    }
#endif                                                      /* ... 09.02.03 */
#if defined(PTHREAD_INTLIB)
#if defined(LINUX)                                          /* 06.06.03 */
     if (flag)
       LocalThPid = 0;
     else
       NetThPid = 0;
#endif
     pthread_mutex_unlock(&IntlibMutex);
#if defined(OS2000)                                         /* 08.12.04 .. */
     if ( ( LocOk != 0 && NetOk != 0 ) )
     {
       RunUninitThread();
     }
#endif                                                      /* .. 08.12.04 */
#endif /* PTHREAD_INTLIB */                                 /* ... 22.08.02 */
#if !( ((defined(QNX) && !defined(QNXNATIVE)) || defined(OS2000)) && defined(PTHREAD_INTLIB) ) /* 25.07.07 17.03.10 */
#if !defined(PTHREAD_INTLIB)                                /* 11.04.13 */
     return;
#else                                                       /* 11.04.13 ... */
#if defined(QNXNATIVE)                                     /* 30.05.14 ... */
     if (flag)
     {
       if (NetThreadID == 0)
         alarm(0);
     }
     else
     {
       if (LocalThreadID = 0)
         alarm(0);
     }
#endif /* QNXNATIVE */                                     /* ... 30.05.14 */
     return NULL;
#endif                                                      /* ... 11.04.13 */
#endif     
     }
   else
     RcvMsg.CBL.SysErr = errno;
   errno = 0;
   RcvMsg.CBL.CodErr = ERRREADMSG;
#if !defined(PTHREAD_INTLIB)                                /* 22.08.02 ... */
   return;                                                 /* 28.06.98 */
#else
   continue;                             /* Continue work of read thread */
#endif                                                      /* ... 22.08.02 */
   }
#if !defined(QNXNATIVE)                                     /* 14.12.09 */
 ReceiveData(&RcvMsg, NULL, 0, NULL, NULL);
#else                                                       /* 14.12.09 ... */
 {
#if defined(PTHREAD_INTLIB)
   struct _pulse* Pulse = (struct _pulse*)&RcvMsg;
   
   if ( Pulse->code != _PULSE_CODE_MINAVAIL+3)              /* 17.03.10 ... */
   {
     UninitLinter(Server->Pid);                             /* 09.07.10 */
   }
   else                                                     /* ... 17.03.10 */
   if ( Pulse->value.sival_int == 0  )
#else /* PTHREAD_INTLIB */
   if ( Server->Shm != NULL && Server->Shm != MAP_FAILED )
#endif /* PTHREAD_INTLIB */
   {
     L_ULONG WPos = ((L_ULONG*)(Server->Shm + SRV_HDR_OFFSET))[0]; /* 11.11.10 */
     L_ULONG RPos = ((L_ULONG*)(Server->Shm + SRV_HDR_OFFSET))[1]; /* 11.11.10 */

     if ( WPos != RPos )
     {
       L_ULONG TRPos = RPos & SPACE_MASK;                   /* 11.11.10 */
       L_ULONG MsgLen = ((IPC_TYP*)(Server->Shm + SRV_DATA_OFFSET + TRPos))[0]; /* 11.11.10 */
       
       MsgLen = (MsgLen + 3) / 4 * 4;
       
       if ( TRPos + MsgLen <= SPACE_SIZE )                  /* 11.11.10 */
       {
         memcpy(&RcvMsg, Server->Shm + SRV_DATA_OFFSET + TRPos, MsgLen); /* 11.11.10 */
       }
       else
       {
         L_ULONG LenToEndOfBuf = SPACE_SIZE - TRPos;        /* 11.11.10 */
         
         memcpy(&RcvMsg, Server->Shm + SRV_DATA_OFFSET + TRPos, LenToEndOfBuf); /* 11.11.10 */
         memcpy(((char*)&RcvMsg)+LenToEndOfBuf, Server->Shm + SRV_DATA_OFFSET,
                  MsgLen - LenToEndOfBuf);
       }
       ((L_ULONG*)(Server->Shm + SRV_HDR_OFFSET))[1] = (RPos + MsgLen) /* 11.11.10 */
#if defined(QNXNATIVECOMPAT)
                                                              & SPACE_MASK
#endif
                                                                        ;
       ReceiveData( &RcvMsg, NULL, 0, NULL, NULL);
     }
   }
 }
#endif /* QNXNATIVE */                                      /* ... 14.12.09 */
#if !defined(VXWORKS) && !defined(QNX) && defined(SOCKETS) && !defined(NO_ASYNC) /* 05.11.00 */  /* 28.03.00 *//* 13.04.00 */
 goto try_next;                                            /* 01.03.00 */
#endif
#if (defined(QNX) && !defined(QNXNATIVE)) || defined(VXWORKS)   /* 14.12.09 27.02.04 */
 goto l5;
#endif

#if defined(PTHREAD_INTLIB)                                 /* 22.08.02 */
 }
#endif

#endif
#ifdef WIN32
 DWORD  dwTmp;                                              /* 03.12.98 */
 TMSG RcvMsg;
/* 16.05.03 ...*/
 HANDLE wait[2];
 DWORD  dwRc;
/*... 16.05.03 */

 do
 {
/* 08.01.03 ...*/
   switch(gExchType)
   {
     case slot_exch:
       dwTmp = 0;                                          /* 19.06.08 */
       if (!(dwRc = ReadFile(gpInput->hMailslotClient, &RcvMsg, LENMSG, &dwTmp, NULL)))
       {
         RcvMsg.CBL.SysErr = GetLastError();
         RcvMsg.CBL.CodErr = ERRREADMSG;
         dwTmp = 0;                                        /* 19.06.08 */
       } 
/* 16.05.03 ...*/
       if (!dwTmp) /* break by timeout=0 from UninitInputObjects */
       {
         bThrdStopped = TRUE;                              /* 21.05.03 */
         return;
       }
/*... 16.05.03 */
       ReceiveData(&RcvMsg, NULL, 0, NULL, NULL);
       break;
     case memory_exch:
/* 16.05.03 ...*/
       wait[0] = gpInput->hEventClientAdd;
       wait[1] = hEventStopAstThrd;
       dwRc = WaitForMultipleObjects(2, wait, FALSE, INFINITE);
       switch(dwRc)
       {
         case WAIT_OBJECT_0+0:
           ReceiveData(gpInput->lpvMapViewClient, NULL, 0, NULL, NULL);
           break;
         case WAIT_OBJECT_0+1:
           return;
         case WAIT_FAILED:
           RcvMsg.CBL.SysErr = GetLastError();
           RcvMsg.CBL.CodErr = ERRREADMSG;
           break;
       }
/*... 16.05.03 */
       TRACE_EVENT("Set event in ReadMsgFromServer", gpInput->hEventClientReady);  /* 01.08.03 */
       SetEvent(gpInput->hEventClientReady);
       break;
   } /* switch gExchType */
/*... 08.01.03 */
   } while(hAstThrd);
#endif
}

/* 10.02.97 */
#ifndef VXWORKS                                          /* 11.01.01 */
static                                                   /* 08.12.98 */
# if defined(UNIX) || defined(QNX) || defined(VMS)       /* 30.07.12 22.08.02 */
void CheckServer(int i)
# else
#  if defined(OS9000)
int CheckServer(int *Srv, int Time)
#  elif defined(WIN32)
VOID WINAPI CheckServer(LPVOID lpvArg)
#  else
L_BOOL CheckServer(int i)
#  endif
# endif
{
#ifdef WIN32
/* 08.01.03 ...*/
#ifndef RTOS32
 HANDLE   wait[4];                                       /* 16.05.03 */
#else
 HANDLE   wait[3];                                       /* 16.05.03 */
#endif

 DWORD    dwRc, dwWaitCount;
 DWORD    dwAgentDie, dwLinterDie;
 DWORD    dwStopThrd;                                    /* 16.05.03 */

 while(TRUE)
 {
   WaitForSingleObject(hMutexExchLock, INFINITE);          /* 22.01.03 */
   dwWaitCount = 0;
   dwAgentDie = dwLinterDie = -1;
   if(gpOutput[0])
   {
     wait[dwWaitCount] = gpOutput[0]->hServer;  /* agent */
     dwAgentDie = WAIT_OBJECT_0 + dwWaitCount;
     dwWaitCount++;
   }
   if(gpOutput[1])
   {
     wait[dwWaitCount] = gpOutput[1]->hServer;  /* local linter */
     dwLinterDie = WAIT_OBJECT_0 + dwWaitCount;
     dwWaitCount++;
   }
/* 16.05.03 ...*/
   wait[dwWaitCount] = hEventStopChkThrd; /* Stop thread */
   dwStopThrd = WAIT_OBJECT_0 + dwWaitCount;
   dwWaitCount++;
/*... 16.05.03 */
   wait[dwWaitCount++] = gpInput->hEventSyncChkThrd;
   ReleaseMutex(hMutexExchLock);                           /* 22.01.03 */
   dwRc = WaitForMultipleObjects(dwWaitCount, wait, FALSE, INFINITE);
   switch(dwRc)
   {
     case WAIT_OBJECT_0+0:
     case WAIT_OBJECT_0+1:
     case WAIT_OBJECT_0+2:
       if(dwRc == dwAgentDie)
         UninitOutputObjects(&gpOutput[0]);
       else if(dwRc == dwLinterDie)
         UninitOutputObjects(&gpOutput[1]);
       else if(dwRc == dwStopThrd)                         /* 16.05.03 */
         goto l_end;
       break;
   } /* Switch WaitForMultipleObjects */
 } /* main loop */
/*... 08.01.03 */
l_end:
  bThrdStopped = TRUE;                                      /* 21.05.03 */
  return;                                                   /* 16.05.03 */
#endif
#if (defined(QNX) || (defined(UNIX) && !defined(SOCKETS))) && !defined(PTHREAD_INTLIB)    /* 27.10.99 02.03.99 22.08.02 */
/* 12.02.97 */
 int j;                                                     /* 22.08.02 ... */

#if defined(QNXNATIVE)                                      /* 15.07.10 ... */
 struct _pulse Pulse;
 
 if ( Linter.Pid )
 {
   if ( MsgSendPulse(Linter.ClntConnId, Linter.Prior,
                            _PULSE_CODE_MINAVAIL + 7, 0xFF00) == 0 )
   {
    if ( (MsgReceivePulse(Linter.ClntChId, &Pulse, sizeof(Pulse), NULL ) == -1) ||
        ( Pulse.code != _PULSE_CODE_MINAVAIL + 7 ) )
    {
      UninitLinter(Linter.Pid);
      ReadMsgFromServer(1);
    }
   }
 }

 if ( Dbc.Pid )
 {
   if ( MsgSendPulse(Dbc.ClntConnId, Dbc.Prior,
                            _PULSE_CODE_MINAVAIL + 7, 0xFF00) == 0 )
   {
    if ( (MsgReceivePulse(Dbc.ClntChId, &Pulse, sizeof(Pulse), NULL ) == -1) ||
        ( Pulse.code != _PULSE_CODE_MINAVAIL + 7 ) )
    {
      UninitLinter(Dbc.Pid);
      ReadMsgFromServer(0);
    }
   }
 }
#endif /* QNXNATIVE */                                      /* ... 15.07.10 */
 for(i = 0; i < AstListSize; i++)
  for ( j = 0; j < ASTBLOCKSIZE; j++ )
  {
    TASTMSG* Ast = AstList[i] + j;

    if (Ast->Actual && kill(Ast->Server, 0) != 0 && errno != EPERM )
    {
#if defined(QNXNATIVE)
      L_LONG IsLocal = (Ast->Server == Linter.Pid?1:0);     /* 14.12.09 */
#endif
#ifdef QNX                                                  /* 29.12.04 .. */
      UninitLinter(Ast->Server);                         /* 27.10.99 */
#else
      *(Ast->msg_id) = -1;
#endif
#if !defined(QNXNATIVE)
      ReadMsgFromServer((MSGID_TYPE)-1);                    /* .. 29.12.04 */
#else
      ReadMsgFromServer(IsLocal);                           /* 14.12.09 */
#endif
    }
  }                                                         /* ... 22.08.02 */

#if !defined(QNXNATIVE)                                     /* 14.12.09 */
 if (alrm_handler == NULL)                                    /* 11.01.99 ... */
   alarm(TIMEOUT);
 else
   alrm_handler(SIGALRM);
 signal(SIGALRM, CheckServer);                                /* ... 11.01.99 */
#endif /* QNXNATIVE */
/* 12.02.97 */
#endif
#ifdef OS9000
 int        ri;
 L_BYTE       rflag = 0;
 char       rbuf[CNT_FOR_INSPEC];
 Pr_desc    prd  = (Pr_desc)rbuf;
 u_int32    size = CNT_FOR_INSPEC;
 error_code err;

 if ((err = _os_gprdsc(*Srv, prd, &size)))
   {
   if (Time)
     rflag = (err == EOS_IPRCID);
   }
 else
   {
   if (prd->p_state & (DEADBT | CONDEMNBT))                     /* 29.02.00 */
     rflag = true;
   else
     {
     if (Time)
#ifdef OS9
       rflag = (prd->_timbeg != Time);
     else
       return(prd->_timbeg);
#else
       rflag = (prd->p_timbeg != Time);
     else
       return(prd->p_timbeg);
#endif
     }
   }
 if (rflag)
   {
   if ( *Srv == pid_lin )
     LinChan = &LinOutChan;
   else
     if ( *Srv == pid_net )                                /* 10.06.00 */
       LinChan = &NetOutChan;
     else
       return L_TFALSE;
   while ((ri = _gs_rdy(*LinChan)) > 0)
     {
     if (ri > CNT_FOR_INSPEC)
       ri = CNT_FOR_INSPEC;
     read(*LinChan, rbuf, ri);
     }
   if (*LinChan)                                        /* 10.06.00 */
     close(*LinChan);
   *LinChan = 0;                                /* 05.06.00 */
   *Srv = 0;
   return(L_TFALSE);
   }
 return (L_TTRUE);
#endif
#ifdef VMS                                                  /* 30.07.12 ... */
  int j;
  int MbxCounter;
  L_WORD *MailBoxArray[2];
  L_WORD *SecondMailBox;
  int Status;
  IOSB Iosb;
  
  MailBoxArray[0] = &LinMailbox;
  MailBoxArray[1] = &NetMailbox;
  
  for ( MbxCounter = 0; MbxCounter < 2; MbxCounter++ )
  {
    L_WORD* OutMailBox = MailBoxArray[MbxCounter];

    if ( *OutMailBox == 0 )
      continue;

    Status = SYS$QIOW(EFN$C_ENF, /* No event */
                 *OutMailBox, /* Channel to the mailbox */
                 IO$_SENSEMODE|IO$M_READERCHECK, /* Checking reader */
                 &Iosb, /* Status block */
                 0, 0,
                 NULL, /* Data */
                 0, /* Data len */
                 0,0,0,0);
    if ( Status == SS$_NORMAL)
    {
      Status = Iosb.iosb$w_status;
    }

    if ( Status == SS$_NORMAL)
      continue;

    SYS$DASSGN(*OutMailBox);
    for(i = 0; i < AstListSize; i++)
    {
      for ( j = 0; j < ASTBLOCKSIZE; j++ )
      {
        TASTMSG* Ast = AstList[i] + j;

        if (Ast->Actual && Ast->Mailbox == *OutMailBox)
        {
          memset((char *)(GlRcvMsg.Buf.Byte), 0, 8);
          ((TAILMSG *)(&GlRcvMsg.Buf.Byte[8]))->AstInd = Ast->AstIndex;
          GlRcvMsg.CBL.CodErr = ERRREADMSG;
          GlRcvMsg.CBL.SysErr = Status;
          ReadMsgFromServer(&GlRcvMsg);
        }/* If */
      }
    }

    *OutMailBox = 0;
    break;
  } /* For net and local MBX */

  if ( MbxCounter < 2 ) /* Some mbx was closed */
  {
    if ( LinMailbox == 0 && NetMailbox == 0 )
    {
      UninitLinterClient();
      return;
    }
  }

  i = SYS$SETIMR(EFN$C_ENF, /* Event */
              &RelativeTime, /* Interval */
              CheckServer, /* AST */
              20561, /* Timer ID and AST arg */
              0);
  return;                                                   /* ... 30.07.12 */
#endif
}
/* 10.02.97 */
#endif


void UninitLinterClient(void)
{
 int i;                                                  /* 22.08.02 */
 THREAD_RELEASE(g_DefCpSync);           /* 22.05.03 */    /* 03.06.03 */
#if defined(VMS)                                            /* 30.07.12 ... */
 if (LinMailbox != 0)
 {
  SYS$DASSGN(LinMailbox);
  LinMailbox = 0;
 }
 if (NetMailbox != 0)
 {
  SYS$DASSGN(NetMailbox);
  LinMailbox = 0;
 }
 if (InputMailbox != 0)
 {
  SYS$DASSGN(InputMailbox);
  InputMailbox = 0;
  MyPid = 0;
 }
 SYS$CANTIM(20561, 0);
#endif /* VMS */                                            /* ... 30.07.12 */
#ifdef WIN32

/* 08.01.03 ...*/
 UninitInputObjects();

 UninitOutputObjects(&gpOutput[local_ker]);
 UninitOutputObjects(&gpOutput[remote_ker]);
/*... 08.01.03 */

#endif
#ifdef UNIX                                                 /* 12.11.98 ... */
#ifdef SOCKETS
 if (msg_lin >= 0)
 {
   close(msg_lin);
   msg_lin = -1;                                            /* 05.07.02 */
 }
 if (msg_net >= 0)
 {
   close(msg_net);
   msg_net = -1;                                            /* 05.07.02 */
 }
#else
 msg_lin = -1;                                              /* 08.02.07 */
 msg_net = -1;                                              /* 08.02.07 */
#endif
 my_pid = 0;                                                /* 08.02.07 29.04.99 */

#if defined(PTHREAD_INTLIB)                                 /* 22.08.02 ... */
 {
   ASTCALLDESC* AstCallDesc = AstCallDescFirst.Next;
   while ( AstCallDesc != &AstCallDescFirst )
   {
     ASTCALLDESC* Tmp = AstCallDesc;

#if !defined(ANDROID)                                       /* 09.06.12 */
     pthread_cancel(Tmp->ThreadID);
#else                                                       /* 09.06.12 ... */
     /* TODO: replace pthread_cancel() call in a proper way. */
#endif                                                      /* ... 09.06.12 */
     pthread_cond_destroy(&Tmp->LCond.Cond);
     pthread_mutex_destroy(&Tmp->Mutex);
     AstCallDesc = Tmp->Next;
     free(Tmp);
   }
   if ( AstCallDescFirst.ThreadID )                         /* 07.06.07 .. */
   {
#if !defined(ANDROID)                                       /* 09.06.12 */
     pthread_cancel(AstCallDescFirst.ThreadID);
#else                                                       /* 09.06.12 ... */
     /* TODO: replace pthread_cancel() call in a proper way. */
#endif                                                      /* ... 09.06.12 */
     pthread_cond_init(&AstCallDescFirst.LCond.Cond, NULL);
     pthread_mutex_init(&AstCallDescFirst.Mutex, NULL);
     AstCallDescFirst.ThreadID = (pthread_t)0;
   }                                                        /* .. 07.06.07 */
 }
# if defined(LINUX)
 LocalThPid = 0;                                            /* 06.06.03 */
 NetThPid = 0;
# endif
 IntlibIsInit = 0;
#endif   /* PTHREAD_INTLIB */                               /* ... 22.08.02 */

 pid_lin = pid_net = 0;
#endif                                                      /* ... 12.11.98 */
 for ( i = 0; i < AstListSize; i++)                         /* 22.08.02 ... */
 {
   free(AstList[i]);
   AstList[i] = NULL;
 }                                                          /* ... 22.08.02 */
 AstListSize = 0;
#if defined(VXWORKS) || defined(QNX)                        /* 08.02.07 15.05.01 */
 UninitLinterAll();                                         /* 03.04.08e */
#endif
#if defined(QNX) && !defined(QNXNATIVE)                     /* 14.12.09 */
 my_pid = 0;                                                /* 08.02.07 */
#endif
}
#endif /* not INTER_MSDOS && INTER_DPMI */

#ifdef INTER_DPMI
#ifndef __WATCOMC__
#include "lowmem.h"
#else
#define copyToLow(sel, off, src, len)   memcpy ((void *)(sel<<4), src, len)
#define copyFromLow(dst, sel, off, len) memcpy (dst, (void *)(sel<<4), len)
#endif

static unsigned int DosAllocRealSeg(unsigned long size,
                                    unsigned int *seg,
                                    unsigned int *sel)
{
#ifdef __FLAT__
 regs.x.eax = 0x0100;
 regs.x.ebx = (int)(size+15)>>4;
 int386(0x31, &regs, &regs);
 *seg = regs.x.eax;
 *sel = regs.x.edx;
#else
 regs.x.ax = 0x0100;
 regs.x.bx = (short)((size+15)>>4);
 int86(0x31, &regs, &regs);
 *seg=regs.x.ax;
 *sel=regs.x.dx;
#endif
 return (regs.x.cflag);
}

static unsigned int DosFreeRealSeg(unsigned int sel)
{
#ifdef __FLAT__
 regs.x.eax = 0x0101;
 regs.x.edx = sel;
 int386(0x31, &regs, &regs);
#else
 regs.x.ax = 0x0101;
 regs.x.dx = sel;
 int86(0x31, &regs, &regs);
#endif
 return (regs.x.cflag);
}

static int InterDPMI(char *Message, unsigned int MessageLen)
{
 unsigned int segment;
 unsigned int selector;
 unsigned int DpmiError;

 DpmiError=0;
 if (DosAllocRealSeg( (unsigned long)MessageLen, &segment, &selector))
   return(EALLSEG);
#ifdef __WATCOMC__
 copyToLow(segment, 0, Message, MessageLen);
#else
 copyToLow(selector, 0, Message, MessageLen);
#endif
 memset(&regs, 0, sizeof(regs));
 regs.w.ax = 0x0201;
 regs.h.bl = 0x98;
 regs.w.cx = segment;
#ifdef __FLAT__
 regs.x.edx = 0;
 int386(0x31, &regs, &regs);
#else
 regs.x.dx = 0;
 int86(0x31, &regs, &regs);
#endif
 if (regs.x.cflag & 1)
   {
   DpmiError = ESETPNTR;
   goto ERROR;
   }
#ifdef __FLAT__
 int386(0x96, &regs,&regs);
#else
 int86(0x96, &regs, &regs);
#endif
 if (regs.x.cflag & 1)
   {
   DpmiError=EINT;
   goto ERROR;
   }
#ifdef __WATCOMC__
 copyFromLow(Message, segment, 0, MessageLen);
#else
 copyFromLow(Message, selector, 0, MessageLen);
#endif
ERROR:
 if (DosFreeRealSeg( selector))
   DpmiError = EFRSEG;
 return(DpmiError);
}
#endif /* INTER_DPMI */

#if defined(UNIX) || (defined(QNX) && !defined(QNXNATIVE)) || defined(VXWORKS) /* 14.12.09 02.04.98 */ /* 11.01.01 */
#if !defined(PTHREAD_INTLIB)                               /* 22.08.02 */
static void AstReceive(int sig)                            /* 05.02.99 */
{
  int saved = errno;                                       /* 09.12.08 */
#if defined(QNX) || defined(VXWORKS)                       /* 11.01.01 */
  struct sigevent se;

  se.sigev_signo = SIGUSR1;
#if defined OS2000                                         /* 03.05.01 */
  se.sigev_notify = SIGEV_SIGTHR;
  se.sigev_thread = /*getpid()*/pid_user;                  /* 10.03.04 04.06.01 */
#elif defined(QNX6)
  se.sigev_notify = SIGEV_SIGNAL;                          /* 10.07.02 */
#endif
  mq_notify(msg_user,&se);
#endif
#ifdef UNIX
 if (msg_lin >= 0)
   ReadMsgFromServer(msg_lin);
 if (msg_net >= 0)
   ReadMsgFromServer(msg_net);
#else
   ReadMsgFromServer(msg_user);
#endif
  errno = saved;                                           /* 09.12.08 */
}
#endif /* PTHREAD_INTLIB */                                 /* 22.08.02 */
#endif /* UNIX */

#if defined(QNXNATIVE) && !defined(PTHREAD_INTLIB)          /* 14.12.09 ... */
static void AstReceive(int Signal, siginfo_t* SigInfo, void* Arg)
{
  int saved = errno;

  if (SigInfo->si_value.sival_int == 0)
     ReadMsgFromServer(1); /* Linter message */
  else
     ReadMsgFromServer(0);
  errno = saved;
}
#endif                                                      /* ... 14.12.09 */

static void ZAPBUF(TCBL *CBL, L_CHAR *Buf, L_LONG *Len, L_LONG Err) /* 30.03.98 */
{
 if (Buf)
   {
   *Len = strlen(Buf);
   while(*Len && Buf[(*Len)-1] != ';')
     (*Len)--;
   if (*Len == 0)
     CBL->CodErr = Err;
   }
 else
   CBL->CodErr = NULLPOINTER;
}

static void ZAPKORTEG(TCBL    *CBL,
                      HPCHAR   Korteg,
                      L_LONG *Len)
{
 if (!Korteg && CBL->LnBufRow)
   CBL->CodErr = NULLPOINTER;
 else
   *Len = CBL->LnBufRow;
}

static void ZAPKORTEGB(TCBL    *CBL,                       /* 23.05.00 ... */
                       HPCHAR   Korteg,
                       L_LONG *Len)
{
 if (!Korteg && CBL->LnBufRow)
   CBL->CodErr = NULLPOINTER;
 else
   {
   *Len = CBL->LnBufRow;
   while (*Len && !Korteg[*Len - 1])
     (*Len)--;                                             /* 31.07.00 */
   if ( ( *Len & 1 ) && ( *Len < CBL->LnBufRow ) )         /* 31.08.00 */
     (*Len)++;
   }
}                                                          /* ... 23.05.00 */

#if defined(QNX) || defined(VXWORKS)                       /* 02.04.98 */ /* 11.01.01 */
#ifdef VXWORKS                                             /* 11.01.01 */
#ifndef OS2000
void TaskExit(WIND_TCB *pTcb)
{
  int tcb, i, j;

  tcb = (int)pTcb;
  if ( (tcb == pid_lin) || (tcb == pid_net) )
    for (i = 0; i < AstListSize; i++)
     for ( j = 0; j < ASTBLOCKSIZE; j++)
     {
       TASTMSG* AstTmp = AstList[i] + j;

       if (AstTmp->Actual && (AstTmp->Server == tcb))
       {
         UninitLinter(tcb);
         AstTmp->Status = 0;
         kill(pid_user, SIGUSR1);
       }
     }
  else if ( tcb == pid_user )
    {
    taskDeleteHookDelete((FUNCPTR)TaskExit);
    if ( msg_user != -1 )                                 /* 04.06.01 */
      i = mq_notify(msg_user,NULL);
    UninitLinterAll();
    }
  else
    return;
}
#endif
#endif
void UninitLinter(int Pid)                                 /* 27.10.99 ... */
{
#if !defined(QNXNATIVE)                                     /* 14.12.09 */
  if (Pid != -1)
    {
    if (Pid == pid_lin)
      {
      if (sh_linter != -1)
        close(sh_linter);
      sh_linter = -1;
      if (msg_lin != (mqd_t)-1)
        mq_close(msg_lin);
      msg_lin = (MSGID_TYPE)-1;                             /* 15.09.04 */
      pid_lin = -1;
      }
    if (Pid == pid_net)
      {
      if (sh_net != -1)
        close(sh_net);
      sh_net = -1;
      if (msg_net != (MSGID_TYPE)-1)                        /* 15.09.04 */
        mq_close(msg_net);
      msg_net = (MSGID_TYPE)-1;                             /* 15.09.04 */
      pid_net = -1;
      }
    }
#else /* QNXNATIVE */                                       /* 14.12.09 ... */
  if ( Pid == Linter.Pid && Pid != 0 )
  {
    if ( Linter.Shm != MAP_FAILED)
    {
      munmap(Linter.Shm, SHMSIZE);
      Linter.Shm = MAP_FAILED;
    }
    if ( Linter.ShmId != -1 )
    {
      close(Linter.ShmId);
      Linter.ShmId = -1;
    }
    if ( Linter.ConnId != -1 )
    {
      ConnectDetach(Linter.ConnId);
      Linter.ConnId = -1;
    }
    if ( Linter.ClntConnId != -1 )
    {
      ConnectDetach(Linter.ClntConnId);
      Linter.ClntConnId = -1;
    }
    if ( Linter.ClntChId != -1 )
    {
      ChannelDestroy(Linter.ClntChId);
      Linter.ClntChId = -1;
    }
    Linter.Pid = 0;
#if defined(PTHREAD_INTLIB)
    pthread_kill(LocalThreadID, SIGALRM);                  /* 30.05.14 */
    LocalThreadID = 0;
#endif
    return;
  }
  
  if ( Pid == Dbc.Pid && Pid != 0 )
  {
    if ( Dbc.Shm != MAP_FAILED)
    {
      munmap(Dbc.Shm, SHMSIZE);
      Dbc.Shm = MAP_FAILED;
    }
    if ( Dbc.ShmId != -1 )
    {
      close(Dbc.ShmId);
      Dbc.ShmId = -1;
    }
    if ( Dbc.ConnId != -1 )
    {
      ConnectDetach(Dbc.ConnId);
      Dbc.ConnId = -1;
    }
    if ( Dbc.ClntConnId != -1 )
    {
      ConnectDetach(Dbc.ClntConnId);
      Dbc.ClntConnId = -1;
    }
    if ( Dbc.ClntChId != -1 )
    {
      ChannelDestroy(Dbc.ClntChId);
      Dbc.ClntChId = -1;
    }
    Dbc.Pid = 0;
#if defined(PTHREAD_INTLIB)
    pthread_kill(NetThreadID, SIGALRM);                    /* 30.05.14 */
    NetThreadID = 0;
#endif
    return;
  }
#endif                                                      /* ... 14.12.09 */
}

void UninitLinterAll(void)
{
#if !defined(QNXNATIVE)                                     /* 14.12.09 */
  if (msg_user != (MSGID_TYPE)-1)                           /* 15.09.04 */
  {
    mq_close(msg_user);
    mq_unlink(ThisMbxKey);                                  /* 08.02.07 */
    msg_user = (MSGID_TYPE)-1;                              /* 15.09.04 */
  }
#ifdef VXWORKS                                          /* 11.01.01 */
  pthread_cancel(pid_user);                             /* 28.05.10 */
  pid_user = -1;                                        /* 07.04.01 */
#endif
  UninitLinter(pid_lin);                                /* 27.10.99 */
  UninitLinter(pid_net);
#else /* QNXNATIVE */                                   /* 14.12.09 ... */
  Client.Pid = 0;
#if !defined(PTHREAD_INTLIB)                                /* 27.04.12 ... */
  if ( Client.Timer != (timer_t)0 )
  {
    timer_delete(Client.Timer);
    Client.Timer = (timer_t)0;
  }
#endif /* PTHREAD_INTLIB */                                 /* ... 27.04.12 */
  UninitLinter(Linter.Pid);
  UninitLinter(Dbc.Pid);                                /* ... 14.12.09 */
#endif /* QNXNATIVE */
}                                                       /* ... 27.10.99 */
#endif  /* QNX VXWORKS */

#if defined(PTHREAD_INTLIB)                                 /* 22.08.02 ... */
static int RunReaderThread(int flag)
{
  pthread_t TmpThread;
  pthread_attr_t Attr;                                         /* 04.11.02 */
#if defined(LINUX)
  pid_t*    ReaderPid = flag?&LocalThPid:&NetThPid;
#endif

  pthread_attr_init(&Attr);                                    /* 04.11.02 */
  pthread_attr_setstacksize(&Attr, IntlibStackSize);        /* 10.05.12 26.04.10 */

  if ( pthread_create(&TmpThread, &Attr, ReadMsgFromServer, (void*)(L_ADRTYPE)flag) ) /* 10.05.12 */
    return -1;

  if ( pthread_detach(TmpThread) )
  {
#if !defined(ANDROID)                                       /* 09.06.12 */
    pthread_cancel(TmpThread);
#else                                                       /* 09.06.12 ... */
    /* TODO: replace pthread_cancel() call in a proper way. */
#endif                                                      /* ... 09.06.12 */
    return -1;
  }
#if defined(LINUX)
  pthread_mutex_lock(&InternalMutex);

  if ( *ReaderPid == 0 )                                    /* 06.06.03 */
    pthread_cond_wait(&RunReaderCond.Cond, &InternalMutex);
  pthread_mutex_unlock(&InternalMutex);
#endif
#if defined(OS2000)                                         /* 08.12.04 */
#if defined(MTOS2000)                                       /* 29.11.05 */
  ((struct PidMsg*)flag)->Pid = TmpThread;
#else
  return (int)TmpThread;                                    /* 29.11.05 */
#endif
#endif
  return 0;
}
#endif  /* PTHREAD_INTLIB */

#if (defined(QNX) && !defined(QNXNATIVE)) || defined(VXWORKS) /* 14.12.09 */
static mqd_t CreateUserQueue()
{
  struct mq_attr  attr, attr_old;
  struct sigevent se;
  mqd_t msg_user;

  memset(&attr,0,sizeof(attr));
  attr.mq_maxmsg  = 3;
  attr.mq_msgsize = MAXMSGSIZE;
  attr.mq_flags   = 0;

#if defined(VXWORKS)
#if defined(MTOS2000) || !defined(OS2000)
  sprintf(ThisMbxKey, "%s_%x", LinterMbxKey, getpid());     /* 08.02.07 */
#else
  sprintf(ThisMbxKey, "%s_%x", LinterMbxKey, pid_user);     /* 08.02.07 */
#endif
#endif

  if ((msg_user = mq_open(ThisMbxKey, O_CREAT | O_RDWR ,0664,&attr)) == (mqd_t) -1) /* 08.02.07 */
    return (mqd_t)-1;

#if !defined(PTHREAD_INTLIB)
  mq_getattr(msg_user,&attr);
#if defined(VXWORKS) || defined(QNX6)
  attr.mq_flags = O_NONBLOCK;
#else
  attr.mq_flags = MQ_NONBLOCK;
#endif
  mq_setattr(msg_user,&attr,&attr_old);
#endif /* PTHREAD_INTLIB */

#if defined(VXWORKS) && !defined(OS2000)
  se.sigev_signo = SIGUSR1;
  if ( signal(SIGUSR1,AstReceive) == SIG_ERR || (mq_notify(msg_user,&se) < 0) )
  {
    mq_close(msg_user);
    mq_unlink(ThisMbxKey);                                 /* 18.06.14 */
    return (mqd_t)-1;
  }
#else   /* OS2000 QNX6*/                                            /* 03.05.01 */
#if !defined(PTHREAD_INTLIB)
  {
    struct sigaction action;

    sigemptyset(&newmask);                                   /* 17.01.07b 22.08.02 ... */
    sigaddset(&newmask, SIGUSR1);
#if defined(OS2000)
    pthread_sigmask(SIG_UNBLOCK, &newmask, NULL);
#else /* QNX */
    sigprocmask(SIG_UNBLOCK, &newmask, NULL);
#endif                                                      /* ... 17.01.07b 22.08.02 */

    memset(&action, 0, sizeof(struct sigaction));
    action.sa_handler = AstReceive;

#if defined(OS2000)
    se.sigev_notify = SIGEV_SIGTHR;
    se.sigev_thread = pthread_self();
#elif defined(QNX6)                                         /* 24.03.03 */
    se.sigev_notify = SIGEV_SIGNAL;
#endif
    se.sigev_signo  = SIGUSR1;
    if (sigaction(SIGUSR1, &action, NULL) < 0 || mq_notify(msg_user, &se) < 0)
    {
      mq_close(msg_user);
      mq_unlink(ThisMbxKey);                                         /* 18.06.14 */
      return (mqd_t)-1;
    }
  }
#endif /* PTHREAD_INTLIB */
#endif
  return msg_user;
}
#endif /* QNX VXWORKS */

#if defined(QNX) || defined(VXWORKS)                        /* 14.12.09 */
static int TestServerQueue(int IsLocalQueue)
{
#if !defined(QNXNATIVE)                                     /* 14.12.09 */
  mqd_t* Queue;
  char* str;
#if defined(QNX)                                            /* 27.05.08 ... */
  int*  ShmId;
  int*  Pid;
  void** Shm;
#endif                                                      /* ... 27.05.08 */

  if (IsLocalQueue)
  {
#if defined(VXWORKS)
    if ( msg_lin != (MSGID_TYPE)-1 )                        /* 15.09.04 27.02.04 */
      return 0;
#endif
#if defined(QNX)                                            /* 27.05.08 ... */
    Pid   = &pid_lin;
    ShmId = &sh_linter;
    Shm = &shm_linter;
#endif                                                      /* .. 27.05.08 */
    Queue = &msg_lin;
    str = LinterMbxKey;                                      /* 08.02.07 */
  }
  else
  {
#if defined(VXWORKS)
    if ( msg_net != (MSGID_TYPE)-1)                         /* 15.09.04 */
      return 0;
#endif
#if defined(QNX)                                            /* 27.05.08 ... */
    Pid   = &pid_net;
    ShmId = &sh_net;
    Shm = &shm_net;
#endif                                                      /* ... 27.05.08 */
    Queue = &msg_net;
    str = NetMbxKey;                                        /* 08.02.07 */
  }

#if defined(QNX)                                            /* 27.05.08 ... */
  if ( *Pid != 0 &&
         (kill(*Pid, 0) == 0 || errno == EPERM) && *Queue != -1)
    return 0;
  else
  {
    if ( *Queue != -1 )
    {
      mq_close(*Queue);
      *Queue = -1;
    }

    if ((*ShmId = shm_open(str, O_RDWR, 0666)) == -1)
      return -1;
      
#ifdef QNX6                                            /* 29.07.08 */
    if ( ftruncate(*ShmId, PAGESIZE) == -1 )           /* 25.07.07 */
    {
      shm_close(*ShmId);
      return -1;
    }
#endif
      
    if ((*Shm =
        mmap(NULL, PAGESIZE, PROT_READ, MAP_SHARED, *ShmId, 0)) == (void*)-1)
      {
        close(sh_linter);
        return -1;
      }

      *Pid = *(L_LONG *)(*Shm);
      munmap(*Shm, PAGESIZE);
      close(*ShmId);
      if ( kill(*Pid, 0) != 0 && errno != EPERM )
        return -1;
  }
#endif                                                      /* ... 27.05.08 */

  if ( (*Queue = mq_open(str,O_WRONLY,0664,0)) == (mqd_t)-1 )  /* 12.03.04 */
    return -1;
  return 0;

#else /* QNXNATIVE */                                       /* 14.12.09 ... */
  IntlibServer_t* Server = IsLocalQueue?&Linter:&Dbc;
  int ShmId;
  void* Shm;
  char* Mbx;
  int   RemoteChId;
  L_LONG Pid;
  int ConnId;
  int RemoteConnId;
  int RcvId;
  int ClntConnId;
  int ClntChId;
  char Address[256];
  mode_t Mask;
  
  if ( Server->Pid != 0 )
  {
    if (kill( Server->Pid , 0) != 0 && errno != EPERM)
    {
      UninitLinter(Server->Pid); /* And try reconnect */
    }
    else
      return 0;
  }

  ShmId = -1;
  Shm = MAP_FAILED;
  Mbx = IsLocalQueue?LinterMbxKey:NetMbxKey;
  ClntChId = -1;
  ClntConnId = -1;
  ConnId = -1;
  
  ClntChId = ChannelCreate(_NTO_CHF_FIXED_PRIORITY|_NTO_CHF_UNBLOCK|_NTO_CHF_DISCONNECT); /* 17.03.10 */
  if ( ClntChId == -1 )
  {
    return -1;
  }
    
  ClntConnId = ConnectAttach(ND_LOCAL_NODE, 0, ClntChId, _NTO_SIDE_CHANNEL, _NTO_COF_CLOEXEC);
  if ( ClntConnId == -1 )
  {
    goto AttachFailed;
  }

  {                                                         /* 15.07.10 ... */
    struct sched_param Param;
    int Policy = SchedGet( 0, 0, &Param);
    
    if ( Policy == -1 )
    {
      goto AttachFailed;
    }
    
    {
      int MinPrior = sched_get_priority_min( Policy );
      if ( MinPrior == -1 )
      {
        goto AttachFailed;
      }
      Server->Prior = MinPrior;
    }
  }                                                         /* ... 15.07.10 */
  
  ShmId = shm_open(Mbx, O_RDONLY, 0);
  if ( ShmId == -1 )
  {
    goto AttachFailed;
  }

  {
    struct stat Stat;
    
    if ( fstat(ShmId, &Stat) == -1 )
    {
      goto AttachFailed;
    }
    
    if ( Stat.st_size < PAGESIZE )
      goto AttachFailed;
  }
  
  Shm = mmap(0, PAGESIZE, PROT_READ, MAP_SHARED, ShmId, 0);
  if ( Shm == MAP_FAILED )
  {
    goto AttachFailed;
  }
  
  Pid    = ((L_LONG*)Shm)[0];
  RemoteChId   = ((L_LONG*)Shm)[2];
  RemoteConnId = ((L_LONG*)Shm)[3];
  
  /* Closing shm segment. All needed data was read from it */
  munmap(Shm, PAGESIZE);
  Shm = MAP_FAILED;
  close(ShmId);
  ShmId = -1;
  
  /* Check server alive */
  if (kill( Pid , 0) != 0 && errno != EPERM)
  {
    goto AttachFailed;
  }
  
  /* Opening connection to the server */
  ConnId = ConnectAttach(ND_LOCAL_NODE, Pid, RemoteChId, 
                             _NTO_SIDE_CHANNEL, _NTO_COF_CLOEXEC);
  if ( ConnId == -1 )
  {
    goto AttachFailed;
  }


  if ( strlen(LinterMbxKey) + 20 > sizeof(Address) )
  {
    goto AttachFailed;
  }
    
  sprintf(Address, "%s_%08X%08X", LinterMbxKey, Client.Pid, Pid);
  Mask = umask(0);

  ShmId = shm_open(Address, O_CREAT|O_EXCL|O_RDWR, 0666);
  if ( ShmId == -1 )
  {
    umask(Mask);
    goto AttachFailed;
  }
  
  umask(Mask);

  if ( ftruncate(ShmId, SHMSIZE ) )
  {
    goto AttachFailed;
  }
  
  Shm = mmap(0, SHMSIZE, PROT_NOCACHE|PROT_READ|PROT_WRITE,
                                                MAP_SHARED, ShmId, 0);
  if (Shm == MAP_FAILED)
  {
    goto AttachFailed;
  }
  

  /* Send first connect packet to the server */
  {
    char Buf[sizeof(TCBL) + 64 + sizeof(TAILMSG)];
    TMSG* Msg = (TMSG*)Buf;
    
    memset(Buf, 0, sizeof(Buf));
    
    /* TaskName - ExchId, SysErr - ChId, 
       RowCount- Notify(Pulse, signal), CodErr - ConnId */
    Msg->TaskName = 0xFFFFFFFF;
    Msg->CBL.SysErr = ClntChId;
#if defined(PTHREAD_INTLIB)
    Msg->CBL.RowCount = SIGEV_PULSE;
#else /* PTHREAD_INTLIB */
    Msg->CBL.RowCount = SIGEV_SIGNAL_CODE;
#endif /* PTHREAD_INTLIB */
    Msg->CBL.CodErr   = ClntConnId;
    while ( MsgSend(ConnId, Msg, sizeof(TCBL) + sizeof(TAILMSG) + 8 + 8,
                         Buf, sizeof(Buf)) == -1 )
    {
      if ( errno == EINTR )
        continue;
      goto AttachFailed;
    }
    Server->ExchId = Msg->TaskName;
    
    shm_unlink(Address); /* At the momont the kernel has to attach to the shm */
    
    /* receive packet from server - get RcvId */
    /* Set timeout to receive the answer */
    {
      struct sigaction Action;
      
      memset(&Action, 0, sizeof(Action));
#if defined(PTHREAD_INTLIB)
      Action.sa_handler = AlarmHandler;
#else /* PTHREAD_INTLIB */
      Action.sa_handler = CheckServer;
      sigaddset(&Action.sa_mask, SIGUSR1);
      sigaddset(&Action.sa_mask, SIGALRM);
#endif /* PTHREAD_INTLIB */
      if ( sigaction(SIGALRM, &Action, NULL) )
      {
        goto AttachFailed;
      }

#if defined(PTHREAD_INTLIB)
      alarm(TIMEOUT);
#else /*PTHREAD_INTLIB*/  
      /* SIGALRM every TIMEOUT sec */
      if ( Client.Timer == (timer_t)0 )
      {
        
        if ( timer_create(CLOCK_REALTIME, NULL, &Client.Timer) )
        {
          Client.Timer = (timer_t)0;
          goto AttachFailed;
        }
        
      }
      
      {
        struct itimerspec Timeout;
        
        memset(&Timeout, 0, sizeof(Timeout));
        Timeout.it_value.tv_sec = TIMEOUT;
        Timeout.it_interval.tv_sec = TIMEOUT;
        if ( timer_settime( Client.Timer, 0, &Timeout, NULL) )
        {
          timer_delete(Client.Timer);
          Client.Timer = (timer_t)0;
          goto AttachFailed;
        }
      }
      
      /* Set sigusr1 handler */
      memset(&Action, 0, sizeof(Action));
      Action.sa_handler = AstReceive;
      Action.sa_flags   = SA_SIGINFO; /* Queued signal */
      sigaddset(&Action.sa_mask, SIGUSR1);
      sigaddset(&Action.sa_mask, SIGALRM);
      if ( sigaction(SIGUSR1, &Action, NULL) )
      {
        goto AttachFailed;
      }
      /* Enable sigusr1 */
      sigemptyset(&Action.sa_mask);
      sigaddset(&Action.sa_mask, SIGUSR1);
      if ( sigprocmask(SIG_UNBLOCK, &Action.sa_mask, NULL) )
      {
        goto AttachFailed;
      }
#endif /* PTHREAD_INTLIB */

      RcvId = MsgReceive(ClntChId, Buf, sizeof(Buf), NULL);
      if ( RcvId == -1 )
      {
        goto AttachFailed;
      }
    }
    
    if ( MsgError(RcvId, 0) )
    {
      goto AttachFailed;
    }
  }

#if defined(PTHREAD_INTLIB)
  if ( RunReaderThread(IsLocalQueue) != 0 )
  {
    goto AttachFailed;
  }
#endif /* PTHREAD_INTLIB */

  Server->Pid  = Pid;
  Server->RcvId = RcvId;
  Server->ConnId = ConnId;
  Server->RemoteConnId = RemoteConnId;
  Server->RemoteChId      = RemoteChId;
  Server->Shm = (char*)Shm;
  Server->ShmId = ShmId;
  Server->ClntConnId = ClntConnId;
  Server->ClntChId      = ClntChId;
  return 0;

AttachFailed:
  if ( Shm != MAP_FAILED )
    munmap(Shm, SHMSIZE);
  if ( ShmId != -1)
    close(ShmId);
  if ( ConnId != -1 )
    ConnectDetach(ConnId);
  if ( ClntConnId != -1 )
    ConnectDetach(ClntConnId);
  if ( ClntChId != -1 )
    ChannelDestroy(ClntChId);
  return -1;
#endif                                                      /* ... 14.12.09 */
}
#endif /* QNX VXWORKS */                                    /* ... 22.08.02 */

#if defined(UNIX) || defined(QNX)                           /* 08.02.07 .. */
static void GetEnv()
{
  char *Env;

#if defined(UNIX)
  Env = getenv("LINTER_MBX");
  if ( Env == NULL )
    LinterMbxKey = KEYLIN;
  else
    LinterMbxKey = atoi(Env);

  Env = getenv("NET_MBX");
  if ( Env == NULL )
    NetMbxKey = KEYNET;
  else
    NetMbxKey = atoi(Env);
#elif defined(QNX)

  Env = getenv("LINTER_MBX");
#if defined(QNX6)
  strcpy(LinterMbxKey, "/");
#else
  LinterMbxKey[0] = 0;
#endif
  if ( Env != NULL && strlen(Env) < L_MAX_MBX )             /* 17.02.14 */
  {
    strcat(LinterMbxKey, Env);                              /* 17.02.14 */
  }
  else
    strcat(LinterMbxKey, "linter");

  Env = getenv("NET_MBX");
#if defined(QNX6)
  strcpy(NetMbxKey, "/");
#else
  NetMbxKey[0] = 0;
#endif
  if ( Env != NULL && strlen(Env) < L_MAX_MBX )             /* 17.02.14 */
  {
    strcat(NetMbxKey, Env);                                 /* 17.02.14 */
  }
  else
    strcat(NetMbxKey, "linter_net");

  sprintf(ThisMbxKey, "%s_%x", LinterMbxKey, getpid());
#else
  strcpy(LinterMbxKey, "linter");
  strcpy(NetMbxKey, "linter_net");
#endif
}
#endif                                                      /* .. 08.02.07 */

#if defined(KRB_AUTH)                                       /* 24.08.14 ... */
static L_BOOL CrtQueInt(int flag);
static L_BOOL CrtQue(int flag)
{
  if ( KrbForce == -1 )
  {
    char* krb = getenv("LINTER_KRB");

    KrbForce = 0;
    if (krb)
    {
      KrbForce = 1;
    }
  }
  return CrtQueInt(flag);
}
static L_BOOL CrtQueInt(int flag)
#else
static L_BOOL CrtQue(int flag)                                  /* 08.01.03 */
#endif                                                      /* ... 24.08.14 */
{
#ifdef INTER_MSDOS
 memset(&sregs, 0, sizeof(sregs));
#ifdef __WATCOMC__
 regs.w.ax = 0x35B4;
 int86x(0x21, &regs, &regs, &sregs);
#else
 regs.x.ax = 0x35B4;
 int86x(0x21, &regs, &regs, &sregs);
#endif
 if ('L' == (char)regs.h.bl)
   return (L_TTRUE);
#endif /* INTER_MSDOS */
#ifdef INTER_DPMI
 memset(&regs, 0, sizeof(regs));
 regs.w.ax = 0x0200;
 regs.h.bl = 0xB4;
#ifdef __FLAT__
 int386(0x31, &regs, &regs);
#else
 int86(0x31, &regs, &regs);
#endif
 if ('L' == (char)regs.w.dx)
   return (L_TTRUE);
#endif /* INTER_DPMI */
#ifdef INTER_MSWINDOWS
#ifdef WIN32
#ifndef WINCE /* warnings shutdown */                       /* 08.04.02 */
 TCHAR szMbx[L_MAX_MBX+32];                                 /* 17.02.14 11.12.98 */
 SECURITY_ATTRIBUTES SA;
 PSECURITY_DESCRIPTOR pSD;
 BYTE  SD[SECURITY_DESCRIPTOR_MIN_LENGTH];
#endif /* WINCE */
 OSVERSIONINFO vi;
 TCHAR      szName[32];                                     /* 17.02.14... 03.03.03 */
 TCHAR      lpszKerQue[L_MAX_MBX+32];
 TCHAR      lpszKerReady[L_MAX_MBX+32];
 TCHAR      lpszKerWait[L_MAX_MBX+32];
 TCHAR      lpszProcessInfo[L_MAX_MBX+32];                  /* ... 17.02.14 */
 DWORD      dwProcID;
 DWORD      dwThrdID;
 PExchangeOut  pTmpOut;
 BOOL          bRet = TRUE;
 BOOL          bDeinit = FALSE;
 HANDLE        hFileMap;
 static TCHAR *Pref = TEXT("");                             /* 15.01.04 */
 TCHAR        *CurPref;                                     /* 08.04.07 */

 if (lInit < 0 && (InterlockedIncrement(&lInit) == 0))      /* 03.12.98 */
 {
#ifdef DEBUG_EVENT_TRACE
   InitializeCriticalSection(&sg_critSect);                   /* 13.02.05 */
#endif
   if(gExchType == undefined_exch)
   {
     vi.dwOSVersionInfoSize = sizeof(vi);
     GetVersionEx(&vi);
#ifdef WINCE
     gExchType = memory_exch;
     nOS = OS_WINCE;
#else
     if (vi.dwPlatformId == VER_PLATFORM_WIN32_NT)
     {
       nOS = OS_WINNT;
# if _VER_MAX >= 620 || defined(RTOS32) || defined(MEMORY_EXCHANGE) /* 12.04.12 */
       gExchType = memory_exch;
# else
       gExchType = slot_exch;
# endif
     }
     else
     {
       gExchType = memory_exch;
       nOS = OS_WIN95;
     }
     if (vi.dwMajorVersion >= 5)                          /* 08.04.07 15.01.04 */
       Pref = TEXT("Global\\");                           /* 08.04.07 15.01.04 */
#endif
   }
   if((gpInput = (PExchangeIn)calloc(1, sizeof(ExchangeIn))) == NULL)
   {
     lInit = -1;
#ifdef DEBUG_EVENT_TRACE
     DeleteCriticalSection(&sg_critSect);                   /* 13.02.05 */
#endif
     return(FALSE);
   }
   hMutexExchLock = CreateMutex(NULL, FALSE, NULL);        /* 22.01.03 */
   hLock = CreateMutex(NULL, FALSE, NULL);                  /* 03.12.98 */
#ifndef RTOS32   
   dwProcID = GetCurrentProcessId();
#else
   dwProcID = GetCurrentThreadId(); /*GetCurrentProcessId is a Dummy and returns 1*/
#endif
   switch(gExchType)
   {
#if !defined(WINCE) && !defined(RTOS32)                                             /* 03.03.03 ...*/
   case slot_exch:
       pSD = (PSECURITY_DESCRIPTOR)SD;                       /* 11.03.98 ... */
       InitializeSecurityDescriptor(pSD, SECURITY_DESCRIPTOR_REVISION);
       SetSecurityDescriptorDacl(pSD, TRUE, (PACL)NULL, FALSE);
       SA.nLength = sizeof(SA);
       SA.lpSecurityDescriptor = pSD;
       SA.bInheritHandle = FALSE;                          /* 05.09.03 */
       wsprintf(szName, TEXT("\\\\.\\mailslot\\%u"LVER), dwProcID);
       gpInput->hMailslotClient = CreateMailslot(szName, 0L, (DWORD)-1, &SA);    /* ... 11.03.98 */
       if (gpInput->hMailslotClient == INVALID_HANDLE_VALUE)
       {
         bDeinit = TRUE;
         goto input_end;
       }
       break;
#endif                                                    /*... 03.03.03 */
     case memory_exch:
#ifndef RTOS32      
       wsprintf(szName, TEXT("M%u"), dwProcID);
       gpInput->hFileMapClient = CreateFileMapping((HANDLE)-1, NULL, PAGE_READWRITE, 0, LENMSG, szName);
       gpInput->lpvMapViewClient = MapViewOfFile (gpInput->hFileMapClient, FILE_MAP_WRITE, 0, 0, 0);
       wsprintf(szName, TEXT("E%u"), dwProcID);
       gpInput->hEventClientAdd = CreateEvent(NULL, FALSE, FALSE, szName);
       wsprintf(szName, TEXT("EL%u"), dwProcID);
       gpInput->hEventClientReady = CreateEvent(NULL, FALSE, TRUE, szName);
#else
       sprintf(szName, TEXT("M%u"), dwProcID);
       dwClientBuff=calloc(8196,1);
       LinterProcessesInfo.Intlib.dwProcessId = dwProcID;
       gpInput->lpvMapViewClient=LinterProcessesInfo.Intlib.dwSerBuff = dwClientBuff;
       sprintf(szName, TEXT("E%u"), RTOS32_CHANGE_VALUE);
       gpInput->hEventClientAdd = CreateEvent(NULL, FALSE, FALSE, szName);
       sprintf(szName, TEXT("EL%u"), RTOS32_CHANGE_VALUE);
       gpInput->hEventClientReady = CreateEvent(NULL, FALSE, TRUE, szName);
#endif 
       if (!gpInput->lpvMapViewClient || !gpInput->hEventClientAdd || !gpInput->hEventClientReady)
       {
         bDeinit = TRUE;
         goto input_end;
       }
       break;
   } /* switch gExchType */

   hEventStopAstThrd = CreateEvent(NULL, FALSE, FALSE, NULL); /* 16.05.03 13.10.03 */
   hAstThrd = CreateThread(NULL, IntlibStackSize,(LPTHREAD_START_ROUTINE)ReadMsgFromServer, /* 10.05.12 */
                           NULL, 0, &dwAstTid);           /* 24.07.98 */
   gpInput->hEventSyncChkThrd = CreateEvent(NULL, FALSE, FALSE, NULL); /* 13.10.03 */
   WaitForSingleObject(hMutexExchLock, INFINITE);            /* 13.10.03 */
   gpInput->hChkThrd = CreateThread(NULL, CheckServer_THREAD_STACK_SIZE, (LPTHREAD_START_ROUTINE)CheckServer,
                                    0, 0, &dwThrdID);
   hEventStopChkThrd = CreateEvent(NULL, FALSE, FALSE, NULL); /* 16.05.03 */
   if(!hAstThrd || !hEventStopAstThrd ||
      !gpInput->hChkThrd || !gpInput->hEventSyncChkThrd || !hEventStopChkThrd) /* 16.05.03 */
     bDeinit = TRUE;

input_end:
   if(bDeinit == TRUE)
   {
     ReleaseMutex(hMutexExchLock);                         /* 13.10.03 */
     UninitInputObjects();
     return(FALSE);
   }
 } /* initialize of input objects */
 else
   WaitForSingleObject(hMutexExchLock, INFINITE);            /* 22.01.03 */

#ifndef WINCE                                             /* 03.03.03 ...*/
 {
   DWORD EnvLen;                                            /* 17.02.14 ... */
   
   EnvLen = GetEnvironmentVariable(LIN_MBX, szLinMbxEnv, L_MAX_MBX+1);
   if ( EnvLen == 0 || EnvLen > L_MAX_MBX)
   {
     *szLinMbxEnv = 0;
   }
   EnvLen = GetEnvironmentVariable(NET_MBX, szNetMbxEnv, L_MAX_MBX+1);
   if ( EnvLen == 0 || EnvLen > L_MAX_MBX)
   {
     *szNetMbxEnv = 0;
   }
 }                                                          /* .. 17.02.14 */
#endif                                                    /*... 03.03.03 */

/* check existance of output objects */
 if(gpOutput[flag] == NULL)
 {
   if((pTmpOut = (PExchangeOut)calloc(1, sizeof(ExchangeOut))) == NULL)
   {
     bRet = FALSE;
     goto cs_end;
   }
   gpOutput[flag] = pTmpOut;

   switch(gExchType)
   {
#ifndef WINCE                                             /* 03.03.03 ...*/
   case slot_exch:
       lstrcpy(szMbx, flag == local_ker ? szLinMbxEnv : szNetMbxEnv);     /* 23.05.02 ... */
       if (szMbx[0])
         wsprintf(szName, TEXT("%s(%s)"), flag == local_ker ? LMS : RMS, szMbx);
       else
         lstrcpy(szName, flag == local_ker ? LMS : RMS);                  /* ... 23.05.02 */
       gpOutput[flag]->hMailslotLinter = CreateFile(szName, GENERIC_WRITE, FILE_SHARE_READ | FILE_SHARE_WRITE,
                            NULL, OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL, NULL);
       if (gpOutput[flag]->hMailslotLinter == INVALID_HANDLE_VALUE)
       {
         bRet = FALSE;
         goto cs_end;
       }
       break;
#endif                                                    /*... 03.03.03 */
     case memory_exch:
       if(flag == local_ker)   /* without agent */
       {
         if(szLinMbxEnv[0])
         {
           wsprintf(lpszKerQue, TEXT("Linter(%s)"), szLinMbxEnv);
           wsprintf(lpszKerReady, TEXT("LinterLock(%s)"), szLinMbxEnv);
           wsprintf(lpszKerWait, TEXT("LinterLocal(%s)"), szLinMbxEnv);
         }
         else
         {
           wsprintf(lpszKerQue, TEXT("Linter"));
           wsprintf(lpszKerReady, TEXT("LinterLock"));
           wsprintf(lpszKerWait, TEXT("LinterLocal"));
         }
       }
       else
       {
         wsprintf(lpszKerQue, TEXT("LinterAgent"));
         wsprintf(lpszKerReady, TEXT("LinterAgentReady"));
         wsprintf(lpszKerWait, TEXT("LinterAgentAdd"));
       }

#ifdef WINCE
       gpOutput[flag]->hEventLinterReady = CreateEvent(NULL, FALSE, FALSE, lpszKerReady);
       gpOutput[flag]->hEventLinterAdd = CreateEvent(NULL, FALSE, FALSE, lpszKerWait);
       gpOutput[flag]->hFileMapLinter = CreateFileMapping((HANDLE)-1, NULL, PAGE_READWRITE, 0, 9000, lpszKerQue);
#else
       gpOutput[flag]->hEventLinterReady = OpenEvent(EVENT_ALL_ACCESS, FALSE, lpszKerReady);
       gpOutput[flag]->hEventLinterAdd = OpenEvent(EVENT_ALL_ACCESS, FALSE, lpszKerWait);
#ifndef RTOS32
       gpOutput[flag]->hFileMapLinter = OpenFileMapping(FILE_MAP_WRITE, FALSE, lpszKerQue);
#endif
#endif
#ifndef RTOS32  
       gpOutput[flag]->lpvMapViewLinter = MapViewOfFile (gpOutput[flag]->hFileMapLinter, FILE_MAP_WRITE, 0, 0, 0);
#else
       if (flag == local_ker)   /* without agent */
         gpOutput[flag]->lpvMapViewLinter =LinterProcessesInfo.Linter.dwSerBuff; /*7.05.07*/
       else
         gpOutput[flag]->lpvMapViewLinter =LinterProcessesInfo.Dbc.dwSerBuff; /*7.05.07*/
#endif
       if (!gpOutput[flag]->hEventLinterReady ||
           !gpOutput[flag]->hEventLinterAdd            /* 19.06.06 */
#ifndef RTOS32
           || !gpOutput[flag]->hFileMapLinter
#endif

          )
       {
         bRet = FALSE;
         goto cs_end;
       }
       break;
   } /* switch gExchType */
   CurPref = Pref;                                               /* 08.04.07 */
RetryLocal:
/* Try to get output process information (kernel or agent) */
#ifndef RTOS32
   if (flag == local_ker)   /* without agent */
   {
     if(szLinMbxEnv[0])
       wsprintf(lpszProcessInfo, TEXT("%sLinterProcessInfo(%s)"),CurPref,szLinMbxEnv); /* 08.04.07 */ /* 15.01.04 */ 
     else
       wsprintf(lpszProcessInfo, TEXT("%sLinterProcessInfo"),CurPref);                 /* 08.04.07 */ /* 15.01.04 */
   }
   else                                                      /* 08.04.07 ... */
   {
     if(szNetMbxEnv[0])
       wsprintf(lpszProcessInfo, TEXT("%sAgentProcessInfo(%s)"),CurPref,szNetMbxEnv); /* 15.01.04 */
     else
       wsprintf(lpszProcessInfo, TEXT("%sAgentProcessInfo"),CurPref);                  /* 15.01.04 */
   }                                                         /* ... 08.04.07 */
#ifdef WINCE
   /* WindowsCE dosn't support READONLY FileMapping */
   hFileMap = CreateFileMapping((HANDLE)-1, NULL, PAGE_READWRITE /*PAGE_READONLY*/, 0, sizeof(OutProcessInfo), lpszProcessInfo); /* 03.03.03 */
#else
   hFileMap = OpenFileMapping(FILE_MAP_READ, FALSE, lpszProcessInfo);
#endif
   if (hFileMap)
   {
     LPVOID     lpvMapView;                                  /* 08.04.07 ... */
     lpvMapView = MapViewOfFile (hFileMap, FILE_MAP_READ, 0, 0, sizeof(OutProcessInfo));
     if (lpvMapView)
     {
       memcpy(&gpOutput[flag]->ProcessInfo, lpvMapView, sizeof(OutProcessInfo));
       UnmapViewOfFile(lpvMapView);
     }
     CloseHandle(hFileMap);
     if (!lpvMapView)
     {
       bRet = FALSE;
       goto cs_end;
     }
   }
   else
   {
     if ((GetLastError() == ERROR_FILE_NOT_FOUND) && CurPref[0])
     {
       CurPref = TEXT("");
       goto RetryLocal;
     }

     bRet = FALSE;
     goto cs_end;
   }                                                         /* ... 08.04.07 */
#else 
   if(flag == local_ker)   /* without agent */ 
     memcpy(&gpOutput[flag]->ProcessInfo, &LinterProcessesInfo.Linter, sizeof(OutProcessInfo));
   else
     memcpy(&gpOutput[flag]->ProcessInfo, &LinterProcessesInfo.Dbc, sizeof(OutProcessInfo));
#endif /* RTOS32 */
#ifndef RTOS32
   gpOutput[flag]->hServer = OpenProcess(SYNCHRONIZE, FALSE, gpOutput[flag]->ProcessInfo.dwProcessId);
#else
   gpOutput[flag]->hServer = RTKToWin32Handle(gpOutput[flag]->ProcessInfo.dwProcessId);
#endif
   if (!gpOutput[flag]->hServer)
   {
     bRet = FALSE;
     goto cs_end;
   }
   TRACE_EVENT("Set event in CrtQue", gpInput->hEventSyncChkThrd);  /* 01.08.03 */
   SetEvent(gpInput->hEventSyncChkThrd);
 }

cs_end:
 if(bRet == FALSE)
   UninitOutputObjects(&gpOutput[flag]);
 ReleaseMutex(hMutexExchLock);                             /* 22.01.03 */
 return (bRet);                  /* 08.12.98 */  /* ... 03.12.98 */
#else
 hLclQue = FindWindow("LinterLocalServer",  NULL);
 hDbcQue = FindWindow("LinterRemoteServer", NULL);
 if (hLclQue || hDbcQue)
   return (TRUE);
#endif
#endif /* INTER_MSWINDOWS */

#if defined(QNX) || defined(VXWORKS)                       /* 02.04.98 */ /* 11.01.01 */
#if defined(QNX) && !defined(QNXNATIVE)                     /* 14.12.09 */
 if ( my_pid != getpid() )
 {
#if defined(PTHREAD_INTLIB)                                 /* 19.07.10 ... */
   pthread_mutex_lock(&IntlibMutex);
   if ( my_pid != getpid() )
   {
#endif /* PTHREAD_INTLIB */                                 /* ... 19.07.10 */
   UninitLinterClient();                                    /* 08.02.07 */
   my_pid = getpid();
   GetEnv();                                                /* 08.02.07 */
#if defined(PTHREAD_INTLIB)                                 /* 19.07.10 ... */
   }
   pthread_mutex_unlock(&IntlibMutex);
#endif /* PTHREAD_INTLIB */                                 /* ... 19.07.10 */
 }                                                          /* .. 20.02.06 */
#endif
#if defined(QNXNATIVE)                                      /* 14.12.09 ... */
 if ( Client.Pid != getpid() )
 {
#if defined(PTHREAD_INTLIB)                                 /* 19.07.10 ... */
   pthread_mutex_lock(&IntlibMutex);
   if ( Client.Pid != getpid() )
   {
#endif /* PTHREAD_INTLIB */                                 /* ... 19.07.10 */
   UninitLinterClient();
   Client.Pid = getpid();
   GetEnv();
#if defined(PTHREAD_INTLIB)                                 /* 19.07.10 ... */
   }
   pthread_mutex_unlock(&IntlibMutex);
#endif /* PTHREAD_INTLIB */                                 /* ... 19.07.10 */
 }
#endif /* QNXNATIVE */                                      /* ... 14.12.09 */


#if defined(PTHREAD_INTLIB)
 pthread_mutex_lock(&IntlibMutex);
#if defined(OS2000) && defined(MTOS2000)                    /* 29.11.05 08.12.04 .. */
 if ( IntlibIsInit == 0 )
 {
   pthread_key_create(&UserKey, UninitSpecific);
   IntlibIsInit++;
 }
#endif /* OS2000 */
#endif /* PTHREAD_INTLIB */
 if ( TestServerQueue( flag ) ) /* Opening network or linter queue */
 {
#if defined(PTHREAD_INTLIB)
   pthread_mutex_unlock(&IntlibMutex);
#endif
   return(L_TFALSE);
 }                                                          /* .. 08.12.04 */

#if !defined(QNXNATIVE)                                     /* 14.12.09 */
#if !defined(OS2000) || !defined(PTHREAD_INTLIB)            /* 08.12.04 */
 if ( msg_user == (MSGID_TYPE)-1)                           /* 15.09.04 */
 {
   if ( (msg_user = CreateUserQueue()) == (mqd_t)-1 )
   {
#if defined(PTHREAD_INTLIB)
     pthread_mutex_unlock(&IntlibMutex);
#endif
     return(L_TFALSE);
   }
#if defined(PTHREAD_INTLIB)
   if ( RunReaderThread(0) )
   {
   pthread_mutex_unlock(&IntlibMutex);
   UninitLinterClient();
   return(L_TFALSE);
   }
#endif /* PTHREAD_INTLIB */
#else /* OS2000 PTHREAD_INTLIB */                           /* 08.12.04 .. */
#if defined(MTOS2000)
  /* create one reader thread for one thread */
  if ( pthread_getspecific(UserKey) == NULL ) /* If first run */
  {
    struct PidMsg* DataThread = (struct PidMsg*)calloc(sizeof(struct PidMsg), 1);

    if ( DataThread == NULL )
    {
      pthread_mutex_unlock(&IntlibMutex);
      return(L_TFALSE);
    }

    if ( (DataThread->MsgId = CreateUserQueue() ) == (mqd_t)-1 )
    {
      pthread_mutex_unlock(&IntlibMutex);
      free(DataThread);
      return(L_TFALSE);
    }

    pthread_setspecific(UserKey, DataThread);
    /* Run reader thread and save its PID into DataThread */
    if ( RunReaderThread((int)DataThread) )
    {
      pthread_mutex_unlock(&IntlibMutex);
      free(DataThread);
      UninitLinterClient();
      return(L_TFALSE);
    }
#else /* MTOS2000 */                                        /* 29.11.05 .. */
  {
    if ( msg_user == (mqd_t)-1 )
    {
      if ( (pid_user = RunReaderThread(0)) == -1 )
      {
        pthread_mutex_unlock(&IntlibMutex);
        UninitLinterClient();
        return(L_TFALSE);
      }

      pthread_mutex_lock(&InternalMutex);

      if ( (msg_user = CreateUserQueue()) == (mqd_t)-1 )
      {
        pthread_mutex_unlock(&IntlibMutex);
        return(L_TFALSE);
      }

      pthread_cond_signal(&RunReaderCond.Cond);
      pthread_mutex_unlock(&InternalMutex);
    }
#endif                                                      /* .. 29.11.05 */
#endif /* OS2000 PTHREAD_INTLIB */                          /* .. 08.12.04 */

#ifdef VXWORKS                                             /* 11.01.01 */
#ifndef OS2000
 if (pid_user == -1)
   {
   while ( taskDeleteHookDelete((FUNCPTR)TaskExit) != ERROR);
   if (taskDeleteHookAdd((FUNCPTR)TaskExit) == ERROR)
     return(L_TFALSE);
   pid_user = getpid();
   }
#else                                                       /* 03.05.01 */
#if defined(MTOS2000)                                       /* 29.11.05 */
  pid_user = getpid();
#endif
#endif
#else
  atexit(UninitLinterClient);                                 /* 08.02.07 27.10.99 ... */
#endif
 }
#endif /* QNXNATIVE */                                      /* 14.12.09 */
#if defined(PTHREAD_INTLIB)
 pthread_mutex_unlock(&IntlibMutex);
#else
#if defined(QNX) &&  !defined(QNXNATIVE)                    /* 14.12.09 */                                  /* 11.01.01 */
 signal(SIGALRM, CheckServer);                               /* 27.10.99 */
 alarm(TIMEOUT);
#endif
#endif
 return(L_TTRUE);

#endif                                                       /* ... 22.08.02 */

#ifdef UNIX
 char *s;
 int   key;
 int  *msg, *pid;
#ifdef SOCKETS                                              /* 12.11.98 */
 int    len;
 SDESC  desc;
#if !defined(IOS)                                           /* 11.04.13 */
 struct sockaddr_un saddr;
#else                                                       /* 11.04.13 ... */
 struct sockaddr_in saddr;
 struct hostent *server;
#endif                                                      /* ... 11.04.13 */
#else
 L_BOOL RetVal = L_TFALSE;                                   /* 14.11.03 */
 int  *shmem, shm_id;
#ifndef NO_SEMAPHORES                                      /* 07.04.99 */
 int *sem;
#endif /* NO_SEMAPHORES */
#endif /* SOCKETS */

#if defined(PTHREAD_INTLIB)                                 /* 22.08.02 ... */
 pthread_mutex_lock(&IntlibMutex);

 if ( !IntlibIsInit )
 {
#if !defined(NO_ATFORK) && !defined(PIC)                    /* 10.06.11 07.12.10 11.09.03a */
   pthread_atfork(NULL, NULL, UninitLinterClient);
#endif
   IntlibIsInit++;
#if !defined(SOCKETS)
   signal(SIGUSR1, SIG_IGN);
#else
   signal(SIGPIPE, SIG_IGN);
#endif
   GetEnv();                                                /* 08.02.07 */
 }
#endif  /* PTHREAD_INTLIB */                                /* ... 22.08.02 */

#if !(defined(LINUX) && defined(PTHREAD_INTLIB)) || defined(BSD)          /* 11.09.03a 22.08.02 */
 if (my_pid != getpid())
 {
   UninitLinterClient();                                   /* 08.02.07 */
   GetEnv();
   my_pid = getpid();                                      /* 08.02.07 */
 }
#endif /* LINUX PTHREAD_INTLIB */                          /* 22.08.02 */

#if !defined(PTHREAD_INTLIB)                               /* 22.08.02 */
#ifndef PHP_LINTER                                         /* 24.10.00 */
 if (!pid_lin && !pid_net)
#endif /* PHP_LINTER */
 {
#if defined(BSD) || defined(LINUX)                         /* 28.08.97 */
   sigemptyset(&sa.sa_mask);                               /* 15.03.99 */
   sa.sa_flags   = 0;
   sa.sa_handler = AstReceive;
#ifdef SOCKETS                                              /* 12.11.98 ... */
# if !defined (NO_ASYNC)                                    /* 06.06.01 */
   sigaction(SIGIO,   &sa, 0);
# endif
#else
   sigaction(SIGUSR1, &sa, 0);
#endif
#else
#ifdef SOCKETS
# if !defined (NO_ASYNC)                                    /* 06.06.01 */
   sigset(SIGIO, AstReceive);                   /* replace to sigaction ? */
# endif
#else
   sigset(SIGUSR1, AstReceive);
#endif
#endif
#ifdef SOCKETS
   signal(SIGPIPE, SIG_IGN);
#ifndef PHP_LINTER                                         /* 24.10.00 */
   msg_lin = msg_net = -1;
#endif /* PHP_LINTER */
#endif                                                      /* ... 12.11.98 */

   /* Enable signal for notification once for this task */
   sigemptyset(&newmask);                                     /* 31.05.05 ... */
#if !defined (NO_ASYNC)                                       /* 06.06.01 */
#ifdef SOCKETS
   sigaddset(&newmask, SIGIO);
#else
   sigaddset(&newmask, SIGUSR1);
#endif
#endif
   while ( sigprocmask(SIG_UNBLOCK, &newmask, NULL) == -1 ) /* 14.12.06 */
   {
     if ( errno == EINTR )
       continue;
     return L_TFALSE;
   }
 }
#else      /* PTHREAD_INTLIB */                             /* 22.08.02 ... */
#endif      /* PTHREAD_INTLIB */                            /* 22.08.02 ... */
 if (flag == 1)
 {
   pid = &pid_lin;
   msg = &msg_lin;
#if ! defined(SOCKETS) && ! defined(NO_SEMAPHORES)          /* 07.04.99 */
   sem = &sem_lin;
#endif
 }
 else
 {
   pid = &pid_net;
   msg = &msg_net;
#if ! defined(SOCKETS) && ! defined(NO_SEMAPHORES)          /* 07.04.99 */
   sem = &sem_net;
#endif
 }

 if (*pid > 0)                                              /* 12.01.99 ... */
 {
   if (kill(*pid, 0) < 0 && errno == ESRCH)
   {
#if defined(SOCKETS)                                        /* 08.02.07 */
     if (*msg >= 0)
       close(*msg);
#endif
     *msg = -1;
   }
   else
   {
#if defined(PTHREAD_INTLIB)
     pthread_mutex_unlock(&IntlibMutex);                    /* 08.02.07 */
#endif     /* PTHREAD_INTLIB */
     return L_TTRUE;                                           /* 08.02.07 */
   }
 }                                                        /* ... 12.01.99 */

#ifdef SOCKETS                                              /* 08.02.07 12.11.98 ... */
 if (*msg < 0)
 {
#if !defined(IOS)                                           /* 11.04.13 */
   char* TmpPath = getenv("LINTER_TMP");                    /* 11.11.02 ...*/
   register int Socket;                                     /* 14.02.03 */

   key = (flag == 1)? LinterMbxKey : NetMbxKey;             /* 08.02.07 */
   if ( TmpPath == NULL ||
        strlen(TmpPath) + sizeof("/linter.internal.") + 10 > sizeof(saddr.sun_path) ) /* 17.02.14 */
     TmpPath = "/tmp/linter";
   memset(&saddr, 0, sizeof(saddr));
   saddr.sun_family = AF_UNIX;
   sprintf(saddr.sun_path, "%s/linter.%d", TmpPath, key);   /* ... 11.11.02 */
   len = strlen(saddr.sun_path) + 1 + sizeof(saddr.sun_family);
   if ((Socket = socket(AF_UNIX, SOCK_STREAM, 0)) < 0)      /* 14.02.03 */
#else                                                       /* 11.04.13 ... */
   register int Socket;

   key = (flag == 1) ? LinterMbxKey : NetMbxKey;
   memset(&saddr, 0, sizeof(saddr));
   saddr.sin_family = AF_INET;
   server = gethostbyname("localhost");
   bcopy((char *)server->h_addr,
              (char *)&saddr.sin_addr.s_addr,
              server->h_length);
   saddr.sin_port = htons(key);
   len = sizeof(saddr);
   if ((Socket = socket(AF_INET, SOCK_STREAM, 0)) < 0)
#endif                                                      /* ... 11.04.13 */
   {
#if defined(PTHREAD_INTLIB)
     pthread_mutex_unlock(&IntlibMutex);                    /* 22.08.02 */
#endif
     return (L_TFALSE);
   }

   if (fcntl(Socket, F_SETFD, FD_CLOEXEC) < 0)                /* 14.02.03 *//* 28.10.02 ... */
   {
     close(Socket);                                         /* 14.02.03 */
#if defined(PTHREAD_INTLIB)
     pthread_mutex_unlock(&IntlibMutex);
#endif
     return L_TFALSE;
   }                                                        /* ... 28.10.02 */

   while (connect(Socket, (struct sockaddr *)&saddr, len) < 0)   /* 16.01.07h 14.02.03 */
   {
     if (errno == EINTR)
       continue;                                            /* 16.01.07h */
     close(Socket);                                           /* 14.02.03 */
#if defined(PTHREAD_INTLIB)
     pthread_mutex_unlock(&IntlibMutex);                    /* 22.08.02 */
#endif
     return (L_TFALSE);
   }

   while (read(Socket, &desc, sizeof(SDESC)) < 0)           /* 16.01.07h 14.02.03 */
   {
     if (errno == EINTR)
       continue;                                          /* 16.01.07h */
     close(Socket);                                         /* 14.02.03 */
#if defined(PTHREAD_INTLIB)
     pthread_mutex_unlock(&IntlibMutex);                    /* 22.08.02 */
#endif
     return (L_TFALSE);
   }
   *pid = desc.TaskName;
#if !defined(NO_ASYNC) && !defined(PTHREAD_INTLIB)                 /* 23.10.99 22.08.02 */
#if defined(AIX) || defined(BSD) || defined(LINUX) || defined(SGI) || defined(CYGWIN) /* 10.05.99 */
   fcntl(Socket, F_SETOWN, getpid());                       /* 14.02.03 */
   fcntl(Socket, F_SETFL, FASYNC | O_NONBLOCK);               /* 02.02.99 */
#elif defined HPUX                                          /* 05.04.01 */
   {
     int int_value = getpid();

     ioctl(Socket, SIOCSPGRP, &int_value);                  /* 14.02.03 */
     int_value = 1;
     ioctl(Socket, FIOASYNC, &int_value);                   /* 14.02.03 */
     fcntl(Socket, F_SETFL, O_NONBLOCK);
   }
#else
   fcntl(Socket, F_SETFL, O_NONBLOCK);                      /* 14.02.03 */  /* 29.03.99 */
   ioctl(Socket, I_SETSIG, (char *)S_RDNORM);               /* 14.02.03 */  /* 28.02.99 */
#endif
#endif /* NO_ASYNC PTHREAD_INTLIB */

   len = MAXMSG;                              /* 21.12.99 *//* 11.05.99 ... */
   if (setsockopt(Socket, SOL_SOCKET, SO_SNDBUF, &len, sizeof(len)) < 0 ||
       setsockopt(Socket, SOL_SOCKET, SO_RCVBUF, &len, sizeof(len)) < 0)  /* 14.02.03 */
     {
     close(Socket);                                         /* 14.02.03 */
#if defined(PTHREAD_INTLIB)
     pthread_mutex_unlock(&IntlibMutex);                    /* 22.08.02 */
#endif
     return (L_TFALSE);
     }                                                      /* ... 11.05.99 */

   *msg = Socket;                                           /* 14.02.03 */

#if defined(PTHREAD_INTLIB)                                 /* 22.08.02 ... */
   if ( RunReaderThread(flag) )
   {
     pthread_mutex_unlock(&IntlibMutex);
     return (L_TFALSE);
   }
#endif  /* PTHREAD_INTLIB */                                /* ... 22.08.02 */
 }

#if defined(PTHREAD_INTLIB)
 pthread_mutex_unlock(&IntlibMutex);                        /* 22.08.02 */
#endif     /* PTHREAD_INTLIB */

 return (L_TTRUE);                                             /* ... 12.11.98 */
#else

 if ( *msg < 0 )                                            /* 08.02.07 .. */
 {
   key = (flag == 1)? LinterMbxKey : NetMbxKey;
#ifndef NO_SEMAPHORES                                       /* 07.04.99 */
   sops.sem_num = 0;
   sops.sem_flg = 0;
   sops.sem_op  = 1;
   if ( (*sem = semget(key, 1, 0)) >= 0 )
#endif
   if ( (*msg = msgget(key, 0) ) >= 0 )
   {
     if ((shm_id = shmget(key,0,0)) >= 0)                     /* .. 08.02.07 12.02.97 */
     {
       if ((shmem = (int*)shmat(shm_id, 0, SHM_RDONLY))    /* 15.01.97 */
           != (void *)(-1))
       {
         if ( kill(*shmem, 0) >= 0 || errno == EPERM )        /* 16.01.07g */
         {
           *pid = *shmem;
           RetVal = L_TTRUE;                                     /* 14.11.03 */
         }
         shmdt(shmem);
       }
     }
   }

#if defined(PTHREAD_INTLIB)                                 /* 22.08.02 ... */
   if ( !(flag?LocalThreadID:NetThreadID) )
     if ( RunReaderThread(flag) )
     {
       pthread_mutex_unlock(&IntlibMutex);
       return (L_TFALSE);
     }
#endif                                                      /* ... 22.08.02 */
 }

#if defined(PTHREAD_INTLIB)                                 /* 05.11.03 */
 pthread_mutex_unlock(&IntlibMutex);                        /* 22.08.02 */
#endif  /* PTHREAD_INTLIB */

 return (RetVal);                                           /* 14.11.03 */

#endif /* SOCKETS */
#endif /* UNIX    */
#ifdef OS9000
 mh_com *mh;
 int    *pid_srv;
 char   *s, pipe[L_MAX_MBX+32];                             /* 17.02.14 */

 if ( !pid_lin && !pid_net )
   {
   PipeMsg.pid = getpid();
   if (!(PipeMsg.timefork = CheckServer(&(PipeMsg.pid), 0)))
     return (L_TFALSE);
   }

 s = getenv(flag ? LIN_MBX : NET_MBX);
 if (s == NULL || strlen(s) > L_MAX_MBX)                    /* 17.02.14 ... */
 {
   s = flag ? KEYLIN: KEYNET;
 }                                                          /* .. 17.02.14 */

 if (flag == 1)
   {
   pid_srv  = &pid_lin;
   LinChan  = &LinOutChan;
   TimeFork = &TimeForkLin;
   }
 else
   {
   pid_srv  = &pid_net;
   LinChan  = &NetOutChan;
   TimeFork = &TimeForkNet;
   }
 sprintf(pipe, "PID_%s", s);
 mh = (mh_com *)modlink(pipe, 0);
 if ((int)mh == -1)
   return (L_TFALSE);
 *pid_srv  = *(ADRDAT(mh));
 *TimeFork = *(ADRDAT(mh) + 1);
 munload(pipe, 0);
 if (!*TimeFork)
   return(L_TFALSE);
 if (!CheckServer(pid_srv, *TimeFork))
   return(L_TFALSE);
 sprintf(pipe,"/pipe/%s", s);
 if (!(*LinChan))
   *LinChan = open(pipe, S_IREAD | S_IWRITE);
 if (*LinChan >= 0)
   return(L_TTRUE);
#endif /* OS9000 */
#ifdef VMS
 struct dsc$descriptor_s dsk;                               /* 30.07.12 ... */
 char mbx_name[L_MAX_MBX+32], *s;                           /* 17.02.14 */
 static DmbxArg = 0;
 L_WORD *OutMailbox;                                        /* 20.11.12 */

 if (flag) /* to local linter */
 {
   if ( LinMailbox != 0 )
     return (L_TTRUE);
   if ((s = getenv(LIN_MBX)) && strlen(s) <= L_MAX_MBX )    /* 17.02.14 */
   {
     strcpy(mbx_name, s);
   }
   else
     strcpy(mbx_name, "LINTER");
   OutMailbox = &LinMailbox;
 }
 else
 {
   if ( NetMailbox != 0 )
     return L_TTRUE;
   strcpy(mbx_name,"linter_net");
   if ((s = getenv(NET_MBX)) && strlen(s) <= L_MAX_MBX)     /* 17.02.14 */
     strcpy(mbx_name, s);
   else
     strcpy(mbx_name,"linter_net");
   OutMailbox = &NetMailbox;
 }

 dsk.dsc$a_pointer = mbx_name;
 dsk.dsc$b_dtype = DSC$K_DTYPE_T;
 dsk.dsc$b_class = DSC$K_CLASS_S;
 dsk.dsc$w_length  = strlen(mbx_name);

 if ( SYS$ASSIGN(&dsk, /* MBX NAME */
                 OutMailbox, /* Channel address */
                 0, NULL, AGN$M_WRITEONLY) != SS$_NORMAL)
   return L_TFALSE;

 if (!MyPid)
 {
   $DESCRIPTOR(RelativeTimeDsc, "0 00:00:02.00");
   
   sprintf(mbx_name, "CLNT%X", getpid());
   dsk.dsc$w_length  = strlen(mbx_name);

   if (SYS$CREMBX(1, /* Permanent */
                  &InputMailbox, /* Adress to save mailbox channel */
                  (MAXMSG+4095)/4096*4096, /* Max message size */
                  MAXMSG*2,/* Messages queue */
                  0 , 0,
                  &dsk, /* MBX name */
                  CMB$M_READONLY) != SS$_NORMAL)
     goto Deassign;
   SYS$DELMBX(InputMailbox); /* Will be deleted after deassign */
   if ((SYS$QIO( EFN$C_ENF, /* Event */
                 InputMailbox, /* mailbox Channel */
                 IO$_READVBLK, /* Operation - read and wait */
                 &GlIOSB, /* Status block */
                 ReadMsgFromServer, /* AST */
                 (__int64)&GlRcvMsg, /* AST Arg */
                 &GlRcvMsg, /* Buffer address */
                 LNMSG, /* Buffer len */
                 0, 0, 0, 0)) != SS$_NORMAL)
   {
     goto Deassign;
   }
   MyPid = getpid();

   /* Setting timer */
   SYS$BINTIM(&RelativeTimeDsc, &RelativeTime);
   flag = SYS$SETIMR(EFN$C_ENF, /* Event */
              &RelativeTime, /* Interval */
              CheckServer, /* AST */
              20561, /* Timer ID and AST arg */
              0);
   if ( flag != SS$_NORMAL)
   {
     goto Deassign;
   }
 }

 return (L_TTRUE);
Deassign:
 if ( *OutMailbox )
 {
   SYS$DASSGN(*OutMailbox); /* Free channel */
 }
 *OutMailbox = 0;
 if ( InputMailbox )
   SYS$DASSGN(InputMailbox); /* Free channel */
 InputMailbox = 0;
 MyPid = 0;
 SYS$CANTIM(20561, 0);
 return L_TFALSE;                                           /* ... 30.07.12 */
#endif /* VMS */
#if !defined(UNIX)
 return (L_TFALSE);
#endif
}

#ifndef LITE                                                   /* 08.04.03 */
static L_BOOL IsLocalServer(TCBL *CBL)
{
 return (CBL->Node[0] == ' ' || CBL->Node[0] == 0);            /* 08.12.98 */
}
#else
# define IsLocalServer(CBL) ((CBL)->Node[0] == ' ' || (CBL)->Node[0] == 0)
#endif

static void inter_sync(TASTMSG *AstMsg, TCBL* Cbl, L_LONG Time)   /* 22.08.02 08.01.03 */
{
#if defined(PTHREAD_INTLIB)                                 /* 23.07.10 */
  int SavedWaitFlag;                                        /* 03.04.08d */
#elif defined(QNX) || defined(VXWORKS) || defined(UNIX)     /* 23.07.10 */
  sigset_t oldmask;                                         /* 17.01.7b */
#endif

 if (AstMsg->Actual != 1 && AstMsg->CBL == Cbl)             /* 17.01.07a */
   return;

#if defined(PTHREAD_INTLIB)                                 /* 22.08.02 ... */
 pthread_mutex_lock(&AstMsg->Mutex);
 SavedWaitFlag = AstMsg->WaitFlag;                          /* 03.04.08d */
 AstMsg->WaitFlag = 1;
 while (AstMsg->Actual == 1 && AstMsg->CBL == Cbl)
 {
   pthread_cond_wait(&AstMsg->LCond.Cond, &AstMsg->Mutex);
 }
 AstMsg->WaitFlag = SavedWaitFlag;                          /* 03.04.08d */
 pthread_mutex_unlock(&AstMsg->Mutex);
#else                                                       /* 22.08.02 */

#if defined(QNX) || defined(VXWORKS)                           /* 11.01.01 */
#if defined(OS2000)
  while ( pthread_sigmask(SIG_BLOCK, &newmask, &oldmask) < 0 && errno == EINTR)
#else
  while ( sigprocmask(SIG_BLOCK, &newmask, &oldmask) < 0 && errno == EINTR)
#endif
    ; /* BLANK */

  while (AstMsg->Actual == 1)                                   /* 04.04.00 */
    sigsuspend(&oldmask);

#if defined(OS2000)
  while ( pthread_sigmask(SIG_SETMASK, &oldmask, NULL) < 0 && errno == EINTR)
#else /* QNX */
  while ( sigprocmask(SIG_SETMASK, &oldmask, NULL) < 0 && errno == EINTR)
#endif
    ; /* BLANK */

#endif /* QNX VXWORKS */

#ifdef UNIX
#ifdef NO_ASYNC                                 /* 23.10.99 */
 ReadMsgFromServer(*AstMsg->msg_id);
#else
 while ( sigprocmask(SIG_BLOCK, &newmask, &oldmask) < 0 && errno == EINTR) /* 16.01.07h */
   ; /* BLANK */
 while (AstMsg->Actual == 1)                      /* 04.04.00 */
   sigsuspend(&oldmask);
 while ( sigprocmask(SIG_SETMASK, &oldmask, NULL) < 0 && errno == EINTR) /* 16.01.07h */
   ; /* BLANK */
#endif
#endif

#endif /* PTHREAD_INTLIB */                      /* 22.08.02 */

#ifdef OS9000
 sigmask(1);
 while (AstMsg->Actual)                                       /* 04.04.00 */
   pause();
 sigmask(0);
#endif
#ifdef WIN32

 WaitForSingleObject(AstMsg->hAsyncEvent, (Time == -1)?INFINITE:Time * 1000L); /* 08.01.03 */
 TRACE_EVENT("WAIT for event in inter_sync", AstMsg->hAsyncEvent);  /* 01.08.03 */
#endif
#ifdef VMS
 while (AstMsg->Actual)                                    /* 04.04.00 */
   SYS$HIBER(); /* Waiting prev or next AST call */
#endif
}

#if !defined(KRB_AUTH)                                     /* 24.08.14 ... */
static
#endif
 void SendMsgToServerByAst(TMSG *SndMsg, TASTMSG *AstMsg, L_BOOL bAsync);

void SendMsgToServer(TMSG *SndMsg, TASTMSG *Ast)
{
#if defined(INTER_MSDOS) || defined(INTER_DPMI)
 TASTMSG *AstMsg = Ast;
#else
 TASTMSG *AstMsg = FindFreeDesc(Ast);
#endif
 TCBL* BeginCbl = AstMsg->CBL;                        /* 22.08.02 */
 L_BOOL  bAsync = ((SndMsg->CBL.PrzExe & Q_ASYNC) != 0)
# if defined(WIN32)
    && hAstThrd
#endif
    ;
 SendMsgToServerByAst(SndMsg, AstMsg, bAsync);

#ifdef NO_ASYNC                                        /* 17.05.01 */
 bAsync = L_TFALSE;
#endif
 if (!bAsync)                                          /* 31.07.97 */  /* 01.08.03 ... */
 {
#if !defined(INTER_MSDOS) && !defined(INTER_DPMI)
   inter_sync(AstMsg, BeginCbl, -1);      /* 22.08.02 08.01.03 */
#endif
# if defined (WIN32)
   TRACE_EVENT("Actual cleared for nonasync ", AstMsg->hAsyncEvent); /* 28.07.04 */
   if (AstMsg->CBL->PrzExe & Q_EV_CATCH)
     AstMsg->Actual = 2;
   else
     AstMsg->Actual = 0;
# endif     /* PTHREAD_INTLIB */
 }                                                                     /* ... 01.08.03 */
}

#if !defined(KRB_AUTH)
static
#endif
 void SendMsgToServerByAst(TMSG *SndMsg, TASTMSG *AstMsg, L_BOOL bAsync)
{
 TAILMSG *Tail;                                            /* ... 24.08.14 */
 L_LONG  Length = CalcMsgLen(SndMsg, &Tail, AstMsg);
#if defined(UNIX) && defined(SOCKETS)                      /* 17.02.00 */
 char *Tptr = (char *)SndMsg;                              /* 01.03.00 */
 int  Cnt   = 0;
#endif
#if defined(OS2000) && defined(PTHREAD_INTLIB) && defined(MTOS2000)  /* 29.11.05  18.11.05a */
 int pid_user;
#endif /* OS2000 PTHREAD_INTLIB */
#if defined VMS
 L_WORD Channel;                                            /* 20.11.12 */
 IOSB Iosb;
#endif

#ifdef INTER_MSDOS
 Tail->Os = OS_MSDOS;
 Tail->ClientRealPid = (L_LONG)SndMsg;
 SndMsg->TaskName = (L_LONG)SndMsg;
 memset(&sregs, 0, sizeof(sregs));
#ifdef __WATCOMC__
 regs.w.ax = 0x2598;
 regs.w.dx = FP_OFF(SndMsg);
#else
 regs.x.ax = 0x2598;
 regs.x.dx = FP_OFF(SndMsg);
#endif
 sregs.ds  = FP_SEG(SndMsg);
 int86x(0x21, &regs, &regs, &sregs);
 int86x(0x96, &regs, &regs, &sregs);
#endif /* INTER_MSDOS */
#ifdef INTER_DPMI
 Tail->Os = OS_MSDOS;
 Tail->ClientRealPid = (long)getpid();
 SndMsg->TaskName    = (long)getpid();
 InterDPMI ((char *)SndMsg, sizeof(TMSG));
#endif /* INTER_DPMI */
#ifdef INTER_MSWINDOWS
 BOOL    bOK;
 DWORD   dwProcID, dwThrdID;                               /* 08.01.03 */
 DWORD   dwTmp;
# ifndef WIN32
 DATASTRUCT ds;
# endif

/* 08.01.03 ...*/
# ifdef WIN32
 WaitForSingleObject(hMutexExchLock, INFINITE);            /* 22.01.03 */
 AstMsg->ppOutput = IsLocalServer(&SndMsg->CBL) && gpOutput[local_ker] ? &gpOutput[local_ker] : &gpOutput[remote_ker];
#ifndef RTOS32
 dwProcID = GetCurrentProcessId();
 dwThrdID = GetCurrentThreadId();
#else
 dwProcID = GetCurrentThreadId();
 dwThrdID = dwProcID;
#endif
 SndMsg->TaskName = dwThrdID; 
#  ifdef LONG_ALIGN
 CopyMemory((char *)&Tail->ClientRealPid, (char *)&dwProcID, sizeof(DWORD)); /* 11.12.05 */
 CopyMemory((char *)&Tail->ServerDesc, (char *)&dwProcID, sizeof(L_LONG));   /* 11.12.05 */
#  else
 Tail->ClientRealPid = dwProcID;
#ifndef RTOS32 
 Tail->ServerDesc = dwProcID;
#else
 Tail->ServerDesc = RTOS32_CHANGE_VALUE;
#endif
#  endif
 if (!AstMsg->hAsyncEvent)
   AstMsg->hAsyncEvent = CreateEvent(NULL, FALSE, FALSE, NULL);
 TRACE_EVENT("Reset event in SendMsgToServer", AstMsg->hAsyncEvent);  /* 01.08.03 */
 ResetEvent(AstMsg->hAsyncEvent);
 if (*AstMsg->ppOutput == NULL)    /* if output objects not exists */
# else
 AstMsg->hServer = IsLocalServer(&SndMsg->CBL) && hLclQue ? &hLclQue : &hDbcQue;
 if (AstMsg->hServer == NULL)
# endif
/*... 08.01.03 */
 {
   AstMsg->CBL->CodErr = ERROPENQUE;
   AstMsg->Actual = 0;
# ifdef WIN32
   AstMsg->CBL->SysErr = GetLastError();
   ReleaseMutex(hMutexExchLock);                           /* 22.01.03 */
# endif
   return;
 }
# ifndef WIN32
 ds.dwData = 0L;
 ds.cbData = Length;
 ds.lpData = SndMsg;
 Tail->Os  = OS_WIN3XX;
 SndMsg->TaskName = (L_LONG)SndMsg;
 bOK = (BOOL)SendMessage(*AstMsg->hServer, WM_LINTER_CALL, 0, (LPARAM)&ds);
# else
#  ifdef LONG_ALIGN
 CopyMemory((char *)&Tail->Os, (char *)&nOS, sizeof(Tail->Os));       /* 11.12.05 */
#  else
 Tail->Os = nOS;
#  endif

/* 08.01.03 ...*/
 switch(gExchType)
 {
   case slot_exch:
     bOK = WriteFile((*AstMsg->ppOutput)->hMailslotLinter, SndMsg, Length, &dwTmp, NULL);
     break;
   case memory_exch:
     WaitForSingleObject((*AstMsg->ppOutput)->hEventLinterReady, INFINITE);                   /* 23.12.98 */
     CopyMemory((*AstMsg->ppOutput)->lpvMapViewLinter, SndMsg, Length);
     TRACE_EVENT("Set event in SendMsgToServer", (*AstMsg->ppOutput)->hEventLinterAdd);  /* 01.08.03 */
     bOK = SetEvent((*AstMsg->ppOutput)->hEventLinterAdd);
     break;
 } /* switch gExchType */
/*... 08.01.03 */
 ReleaseMutex(hMutexExchLock);                           /* 22.01.03 */
# endif
 if (!bOK)
 {
   AstMsg->CBL->CodErr = ERRWRITEMSG;                       /* 16.12.02 */
# ifdef WIN32
   AstMsg->CBL->SysErr = GetLastError();                   /* 03.12.98 */
   UninitOutputObjects(AstMsg->ppOutput);                  /* 08.01.03 */
# else
   CloseHandle(*AstMsg->hServer);                          /* 13.02.99 */
   *AstMsg->hServer = NULL;                                /* 13.02.99 */
# endif
   AstMsg->Actual = 0;
   return;
 }
# ifdef WIN32                                               /* 01.08.00 */
 if (bAsync)
 {
#ifndef RTOS32
   Sleep(0L);
#else
   RTKDelay(0); 
#endif
 }
# endif
#endif /* INTER_MSWINDOWS */
#ifdef OS9000
  Tail->ClientRealPid = getpid();                               /* 10.06.00 */
  signal(SIGCHECKSHUT, ReadMsgFromServer);
  signal(BASESIG + Tail->AstInd, ReadMsgFromServer);
  if (IsLocalServer(&SndMsg->CBL) && pid_lin)
    AstMsg->Server = pid_lin;
  else
    AstMsg->Server = pid_net;
  if (!AstMsg->ShMem)
    if (!(AstMsg->ShMem = (TMSG *)calloc(LENMSG, 1)))
      {
      SndMsg->CBL.CodErr = ERRWRITEMSG;
      SndMsg->CBL.SysErr = errno;
      ReceiveData(SndMsg, NULL, 0, NULL, NULL);
      return;
      }
  errno = _os_permit(AstMsg->ShMem, LENMSG, MP_WORLD_ACCESS, AstMsg->Server);
  if (errno)
    {
    AstMsg->ShMem->CBL.CodErr = ERRWRITEMSG;
    AstMsg->ShMem->CBL.SysErr = errno;
    raise(BASESIG + Tail->AstInd);
    free(AstMsg->ShMem);
    AstMsg->ShMem = NULL;
    }
  SndMsg->TaskName = getpid();
  memcpy(AstMsg->ShMem, SndMsg, Length);
  PipeMsg.addr  = (int)AstMsg->ShMem;
  PipeMsg.sigcod = BASESIG + Tail->AstInd;
  AstMsg->ShMem->Msg_Type = Length;
  if (!AlmId)
    AlmId = alm_set(SIGCHECKSHUT, TIMECHECK(10));
  if (write(*LinChan, &PipeMsg, sizeof(PipeMsg)) < 0)
    {
    AstMsg->ShMem->CBL.CodErr = ERRWRITEMSG;
    AstMsg->ShMem->CBL.SysErr = errno;
    raise(BASESIG + Tail->AstInd);
    }
#endif
#if defined(QNXNATIVE)                                      /* 14.12.09 ... */
  if (IsLocalServer(&SndMsg->CBL) && Linter.Pid != 0)
  {
    AstMsg->Server = Linter.Pid;
  }
  else
  {
    AstMsg->Server = Dbc.Pid;
    if ( Dbc.Pid == 0 )
    {
      AstMsg->CBL->CodErr = ERROPENQUE;
      AstMsg->Actual = 0;
      return;
    }
  }
#endif /* QNXNATIVE */                                      /* ... 14.12.09 */
#if (defined(QNX) && !defined(QNXNATIVE)) || defined(VXWORKS)   /* 14.12.09 02.04.98 */ /* 11.01.01 */
 if (IsLocalServer(&SndMsg->CBL) && msg_lin != (MSGID_TYPE)-1)    /* 15.09.04 04.06.01 */
   {
   AstMsg->Server = pid_lin;                          /* 12.02.97 */
   AstMsg->msg_id = &msg_lin;
   }
 else
   {
   AstMsg->Server = pid_net;                          /* 12.02.97 */
   AstMsg->msg_id = &msg_net;
   }
 if (*(AstMsg->msg_id) == (MSGID_TYPE)-1)                   /* 15.09.04 */
   {
   AstMsg->CBL->CodErr = ERROPENQUE;
   AstMsg->Actual = 0;
   return;
   }
#ifdef VXWORKS                                        /* 11.01.01 */
 Tail->Os            = OS_VXWORKS;
#if defined(OS2000) && defined(PTHREAD_INTLIB)              /* 08.12.04 */
#if defined(MTOS2000)                                       /* 29.11.05 */
 pid_user = getpid();
#endif
#endif /* OS2000 PTHREAD_INTLIB */
#if !defined LONG_ALIGN                                 /* 04.06.01 */
 Tail->ClientRealPid = pid_user;
 Tail->lReserv = getpid();                                  /* 10.03.04 */
#else
 {
  L_LONG pid = pid_user;
  L_WORD* TmpPtr = (L_WORD*)(&Tail->ClientRealPid);         /* 18.11.05 .. */
  
  TmpPtr[0] = ((L_WORD*)&pid)[0];
  TmpPtr[1] = ((L_WORD*)&pid)[1];

  TmpPtr = (L_WORD*)(&Tail->lReserv);
  pid = (L_LONG)getpid();                                  /* 10.03.04 */
  TmpPtr[0] = ((L_WORD*)&pid)[0];
  TmpPtr[1] = ((L_WORD*)&pid)[1];                           /* .. 18.11.05 */
 }
#endif
 SndMsg->TaskName = pid_user;
#else
 Tail->Os            = OS_QNX;
 Tail->ClientRealPid = getpid();
 SndMsg->TaskName    = getpid();
#endif
 AstMsg->Status = 1;

Label1:
 if (mq_send(*(AstMsg->msg_id), (char*)SndMsg, Length, 0) < 0)      /* 03.06.98 */
   {
   if ((errno == EAGAIN) || (errno == EINTR))
     goto Label1;
   AstMsg->CBL->CodErr = ERRWRITEMSG;
   AstMsg->CBL->SysErr = errno;
   AstMsg->Actual      = 0;
   return;
   }
#endif

#if defined(QNXNATIVE)                                      /* 14.12.09 ... */
  Tail->Os            = OS_QNX;
  Tail->ClientRealPid = Client.Pid;
  SndMsg->TaskName    = Client.Pid;
  SndMsg->Msg_Type        = Length;
  AstMsg->Status = 1;
  {
    IntlibServer_t* Server = AstMsg->Server == Linter.Pid?&Linter:&Dbc;
    L_ULONG RPos;                                           /* 11.11.10 */
    L_ULONG WPos;                                           /* 11.11.10 */
    
    if ( Server->Pid == 0 )
    {
      AstMsg->CBL->CodErr = ERRWRITEMSG;
      AstMsg->Actual      = 0;
      return;
    }
    
#if defined(PTHREAD_INTLIB)
    pthread_mutex_lock(&InternalMutex);                     /* 16.07.10 */
#endif

    WPos = ((L_ULONG*)(Server->Shm + CLNT_HDR_OFFSET))[0];  /* 11.11.10 */
    RPos = ((L_ULONG*)(Server->Shm + CLNT_HDR_OFFSET))[1];  /* 11.11.10 */
    
    Length = (Length + 3) / 4 * 4;
    
    if ( (WPos == RPos) || (((RPos - WPos)&SPACE_MASK) > Length) ) /* Shered memory has free space */
    {
      struct sigevent Event;
      L_ULONG TWPos = WPos & SPACE_MASK;                    /* 11.11.10 */
    
      SIGEV_PULSE_INIT( &Event, Server->RemoteConnId, SIGEV_PULSE_PRIO_INHERIT,
                      _PULSE_CODE_MINAVAIL+3, Server->ExchId);

      if ( TWPos + Length <= SPACE_SIZE )                   /* 11.11.10 */
      {
        memcpy(Server->Shm + CLNT_DATA_OFFSET + TWPos, SndMsg, Length); /* 11.11.10 */
      }
      else
      {
        L_ULONG LenToEndBuf = SPACE_SIZE - TWPos;           /* 11.11.10 */
        
        memcpy(Server->Shm + CLNT_DATA_OFFSET + TWPos, SndMsg, LenToEndBuf); /* 11.11.10 */
        memcpy(Server->Shm + CLNT_DATA_OFFSET, ((char*)SndMsg) + LenToEndBuf,
                 Length - LenToEndBuf);
      }
      
      ((L_ULONG*)(Server->Shm + CLNT_HDR_OFFSET))[0] = (Length + WPos) /* 11.11.10 */
#if defined(QNXNATIVECOMPAT)
                                                          & SPACE_MASK
#endif
                                                                          ;
      if ( MsgDeliverEvent(Server->RcvId, &Event) == -1 ) /* Error deliver pulse */
      {
        ((L_LONG*)(Server->Shm + CLNT_HDR_OFFSET))[0] = WPos;

#if defined(PTHREAD_INTLIB)
        pthread_mutex_unlock(&InternalMutex);               /* 16.07.10 */
#endif

        if ( errno == EAGAIN )
        {
          /* Try send message in sync mode */
          goto SendByMsg;
        }
        AstMsg->CBL->CodErr = ERRWRITEMSG;
        AstMsg->CBL->SysErr = errno;
        AstMsg->Actual      = 0;
        return;
      }

#if defined(PTHREAD_INTLIB)
      pthread_mutex_unlock(&InternalMutex);                 /* 16.07.10 */
#endif

      goto SkipMsgSending;
    }

#if defined(PTHREAD_INTLIB)
    pthread_mutex_unlock(&InternalMutex);                   /* 16.07.10 */
#endif

SendByMsg:
    SndMsg->TaskName    = Server->ExchId;
    while ( MsgSend(Server->ConnId, SndMsg, Length, NULL, 0) != 0 )
    {
      if ( errno == EINTR )
        continue;
      /* Sending failed */
      AstMsg->CBL->CodErr = ERRWRITEMSG;
      AstMsg->CBL->SysErr = errno;
      AstMsg->Actual      = 0;
      return;
    }
  }
SkipMsgSending:
  ; /* BLANK */                                            /* 24.10.14 */
#endif /* QNXNATIVE */                                      /* ... 14.12.09 */

#ifdef UNIX
 pid_t MyPid = getpid();                                    /* 22.08.02 */

 if (IsLocalServer(&SndMsg->CBL) && msg_lin >= 0)
   {
   AstMsg->Server = pid_lin;                          /* 12.02.97 */
   AstMsg->msg_id = &msg_lin;
#if !defined(SOCKETS) && !defined(NO_SEMAPHORES)      /* 07.04.99 */
   AstMsg->sem_id = &sem_lin;
#endif
#if defined(LINUX) && defined(PTHREAD_INTLIB)
   MyPid = LocalThPid;                                /* 22.08.02 */
#endif
   }
 else
   {
   AstMsg->Server = pid_net;                          /* 12.02.97 */
   AstMsg->msg_id = &msg_net;
#if !defined(SOCKETS) && !defined(NO_SEMAPHORES)      /* 07.04.99 */
   AstMsg->sem_id = &sem_net;
#endif
#if defined(LINUX) && defined(PTHREAD_INTLIB)
   MyPid = NetThPid;                                  /* 22.08.02 */
#endif
   }
 if (*(AstMsg->msg_id) < 0)
   {
   AstMsg->CBL->CodErr = ERROPENQUE;
   AstMsg->Actual = 0;
   return;
   }
#ifndef SOCKETS                                       /* 12.11.98 */
 Length -= sizeof(int);
 SndMsg->Msg_Type = 1;                                /* 07.02.99 */
#else
 SndMsg->Msg_Type = Length;                           /* 07.02.99 */
#endif

#if defined(PTHREAD_INTLIB)                           /* 22.08.2 ... */
 pthread_mutex_lock(&InternalMutex);
#endif                                                /* ... 22.08.02 */

l1:
 errno = 0;                                           /* 12.02.97 */
 SndMsg->TaskName = MyPid;                            /* 22.08.02 */
 Tail->Os = OS_UNIX;
#ifdef UNIXWARE
 Tail->Os = OS_UNIXWARE;
#endif
#ifdef USIX
 Tail->Os = OS_USIX;
#endif
#ifdef BSD
 Tail->Os = OS_FREEBSD;
#endif
#ifdef LINUX
 Tail->Os = OS_LINUX;
#endif
#ifdef SCO
 Tail->Os = OS_UNIXSCO;
#endif
#ifdef SINIX
 Tail->Os = OS_SINIX;
#endif
#ifdef MSVS                                        /* 05.01.98 ... */
 Tail->Os = OS_MSVS;
#endif
#ifdef MSVT
 Tail->Os = OS_MSVT;
#endif                                             /* ... 05.01.98 */
#ifdef DIGITAL_UNIX                                /* 13.02.99 */
 Tail->Os = OS_DIGITAL;
#endif
#if defined(SUN) && defined(DATA_MSBF)             /* 13.02.99 */
 Tail->Os = OS_SUNSPARC;
#endif
#if defined(SUN) && !defined(DATA_MSBF)
 Tail->Os = OS_SUNI386;
#endif
#ifdef AIX                                          /* 16.03.99 */
 Tail->Os = OS_AIX;
#endif
#ifdef SGI                                          /* 16.03.99 */
 Tail->Os = OS_SGI;
#endif
#ifdef RTOS32                                          /* 22.05.07 */
 Tail->Os = RTOS32;
#endif
#ifdef ANDROID                                      /* 29.05.13 ... */
 Tail->Os = OS_ANDROID;
#endif                                              /* ... 29.05.13 */
#ifdef IOS                                          /* 11.04.13 ... */
 Tail->Os = OS_IOS;
#endif                                              /* ... 11.04.13 */

#if defined(LONG_ALIGN) && defined(UNIX)           /* 31.05.98 */
 {
   L_LONG Tmp = MyPid;                                      /* 06.02.12 */

   L_MEMCPY4(&Tail->ClientRealPid, &Tmp);                   /* 06.02.12 */
 }
#else
 Tail->ClientRealPid = MyPid;                               /* 22.08.02 */
#endif
/* 12.02.97 */
#if !defined(PTHREAD_INTLIB)                                /* 22.08.02 */
 AstMsg->Status = 1;
#endif /* PTHREAD_INTLIB */                                 /* 22.08.02 */

#ifdef SOCKETS                                                  /* 12.11.98 */
rest:                                                           /* 17.02.00 */
 if ( (Cnt = write(*AstMsg->msg_id, Tptr, Length)) < 0)
#else
 if (msgsnd(*AstMsg->msg_id, SndMsg, Length, IPC_NOWAIT) < 0)
#endif
   {
   if (errno == EINTR)
     goto l1;
#ifdef SOCKETS
   if (errno == EAGAIN)                                         /* 10.03.00 */
     goto l1;
#endif
   AstMsg->CBL->CodErr = ERRWRITEMSG;
   AstMsg->CBL->SysErr = errno;
   AstMsg->Actual      = 0;
#if defined(PTHREAD_INTLIB)                                 /* 22.08.02 ... */
#if defined(SOCKETS)
   close(*AstMsg->msg_id);
#endif
   pthread_mutex_unlock(&InternalMutex);
#endif /* PTHREAD_INTLIB */                                 /* ... 22.08.02 */
   return;
   }
#ifdef SOCKETS                                                  /* 17.02.00 */
 if (Cnt < Length)
    {
    Length -= Cnt;
    Tptr += Cnt;
    goto rest;
    }
#endif
#if defined(PTHREAD_INTLIB)
   pthread_mutex_unlock(&InternalMutex);
#endif
#ifndef SOCKETS                                                 /* 12.11.98 */
#ifndef NO_SEMAPHORES                                           /* 07.04.99 */
l2:
 errno = 0;
 semop(*AstMsg->sem_id, &sops, 1);
 if (errno == EINTR)
   goto l2;
#endif /* NO_SEMAPHORES */

#if !defined(PTHREAD_INTLIB)                                    /* 22.08.02 */
 alrm_handler = signal(SIGALRM, CheckServer);                   /* 11.01.99 */
 if (alrm_handler == CheckServer)                               /* 19.05.99 */
   alrm_handler = NULL;                                         /* 19.05.99 */
 if (alrm_handler == NULL)                                      /* 11.01.99 */
   alarm(TIMEOUT);
/* 12.02.97 */
#endif

#endif  /* NOT SOCKETS */
#endif /* UNIX */
#ifdef VMS
 if (IsLocalServer(&SndMsg->CBL) && LinMailbox != 0)        /* 30.07.12 ... */
   Channel = LinMailbox;
 else
   Channel = NetMailbox;
 Tail->Os = OS_VAXVMS;
 Tail->ClientRealPid = MyPid;
 SndMsg->TaskName    = MyPid;
 SndMsg->Msg_Type    = Length;
 AstMsg->Mailbox = Channel;

 if ((AstMsg->CBL->SysErr =
        SYS$QIOW(EFN$C_ENF, /* No event */
                 Channel, /* Channel to the mailbox */
                 IO$_WRITEVBLK | IO$M_NOW, /* writing and waiting only if the buffer is full */
                 &Iosb, /* Status block */
                 0, 0,
                 SndMsg, /* Data */
                 Length, /* Data len */
                 0,0,0,0)) == SS$_NORMAL)                   /* ... 30.07.12 */
 {
   if (Iosb.iosb$w_status != SS$_NORMAL)
   {
     AstMsg->CBL->CodErr = ERRWRITEMSG;
     AstMsg->CBL->SysErr = Iosb.iosb$w_status;              /* 30.07.12 */
     AstMsg->Actual      = 0;
     return;
   }
 }
 else
 {
   AstMsg->CBL->CodErr = ERRWRITEMSG;
   return;
 }
#endif /* VMS */

}

void RecvMsgFromServer(TMSG *SndMsg, TASTMSG *AstMsg)  /* 16.11.97 ... */
{
 TCBL    RCBL;
 L_LONG  Recv   = 0;                                       /* 16.06.03 */
 L_BYTE *Korteg = (L_BYTE*)AstMsg->Korteg;
 L_LONG  Sz = AstMsg->CBL->LnBufRow;                       /* 02.11.02 ... */
 L_BYTE  Buf[MAXBUF];
 L_LONG  Ind=0;                                            /* 16.06.03 ...*/
 L_LONG  lRestLen;                                         /*... 16.06.03 */

 AstMsg->CBL->CodErr = NORMAL;                             /* 26.05.98 ... */
 if (Korteg /* don't recieve long message if buffer is empty */) /* 12.02.10 */
 {
   memcpy(&RCBL, AstMsg->CBL, LENCBL);
   AstMsg->CBL->PrzExe &= ~Q_ASYNC;
   COM2INT(AstMsg->CBL->Command) = MGET;                   /* 16.01.07 */
   do
   {                                                       /* 16.06.03 ...*/
     AstMsg->CBL->RowId    = Recv + 8;
     AstMsg->CBL->LnBufRow = MAXBUF;
     AstMsg->Korteg = Buf;
     memset(SndMsg->Buf.Byte, 0, 8);
     memcpy(&SndMsg->CBL, AstMsg->CBL, LENCBL);            /* ... 26.05.98 */
     SendMsgToServer(SndMsg, AstMsg);
/*
  IMPORTANT !!!
  This code doesn't check real nullbuf length in first 2 bytes of nullbuf.
  If lenght of buffer returned by kernel is larger than real data+null buffers length
  (i.e. alligned buffer - ??) this may cause segmantation fault.
*/
     if (Sz > Recv)
     {
       lRestLen = Sz - Recv;
       memcpy(&Korteg[Recv],Buf, lRestLen >=MAXBUF ? MAXBUF : lRestLen);
       if (lRestLen < MAXBUF && (AstMsg->Range && !AstMsg->CBL->CodErr))
       {
         memcpy( (char*)AstMsg->Range + Ind,
                 &Buf[lRestLen], AstMsg->CBL->LnBufRow - lRestLen);
         Ind += AstMsg->CBL->LnBufRow - lRestLen;
       }
     }
     else
     {
       if(AstMsg->Range && !AstMsg->CBL->CodErr)
       {
         memcpy( (char*)AstMsg->Range + Ind, Buf,AstMsg->CBL->LnBufRow);
         Ind += AstMsg->CBL->LnBufRow;
       }
     }                                                     /*... 16.06.03 */
     Recv += AstMsg->CBL->LnBufRow;
   } 
   while (AstMsg->CBL->LnBufRow == MAXBUF/*Recv < RCBL.LnBufRow+8*/ && !AstMsg->CBL->CodErr);
   if (AstMsg->CBL->CodErr)
     RCBL.CodErr = AstMsg->CBL->CodErr;
   memset(SndMsg->Buf.Byte, 0, 8);
   COM2INT(SndMsg->CBL.Command) = MCLR;                    /* 16.01.07 */
   SendMsgToServer(SndMsg, AstMsg);
   memcpy(AstMsg->CBL, &RCBL, LENCBL);                     /* ... 02.11.02 */
 }
 AstMsg->CBL->PrzExe &= ~Q_RECEIVE;                        /* 01.10.98 */
 AstMsg->Korteg = Korteg;
#if defined(PTHREAD_INTLIB)                                /* 12.04.07 .. */
 pthread_mutex_init(&AstMsg->Mutex, NULL);
 pthread_cond_init(&AstMsg->LCond.Cond, NULL);
 AstMsg->Actual = 1;
#endif                                                     /* .. 12.04.07 */
 CallUserProc(AstMsg);
#if defined(PTHREAD_INTLIB)                                /* 12.04.07 .. */
 if ( AstMsg->Actual && (AstMsg->CBL->PrzExe & !Q_ASYNC) )
   inter_sync(AstMsg, AstMsg->CBL, 0);
 pthread_mutex_destroy(&AstMsg->Mutex);
 pthread_cond_destroy(&AstMsg->LCond.Cond);                /* 03.04.08b */
#endif                                                     /* .. 12.04.07 */
}

L_LONG CreateArea(TMSG *SndMsg, TASTMSG *AstMsg, L_LONG Len)
{
 SndMsg->CBL.LnBufRow = (L_WORD)Len;
 SndMsg->CBL.PrzExe  &= ~Q_ASYNC;
 memset(SndMsg->Buf.Word, 0, 8);
 COM2INT(SndMsg->CBL.Command) = MALL;                       /* 16.01.07 */
 SendMsgToServer(SndMsg, AstMsg);
 return (AstMsg->CBL->CodErr);
}

L_LONG SendBuffer(TMSG *SndMsg, TASTMSG *AstMsg,
                  L_CHAR *Buf, L_LONG Len,                /* 30.03.98 */
                  L_LONG *Total, L_LONG *Offs)
{
 L_LONG Sent = 0L, Size;

 SndMsg->Buf.Word[*Offs/2] = (L_WORD)(Len + (Len & 1));
 *Offs += 2L;
 while (Len > Sent && !AstMsg->CBL->CodErr)
   {
   Size = min(Len - Sent, (L_LONG)MAXBUF - *Offs);       /* 04.06.98 */
   memcpy(&SndMsg->Buf.Byte[*Offs], &Buf[Sent], Size);
   Sent += Size;
   if ((Size + *Offs) == MAXBUF &&                        /* 22.07.02 */
        (Len + *Offs) > MAXBUF)                           /* 22.07.02 */
     {
     Size = Size + *Offs - 8;
     SndMsg->CBL.RowId = *Total;
     SndMsg->CBL.LnBufRow = (L_WORD)Size;                   /* 04.06.98 */
     memset(SndMsg->Buf.Word, 0, 6);
     COM2INT(SndMsg->CBL.Command) = MPUT;                   /* 16.01.07 */
     SndMsg->Buf.Word[3] = (L_WORD)(Size + (Size & 1));
     SendMsgToServer(SndMsg, AstMsg);
     *Total += Size;
     *Offs = 8L;
     }
   else
     *Offs += Size;
   }
 if (Len & 1)
   SndMsg->Buf.Byte[(*Offs)++] = ' ';
 return (AstMsg->CBL->CodErr);
}                                                      /* ... 16.11.97 */

static L_LONG ErrorHandler(TCBL *CBL, L_CHAR *WHERE, HPCHAR KORTEG)  /* 27.07.98 */ /* 29.07.98 */
{
#if !defined(PTHREAD_INTLIB)                                /* 22.08.02 */
 TASTMSG AstMsg;

 AstMsg.CBL     = CBL;
 AstMsg.Korteg  = KORTEG;
 AstMsg.UsrProc = (USR_PROC)WHERE;                    /* 29.07.98 */
 CallUserProc(&AstMsg);
#else                                                       /* 22.08.02 ... */
 if ( (CBL->PrzExe & Q_ASYNC) && WHERE )                    /* 04.11.02 */
   ((USR_PROC)WHERE)(CBL, KORTEG, NULL);
#endif
 return (CBL->CodErr);
}

/*
 Function set def_codepage for V6.0 and higher.
 Called if first inter call,first GET or RESET by inter_control
 For WIN32 autodetict local codepage
*/

#if _VER_MAX >= 600
#if defined(UNIX) || defined(QNX)                           /* 20.04.06 ... */
typedef struct
{
  const char* m_codeset;
#  ifdef SUNOS
  const char** m_alias;
#  else
  const int m_alias;
#  endif
} CodeSet;

typedef struct
{
  const char*    m_locale;
  const CodeSet* m_codeset;
} LocaleSet;


#  ifdef SUNOS
static const char * aliasTab[] = {
  /* 0 */  "ISO-8859-5", "iso", "iso5", 0,
  /* 4 */  "CP1251",     "win", "win5", 0,
  /* 8 */  "KOI8-R",     "koi", "koi8", 0,
  /* 12 */ "KOI8-U",     "koi", "koi8", 0,
  /* 16 */ "KOI8-T",     "koi", "koi8", 0,
  /* 20 */ "CP1252", 0,
  /* 22 */ "CP1253", 0,
  /* 24 */ "CP1254", 0,
  /* 26 */ "CP1255", 0,
  /* 28 */ "CP1256", 0,
  /* 30 */ "CP1257", 0,
  /* 32 */ "CP1258", 0,
  /* 34 */ "CP819", 0,
  /* 36 */ "CP850",0,
  /* 38 */ "CP862",0,
  /* 40 */ "CP866",0,
  /* 42 */ "CP874",0,
  /* 44 */ "CP932",0,
  /* 46 */ "CP936",0,
  /* 48 */ "CP949",0,
  /* 50 */ "CP950",0,
  /* 52 */ "CP1133",0,
  /* 54 */ "CP1361",0,
  /* 56 */ "EUCJP", 0,
  /* 58 */ "EUCKR", 0,
  /* 60 */ "EUCTW", 0,
  /* 62 */ "EUCCN", 0,
  /* 64 */ "SHIFT_JIS", "SJIS", 0,
  /* 67 */ "ISO-8859-1", "ISO_8859-1", "ISO8859-1", "8859-1", "C", 0,
  /* 73 */ "ISO-8859-2", "ISO_8859-2", "ISO8859-2", "8859-2", 0,
  /* 78 */ "ISO-8859-3", "ISO_8859-3", "ISO8859-3", "8859-3", 0,
  /* 83 */ "ISO-8859-4", "ISO_8859-4", "ISO8859-4", "8859-4", 0,
  /* 88 */ "ISO-8859-5", "ISO_8859-5", "ISO8859-5", "8859-5", 0,
  /* 93 */ "ISO-8859-6", "ISO_8859-6", "ISO8859-6", "8859-6", 0,
  /* 98 */ "ISO-8859-7", "ISO_8859-7", "ISO8859-7", "8859-7", 0,
  /* 103 */ "ISO-8859-8", "ISO_8859-8", "ISO8859-8", "8859-8", 0,
  /* 108 */ "ISO-8859-9", "ISO_8859-9", "ISO8859-9", "8859-9", 0,
  /* 113 */ "ISO-8859-10", "ISO_8859-10", "ISO8859-10", "8859-10", 0,
  /* 118 */ "ISO-8859-11", "ISO_8859-11", "ISO8859-11", "8859-11", 0,
  /* 123 */ "ISO-8859-12", "ISO_8859-12", "ISO8859-12", "8859-12", 0,
  /* 128 */ "ISO-8859-13", "ISO_8859-13", "ISO8859-13", "8859-13", 0,
  /* 133 */ "ISO-8859-14", "ISO_8859-14", "ISO8859-14", "8859-14", 0,
  /* 138 */ "ISO-8859-15", "ISO_8859-15", "ISO8859-15", "8859-15", 0,
  /* 143 */ "ISO-8859-16", "ISO_8859-16", "ISO8859-16", "8859-16", 0
};
#  else
#    define aliasTab 0
#  endif
static const CodeSet codesetTab[] = {
  /* 0 */   {"ISO-8859-1", aliasTab + 67},
  /* 1 */   {"ISO-8859-2", aliasTab + 73},
  /* 2 */   {"ISO-8859-3", aliasTab + 78},
  /* 3 */   {"ISO-8859-4", aliasTab + 83},
  /* 4 */   {"ISO-8859-5", aliasTab + 88},
  /* 5 */   {"ISO-8859-6", aliasTab + 93},
  /* 6 */   {"ISO-8859-7", aliasTab + 98},
  /* 7 */   {"ISO-8859-8", aliasTab + 103},
  /* 8 */   {"ISO-8859-9", aliasTab + 108},
  /* 9 */   {"ISO-8859-10", aliasTab + 113},
  /* 10 */  {"ISO-8859-11", aliasTab + 118},
  /* 11 */  {"ISO-8859-12", aliasTab + 123},
  /* 12 */  {"ISO-8859-13", aliasTab + 128},
  /* 13 */  {"ISO-8859-14", aliasTab + 133},
  /* 14 */  {"ISO-8859-15", aliasTab + 138},
  /* 15 */  {"ISO-8859-16", aliasTab + 143},
  /* Windows *******/
  /* 16 */  {"Windows-1251", aliasTab + 4},
  /* 17 */  {"Windows-1252", aliasTab + 20},
  /* 18 */  {"Windows-1253", aliasTab + 22},
  /* 19 */  {"Windows-1254", aliasTab + 24},
  /* 20 */  {"Windows-1255", aliasTab + 26},
  /* 21 */  {"Windows-1256", aliasTab + 28},
  /* 22 */  {"Windows-1257", aliasTab + 30},
  /* 23 */  {"Windows-1258", aliasTab + 32},
  /* KOI8 **********/
  /* 24 */  {"KOI8-R", aliasTab + 8},
  /* 25 */  {"KOI8-U", aliasTab + 12},
  /* 26 */  {"KOI8-T", aliasTab + 16},
  /* IBM ***********/
  /* 27 */  {"IBM819", aliasTab + 34},
  /* 28 */  {"IBM850", aliasTab + 36},
  /* 29 */  {"IBM862", aliasTab + 38},
  /* 30 */  {"IBM866", aliasTab + 40},
  /* 31 */  {"IBM874", aliasTab + 42},
  /* 32 */  {"IBM932", aliasTab + 44},
  /* 33 */  {"IBM936", aliasTab + 46},
  /* 34 */  {"IBM949", aliasTab + 48},
  /* 35 */  {"IBM950", aliasTab + 50},
  /* 36 */  {"IBM-CP1133", aliasTab + 52},
  /* 37 */  {"IBM1361", aliasTab + 54},
  /* EUC ************/
  /* 38 */  {"EUC-JP", aliasTab + 56},
  /* 39 */  {"EUC-KR", aliasTab + 58},
  /* 40 */  {"EUC-TW", aliasTab + 60},
  /* 41 */  {"EUC-CN", aliasTab + 62},
  /* Japan **********/
  /* 42 */  {"SHIFT-JIS", aliasTab + 64},
  /* 43 */  {"MS_KANJI", 0},
  /* China **********/
  /* 44 */  {"GBK", 0},
  /* 45 */  {"GB18030", 0},
  /* 46 */  {"BIG-5", 0},
  /* 47 */  {"BIG5-HKSCS", 0},
  /* 48 */  {"GB2312", 0},
  /* Thai ***********/
  /* 49 */  {"TIS-620", 0},
  /* UTF-8 Unicode **/
  /* 50 */  {"UTF-8", 0},
  /* Other **************/
  /* 51 */  {"PT154", 0},
  /* 52 */  {"ISCII-DEV", 0},
  /* 53 */  {"GEORGIAN-PS", 0}
};

#  ifdef SUNOS
static int cmpCodeSet(const char* str, const CodeSet* cs)
{
  if (!strcmp(str, cs->m_codeset))
    return 1;
  if (cs->m_alias)
  {
    int i;
    for(i = 0; cs->m_alias[i]; i++)
      if (!strcmp(str, cs->m_alias[i]))
        return 1;
  }
  return 0;
}

static const CodeSet* getCodeset()
{
  int  i;
  char str[256];
  strncpy(str, nl_langinfo(CODESET), sizeof(str));
  for (i = 0; i < sizeof(codesetTab) / sizeof(CodeSet); i++)
  {
    if (cmpCodeSet(str, codesetTab + i))
      return codesetTab + i;
  }
  return codesetTab;
}
#  else
static const LocaleSet localeTab[] = {
  {"C", codesetTab},
  {"POSIX", codesetTab},
  {"af_ZA", codesetTab},
  {"ar_AE", codesetTab + 6},
  {"ar_BH", codesetTab + 6},
  {"ar_DZ", codesetTab + 6},
  {"ar_EG", codesetTab + 6},
  {"ar_IN", codesetTab + 6},
  {"ar_IQ", codesetTab + 6},
  {"ar_JO", codesetTab + 6},
  {"ar_KW", codesetTab + 6},
  {"ar_LB", codesetTab + 6},
  {"ar_LY", codesetTab + 6},
  {"ar_MA", codesetTab + 6},
  {"ar_OM", codesetTab + 6},
  {"ar_QA", codesetTab + 6},
  {"ar_SA", codesetTab + 6},
  {"ar_SD", codesetTab + 6},
  {"ar_SY", codesetTab + 6},
  {"ar_TN", codesetTab + 6},
  {"ar_YE", codesetTab + 6},
  {"be_BY", codesetTab + 16}, /* Belarus CP1251 */
  {"bg_BG", codesetTab + 16}, /* Bulgaria CP1251 */
  {"br_FR", codesetTab},
  {"bs_BA", codesetTab + 1},
  {"ca_ES", codesetTab},
  {"cs_CZ", codesetTab + 1},
  {"cy_GB", codesetTab + 13},
  {"da_DK", codesetTab},
  {"de", codesetTab},
  {"de_AT", codesetTab},
  {"de_BE", codesetTab},
  {"de_CH", codesetTab},
  {"de_DE", codesetTab},
  {"de_LU", codesetTab},
  {"el_GR", codesetTab + 6},
  {"en", codesetTab},
  {"en_AU", codesetTab},
  {"en_BE", codesetTab},
  {"en_BW", codesetTab},
  {"en_CA", codesetTab},
  {"en_DK", codesetTab},
  {"en_DK", codesetTab},
  {"en_GB", codesetTab},
  {"en_HK", codesetTab},
  {"en_IE", codesetTab},
  {"en_IN", codesetTab + 50},
  {"en_NZ", codesetTab},
  {"en_PH", codesetTab},
  {"en_SG", codesetTab},
  {"en_US", codesetTab},
  {"en_ZA", codesetTab},
  {"en_ZW", codesetTab},
  {"es", codesetTab},
  {"es_AR", codesetTab},
  {"es_BO", codesetTab},
  {"es_CL", codesetTab},
  {"es_CO", codesetTab},
  {"es_CR", codesetTab},
  {"es_DO", codesetTab},
  {"es_EC", codesetTab},
  {"es_ES", codesetTab},
  {"es_GT", codesetTab},
  {"es_HN", codesetTab},
  {"es_MX", codesetTab},
  {"es_NI", codesetTab},
  {"es_PA", codesetTab},
  {"es_PE", codesetTab},
  {"es_PR", codesetTab},
  {"es_PY", codesetTab},
  {"es_SV", codesetTab},
  {"es_US", codesetTab},
  {"es_UY", codesetTab},
  {"es_VE", codesetTab},
  {"et", codesetTab + 14},
  {"et_EE", codesetTab + 14},
  {"eu_ES", codesetTab},
  {"fa_IR", codesetTab + 50},
  {"fi_FI", codesetTab},
  {"fo_FO", codesetTab},
  {"fr", codesetTab},
  {"fr_BE", codesetTab},
  {"fr_CA", codesetTab},
  {"fr_CH", codesetTab},
  {"fr_FR", codesetTab},
  {"fr_LU", codesetTab},
  {"ga_IE", codesetTab},
  {"gl_ES", codesetTab},
  {"gv_GB", codesetTab},
  {"he_IL", codesetTab + 7},
  {"hi_IN", codesetTab + 50},
  {"hi_IN.ISCII-DEV", codesetTab + 52},
  {"hr_HR", codesetTab + 1},
  {"hu_HU", codesetTab + 1},
  {"id_ID", codesetTab},
  {"is_IS", codesetTab},
  {"it", codesetTab},
  {"it_CH", codesetTab},
  {"it_IT", codesetTab},
  {"iw_IL", codesetTab + 7},
  {"ja", codesetTab + 38},
  {"ja_JP.eucJP", codesetTab + 38},
  {"ja_JP.eucjp", codesetTab + 38},
  {"ja_JP.EUC", codesetTab + 38},
  {"ja_JP.EUC-JP", codesetTab + 38},
  {"ja_JP.euc", codesetTab + 38},
  {"ja_JP.PCK", codesetTab + 43},
  {"ja_JP.ujis", codesetTab + 38},
  {"ja_JP.SJIS", codesetTab + 42},
  {"ja_JP.mscode", codesetTab + 42},
  {"ja_JP.Shift_JIS", codesetTab + 42},
  {"ka_GE", codesetTab + 53},
  {"kk_KZ", codesetTab + 51},
  {"kl_GL", codesetTab},
  {"ko", codesetTab + 39},
  {"ko_KR.EUC", codesetTab + 39},
  {"ko_KR.eucKR", codesetTab + 39},
  {"kw_GB", codesetTab},
  {"ky_KG", codesetTab + 51},
  {"lt", codesetTab + 12},
  {"lt_LT", codesetTab + 12},
  {"lv", codesetTab + 12},
  {"lv_LV", codesetTab + 12},
  {"mi_NZ", codesetTab + 12},
  {"mk_MK", codesetTab + 4},
  {"mr_IN", codesetTab + 50},
  {"ms_MY", codesetTab},
  {"mt_MT", codesetTab + 2},
  {"nl_BE", codesetTab},
  {"nl_NL", codesetTab},
  {"nn_NO", codesetTab},
  {"no", codesetTab},
  {"no_NO", codesetTab},
  {"nr", codesetTab + 1},
  {"oc_FR", codesetTab},
  {"pl", codesetTab + 1},
  {"pl_PL", codesetTab + 1},
  {"pt_BR", codesetTab},
  {"pt_PT", codesetTab},
  {"ro_RO", codesetTab + 1},
  {"ru", codesetTab + 24}, /* Russian KOI8-R */
  {"ru_RU", codesetTab + 24},
  {"ru_SU", codesetTab + 24},
  {"ru_UA", codesetTab + 25},
  {"ru_RU.PT154", codesetTab + 51},
  {"ru.koi8r", codesetTab + 24},                                                                   /* 01.08.10 */
  {"ru.koi8-r", codesetTab + 24},
  {"ru_RU.koi8r", codesetTab + 24},                                                                /* 01.08.10 */
  {"ru_RU.koi8-r", codesetTab + 24},                                                               /* 01.08.10 */
  {"ru_RU.cp1251", codesetTab + 16}, /* Russian CP1251 */
  {"ru_RU.CP1251", codesetTab + 16}, /* Russian CP1251 */
  {"ru_RU.utf8", codesetTab + 50}, /* utf8 */ /* 26.01.12 */
  {"sh_BA", codesetTab + 1},
  {"sk_SK", codesetTab + 1},
  {"sl_SI", codesetTab + 1},
  {"sq_AL", codesetTab + 1},
  {"sr_SP", codesetTab + 4},
  {"sr_YU", codesetTab + 4},
  {"sv", codesetTab},
  {"sv_FI", codesetTab},
  {"sv_SE", codesetTab},
  {"ta_IN", codesetTab + 50},
  {"tg_TJ", codesetTab + 26},
  {"th", codesetTab + 46},
  {"th_TH", codesetTab + 46},
  {"tl_PH", codesetTab},
  {"tr", codesetTab + 8},
  {"tr_TR", codesetTab + 8},
  {"uk", codesetTab + 25},
  {"uk_UA", codesetTab + 25},
  {"ur_PK", codesetTab + 50},
  {"uz_UZ", codesetTab},
  {"vi_VN", codesetTab + 50},
  {"yi_US", codesetTab + 20},
  {"zh", codesetTab + 41},
  {"zh_CN", codesetTab + 48},
  {"zh_HK", codesetTab + 47},
  {"zh_TW", codesetTab + 46},
  {"zh_CN.EUC", codesetTab + 41},
  {"zh_CN.eucCN", codesetTab + 41},
  {"sr_YU@cyrillic", codesetTab + 4},
  {"ca_ES@euro", codesetTab + 14},
  {"de_AT@euro", codesetTab + 14},
  {"de_BE@euro", codesetTab + 14},
  {"de_DE@euro", codesetTab + 14},
  {"de_LU@euro", codesetTab + 14},
  {"en_IE@euro", codesetTab + 14},
  {"es_ES@euro", codesetTab + 14},
  {"eu_ES@euro", codesetTab + 14},
  {"fi_FI@euro", codesetTab + 14},
  {"fr_BE@euro", codesetTab + 14},
  {"fr_FR@euro", codesetTab + 14},
  {"fr_LU@euro", codesetTab + 14},
  {"ga_IE@euro", codesetTab + 14},
  {"gl_ES@euro", codesetTab + 14},
  {"it_IT@euro", codesetTab + 14},
  {"nl_BE@euro", codesetTab + 14},
  {"nl_NL@euro", codesetTab + 14},
  {"pt_PT@euro", codesetTab + 14},
  {"sv_FI@euro", codesetTab + 14},
  {"en_BE@euro", codesetTab + 14},
  {"en_IE@euro", codesetTab + 14},
  {"en@euro", codesetTab + 14},
  0
};

static const CodeSet* getCodeset()
{
  char cp1[] = "ANSI1251";
  char cp2[] = "CP1251";
  int   i;
  char  str[256];
#ifdef QNX                                                       /* 06.08.06 */
  char  strup[256];                                              /* 01.08.06 */
#endif
  char *p    = setlocale(LC_CTYPE, NULL);
  if (!p)
    return codesetTab;
  strncpy(str, p, 256);
  str[255] = 0;
#ifdef QNX                                                       /* 06.08.06 */
  strcpy(strup,str);
  strupr(strup);                                                 /* 01.08.06 */
#endif

  /* by some key */
  if (strstr(str, "ASCII"))
    return codesetTab;
  if (strstr(str, "KOI8-R"))
    return codesetTab + 24;
  if (strstr(str, "KOI8-U"))
      return codesetTab + 25;
  if (strstr(str, "KOI8-T"))
    return codesetTab + 26;
  if (strstr(str, "GBK"))
    return codesetTab + 44;
  if (strstr(str, "GB18030"))
    return codesetTab + 45;
  if (strstr(str, "BIG5") || strstr(str, "Big5"))
    return codesetTab + 46;
  if (strstr(str, "TIS620") || strstr(str, "TIS-620"))
    return codesetTab + 49;
  if (strstr(str, "UTF-8"))
    return codesetTab + 50;

  /* ISO */
  if ( (p = strstr(str, "ISO-8859-")) )
    p += sizeof("ISO-8859-") - 1;
  else if ( (p = strstr(str, "ISO8859-")) )
    p += sizeof("ISO8859-") - 1;
  else if ( (p = strstr(str, "ISO_8859-")) )
    p += sizeof("ISO_8859-") - 1;
  else if ( (p = strstr(str, "DIS_8859-")) )
    p += sizeof("DIS_8859-") - 1;
  else if ( (p = strstr(str, "iso_8859_")) )
    p += sizeof("iso_8859_") - 1;
  if (p)
  {
    i = 0;
    sscanf(p, "%d", &i);
    if ( i > 0)
      return codesetTab + i - 1;
    else
    {
      return codesetTab;
    }
  }

  /* Windows */
  for (i = 0; i < 8;
       i++, cp1[sizeof(cp1) - 2]++, cp2[sizeof(cp2) - 2]++)
  {
    char *p1, *p2;
#ifdef QNX                                                   /* 06.08.06 */
    if ( (p1 = strstr(strup, cp1)) ||                        /* 01.08.06 */
         (p2 = strstr(strup, cp2)))
#else
    if ( (p1 = strcasestr(str, cp1)) ||                        
         (p2 = strcasestr(str, cp2)))
#endif
    {
      return codesetTab + 16 + i;
    }
  }

  /* IBM */
#define NUMBER_IBMCS 11
#define IBM "IBM"
#define CP "CP"
  for (i = 0; i < NUMBER_IBMCS; i++)
  {
    const char * const imb[] = {"819", "850", "862", "866", "874",
                                "932", "936", "949", "950", "-CP1133", "1361" };
    if ( (p = strstr(str, imb[i])) )
    {
      char *pp = p;
      if ( (*(--p) == 'P' && *(--p) == 'C') ||
           (*(--pp) == 'M' && *(--pp) == 'B' && *(--pp) == 'I') )
        return codesetTab + 27 + i;
    }
  }

  /* CP1251, ISO-8859-5, KOI8-R CP866 */
  if (!strcmp(str, "ru")    ||
      !strcmp(str, "ru_SU") ||
      !strcmp(str, "ru_RU") ||
      !strcmp(str, "bg_BG") ||
      !strcmp(str, "ru_UA") ||
      !strcmp(str, "uk")    ||
      !strcmp(str, "uk_UA") ||
      !strcmp(str, "be_BY"))
  {
    if (!isalpha(0xBB))
    {
      if (isalpha(0xC7))
      {
        if (islower(0xC1))
        {/* KOI8-R or  KOI8-U */
          if (!strcmp(str, "ru_UA"))
            return codesetTab + 25;
          else
            return codesetTab + 24;
        }
        else
          return codesetTab + 16; /* CP1251 */
      }
      else
        return codesetTab + 30; /* CP866 */
    }
    else
      return codesetTab + 4; /* ISO8859-5 */
  }

  /* misc. */
  for (i = 0; i < sizeof(localeTab) / sizeof(LocaleSet) - 1; i++)
  {
    if (!strcmp(str, localeTab[i].m_locale))
    {
      if (localeTab[i].m_codeset)
        return localeTab[i].m_codeset;
      else
      {
        return codesetTab;
      }
    }
  }
  return codesetTab;
}
#  endif /* #  ifdef SUNOS */

#endif                                                      /* ... 20.04.06 */
static void SetDefCodepage()                                /* 22.11.02 ... */
{

#ifndef WINCE                                               /* 08.04.02 */
  if (!LINTER_CP_R)
  {
    LINTER_CP = getenv("LINTER_CP");
    LINTER_CP_R = L_TTRUE;
  }
#endif  /* WINCE */                                        /* ... 27.05.02 */
  if (LINTER_CP)
  {
    def_codepage.cp_type = CP_AS_STR;
    def_codepage.cp_len = min(strlen(LINTER_CP), MAX_IDENT);
    strncpy(def_codepage.cp.s_cp, LINTER_CP, def_codepage.cp_len);
  }
  else
    {
#ifdef WIN32                                               /* 02.11.01 ... */
      TCHAR cp_str[MAX_IDENT + 1] = TEXT("CP");                   /* 26.12.01 ... */ /* 16.11.02 ... */

      if ( GetLocaleInfo( LOCALE_USER_DEFAULT, bUse_OEM_CP ?
                         LOCALE_IDEFAULTCODEPAGE : LOCALE_IDEFAULTANSICODEPAGE,
                         &cp_str[2], sizeof(cp_str) - 2 * sizeof( TCHAR ) ) )
      {
        def_codepage.cp_type = CP_AS_STR;                    /* ... 26.12.01 */
        def_codepage.cp_len  = lstrlen( cp_str );
#if defined(UNICODE) /* include WINCE */                     /* 27.10.06 */
        WideCharToMultiByte( CP_ACP,
                         0, &cp_str[0], -1, def_codepage.cp.s_cp,
                         MAX_IDENT + 1, NULL, NULL );
#  else
        lstrcpyn(def_codepage.cp.s_cp, cp_str, MAX_IDENT);
#  endif                                                   /* ... 16.11.02 */
        return;
      }
#elif defined(UNIX) || defined(QNX)                         /* 20.04.06 ... */
      {
          const CodeSet* codeSet = getCodeset();
          if (codeSet && codesetTab != codeSet)
          {
              def_codepage.cp_type = CP_AS_STR;
              def_codepage.cp_len  = strlen(codeSet->m_codeset);
              strncpy(def_codepage.cp.s_cp, codeSet->m_codeset, MAX_IDENT);
              return;
          }
      }                                                     /* ... 20.04.06 */
#endif                                                     /* ... 02.11.01 */
      def_codepage.cp_type = CP_AS_STR;                    /* ... 26.12.01 */
      def_codepage.cp_len  = strlen("CP437");
      strcpy(def_codepage.cp.s_cp,"CP437");
    }
}                                                          /* ... 22.11.02 */
#endif

static L_LONG detectLCP(char * envvar)                    /* 01.12.05 */
{
#ifdef _ENGLATIN1                                            /* 28.11.05 */
  if ( !strcmp(envvar,ANSI_CP) ||                        /* 02.06.00 ... */  /* 26.09.02 */
       !strcmp(envvar,ISO_CP) ||
       !strcmp(envvar,OEM_CP) )                          /* ... 02.06.00 */
    { /* Take LINTER_CP setting */
      if (!strcmp(envvar,ANSI_CP))
        return(Q_ENCODE);
      if (!strcmp(envvar,ISO_CP))
        return(Q_KOI8);
      return(Q_866);
    }
#else                                                          /* 28.11.05 ... */
  if ( !strcmp(envvar,ANSI_CP) ||                        /* 02.06.00 ... */  /* 26.09.02 */
       !strcmp(envvar,ISO_CP) ||
       !strcmp(envvar,OEM_CP) ||
       !strcmp(envvar,ANSI_CP6) ||
       !strcmp(envvar,ISO_CP6) ||
       !strcmp(envvar,OEM_CP6) )
    { /* Take LINTER_CP setting */
      if (!strcmp(envvar,ANSI_CP) || !strcmp(envvar,ANSI_CP6))
        return(Q_ENCODE);
      if (!strcmp(envvar,ISO_CP) || !strcmp(envvar,ISO_CP6))
        return(Q_KOI8);
      return(Q_866);
    }
#endif                                                          /* ... 28.11.05 */
  return(0);
}

static void detect_codepage(TCBL *CBL, L_CHAR **rbuf,        /* 22.10.01 ... */
                            L_LONG *len)
{
#if _VER_MAX >= 600 && !defined(RUSSIAN_COMPAT)
  static LCODEPAGE lcp;                                    /* 22.11.02 ... */
#endif

#ifndef WINCE                                              /* 08.04.02 */
  if (!LINTER_CP_R)                                        /* 22.11.02 */
  {
    LINTER_CP = getenv("LINTER_CP");
    LINTER_CP_R = L_TTRUE;
    if (LINTER_CP)
      LINTER_CP_ENCODE = detectLCP(LINTER_CP);
  }
#endif  /* WINCE */                                        /* ... 27.05.02 */
#if _VER_MAX < 600 || defined(RUSSIAN_COMPAT)              /* 24.01.00 ... */  
  if (CBL->PrzExe & (Q_ENCODE + Q_KOI8))
  { /* Take user's flags */
    if ((CBL->PrzExe & Q_ENCODE) && (CBL->PrzExe & Q_KOI8)) /* 866 */
      CBL->PrzExe &= ~(Q_ENCODE + Q_KOI8);
  }
  else
  {                                                        /* 29.08.00 */
    if (LINTER_CP && LINTER_CP_ENCODE)                     /* 01.12.05 ... 12.01.10 */
    {
      if (LINTER_CP_ENCODE == Q_866)
        CBL->PrzExe &= ~(Q_ENCODE + Q_KOI8);
      else
        CBL->PrzExe |= LINTER_CP_ENCODE;
    }                                                      /* ... 01.12.05 */
    else
    { /* Take platform defaults */
#if defined(UNIX) || defined(OS2000)                       /* 16.10.01 */
      CBL->PrzExe &= ~Q_ENCODE;
      CBL->PrzExe |= Q_KOI8;
#endif
#if defined(INTER_MSWINDOWS) && !defined(WIN32)
      CBL->PrzExe &= ~Q_KOI8;
      CBL->PrzExe |= Q_ENCODE;
#endif
    }
  }                                                        /* ... 24.01.00 */
#else
  *len = sizeof(LCODEPAGE);
  if (*rbuf && **rbuf)                                     /* 16.11.05 */
  {
    lcp.cp_len = min(strlen(*rbuf), MAX_IDENT);
    strncpy(lcp.cp.s_cp, *rbuf, lcp.cp_len);
    lcp.cp_type = CP_AS_STR;
    *rbuf = (L_CHAR *)&lcp;
  }
  else
  {
    THREAD_INIT(g_DefCpSync)                               /* 22.05.03 ... 03.06.03 ...*/
    THREAD_LOCK(g_DefCpSync)
    if (!def_codepage.cp_type)
      SetDefCodepage();
    *rbuf = (L_CHAR *)&def_codepage;
    THREAD_UNLOCK(g_DefCpSync)                             /*... 22.05.03 ... 03.06.03 */
  }
#endif                                                     /* ... 16.11.02 */
}                                                          /* ... 22.10.01 */

#if defined VXWORKS && ( !(defined(OS2000) && defined(PTHREAD_INTLIB)) )  /* 08.12.04 */                                      /* 29.06.01.. */
L_LONG DLLAPI inter_internal(TCBL *CBL, L_CHAR *RANGE, L_CHAR *RETR, L_CHAR *WHERE, HPCHAR KORTEG);
#if !defined OS2000
#define SEM_FAILED ((sem_t*)-1)
#endif

L_LONG DLLAPI inter(TCBL *CBL, void *range, void *retr, void *where, HPVOID korteg)
  {
  L_LONG RetValue;
  sem_t* Semaphore;
  L_CHAR *RANGE = range, *RETR = retr, *WHERE = where;
  HPCHAR  KORTEG = korteg;

LoopSemOpen:
  Semaphore = sem_open("/IntlibSem", O_CREAT,  0777, 1);
  if (Semaphore == SEM_FAILED)
    {
    if (errno == EINTR)
      goto LoopSemOpen;
    CBL->CodErr = ERROPENQUE;
    CBL->SysErr = errno;
    return (ErrorHandler(CBL, WHERE, KORTEG));
    }
LoopSemWait:
  if ( sem_wait(Semaphore) != 0 )
    {
    if (errno == EINTR)
      goto LoopSemWait;
    CBL->CodErr = ERROPENQUE;
    CBL->SysErr = errno;
    return (ErrorHandler(CBL, WHERE, KORTEG));
    }

  RetValue = inter_internal(CBL, RANGE, RETR, WHERE, KORTEG);
/*  UninitLinterAll();                                      27.02.04 */
  sem_post(Semaphore);
  sem_close(Semaphore);
  return RetValue;
  }
#define inter(CBL, RANGE, RETR, WHERE, KORTEG) inter_internal(CBL, RANGE, RETR, WHERE, KORTEG)
#endif                                                      /* ..29.06.01 */

L_LONG DLLAPI inter(TCBL *CBL, void *range, void *retr, void *where, HPVOID korteg)
{
 TCBL    RCBL;                                           /* 25.09.98 */
 L_CHAR *RANGE = range, *RETR = retr, *WHERE = where;
 HPCHAR  KORTEG = korteg;
 L_WORD  NmrCom;
 L_CHAR *NulInd = RANGE;
 L_LONG  lTotal  = 0L;
 L_LONG  lRange  = 0L;
 L_LONG  lRetr   = 0L;
 L_LONG  lWhere  = 0L;
 L_LONG  lKorteg = 0L;
 L_LONG  ri, rj;
 TASTMSG AstMsg;
#if !defined(INTER_MSDOS) && !defined(INTER_DPMI)
 TMSG    InterMsg;
#endif
 L_LONG  r_Cmnd;                                            /* 16.01.07 22.12.98 */
#ifdef WINCE                                              /* 21.05.03a ...*/
 L_BYTE  bAttempt = 0;
#endif                                                    /*... 21.05.03a */

 if (CBL == NULL)
   return (NULLPOINTER);

 CBL->CodErr = 0L;                                     /* 22.07.98 ... */
 CBL->SysErr = 0L;

 if (AstCall && !(CBL->PrzExe & Q_ASYNC))
   {
#ifdef WIN32
   if (GetCurrentThreadId() == dwAstTid)               /* 24.07.98 */
#endif
     {
     CBL->CodErr = CALLFROMHANDLER;
     return (ErrorHandler(CBL, WHERE, KORTEG));
     }
   }                                                   /* ... 22.07.98 */
#ifdef WINCE                                              /* 23.08.03 */
l_connect:                                                /* 21.05.03a */
#endif
 if ((ri = IsLocalServer(CBL)) != 0)                   /* 03.12.98 */
#ifdef WIN32                                           /*08.01.03 ... 23.05.02 ... */
   rj = gpOutput[local_ker] ? 1 : CrtQue(local_ker);
 if (!ri || !rj)
   rj = gpOutput[remote_ker] ? 1 : CrtQue(remote_ker);
#else                                                  /* ... 23.05.02 ... 08.01.03 */
   rj = CrtQue(1);
 if (!ri || !rj)
   rj = CrtQue(0);
#endif /* WIN32 */
 if (!rj)
   {
#ifdef WINCE                                              /* 21.05.03a ...*/
     if ( COM2INT(CBL->Command) == OPEN ||                  /* 16.01.07 28.11.03 */
          COM2INT(CBL->Command) == DESC )
     {
       if(!bAttempt)
       {
          if(LaunchDefServer())
          {
            bAttempt++;
            goto l_connect;
          }
       }
       if(bAttempt && bAttempt < 10)
       {
         bAttempt++;
         Sleep(1000);
         goto l_connect;
       }
     }                                                    /* 28.11.03 */
#endif /*WINCE*/                                          /*... 21.05.03a */
   CBL->CodErr = ERROPENQUE;
#if defined(UNIX) || defined(QNX) || defined(VXWORKS)   /* 02.04.98 */ /* 11.01.01 */
   CBL->SysErr = errno;
#endif
#ifdef WIN32
   CBL->SysErr = GetLastError();
#endif
   return (ErrorHandler(CBL, WHERE, KORTEG));
   }

recv_data:                                                    /* 21.06.99 */
 if (CBL->PrzExe & Q_RECEIVE)                                 /* 01.10.98 */
   {
   r_Cmnd = COM2INT(CBL->Command);                          /* 16.01.07 22.12.98 */
   COM2INT(CBL->Command) = MGET;                            /* 16.01.07 22.12.98 */
   }
 for (NmrCom = 0; NmrCom < siz; NmrCom++)
   {
   if ( COM2INT(CBL->Command) == comtab[NmrCom].com )       /* 16.01.07 */
     {
     switch(comtab[NmrCom].p2)
       {
       case 0:goto ALLPAR;
       case 1:goto FPAR;
       case 2:switch (comtab[NmrCom].p1)
                 {
                 case 3:if (!RANGE)                          /* 27.04.06 ... */
                        {
                          CBL->CodErr = NULLPOINTER;
                          return (ErrorHandler(CBL, WHERE, KORTEG));
                        }                                    /* ... 27.04.06 */
                        rj = strlen(RANGE);
                        if (rj > MAX_IDENT)
                          rj = MAX_IDENT;
                        break;
                 case 4:rj = CBL->LnBufRow;
                        break;
                 case 5:rj = RANGE ? sizeof(SAVE_POINT) : 0;  /* 29.06.98 */
                        break;                                /* 29.06.98 */
                 case 6:rj = CBL->RowCount;                   /* 12.09.00 */
                        break;                                /* 12.09.00 */
                 default:rj = MAX_IDENT+4;                    /* 26.12.00 */
                 }
              goto ZAPFREL;
       case 3:goto ZAPRET;
       case 4:goto NOPAR;
       case 5:goto ZAPOPEN;
       case 6:goto KORT;
       case 7:goto EXEC_COMPILED;
       case 8:goto ZAPBKUP;
       }
     }
   }
 CBL->CodErr = NOCOMMAND;
 return (ErrorHandler(CBL, WHERE, KORTEG));

FPAR:
 ZAPKORTEG(CBL, KORTEG, &lKorteg);
ALLPAR:
 ZAPBUF(CBL, RANGE, &lRange, ERRRANGE);
 if (CBL->CodErr)
   return (ErrorHandler(CBL, WHERE, KORTEG));
 ZAPBUF(CBL, RETR, &lRetr, ERRRETR);
 if (CBL->CodErr)
   return (ErrorHandler(CBL, WHERE, KORTEG));
 ZAPBUF(CBL, WHERE, &lWhere, ERRWHERE);
 if (CBL->CodErr)
   return (ErrorHandler(CBL, WHERE, KORTEG));
 goto EXECLab;
KORT:
 ZAPKORTEG(CBL, KORTEG, &lKorteg);
 if (CBL->CodErr)
   return (ErrorHandler(CBL, WHERE, KORTEG));
 goto EXECLab;
ZAPRET:
 ZAPBUF(CBL, RETR, &lRetr, ERRRETR);
 if (CBL->CodErr != 0)
   return (ErrorHandler(CBL, WHERE, KORTEG));
NOPAR:
  if (comtab[NmrCom].UseCodepage)
    detect_codepage(CBL, &RETR, &lRetr);                      /* 22.10.01 */
 goto EXECLab;
EXEC_COMPILED:
 {
 RANGE = RETR;
#ifdef PREPARE
 inter_fill_parameters(CBL, RANGE);
#endif
 lRange = (L_LONG)(*(L_WORD*)(RANGE));                      /* 29.11.01 */
 if (CBL->CodErr)
   return (ErrorHandler(CBL, WHERE, KORTEG));
 RANGE += sizeof(L_WORD);
 goto EXECLab;
 }
ZAPFREL:
 lRange = rj; /* + (rj & 1); 29.06.98 */
 goto EXECLab;
ZAPOPEN:
 if (RANGE)
 {
   while (RANGE[lRange] && lRange < (MAX_IDENT * 4 + 5))   /* 27.12.00 */
     lRange++;
 }
 if ( COM2INT(CBL->Command) != KILL )                     /* 16.01.07 22.10.01 ... */
   CBL->NumChan = 0;
 detect_codepage(CBL, &RETR, &lRetr);                      /* ... 22.10.01 */
 goto EXECLab;
ZAPBKUP:                                         /* 20.11.97 -> */
 /*
 This command used for BACKUP command.
 Send KORTEG(last) argument and
 RANGE(second) argument (if present). (May not be needed).
 */
 if (RANGE)
   lRange = strlen(RANGE);
 ZAPKORTEGB(CBL, KORTEG, &lKorteg);              /* 23.05.00 */
 if (CBL->CodErr != 0)
   return (ErrorHandler(CBL, WHERE, KORTEG));
 goto EXECLab;                                      /* <- 20.11.97 */

EXECLab:
 CBL->PrzExe &= (Q_ASYNCDONE | Q_ASYNC | Q_ENCODE | Q_KOI8 | Q_RECEIVE |        /* 18.05.05 */ /* 25.01.00 */
                 M_SPEC | M_OPTIMISTIC | M_SHARE | M_EXCLUSIVE 
#if defined(LOLTP)
                 | Q_LOLTP                                  /* 16.04.07 */
#endif
                 | Q_EV_CATCH | Q_REPLICATOR | Q_USE_UTF8 | Q_USE_ADO   /* 28.10.02 */ /* 15.11.02 */
#if _VER_MAX >= 610                                                        /* 03.07.02 */
                 | M_READ_UNCOMMITTED | M_READ_COMMITTED | M_SERIALIZABLE
#endif /* _VER_MAX >= 610 */
#if defined(KRB_AUTH)                                      /* 24.08.14 ... */
                 | Q_KRBREQ
#endif                                                     /* ... 24.08.14 */
                );
#if defined(LARGE_EXCHANGE)
  CBL->PrzExe |= M_LARGE_EXCHANGE;                          /* 19.10.09 */
#endif
 AstMsg.CBL     = CBL;
 if ( COM2INT(CBL->Command) == TRAN )                       /* 16.01.07 08.02.06 */
   NulInd = NULL;                                        /* 08.02.06 */
#if defined(KRB_AUTH)                                      /* 24.08.14 ... */
 switch(COM2INT(CBL->Command))
 {
   case OPEN:
   case KILL:
   case SHUT:
   {
     if (KrbForce == 1)
     {
#if !defined(PTHREAD_INTLIB) && !defined(WIN32) /* Only sync OPEN queries */
       if ( CBL->PrzExe & Q_ASYNC )
       {
         CBL->CodErr = KRB_ONLYSYNC;
         return (ErrorHandler(CBL, WHERE, KORTEG));
       }
#endif
        CBL->PrzExe |= Q_KRBREQ;
     }
     KrbInit();                                            /* 28.10.14 */
     NulInd = RETR; /* Saving codepage */
     break;
   }
 }
#endif                                                     /* ... 24.08.14 */
 AstMsg.Range   = NulInd;
 AstMsg.Korteg  = KORTEG;
 AstMsg.NmrCom  = NmrCom;
 AstMsg.UsrProc = (USR_PROC)WHERE;
 if (CBL->PrzExe & Q_RECEIVE)                                 /* 01.10.98 ... */
   {
   COM2INT(CBL->Command) = r_Cmnd;                          /* 16.01.07 22.12.98 */
   RecvMsgFromServer(&InterMsg, &AstMsg);
   return (ErrorHandler(CBL, WHERE, KORTEG));
   }                                                          /* ... 01.10.98 */
 if ( COM2INT(CBL->Command) != RPAG &&                      /* 16.01.07 27.10.00 */
      COM2INT(CBL->Command) != WPAG )                       /* 27.10.00 */
 {
#ifdef DATA_MSBF
   CBL->PrzExe |= 0x10L;                                   /* 04.04.98 */
#else
   CBL->PrzExe &= ~0x10L;                                  /* 05.02.99 */
#endif
 }
 ri = rj = 0;
 lTotal = lRange + (lRange & 1) + lRetr + (lRetr & 1) +       /* 16.11.97 ... */
          lWhere + (lWhere & 1) + lKorteg + (lKorteg & 1) + 8;
 if (lTotal > MAXBUF)
 {                                                            /* 25.09.98 ... */
#ifdef LARGE_EXCHANGE                                         /* 03.02.12 ... */
   CBL->CodErr = ERRWRITEMSG;
   return ErrorHandler(CBL, WHERE, KORTEG);
#else
   if (lTotal > 0xFFFF)
   {
     CBL->CodErr = ERRWRITEMSG;
     return ErrorHandler(CBL, WHERE, KORTEG);
   }                                                          /* ... 03.02.12 */
   memcpy(&RCBL, CBL, sizeof(TCBL));
   CBL->PrzExe &= ~Q_ASYNC;
   memcpy(&InterMsg.CBL, CBL, sizeof(TCBL));                  /* ... 25.09.98 */
   if (CreateArea(&InterMsg, &AstMsg, lTotal))
     return (ErrorHandler(CBL, WHERE, KORTEG));
   rj = 8L;
#endif
 }
 if (SendBuffer(&InterMsg, &AstMsg, RANGE, lRange, &ri, &rj))
   return (ErrorHandler(CBL, WHERE, KORTEG));
 if (SendBuffer(&InterMsg, &AstMsg, RETR, lRetr, &ri, &rj))
   return (ErrorHandler(CBL, WHERE, KORTEG));
 if (SendBuffer(&InterMsg, &AstMsg, WHERE, lWhere, &ri, &rj))
   return (ErrorHandler(CBL, WHERE, KORTEG));
 if (SendBuffer(&InterMsg, &AstMsg, KORTEG, lKorteg, &ri, &rj))
   return (ErrorHandler(CBL, WHERE, KORTEG));
 if (lTotal > MAXBUF)
   {
   rj = lTotal - ri;
   InterMsg.Buf.Word[3] = (L_WORD)(rj + (rj & 1));
   memcpy(CBL, &RCBL, sizeof(TCBL));                          /* 25.09.98 */
   }
 memcpy(&InterMsg.CBL, CBL, sizeof(TCBL));                    /* ... 16.11.97 */
 SendMsgToServer(&InterMsg, &AstMsg);
#if defined(INTER_MSDOS) || defined(INTER_DPMI)
 ReceiveData(&InterMsg, CBL, NmrCom, KORTEG, RANGE);
#endif
 if (CBL->PrzExe & Q_RECEIVE)                                /* 21.06.99 */
   goto recv_data;
#if ( (defined(QNX) && !defined(QNXNATIVE)) || /* 14.12.09 */\
  (defined(UNIX) && !defined(SOCKETS))) && !defined(PTHREAD_INTLIB)     /* 22.08.02 27.10.99 02.03.99 */
 if (alrm_handler == NULL)                                   /* 11.01.99 */
   alarm(0);
 else
   signal(SIGALRM, alrm_handler);                            /* 11.01.99 */
 for(ri = 0; ri < AstListSize; ri++)
  for ( rj = 0; rj < ASTBLOCKSIZE; rj++)                     /* 22.08.02 */
   if (AstList[ri][rj].Actual)
     {
     alrm_handler = signal(SIGALRM, CheckServer);            /* 11.01.99 */
     if (alrm_handler == CheckServer)                        /* 19.05.99 */
       alrm_handler = NULL;                                  /* 19.05.99 */
     if (alrm_handler == NULL)                               /* 11.01.99 */
       alarm(TIMEOUT);
     break;
     }
#endif
 if (CBL->PrzExe & Q_ASYNC)
   return (NORMAL);                                          /* 10.10.98 */
 else
   return (CBL->CodErr);                                     /* 10.10.98 */
} /* inter(...) */

/* 31.07.97 */
#ifndef LITE                                                               /* 08.04.03 */
L_BOOL DLLAPI inter_wait_single(TCBL *CBL, L_LONG Time)    /* 01.07.98 */
{

 int  i;                                                    /* 28.08.02 ... */
 int  j;
 TASTMSG* TmpAst = NULL;

 for(i = 0; i < AstListSize; i++)
  for ( j = 0; j < ASTBLOCKSIZE; j++ )
  {
    TASTMSG* Ast = &AstList[i][j];

    if (Ast->CBL == CBL)
    {
#if defined(PTHREAD_INTLIB)
      if ( !Ast->Actual )
        continue;
#endif
      TmpAst = Ast;
      goto inter_wait_single_after_for;
    }
  }
inter_wait_single_after_for:
 if (TmpAst == NULL)
   return (L_TFALSE);                                            /* 01.07.98 */
 if (TmpAst->Actual)                                          /* 04.04.00 15.09.08 */
   inter_sync(TmpAst, CBL, Time);                       /*... 22.08.02 08.01.03 */
 return (L_TTRUE);                                               /* 01.07.98 */
}  /* inter_wait_single(...) */
#endif
/* 31.07.97 */

#ifndef LITE                                                               /* 08.04.03 */
L_BOOL DLLAPI inter_control(TCBL *CBL, int request,        /* 22.10.01 ... */
                            void *arg, int arg_len)
{
  L_BOOL  bResult = L_TTRUE;                               /* 22.05.03 ...*/

  THREAD_INIT(g_DefCpSync)                                /* 03.06.03 */
  THREAD_LOCK(g_DefCpSync)                                /* 03.06.03 */
  switch(request)
    {
    case ICR_ENABLE_MSG:
      msg_enabled = (*(char *)arg != 0);
      break;
    case ICR_USE_UTF8:                                     /* 26.12.01 */
      use_UTF8 = ((*(L_BOOL *)arg) != 0);
      break;
#if defined(WIN32) || defined(PTHREAD_INTLIB)
    case ICR_SET_STACK_SIZE:                                /* 10.05.12 */
      if (arg_len < ReadMsgFromServer_THREAD_STACK_SIZE)
      {
        bResult = L_TFALSE;
        break;
      }
      IntlibStackSize = arg_len;
      break;
#endif /* WIN32 PTHREAD_INTLIB */
#if _VER_MAX >= 600                                         /* 22.11.02 */
    case ICR_GET_CP:
      if (arg_len < sizeof(LCODEPAGE))
      {                                                     /* 03.06.03 ...*/
        bResult = L_TFALSE;
        break;
      }                                                     /*... 03.06.03 */
      if (!def_codepage.cp_type)
        SetDefCodepage();                                   /* 22.11.02 */
      memcpy(arg, &def_codepage, arg_len);
      break;
    case ICR_SET_CP:
      def_codepage.cp_type = (L_WORD)arg_len;
      if (def_codepage.cp_type == CP_AS_NUM)
        def_codepage.cp.n_cp = (L_WORD)(int)arg;            /* 29.07.02 */
      else
        {
        def_codepage.cp_len = min(strlen((char *)arg), MAX_IDENT);
        strncpy(def_codepage.cp.s_cp, (char *)arg, MAX_IDENT);
        }
      break;
#ifdef WIN32                                               /* 26.12.01 ... */
    case ICR_USE_OEM_CP:
      bUse_OEM_CP = ((*(L_BOOL *)arg) != 0);
      SetDefCodepage();                                    /* 25.02.03 */
      break;
#endif                                                     /* ... 26.12.01 */
#endif
    case ICR_REREAD_ENV:                                   /* 24.06.02 ... */
#if defined(WIN32) && !defined(WINCE)
     {
       DWORD EnvLen;                                            /* 17.02.14 ... */
   
       EnvLen = GetEnvironmentVariable(LIN_MBX, szLinMbxEnv, L_MAX_MBX+1);
       if ( EnvLen == 0 || EnvLen > L_MAX_MBX)
       {
         *szLinMbxEnv = 0;
       }
       EnvLen = GetEnvironmentVariable(NET_MBX, szNetMbxEnv, L_MAX_MBX+1);
       if ( EnvLen == 0 || EnvLen > L_MAX_MBX)
       {
         *szNetMbxEnv = 0;
       }
     }                                                          /* .. 17.02.14 */
#endif
#ifndef WINCE
      LINTER_CP = getenv("LINTER_CP");
      LINTER_CP_R = L_TTRUE;
#endif
#if _VER_MAX >= 600
      SetDefCodepage();                                    /* 22.11.02 */
#endif
      break;                                               /* ... 24.06.02 */
#if defined(KRB_AUTH)                                      /* 24.08.14 ... */
    case ICR_FORCE_KRB:
      KrbForce = *(char *)arg;
      break;
#endif                                                     /* ... 24.08.14 */
    default:
      bResult = L_TFALSE;
    }
  THREAD_UNLOCK(g_DefCpSync);                             /* 03.06.03 */
  return (bResult);                                       /*... 22.05.03 */
}                                                          /* ... 22.10.01 */
#endif

#ifdef WINCE                                              /* 21.05.03a ...*/
L_BOOL LaunchDefServer()
{
  HKEY                hRootKey = HKEY_LOCAL_MACHINE;
  HKEY                hKey     = NULL;
  DWORD               dwDispos = REG_OPENED_EXISTING_KEY;
  DWORD               dwIndex  = 0;
  DWORD               dwNamLen;
  TCHAR               szStrTemp[MAX_PATH];
  TCHAR               szKeyName[MAX_PATH];
  TCHAR               szLinHome[MAX_PATH];
  PROCESS_INFORMATION piSrv;
  BOOL                bResult  = FALSE;
  lstrcpy(szKeyName, REG_KEY);
  memset(szLinHome,0,sizeof(szLinHome));
  RegCreateKeyEx(hRootKey, szKeyName, 0L, NULL, 0L, KEY_READ, NULL, &hKey, &dwDispos);
  if (hKey)
  {
    dwDispos = sizeof(szStrTemp);
    dwNamLen = sizeof(szKeyName);
    while(RegEnumValue(hKey, dwIndex++, szKeyName, &dwNamLen, NULL,
                       NULL, (LPBYTE) szStrTemp, &dwDispos) == ERROR_SUCCESS)
    {
      if(CompareString(LOCALE_USER_DEFAULT, NORM_IGNORECASE,
                        szKeyName, -1, LINTER_HOME, -1) == CSTR_EQUAL)
      {
        wsprintf(szLinHome,TEXT("%s\\"),szStrTemp);
        break;
      }
    }
    RegCloseKey(hKey);
  }
  _tcsncat(szLinHome, LINTER_BIN, sizeof(szLinHome)/sizeof(TCHAR) -
                                   _tcslen(szLinHome));
  if(CreateProcess(szLinHome, TEXT("/local"), NULL, NULL, 0,
                   0, NULL, NULL, NULL, &piSrv))
    bResult = L_TTRUE;
  return bResult;
}
#endif                                                    /*... 21.05.03a */
/* -- End of file intlib.c ------------------------------------------------- */
